dojo.provide("dojox.widget.AnalogGauge2");

dojo.require("dojox.gfx");
dojo.require("dojo.number");
dojo.require("dojox.widget.AnalogGauge2._Style"); 
dojo.require("dojox.widget.AnalogGauge2.ReallySimpleStyle");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.declare("dojox.widget.AnalogGaugeThreshold", [dijit._Widget], {
    // summary:
    //          AnalogGaugeThreshold objects are used to denote a threshold for 
    //          an AnalogGauge (duh!)
    // description:
    //          When specifying thresholds for an AnalogGauge, you do not *need* 
    //          to use an AnalogGaugeThreshold object, this class is more for 
    //          the easy support for markup-based instances of an AnalogGauge.
    // examples:
    //          <div dojoType="dojox.widget.AnalogGaugeThreshold" 
    //                  name="First Threshold" start="0" status="Normal" />
    //          
    //          new dojox.widget.AnalogGaugeThreshold({
    //              start: 0,
    //              name: "First Threshold",
    //              status: dojox.widget.AnalogGaugeThreshold.NORMAL_STATUS
    //          });
    
    // start: int
    //          Starting value for the threshold.  It's important to note that 
    //          the ending value for a threshold is specified not by the 
    //          threshold object itself, but by the starting value of the next 
    //          threshold in the AnalogGauge's threshold array.
    start: 0,
    
    // name: String
    //          Name for the threshold, this can be used to look the threshold 
    //          up in the AnalogGauge's threshold array, some styles/themes can 
    //          use this as well.
    name: "",
    
    // status: String
    //          Used to denote the semantic connotation of a gauge's value lying 
    //          within a threshold, the status can tell gauge styles how to 
    //          render the threshold as well.
    status: ""
});
dojox.widget.AnalogGaugeThreshold.CLEAR_STATUS = "Clear";
dojox.widget.AnalogGaugeThreshold.NORMAL_STATUS = "Normal";
dojox.widget.AnalogGaugeThreshold.CRITICAL_STATUS = "Critical";
dojox.widget.AnalogGaugeThreshold.FATAL_STATUS = "Fatal";
dojox.widget.AnalogGaugeThreshold.MAJOR_STATUS = "Major";
dojox.widget.AnalogGaugeThreshold.MINOR_STATUS = "Minor";
dojox.widget.AnalogGaugeThreshold.INDETERMINANT_STATUS = "Intermediate";
dojox.widget.AnalogGaugeThreshold.NOT_APPLICABLE_STATUS = "Not Applicable";
dojox.widget.AnalogGaugeThreshold.UNKNOWN_STATUS = "Unknown";
dojox.widget.AnalogGaugeThreshold.UNDEFINED_STATUS = "Undefined";
dojox.widget.AnalogGaugeThreshold.INFORMATIONAL_STATUS = "Informational";


dojo.declare("dojox.widget.AnalogGauge2",[dijit._Widget, dijit._Templated],{
    // summary:
    //          AnalogGauge2 defines a Widget that is capable of displaying a 
    //          metric as an "analog"-style meter (similar to a spedometer).
    // description:
    //          The dojox.widget.AnalogGauge2 widget will display a metric as a 
    //          spedometer-style gauge.  It is a highly customizable/themeable/
    //          skinnable widget.  
    //          
    //          The base class, dojox.widget.AnalogGauge2 is not responsible for 
    //          any of the visuals.  Visual display is handled by any class 
    //          implementing the dojox.widget.AnalogGauge2._Style interface. (see the 
    //          documentation for dojox.widget.AnalogGauge2._Style for more information 
    //          on styles).  
    //
    //          The AnalogGauge2 class is responsible for handling data, firing 
    //          events, and using the _Style implementation it's assigned to 
    //          draw itself.  It also has the responsibility of keeping track of 
    //          its overall dojox.gfx.Group object, in the case that the gauge 
    //          should be drawn on a larger dojox.gfx.Surface than the default 
    //          one that the gauge will create for itself unless otherwise 
    //          specified.
    
    // styler: Object
    //          Instance of dojox.widget.AnalogGauge2._Style, it will be interrogated 
    //          to find out which functions should be called to draw the gauge 
    //          correctly.  
    styler: null,
    
    // store: Object
    //          Implementation of the Dojo Data Read API, a "data store".  If it 
    //          also implements the notification data api, we will ignore 
    //          "pollInterval" and just wait for notifications of changes to 
    //          "storeItem"
    store: null,
    
    // storeItem: Object
    //          Item object from the "store".
    storeItem: null,
    
    // storeAttribute: String
    //          Name of the attribute we should use
    storeAttribute: "",
    
    // pollInterval: Integer
    //          Number of milliseconds between each attempt to poll the data 
    //          store for updates to the value.  If zero, we will not poll. 
    //          TODO: Write an update() method.
    pollInterval: 5000,
    
    // width: Integer
    //          Width of the gauge, in pixels.
    width: 120,
    
    // height: Integer
    //          Height of the gauge, in pixels.
    height:110,
    
    // surface: Object
    //          Instance of dojox.gfx.Surface, used to draw the gauge onto.  If 
    //          unspecified on creation of the gauge, we will create a new one 
    //          and associate it with the domNode assigned to the widget.
    surface: null,
    
    // minValue: Number
    //          Minimum displayable value for the gauge.
    minValue: 0,
    
    // maxValue: Number
    //          Maximum displayable value for the gauge.
    maxValue: 180,

    
	//  The field is like the 
    //        dojo.number.__FormatOptions
    //   eg: dojo.number.format(10, {places:2})  produces "10.00"
    //
	// In this case you would set 
	//      formatOptions: {places:2}
	//
    // More complicated can be used such as pattern
    //
    //   eg:  dojo.number.format(10, {pattern:"00000"}) produces  "00010" using the Unicode CLDR specification
    //
	// In this case you would set
	//     formatOptions: {pattern:"00000"}
	//
    // In this case to produce $57.40 from setValue(57.4) the options wouldbe
    //
	//     formatOptions: {symbol: "$",pattern: "\u00A4####", places: 2},
    //
    //
	// use null for no formating the number which is defaulted to setValue. you can use {places:0} for Floor
	formatOptions: null,


    // displayText: String
	//
	displayText: null,

    // units: String
    //          Units for the metric the gauge is displaying. (Will be passed to 
    //          the _Style at render-time, in case it wishes to show this)
    units: "Feet",
    
    // majorTickSeparation: Number
    //          Distance between each major tick on the gauge. (in the same 
    //          units as the display metric)
    majorTickSeparation: 20,
    
    // minorTickSeparation: Number
    //          Distance between each minor tick on the gauge. (in the same 
    //          units as the display metric)
    minorTickSeparation: 5,
    
    // trendValue: Number
    //          Trend value for the gauge, this is a historical average number 
    //          for the gauge's metric over time.  Can be average of values 
    //          since creation of the widget, or truely historical average (set 
    //          on widget creation).
    trendValue: 0,
    
    // ceilingValue: Number
    //
    ceilingValue: 0,    
    
    // leap: Boolean
    //          Whether or not we should cause the gauge to leap when the value 
    //          crosses the leapValue point.
    leap: false,
    
    // background: String or Object
    //          Background color/gradient for the gauge.  This is only used in 
    //          the default dojox.gfx.Surface case where the gauge has its own 
    //          surface. This background color will be drawn first (and 
    //          therefore behind) the shapes and groups drawn by the gauge's 
    //          _Style.
    background: "none",
    
    // leapfuncname: String
    //          Name of the function to call when the gauge leaps. 
    //          TODO: this should instead be an event that can be bound-to using 
    //          myGauge.connect("leap", myFunc);
    leapfuncname: "undefined",
    
    // attentionOnClick: Boolean
    //          Whether or not to "leap" when clicked.
    attentionOnClick: false,

    //private
    _currentValue: 0,
    _originWidth: 120,
    _originHeight: 110,
    _savecolor: {gcolor: 'black', major: 'none', minor: 'none', ncolor: 'none'},
    _reset: false,   //reset the gauge after a leap
    _leaped: false,      //the gauge has leaped
    
    // templateString: String
    //          dijit._Templated template string.
    //          TODO: put an accessible paragraph/text stuff in here (css 
    //          shifted off the page) with the value, units, and label (if 
    //          necessary)
    templateString:'<div class="dojotSimpleGauge" dojoAttachPoint="containerNode"></div>',
    
    constructor: function(params, domNode){
        // summary:
        //          Constructor for the AnalogGauge2 widget.  This is called 
        //          when the user creates a new instance of 
        //          dojox.widget.AnalogGauge2.
        // description:
        //          We'll set up arrays here.
        // params: Object
        //          Parameters for the widget.
        // domNode: Object or String
        //          Object or String (ID string) of the dom node to associate 
        //          with the widget.
        // tags:
        //          extension
        this._domStyle = dojo.attr(domNode, "style");
    },
    
    postMixInProperties: function(){
        // summary:
        //          Run after the properties have been mixed into the object.  
        //          This is like the second half of the constructor.  We'll wait 
        //          until this function is called to do any initialization based 
        //          on the input parameters.
        // tags:
        //          extension
        
        if(this.styler == null){
            this.styler = new dojox.widget.AnalogGauge2.ReallySimpleStyle();
        }
        
        
        // If we set a store, let's take advantage of it.
        if(this.store != null){
            // If the user didn't set a storeItem or a storeAttribute, throw an 
            //  error.
            this.setStore(this.store, this.storeItem, this.storeAttribute);
        }
        
        if (this.background=='default'){
            this.background={
                'type': 'linear',
                'x1': 0,
                'x2': 0,
                'y2': 0,
                'y1': 100,
                'colors': [
                    {
                        offset: 0, 
                        color: "#9090EC"
                    },{
                        offset: 1, 
                        color: "white"
                    }
                ]
            };
        }
    },
    
    postCreate: function(){
        // summary:
        //          Finalizer on the constructor...
        // tags:
        //          extension
        
        this.inherited("postCreate", arguments);
    },
    
    startup: function(){
        // summary:
        //          More initialization, called after the template has been 
        //          applied and used to initiate things like the dojox.gfx 
        //          drawing.
        // tags:
        //          extension
        var thresholds=[];

        if (this.getChildren && this.thresholds == null ){
            // Handle the markup case where we'll let the user specify 
            //  thresholds and style as sub-nodes.
            this.thresholds = [];
            var me = this;
            
            dojo.forEach(this.getChildren(), function(child){
                child.startup();
                if(/\.AnalogGaugeThreshold/.test(child.declaredClass)){
                    thresholds.push(child);
                } else if(/\._Style/.test(child.constructor.superclass.declaredClass)){
                    me.styler = child;
                }
            });
            this.thresholds= thresholds;
        }
        if (typeof(this.thresholds)!='undefined')
          this.thresholds = this.thresholds.sort(function(a, b){
            return a.start - b.start;
          });
          
        // Fix for defect 14114 - If thresholds are defined and there is not one
        // that starts at begin range for the gauge, the gauge just defaults to the 1st
        // threshold defined. If for instance it is critical, the gauge will always be
        // showing critical status.  Fix for beta is to define "Normal" for everything
        // up to that point and that can be done right here and only here.  After beta
        // revist and update each style to have a "default" threshold which is default
        // color for gauge and used instead of normal.
        if (this.thresholds && this.thresholds.length > 0) {
            console.log("Checking thresholds to make sure 1st one starts at minValue");
            // Recall we just sorted thresholds
            if (this.thresholds[0].start > this.minValue) {
                console.log("First threshold does NOT start at minValue!");
                this.thresholds.push({start: this.minValue, status: "Normal", name: "Normal"});
                // And just sort again to guarantee correct order
                this.thresholds = this.thresholds.sort(function(a, b){
                    return a.start - b.start;
                })
            }
        }   

        this.domNode.style.width = this.width+"px";
        this.domNode.style.height = this.height+"px";    
        
        if(this._domStyle && !dojo.isString(this._domStyle)){
            for(var s in this.domNode.style){
                if(s == "font"){
                    continue; // for some reason we have to skip the 'font' style in IE.
                }
                this.domNode.style[s] = this._domStyle[s];
            }
        } else {
            dojo.attr(this.domNode, "style", this._domStyle);
        }
        
        this.init();
    },
    
    destroy: function(){
        // summary:
        //          Called when the widget is destroyed by dojo, used to clean 
        //          up memory before disassociating the widget from the dom.
    },
    
    init: function(){
        // summary:
        //          Builds out the gauge initially. (hence the name!)
        // description:
        //          Creates the surface and draws the gauge initially... to 
        //          whatever value we're located at.
        // divId:
        //          ID of the div where we're building our gauge.

        // set up the dimensions and save the current dimensions so we can return to them if we need to
        this.cx = this.width/2;
        this.cy = this.height/2;
        this._originWidth=this.width;
        this._originHeight=this.height;
        this.hasAttention = false;
        
        this.draw(); // TODO: support external surface reference...
        
        this.setValue(this._currentValue);
        
        var me = this;     
        
        dojo.connect(this.domNode, "onclick", function(){
            if(me.hasAttention || me.attentionOnClick){
				me.attention();  
            } 
        });
    },
    
    _animateScaleChange: function(startScale, endScale, fns){
        // summary:
        //          Resizes the whole shebang, animated.
        // startScale: Number
        //          Starting scale to resize from.
        // endScale: Number
        //          Ending scale to resize to.
		if(startScale == endScale){
            return;
        }
        
        var me = this;
        
        var group = this._masterGroup;
        var minValue = this.minValue;
        var maxValue = this.maxValue;
        var startValue = this.getValue();
        var endValue = this.getValue();
        var	formatOptions= this.formatOptions;
        var	displayText= this.displayText;
		    var startTrendValue = this.trendValue;
        var endTrendValue = this.trendValue;
        var startCeilingValue = this.ceilingValue;
        var endCeilingValue = this.ceilingValue;        
        var units = this.units;
        var thresholds = this.thresholds;
        var minorTickSeparation = this.minorTickSeparation;
        var majorTickSeparation = this.majorTickSeparation;
        
        new dojo._Animation({
            curve: [0, 1],
            duration: this.duration | 350,
            easing: dojo.fx.easing.expoIn,
            onBegin: function(){
            },
            onAnimate: function(step){
                var percent = step;
                var currentScale = (endScale - startScale)*percent + startScale;
				me.adjust({
                    width: me._originWidth * currentScale, 
                    height: me._originHeight * currentScale 
                });

                for(var i = 0, len = fns.length; i < len; i++){
                    me.styler[fns[i]]({
                        group: group,
                        minValue: minValue,
                        maxValue: maxValue,
                        startValue: startValue,
                        formatOptions: formatOptions,
                        displayText: displayText,
						            endValue: endValue,
						            startTrendValue: startTrendValue,
                        endTrendValue: endTrendValue,
                        units: units,
                        thresholds: thresholds,
                        minorTickSeparation: minorTickSeparation,
                        majorTickSeparation: majorTickSeparation,
                        width: me.width,
                        height: me.height,
                        startCeilingValue: startCeilingValue,
                        endCeilingValue: endCeilingValue
                    });
                }
            },
            onEnd: function(){
                dojo.style(me.domNode, "width", (endScale*me._originWidth) + "px");
                dojo.style(me.domNode, "height", (endScale*me._originHeight) + "px");
                
                if(endScale == 1) {
                    dojo.style(me.domNode, "zIndex", 1); // TODO: should it be 1? or should it be something relative to the original?
                }
            }
        }).play();
    },

    attention: function(size){
        // summary:
        //          Responsible for implementation of the leap idea, this 
        //          function calls the "attention" draw semantic functions 
        //          for the gauge style.
        
		if(arguments.length == 0 && !this.hasAttention){
            size = 2.0;
            
            if(this.leapMagnify){
                size = this.leapMagnify;
            }
        }
        
        var attentionFunctions = [];
        var semantics= this.styler.getDrawSemantics();
        if(this.hasAttention){
            dojo.style(this.domNode, "zIndex", 1);
            
            if(semantics.attentionDown){
                attentionFunctions = dojo.isArray(semantics.attentionDown) ? semantics.attentionDown : [semantics.attentionDown];
            }
            
            this.hasAttention = false;
            this._animateScaleChange(this.width / this._originWidth, 1.0, attentionFunctions);
        } else if(!this.hasAttention) {
            dojo.style(this.domNode, "zIndex", 999);
            
            if(semantics.attentionDown){
                attentionFunctions = dojo.isArray(semantics.attentionUp) ? semantics.attentionUp : [semantics.attentionUp];
            }
            
            this.hasAttention = true;
            this._animateScaleChange(1.0, size, attentionFunctions);
        }
    },


	adjust: function(opt){
        // summary:
        //          This function is used to adjust any of the attributes of the 
        //          gauge. It could be abstracted further to provide resize 
        //          capability, for example.
        // opt: Object
        //          Object containing key/value pairs to mix into the gauge.
        // examples:
        //          gauge.adjust({
        //              width:value, 
        //              height:value, 
        //              radius:value, 
        //              gcolor:"color", 
        //              label: "mylabel"
        //          });
        dojo.mixin(this, opt);
        this.styler.reset();
        this.styler.adjust(opt);
        // adjust center x and center y locations if the width/height changed.. 
        this.cx = Math.floor(this.width/2);
        this.cy = Math.floor(this.height/2);
        
        // adjust the outer DIV
        this.domNode.style.width = this.width+"px";
        this.domNode.style.height = this.height+"px";
        
        if(!this._sharesSurface){ // TODO: Implement surface sharing in the constructor.......
			this.surface.setDimensions(this.width, this.height);
		}
        this.draw();        
    },

    _groups: [],
    _masterGroup: null,
    draw: function(surface){
        // summary: 
        //          Renamed from "createSurface", the "draw" function is 
        //          responsible for drawing the gauge onto the graphics surface.
        // description:
        //          This function uses the dojox.widget.gauge._Style interface 
        //          of it's 'style' attribute in order to correctly draw the 
        //          gauge onto a surface.  
        //
        //          It can either be passed a valid surface object (from a 
        //          larger dojox.gfx canvas application) or can create its own 
        //          surface object from the root dom node the widget was 
        //          assigned on construction.
        //          
        //          If the widget already has a surface assigned to it, it will 
        //          clear that surface of any graphics associated with the gauge 
        //          and re-draw everything.
        // surface: Object 
        //          Optional. A dojox.gfx.Surface object on which to draw the 
        //          gauge.
        
        // check the argument..
        if(surface != null && surface.declaredClass == "dojox.gfx.Surface"){
            this.surface = surface;
        } else if (surface != null){
            throw new Error("AnalogGauge2.draw() must be passed either zero arguments, or a valid dojox.gfx.Surface object.");
        } 
        
        var callback = dojo.hitch(this, function(){
            var surface = this.surface;
            
            this._masterGroup = surface.createGroup();
        
            // draw the background if we're supposed to..
            if (this.background != 'none') {
                var shape = this._masterGroup.createRect({x: -50, y: -50, width: 100, height: this.height / this.width * 100 });
                shape.setFill(this.background);
            }
    
            var drawFunctions = this.styler.getDrawOrder();
            this._groups = [];
            for(var i = 0, len = drawFunctions.length; i < len; i++){
                this._groups.push(this.styler[drawFunctions[i]]({
                    group: this._masterGroup,
                    minValue: this.minValue,
                    maxValue: this.maxValue,
                    startValue: 0,
                    formatOptions: this.formatOptions,
                    displayText: this.displayText,
					          endValue: this._currentValue,
                    startTrendValue: 0,
                    endTrendValue: this.trendValue,
                    units: this.units,
                    thresholds: this.thresholds,
                    minorTickSeparation: this.minorTickSeparation,
                    majorTickSeparation: this.majorTickSeparation,
                    width: this.width,
                    height: this.height,
                    startCeilingValue: 0,
                    endCeilingValue: this.ceilingValue                 
                }));
            }
            
//           var scaler = this.width > this.height ? this.height / 105.0 : this.width / 105.0;
           var scaler = this.width > this.height ? this.height / 100.0 : this.width / 100.0;
//scaler=1.0;
			if (this.styler.dir=='rtl')
			  this._masterGroup.setTransform({xx: -scaler, yy: scaler, dx: this.cx, dy: this.cy});
            else
			this._masterGroup.setTransform({xx: scaler, yy: scaler, dx: this.cx, dy: this.cy});

	   });

        // If we don't have a surface to draw to, create one.
        if (this.surface == null){
            this.surface = dojox.gfx.createSurface(this.domNode, this.width, this.height);
			this.surface.whenLoaded(callback);
        } else {
            this.surface.remove(this._masterGroup); // TODO: THIS IS WRONG for shared surfaces...
            delete this._masterGroup;
            this._masterGroup = null;
            this.styler.reset();
			this.surface.setDimensions(this.width,this.height);
            
            // call the callback manually, since "whenLoaded" won't get fired again on a surface taht already exists...
            callback();
        }
    },
    
    getValue: function(){
        // summary: 
        //          Gets the current value for the gauge.
        // returns:
        //          Current value for the gauge.
        return this._currentValue;
    },
    
    setValue: function(/*number*/ value){
        // summary:
        //          Animates the arm/needle to the specified value.
        // value: (Number)
        //          Value to which the needle should be moved.
        
        // Update the needle
        var oldValue = this._currentValue;
        this._currentValue = parseFloat(value);
        
        if(this._masterGroup != null){
            var drawSemantics = this.styler.getDrawSemantics();
                
            var updateFunctions = dojo.isArray(drawSemantics.updateValue) ? drawSemantics.updateValue : [drawSemantics.updateValue];
            for(var i = 0, len = updateFunctions.length; i < len; i++){
                this.styler[updateFunctions[i]]({
                    group: this._masterGroup,
                    minValue: this.minValue,
                    maxValue: this.maxValue,
                    startValue: oldValue,
                    formatOptions: this.formatOptions,
                    displayText: this.displayText,
					          endValue: value,
                    startTrendValue: this.trendValue,
                    endTrendValue: this.trendValue,
                    units: this.units,
                    thresholds: this.thresholds,
                    minorTickSeparation: this.minorTickSeparation,
                    majorTickSeparation: this.majorTickSeparation,
                    width: this.width,
                    height: this.height,
                    startCeilingValue: this.ceilingValue,
                    endCeilingValue: this.ceilingValue
                });
            }
            
            if((this.leap) && (value >= this.leapValue) && !this.hasAttention){
                this._reset = true;
                if (this.leapMagnify == 1){
                    this.attention(2.0);
                } else {
                    this.attention(this.leapMagnify | 2.0);
                }
            }
        }
    },
    
    setStore: function(store, item, attribute){
        // summary: 
        //          Sets the store information.
        // description:
        //          Sets the attributes required to associate the gauge with a 
        //          store.  It will also call setStoreItem, using the item and 
        //          attribute arguments as well - to kick off notification event 
        //          listening, polling, etc.
        // store: Object
        //          A dojo data Read api-implementor. (can/should implement 
        //          Notification as well)
        // item: Object
        //          A valid item object from the store.
        // attribute: String
        //          Name of the attribute we want to display about the item. 
        //          store.getValue(item,attribute) will be the value displayed 
        //          in the gauge.
        
        this.store = store;
        this.storeItem = item;
        this.storeAttribute = attribute;
    
        if(this.storeItem == null || this.storeAttribute == null){
            throw new Error("A data store was set, but either a storeItem or a storeAttribute wasn't specified.");
        }
        
        var features = this.store.getFeatures();
        
        if(!features['dojo.data.api.Read'] || !features['dojo.data.api.Identity']){
            throw new Error("All data stores associated with a gauge must support at least the Read and Identity data APIs");
        }

        // set up the item/attribute
        this.setStoreItem(this.storeItem, this.storeAttribute);
    },
     
    _notificationConnection: null,
    _pollingTimer: null,
    setStoreItem: function(item, attribute){
        // summary:
        //          Sets the item and attribute that the gauge should display.
        // description:
        //          Sets the store item, resets the notification event listeners 
        //          and timers.
        // item: Object
        //          A valid item object from the store.
        // attribute: String
        //          Name of the attribute we want to display about the item. 
        //          store.getValue(item,attribute) will be the value displayed 
        //          in the gauge.
        
        this.storeItem = item;
        this.storeAttribute = attribute;
        
        // get the identity of the storeItem
        var itemIdentity = this.store.getIdentity(item);
        
        // Set the value initially
        this.setValue(this.store.getValue(this.storeItem, this.storeAttribute));
        
        // disconnect the notification connection and cancel the timer, if we have them.
        this._cancelStoreUpdates();
        
        // If the store implements the notification API, set it up to listen
        var features = this.store.getFeatures();
        
        if(features['dojo.data.api.Notification'] === true){
            // Setup event handler.
            this._notificationConnection = this.connect(this.store, "onSet", 
                dojo.hitch(this, function(item, attribute, oldValue, newValue){
                    if(this.store.getIdentity(item).toString() === itemIdentity.toString()){
                        this.setValue(newValue);
                    }
                })
            );
        } else if(this.pollInterval > 0) {
            // Poll for changes...
            this._pollingTimer = setInterval(dojo.hitch(this, function(){
                this.setValue(this.store.getValue(this.storeItem, this.storeAttribute));
            }), this.pollInterval);
        } else {
            // don't poll or listen for events.
        }
    },
    
    _cancelStoreUpdates: function(){
        // summary:
        //          Cancels the timers and event handlers that watch for changes 
        //          to the data store items.
        
        if(this._pollingTimer != null){
            clearInterval(this._pollingTimer);
        }
        this._pollingTimer = null;
        
        if(this._notificationConnection != null){
            this.disconnect(this._notificationConnection);
        }
        this._notificationConnection = null;
    }
});