dojo.provide("dojox.widget.AnalogGauge2.AnalogStyle");

dojo.require("dojo.fx");
dojo.require("dojo.fx.easing");
dojo.require("dojox.gfx");
dojo.require("dojox.gfx.matrix");
dojo.require("dojox.widget.AnalogGauge2._Style");
dojo.require("dojox.widget.AnalogGauge2"); // for the AnalogGaugeThreshold

dojo.declare("dojox.widget.AnalogGauge2.AnalogStyle", [dojox.widget.AnalogGauge2._Style], {

    /** colorMap: Object
     *          Mapping from dojox.widget.AnalogGaugeThreshold.***_STATUS values 
     *         to colors.
	 */
        colorMap: null,
	/**
     *  Gauge color
     */
	gcolor: "#79818b",

	trendColor: "#ffffff",
    needleColor: "#be1e2d",
    ceilingColor: "#be1e2d",

    constructor: function(args){
        dojo.mixin(this, args);
        // if the colorMap wasn't set, use a default one.
        if(this.colorMap == null){
            this.colorMap = {};
            this.colorMap[dojox.widget.AnalogGaugeThreshold.CLEAR_STATUS         ]   = "#33cc14";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.NORMAL_STATUS        ]   = "green";//#33cc14";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.CRITICAL_STATUS      ]   = "red"; //#e51717";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.FATAL_STATUS         ]   = "#222222";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.MAJOR_STATUS         ]   = "#FFB31A";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.MINOR_STATUS         ]   = "#ffff1a";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.INDETERMINANT_STATUS ]   = "#2EA8E5";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.NOT_APPLICABLE_STATUS]   = "#6963A6";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.UNKNOWN_STATUS       ]   = "#A6A6A6";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.UNDEFINED_STATUS     ]   = "#9294f3";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.INFORMATIONAL_STATUS ]   = "#2ea8e5";
        }
	},


	valueFormatter: function(value){
        return value+"";
    },
    
    getDrawOrder: function(){
        return ["drawBackground", "drawOdometerBacking", "drawOdometerText", "drawHighlight", "drawTicks", "drawBorder", "drawTrendIndicator", "drawNeedle", "drawCeilingIndicator", "drawNubbin"];
    },
    
    getDrawSemantics: function(){
        return {
            background: ["drawBackground", "drawOdometerBacking", "drawBorder"],
            ticks: ["drawTicks"],
            needle: "drawNeedle",
            foreground: ["drawHighlight"],
            updateValue: ["drawBackground", "drawTrendIndicator", "drawOdometerText", "drawNeedle", "drawCeilingIndicator"]
        };
    },
        
    reset: function(){
        // summary:
        //          Deletes any locally-stored shape objects when the gauge 
        //          needs to be reset.
        delete this._backGroup;
        this._backGroup=null;

		delete this._needleGroup;
        this._needleGroup = null;
        delete this._ceilingGroup;
        this._ceilingGroup = null;
        delete this._trendGroup;
        this._trendGroup = null;
        delete this._odometerTextGroup;
        this._odometerText = null;
    },
    
	/**
	 * Note you can use the word rainbow for  threshold color
	 */
	_getColor: function(state){
		  if (state.thresholds.length==0)
			  return this.gcolor;
		  var c=this.getThresholdFromValue(state.endValue, state.thresholds);
		  return this.colorMap[c.name];
	},

    _backGroup: null,
	drawBackground: function(state){
        if(this._backGroup === null){
           this._backGroup = state.group.createGroup();
		}
       this._backGroup.clear();

        var face = this._backGroup.createCircle({cx: 0, cy: 0, r: 50});
        var c=this._getColor(state);
		face.setStroke(null).setFill({type: "linear", x1: 0, y1: -50, x2: 0, y2: 50, 
//			colors: [{offset: 0, color: "#79818b"}, {offset: 1, color: "#97a3b3"}]});
			colors: [{offset: 0, color: c}, {offset: 1, color: "#97a3b3"}]});


        return this._backgroup;
    },
    
    drawTicks: function(state){
        var result = state.group.createGroup();
        
        var startVal = state.minValue;
        var endVal = state.maxValue;
        var startAngle = 315;
        var endAngle = 45;
        var minorTickInterval = state.minorTickSeparation;
        var majorTickInterval = state.majorTickSeparation;
        
        var outerRadius = 47;
        var minorTickInnerRadius = 41;
        var majorTickInnerRadius = 35;
        var textRadius = 30;
        
        // draw the minor ticks.
        for(var i = startVal; i <= endVal; i = i+minorTickInterval){
            var radians = this._getRadianForValue(i, startVal, endVal, startAngle, endAngle);
            
            var tick = result.createLine({
                x1: minorTickInnerRadius * Math.sin(radians),
                y1: minorTickInnerRadius * Math.cos(radians),
                x2: outerRadius * Math.sin(radians),
                y2: outerRadius * Math.cos(radians)
            });
            tick.setStroke({width: 0.3, color: "#363636"}).setFill(null);
            
            // draw the start and end labels
			var st= ""+i;
			if (this.dir=='rtl') {
              st=""+ endVal-i;
			}

			if(i == startVal || i == endVal){
                var text = result.createText({
                    x: textRadius * Math.sin(radians),
                    y: textRadius * Math.cos(radians) + 2,
                    text: st,
                    align: "middle"
                });
                text.setFont({
                    size: "4px",
                    family: "Verdana"
                }).setFill("#363636").setStroke(null);
                if (this.dir=='rtl') 
                   text.setTransform(dojox.gfx.matrix.flipX);
			}
        }
        
        // draw the major ticks and the labels

		for(var i = startVal+majorTickInterval; i < endVal; i = i+majorTickInterval){
            var radians = this._getRadianForValue(i, startVal, endVal, startAngle, endAngle);
            
            var tick = result.createLine({
                x1: majorTickInnerRadius * Math.sin(radians),
                y1: majorTickInnerRadius * Math.cos(radians),
                x2: outerRadius * Math.sin(radians),
                y2: outerRadius * Math.cos(radians)
            });
            tick.setStroke({width: 0.3, color: "#363636"}).setFill(null);
            
            // draw the labels
            var st= ""+i;
			if (this.dir=='rtl') {
              st=""+ endVal-i;
			}

			var text = result.createText({
                x: textRadius * Math.sin(radians),
                y: textRadius * Math.cos(radians) + 2,
                text: st,
                align: "middle"
            });
            text.setFont({
                size: "4px",
                family: "Verdana"
            }).setFill("#363636").setStroke(null);
            if (this.dir=='rtl') 
            	text.setTransform(dojox.gfx.matrix.flipX);
		}
        
        
        return result;
    },
    
    drawBorder: function(state){
        var result = state.group.createGroup();
        
        var border = result.createImage({
            x: -50, y: -50, 
            width: 100, height: 100, 
            src: dojo.moduleUrl("dojox.widget.AnalogGauge2.AnalogStyle", "default_border.png")
        });
        
        return result;
    },
    
    drawHighlight: function(state){
        var result = state.group.createGroup();
        
        var highlight = result.createImage({
            x: -50, y: -50, 
            width: 100, height: 100, 
            src: dojo.moduleUrl("dojox.widget.AnalogGauge2.AnalogStyle", "default_light.png")
        });
        
        return result;
    },
    
    drawOdometerBacking: function(state){
        var result = state.group.createGroup();
        
        var odometerBacking = result.createImage({
            x: -50, y: -50, 
            width: 100, height: 100, 
            src: dojo.moduleUrl("dojox.widget.AnalogGauge2.AnalogStyle", "default_odometer.png")
        });
        
        return result;
    },
    
    _odometerTextGroup: null,
    _odometerText: null,
    drawOdometerText: function(state){
        var fontSize = "5px";
        var align = "end";
        var x = 14;
        var y = 33.5;
    
        if(this._odometerTextGroup === null){
            this._odometerTextGroup = state.group.createGroup();
            
            this._odometerText = this._odometerTextGroup.createText({
                x: x, y: y, 
                text: this.valueFormatter(state.endValue) + "" + state.units, 
                align: align
            }).setFill("#000000");
            this._odometerText.setFont({
                size: fontSize, family: "Verdana, sans-serif"
            });
            if (this.dir=='rtl') 
                this._odometerText.setTransform(dojox.gfx.matrix.flipX);
		}
        
        
        // Animate it! (if it's an integer value to an integer value... decimals look weird)
        if(this.dontAnimate){
			this._odometerText.setShape({
                x: x, y: y, 
                text: this.valueFormatter(state.startValue), 
                align: align
            });
            new dojo._Animation({
                curve: [0, 1], 
                duration: 1000, 
                easing: dojo.fx.easing.quadratic,
                onAnimate: dojo.hitch(this, function(step){
                    var percent = step;
                    var currentValue = Math.floor(percent * (state.endValue - state.startValue) + state.startValue);
                    if(this._odometerText != null){
						if (step==1) {
                           currentValue = state.endValue;
						   if (state.formatOptions!=null) 
                             currentValue=    dojo.number.format(currentValue, state.formatOptions);
						}
						this._odometerText.setShape({
                            x: x, y: y, 
                            text: this.valueFormatter(currentValue) + "" + state.units, 
                            align: align
                        });
                    }
                })
            }).play();
        } else {
			var currentValue = state.endValue;
			if (state.formatOptions!=null) 
                  currentValue=    dojo.number.format(state.endValue, state.formatOptions);
			this._odometerText.setShape({
                x: x, y: y, 
                text: currentValue + "" + state.units, 
                align: align
            });
        }
        
        return this._odometerTextGroup;
    },
    
    _getRadianForValue: function(value, minValue, maxValue, minAngle, maxAngle){
        var minRadians = minAngle * Math.PI / 180.0;
        var maxRadians = maxAngle * Math.PI / 180.0;
        var radianRange = maxRadians - minRadians;
        
        return minRadians + radianRange * ((value - minValue) / (maxValue - minValue));
    },
    
    _getRadianBounds: function(startValue, endValue, minValue, maxValue, minAngle, maxAngle){
        var result = {};
        result.start = this._getRadianForValue(startValue, minValue, maxValue, minAngle, maxAngle);
        result.end = this._getRadianForValue(endValue, minValue, maxValue, minAngle, maxAngle);
        return result;
    },
    
    _needleGroup: null,
    drawNeedle: function(state){
        if(this._needleGroup === null){
            this._needleGroup = state.group.createGroup();
            
            var pointerShadow = this._needleGroup.createPath();
            pointerShadow.moveTo({x: -1.0, y: -11.667}).lineTo({x: 0.5, y: -36.433 - 11.667}).lineTo({x: 2.0, y: -11.67}).lineTo({x: -1.0, y: -11.667});
            pointerShadow.setStroke(null).setFill("rgba(0,0,0,0.5)");
            
            var pointer = this._needleGroup.createPath();
            pointer.moveTo({x: -1.5, y: -11.667}).lineTo({x: 0, y: -36.433 - 11.667}).lineTo({x: 1.5, y: -11.67}).lineTo({x: -1.5, y: -11.667});
            pointer.setStroke(null).setFill(this.needleColor);
            
            
            var baseShadow = this._needleGroup.createRect({x: -1.0, y: -11.222, width: 3.0, height: 11.222});
            baseShadow.setStroke(null).setFill("rgba(0,0,0,0.5)");
            
            var base = this._needleGroup.createRect({x: -1.5, y: -11.222, width: 3.0, height: 11.222});
            base.setStroke(null).setFill("#414042");
        }
        
        var radianBounds = this._getRadianBounds(state.startValue, state.endValue, state.minValue, state.maxValue, -135, 135);
        
        var startRadians = radianBounds.start;
        var endRadians = radianBounds.end;
        
        this._needleGroup.setTransform(dojox.gfx.matrix.rotate(startRadians));
        
        if(!this.dontAnimate){
            new dojo._Animation({
                curve: [0, 1], 
                duration: 1000, 
                easing: dojo.fx.easing.quadratic,
                onAnimate: dojo.hitch(this, function(step){
                    var percent = step;
                    var currentRadians = percent * (endRadians - startRadians) + startRadians;
                    if(this._needleGroup){
                        this._needleGroup.setTransform(dojox.gfx.matrix.rotate(currentRadians));
                    }
                })
            }).play();
        } else {
            this._needleGroup.setTransform(dojox.gfx.matrix.rotate(endRadians));
        }
        
        return this._needleGroup;
    },
    
    _ceilingGroup: null,
    drawCeilingIndicator: function(state){
        if(this._ceilingGroup === null){
            this._ceilingGroup = state.group.createGroup();
            
            var ballShadow = this._ceilingGroup.createCircle({cx: 0, cy: 0.5, r: 3.33});
            ballShadow.setStroke(null).setFill("rgba(0,0,0,0.5)");
            
            var ball = this._ceilingGroup.createCircle({cx: 0, cy: 0.0, r: 3.33});
            ball.setStroke(null).setFill(this.ceilingColor);
            
            var ballHighlight = this._ceilingGroup.createEllipse({cx: 0, cy: -0.5, rx: 2.67, ry: 2.266});
            ballHighlight.setStroke(null).setFill({type: "linear", x1: 0, y1: -2.7, x2: 0, y2: 1.7, colors: [{offset: 0, color: "rgba(209,211,212,1.0)"}, {offset: 1, color: "rgba(209,211,212,0.0)"}]});
        }
        
        //if(!state.startCeilingValue || !state.endCeilingValue){        
        // Line above causes problems if startCeilingValue is 0 which is very possible!
        if(!state.endCeilingValue){
            this._ceilingGroup.clear();
            return this._ceilingGroup;
        }
        
        var radianBounds = this._getRadianBounds(state.startCeilingValue, state.endCeilingValue, state.minValue, state.maxValue, 315, 45);
        
        var startRadians = radianBounds.start;
        var endRadians = radianBounds.end;
        
        function generateMatrix(radians){
            return {xx: 1, xy: 0, yx: 0, yy: 1, dy: 42 * Math.cos(radians), dx: 42 * Math.sin(radians)};
        }
        
        this._ceilingGroup.setTransform(generateMatrix(startRadians));
        
        if(!this.dontAnimate){
            // Animate it!
            new dojo._Animation({
                curve: [0, 1], 
                duration: 1000, 
                easing: dojo.fx.easing.quadratic,
                onAnimate: dojo.hitch(this, function(step){
                    var percent = step;
                    var currentRadians = percent * (endRadians - startRadians) + startRadians;
                    
                    if(this._ceilingGroup){
                        this._ceilingGroup.setTransform(generateMatrix(currentRadians));
                    }
                })
            }).play();
        } else {
            this._ceilingGroup.setTransform(generateMatrix(endRadians));
        }
        
        return this._ceilingGroup;
    },
    
    _trendGroup: null,
    drawTrendIndicator: function(state){
        if(this._trendGroup === null){
            this._trendGroup = state.group.createGroup();
            
            var pointerShadow = this._trendGroup.createPath();
            pointerShadow.moveTo({x: -1.0, y: -11.667}).lineTo({x: 0.5, y: -36.433 - 11.667}).lineTo({x: 2.0, y: -11.67}).lineTo({x: -1.0, y: -11.667});
            pointerShadow.setStroke(null).setFill("rgba(0,0,0,0.5)");
            
            var pointer = this._trendGroup.createPath();
            pointer.moveTo({x: -1.5, y: -11.667}).lineTo({x: 0, y: -36.433 - 11.667}).lineTo({x: 1.5, y: -11.67}).lineTo({x: -1.5, y: -11.667});
            pointer.setStroke(null).setFill(this.trendColor);
            
            
            var baseShadow = this._trendGroup.createRect({x: -1.0, y: -11.222, width: 3.0, height: 11.222});
            baseShadow.setStroke(null).setFill("rgba(0,0,0,0.5)");
            
            var base = this._trendGroup.createRect({x: -1.5, y: -11.222, width: 3.0, height: 11.222});
            base.setStroke(null).setFill("#414042");
        }
        
        //if(!state.startTrendValue || !state.endTrendValue){
        // Line above causes problems if startTrendValue is 0 which is very possible!
        if(!state.endTrendValue){
            this._trendGroup.clear();
            return this._trendGroup;
        }
        
        var radianBounds = this._getRadianBounds(state.startTrendValue, state.endTrendValue, state.minValue, state.maxValue, -135, 135);
        
        var startRadians = radianBounds.start;
        var endRadians = radianBounds.end;
        
        this._trendGroup.setTransform(dojox.gfx.matrix.rotate(startRadians));
        
        if(!this.dontAnimate){
            new dojo._Animation({
                curve: [0, 1], 
                duration: 1000, 
                easing: dojo.fx.easing.quadratic,
                onAnimate: dojo.hitch(this, function(step){
                    var percent = step;
                    var currentRadians = percent * (endRadians - startRadians) + startRadians;
                    
                    if(this._trendGroup){
                        this._trendGroup.setTransform(dojox.gfx.matrix.rotate(currentRadians));
                    }
                })
            }).play();
        } else {
            this._trendGroup.setTransform(dojox.gfx.matrix.rotate(endRadians));
        }
        
        return this._trendGroup;
    },
    
    drawNubbin: function(state){
        var result = state.group.createGroup();
        
        var shadow = result.createCircle({cx: 0, cy: 1, r: 6.666});
        shadow.setStroke(null).setFill("rgba(65,64,66,0.5)");;
        
        var nubbin = result.createCircle({cx: 0, cy: 0, r: 6.666});
        nubbin.setStroke(null).setFill("#414042");
        
        var highlight = result.createEllipse({cx: 0, cy: -1.4, rx: 5.6, ry: 4.6});
        highlight.setStroke(null).setFill({type: "linear", x1: 0, y1: -6, x2: 0, y2: 4, colors: [{offset: 0, color: "rgba(209,211,212,1.0)"}, {offset: 1, color: "#414042"}]});
        
        return result;
    }
});