dojo.provide("dojox.widget.AnalogGauge2.RingStyle");

dojo.require("dojo.fx");
dojo.require("dojo.fx.easing");
dojo.require("dojox.widget.AnalogGauge2._Style");
dojo.require("dojox.widget.AnalogGauge2"); // for the AnalogGaugeThreshold

dojo.declare("dojox.widget.AnalogGauge2.RingStyle", dojox.widget.AnalogGauge2._Style, {
    //
	// Set the display text size
	fs: 22,
	// summary:
    //          This class defines a style similar to the gauges used in the 
    //          Pulse "main stage" demo for 2010.  
    // description:
    //          They were originally designed by Brian Brandenburg and 
    //          implemented in ILOG's JView.  This iteration has been redesigned 
    //          by Kimberly Holmes and, obviously, is meant for dojo.  
    //
    //          The background color transitions between colors as the needle 
    //          moves through the various thresholds.  In the constructor for 
    //          this style, you can define a mapping of 
    //          dojox.widget.AnalogGaugeThreshold Statuses to colors.  The style 
    //          will automatically figure out how to transition between the 
    //          colors as the needle moves through the thresholds.

    // colorMap: Object
    //          Mapping from dojox.widget.AnalogGaugeThreshold.***_STATUS values 
    //          to colors.
    colorMap: null,
    
    // graphicMap: Object
    //          Mapping from dojox.widget.AnalogGaugeThreshold.***_STATUS values 
    //          to images which convey status without relying on colors (for 
    //          colorblind-accessibility)
    graphicMap: null,
    
    // needlRange: Integer
    //          Range of movement, in degrees, that the needle can pass through, 
    //          centered at vertical.
    needleRange: 260, 

    constructor: function(args){
        dojo.mixin(this, args);
        
        // if the colorMap wasn't set, use a default one.
        if(this.colorMap == null){
            this.colorMap = {};
            this.colorMap[dojox.widget.AnalogGaugeThreshold.CLEAR_STATUS         ]   = "#33cc14";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.NORMAL_STATUS        ]   = "green"; //#33cc14";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.CRITICAL_STATUS      ]   = "red"; //#e51717";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.FATAL_STATUS         ]   = "#222222";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.MAJOR_STATUS         ]   = "#FFB31A";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.MINOR_STATUS         ]   = "#ffff1a";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.INDETERMINANT_STATUS ]   = "#2EA8E5";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.NOT_APPLICABLE_STATUS]   = "#6963A6";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.UNKNOWN_STATUS       ]   = "#A6A6A6";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.UNDEFINED_STATUS     ]   = "#9294f3";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.INFORMATIONAL_STATUS ]   = "#2ea8e5";
        }
        
        if(this.graphicMap == null){
            this.graphicMap = {};
            this.graphicMap[dojox.widget.AnalogGaugeThreshold.CLEAR_STATUS         ]   = dojo.moduleUrl("dojox.widget.AnalogGauge2.RingStyle","ring_normal.png");
            this.graphicMap[dojox.widget.AnalogGaugeThreshold.NORMAL_STATUS        ]   = dojo.moduleUrl("dojox.widget.AnalogGauge2.RingStyle","ring_normal.png");
            this.graphicMap[dojox.widget.AnalogGaugeThreshold.CRITICAL_STATUS      ]   = dojo.moduleUrl("dojox.widget.AnalogGauge2.RingStyle","ring_critical.png");
            this.graphicMap[dojox.widget.AnalogGaugeThreshold.FATAL_STATUS         ]   = dojo.moduleUrl("dojox.widget.AnalogGauge2.RingStyle","ring_critical.png");
            this.graphicMap[dojox.widget.AnalogGaugeThreshold.MAJOR_STATUS         ]   = dojo.moduleUrl("dojox.widget.AnalogGauge2.RingStyle","ring_warning_major.png");
            this.graphicMap[dojox.widget.AnalogGaugeThreshold.MINOR_STATUS         ]   = dojo.moduleUrl("dojox.widget.AnalogGauge2.RingStyle","ring_warning.png");
            this.graphicMap[dojox.widget.AnalogGaugeThreshold.INDETERMINANT_STATUS ]   = dojo.moduleUrl("dojox.widget.AnalogGauge2.RingStyle","ring_indeterminate.png");
            this.graphicMap[dojox.widget.AnalogGaugeThreshold.NOT_APPLICABLE_STATUS]   = dojo.moduleUrl("dojox.widget.AnalogGauge2.RingStyle","ring_indeterminate.png");
            this.graphicMap[dojox.widget.AnalogGaugeThreshold.UNKNOWN_STATUS       ]   = dojo.moduleUrl("dojox.widget.AnalogGauge2.RingStyle","ring_unknown_undefined.png");
            this.graphicMap[dojox.widget.AnalogGaugeThreshold.UNDEFINED_STATUS     ]   = dojo.moduleUrl("dojox.widget.AnalogGauge2.RingStyle","ring_unknown_undefined.png");
            this.graphicMap[dojox.widget.AnalogGaugeThreshold.INFORMATIONAL_STATUS ]   = dojo.moduleUrl("dojox.widget.AnalogGauge2.RingStyle","ring_info.png");
        }
    },

    getDrawOrder: function(){
        return ["drawStatus", "drawStatusShadow", "drawBacking", "drawStatusIcon", "drawTicks", "drawNeedle", "drawText", "drawHighlight"];
    },
    
    getDrawSemantics: function(){
        return {
            background: ["drawStatus", "drawStatusShadow", "drawBacking"],
            ticks: "drawTicks",
            needle: "drawNeedle",
            foreground: "drawHighlight",
            textDisplay: "drawText",
            updateValue: ["drawStatus", "drawStatusIcon", "drawNeedle", "drawText"]
        };
    },
    
    reset: function(){
        // summary:
        //          Deletes any locally-stored shape objects when the gauge 
        //          needs to be reset.
        
        delete this._statusColorCircle;
        delete this._statusColorCircleGroup;
        delete this._needle;
        delete this._needleGroup;
        delete this._textDisplay;
        delete this._textGroup;
        delete this._statusIconGroup;
        
        this._statusColorCircle = null;
        this._statusColorCircleGroup = null;
        this._needle = null;
        this._needleGroup = null;
        this._textDisplay = null;
        this._textGroup = null;
        this._statusIconGroup = null;
    },
    
    _statusColorCircle: null,
    _statusColorCircleGroup: null,
    _lastStatusColor: "#000000",
    drawStatus: function(state){
        // summary:
        //          Draws the colored circle that backs the whole gauge, and 
        //          animates the change in color when necessary (if a threshold 
        //          has just been entered and the threshold's color is different 
        //          from the current backing color.
        // state: Object
        //          {
        //              group: Object
        //                      The dojox.gfx Group object used by the gauge.  All 
        //                      result groups returned by these functions should be 
        //                      created from this group.
        //              minValue: int
        //                      Minimum displayable value on the gauge.
        //              maxValue: int
        //                      Maximum displayable value on the gauge.
        //              startValue: int
        //                      The previous value for the gauge needle. (used with 
        //                      the endValue for animation of a needle) 
        //              endValue: int
        //                      The current value for the gauge needle.
        //              startTrendValue: int
        //                      The previous value for the trend needle.
        //              endTrendValue: int
        //                      The current value for the trend needle.
        //              units: String
        //                      Unit string (like ft. or mi. or cubic meters per hr)
        //              thresholds: Array
        //                      Array of dojox.widget.AnalogGaugeThreshold 
        //                      objects, in order of increasing "start" 
        //                      parameter. Styles can use these to do cool stuff 
        //                      by rendering themselves differently based on the 
        //                      location of the needle relative to the 
        //                      thresholds.
        //              minorTickSeparation: int
        //                      Configured distance between minor ticks. If non-
        //                      positive, styles shouldn't render ticks.
        //              majorTickSeparation: int
        //                      Configured distance between major ticks. If non-
        //                      positive, styles shouldn't render ticks.
        //          }
        // returns:
        //          A dojox.gfx.Group object representing the circle.
        
        if(this._statusColorCircleGroup == null){
            this._statusColorCircleGroup = state.group.createGroup();
            this._statusColorCircle = this._statusColorCircleGroup.createCircle({cx: 0, cy: 0, r: 49});
            this._statusColorCircle.setFill(this._lastStatusColor);
        }
        
        var lastThreshold = this.getThresholdFromValue(state.startValue, state.thresholds);
        var currentThreshold = this.getThresholdFromValue(state.endValue, state.thresholds);
        
        // reaffirm the background color... just in case.
        this._lastStatusColor = this.colorMap[lastThreshold.status];
        this._statusColorCircle.setFill(this._lastStatusColor);
        
        // if the new threshold isn't the same as the last threshold, animate 
        //  the color difference.
        if(lastThreshold.status != currentThreshold.status){
            var newStatusColor = this.colorMap[currentThreshold.status];
            
            var fromRGB = new dojo.Color(this._lastStatusColor).toRgb();
            var toRGB = new dojo.Color(newStatusColor).toRgb();
            
            new dojo._Animation({
                curve: [0, 1], 
                duration: 1000, 
                easing: dojo.fx.easing.expoIn,
                onAnimate: dojo.hitch(this, function(step){
                    var percent = step;
                    var currentColor = [
                        Math.floor(percent * (toRGB[0] - fromRGB[0]) + fromRGB[0]),// red,
                        Math.floor(percent * (toRGB[1] - fromRGB[1]) + fromRGB[1]),// green,
                        Math.floor(percent * (toRGB[2] - fromRGB[2]) + fromRGB[2]) // blue
                    ];
                    if(this._statusColorCircle != null){
                        this._statusColorCircle.setFill(currentColor);
					}
                })
            }).play();
            
            this._lastStatusColor = newStatusColor;
        }
        

		return this._statusColorCircleGroup;
    },
    
    _statusIcon: null,
    _statusIconGroup: null,
    drawStatusIcon: function(state){
        // summary:
        //          Draws the correct icon shape given the value and the list of 
        //          thresholds.
        // state: 
        //          Common state object, see dojox.widget.AnalogGaugeThreshold 
        //          for mroe information.
        // returns:
        //          A dojox.gfx.Group object representing the status graphic.
        
        if(this._statusIconGroup === null){
            this._statusIconGroup = state.group.createGroup();
        } 
        
        setTimeout(dojo.hitch(this, function(){
            this._statusIconGroup.clear();
            delete this._statusIcon;
            this._statusIcon = this._statusIconGroup.createImage({
                x: -47.5, y: -50, 
                width: 100, height: 100, 
                src: this.graphicMap[this.getThresholdFromValue(state.endValue, state.thresholds).status]
            });
        }), 900);
        
        if (this.dir=='rtl') 
          	this._statusIconGroup.setTransform(dojox.gfx.matrix.flipX);

		return this._statusColorCircleGroup;
    },

    
    drawStatusShadow: function(state){
        // summary:
        //          Draws the shadow thing on top of the background color.
        // state: Object
        //          Big state object, see dojox.widget.AnalogGauge2._Style for info.
        // returns: 
        //          Group with the necessary shapes.
        var result = state.group.createGroup();
        
        
        var img = result.createImage({
            x: -50, y: -50, 
            width: 100, height: 100, 
            src: dojo.moduleUrl("dojox.widget.AnalogGauge2.RingStyle","standard_dark_grad.png")
        });
        
        img = result.createImage({
            x: -50, y: -50, 
            width: 100, height: 100, 
            src: dojo.moduleUrl("dojox.widget.AnalogGauge2.RingStyle","standard_bg.png")
        });
        
        return result;
    },
    
    drawBacking: function(state){
        // summary:
        //          Draws the backing thing on top of the background color.
        // state: Object
        //          Big state object, see dojox.widget.AnalogGauge2._Style for info.
        // returns: 
        //          Group with the necessary shapes.
        var result = state.group.createGroup();
        
        var img = result.createImage({
            x: -50, y: -50, 
            width: 100, height: 100, 
            src: dojo.moduleUrl("dojox.widget.AnalogGauge2.RingStyle","standard_center.png")
        });
        
        return result;
    },
    
    drawTicks: function(state){
        // summary:
        //          Draws nothing, this style doesn't use ticks...
        // state: Object
        //          Big state object, see dojox.widget.AnalogGauge2._Style for info.
        // returns: 
        //          Empty group.
        var result = state.group.createGroup();
        
        return result;
    },
    
    _needle: null,
    _needleGroup: null,
    drawNeedle: function(state){
        // summary: 
        //          Draws and animates the gauge's needle.
        // description:
        //          If this._needleGroup is null, we haven't ever drawn a needle 
        //          before.. so we need to do that.  After the initial needle is 
        //          drawn (or if this._needleGroup was not null), we must 
        //          animate the needle from the startValue to the endValue in 
        //          the settings argument.
        // settings: 
        //          The standard style settings object. See  
        //          dojox.widget.AnalogGauge2._Style for more information. 
        // returns: 
        //          A dojox.gfx.Group containing the needle for the gauge. 
        if(this._needleGroup == null || this._needle == null){
            var parentGroup = state.group;
            this._needleGroup = parentGroup.createGroup();
            
            var stick = this._needleGroup.createPath();
            stick.moveTo(0,0)
                .lineTo(-1.5,0)
                .lineTo(-1.5,-47)
                .lineTo(1.5,-47)
                .lineTo(1.5,0)
                .lineTo(0,0)
                .setFill("white")
                .setStroke("black");
            this._needle = stick;
            var nubbin = this._needleGroup.createCircle({cx: 0, cy: 0, r: 5}).setFill("white").setStroke("black");
        }
        
        var minValue = state.minValue;
        var maxValue = state.maxValue;
        var startValue = state.startValue;
        var endValue = state.endValue;
        
        if(startValue > maxValue){
            startValue = maxValue;
        } else if (startValue < minValue){
            startValue = minValue;
        }
        if(endValue > maxValue){
            endValue = maxValue;
        } else if (endValue < minValue){
            endValue = minValue;
        }
        
        var range = maxValue - minValue;
        var startRadians = ((startValue - minValue) / range * this.needleRange - this.needleRange / 2.0 ) * (Math.PI / 180.0); // TODO: Bug here!
        var endRadians = ((endValue - minValue) / range * this.needleRange - this.needleRange / 2.0 ) * (Math.PI / 180.0);    // TODO: bug here!
        
        // set initial position
        this._needle.setTransform(dojox.gfx.matrix.rotate(startRadians));
        
        // Animate it!
        new dojo._Animation({
            curve: [0, 1], 
            duration: 1000, 
            easing: dojo.fx.easing.quadratic,
            onAnimate: dojo.hitch(this, function(step){
                var percent = step;
                var currentRadians = percent * (endRadians - startRadians) + startRadians;
                if(this._needle != null){
                    this._needle.setTransform(dojox.gfx.matrix.rotate(currentRadians));
                }
            })
        }).play();
        
        return this._needleGroup;
    },
    
    _textDisplay: null,
    _textGroup: null,
    drawText: function(state){
        // summary:
        //          Draws the text display of the gauge.
        // state: Object
        //          Big state object, see dojox.widget.AnalogGauge2._Style for info.
        // returns:
        //          Group with the necessary shapes.
        if(this._textGroup == null){
            this._textGroup = state.group.createGroup();
            
            this._textDisplay = this._textGroup.createText({
                x: 0, y: this.fs, 
                text: state.endValue, 
                align: "middle"
            }).setFill("#FFFFFF");
            this._textDisplay.setFont({
                size: this.fs+"px", family: "Verdana, sans-serif"
            });
			var third= (this.fs/3)|0;
			var fold = this.fs + third;
            var units = this._textGroup.createText({
                x: 0, y: fold, // above fs + fs/3
                text: state.units, 
                align: "middle"
            }).setFont({
                size: third+"px", family: "Verdana, sans-serif"
            }).setFill("#FFFFFF");
            if (this.dir=='rtl') 
    	          units.setTransform(dojox.gfx.matrix.flipX);
		}
        
        this._textDisplay.setShape({
            x: 0, y: this.fs,
            text: state.endValue + "" + state.units, 
            align: "middle"
        });
        
        
        // Animate it! (if it's an integer value to an integer value... decimals look weird)
			this._textDisplay.setShape({
                x: 0, y: this.fs,
                text: state.startValue, 
                align: "middle"
            });
			new dojo._Animation({
                curve: [0, 1], 
                duration: 1000, 
                easing: dojo.fx.easing.quadratic,
                onAnimate: dojo.hitch(this, function(step){
                    var percent = step;
                    var currentValue = Math.floor(percent * (state.endValue - state.startValue) + state.startValue);

					if(this._textDisplay != null){
						if (step==1) {
                           currentValue = state.endValue;
						   if (state.formatOptions!=null) 
                             currentValue=    dojo.number.format(currentValue, state.formatOptions);
						}
						this._textDisplay.setShape({
                            x: 0, y: this.fs,
                            text:   currentValue, 
                            align: "middle"
                        });
                    }
					if (this.dir=='rtl') 
          	            this._textDisplay.setTransform(dojox.gfx.matrix.flipX);
				})
            }).play();
        
        return this._textGroup;
    },
    
    drawHighlight: function(state){
        // summary:
        //          Draws the highlight.
        // state: Object
        //          Big state object, see dojox.widget.AnalogGauge2._Style for info.
        // returns: 
        //          Group with the necessary shapes.
        var result = state.group.createGroup();
        
        var img = result.createImage({
            x: -50, y: -50, 
            width: 100, height: 100, 
            src: dojo.moduleUrl("dojox.widget.AnalogGauge2.RingStyle","standard_light.png")
        });
        
        return result;
    }
});