dojo.provide("dojox.widget.AnalogGauge2.StatusStyle");

dojo.require("dojo.fx");
dojo.require("dojo.fx.easing");
dojo.require("dojox.widget.AnalogGauge2._Style");
dojo.require("dojox.widget.AnalogGauge2"); // for the AnalogGaugeThreshold

dojo.declare("dojox.widget.AnalogGauge2.StatusStyle", dojox.widget.AnalogGauge2._Style, {
    // summary:
    //          This class defines a style which, instead of rendering a normal 
    //          spedometer-style gauge, shows simply the appropriate status icon 
    //          for the value given the thresholds assigned to the gauge. 

    //displayText: false,

    urlMap: null,
    pw:80,
    ph:80,
    fs:9,
    font: {family: "Arial", style: "italic", variant: 'small-caps', weight: 'bold', size:'8px'},
    fontSize:9,

    constructor: function(args){
        dojo.mixin(this, args);
        
        // if the colorMap wasn't set, use a default one.
        if(this.urlMap == null){
            this.urlMap = {};
            var baseString = "dojox.widget.AnalogGauge2.StatusStyle";
            this.urlMap[dojox.widget.AnalogGaugeThreshold.CLEAR_STATUS         ]   = dojo.moduleUrl(baseString, "normal.png");
            this.urlMap[dojox.widget.AnalogGaugeThreshold.NORMAL_STATUS        ]   = dojo.moduleUrl(baseString, "normal.png");
            this.urlMap[dojox.widget.AnalogGaugeThreshold.CRITICAL_STATUS      ]   = dojo.moduleUrl(baseString, "critical.png");
            this.urlMap[dojox.widget.AnalogGaugeThreshold.MAJOR_STATUS         ]   = dojo.moduleUrl(baseString, "warning_major.png");
            this.urlMap[dojox.widget.AnalogGaugeThreshold.MINOR_STATUS         ]   = dojo.moduleUrl(baseString, "warning.png");
            this.urlMap[dojox.widget.AnalogGaugeThreshold.INDETERMINANT_STATUS ]   = dojo.moduleUrl(baseString, "indeterminate.png");
            this.urlMap[dojox.widget.AnalogGaugeThreshold.NOT_APPLICABLE_STATUS]   = dojo.moduleUrl(baseString, "unknown.png");
            this.urlMap[dojox.widget.AnalogGaugeThreshold.UNKNOWN_STATUS       ]   = dojo.moduleUrl(baseString, "unknown.png");
            this.urlMap[dojox.widget.AnalogGaugeThreshold.UNDEFINED_STATUS     ]   = dojo.moduleUrl(baseString, "undefined.png");
            this.urlMap[dojox.widget.AnalogGaugeThreshold.INFORMATIONAL_STATUS ]   = dojo.moduleUrl(baseString, "informational.png");
            this.urlMap[dojox.widget.AnalogGaugeThreshold.FATAL_STATUS         ]   = dojo.moduleUrl(baseString, "fatal.png");
        }
    },

    getDrawOrder: function(){
        return ["drawIcon","drawText"];
    },
    
    getDrawSemantics: function(){
        return {
            background: [],
            ticks: [],
            needle: [],
            foreground: [],
            textDisplay: [],
            updateValue: ["drawIcon","drawText"]
        };
    },
    
    reset: function(){
    	console.log("reset");
        // summary:
        //          Deletes any locally-stored shape objects when the gauge 
        //          needs to be reset.
        
        delete this._iconDisplay;
        delete this._iconGroup;
        delete this._textGroup;
        
        this._iconDisplay = null;
        this._iconGroup = null;
        this._textGroup = null;
    },
    
    _computeSize: function(state){
    	console.log("_computeSize");
    	var maxLength = (state.endValue+" ").length;
    	if(maxLength < (state.units+"").length)maxLength=(state.units+"").length;
    	if(maxLength/2*this.fontSize > 100){
    		this.fontSize = Math.abs(100 / maxLength * 2 -1);
    	}
    },
    
    _textGroup:null,
    drawText:function(state){
    	console.log("drawText")
    	if(this._textGroup == null){
            this._textGroup = state.group.createGroup();
        } 
        this._textGroup.clear();
        
        this.font.size=this.fontSize+"px";
        this._textGroup.createText({x:0,y:50-1,text:state.units,align:"middle"}).setFont(this.font).setStroke("black");
        this._textGroup.createText({x:0,y:40-1,text:state.endValue,align:"middle"}).setFont(this.font).setStroke("black");
        
        return this._textGroup;
    },
    
    _iconDisplay: null,
    _iconGroup: null,
    drawIcon: function(state){
    	console.log("drawIcon")
        // summary:
        //          Draws the status icon.
        // state: Object
        //          Big state object, see dojox.widget.AnalogGauge2._Style for info.
        // returns: 
        //          Group with the necessary shapes.
        this._computeSize(state);
        if(this._iconGroup == null){
            this._iconGroup = state.group.createGroup();
        } else {
            this._iconGroup.clear();
            delete this._iconDisplay;
        }

        this._iconDisplay = this._iconGroup.createImage(
                {
                  x: -40, y: -50, 
                  width: 80, height: 80, 

                  src: this.urlMap[this.getThresholdFromValue(state.endValue, state.thresholds).status]
                }
            );
        
        return this._iconGroup;
    }
});