dojo.provide("dojox.widget.AnalogGauge2.WiFiBarStyle");

dojo.require("dojo.fx");
dojo.require("dojo.fx.easing");
dojo.require("dojox.widget.AnalogGauge2._Style");


dojo.declare("dojox.widget.AnalogGauge2.WiFiBarStyle", [dojox.widget.AnalogGauge2._Style], {
	/**
	 * lable and value's text style
	 */
	textColor: [0,0,0],
	textFont:  {family: 'Arial', weight: 'bold'},
	
	_label: "label",
	_height:170,
	_width:250,
	_scaler:170,
	_highlightImage:"",
	
    /**
     * Set the max number of bars
     */
    segments: 5,
	
	// colorMap: Object
    //          Mapping from dojox.widget.AnalogGaugeThreshold.***_STATUS values 
    //          to colors.
    colorMap: null,

    /**
     * Set outline thickness and color, set to zero to remove outline
     */
    outlinewidth: 1,
    outlinecolor: "black",
	
	rainbow : {type: "linear",
             x1: 0, y1: 0,
             x2: 80, y2: 0,
             colors: [
                { offset: 0.1, color: "#FFB31A" },
                { offset: 0.9, color: "#ffffff" }
                 ]
            },
	
    constructor: function(args){
		dojo.mixin(this, args);
		// if the colorMap wasn't set, use a default one.
        if(this.colorMap == null){
            this.colorMap = {};
            this.colorMap[dojox.widget.AnalogGaugeThreshold.CLEAR_STATUS         ]   = "#33cc14";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.NORMAL_STATUS        ]   = "#33cc14";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.CRITICAL_STATUS      ]   = "#e51717";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.FATAL_STATUS         ]   = "#222222";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.MAJOR_STATUS         ]   = "#FFB31A";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.MINOR_STATUS         ]   = "#ffff1a";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.INDETERMINANT_STATUS ]   = "#2EA8E5";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.NOT_APPLICABLE_STATUS]   = "#6963A6";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.UNKNOWN_STATUS       ]   = "#A6A6A6";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.UNDEFINED_STATUS     ]   = "#9294f3";
            this.colorMap[dojox.widget.AnalogGaugeThreshold.INFORMATIONAL_STATUS ]   = "#2ea8e5";
        }
		
	},


	/**
	 * private fnciton compute
	 */
    _computeSizing: function(state){
		if(state.width*0.68>state.height){
			this._width = state.height/0.68;
			this._height = state.height;
		}
		else{
			this._width = state.width;
			this._height = state.width*0.68;
		}
		
		if(state.width>state.height){
			this._scaler = 100.0/state.height;
		}
		else{
			this._scaler = 100.0/state.width;
		}
		
		//the color shade is light or dark
		var thColor = this.colorMap[this.getThresholdFromValue(state.endValue, state.thresholds).status];
		var shade = parseInt(thColor.substring(1,3),16)*0.299 + parseInt(thColor.substring(3,5),16)*0.587 + parseInt(thColor.substring(5,7),16)*0.114;
		if(shade>191){
			this._highlightImage = "transparency_light.png";
		}
		else{
			this._highlightImage = "transparency5.png";
		}
		
		
    },


    getDrawOrder: function(){
        return ["drawBackground",  "drawForeground", "drawText"];
	},
  
    
    getDrawSemantics: function(){
        return {
            background: "drawBackground",
            foreground: "drawForeground",
			text: "drawText",
            updateValue: ["drawBackground", "drawForeground", "drawText"]
        };
    },
    


	_backGroup: null,

	drawBackground: function(state){
		
		this._computeSizing(state);
		
		var ph=this._height*0.875;
        var pw=this._width*0.875;
		var cx = this._width*0.5;
        var cy = this._height*0.5;
		
		var shape;
		if (this._backGroup==null){
           this._backGroup= state.group.createGroup();
        }
        this._backGroup.clear();
		shape = this._backGroup.createImage({x:-cx,y:-cy,width:pw,height:ph,src:dojo.moduleUrl("dojox.widget.AnalogGauge2.WiFiStyle", "background3.png")});

		this._backGroup.setTransform({xx: this._scaler, yy: this._scaler, dx:0, dy:0});
		return this._backGroup;
	
	},

    _foreGroup: null,
    drawForeground: function(state){
		var ph=this._height*0.875;
        var pw=this._width*0.875;

        var cx = this._width*0.5;
        var cy = this._height*0.5;


        if (this._foreGroup==null){
           this._foreGroup= state.group.createGroup();
        }
        this._foreGroup.clear();

        var shape,i,x,j;
		
		
		//how many bars
		var n = Math.min(this.segments, Math.floor((parseInt(state.endValue)+10)/20));
		
		var bw = pw/12; //bar width
		var mw = pw*0.15;//pw/10;  //move bar horizontal

		for (i=0; i<5; i++){
			var bh = ph*(i+1)/7;  //bar height
			var mh = ph - bh - ph*0.12;  //move bar vertical 
			
			shape = this._foreGroup.createRect({x:-cx+mw,y:-cy+mh,width:bw,height:bh}).setStroke({color: this.outlinecolor, width: this.outlinewidth});
	
			this.rainbow.colors[0].color = this.colorMap[this.getThresholdFromValue(state.endValue, state.thresholds).status];
			this.rainbow.x2 = bw * 2;
			if(i<n){
				shape.setFill(this.rainbow);
				//draw high light
				var highlight = this._foreGroup.createImage({
				x: -cx+mw, y: -cy+mh, 
				width: bw/2, height: bh, 
				src: dojo.moduleUrl("dojox.widget.AnalogGauge2.WiFiStyle", this._highlightImage)
			});
			}
			else{
				shape.setFill("white");
			}

			cx-=(pw/5);
			mw-=pw/20;
		
		}

		this._foreGroup.setTransform({xx: this._scaler, yy: this._scaler, dx:0, dy:0});
		return this._foreGroup;
    },

	
	/**
     *  draw lable and value
     */
    _textGroup: null,
    drawText: function(state){
		var cx = this._width *0.32;//*0.31;//*0.4;
		var cy = this._height *0.2;//*0.15;//*0.05;
		
		var cx1 = this._width *0.06;//*0.26;//*0.1;
		var cy1 = this._height *0.48;//*0.24;//*0.44;
		
		if (this._textGroup == null) {
			this._textGroup = state.group.createGroup();
		}
		this._textGroup.clear();
		

			var value = this._textGroup.createText({
				x: -cx,
				y: -cy,
				text: state.endValue,
				//text: Math.min(this.segments, Math.floor((parseInt(state.endValue)+10)/20)),
				align: "middle"
			}).setFill(this.textColor);
			this.textFont.size=25*this._width/200;
			value.setFont(this.textFont);

			var label = this._textGroup.createText({
				x: -cx1,
				y: cy1,
				text: state.units,//this._label,  //state.displayText
				align: "middle"
			}).setFill(this.textColor);
			this.textFont.size=20*this._width/200;
			label.setFont(this.textFont);

			this._textGroup.setTransform({xx: this._scaler, yy: this._scaler, dx:0, dy:0});
			return this._textGroup;
		
	},

    reset: function(){
        // summary:
        //          Deletes any locally-stored shape objects when the gauge 
        //          needs to be reset.

        delete this._backGroup;
        delete this._foreGroup;
		delete this._textGroup;

        this._backGroup=null;
        this._foreGroup=null;
		this._textGroup=null; 
    }
});