dojo.provide("dojot.visual.ClusteredBarsEmbossed");

dojo.require("dojot.visual.BarsEmbossed");
dojo.require("dojox.charting.plot2d.common");
dojo.require("dojox.lang.functional");
dojo.require("dojox.lang.functional.reversed");

(function(){
	var df = dojox.lang.functional, dc = dojox.charting.plot2d.common,
		purgeGroup = df.lambda("item.purgeGroup()");

	dojo.declare("dojot.visual.ClusteredBarsEmbossed", dojot.visual.BarsEmbossed, {
		//	summary:
		//		A plot representing grouped or clustered bars (horizontal bars)
		render: function(dim, offsets){
			//	summary:
			//		Run the calculations for any axes for this plot.
			//	dim: Object
			//		An object in the form of { width, height }
			//	offsets: Object
			//		An object of the form { l, r, t, b}.
			//	returns: dojot.visual.ClusteredBarsEmbossed
			//		A reference to this plot for functional chaining.
			if(this.zoom && !this.isDataDirty()){
				return this.performZoom(dim, offsets);
			}
			this.resetEvents();
			this.dirty = this.isDirty();
			if(this.dirty){
				dojo.forEach(this.series, purgeGroup);
				this._eventSeries = {};
				this.cleanGroup();
				var s = this.group;
				df.forEachRev(this.series, function(item){ item.cleanGroup(s); });
			}

			this._hScaler.bounds.span-=this.opt.depth;
            var diff=this._hScaler.bounds.to-this._hScaler.bounds.from;
            this._hScaler.bounds.scale= this._hScaler.bounds.span/diff;

			var t = this.chart.theme, f, gap, height, thickness,
				ht = this._hScaler.scaler.getTransformerFromModel(this._hScaler),
				vt = this._vScaler.scaler.getTransformerFromModel(this._vScaler),
				baseline = Math.max(0, this._hScaler.bounds.lower),
				baselineWidth = ht(baseline),
				events = this.events();
			f = dc.calculateBarSize(this._vScaler.bounds.scale, this.opt, this.series.length);
			gap = f.gap;
			height = thickness = f.size;



			for(var i = this.series.length - 1; i >= 0; --i){
				var run = this.series[i], shift = thickness * (this.series.length - i - 1);
				if(!this.dirty && !run.dirty){
					t.skip();
					this._reconnectEvents(run.name);
					continue;
				}
				run.cleanGroup();
				var theme = t.next("bar", [this.opt, run]), s = run.group,
					eventSeries = new Array(run.data.length);
				for(var j = 0; j < run.data.length; ++j){
					var value = run.data[j];
					if(value !== null){
						var v = typeof value == "number" ? value : value.y,
							hv = ht(v),
							width = hv - baselineWidth,
							w = Math.abs(width),
							finalTheme = typeof value != "number" ?
								t.addMixin(theme, "bar", value, true) :
								t.post(theme, "bar");
						if(w >= 0 && height >= 1){

							var specialFill = this._plotFill(finalTheme.series.fill, dim, offsets);
							var work= new dojo.Color(specialFill);
                            var darker = new dojo.Color([ work.r*50/100|0, work.g*50/100|0, work.b*50/100|0 ]);
                            var dark = new dojo.Color([ work.r*90/100|0, work.g*90/100|0, work.b*90/100|0 ]);

//begin redpath
                        var x= offsets.l + (v < baseline ? hv : baselineWidth);
                        var y= dim.height - offsets.b - vt(j + 1.5) + this.opt.gap + shift+ this.opt.depth;
                        var off= this.opt.depth;

                        var apath = "M" + x          +  "," + y +
                             " L" + (x+off)       +  "," + (y-off) +
                             " L" + (x+width+off) +  "," + (y-off) +
                             " L" + (x+width)   +  "," + (y) +
                             " L" + x           +  "," + y+ "z";
                        s.createPath(apath).setFill(dark);

						var h= height-off;
                        apath = " M" + (x+width+off) +  "," + (y-off) +
                             " L" + (x+width+off) +  "," + (y+h-off) +
                             " L" + (x+width) +  "," + (y+h)+
                             " L" + (x+width) +  "," + y+" z";
                        s.createPath(apath).setFill(darker);


						var shape = s.createRect({
							x: offsets.l + (v < baseline ? hv : baselineWidth),
							y: dim.height - offsets.b - vt(j + 1.5) + this.opt.gap + shift +this.opt.depth,
							width: w, height: height-this.opt.depth
						}).setFill(specialFill);

							run.dyn.fill   = shape.getFill();
							run.dyn.stroke = shape.getStroke();
							if(events){
								var o = {
									element: "bar",
									index:   j,
									run:     run,
									shape:   shape,
									x:       v,
									y:       j + 1.5
								};
								this._connectEvents(o);
								eventSeries[j] = o;
							}

						}
					}
				}
				this._eventSeries[run.name] = eventSeries;
				run.dirty = false;
			}
			this.dirty = false;
			return this;	//	dojot.visual.ClusteredBarsEmbossed
		}
	});
})();
