dojo.provide("dojot.visual.ClusteredColumnsEmbossed");

dojo.require("dojot.visual.ColumnsEmbossed");
dojo.require("dojox.charting.plot2d.common");
dojo.require("dojox.charting.plot2d.Columns");

dojo.require("dojox.lang.functional");
dojo.require("dojox.lang.functional.reversed");

(function(){
	var df = dojox.lang.functional, dc = dojox.charting.plot2d.common,
		purgeGroup = df.lambda("item.purgeGroup()");

	dojo.declare("dojot.visual.ClusteredColumnsEmbossed", dojot.visual.ColumnsEmbossed, {
		//	summary:
		//		A plot representing grouped or clustered columns (vertical bars).
		render: function(dim, offsets){
			//	summary:
			//		Run the calculations for any axes for this plot.
			//	dim: Object
			//		An object in the form of { width, height }
			//	offsets: Object
			//		An object of the form { l, r, t, b}.
			//	returns: dojot.visual.ClusteredColumnsEmbossed
			//		A reference to this plot for functional chaining.
			if(this.zoom && !this.isDataDirty()){
				return this.performZoom(dim, offsets);
			}
			this.resetEvents();
			this.dirty = this.isDirty();
			if(this.dirty){
				dojo.forEach(this.series, purgeGroup);
				this._eventSeries = {};
				this.cleanGroup();
				var s = this.group;
				df.forEachRev(this.series, function(item){ item.cleanGroup(s); });
			}
			this._vScaler.bounds.span-=this.opt.depth;
            var diff=this._vScaler.bounds.to-this._vScaler.bounds.from;
            this._vScaler.bounds.scale= this._vScaler.bounds.span/diff;

			var t = this.chart.theme, f, gap, width, thickness,
				ht = this._hScaler.scaler.getTransformerFromModel(this._hScaler),
				vt = this._vScaler.scaler.getTransformerFromModel(this._vScaler),
				baseline = Math.max(0, this._vScaler.bounds.lower),
				baselineHeight = vt(baseline),
				events = this.events();
			f = dc.calculateBarSize(this._hScaler.bounds.scale, this.opt, this.series.length);
			gap = f.gap;
			width = thickness = f.size;
            width-=this.opt.depth;

			for(var i = 0; i < this.series.length; ++i){
				var run = this.series[i], shift = thickness * i;
				if(!this.dirty && !run.dirty){
					t.skip();
					this._reconnectEvents(run.name);
					continue;
				}
				run.cleanGroup();
				var theme = t.next("column", [this.opt, run]), s = run.group,
					eventSeries = new Array(run.data.length);
				for(var j = 0; j < run.data.length; ++j){
					var value = run.data[j];
					if(value !== null){
						var v = typeof value == "number" ? value : value.y,
							vv = vt(v),
							height = vv - baselineHeight,
							h = Math.abs(height),
							finalTheme = typeof value != "number" ?
								t.addMixin(theme, "column", value, true) :
								t.post(theme, "column");
						if(width >= 1 && h >= 0){
							var specialFill = this._plotFill(finalTheme.series.fill, dim, offsets);
							var work= new dojo.Color(specialFill);
                            var darker = new dojo.Color([ work.r*50/100|0, work.g*50/100|0, work.b*50/100|0 ]);
                            var dark = new dojo.Color([ work.r*90/100|0, work.g*90/100|0, work.b*90/100|0 ]);
//begin redpath
                            var x= offsets.l + ht(j + 0.5) + gap + shift;
                            var y= dim.height - offsets.b - vt(v);
                            var off= this.opt.depth;
    
    
    
                            var apath = "M" + x           +  "," + y   +
                                 " L" + (x+off)       +  "," + (y-off) +
                                 " L" + (x+width+off) +  "," + (y-off) +
                                 " L" + (x+width)   +  "," + (y) +
                                 " L" + x           +  "," + y+ "z";
                            s.createPath(apath).setFill(dark);
    
                            apath = " M" + (x+width+off) +  "," + (y-off) +
                                 " L" + (x+width+off) +  "," + (y+height-off) +
                                 " L" + (x+width) +  "," + (y+height)+
                                 " L" + (x+width) +  "," + y+" z";
                            s.createPath(apath).setFill(darker);
    
                            var shape = s.createRect({
                                x: offsets.l + ht(j + 0.5) + gap + shift,
    							y: dim.height - offsets.b - (v > baseline ? vv : baselineHeight),
    							width: width, height: h
    						}).setFill(specialFill);
  							run.dyn.fill   = shape.getFill();
   							run.dyn.stroke = shape.getStroke();
   							if(events){
   								var o = {
   									element: "column",
   									index:   j,
   									run:     run,
									shape:   shape,
									x:       j + 0.5,
									y:       v
								};
							this._connectEvents(o);
							eventSeries[j] = o;
							}
						}
					}
				}
				this._eventSeries[run.name] = eventSeries;
				run.dirty = false;
			}
			this.dirty = false;
			return this;	//	dojot.visual.ClusteredColumnsEmbossed
		}
	});
})();
