/**
 * @author jwfeinst
 */
(function(dojo){
    dojo.provide("ibm.tivoli.data.HierarchicalItemReadStore");
    
    dojo.require("dojo.data.ItemFileReadStore");
    dojo.require("ibm.tivoli.data.api.Hierarchy");
    
    dojo.declare("ibm.tivoli.data.HierarchicalItemReadStore", [dojo.data.ItemFileReadStore, ibm.tivoli.data.api.Hierarchy], {
        childField: "",
        parentField: "",
        
        constructor: function(){
            console.log("Arguments: ", Array.prototype.slice.call(arguments));
            dojo.mixin(this, arguments[0]);
        },
        
        /**
         * Fetches the children of a specified item (e.g. Children nodes
         * for a node in a tree structure)
         * 
         * exceptions:
         *     Throws an exception if the query is not valid, or if the query 
         *     is required but was not supplied.
         * 
         * @param {Object} keywordArgs
         *     {
         *         baseItem: Item,
         *         onBegin: Function(size, request), 
         *         onItem: Function(item, request), 
         *         onComplete: Function(items, request), 
         *         onError: Function(errorData, request),
         *         scope: object
         *     }
         */
        fetchChildren: function(keywordArgs){
            var childrenIds = this.getValues(keywordArgs.baseItem, this.childField);
            
            keywordArgs.onBegin.call(keywordArgs.scope, childrenIds.length, keywordArgs);
            var items = [];
            for(var i = 0; i < childrenIds.length; i++){
                this.fetchItemByIdentity({
                    identity: childrenIds[i],
                    scope: keywordArgs.scope,
                    onItem: function(item, request){
                        items.push(item);
                        keywordArgs.onItem.call(keywordArgs.scope, item, keywordArgs);
                    },
                    onError: keywordArgs.onError
                });
            }
            keywordArgs.onComplete.call(keywordArgs.scope, items, keywordArgs);
            
            return keywordArgs;
        },
        
        /**
         * Fetches the parents of a specified item (e.g. Parent nodes
         * for a node in a tree structure)
         * 
         * exceptions:
         *     Throws an exception if the query is not valid, or if the query 
         *     is required but was not supplied.
         * 
         * @param {Object} keywordArgs
         *     {
         *         baseItem: Item,
         *         onBegin: Function(size, request), 
         *         onItem: Function(item, request), 
         *         onComplete: Function(items, request), 
         *         onError: Function(errorData, request),
         *         scope: object
         *     }
         */
        fetchParents: function(keywordArgs){
            var parentIds = this.getValues(keywordArgs.baseItem, this.parentField);
            
            keywordArgs.onBegin.call(keywordArgs.scope, parentIds.length, keywordArgs);
            var items = [];
            for(var i = 0; i < parentIds.length; i++){
                this.fetchItemByIdentity({
                    identity: parentIds[i],
                    scope: keywordArgs.scope,
                    onItem: function(item, request){
                        items.push(item);
                        keywordArgs.onItem.call(keywordArgs.scope, item, keywordArgs);
                    },
                    onError: keywordArgs.onError
                });
            }
            keywordArgs.onComplete.call(keywordArgs.scope, items, keywordArgs);
            
            return keywordArgs;
        }
    });
    
})(dojo);