/**
 * @author jwfeinst
 */
(function(dojo){
    
    //if (dojo._hasResource["ibm.tivoli.data.NavModelStore"]) 
    //    return;
        
        
    //dojo._hasResource["ibm.tivoli.data.NavModelStore"] = true;
    dojo.provide("ibm.tivoli.data.NavModelItemLinkStore");
    
    /*
     * APIs we plan to implement.
     */
    dojo.require("dojo.data.api.Identity");
    dojo.require("dojo.data.api.Notification");
    dojo.require("dojo.data.api.Read");
    dojo.require("dojo.data.api.Request");
    
    /*
     * Other imports
     */
    dojo.require("dojo.data.api.Request");
    
    /**
     * "Throw away" class used for the demo that the director team needs to give
     * in late October, 2009. It is a modified version of the early
     * NavModelStore to give preliminary Links and Item responses based on query.
     */
    dojo.declare("ibm.tivoli.data.NavModelItemLinkStore", [dojo.data.api.Read], {
        
        /**
         * Required Parameters
         */
        baseUrl : "",
        provider: "",
        datasource: "",
        dataset: "",
        
        /**
         * Optional Parameters
         */
        columns: null,
        id: "",
        pollImmediately: true,
        availableColumns: null,
        
        /**
         * Instance vars.
         */
        indexedCache: {
                    items: {},
                    links: {},
                    tasks: {}
                },
        cacheExpiration: 30000, // 30 seconds
        
        /**
         * Called before the constructor is called, by dojo.
         */
        preamble: function(){
            this._TOPIC_LOG = "ibm.tivoli.data.NavModelItemLinkStore.log";//= ibm.tivoli.data.NavModelStore.TOPIC_LOG;
            this._logLevel = 1;
        },
        
        /**
         * Called when an instance of this 'class' is created, by dojo. Knows 
         * how to accept a single argument, an object, structured like this:
         * 
         * {
         *        id : "some id", <-- id for the data store, will be used when 
         *                             emitting events (optional, one will be 
         *                             generated automatically if none is specified)
         *        baseUrl : "http://9.42.57.155:16315/ibm/tivoli/rest", <-- base 
         *                            url of the navmodel rest service
         *        provider: "DEMO", <-- navmodel provicer
         *        dataset: "DEMO", <-- navmodel data set
         *        columns: ["NAME", "TYPE", "DESCRIPTION"]  <-- item parameters 
         *                            to retrieve automatically  (optional, will 
         *                            use service's default columns if none are 
         *                            specified)
         * }
         */
        constructor: function(){
            dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "constructor()", "Entered."]);
            
	    
            dojo.mixin(this, arguments[0]);
            if(!this.id || this.id == ""){
                this.id = (new Date()).getTime()+"NavModelStore"+(Math.floor(Math.random()*100000));
            }
            
            dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "constructor()", "Exited."]);
        },
        
        /**
         * Called after the constructor is called, by dojo.
         */
        postscript: function(){
            dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "postscript(args)", "Entered."]);
            
            if(this.pollImmediately){
                dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "postscript(args)", "pollImmediately was set to true, fetching "+(this.columns == null || !(this.columns instanceof Array) ? "default" : this.columns.join(", "))+" columns."]);
                //this._pollItems();  // TODO remove this, enable caching some other way?
            }
            
            dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "postscript(args)", "Exited."]);
        },
        
        /**
         * Gets an array of default columns..
         * TODO: finish this.
         */
        getDefaultColumns: function(){
            var result = [];
            
            dojo.xhrGet(_buildDefaultColumnsRequest(false, 
                function(response){
                    for(var i = 0; i < response.items.length; i=i+1){
                        
                    }
                }, 
                function(error){
                    throw new Error(error);
                }));
        },
        
        /**
         * This function is called when we need to poll for items.
         */
        _pollItems: function(){
            dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "_pollItems()", "Entered."]);
            
            this.fetch({
                onItem: function(item, request){
                    var name = this.getValue(item,"name");
                    dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "_pollItems()", "Found item: "+name]);
                },
                onBegin: function(){
                    dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "_pollItems()", "Starting to find items.."]);
                },
                scope: this
            });
            
            
            dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "_pollItems()", "Exited."]);
        },
        
        /**
         * Constructs the parameters object used in an xhrGet for retrieving the items for a dataset.  Expects this.columns to be set.
         * @param {boolean} async Whether or not the request should be asynchronous.
         * @param {Function} loadFunc Function to execute when the response is received.
         * @param {Function} errorFunc Function to execute when an error is received.
         */
        _buildItemPollingRequest: function(async, loadFunc, errorFunc){
            dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "_buildItemPollingRequest("+arguments+")", "Entering."]);
            var result = this._buildSkeletonDatasetRequest(async, loadFunc, errorFunc);
            result.url = result.url + "/items";
            result.content['properties'] = this.columns.join(",");
	    
            dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "_buildItemPollingRequest("+arguments+")", "Exiting with result:", result]);
            return result;
        },
	
	/**
         * Constructs the parameters object used in an xhrGet for retrieving the items for a dataset.  Expects this.columns to be set.
         * @param {boolean} async Whether or not the request should be asynchronous.
         * @param {Function} loadFunc Function to execute when the response is received.
         * @param {Function} errorFunc Function to execute when an error is received.
         */
        _buildLinkPollingRequest: function(async, loadFunc, errorFunc){
            dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "_buildLinkPollingRequest("+arguments+")", "Entering."]);
            var result = this._buildSkeletonDatasetRequest(async, loadFunc, errorFunc);
            result.url = result.url + "/links";
            result.content['properties'] = this.columns.join(",");
	    
            dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "_buildLinkPollingRequest("+arguments+")", "Exiting with result:", result]);
            return result;
        },
        
        /**
         * Contstructs the parameters object used in an xhrGet for retrieving the defaultColumns of a dataset's items.
         * @param {boolean} async Whether or not the request should be asynchronous.
         * @param {Function} loadFunc Function to execute when the response is received.
         * @param {Function} errorFunc Function to execute when an error is received.
         */
        _buildDefaultColumnsRequest: function(async, loadFunc, errorFunc){
            var result = this._buildSkeletonDatasetRequest(async, loadFunc,errorFunc);
            result.url = result.url +"/defaultColumns";
            return result;
        },
        
        
        /**
         * Contstructs the parameters object used in an xhrGet for retrieving the availableColumns of a dataset's items.
         * @param {boolean} async Whether or not the request should be asynchronous.
         * @param {Function} loadFunc Function to execute when the response is received.
         * @param {Function} errorFunc Function to execute when an error is received.
         */
        _buildAvailableColumnsRequest: function(async, loadFunc, errorFunc){
            var result = this._buildSkeletonDatasetRequest(async, loadFunc,errorFunc);
            result.url = result.url +"/availableColumns";
            return result;
        },
            
        /**
         * Constructs the parameters object used in an xhrGet for retrieving the total set of info for an item.
         * @param {Object} item Item to evaluate.
         * @param {Object} async Wether or not to make the request asynchronous.
         * @param {Object} loadFunc Function to call when the response is received.
         * @param {Object} errorFunc Function to call when an error is received.
         */            
        _buildItemRequest: function(item, async, loadFunc, errorFunc){
            var result = this._buildSkeletonRequest(async, loadFunc, errorFunc);
            result.url = result.url + item.itemUri;
            result.content['properties'] = 'all';
            return result;
        },
	
        /**
         * Constructs the parameters object used in an xhrGet for retrieving a 
         * particular item,w task, or link.
         * @param {Object} identity
         *    ID of the item/task/link, the ID that appears in a URL
         * @param {Object} type
         *    Type of object (either 'item', 'task', or 'link')
         * @param {Object} async
         *    Wether or not to make the request asynchronous.
         * @param {Object} loadFunc
         *    Function to call when the response is received.
         * @param {Object} errorFunc
         *    Function to call if an error is received.
         */
        _buildIdentityRequest: function(identity, type, async, loadFunc, errorFunc){
            var result = this._buildSkeletonDatasetRequest(async, loadFunc, errorFunc);
            result.url = result.url + "/" + type +"s/" + identity;
            result.content['properties'] = 'all';
            return result;
        },
        
        
        /**
         * Constructs the base parameters object used in making xhrGet requests to the dataset that this Data Store is instantiated for.
         * @param {boolean} async Whether or not the request should be asynchronous.
         * @param {Function} loadFunc Function to execute when the response is received.
         * @param {Function} errorFunc Function to execute when an error is received.
         */
        _buildSkeletonDatasetRequest: function(async, loadFunc, errorFunc){
            var result = this._buildSkeletonRequest(async, loadFunc, errorFunc);
            result.url = result.url + "/providers/" + this.provider+(this.datasource != "" ? "/datasources/"+this.datasource : "")+"/datasets/"+this.dataset;
            return result;
        },
        
        /**
         * Constructs the base parameters object used in making xhrGet requests back end.
         * @param {boolean} async Whether or not the request should be asynchronous.
         * @param {Function} loadFunc Function to execute when the response is received.
         * @param {Function} errorFunc Function to execute when an error is received.
         */
        _buildSkeletonRequest: function(async, loadFunc, errorFunc){
            return {
                url: this.baseUrl,
                handleAs: "json",
                content: {},
                sync: !async,
                timeout: 30000,
                load: dojo.hitch(this, loadFunc),
                error: dojo.hitch(this, errorFunc)
            };
        },
        
        /**
         * summary:
         *        Returns a single attribute value.
         *        Returns defaultValue if and only if *item* does not have a value for *attribute*.
         *        Returns null if and only if null was explicitly set as the attribute value.
         *        Returns undefined if and only if the item does not have a value for the
         *        given attribute (which is the same as saying the item does not have the attribute). 
         * description:
         *        Saying that an "item x does not have a value for an attribute y"
         *        is identical to saying that an "item x does not have attribute y". 
         *        It is an oxymoron to say "that attribute is present but has no values" 
         *        or "the item has that attribute but does not have any attribute values".
         *        If store.hasAttribute(item, attribute) returns false, then
         *        store.getValue(item, attribute) will return undefined.
         *
         * exceptions:
         *        Throws an exception if *item* is not an item, or *attribute* is not a string
         *
         * @param {Object} item The item to access values on.
         * @param {Object} attribute The attribute to access represented as a 
         *                     string.
         * @param {Object} defaultValue Optional.  A default value to use for 
         *                     the getValue return in the attribute does not exist 
         *                     or has no value.
         */
        getValue: function(item, attribute, defaultValue){
            dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "getvalue(item, '"+attribute+"', "+defaultValue+")", "Entering.."]);
            if(typeof attribute != 'string'){
                dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "getvalue(item, '"+attribute+"', "+defaultValue+")", "Exiting early, attribute param wasn't a string."]);
                throw new Error("Attribute not a string.");
            }                
            
            var attributeValue = null;
            var foundValue = false;
            attribute = attribute.toLowerCase();
            
            if(!item.properties[attribute]){
                if (!this.isItemLoaded(item)) {
                    dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "getvalue(item, '" + attribute + "', " + defaultValue + ")", "Could not find attribute and item is not fully loaded, loading it now."]);
                    this.loadItem({
                            item: item, 
                            onLoad:  function(loadedItem){
                                dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "getvalue(item, '" + attribute + "', " + defaultValue + ")", "Loaded item."]);
                                item = loadedItem;
                            }, 
                            onError: function(error){
                                dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "getvalue(item, '" + attribute + "', " + defaultValue + ")", "Error occurred while loading item: "+error]);
                                throw new Error(error);
                            }
                        }, false);
                }
                
                if(!item.properties[attribute]){
                    dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "getvalue(item, '"+attribute+"', "+defaultValue+")", "Exiting with default value, could not find attribute even after loading."]);
                    return defaultValue;
                }
            }
            
            foundValue = item.properties[attribute].value;
            
            dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "getvalue(item, '"+attribute+"', "+defaultValue+")", "Exiting with "+foundValue]);
            return foundValue; // a literal, an item, null, or undefined (never an array)
        },

        /**
         * This getValues() method works just like the getValue() method, but getValues()
         * always returns an array rather than a single attribute value.  The array
         * may be empty, may contain a single attribute value, or may contain
         * many attribute values.
         * If the item does not have a value for the given attribute, then getValues()
         * will return an empty array: [].  (So, if store.hasAttribute(item, attribute)
         * has a return of false, then store.getValues(item, attribute) will return [].)
         * 
         * exceptions:
         * Throws an exception if *item* is not an item, or *attribute* is not a string
         * 
         * example:
         * var friendsOfLuke = store.getValues(lukeSkywalker, "friends");
         * 
         * @param {Object} item The item to access values on.
         * @param {Object} attribute The attribute to access represented as a string.
         */
        getValues: function(item, attribute){
            if(typeof attribute != 'string'){
                throw new Error("Attribute not a string.");
            }
            
            var array = [];
            array = this.getValue(item, attribute, []);
            if(array == null)
                array = [];
            else if (!(array instanceof Array)){
                array = [array];
            }
            
            return array; // an array that may contain literals and items
        },
    
        /**
         * Returns an array with all the attributes that this item has.  This
         * method will always return an array; if the item has no attributes
         * at all, getAttributes() will return an empty array: [].
         * 
         * exceptions:
         *  Throws an exception if *item* is not an item
         *  
         * example:
         *  var array = store.getAttributes(kermit);
         * 
         * @param {Object} item The item to access attributes on.
         */   
        getAttributes: function(item){  
            if(!this.isItem(item)){
                throw new Error("Argument is not a valid item.");
            }
            
            if(this.availableColumns == null){
                dojo.xhrGet(this._buildAvailableColumnsRequest(false, 
                        dojo.hitch(this, function(response){
                            this.availableColumns = new Array();
                            for(var i = 0; i < response.items.length; i=i+1){
                                this.availableColumns.push(response.items[i].columnId.toLowerCase());
                            }
                        }), 
                        dojo.hitch(this, function(error){
                            throw error;
                        })));
            }
            
            return this.availableColumns; // array
        },
        
        /**
         * Returns true if the given *item* has a value for the given *attribute*.
         * 
         * exceptions:
         *  Throws an exception if *item* is not an item, or *attribute* is not a string
         * example:
         *  var trueOrFalse = store.hasAttribute(kermit, "color");
         * 
         * @param {Object} item The item to access attributes on.
         * @param {Object} attribute The attribute to access represented as a string.
         */
        hasAttribute: function(item, attribute){   
            if(!this.isItem(item)){
                /*
                 * Throw an exception like we're supposed to.
                 */
                throw new Error("Item argument is not a valid item.");
            }
            
            if(!(typeof(attribute) == 'string')){
                throw new Error("Attribute argument is not a string.");
            }
            
            var attributes = this.getAttributes(item);
            
            return (new RegExp(","+attribute+",|^"+attribute+",|,"+attribute+"$")).test(attributes.join(",")); // boolean
        },

        /**
         * Returns true if the given *value* is one of the values that getValues()
         * would return.
         * 
         * exceptions:
         *  Throws an exception if *item* is not an item, or *attribute* is not a string
         * example:
         *  var trueOrFalse = store.containsValue(kermit, "color", "green");
         * 
         * @param {Object} item The item to access values on.
         * @param {Object} attribute The attribute to access represented as a string.
         * @param {Object} value The value to match as a value for the attribute.
         */
        containsValue: function(item, attribute, value){ 
            if(!this.isItem(item)){
                throw new Error("Value for item argument is not a valid item.");
                return false;
            }
            
            var values = this.getValues(item, attribute, value);

            for(var i = 0; i < values.length; i++){
                if(values[i] == value)
                    return true;
            }
            
            return false; // boolean
        },

        /**
         * Returns true if *something* is an item and came from the store 
         * instance.  Returns false if *something* is a literal, an item from 
         * another store instance, or is any object other than an item.
         * 
         * SPECIFICS:
         *      Returns true if the object has values for something.itemId, 
         *      something.itemUrl, and something.loaded
         * 
         * example:
         *    var yes = store.isItem(store.newItem());
         *    var no  = store.isItem("green");
         * 
         * @param {Object} something Can be anything.
         */
        isItem: function(something){ 
            if(!something)
                return false;
        
            return (
                (
		    (something.itemId != undefined && something.itemId != null) ||
		    (something.linkId != null)
		) && 
                (something.loaded != undefined && 
                something.loaded != null));
        },

        /**
         * Returns false if isItem(something) is false.  Returns false if if 
         * isItem(something) is true but the the item is not yet loaded in 
         * local memory (for example, if the item has not yet been read from 
         * the server).
         * 
         * example:
         *    var yes = store.isItemLoaded(store.newItem());
         *    var no  = store.isItemLoaded("green");
         * 
         * @param {Object} something Can be anything.
         */
        isItemLoaded: function(something) { 
            dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "isItemLoaded('"+something+"')", "Entering..."]);
            if(something.loaded == undefined) something.loaded = false;
            
            var result = this.isItem(something) && something.loaded == true;
            dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "isItemLoaded('"+something+"')", "Exiting with "+result]);
            return result;
        },
    
        /**
         * Given an item, this method loads the item so that a subsequent call 
         * to store.isItemLoaded(item) will return true.  If a call to 
         * isItemLoaded() returns true before loadItem() is even called, then 
         * loadItem() need not do any work at all and will not even invoke the 
         * callback handlers.  So, before invoking this method, check that the 
         * item has not already been loaded.
         * 
         * @param {Object} keywordArgs An anonymous object that defines the 
         *      item to load and callbacks to invoke when the load has 
         *      completed.  The format of the object is as follows: 
         *      { 
         *          item: object, 
         *                  The item parameter is an object that represents the 
         *                  item in question that should be contained by the 
         *                  store.  This attribute is required.
         *          onItem: Function, 
         *                  The onItem parameter is the callback to invoke when 
         *                  the item has been loaded.  It takes only one 
         *                  parameter, the fully loaded item.
         *          onError: Function, 
         *                  The onError parameter is the callback to invoke 
         *                  when the item load encountered an error.  It takes 
         *                  only one parameter, the error object
         *          scope: object 
         *                  If a scope object is provided, all of the callback 
         *                  functions (onItem,  onError, etc) will be invoked 
         *                  in the context of the scope object. In the body of 
         *                  the callback function, the value of the "this" 
         *                  keyword will be the scope object.   If no scope 
         *                  object is provided, the callback functions will be 
         *                  called in the context of dojo.global(). For example, 
         *                  onItem.call(scope, item, request) vs.  
         *                  onItem.call(dojo.global(), item, request)
         *      }
         */
        loadItem: function(keywordArgs){        
            dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "loadItem('"+keywordArgs+"')", "Entering..."]);
            //console.dir(keywordArgs);
            
            var async = true;
            if(arguments.length == 2){
                async = arguments[1];
            }
            
            var item = this.indexedItems[keywordArgs.item.itemId];
            //console.dir(item);
            var error = null;
            if (!this.isItemLoaded(item)) {
                dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "loadItem('"+keywordArgs+"')", "Item wasn't loaded already, loading now. ("+(async ? "asynchronously" : "synchronously")+")"]);
                /*
                 * Make an xhr call to load the item with all the columns..
                 */
                dojo.xhrGet(this._buildItemRequest(item,
                        async, 
                        dojo.hitch(this, function (result){
                            dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "loadItem('"+keywordArgs+"')", "Loaded it..."]);

                            /*
                             * Munge the result.
                             */
                            for(var i = 0; i < result.properties.length; i=i+1){
                                item.properties[result.properties[i].propertyId.toLowerCase()] = result.properties[i];
                            }
                            item.properties.__id = {
                                propertyId: "__id",
                                value: result.itemId
                            };
                            item.loaded = true;

                            /*
                             * Notify the onLoad callback.
                             */
                            if (keywordArgs.onLoad && keywordArgs.onLoad instanceof Function) {
                                dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "loadItem('" + keywordArgs + "')", "Calling the onItem function in scope: " + keywordArgs.scope + "."]);
                                if (keywordArgs.scope) {
                                    try {
                                        keywordArgs.onLoad.call(keywordArgs.scope, item);
                                    } catch(e){
                                        throw e;
                                    }
                                } else {
                                    try {
                                        keywordArgs.onLoad.call(window, item);
                                    } catch (e){
                                        throw e;
                                    }
                                }
                            }
                        }), 
                        dojo.hitch(this, function (errorVal){
                            if(keywordArgs.onError && keywordArgs.onError instanceof Function){
                                if(keywordArgs.scope){
                                    keywordArgs.onError.call(keywordArgs.scope, errorVal);
                                } else {
                                    keywordArgs.onError.call(dojo.global(), errorVal);
                                }
                            }
                        })));
            }
            
            dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "loadItem('"+keywordArgs+"')", "Exiting..."]);
        },
    
        /**
         * Given a query and set of defined options, such as a start and count 
         * of items to return, this method executes the query and makes the 
         * results available as data items. The format and expectations of 
         * stores is that they operate in a generally asynchronous manner, 
         * therefore callbacks are always used to return items located by the 
         * fetch parameters.
         * 
         * exceptions:
         *     Throws an exception if the query is not valid, or if the query 
         *     is required but was not supplied.
         * 
         * @param {Object} keywordArgs
         *     The keywordArgs parameter may either be an instance of 
         *     conforming to dojo.data.api.Request or may be a simple anonymous 
         *     object that may contain any of the following:
         *     { 
         *         query: query-object or query-string, 
         *             { 
         *                 attrFoo: valueBar, 
         *                 attrFoo1: valueBar1
         *             }.  
         *         queryOptions: object,
         *             {
         *                 ignoreCase: boolean, 
         *                     Whether or not the query should match case 
         *                     sensitively or not.  Default behaviour is false.
         *                     
         *                 deep: boolean 
         *                     Whether or not a fetch should do a deep search 
         *                     of items and all child items instead of just 
         *                     root-level items in a datastore.  Default is 
         *                     false.
         *             }
         *         onBegin: Function(size, request), 
         *         onItem: Function(item, request),     
         *         onComplete: Function(items, request), 
         *         onError: Function(errorData, request),
         *         scope: object, 
         *         start: int, 
         *         count: int,
         *         sort: array 
         *             {
         *                 attribute: attribute || attribute-name-string, 
         *                 descending: true|false;   
         *                     Optional.  Default is false. 
         *             }
         *     }
         * @return
         */
        fetch: function(keywordArgs){
            dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "fetch('"+keywordArgs+"')", "Entering..."]);
            
            var request = new dojo.data.api.Request(); 
            request.query = keywordArgs.query;
            request.sort = keywordArgs.sort;
            request.start = keywordArgs.start;
            request.count = keywordArgs.count;
            request.scope = keywordArgs.scope;
            request.onBegin = keywordArgs.onBegin;
            request.onItem = keywordArgs.onItem;
            request.onComplete = keywordArgs.onComplete;
            request.onError = keywordArgs.onError;
	    if(keywordArgs.query){
		request.type = keywordArgs.query.type == "link" ? "link" : "item";
	    }else {
		request.type = "item";
	    }
            
            dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "fetch('"+keywordArgs+"')", "Filtering the keywordArgs..."]);
            
            
            if(keywordArgs.onBegin == null){
                keywordArgs.onBegin = function(total, request){};
            }
            //*/
            if(keywordArgs.onComplete == null){
                keywordArgs.onComplete = function(items, request){};
            }
            if(keywordArgs.onItem == null){
                keywordArgs.onItem = function(item, request){};
            }
            if(keywordArgs.onError == null){
                keywordArgs.onError = function(error, request){};
            }
            if(keywordArgs.start == null){
                keywordArgs.start = 0;
            }
            
            /*
             * Construct the request object.
             */
            dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "fetch('"+keywordArgs+"')", "Constructing the request block..."]);
            this.columns = ["all"]; // TODO: clean this up
	    try{
	    var buildFunc = dojo.hitch(this, request.type == "link" ? this._buildLinkPollingRequest : this._buildItemPollingRequest);
            var xhr = buildFunc(true, 
                    function(result){
                        dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "fetch('"+keywordArgs+"')", "Response Received..."]);

                        keywordArgs.onBegin.call(keywordArgs.scope, result.totalRows, request);
                        
			var identifier = result.identifier;
                        /*
                         * Parse the items.
                         */
                        var foundItems = new Array();
                        for(var i = 0; i < result.items.length; i=i+1){
                            var props = {};
                            for(var j = 0; j < result.items[i].properties.length; j=j+1){
                                props[result.items[i].properties[j].propertyId.toLowerCase()] = result.items[i].properties[j];
                            }
                            props.__id = {
                                propertyId: "__id",
                                value: result.items[i][identifier]
                            };
			    if(request.type == "link"){
				for(var j in result.items[i]){
				    if(j != "properties"){
					props[j] = {
					    propertyId: j,
					    value: result.items[i][j]
					};
				    }
				}
			    }
                            result.items[i].properties = props;
                            result.items[i].loaded = true;
                            result.items[i].expired = false;
                            result.items[i].changed = false;
			    result.items[i].navModelType = request.type;
                            foundItems.push(result.items[i]);
                            keywordArgs.onItem.call(keywordArgs.scope, result.items[i], request);
                        }
                        
                        /*
                         * Merge the found items into the cache.
                         */
                        if(!this.indexedItems){
                            this.indexedItems = {};
                        }
                        for(var i = 0; i < foundItems.length; i=i+1){
                            if(!this.indexedItems[foundItems[i][identifier]]){
                                this.indexedItems[foundItems[i][identifier]] = foundItems[i];
                            }
                        }
                        
                        keywordArgs.onComplete.call(keywordArgs.scope, result.items, request);
                    }, 
                    function(error){
                        keywordArgs.onError.call(keywordArgs.scope, error, request);
                    });
	    } catch (e){
		console.error(e);
	    }
            dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "fetch('"+keywordArgs+"')", "Setting the keywords..."]);
            xhr.content.start = keywordArgs.start;
            if (keywordArgs.count != Infinity && keywordArgs.count != null) {
                xhr.content.count = keywordArgs.count;
            }
            if (keywordArgs.query){
                var conditions = new Array();
                for(var i in keywordArgs.query){
                    if (typeof(keywordArgs.query[i]) == "object") {
                       var operator = keywordArgs.query[i].type ? keywordArgs.query[i].type : "=";
                        conditions.push(i + " "+operator+" '" + keywordArgs.query[i].value + "'");
                    } else if(i != "type"){
                        conditions.push(i + " = '" + keywordArgs.query[i] + "'");
                    }
                }
                xhr.content.condition = conditions.join(" AND ");
            }
            if(keywordArgs.sort){
                xhr.content.sort = "";
                var sortArray = new Array();
                for(var i = 0; i < keywordArgs.sort.length; i=i+1){
                    sortArray.push((keywordArgs.sort[i].descending ? "-" : "")+keywordArgs.sort[i].attribute);
                }
                xhr.content.sort = sortArray.join(",");
            }
            
            /*
             * Fire off the request.
             */
            dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "fetch('"+keywordArgs+"')", "Firing off the request..."]);
            dojo.xhrGet(xhr);
            
            return request; // an object conforming to the dojo.data.api.Request API
        },
    
        /**
         * The getFeatures() method returns an simple keyword values object 
         * that specifies what interface features the datastore implements.  
         * A simple CsvStore may be read-only, and the only feature it 
         * implements will be the 'dojo.data.api.Read' interface, so the
         * getFeatures() method will return an object like this one:
         * {'dojo.data.api.Read': true}.
         * A more sophisticated datastore might implement a variety of
         * interface features, like 'dojo.data.api.Read', 'dojo.data.api.Write', 
         * 'dojo.data.api.Identity', and 'dojo.data.api.Attribution'.
         */
        getFeatures: function(){
            return {
                'dojo.data.api.Read': true,
                'dojo.data.api.Identity': true,
		'ibm.tivoli.data.api.Hierarchy':true
            };
        },

        /**
         * The close() method is intended for instructing the store to 'close' 
         * out any information associated with a particular request.
         * 
         * The close() method is intended for instructing the store to 'close' 
         * out any information associated with a particular request.  In 
         * general, this APIexpects to recieve as a parameter a request object 
         * returned from a fetch.  It will then close out anything associated 
         * with that request, such as clearing any internal datastore caches 
         * and closing any 'open' connections.For some store implementations, 
         * this call may be a no-op.
         * 
         * example:
         *      var request = store.fetch({onComplete: doSomething});
         *      ...
         *      store.close(request);
         * 
         * @param {Object} 
         *      (dojo.data.api.Request || keywordArgs || null)request 
         *      An instance of a request for the store to use to identify what 
         *      to close out.If no request is passed, then the store should 
         *      clear all internal caches (if any)and close out all 'open' 
         *      connections.  It does not render the store unusable fromthere 
         *      on, it merely cleans out any current data and resets the store 
         *      to initial state.
         */
        close: function(/*dojo.data.api.Request || keywordArgs || null */ request){ 
            throw new Error('Unimplemented API: dojo.data.api.Read.close');
        },

        /**
         * Method to inspect the item and return a user-readable 'label' for the item
         * that provides a general/adequate description of what the item is.
         * 
         * Method to inspect the item and return a user-readable 'label' for the item
         * that provides a general/adequate description of what the item is.  In general
         * most labels will be a specific attribute value or collection of the attribute
         * values that combine to label the item in some manner.  For example for an item
         * that represents a person it may return the label as:  "firstname lastlame" where
         * the firstname and lastname are attributes on the item.  If the store is unable 
         * to determine an adequate human readable label, it should return undefined.  Users that wish
         * to customize how a store instance labels items should replace the getLabel() function on 
         * their instance of the store, or extend the store and replace the function in 
         * the extension class.
         * 
         * returns: 
         *  A user-readable string representing the item or undefined if no user-readable label can 
         *  be generated.
         * 
         * @param {Object} item The item to return the label for.
         */
        getLabel: function(item){
            if(!this.isItem(item)){
                throw new Error("Argument is not a valid item.");
            }
            
            // TODO maybe allow users to specify a label function on the constructor?
            
            return item.itemId;
        },
    
        /**
         * Method to inspect the item and return an array of what attributes of the item were used 
         * to generate its label, if any.
         * 
         * Method to inspect the item and return an array of what attributes of the item were used 
         * to generate its label, if any.  This function is to assist UI developers in knowing what
         * attributes can be ignored out of the attributes an item has when displaying it, in cases
         * where the UI is using the label as an overall identifer should they wish to hide 
         * redundant information.
         * 
         * returns:
         *  An array of attribute names that were used to generate the label, or null if public attributes 
         *  were not used to generate the label.
         * 
         * @param {Object} item  The item to return the list of label attributes for.
         */
        getLabelAttributes: function(item){     
            return null;
        },
    
        /**
         * Returns a unique identifier for an item.  The return value will be 
         * either a string or something that has a toString() method (such as, 
         * for example, a dojox.uuid.Uuid object).
         * 
         * exceptions: 
         *      Conforming implementations may throw an exception or return 
         *      null if item is not an item.
         * 
         * @param {Object} item
         *      The item from the store from which to obtain its identifier.
         */
        getIdentity: function(item){
            if(!this.isItem(item)){
                throw new Exception("Item argument is not a valid item.");
                return null;
            }
            var itemIdentityString = item.itemId == null ? item.linkId : item.itemId;
            return itemIdentityString; // string
        },
    
        /**
         * Returns an array of attribute names that are used to generate the 
         * identity. For most stores, this is a single attribute, but for some 
         * complex stores such as RDB backed stores that use compound (multi-
         * attribute) identifiers it can be more than one.  If the identity is 
         * not composed of attributes on the item, it will return null.  This 
         * function is intended to identify the attributes that comprise the 
         * identity so that so that during a render of all attributes, the UI 
         * can hide the the identity information if it chooses.
         * 
         * @param {Object} item
         *      The item from the store from which to obtain the array of 
         *      public attributes that compose the identifier, if any.
         */
        getIdentityAttributes: function(item){
            return null; // string
        },
    
        /**
         * Given the identity of an item, this method returns the item that 
         * has that identity through the onItem callback.  Conforming 
         * implementations should return null if there is no item with the 
         * given identity.
         * 
         * Implementations of fetchItemByIdentity() may sometimes return an 
         * item from a local cache and may sometimes fetch an item from a 
         * remote server,
         * 
         * @param {Object} keywordArgs
         *      An anonymous object that defines the item to locate and 
         *      callbacks to invoke when the item has been located and load has 
         *      completed.  The format of the object is as follows: 
         *      { 
         *          identity: string|object, 
         *              The identity parameter is the identity of the item you 
         *              wish to locate and load This attribute is required.  It 
         *              should be a string or an object that toString() can be 
         *              called on.
         *              
         *          onItem: Function(item), 
         *              The onItem parameter is the callback to invoke when the 
         *              item has been loaded.  It takes only one parameter, the 
         *              item located, or null if none found.
         *              
         *          onError: Function(error), 
         *              The onError parameter is the callback to invoke when 
         *              the item load encountered an error.  It takes only one 
         *              parameter, the error object
         *              
         *          scope: object 
         *              If a scope object is provided, all of the callback 
         *              functions (onItem, onError, etc) will be invoked in the 
         *              context of the scope object. In the body of the 
         *              callback function, the value of the "this" keyword will 
         *              be the scope object.   If no scope object is provided, 
         *              the callback functions will be called in the context of 
         *              dojo.global.
         *              
         *              For example, onItem.call(scope, item, request) vs. 
         *              onItem.call(dojo.global, item, request)
         *      }
         */
	fetchItemByIdentity: function(keywordArgs){     
	    /*
	     * if we were only given a string as the identity, assume they want 
	     * an 'item'
	     */
	    if((typeof(keywordArgs.identity)).toLowerCase() == "string"){
		var idstr = keywordArgs.identity;
		keywordArgs.identity = {
		    type: 'item',
		    id: idstr
		};
	    }
	    
	    if(!keywordArgs.identity.id){
		throw new Error("No id value was specified for the identity, an ID is required.");
	    }
	    if(!keywordArgs.identity.type){
		keywordArgs.identity.type = "item";
	    }
	    
	    if(!keywordArgs.onItem){
		keywordArgs.onItem = function(item){};
	    }
	    if(!keywordArgs.onError){
		keywordArgs.onError = function(error){throw new Error(error);};
	    }
	    if(!keywordArgs.scope){
		keywordArgs.scope = dojo.global;
	    }
	    
	    /*
	     * Actually get down to business.
	     * TODO use the indexedCache instead of the indexedItems for now on.
	     */
	    if(!this.indexedCache){
		this.indexedCache = {
		    items: {},
		    links: {},
		    tasks: {}
		};
	    }
	    var currentTime = (new Date()).getTime();
	    /*
	     * Check the cache.
	     */
	    if(this.indexedCache[keywordArgs.identity.type+"s"][keywordArgs.identity.id]){
	    var val = this.indexedCache[keywordArgs.identity.type+"s"][keywordArgs.identity.id];
	    if(val.cachetime > currentTime - this.cacheExpiration){
		/*
		 * we have a valid cache hit, call the onItem function
		 */ 
		keywordArgs.onItem.call(keywordArgs.scope, val);
		return;
	    }
	    }
        
	    /*
	     * When a cache miss occurrs, do a normal retrieval.
	     */
	    dojo.xhrGet(this._buildIdentityRequest(keywordArgs.identity.id, keywordArgs.identity.type, true, 
		dojo.hitch(this, function (result){
		    dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "fetchItemByIdentity('"+keywordArgs+"')", "Loaded it..."]);
	
		    var item = {};
		    for(var i in result){
			item[i] = result[i];
		    }
		    /*
		     * Munge the result.
		     */
		    item.properties = {};
		    for(var i = 0; i < result.properties.length; i=i+1){
			item.properties[result.properties[i].propertyId.toLowerCase()] = result.properties[i];
		    }
		    item.properties.__id = {
			propertyId: "__id",
			value: result.itemId
		    };
		    item.loaded = true;
		    item.cachetime = currentTime;
		    this.indexedCache[keywordArgs.identity.type+'s'][keywordArgs.identity.id] = item;
				
		    /*
		     * Notify the onLoad callback.
		     */
		    dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "fetchItemByIdentity('" + keywordArgs + "')", "Calling the onItem function in scope: " + keywordArgs.scope + "."]);
		    keywordArgs.onItem.call(keywordArgs.scope, item);
		}), 
		dojo.hitch(this, function(error){
		    dojo.publish(this._TOPIC_LOG, [this._logLevel, this.id, "fetchItemByIdentity('"+keywordArgs+"')", "Error Happened..."]);
		    keywordArgs.onError.call(keywordArgs.scope, error);
		})));
        }
    });
    
    ibm.tivoli.data.NavModelItemLinkStore.TOPIC_LOG = "ibm.tivoli.data.NavModelItemLinkStore.log";
    
})(dojo);
