/**
 * Adds some extra functionality to dojo...
 * Utilities.
 *
 * @author Jason Feinstein (jwfeinst@us.ibm.com)
 */
(function(dojo){
    dojo.provide("ibm.tivoli.data.Utilities");
    
    /*
     * Check if the plugin methods exist already and warn if so.
     */
    if(dojo.advancedMixin != null || dojo.isObjectObject != null){
        throw new Error("Could not extend dojo to add advancedMixin and isObjectObject functionality in ibm.tivol.data.Utilities.js: Functions already exist.");
    }
    
    dojo.mixin(dojo, {
        /**
         * Checks if an object is an object (and not an array or null or 
         * something)... yes, I was surprised dojo didn't have this already too.
         *
         * @param
         *     Object/variable to check.
         * @return
         *     Whether or not the candidate is an object.
         */
        isObjectObject: function(candidate){
            return Object.prototype.toString.call(candidate) == "[object Object]";
        },
        
        /**
         * Deep-mixin of objects in dojo.
         *
         * @param 
         *     Target object.
         * @param
         *     Source object.
         * @param [optional]
         *     Boolean - should we do a deep mixin?
         * @return 
         *     Mixed in target.
         */
        advancedMixin: function (){
            var target = arguments[0];
            var source = arguments[1];
            
            var deep = true;
            if(arguments.length === 3){
                deep = arguments[2];
            }
            
            if(!deep){
                return dojo.mixin(target,source);
            }
        
            if(!target){target = {};}
            if(!source){source = {};}
            
            for(var attr in source){
                if(dojo.isFunction(source[attr])){
                    target[attr] = source[attr];
                } else if(dojo.isArray(source[attr])) {
                    target[attr] = source[attr];
                } else if(dojo.isObjectObject(source[attr])){
                    if(target[attr] === null){
                        target[attr] = {};
                    }
                    target[attr] = dojo.advancedMixin(target[attr], source[attr], true);
                } else {
                    target[attr] = source[attr];
                }
            }
            return target;
        }
    });
})(dojo);