
/**
 * This class is responsible for providing the mechanisms to make calls to
 * a NavModel REST API from within an iWidget Environment. It simply extends
 * the AJAXNavModelStore and overrides the _build___URL methods to introduce a 
 * call to  iContextIO.rewriteURI().
 *
 * @author Jason Feinstein (jwfeinst@us.ibm.com)
 */
(function(dojo){
    dojo.provide("ibm.tivoli.data.iWidgetNavModelStore");
    dojo.require("ibm.tivoli.data._NavModelStore");
    dojo.require("ibm.tivoli.data.AJAXNavModelStore");
    dojo.require("ibm.tivoli.data.Utilities");
    
    dojo.declare("ibm.tivoli.data.iWidgetNavModelStore",[ibm.tivoli.data.AJAXNavModelStore],{
        iContextIO: null,
    
        
        /**
         * Gets the implementation-specific settings for the implementing NavModelStore.
         */
        _getNavModelSettings: function(){
            var settings = {};
            settings.baseUrl = this.baseUrl;
            settings.provider = this.provider;
            settings.dataset = this.dataset;
            settings.datasource = this.datasource;
            settings.parameters = this.parameters;
            settings.defaultProperties = this.defaultProperties;
            settings.iContextIO = this.iContextIO;
            return settings;
        },
        
        _buildFetchURL: function(settings){
            if(settings.query != null && settings.query.navModel != null){
                var itemTypes = settings.query.navModel.itemTypes;
                if(itemTypes[0] === "item" && settings.query.navModel.origin === true){
                    return this.iContextIO.rewriteURI(this.baseUrl + "/providers/"+this.provider+(this.datasource !== "" ?  ("/datasources/" + this.datasource) : "") + "/datasets/"+ this.dataset + "/originItems");
                } else if(itemTypes.length == 1){
                    return this.iContextIO.rewriteURI(this.baseUrl + "/providers/"+this.provider+(this.datasource !== "" ?  ("/datasources/" + this.datasource) : "") + "/datasets/"+ this.dataset + "/"+itemTypes[0]+"s");
                } 
            }
            return this.iContextIO.rewriteURI(this.baseUrl + "/providers/"+this.provider+(this.datasource !== "" ?  ("/datasources/" + this.datasource) : "") + "/datasets/"+ this.dataset);            
        },
        
                
        _buildRelatedItemsURL: function(settings){
            if(settings.sourceItem == null){
                return this.iContextIO.rewriteURI(this.baseUrl + "/providers/"+this.provider+(this.datasource !== "" ?  ("/datasources/" + this.datasource) : "") + "/datasets/"+ this.dataset + "/originItems");
            } else {
                return this.iContextIO.rewriteURI(this.baseUrl + settings.sourceItem.relatedItemsUri);
            }
        },
        
        _buildGetElementURL: function(settings){
            return this.iContextIO.rewriteURI(this.baseUrl + settings.item.uri);
        },
        
        /**
         * Build a new instance of an implementation of _NavModelStore, based on 
         * a child dataset of the dataset described by this data store.
         *
         * @param item
         *      Must be a valid Child Dataset "item" object.  If not, an 
         *      exception will be thrown.
         * @return 
         *      New instance of a _NavModelStore implementation (the same 
         *      implementation as the calling object)
         */
        createDataStoreFromChildDataset: function(item){
            if(!this._isChildDatasetItem(item)){
                throw new Error("Specified item is not a valid child dataset.");
            }
            
            var id = item.id;
            var uriRegex = /\/providers\/([a-zA-Z0-9\._-]+)(\/datasources\/([a-zA-Z0-9\._-]+))?\/datasets\/([a-zA-Z0-9\._-]+)/;
            var match = uriRegex.exec(item.uri);
            var provider = match[1] === undefined ? "" : match[1];
            var datasource = match[3] === undefined ? "" : match[3];
            var dataset = match[4] === undefined ? "" : match[4];
            
            return new (eval(this.declaredClass))({
                iContextIO: this.iContextIO,
                dataset: dataset,
                baseUrl: this.baseUrl,
                datasource: datasource,
                provider: provider,
                defaultProperties: this.defaultProperties,
                parameters: this.parameters
            });
        }
    });
})(dojo);