/**
 * This file contains the unit tests for the Utilities 'plugin'
 * @author Jason Feinstein (jwfeinst@us.ibm.com)
 */

//Declare out the name of the test module to make dojo's module loader happy.
dojo.provide("ibm.tivoli.data.tests.Utilities");
dojo.require("ibm.tivoli.data.Utilities");

/**
 * Makes sure that the element ID decoder function works correctly.
 */
doh.registerGroup("Utilities.advancedMixin",
    [
        function test(t){
            var target = {
                a: 1,
                b: true,
                c: null,
                d: function(){
                    return '0';
                },
                e: {
                    f: {
                        l: -1,
                        m: -2,
                        n: -3,
                        o: -4
                    },
                    g: 1,
                    h: 2,
                    i: 3,
                    j: 4
                }
            };
            var source = {
                test: {
                    key : "value"
                },
                c: "not null anymore!",
                e: { f: {
                    l: 1000,
                    other: "other"
                }}
            };
            dojo.advancedMixin(target, source, true);
            
            /*
             * Make sure things were kept intact.
             */
            t.assertEqual(1, target.a);
            t.assertEqual(true, target.b);
            t.assertTrue(Object.prototype.toString.call(target.d) == "[object Function]");
            t.assertEqual(-2, target.e.f.m);
            t.assertEqual(-3, target.e.f.n);
            t.assertEqual(-4, target.e.f.o);
            t.assertEqual(1, target.e.g);
            t.assertEqual(2, target.e.h);
            t.assertEqual(3, target.e.i);
            t.assertEqual(4, target.e.j);
            
            /*
             * Make sure things got overwritten/created correctly
             */
            t.assertEqual(source.e.f.l, target.e.f.l);
            t.assertEqual(source.e.f.other, target.e.f.other);
            t.assertEqual(source.test.key, target.test.key);
            t.assertEqual(source.c, target.c);
        }
    ],
    function(){}, function(){}
);