/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.AbstractDataset"); 
 
// includes
dojo.require("ibm.tivoli.navmodel.Item");
dojo.require("ibm.tivoli.navmodel.ItemStyle");
dojo.require("ibm.tivoli.navmodel.Link");
dojo.require("ibm.tivoli.navmodel.LinkStyle");
dojo.require("ibm.tivoli.navmodel.Relationship");
dojo.require("ibm.tivoli.navmodel.Property");
dojo.require("ibm.tivoli.navmodel.Task");
dojo.require("ibm.tivoli.navmodel.Status");
dojo.require("ibm.tivoli.navmodel.Column");
dojo.require("ibm.tivoli.navmodel.EventRouter");
dojo.require("ibm.tivoli.navmodel.Event");
dojo.require("ibm.tivoli.navmodel.Selection");
dojo.require("ibm.tivoli.navmodel.NavWidgetDef");
dojo.require("ibm.tivoli.navmodel.TopoWidgetDef");
dojo.require("ibm.tivoli.navmodel.TableWidgetDef");
dojo.require("ibm.tivoli.navmodel.NotebookWidgetDef");
dojo.require("ibm.tivoli.navmodel.Exception");
dojo.require("ibm.tivoli.navmodel.Utils");
dojo.require("ibm.tivoli.navmodel.Trace");

ibm.tivoli.navmodel.AbstractDataset._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.AbstractDataset");

////////////////////////////////////
// AbstractDataset
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.AbstractDataset", null, {
	// private members       
	_args: null,
	
	// dataset attributes
	_dataset: null,
	_version: null,
	
	// widget definition supplied by server
	_widgetDefs: null,
	
	// item, link, relationship, caches
	_itemsById: null,
	_linksById: null,
	_relationshipsById: null,
	_cacheIntervalId: null,
	
	// styles and status caches
	_itemStyles: null,
	_itemStylesById: null,
	_itemStatus: null,
	_itemStatusById: null,	
	_linkStyles: null,
	_linkStylesById: null,
	_linkStatus: null,
	_linkStatusById: null,
	_relStyles: null,
	_relStylesById: null,
	_relStatus: null,
	_relStatusById: null,
	_status: null,
		
	// miscellaneous
	_toolbar: null,
	_eventRouter: null,
	_listeners: null,
	_destroyed: false,
   _remoteKeepAliveIntervalId: null,     // 255243
	_trace: ibm.tivoli.navmodel.AbstractDataset._trace, 
	
	/*=====
	constructor_onComplete: function(dataset, kwArgs){},
	=====*/
	
	/*=====
	constructor_onPage: function(start, count, kwArgs){},
	=====*/
	
	/*=====
	constructor_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	 constructor_kwArgs: function(onComplete, onError, onPage, baseUrl, provider, datasource, dataset, 
	 					parameters, defaultProperties, defaultLinkProperties, defaultRelProperties, 
	 					cacheThreshold, cacheValidationInterval, cacheExpirationTime, version, eventRouter) {
	 	// onComplete: constructor_onComplete
	 	//		callback to be invoked when the Dataset is initialized and ready for methods
	 	//		to be invoked  (i.e. asynchronous initialization must be performed). 
	 	//		Note: A null value will result in synchronous initialization.
	 	// onError: constructor_onError
	 	//		callback to be invoked if there is a problem constructing the Dataset
	 	// onPage:  constructor_onPage
	 	//		callback to be invoked when a new page is requested
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// dataset: String
		//		dataset id
		// parameters: Object
		//		optional: extra parameters sent to dataset provider
		// defaultProperties: String[]
		//		optional: item property ids to fetch by default (can specify "all")
		// defaultLinkProperties: String[]
		//		optional: link property ids to fetch by default (can specify "all")
		// defaultRelProperties: String[]
		//		optional: relationship property ids to fetch by default (can specify "all")
		// cacheThreshold: int
		//		optional: cache objects if number of dataset items >= threshold
		// cacheValidationInterval: int
		//		optional: number of minutes in between cache validations
		// cacheExpirationTime: int
		//		optional: number of minutes objects can stay in cache while inactive
		// version: Float
		//		optional: version used to communicate with rest service (i.e. json version)
		// eventRouter: Object
		//		optional: object contains the following values (defaults shown)
		//		eventRouter: {
		//			timeout : 20,    		* number of seconds to wait for events  (optional)
		//			pollDelay : 0,			* number of ms to wait after an error before next poll  (optional)
		//			pollErrorDelay : 5000	* number of ms to wait after an error before next poll  (optional)
	   //		},	   	
		//		Use With Caution:  If you are the first Dataset to create an EventRouter object, the parameters 
		// 		specified here will dictate timeout, pollDelay and pollErrorDelay for ALL Datasets in the browser. 
  		//		If a Dataset has already been constructed, these parameters will have NO effect.  If the Dataset 
  		//		is destroyed, the event router will shutdown and new Dataset instances and their settings will be used.	
      // datasetImplementation: String
      //    optional: dataset data table type - 
      //                ibm.tivoli.navmodel.AbstractDataset.implementation.SIMPLE
      //                ibm.tivoli.navmodel.AbstractDataset.implementation.HYBRID
      //             or ibm.tivoli.navmodel.AbstractDataset.implementation.LARGE
      //                (default is ibm.tivoli.navmodel.AbstractDataset.implementation.SIMPLE)
      // remote: boolean
      //    optional: indicates whether the data provider is remote or local (default is false)
      // remoteKeepAliveInterval: int
      //    optional: number of minutes in between sending a request to keep a connection alive
      //              with a provider (used only if the provider is a remote provider)
	 },
	=====*/
	
	constructor: function(/*constructor_kwArgs*/ kwArgs, /*Object?*/ dataset) {
		// summary:
		//		Constructs AbstractDataset object.
		// description:
		//		The following options are normally not included in constructor (internal use only)
		//			createEventRouter: boolean - True indicates that a new event router should be created (vs. shared)
		//			dataset: object - Optional dataset object; will fetch from server if not provided
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		this._args = {
			onComplete: null,
			onError: null,
			onPage: null,
			baseUrl: "/ibm/tivoli/rest",
			provider: "",
			datasource: "",
			dataset: "",
			parameters: {},
			defaultProperties: [],
			defaultLinkProperties: [],
			defaultRelProperties: [],
			cacheThreshold:	250,
			cacheValidationInterval: 5,
			cacheExpirationTime: 15,
			version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion(),
			
			// The following options are normally not included in constructor
			createEventRouter: true,
         fetchMetaData: true,
         datasetImplementation: ibm.tivoli.navmodel.AbstractDataset.implementation.SIMPLE,
         remote: false,                   // 255243
         remoteKeepAliveInterval: 15      // 255243
		};
		dojo.mixin(this._args, kwArgs);
		
		// init empty caches
		this._itemsById = {};
		this._linksById = {};
		this._relationshipsById = {};
	
		// add param_pageId to help build unique event router queue: <user>_<sessionId>_<pageId>
		this._args.parameters.pageId = ibm.tivoli.navmodel.EventRouter.getPageId();
		
		// add param_refId to make sure server dataset is unique
		if(typeof(this._args.parameters.refId) == "undefined"){ 
			this._args.parameters.refId = Math.random();
		}
		// empty refId supplied; so don't include in parameters
		else if (this._args.parameters.refId.length === 0) {
			delete this._args.parameters.refId;
		}

		// add param_tz for the user timezone
		if (!this._args.parameters.tz) {
			this._args.parameters.tz = ibm.tivoli.navmodel.Utils.getTimeZone();
		}
		
		// subclass can initialize the dataset
		var initArgs = {
			onComplete: this._args.onComplete,
			onError: this._args.onError,
			onPage: this._args.onPage,
			optimize: true,
			dataset: dataset,
			createEventRouter: this._args.createEventRouter,
         fetchMetaData: this._args.fetchMetaData
		};
		this._initDataset(initArgs);		
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	refresh: function() {
		// summary:
		//		This method forces the dataset object to do a full refresh of its 
		//		data and caches
		
		// clear caches, refresh, and notify listeners
		this.clientChangedEvent();
	},

	destroy: function() {
		// summary:
		//		This method should be called to clean up the Dataset object
		
		if (!this._destroyed) {
			// cancel the cache validation timer
			if (this._cacheIntervalId) {
				clearInterval(this._cacheIntervalId);
			}
			
         // cancel the remote keep alive timer
         if (this._remoteKeepAliveIntervalId) {            // 255243
            clearInterval(this._remoteKeepAliveIntervalId);
         }
			
			// do any clean up like unregistering from event router, etc
			this._cleanupDataset();
			
			// destroy widget definition if we have one
			if (this._widgetDefs) {
				var length = this._widgetDefs.length;
				for (var index = 0;  index < length;  ++index) {
					if (this._widgetDefs[index]) {
						this._widgetDefs[index].destroy();
					}
				}								
			}
			
			this._destroyed = true;
		}
	},
		
	getArgs: function() {
		// summary:
		//		Returns the initial arguments passed to the dataset
		return dojo.clone(this._args);
	},

	getArg: function(key) {
		// summary:
		//		Returns a specific initial argument passed to the dataset
		return this._args[key];
	},

	getAttribute: function(key) {
		// summary:
		//		Returns general dataset attribute for requested key
		return this._dataset[key];
	},

	getAttributes: function(keyPrefix) {
		// summary:
		//		Returns attribute map for all keys that begin with keyPrefix;  if no
		//		keyPrefix is supplied; all attributes are returned
		var attr = {};		
		if (keyPrefix) {
			for (var key in this._dataset) {
				if (this._dataset.hasOwnProperty(key)) {
					if (key.indexOf(keyPrefix) === 0) {
						attr[key] = this._dataset[key];
					}			
				}
			}
		}
		else {
			dojo.mixin(attr, this._dataset);
		}
		
		return attr;
	},

	setAttribute: function(key, value) {
		// summary:
		//		Set general dataset attribute for requested key
		this._dataset[key] = value;		
	},

	mixinAttributes: function(attr) {
		// summary:
		//		Mixin general dataset attributes
		dojo.mixin(this._dataset, attr);		
	},
	
	isClearCacheOnRefresh: function() {
		// summary:
		//		Returns true if the dataset is requesting that
		//		?clearCache be placed on url when doing a Refresh;
		//		which will cause CURI cache to purge the dataset and reconstruct		
		return (!this._dataset.eventsEnabled || this._dataset.clearCacheOnRefresh);
	},
	
	isEventsEnabled: function() {
		// summary:
		//		Returns true if the dataset is capable of producing events
		return this._dataset.eventsEnabled;
	},

	getEventListenerCount: function() {
		// summary:
		//		Returns number of registered event listeners
		var listenerCount = 0;
		if (this._listeners) {
			listenerCount = this._listeners.length;
		}
		
		return listenerCount;
	},
	
	/*=====
	_listenerCallback: function(listenerObj, event) {
		// listenerObj
		// event
	},
	=====*/

	addEventListener: function(/*Object*/ listenerObj, /*_listenerCallback*/ listenerCallback) {
		// summary:
		//		Adds an event listener to the Dataset
		// listenerObj:
		//		unique object (i.e. same object must be passed to removeEventListener)
		// listenerCallback
		//		listener callback function for receiving the events
		
		// remove listener if it already exists
		this.removeEventListener(listenerObj);
			
		// add new listener
		if (!this._listeners) {
			this._listeners = [];
		}		
		this._listeners.push({listenerObj: listenerObj, listenerCallback: listenerCallback});
	},
	
	removeEventListener: function(/*Object*/ listenerObj) {
		// summary:
		//		Removes an event listener from the Dataset
		// listernObj:
		//		previously registered listener object
		if (this._listeners) {
			var listenerLength = this._listeners.length;
			for (var listenerIndex = 0;  listenerIndex < listenerLength;  ++listenerIndex) {
				if (this._listeners[listenerIndex].listenerObj === listenerObj) {
					// remove listener
					this._listeners.splice(listenerIndex, 1);
					
					// last listener so null out array					
					if (this._listeners.length === 0) {
						this._listeners = null;
					}
					
					break;
				}
			}	
		}
	},	

	getVersion: function() {
		// summary:
		//		Returns the version used to communicate with rest service
		//		(i.e. json version)
		return this._version;
	},
	
	_setVersion: function(version) {
		// summary:
		//		Sets the version used to communicate with the rest servce
		//		(i.e. json version)
		this._version = version;
	},

	getId: function() {
		// summary:
		//		Returns the dataset id
		return this._dataset.id;
	},

	setId: function(id) {
		// summary:
		//		Set the dataset id
		this._dataset.id = id;
	},

	getProviderId: function() {
		// summary:
		//		Returns the provider id
		return this._dataset.providerId;
	},

	setProviderId: function(providerId) {
		// summary:
		//		Set the provider id
		this._dataset.providerId = providerId;
	},

	getDatasourceId: function() {
		// summary:
		//		Returns the datasource id
		return this._dataset.datasourceId;
	},
	
	setDatasourceId: function(datasourceId) {
		// summary:
		//		Set the datasource id
		this._dataset.datasourceId = datasourceId;
	},

	getEventId: function() {
		// summary:
		//		Returns the event id
		return this._dataset.eventId;
	},

	setEventId: function(eventId) {
		// summary:
		//		Set the event id
		this._dataset.eventId = eventId;
	},

	getType: function() {
		// summary:
		//		Returns the dataset type
		return this._dataset.type;
	},

	setType: function(type) {
		// summary:
		//		Set the dataset type
		this._dataset.type = type;		
	},

	getViewType: function() {
		// summary:
		//		Returns the preferred view type for rendering
		//		
		//		values: "graph", "table", "tree"
		return this._dataset.viewType;
	},

	setViewType: function(viewType) {
		// summary:
		//		Set the preferred view type for rendering
		this._dataset.viewType = viewType;		
	},

   getDatasetImplementation: function() {        // 255211
       // summary:
       //		Returns the dataset implementation type
       //		
       //		values: ibm.tivoli.navmodel.AbstractDataset.implementation.SIMPLE, 
       //              ibm.tivoli.navmodel.AbstractDataset.implementation.HYBRID,
       //              ibm.tivoli.navmodel.AbstractDataset.implementation.LARGE_TABLE
       return this._dataset.datasetImplementation;
   },

   setDatasetImplementation: function(implementationType) {     // 255211
       // summary:
       //		Set the dataset implementation type
       this._dataset.datasetImplementation = implementationType;		
   },
	
	isGraph: function() {
		// summary:
		//		Returns true if the dataset represents a graph
		return this.getViewType() === "graph";
	},

	isTable: function() {
		// summary:
		//		Returns true if the dataset represents a table
		return this.getViewType() === "table";
	},

	isTree: function() {
		// summary:
		//		Returns true if the dataset represents a tree
		return this.getViewType() === "tree";
	},

	isLazyLoad: function() {
		// summary:
		//		Returns true if the dataset designates that it should be lazy loaded
		return (this._dataset.lazyLoad !== undefined ? this._dataset.lazyLoad : false);
	},
	
	setLazyLoad: function(lazyLoad) {
		// summary:
		//		Set lazy load attribute for the dataset
		this._dataset.lazyLoad = lazyLoad;
	},
	
	getLazyLoadRelationships: function() {		
		// summary:
		//		Returns the relationships that should be explored for related items
		return this._dataset.relationshipsKey;
	},

	setLazyLoadRelationships: function(relationshipsKey) {		
		// summary:
		//		Set the relationships that should be explored for related items
		this._dataset.relationshipsKey = relationshipsKey;
	},
	
	getLabel: function() {	
		// summary:
		//		Returns globalized label for the dataset
		return this._dataset.label;
	},

	setLabel: function(label) {
		// summary:
		//		Set globalized label for the dataset
		this._dataset.label = label;		
	},

	getLabelQualifier: function() {
		// summary:
		//		Returns globalized label qualifier that can be used in conjunction with the label
		//		i.e.    label (labelQualifier)
		return this._dataset.labelQualifier;
	},

	setLabelQualifier: function(labelQualifier) {
		// summary:
		//		Set globalized label qualifier that can be used in conjunction with the label
		this._dataset.labelQualifier = labelQualifier;		
	},
	
	/*=====
	depth_kwArgs1: function(widgetDef) {
		// widgetDef: Object
		//		widget definition can override the depth value
	},
	=====*/

	getDepth: function(/*depth_kwArgs*/ kwArgs) {
		// summary:
		//		Returns the default depth for the dataset if it is a graph
		//		(otherwise -1)
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		if (args.widgetDef) {
			return args.widgetDef.getDepth();
		}
				
		return (this._dataset.depth ? this._dataset.depth : -1);		
	},

	setDepth: function(depth) {
		// summary:
		//		Set the depth for the dataset if it is a graph
		// depth:
		//		New topology depth
		
		// update depth parameter
		var found = false;
		for (var parm in this._args.parameters) {
			if (parm === "depth" && this._args.parameters.hasOwnProperty(parm)) {
				this._args.parameters[parm] = depth;
				found = true;
				break;
			}
		}
		if (!found) {
			this._args.parameters.depth = depth;
		}
		
		// clear caches, refresh, and notify listeners
		var event = ibm.tivoli.navmodel.ClientEvent.getChangedEvent(this);
		this._eventHandler(event);		
	},

	getMaxDepth: function(/*depth_kwArgs*/ kwArgs) {
		// summary:
		//		Returns the maximum depth for the dataset if it is a graph
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		if (args.widgetDef) {
			return args.widgetDef.getMaxDepth();
		}
		
		return (this._dataset.maxDepth ? this._dataset.maxDepth : -1);		
	},

	getDefaultProperties: function() {
		// summary:
		//		Returns default properties that are fetched with each item
		return this._args.defaultProperties;
	},
	
	setDefaultProperties: function(itemProperties) {
		// summary:
		//		Sets default properties that are fetched with each item
		// itemProperties:
		//		Array of item property ids
		this._args.defaultProperties = itemProperties;
	},

	getDefaultLinkProperties: function() {
		// summary:
		//		Returns default properties that are fetched with each link
		return this._args.defaultLinkProperties;
	},
	
	setDefaultLinkProperties: function(linkProperties) {
		// summary:
		//		Sets default properties that are fetched with each link
		// linkProperties:
		//		Array of link properties
		this._args.defaultLinkProperties = linkProperties;
	},

	getDefaultRelProperties: function() {
		// summary:
		//		Returns default properties that are fetched with each relationship
		return this._args.defaultRelProperties;
	},
	
	setDefaultRelProperties: function(relProperties) {
		// summary:
		//		Sets default properties that are fetched with each relationship
		// relProperties:
		//		Array of relationship properties
		this._args.defaultRelProperties = relProperties;
	},
	
	cacheExpires: function() {
		// summary:
		//		Returns true if the items, links, and relationships cache expires for this dataset.  If
		//		there are only a small number of objects the dataset will not bother with expiration.
		return (this._cacheIntervalId ? true : false);
	},

	getNewExpirationTime: function() {
		// summary:
		//		Returns an expiration time for a new item, link, or relationship
		var now = new Date();
		return now.getTime() + (this._args.cacheExpirationTime*60000);
	},
	
	getItemCount: function() {
		// summary:
		//		Returns the number of items associated with the dataset
		return this._dataset.itemCount;
	},
	
	/*=====
	itemsAndLinks_onComplete: function(items, links, kwArgs){},
	=====*/
	
	/*=====
	_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	getItemsAndLinks_kwArgs: function(onComplete, onError, sync) {
		// onComplete: itemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getItemsAndLinks: function(/*getItemsAndLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns both the items and links associated with the dataset
		var args = {                
			onComplete: function(items, links, kwArgs) {},
			onError: function(error, exception, ioArgs) {},
			sync: false
		};   
		dojo.mixin(args, kwArgs);
		
		// fetch items and links from rest service
		var onComplete = args.onComplete;		
		args.onComplete = dojo.hitch(this, function(items, links, toolbar, args) {
			// cache items and links
			items = this._cacheItems(items);
			links = this._cacheLinks(links);
				
			// cache toolbar tasks				
			this._toolbar = toolbar;
				
			// call orignal callback
			onComplete(items, links, args);
		});			
		this._fetchItemsAndLinks(args);		
	},
	
	/*=====
	items_onComplete: function(items, kwArgs){},
	=====*/

	/*=====
	getItems_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions 
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getItems: function(/*getItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items associated with the dataset
		var args = {                
			onComplete: function(items, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			start: 0,
			count: null,
			properties: this._args.defaultProperties,
			metadata: false, 
			sort: null,
			condition: null,
			sync: false
		};   
		dojo.mixin(args, kwArgs);         
		
		///////////////////////////////////////////////////////////
		// make sure we always return asynchronously in case we are 
		// being invoked by a dojo datastore
		///////////////////////////////////////////////////////////
		
		// are NOT any items
		if (this.getItemCount() === 0) {		
			args.numRows = 0;
			args.filteredRows = 0;
			args.totalRows = 0;				
			if (args.sync) {
				args.onComplete(null, args);
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, null, args), 0);
			}			
		}
		// fetch items from rest service
		else {
			var onComplete = args.onComplete;		
			args.onComplete = dojo.hitch(this, function(items, args) {
				// cache items
				items = this._cacheItems(items);
				
				// call orignal callback
				onComplete(items, args);
			});
			this._fetchItems(args);
		}		
	},
	
	hasCachedItems: function() {
		// summary:
		//		indicates if dataset currently contains cached items			
		if (this._itemsById) {
			for (var id in this._itemsById) { 
    			if (this._itemsById.hasOwnProperty(id)) {
    				return true;
    			}
    		}
		}
		
		return false;
	},
	
	getCachedItems: function() {
		// summary:
		//		returns array with currently cached items	
		var items = [];
		this._traverseCachedItems(items);
		return items;
	},
	
	_traverseCachedItems: function(/*ibm.tivoli.navmodel.Item[]*/ items) {
		if (this._itemsById) {
			for (var id in this._itemsById) { 
    			if (this._itemsById.hasOwnProperty(id)) {
    				items.push(this._itemsById[id]);
    			}
    		}
		}
	},
	
	/*=====
	getItem_kwArgs: function(itemId, properties, metadata) {
		// itemId: String
		//		item id to get
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/

	getItem: function(/*getItem_kwArgs*/ kwArgs) {
		// summary:
		//		Returns the requested item
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var args = {
			itemId: null,
			properties: null,
			metadata: false
		};
		dojo.mixin(args, kwArgs);
		
		// check cache
		var item = this._getCachedItem(args.itemId);
		
		// force a fetch if we do not have all the requested properties		
		if (item && ((args.metadata && !item.hasCachedMetadata()) || (args.properties && !item.hasCachedProperties(args.properties)))) {
			item = null;			
		}
		
		// need to fetch individual item
		if (!item) {
			if (!args.properties || args.properties.length === 0) {
				args.properties = this._args.defaultProperties;
			}
			
			item = this._fetchItem(args);
			item = this._cacheItem(item);			
		}	

		return item;
	},
	
	/*=====
	getOriginItems_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: itemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metdata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getOriginItems: function(/*getOriginItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns orign items of the dataset
		var args = {                
			onComplete: function(items, links, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			start: 0,
			count: null,
			properties: this._args.defaultProperties,
			metadata: false,
			sort: null,
			condition: null,
			sync: false
		};   
		dojo.mixin(args, kwArgs);
		
		var onComplete = args.onComplete;		
		args.onComplete = dojo.hitch(this, function(items, args) {
			// cache items
			items = this._cacheItems(items, true);
				
			// call orignal callback
			onComplete(items, args);
		});
		this._fetchOriginItems(args);
	},
	
	/*=====
	getRelatedItems_kwArgs: function(onComplete, onError, itemId, relationships, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: itemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemId: String
		//		id of the base item
		// relationships: String
		//		comma delimited list of relationships to determine related items; 
		//		can also be keywords: child, parent, all
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getRelatedItems: function(/*getRelatedItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items that are related to the requested item
		var args = {                
			onComplete: function(items, links, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			itemId: null,
			relationships: null,
			start: 0,
			count: null,                
			properties: this._args.defaultProperties,
			metadata: false,
			sort: null,
			condition: null,
			sync: false
		};   
		dojo.mixin(args, kwArgs);
		
		var onComplete = args.onComplete;		
		args.onComplete = dojo.hitch(this, function(items, links, args) {
			// cache items and links
			items = this._cacheItems(items, true);
			links = this._cacheLinks(links);			
				
			// if no related items were returned; reset lazyLoad indicators
			if (!items || items.length === 0) {
				var item = this._getCachedItem(kwArgs.itemId);
				if (item) {
					item.resetLazyLoad();
				}
			}
			
			// call orignal callback
			onComplete(items, links, args);
		});		
		this._fetchRelatedItems(args);
	},

	/*=====
	queryItems_kwArgs: function(onComplete, onError, query, start, count, properties, metadata, sync) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// query: Object
		//		custom query object to be sent as request body
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	queryItems: function(/*queryItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items associated with the dataset that match
		//		the provided custom query
		var args = {                
			onComplete: function(items, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			query: {},                
			start: 0,
			count: null,
			properties: this._args.defaultProperties,
			metadata: false,
			sync: false
		};   
		dojo.mixin(args, kwArgs);         
		
		var onComplete = args.onComplete;		
		args.onComplete = dojo.hitch(this, function(items, args) {
			// cache items
			items = this._cacheItems(items);
						
			// call orignal callback
			onComplete(items, args);
		});		
		this._queryItems(args);	
	},

	getLinkCount: function() {
		// summary:
		//		Returns the number of links associated with the dataset
		return this._dataset.linkCount;
	},

	/*=====
	links_onComplete: function(items, kwArgs){},
	=====*/

	/*=====
	getLinks_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: links_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions
		// start: int
		//		start index to fetch
		// count: int
		//		number of links to fetch; null means all
		// properties: String[]
		//		link property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	getLinks: function(/*getLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns links associated with the dataset
		var args = {                
			onComplete: function(links, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			start: 0,
			count: null,
			properties: this._args.defaultLinkProperties,
			metadata: false,
			sort: null,
			condition: null,
			sync: false
		};   
		dojo.mixin(args, kwArgs);         

		// are NOT any links
		if (this.getLinkCount() === 0) {
			args.numRows = 0;
			args.filteredRows = 0;
			args.totalRows = 0;				
			if (args.sync) {
				args.onComplete(null, args);
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, null, args), 0);
			}
		}
		// fetch links from rest service
		else {
			var onComplete = args.onComplete;		
			args.onComplete = dojo.hitch(this, function(links, args) {
				// cache links
				links = this._cacheLinks(links);
				
				// call orignal callback
				onComplete(links, args);
			});
			this._fetchLinks(args);
		}
	},
	
	hasCachedLinks: function() {
		// summary:
		//		indicates if dataset currently contains cached links			
		if (this._linksById) {
			for (var id in this._linksById) { 
    			if (this._linksById.hasOwnProperty(id)) {
    				return true;
    			}
    		}
		}
		
		return false;
	},
	
	getCachedLinks: function() {
		// summary:
		//		returns array with currently cached links		
		var links = [];
		this._traverseCachedLinks(links);		
		return links;
	},
	
	_traverseCachedLinks: function(/*ibm.tivoli.navmodel.Link[]*/ links) {
		if (this._linksById) {
			for (var id in this._linksById) { 
    			if (this._linksById.hasOwnProperty(id)) {
    				links.push(this._linksById[id]);
    			}
    		}
		}
	},

	/*=====
	getLink_kwArgs: function(itemId, properties, metadata) {
		// linkId: String
		//		link id to get
		// properties: String[]
		//		array of property ids to pre-fetch
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/
	
	getLink: function(/*getLink_kwArgs*/ kwArgs) {
		// summary:
		//		Returns the requested link
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var args = {
			linkId: null,
			properties: null,
			metadata: false
		};
		dojo.mixin(args, kwArgs);
		
		// check cache
		var link = this._getCachedLink(args.linkId);
		
		// force a fetch if we do not have all the requested properties		
		if (link && ((args.metadata && !link.hasCachedMetadata()) || (args.properties && !link.hasCachedProperties(args.properties)))) {
			link = null;			
		}
			
		// need to fetch individual link
		if (!link) {
			if (!args.properties || args.properties.length === 0) {
				args.properties = this._args.defaultLinkProperties;
			}
			
			link = this._fetchLink(args);
			link = this._cacheLink(link);			
		}	
		
		return link;
	},

	getRelationshipCount: function() {
		// summary:
		//		Returns the number of items associated with the dataset
		return this._dataset.relationshipCount;
	},
	
	/*=====
	relationships_onComplete: function(relationships, kwArgs){},
	=====*/
	
	/*=====
	getRelationships_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: relationships_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions
		// start: int
		//		start index to fetch
		// count: int
		//		number of relationships to fetch; null means all
		// properties: String[]
		//		relationship property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getRelationships: function(/*getRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Returns relationships associated with the dataset
		var args = {                
			onComplete: function(relationships, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			start: 0,
			count: null,
			properties: this._args.defaultRelProperties,
			metadata: false,
			sort: null,
			condition: null,
			sync: false
		};   
		dojo.mixin(args, kwArgs);         

		///////////////////////////////////////////////////////////
		// make sure we always return asynchronously in case we are 
		// being invoked by a dojo datastore
		///////////////////////////////////////////////////////////
		
		// are NOT any links/relationships
		if (this.getLinkCount() === 0) {
			args.numRows = 0;
			args.filteredRows = 0;
			args.totalRows = 0;				
			if (args.sync) {
				args.onComplete(null, args);
			}
			else {		
				setTimeout(dojo.hitch(this, args.onComplete, null, args), 0);
			}
		}
		// fetch relationships from rest service
		else {
			var onComplete = args.onComplete;		
			args.onComplete = dojo.hitch(this, function(relationships, args) {
				// cache relationships
				relationships = this._cacheRelationships(relationships);
				
				// call orignal callback
				onComplete(relationships, args);
			});
			this._fetchRelationships(args);
		}
	},
	
	hasCachedRelationships: function() {
		// summary:
		//		indicates if dataset currently contains cached relationships
		if (this._relationshipsById) {
			for (var id in this._relationshipsById) { 
    			if (this._relationshipsById.hasOwnProperty(id)) {
    				return true;
    			}
    		}
		}
		
		return false;
	},
	
	getCachedRelationships: function() {
		// summary:
		//		return array with currently cached relationships	
		var relationships = [];
		this._traverseCachedRelationships(relationships);
		return relationships;
	},
	
	_traverseCachedRelationships: function(/*ibm.tivoli.navmodel.Relationship[]*/ relationships) {
		if (this._relationshipsById) {
			for (var id in this._relationshipsById) { 
    			if (this._relationshipsById.hasOwnProperty(id)) {
    				relationships.push(this._relationshipsById[id]);
    			}
    		}
		}
	},
	
	/*=====
	getRelationship_kwArgs: function(relationshipId, properties, metadata) {
		// relationshipId: String
		//		relationship id to fetch
		// properties: String[]
		//		array of property ids to pre-fetch
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/
	
	getRelationship: function(/*getRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Returns the requested relationship
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var args = {
			relationshipId: null,
			properties: null,
			metadata: false
		};
		dojo.mixin(args, kwArgs);
		
		// check cache
		var relationship = this._getCachedRelationship(args.relationshipId);

		// force a fetch if we do not have all the requested properties		
		if (relationship && ((args.metadata && !relationship.hasCachedMetadata()) || (args.properties && !relationship.hasCachedProperties(args.properties)))) {
			relationship = null;			
		}
						
		// need to fetch individual relationship
		if (!relationship) {
			if (!args.properties || args.properties.length === 0) {
				args.properties = this._args.defaultRelProperties;
			}
			
			relationship = this._fetchRelationship(args);
			relationship = this._cacheRelationship(relationship);
		}	
		
		return relationship;
	},
	
	/*=====
	tasks_onComplete: function(tasks, kwArgs){},
	=====*/
	
	/*=====
	getTasks_kwArgs: function(onComplete, onError, taskCategory, selection, itemIds, linkIds, itemIndexes, sync) {
		// onComplete: tasks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// taskCategory: String
		//		comma delimited list indicating type of tasks to fetch
		//		legal values are: menu, toolbar, and breadcrumb
		// selection: Object
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// itemIds: String
		//		comma delimited list of item ids that are selected
		// linkIds, String
		//		comma delimited list of link ids that are selected
		// itemIndexes: String
		//		comma delimited list of item ids or ranges that are selected
		//		ex: "0,1,10,20" or "0-10,30,40-50"
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getTasks: function(/*getTasks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns tasks associated with the dataset
		var args = {
			onComplete: function(tasks, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			taskCategory: "menu",
			selection: null,
			itemIds: null,
			linkIds: null,
			itemIndexes: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		// asking for "toolbar" without any selection and we previously
		// fetched the toolbar tasks with the items and links
		if (args.taskCategory === "toolbar" && this._toolbar &&
			(!args.selection || args.selection.isEmpty()) && !args.itemIds && !args.linkIds && !args.itemIndexes) {
			if (args.sync) {
				args.onComplete(this._toolbar, args);
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, this._toolbar, args), 0);
			}		
		}		
		// fetch tasks from rest service
		else {
			this._fetchTasks(args);
		}
	},

	/*=====
	dataset_onComplete: function(dataset, kwArgs){},
	=====*/
	
	/*=====
	navigate_kwArgs: function(onComplete, onError, taskId, taskCategory, selection, sync) {
		// onComplete: dataset_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// taskId: String
		//		navigation task id
		// taskCategory: String
		//		navigation task category
		// selection: Object
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	navigate: function(/*navigate_kwArgs*/ kwArgs) {
		// summary:
		//		Navigate to another Dataset from a navigation task
		var args = {                
			onComplete: function(dataset, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},                
			taskId: null,
			taskCategory: "menu:navigation",
			selection: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		this._navigate(args);     
	},

	/*=====
	itemStyles_onComplete: function(itemStyles, kwArgs){},
	=====*/
	
	/*=====
	getItemStyles_kwArgs: function(onComplete, onError, start, count, widgetDef, sync) {
		// onComplete: itemStyles_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of item styles to fetch;  null means all
		// widgetDef: Object
		//		widget definition that can override attributes
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getItemStyles: function(/*getItemStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current item styles for the Dataset
		var args = {
			onComplete: function(itemStyles, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			start: 0,
			count: null,
			widgetDef: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		// already have the item styles
		if (this._itemStyles) {
			if (args.widgetDef) {
				if (args.sync) {
					args.onComplete(this._mergeItemStyles(this._itemStyles, args.widgetDef), args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, this._mergeItemStyles(this._itemStyles, args.widgetDef), args), 0);
				}
			}
			else if (args.sync) {
				args.onComplete(this._itemStyles, args);
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, this._itemStyles, args), 0);
			}			
		}
		// need to fetch the item styles
		else {
			var onComplete = args.onComplete;		
			args.onComplete = dojo.hitch(this, function(itemStyles, args) {
				// cache styles
				if (args.start === 0 && args.count === null) {
					this._cacheItemStyles(itemStyles);					
				}
				
				// call orignal callback
				if (args.widgetDef) {
					onComplete(this._mergeItemStyles(itemStyles, args.widgetDef), args);
				}
				else {
					onComplete(itemStyles, args);
				}				
			});			
			this._fetchItemStyles(args);
		}
	},
	
	/*=====
	getItemStyle_kwArgs: function(itemType, widgetDef) {
		// itemType: String
		//		type of itemStyle
		// widgetDef: Object
		//		widget definition that can override attributes
	},
	=====*/

	getItemStyle: function(/*getItemStyle_kwArgs*/ kwArgs) {
		// summary:
		//		Returns item style for requested itemType
		var itemStyle = null;
		var args = {
			itemType: "",
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		// if we don't have item styles fetch them
		if (!this._itemStyles) {
			var fetchArgs = {
				onComplete: dojo.hitch(this, function(itemStyles, args){
					this._cacheItemStyles(itemStyles);					
				}),
				onError: function(error, exception, ioArgs){
				},
				start: 0,
				count: null,
				sync: true
			};
			this._fetchItemStyles(fetchArgs);
		}
		
		// locate requested item style in cache
		if (this._itemStylesById) {
			itemStyle = this._itemStylesById[args.itemType];
		}
		
		if (args.widgetDef) {
			// clone and merge widget definition attributes				
			if (itemStyle) {
				itemStyle = itemStyle.clone(args.widgetDef);					
			}
			// unknown style to dataset; see if it exists in widget definition
			else {
				var widgetStyle = args.widgetDef.getItemStyle(args.itemType);
				if (widgetStyle) {
					itemStyle = this._createItemStyle(args.widgetDef, widgetStyle);
				}					
			}
		}
		
		return itemStyle;
	},
	
	/*=====
	linkStyles_onComplete: function(linkStyles, kwArgs){},
	=====*/
	
	/*=====
	getLinkStyles_kwArgs: function(onComplete, onError, start, count, widgetDef, sync) {
		// onComplete: linkStyles_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of link styles to fetch; null means all
		// widgetDef: Object
		//		widget definition that can override attributes
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getLinkStyles: function(/*getLinkStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current link styles for the Dataset
		var args = {
			onComplete: function(linkStyles, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			start: 0,
			count: null,
			widgetDef: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		// already have the link styles
		if (this._linkStyles) {
			if (args.widgetDef) {
				if (args.sync) {
					args.onComplete(this._mergeLinkStyles(this._linkStyles, args.widgetDef, "link"), args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, this._mergeLinkStyles(this._linkStyles, args.widgetDef, "link"), args), 0);
				}
			}
			else if (args.sync) {
				args.onComplete(this._linkStyles, args);
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, this._linkStyles, args), 0);
			}
		}
		// need to fetch the link styles
		else {
			var onComplete = args.onComplete;		
			args.onComplete = dojo.hitch(this, function(linkStyles, args) {
				// cache styles
				if (args.start === 0 && args.count === null) {
					this._cacheLinkStyles(linkStyles);					
				}
				
				// call orignal callback
				if (args.widgetDef) {
					onComplete(this._mergeLinkStyles(linkStyles, args.widgetDef, "link"), args);
				}
				else {
					onComplete(linkStyles, args);
				}				
			});
			this._fetchLinkStyles(args);
		}
	},
	
	/*=====
	getLinkStyle_kwArgs: function(linkType, widgetDef) {
		// linkType: String
		//		type of linkStyle
		// widgetDef: Object
		//		widget definition that can override attributes
	},
	=====*/
	
	getLinkStyle: function(/*getLinkStyle_kwArgs*/ kwArgs) {
		// summary:
		//		Returns link style for requested linkType
		var linkStyle = null;
		var args = {
			linkType: "",
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		// if we don't have link styles fetch them
		if (!this._linkStyles) {
			var fetchArgs = {
				onComplete: dojo.hitch(this, function(linkStyles, args){
					this._cacheLinkStyles(linkStyles);
				}),
				onError: function(error, exception, ioArgs){
				},
				sync: true
			};
			this._fetchLinkStyles(fetchArgs);
		}
		
		// locate requested link style in cache
		if (this._linkStylesById) {
			linkStyle = this._linkStylesById[args.linkType];
		}
		
		if (args.widgetDef) {
			// clone and merge widget definition attributes				
			if (linkStyle) {
				linkStyle = linkStyle.clone(args.widgetDef, "link");					
			}
			// unknown style to dataset; see if it exists in widget definition
			else {
				var widgetStyle = args.widgetDef.getLinkStyle(args.linkType);
				if (widgetStyle) {
					linkStyle = this._createLinkStyle(args.widgetDef, widgetStyle, "link");
				}
			}
		}
				
		return linkStyle;
	},
	
	/*=====
	relStyles_onComplete: function(relStyles, kwArgs){},
	=====*/
	
	/*=====
	getRelationshipStyles_kwArgs: function(onComplete, onError, start, count, widgetDef, sync) {
		// onComplete: relStyles_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of relationship styles to fetch; null means all
		// widgetDef: Object
		//		widget definition that can override attributes
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getRelationshipStyles: function(/*getRelationshipStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current relationship styles for the Dataset
		var args = {
			onComplete: function(relStyles, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			start: 0,
			count: null,
			widgetDef: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		// already have the relationship styles
		if (this._relStyles) {
			if (args.widgetDef) {
				if (args.sync) {
					args.onComplete(this._mergeLinkStyles(this._relStyles, args.widgetDef, "relationship"), args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, this._mergeLinkStyles(this._relStyles, args.widgetDef, "relationship"), args), 0);
				}
			}
			else if (args.sync) {
				args.onComplete(this._relStyles, args);
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, this._relStyles, args), 0);
			}
		}
		// need to fetch the relationship styles
		else {
			var onComplete = args.onComplete;		
			args.onComplete = dojo.hitch(this, function(relStyles, args) {
				// cache styles
				if (args.start === 0 && args.count === null) {
					this._cacheRelationshipStyles(relStyles);					
				}
				
				// call orignal callback
				if (args.widgetDef) {
					onComplete(this._mergeLinkStyles(relStyles, args.widgetDef, "relationship"), args);
				}
				else {
					onComplete(relStyles, args);
				}				
			});
			this._fetchRelationshipStyles(args);
		}
	},
	
	/*=====
	getRelationshipStyle_kwArgs: function(relationshipType, widgetDef) {
		// relationshipType: String
		//		type of relationshipStyle
		// widgetDef: Object
		//		widget definition that can override attributes
	},
	=====*/

	getRelationshipStyle: function(/*getRelationshipStyle_kwArgs*/ kwArgs) {
		// summary:
		//		Returns relationship style for requested relationshipType
		var relStyle = null;
		var args = {
			relationshipType: "",
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		// if we don't have relationship styles fetch them
		if (!this._relStyles) {
			var fetchArgs = {
				onComplete: dojo.hitch(this, function(relStyles, args){
					this._cacheRelationshipStyles(relStyles);
				}),
				onError: function(error, exception, ioArgs){
				},
				sync: true
			};
			this._fetchRelationshipStyles(fetchArgs);
		}
		
		// locate requested relationship style in cache
		if (this._relStylesById) {
			relStyle = this._relStylesById[args.relationshipType];
		}
		
		if (args.widgetDef) {
			// clone and merge widget definition attributes				
			if (relStyle) {
				relStyle = relStyle.clone(args.widgetDef, "relationship");					
			}
			// unknown style to dataset; see if it exists in widget definition
			else {
				var widgetStyle = args.widgetDef.getRelationshipStyle(args.relationshipType);
				if (widgetStyle) {
					relStyle = this._createLinkStyle(args.widgetDef, widgetStyle, "relationship");
				}
			}
		}	
		
		return relStyle;
	},
		
	/*=====
	statuses_onComplete: function(statuses, kwArgs){},
	=====*/
	
	/*=====
	getStatuses_kwArgs: function(onComplete, onError, type, start, count, widgetDef, sync) {
		// onComplete: statuses_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// type: String
		//		object type: "item", "link", "relationship" 
		// start: int
		//		start index to fetch
		// count: int
		//		number of status values to fetch; null means all
		// widgetDef: Object
		//		widget definition that can override attributes
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getStatuses: function(/*getStatuses_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of possible status values for the object type
		var args = {
			onComplete: function(statuses, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			type: "item",
			start: 0,
			count: null,
			widgetDef: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		var statuses = null;
		var statusesById = null;
		switch(args.type) {
			case "item":
				statuses = this._itemStatus;
				statusesById = this._itemStatusById;
				break;
			case "link":
				statuses = this._linkStatus;
				statusesById = this._linkStatusById;
				break;
			case "relationship":
				statuses = this._relStatus;
				statusesById = this._relStatusById;			
		}
		
		// already have the status values
		if (statuses) {
			if (args.widgetDef) {
				if (args.sync) {
					args.onComplete(this._mergeStatuses(statuses, args.widgetDef, args.type), args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, this._mergeStatuses(statuses, args.widgetDef, args.type), args), 0);
				}
			}
			else if (args.sync) {
				args.onComplete(statuses, args);
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, statuses, args), 0);
			}			
		}
		// need to fetch the status values
		else {
			var onComplete = args.onComplete;
			args.onComplete = dojo.hitch(this, function(fetchedStatuses, args) {
				// cache statuses
				if (args.start === 0 && args.count === null) {
					switch(args.type) {
						case "item":
							this._cacheItemStatuses(fetchedStatuses);
							break;
					case "link":
							this._cacheLinkStatuses(fetchedStatuses);
							break;
					case "relationship":
							this._cacheRelationshipStatuses(fetchedStatuses);
					}
				}
				
				// call orignal callback
				if (args.widgetDef) {
					onComplete(this._mergeStatuses(fetchedStatuses, args.widgetDef, args.type), args);
				}
				else {
					onComplete(fetchedStatuses, args);
				}
			});
			this._fetchStatuses(args);
		}
	},
	
	/*=====
	getStatus_kwArgs: function(type, statusId, statusValue, widgetDef) {
		// type: String
		//		object type: "item", "link", "relationship" 
		// statusId: String
		//		id of the status
		// statusValue: int
		//		value of the status
		// widgetDef: Object
		//		widget definition that can override attributes
	},
	=====*/

	getStatus: function(/*getStatus_kwArgs*/ kwArgs) {
		// summary:
		//		Returns status for requested status id and value
		var status = null;
		var args = {
			type: "item",
			statusId: "",
			statusValue: 0,
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		var statuses = null;
		switch(args.type) {
			case "item":
				statuses = this._itemStatus;
				break;
			case "link":
				statuses = this._linkStatus;
				break;
			case "relationship":
				statuses = this._relStatus;
		}
		
		// if we don't have statuses fetch them
		if (!statuses) {
			var fetchArgs = {
				onComplete: dojo.hitch(this, function(fetchedStatuses, args){
					switch(args.type) {
						case "item":
							this._cacheItemStatuses(fetchedStatuses);
							break;
						case "link":
							this._cacheLinkStatuses(fetchedStatuses);
							break;
						case "relationship":
							this._cacheRelationshipStatuses(fetchedStatuses);
					}					
				}),
				onError: function(error, exception, ioArgs){
				},
				type: args.type,
				start: 0,
				count: null,
				sync: true
			};
			this._fetchStatuses(fetchArgs);
		}
		
		var statusesById = null;
		switch(args.type) {
			case "item":
				statusesById = this._itemStatusById;
				break;
			case "link":
				statusesById = this._linkStatusById;
				break;
			case "relationship":
				statusesById = this._relStatusById;			
		}
		
		// locate requested item status in cache
		if (statusesById) {
			var id = args.statusId + "_" + args.statusValue;
			status = statusesById[id];
		}
		
		if (status && args.widgetDef) {
			status = status.clone(widgetDef, args.type);			
		}
		
		return status;
	},
	
	/*=====
	cols_onComplete: function(cols, kwArgs){},
	=====*/
	
	/*=====
	getColumns_kwArgs: function(onComplete, onError, type, properties, start, count, metadata, sync) {
		// onComplete: cols_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// type: String
		//		object type: "item", "link", "relationship" 
		// properties: String[]
		//		array of column ids; or keyword "current", "default", or "all"
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// metadata: Boolean
		//		indicates if filter metadata should be returned with columns
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	getColumns: function(/*getColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current columns for the Dataset
		//		(i.e. the ones that should currently be dispayed)
		var args = {
			onComplete: function(cols, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			source: "dataset",
			type: "item",
			properties: ["current"],
			start: 0,
			count: null,
			metadata: true,
			sync: false
		};
		dojo.mixin(args, kwArgs);
				
		this._fetchColumns(args);	
	},
	
	/*=====
	setColumns_kwArgs: function(onComplete, onError, columns, type, sync) {
		// onComplete: cols_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// columns: ibm.tivoli.navmodel.Column[]
		//		array of columns to be saved
		// type: String
		//		object type: "item", "link", "relationship" 
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	setColumns: function(/*setColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Set list of current columns for the Dataset
		//		(with updated attributes for width, sort, etc)
		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			columns: null,
			type: "item",
			sync: false
		};
		dojo.mixin(args, kwArgs);
				
		this._setColumns(args);	
	},
	
	keepAlive: function(/*keepAlive_kwArgs*/ kwArgs) {       // 255243
		// summary:
		//		Makes a request to keep a connection to a (remote) data provider alive

		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			sync: false
		};
		dojo.mixin(args, kwArgs);
				
		this._doKeepAlive(args);	
	},
	
	hasWidgetDefs: function() {
		// summary:
		//		Returns true if the dataset defines widget definitions	
		return (this._dataset.widgetDefinitions && dojo.isArray(this._dataset.widgetDefinitions) && this._dataset.widgetDefinitions.length > 0);
	},
	
	/*=====
	getWidgetDef_kwArgs: function() {
		// widgetType: String
		//		the type of widget def (i.e. "navwidget", "topology", "table", etc)
	},
	=====*/
	
	getWidgetDef: function(/*getWidgetDef_kwArgs*/ kwArgs) {
		// summary:
		//		Returns widget definition for the requested widget type
		var args = {
			widgetType: null
		};
		dojo.mixin(args, kwArgs);
		
		return this._getWidgetDef(this._getWidgetDefIndex(args.widgetType));		
	},
	
	getWidgetDefs: function() {
		// summary:
		//		Returns all widget definitions defined by the dataset
		var widgetDefs = null;		
		if (this.hasWidgetDefs()) {
			widgetDefs = [];			
			var length = this._dataset.widgetDefinitions.length;
			for (var index = 0;  index < length;  ++index) {
				var widgetDef = this._getWidgetDef(index);
				if (widgetDef) {
					widgetDefs.push(widgetDef);
				}
			}
		}
		
		return widgetDefs;
	},
	
	/*=====
	item_onComplete: function(item, kwArgs){},
	=====*/
	
	/*=====
	createItem_kwArgs: function(onComplete, onError, item) {
		// onComplete: item_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// item: Object
		//		ibm.tivoli.navmodel.Item to be created on server
	},
	=====*/

	createItem: function(/*createItem_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new item in the dataset
		var args = {
			onComplete: function(item, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			item: null
		};
		dojo.mixin(args, kwArgs);
		
		this._createItem(args);
	},
	
	/*=====
	createItems_kwArgs: function(onComplete, onError, item) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// item: Object[]
		//		array of ibm.tivoli.navmodel.Item to be created on server
	},
	=====*/

	createItems: function(/*createItems_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new items in the dataset
		var args = {
			onComplete: function(items, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			items: []
		};
		dojo.mixin(args, kwArgs);
		
		this._createItems(args);
	},
	
	/*=====
	_onComplete: function(kwArgs){},
	=====*/
	
	/*=====
	deleteItem_kwArgs: function(onComplete, onError, itemId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemId: String
		//		id of item to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of item
	},
	=====*/

	deleteItem: function(/*deleteItem_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a item from the dataset
		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			itemId: null,
			remove: false
		};
		dojo.mixin(args, kwArgs);
		
		this._deleteItem(args);
	},
	
	/*=====
	deleteItems_kwArgs: function(onComplete, onError, itemIds, condition, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemIds: String[]
		//		array of item ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of items
	},
	=====*/

	deleteItems: function(/*deleteItems_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete items from the dataset
		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			itemIds: [],
			condition: null,
			remove: false
		};
		dojo.mixin(args, kwArgs);
		
		this._deleteItems(args);
	},
	
	/*=====
	link_onComplete: function(link, kwArgs){},
	=====*/
	
	/*=====
	createLink_kwArgs: function(onComplete, onError, link) {
		// onComplete: link_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// link: Object
		//		ibm.tivoli.navmodel.Link to be created on server
	},
	=====*/

	createLink: function(/*createLink_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new link in the dataset
		var args = {
			onComplete: function(link, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			link: null
		};
		dojo.mixin(args, kwArgs);
		
		this._createLink(args);	
	},
	
	/*=====
	createLinks_kwArgs: function(onComplete, onError, links) {
		// onComplete: links_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// links: Object[]
		//		array of ibm.tivoli.navmodel.Link to be created on server
	},
	=====*/

	createLinks: function(/*createLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new links in the dataset
		var args = {
			onComplete: function(links, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			links: []
		};
		dojo.mixin(args, kwArgs);
		
		this._createLinks(args);	
	},
	
	/*=====
	deleteLink_kwArgs: function(onComplete, onError, linkId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// linkId: String
		//		id of link to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of link
	},
	=====*/

	deleteLink: function(/*deleteLink_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a link from the dataset
		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			linkId: null,
			remove: false
		};
		dojo.mixin(args, kwArgs);
		
		this._deleteLink(args);
	},
	
	/*=====
	deleteLinks_kwArgs: function(onComplete, onError, linkIds, condition, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// linkIds: String[]
		//		array of link ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of links
	},
	=====*/

	deleteLinks: function(/*deleteLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete links from the dataset
		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			linkIds: [],
			condition: null,
			remove: false
		};
		dojo.mixin(args, kwArgs);
		
		this._deleteLinks(args);
	},
	
	/*=====
	relationship_onComplete: function(relationship, kwArgs){},
	=====*/
	
	/*=====
	createRelationship_kwArgs: function(onComplete, onError, relationship) {
		// onComplete: relationship_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationship: Object
		//		ibm.tivoli.navmodel.Relationship to be created on server
	},
	=====*/
	
	createRelationship: function(/*createRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new relationship in the dataset
		var args = {
			onComplete: function(relationship, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			relationship: null
		};
		dojo.mixin(args, kwArgs);
		
		this._createRelationship(args);	
	},
	
	/*=====
	createRelationships_kwArgs: function(onComplete, onError, relationships) {
		// onComplete: relationships_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationship: Object[]
		//		array of ibm.tivoli.navmodel.Relationship to be created on server
	},
	=====*/
	
	createRelationships: function(/*createRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new relationships in the dataset
		var args = {
			onComplete: function(relationships, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			relationships: []
		};
		dojo.mixin(args, kwArgs);
		
		this._createRelationships(args);	
	},
	
	/*=====
	deleteRelationship_kwArgs: function(onComplete, onError, relationshipId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationshipId: String
		//		id of relationship to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of relationship
	},
	=====*/

	deleteRelationship: function(/*deleteRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a relationship from the dataset
		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			relatonshipId: null,
			remove: false
		};
		dojo.mixin(args, kwArgs);
		
		this._deleteRelationship(args);
	},
	
	/*=====
	deleteRelationships_kwArgs: function(onComplete, onError, relationshipIds, condition, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationshipIds: String[]
		//		array of relationship ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of relationships
	},
	=====*/

	deleteRelationships: function(/*deleteRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete relationships from the dataset
		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			relatonshipIds: [],
			condition: null,
			remove: false
		};
		dojo.mixin(args, kwArgs);
		
		this._deleteRelationships(args);
	},

	clientChangedEvent: function() {
		// summary:
		//		Fire client side "changed" event
		var event = ibm.tivoli.navmodel.ClientEvent.getChangedEvent(this);
		this._eventHandler(event);	
	},

	clientItemsAddedEvent: function(items) {
		// summary:
		//		Fire client side "itemsAdded" event
		var event = ibm.tivoli.navmodel.ClientEvent.getItemsAddedEvent(this, items);
		this._eventHandler(event);
	},

	clientItemsRemovedEvent: function(items) {
		// summary:
		//		Fire client side "itemsRemoved" event
		var event = ibm.tivoli.navmodel.ClientEvent.getItemsRemovedEvent(this, items);
		this._eventHandler(event);
	},

	clientItemStatusChangedEvent: function(items) {
		// summary:
		//		Fire client side "itemStatusChanged" event
		var event = ibm.tivoli.navmodel.ClientEvent.getItemStatusChangedEvent(this, items);
		this._eventHandler(event);	
	},

	clientItemPropertiesChangedEvent: function(items) {
		// summary:
		//		Fire client side "itemPropertiesChanged" event
		var event = ibm.tivoli.navmodel.ClientEvent.getItemPropertiesChangedEvent(this, items);
		this._eventHandler(event);	
	},

	clientLinksAddedEvent: function(links) {
		// summary:
		//		Fire client side "linksAdded" event
		var event = ibm.tivoli.navmodel.ClientEvent.getLinksAddedEvent(this, items);
		this._eventHandler(event);
	},

	clientLinksRemovedEvent: function(links) {
		// summary:
		//		Fire client side "linksRemoved" event
		var event = ibm.tivoli.navmodel.ClientEvent.getLinksRemovedEvent(this, links);
		this._eventHandler(event);
	},

	clientLinkStatusChangedEvent: function(links) {
		// summary:
		//		Fire client side "linkStatusChanged" event
		var event = ibm.tivoli.navmodel.ClientEvent.getLinkStatusChangedEvent(this, links);
		this._eventHandler(event);
	},

	clientLinkPropertiesChangedEvent: function(links) {
		// summary:
		//		Fire client side "linkPropertiesChanged" event
		var event = ibm.tivoli.navmodel.ClientEvent.getLinkPropertiesChangedEvent(this, links);
		this._eventHandler(event);
	},

	clientLabelChangedEvent: function(newLabel) {
		// summary:
		//		Fire client side "labelChanged" event
		var event = ibm.tivoli.navmodel.ClientEvent.getLinkPropertiesChangedEvent(this, newLabel);
		this._eventHandler(event);	
	},
		
	/////////////////////////////
	// protected methods
	/////////////////////////////	
	_initEmptyDataset: function() {
		// summary:
		//		Reset internal dataset to an empty state
		this._version = this._args.version;		
		this._dataset = {};
		this._dataset.providerId = this._args.provider;
		this._dataset.datasourceId = this._args.datasource;
      this._dataset.datasetImplementation = this._args.datasetImplementation;    // 255243
		this._dataset.id = this._args.dataset;
		this._dataset.eventId = this._dataset.id;
		this._dataset.eventsEnabled = false;
		this._dataset.viewType = "graph";		
		this._dataset.itemCount = 0;
		this._dataset.linkCount = 0;
		this._dataset.relationshipCount = 0;
      this._dataset.remote = this._args.remote;   // 255243
		
		// establish our base dataset uri
		this._dataset.uri  = "/providers/" + encodeURIComponent(this._dataset.providerId);
		this._dataset.uri += (this._dataset.datasourceId !== "" ? ("/datasources/" + encodeURIComponent(this._dataset.datasourceId)) : "");
		this._dataset.uri += "/datasets/" + encodeURIComponent(this._dataset.id);		
	},

	_cleanupDataset: function() {
		// summary:
		//		Clean up by removing listener and attempting a deleteDataset if no other listeners
		
		// unregister from event router
		if (this._eventRouter) {
			var remainingListeners = this._eventRouter.removeListener(this.getEventId());
				
			// ok to delete dataset from REST service cache
			if (remainingListeners === 0) {
				this._deleteDataset();
			}
			
			this._eventRouter = null;
		}
		// no event router;  just do the delete
		else {
			this._deleteDataset();
		}
		
		// clear all item and link caches
		this._clearAllCaches();		
		this._initEmptyDataset();
	},

	_getWidgetDefIndex: function(/*String*/ widgetType) {
		// summary:
		//		Returns the index into widgetDefinitions array for the requested widgetType		
		if (widgetType && this.hasWidgetDefs()) {
			var length = this._dataset.widgetDefinitions.length;
			for (var index = 0;  index < length;  ++index) {
				var entry = this._dataset.widgetDefinitions[index];
				if (entry) {
					// entry should be in the form of  <widget type>:<widget id>:<widget class>
					var parts = entry.split(":");
					if (parts && parts.length == 3 && parts[0] === widgetType) {
						return index;
					}
				}
			}
		}
		
		return -1;
	},
	
	_getWidgetDef: function(/*int*/ index) {
		// summary:
		//		Return WidgetDef for the requested index
		if (!this._widgetDefs && this.hasWidgetDefs()) {
			this._widgetDefs = [];			
			var widgetDefsLength = this._dataset.widgetDefinitions.length;
			for (var widgetDefIndex = 0;  widgetDefIndex < widgetDefsLength;  ++widgetDefIndex) {
				this._widgetDefs[widgetDefIndex] = null;
			}
		}
		
		if (!this._widgetDefs || index < 0 || index >= this._widgetDefs.length) {
			return null;
		}
		
		if (!this._widgetDefs[index]) {
			var parts = this._dataset.widgetDefinitions[index].split(":");
			var widgetType = parts[0];
			var widgetId = parts[1];
			var widgetClass = parts[2];
				
			var objectClass = dojo.getObject(widgetClass);
			if (objectClass) {
				var args = {
					baseUrl: this._args.baseUrl,
					provider: this.getProviderId(),
					datasource: this.getDatasourceId(),
					widget: widgetType,
					definition: widgetId,
					type: "user",
					version: this.getVersion()
				};					
					
				try {
					this._widgetDefs[index] = new objectClass(args);
				}
				catch(ex) {
					this._trace.error("Exception constructing widget definition: ex=" + ex, "_getWidgetDef");					
				}
			}
			else {
				this._trace.error("Cannot instantiate widget def class=" + widgetClass, "_initWidgetDef");				
			}
		}
			
		return this._widgetDefs[index];		
	},

	_setEventRouter: function(eventRouter) {
		// summary:
		//		Associate our dataset with an existing EventRouter object
		if (eventRouter && this.isEventsEnabled()) {
			this._eventRouter = eventRouter;
			this._eventRouter.addListener(this.getEventId(), dojo.hitch(this, this._jsonEventHandler));
		}
	},

	_clearEventRouter: function() {
		// summary:
		//		Clear EventRouter for our dataset
		if (this._eventRouter) {
			this._eventRouter.removeListener(this.getEventId());
			this._eventRouter = null;
		}
	},

	_getEventRouter: function() {
		// summary:
		//		Returns current event router
		return this._eventRouter;
	},
	
	_setItemCount: function(itemCount) {
		// summary:
		//		Set internal dataset item count
		this._dataset.itemCount = itemCount;
	},

	_setLinkCount: function(linkCount) {
		// summary:
		//		Set internal dataset link count
		this._dataset.linkCount = linkCount;
	},
	
	_setRelationshipCount: function(relationshipCount) {
		// summary:
		//		Set internal dataset relationship count
		this._dataset.relationshipCount = relationshipCount;
	},

	_clearAllCaches: function() {
		// summary:
		//		Clear all item and link related caches
		this._clearItemCache();
		this._clearLinkCache();
		this._clearRelationshipCache();
		this._clearToolbarCache();
		this._clearItemStyleCache();
		this._clearItemStatusCache();
		this._clearLinkStyleCache();
		this._clearLinkStatusCache();
		this._clearRelationshipStyleCache();
		this._clearRelationshipStatusCache();
	},

	_clearItemCache: function() {
		// summary:
		//		Clear the item cache
		this._itemsById = {};		
	},
	
	_clearLinkCache: function() {
		// summary:
		//		Clear the link cache
		this._linksById = {};
	},

	_clearRelationshipCache: function() {
		// summary:
		//		Clear the relationship cache
		this._relationshipsById = {};
	},

	_clearToolbarCache: function() {
		// summary:
		//		Clear the "toolbar" cache
		this._toolbar = null;
	},

	_clearItemStyleCache: function() {
		// summary:
		//		Clear item style cache
		this._itemStyles = null;
		this._itemStylesById = null;
	},

	_clearItemStatusCache: function() {
		// summary:
		//		Clear item status cache
		this._itemStatus = null;
		this._itemStatusById = null;
	},

	_clearLinkStyleCache: function() {
		// summary:
		//		Clear link style cache
		this._linkStyles = null;
		this._linkStylesById = null;
	},

	_clearLinkStatusCache: function() {
		// summary:
		//		Clear link status cache
		
		//		protected
		this._linkStatus = null;
		this._linkStatusById = null;
	},

	_clearRelationshipStyleCache: function() {
		// summary:
		//		Clear relationship style cache
		this._relStyles = null;
		this._relStylesById = null;
	},

	_clearRelationshipStatusCache: function() {
		// summary:
		//		Clear relationship status cache
		this._relStatus = null;
		this._relStatusById = null;
	},

	_cacheItems: function(items, resetLazyLoad) {
		// summary:
		//		Cache items; merge if already cached
		if (items) {
			var itemsLength = items.length;
			for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
				items[itemIndex] = this._cacheItem(items[itemIndex], resetLazyLoad);
			}
		}
		
		return items;
	},

	_cacheItem: function(item, resetLazyLoad) {
		// summary:
		//		Cache item; merge if already cached
		var cachedItem = null;
		
		if (item) {
			// check item cache		
			cachedItem = this._getCachedItem(item.getId());
			
			// item is already in cache; merge property values
			if (cachedItem) {
				cachedItem.mixin(item, resetLazyLoad);
				
				// reset expiration to (now + interval) since item was just referenced
				if (this.cacheExpires()) {
					cachedItem.resetExpiration();
				}		
			}
			// cache new item
			else {
				this._itemsById[item.getId()] = item;
				cachedItem = item;				
			}
		}
		
		return cachedItem;
	},

	_getCachedItem: function(itemId) {
		// summary:
		//		look up item in cache
		return this._itemsById[itemId];
	},

	_cacheLinks: function(links) {
		// summary:
		//		Cache links; merge if already cached
		if (links) {
			var linksLength = links.length;
			for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
				links[linkIndex] = this._cacheLink(links[linkIndex]);
			}
		}
		
		return links;
	},

	_cacheLink: function(link) {
		// summary:
		//		Cache link; merge if already cached
		var cachedLink = null;
		
		if (link) {
			// check link cache		
			cachedLink = this._getCachedLink(link.getId());
			
			// link is already in cache; merge property values
			if (cachedLink) {
				cachedLink.mixin(link);
				
				// reset expiration to (now + interval) since link was just referenced
				if (this.cacheExpires()) {
					cachedLink.resetExpiration();
				}		
			}
			// cache new link
			else {
				this._linksById[link.getId()] = link;
				cachedLink = link;				
			}
		}
		
		return cachedLink;
	},

	_getCachedLink: function(linkId) {
		// summary:
		//		look up link in cache
		return this._linksById[linkId];
	},

	_cacheRelationships: function(relationships) {
		// summary:
		//		Cache relationships; merge if already cached
		if (relationships) {
			var relsLength = relationships.length;
			for (var relIndex = 0;  relIndex < relsLength;  ++relIndex) {
				relationships[relIndex] = this._cacheRelationship(relationships[relIndex]);				
			}
		}
		
		return relationships;
	},

	_cacheRelationship: function(relationship) {
		// summary:
		//		Cache relationship; merge if already cached
		var cachedRelationship = null;
		
		if (relationship) {
			// check relationship cache		
			cachedRelationship = this._getCachedRelationship(relationship.getId());
			
			// relationship is already in cache; merge property values
			if (cachedRelationship) {
				cachedRelationship.mixin(relationship);
				
				// reset expiration to (now + interval) since relationship was just referenced
				if (this.cacheExpires()) {
					cachedRelationship.resetExpiration();
				}			
			}
			// cache new relationship
			else {
				this._relationshipsById[relationship.getId()] = relationship;
				cachedRelationship = relationship;
			}
		}
		
		return cachedRelationship;
	},

	_getCachedRelationship: function(relationshipId) {
		// summary:
		//		look up relationship in cache
		return this._relationshipsById[relationshipId];
	},

	_cacheItemStyles: function(styles) {
		// summary:
		//		Set item style cache
		var stylesById = {};
		if (styles) {
			var stylesLength = styles.length;
			for (var styleIndex = 0;  styleIndex < stylesLength;  ++styleIndex) {
				var style = styles[styleIndex];
				stylesById[style.getType()] = style;
			}
		}
		
		this._itemStylesById = stylesById;
		this._itemStyles = styles;
	},

	_cacheItemStatuses: function(statuses) {
		// summary:
		//		Set item status cache
		var statusesById = {};
		if (statuses) {
			var statusLength = statuses.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				var id = statuses[statusIndex].getId() + "_" + statuses[statusIndex].getValue();
				statusesById[id] = statuses[statusIndex];
			}
		}
		
		this._itemStatusById = statusesById;
		this._itemStatus = statuses;	
	},

	_cacheLinkStyles: function(styles) {
		// summary:
		//		Set link style cache
		var stylesById = {};
		if (styles) {
			var stylesLength = styles.length;
			for (var styleIndex = 0;  styleIndex < stylesLength;  ++styleIndex) {
				var style = styles[styleIndex];
				stylesById[style.getType()] = style;
			}
		}
		
		this._linkStylesById = stylesById;
		this._linkStyles = styles;
	},

	_cacheLinkStatuses: function(statuses) {
		// summary:
		//		Set link status cache
		var statusesById = {};
		if (statuses) {
			var statusLength = statuses.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				var id = statuses[statusIndex].getId() + "_" + statuses[statusIndex].getValue();
				statusesById[id] = statuses[statusIndex];
			}
		}
		
		this._linkStatusById = statusesById;
		this._linkStatus = statuses;
	},

	_cacheRelationshipStyles: function(styles) {
		// summary:
		//		Set relationship style cache
		var stylesById = {};
		if (styles) {
			var stylesLength = styles.length;
			for (var styleIndex = 0;  styleIndex < stylesLength;  ++styleIndex) {
				var style = styles[styleIndex];
				stylesById[style.getType()] = style;
			}
		}
		
		this._relStylesById = stylesById;
		this._relStyles = styles;
	},
	
	_cacheRelationshipStatuses: function(statuses) {
		// summary:
		//		Set relationship status cache
		var statusesById = {};
		if (statuses) {
			var statusLength = statuses.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				var id = statuses[statusIndex].getId() + "_" + statuses[statusIndex].getValue();
				statusesById[id] = statuses[statusIndex];
			}
		}
		
		this._relStatusById = statusesById;
		this._relStatus = statuses;
	},

	_mergeItemStyles: function(itemStyles, widgetDef) {
		// summary:
		//		Merge item styles with any pertinent information from widget definition
		var mergedStyles = itemStyles;
				
		// widget definition was provided
		if (widgetDef) {
			mergedStyles = itemStyles.slice(0);
			
			var styleMap = {};
			var styleLength = itemStyles.length;
			for (var styleIndex = 0;  styleIndex < styleLength;  ++styleIndex) {
				styleMap[itemStyles[styleIndex].getType()] = itemStyles[styleIndex];
				mergedStyles[styleIndex] = itemStyles[styleIndex].clone(widgetDef);					
			}
			
			// see if any styles are in widget definition that are not known to dataset
			var widgetStyles = widgetDef.getItemStyles();
			if (widgetStyles) {
				styleLength = widgetStyles.length;
				for (styleIndex = 0;  styleIndex < styleLength;  ++styleIndex) {
					var widgetStyle = widgetStyles[styleIndex];
					if (!styleMap[widgetStyle.type]) {
						mergedStyles.push(this._createItemStyle(widgetDef, widgetStyle));
					}
				}
			}			
		}
		
		return mergedStyles;
	},

	_createItemStyle: function(widgetDef, widgetStyle) {
		// summary:
		//		Create an ItemStyle from information in the widget definition
		var styleAttributes = {};
		var widgetDefaultStyle = widgetDef.getDefaultItemStyle();
		if (widgetDefaultStyle) {
			dojo.mixin(styleAttributes, widgetDefaultStyle);
		}
							
		dojo.mixin(styleAttributes, widgetStyle);
					
		return new ibm.tivoli.navmodel.ItemStyle({style: styleAttributes, version: this.getVersion()});
	},

	_mergeLinkStyles: function(linkStyles, widgetDef, objType) {
		// summary:
		//		Merge link styles with any pertinent information from widget definition
		var mergedStyles = linkStyles;
		
		// widget definition was provided
		if (widgetDef) {
			mergedStyles = linkStyles.slice(0);
			
			var styleMap = {};
			var styleLength = linkStyles.length;
			for (var styleIndex = 0;  styleIndex < styleLength;  ++styleIndex) {
				styleMap[linkStyles[styleIndex].getType()] = linkStyles[styleIndex];
				mergedStyles[styleIndex] = linkStyles[styleIndex].clone(widgetDef, objType);				
			}
			
			// see if any styles are in widget definition that are not known to dataset
			var widgetStyles = widgetDef.getLinkStyles();
			if (widgetStyles) {
				styleLength = widgetStyles.length;
				for (styleIndex = 0;  styleIndex < styleLength;  ++styleIndex) {
					var widgetStyle = widgetStyles[styleIndex];
					if (!styleMap[widgetStyle.type]) {
						mergedStyles.push(this._createLinkStyle(widgetDef, widgetStyle, objType));
					}
				}
			}			
		}
		
		return mergedStyles;
	},

	_createLinkStyle: function(widgetDef, widgetStyle, objType) {
		// summary:
		//		Create a LinkStyle from information in the widget definition
		var styleAttributes = {};
		var widgetDefaultStyle = null;
		switch(objType) {
			case "link":
				widgetDefaultStyle = widgetDef.getDefaultLinkStyle();
				break;
			case "relationship":
				widgetDefaultStyle = widgetDef.getDefaultRelationshipStyle();
		}
		
		if (widgetDefaultStyle) {
			dojo.mixin(styleAttributes, widgetDefaultStyle);
		}
							
		dojo.mixin(styleAttributes, widgetStyle);
					
		return new ibm.tivoli.navmodel.LinkStyle({style: styleAttributes, version: this.getVersion()});
	},

	_mergeStatuses: function(status, widgetDef, objType) {
		// summary:
		//		Merge status with any pertinent information from widget definition
		var mergedStatus = status;
		
		// widget definition was provided
		if (widgetDef) {
			mergedStatus = status.slice(0);
			
			var statusMap = {};
			var statusLength = status.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				statusMap[status[statusIndex].getId()] = status[statusIndex];
				mergedStatus[statusIndex] = status[statusIndex].clone(widgetDef, objType);
			}
			
			// see if any status values are in widget definition that are not known to dataset
			var widgetStatuses = null;
			switch(objType) {
				case "item":
					widgetStatuses = widgetDef.getItemStatuses();
					break;
				case "link":
					widgetStatuses = widgetDef.getLinkStatuses();
					break;
				case "relationship":
					widgetStatuses = widgetDef.getRelationshipStatuses();
			}
			
			if (widgetStatuses) {
				statusLength = widgetStatuses.length;
				for (statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
					var widgetStatus = widgetStatuses[statusIndex];
					if (!statusMap[widgetStatus.id]) {
						mergedStatus.push(this._createStatus(widgetDef, widgetStatus, objType));
					}
				}
			}
		}
		
		return mergedStatus;
	},

	_createStatus: function(widgetDef, widgetStatus, objType) {
		// summary:
		//		Create an Status from information in the widget definition
		var statusAttributes = {};
		var widgetDefaultStatus = null;
		switch(objType) {
			case "item":
				widgetDefaultStatus = widgetDef.getDefaultItemStatus();
				break;
			case "link":
				widgetDefaultStatus = widgetDef.getDefaultLinkStatus();
				break;
			case "relationship":
				widgetDefaultStatus = widgetDef.getDefaultRelationshipStatus();
		}
		
		if (widgetDefaultStatus) {
			dojo.mixin(statusAttributes, widgetDefaultStatus);
		}
							
		dojo.mixin(statusAttributes, widgetStatus);
					
		return new ibm.tivoli.navmodel.Status({status: statusAttributes, version: this.getVersion()});
	},
		
	_notifyEventListeners: function(event) {
		// summary:
		//		Pass ibm.tivoli.navmodel.Event to all registered listeners
		if (this._listeners) {
			var listenerLength = this._listeners.length;
			for (var listenerIndex = 0;  listenerIndex < listenerLength;  ++listenerIndex) {
				try {
					var listenerObj = this._listeners[listenerIndex].listenerObj;
					this._listeners[listenerIndex].listenerCallback(listenerObj, event);
				}
				catch (ex) {
					this._trace.error("Exception forwarding eventId=" + event.getId() + " type=" + event.getType() + " ex=" + ex, "_sendEvent");
				}
			}				
		}		
	},
	
	_jsonEventHandler: function(jsonEventObj) {
		// summary:
		//		This is the routine that we register with the EventRouter to handle
		//		raw json events that come from the server
		var event = new ibm.tivoli.navmodel.Event({dataset: this, event: jsonEventObj});
		this._eventHandler(event);
	},

	_eventHandler: function(event) {
		// summary:
		//		Handles ibm.tivoli.navmodel.Event objects
		if (event.getId() === this.getEventId()) {
			if (this._trace.isFine()) {
				this._trace.fine("datasetId=" + this.getId() + " event=" + event.getType() + " eventId=" + event.getId(), "_eventHandler");
			}
			
			switch(event.getType()) {
				case "changed":
					this._handleChangedEvent(event);
					break;
				case "labelChanged":
					this._handleLabelChangedEvent(event);
					break;
				case "statusChanged":
					this._handleStatusChangedEvent(event);
					break;					
				case "itemsAdded":
					this._handleItemsAddedEvent(event);
					break;
				case "itemsRemoved":
					this._handleItemsRemovedEvent(event);
					break;
				case "itemStatusChanged":
					this._handleItemStatusChangedEvent(event);
					break;
				case "itemPropertiesChanged":
					this._handleItemPropertiesChangedEvent(event);
					break;
				case "linksAdded":
					this._handleLinksAddedEvent(event);
					break;
				case "linksRemoved":
					this._handleLinksRemovedEvent(event);
					break;
				case "linkStatusChanged":
					this._handleLinkStatusChangedEvent(event);
					break;
				case "linkPropertiesChanged":
					this._handleLinkPropertiesChangedEvent(event);
					break;
				case "linkChanged":
					this._handleLinkChangedEvent(event);
					break;
				case "relationshipStatusChanged":
					this._handleRelationshipStatusChangedEvent(event);
					break;
				case "relationshipPropertiesChanged":
					this._handleRelationshipPropertiesChangedEvent(event);
					break;
				case "exception":
					if (this._trace.isFine()) {
						this._trace.fine("	exception msgText=" + event.getException().getMsgText(), "_eventHandler");
					}
					break;
				case "eventRouter":
					if (this._trace.isFine()) {
						this._trace.fine("	eventRouterStatus=" + event.getEventRouterStatus(), "_eventHandler");
					}
			}
			
			this._notifyEventListeners(event);
		}		
	},

	_handleChangedEvent: function(event) {
		// summary:
		//		Handle "changed" event type
		this._clearAllCaches();
		
		var initArgs = {
			optimize: true,
			createEventRouter: event.createEventRouter,
			fetchMetaData: true			
		};
		this._initDataset(initArgs);
	},
	
	_handleLabelChangedEvent: function(event) {
		// summary:
		//		Handle "labelChanged" event type
		this._dataset.label = event._getEventNewLabel();
		
		if (this._trace.isFinest()) {
			this._trace.finest("datasetId=" + this.getId() + " label=" + this._dataset.label, "_handleLabelChangedEvent");
		}
	},

	_handleStatusChangedEvent: function(event) {
		// summary:
		//		Handle "statusChanged" event type
		var eventStatus = event._getEventStatuses();
		if (eventStatus) {		
			var status = [];
			var statusLength = eventStatus.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				status[statusIndex] = new ibm.tivoli.navmodel.Status({status: eventStatus[statusIndex], version: this.getVersion()});
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + this.getId() + " status=" + status[statusIndex].getLabel(), "_handleStatusChangedEvent");
				}
			}
		}
	},

	_handleItemsAddedEvent: function(event) {
		// summary:
		//		Handle "itemsAdded" event type
		
		// get new item count for dataset
		var itemCount = event._getEventDatasetItemCount();
		if (itemCount !== undefined) {
			this._setItemCount(itemCount);
		}
		
		// might have new list of styles
		this._clearItemStyleCache();
			
		var eventItems = event._getEventItems();
		if (eventItems) {
			var itemsLength = eventItems.length;
			for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + this.getId() + " itemId=" + eventItems[itemIndex].id, "_handleItemsAddedEvent");
				}
				
				var eventItem = eventItems[itemIndex];
				var newItem = new ibm.tivoli.navmodel.Item({item: eventItem, dataset: this});
				newItem = this._cacheItem(newItem);													
				event._addItem(newItem);				
			}					
		}
	},

	_handleItemsRemovedEvent: function(event) {
		// summary:
		//		Handle "itemsRemoved" event type
		
		// get new item count for dataset
		var itemCount = event._getEventDatasetItemCount();
		if (itemCount !== undefined) {
			this._setItemCount(itemCount);
	    	if (itemCount === 0) {
	    		this._clearItemCache();
			}
		}
		
		// might have new list of styles
		this._clearItemStyleCache();
			
		var eventItems = event._getEventItems();
		if (eventItems) {
			var itemsLength = eventItems.length;
			for (var itemIndex = itemsLength - 1;  itemIndex >= 0;  --itemIndex) {
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + this.getId() + " itemId=" + eventItems[itemIndex].id, "_handleItemsRemovedEvent");
				}
				
				var eventItem = eventItems[itemIndex];
				var deletedItem = new ibm.tivoli.navmodel.Item({item: eventItem, dataset: this});		
				var item = this._getCachedItem(eventItem.id);
				if (item) {
					delete this._itemsById[item.getId()];
					event._addItem(item);
				}
				else {	
					event._addItem(deletedItem);
				}			
			}	
		}
	},

	_handleItemStatusChangedEvent: function(event) {
		// summary:
		//		Handle "itemStatusChanged" event type
		var eventItems = event._getEventItems();
		if (eventItems) {
			var itemsLength = eventItems.length;
			for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
				var eventItem = eventItems[itemIndex];	
				var item = new ibm.tivoli.navmodel.Item({item: eventItem, dataset: this});		
				item = this._cacheItem(item);					
				event._addItem(item);
				
				if (this._trace.isFinest()) {
					var status = item.getStatuses({widgetDef: null});
					if (status) {
						var statusLength = status.length;
						for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
							this._trace.finest("datasetId=" + this.getId() + " itemId=" + eventItem.id + " status=" + status[statusIndex].getLabel(), "_handleItemStatusChangedEvent");
						}					
					}
				}			
			}
		}
	},

	_handleItemPropertiesChangedEvent: function(event) {
		// summary:
		//		Handle "itemPropertiesChanged" event type
		var eventItems = event._getEventItems();
		if (eventItems) {
			var itemsLength = eventItems.length;
			for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
				var eventItem = eventItems[itemIndex];							
				var props = [];
				var properties = eventItem.properties;					
				if (properties) {
					var propsLength = properties.length;
					for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
						var property = new ibm.tivoli.navmodel.Property({property: properties[propIndex], dataset: this, objectType: "item"});
						props[propIndex] = property;						
						if (this._trace.isFinest()) {
							this._trace.finest("datasetId=" + this.getId() + " itemId=" + eventItem.id + " prop=" + property.getId() + " value=" + property.getDisplayValue(), "_handleItemPropertiesChangedEvent");
						}																	
					}
				}
								
				var item = new ibm.tivoli.navmodel.Item({item: eventItem, dataset: this});				
				item = this._cacheItem(item);					
				event._addItem(item, props);				
			}
		}
	},

	_handleLinksAddedEvent: function(event) {
		// summary:
		//		Handle "linksAdded" event type
		
		// get new link and relationship counts for dataset
		var linkCount = event._getEventDatasetLinkCount();
		var relCount = event._getEventDatasetRelationshipCount();
		if (linkCount !== undefined) {
			this._setLinkCount(linkCount);
		}
		if (relCount !== undefined) {
			this._setRelationshipCount(relCount);
		}
		
		// might have new list of styles
		this._clearLinkStyleCache();
		this._clearRelationshipStyleCache(); 
			
		var eventLinks = event._getEventLinks();
		if (eventLinks) {
		    var eventLinksLength = eventLinks.length;
			for (var linkIndex = 0;  linkIndex < eventLinksLength;  ++linkIndex) {
				var eventLink = eventLinks[linkIndex];
				var newLink = new ibm.tivoli.navmodel.Link({link: eventLink, dataset: this});
				newLink = this._cacheLink(newLink);				
				event._addLink(newLink);
				
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + this.getId() + " linkId=" + newLink.getId(), "_handleLinksAddedEvent");
				}				
			}
		}
	},

	_handleLinksRemovedEvent: function(event) {
		// summary:
		//		Handle "linksRemoved" event type
		
		// get new link count for dataset
		var linkCount = event._getEventDatasetLinkCount();
		var relCount = event._getEventDatasetRelationshipCount();
		if (linkCount !== undefined) {
			this._setLinkCount(linkCount);
			if (linkCount === 0) {
	    		this._clearLinkCache();	    		
			}
		}
		if (relCount !== undefined) {
		 	this._setRelationshipCount(relCount);			
		}
		
		// might have new list of styles
		this._clearLinkStyleCache();
		this._clearRelationshipStyleCache();
		    
		var eventLinks = event._getEventLinks();
		if (eventLinks) {
		    var eventLinksLength = eventLinks.length;
			for (var linkIndex = eventLinksLength - 1;  linkIndex >= 0;  --linkIndex) {
				var eventLink = eventLinks[linkIndex];
				var deletedLink = new ibm.tivoli.navmodel.Link({link: eventLink, dataset: this});
				var link = this._getCachedLink(eventLink.id);
				if (link) {
					delete this._linksById[link.getId()];
					event._addLink(link);
				}
				else {	
					event._addLink(deletedLink);
				}
				
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + this.getId() + " linkId=" + deletedLink.getId(), "_handleLinksRemovedEvent");
				}	
			}
		}
	},
	
	_handleLinkStatusChangedEvent: function(event) {
		// summary:
		//		Handle "linkStatusChanged" event type
		var eventLinks = event._getEventLinks();
		if (eventLinks) {
			var eventLinksLength = eventLinks.length;
			for (var linkIndex = 0;  linkIndex < eventLinksLength;  ++linkIndex) {
				var eventLink = eventLinks[linkIndex];
				var link = new ibm.tivoli.navmodel.Link({link: eventLink, dataset: this});
				link = this._cacheLink(link);					
				event._addLink(link);
				
				if (this._trace.isFinest()) {
					var status = link.getStatuses({widgetDef: null});
					if (status) {
						var statusLength = status.length;
						for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
							this._trace.finest("datasetId=" + this.getId() + " linkId=" + link.getId() + " status=" + status[statusIndex].getLabel(), "_handleLinkStatusChangedEvent");
						}					
					}
				}		
			}
		}
	},

	_handleLinkPropertiesChangedEvent: function(event) {
		// summary:
		//		Handle "linkPropertiesChanged" event type
		var eventLinks = event._getEventLinks();
		if (eventLinks) {
			var eventLinksLength = eventLinks.length;
			for (var linkIndex = 0;  linkIndex < eventLinksLength;  ++linkIndex) {
				var eventLink = eventLinks[linkIndex];
				var link = new ibm.tivoli.navmodel.Link({link: eventLink, dataset: this});
				var props = [];
				var properties = eventLink.properties;					
				if (properties) {
					var propsLength = properties.length;
					for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
						var property = new ibm.tivoli.navmodel.Property({property: properties[propIndex], dataset: this, objectType: "link"});
						props[propIndex] = property;						
						if (this._trace.isFinest()) {
							this._trace.finest("datasetId=" + this.getId() + " linkId=" + link.getId() + " prop=" + property.getId() + " value=" + property.getDisplayValue(), "_handleLinkPropertiesChangedEvent");
						}
					}
				}
				
				link = this._cacheLink(link);					
				event._addLink(link, props);
			}
		}		
	},

	_handleLinkChangedEvent: function(event) {
		// summary:
		//		Handle "linkChanged" event type
		
		// might have new list of styles
		this._clearLinkStyleCache();
		this._clearRelationshipStyleCache();
		    
		var eventLink = event._getEventLink();
		var link = new ibm.tivoli.navmodel.Link({link: eventLink, dataset: this});
		link = this._cacheLink(link);	
		event._addLink(link);
				
		if (this._trace.isFinest()) {
			this._trace.finest("datasetId=" + this.getId() + " linkId=" + link.getId() + " reason=" + event.getLinkChangedReason(), "_handleLinkChangedEvent");
		}						
		
		// affected relationships (reason is "relationshipAdded" or "relationshipRemoved")
		var eventRelationships = event._getEventRelationships();
		if (eventRelationships) {
			var eventRelLength = eventRelationships.length;
			for (var eventRelIndex = 0;  eventRelIndex < eventRelLength;  ++eventRelIndex) {
				var relationship = new ibm.tivoli.navmodel.Relationship({relationship: eventRelationships[eventRelIndex], dataset: this});
				event._addRelationship(relationship);
			}
		}
	},

	_handleRelationshipStatusChangedEvent: function(event) {
		// summary:
		//		Handle "relationshipStatusChanged" event type
		var eventRels = event._getEventRelationships();
		if (eventRels) {
			var relLength = eventRels.length;
			for (var relIndex = 0;  relIndex < relLength;  ++relIndex) {
				var eventRel = eventRels[relIndex];
				var relationship = new ibm.tivoli.navmodel.Relationship({relationship: eventRel, dataset: this});
				relationship = this._cacheRelationship(relationship);
				event._addRelationship(relationship);
				
				if (this._trace.isFinest()) {
					var status = relationship.getStatuses({widgetDef: null});
					if (status) {
						var statusLength = status.length;
						for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
							this._trace.finest("datasetId=" + this.getId() + " relationshipId=" + eventRel.id + " status=" + status[statusIndex].getLabel(), "_handleRelationshipStatusChangedEvent");
						}					
					}
				}
			}
		}		
	},

	_handleRelationshipPropertiesChangedEvent: function(event) {
		// summary:
		//		Handle "relationshipPropertiesChanged" event type
		var eventRels = event._getEventRelationships();
		if (eventRels) {
			var relsLength = eventRels.length;
			for (var relIndex = 0;  relIndex < relsLength;  ++relIndex) {
				var eventRel = eventRels[relIndex];
				var props = [];
				var properties = eventRel.properties;					
				if (properties) {
					var propsLength = properties.length;
					for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
						var property = new ibm.tivoli.navmodel.Property({property: properties[propIndex], dataset: this, objectType: "relationship"});
						props[propIndex] = property;							
						if (this._trace.isFinest()) {
							this._trace.finest("datasetId=" + this.getId() + " relationshipId=" + eventRel.id + " prop=" + property.getId() + " value=" + property.getDisplayValue(), "_handleRelationshipPropertiesChangedEvent");
						}
					}					
				}
					
				var relationship = new ibm.tivoli.navmodel.Relationship({relationship: eventRel, dataset: this});
				relationship = this._cacheRelationship(relationship);
				event._addRelationship(relationship, props);				
			}
		}
	},
		
	_validateCache: function() {
		// summary:
		//		Check cache for items, links, and relationships that haven't been used for awhile;
		//		remove from cache
		this._trace.entering("starting validate cache", "_validateCache");
		
		this._validate(this._itemsById);
		this._validate(this._linksById);
		this._validate(this._relationshipsById);
		
		this._trace.exiting("ending validate cache", "_validateCache");				
	},
	
	_validate: function(cache) {
		// summary:
		//		Check expiration for supplied cache; items, links, or relationships
		var nowDate = new Date();
		var now = nowDate.getTime();
		
		for (var id in cache) { 
    		if (cache.hasOwnProperty(id)) {
    			var obj = cache[id];
    			if (obj && obj.checkExpiration(now)) {
    				delete cache[id];
    			}
    		}        
		} 		
	},
	
	/////////////////////////////
	// abstract methods
	/////////////////////////////
	
	/*=====
	initDataset_kwArgs: function(createEventRouter) {
		// createEventRouter: Boolean
		//		boolean indicating whether new event router should be created
	},
	=====*/
	
	_initDataset: function(/*initDataset_kwArgs*/ kwArgs) {
		// summary:
		//		Initialize internal dataset object
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		
		// determine if we need to start up the cache interval to purge
		// resources that have not been touched in a while
		if (!this._cacheIntervalId && this.getItemCount() >= this._args.cacheThreshold && this._args.cacheValidationInterval) {
			this._cacheIntervalId = setInterval(dojo.hitch(this, this._validateCache), (this._args.cacheValidationInterval*60000));
		}

      // determine if we need to start up the remote provider keep-alive interval
      // to ensure session connection through passthrough to remote provider is maintained          255243
      if (!this._remoteKeepAliveIntervalId && (this.getAttribute("remote") != null) && this.getAttribute("remote")) {
         if (this._trace.isFine()) {
             this._trace.fine("Setting up remote dataset keep-alive interval = " + this._args.remoteKeepAliveInterval + " min, for dataset = " + this.getId(), "_initDataset");
         }
         this._remoteKeepAliveIntervalId = setInterval(dojo.hitch(this, this.keepAlive), (this._args.remoteKeepAliveInterval*60000));
      }
	},

	_deleteDataset: function() {
		// summary:
		//		Dataset is no longer in use	
	},
	
	/*=====
	fetchItemsAndLinks_onComplete: function(items, links, toolbar, kwArgs){},
	=====*/
	
	/*=====
	fetchItemsAndLinks_kwArgs: function(onComplete, onError, sync) {
		// onComplete: fetchItemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	_fetchItemsAndLinks: function(/*fetchItemsAndLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns both the items and links associated with the dataset
	},
	
	_fetchItems: function(/*getItems_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch items from rest service
	},

	_fetchItem: function(/*getItem_kwArgs*/ kwArgs) {
		// summary:
		//		Synchronously fetch an individual item from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		return null;
	},
	
	_fetchOriginItems: function(/*getOriginItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns orign items of the dataset
	},
	
	_fetchRelatedItems: function(/*getRelatedItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items that are related to the requested item
	},

	_queryItems: function(/*queryItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items associated with the dataset that match
		//		the provided custom query
	},

	_fetchLinks: function(/*getLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch links from rest service
	},

	_fetchLink: function(/*getLink_kwArgs*/ linkId) {
		// summary:
		//		Synchronously fetch an individual link from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		return null;
	},
		
	_fetchRelationships: function(/*getRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch relationships from rest service
	},

	_fetchRelationship: function(/*getRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Synchronously fetch an individual relationship from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		return null;
	},

	_fetchItemStyles: function(/*getItemStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current item styles for the Dataset
	},
	
	_fetchLinkStyles: function(/*getLinkStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current link styles for the Dataset
	},
	
	_fetchRelationshipStyles: function(/*getRelationshipStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current relationship styles for the Dataset
	},
	
	/*=====
	fetchStatuses_kwArgs: function(onComplete, onError, start, count, sync) {
		// onComplete: relStatuses_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// type: String
		//		object type for columns: "item", "link", "relationship"
		// start: int
		//		start index to fetch
		// count: int
		//		number of status values to fetch; null means all
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchStatuses: function(/*fetchStatuses_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of possible relationship status values for the Dataset
	},
	
	_fetchTasks: function(/*getTasks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns tasks associated with the dataset
	},
	
	_navigate: function(/*navigate_kwArgs*/ kwArgs) {
		// summary:
		//		Navigate to another Dataset from a navigation task
	},
	
	/*=====
	fetchColumns_kwArgs: function(onComplete, onError, type, properties, start, count, metadaa, sync) {
		// onComplete: cols_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// type: String
		//		object type: "item", "link", "relationship" 
		// properties: String[]
		//		array of ids in which to fetch columns;
		//		if not specified the "current" columns will be fetched
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// metadata: Boolean
		//		indicates if filter metadata should be returned with columns
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	_fetchColumns: function(/*fetchColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch columns from rest service
	},
	
	_setColumns: function(/*setColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Save columns to rest service
	},

	_createItem: function(/*createItem_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new item in the dataset
	},
	
	_createItems: function(/*createItems_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new items in the dataset
	},
	
	_deleteItem: function(/*deleteItem_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a item from the dataset
	},
	
	_deleteItems: function(/*deleteItems_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete items from the dataset
	},
	
	_createLink: function(/*createLink_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new link in the dataset
	},
	
	_createLinks: function(/*createLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new links in the dataset
	},
	
	_deleteLink: function(/*deleteLink_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a link from the dataset
	},
	
	_deleteLinks: function(/*deleteLinks_kwArgs*/  kwArgs) {
		// summary:
		//		Asynchronously delete links from the dataset
	},

	_createRelationship: function(/*createRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new relationship in the dataset
	},

	_createRelationships: function(/*createRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new relationships in the dataset
	},
	
	_deleteRelationship: function(/*deleteRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a relationship from the dataset
	},
	
	_deleteRelationships: function(/*deleteRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete relationships from the dataset
	},

   _doKeepAlive: function(/*doKeepAlive_kwArgs*/ kwArgs) {       // 255243
       // summary:
       //		Query the dataset attributes again to keep the connection with the provider alive
   }
});

// 255211
// public constants for values of getDatasetImplementation() as defined by CURI INavCollection.getCollectionType()
//    SIMPLE = INavCollection.SIMPLE_COLLECTION
//    HYBRID = INavCollection.HYBRID_COLLECTION
//    LARGE_TABLE = INavCollection.LARGE_TABLE_COLLECTION
// usage:
//      if ( myDataset.getDatasetImplementation() == ibm.tivoli.navmodel.AbstractDataset.implementation.LARGE_TABLE )
//      {  /* do something */ }
ibm.tivoli.navmodel.AbstractDataset.implementation = {     // 255247
   SIMPLE: "simple",
   HYBRID: "hybrid",
   LARGE_TABLE: "large"
};
