/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.ClientDataset"); 
 
// includes
dojo.require("ibm.tivoli.navmodel.Dataset");

ibm.tivoli.navmodel.ClientDataset._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.ClientDataset");

////////////////////////////////////
// ClientDataset
//
// NOTE: do not call any abstract methods directly;
// call the public version of the methods defined in
// AbstractDataset instead and it will in turn call
// the abstract overrides.
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.ClientDataset", ibm.tivoli.navmodel.Dataset, {
	// private members
	_trace: ibm.tivoli.navmodel.ClientDataset._trace,
	_json: null,
	_items: null,
	_links: null,
	
	/*=====
	constructor_kwArgs: function(onComplete, onError, provider, datasource, dataset, dataUrl, data) {
		// onComplete: constructor_onComplete
	 	//		callback to be invoked when the Dataset is initialized and ready for methods
	 	//		to be invoked  (i.e. asynchronous initialization must be performed). 
	 	//		Note: A null value will result in synchronous initialization.
	 	// onError: constructor_onError
	 	//		callback to be invoked if there is a problem constructing the Dataset
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// dataset: String
		//		dataset id
		// dataUrl: String
		//		url to file containing dataset json
		// data: String
		//		alternative to the fileUrl (i.e. provide the json directly)
	},
	=====*/

	constructor: function(/*constructor_kwArgs*/ kwArgs) {	
		// summary:
		//		Constructs ClientDataset object.
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
	},
	
	/////////////////////////////
	// abstract method overrides
	//
	// NOTE: do not call any abstract methods directly;
	// call the public version of the methods defined in
	// AbstractDataset instead and it will in turn call
	// the abstract overrides.
	/////////////////////////////
	
	/*=====
	initDataset_kwArgs: function(createEventRouter) {
		// createEventRouter: Boolean
		//		boolean indicating whether new event router should be created
	},
	=====*/
	
	_initDataset: function(/*initDataset_kwArgs*/ kwArgs) {
		// summary:
		//		Initialize internal dataset object
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		
		// reset internal dataset info
		this._initEmptyDataset();				
		
		// see if json was passed directly or a file url was provided
		this._json = this.getArg("data");
		var dataUrl = this.getArg("dataUrl");
		
		// make synchronous call if no onComplete callback
		var synchronous = (kwArgs.onComplete ? false : true);
		var exception = null;
		
		// url to fetch json
		if (!this._json && dataUrl) {
			var now = new Date();		
			var content = {d: now.getTime(), r: Math.random()};
						
			dojo.xhrGet({
				url: dataUrl,
				content: content,
				handleAs: "json",
				sync: synchronous,
				load: dojo.hitch(this, function(result, ioArgs) {
					this._json = result;
					
					if (this._json.items) {
						this._items = this._extractItems();		
						this._setItemCount(this._items.length);
						this._cacheItems(this._items);
					}
					
					if (this._json.links) {
						this._links = this._extractLinks();
						this._setLinkCount(this._links.length);
						this._cacheLinks(this._links);
					}
					
					// misc attributes of the dataset
					if (this._json.dataset) {
						this.mixinAttributes(this._json.dataset);		
					}
					
					// call onComplete callback
					if (kwArgs.onComplete) {
						kwArgs.onComplete(this, kwArgs);
					}					
				}),
				error: dojo.hitch(this, function(error, ioArgs){
					exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_initDataset", this._trace, this.getVersion());
					if (kwArgs.onError) {
						kwArgs.onError(error, exception, ioArgs);
					}	
				})
			});
		}
		
		if (synchronous && exception) {
			throw exception;
		}		
	},

	_deleteDataset: function() {
		// summary:
		//		Dataset is no longer in use
		
		// nothing to do
	},
	
	/*=====
	fetchItemsAndLinks_onComplete: function(items, links, datasetItems, toolbar, kwArgs){},
	=====*/
	
	/*=====
	_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	fetchItemsAndLink_kwArgs: function(onComplete, onError, sync) {
		// onComplete: fetchItemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchItemsAndLinks: function(/*fetchItemsAndLink_kwArgs*/ kwArgs) {
		// summary:
		//		Returns both the items and links associated with the dataset
		
		// build topology
		var items = this._extractItems();
		var links = this._extractLinks();
		var toolbar = this._extractToolbar();
		
		// execute the callback
		if (kwArgs.sync) {
			kwArgs.onComplete(items, links, toolbar, kwArgs);
		}
		else {
			setTimeout(dojo.hitch(this, kwArgs.onComplete, items, links, toolbar, kwArgs), 0);
		}		
	},
	
	/*=====
	items_onComplete: function(items, kwArgs){},
	=====*/

	/*=====
	fetchItems_kwArgs: function(onComplete, onError, start, count, properties, metadata, condition, sync) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions 
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	_fetchItems: function(/*fetchItems_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch items from rest service
		var items = this._items;			
		var itemLength = items.length;
		var start = kwArgs.start;
		var count = kwArgs.count;
		if (!count) {
			count = itemLength;
		}
		
		// need to pull out just the items requested
		if (start !== 0 || count !== itemLength) {
			items = items.slice(start, start + count);
		}
		
		// place counts in the arguments map
		kwArgs.numRows = items.length;
		kwArgs.filteredRows = itemLength;
		kwArgs.totalRows = itemLength;
							
		// execute the callback
		if (kwArgs.sync) {
			kwArgs.onComplete(items, kwArgs);
		}
		else {
			setTimeout(dojo.hitch(this, kwArgs.onComplete, items, kwArgs), 0);
		}
	},

	/*=====
	fetchItem_kwArgs: function(itemId, properties, metadata) {
		// itemId: String
		//		item id to get
		// properties: String[]
		//		array of property ids to pre-fetch
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/
	
	_fetchItem: function(/*fetchItem_kwArgs*/ kwArgs) {
		// summary:
		//		Synchronously fetch an individual item from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		return this._getCachedItem(kwArgs.itemId);		
	},

	/*=====
	fetchOriginItems_kwArgs: function(onComplete, onError, start, count, properties, metadata, sync) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchOriginItems: function(/*fetchOriginItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns orign items of the dataset
		var items = this._items;		
		var origins = [];
		var itemsLength = items.length;
		for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
			if (items[itemIndex].isOrigin()) {
				origins.push(items[itemIndex]);
			}
		}
		
		// execute the callback
		if (kwArgs.sync) {
			kwArgs.onComplete(origins, kwArgs);
		}
		else {
			setTimeout(dojo.hitch(this, kwArgs.onComplete, origins, kwArgs), 0);
		}
	},
	
	/*=====
	fetchRelatedItems_kwArgs: function(onComplete, onError, itemId, relationships, start, count, properties, metadata, sync) {
		// onComplete: itemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemId: String
		//		id of the base item
		// relationships: String
		//		comma delimited list of relationships to determine related items; 
		//		can also be keywords: child, parent, all
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchRelatedItems: function(/*fetchRelatedItems_kwArgs*/ kwArgs) {
		var items = [];
		var links = [];
		var datasetItems = [];
		
		//TODO: find related items
		
		// execute the callback
		if (kwArgs.sync) {
			kwArgs.onComplete(items, links, datasetItems, kwArgs);
		}
		else {
			setTimeout(dojo.hitch(this, kwArgs.onComplete, items, links, datasetItems, kwArgs), 0);
		}
	},

	/*=====
	queryItems_kwArgs: function(onComplete, onError, query, start, count, properties, metadata, sync) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// query: Object
		//		custom query object to be sent as request body
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	 _queryItems: function(/*queryItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items associated with the dataset that match
		//		the provided custom query
	 	var items = [];
		
		//TODO: find items
		
		// execute the callback
		if (kwArgs.sync) {
			kwArgs.onComplete(items, kwArgs);
		}
	 	else {
	 		setTimeout(dojo.hitch(this, kwArgs.onComplete, items, kwArgs), 0);
	 	}
	 },

	/*=====
	links_onComplete: function(items, kwArgs){},
	=====*/
	 
	/*=====
	fetchLinks_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: links_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions
		// start: int
		//		start index to fetch
		// count: int
		//		number of links to fetch; null means all
		// properties: String[]
		//		link property ids to fetch by default (can specify \"all\")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	 
	_fetchLinks: function(/*fetchLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch links from rest service
		var links = this._links;		
		var linkLength = links.length;
		var start = kwArgs.start;
		var count = kwArgs.count;
		if (!count) {
			count = linkLength;
		}
		
		// need to pull out just the links requested
		if (start !== 0 || count !== linkLength) {
			links = links.slice(start, start + count);
		}
		
		// place counts in the arguments map
		kwArgs.numRows = links.length;
		kwArgs.filteredRows = linkLength;
		kwArgs.totalRows = linkLength;
		
		// execute the callback
		if (kwArgs.sync) {
			kwArgs.onComplete(links, kwArgs);
		}
		else {
			setTimeout(dojo.hitch(this, kwArgs.onComplete, links, kwArgs), 0);
		}
	},
	
	/*=====
	fetchLink_kwArgs: function(itemId, properties, metadata) {
		// linkId: String
		//		link id to get
		// properties: String[]
		//		array of property ids to pre-fetch
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/
	
	_fetchLink: function(/*fetchLink_kwArgs*/ kwArgs) {
		// summary:
		//		Synchronously fetch an individual link from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		return this._getCachedLink(kwArgs.linkId);		
	},
	
	/*=====
	relationships_onComplete: function(relationships, kwArgs){},
	=====*/
	
	/*=====
	fetchRelationships_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: relationships_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions
		// start: int
		//		start index to fetch
		// count: int
		//		number of relationships to fetch; null means all
		// properties: String[]
		//		relationship property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchRelationships: function(/*fetchRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch relationships from rest service
		var relationships = [];		
		var links = this._links;
		var linkLength = links.length;
		for (var linkIndex = 0;  linkIndex < linkLength;  ++linkIndex) {
			var rels = links[linkIndex].getRelationships();
			if (rels) {
				relationships = relationships.concat(rels);
			}
		}
		
		var start = kwArgs.start;
		var count = kwArgs.count;
		if (!count) {
			count = relationships.length;
		}
		
		// need to pull out just the relationships requested
		if (start !== 0 || count !== relationships.length) {
			relationships = relationships.slice(start, start + count);
		}
		
		// place counts in the arguments map
		kwArgs.numRows = relationships.length;
		kwArgs.filteredRows = relationshipLength;
		kwArgs.totalRows = relationshipLength;
		
		// execute the callback
		if (kwArgs.sync) {
			kwArgs.onComplete(relationships, kwArgs);
		}
		else {
			setTimeout(dojo.hitch(this, kwArgs.onComplete, relationships, kwArgs), 0);
		}
	},
	
	/*=====
	fetchRelationship_kwArgs: function(relationshipId, properties, metadata) {
		// relationshipId: String
		//		relationship id to fetch
		// properties: String[]
		//		array of property ids to pre-fetch
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/
	
	_fetchRelationship: function(/*fetchRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Synchronously fetch an individual relationship from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var relationship = this._getCachedRelationship(kwArgs.relationshipId);
		if (relationship) {
			return relationship;
		}
		
		var links = this._links;
		var linkLength = links.length;
		for (var linkIndex = 0;  linkIndex < linkLength;  ++linkIndex) {
			var relationships = links[linkIndex].getRelationships();
			if (relationships) {
				var relLength = relationships.length;
				for (var relIndex = 0;  relIndex < relLength;  ++relIndex) {
					if (relationships[relIndex].getId() === kwArgs.relationshipId) {
						this._cacheRelationship(relationships[relIndex]);
						return relationships[relIndex];
					}
				}										
			}
		}
		
		return null;
	},
	
	/*=====
	itemStyles_onComplete: function(itemStyles, kwArgs){},
	=====*/
	
	/*=====
	fetchItemStyles_kwArgs: function(onComplete, onError, start, count, sync) {
		// onComplete: itemStyle_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of item styles to fetch;  null means all
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	_fetchItemStyles: function(/*fetchItemStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current item styles for the Dataset
		var styles = [];
		var styleObjs = this._json.itemStyles;
		if (styleObjs) {
			var styleLength = styleObjs.length;			
			var start = kwArgs.start;
			var count = kwArgs.count;
			if (!count) {
				count = styleLength;
			}			
			for (var styleIndex = start;  styleIndex < styleLength && count > 0;  ++styleIndex, --count) {
				styles[styleIndex] = this._newItemStyle(styleObjs[styleIndex]);
			}			
		}
		
		// execute the callback
		if (kwArgs.sync) {
			kwArgs.onComplete(styles, kwArgs);
		}
		else {
			setTimeout(dojo.hitch(this, kwArgs.onComplete, styles, kwArgs), 0);
		}
	},
	
	/*=====
	linkStyles_onComplete: function(linkStyles, kwArgs){},
	=====*/
	
	/*=====
	fetchLinkStyles_kwArgs: function(onComplete, onError, start, count, sync) {
		// onComplete: linkStyles_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of link styles to fetch; null means all
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	_fetchLinkStyles: function(/*fetchLinkStyles_kwArgs*/ kwArgs) {
		var styles = [];
		var styleObjs = this._json.linkStyles;
		if (styleObjs) {
			var styleLength = styleObjs.length;
			var start = kwArgs.start;
			var count = kwArgs.count;
			if (!kwArgs.count) {
				count = styleLength;
			}
			for (var styleIndex = start;  styleIndex < styleLength && count > 0;  ++styleIndex, --count) {
				styles[styleIndex] = this._newLinkStyle(styleObjs[styleIndex]);
			}			
		}
		
		// execute the callback
		if (kwArgs.sync) {
			kwArgs.onComplete(styles, kwArgs);
		}
		else {
			setTimeout(dojo.hitch(this, kwArgs.onComplete, styles, kwArgs), 0);
		}
	},
	
	/*=====
	relStyles_onComplete: function(relStyles, kwArgs){},
	=====*/
	
	/*=====
	fetchRelationshipStyles_kwArgs: function(onComplete, onError, start, count, sync) {
		// onComplete: relStyles_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of relationship styles to fetch; null means all
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	_fetchRelationshipStyles: function(/*fetchRelationshipStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current relationship styles for the Dataset
		var styles = [];
		var styleObjs = this._json.relationshipStyles;
		if (styleObjs) {
			var styleLength = styleObjs.length;
			var start = kwArgs.start;
			var count = kwArgs.count;
			if (!kwArgs.count) {
				count = styleLength;
			}
			for (var styleIndex = start;  styleIndex < styleLength && count > 0;  ++styleIndex, --count) {
				styles[styleIndex] = this._newLinkStyle(styleObjs[styleIndex]);
			}			
		}
		
		// execute the callback
		if (kwArgs.sync) {
			kwArgs.onComplete(styles, kwArgs);
		}
		else {
			setTimeout(dojo.hitch(this, kwArgs.onComplete, styles, kwArgs), 0);
		}
	},
	
	/*=====
	itemStatuses_onComplete: function(itemStatuses, kwArgs){},
	=====*/
	
	/*=====
	fetchStatuses_kwArgs: function(onComplete, onError, type, start, count, sync) {
		// onComplete: itemStatuses_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// type: String
		//		object type for columns: "item", "link", "relationship"
		// start: int
		//		start index to fetch
		// count: int
		//		number of status values to fetch; null means all
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchStatuses: function(/*fetchStatuses_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of possible item status values for the Dataset
		var status = [];
		var statusObjs = null;
		
		switch(kwArgs.type) {
			case "item":
				statusObjs = this._json.itemStatus;
				break;
			case "link":
				statusObjs = this._json.linkStatus;
				break;
			case "relationship":
				statusObjs = this._json.relationshipStatus;
		}
		
		if (statusObjs) {
			var statusLength = statusObjs.length;
			var start = kwArgs.start;
			var count = kwArgs.count;
			if (!count) {
				count = statusLength;
			}
			for (var statusIndex = start;  statusIndex < statusLength && count > 0;  ++statusIndex, --count) {
				status[statusIndex] = this._newStatus(statusObjs[statusIndex]);
			}			
		}
		
		// execute the callback
		if (kwArgs.sync) {
			kwArgs.onComplete(status, kwArgs);
		}
		else {
			setTimeout(dojo.hitch(this, kwArgs.onComplete, status, kwArgs), 0);
		}		
	},

	/*=====
	tasks_onComplete: function(tasks, kwArgs){},
	=====*/
	
	/*=====
	fetchTasks_kwArgs: function(onComplete, onError, taskCategory, selection, itemIds, linkIds, itemIndexes, sync) {
		// onComplete: tasks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// taskCategory: String
		//		comma delimited list indicating type of tasks to fetch
		//		legal values are: menu, toolbar, and breadcrumb
		// selection: Object
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// itemIds: String
		//		comma delimited list of item ids that are selected
		// linkIds: String
		//		comma delimited list of link ids that are selected
		// itemIndexes: String
		//		comma delimited list of item ids or ranges that are selected
		//		ex: "0,1,10,20" or "0-10,30,40-50"
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	_fetchTasks: function(/*fetchTasks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns tasks associated with the dataset
		var tasks = [];
		var categories = kwArgs.taskCategory.split(",");
		var length = categories.length;
		
		if (this._json.tasks) {
			for (var index = 0;  index < length;  ++index) {
				var taskObjs = this._json.tasks[categories[index]];
				if (taskObjs) {
					var taskLength = taskObjs.length;
					for (var taskIndex = 0;  taskIndex < taskLength;  ++taskIndex) {
						tasks.push(this._newTask(taskObjs[taskIndex]));
					}
				}
			}
		}
		
		// execute the callback
		if (kwArgs.sync) {
			kwArgs.onComplete(tasks, kwArgs);
		}
		else {
			setTimeout(dojo.hitch(this, kwArgs.onComplete, tasks, kwArgs), 0);
		}
	},

	/*=====
	dataset_onComplete: function(dataset, kwArgs){},
	=====*/
	
	/*=====
	navigate_kwArgs: function(onComplete, onError, taskId, taskCategory, selectionsync) {
		// onComplete: dataset_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// taskId: String
		//		navigation task id
		// taskCategory: String
		//		navigation task category
		// selection: Object
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	_navigate: function(/*navigate_kwArgs*/ kwArgs) {
		// summary:
		//		Navigate to another Dataset from a navigation task
	},
	
	/*=====
	cols_onComplete: function(cols, kwArgs){},
	=====*/
	
	/*=====
	fetchColumns_kwArgs: function(onComplete, onError, type, properties, start, count, sync) {
		// onComplete: cols_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// type: String
		//		object type: "item", "link", "relationship" 
		// properties: String[]
		//		array of column ids; or keyword "current", "default", or "all"
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	_fetchColumns: function(/*fetchColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch columns from rest service
		var cols = [];
		var colObjs = this._extractColumnObjs(kwArgs.type, kwArgs.properties);
		if (colObjs) {
			var colLength = colObjs.length;
			var start = kwArgs.start;
			var count = kwArgs.count;
			if (!count) {
				count = colLength;
			}
			for (var colIndex = start;  colIndex < colLength && count > 0;  ++colIndex, --count) {
				cols[colIndex] = this._newColumn(colObjs[colIndex], kwArgs.type);
			}
		}
		
		// execute the callback
		if (kwArgs.sync) {
			kwArgs.onComplete(cols, kwArgs);
		}
		else {
			setTimeout(dojo.hitch(this, kwArgs.onComplete, cols, kwArgs), 0);
		}
	},
	
	/*=====
	setColumns_kwArgs: function(onComplete, onError, columns, type, sync) {
		// onComplete: cols_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// columns: ibm.tivoli.navmodel.Column[]
		//		array of columns to be saved
		// type: String
		//		object type: "item", "link", "relationship" 
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_setColumns: function(/*setColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Save columns
	},
	
	/*=====
	item_onComplete: function(item, kwArgs){},
	=====*/
	
	/*=====
	createItem_kwArgs: function(onComplete, onError, item) {
		// onComplete: item_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// item: Object
		//		ibm.tivoli.navmodel.Item to be created on server
	},
	=====*/
	_createItem: function(/*createItem_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new item in the dataset
	},
	
	/*=====
	createItems_kwArgs: function(onComplete, onError, item) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// item: Object[]
		//		array of ibm.tivoli.navmodel.Item to be created on server
	},
	=====*/
	_createItems: function(/*createItems_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new items in the dataset
	},
	
	/*=====
	_onComplete: function(kwArgs){},
	=====*/
	
	/*=====
	deleteItem_kwArgs: function(onComplete, onError, itemId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemId: String
		//		id of item to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of item
	},
	=====*/
	
	_deleteItem: function(/*deleteItem_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a item from the dataset
	},

	/*=====
	deleteItems_kwArgs: function(onComplete, onError, itemIds, conditions, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemIds: String[]
		//		array of item ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of items
	},
	=====*/
	
	_deleteItems: function(/*deleteItems_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete items from the dataset
	},

	/*=====
	link_onComplete: function(link, kwArgs){},
	=====*/
	
	/*=====
	createLink_kwArgs: function(onComplete, onError, link) {
		// onComplete: link_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// link: Object
		//		ibm.tivoli.navmodel.Link to be created on server
	},
	=====*/
	
	_createLink: function(/*createLink_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new link in the dataset
	},

	/*=====
	createLinks_kwArgs: function(onComplete, onError, links) {
		// onComplete: links_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// links: Object[]
		//		array of ibm.tivoli.navmodel.Link to be created on server
	},
	=====*/
	
	_createLinks: function(/*createLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new links in the dataset
	},
	
	/*=====
	deleteLink_kwArgs: function(onComplete, onError, linkId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// linkId: String
		//		id of link to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of link
	},
	=====*/
	
	_deleteLink: function(/*deleteLink_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a link from the dataset
	},

	/*=====
	deleteLinks_kwArgs: function(onComplete, onError, linkIds, conditions, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// linkIds: String[]
		//		array of link ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of links
	},
	=====*/
	
	_deleteLinks: function(/*deleteLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete links from the dataset
	},

	/*=====
	relationship_onComplete: function(relationship, kwArgs){},
	=====*/
	
	/*=====
	createRelationship_kwArgs: function(onComplete, onError, relationship) {
		// onComplete: relationship_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationship: Object
		//		ibm.tivoli.navmodel.Relationship to be created on server
	},
	=====*/
	
	_createRelationship: function(/*createRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new relationship in the dataset
	},

	/*=====
	createRelationships_kwArgs: function(onComplete, onError, relationships) {
		// onComplete: relationships_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationship: Object[]
		//		array of ibm.tivoli.navmodel.Relationship to be created on server
	},
	=====*/
	
	_createRelationships: function(/*reateRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new relationships in the dataset
	},
	
	/*=====
	deleteRelationship_kwArgs: function(onComplete, onError, relationshipId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationshipId: String
		//		id of relationship to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of relationship
	},
	=====*/
	
	_deleteRelationship: function(/*deleteRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a relationship from the dataset
	},

	/*=====
	deleteRelationships_kwArgs: function(onComplete, onError, relationshipIds, conditions, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationshipIds: String[]
		//		array of relationship ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of relationship
	},
	=====*/
	
	_deleteRelationships: function(/*deleteRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete relationships from the dataset
	},
	
	/////////////////////////////
	// protected methods
	/////////////////////////////

	_newItem: function(itemObj) {
		// summary:
		//		Create a new item
		return new ibm.tivoli.navmodel.Item({item: itemObj, dataset: this, allProperties: true, allMetadata: true});		
	},

	_newLink: function(linkObj) {
		// summary:
		//		Create a new link
		return new ibm.tivoli.navmodel.Link({link: linkObj, dataset: this, allProperties: true, allMetadata: true});		
	},

	_newItemStyle: function(styleObj) {
		// summary:
		//		Create a new item style	
		return new ibm.tivoli.navmodel.ItemStyle({style: styleObj, version: this.getVersion()});
	},
	
	_newLinkStyle: function(styleObj) {
		// summary:
		//		Create a new link style
		return new ibm.tivoli.navmodel.LinkStyle({style: styleObj, version: this.getVersion()});
	},

	_newStatus: function(statusObj) {
		// summary:
		//		Create a new status
		return new ibm.tivoli.navmodel.Status({status: statusObj, version: this.getVersion()});
	},

	_newTask: function(taskObj) {
		// summary:
		//		Create a new task
		return new ibm.tivoli.navmodel.Task({task: taskObj, version: this.getVersion()});
	},

	_newColumn: function(colObj, objectType) {
		// summary:
		//		Create a new column
		return new ibm.tivoli.navmodel.Column({column: colObj, objectType: objectType, dataset: this});
	},
	
	_validateItems: function() {
		// summary:
		//		Fetch items if we haven't already done so
		
		// get cached items
		if (!this._items) {
			this._this._extractItems();						
		}
	},

	_extractItems: function() {
		// summary:
		//		Extract items from json
		var items = [];
		
		// create items
		var itemObjs = this._json.items;
		if (itemObjs) {
			var itemLength = itemObjs.length;
			for (var itemIndex = 0;  itemIndex < itemLength;  ++itemIndex) {
				items[itemIndex] = this._newItem(itemObjs[itemIndex]);
			}
		}
		
		return items;
	},

	_extractLinks: function() {
		// summary:
		//		Extract links from json
		var links = [];
		
		// create links
		var linkObjs = this._json.links;
		if (linkObjs) {
			var linkLength = linkObjs.length;
			for (var linkIndex = 0;  linkIndex < linkLength;  ++linkIndex) {
				links[linkIndex] = this._newLink(linkObjs[linkIndex]);
			}
		}
		
		return links;	
	},

	_extractToolbar: function() {
		// summary:
		//		Extract toolbar from json
		var toolbar = [];
		return toolbar;
	},

	_extractColumnObjs: function(type, properties) {
		// summary:
		//		Extract json columns 
		var cols = [];
		var columnType = null;
		switch(type) {
			case "item":
				columnType = this._json.itemColumns;
				break;
			case "link":
				columnType = this._json.linkColumns;
				break;
			case "relationship":
				columnType = this._json.relationshipColumns;
		}
		
		if (columnType) {	
			var keyWord = properties.join(",");
			switch(keyWord) {
				case "current":
					cols = columnType.current;
					if (!cols || cols.length === 0) {
						cols = columnType.all;
					}
					break;
					
				case "default":
					cols = columnType["default"];
					if (!cols || cols.length === 0) {
						cols = columnType.all;
					}
					break;
					
				case "all":
					cols = columnType.all;
					break;
					
				default:
					var all = columnType.all;
					if (all) {
						var ids = properties;
						var idLength = ids.length;
						var length = all.length;
				
						for (var index = 0;  index < length;  ++index) {
							var colId = all[index].id;					
							for (var idIndex = 0;  idIndex < idLength;  ++idIndex) {
								if (colId === ids[idIndex]) {
									cols.push(all[index]);
									break;							
								}
							}
						}
					}
			}
		}
			
		return cols;	
	}
});