/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Column"); 

// includes
dojo.require("ibm.tivoli.navmodel.Utils");

////////////////////////////////////
// Column
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Column", null, {
	// private members
	_col: null,
	_objectType: null,
	_dataset: null,
	_condition: null,
	
	/*=====
	constructor_kwArgs: function(column, objectType, dataset) {
		// column: Object
		//		json column object
		// objectType: String
		//		object type: "item", "link", or "relationship"
		// dataset: Object
		//		ibm.tivoli.navmodel.Dataset that owns this column
	},
	=====*/

	constructor: function(/*constructor_kwArgs*/ kwArgs) {		
		// summary:
		//		Constructs Column object.		
		this._col = kwArgs.column;
		if (!this._col) {
			this._col = {};
		}
		
		this._objectType = kwArgs.objectType;
		this._dataset = kwArgs.dataset;
	},

	getAttribute: function(key) {
		// summary:
		//		Returns general attribute for requested key
		return this._col[key];
	},

	setAttribute: function(key, value) {
		// summary:
		//		Set general attribute for requested key
		this._col[key] = value;
	},
	
	getAttributes: function() {		
		// summary:
		//		Get all general attributes for the column
		return dojo.clone(this._col);			
	},

	getDataset: function() {
		// summary:
		//		Returns the dataset that owns the column
		return this._dataset;
	},
	
	getId: function() {
		// summary:
		//		Returns column id
		return this._col.id;
	},

	setId: function(id) {
		// summary:
		//		Set column id
		this._col.id = id;
	},

	getLabel: function() {
		// summary:
		//		Returns globalized column header label
		if (this._col.label) {
			// string
			if (typeof this._col.label === "string") {
				return this._col.label;
			}
			// object with key/value info
			else {
				return this._col.label.value;
			}
		}
		
		return null;		
	},

	setLabel: function(label) {
		// summary:
		//		Set globalized column header label
		this._col.label = label;		
	},	
	
	getDescription: function() {
		// summary:
		//		Returns globalized description
		if (this._col.description) {
			// string
			if (typeof this._col.description === "string") {
				return this._col.description;
			}
			// object with key/value info
			else {
				return this._col.description.value;
			}
		}
		
		return null;
	},

	setDescription: function(description) {
		// summary:
		//		Set globalized description
		this._col.description = description;		
	},	
	
	getValueType: function() {
		// summary:
		//		Returns the type associated with the column
		//		
		//		|	values:	"itemName", "itemType", "linkName", "linkType", "relationshipName", "relationshipType",
		//		|			string", "string[]",
		//		|			"byte", "short", "int", "long", "byte[]", "short[]", "int[]", "long[]",
		//		|			"ubyte, "ushort", "uint", "ulong", "ubyte[]", "ushort[]", "uint[]", "ulong[]",
		//		|			"boolean", "boolean[]"
		//		|			"enum",
		//		|			"status"
		//		|			"date", "datetime", "time", "date[]", "datetime[]", "time[]"
		//		|			"isodate", isodatetime", "isotime", "isodate[]", "isodatetime[], "isotime[]", "isointerval", "isoduration"
		//		|			"float", "double", "percentage", "float[]", "double[]"
		return this._col.valueType;
	},

	setValueType: function(valueType) {
		// summary:
		//		Set the type associated with the column	
		this._col.valueType = valueType;
	},
	
	getWidth: function() {
		// summary:
		//		Returns column width string as: pixels, percentage, or em value
		return this._col.width;
	},

	setWidth: function(width) {
		// summary:
		//		Set the column width string as: pixels, percentage, or em value
		// width: String
		//		Column width value
		this._col.width = width;
	},
	
	isResizeable: function() {
		// summary:
		//		Returns true if column is resizeable by the user
		return (this._col.resizeable !== undefined ? this._col.resizeable : true);
	},
	
	setResizeable: function(resizeable) {
		// summary:
		//		Set if the column is to be resizeable by the user
		this._col.resizeable = resizeable;
	},
	
	isLineWrap: function() {
		// summary:
		//		Returns true if column allows line wrap to multiple lines
		return (this._col.lineWrap !== undefined ? this._col.lineWrap : false);
	},
	
	setLineWrap: function(lineWrap) {
		// summary:
		//		Set if the column allows line wrap to multiple lines
		this._col.lineWrap = lineWrap;
	},
	
	isLocked: function() {
		// summary:
		//		Returns true if column locked in non-scrollable region
		return (this._col.locked !== undefined ? this._col.locked : false);
	},
	
	setLocked: function(Locked) {
		// summary:
		//		Set whether the column is locked in non-scrollable region
		this._col.locked = locked;
	},
		
	isHidden: function() {
		// summary:
		//		Returns true if the column is to be hidden from the user
		return (this._col.hidden !== undefined ? this._col.hidden : false);
	},
	
	setHidden: function(hidden) {
		// summary:
		//		Set if the column is to be hidden from the user
		this._col.hidden = hidden;
	},
	
	getImage: function() {
		// summary:
		//		Returns optional image for the column
		return this._col.image;
	},
	
	setImage: function(image) {
		// summary:
		//		Set optional image for the column
		this._col.image = image;
	},
	
	getSortOrder: function() {
		// summary:
		//		Returns sort order priority (value is 0 based and should be -1 if not sorted)
		return (this._col.sortOrder !== undefined ? this._col.sortOrder : -1);
	},

	setSortOrder: function(sortOrder) {
		// summary:
		//		Set sort order priority (value is 0 based and should be -1 if not sorted)
		this._col.sortOrder = sortOrder;
	},

	isSortable: function() {
		// summary:
		//		Returns true if the column is sortable
		return (this._col.sortable !== undefined ? this._col.sortable : true);
	},

	setSortable: function(sortable) {
		// summary:
		//		Set if the column is sortable
		this._col.sortable = sortable;
	},

	isSortAscending: function() {
		// summary:
		//		Returns true if the sort is ascending or descending
		return (this._col.sortAscending !== undefined ? this._col.sortAscending : true);
	},

	setSortAscending: function(ascending) {
		// summary:
		//		Set if the sort is ascending or descending
		this._col.sortAscending = ascending;
	},

	isSearchable: function() {
		// summary:
		//		Returns true if the column is searchable
		return (this._col.searchable !== undefined ? this._col.searchable : true);
	},

	setSearchable: function(searchable) {
		// summary:
		//		Set if the column is searchable
		this._col.searchable = searchable;
	},

	isFilterable: function() {
		// summary:
		//		Returns true if the column is filterable
		return (this._col.filterable !== undefined ? this._col.filterable : true);
	},
	
	setFilterable: function(filterable) {
		// summary:
		//		Set if the column is filterable
		this._col.filterable = filterable;
	},
	
	isFiltered: function() {
		// summary:
		//		Returns true if the column is currently filtered
		return (this._col.filtered !== undefined ? this._col.filtered : false);
	},

	setFiltered: function(filtered) {
		// summary:
		//		Set if the column is currently filtered
		this._col.filtered = filtered;
	},
	
	getFilter: function() {
		// summary:
		//		Returns filter object for the column		
		return this._col.filter;
	},
	
	getUnsupportedFilterOperators: function() {
		// summary:
		//		Returns String[] of operators that are not supported for this
		//		column or null if all operators are supported.
		return (this._col.filter ? this._col.filter.unsupportedOperators : null);		
	},
	
	hasPossibleValues: function() {
		// summary:
		//		Returns true if the column supports a list of possible values		
		var filter = this._col.filter;
		return ((filter && (filter.possibleValues || filter.possibleValuesUri)) ? true : false);
	},
	
	hasInlinedPossibleValues: function() {
		// summary:
		//		Returns true if the column supports possible values and
		//		all of those values have been inlined in the json
		var filter = this._col.filter;
		return (filter && filter.possibleValues ? true : false);
	},
	
	hasUrlAddressablePossibleValues: function() {
		// summary:
		//		Returns true if the column supports possible values and
		//		the values are addressable via a url
		var filter = this._col.filter;
		return (filter && filter.possibleValuesUri ? true : false);
	},
	
	getPossibleValues: function(kwArgs) {
		// summary:
		//		Retrieves a list of possible values for columns with a valueType of
		//		enum, map, itemType, linkType, or relationshipType
		var args = {
			onComplete: function(values, totalCount, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			start: 0,
			count: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
	
		var filter = this._col.filter;
		
		// possible values were inlined; just return them
		if (filter && filter.possibleValues) {
			if (args.start === 0 && args.count === null) {
				args.onComplete(filter.possibleValues, filter.possibleValues.length, args);
			}
			else {
				var values = [];
				var length = filter.possibleValues.length;
				var count = args.count ? args.count : length;
				for (var index = args.start;  index < length && count > 0;  ++index, --count) {
					values.push(filter.possibleValues[index]);
				}
				args.onComplete(values, length, args);
			}
		}
		// possible values are available via uri
		else if (filter && filter.possibleValuesUri) {
			var content = ibm.tivoli.navmodel.Utils.getContent();
			ibm.tivoli.navmodel.Utils.mixinParams(content, this._dataset);
			if (args.start) {
				content.start = args.start;
			}
			if (args.count) {
				content.count = args.count;
			}
			if (this._objectType) {
				content.type = this._objectType;
			}
			
			dojo.xhrGet({
				url: this._dataset.getArg("baseUrl") + filter.possibleValuesUri,
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "columnValueList")
				},
				handleAs: "json",
				sync: args.sync,
				content: content,		
				load: dojo.hitch(this, function(result, ioArgs){
					args.onComplete(result.items, result.totalRows, args);					
				}),
				error: dojo.hitch(this, function(error, ioArgs){
					args.onError(error, ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "getPossibleValues", this._trace, this._version()), ioArgs);
				})
			});	
		}
		// doesn't have any
		else {
			args.onComplete(null, 0, args);
		}		
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////

	_version: function() {		
		// summary:
		//		Returns rest version
		return this._dataset.getVersion();
	}
});
