/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Dataset"); 
 
// includes
dojo.require("ibm.tivoli.navmodel.AbstractDataset");

ibm.tivoli.navmodel.Dataset._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.Dataset");

////////////////////////////////////
// Dataset
//
// NOTE: do not call any abstract methods directly;
// call the public version of the methods defined in
// AbstractDataset instead and it will in turn call
// the abstract overrides.
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Dataset", ibm.tivoli.navmodel.AbstractDataset, {
	// private members
	_optimize: true,
	_trace: ibm.tivoli.navmodel.Dataset._trace, 
    _oneTimeSetup: false,
    _createEventRouterFlag: false,
    
	
	/*=====
	constructor_onComplete: function(dataset, kwArgs){},
	=====*/
	
	/*=====
	constructor_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	constructor_onPage: function(start, count, ioArgs){},
	=====*/
	
	/*=====
	 constructor_kwArgs: function(onComplete, onError, onPage, baseUrl, provider, datasource, dataset, 
	 					parameters, defaultProperties, defaultLinkProperties, defaultRelProperties, 
	 					cacheThreshold, cacheValidationInterval, cacheExpirationTime, version, eventRouter) {
		// onComplete: constructor_onComplete
	 	//		callback to be invoked when the Dataset is initialized and ready for methods
	 	//		to be invoked  (i.e. asynchronous initialization must be performed). 
	 	//		Note: A null value will result in synchronous initialization.
	 	// onError: constructor_onError
	 	//		callback to be invoked if there is a problem constructing the Dataset
	 	// onPage:  constructor_onPage
	 	//		callback to be invoked when a new page is requested
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// dataset: String
		//		dataset id
		// parameters: Object
		//		optional: extra parameters sent to dataset provider
		// defaultProperties: String[]
		//		optional: item property ids to fetch by default (can specify "all")
		// defaultLinkProperties: String[]
		//		optional: link property ids to fetch by default (can specify "all")
		// defaultRelProperties: String[]
		//		optional: relationship property ids to fetch by default (can specify "all")
		// cacheThreshold: int[]
		//		optional: cache objects if number of dataset items >= threshold
		// cacheValidationInterval: int[]
		//		optional: number of minutes in between cache validations
		// cacheExpirationTime: int[]
		//		optional: number of minutes objects can stay in cache while inactive
		// version: Float
		//		optional: version used to communicate with rest service (i.e. json version)
		// eventRouter: Object
		//		optional: object contains the following values (defaults shown)
		//		eventRouter: {
		//			timeout : 20,    		* number of seconds to wait for events  (optional)
		//			pollDelay : 0,			* number of ms to wait after an error before next poll  (optional)
		//			pollErrorDelay : 5000	* number of ms to wait after an error before next poll  (optional)
	   	//		},	   	
		//		Use With Caution:  If you are the first Dataset to create an EventRouter object, the parameters 
		// 		specified here will dictate timeout, pollDelay and pollErrorDelay for ALL Datasets in the browser. 
  		//		If a Dataset has already been constructed, these parameters will have NO effect.  If the Dataset 
  		//		is destroyed, the event router will shutdown and new Dataset instances and their settings will be used.	
	},
	=====*/

	constructor: function(/*constructor_kwArgs_kwArgs*/ kwArgs, dataset) {	
		// summary: 
		//		Constructs Dataset object.
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception		
	},

	//////////////////////////////////////////////////////
	// abstract method overrides
	//
	// NOTE: do not call any abstract methods directly;
	// call the public version of the methods defined in
	// AbstractDataset instead and it will in turn call
	// the abstract overrides.
	//////////////////////////////////////////////////////

	/*=====
	initDataset_kwArgs: function(createEventRouter, optimize) {
		// createEventRouter: Boolean
		//		boolean indicating whether new event router should be created
		// optimize: Boolean
		//		optimize json that is returned; remove items that can be calculated
		//		or can be retrieved via additional urls
	},
	=====*/
	
	_initDataset: function(/*initDataset_kwArgs*/ kwArgs) {
		// summary:
		//		Initialize internal dataset object
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		
		// start out with empty dataset
		this._initEmptyDataset();		
		this._optimize = kwArgs.optimize;
		
		// make synchronous call if no onComplete callback
		var synchronous = (kwArgs.onComplete ? false : true);
		var args = arguments;
		var exception = null;
		
		// dataset was NOT provided; fetch it
		if (!kwArgs.dataset) {			
			var content = ibm.tivoli.navmodel.Utils.getContent();
			this._mixinStandard(content, kwArgs);			
			ibm.tivoli.navmodel.Utils.mixinParams(content, this);
			
            if (kwArgs.fetchMetaData) {
                dojo.xhrGet({
                    url: this._getDatasetURL(),
                    headers: {
                        'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "dataset")
                    },
                    handleAs: "json",
                    sync: synchronous,
                    content: content,
                    load: dojo.hitch(this, function(result, ioArgs) {
                        // pull version from the Content-Type response header
                        this._setVersion(this._extractContentTypeVersion(ioArgs));
                        
                        // mixin all dataset attributes
                        this.mixinAttributes(result);
					
                        // listen for asynchronous events
                        if (kwArgs.createEventRouter && this.isEventsEnabled()) {
                            this._createEventRouter();
                        }
		
                        // call base class
                        this.inherited(args);
					
                        // call onComplete callback
                        if (kwArgs.onComplete) {
                            kwArgs.onComplete(this, kwArgs);
                        }							
                    }),
                    error: dojo.hitch(this, function(error, ioArgs){
                        exception = this._getException(error, ioArgs, "_initDataset");					
                        if (kwArgs.onError) {
                            kwArgs.onError(error, exception, ioArgs);
                        }						
                    })
                });	
            } else {
                this._createEventRouterFlag = kwArgs.createEventRouter;
				// call onComplete callback
				if (kwArgs.onComplete) {
                    // to support do not fetch Meta Data
					this._dataset.itemCount = -1;
					kwArgs.onComplete(this, kwArgs);
				} else {
                    console.log ("onComplete does not exist: " + kwArgs.onComplete);
                }
            }
			
			if (synchronous && exception) {
				throw exception;
			}						
		}
		
		// dataset was provided
		else {
			this.mixinAttributes(kwArgs.dataset);
			
			// listen for asynchronous events
			if (kwArgs.createEventRouter && this.isEventsEnabled()) {
				this._createEventRouter();
			}
		
			// call base class
			this.inherited(args);			
		}
	},
	
	_createEventRouter: function() {
		// summary:
		//		create event router object and set on the base class
		var baseUrl = this.getArg("baseUrl");
		var contextRoot = baseUrl.substring(0, baseUrl.lastIndexOf("/"));
		var eventRouterArgs = { 
			eventDestination: this.getAttribute("eventDestination"),
			contextRoot: contextRoot
		};		
		var evArgs = this.getArg("eventRouter");
		if (evArgs) {
			dojo.mixin(eventRouterArgs, evArgs);
		}
				
		var eventRouter = new ibm.tivoli.navmodel.EventRouter(eventRouterArgs);
			
		this._setEventRouter(eventRouter);		
	},

	_deleteDataset: function() {
		// summary:
		//		Dataset is no longer in use
		if (this._trace.isFinest()) {
			this._trace.finest("sending DELETE for dataset=" + this.getId(), "_deleteDataset");
		}
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		
		return dojo.xhrDelete({
			url: this._getDatasetURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion())
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){			
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				this._getException(error, ioArgs, "_deleteDataset");
			})
		});
	},
	
	
	_doKeepAlive: function(/*fetchVersion_kwArgs*/ kwArgs) {        // 255243
		// summary:
		//		Query the dataset attributes again to keep the connection with the provider alive
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception

      if (this._trace.isFine()) {
          this._trace.fine("sending keep-alive for remote dataset=" + this.getId(), "_doKeepAlive");
      }
		var exception = null;

		// need original "param_" values to locate cached dataset	
      var content = ibm.tivoli.navmodel.Utils.getContent();
      this._mixinStandard(content, kwArgs);			
      ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		
		dojo.xhrGet({
			url: this._getDatasetURL(),
			headers: {
				'Accept': "application/json"
			},
			handleAs: "json",
			sync: false,
         content: content,
			load: dojo.hitch(this, function(result, ioArgs){
                  // nothing to do
                  if (this._trace.isFine()) {
                      this._trace.fine("...keep-alive request returned for remote dataset=" + this.getId(), "_doKeepAlive");
                  }
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				exception = this._getException(error, ioArgs, "_doKeepAlive");
			})
		});
		
		if (exception) {
			throw exception;
		}
	},
	

	/*=====
	fetchItemsAndLinks_onComplete: function(items, links, datasetItems, toolbar, kwArgs){},
	=====*/
	
	/*=====
	_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	fetchItemsAndLinks_kwArgs: function(onComplete, onError, sync) {
		// onComplete: fetchItemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchItemsAndLinks: function(/*fetchItemsAndLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns both the items and links associated with the dataset
		var defaultProperties = this.getArg("defaultProperties");
		var defaultLinkProperties = this.getArg("defaultLinkProperties");
		var defaultRelProperties = this.getArg("defaultRelProperties");
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "itemProperties", defaultProperties);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "linkProperties", defaultLinkProperties);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "relationshipProperties", defaultRelProperties);		
		content.optimize = this._optimize;
		content.items = true;
		content.links = true;
		content.toolbarTasks = true;
		
		return dojo.xhrGet({
			url: this._getDatasetURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "dataset")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var expiration = this.getNewExpirationTime();
				var items = [];				
				var links = [];
				var toolbar = [];
				var styles = null;
				var status = null;
				var length;
				var index;
				var version = this.getVersion();
				
				this._setItemCount(result.itemCount);
				this._setLinkCount(result.linkCount);
				this._setRelationshipCount(result.relationshipCount);
										
				if (result.itemStyles) {
					styles = [];
					length = result.itemStyles.length;
					for (index = 0;  index < length;  ++index) {
						styles[index] = new ibm.tivoli.navmodel.ItemStyle({style: result.itemStyles[index], version: version});
					}
					this._cacheItemStyles(styles);
				}
				
				if (result.itemStatus) {
					status = [];
					length = result.itemStatus.length;
					for (index = 0;  index < length;  ++index) {
						status[index] = new ibm.tivoli.navmodel.Status({status: result.itemStatus[index], version: version});
					}
					this._cacheItemStatuses(status);
				}
				
				if (result.items) {
					var allProps = this._isAllProps(defaultProperties);					
					var propIds = this._getPropIds(defaultProperties, allProps);
					length = result.items.length;
					for (index = 0;  index < length;  ++index) {
						items[index] = new ibm.tivoli.navmodel.Item({item: result.items[index], dataset: this, propIds: propIds, allProperties: allProps, expiration: expiration});
					}
				}
				
				if (result.linkStyles) {
					styles = [];
					length = result.linkStyles.length;
					for (index = 0;  index < length;  ++index) {
						styles[index] = new ibm.tivoli.navmodel.LinkStyle({style: result.linkStyles[index], version: version});
					}
					this._cacheLinkStyles(styles);
				}
				
				if (result.relationshipStyles) {
					styles = [];
					length = result.relationshipStyles.length;
					for (index = 0;  index < length;  ++index) {
						styles[index] = new ibm.tivoli.navmodel.LinkStyle({style: result.relationshipStyles[index], version: version});
					}
					this._cacheRelationshipStyles(styles);
				}
				
				if (result.linkStatus) {
					status = [];
					length = result.linkStatus.length;
					for (index = 0;  index < length;  ++index) {
						status[index] = new ibm.tivoli.navmodel.Status({status: result.linkStatus[index], version: version});
					}
					this._cacheLinkStatuses(status);
				}
				
				if (result.relationshipStatus) {
					status = [];
					length = result.relationshipStatus.length;
					for (index = 0;  index < length;  ++index) {
						status[index] = new ibm.tivoli.navmodel.Status({status: result.relationshipStatus[index], version: version});
					}
					this._cacheRelationshipStatuses(status);
				}
				
				if (result.links) {
					var allLinkProps = this._isAllProps(defaultLinkProperties);
					var linkPropIds = this._getPropIds(defaultLinkProperties, allLinkProps);
					length = result.links.length;
					for (index = 0;  index < length;  ++index) {
						links[index] = new ibm.tivoli.navmodel.Link({link: result.links[index], dataset: this, propIds: linkPropIds, allProperties: allLinkProps, expiration: expiration});
					}
				}
				
				if (result.toolbarTasks) {
					length = result.toolbarTasks.length;
					for (index = 0;  index < length;  ++index) {
						toolbar[index] = new ibm.tivoli.navmodel.Task({task: result.toolbarTasks[index], dataset: this});
					}
				}
				
				kwArgs.onComplete(items, links, toolbar, kwArgs);				
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchItemsAndLinks"), ioArgs);
			})
		});
	},

	/*=====
	items_onComplete: function(items, kwArgs){},
	=====*/

	/*=====
	fetchItems_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	_fetchItems: function(/*fetchItems_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch items from rest service
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);

		console.log ("url: " + this._getItemsURL());
		
		return dojo.xhrGet({
			url: this._getItemsURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemViewList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				console.log ("in Dataset._fetchItems.load");
                // Begin JSA Change for no Meta Data
				if (this._oneTimeSetup == false) {
				  this._oneTimeSetup = true;
				  // pull version from the Content-Type response header
				  this._setVersion(this._extractContentTypeVersion(ioArgs));
				
				  // listen for asynchronous events
				  if (this._createEventRouterFlag && this.isEventsEnabled()) {
					  this._createEventRouter();
				  }
				}            
                // End JSA Change for no Meta Data
            
				var items = [];				
				if (result.items) {
					var expiration = this.getNewExpirationTime();
					var allProps = this._isAllProps(kwArgs.properties);
					var propIds = this._getPropIds(kwArgs.properties, allProps);
					var itemsLength = result.items.length;
					for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
						var item = result.items[itemIndex];
						if (item.datasetId && item.datasetId !== this.getId()) {
							var dataset = this.getDescendentDataset(item.datasetId);
							items[itemIndex] = new ibm.tivoli.navmodel.Item({item: item, dataset: dataset, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});							
						}
						else {
							items[itemIndex] = new ibm.tivoli.navmodel.Item({item: item, dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
						}
					}
				}
				
				// make sure our count is accurate
				this._setItemCount(result.totalRows);
					
				// place counts in the arguments map
				kwArgs.numRows = result.numRows;
				kwArgs.filteredRows = result.filteredRows;
				kwArgs.totalRows = result.totalRows;
				kwArgs.onComplete(items, kwArgs);	
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchItems"), ioArgs);
			})
		});
	},
	
	/*=====
	fetchItem_kwArgs: function(itemId, properties, metadata) {
		// itemId: String
		//		item id to get
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/
	
	_fetchItem: function(/*fetchItem_kwArg*/ kwArgs) {
		// summary:
		//		Synchronously fetch an individual item from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var item = null;
		var exception = null;
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		dojo.xhrGet({
			url: this._getItemURL(kwArgs.itemId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemView")
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var allProps = this._isAllProps(kwArgs.properties);
				var propIds = this._getPropIds(kwArgs.properties, allProps);
				item = new ibm.tivoli.navmodel.Item({item: result, dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata});											
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				exception = this._getException(error, ioArgs, "_fetchItem");
			})
		});
		
		if (exception) {
			throw exception;
		}
				
		return item;
	},

	/*=====
	fetchOriginItems_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metdata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchOriginItems: function(/*fetchOriginItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns orign items of the dataset
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		return dojo.xhrGet({
			url: this._getOriginItemsURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "originItemViewList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var expiration = this.getNewExpirationTime();				
				var items = [];
				if (result.items) {
					var allProps = this._isAllProps(kwArgs.properties);
					var propIds = this._getPropIds(kwArgs.properties, allProps);
					var itemsLength = result.items.length;
					for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
						items[itemIndex] = new ibm.tivoli.navmodel.Item({item: result.items[itemIndex], dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
					}
				}
				
				// place counts in the arguments map
				kwArgs.numRows = result.numRows;
				kwArgs.filteredRows = result.filteredRows;
				kwArgs.totalRows = result.totalRows;
				
				kwArgs.onComplete(items, kwArgs);				
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchOriginItems"), ioArgs);
			})
		});
	},

	/*=====
	itemsAndLinks_onComplete: function(items, links, datasetItems, kwArgs){},
	=====*/
	
	/*=====
	fetchRelatedItems_kwArgs: function(onComplete, onError, itemId, relationships, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: itemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemId: String
		//		id of the base item
		// relationships: String
		//		comma delimited list of relationships to determine related items; 
		//		can also be keywords: child, parent, all
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchRelatedItems: function(/*fetchRelatedItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items that are related to the requested item
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		// relationships to explore
		if (kwArgs.relationships) {
			content.relationships = kwArgs.relationships;
		}
		
		return dojo.xhrGet({
			url: this._getRelatedItemsURL(kwArgs.itemId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relatedItemViewList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var expiration = this.getNewExpirationTime();
				var items = [];
				if (result.items) {
					if (result.datasetItemCount !== undefined) {
						this._setItemCount(result.datasetItemCount);
					}
					
					var allProps = this._isAllProps(kwArgs.properties);
					var propIds = this._getPropIds(kwArgs.properties, allProps);
					var itemsLength = result.items.length;
					for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
						items[itemIndex] = new ibm.tivoli.navmodel.Item({item: result.items[itemIndex], dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});							
					}
				}
				
				var links = [];
				if (result.links) {
					if (result.datasetLinkCount !== undefined) {
						this._setLinkCount(result.datasetLinkCount);
					}
					
					if (result.datasetRelationshipCount !== undefined) {
						this._setRelationshipCount(result.datasetRelationshipCount);
					}					
					
					var linksLength = result.links.length;
					for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
						links[linkIndex] = new ibm.tivoli.navmodel.Link({link: result.links[linkIndex], dataset: this, expiration: expiration});
					}
				}
				
				// place counts in the arguments map
				kwArgs.numRows = result.numRows;
				kwArgs.filteredRows = result.filteredRows;
				kwArgs.totalRows = result.totalRows;
				
				kwArgs.onComplete(items, links, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchRelatedItems"), ioArgs);
			})
		});
	},
	
	/*=====
	queryItems_kwArgs: function(onComplete, onError, query, start, count, properties, metadata, sync) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// query: Object
		//		custom query object to be sent as request body
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	 _queryItems: function(/*queryItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items associated with the dataset that match
		//		the provided custom query
		var json = dojo.toJson(kwArgs.query, false);
				
		return dojo.xhrPost({
			url: this._getItemsQueryURL(kwArgs),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemViewList"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "customQuery"),
				'Content-Length': json.length,
				'X-Method-Override': 'Custom-GET'
			},
			handleAs: "json",
			postData: json,
			sync: kwArgs.sync,
			load: dojo.hitch(this, function(result, ioArgs){
				var expiration = this.getNewExpirationTime();
				var items = [];				
				if (result.items) {
					var allProps = this._isAllProps(kwArgs.properties);				
					var propIds = this._getPropIds(kwArgs.properties, allProps);
					var itemsLength = result.items.length;
					for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
						items[itemIndex] = new ibm.tivoli.navmodel.Item({item: result.items[itemIndex], dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
					}
				}
				
				kwArgs.onComplete(items, kwArgs);	
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_queryItems"), ioArgs);
			})
		});
	},
	
	/*=====
	links_onComplete: function(items, kwArgs){},
	=====*/

	/*=====
	fetchLinks_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: links_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions
		// start: int
		//		start index to fetch
		// count: int
		//		number of links to fetch; null means all
		// properties: String[]
		//		link property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchLinks: function(/*fetchLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch links from rest service
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		return dojo.xhrGet({
			url: this._getLinksURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkViewList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var expiration = this.getNewExpirationTime();
				var links = [];							
				if (result.items) {
					var allProps = this._isAllProps(kwArgs.properties);
					var propIds = this._getPropIds(kwArgs.properties, allProps);
					var linksLength = result.items.length;
					for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
						var link = result.items[linkIndex];
						if (link.datasetId && link.datasetId !== this.getId()) {
							var dataset = this.getDescendentDataset(link.datasetId);
							links[linkIndex] = new ibm.tivoli.navmodel.Link({link: link, dataset: dataset, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
						}
						else {									
							links[linkIndex] = new ibm.tivoli.navmodel.Link({link: link, dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
						}
					}
				}
				
				// make sure our count is accurate
				this._setLinkCount(result.totalRows);
					
				// place counts in the arguments map
				kwArgs.numRows = result.numRows;
				kwArgs.filteredRows = result.filteredRows;
				kwArgs.totalRows = result.totalRows;				
				kwArgs.onComplete(links, kwArgs);	
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchLinks"), ioArgs);
			})
		});
	},
	
	/*=====
	fetchLink_kwArgs: function(itemId, properties, metadata) {
		// linkId: String
		//		link id to get
		// properties: String[]
		//		array of property ids to pre-fetch
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/
	
	_fetchLink: function(/*fetchLink_kwArgs*/ kwArgs) {
		// summary:
		//		Synchronously fetch an individual link from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var link = null;
		var exception = null;
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		dojo.xhrGet({
			url: this._getLinkURL(kwArgs.linkId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkView")
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var allProps = this._isAllProps(kwArgs.properties);
				var propIds = this._getPropIds(kwArgs.properties, allProps);
				link = new ibm.tivoli.navmodel.Link({link: result, dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata});			
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				exception = this._getException(error, ioArgs, "_fetchLink");
			})
		});
		
		if (exception) {
			throw exception;
		}
		
		return link;
	},

	/*=====
	relationships_onComplete: function(relationships, kwArgs){},
	=====*/
	
	/*=====
	fetchRelationships_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: relationships_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions
		// start: int
		//		start index to fetch
		// count: int
		//		number of relationships to fetch; null means all
		// properties: String[]
		//		relationship property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchRelationships: function(/*fetchRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch relationships from rest service
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		return dojo.xhrGet({
			url: this._getRelationshipsURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relationshipViewList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var expiration = this.getNewExpirationTime();
				var relationships = [];				
				if (result.items) {
					var allProps = this._isAllProps(kwArgs.properties);
					var propIds = this._getPropIds(kwArgs.properties, allProps);
					var relLength = result.items.length;
					for (var relIndex = 0;  relIndex < relLength;  ++relIndex) {
						var relationship = result.items[relIndex];
						if (relationship.datasetId && relationship.datasetId !== this.getId()) {
							var dataset = this.getDescendentDataset(relationship.datasetId);
							relationships[relIndex] = new ibm.tivoli.navmodel.Relationship({relationship: relationship, dataset: dataset, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
						}
						else {									
							relationships[relIndex] = new ibm.tivoli.navmodel.Relationship({relationship: relationship, dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
						}						
					}
				}
								
				// make sure our count is accurate
				this._setRelationshipCount(result.totalRows);
					
				// place counts in the arguments map
				kwArgs.numRows = result.numRows;
				kwArgs.filteredRows = result.filteredRows;
				kwArgs.totalRows = result.totalRows;				
				kwArgs.onComplete(relationships, kwArgs);	
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchRelationships"), ioArgs);
			})
		});
	},

	/*=====
	fetchRelationship_kwArgs: function(relationshipId, properties, metadata) {
		// relationshipId: String
		//		relationship id to fetch
		// properties: String[]
		//		array of property ids to pre-fetch
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/
	
	_fetchRelationship: function(/*fetchRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Synchronously fetch an individual relationship from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var relationship = null;
		var exception = null;
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		dojo.xhrGet({
			url: this._getRelationshipURL(kwArgs.relationshipId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relationshipView")
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var allProps = this._isAllProps(kwArgs.properties);
				var propIds = this._getPropIds(kwArgs.properties, allProps);
				relationship = new ibm.tivoli.navmodel.Relationship({relationship: result, dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata});			
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				exception = this._getException(error, ioArgs, "_fetchRelationship");
			})
		});
		
		if (exception) {
			throw exception;
		}
		
		return relationship;
	},
	
	/*=====
	itemStyles_onComplete: function(itemStyles, kwArgs){},
	=====*/
	
	/*=====
	fetchItemStyles_kwArgs: function(onComplete, onError, start, count, sync) {
		// onComplete: itemStyle_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of item styles to fetch;  null means all
		// sync: Boolean
		//		indicates if fetch is synchronous		
	},
	=====*/
	
	_fetchItemStyles: function(/*fetchItemStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current item styles for the Dataset
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		
		return dojo.xhrGet({
			url: this._getItemStylesURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemStyleList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var styles = [];
				var stylesLength = result.items.length;
				for (var index = 0;  index < stylesLength;  ++index) {
					styles[index] = new ibm.tivoli.navmodel.ItemStyle({style: result.items[index], version: this.getVersion()});
				}
				
				kwArgs.onComplete(styles, kwArgs);				
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchItemStyles"), ioArgs);
			})
		});
	},

	/*=====
	linkStyles_onComplete: function(linkStyles, kwArgs){},
	=====*/
	
	/*=====
	fetchLinkStyles_kwArgs: function(onComplete, onError, start, count, sync) {
		// onComplete: linkStyles_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of link styles to fetch; null means all
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchLinkStyles: function(/*fetchLinkStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current link styles for the Dataset
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		
		return dojo.xhrGet({
			url: this._getLinkStylesURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkStyleList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var styles = [];
				var stylesLength = result.items.length;
				for (var index = 0;  index < stylesLength;  ++index) {
					styles[index] = new ibm.tivoli.navmodel.LinkStyle({style: result.items[index], version: this.getVersion()});
				}
				
				kwArgs.onComplete(styles, kwArgs);				
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchLinkStyles"), ioArgs);
			})
		});		
	},

	/*=====
	relStyles_onComplete: function(relStyles, kwArgs){},
	=====*/
	
	/*=====
	fetchRelationshipStyles_kwArgs: function(onComplete, onError, start, count, sync) {
		// onComplete: relStyles_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of relationship styles to fetch; null means all
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchRelationshipStyles: function(/*fetchRelationshipStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current relationship styles for the Dataset
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		
		return dojo.xhrGet({
			url: this._getRelationshipStylesURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkStyleList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var styles = [];
				var stylesLength = result.items.length;
				for (var index = 0;  index < stylesLength;  ++index) {
					styles[index] = new ibm.tivoli.navmodel.LinkStyle({style: result.items[index], version: this.getVersion()});
				}
				
				kwArgs.onComplete(styles, kwArgs);				
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchRelationshipStyles"), ioArgs);
			})
		});		
	},

	/*=====
	itemStatuses_onComplete: function(itemStatuses, kwArgs){},
	=====*/
	
	/*=====
	fetchStatuses_kwArgs: function(onComplete, onError, type, start, count, sync) {
		// onComplete: itemStatuses_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// type: String
		//		object type for columns: "item", "link", "relationship"
		// start: int
		//		start index to fetch
		// count: int
		//		number of status values to fetch; null means all
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	_fetchStatuses: function(/*fetchStatuses_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of possible item status values for the Dataset
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);		
		if (kwArgs.type) {
			content.type = kwArgs.type;
		}
		
		return dojo.xhrGet({
			url: this._getStatusURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "statusList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var status = [];
				var statusLength = result.items.length;
				for (var index = 0;  index < statusLength;  ++index) {
					status[index] = new ibm.tivoli.navmodel.Status({status: result.items[index], version: this.getVersion()});
				}
				
				kwArgs.onComplete(status, kwArgs);				
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchStatuses"), ioArgs);
			})
		});
	},

	/*=====
	tasks_onComplete: function(tasks, kwArgs){},
	=====*/
	
	/*=====
	fetchTasks_kwArgs: function(onComplete, onError, taskCategory, selection, itemIds, linkIds, itemIndexes, sync) {
		// onComplete: tasks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// taskCategory: String
		//		comma delimited list indicating type of tasks to fetch
		//		legal values are: menu, toolbar, and breadcrumb
		// selection: Object
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// itemIds: String
		//		comma delimited list of item ids that are selected
		// linkIds, String
		//		comma delimited list of link ids that are selected
		// itemIndexes: String
		//		comma delimited list of item ids or ranges that are selected
		//		ex: "0,1,10,20" or "0-10,30,40-50"
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchTasks: function(/*fetchTasks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns tasks associated with the dataset
		
		// use "selection"
		if (kwArgs.selection) {
			var json = dojo.toJson(kwArgs.selection.getAttributes(), false);
			
			return dojo.xhrPost ({
				url: this._getTasksParamURL(kwArgs),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "taskList"),
					'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "taskContext"),
					'Content-Length': json.length
				},
				handleAs: "json",
				postData: json,
				sync: kwArgs.sync,			
				load: dojo.hitch(this, function(result, ioArgs) {
					var tasks = [];
					var rawTasks = result.items;
					var rawTasksLength = rawTasks.length;
					for (var index = 0;  index < rawTasksLength;  ++index) {
						tasks[index] = new ibm.tivoli.navmodel.Task({task: rawTasks[index], dataset: this});
					}
					kwArgs.onComplete(tasks, kwArgs);
				}),
				error: dojo.hitch(this, function(error, ioArgs) {
					kwArgs.onError(error, this._getException(error, ioArgs, "getTasks"), ioArgs);	               
				})
			});
		}
		
		// use "itemIds", "linkIds", and "itemIndexes"
		else {
			// need original "param_" values to locate cached dataset
			var content = ibm.tivoli.navmodel.Utils.getContent();
			this._mixinStandard(content, kwArgs);
			ibm.tivoli.navmodel.Utils.mixinParams(content, this);
			
			// comma delimitted categories (i.e. "menu,toolbar,breadcrumb")
			content.category = kwArgs.taskCategory;

			// indicate what is selected
			if (kwArgs.itemIds) {
				content.itemIds = kwArgs.itemIds;
			}
			if (kwArgs.linkIds) {
				content.linkIds = kwArgs.linkIds;
			}
			if (kwArgs.itemIndexes) {
				content.itemIndexes = kwArgs.itemIndexes;
			}

			return dojo.xhrGet({
				url: this._getTasksURL(),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "taskList")
				},
				handleAs: "json",
				sync: kwArgs.sync,
				content: content,
				load: dojo.hitch(this, function(result, ioArgs){
					var tasks = [];
					var rawTasks = result.items;
					var rawTasksLength = rawTasks.length;
					for (var index = 0;  index < rawTasksLength;  ++index) {
						tasks[index] = new ibm.tivoli.navmodel.Task({task: rawTasks[index], dataset: this});
					}
					kwArgs.onComplete(tasks, kwArgs);
				}),
				error: dojo.hitch(this, function(error, ioArgs){
					kwArgs.onError(error, this._getException(error, ioArgs, "getTasks()"), ioArgs);
				})
			});
		}
	},

	/*=====
	dataset_onComplete: function(dataset, kwArgs){},
	=====*/
	
	/*=====
	navigate_kwArgs: function(onComplete, onError, taskId, taskCategory, selection, sync) {
		// onComplete: dataset_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// taskId: String
		//		navigation task id
		// taskCategory: String
		//		navigation task category
		// selection: Object
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_navigate: function(/*navigate_kwArgs*/ kwArgs) {
		// summary:
		//		Navigate to another Dataset from a navigation task
				
		// use "selection"
		var json = dojo.toJson(kwArgs.selection.getAttributes(), false);
			
		return dojo.xhrPost ({
			url: this._getNavigateParamURL(kwArgs),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "dataset"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "taskContext"),
				'Content-Length': json.length
			},
			handleAs: "json",
			postData: json,
			sync: kwArgs.sync,			
			load: dojo.hitch(this, function(result, ioArgs) {
				var navigateArgs = this.getArgs();
				navigateArgs.dataset = result.id;
				navigateArgs.parameters = {refId: ""};  // CURI compatability: include empty refId so we don't create one
				navigateArgs.defaultProperties = [];
				navigateArgs.defaultLinkProperties = [];
				navigateArgs.defaultRelProperties = [];
				navigateArgs.createEventRouter = false; // will share EventRouter
				navigateArgs.onComplete = null;
				navigateArgs.onError = null;
						
		  		// construct navigation dataset (share EventRouter)
				var dataset = new ibm.tivoli.navmodel.Dataset(navigateArgs, result);
				dataset._setEventRouter(this._getEventRouter());
			
				// pass new dataset back to caller
				kwArgs.onComplete(dataset, kwArgs);		
			}),
			error: dojo.hitch(this, function(error, ioArgs) {
				kwArgs.onError(error, this._getException(error, ioArgs, "navigate"), ioArgs);	               
			})
		});			      
	},

	/*=====
	cols_onComplete: function(cols, kwArgs){},
	=====*/
	
	/*=====
	fetchColumns_kwArgs: function(onComplete, onError, type, properties, start, count, metadata, sync) {
		// onComplete: cols_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// type: String
		//		object type: "item", "link", "relationship" 
		// properties: String[]
		//		array of column ids; or keyword "current", "default", or "all"
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// metadata: Boolean
		//		indicates if filter metadata should be returned with columns		
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchColumns: function(/*fetchColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch columns from rest service
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);		
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		if (kwArgs.source) {
			content.source = kwArgs.source;
		}
		if (kwArgs.type) {
			content.type = kwArgs.type;
		}
		if (kwArgs.properties) {
			content.properties = kwArgs.properties.join(",");
		}		

		return dojo.xhrGet({
			url: this._getColumnsURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "columnList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var cols = [];
				var colsLength = result.items.length;
				for (var index = 0;  index < colsLength;  ++index) {
					cols[index] = new ibm.tivoli.navmodel.Column({column: result.items[index], objectType: kwArgs.type, dataset: this});
				}
				
				kwArgs.onComplete(cols, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchColumns"), ioArgs);
			})
		});
	},

	/*=====
	setColumns_kwArgs: function(onComplete, onError, columns, type, sync) {
		// onComplete: cols_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// columns: ibm.tivoli.navmodel.Column[]
		//		array of columns to be saved
		// type: String
		//		object type: "item", "link", "relationship" 
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_setColumns: function(/*setColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Save columns to rest service
		
		// pull out raw javascript column data
		var columns = [];
		if (kwArgs.columns) {
			var count = kwArgs.columns.length;
			for (var index = 0;  index < count;  ++index) {
				var col = kwArgs.columns[index].getAttributes();
				delete col.filter;
				columns.push(col);
			}
		}		
		var json = dojo.toJson({columns: columns}, false);
		
		// need original "param_" values to locate cached dataset
		return dojo.xhrPost({
			url: this._getColumnsParamURL(kwArgs),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion()),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "columnList"),
				'Content-Length': json.length,
				'X-Method-Override': 'Multi-PUT'
			},
			handleAs: "json",
			postData: json,
			sync: kwArgs.sync,
			load: dojo.hitch(this, function(result, ioArgs){
				kwArgs.onComplete(kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_setColumns"), ioArgs);
			})
		});
	},

	/*=====
	item_onComplete: function(item, kwArgs){},
	=====*/
	
	/*=====
	createItem_kwArgs: function(onComplete, onError, item) {
		// onComplete: item_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// item: Object
		//		ibm.tivoli.navmodel.Item to be created on server
	},
	=====*/
	
	_createItem: function(/*createItem_kwArgs*/ kwArgs) {
		// summary:
		//			Asynchronously create a new item in the dataset	
		var item = kwArgs.item;
		var json = dojo.toJson(item.getAttributes(), false);
		
		return dojo.xhrPost({
			url: this._getItemParamURL(item.getId()),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemView"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemView"),
				'Content-Length': json.length
			},
			handleAs: "json",
			postData: json,
			sync: false,
			load: dojo.hitch(this, function(result, ioArgs){
				var newItem = new ibm.tivoli.navmodel.Item({item: result, dataset: this, allProperties: true, allMetadata: false});
				kwArgs.onComplete(newItem, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_createItem"), ioArgs);
			})
		});
	},

	/*=====
	createItems_kwArgs: function(onComplete, onError, item) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// item: Object[]
		//		array of ibm.tivoli.navmodel.Item to be created on server
	},
	=====*/
	
	_createItems: function(/*createItems_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new items in the dataset
		var itemsObj = {"items": []};
		if (kwArgs.items) {
			var itemsLength = kwArgs.items.length;
			for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
				itemsObj.items[itemIndex] = kwArgs.items[itemIndex].getAttributes();
			}
		}
	
		var json = dojo.toJson(itemsObj, false);
		
		return dojo.xhrPost({
			url: this._getItemsParamURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemViewList"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemViewList"),
				'Content-Length': json.length,
				'X-Method-Override': 'Multi-PUT'
			},
			handleAs: "json",
			postData: json,
			sync: false,
			load: dojo.hitch(this, function(result, ioArgs){
				var newItems = [];				
				if (result.items) {
					var itemsLength = result.items.length;
					for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
						newItems[itemIndex] = new ibm.tivoli.navmodel.Item({item: result.items[itemIndex], dataset: this, allProperties: true, allMetadata: false});
					}
				}
				kwArgs.onComplete(newItems, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_createItems"), ioArgs);
			})
		});
	},

	/*=====
	_onComplete: function(kwArgs){},
	=====*/
	
	/*=====
	deleteItem_kwArgs: function(onComplete, onError, itemId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemId: String
		//		id of item to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of item
	},
	=====*/
	
	_deleteItem: function(/*deleteItem_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a item from the dataset
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		if (kwArgs.remove) {
			content.remove = kwArgs.remove;
		}
		
		return dojo.xhrDelete({
			url: this._getItemURL(kwArgs.itemId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion())
			},
			handleAs: "json",
			sync: false,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				kwArgs.onComplete(kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_deleteItem"), ioArgs);
			})
		});
	},
	
	/*=====
	deleteItems_kwArgs: function(onComplete, onError, itemIds, condition, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemIds: String[]
		//		array of item ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of items
	},
	=====*/
	
	_deleteItems: function(/*deleteItems_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete items from the dataset
		
		// build up the delete filter
		var deleteFilter = {};
		if (kwArgs.itemIds && kwArgs.itemIds.length > 0) {
			deleteFilter.identifiers = kwArgs.itemIds;
		}
		if (kwArgs.condition) {
			deleteFilter.condition = kwArgs.condition;
		}
		if (kwArgs.remove) {
			deleteFilter.remove = kwArgs.remove;
		}
		
		var json = dojo.toJson(deleteFilter, false);
				
		return dojo.xhrPost({
			url: this._getItemsParamURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion()),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "deleteFilter"),
				'Content-Length': json.length,
				'X-Method-Override': 'Multi-DELETE'
			},
			handleAs: "json",
			postData: json,
			sync: false,			
			load: dojo.hitch(this, function(result, ioArgs){
				kwArgs.onComplete(kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_deleteItems"), ioArgs);
			})
		});
	},

	/*=====
	link_onComplete: function(link, kwArgs){},
	=====*/
	
	/*=====
	createLink_kwArgs: function(onComplete, onError, link) {
		// onComplete: link_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// link: Object
		//		ibm.tivoli.navmodel.Link to be created on server
	},
	=====*/
	
	_createLink: function(/*createLink_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new link in the dataset
		var link = kwArgs.link;
		var json = dojo.toJson(link.getAttributes(), false);
		
		return dojo.xhrPost({
			url: this._getLinkParamURL(link.getId()),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkView"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkView"),
				'Content-Length': json.length
			},
			handleAs: "json",
			postData: json,
			sync: false,
			load: dojo.hitch(this, function(result, ioArgs){
				var newLink = new ibm.tivoli.navmodel.Link({link: result, dataset: this, allProperties: true, allMetadata: false});
				kwArgs.onComplete(newLink, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_createLink"), ioArgs);
			})
		});
	},
	
	/*=====
	createLinks_kwArgs: function(onComplete, onError, links) {
		// onComplete: links_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// links: Object[]
		//		array of ibm.tivoli.navmodel.Link to be created on server
	},
	=====*/
	
	_createLinks: function(/*createLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new links in the dataset
		var linksObj = {"links": []};
		if (kwArgs.links) {
			var linksLength = kwArgs.links.length;
			for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
				linksObj.links[linkIndex] = kwArgs.links[linkIndex].getAttributes();
			}
		}
	
		var json = dojo.toJson(linksObj, false);
		
		return dojo.xhrPost({
			url: this._getLinksParamURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkViewList"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkViewList"),
				'Content-Length': json.length,
				'X-Method-Override': 'Multi-PUT'
			},
			handleAs: "json",
			postData: json,
			sync: false,
			load: dojo.hitch(this, function(result, ioArgs){
				var newLinks = [];				
				if (result.items) {
					var linksLength = result.items.length;
					for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
						newLinks[linkIndex] = new ibm.tivoli.navmodel.Link({link: result.items[linkIndex], dataset: this, allProperties: true, allMetadata: false});
					}
				}
				kwArgs.onComplete(newLinks, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_createLinks"), ioArgs);
			})
		});
	},

	/*=====
	deleteLink_kwArgs: function(onComplete, onError, linkId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// linkId: String
		//		id of link to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of link
	},
	=====*/
	
	_deleteLink: function(/*deleteLink_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a link from the dataset
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		if (kwArgs.remove) {
			content.remove = kwArgs.remove;
		}
		
		return dojo.xhrDelete({
			url: this._getLinkURL(kwArgs.linkId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion())
			},
			handleAs: "json",
			sync: false,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				kwArgs.onComplete(kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_deleteLink"), ioArgs);
			})
		});
	},

	/*=====
	deleteLinks_kwArgs: function(onComplete, onError, linkIds, condition, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// linkIds: String[]
		//		array of link ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of links
	},
	=====*/
	
	_deleteLinks: function(/*deleteLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete links from the dataset
		
		// build up the delete filter
		var deleteFilter = {};
		if (kwArgs.linkIds && kwArgs.linkIds.length > 0) {
			deleteFilter.identifiers = kwArgs.linkIds;
		}
		if (kwArgs.condition) {
			deleteFilter.condition = kwArgs.condition;
		}
		if (kwArgs.remove) {
			deleteFilter.remove = kwArgs.remove;
		}
		
		var json = dojo.toJson(deleteFilter, false);
				
		return dojo.xhrPost({
			url: this._getLinksParamURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion()),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "deleteFilter"),
				'Content-Length': json.length,
				'X-Method-Override': 'Multi-DELETE'
			},
			handleAs: "json",
			postData: json,
			sync: false,
			load: dojo.hitch(this, function(result, ioArgs){
				kwArgs.onComplete(kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_deleteLinks"), ioArgs);
			})
		});
	},
	
	/*=====
	relationship_onComplete: function(relationship, kwArgs){},
	=====*/
	
	/*=====
	createRelationship_kwArgs: function(onComplete, onError, relationship) {
		// onComplete: relationship_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationship: Object
		//		ibm.tivoli.navmodel.Relationship to be created on server
	},
	=====*/
	_createRelationship: function(/*createRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new relationship in the dataset
		var relationship = kwArgs.relationship;
		var json = dojo.toJson(relationship.getAttributes(), false);
		
		return dojo.xhrPost({
			url: this._getRelationshipParamURL(relationship.getId()),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relationshipView"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relationshipView"),
				'Content-Length': json.length
			},
			handleAs: "json",
			postData: json,
			sync: false,
			load: dojo.hitch(this, function(result, ioArgs){
				var newRelationship = new ibm.tivoli.navmodel.Relationship({relationship: result, dataset: this, allProperties: true, allMetadata: false});				
				kwArgs.onComplete(newRelationship, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_createRelationship"), ioArgs);
			})
		});
	},
	
	/*=====
	createRelationships_kwArgs: function(onComplete, onError, relationships) {
		// onComplete: relationships_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationship: Object[]
		//		array of ibm.tivoli.navmodel.Relationship to be created on server
	},
	=====*/
	
	_createRelationships: function(/*createRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new relationships in the dataset
		var relationshipsObj = {"relationships": []};
		if (kwArgs.relationships) {
			var relLength = kwArgs.relationships.length;
			for (var relIndex = 0;  relIndex < relLength;  ++relIndex) {
				relationshipsObj.relationships[relIndex] = kwArgs.relationships[relIndex].getAttributes();
			}
		}
	
		var json = dojo.toJson(relationshipsObj, false);
		
		return dojo.xhrPost({
			url: this._getRelationshipsParamURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relationshipViewList"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relationshipViewList"),
				'Content-Length': json.length,
				'X-Method-Override': 'Multi-PUT'
			},
			handleAs: "json",
			postData: json,
			sync: false,
			load: dojo.hitch(this, function(result, ioArgs){
				var newRelationships = [];				
				if (result.items) {
					var relLength = result.items.length;
					for (var relIndex = 0;  relIndex < relLength;  ++relIndex) {
						newRelationships[relIndex] = new ibm.tivoli.navmodel.Relationship({relationship: result.items[relIndex], dataset: this, allProperties: true, allMetadata: false});
					}
				}
				kwArgs.onComplete(newRelationships, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_createRelationships"), ioArgs);
			})
		});
	},

	/*=====
	deleteRelationship_kwArgs: function(onComplete, onError, relationshipId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationshipId: String
		//		id of relationship to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of relationship
	},
	=====*/
	
	_deleteRelationship: function(/*deleteRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a relationship from the dataset
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		if (kwArgs.remove) {
			content.remove = kwArgs.remove;
		}
				
		return dojo.xhrDelete({
			url: this._getRelationshipURL(kwArgs.relationshipId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion())
			},
			handleAs: "json",
			sync: false,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				kwArgs.onComplete(kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_deleteRelationship"), ioArgs);
			})
		});
	},

	/*=====
	deleteRelationships_kwArgs: function(onComplete, onError, relationshipIds, condition, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationshipIds: String[]
		//		array of relationship ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of relationships
	},
	=====*/
	
	_deleteRelationships: function(/*deleteRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete relationships from the dataset
		
		// build up the delete filter
		var deleteFilter = {};
		if (kwArgs.relationshipIds && kwArgs.relationshipIds.length > 0) {
			deleteFilter.identifiers = kwArgs.relationshipIds;
		}
		if (kwArgs.condition) {
			deleteFilter.condition = kwArgs.condition;
		}
		if (kwArgs.remove) {
			deleteFilter.remove = kwArgs.remove;
		}
		
		var json = dojo.toJson(deleteFilter, false);
			
		return dojo.xhrPost({
			url: this._getRelationshipsParamURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion()),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "deleteFilter"),
				'Content-Length': json.length,
				'X-Method-Override': 'Multi-DELETE'
			},
			handleAs: "json",
			postData: json,
			sync: false,			
			load: dojo.hitch(this, function(result, ioArgs){
				kwArgs.onComplete(kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_deleteRelationships"), ioArgs);
			})
		});
	},	
	
	/////////////////////////////
	// private methods
	/////////////////////////////
	
	_extractContentTypeVersion: function(ioArgs) {
		var version = this.getVersion();
		var contentType = ioArgs.xhr.getResponseHeader("Content-Type");
		if (contentType) {
			var parms = contentType.split(";");
			var length = parms.length;
			for (var index = 0;  index < length;  ++index) {
				if (parms[index].indexOf("version=") === 0) {								
					version = parms[index].substring(8)*1;
				}
			}
		}
		
		return version;		
	},
									
	_mixinStandard: function(content, kwArgs) {
		// summary:
		//		Add common arguments if provided in kwArgs
		if (kwArgs.clearCache) {
			content.clearCache = kwArgs.clearCache;
		}
		
		if (this._optimize) {
			content.optimize = this._optimize;
		}
		
		if (kwArgs.start && kwArgs.start > 0) {
			content.start = kwArgs.start;
		}
		
		if (kwArgs.count && kwArgs.count > 0) {
			content.count = kwArgs.count;
		}
		
		if (kwArgs.metadata) {
			content.metadata = true;
		}
		
		if (kwArgs.sort) {
			content.sort = kwArgs.sort; 
		}
		
		if (kwArgs.condition) {
			content.condition = kwArgs.condition;
		}
	},
	
	_getDatasetURL: function() {
		// summary:
		//		Returns url for for fetching dataset
		
		return this.getArg("baseUrl") + this.getAttribute("uri");
	},
	
	_getOriginItemsURL: function() {
		// summary:
		//		Returns url for fetching origin items
		return  this._getDatasetURL() + "/originItems";
	},

	_getItemsURL: function() {
		// summary:
		//		Returns url for fetching items for a dataset 
		return this._getDatasetURL() + "/items";		
	},

	_getItemsParamURL: function() {
		// summary:
		//		Returns url for fetching items and also includes
		//		all of the "param_" and "version" arguments
		return this._appendParams(this._getItemsURL());	
	},

	_getItemsQueryURL: function(kwArgs) {
		// summary:
		//		Returns url for custom query of items
		var url = this._getItemsParamURL();
		var questionMark = (url.indexOf('?') > 0);
		
		if (this._optimize) {
			if (!questionMark) {
				questionMark = true;
				url += "?optimize=" + this._optimize;
			}
			else {
				url += "&optimize=" + this._optimize;
			}			
		}
		
		if (kwArgs.start && kwArgs.start > 0) {
			if (!questionMark) {
				questionMark = true;
				url += "?start=" + kwArgs.start;
			}
			else {
				url += "&start=" + kwArgs.start;
			}
		}
		
		if (kwArgs.count && kwArgs.count > 0) {
			if (!questionMark) {
				questionMark = true;
				url += "?count=" + kwArgs.count;
			}
			else {
				url += "&count=" + kwArgs.count;
			}
		}
		
		if (kwArgs.properties && kwArgs.properties.length > 0) {
			if (!questionMark) {
				questionMark = true;
				url += "?properties=" + kwArgs.properties.join(",");
			}
			else {
				url += "&properties=" + kwArgs.properties.join(",");
			}
			
			if (kwArgs.metadata) {
				url += "&metadata=true";
			}
		}
				
		return url;
	},

	_getItemURL: function(itemId) {
		// summary:
		//		Returns url for fetching a specific item
		return this._getItemsURL() + "/" + encodeURIComponent(itemId);		
	},
	
	_getItemParamURL: function(itemId) {
		// summary:
		//		Returns url for item and also includes
		//		all of the "param_" and "version" arguments
		return this._appendParams(this._getItemURL(itemId));				
	},

	_getRelatedItemsURL: function(itemId) {
		// summary:
		//		Returns url for fetching related items
		return this._getItemURL(itemId) + "/relatedItems";	
	},

	_getLinksURL: function() {
		// summary:
		//		Returns url for fetching links for a dataset
		return this._getDatasetURL() + "/links";
	},

	_getLinksParamURL: function() {
		// summary:
		//		Returns url for fetching links and also includes
		//		all of the "param_" and "version" arguments
		return this._appendParams(this._getLinksURL()); 		
	},

	_getLinkURL: function(linkId) {
		// summary:
		//		Returns url for fetching a specific link
		return this._getLinksURL() + "/" + encodeURIComponent(linkId);		
	},

	_getLinkParamURL: function(linkId) {
		// summary:
		//		Returns url for link and also includes
		//		all of the "param_" and "version" arguments
		return this._appendParams(this._getLinkURL(linkId));				
	},

	_getRelationshipsURL: function() {
		// summary:
		//		Returns url for fetching relationships for a dataset
		return this._getDatasetURL() + "/relationships";		
	},

	_getRelationshipsParamURL: function() {
		// summary:
		//		Returns url for fetching relationships and also includes
		//		all of the "param_" and "version" arguments
		return this._appendParams(this._getRelationshipsURL()); 		
	},

	_getRelationshipURL: function(relationshipId) {
		// summary:
		//		Returns url for fetching a specific relationship
		return this._getRelationshipsURL() + "/" + encodeURIComponent(relationshipId);		
	},
	
	_getRelationshipParamURL: function(relationshipId) {
		// summary:
		//		Returns url for relationship and also includes
		//		all of the "param_" and "version" arguments
		return this._appendParams(this._getRelationshipURL(relationshipId));				
	},
	
	_getTasksURL: function() {
		// summary:
		//		Returns url for fetching tasks for a dataset
		return this._getDatasetURL() + "/tasks";		
	},

	_getTasksParamURL: function(kwArgs) {
		// summary:
		//		Returns url for fetching tasks for a dataset and also includes
		//		all of the "param_" and "version" arguments
		var url = this._appendParams(this._getTasksURL());		
		var questionMark = (url.indexOf('?') > 0);
		
		// optimize
		if (this._optimize) {
			if (!questionMark) {
				questionMark = true;
				url += "?optimize=" + this._optimize;
			}
			else {
				url += "&optimize=" + this._optimize;
			}			
		}
		
		// task category
		if (kwArgs.taskCategory) {
			if (!questionMark) {
				questionMark = true;
				url += "?category=" + kwArgs.taskCategory;
			}
			else {
				url += "&category=" + kwArgs.taskCategory;
			}			
		}
		
		return url;		
	},

	_getTaskURL: function(taskId) {
		// summary:
		//		Returns url for fetching a specific task
		return this._getTasksURL() + "/" + encodeURIComponent(taskId);		
	},

	_getNavigateParamURL: function(kwArgs) {
		// summary:
		//		Returns url for navigating a specific task and also includes
		//		all of the "param_" and "version" arguments
		var url = this._appendParams(this._getTaskURL(kwArgs.taskId));
		var questionMark = (url.indexOf('?') > 0);
		
		// optimize
		if (this._optimize) {
			if (!questionMark) {
				questionMark = true;
				url += "?optimize=" + this._optimize;
			}
			else {
				url += "&optimize=" + this._optimize;
			}			
		}
		
		// task category; should indicate navigation
		if (kwArgs.taskCategory) {
			if (!questionMark) {
				questionMark = true;
				url += "?category=" + kwArgs.taskCategory;
			}
			else {
				url += "&category=" + kwArgs.taskCategory;
			}			
		}
			
		return url;
	},
	
	_getItemStylesURL: function() {
		// summary:
		//		Returns url for fetching current item styles for a dataset
		return this._getDatasetURL() + "/itemStyles";	
	},

	_getLinkStylesURL: function() {
		// summary:
		//		Returns url for fetching current link styles for a dataset
		return this._getDatasetURL() + "/linkStyles";
	},

	_getRelationshipStylesURL: function() {
		// summary:
		//		Returns url for fetching current relationship styles for a dataset
		return this._getDatasetURL() + "/relationshipStyles";
	},

	_getStatusURL: function() {
		// summary:
		//		Returns url for fetching possible status values
		return this._getDatasetURL() + "/status";
	},

	_getColumnsURL: function() {
		// summary:
		//		Returns url for fetching current columns for a dataset
		return this._getDatasetURL() + "/columns";
	},
	
	_getColumnsParamURL: function(kwArgs) {
		// summary:
		//		Returns url for saving current tasks and also includes
		//		all of the "param_" and "version" arguments
		var url = this._appendParams(this._getColumnsURL());
		var questionMark = (url.indexOf('?') > 0);
		
		// type
		if (kwArgs.type) {
			if (!questionMark) {
				questionMark = true;
				url += "?type=" + kwArgs.type;
			}
			else {
				url += "&type=" + kwArgs.type;
			}			
		}
			
		return url;
	},

	_appendParams: function(url) {
		// summary:
		//		Append "param_" and "version" request parameters to the url
		//		so that dataset can be obtained from the REST service cache
		var questionMark = false;
		
		// add "param_" parms
		var parameters = this.getArg("parameters");
		for (var parm in parameters) {
			if (parameters.hasOwnProperty(parm)) {
				if (!questionMark) {
					questionMark = true;
					url += "?";
				}
				else {
					url += "&";
				}
				url += ("param_" + parm + "=" + parameters[parm]);				
			}
		}
		
		// IE will sometimes return http requests from cache without unique parms
		var now = new Date();
		if (!questionMark) {
			questionMark = true;
			url += "?d=" + now.getTime() + "&r=" + Math.random();
		}
		else {
			url += "&d=" + now.getTime() + "&r=" + Math.random();
		}	
		
		return url;
	},
	
	_getException: function(error, ioArgs, functionName) {
		return ibm.tivoli.navmodel.Utils.getException(error, ioArgs, functionName, this._trace, this.getVersion());
	},
	
	_isAllProps: function(properties) {
		return (properties && properties.length === 1 && properties[0] === "all");
	},
	
	_getPropIds: function(properties, allProps) {
		if (properties && properties.length > 0 && !allProps) {
			return properties;
		}
		
		return null;
	}
});
