/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */
 
dojo.provide("ibm.tivoli.navmodel.DatasetLazyTreeStore");

// includes
dojo.require("ibm.tivoli.navmodel.DatasetStore");
dojo.require("ibm.tivoli.navmodel.Dataset");
	
///////////////////////////////////////////
// This dojo datastore is compatible with:
// 		dojox.grid.LazyTreeGrid	
//
// Do NOT use for:
//		dijit.Tree
//		dojox.grid.TreeGrid
// instead use DatasetTreeStore
///////////////////////////////////////////	
ibm.tivoli.navmodel.DatasetLazyTreeStore._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.DatasetLazyTreeStore");
	
dojo.declare("ibm.tivoli.navmodel.DatasetLazyTreeStore", ibm.tivoli.navmodel.DatasetStore, {
	// private members 
	_clearCache: false,
	_origins: null,
	_children: null,
	_sort: null,
	_condition: null,
	_sync: false,
	_trace: ibm.tivoli.navmodel.DatasetLazyTreeStore._trace,
	
	/*=====
	constructor_kwArgs: function(dataset, defaultProperties, relationships) {
		// summary:
		//		An anonymous object used to configure the datastore
		// dataset: Object
		//		A ibm.tivoli.navmodel.Dataset instance
		// defaultProperties: String[]
		//		An array of property id values that are pre-fetched with the items
		// relationships: String  (optional)
		//		Comma delimited list of relationships to determine related items; 
		//		can also be keywords: child, parent, all.  If not specified the value is
		//		retrieved from the individual item that is being expanded			
	},
	=====*/
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructor for dojo datastore;
		//		that receives item data (in tree format) from an ibm.tivoli.navmodel.Dataset
		//		Base class will mixin kwArgs
		
		// LazyTreeGrid seems to expects us to behave in a synchronous manner
		// when dynamic refresh is being processed
		this._sync = this.dataset.isEventsEnabled();
		
		this._clearOrigins();
		this._clearChildren();		
	},		
	
	//////////////////////////////////////////////////////////////
	// dojo.data.api.Identity
	//////////////////////////////////////////////////////////////
	
    getIdentity: function(/*Object*/ item){
		// summary:
    	//		Returns a unique identifier for an item. The return value will be
    	//		either a string or something that has a toString() method.
    	// item:
    	//		The item from the store from which to obtain its identifier.
		// exceptions:
    	//		Conforming implementations may throw an exception or return null if
    	//		item is not an item.
		if (!this.isItem(item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
		return item.getId();		
    },
    
	/*=====
	_onItem: function(item) {},
	=====*/
	
	/*=====
	_onError: function(error) {},
	=====*/
	
	/*=====
	fetchItemByIdentity_kwArgs: function(identity, onItem, onError, scope) {
		// identity: String|Object
		//		The identity parameter is the identity of the item you wish to locate and load
		//		This attribute is required. It should be a string or an object that toString()
		//		can be called on.
		// onItem: _onItem
		//		The onItem parameter is the callback to invoke when the item has been loaded. It takes only one
		//		parameter, the item located, or null if none found.
		// onError: _onError
		//		The onError parameter is the callback to invoke when the item load encountered an error. It takes 
		//		only one parameter, the error object
		// scope: Object
		//		If a scope object is provided, all of the callback functions (onItem,
		//		onError, etc) will be invoked in the context of the scope object.
	},
	=====*/
	
 	fetchItemByIdentity: function(/*fetchItemByIdentity_kwArgs*/ kwArgs) {
		// summary:
		//		Given the identity of an item, this method returns the item that has
		//		that identity through the onItem callback. Conforming implementations
		//		should return null if there is no item with the given identity.
		//		Implementations of fetchItemByIdentity() may sometimes return an item
		//		from a local cache and may sometimes fetch an item from a remote server
	 	var args = {
 			identity: null,
 			onItem: function(item) {},
 			onError: function(error) {},
 			scope: null
 		};
 		dojo.mixin(args, kwArgs);
 		
 		var item = null;
 		var error = null;
 		if (args.identity) {
 			try { 			
  				item = this.dataset.getItem({itemId: args.identity, properties: this.defaultProperties});
  			}
  			catch(err) {
  				error = err;
  			}  			
  		}  		
  		
 		if (item) {
 			args.onItem.call(args.scope, item);
 		}
 		else {
 			args.onError.call(args.scope, error ? error : "Could not fetch item for identity=" + args.identity);
 		}		
	},
	
	
	//////////////////////////////////////////////////////////////
	// dojo.data.api.Read
	//////////////////////////////////////////////////////////////
	
	/*=====
	fetch_onBegin: function(size, request){},
	=====*/
	
	/*=====
	fetch_onItem: function(item, request){},
	=====*/
	
	/*=====
	fetch_onComplete: function(items, request){},
	=====*/
	
	/*=====
	fetch_onError: function(errorData, request){},
	=====*/
	
	/*=====
	fetch_kwArgs: function(query, queryOptions, onBegin, onItem, onComplete, onError, scope, start, count, sort) {
		// summary:
		//		The kwArgs parameter may either be an instance of conforming to 
		//		dojo.data.api.Request or may be a simple anonymous object
		//		containing any of the above parameters.
		// query: String|Object
		//		The query parameter is used to specify search and filter condition information for the fetch.
		//		Only those items that match the search and/or filter will be returned.
		//		|	{	ids: {
		//		|			propId: boolean		// indicates propIds involved in the expression
		//		|		},
		//		|		expression: string,		// expression involiving propId's and values  (and/or together)
		//		|								// example:  ((Name contains 'abc' || Type contains 'abc)) and (Size > 100)
		//		|
		//		|		// the following is for compatability with legacy datastore
		//		| 		navModel: {
		//		|			search: {
		//		|				term: string,		the search text
		//		|				attributes: array	["property id1", "property id2", ...]
		//		|			}
		//		|		}
		//		|	}
		// queryOptions: Object?
		//		The queryOptions parameter is an optional parameter that can be used to modify the query.  The
		//		ignoreCase case can be specified to globally make all of the filter conditions case sensitive/insensitive.
		//		|	{ ignoreCase: boolean
		//		|	}
		// onBegin: fetch_onBegin
		//		If an onBegin callback function is provided, the callback function will be called just once, 
		//		before the first onItem callback is called. The onBegin callback function will be passed two arguments, 
		//		the the total number of items identified and the Request object.  If the total number is
		//		unknown, then size will be -1.  Note that size is not necessarily the size of the collection of items 
		//		returned from the query, as the request may have specified to return only a subset of the total set 
		//		of items through the use of the start and count parameters.
		// onItem: fetch_onItem
		//		If an onItem callback function is provided, the callback function will be called as each item 
		//		in the result is received. The callback function will be passed two arguments: the item itself, 
		//		and the Request object.
		// onComplete: fetch_onComplete
		//		If an onComplete callback function is provided, the callback function will be called just once,
		//		after the last onItem callback is called. Note that if the onItem callback is not present,
		//		then onComplete will be passed an array containing all items which matched the query and the 
		//		request object. If the onItem callback is present, then onComplete is called as: onComplete(null, request).
		// onError: fetch_onError
		//		If an onError callback function is provided, the callback function will be called if there is any 
		//		sort of error while attempting to execute the query. The onError callback function will be passed two arguments:
		//		an Error object and the Request object.
		// scope: Object
		//		If a scope object is provided, all of the callback functions (onItem, onComplete, onError, etc) 
		//		will be invoked in the context of the scope object.
		// start: int
		//		If a start parameter is specified, this is a indication to the datastore to
		//		only start returning items once the start number of items have been located and
		//		skipped.  When this parameter is paired with 'count', the store should be able
		//		to page across queries with millions of hits by only returning subsets of the
		//		hits for each query
		// count: int
		//		 If a count parameter is specified, this is a indication to the datastore to
		//		 only return up to that many items.  This allows a fetch call that may have
		//		millions of item matches to be paired down to something reasonable.
		// sort: Object[]
		//		If a sort parameter is specified, this is a indication to the datastore to
		//		sort the items in some manner before returning the items.  The array is an array of
		//		javascript objects that must conform to the following format to be applied to the
		//		fetching of items:
		//		|	{
		//		|		attribute: attribute || attribute-name-string,
		//		|		descending: true|false;   // Optional.  Default is false.
		//		|	}
	},
	=====*/
		
	fetch: function(/*fetch_kwArgs*/ kwArgs) {
		// summary:
		//		This datastore exposes the tree origin nodes of the Dataset and allows for
		//		lazy loading of children afterwards.
		//		The format and expectations of stores is that they operate in a generally asynchronous
		//		manner, therefore callbacks are always used to return items located by the fetch parameters.
		// description:
		//		A Request object will always be returned and is returned immediately.
		//		The basic request is nothing more than the keyword args passed to fetch and
		//		an additional function attached, abort().  The returned request object may then be used
		//		to cancel a fetch.  All data items returns are passed through the callbacks defined in the
		//		fetch parameters and are not present on the 'request' object.
		// returns:
		//		The fetch() method will return a javascript object conforming to the API
		//		defined in dojo.data.api.Request.  In general, it will be the keywordArgs
		//		object returned with the required functions in Request.js attached.
		//		Its general purpose is to provide a convenient way for a caller to abort an
		//		ongoing fetch.
		// exceptions:
		//		Throws an exception if the query is not valid, or if the query is required but was not supplied.
		var args = {
			onBegin: function(size, request) {},
			onItem: function(item, request) {},
			onComplete: function(items, request){},
			onError: function(errorData, request){},
			scope: null,
			abort: function() {}
		};
		dojo.mixin(args, kwArgs);
		
		// make sure event handling is set up
		this._validateEventHandler();
		
		// TED refresh is occurring
		this._clearCache = false;
		if (this.isRefresh()) {
			this._clearOrigins();
			this._clearChildren();
			this._clearCache = this.dataset.isClearCacheOnRefresh();						
		}
		
		// extract sort (see if it has changed from last time)
		var sort = this._getSort(kwArgs.sort);
		if ((sort && !this._sort) || (!sort && this._sort) || (sort !== this._sort)) {
			this._clearOrigins();
			this._clearChildren();			
		}
		this._sort = sort;			
						
		// extract condition (see if it has changed from last time)
		var condition = this._getCondition(args.query);	
		if ((condition && !this._condition) || (!condition && this._condition) || (condition !== this._condition)) {
			this._clearOrigins();
			this._clearChildren();			
		}
		this._condition = condition;
		
		// query was supplied
		if (args.query) {
			// fetch flat filtered list with no hierarchy
			if (this._condition) {
				this._fetchItems(args);
			}
			// fetch children for "parentId"
			else if (args.query.parentId) {
				if (this._trace.isFinest()) {
					this._trace.finest("fetching children for parent=" + args.query.parentId + " start=" + kwArgs.start + " count=" + kwArgs.count, "fetch");
				}
				var parentItem = this._getItem(args.query.parentId);
				this._fetchChildren(args, parentItem);
			}
			// fetch origin items
			else if (args.query.origins) {
				if (this._trace.isFinest()) {
					this._trace.finest("fetching origins start=" + kwArgs.start + " count=" + kwArgs.count, "fetch");
				}				
				this._fetchOrigins(args);
			}
			// fetch all items
			else {
				this._fetchItems(args);
			}
		}		
		// query was not supplied; fetch all items
		else {
			this._fetchItems(args);
		}
		       
        return args;
    },
    
    getAttributes: function(/*Object*/ item) {
		// summary:
		//		Returns an array with all the attributes that this item has.  This
		//		method will always return an array; if the item has no attributes
		//		at all, getAttributes() will return an empty array: [].
		// item:
		//		The item to access attributes on.
		//		ibm.tivoli.navmodel.Item object
		// exceptions:
		//		Throws an exception if *item* is not an item
		var attributes = this.inherited(arguments);
  		
		// add "hasChildren" attribute
		if (this._hasChildren(item)) {
			attributes.push("hasChildren");			
		}  		
		
		return attributes;		
    },
    
    getValue: function(/*Object*/ item, /*String*/ attribute, /*Object?*/ defaultValue) {
		// summary:
		//		Returns a single attribute value. Returns defaultValue if and only if item does not have a 
		//		value for attribute. Returns null if and only if null was explicitly set as the attribute value. 
		//		Returns undefined if and only if the item does not have a value for the given attribute 
		//		(which is the same as saying the item does not have the attribute).
		// item:
		//		The item to access value on.
		//		ibm.tivoli.navmodel.Item object
		// attribute:
		//		The attribute to access represented as a string.
		// defaultValue:
		//		Optional. A default value to use for the getValue return if the attribute does not exist or 
		//		has no value.
		// exceptions:
		//		Throws an exception if *item* is not an item, or *attribute* is not a string
    	if (arguments.length < 2) {
            throw new Error("Incorrect number of arguments specified: expecting 2 (item, attribute)");
        }
        
        if (!this.isItem(item)) {
            throw new Error("Specified item argument is not a valid item.");
        }
        
        if (!dojo.isString(attribute)) {
            throw new Error("Specified attribute argument is not a string.");
        }  
               
        var value = defaultValue;
        
        // asking if item has children
        if (attribute === "hasChildren") {
        	value = this._hasChildren(item);
        }
        // asking for entire item
        else if (this.returnItem) {
        	value = item;
        }
		// asking for item property
        else {
        	var prop = item.getProperty(attribute);
        	if (prop) {
        		value = prop.getDisplayValue();            
        	}
        }            
                           	
        return value;
    },
        
    
    /*=====
	loadItem_kwArgs: function(item, onItem, onError, scope) {
		// summary:
		//		An anonymous object that defines the item to load and callbacks to invoke when the
		//		load has completed.
		// item: Object
		//		The item parameter is an object that represents the item in question that should be
		//		contained by the store.  This attribute is required.
		//		ibm.tivoli.navmodel.Item object
		// onItem: _onItem
		//		The onItem parameter is the callback to invoke when the item has been loaded.	
		//		It takes only one parameter, the fully loaded item.
		// onError: _onError
		//		The onError parameter is the callback to invoke when the item load encountered an error.
		//		It takes only one parameter, the error object.
		// scope: Object
		//		If a scope object is provided, all of the callback functions (onItem, onError, etc)
		//		will be invoked in the context of the scope object.
		//		onItem.call(dojo.global(), item, request)
	},
	=====*/
	
	loadItem: function(/*loadItem_kwArgs*/ kwArgs) {
		var args = {
			item : null,
			onItem: function(item) {},
			onError: function(error){},
			scope: null
		};		
		dojo.advancedMixin(args, kwArgs);
	
		if (!this.isItem(args.item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
		if (this.isItemLoaded(args.item)){
			return args.item;
		}
		
		try {
			var dataset = args.item.getDataset();
			var item = dataset.getItem({itemId: args.item.getId(), properties: this.defaultProperties});			
			args.onItem.call(args.scope, item);
		}
		catch(err) {
			args.onError.call(args.scope, err);
		}
	},
	
	//////////////////////////////////////////////////////////////
	// private methods
	///////////////////////////////////////////////////////////////
	
	_getObjectTypes: function() {
		// summary:
		//		Returns String[] of object types that can in the datastore
    	return {
    		"item": true
		};
    },
    
    _fetchOrigins: function(/*fetch_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch origin items for the tree
		
		// origin items cached
		if (this._origins) {
			var length = this._origins.length;
			var start = kwArgs.start ? kwArgs.start : 0;
			var count = kwArgs.count ? kwArgs.count : length;
			if (start + count > length) {
				count = length - start;
			}
			
			if (this._trace.isFinest()) {
				this._trace.finest("datasetId=" + this.dataset.getId() + " returning cached origins; start=" + start + " count=" + count, "_fetchOrigins");
			}
				
			kwArgs.onBegin.call(kwArgs.scope, length, kwArgs);				
			for(var index = 0;  index < count;  ++index) {
				if (this._trace.isFinest()) {
					this._trace.finest("cached origins[" + (start+index) + "]=" + this._origins[start+index].getId(), "_fetchOrigins");
				}                 	                	
                kwArgs.onItem.call(kwArgs.scope, this._origins[start+index], kwArgs);                	
			}					
			kwArgs.onComplete.call(kwArgs.scope, this._origins.slice(start,start+count), kwArgs);			
		}
		// fetch origin items
		else {
			var originArgs = {
				clearCache: this._clearCache,
				start: kwArgs.start,
				count: kwArgs.count,
				sort: this._sort,
				properties: this.defaultProperties,
				sync: this._sync,
				onComplete: dojo.hitch(this, function(originItems, originArgs) {				
					// we have origin items		
					if (originItems && originItems.length > 0) {
						var originsLength = originItems.length;
						
						if (this._trace.isFinest()) {
							this._trace.finest("datasetId=" + this.dataset.getId() + " returning origins; start=" + kwArgs.start + " count=" + kwArgs.count, "_fetchOrigins");
						}
			
						kwArgs.onBegin.call(kwArgs.scope, originArgs.filteredRows, kwArgs);				
						for(var originIndex = 0;  originIndex < originsLength;  ++originIndex) {
							if (this._trace.isFinest()) {
								this._trace.finest("origins[" + (kwArgs.start+originIndex) + "]=" + originItems[originIndex].getId(), "_fetchOrigins");
							}                	                	
              				kwArgs.onItem.call(kwArgs.scope, originItems[originIndex], kwArgs);                	
						}					
						kwArgs.onComplete.call(kwArgs.scope, originItems, kwArgs);
						
						// we received all of the origins so cache them
						if (originsLength === originArgs.filteredRows) {
							this._origins = originItems;
						}				
					}
					// no origins declared
					else {
						this._fetchItems(kwArgs);
					}
				}),			
				onError: dojo.hitch(this, function(error, exception, ioArgs) {
					kwArgs.onError.call(kwArgs.scope, error, kwArgs);
				})						
			}; 		
			this.dataset.getOriginItems(originArgs);
		}	
    },    
    
    _hasChildren: function(/*Object*/ item) {
		// summary:
		//		Indicates if the item has children items
		
		// if we have a filter; then we are showing a flat list
		// so we would never want to indicate children present
		if (this._condition) {
			return false;
		}
		
		// lazyLoad indicates that there might be children;
		// lazyLoadRelationships indicates that there is children and this is the relationship to fetch them
    	return (item.isLazyLoad() || item.getLazyLoadRelationships()) ? true : false;
    },
    
    _fetchChildren: function(kwArgs, item) {
    	// children cached
    	var children = this._getChildren(item.getId());
    	if (children) {
    		var length = children.length;
			var start = kwArgs.start ? kwArgs.start : 0;
			var count = kwArgs.count ? kwArgs.count : length;
			if (start + count > length) {
				count = length - start;
			}
					
			if (this._trace.isFinest()) {
				this._trace.finest("datasetId=" + this.dataset.getId() + " returning cached children for item=" + item.getId() + " start=" + start + " count=" + count, "_fetchChildren");
			}
				    			
			kwArgs.onBegin.call(kwArgs.scope, length, kwArgs);				
			for(var index = 0;  index < count;  ++index) {
				if (this._trace.isFinest()) {
					this._trace.finest("cached children[" + (start+index) + "]=" + children[start+index].getId(), "_fetchChildren");
				}                 	                	
                kwArgs.onItem.call(kwArgs.scope, children[start+index], kwArgs);                	
			}					
			kwArgs.onComplete.call(kwArgs.scope, children.slice(start,start+count), kwArgs);	
    	}
    	// fetch children
    	else {
    		var relatedArgs = {
    			clearCache: this._clearCache,
    			start: kwArgs.start,
    			count: kwArgs.count,
    			relationships: this.relationships,
    			properties: this.defaultProperties,
    			sort: this._sort,
    			sync: this._sync,    		
    			onComplete: dojo.hitch(this, function(items, links, relatedArgs) {
    				if (this._trace.isFinest()) {
						this._trace.finest("datasetId=" + this.dataset.getId() + " returning children for item=" + item.getId() + " start=" + kwArgs.start + " count=" + kwArgs.count, "_fetchChildren");
					}			
					kwArgs.onBegin.call(kwArgs.scope, relatedArgs.filteredRows, kwArgs);
								
					if (items) {
						var itemsLength = items.length;
						for(var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {                 	                	
                			if (this._trace.isFinest()) {
								this._trace.finest("children[" + (kwArgs.start+itemIndex) + "]=" + items[itemIndex].getId(), "_fetchChildren");
							} 
							kwArgs.onItem.call(kwArgs.scope, items[itemIndex], kwArgs);                	
						}
						
						// we received all of the children so cache them
						if (itemsLength === relatedArgs.filteredRows) {
							this._setChildren(item.getId(), items);
						}
					}				
					
					kwArgs.onComplete.call(kwArgs.scope, items, kwArgs);
				}),			
				onError: dojo.hitch(this, function(error, exception, ioArgs) {
					kwArgs.onError.call(kwArgs.scope, error, kwArgs);				
				})
    		};
    		item.getRelatedItems(relatedArgs);
    	}
    },
    
    _getItem: function(/*String*/ identity) {
    	return this.dataset.getItem({itemId: identity, properties: this.defaultProperties}); 
    },
        
    _fetchItems: function(/*fetch_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch all items for the dataset if the query did not indicate that origins should be returned
		
    	var itemArgs = {
			clearCache: this._clearCache,		
			properties: this.defaultProperties,
			sort: this._sort,
			condition: this._condition,
			sync: this._sync,
			onComplete: dojo.hitch(this, function(items, itemArgs) {
				kwArgs.onBegin.call(kwArgs.scope, itemArgs.filteredRows, kwArgs);
								
				if (items) {
					var itemsLength = items.length;
					for(var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {                 	                	
                		kwArgs.onItem.call(kwArgs.scope, items[itemIndex], kwArgs);                	
					}
				}				
					
				kwArgs.onComplete.call(kwArgs.scope, items, kwArgs);
			}),			
			onError: dojo.hitch(this, function(error, exception, ioArgs) {
				kwArgs.onError.call(kwArgs.scope, error, kwArgs);
			})
		}; 		
		this.dataset.getItems(itemArgs);
    },
    
    _clearOrigins: function() {
    	// summary:
		//		Clear cached origins
    	this._origins = null;
    },
    
    _clearChildren: function() {
		// summary:
		//		Clear cached children
    	this._children = {};    	
    },
    
    _getChildren: function(/*String*/ id) {
		// summary:
		//		Get cached children for parent id
    	return this._children[id];
    },
    
	_setChildren: function(/*String*/ id, /*String*/ children) {
		// summary:
		//		Set cached children for parent id
    	this._children[id] = children;
    },
    
    _getCondition: function(/*Object*/ query) {
		// summary: 
		//		Returns array of condition objects involved in the current filter
		//			ibm.tivoli.navmodel.[String|Number|Date|Boolean|Enum|Map]Condition
		this._conditionIds = null;
		var condition = null;
 		 		
		if (query && query.filterText && query.filterText.length > 0) {
			var filterText = query.filterText.replace(/'/g, "''");
			var propIds = this.defaultProperties;
			if (propIds) {
				this._conditionIds = {};				
				condition = "(";
				var length = propIds.length;
				for (var index = 0;  index < length;  ++index) {
					if (index > 0) {
						condition += " or ";
					}					
					var id = propIds[index];
					condition += "(" + id + " contains '" + filterText + "')";
					this._conditionIds[id] = true;
				}
				condition += ")";
			}
		}
 		
		return condition;
 	},
 	 	
    //////////////////////////////////////////////////////////////
	// async event handlers
	///////////////////////////////////////////////////////////////
	
	_handleChangedEvent: function(/*Object*/ event) {
    	this._clearOrigins();
    	this._clearChildren();
    	this.onRefresh();
	},
	
	_handleItemPropertiesChangedEvent: function(/*Object*/ event) {
		var items = event.getItems();    	
    	var propsArray = event.getProperties();			
		if (items && propsArray) {
			var refresh = false;
    		var length = items.length;			
			for (var index = 0;  index < length;  ++index) {
				var properties = propsArray[index];				
				if (properties) {
					var propsLength = properties.length;
					for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
						var item = items[index];
						var property = properties[propIndex];						
						if (this._trace.isFinest()) {
							this._trace.finest("datasetId=" + event.getDataset().getId() + " itemId=" + item.getId() + " prop=" + property.getId() + " value=" + property.getDisplayValue(), "_handleItemPropertiesChangedEvent");
						}
						
						this.onSet(item, property.getId(), null, property.getDisplayValue());
						
						if (!refresh && (this._affectsSort(propId) || this._affectsCondition(propId))) {
							refresh = true;							
						}
					}										
				}				
			}
			
			if (refresh) {
				this.onRefresh();
			}		
		}	
	},
	
    _handleItemsAddedEvent: function(/*Object*/ event) {
    	var items = event.getItems();
		if (items) {
			var refresh = false;
    		var length = items.length;			
			for (var index = 0;  index < length;  ++index) {
				var item = items[index];				
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + event.getDataset().getId() + " itemId=" + item.getId() + " origin=" + item.isOrigin(), "_handleItemsAddedEvent");
				}
				
				this._clearChildren();
				
				if (this._hasCondition()) {
					refresh = true;
					this.onNew(item);					
				}
				else if (item.isOrigin()) {
					this._clearOrigins();
					this.onNew(item);					
				}
			}
			
			if (refresh) {
				this.onRefresh();
			}
		}    		
	},
	
	_handleItemsRemovedEvent: function(/*Object*/ event) {
		var items = event.getItems();
		if (items) {
			var length = items.length;			
			for (var index = 0;  index < length;  ++index) {
				var item = items[index];				
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + event.getDataset().getId() + " itemId=" + item.getId(), "_handleItemsRemovedEvent");
				}
				
				this._clearChildren();								
				this._clearOrigins();				
				this.onDelete(item);
			}
		}
	},
	
	_handleLinksAddedEvent:function(/*Object*/ event) {
		var links = event.getLinks();
		
		// if condition is set we have a flat list and don't need
		// to worry about any parent/child hierarchy from relationships
		if (links && !this._hasCondition()) {
			var linksLength = links.length;
			for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
				var link = links[linkIndex];
				var item1 = link.getItem1();
				var item2 = link.getItem2();
				var parent = null;
				var child = null;
								
				switch(link.getDirection()) {				
					case "1_2":
					case "both":
						parent = {item: item1, attribute: 'hasChildren'};
						if (!item1.getLazyLoadRelationships()) {
							item1.setLazyLoadRelationships(this.relationships ? this.relationships : "child");
						}
						child = item2;
						break;
					case "2_1":
						parent = {item: item2, attribute: 'hasChildren'};
						if (!item2.getLazyLoadRelationships()) {
							item2.setLazyLoadRelationships(this.relationships ? this.relationships : "parent");
						}
						child = item1;
				}					
				
				if (parent && child) {
					if (this._trace.isFinest()) {
						this._trace.finest("datasetId=" + event.getDataset().getId() + " parent=" + parent.item.getId() + " child=" + child.getId(), "_handleLinksAddedEvent");
					}
					this._clearChildren();
					this.onNew(child, parent);
				}
				else if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + event.getDataset().getId() + " ignoring linkId=" + link.getId(), "_handleLinksAddedEvent");
				}	
			}			
		}
	}	
});