/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */
 
dojo.provide("ibm.tivoli.navmodel.DatasetLinksStore");

dojo.require("ibm.tivoli.navmodel.DatasetStore");
dojo.require("ibm.tivoli.navmodel.Dataset");
	
ibm.tivoli.navmodel.DatasetLinksStore._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.DatasetLinksStore");
	
dojo.declare("ibm.tivoli.navmodel.DatasetLinksStore", ibm.tivoli.navmodel.DatasetStore, {
	// private members
	_clearCache: false,
	_trace: ibm.tivoli.navmodel.DatasetLinksStore._trace, 
	
	/*=====
	constructor_kwArgs: function(dataset, defaultProperties, returnItem) {
		// summary:
		//		An anonymous object used to configure the datastore
		// dataset: Object
		//		A ibm.tivoli.navmodel.Dataset instance
		// defaultProperties: String[]
		//		An array of property id values that are pre-fetched with the items
		// returnItem: Boolean
		//		A boolean indicating if the getValue() method should just return the actual
		//		item so that it can be used for custom formatting of the value
	},
	=====*/
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructor for dojo datastore;
		//		that receives link data from an ibm.tivoli.navmodel.Dataset
		//		Base class will mixin kwArgs      
	},


	//////////////////////////////////////////////////////////////
	// dojo.data.api.Identity
	//////////////////////////////////////////////////////////////
	
	getIdentity: function(/*Object*/ item){
		// summary:
    	//		Returns a unique identifier for an item. The return value will be
    	//		either a string or something that has a toString() method.
    	// item:
    	//		The item from the store from which to obtain its identifier.
		// exceptions:
    	//		Conforming implementations may throw an exception or return null if
    	//		item is not an item.
		if (!this.isItem(item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
  		var identity = item.getId();
		
		return identity;		
    },
    
	/*=====
	_onItem: function(item) {},
	=====*/
	
	/*=====
	_onError: function(error) {},
	=====*/
	
	/*=====
	fetchItemByIdentity_kwArgs: function(identity, onItem, onError, scope) {
		// summary:
		//		An anonymous object that defines the item to locate and callbacks to invoke when the
		//		item has been located and load has completed.
		// identity: String|Object
		//		The identity parameter is the identity of the item you wish to locate and load
		//		This attribute is required. It should be a string or an object that toString()
		//		can be called on.
		// onItem: _onItem
		//		The onItem parameter is the callback to invoke when the item has been loaded.
		//		It takes only one parameter, the item located, or null if none found.
		// onError: _onError
		//		The onError parameter is the callback to invoke when the item load encountered an error.
		//		It takes only one parameter, the error object
		// scope: Object
		//		If a scope object is provided, all of the callback functions (onItem, onError, etc)
		//		will be invoked in the context of the scope object.
	},
	=====*/

 	fetchItemByIdentity: function(/*fetchItemByIdentity_kwArgs*/ kwArgs) {
		// summary:
		//		Given the identity of an item, this method returns the item that has
		//		that identity through the onItem callback. Conforming implementations
		//		should return null if there is no item with the given identity.
		//		Implementations of fetchItemByIdentity() may sometimes return an item
		//		from a local cache and may sometimes fetch an item from a remote server.
		var args = {
 			identity: null,
 			onItem: function(item) {},
 			onError: function(error) {},
 			scope: null
 		};
 		dojo.mixin(args, kwArgs);
 		
 		var item = null;
 		var error = null;
 		if (args.identity) {
 			try {
 				var linkId  = args.identity;
 				if (this.dataset) {
 					item = this.dataset.getLink({linkId: linkId, properties: this.defaultProperties});
 				}
  			}
  			catch(err) {
  				error = err;  				
  			}  			
  		}  		
  		
 		if (item) {
 			args.onItem.call(args.scope, item);
 		}
 		else {
 			args.onError.call(args.scope, error ? error : "Could not fetch item for identity=" + args.identity);
 		}		
	},
	
	
	//////////////////////////////////////////////////////////////
	// dojo.data.api.Read
	//////////////////////////////////////////////////////////////
	
	/*=====
	fetch_onBegin: function(size, request){},
	=====*/
	
	/*=====
	fetch_onItem: function(item, request){},
	=====*/
	
	/*=====
	fetch_onComplete: function(items, request){},
	=====*/
	
	/*=====
	fetch_onError: function(errorData, request){},
	=====*/
	
	/*=====
	fetch_kwArgs: function(query, queryOptions, onBegin, onItem, onComplete, onError, scope,
							start, count, sort) {
		// summary:
		//		The kwArgs parameter may either be an instance of conforming to 
		//		dojo.data.api.Request or may be a simple anonymous object
		//		containing any of the above parameters.
		// query: String|Object
		//		The query parameter is used to specify search and filter condition information for the fetch.
		//		Only those items that match the search and/or filter will be returned.
		//		|	{	ids: {
		//		|			propId: boolean		// indicates propIds involved in the expression
		//		|		},
		//		|		expression: string,		// expression involiving propId's and values  (and/or together)
		//		|								// example:  ((Name contains 'abc' || Type contains 'abc)) and (Size > 100)
		//		|
		//		|		// the following is for compatability with legacy datastore
		//		| 		navModel: {
		//		|			search: {
		//		|				term: string,		the search text
		//		|				attributes: array	["property id1", "property id2", ...]
		//		|			}
		//		|		}
		//		|	}
		// queryOptions: Object?
		//		The queryOptions parameter is an optional parameter that can be used to modify the query.  The
		//		ignoreCase case can be specified to globally make all of the filter conditions case sensitive/insensitive.
		//		|	{ ignoreCase: boolean
		//		|	}
		// onBegin: fetch_onBegin
		//		If an onBegin callback function is provided, the callback function will be called just once, 
		//		before the first onItem callback is called. The onBegin callback function will be passed two arguments, 
		//		the the total number of items identified and the Request object.  If the total number is
		//		unknown, then size will be -1.  Note that size is not necessarily the size of the collection of items 
		//		returned from the query, as the request may have specified to return only a subset of the total set 
		//		of items through the use of the start and count parameters.
		// onItem: fetch_onItem
		//		If an onItem callback function is provided, the callback function will be called as each item 
		//		in the result is received. The callback function will be passed two arguments: the item itself, 
		//		and the Request object.
		// onComplete: fetch_onComplete
		//		If an onComplete callback function is provided, the callback function will be called just once,
		//		after the last onItem callback is called. Note that if the onItem callback is not present,
		//		then onComplete will be passed an array containing all items which matched the query and the 
		//		request object. If the onItem callback is present, then onComplete is called as: onComplete(null, request).
		// onError: fetch_onError
		//		If an onError callback function is provided, the callback function will be called if there is any 
		//		sort of error while attempting to execute the query. The onError callback function will be passed two arguments:
		//		an Error object and the Request object.
		// scope: Object
		//		If a scope object is provided, all of the callback functions (onItem, onComplete, onError, etc) 
		//		will be invoked in the context of the scope object.
		// start: int
		//		If a start parameter is specified, this is a indication to the datastore to
		//		only start returning items once the start number of items have been located and
		//		skipped.  When this parameter is paired with 'count', the store should be able
		//		to page across queries with millions of hits by only returning subsets of the
		//		hits for each query
		// count: int
		//		 If a count parameter is specified, this is a indication to the datastore to
		//		 only return up to that many items.  This allows a fetch call that may have
		//		millions of item matches to be paired down to something reasonable.
		// sort: Object[]
		//		If a sort parameter is specified, this is a indication to the datastore to
		//		sort the items in some manner before returning the items.  The array is an array of
		//		javascript objects that must conform to the following format to be applied to the
		//		fetching of items:
		//		|	{
		//		|		attribute: attribute || attribute-name-string,
		//		|		descending: true|false;   // Optional.  Default is false.
		//		|	}
	},
	=====*/
	
	fetch: function(/*fetch_kwArgs*/ kwArgs) {
		// summary:
		//		Given a query and set of defined options, such as a start and count of items to return,
		//		this method executes the query and makes the results available as data items.
		//		The format and expectations of stores is that they operate in a generally asynchronous
		//		manner, therefore callbacks are always used to return items located by the fetch parameters.
		// description:
		//		A Request object will always be returned and is returned immediately.
		//		The basic request is nothing more than the keyword args passed to fetch and
		//		an additional function attached, abort().  The returned request object may then be used
		//		to cancel a fetch.  All data items returns are passed through the callbacks defined in the
		//		fetch parameters and are not present on the 'request' object.
		// returns:
		//		The fetch() method will return a javascript object conforming to the API
		//		defined in dojo.data.api.Request.  In general, it will be the keywordArgs
		//		object returned with the required functions in Request.js attached.
		//		Its general purpose is to provide a convenient way for a caller to abort an
		//		ongoing fetch.
		// exceptions:
		//		Throws an exception if the query is not valid, or if the query is required but was not supplied.
		var args = {
			query: null,
			queryOptions: null,
			onBegin: function(size, request) {},
			onItem: function(item, request) {},
			onComplete: function(items, request){},
			onError: function(errorData, request){},
			scope: null,
			start: 0,
			count: null, //default should be to get all
			sort: [],
			abort: function() {}
		};
		dojo.mixin(args, kwArgs);
		
		if (this._trace.isFinest()) {
			this._trace.finest("dataset=" + this.dataset.getId() + " start=" + args.start + " count=" + args.count, "fetch");
		}
		
		// make sure event handling is set up
		this._validateEventHandler();
		
		// TED refresh is occurring
		this._clearCache = false;
		if (this.isRefresh()) {
			this._clearCache = this.dataset.isClearCacheOnRefresh();						
		}
		
		// translate information from the args
		var sort = this._getSort(args.sort);
		var condition = this._getCondition(args.query);		
					
		// if links are being viewed without context that includes items;
		// then we will have to fetch all the items to avoid fetching them one at
		// a time when the item1 and item2 are evaluated
		if (!this.dataset.hasCachedItems()) {
			this.dataset.getItems({sync: true});
		}
			
		var linkArgs = {
			onComplete: dojo.hitch(this, function(links, linkArgs) {
				args.onBegin.call(args.scope, linkArgs.filteredRows, args);
				
				if (links) {
					var linksLength = links.length;
					for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
						args.onItem.call(args.scope, links[linkIndex], args);                	
					}
				}
				
				args.onComplete.call(args.scope, links, args);				
			}),
			
			onError: dojo.hitch(this, function(error, exception, ioArgs) {
				args.onError.call(args.scope, error, args);
			}),
			
			clearCache: this._clearCache,
			start: args.start,
			count: args.count,	
			properties: this.defaultProperties,
			sort: sort,
			condition: condition
		}; 		
		this.dataset.getLinks(linkArgs);	
		       
        return args;
    },
    
	getAttributes: function(/*Object*/ item) {
		// summary:
		//		Returns an array with all the attributes that this item has.  This
		//		method will always return an array; if the item has no attributes
		//		at all, getAttributes() will return an empty array: [].
		//		Throws an exception if *item* is not an item
		// item:
		//		The item to access attributes on. Type: ibm.tivoli.navmodel.Item    
		var attributes = this.inherited(arguments);
		attributes.push("label");
		attributes.push("item1");
		attributes.push("item2");
		attributes.push("dataset1");
		attributes.push("dataset2");
		attributes.push("identity1");
		attributes.push("identity2");
		
		return attributes;		
    },
    
    getValue: function(/*Object*/ item, /*String*/ attribute, /*Object?*/ defaultValue) {
		// summary:
		//		Returns a single attribute value. Returns defaultValue if and only if item does not 
		//		have a value for attribute. Returns null if and only if null was explicitly set as the 
		//		attribute value. Returns undefined if and only if the item does not have a value for the 
		//		given attribute (which is the same as saying the item does not have the attribute).
		// item:
		//		The item to access attributes on.
		//		ibm.tivoli.navmodel.Item object
		// attribute:
		//		The attribute to access represented as a string.
		// defaultValue:
		//		Optional. A default value to use for the getValue return if the attribute does not exist 
		//		or has no value.
		// exceptions:
		//		Throws an exception if *item* is not an item, or *attribute* is not a string 
		if (!this.isItem(item)) {
			throw new Error("Specified item argument is not a valid item.");
        }
        
 		if (!dojo.isString(attribute)) {
			throw new Error("Specified attribute argument is not a string.");
		}  
		
		if (this.returnItem) {
			return item;
		}
		
		var value = defaultValue;    
        switch(attribute) {
        	case "label":
        		value = item.getLabel();
        		break;
        		
        	case "item1":
        		var item1 = item.getItem1();
				value = (item1 ? item1.getLabel() : "");
				break;
        		
        	case "item2":
        		var item2 = item.getItem2();
				value = (item2 ? item2.getLabel() : "");
				break;
        		
        	case "dataset1":
        		var d1 = item.getDataset1();
    			value = (d1 ? d1.getLabel() : "");
        		break;
        		
        	case "dataset2":
        		var d2 = item.getDataset2();
    			value = (d2 ? d2.getLabel() : "");
    			break;
    			
    		case "identity1":
    			value = item.getItem1Id();
        		break;
        		
        	case "identity2":
        		value = item.getItem2Id();
        		break;
        		
    		default:
    			var prop = item.getProperty(attribute);
        		if (prop) {
        			value = prop.getDisplayValue();
        		}
        }
        
        return value;        
    },
    
	/*=====
	loadItem_kwArgs: function(item, onItem, onError, scope) {
		// summary:
		//		An anonymous object that defines the item to load and callbacks to invoke when the
		//		load has completed.
		// item: Object
		//		The item parameter is an object that represents the item in question that should be
		//		contained by the store.  This attribute is required.
    	//		ibm.tivoli.navmodel.Item object
		// onItem: _onItem
		//		The onItem parameter is the callback to invoke when the item has been loaded.  
		//		It takes only one parameter, the error object
		// onError: _onError
		//		The onError parameter is the callback to invoke when the item load encountered an error.
		//		It takes only one parameter, the error object.
		// scope: Object
		//		If a scope object is provided, all of the callback functions (onItem, onError, etc)
		//		will be invoked in the context of the scope object.
	},
	=====*/
	
 	loadItem: function(/*loadItem_kwArgs*/ kwArgs) {
		// summary:
		//		Given an item, this method loads the item so that a subsequent call to store.isItemLoaded(item) 
		//		will return true.  If a call to isItemLoaded() returns true before loadItem() is even called,
		//		then loadItem() need not do any work at all and will not even invoke the callback handlers.  
		//		So, before invoking this method, check that the item has not already been loaded.
		var args = {
			item : null,
			onItem: function(item) {},
			onError: function(error){},
			scope: null
		};		
		dojo.advancedMixin(args, kwArgs);
	
		if (!this.isItem(args.item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
		if (this.isItemLoaded(args.item)){
			return args.item;
		}
		
		try {
			var dataset = args.item.getDataset();
			var item = dataset.getLink({linkId: args.item.getId(), properties: this.defaultProperties});
			args.onItem.call(args.scope, item);
		}
		catch(err) {
			args.onError.call(args.scope, err);
		}
	},
	
	
	//////////////////////////////////////////////////////////////
	// private methods
	///////////////////////////////////////////////////////////////
	
    _getObjectTypes: function() {
		// summary:
		//		Returns String[] of object types that can in the datastore
    	return {"link" : true};
    },
    
    _handleLinkPropertiesChangedEvent: function(/*Object*/ event) {
		var links = event.getLinks();    	
    	var propsArray = event.getProperties();			
		if (links && propsArray) {
			var refresh = false;
    		var length = links.length;			
			for (var index = 0;  index < length;  ++index) {
				var properties = propsArray[index];				
				if (properties) {
					var propsLength = properties.length;
					for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
						var link = links[index];
						var property = properties[propIndex];
						var propId = property.getId();
						if (this._trace.isFinest()) {
							this._trace.finest("datasetId=" + event.getDataset().getId() + " linkId=" + link.getId() + " prop=" + propId + " value=" + property.getDisplayValue(), "_handleLinkPropertiesChangedEvent");
						}
						
						this.onSet(link, propId, null, property.getDisplayValue());
						
						if (!refresh && (this._affectsSort(propId) || this._affectsCondition(propId))) {
							refresh = true;							
						}
					}										
				}				
			}
			
			if (refresh) {
				this.onRefresh();
			}	
		}
	},
	
	_handleLinksAddedEvent:function(/*Object*/ event) {
		var links = event.getLinks();
		if (links) {
			var length = links.length;
			for (var index = 0;  index < length;  ++index) {
				var link = links[index];
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + event.getDataset().getId() + " linkId=" + link.getId(), "_handleLinksAddedEvent");
				}
				this.onNew(link);
			}
			
			if (this._hasSort() || this._hasCondition()) {
				this.onRefresh();
			}
		}
	},
	
	_handleLinksRemovedEvent: function(/*Object*/ event) {
		var links = event.getLinks();
		if (links) {
			var length = links.length;
			for (var index = 0;  index < length;  ++index) {
				var link = links[index];
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + event.getDataset().getId() + " linkId=" + link.getId(), "_handleLinksRemovedEvent");
				}
				this.onDelete(link);
			}			
		}
	},
	
	_handleItemPropertiesChangedEvent: function(/*Object*/ event) {
		var items = event.getItems();
		var propsArray = event.getProperties();			
		if (items && propsArray) {
			var itemIds = [];
			var itemValues = [];
			
			// see if any "itemName" properties changed;  we use these to 
			// display the "item1" and "item2" values
    		var length = items.length;			
			for (var index = 0;  index < length;  ++index) {
				var properties = propsArray[index];				
				if (properties) {
					var propsLength = properties.length;
					for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
						if (properties[propIndex].getValueType() === "itemName") {						
							itemIds.push(items[index].getId());
							itemValues.push(properties[propIndex].getDisplayValue());
							if (this._trace.isFinest()) {
								this._trace.finest("datasetId=" + event.getDataset().getId() + " itemName property changed for itemId=" + items[index].getId(), "_handleItemPropertiesChangedEvent");
							}
							break;
						}
					}										
				}				
			}	
			
			// go check links that might be affected
			if (itemIds.length > 0) {
				var fetchArgs = {
					onComplete: dojo.hitch(this, function(links, args) {
						if (links) {
							var linksLength = links.length;
							for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
								var item1Id = links[linkIndex].getItem1Id();
								var item2Id = links[linkIndex].getItem2Id();
								var idLength = itemIds.length;
								
								for (var idIndex = 0;  idIndex < idLength;  ++idIndex) {
									if (item1Id && item1Id === itemIds[idIndex]) {
										this.onSet(links[linkIndex], "item1", null, itemValues[idIndex]);
									}
									else if (item2Id && item2Id === itemIds[idIndex]) {
										this.onSet(links[linkIndex], "item2", null, itemValues[idIndex]);
									}
								}																
							}
						}
					}),
					start: 0,
					count: null
				};
				this.fetch(fetchArgs);
			}			
		}
	}
});