/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.DatasetParameter");

// includes
dojo.require("ibm.tivoli.navmodel.Utils");

////////////////////////////////////
// DatasetParameter
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.DatasetParameter", null, {
	// private members
	_param: null,
	_baseUrl: null,
	_version: null,

	/*=====
	constructor_kwArgs: function(parameter, baseUrl, version) {
		// parameter: Object
		//		json parameter object
	},
	=====*/

	 constructor: function(/*constructor_kwArgs*/ kwArgs) {		
		// summary:
		//		Constructs Property object.
		this._param = kwArgs.parameter;
		this._baseUrl = kwArgs.baseUrl;
		this._version = (kwArgs.version ? kwArgs.version : ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion());
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	getAttribute: function(key) {		
		// summary:
		//		Returns general attribute for requested key
		return this._param[key];
	},

	setAttribute: function(key, value) {		
		// summary:
		//		Set general attribute for requested key
		this._param[key] = value;
	},
	
	getAttributes: function() {		
		// summary:
		//		Get all general attributes for the property
		return dojo.clone(this._param);		
	},
	
	removeAttribute: function(key) {
		// summary:
		//		Remove general attribute for requested key
		delete this._param[key];
	},
	
	getId: function() {		
		// summary:
		//		Returns the parameter id
		return this._param.id;
	},

	setId: function(id) {		
		// summary:
		//		Set the parameter id
		this._param.id = id;
	},

	getType: function() {		
		// summary:
		//		Returns the parameter type
		//		|	values:	
		//		|		"string", "string[]",
		//		|		"byte", "short", "int", "long", "byte[]", "short[]", "int[]", "long[]",
		//		|		"ubyte, "ushort", "uint", "ulong", "ubyte[]", "ushort[]", "uint[]", "ulong[]",	
		//		|		"boolean", "boolean[]"	
		//		|		"enum", "map"
		//		|		"date", "datetime", "time", "date[]", "datetime[]", "time[]"
		//		|		"isodate", isodatetime", "isotime", "isodate[]", "isodatetime[], "isotime[]", "isointerval", "isoduration"
		//		|		"float", "double", "float[]", "double[]"
		return this._param.type;
	},

	setType: function(type) {		
		// summary:
		//		Set the parameter type
		//		|	values:	
		//		|		"string", "string[]",
		//		|		"byte", "short", "int", "long", "byte[]", "short[]", "int[]", "long[]",
		//		|		"ubyte, "ushort", "uint", "ulong", "ubyte[]", "ushort[]", "uint[]", "ulong[]",	
		//		|		"boolean", "boolean[]"	
		//		|		"enum", "map"
		//		|		"date", "datetime", "time", "date[]", "datetime[]", "time[]"
		//		|		"isodate", isodatetime", "isotime", "isodate[]", "isodatetime[], "isotime[]", "isointerval", "isoduration"
		//		|		"float", "double", "float[]", "double[]"
		this._param.type = type;
	},
	
	getLabel: function() {		
		// summary:
		//		Returns the parameter label
		return this._param.label;
	},

	setLabel: function(label) {		
		// summary:
		//		Set the parameter label
		this._param.label = label;
	},
	
	getDescription: function() {		
		// summary:
		//		Returns the parameter description
		return this._param.description;
	},

	setDescription: function(description) {		
		// summary:
		//		Set the parameter description
		this._param.description = description;
	},

	isHidden: function() {
		// summary:
		//		Indicates if parameter should be hidden from user
		return this._param.hidden;
	},
	
	setHidden: function(hidden) {
		// summary:
		//		Set value indicating if parameter should be hidden from user
		this._param.hidden = hidden;
	},
	
	isRequired: function() {
		// summary:
		//		Indicates if parameter is required when creating dataset
		return this._param.required;
	},
	
	setRequired: function(required) {
		// summary:
		//		Set value indicating if parameter is required when creating dataset
		this._param.required = required;
	},
	
	hasPossibleValues: function() {
		// summary:
		//		Returns true if the parameter supports a list of possible values		
		return (this._param.values || this._param.valuesUri ? true : false);
	},
	
	hasInlinedPossibleValues: function() {
		// summary:
		//		Returns true if the paramter supports possible values and
		//		all of those values have been inlined in the json
		return (this._param.values ? true : false);
	},
	
	hasUrlAddressablePossibleValues: function() {
		// summary:
		//		Returns true if the parameter supports possible values and
		//		the values are addressable via a url
		return (this._param.valuesUri ? true : false);
	},
	
	getPossibleValues: function(kwArgs) {
		// summary:
		//		Retrieves a list of possible values for parameter
		var args = {
			onComplete: function(values, totalCount, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			start: 0,
			count: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
	
		// possible values were inlined; just return them
		if (this._param.values) {
			if (args.start === 0 && args.count === null) {
				args.onComplete(this._param.values, this._param.values.length, args);
			}
			else {
				var values = [];
				var length = this._param.values.length;
				var count = args.count ? args.count : length;
				for (var index = args.start;  index < length && count > 0;  ++index, --count) {
					values.push(this._param.values[index]);
				}
				args.onComplete(values, length, args);
			}
		}
		// possible values are available via uri
		else if (this._param.valuesUri) {
			var content = ibm.tivoli.navmodel.Utils.getContent();
			if (args.start) {
				content.start = args.start;
			}
			if (args.count) {
				content.count = args.count;
			}
			
			dojo.xhrGet({
				url: this._baseUrl + this._param.valuesUri,
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version, "datasetParmValueList")
				},
				handleAs: "json",
				sync: args.sync,
				content: content,		
				load: dojo.hitch(this, function(result, ioArgs){
					args.onComplete(result.items, result.totalRows, args);					
				}),
				error: dojo.hitch(this, function(error, ioArgs){
					args.onError(error, ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "getPossibleValues", this._trace, this._version), ioArgs);
				})
			});	
		}
		// doesn't have any
		else {
			args.onComplete(null, 0, args);
		}		
	},
	
	getDefaultValue: function() {
		// summary:
		//		Returns default value for parameter
		return this._param["default"];
	},
	
	setDefaultValue: function(defaultVal) {
		// summary:
		//		Set the default value for parameter
		this._param["default"] = defaultVal;
	},
	
	getMinValue: function() {		
		// summary:
		//		Returns minimum value;  for string parms this is minimum length
		return this._prop.min;
	},

	setMinValue: function(min) {		
		// summary:
		//		Set the minimum value
		this._prop.min = min;
	},
	
	getMaxValue: function() {		
		// summary:
		//		Returns maximum value;  for string parms this is maximum length
		return this._prop.max;
	},

	setMaxValue: function(max) {		
		// summary:
		//		Set the maximum value
		this._prop.max = max;
	}
});