/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */
 
dojo.provide("ibm.tivoli.navmodel.DatasetTreeStore");

// includes
dojo.require("ibm.tivoli.navmodel.DatasetStore");
dojo.require("ibm.tivoli.navmodel.Dataset");
	
ibm.tivoli.navmodel.DatasetTreeStore._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.DatasetTreeStore");
	
///////////////////////////////////////////
// This dojo datastore is compatible with:
// 		dijit.Tree
//		dojox.grid.TreeGrid
//
// Do NOT use for:
//		dojox.grid.LazyTreeGrid
// instead use DatasetLazyTreeStore
///////////////////////////////////////////

dojo.declare("ibm.tivoli.navmodel.DatasetTreeStore", ibm.tivoli.navmodel.DatasetStore, {
	// private members 
	_clearCache: false,
	_parent: null,
	_children: null,
	_isTreeGrid: false,
	_sort: null,
	_trace: ibm.tivoli.navmodel.DatasetTreeStore._trace,
	
	/*=====
	constructor_kwArgs: function(dataset, defaultProperties) {
		// summary:
		//		An anonymous object used to configure the datastore
		// dataset: Object
		//		A ibm.tivoli.navmodel.Dataset instance
		// defaultProperties: String[]
		//		An array of property id values that are pre-fetched with the items
		// relationships: String  (optional)
		//		Comma delimited list of relationships to determine related items; 
		//		can also be keywords: child, parent, all.  If not specified the value is
		//		retrieved from the individual item that is being expanded
		// isTreeGrid: boolean  (optional)
		//		Indicates that the store is being used by a TreeGrid component; and a 
		//		onRefresh should be used instead of onDelete (there are bugs in the TreeGrid) 
	},
	=====*/
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructor for dojo datastore;
		//		that receives item data (in tree format) from an ibm.tivoli.navmodel.Dataset
		//		Base class will mixin kwArgs
		
		this._isTreeGrid = kwArgs.isTreeGrid;
		this._clearParents();
		this._clearChildren();		
	},
	
	
	//////////////////////////////////////////////////////////////
	// dojo.data.api.Identity
	//////////////////////////////////////////////////////////////
	
	getIdentity: function(/*Object*/ item){
		// summary:
		//		Returns a unique identifier for an item. The return value will be
		//		either a string or something that has a toString() method.
		// item:
		//		The item from the store from which to obtain its identifier.
		//		ibm.tivoli.navmodel.Item object
		// exceptions:
		//		Conforming implementations may throw an exception or return null if
		//		item is not an item.
		if (!this.isItem(item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
  		var identity;
  		switch(item.getObjectType()) {
  			case "item":
  				identity = item.getId();
  				break;
  		}
  		  		
  		return identity;
    },
    
	/*=====
	_onItem: function(item) {},
	=====*/
	
	/*=====
	_onError: function(error) {},
	=====*/
    
    /*=====
	fetchItemByIdentity_kwArgs: function(identity, onItem, onError, scope) {
		// summary:
		//		An anonymous object that defines the item to locate and callbacks to invoke when the
		//		item has been located and load has completed.
		// identity: String|Object
		//		The identity parameter is the identity of the item you wish to locate and load
		//		This attribute is required. It should be a string or an object that toString()
		//		can be called on.
		// onItem: _onItem
		//		The onItem parameter is the callback to invoke when the item has been loaded.
		//		It takes only one parameter, the item located, or null if none found.
		// onError: _onError
		//		The onError parameter is the callback to invoke when the item load encountered an error.
		//		It takes only one parameter, the error object.
		// scope: Object
		//		If a scope object is provided, all of the callback functions (onItem, onError, etc)
		//		will be invoked in the context of the scope object.
	},
	=====*/
	
 	fetchItemByIdentity: function(/*fetchItemByIdentity_kwArgs*/ kwArgs) {
		// summary:
		//		Given the identity of an item, this method returns the item that has
		//		that identity through the onItem callback. Conforming implementations
		//		should return null if there is no item with the given identity.
		//		Implementations of fetchItemByIdentity() may sometimes return an item
		//		from a local cache and may sometimes fetch an item from a remote server.
 		var args = {
 			identity: null,
 			onItem: function(item) {},
 			onError: function(error) {},
 			scope: null
 		};
 		dojo.mixin(args, kwArgs);
 		
 		var item = null;
 		var error = null;
 		if (args.identity) {
 			// <itemId>
 			try {
				item = this.dataset.getItem({itemId: args.identity, properties: this.defaultProperties});
			}
			catch(error1) {
				error = error1;
			}
  		}
  		
  		if (item) {
 			args.onItem.call(args.scope, item);
 		}
 		else {
 			args.onError.call(args.scope, error ? error : "Could not fetch item for identity=" + args.identity);
 		}
	},
	
	
	//////////////////////////////////////////////////////////////
	// dojo.data.api.Read
	//////////////////////////////////////////////////////////////
	
	/*=====
	fetch_onBegin: function(size, request){},
	=====*/
	
	/*=====
	fetch_onItem: function(item, request){},
	=====*/
	
	/*=====
	fetch_onComplete: function(items, request){},
	=====*/
	
	/*=====
	fetch_onError: function(errorData, request){},
	=====*/
	
	/*=====
	fetch_kwArgs: function(query, queryOptions, onBegin, onItem, onComplete, onError, scope, start, count, sort) {
		// summary:
		//		The kwArgs parameter may either be an instance of conforming to 
		//		dojo.data.api.Request or may be a simple anonymous object
		//		containing any of the above parameters.
		// query: String|Object
		//		The query parameter is used to specify search and filter condition information for the fetch.
		//		Only those items that match the search and/or filter will be returned.
		//		|	{	ids: {
		//		|			propId: boolean		// indicates propIds involved in the expression
		//		|		},
		//		|		expression: string,		// expression involiving propId's and values  (and/or together)
		//		|								// example:  ((Name contains 'abc' || Type contains 'abc)) and (Size > 100)
		//		|
		//		|		// the following is for compatability with legacy datastore
		//		| 		navModel: {
		//		|			search: {
		//		|				term: string,		the search text
		//		|				attributes: array	["property id1", "property id2", ...]
		//		|			}
		//		|		}
		//		|	}
		// queryOptions: Object?
		//		The queryOptions parameter is an optional parameter that can be used to modify the query.  The
		//		ignoreCase case can be specified to globally make all of the filter conditions case sensitive/insensitive.
		//		|	{ ignoreCase: boolean
		//		|	}
		// onBegin: fetch_onBegin
		//		If an onBegin callback function is provided, the callback function will be called just once, 
		//		before the first onItem callback is called. The onBegin callback function will be passed two arguments, 
		//		the the total number of items identified and the Request object.  If the total number is
		//		unknown, then size will be -1.  Note that size is not necessarily the size of the collection of items 
		//		returned from the query, as the request may have specified to return only a subset of the total set 
		//		of items through the use of the start and count parameters.
		// onItem: fetch_onItem
		//		If an onItem callback function is provided, the callback function will be called as each item 
		//		in the result is received. The callback function will be passed two arguments: the item itself, 
		//		and the Request object.
		// onComplete: fetch_onComplete
		//		If an onComplete callback function is provided, the callback function will be called just once,
		//		after the last onItem callback is called. Note that if the onItem callback is not present,
		//		then onComplete will be passed an array containing all items which matched the query and the 
		//		request object. If the onItem callback is present, then onComplete is called as: onComplete(null, request).
		// onError: fetch_onError
		//		If an onError callback function is provided, the callback function will be called if there is any 
		//		sort of error while attempting to execute the query. The onError callback function will be passed two arguments:
		//		an Error object and the Request object.
		// scope: Object
		//		If a scope object is provided, all of the callback functions (onItem, onComplete, onError, etc) 
		//		will be invoked in the context of the scope object.
		// start: int
		//		If a start parameter is specified, this is a indication to the datastore to
		//		only start returning items once the start number of items have been located and
		//		skipped.  When this parameter is paired with 'count', the store should be able
		//		to page across queries with millions of hits by only returning subsets of the
		//		hits for each query
		// count: int
		//		 If a count parameter is specified, this is a indication to the datastore to
		//		 only return up to that many items.  This allows a fetch call that may have
		//		millions of item matches to be paired down to something reasonable.
		// sort: Object[]
		//		If a sort parameter is specified, this is a indication to the datastore to
		//		sort the items in some manner before returning the items.  The array is an array of
		//		javascript objects that must conform to the following format to be applied to the
		//		fetching of items:
		//		|	{
		//		|		attribute: attribute || attribute-name-string,
		//		|		descending: true|false;   // Optional.  Default is false.
		//		|	}
	},
	=====*/
		
	fetch: function(/*fetch_kwArgs*/ kwArgs) {
		// summary:
		//		This datastore exposes the tree origin nodes of the Dataset and allows for
		//		lazy loading of children afterwards.
		//		The format and expectations of stores is that they operate in a generally asynchronous
		//		manner, therefore callbacks are always used to return items located by the fetch parameters.
		// description:
		//		A Request object will always be returned and is returned immediately.
		//		The basic request is nothing more than the keyword args passed to fetch and
		//		an additional function attached, abort().  The returned request object may then be used
		//		to cancel a fetch.  All data items returns are passed through the callbacks defined in the
		//		fetch parameters and are not present on the 'request' object.
		// returns:
		//		The fetch() method will return a javascript object conforming to the API
		//		defined in dojo.data.api.Request.  In general, it will be the keywordArgs
		//		object returned with the required functions in Request.js attached.
		//		Its general purpose is to provide a convenient way for a caller to abort an
		//		ongoing fetch.
		// exceptions:
		//		Throws an exception if the query is not valid, or if the query is required but was not supplied.
		var args = {
			onBegin: function(size, request) {},
			onItem: function(item, request) {},
			onComplete: function(items, request){},
			onError: function(errorData, request){},
			scope: null,
			abort: function() {}
		};
		dojo.mixin(args, kwArgs);
		
		if (this._trace.isFinest()) {
			this._trace.finest("dataset=" + this.dataset.getId() + " fetching origin node(s)", "fetch");
		}
		
		// make sure event handling is set up
		this._validateEventHandler();
		
		// TED refresh is occurring
		this._clearCache = false;
		if (this.isRefresh()) {
			this._clearParents();
			this._clearChildren();
			this._clearCache = this.dataset.isClearCacheOnRefresh();						
		}
		
		// fetching "origin" items
		if (args.query && args.query.origins) {
			this._clearChildren();			
			this._fetchOrigins(args);
		}		
		// fetch all items
		else {
			this._fetchItems(args);
		}
		       
        return args;
    },
    
    getAttributes: function(/*Object*/ item) {
		// summary:
		//		Returns an array with all the attributes that this item has.  This
		//		method will always return an array; if the item has no attributes
		//		at all, getAttributes() will return an empty array: [].
		// item:
		//		The item to access attributes on.
		//		ibm.tivoli.navmodel.Item object
		// exceptions:
		//		Throws an exception if *item* is not an item
		var attributes = this.inherited(arguments);
  		
		// add in "children" attribute if appropriate
		if (this._hasChildren(item)) {
			attributes.push("children");
		}
		
		return attributes;		
    },
    
    getValue: function(/*Object*/ item, /*String*/ attribute, /*Object?*/ defaultValue) {
		// summary:
		//		Returns a single attribute value. Returns defaultValue if and only if item does not have a 
		//		value for attribute. Returns null if and only if null was explicitly set as the attribute value. 
		//		Returns undefined if and only if the item does not have a value for the given attribute 
		//		(which is the same as saying the item does not have the attribute).
		// item:
		//		The item to access value on.
		//		ibm.tivoli.navmodel.Item object
		// attribute:
		//		The attribute to access represented as a string.
		// defaultValue:
		//		Optional. A default value to use for the getValue return if the attribute does not exist or 
		//		has no value.
		// exceptions:
		//		Throws an exception if *item* is not an item, or *attribute* is not a string
    	if (arguments.length < 2) {
            throw new Error("Incorrect number of arguments specified: expecting 2 (item, attribute)");
        }
        
        if (!this.isItem(item)) {
            throw new Error("Specified item argument is not a valid item.");
        }
        
        if (!dojo.isString(attribute)) {
            throw new Error("Specified attribute argument is not a string.");
        }  
               
        var value = defaultValue;
        
        // general attribute value
    	if (this.returnItem) {
    		return item;
    	}
    
    	var prop = item.getProperty(attribute);
    	if (prop) {
    		value = prop.getDisplayValue();            
    	}	         
                           	
        return value;
    },
    
    getValues: function(/*Object*/ item, /*String*/ attribute) {
		// summary:
		//		This getValues() method works just like the getValue() method, but getValues()
		//		always returns an array rather than a single attribute value.  The array
		//		may be empty, may contain a single attribute value, or may contain many
		//		attribute values.
		//		
		//		If the item does not have a value for the given attribute, then getValues()
		//		will return an empty array: [].  (So, if store.hasAttribute(item, attribute)
		//		returns false, then store.getValues(item, attribute) will return [].)
		// item:
		//		The item to access values on.
		//		ibm.tivoli.navmodel.Item object
		// attribute:
		//		The attribute to access represented as a string.
		// exceptions:
		//		Throws an exception if *item* is not an item, or *attribute* is not a string
    	if (arguments.length < 2) {
            throw new Error("Incorrect number of arguments specified: expecting 2 (item, attribute)");
        }
        
        if (!this.isItem(item)) {
            throw new Error("Specified item argument is not a valid item.");
        }
        
        if (!dojo.isString(attribute)) {
            throw new Error("Specified attribute argument is not a string.");
        }
        
        var values = null;
        
        // asking for the "children" of this item
        if (attribute === "children") {
        	values = this._fetchChildren(item);
        }
        // general attribute value
    	else {
    		values = [this.getValue(item, attribute)];
    	}
    	
    	return values;
    },
    
    
    /*=====
	loadItem_kwArgs: function(item, onItem, onError, scope) {
		// summary:
		//		An anonymous object that defines the item to load and callbacks to invoke when the
		//		load has completed.
		// item: Object
		//		The item parameter is an object that represents the item in question that should be
		//		contained by the store.  This attribute is required.
		//		ibm.tivoli.navmodel.Item object
		// onItem: _onItem
		//		The onItem parameter is the callback to invoke when the item has been loaded.	
		//		It takes only one parameter, the fully loaded item.
		// onError: _onError
		//		The onError parameter is the callback to invoke when the item load encountered an error.
		//		It takes only one parameter, the error object.
		// scope: Object
		//		If a scope object is provided, all of the callback functions (onItem, onError, etc)
		//		will be invoked in the context of the scope object.
		//		onItem.call(dojo.global(), item, request)
	},
	=====*/
	
	loadItem: function(/*loadItem_kwArgs*/ kwArgs) {
		var args = {
			item : null,
			onItem: function(item) {},
			onError: function(error){},
			scope: null
		};		
		dojo.advancedMixin(args, kwArgs);
	
		if (!this.isItem(args.item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
		if (this.isItemLoaded(args.item)){
			return args.item;
		}
		
		try {
			var item = null;
			var dataset = args.item.getDataset();
			switch(item.getObjectType()) {
				case "item":					
					item = dataset.getItem({itemId: args.item.getId(), properties: this.defaultProperties});
					break;
			}	
			args.onItem.call(args.scope, item);
		}
		catch(err) {
			args.onError.call(args.scope, err);
		}
	},
	
	//////////////////////////////////////////////////////////////
	// private methods
	///////////////////////////////////////////////////////////////
	
	_getObjectTypes: function() {
		// summary:
		//		Returns String[] of object types that can in the datastore
    	return {
    		"item": true
		};
    },
    
    _fetchOrigins: function(/*fetch_kwArgs*/ args) {
		// summary:
		//		Fetch origin items for the tree
		
		// we do have some possible origins
    	if (this.dataset.getItemCount() > 0) {
    		this._sort = this._getSort(args.sort);
				
			var originArgs = {
				clearCache: this._clearCache,
				properties: this.defaultProperties,
				sort: this._sort,
				onComplete: dojo.hitch(this, function(originItems, originArgs) {
					// we have origin items		
					var origins = [];
					if (originItems && originItems.length > 0) {
						origins = this._concat(origins, originItems, "root");
					
						var originsLength = origins.length;
						args.onBegin.call(args.scope, originsLength, args);				
						for(var originIndex = 0;  originIndex < originsLength;  ++originIndex) {                 	                	
                			args.onItem.call(args.scope, origins[originIndex], args);                	
						}					
						args.onComplete.call(args.scope, origins, args);					
					}
					// no origins declared
					else {
						this._finishFetchWhenNoOrigins(args);
					}
				}),			
				onError: dojo.hitch(this, function(error, exception, ioArgs) {
					args.onError.call(args.scope, error, args);
				})
			}; 		
			this.dataset.getOriginItems(originArgs);
		}
		// no items; so no origins
		else {
			this._finishFetchWhenNoOrigins(args);
		}
    },    
    
    _finishFetchWhenNoOrigins: function(/*fetch_kwArgs*/ args) {
		// summary:
		//		Fetch origin items for the tree when there are no items marked as origins		
    	var origins = [];    			
    	
    	// has items and possible child datasets
    	if (this.dataset.getItemCount() > 0) {
    		var itemsArgs = {
    			properties: this.defaultProperties,
    			onComplete: dojo.hitch(this, function(items, itemsArgs) {
    				if (items && items.length > 0) {
    					origins = this._concat(origins, items, "root");
    				}
    		
					var originsLength = origins.length;
					args.onBegin.call(args.scope, originsLength, args);				
					for(var originIndex = 0;  originIndex < originsLength;  ++originIndex) {                 	                	
                		args.onItem.call(args.scope, origins[originIndex], args);                	
					}					
					args.onComplete.call(args.scope, origins, args);
				}),			
				onError: dojo.hitch(this, function(error, exception, ioArgs) {
					args.onError.call(args.scope, error, args);
				})
    		};
    		this.dataset.getItems(itemsArgs);
    	}
    	// does NOT have items; check child datasets
    	else {
			var originsLength = origins.length;
			args.onBegin.call(args.scope, originsLength, args);				
			for(var originIndex = 0;  originIndex < originsLength;  ++originIndex) {                 	                	
                args.onItem.call(args.scope, origins[originIndex], args);                	
			}					
			args.onComplete.call(args.scope, origins, args);
    	}
    },
    
    _fetchItems: function(/*fetch_kwArgs*/ args) {
		// summary:
		//		Fetch all items for the dataset if the query did not indicate that origins should be returned
		
    	// we do have some possible origins
    	this._sort = this._getSort(args.sort);
				
		var itemArgs = {
			clearCache: this._clearCache,
			properties: this.defaultProperties,
			sort: this._sort,
			onComplete: dojo.hitch(this, function(items, itemArgs) {
				args.onBegin.call(args.scope, itemArgs.filteredRows, args);
								
				if (items) {
					var itemsLength = items.length;
					for(var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {                 	                	
                		args.onItem.call(args.scope, items[itemIndex], args);                	
					}
				}				
					
				args.onComplete.call(args.scope, origins, args);
			}),			
			onError: dojo.hitch(this, function(error, exception, ioArgs) {
				args.onError.call(args.scope, error, args);
			})	
		}; 		
		this.dataset.getItems(itemArgs);
    },
    
	_hasChildren: function(/*Object*/ item) {
		// summary:
		//		Indicates if the item has children items
    	var hasChildren = false;    	
    	switch(item.getObjectType()) {
    		case "item":
    			hasChildren = (item.isLazyLoad() || item.getLazyLoadRelationships());
    			break;
    	}
    	
    	return hasChildren;
    },
    
	_fetchChildren: function(/*Object*/ item) {
		// summary:
		//		Fetch children for item from the dataset
    	var children = [];
    	
		if (this._trace.isFinest()) {
			this._trace.finest("datasetId=" + this.dataset.getId() + " getting children for item=" + item.getId(), "_fetchChildren");
		}
						
    	if (this._hasChildren(item)) {
    		var cachedChildren = this._getChildren(item.getId());
    		if (cachedChildren) {
    			children = cachedChildren;
    			if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + this.dataset.getId() + " returning cached children for item=" + item.getId(), "_fetchChildren");
				}
    		}
    		else {
        		switch(item.getObjectType()) {
        			case "item":
						children = this._fetchChildrenForItem(item);
        				break;
				}
				
				// cache the children for this item
				this._setChildren(item.getId(), children);
			}
		}
					
		return children;		
    },
    		
    _fetchChildrenForItem: function(/*Object*/ item) {
		// summary:
		//		Fetch children for an "item" object
    	var children = [];  
    	var relatedArgs = {
        	onComplete: dojo.hitch(this, function(items, links, kwArgs) {
				if (items && items.length > 0) {
					children = this._concatSiblings(children, items, item);									
				}
			}),			
			onError: dojo.hitch(this, function(error, exception, ioArgs) {				
			}),
			relationships: this.relationships,
			properties: this.defaultProperties,
			sort: this._sort,
			sync: true
        };
        item.getRelatedItems(relatedArgs);
        			
    	return children;    	
    },
    
    _getParentId: function(/*String*/ id) {
		// summary:
		//		Get parentId for the requested child id
    	return this._parent[id];
    },
    
	_setParentId: function(/*String*/ id, /*String*/ parentId) {
		// summary:
		//		Set parentId for the requested child id
    	this._parent[id] = parentId;
    },
    
	_clearParents: function(/*String*/ id) {
		// summary:
		//		Clear parentId for the requested child id or all if not specified
    	if (id) {
    		delete this._parent[id];
    	}
    	else {
    		this._parent = {};
    	}
    },
    
	_getChildren: function(/*String*/ id) {
		// summary:
		//		Get cached children for parent id
    	return this._children[id];
    },
    
	_setChildren: function(/*String*/ id, /*String*/ children) {
		// summary:
		//		Set cached children for parent id
    	this._children[id] = children;
    },
    
	_clearChildren: function(/*String*/ id) {
		// summary:
		//		Clear cached children for parent id or all if not specified
    	if (id) {
    		delete this._children[id];
    	}
    	else {
    		this._children = {};
    	}
    },
      
    _concat: function(/*Object[]*/ list, /*Object[]*/ items, /*String*/ parentId) {
		var itemsLength = items.length;
    	for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
    		var item = items[itemIndex];
    		var prevParentId = this._getParentId(item.getId());    			
    		if (!prevParentId || prevParentId === parentId) {
				this._setParentId(item.getId(), parentId);
				list.push(item);
					
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + this.dataset.getId() + " parent=" + parentId + " child=" + item.getId(), "_concat");
				}
			}				
    	}
    	
    	return list;
    },
    
    _concatItem: function(/*Object[]*/ list, /*Object*/ item, /*String*/ parentId) {
    	var prevParentId = this._getParentId(item.getId());
    	if (!prevParentId || prevParentId === parentId) {
			this._setParentId(item.getId(), parentId);
			list.push(item);
				
			if (this._trace.isFinest()) {
				this._trace.finest("datasetId=" + this.dataset.getId() + " parent=" + parentId + " child=" + item.getId(), "_concatItem");
			}
		}				    	
    	
    	return list;    	
    },
    
    _concatSiblings: function(/*Object[]*/ list, /*Object[]*/ items, /*Object*/ siblingItem) {
    	var parentId = siblingItem.getId();
    	var itemsLength = items.length;
    	for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
    		var item = items[itemIndex];
    		if (item.getDataset() === siblingItem.getDataset()) {    			
    			var prevParentId = this._getParentId(item.getId());    		
    			if (!prevParentId || prevParentId === parentId) {
					this._setParentId(item.getId(), parentId);
					list.push(item);
					
					if (this._trace.isFinest()) {
						this._trace.finest("datasetId=" + this.dataset.getId() + " parent=" + siblingItem.getId() + " child=" + item.getId(), "_concatSiblings");
					}
				}					
    		}    			    		
    	}
    	
    	return list;
    },
    
    _handleChangedEvent: function(/*Object*/ event) {
    	this._clearParents();
    	this._clearChildren();
    	this.onRefresh();
	},
	
    _handleLabelChangedEvent: function(/*Object*/ event) {
    	if (this._trace.isFinest()) {
			this._trace.finest("datasetId=" + event.getDataset().getId() + " label=" + event.getNewLabel(), "_handleLabelChangedEvent");
		}		
    },
	
	_handleItemPropertiesChangedEvent: function(/*Object*/ event) {
		var items = event.getItems();    	
    	var propsArray = event.getProperties();			
		if (items && propsArray) {
    		var length = items.length;			
			for (var index = 0;  index < length;  ++index) {
				var properties = propsArray[index];				
				if (properties) {
					var propsLength = properties.length;
					for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
						var item = items[index];
						var property = properties[propIndex];						
						if (this._trace.isFinest()) {
							this._trace.finest("datasetId=" + event.getDataset().getId() + " itemId=" + item.getId() + " prop=" + property.getId() + " value=" + property.getDisplayValue(), "_handleItemPropertiesChangedEvent");
						}
						
						this.onSet(item, property.getId(), null, property.getDisplayValue());
					}										
				}				
			}	
		}	
	},
	
    _handleItemsAddedEvent: function(/*Object*/ event) {
    	var dataset = event.getDataset();
    	var isDescendent = dataset.getId() != this.dataset.getId();
		var items = event.getItems();
		if (items) {
    		var length = items.length;			
			for (var index = 0;  index < length;  ++index) {
				var item = items[index];				
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + event.getDataset().getId() + " itemId=" + item.getId() + " origin=" + item.isOrigin(), "_handleItemsAddedEvent");
				}
				
				if (item.isOrigin()) {
					// new origin for main dataset
					this.onNew(item);
				}
			}
		}    		
	},
	
	_handleItemsRemovedEvent: function(/*Object*/ event) {
		if (this._isTreeGrid) {
			this._trace.finest("datasetId=" + event.getDataset().getId() + " issuing onRefresh because isTreeGrid=true", "_handleItemsRemovedEvent");			
			this.onRefresh();
			return;
		}
			
		var items = event.getItems();
		if (items) {
			var length = items.length;			
			for (var index = 0;  index < length;  ++index) {
				var item = items[index];				
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + event.getDataset().getId() + " itemId=" + item.getId(), "_handleItemsRemovedEvent");
				}
				
				this._clearChildren(this._getParentId(item.getId()));
				this._clearParents(item.getId());
				this.onDelete(item);
			}			
		}
	},
	
	_handleLinksAddedEvent:function(/*Object*/ event) {
		var links = event.getLinks();
		if (links) {
			var linksLength = links.length;
			for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
				var link = links[linkIndex];
				var item1 = link.getItem1();
				var item2 = link.getItem2();
				var parent = null;
				var child = null;
					
				// we have a tree			
				if (this.dataset.getViewType() === "tree") {
					switch(link.getDirection()) {				
						case "1_2":
							parent = {item: item1};
							if (!item1.getLazyLoadRelationships()) {
								item1.setLazyLoadRelationships(this.relationships ? this.relationships : "child");
							}
							child = item2;
							break;
						case "2_1":
							parent = {item: item2};
							if (!item2.getLazyLoadRelationships()) {
								item2.setLazyLoadRelationships(this.relationships ? this.relationships : "parent");
							}
							child = item1;
					}					
				}
				// not necessarily a tree; see if item1 is in the tree and item2 is not
				else if (this._getParentId(item1.getId()) && !this._getParentId(item2.getId())) {
					parent = {item: item1};
					if (!item1.getLazyLoadRelationships()) {
						item1.setLazyLoadRelationships(this.relationships ? this.relationships : "all");
					}						
					child = item2;
				}
				// not necessarily a tree; see if item2 is in the tree and item1 is not
				else if (this._getParentId(item2.getId()) && !this._getParentId(item1.getId())) {
					parent = {item: item2};
					if (!item2.getLazyLoadRelationships) {
						item2.setLazyLoadRelationships(this.relationships ? this.relationships : "all");
					}							
					child = item1;
				}
				
				if (parent && child) {
					if (this._trace.isFinest()) {
						this._trace.finest("datasetId=" + event.getDataset().getId() + " parent=" + parent.item.getId() + " child=" + child.getId(), "_handleLinksAddedEvent");
					}	
					this._clearChildren(parent.item.getId());
					this.onNew(child, parent);
				}
				else if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + event.getDataset().getId() + " ignoring linkId=" + link.getId(), "_handleLinksAddedEvent");
				}	
			}			
		}
	}
});