/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.DatasetUtils"); 

// includes
dojo.require("ibm.tivoli.navmodel.DatasetParameter");
dojo.require("ibm.tivoli.navmodel.Utils");
dojo.require("ibm.tivoli.navmodel.Trace");

ibm.tivoli.navmodel.DatasetUtils._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.DatasetUtils");

ibm.tivoli.navmodel.DatasetUtils.getParameters = function(kwArgs) {
	// summary: 
	//		Retrieve available parameters that can be used when creating a dataset instance
	// onComplete: onComplete(parameters, kwArgs)
	//		callback for successful fetch
	// onError: _onError(error, exception, ioArgs)
	//		callback for error condition
	// baseUrl: String
	//		base url for identifying rest service
	// provider: String
	//		provider id
	// datasource: String
	//		datasource id
	// dataset: String
	//		datasetId
	// start: int
	//		start index to fetch
	// count: int
	//		number of items to fetch; null means all
	// sync: Boolean
	//		indicates if fetch is synchronous	
	// version: Float?
	//		optional: version used to communicate with rest service (i.e. json version)
	var args = {
		onComplete: function(parameters, parameterCount, kwArgs) {},
		onError: function(error, exception, ioArgs) {},
		baseUrl: null,
		provider: null,
		datasource: null,
		dataset: null,
		start: 0,
		count: null,
		sync: false,
		version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
	};
	dojo.mixin(args, kwArgs);

	// url needed to fetch parameters
	var url = args.baseUrl;
	url += "/providers/" + encodeURIComponent(args.provider);
	url += args.datasource ? ("/datasources/" + encodeURIComponent(args.datasource)) : "";
	url += "/datasets/"+ encodeURIComponent(args.dataset); 
	url += "/parameters";
		
	var content = ibm.tivoli.navmodel.Utils.getContent();
	if (args.start) {
		content.start = args.start;
	}
	if (args.count) {
		content.count = args.count;
	}
			
	return dojo.xhrGet({
		url: url,
		headers: {
			'Accept': ibm.tivoli.navmodel.Utils.getContentType(args.version, "datasetParmList")	
		},
		handleAs: "json",
		sync: args.sync,
		content: content,
		load: dojo.hitch(this, function(result, ioArgs) {
			var parms = [];			
			if (result.items) {
				var length = result.items.length;
				for (var index = 0;  index < length;  ++index) {
					var parm = new ibm.tivoli.navmodel.DatasetParameter({parameter: result.items[index], baseUrl: args.baseUrl, version: args.version});
					parms.push(parm);
				}
			}
			
		 	args.onComplete(parms, result.totalRows, result.iargs);							
		}),
		error: dojo.hitch (this, function(error, ioArgs){
			var exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "getParameters", ibm.tivoli.navmodel.DatasetUtils._trace, args.version);	
			args.onError(error, exception, ioArgs);			
		})
	});
};


ibm.tivoli.navmodel.DatasetUtils.getParameterById = function(kwArgs) {
	// summary: 
	//		Retrieve a particular parameter that can be used to create a dataset	
	// baseUrl: String
	//		base url for identifying rest service
	// provider: String
	//		provider id
	// datasource: String
	//		datasource id
	// dataset: String
	//		datasetId
	// parameter: String
	//		parameter id
	// version: Float?
	//		optional: version used to communicate with rest service (i.e. json version)
	
	var args = {
		baseUrl: null,
		provider: null,
		datasource: null,		
		dataset: null,
		parameter: null,
		version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
	};
	dojo.mixin(args, kwArgs);

	var parameter = null;
	var exception = null;
		
	// url needed to fetch parameters
	var url = args.baseUrl;
	url += "/providers/" + encodeURIComponent(args.provider);
	url += args.datasource ? ("/datasources/" + encodeURIComponent(args.datasource)) : "";
	url += "/datasets/"+ encodeURIComponent(args.dataset); 
	url += "/parameters/" + encodeURIComponent(args.parameter);
		
	dojo.xhrGet({
		url: url,
		headers: {
			'Accept': ibm.tivoli.navmodel.Utils.getContentType(args.version, "datasetParm")	
		},
		handleAs: "json",
		sync: true,
		load: dojo.hitch(this, function(result, ioArgs) {
			parameter = new ibm.tivoli.navmodel.DatasetParameter({parameter: result, baseUrl: args.baseUrl, version: args.version});										
		}),
		error: dojo.hitch (this, function(error, ioArgs){
			exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "getParameters", ibm.tivoli.navmodel.DatasetUtils._trace, args.version);				
		})
	});
	
	if (exception) {
		throw exception;
	}
	
	return parameter;
};