/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */
dojo.provide("ibm.tivoli.navmodel.Datasource"); 

// includes
dojo.require("ibm.tivoli.navmodel.ProviderUtils");

////////////////////////////////////
// Datasource
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Datasource", null, {
	// private members       
	_datasource: null,
	_baseUrl: null,
	_version: null,
	
	/*=====
	constructor_kwArgs: function(datasource, baseUrl, version) {
		// datasource: Object
		//		json datasource object
		// baseUrl: String
		//		base url for identifying rest service
		// version: Float
		//		optional: version used to communicate with rest service (i.e. json version)
	},
	=====*/
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs Provider object.
		this._baseUrl = kwArgs.baseUrl;
		this._version = kwArgs.version;
		this._datasource = kwArgs.datasource;
		if (!this._datasource) {
			this._datasource = {};
		}
	},

	getAttribute: function(key) {
		// summary:
		//		Returns general attribute for requested key
		return this._datasource[key];
	},

	setAttribute: function(key, value) {		
		// summary:
		//		Set general attribute for requested key
		this._datasource[key] = value;
	},
	
	removeAttribute: function(key) {
		// summary:
		//		Remove general attribute for requested key
		delete this._datasource[key];
	},
	
	getProviderId: function() {
		// summary:
		//		Returns the providerId of this datasource
		return this._datasource.providerId;
	},
	
	getId: function() {
		// summary:
		//		Returns the id of this datasource
		return this._datasource.id;
	},
	
	getLabel: function() {
		// summary:
		//		Returns the label of this datasource
		return this._datasource.label;
	},
	
	getDescription: function() {
		// summary:
		//		Returns the description of this datasource
		return this._datasource.description;
	},
	
	getType: function() {
		// summary:
		//		Returns the type of this datasource.
		return this._datasource.type;
	},
	
	/*=====
	getDatasets_kwArgs: function(onComplete, onError, start, count, condition, sync) {
		// onComplete: function(datasets, datasetCount, kwArgs)
		//		callback for successful fetch
		// onError: function(error, exception, ioArgs)
		//		callback for error conditions
		// start: int
		//		start index to fetch
		// count: int
		//		number of datasources to fetch; null means all
		// sort: String
		//		comma deliminated attribute string (preceded with optional +/- indicating ascending/descending)
		//		example:  label,type
		// condition: String
		//		condition expression string 
		//		example: description contains 'abcd'
		// sync: Boolean
		//		indicates if fetch is synchronous		
	},
	=====*/
	
	getDatasets: function(/*getDatasets_kwArgs*/ kwArgs) {
		var args = {
			onComplete: function(datasets, datasetCount, kwArgs) {
			},
			onError: function(error, exception, ioArgs) {
			},
			baseUrl: this._baseUrl,
			provider: this._datasource.providerId,
			datasource: this._datasource.id,		
			start: 0,
			count: null,
			sourt: null,
			condition: null,
			sync: false,
			version: this._version		
		};
		dojo.mixin(args, kwArgs);
		
		return ibm.tivoli.navmodel.ProviderUtils.getDatasets(args);
	}
});
