/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Event"); 

// includes
dojo.require("ibm.tivoli.navmodel.Exception");

////////////////////////////////////
// Event
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Event", null, {
	// private members
	_dataset: null,
	_event: null,
	_items: null,
	_links: null,
	_relationships: null,
	_props: null,
	_eventRouterStatus: null,
	
	/*=====
	constructor_kwArgs: function(dataset, event) {
		// dataset: Object
		//		the dataset that the event is for
		// event: Object
		//		raw event object
	},
	=====*/

	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs Event object.
		this._dataset = kwArgs.dataset;		
		this._event = kwArgs.event;
		this._items = [];
		this._links = [];
		this._relationships = [];
		this._props = [];
		
		// "eventRouterStatus" events originate from the EventRouter;
		// fill in this info to be consistent with other events
		if (this._event.id == "eventRouter") {
			this._event.providerId = this._dataset.getProviderId();
			this._event.datasourceId = this._dataset.getDatasourceId();
			this._event.datasetId = this._dataset.getId();
			this._event.version = this._dataset.getVersion();
		}
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	getDataset: function() {
		return this._dataset;
	},
	
	getVersion: function() {
		return this._event.version;
	},
	
	getProviderId: function() {
		return this._event.providerId;
	},
	
	getDatasourceId: function() {
		return this._event.datasourceId;
	},
	
	getDatasetId: function() {
		return this._event.datasetId;
	},
	
	getId: function() {
		return this._event.id;
	},
	
	getType: function() {
		return this._event.type;
	},
	
	getItems: function() {
		return this._items;
	},
	
	getLinks: function() {
		return this._links;
	},
	
	getRelationships: function() {
		return this._relationships;
	},
	
	getProperties: function() {
		return this._props;
	},
	
	getLinkChanged: function() {
		return this._links[0];
	},
	
	getLinkChangedReason: function() {
		return this._event.reason;
	},
	
	getLinkChangedRelationships: function() {
		return this._relationships;
	},
	
	getNewLabel: function() {
		return this._event.label;
	},
	
	getException: function() {
		return new ibm.tivoli.navmodel.Exception({exception: this._event.exception, version: this._dataset.getVersion()});
	},
	
	getEventRouterStatus: function() {
		return this._event.eventRouterStatus;
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////
	
	_addItem: function(item, props) {
		this._items.push(item);
		this._props.push(props);
	},
	
	_addLink: function(link, props) {
		this._links.push(link);
		this._props.push(props);
	},
	
	_addRelationship: function(relationship, props) {
		this._relationships.push(relationship);
		this._props.push(props);
	},
	
	_getEvent: function() {
		return this._event;
	},
	
	_getEventItems: function() {
		return this._event.items;
	},
	
	_getEventLinks: function() {
		return this._event.links;
	},
	
	_getEventLink: function() {
		return this._event.link;
	},
	
	_getEventRelationships: function() {
		return this._event.relationships;
	},
	
	_getEventNewLabel: function() {
		return this._event.label;
	},
	
	_getEventStatuses: function() {
		return this._event.status;
	},
	
	_getEventDatasetItemCount: function() {
		return this._event.datasetItemCount;
	},
	
	_getEventDatasetLinkCount: function() {
		return this._event.datasetLinkCount;
	},
	
	_getEventDatasetRelationshipCount: function() {
		return this._event.datasetRelationshipCount;
	}
});


////////////////////////////////////////////////////////////////////////
// ibm.tivoli.navmodel.ClientEvent
//
// For creating client initiated ibm.tivoli.navmodel.Event objects
////////////////////////////////////////////////////////////////////////

ibm.tivoli.navmodel.ClientEvent = {
	getChangedEvent: function(dataset) {
		// summary:
		//		Creates client-side event for type "changed"
		var eventObj = this._eventObj(dataset, "changed");		
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getItemsAddedEvent: function(dataset, items) {
		var eventObj = this._eventObj(dataset, "itemsAdded");
		eventObj.datasetItemCount = dataset.getItemCount() + items.length;		
		this._mixinItems(eventObj, items);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getItemsRemovedEvent: function(dataset, items) {
		var eventObj = this._eventObj(dataset, "itemsRemoved");
		eventObj.datasetItemCount = dataset.getItemCount() - items.length;		
		this._mixinItems(eventObj, items);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getItemStatusChangedEvent: function(dataset, items) {
		var eventObj = this._eventObj(dataset, "itemStatusChanged");
		this._mixinItems(eventObj, items);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getItemPropertiesChangedEvent: function(dataset, items) {
		var eventObj = this._eventObj(dataset, "itemPropertiesChanged");
		this._mixinItems(eventObj, items);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getLinksAddedEvent: function(dataset, links) {
		var eventObj = this._eventObj(dataset, "linksAdded");
		eventObj.datasetLinkCount = dataset.getLinkCount() + links.length;		
		this._mixinLinks(eventObj, links);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getLinksRemovedEvent: function(dataset, links) {
		var eventObj = this._eventObj(dataset, "linksRemoved");
		eventObj.datasetLinkCount = dataset.getLinkCount() - links.length;		
		this._mixinLinks(eventObj, links);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getLinkStatusChangedEvent: function(dataset, links) {
		var eventObj = this._eventObj(dataset, "linkStatusChanged");
		this._mixinLinks(eventObj, links);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getLinkPropertiesChangedEvent: function(dataset, links) {
		var eventObj = this._eventObj(dataset, "linkPropertiesChanged");
		this._mixinLinks(eventObj, links);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getLinkChangedEvent: function(dataset, link, relationships, relationshipAdded) {
		var eventObj = this._eventObj(dataset, "linkPropertiesChanged");
		eventObj.reason = (relationshipAdded ? "relationshipAdded" : "relationshipRemoved");
		eventObj.link = link.getAttributes();
		this._mixinRelationships(eventObj, links);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getLabelChangedEvent: function(dataset, newLabel) {
		var eventObj = this._eventObj(dataset, "labelChanged");
		eventObj.label = label;
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	_eventObj: function(dataset, type) {
		return {
			"id": dataset.getEventId(),
			"type": type,
			"providerId": dataset.getProviderId(),
			"datasourceId": dataset.getDatasourceId(),
			"datasetId": dataset.getId(),
			"version": dataset.getVersion()
		};		
	},
	
	_mixinItems: function(eventObj, items) {
		var itemObjs = [];
		var itemLength = items.length;
		for (var itemIndex = 0;  itemIndex < itemLength;  ++itemIndex) {
			itemObjs[itemIndex] = items[itemIndex].getAttributes();
		}
		eventObj.items = itemObjs;
	},
	
	_mixinLinks: function(eventObj, links) {
		var linkObjs = [];
		var linkLength = links.length;
		for (var linkIndex = 0;  linkIndex < linkLength;  ++linkIndex) {
			linkObjs[linkIndex] = links[linkIndex].getAttributes();
		}
		eventObj.links = linkObjs;
	},
	
	_mixinRelationships: function(eventObj, relationships) {
		var relationshipObjs = [];
		var relationshipLength = relationships.length;
		for (var relationshipIndex = 0;  relationshipIndex < relationshipLength;  ++relationshipIndex) {
			relationshipObjs[relationshipIndex] = relationships[relationshipIndex].getAttributes();
		}
		eventObj.relationships = relationshipObjs;
	}
};