/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.ItemStyle"); 
 
////////////////////////////////////
// ItemStyle
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.ItemStyle", null, {
	// private members       
	_version: null,
	_style: null,
	
	/*=====
	constructor_kwArgs: function(version, style) {
		// version: Float
		//		version used to communicate with rest service
		// style: Object
		//		json item style object
	},
	=====*/

	 constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary
		//		Constructs ItemStyle object.
		this._version = kwArgs.version;
		this._style = kwArgs.style;		
		if (!this._style) {
			this._style = {};
		}
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	getAttribute: function(key) {
		return this._style[key];
	},
	
	setAttribute: function(key, value) {
		this._style[key] = value;
	},
	
	getId: function() {
		return this._style.id;
	},
	
	setId: function(id) {
		this._style = id;
	},
	
	getType: function() {
		return this._style.type;
	},
	
	setType: function(type) {
		this._style.type = type;
	},
	
	getLabel: function() {
		if (this._style.label) {
			// string
			if (typeof this._style.label === "string") {
				return this._style.label;
			}
			// object with key/value info
			else {
				return this._style.label.value;
			}
		}
		
		return null;
	},
	
	setLabel: function(label) {
		this._style.label = label;		
	},
	
	getDescription: function() {
		if (this._style.description) {
			// string
			if (typeof this._style.description === "string") {
				return this._style.description;
			}
			// object with key/value info
			else {
				return this._style.description.value;
			}
		}
		
		return null;		
	},
	
	setDescription: function(description) {
		this._style.description = description;			
	},
	
	getSmallImage: function() {
		return this._style.smallImage;
	},
	
	setSmallImage: function(image) {
		this._style.smallImage = image;
	},
	
	getSmallImageWidth: function() {
		return this._style.smallImageWidth;
	},
	
	setSmallImageWidth: function(width) {
		this._style.smallImageWidth = width;
	},
	
	getSmallImageHeight: function() {
		return this._style.smallImageHeight;
	},
	
	setSmallImageHeight: function(height) {
		this._style.smallImageHeight = height;
	},
	
	getLargeImage: function() {
		return this._style.largeImage;
	},
	
	setLargeImage: function(image) {
		this._style.largeImage = image;
	},
	
	getLargeImageWidth: function() {
		return this._style.largeImageWidth;
	},
	
	setLargeImageWidth: function(width) {
		this._style.largeImageWidth = width;
	},
	
	getLargeImageHeight: function() {
		return this._style.largeImageHeight;
	},
	
	setLargeImageHeight: function(height) {
		this._style.largeImageHeight = height;
	},
	
	getScalarImage: function() {
		return this._style.scalarImage;
	},
	
	setScalarImage: function(image) {
		this._style.scalarImage = image;
	},
	
	getScalarImageWidth: function() {
		return this._style.scalarImageWidth;
	},
	
	setScalarImageWidth: function(width) {
		this._style.scalarImageWidth = width;
	},
	
	getScalarImageHeight: function() {
		return this._style.scalarImageHeight;
	},
	
	setScalarImageHeight: function(height) {
		this._style.scalarImageHeight = height;
	},
	
	isScalarImageVisible: function() {
		return (this._style.scalarImageVisible !== undefined ? this._style.scalarImageVisible : true);
	},
	
	setScalarImageVisible: function(visible) {
		this._style.scalarImageVisible = visible;
	},
	
	clone: function(widgetDef) {
		var args = {
			style: {},
			version: this._version
		};		
		
		if (widgetDef) {
			// start with widgetDef default style
			var widgetDefaultStyle = widgetDef.getDefaultItemStyle();
			if (widgetDefaultStyle) {
				dojo.mixin(args.style, widgetDefaultStyle);
			}
		
			// mixin our item style
			dojo.mixin(args.style, this._style);
			
			// mixin widgetDef style
			var widgetStyle = widgetDef.getItemStyle(this.getType());			
			if (widgetStyle) {
				dojo.mixin(args.style, widgetStyle);
			}
		}
		// just mixin our item style
		else {
			dojo.mixin(args.style, this._style);
		}
			
		return new ibm.tivoli.navmodel.ItemStyle(args);
	}
});
