/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Link"); 
 
// includes
dojo.require("ibm.tivoli.navmodel.Dataset");
dojo.require("ibm.tivoli.navmodel.Relationship");
dojo.require("ibm.tivoli.navmodel.Property");
dojo.require("ibm.tivoli.navmodel.Status");
dojo.require("ibm.tivoli.navmodel.LinkStyle");
dojo.require("ibm.tivoli.navmodel.Exception");
dojo.require("ibm.tivoli.navmodel.Utils");
dojo.require("ibm.tivoli.navmodel.Trace");

ibm.tivoli.navmodel.Link._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.Link");

////////////////////////////////////
// Link
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Link", null, {
	// private members       
	_link: null,
	_dataset: null,
	_relationships: null,
	_props: null,
	_propsById: null,
	_nullPropIds: null,
	_allProps: false,
	_allMetadata: false,
	_status: null,
	_style: null,
	_defaultTask: null,
	_expiration: null,
	_trace: ibm.tivoli.navmodel.Link._trace,
	
	/*=====
	constructor_kwArgs: function(link, dataset, propIds, allProperties, allMetadata, expiration) {
		// link: Object
		//		json link object
		// dataset: Object
		//		ibm.tivoli.navmodel.Dataset that owns this link
		// propIds: String[]
		//		optional array indicating the property Id's that were requested with 
		//		the item; so it can be determined if any properties are null
		// allProperties: Boolean
		//		optional: indicates if the link already has "all" properties
		// allMetadata: Boolean
		//		optional: indicates if link properties already have metadata
		// expiration: Long
		//		optional: expiration time;  will use (current time + interval) if not supplied
	},
	=====*/

	 constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs Link object.
		this._link = kwArgs.link;
		if (this._link) {
			// optimization of json on the server can leave out the "id"
			// but we know the default format
			if (!this._link.id) {
				if (this._link.item1 < this._link.item2) {
					this._link.id = this._link.item1 + "_" + this._link.item2;
				}
				else {
					this._link.id = this._link.item2 + "_" + this._link.item1;
				}
			}
		}
		else {
			this._link = {};
		}
		
		this._dataset = kwArgs.dataset;
		this._props = [];
		this._propsById = {};
		this._nullPropIds = {};
		this._allProps = kwArgs.allProperties;		
		this._allMetadata = kwArgs.allMetadata;
		
		if (this._link.properties) {
			var propsLength = this._link.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var prop = new ibm.tivoli.navmodel.Property({property: this._link.properties[propIndex], dataset: this._dataset, objectType: "link"});
				this._props[propIndex] = prop;
				this._propsById[prop.getId()] = prop;
			}
			
			// see if we should mark any properties as null because they were not returned
			if (kwArgs.propIds) {
				propsLength = kwArgs.propIds.length;
				for (propIndex = 0;  propIndex < propsLength;  ++propIndex) {
					var propId = kwArgs.propIds[propIndex];				
					if (!this._propsById[propId]) {
						this._nullPropIds[propId] = true;
					}
				}
			}
		}
				
		// expiration was supplied
		if (kwArgs.expiration) {
			this._expiration = kwArgs.expiration;
		}
		// reset to current time + interval
		else {
			this.resetExpiration();
		}
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	mixin: function(link) {		
		// summary:
		//		Mixin properties and status from this Link object
		if (link && link !== this) {
			this.removeAttribute("style");
			this._style = null;
			
			this.mixinAttributes(link.getAttributes());
			this.mixinProperties(link._props, link._nullPropIds);
			this._status = link.getStatuses({widgetDef: null});
			this._defaultTask = link.getDefaultTask();
			this._relationships = link.getRelationships();
			this._allProps = (this._allProps || link._allProps);
			this._allMetadata = (link._allProps && link._allMetadata);			
		}		
	},
	
	getDataset: function() {		
		// summary:
		//		Returns the dataset that owns the link
		return this._dataset;
	},

	getObjectType: function() {		
		// summary:
		//		Indicate that this object is a "link"
		return "link";
	},

	getAttribute: function(key) {		
		// summary:
		//		Returns general attribute for requested key
		return this._link[key];
	},

	setAttribute: function(key, value) {		
		// summary:
		//		Set general attribute for requested key
		this._link[key] = value;
	},

	removeAttribute: function(key) {
		// summary:
		//		Remove general attribute for requested key
		delete this._link[key];
	},
	
	getAttributes: function() {		
		// summary:
		//		Get all general attributes for the link
		var link = dojo.clone(this._link);
		
		// make sure "properties" are current as we may have received events
		if (this._props && this._props.length > 0) {
			link.properties = [];
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				link.properties[propIndex] = this._props[propIndex].getAttributes();
			}
		}
		
		return link;
	},

	mixinAttributes: function(attr) {		
		// summary:
		//		Mixin general attributes for the link
		dojo.mixin(this._link, attr);		
	},

	getUri: function() {
		if (!this._link.uri) {
			this._link.uri = this._dataset.getAttribute("uri") + "/links/" + encodeURIComponent(this.getId());
		}
	
		return this._dataset.getArg("baseUrl") + this._link.uri;
	},
	
	getId: function() {		
		// summary:
		//		Returns link id
		return this._link.id;
	},

	setId: function(id) {		
		// summary:
		//		Set the link id
		this._link.id = id;		
	},

	getType: function() {		
		// summary:
		//		Returns the type of link
		return this._link.type;
	},

	setType: function(type) {		
		// summary:
		//		Set the link type
		this._link.type = type;		
	},

	getLabel: function() {		
		// summary:
		//		Returns globalized label for the link
		return this._link.label;
	},

	setLabel: function(label) {		
		// summary:
		//		Set the globalized label for the link
		this._link.label = label;		
	},

	getTooltip: function() {		
		// summary:
		//		Returns globalized tooltip for the link
		return this._link.tooltip;
	},

	setTooltip: function(tooltip) {		
		// summary:
		//		Set the globalized tooltip for the link
		this._link.tooltip = tooltip;
	},
	
	isContainment: function() {		
		// summary:
		//		Returns true if this link represents containment instead of
		//		connection via normal lines
		return (this._link.containment ? this._link.containment : false);
	},
	
	setContainment: function(isContainment) {		
		// summary:
		//		Set the value indicating containment
		this._link.containment = isContainment;
	},

	getDirection: function() {		
		// summary:
		//		Returns the direction associated with the link
		//		
		//		|	values:	"1_2"   	means direction of arrow flows from item1 to item2
		//		|		"2_1"		means direction of arrow flows from item2 to item1
		//		|		"both"	means that arrow flows in both directions
		return (this._link.direction ? this._link.direction : "1_2");
	},

	setDirection: function(direction) {		
		// summary:
		//		Set the direction associated with the link
		//		
		//		|	values:	"1_2"   	means direction of arrow flows from item1 to item2
		//		|		"2_1"		means direction of arrow flows from item2 to item1
		//		|		"both"	means that arrow flows in both directions
		this._link.direction = direction;
	},
	
	/*=====
	getStatuses_kwArgs: function(widgetDef) {
		// widgetDef: Object
		//		widget definition can override attributes
	},
	=====*/

	getStatuses: function(/*getStatuses_kwArgs*/ kwArgs) {		
		// summary:
		//		Returns ibm.tivoli.navmodel.Status[] objects indicating the link status values
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		if (!this._status && this._link.status) {
			this._status = [];
			var statusLength = this._link.status.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				if (this._link.status.label && this._link.status.image) {
					this._status.push(new ibm.tivoli.navmodel.Status({status: this._link.status[statusIndex], version: this._version()}));				
				}				
				else {
					var statusArgs = {
						type: "link",
						statusId: this._link.status[statusIndex].id,
						statusValue: this._link.status[statusIndex].value				
					};
				
					var statusObj = this._dataset.getStatus(statusArgs);
					if (statusObj) {				
						this._status.push(statusObj);
					}
					else {
						this._trace.error("Could not locate status with id=" + statusArgs.statusId + " value=" + statusArgs.statusValue, "getStatuses");
					}
				}
			}			
		}
		
		// merge widget definition status
		var status = this._status;
		if (status && args.widgetDef) {
			status = this._status.slice(0);
			var statusLength2 = this._status.length;
			for (var statusIndex2 = 0;  statusIndex2 < statusLength2;  ++statusIndex2) {
				status[statusIndex2] = this._status[statusIndex2].clone(args.widgetDef, "link");
			}
		}					
		
		return status;		
	},

	setStatuses: function(statuses) {		
		// summary:
		//		Set ibm.tivoli.navmodel.Status[] objects indicating the link status values
		this._status = statuses;
	},


	getItem1: function() {
		// summary:
		//		Returns the item1 involved in the link		
		var item = this._dataset.getItem({itemId: this.getItem1Id()});		
		return item;
	},
	
	getItem1Id: function() {		
		// summary:
		//		Returns the id of item1 involved in the link;  this value can be undefined
		//		if the link is between two different datasets and the connection point
		//		is to the dataset and not a link within it
		return this._link.item1;
	},

	setItem1Id: function(item1) {		
		// summary:
		//		Set the id of item1 involved in the link
		this._link.item1 = item1;
	},

	getItem2: function() {
		// summary:
		//		Returns the item2 involved in the link		
		var item = this._dataset.getItem({itemId: this.getItem2Id()});	
		return item;
	},
		
	getItem2Id: function() {		
		// summary:
		//		Returns the id of item2 involved in the link;  this value can be undefined
		//		if the link is between two different datasets and the connection point
		//		is to the dataset and not a link within it
		return this._link.item2;
	},

	setItem2Id: function(item2) {		
		// summary:
		//		Set the id of item1 involved in the link
		this._link.item2 = item2;
	},

	getRelationshipCount: function() {		
		// summary:
		//		Returns the number of relationships involved in the link
		return this.getRelationships().length;
	},

	getRelationship: function(relationshipId) {		
		// summary:
		//		Returns requested relationship if it exists
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		var relationships = this.getRelationships();
		if (relationships) {
			var relLength = relationships.length;
			for (var relIndex = 0;  relIndex < relLength;  ++relIndex) {
				if (relationships[relIndex].getId() === relationshipId) {
					return relationships[relIndex];
				}
			}
		}
		
		return null;
	},

	getRelationships: function() {		
		// summary:
		//		Returns the ibm.tivoli.navmodel.Relationship[] objects that are involved in the link
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		if (!this._relationships) {
			var direction = this.getDirection();
			this._relationships = [];
			var relationshipsLength = this._link.relationships.length;
			for (var relIndex = 0;  relIndex < relationshipsLength;  ++relIndex) {			
				var relationshipObj = this._link.relationships[relIndex];
				
				if (!relationshipObj.linkId) {
					relationshipObj.linkId = this.getId();
				}
				
				if (!relationshipObj.label) {
					relationshipObj.label = this.getLabel();
				}
				
				if (!relationshipObj.type) {
					relationshipObj.type = this.getType();
				}
				
				if (!relationshipObj.sourceItem) {
					switch(direction) {
						case "1_2":
						case "none":
							if (this.getItem1Id()) {
								relationshipObj.sourceItem = this.getItem1Id();
							}
							break;
						case "2_1":
							if (this.getItem2Id()) {
								relationshipObj.sourceItem = this.getItem2Id();
							}
					}	
				}
				
				if (!relationshipObj.targetItem) {
					switch(direction) {
						case "1_2":
						case "none":
							if (this.getItem2Id()) {
								relationshipObj.targetItem = this.getItem2Id();
							}
							break;
						case "2_1":
							if (this.getItem1Id()) {
								relationshipObj.targetItem = this.getItem1Id();
							}
					}					
				}
				
				var relationship = new ibm.tivoli.navmodel.Relationship({relationship: relationshipObj, dataset: this._dataset});
				this._relationships[relIndex] = this._dataset._cacheRelationship(relationship);
			}
		}

		return this._relationships;
	},

	setRelationships: function(relationships) {		
		// summary:
		//		Set the ibm.tivoli.navmodel.Relationship[] objects that are involved in the link
		this._relationships = relationships;
	},

	supportsProperties: function() {		
		// summary:
		//		Indicates if "Properties" is supported for the link
		return (this._link.supportsProperties !== undefined ? this._link.supportsProperties : true);
	},

	setSupportsProperties: function(supportsProperties) {		
		// summary:
		//		Set whether link  supports "Properties"
		this._link.supportsProperties = supportsProperties;
	},

	hasCachedMetadata: function() {		
		// summary:
		//		Returns true if link has all properties and associated metadata cached
		return this._allMetadata;
	},

	hasAllCachedProperties: function() {		
		// summary:
		//		Returns true if link has all proprties cached
		return this._allProps;
	},

	hasCachedProperty: function(/*String*/ propId) {
		// summary:
		//		Returns true if we have the requested property in the cache
		// propId
		//		property id
		return (this._propsById[propId] ? true : false);
	},
	
	hasCachedProperties: function(/*String[]*/ properties) {		
		// summary:
		//		Returns true if we have all the requested properties in the cache
		// properties:
		//		array of property ids
		if (properties) {
			var propsLength = properties.length;
			
			// asking if we have "all" properties
			if (propsLength === 1 && properties[0] === "all") {
				return this._allProps;
			}
			
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				if (!this._propsById[properties[propIndex]]) {
					return false;
				}
			}
		}
		
		return true;
	},

	getCachedPropertyIds: function(expandAll) {		
		// summary:
		//		Returns array of property Ids currently cached in the item
		var propIds = [];
		
		// have all the properties
		if (!expandAll && this._allProps) {
			propIds.push("all");
		}
		// get individual property ids	
		else if (this._props) {
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				propIds.push(this._props[propIndex].getId());
			}
		}
		return propIds;
	},
	
	/*=====
	properties_onComplete: function(properties, kwArgs){},
	=====*/
	
	/*=====
	_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	getProperties_kwArgs: function(onComplete, onError, properties, metadata, sync) {
		// onComplete: properties_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		property ids to fetch or "all"
		// metadata: Boolean
		//		indicates whether metadata is required
		// sync: Boolean
		//		indicates if call is synchronous or asynchronous
	},
	=====*/

	getProperties: function(/*getProperties_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously returns properties for the link
		var args = {                
			onComplete: function(properties, args){
			},
			onError: function(error, exception, ioArgs){				
			},                
			properties: ["all"],
			metadata: false,
			sync: false
		};   
		dojo.mixin(args, kwArgs);         
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		// fetch all properties
		if (args.properties.length === 1 && args.properties[0] === "all") {
			if (this._allProps && (!args.metadata || this._allMetadata)) {				
				if (args.sync) {
					args.onComplete(this._props, args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, this._props, args), 0);
				}		
			}
			else {
				this._fetchProperties(args);
			}
		}		
		// fetch selective properties
		else {
			// look up properties in cache
			var props = [];
			var propsLength = args.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var propId = args.properties[propIndex];
				var prop = this._propsById[propId];				
				if (prop && (!args.metadata || prop.hasMetadata())) {
					props[propIndex] = prop;					
				}
				else {
					break;
				}
			}
			
			// if we found all properties in cache then return them
			if (props.length === propsLength) {
				if (args.sync) {
					args.onComplete(props, args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, props, args), 0);
				}
			}
			// otherwise we need to fetch
			else {
				this._fetchProperties(args);
			}
		}		
	},
	
	setProperties: function(properties) {		
		// summary:
		//		Set the ibm.tivoli.navmodel.Property[] objects for the link
		//		
		//		Note: client side only operation;  no request to rest service
		this._propsById = {};
		this._nullPropIds = {};
		this._props = [];
		
		if (properties) {
			var propsLength = properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var prop = properties[propIndex];
				this._propsById[prop.getId()] = prop;
			}
		}		
	},

	mixinProperties: function(properties, nullPropIds) {		
		// summary:
		//		Mixin the ibm.tivoli.navmodel.Property[] objects for the link
		//		
		//		Note: client side only operation;  no request to rest service
		
		// mixin property objects
		if (properties) {
			var propsLength = properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				this.mixinProperty(properties[propIndex]);
			}
		}
		
		// mixin any propIds that were null
		if (nullPropIds) {
			for (var propId in nullPropIds) {
				if (nullPropIds.hasOwnProperty(propId)) {
					this._nullPropIds[propId] = true;
				}
			}
		}
	},

	 getProperty: function(/*String*/ propId) {		
		// summary:
		//		Synchronously returns requested property
		// propId:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		// check prop cache
		var prop = this._propsById[propId];
			
		if (!prop) {
			// we have asked for this property before and it wasn't found
			if (this._nullPropIds[propId]) {
				return null;
			}
			
			// check default properties on link	
			if (this._link.properties) {
				var propsLength = this._link.properties.length;
				for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
					var rawProp = this._link.properties[propIndex];
					if (rawProp.id === propId) {
						prop = new ibm.tivoli.navmodel.Property({property: rawProp, dataset: this._dataset, objectType: "link"});
						
						// cache property
						this._propsById[propId] = prop;
						break;
					}
				}				
			}
		}	
		
		// fetch it
		if (!prop && !this._allProps) {
			prop = this._fetchProperty(propId);
			
			// cache property
			if (prop) {
				this._propsById[propId] = prop;
			}
		}		
		
		return prop;
	},

	setProperty: function(property) {		
		// summary:
		//		Set ibm.tivoli.navmodel.Property for the item
		//		
		//		Note: client side only operation;  no request to rest service
		var found = this._propsById[property.getId()];		
		if (found) {
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				if (this._props[propIndex].getId() === property.getId()) {
					this._props[propIndex] = property;
					break;
				}
			}
		}
		else {
			this._props.push(property);
		}		
		
		this._propsById[property.getId()] = property;
	},

	mixinProperty: function(property) {		
		// summary:
		//		Mixin ibm.tivoli.navmodel.Property for the link
		//		
		//		Note: client side only operation;  no request to rest service
		var prop = this._propsById[property.getId()];
		if (prop) {
			prop.mixin(property);
		}
		else {
			this._props.push(property);
			this._propsById[property.getId()] = property;
		}	
	},
	
	/*=====
	_onComplete: function(kwArgs){},
	=====*/
	
	/*=====
	editProperties_kwArgs: function(onComplete, onError, properties) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		array of property ids to set
	},
	=====*/

	editProperties: function(/*editProperties_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously modify properties for the link
		var args = {                
			onComplete: function(properties, args){
			},
			onError: function(error, exception, ioArgs){				
			},                
			properties: null
		};
		dojo.mixin(args, kwArgs);
		
		if (args.properties && args.properties.length > 0) {
			var link = dojo.clone(this._link);
			link.properties = [];
			
			var propsLength = args.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				link.properties[propIndex] = args.properties[propIndex].getAttributes();
			}
			
			var json = dojo.toJson(link, false);
			
			dojo.xhrPost ({
				url: ibm.tivoli.navmodel.Utils.getParamUrl(this.getUri(), this._dataset),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
					'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "linkView"),
					'Content-Length': json.length
				},
				handleAs: "json",
				postData: json,
				sync: false,			
				load: dojo.hitch(this, function(result, ioArgs) {
					args.onComplete(args);						
				}),
				error: dojo.hitch(function(error, ioArgs) {
					exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "setProperties", this._trace, this._version());				               
				})
			});	
		}
		// nothing to set
		else {
			setTimeout(dojo.hitch(this, args.onComplete, args), 0);
		}		
	},

	 editProperty: function(/*String*/ property) {		
		// summary:
		//		Synchronously modify property for the link
		// propId:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var exception = null;
		
		if (property) {
			var link = dojo.clone(this._link);
			link.properties = [property.getAttributes()];
			
			var json = dojo.toJson(link, false);
			
			dojo.xhrPost ({
				url: ibm.tivoli.navmodel.Utils.getParamUrl(this.getUri(), this._dataset),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
					'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "linkView"),
					'Content-Length': json.length
				},
				handleAs: "json",
				postData: json,
				sync: true,			
				load: dojo.hitch(this, function(result, ioArgs) {						
				}),
				error: dojo.hitch(this, function(error, ioArgs) {
					exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "setProperty", this._trace, this._version());									              
				})
			});				
		}
		
		if (exception) {
			throw exception;
		}
	},

	hasOwnLinkStyle: function() {		
		// summary:
		//		Returns true if the link has a LinkStyle that overrides the default
		//		link style for the link type  (i.e. different than the one retrieved
		//		from the dataset)
		return (this._link.style ? true: false);
	},
	
	/*=====
	getLinkStyle_kwArgs: function(widgetDef) {
		// widgetDef: Object
		//		widget definition can override
	},
	=====*/
	
	getLinkStyle: function(/*getLinkStyle_kwArgs*/ kwArgs) {		
		// summary:
		//		Returns the ibm.tivoli.navmodel.LinkStyle for this link
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		if (!this._style) {
			if (this._link.style) {
				this._style = new ibm.tivoli.navmodel.LinkStyle({style: this._link.style, version: this._version()});
			}
			else {
				this._style = this._dataset.getLinkStyle({linkType: this.getType(), widgetDef: null});
			}
		}
		
		// merge widget definition styles 
		var style = this._style;
		if (style && args.widgetDef) {
			// clone and merge widget definition attributes				
			if (style) {
				style = style.clone(args.widgetDef, "link");
			}
			// unknown style; see if it exists in widget definition
			else {
				var widgetStyle = args.widgetDef.getLinkStyle(this.getType());
				if (widgetStyle) {
					style = this._dataset._createLinkStyle(args.widgetDef, widgetStyle, "link");
				}
			}							
		}
		
		return style;
	},

	setLinkStyle: function(style) {		
		// summary:
		//		Set link style for the link
		this._style = style;
	},

	getDefaultTask: function() {		
		// summary:
		//		Returns optional default task associated with the link
		if (!this._defaultTask && this._link.defaultTask) {
			this._defaultTask = new ibm.tivoli.navmodel.Task({task: this._link.defaultTask, dataset: this._dataset});
		}
		
		return this._defaultTask;
	},

	setDefaultTask: function(defaultTask) {		
		// summary:
		//		Set optional default task associated with the link
		this._defaultTask = defaultTask;
	},

	isExpired: function() {		
		// summary:
		//		Returns true if the item is currently expired
		return (this._expiration ? false : true);
	},

	checkExpiration: function(nowTime) {		
		// summary:
		//		Check the expiration time on the item; return true if expired
		var expired = false;
		
		// already set to null from a previous check
		if (!this._expiration) {
			expired = true;
		}
		// check agaist the time to see if expired
		else if (nowTime >= this._expiration) {
			this._expiration = null;
			expired = true;
		}
		
		return expired;		
	},

	resetExpiration: function() {		
		// summary:
		//		Rest the expiration time for this item
		if (this._dataset.cacheExpires()) {
			this._expiration = this._dataset.getNewExpirationTime();
		}
		else {
			this._expiration = -1;
		}
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////

	_version: function() {		
		// summary:
		//		Returns rest version
		return this._dataset.getVersion();
	},

	_validateExpiration: function(kwArgs) {		
		// summary:
		//		If item is expired; mixin with a current item from the dataset cache
		if (this._dataset.cacheExpires()) {
			// expired
			if (this.isExpired()) {
				var properties = null;
				var metadata = false;
				if (kwArgs) {
					properties = kwArgs.properties;
					metadata = kwArgs.metadata;
				}			
				if (!properties) {
					properties = this.getCachedPropertyIds();
				}
		
				var link = this._dataset.getLink({linkId: this.getId(), properties: properties, metadata: metadata});
				if (link) {
					this.mixin(link);
				}			
			}
			// NOT expired; reset expiration since touched
			else {
				this.resetExpiration();
			}
		}
	},	
        
	/*=====
	fetchProperties_kwArgs: function(onComplete, onError, properties, metadata, sync) {
		// onComplete: properties_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		property ids to fetch or "all"
		// metadata: Boolean
		//		indicates whether metadata is required
		// sync: Boolean
		//		indicates if call is synchronous or asynchronous	
	},
	=====*/
	
	_fetchProperties: function(/*fetchProperties_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously returns properties for the link
		this._props = [];
		this._allProps = false;
		this._allMetadata = false;
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this._dataset);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		content.optimize = true;
		content.metadata = kwArgs.metadata;
		
		// are we fetching "all" properties
		var fetchAll = (kwArgs.properties.length === 1 && kwArgs.properties[0] === "all");
		
		return dojo.xhrGet({
			url: this.getUri(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "linkView")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var props = [];
				var rawProps = result.properties;
				var rawPropsLength = rawProps.length;
				for (var propIndex = 0;  propIndex < rawPropsLength;  ++propIndex) {
					var prop = new ibm.tivoli.navmodel.Property({property: rawProps[propIndex], dataset: this._dataset, objectType: "link"});
					props[propIndex] = prop;
					
					if (fetchAll) {
						this._props[propIndex] = prop;
					}
					
					this._propsById[prop.getId()] = prop;
				}
				
				if (fetchAll) {
					this._allProps = true;
					this._allMetadata = kwArgs.metadata;
				}
				
				kwArgs.onComplete(props, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_fetchProperties", this._trace, this._version()), ioArgs);			
			})
		});		
	},

	_fetchProperty: function(/*String*/ propId) {		
		// summary:
		//		Synchronously returns requested property
		// propId:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var exception = null;
		var prop = null;
			
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this._dataset);
		content.optimize = true;
		content.properties = propId;
		
		dojo.xhrGet({
			url: this.getUri(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "linkView")
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				if (result.properties && result.properties.length > 0) {
					prop = new ibm.tivoli.navmodel.Property({property: result.properties[0], dataset: this._dataset, objectType: "link"});
				
					// cache property
					this._propsById[propId] = prop;
				}
				// property was not found; remember so we don't ask again
				else {
					this._nullPropIds[propId] = true;
				}
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_fetchProperty", this._trace, this._version());				
			})
		});
		
		if (exception) {
			throw exception;
		}
		
		return prop;
	}
});