/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.LinkStyle"); 
 
////////////////////////////////////
// LinkStyle
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.LinkStyle", null, {
	// private members       
	_version: null,
	_style: null,
	
	/*=====
	constructor_kwArgs: function(version, style) {
		// version: Float
		//		version used to communicate with rest service
		// style: Object
		//		json item style object
	},
	=====*/
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary
		//		Constructs LinkStyle object.
		this._version = kwArgs.version;
		this._style = kwArgs.style;		
		if (!this._style) {
			this._style = {};
		}	
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	getAttribute: function(key) {
		return this._style[key];
	},
	
	setAttribute: function(key, value) {
		this._style[key] = value;
	},
	
	getId: function() {
		return this._style.id;
	},
	
	setId: function(id) {
		this._style = id;
	},
	
	getType: function() {
		return this._style.type;
	},
	
	setType: function(type) {
		this._style.type = type;
	},
	
	getLabel: function() {
		if (this._style.label) {
			// string
			if (typeof this._style.label === "string") {
				return this._style.label;
			}
			// object with key/value info
			else {
				return this._style.label.value;
			}
		}
		
		return null;
	},
	
	setLabel: function(label) {
		this._style.label = label;		
	},
	
	getLineStyle: function() {
		return this._style.lineStyle;
	},
	
	setLineStyle: function(lineStyle) {
		this._style.lineStyle = lineStyle;
	},
	
	getLineWidth: function() {
		return this._style.lineWidth;
	},
	
	setLineWidth: function(width) {
		this._style.lineWidth = width;
	},
	
	getLineColor: function() {
		return this._style.lineColor;
	},
	
	setLineColor: function(color) {
		this._style.lineColor = color;
	},
	
	getArrowColor: function() {
		return this._style.arrowColor;
	},

	setArrowColor: function(color) {
		this._style.arrowColor = color;
	},
	
	isLabelVisible: function() {
		return (this._style.labelVisible !== undefined ? this._style.labelVisible : false);
	},
	
	setLabelVisible: function(visible) {
		this._style.labelVisible = visible;
	},
	
	isLabelAutoRotate: function() {
		return (this._style.labelAutoRotate !== undefined ? this._style.labelAutoRotate : false);
	},
	
	setLabelAutoRotate: function(autoRotate) {
		this._style.labelAutoRotate = autoRotate;
	},

	getLabelParallelAlignment: function() {
		return (this._style.labelParallelAlignment !== undefined ? this._style.labelParallelAlignment : "middle");
	},
	
	setLabelParallelAlignment: function(parallelAlignment) {
		this._style.labelParallelAlignment = parallelAlignment;
	},
	
	getLabelOrthogonalAlignment: function() {
		return (this._style.labelOrthogonalAlignment !== undefined ? this._style.labelOrthogonalAlignment : "left");
	},
	
	setLabelOrthogonalAlignment: function(orthogonalAlignment) {
		this._style.labelOrthogonalAlignment = orthogonalAlignment;
	},

	getLabelDistance: function() {
		return (this._style.labelDistance !== undefined ? this._style.labelDistance : 5);
	},
	
	setLabelDistance: function(distance) {
		this._style.labelDistance = distance;
	},

	getLabelPositionType: function() {
		return (this._style.labelPositionType !== undefined ? this._style.labelPositionType : "relative");
	},
	
	setLabelPositionType: function(positionType) {
		this._style.labelPositionType = positionType;
	},

	getLabelPosition: function() {
		return (this._style.labelPosition !== undefined ? this._style.labelPosition : 0.5);
	},
	
	setLabelPosition: function(position) {
		this._style.labelPosition = position;
	},
	
	clone: function(widgetDef, objType) {
		var args = {
			style: {},
			version: this._version
		};		
		
		if (widgetDef) {
			// start with widgetDef default style
			var widgetDefaultStyle = null;
			switch(objType) {
				case "link":
					widgetDefaultStyle = widgetDef.getDefaultLinkStyle();
					break;
				case "relationship":
					widgetDefaultStyle = widgetDef.getDefaultRelationshipStyle();
			}
			if (widgetDefaultStyle) {
				dojo.mixin(args.style, widgetDefaultStyle);
			}
		
			// mixin our link style
			dojo.mixin(args.style, this._style);
			
			// mixin widgetDef style
			var widgetStyle = null;
			switch(objType) {
				case "link":
					widgetStyle = widgetDef.getLinkStyle(this.getType());
					break;
				case "relationship":
					widgetStyle = widgetDef.getRelationshipStyle(this.getType());
			}		
			if (widgetStyle) {
				dojo.mixin(args.style, widgetStyle);
			}
		}
		// just mixin our link style
		else {
			dojo.mixin(args.style, this._style);
		}
			
		return new ibm.tivoli.navmodel.LinkStyle(args);
	}
});
