/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */
dojo.provide("ibm.tivoli.navmodel.Provider"); 

// includes
dojo.require("ibm.tivoli.navmodel.ProviderUtils");

////////////////////////////////////
// Provider
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Provider", null, {
	// private members       
	_provider: null,
	_baseUrl: null,
	_version: null,
	
	/*=====
	constructor_kwArgs: function(provider, baseUrl, version) {
		// provider: Object
		//		json provider object
		// baseUrl: String
		//		base url for identifying rest service
		// version: Float
		//		optional: version used to communicate with rest service (i.e. json version)
	},
	=====*/
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs Provider object.
		this._baseUrl = kwArgs.baseUrl;
		this._version = kwArgs.version;
		this._provider = kwArgs.provider;
		if (!this._provider) {
			this._provider = {};
		}
	},

	getAttributes: function() {
		// summary:
		//		Get all general attributes for the provider
		return dojo.clone(this._provider);
	},
	
	getAttribute: function(key) {
		// summary:
		//		Returns general attribute for requested key
		return this._provider[key];
	},

	setAttribute: function(key, value) {		
		// summary:
		//		Set general attribute for requested key
		this._provider[key] = value;
	},
	
	removeAttribute: function(key) {
		// summary:
		//		Remove general attribute for requested key
		delete this._provider[key];
	},
	
	isRemote: function() {
		// summary:
		//		Returns true if the provider is remote; false
		//		if the provider is local to the server
		return this._provider.remote;
	},	
	
	getMSSName: function() {
		// summary:
		//		Returns the MSSName of this provider.
		return this._provider.MSSName;
	},	
		
	getId: function() {
		// summary:
		//		Returns the id of this provider
		return this._provider.id;
	},
	
	setId: function(id) {
		// summary:
		//		Sets the provider id
		this._provider.id = id;
	},
	
	getExternalProviderId: function() {
		// summary:
		//		Returns the externalProviderId (i.e. the id that the provider
		//		is known by on the remote server)
		return this._provider.externalProviderId;
	},
	
	setExternalProviderId: function(externalProviderId) {
		// summary:
		//		Sets the externalProviderId (i.e. the id that the provider
		//		is known by on the remote server)
		this._provider.externalProviderId = externalProviderId;
	},
	
	getLabel: function() {
		// summary:
		//		Returns the label of this provider
		return this._provider.label;
	},
	
	setLabel: function(label) {
		// summary:
		//		Sets the provider label
		this._provider.label = label;
	},
	
	getDescription: function() {
		// summary:
		//		Returns the description of this provider
		return this._provider.description;
	},
	
	setDescription: function(description) {
		// summary:
		//		Sets the provider description
		this._provider.description = description;
	},
	
	getType: function() {
		// summary:
		//		Returns the type of this provider.
		return this._provider.type;
	},
	
	setType: function(type) {
		// summary:
		//		Sets the provider type
		this._provider.type = type;
	},
	
	getAuthUser: function() {
		// summary:
		//		Returns optional authUser attribute
		return this._provider.authUser;
	},
	
	setAuthUser: function(authUser) {
		// summary:
		//		Sets optional authUser attribute
		this._provider.authUser = authUser;
	},
	
	setAuthPassword: function(authPassword) {
		// summary:
		//		Sets optional authPassword attribute
		this._provider.authPassword = authPassword;
	},
	
	getBaseUrl: function() {
		// summary:
		//		Returns baseUrl attribute
		return this._provider.baseUrl;
	},
	
	setBaseUrl: function(baseUrl) {
		// summary:
		//		Sets baseUrl attribute
		this._provider.baseUrl = baseUrl;
	},
	
	getAuthUrl: function() {
		// summary:
		//		Returns authUrl attribute
		return this._provider.authUrl;
	},
	
	setAuthUrl: function(authUrl) {
		// summary:
		//		Sets baseUrl attribute
		this._provider.authUrl = authUrl;
	},
	
	getProxyHost: function() {
		// summary:
		//		Returns proxyHost attribute
		return this._provider.proxyHost;
	},
	
	getProxyPort: function() {
		// summary:
		//		Returns proxyPort attribute
		return this._provider.proxyPort;
	},
	
	setProxyHostAndPort: function(proxyHost, proxyPort) {
		// summary:
		//		Sets proxyHost and proxyPort attributes
		this._provider.proxyHost = proxyHost;
		this._provider.proxyPort = proxyPort;
	},

	getClientFacingHost: function() {
		// summary:
		//		Returns clientFacingHost attribute
		return this._provider.clientFacingHost;
	},
	
	getClientFacingPort: function() {
		// summary:
		//		Returns clientFacingPort attribute
		return this._provider.clientFacingPort;
	},

	setClientFacingHostAndPort: function(clientFacingHost, clientFacingPort) {
		// summary:
		//		Sets clientFacingHost and clientFacingPort attributes
		this._provider.clientFacingHost = clientFacingHost;
		this._provider.clientFacingPort = clientFacingPort;
	},
	 
	isUseFIPS: function() {
		// summary:
		//		Returns useFIPS attribute
		return (this._provider.useFIPS !== undefined ? this._provider.useFIPS : false);
	},
	
	setUseFIPS: function(useFIPS) {
		// summary:
		//		Sets useFIPS attribute
		this._provider.useFIPS = useFIPS;
	},
	
	isSSO: function() {
		// summary:
		//		Returns sso attribute
		return (this._provider.sso !== undefined ? this._provider.sso : false);
	},
	
	setSSO: function(sso) {
		// summary:
		//		Sets sso attribute
		this._provider.sso = sso;
	},
    
	/*=====
	getDatasources_kwArgs: function(onComplete, onError, start, count, condition, sync) {
		// onComplete: function(datasources, datasourceCount, kwArgs)
		//		callback for successful fetch
		// onError: function(error, exception, ioArgs)
		//		callback for error conditions
		// start: int
		//		start index to fetch
		// count: int
		//		number of datasources to fetch; null means all
		// sort: String
		//		comma deliminated attribute string (preceded with optional +/- indicating ascending/descending)
		//		example:  label,type
		// condition: String
		//		condition expression string 
		//		example: description contains 'abcd'
		// sync: Boolean
		//		indicates if fetch is synchronous		
	},
	=====*/
	
	getDatasources: function(/*getDatasources_kwArgs*/ kwArgs) {
		var args = {
			onComplete: function(datasources, datasourceCount, kwArgs) {
			},
			onError: function(error, exception, ioArgs) {
			},
			baseUrl: this._baseUrl,
			provider: this._provider.id,			
			start: 0,
			count: null,
			sourt: null,
			condition: null,
			sync: false,
			version: this._version		
		};
		dojo.mixin(args, kwArgs);
		
		return ibm.tivoli.navmodel.ProviderUtils.getDatasources(args);
	},
	
	/*=====
	getDatasets_kwArgs: function(onComplete, onError, datasource, start, count, condition, sync) {
		// onComplete: function(datasets, datasetCount, kwArgs)
		//		callback for successful fetch
		// onError: function(error, exception, ioArgs)
		//		callback for error conditions
		// datasource: String
		// 		datasource id
		// start: int
		//		start index to fetch
		// count: int
		//		number of datasources to fetch; null means all
		// sort: String
		//		comma deliminated attribute string (preceded with optional +/- indicating ascending/descending)
		//		example:  label,type
		// condition: String
		//		condition expression string 
		//		example: description contains 'abcd'
		// sync: Boolean
		//		indicates if fetch is synchronous		
	},
	=====*/
	
	getDatasets: function(/*getDatasets_kwArgs*/ kwArgs) {
		var args = {
			onComplete: function(datasets, datasetCount, kwArgs) {
			},
			onError: function(error, exception, ioArgs) {
			},
			baseUrl: this._baseUrl,
			provider: this._provider.id,
			datasource: null,		
			start: 0,
			count: null,
			sourt: null,
			condition: null,
			sync: false,
			version: this._version	
		};
		dojo.mixin(args, kwArgs);
		
		return ibm.tivoli.navmodel.ProviderUtils.getDatasets(args);
	}
});
