/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Selection"); 
 
////////////////////////////////////
// Selection
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Selection", null, {
	// private members
	_version: null,
	_selection: null,
	
	/*=====
	constructor_kwArgs: function(version) {
		// version: Float
		//		version used to communicate with rest service
	},
	=====*/

	constructor: function(/*constructor_kwArgs*/ kwArgs) {		
		// summary:
		//		Constructs Selection object.
		this._version = kwArgs.version;
		this.clear();
	},
	
	/////////////////////////////
	// public methods
	/////////////////////////////

	getAttributes: function() {		
		// summary:
		//		Returns selection value
		return this._selection;
	},

	clear: function() {		
		// summary:
		//		Clear out selection contents
		this._selection = {selected: false, itemIds: [], linkIds: [], relationshipIds: []};
	},

	isEmpty: function() {		
		// summary:
		//		Returns true if selection is empty (i.e. no itemIds or linkIds)
		return this._isEmpty(this._selection);
	},

	addItem: function(itemId) {		
		// summary:
		//		Add an item id to the selection
		var selection = this._getSelection();		
		if (selection) {
			this._addItem(selection, itemId);
		}
	},

	addItems: function(itemIds) {		
		// summary:
		//		Add multiple item ids to the selection
		var idsLength = itemIds.length;
		for (var idIndex = 0;  idIndex < idsLength;  ++idIndex) {
			var selection = this._getSelection();		
			if (selection) {
				this._addItem(selection, itemIds[idIndex]);
			}
		}		
	},

	removeItem: function(itemId) {		
		// summary:
		//		Remove an item id from the selection
		var selection = this._getSelection();		
		if (selection) {
			this._removeItem(selection, itemId);						
		}
		
		// delete any empty selections in the hierarchy
		this._clean(this._selection);
	},

	removeItems: function(itemIds) {		
		// summary:
		//		Remove multiple item ids from the selection
		var idsLength = itemIds.length;
		for (var idIndex = 0;  idIndex < idsLength;  ++idIndex) {
			var selection = this._getSelection(ids);		
			if (selection) {
				this._removeItem(selection, itemIds[idIndex]);						
			}			
		}
		
		// delete any empty selections in the hierarchy
		this._clean(this._selection);
	},

	addLink: function(linkId) {		
		// summary:
		//		Add a link id to the selection
		var selection = this._getSelection();		
		if (selection) {
			this._addLink(selection, linkId);
		}
	},

	addLinks: function(linkIds) {		
		// summary:
		//		Add multiple link ids to the selection
		var idsLength = linkIds.length;
		for (var idIndex = 0;  idIndex < idsLength;  ++idIndex) {
			var selection = this._getSelection();		
			if (selection) {
				this._addLink(selection, linkIds[idIndex]);
			}
		}
	},

	removeLink: function(linkId) {		
		// summary:
		//		Remove a link id from the selection
		var selection = this._getSelection();		
		if (selection) {
			this._removeLink(selection, linkId);			
		}
		
		// delete any empty selections in the hierarchy
		this._clean(this._selection);
	},

	removeLinks: function(linkIds) {		
		// summary:
		//		Remove multiple link ids from the selection
		var idsLength = linkIds.length;
		for (var idIndex = 0;  idIndex < idsLength;  ++idIndex) {
			var selection = this._getSelection();		
			if (selection) {
				this._removeLink(selection, linkIds[idIndex]);			
			}
		}
		
		// delete any empty selections in the hierarchy
		this._clean(this._selection);
	},
	
	addRelationship: function(relationshipId) {		
		// summary:
		//		Add a relationship id to the selectio
		var selection = this._getSelection();		
		if (selection) {
			this._addRelationship(selection, relationshipId);
		}
	},

	addRelationships: function(relationshipIds) {		
		// summary:
		//		Add multiple relationship ids to the selection
		var idsLength = relationshipIds.length;
		for (var idIndex = 0;  idIndex < idsLength;  ++idIndex) {
			var selection = this._getSelection();		
			if (selection) {
				this._addRelationship(selection, relationshipIds[idIndex]);
			}
		}
	},

	removeRelationship: function(elationshipId) {		
		// summary:
		//		Remove a relationship id from the selection
		var selection = this._getSelection();		
		if (selection) {
			this._removeRelationship(selection, relationshipId);			
		}
		
		// delete any empty selections in the hierarchy
		this._clean(this._selection);
	},

	removeRelationships: function(relationshipIds) {		
		// summary:
		//		Remove multiple relationship ids from the selection
		var idsLength = relationshipIds.length;
		for (var idIndex = 0;  idIndex < idsLength;  ++idIndex) {
			var selection = this._getSelection();		
			if (selection) {
				this._removeRelatinship(selection, relationshipIds[idIndex]);			
			}
		}
		
		// delete any empty selections in the hierarchy
		this._clean(this._selection);
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////

	_getSelection: function(ids) {		
		// summary:
		//		Returns the selection object associated
		
		var selection = this._selection;
		return selection;	
	},

	_clean: function(selection) {		
		return this._isEmpty(selection);
	},

	_isEmpty: function(selection) {		
		// summary:
		//		Returns true if the selection is empty
		if (selection.selected) {
			return false;
		}
		
		if (selection.itemIds.length > 0) {
			return false;
		}
		
		if (selection.linkIds.length > 0) {
			return false;
		}
		
		if (selection.relationshipIds.length > 0) {
			return false;
		}
		
		return true;
	},

	_addItem: function(selection, itemId) {		
		// summary:
		//		Add item id to the selection object
		selection.itemIds.push(itemId);
	},
	
	_removeItem: function(selection, itemId) {		
		// summary:
		//		Remove item id from the selection object
		var itemIdsLength = selection.itemIds.length;
		for (var itemIdIndex = 0;  itemIdIndex < itemIdsLength;  ++itemIdIndex) {
			if (itemId === selection.itemIds[itemIdIndex]) {
				selection.itemIds.splice(itemIdIndex, 1);
				break;
			}
		}
	},

	_addLink: function(selection, linkId) {		
		// summary:
		//		Add link id to the selection object
		selection.linkIds.push(linkId);
	},

	_removeLink: function(selection, linkId) {		
		// summary:
		//		Remove link id from the selection object
		var linkIdsLength = selection.linkIds.length;
		for (var linkIdIndex = 0;  linkIdIndex < linkIdsLength;  ++linkIdIndex) {
			if (linkId === selection.linkIds[linkIdIndex]) {
				selection.linkIds.splice(linkIdIndex, 1);
				break;
			}
		}
	},

	_addRelationship: function(selection, relationshipId) {		
		// summary:
		//		Add relationship id to the selection object
		selection.relationshipIds.push(relationshipId);
	},

	_removeRelationship: function(selection, relationshipId) {		
		// summary:
		//		Remove relationship id from the selection object
		var relIdsLength = selection.relationshipIds.length;
		for (var relIdIndex = 0;  relIdIndex < relIdsLength;  ++relIdIndex) {
			if (relationshipId === selection.relationshipIds[relIdIndex]) {
				selection.relationshipIds.splice(relIdIndex, 1);
				break;
			}
		}
	}
});
