/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Status"); 
 
////////////////////////////////////
// Status
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Status", null, {
	// private members
	_version: null,
	_status: null,

	/*=====
	constructor_kwArgs: function(version, style) {
		// version: Float
		//		version used to communicate with rest service
		// style: Object
		//		json item style object
	},
	=====*/
	
	 constructor: function(/*constructor_kwArgs*/ kwArgs) {		
		// summary:
		//		Constructs Status object.
		this._version = kwArgs.version;
		this._status = kwArgs.status;		
		if (!this._status) {
			this._status = {};
		}
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	getAttribute: function(key) {
		return this._status[key];
	},
	
	setAttribute: function(key, value) {
		this._status[key] = value;
	},
	
	getId: function() {
		return this._status.id;
	},
	
	setId: function(id) {
		this._status.id = id;		
	},
	
	getValue: function() {
		return this._status.value;
	},
	
	setValue: function(value) {
		this._status.value = value;
	},
	
	getLabel: function() {
		if (this._status.label) {
			// string
			if (typeof this._status.label === "string") {
				return this._status.label;
			}
			// object with key/value info
			else {
				return this._status.label.value;
			}
		}
		
		return null;
	},
	
	setLabel: function(label) {
		this._status.label = label;
	},
	
	getImage: function() {
		return this._status.image;
	},
	
	setImage: function(image) {
		this._status.image = image;
	},
	
	getImageWidth: function() {
		return this._status.imageWidth;
	},
	
	setImageWidth: function(width) {
		this._status.imageWidth = width;
	},
	
	getImageHeight: function() {
		return this._status.imageHeight;
	},
	
	setImageHeight: function(height) {
		this._status.imageHeight = height;
	},
	
	isImageVisible: function() {
		return (this._status.imageVisible !== undefined ? this._status.imageVisible : true);
	},
	
	setImageVisible: function(visible) {
		this._status.imageVisible = visible;
	},
	
	getScalarImage: function() {
		return this._status.scalarImage;
	},
	
	setScalarImage: function(image) {
		this._status.scalarImage = image;
	},
	
	getScalarImageWidth: function() {
		return this._status.scalarImageWidth;
	},
	
	setScalarImageWidth: function(width) {
		this._status.scalarImageWidth = width;
	},
	
	getScalarImageHeight: function() {
		return this._status.scalarImageHeight;
	},
	
	setScalarImageHeight: function(height) {
		this._status.scalarImageHeight = height;
	},
	
	isScalarImageVisible: function() {
		return (this._status.scalarImageVisible !== undefined ? this._status.scalarImageVisible : true);
	},
	
	setScalarImageVisible: function(visible) {
		this._status.scalarImageVisible = visible;
	},
	
	getBackgroundScalarImage: function() {
		return this._status.backgroundScalarImage;
	},
	
	setBackgroundScalarImage: function(image) {
		this._status.backgroundScalarImage = image;
	},
	
	getBackgroundScalarImageWidth: function() {
		return this._status.backgroundScalarImageWidth;
	},
	
	setBackgroundScalarImageWidth: function(width) {
		this._status.backgroundScalarImageWidth = width;
	},
	
	getBackgroundScalarImageHeight: function() {
		return this._status.backgroundScalarImageHeight;
	},
	
	setBackgroundScalarImageHeight: function(height) {
		this._status.backgroundScalarImageHeight = height;
	},
	
	isBackgroundScalarImageVisible: function() {
		return (this._status.backgroundScalarImageVisible !== undefined ? this._status.backgroundScalarImageVisible : true);
	},
	
	setBackgroundScalarImageVisible: function(visible) {
		this._status.backgroundScalarImageVisible = visible;
	},
	
	getTooltip: function() {
		if (this._status.tooltip) {
			// string
			if (typeof this._status.tooltip === "string") {
				return this._status.tooltip;
			}
			// object with key/value info
			else {
				return this._status.tooltip.value;
			}
		}
		
		return null;
	},
	
	setTooltip: function(tooltip) {
		this._status.tooltip = tooltip;
	},
	
	getMessage: function() {
		return this._status.message;
	},
	
	setMessage: function(message) {
		this._status.message = message;
	},
	
	getColor: function() {
		return this._status.color;
	},
	
	setColor: function(color) {
		this._status.color = color;
	},
	
	clone: function(widgetDef, objectType) {
		var args = {
			status: {},
			version: this._version
		};		
		
		if (widgetDef) {
			// start with widgetDef default status
			var widgetDefaultStatus = null;
			switch(objectType) {
				case "item":
					widgetDefaultStatus = widgetDef.getDefaultItemStatus();
					break;
				case "link":
					widgetDefaultStatus = widgetDef.getDefaultLinkStatus();
					break;
				case "relationship":
					widgetDefaultStatus = widgetDef.getDefaultRelationshipStatus();					
			}			
			if (widgetDefaultStatus) {
				dojo.mixin(args.status, widgetDefaultStatus);
			}
		
			// mixin our status
			dojo.mixin(args.status, this._status);
			
			// mixin widgetDef status
			var widgetStatus = null;
			switch(objectType) {
				case "item":
					widgetStatus = widgetDef.getItemStatus(this.getId(), this.getValue());
					break;
				case "link":
					widgetStatus = widgetDef.getLinkStatus(this.getId(), this.getValue());
					break;
				case "relationship":
					widgetStatus = widgetDef.getRelationshipStatus(this.getId(), this.getValue());					
			}					
			if (widgetStatus) {
				dojo.mixin(args.status, widgetStatus);
			}
		}
		// just mixin our status
		else {
			dojo.mixin(args.status, this._status);
		}
			
		return new ibm.tivoli.navmodel.Status(args);
	}
});
