/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.TableWidgetDef"); 
 
// includes
dojo.require("ibm.tivoli.navmodel.WidgetDef");

////////////////////////////////////
// TableWidgetDef
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.TableWidgetDef", ibm.tivoli.navmodel.WidgetDef, {

	/*=====
	constructor_kwArgs: function(baseUrl, provider, datasource, widget, definition, type, version) {
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// widget: String
		//		widget type in form <type>-<version>
		// definition: String
		//		widget definition id in form <id>-<version>
		// type: String
		//		definition type:  "user", "shared", or "base"
		// version: Float
		//		version used to communicate with rest service
	},
	=====*/ 
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs TopoWidgetDef object.
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	destroy: function() {
		// summary:
		//		Do any necessary clean up 
		if (!this.isDestroyed()) {
			// call base class
			this.inherited(arguments);
		}
	},
		
	/////////////////////////////
	// dataset section data
	/////////////////////////////	
	getDataSet: function() {
		if (!this._widgetDef.datasets) {
			this._widgetDef.datasets = [];
		}
		
		if (this._widgetDef.datasets.length === 0) {
			this._widgetDef.datasets[0] = {};
		}
		
		return this._widgetDef.datasets;		
	},

	getDataSetID: function() {
		var datasets = this.getDataSet();
		return datasets[0].id;
	},

	getDataSetLabelKey: function() {
		var datasets = this.getDataSet();
		return datasets[0].label.key;
	},
	
	getDataSetLabelValue: function() {
		var datasets = this.getDataSet();
		return datasets[0].label.value;
	},
	
	getDataSetQuery: function() {
		var datasets = this.getDataSet();
		return datasets[0].query;
	},
	
	getDataSetColumns: function() {
		var datasets = this.getDataSet();
		return datasets[0].columns;		
	},
	
	getDataSetThresholds: function(){
		var datasets = this.getDataSet();
		return datasets[0].thresholds;
	},

	getDataSetParameters: function() {
		var datasets = this.getDataSet();
		return datasets[0].parameters;
	},
	
	getDataSetResourceBundle: function() {
		var datasets = this.getDataSet();
		return datasets[0].label.resourceBundle;
	},
	
	
	getDataSetDataSource: function() {
		var datasets = this.getDataSet();
		return datasets[0].datasource;
	},
	
	getDataSetDatasourceType: function() {
		var datasets = this.getDataSet();
		datasets[0].datasourceType = type;
	},
	
	setDataSet: function(datasets) {
		this._widgetDef.datasets = datasets;		
	},

	setDataSetID: function(id) {
		var datasets = this.getDataSet();
		datasets[0].id = id;
	},

	setDataSetLabelKey: function(key) {
		var datasets = this.getDataSet();
		datasets[0].label.key = key;
	},
	
	setDataSetLabelValue: function(value) {
		var datasets = this.getDataSet();
		datasets[0].label.value = value;
	},
	
	setDataSetQuery: function(query) {
		var datasets = this.getDataSet();
		datasets[0].query = query;
	},
	
	setDataSetColumns: function(columns) {
		var datasets = this.getDataSet();
		datasets[0].columns = columns;
	},

	setDataSetParameters: function(parameters) {
		var datasets = this.getDataSet();
		datasets[0].parameters = parameters;
	},
	
	setDataSetResourceBundle: function(bundle) {
		var datasets = this.getDataSet();
		datasets[0].label.resourceBundle = bundle;
	},
	
	/////////////////////////////
	// table section data
	/////////////////////////////	
	getTable: function() {
		if (!this._widgetDef.table) {
			this._widgetDef.table = {};
		}
		
		return this._widgetDef.table;		
	},
	
	getTableID: function() {
		var table = this.getTable();
		return table.id;		
	},
	
	getTableView: function() {
		var table = this.getTable();
		if (!table.view) {
			table.view = [];
		}
		
		return table.view;		
	},
	
	getTableViewElement: function(id) {
		var entry = null;
		var view = this.getTableView();
		if (view) {
			var length = view.length;
			for (var index = 0;  index < length;  ++index) {
				if (view[index].id === id) {
					entry = view[index];
					break;
				}
			}
		}
		
		return entry;
	},
	
	getTableCommon: function() {
		var table = this.getTable();
		return table.common;		
	},

	getTableCommonAutoUpdate: function() {
		var table = this.getTable();
		return table.common.autoUpdate;		
	},
	
	getTableCommonShowLastUpdateTime: function() {
		var table = this.getTable();
		return table.common.showLastUpdateTime;		
	},
	
	getTableCommonPaging: function() {
		var table = this.getTable();
		return table.common.paging;		
	},
	
	getTableCommonPagingSize: function() {
		var table = this.getTable();
		return table.common.pagingSize;		
	},
	
	getTableShowAdvancedFilter: function() {
		var table = this.getTable();
		return table.showAdvancedFilter;
	},
	
	getTableShowQuickFilter: function() {
		var table = this.getTable();
		return table.showQuickFilter;
	},
	
	getTableShowClearFilterConfirmation: function() {
		var table = this.getTable();
		return table.showClearFilterConfirmation;
	},
		
	getTableFilter: function() {
		var table = this.getTable();
		return table.filter;
	},
	
	getTableLabelKey: function() {
		var table = this.getTable();
		if (!table.label) {
			table.label = {};
		}
		
		return table.label.key;		
	},
	
	getTableLabelValue: function() {
		var table = this.getTable();
		if (!table.label) {
			table.label = {};
		}
		
		return table.label.value;		
	},
	
	getTableDescriptionKey: function() {
		var table = this.getTable();
		if (!table.description) {
			table.description = {};
		}
		
		return table.description.key;		
	},
	
	getTableDescriptionValue: function() {
		var table = this.getTable();
		if (!table.description) {
			table.description = {};
		}
		
		return table.description.value;		
	},
	
	getTableColumnReordering: function() {
		var table = this.getTable();
		return table.columnReordering;		
	},

	setTable: function(table) {
		this._widgetDef.table = table;		
	},
	
	setTableID: function(id) {
		var table = this.getTable();
		table.id = id;		
	},
	
	setTableView: function(view) {
		var table = this.getTable();
		table.view = view;
	},
	
	setTableCommon: function(common) {
		var table = this.getTable();
		table.common = common;		
	},

	setTableCommonAutoUpdate: function(autoUpdate) {
		var table = this.getTable();
		table.common.autoUpdate = autoUpdate;		
	},
	
	setTableCommonShowLastUpdateTime: function(showLastUpdateTime) {
		var table = this.getTable();
		table.common.showLastUpdateTime = showLastUpdateTime;		
	},
	
	setTableCommonPaging: function(paging) {
		var table = this.getTable();
		table.common.paging = paging;		
	},
	
	setTableCommonPagingSize: function(pagingSize) {
		var table = this.getTable();
		table.common.pagingSize = pagingSize;		
	},
	
	setTableShowAdvancedFilter: function(showAdvancedFilter) {
		var table = this.getTable();
		table.showAdvancedFilter = showAdvancedFilter;
	},
	
	setTableFilter: function(filter) {
		var table = this.getTable();
		table.filter = filter;
	},
	
	setTableLabelKey: function(key) {
		var table = this.getTable();
		if (!table.label) {
			table.label = {};
		}
		table.label.key = key;		
	},
	
	setTableLabelValue: function(value) {
		var table = this.getTable();
		if (!table.label) {
			table.label = {};
		}
		table.label.value = value;		
	},
	
	setTableDescriptionKey: function(key) {
		var table = this.getTable();
		if (!table.description) {
			table.description = {};
		}
		table.description.key = key;		
	},
	
	setTableDescriptionValue: function(value) {
		var table = this.getTable();
		if (!table.description) {
			table.description = {};
		}
		table.description.value = value;		
	},
	
	setTableColumnReordering: function(columnReordering) {
		var table = this.getTable();
		table.columnReordering = columnReordering;		
	}
});
