/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Task"); 
 
// includes
dojo.require("ibm.tivoli.navmodel.Dataset");
dojo.require("ibm.tivoli.navmodel.Selection");
dojo.require("ibm.tivoli.navmodel.Utils");
dojo.require("ibm.tivoli.navmodel.Trace");
ibm.tivoli.navmodel.Task._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.Task");

////////////////////////////////////
// Task
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Task", null, {
	// private members
	_task: null,
	_dataset: null,
	_childTasks: null,
	_trace: ibm.tivoli.navmodel.Task._trace,
	
	/*=====
	constructor_kwArgs: function(task, dataset) {
		// task: Object
		//		json task object
		// dataset: Object
		//		ibm.tivoli.navmodel.Dataset that created this task
	},
	=====*/
	
	 constructor: function(/*constructor_kwArgs*/ kwArgs) {				
		// summary:
		//		Constructs Task object.
		this._dataset = kwArgs.dataset;
		this._task = kwArgs.task;
		if (!this._task) {
			this._task = {};
		}
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	getAttribute: function(key) {		
		// summary:
		//		Returns general attribute for requested key
		return this._task[key];
	},

	setAttribute: function(key, value) {		
		// summary:
		//		Set general attribute for requested key
		this._task[key] = value;
	},

	getVersion: function() {		
		// summary:
		//		Returns rest version
		return this._dataset.getVersion();
	},
		
	getUri: function() {
		if (!this._task.uri) {
			this._task.uri = this._dataset.getAttribute("uri") + "/tasks/" + encodeURIComponent(this.getId());
		}
	
		return this._dataset.getArg("baseUrl") + this._task.uri;
	},
	
	getId: function() {
		return this._task.id;
	},
	
	setId: function(id) {
		this._task.id = id;
	},
	
	getLabel: function() {
		return this._task.label;
	},
	
	setLabel: function(label) {
		this._task.label = label;
	},
	
	getDescription: function() {
		return this._task.description;
	},
	
	setDescription: function(description) {
		this._task.description = description;
	},
	
	getImage: function() {
		return this._task.image;
	},
	
	setImage: function(image) {
		this._task.image = image;
	},

	getCategory: function() {		
		// summary:
		//		Returns the task category (i.e. where the task is visualized)
		//		
		//		values are: menu, menu:navigation, toolbar, toolbar:button, toolbar:menu,
		//					breadcrumb, breadcrumb:navigation, and property
		return this._task.category;
	},
	
	setCategory: function(category) {
		this._task.category = category;
	},

	isServer: function() {		
		// summary:
		//		Indicates that task should be sent back to the server for execution
		return (this._task.server !== undefined ? this._task.server : false);
	},
	
	setServer: function(isServer) {
		this._task.server = isServer;
	},
	 
	getType: function() {		
		// summary:
		//		Returns the task type
		//		
		//		values are: none, url, page, portlet, clientevent, javascript, native,
		//					dialog, noninteractive, cascade, and separator
		return this._task.type;
	},

	setType: function(type) {		
		// summary:
		//		Set the task type
		//		
		//		values are: none, url, page, portlet, clientevent, javascript, native,
		//					dialog, noninteractive, cascade, and separator
		this._task.type = type;
	},
	
	isNormal: function() {
		return (!this.isNavigation() && !this.isCascade() && !this.isSeparator());
	},

	isNavigation: function() {
		return (this._task.type === "navigation");
	},
	
	isPageLaunch: function() {
		return (this._task.type === "page");
	},
	
	isPortletLaunch: function() {
		return (this._task.type === "portlet");
	},
	
	isUrlLaunch: function() {
		return (this._task.type === "url");
	},
	
	isJavascript: function() {
		return (this._task.type === "javascript");
	},
	
	isClientEvent: function() {
		return (this._task.type === "clientevent");
	},
	
	isCascade: function() {
		return (this._task.type === "cascade" && this._task.subTasks);
	},
	
	isSeparator: function() {
		return (this._task.type === "separator");
	},
	
	isResolved: function() {
		return (this._task.resolved !== undefined ? this._task.resolved : true);
	},
	
	setResolved: function(isResolved) {
		this._task.resolved = isResolved;
	},
	
	isDefault: function() {
		return (this._task["default"] !== undefined ? this._task["default"] : false);
	},
	
	setDefault: function(isDefault) {
		this._task["default"] = isDefault;
	},
	
	isEnabled: function() {
		return (this._task.enabled !== undefined ? this._task.enabled : true);
	},
	
	setEnabled: function(isEnabled) {
		this._task.enabled = isEnabled;
	},
	
	isDisabled: function() {
		return (this._task.enabled !== undefined ? !this._task.enabled : false);
	},
	
	setDisabled: function(isDisabled) {
		this._task.enabled = !isDisabled;
	},
	
	/*=====
	resolve_kwArgs: function(selection, itemIds, linkIds, itemIndexes) {
		// selection: Object
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// itemIds: String
		//		comma delimited list of item ids that are selected
		// linkIds: String
		//		comma delimited list of link ids that are selected
		// itemIndexes: String
		//		comma delimited list of item ids or ranges that are selected
		//		ex: "0,1,10,20" or "0-10,30,40-50"
	},
	=====*/

	resolve: function(/*resolve_kwArgs*/ kwArgs) {		
		// summary:
		//		Re-fetch the task to resolve all client side launch parameters
		var args = {
			selection: null,
			itemIds: null,
			linkIds: null,
			itemIndexes: null
		};
		dojo.mixin(args, kwArgs);
		
		// selection was provided
		var json = "";
		if (args.selection) {
			json = dojo.toJson(args.selection.getAttributes(), false);
		}
			
		return dojo.xhrPost ({
			url: this._getParamURL(args),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "task"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "taskContext"),
				'Content-Length': json.length,
				'X-Method-Override': 'GET'
			},
			handleAs: "json",
			postData: json,
			sync: true,			
			load: dojo.hitch(this, function(result, ioArgs) {
				this._task = result;
			}),
			error: dojo.hitch(function(error, ioArgs) {
				ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "resolveLaunchParms", this._trace, this._version());					              
			})
		});
	},
	
	getLaunchUrl: function() {
		return this._task.url;
	},
	
	setLaunchUrl: function(url) {
		this._task.url = url;
	},
	
	getLaunchBrowserWindowId: function() {
		return this._task.browserWindowId;
	},
	
	setLaunchBrowserWindowId: function(windowId) {
		this._task.browserWindowId = windowId;
	},
	
	getLaunchPageId: function() {
		return this._task.pageId;
	},
	
	setLaunchPageId: function(pageId) {
		this._task.pageId = pageId;
	},
	
	getLaunchPortletName: function() {
		return this._task.portletName;
	},
	
	setLaunchPortletName: function(portletName) {
		this._task.portletName = portletName;
	},
	
	getLaunchPortletReuse: function() {
		return this._task.portletReuse;
	},
	
	setLaunchPortletReuse: function(reuse) {
		this._task.portletReuse = reuse;
	},
	
	getLaunchPortletAppUID: function() {
		return this._task.portletAppUID;
	},
	
	setLaunchPortletAppUID: function(appUID) {
		this._task.portletAppUID = appUID;
	},
	
	getLaunchEvent: function() {
		return this._task.event;
	},
	
	setLaunchEvent: function(event) {
		this._task.event = event;
	},
	
	getLaunchJavascript: function() {
		return this._task.javascript;
	},
	
	setLaunchJavascript: function(javascript) {
		this._task.javascript = javascript;
	},
	
	getLaunchApplicationParms: function() {
		return this._task.applicationParms;
	},
	
	setLaunchApplicationParms: function(applicationParms) {
		this._task.applicationParms = applicationParms;
	},
	
	/*
	 * Returns any child tasks if this task is a cascade
	 */
	getChildTasks: function() {
		if (this.isCascade()) {
			if (!this._childTasks) {
				this._childTasks = [];
				var subTaskLength = this._task.subTasks.length;
				for (var index = 0;  index < subTaskLength;  ++index) {
					this._childTasks[index] = new ibm.tivoli.navmodel.Task({task: this._task.subTasks[index], dataset: this._dataset});
				}
			}
			
			return this._childTasks;
		}
		
		return null;
	},
	
	setChildTasks: function(childTasks) {
		if (this.isCascade()) {
			this._childTasks = childTasks;
		}
	},
	
	/*=====
	result_onComplete: function(result, kwArgs){},
	=====*/
	
	/*=====
	_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	execute_kwArgs: function(onComplete, onError, selection, sync) {
		// onComplete: result_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for successful fetch
		// selection: Object:
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// selectionParms: Object:
		//		extra parameters that can be included in the selection when PUT
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	execute: function(/*execute_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously execute remote task
		
		// this is only for executing regular tasks
		if (this.isNavigation()) {
			alert('execute is only for non-navigation tasks');
			return;
		}
		
		var args = {                
			onComplete: function(result, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},         
			selection: null,
			selectionParms: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		// get selection and add additional parameters	
		var selection = args.selection.getAttributes();
		if (args.selectionParms) {
			selection.parameters = args.selectionParms;
		}
			
		var json = dojo.toJson(selection, false);
			
		return dojo.xhrPost ({
			url: this._getParamURL({}),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "taskContext"),
				'Content-Length': json.length				
			},
			handleAs: "json",
			postData: json,
			sync: args.sync,		
			load: dojo.hitch(this, function(result, ioArgs) {
				if (this._trace.isFine()) {
					this._trace.fine("Execute for taskId=" + this.getId() + " returned successfully", "execute");
				}
							
				args.onComplete(result, args);				
			}),
			error: dojo.hitch(function(error, ioArgs) {
				args.onError(error, ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "execute", this._trace, this._version()), ioArgs);	               
			})
		});		
	},
	
	/*=====
	dataset_onComplete: function(dataset, kwArgs){},
	=====*/
	
	/*=====
	navigate_kwArgs: function(onComplete, onError, selection, sync) {
		// onComplete: dataset_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for successful fetch
		// selection: Object:
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	navigate: function(/*navigate_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously execute navigation task (i.e. returns new Dataset)
		
		// this is only for executing regular tasks
		if (!this.isNavigation()) {
			alert('navigate is only for navigation tasks');
			return;
		}
		
		var args = {                
			onComplete: function(dataset, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			taskId: this.getId(),
			taskCategory: this.getCategory(),                
			selection: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		// call the dataset to do the navigation
		this._dataset.navigate(args);
	},	
	
	/////////////////////////////
	// private methods
	/////////////////////////////
	
	_version: function() {
		return this._dataset.getVersion();
	},

	_getParamURL: function(kwArgs) {		
		// summary:
		//		Build url necessary to do reference task and also includes
		//		all of the "param_" and "version" arguments
		var url = this.getUri();
		var questionMark = false;
		
		// add "param_" parms
		var parameters = this._dataset.getArg("parameters");
		for (var parm in parameters) {
			if (parameters.hasOwnProperty(parm)) {
				if (!questionMark) {
					questionMark = true;
					url += "?";
				}
				else {
					url += "&";
				}
				url += ("param_" + parm + "=" + parameters[parm]);				
			}
		}
		
		// IE will sometimes return http requests from cache without unique parms
		var now = new Date();
		if (!questionMark) {
			questionMark = true;
			url += "?d=" + now.getTime() + "&r=" + Math.random();
		}
		else {
			url += "&d=" + now.getTime() + "&r=" + Math.random();
		}
		
		// add "category"
		if(!questionMark) {
			questionMark = true;
			url += "?category=" + this.getCategory();
		}
		else {
			url += "&category=" + this.getCategory();
		}
		
		// indicate selected items
		if (kwArgs.itemIds) {
			if (!questionMark) {
				questionMark = true;
				url += "?itemIds=" + kwArgs.itemIds;
			}
			else {
				url += "&itemIds=" + kwArgs.itemIds;
			}			
		}
		
		// indicate selected links
		if (kwArgs.linkIds) {
			if (!questionMark) {
				questionMark = true;
				url += "?linkIds=" + kwArgs.linkInds;
			}
			else {
				url += "&linkIds=" + kwArgs.linkIds;
			}			
		}
		
		// indicate seleted item ranges
		if (kwArgs.itemIndexes) {
			if (!questionMark) {
				questionMark = true;
				url += "?itemIndexes=" + kwArgs.itemIndexes;
			}
			else {
				url += "&itemIndexes=" + kwArgs.itemIndexes;
			}			
		}
		
		return url;	
	}
});
