/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.TopoWidgetDef"); 
 
// includes
dojo.require("ibm.tivoli.navmodel.WidgetDef");

////////////////////////////////////
// TopoWidgetDef
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.TopoWidgetDef", ibm.tivoli.navmodel.WidgetDef, {

	/*=====
	constructor_kwArgs: function(baseUrl, provider, datasource, widget, definition, type, version) {
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// widget: String
		//		widget type in form <type>-<version>
		// definition: String
		//		widget definition id in form <id>-<version>
		// type: String
		//		definition type:  "user", "shared", or "base"
		// version: Float
		//		version used to communicate with rest service
	},
	=====*/ 
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs TopoWidgetDef object.
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	validateDefaults: function() {
		// summary:
		//		This method is called after the widget def has been read from the server.
		//		Validate defaults that might not be present in the widget def contents.
		
		var layouts = this.getLayouts();
		if (layouts) {
			var length = layouts.length;
			for (var index = 0;  index < length;  ++index) {
				var layout = layouts[index];
				switch (layout.algorithm) {
					case "tree":
						if (!layout.layoutMode) {
							layout.layoutMode = "alternating radial";
						}
						if (!layout.flowDirection) {
							layout.flowDirection = "bottom";
						}
						if (!layout.globalLinkStyle) {
							layout.globalLinkStyle = "straight";
						}
						if (!layout.connectorStyle) {
							layout.connectorStyle = "automatic";
						}
						if (!layout.globalAlignment) {
							layout.globalAlignment = "center";
						}
						if (!layout.levelAlignment) {
							layout.levelAlignment = "center";
						}
						if (!layout.aspectRatio) {
							layout.aspectRatio = "1";
						}
						break;
					case "hierarchical":
						if (!layout.flowDirection) {
							layout.flowDirection = "bottom";
						}
						if (!layout.levelJustification) {
							layout.levelJustification = "bottom";
						}
						if (!layout.levelingStrategy) {
							layout.levelingStrategy = "optimal";
						}
						if (!layout.globalLinkStyle) {
							layout.globalLinkStyle = "straight";
						}
						if (!layout.connectorStyle) {
							layout.connectorStyle = "automatic";
						}
						break;
					case "forceDirected":
						if (!layout.layoutMode) {
							layout.layoutMode = "fast multilevel";
						}
						if (!layout.preferredLinksLength) {
							layout.preferredLinksLength = "25";
						}
						if (!layout.linkStyle) {
							layout.linkStyle = "straight";
						}
						if (!layout.maxAllowedMovePerIteration) {
							layout.maxAllowedMovePerIteration = "5";
						}
						if (!layout.allowedNumberOfIterations) {
							layout.allowedNumberOfIterations = "250";
						}
						if (!layout.convergenceThreshold) {
							layout.convergenceThreshold = "0.1";
						}						
						break;
					case "shortLink":
						if (!layout.globalLinkStyle) {
							layout.globalLinkStyle = "direct";
						}
						if (!layout.globalSelfLinkStyle) {
							layout.globalSelfLinkStyle = "two bends orthogonal";
						}
						if (!layout.globalConnectorStyle) {
							layout.globalConnectorStyle = "automatic";
						}
						if (!layout.flowDirection) {
							layout.flowDirection = "bottom";
						}
						if (!layout.levelJustification) {
							layout.levelJustification = "bottom";
						}
						if (!layout.levelingStrategy) {
							layout.levelingStrategy = "optimal";
						}
						if (!layout.connectorStyle) {
							layout.connectorStyle = "automatic";
						}
						if (!layout.linkOffset) {
							layout.linkOffset = "1";
						}
						if (!layout.minFinalSegmentLength) {
							layout.minFinalSegmentLength = "10";
						}
						break;
					case "longLink":
						if (!layout.globalLinkStyle) {
							layout.globalLinkStyle = "direct";
						}
						if (!layout.flowDirection) {
							layout.flowDirection = "bottom";
						}
						if (!layout.levelJustification) {
							layout.levelJustification = "bottom";
						}
						if (!layout.levelingStrategy) {
							layout.levelingStrategy = "optimal";
						}
						if (!layout.connectorStyle) {
							layout.connectorStyle = "automatic";
						}
						if (!layout.verticalMinOffset) {
							layout.verticalMinOffset = "3";
						}
						if (!layout.horizontalMinOffset) {
							layout.horizontalMinOffset = "3";
						}
						break;
					case "grid":
						if (!layout.layoutMode) {
							layout.layoutMode = "tile to rows";
						}
						if (!layout.globalHorizontalAlignment) {
							layout.globalHorizontalAlignment = "center";
						}
						if (!layout.globalVerticalAlignment) {
							layout.globalVerticalAlignment = "center";
						}
						if (!layout.maxNumberOfNodesPerRowOrColumn) {
							layout.maxNumberOfNodesPerRowOrColumn = "3";
						}
				}
			}
		}
	},
	
	destroy: function() {
		// summary:
		//		Do any necessary clean up 
		if (!this.isDestroyed()) {
			// call base class
			this.inherited(arguments);
		}
	},

	getDepth: function() {
		var returnValue = this._widgetDef.topology.depth;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getDepth) {
				returnValue = parent.getDepth();
			}
		}
		if(returnValue){
			return returnValue;
		} else {
			return -1;
		}
	},
	
	getInitialZoomLevel: function() {
		var returnValue = this._widgetDef.topology.initialZoomLevel;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getInitialZoomLevel) {
				returnValue = parent.getInitialZoomLevel();
			}
		}
		if(returnValue){
			return returnValue;
		} else {
			return -1; //Default to -1 which would result in just fitting the contents
		}
	},	

	zoomOnObjectAdded: function() {
		return this._widgetDef.topology.zoomOnObjectAdded;
	},

	zoomLevelOnObjectAdded: function() {
		return this._widgetDef.topology.zoomLevelOnObjectAdded;
	},

	getMaxDepth: function() {
		var returnValue = this._widgetDef.topology.maxDepth;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getMaxDepth) {
				returnValue = parent.getMaxDepth();
			}
		}
		if(returnValue){
			return returnValue;
		} else {
			return -1;
		}
	},

	useNodeLevelAndPosition: function() {
		return this._widgetDef.topology.useNodeLevelAndPosition;
	},

	getNodeOrderDirection: function() {
		return this._widgetDef.topology.nodeOrderDirection;
	},

	getNodeOrderProperty: function() {
		return this._widgetDef.topology.nodeOrderProperty;
	},

	getBackgroundColor: function() {
		var returnValue = this._widgetDef.topology.backgroundColor;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getBackgroundColor) {
				returnValue = parent.getBackgroundColor();
			}
		}
		if(returnValue){
			return returnValue;
		} else {
			return -1;
		}
	},
	
	/////////////////////////////
	// footer section data
	/////////////////////////////
	
	getIncludeFooter: function() {
		var returnValue = this._widgetDef.topology.includeFooter;
		if(returnValue===undefined) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getIncludeFooter) {
				returnValue = parent.getIncludeFooter();
			}
		}
		if(returnValue!==undefined){
			return returnValue;
		} else {
			return true;
		}
	},
	
	setIncludeFooter: function(includeFooter) {
		this._widgetDef.topology.includeFooter = includeFooter;
	},
	
	getHideFooterDateTime: function(){
		var returnValue = this._widgetDef.topology.hideFooterDateTime;
		if(returnValue===undefined) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getHideFooterDateTime) {
				returnValue = parent.getHideFooterDateTime();
			}
		}
		if(returnValue!==undefined){
			return returnValue;
		} else {
			return false;
		}
	},

	setHideFooterDateTime: function(hideFooterDateTime) {
		this._widgetDef.topology.hideFooterDateTime = hideFooterDateTime;
	},
	
	getFilters: function(){
		var returnValue = this._widgetDef.topology.filters;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getFilters) {
				returnValue = parent.getFilters();
			}
		}		
		return returnValue;			
	},

	/////////////////////////////
	// search section data
	/////////////////////////////

	isAdvSearchEnabled: function() {
		var returnValue = this._widgetDef.topology.isAdvSearchEnabled;
		if(returnValue===undefined) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.isAdvSearchEnabled) {
				returnValue = parent.isAdvSearchEnabled();
			}
		}
		if(returnValue!==undefined){
			return returnValue;
		} else {
			return false;
		}
	},
	
	getAdvSearchProperties: function() {
		var returnValue = this._widgetDef.topology.advSearchProperties;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getAdvSearchProperties) {
				returnValue = parent.getAdvSearchProperties();
			}
		}		
		return returnValue;	
	},	

	/////////////////////////////
	// layout section data
	/////////////////////////////	
	getLayouts: function() {
		var returnValue = this._widgetDef.topology.layout;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getLayouts) {
				returnValue = parent.getLayouts();
			}
		}		
		return returnValue;	
	},
	
	updateLayouts: function(layouts) {
		var delta = this.getWidgetDelta("user");
		if (delta) {
			delta.setSectionAttribute({sectionKey: "topology", attrKey: "layout", attrValue: layouts}); 
		} 
	},
	
	clearLayouts: function() {
		var delta = this.getWidgetDelta("user");
		if (delta) {
			delta.removeSectionAttribute({sectionKey: "topology", attrKey: "layout"}); 
		}
	},
	
	getLayout: function(/*String*/ id) {
		var layouts = this.getLayouts();
		for(var i=0;i<layouts.length;i++){
			if(layouts[i].id === id){
				return layouts[i];
			}
		}
	},
	
	getDefaultLayout: function() {
		var layouts = this.getLayouts();
		for(var i=0;i<layouts.length;i++){
			if(layouts[i].isDefault){
				return layouts[i];
			}
		}
	},
	
	getDefaultLayoutId: function() {
		var layouts = this.getLayouts();
		for(var i=0;i<layouts.length;i++){
			if(layouts[i].isDefault){
				return layouts[i].id;
			}
		}
	},
		
	getRenderRelationshipsInsteadOfLinks: function(){
		var returnValue = this._widgetDef.topology.renderRelationshipsInsteadOfLinks;
		if(returnValue===undefined) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getRenderRelationshipsInsteadOfLinks) {
				returnValue = parent.getRenderRelationshipsInsteadOfLinks();
			}
		}
		if(returnValue!==undefined){
			return returnValue;
		} else {
			return false;
		}		
	}
	
	/////////////////////////////
	// private methods
	/////////////////////////////	

});
