/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Trace");
 
dojo.declare("ibm.tivoli.navmodel.Trace", null, {
    // private members     
	traceName: "",
	level: 0,  // error
        
	constructor: function(name) {
		this.traceName = name;
				
		if (dojo.config.trace) {
			// "all" overrides individual class entries
			if (dojo.config.trace.all) {
				this.level = this._translateLevel("all", dojo.config.trace.all);
			}
			else {
				var lastDot = this.traceName.lastIndexOf(".");
				if (lastDot > 0) {
					var className = this.traceName.substring(lastDot+1);
					if (dojo.config.trace[className]) {
						this.level = this._translateLevel(className, dojo.config.trace[className]);
					}
				}
			}
		}					
 	},
        
	/////////////////////////////
	// public methods
	/////////////////////////////
	
	isError: function() {
		return (this.level >= 0);
	},
	
	isWarn: function() {
		return (this.level >= 1);
	},
	
	isInfo: function() {
		return (this.level >= 2);
	},
	
	isFine: function() {
		return (this.level >= 3);
	},
	
	isFiner: function() {
		return (this.level >= 4);
	},
	
	isFinest: function() {
		return (this.level >= 5);
	},
	
	error: function(dbgString, methodName) {
		if (this.isError()) {
			if (dbgString && dbgString instanceof Error) {
                console.error(this._output("EXCEPTION", methodName, dbgString.toString()));
	    	} 
	    	else if (dbgString && dbgString.isInstanceOf && dbgString.isInstanceOf(ibm.tivoli.navmodel.Exception)) {
	    		console.error(this._output("EXCEPTION", methodName, dbgString.getMsgText()));
	    	} else {
	    		console.error(this._output("ERROR", methodName, dbgString));
	    	}
	    }
	},
	
	warn: function(dbgString, methodName) {
    	if (this.isWarn()) {
    		console.warn(this._output("WARN", methodName, dbgString));
    	}
    },
    
	info: function(dbgString, methodName) {
    	if (this.isInfo()) {
    		console.info(this._output("INFO", methodName, dbgString));
    	}
    },
    
    fine: function(dbgString, methodName) {
    	if (this.isFine()) {
    		console.info(this._output("FINE", methodName, dbgString));
    	}
    },
    
    finer: function(dbgString, methodName) {
    	if (this.isFiner()) {
    		console.info(this._output("FINER", methodName, dbgString));
    	}
    },
    
	finest: function(dbgString, methodName) {
    	if (this.isFinest()) {
    		console.info(this._output("FINEST", methodName, dbgString));
    	}
    },
	
	entering: function(dbgString, methodName) {
    	if (this.isFinest()) {
    		console.info(this._output("ENTERING", methodName, dbgString));
    	}
    },
    
    exiting: function(dbgString, methodName) {
    	if (this.isFinest()) {
    		console.info(this._output("EXITING", methodName, dbgString));
    	}
    },
    
	object: function(dbgObject, methodName) {
    	if (this.isFine()) {
    		console.log(this._output("OBJECT", methodName, dojo.toJson(dbgObject)));
    	}
    },
    
	assert: function(predicate, message, methodName) {
		if (this.isFine()) {
			if (predicate === false ) {
				console.log(this._output("ASSERT", methodName, "assertion failed: " + message));
			}
		}		
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////
	
	_translateLevel: function(className, value) {
		var level = 0;
		
		switch(value) {
			case "off":
				level = -1;
				break;
			case "error":
				level = 0;
				break;
			case "warn":
				level = 1;
				break;
			case "info":
				level = 2;
				break;
			case "fine":
				level = 3;
				break;
			case "finer":
				level = 4;
				break;
			case "finest":
				level = 5;
				break;
			default:
				console.log("className=" + className + " has invalid level=" + value);
		}
		
		return level;
	},
	
	//***** DEBUG::trace=ibm.tivoli.navmodel.Dataset, method=EXITING; event received, type = itemStatusChanged
	_output: function(level, methodName, message) {
		var outputMessage = "***** " + level;
		if (message) {
			outputMessage = outputMessage + ": " + message;
		}
		outputMessage = outputMessage + " ::trace=" + this.traceName;
		if (methodName) {
			outputMessage = outputMessage + ", method=" + methodName;
		}
		return outputMessage;
	}	
});