/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Utils"); 

////////////////////////////////////
// Utils
////////////////////////////////////

ibm.tivoli.navmodel.Utils.isTIP = function() {
	//	summary:
	//		Returns true if environment is running in Tivoli Integrated Portal  
  	var themes = ["TIPLight", "TIPDark", "ISCTheme"];
  	var themeCount = themes.length;
  	var body = dojo.body();
  	for (var index = 0;  index < themeCount;  ++index) {
  		if (dojo.hasClass(body, themes[index])) {
  			return true;
  		}
  	}
  	
  	return false;  
};

ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion = function() {
	return 1.0;
};

ibm.tivoli.navmodel.Utils.getContentType = function(version, format) {
	// summary:
	//		Return the HttpHeader "Content-Type" that REST service is expecting
	//		;format=xxxx   can be appended to indicate the json format object
	var contentType = "application/vnd.ibm.com.tivolidis.json;version=" + version;	
	if (format) {
		contentType += ";format=" + format;
	}
	
	return contentType;
};

ibm.tivoli.navmodel.Utils.getContent = function() {		
	// summary:
	//		Construct a "content" object for sending an xdr request;
	//		IE will sometimes return http requests from cache without unique parms
	var content = {};
		
	// add date and random
	var now = new Date();		
	content.d = now.getTime();
	content.r = Math.random();
		
	return content;
};

ibm.tivoli.navmodel.Utils.mixinParams = function(content, dataset) {		
	// summary:
	//		Mixin the "param_" qualified arguments;  all parameters are 
	//		prefixed with "param_" so the rest service can use them to identify
	//		dataset cache entry
	var parameters = dataset.getArg("parameters");
	for (var parm in parameters) {
		if (parameters.hasOwnProperty(parm)) {
			content["param_" + parm] = parameters[parm];
		}
	}
};

ibm.tivoli.navmodel.Utils.mixinProperties = function(content, key, properties) {		
	// summary:
	//		Mixin the "properties" into the content as a comma delimited list
	if (properties && properties.length > 0) {
		content[key] = properties.join(",");
	}
};

ibm.tivoli.navmodel.Utils.getParamUrl = function(url, dataset) {		
	// summary:
	//		Tack on dataset version and param_* values to the url
	var questionMark = false;
		
	// add "param_" parms
	var parameters = dataset.getArg("parameters");
	for (var parm in parameters) {
		if (parameters.hasOwnProperty(parm)) {
			if (!questionMark) {
				questionMark = true;
				url += "?";
			}
			else {
				url += "&";
			}
			url += ("param_" + parm + "=" + parameters[parm]);				
		}
	}
		
	// IE will sometimes return http requests from cache without unique parms
	var now = new Date();
	if (!questionMark) {
		questionMark = true;
		url += "?d=" + now.getTime() + "&r=" + Math.random();
	}
	else {
		url += "&d=" + now.getTime() + "&r=" + Math.random();
	}
	
	return url;	
};

ibm.tivoli.navmodel.Utils.getException = function(error, ioArgs, funcName, trace, version) {		
	// summary:
	//		Returns Exception object contained in the dojo.xhr ioArgs
	var exception = null;
	var responseText = ioArgs.xhr.responseText;
	if (responseText) {
		try {
			var jsonEx = dojo.fromJson(responseText);
			if (jsonEx.msgId) {
				trace.error(funcName + " generated exception=" + responseText, "getException");				
				exception = new ibm.tivoli.navmodel.Exception({exception: jsonEx, version: version});
			}
			else {
				trace.error(funcName + " generated exception (non rest) = " + error, "getException");				
			}
		}
		catch(e) {
			trace.error(funcName + " generated exception (non rest) = " + error, "getException");
			exception = new ibm.tivoli.navmodel.Exception({exception: {msgText: error}, version: version});
		}
			
		trace.error("exception url=" + ioArgs.url, "getException");							
	}
	
	return exception;
};

ibm.tivoli.navmodel.Utils.NorthernHemisphere = {
	"GMT-10:00": "US/Aleutian",
  	"GMT-9:00":  "US/Alaska",
  	"GMT-8:00":  "US/Pacific",
  	"GMT-7:00":  "US/Mountain",
	"GMT-6:00":  "US/Central",
	"GMT-5:00":  "US/Eastern",
	"GMT-4:00":  "Canada/Atlantic",
	"GMT-3:30":  "Canada/Newfoundland",
	"GMT-3:00":  "America/Godthab",
    "GMT-2:00":  "America/Noronha",
    "GMT-1:00":  "Atlantic/Azores",
    "GMT":       "Europe/London",
    "GMT+1:00":  "Europe/Amsterdam",
    "GMT+2:00":  "Europe/Athens",
    "GMT+3:00":  "Europe/Moscow",
    "GMT+3:30":  "Asia/Tehran",
    "GMT+4:00":  "Asia/Baku",
    "GMT+5:00":  "Asia/Yekaterinburg",
    "GMT+6:00":  "Asia/Novosibirsk",
    "GMT+7:00":  "Asia/Krasnoyarsk",
    "GMT+8:00":  "Asia/Irkutsk",
    "GMT+9:00":  "Asia/Yakutsk",
    "GMT+10:00": "Asia/Vladivostok"
};

ibm.tivoli.navmodel.Utils.SouthernHemisphere = {
	"GMT-4:00":  "America/Asuncion",
	"GMT-3:00":  "America/Montevideo",
	"GMT+2:00":  "Africa/Windhoek",
	"GMT+9:30":  "Australia/Adelaide",
	"GMT+10:00": "Australia/Sydney",
	"GMT+12:00": "Pacific/Fiji"
};

ibm.tivoli.navmodel.Utils.NoDST = {  
	"GMT-11:00": "US/Samoa",
	"GMT-10:00": "US/Hawaii",
	"GMT-7:00":	 "US/Arizona",
	"GMT-6:00":  "Canada/Saskatchewan",
	"GMT-5:00":  "US/East-Indiana",
	"GMT-4:30":  "America/Caracas",
	"GMT-4:00":  "America/Manaus",
	"GMT-3:00":  "America/Buenos_Aires",
	"GMT-2:00":  "Atlantic/South_Georgia",
	"GMT-1:00":  "Atlantic/Cape_Verde",
	"GMT":       "Africa/Monrovia",
	"GMT+1:00":  "Africa/Algiers",
	"GMT+2:00":  "Africa/Harare",
	"GMT+3:00":  "Asia/Baghdad",
	"GMT+4:00":  "Asia/Dubai",
	"GMT+4:30":  "Asia/Kabul",
	"GMT+5:00":  "Asia/Karachi",
	"GMT+5:30":  "Asia/Kolkata",
	"GMT+5:45":  "Asia/Katmandu",
	"GMT+6:00":  "Asia/Dhaka",
	"GMT+6:30":  "Asia/Rangoon",
	"GMT+7:00":  "Asia/Bangkok",
	"GMT+8:00":  "Asia/Hong_Kong",
	"GMT+9:00":  "Asia/Tokyo",
	"GMT+9:30":  "Australia/Darwin",
	"GMT+10:00": "Australia/Brisbane",
	"GMT+11:00": "Asia/Magadan",
	"GMT+12:00": "Pacific/Wake",
	"GMT+13:00": "Pacific/Tongatapu"
};

// OlsonTimeZone name
ibm.tivoli.navmodel.Utils.timeZoneName = null;

ibm.tivoli.navmodel.Utils.setTimeZone = function(timeZoneName) {
	// summary:
	//		Set the timeZoneName that will be returned by getTimeZone()	
	ibm.tivoli.navmodel.Utils.timeZoneName = timeZoneName;
};

ibm.tivoli.navmodel.Utils.getTimeZone = function() {
	// summary:
	//		Calculate the users Olson timezone name
	
	// if one has been set; just return it
	if (ibm.tivoli.navmodel.Utils.timeZoneName) {
		return ibm.tivoli.navmodel.Utils.timeZoneName;
	}
	
	var janUTC = ibm.tivoli.navmodel.Utils.getUTCOffset(1, 1);
	var janGMT = ibm.tivoli.navmodel.Utils.getGMTOffset(1, 1);
	var julyUTC = ibm.tivoli.navmodel.Utils.getUTCOffset(7, 1);
	var julyGMT = ibm.tivoli.navmodel.Utils.getGMTOffset(7, 1);	
	var dst = (janUTC !== julyUTC);	
	var tzName = null;
	
	// DST
	if (dst) {
		if (janUTC < julyUTC) {
			tzName = ibm.tivoli.navmodel.Utils.NorthernHemisphere[janGMT];
		}
		else {
			tzName = ibm.tivoli.navmodel.Utils.SouthernHemisphere[julyGMT];
		}
	}
	// No DST
	else {
		tzName = ibm.tivoli.navmodel.Utils.NoDST[janGMT];
	}
	
	// can't find it in tables; so return GMT offset
	if (!tzName) {
		tzName = ibm.tivoli.navmodel.Utils.getGMTOffset();
	}
	
	//alert('jan=' + janGMT + '  july=' + julyGMT + ' dst=' + dst + ' tz=' + tzName);
		
	return tzName;
};

ibm.tivoli.navmodel.Utils.getUTCOffset = function(month, day) {
	// summary:
	//		Calculate the users current GMT offset
	var date = new Date();
	if (month) {
		date.setMonth(month);
	}
	if (day) {
		date.setDate(day);
	}
	
	return -(date.getTimezoneOffset());	
};

ibm.tivoli.navmodel.Utils.getGMTOffset = function(month, day) {
	// summary:
	//		Calculate the users current GMT offset
	var date = new Date();
	if (month) {
		date.setMonth(month);
	}
	if (day) {
		date.setDate(day);
	}
	
	var timezone = "";
	var minutesFromGMT = date.getTimezoneOffset();
	var extraMinutes = minutesFromGMT%60;
	var gmtHours = null;
	var gmtMinutes = "00";
		
	if (extraMinutes === 0) {
		gmtHours = -(minutesFromGMT/60);
	}
	else {
		gmtHours = -((minutesFromGMT-extraMinutes)/60);
		gmtMinutes = (Math.abs(extraMinutes)).toString();		
	}	
		
	timezone = "GMT";
	if (gmtHours !== 0) {
		if (gmtHours > 0) {
			timezone += "+";				
		}
		timezone += gmtHours;
		timezone += ":";
		timezone += gmtMinutes;
	}
		
	return timezone;
};
