/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.WidgetDef"); 
 
// includes
dojo.require("ibm.tivoli.navmodel.AbstractWidgetDef");


////////////////////////////////////
// WidgetDef
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.WidgetDef", ibm.tivoli.navmodel.AbstractWidgetDef, {
	// private members
	_itemStyles: null,
	_itemStatuses: null,
	_linkStyles: null,
	_linkStatuses: null,
	_relationshipStyles: null,
	_relationshipStatuses: null,
	
	// default styles and status
	_defaultItemStyle: null,
	_defaultItemStatus: null,
	_defaultLinkStyle: null,
	_defaultLinkStatus: null,
	_defaultRelationshipStyle: null,
	_defaultRelationshipStatus: null,
	
	/*=====
	constructor_kwArgs: function(baseUrl, provider, datasource, widget, definition, type, version) {
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// widget: String
		//		widget type in form <type>-<version>
		// definition: String
		//		widget definition id in form <id>-<version>
		// type: String
		//		definition type:  "user", "shared", or "base"
		// version: Float
		//		version used to communicate with rest service
	},
	=====*/ 
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {		
		// summary:
		//		Constructs WidgetDef object.
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		this._itemStyles = {};
		this._itemStatuses = {};
		this._linkStyles = {};
		this._linkStatuses = {};
		this._relationshipStyles = {};
		this._relationshipStatuses = {};
		
		this._defaultItemStyle = {
			"scalarImage": "[widget]/resources/common_assets/common_resource_icons/re64_unknownserver1.swf",
           	"scalarImageWidth": 61.011,
           	"scalarImageHeight": 53.596,
           	"smallImage": "[widget]/resources/common_assets/common_resource_icons/re64_unknownserver1_8.png",
           	"smallImageHeight": 16,
         	"smallImageWidth": 16,
         	"largeImage": "[widget]/resources/common_assets/common_resource_icons/re64_unknownserver1_8.png",
           	"largeImageHeight": 32,
         	"largeImageWidth": 32          	
		};
			
		this._defaultItemStatus = {
			"backgroundScalarImage": "[widget]\/resources\/common_assets\/status_icons\/BK_SWF\/unknownBk141x117gradient.swf",
            "backgroundScalarImageHeight": 117,
            "backgroundScalarImageWidth": 141,
            "backgroundScalarImageVisible": true,            
            "scalarImageHeight": 25,
            "scalarImageWidth": 25,
            "scalarImageVisible": true,
            "imageHeight": 16,
            "imageWidth": 16,
            "imageVisible": true
		};
		
		this._defaultLinkStyle = {
			"lineStyle": "solid",
			"lineWidth": 5, //For best results with link selection and seeing link flyovers on roll over, this is the minimum recommended width for links
			"lineColor": "#000000",
			"labelVisible": false,
			"labelAutoRotate": false,
			"labelParallelAlignment": "middle",
			"labelOrthogonalAlignment": "left",
			"labelDistance": 5,
			"labelPositionType": "relative",
			"labelPosition": 0.5
		};
		
		this._defaultLinkStatus = {            
            "scalarImageHeight": 25,
            "scalarImageWidth": 25,
            "scalarImageVisible": true,
            "imageHeight": 16,
            "imageWidth": 16,
            "imageVisible": true,
            "color": "#000000"
		};
		
		this._defaultRelationshipStyle = this._defaultLinkStyle;
		
		this._defaultRelationshipStatus = this._defaultLinkStatus;
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	destroy: function() {		
		// summary:
		//		Do any necessary clean up
		if (!this.isDestroyed()) {
			// call base class
			this.inherited(arguments);
		}
	},
	
	
	//////////////////
	// pluggableview
	//////////////////
	getPluggableView: function() {
		if (!this._widgetDef.pluggableview) {
			this._widgetDef.pluggableview = {};
		}
		
		return this._widgetDef.pluggableview;
	},
	
	//////////
	// views 
	//////////
	getViews: function(){
		var pluggableview = this.getPluggableView();
		return pluggableview.views;
	},
	
	////////////
	// toolbar 
	////////////
	getToolbar: function() {
		var pluggableview = this.getPluggableView();		
		var returnValue = pluggableview.toolbar;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getToolbar) {
				returnValue = parent.getToolbar();
			}
		}		
		return returnValue;	
	},
	
	getToolbarRows: function(){
		var toolbar = this.getToolbar();
		return (toolbar ? toolbar.rows : []);
	},
	
	isToolbarHidden: function() {
		var toolbar = this.getToolbar();
		var returnValue = (toolbar ? toolbar.hidden : undefined);
		if(returnValue!==undefined){
			return returnValue;
		} else {
			return false;
		}	
	},
	
	//////////////////
	// selectionMode 
	/////////////////
	getSelectionMode: function(){
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.selectionMode;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getSelectionMode){
			returnValue = this._parentWidgetDef.getSelectionMode();
		}		
		return returnValue;			
	},
	
	///////////////
	// labelStyle 
	//////////////
	getLabelStyle: function(){
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.labelStyle;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getLabelStyle){
			returnValue = this._parentWidgetDef.getLabelStyle();
		}		
		return returnValue;
	},
	
	///////////////
	// itemStyles 
	//////////////
	getItemStyles: function() {
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.itemStyles;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getItemStyles){
			returnValue = this._parentWidgetDef.getItemStyles();
		}		
		return returnValue;			
	},
	
	getItemStyle: function(type) {
		var style = this._itemStyles[type];
		
		if (!style) {
			var styles = this.getItemStyles();
			if (styles) {
				var styleLength = styles.length;
				for (var styleIndex = 0;  styleIndex < styleLength;  ++styleIndex) {
					if (styles[styleIndex].type === type) {
						style = styles[styleIndex];
						this._itemStyles[type] = style;
						break;
					}
				}
			}
		}
		
		return style;		
	},
	
	////////////////
	// itemStatuses 
	////////////////
	getItemStatuses: function() {
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.itemStatuses;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getItemStatuses){
			returnValue = this._parentWidgetDef.getItemStatuses();
		}		
		return returnValue;	
	},
	
	getItemStatus: function(id, value) {
		var key = id + "_" + value;
		var status = this._itemStatuses[key];
		
		if (!status) {
			var statuses = this.getItemStatuses();
			if (statuses) {
				var statusLength = statuses.length;
				for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
					if (statuses[statusIndex].id === id &&  statuses[statusIndex].value === value) {
						status = statuses[statusIndex];
						this._itemStatuses[key] = status;
						break;
					}
				}
			}
		}
		
		return status;
	},
	
	///////////////
	// linkStyles 
	///////////////
	getLinkStyles: function() {
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.linkStyles;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getLinkStyles){
			returnValue = this._parentWidgetDef.getLinkStyles();
		}		
		return returnValue;			
	},
	
	getLinkStyle: function(type) {
		var style = this._linkStyles[type];
		
		if (!style) {
			var styles = this.getLinkStyles();
			if (styles) {
				var styleLength = styles.length;
				for (var styleIndex = 0;  styleIndex < styleLength;  ++styleIndex) {
					if (styles[styleIndex].type === type) {
						style = styles[styleIndex];
						this._linkStyles[type] = style;
						break;
					}
				}
			}
		}
		
		return style;		
	},
	
	/////////////////
	// linkStatuses 
	////////////////
	getLinkStatuses: function() {
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.linkStatuses;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getLinkStatuses){
			returnValue = this._parentWidgetDef.getLinkStatuses();
		}		
		return returnValue;			
	},
	
	getLinkStatus: function(id, value) {
		var key = id + "_" + value;
		var status = this._linkStatuses[key];
		
		if (!status) {
			var statuses = this.getLinkStatuses();
			if (statuses) {
				var statusLength = statuses.length;
				for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
					if (statuses[statusIndex].id === id && statuses[statusIndex].value === value) {
						status = statuses[statusIndex];
						this._linkStatuses[key] = status;
						break;
					}
				}
			}
		}
		
		return status;
	},
		
	///////////////////////
	// relationshipStyles 
	///////////////////////
	getRelationshipStyles: function() {
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.relationshipStyles;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getRelationshipStyles){
			returnValue = this._parentWidgetDef.getRelationshipStyles();
		}		
		return returnValue;
	},
	
	getRelationshipStyle: function(type) {
		var style = this._relationshipStyles[type];
		
		if (!style) {
			var styles = this.getRelationshipStyles();
			if (styles) {
				var styleLength = styles.length;
				for (var styleIndex = 0;  styleIndex < styleLength;  ++styleIndex) {
					if (styles[styleIndex].type === type) {
						style = styles[styleIndex];
						this._relationshipStyles[type] = style;
						break;
					}
				}
			}
		}
		
		return style;		
	},
	
	/////////////////////////
	// relationshipStatuses 
	/////////////////////////
	getRelationshipStatuses: function() {
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.relationshipStatuses;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getRelationshipStatuses){
			returnValue = this._parentWidgetDef.getRelationshipStatuses();
		}		
		return returnValue;
	},
	
	getRelationshipStatus: function(id, value) {
		var key = id + "_" + value;
		var status = this._relationshipStatuses[key];
		
		if (!status) {
			var statuses = this.getRelationshipStatuses();
			if (statuses) {
				var statusLength = statuses.length;
				for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
					if (statuses[statusIndex].id === id && statuses[statusIndex].value === value) {
						status = statuses[statusIndex];
						this._relationshipStatuses[key] = status;
						break;
					}
				}
			}
		}
		
		return status;
	},

	getDefaultItemStyle: function() {
		return this._defaultItemStyle;
	},
	
	getDefaultLinkStyle: function() {
		return this._defaultLinkStyle;
	},
	
	getDefaultRelationshipStyle: function() {
		return this._defaultRelationshipStyle;
	},
		
	getDefaultItemStatus: function() {
		return this._defaultItemStatus;
	},
	
	getDefaultLinkStatus: function() {
		return this._defaultLinkStatus;
	},
	
	getDefaultRelationshipStatus: function() {
		return this._defaultRelationshipStatus;
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////
	
	_getTypeSection: function() {
		var type = this.getWidgetType();
		if (type.indexOf("-") > 0) {
			var typeInfo = type.split("-");
			type = typeInfo[0];    		        
        }
        
        return this.getSection(type);
	}
});
