/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.WidgetDelta"); 
 
// includes
dojo.require("ibm.tivoli.navmodel.Trace");

ibm.tivoli.navmodel.WidgetDelta._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.WidgetDelta");

////////////////////////////////////
// WidgetDelta
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.WidgetDelta", null, {
	// private members       
	_widgetDef: null,
	_type: null,
	_delta: null,
	_pending: false,	
	_trace: ibm.tivoli.navmodel.WidgetDelta._trace,

	/*=====
	 constructor_kwArgs: function(widgetDef, type) {
		// widgetDef: ibm.tivoli.navmodel.WidgetDef (or descendent)
		//		widget definition object to manipulate delta
		// type: String
		//		the delta type:  "user" or "shared"		
	},
	=====*/

	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		var args = {
			widgetDef: null,
			type: null
		};
		dojo.mixin(args, kwArgs);
		
		// summary:
		//		Constructs WidgetDelta object.
		this._widgetDef = args.widgetDef;
		this._type = (args.type ? args.type : this._widgetDef.getArg("type"));
		
		switch(this._type) {
			case "user":
				this._delta = this._widgetDef.getUserDelta();
				break;
			case "shared":
				this._delta = this._widgetDef.getSharedDelta();
				break;
			default:
				this._trace.error("Unknown delta type=" + this._type, "constructor");
		}
		
		if (this._delta) {
			this._delta = dojo.clone(this._delta);
		}
		else {
			this._delta = {};
		}		
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
		
	getType: function() {
		// summary:
		//		Return delta type: either "user" or "shared"
		return this._type;
	},
	
	hasPendingChanges: function() {
		// summary:
		//		Returns true if changes are pending
		return this._pending;
	},
	
	save: function() {
		// summary:
		//		Save current delta changes to the widget registry;
		//		delete the delta if it is empty.
		if (this.hasPendingChanges()) {
			switch(this._type) {
				case "user":
					if (this._hasKeys(this._delta)) {
						this._widgetDef.setUserDelta(this._delta);
					}
					else {
						this._widgetDef.deleteUserDelta();
					}
					break;
				case "shared":			
					if (this._hasKeys(this._delta)) {
						this._widgetDef.setSharedDelta(this._delta);
					}
					else {
						this._widgetDef.deleteSharedDeleta();
					}
			}
			this._pending = false;
		}
	},
	
	clear: function() {
		// summary:
		//		Clear the entire delta
		this._delta = {};
		this._pending = true;
	},
	
	/*=====
	 clearSection_kwArgs: function(sectionKey) {
	 	//	sectionKey: String
		//		The key that identifies the section		
	},
	=====*/
	
	clearSection: function(/*clearSection_kwArgs*/ kwArgs) {
		if (this._delta[kwArgs.sectionKey]) {
			delete this._delta[kwArgs.sectionKey];			
		}
		
		this._pending = true;
	},
	
	/*=====
	 sectionAttribute_kwArgs: function(sectionKey, attrKey) {
	 	//	sectionKey: String
		//		The key that identifies the section
		//	attrKey: String
		//		The attribute in the section
	},
	=====*/
	
	getSectionAttribute: function(/*sectionAttribute_kwArgs*/ kwArgs) {
		// summary:
		//		Get a key/value in a specified section
		//
		//	Example:
		//	{	...
		//		"topology: {
		//			"backgroundColor": "white",
		//			...
		//		},
		//		...
		//	}
		//	Get "backgroundColor" attribute in "topology" section 
		//	var mode = delta.getSectionAttribute({sectionKey: "topology", attrKey: "backgroundColor"});		
		var value = null;
		var section = this._delta[kwArgs.sectionKey];
		
		// section exists in delta
		if (section) {
			value = section[kwArgs.attrKey];						
		}
		
		return value;	
	},
	
	/*=====
	 setSectionAttribute_kwArgs: function(sectionKey, attrKey, attrValue) {
	 	//	sectionKey: String
		//		The key that identifies the section
		//	attrKey: String
		//		The attribute in the section
		//	attrValue: any
		//		The value to be set on the attribute
	},
	=====*/
		
	setSectionAttribute: function(/*setSectionAttribute_kwArgs*/ kwArgs) {
		// summary:
		//		Set a key/value in a specified section
		//
		//	Example:
		//	{	...
		//		"topology: {
		//			"backgroundColor": "white",
		//			...
		//		},
		//		...
		//	}
		//	Set "backgroundColor" attribute in "topology" section 
		//	delta.setSectionAttribute({sectionKey: "topology", attrKey: "backgroundColor", attrValue: "gray"});		
		var rc = false;
		var section = this._delta[kwArgs.sectionKey];
		
		// section exists in delta
		if (section) {
			section[kwArgs.attrKey] = kwArgs.attrValue;			
			rc = true;			
		}
		// section does not exist in delta
		else {
			section = {};
			section[kwArgs.attrKey] = kwArgs.attrValue;
			this._delta[kwArgs.sectionKey] = section;
			rc = true;
		}
		
		this._pending = true;
		return rc;		
	},
	
	removeSectionAttribute: function(/*sectionAttribute_kwArgs*/ kwArgs) {
		// summary:
		//		 Remove a key from a specified array element (where section is an array)
		//
		//	Example:
		//	{	...
		//		"topology: {
		//			"backgroundColor": "white",
		//			...
		//		},
		//		...
		//	}
		//	Remove "backgroundColor" attribute from "topology" section
		//	delta.removeSectionAttribute({sectionKey: "topology", attrKey: "backgroundColor"});				
		var rc = false;
		var section = this._delta[kwArgs.sectionKey];
		
		// section exists in delta
		if (section) {
			delete section[kwArgs.attrKey];
			rc = true;
			
			// if section is empty; we can remove from delta
			if (!this._hasKeys(section)) {
				delete this._delta[kwArgs.sectionKey];
			}					
		}
			
		this._pending = true;					
		return rc;		
	},
	
	/*=====
	 arraySectionAttribute_kwArgs: function(sectionKey, elementKey, elementValue, attrKey) {
	 	//	sectionKey: String
		//		The key that identifies the section
		//	elementKey: String
		//		The key used to identify the array element
		//	elementValue: String
		//		The value used to identify the array element (i.e. compare to elementKey)
		//	attrKey: String
		//		The attribute in the array element			
	},
	=====*/
		
	getArraySectionAttribute: function(/*arraySectionAttribute_kwArgs*/ kwArgs) {
		// summary:
		//		Get a key/value in a specified array element (where section is an array)
		//
		//	Example:
		//	{	...
		//		"tables: [
		//			{	"id": "ItemsTable",
		//				"filter": {
		//					...
		//				},
		//				...
		//			},
		//			...
		//		],
		//		...
		//	}
		//	Get "filter" attribute in "tables" section (i.e. the element where "id" is "ItemsTable")		
		//	var filter = delta.getArraySectionAttribute({sectionKey: "tables", elementKey: "id", elementValue: "ItemsTable", attrKey: "filter"});
		
		var value = null;
		var section = this._delta[kwArgs.sectionKey];
		
		// array section exists in delta
		if (section && dojo.isArray(section)) {
			var count = section.length;
			for (var index = 0;  index < count;  ++index) {
				var element = section[index];
				if (element[kwArgs.elementKey] === kwArgs.elementValue) {
					value = element[kwArgs.attrKey];
					break;
				}
			}				
		}
		
		return value;	
	},
		
	/*=====
	 setArraySectionAttribute_kwArgs: function(sectionKey, elementKey, elementValue, attrKey, attrValue) {
	 	//	sectionKey: String
		//		The key that identifies the section
		//	elementKey: String
		//		The key used to identify the array element
		//	elementValue: String
		//		The value used to identify the array element (i.e. compare to elementKey)
		//	attrKey: String
		//		The attribute to be modified in the array element
		//	attrValue: any
		//		The value to be set on the array element attribute
	},
	=====*/
	
	setArraySectionAttribute: function(/*setArraySectionAttribute_kwArgs*/ kwArgs) {
		// summary:
		//		Set a key/value in a specified array element (where section is an array)
		//
		//	Example:
		//	{	...
		//		"tables: [
		//			{	"id": "ItemsTable",
		//				"filter": {
		//					...
		//				},
		//				...
		//			},
		//			...
		//		],
		//		...
		//	}
		//	Set "filter" attribute in "tables" section (i.e. the element that has "id" = "ItemsTable")		
		//	delta.setArraySectionAttribute({sectionKey: "tables", elementKey: "id", elementValue: "ItemsTable", attrKey: "filter", attrValue: value});
		
		var rc = false;
		var section = this._delta[kwArgs.sectionKey];
		
		// array section exists in delta
		if (section && dojo.isArray(section)) {
			var count = section.length;
			for (var index = 0;  index < count;  ++index) {
				var element = section[index];
				if (element[kwArgs.elementKey] === kwArgs.elementValue) {
					element[kwArgs.attrKey] = kwArgs.attrValue;
					rc = true;
					break;
				}
			}				
		}
		// array section does not exist in delta
		else {
			var newElement = {};
			newElement[kwArgs.elementKey] = kwArgs.elementValue;
			newElement[kwArgs.attrKey] = kwArgs.attrValue;
			this._delta[kwArgs.sectionKey + "Identifier"] = kwArgs.elementKey;			
			this._delta[kwArgs.sectionKey] = [newElement];
			rc = true;
		}
		
		this._pending = true;					
		return rc;		
	},
	
	removeArraySectionAttribute: function(/*arraySectionAttribute_kwArgs*/ kwArgs) {
		// summary:
		//		 Remove a key from a specified array element (where section is an array)
		//
		//	Example:
		//	{	...
		//		"tables: [
		//			{	"id": "ItemsTable",
		//				"filter": {
		//					...
		//				},
		//				...
		//			},
		//			...
		//		],
		//		...
		//	}
		//	Remove "filter" attribute from "tables" section (i.e. the element that has "id" = "ItemsTable")		
		//	delta.removeArraySectionAttribute({sectionKey: "tables", elementKey: "id", elementValue: "ItemsTable", attrKey: "filter"});
				
		var rc = false;
		var section = this._delta[kwArgs.sectionKey];
		
		// array section exists in delta
		if (section && dojo.isArray(section)) {
			var count = section.length;
			for (var index = 0;  index < count;  ++index) {
				var element = section[index];
				if (element[kwArgs.elementKey] === kwArgs.elementValue) {					
					delete element[kwArgs.attrKey];
					rc = true;
					
					// if the only key left in the array element is the
					// elementKey; then we can remove this element
					var keys = this._getKeys(element);
					if (keys.length === 1 && keys[0] === kwArgs.elementKey) {
						section.splice(index,1);
					}
					
					// if array is empty; we can remove from the delta
					if (section.length === 0) {
						delete this._delta[kwArgs.sectionKey];
						delete this._delta[kwArgs.sectionKey + "Identifier"];
					}					
					break;
				}
			}				
		}
				
		this._pending = true;						
		return rc;		
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////
	_hasKeys: function(obj) {
		for (var key in obj) {
			if (obj.hasOwnProperty(key)) {
				return true;
			}
		}
		
		return false;
	},
	
	_getKeys: function(obj) {
		var keys = [];		
		for (var key in obj) {
			if (obj.hasOwnProperty(key)) {
				keys.push(key);
			}
		}
		return keys;
	}
});
