function ibm_tivoli_navmodel_EventRouter() {}

// data members
ibm_tivoli_navmodel_EventRouter.initialized = false;
ibm_tivoli_navmodel_EventRouter.terminated = false;
ibm_tivoli_navmodel_EventRouter.timeout = 20;
ibm_tivoli_navmodel_EventRouter.pollDelay = 0;
ibm_tivoli_navmodel_EventRouter.pollErrorDelay = 5000;
ibm_tivoli_navmodel_EventRouter.contextRoot = "/ibm/tivoli/micro";
ibm_tivoli_navmodel_EventRouter.eventDestination = null;
ibm_tivoli_navmodel_EventRouter.registry = {};
ibm_tivoli_navmodel_EventRouter.connected = true;


/*
 * Returns true if already initialized
 */
ibm_tivoli_navmodel_EventRouter.isInitialized = function() {
	return this.initialized;
};


/*
 * Initializes connection with event feed
 */
ibm_tivoli_navmodel_EventRouter.init = function(args) {
	if (!this.initialized) {
		this.info("***** event_router - initializing");
		this.initialized = true;
		
		this.eventDestination = args.eventDestination;
		
		if (args.timeout !== undefined) {
			this.timeout = args.timeout;
		}
		
		if (args.pollDelay !== undefined) {
			this.pollDelay = args.pollDelay;
		}
		
		if (args.pollErrorDelay !== undefined) {
			this.pollErrorDelay = args.pollErrorDelay;
		}
		
		if (args.contextRoot !== undefined) {
			this.contextRoot = args.contextRoot + "/micro";
		}	
	
		// start polling for events
		this.sendPoll();
	}	
};


/*
 * Terminates connection with event feed
 */
ibm_tivoli_navmodel_EventRouter.terminate = function() {
	if (this.initialized) {							
		this.info("***** event_router - terminating");
		this.terminated = true;
		this.initialized = false;			
	}
};


/*
 * Adds listener for the requested eventId
 */
ibm_tivoli_navmodel_EventRouter.addListener = function(listener, eventId, eventCallback) {
	// add entry if it doesn't already exist
	if (!this.registry[eventId]) {		
		this.registry[eventId] = {listeners: []};
	}
		
	// append listener to the entry
	var listenersLength = this.registry[eventId].listeners.length;
	this.info("***** event_router - adding listener for eventId=" + eventId + " at index=" + listenersLength);		
	this.registry[eventId].listeners[listenersLength] = {listener: listener, eventCallback: eventCallback};	
	
	// validate listeners
	this.validate();
};


/*
 * Removes listener for the requested eventId
 */
ibm_tivoli_navmodel_EventRouter.removeListener = function(listener, eventId) {
	var listenersLength = 0;
	
	if (this.registry[eventId]) {
		listenersLength = this.registry[eventId].listeners.length;
		for (var listenerIndex = 0;  listenerIndex < listenersLength;  ++listenerIndex) {
			// remove listener from entry
			if (this.registry[eventId].listeners[listenerIndex].listener === listener) {
				this.info("***** event_router - remove listener for eventId=" + eventId + " at index=" + listenerIndex);
				this.registry[eventId].listeners.splice(listenerIndex, 1);
					
				// remove entry if no more listeners
				listenersLength = this.registry[eventId].listeners.length;
				if (listenersLength === 0) {
					delete this.registry[eventId];
					
					// no more listeners so shut down event feed
					if (!this.haveListeners()) {
						this.terminate();
					}					
				}					
				break;
			}
		}
	}	
		
	return listenersLength;		
};


/*
 * Returns registered listener count
 */
ibm_tivoli_navmodel_EventRouter.getListenerCount = function() {
	var listeners = 0;
	
	for (var eventId in this.registry) {
		if (this.registry.hasOwnProperty(eventId)) {
			var listenerLength = this.registry[eventId].listeners.length;				
			listeners += listenerLength;		
		}
	}
	
	return listeners;
};


/*
 * Returns true if we have at least one listener
 */
ibm_tivoli_navmodel_EventRouter.haveListeners = function() {
	for (var eventId in this.registry) {
		if (this.registry.hasOwnProperty(eventId)) {
			var listenerLength = this.registry[eventId].listeners.length;				
			if (listenerLength > 0) {
				return true;
			}		
		}
	}
	
	return false;
};


/*
 * Validate registered liseners; remove any that can no longer be pinged
 */
ibm_tivoli_navmodel_EventRouter.validate = function() {
	for (var eventId in this.registry) {
		if (this.registry.hasOwnProperty(eventId)) {
			var listenersLength = this.registry[eventId].listeners.length;				
			for (var listenerIndex = listenersLength - 1;  listenerIndex >= 0;  --listenerIndex) {
				var listener = this.registry[eventId].listeners[listenerIndex].listener;
				try {
					listener.ping();								
				}
				catch (e) {	
					this.warn("***** event_router - validate is removing invalid listener ");
					this.removeListener(listener, eventId);					
				}				
			}
		}
	}
};


/*
 * Routes events to the listeners
 */
ibm_tivoli_navmodel_EventRouter.routeEvent = function(jsonEvent) {
	try {
		var event = eval("(" + jsonEvent + ")"); 
		var eventId = event.id;
			
		if (this.registry[eventId]) {
			var listenersLength = this.registry[eventId].listeners.length;						
			for (var listenerIndex = listenersLength - 1;  listenerIndex >= 0;  --listenerIndex) {
				try {
					var eventCallback = this.registry[eventId].listeners[listenerIndex].eventCallback;				
					eventCallback(event);									
				}
				catch (e1) {	
					// see if listener is still pingable				
					var listener = this.registry[eventId].listeners[listenerIndex].listener;
					try {
						listener.ping();
						this.error("***** event_router - Exception when routing eventId=" + eventId + " exception=" + e1);					
					}
					catch (e2) {
						// listener is dead we must remove it
						this.warn("***** event_router - routeEvent is removing invalid listener");
						this.removeListener(listener, eventId);
					}
				}				
			}
		}
	}
	catch (e3) {
		this.error("***** event_router - unexpected exception=" + e3 + " when routing event");
	}
};


/*
 * Routes connection state to listeners
 */
ibm_tivoli_navmodel_EventRouter.routeStatus = function(connected) {
	this.connected = connected;
	
	for (var eventId in this.registry) {
		if (this.registry.hasOwnProperty(eventId)) {
			var listenerLength = this.registry[eventId].listeners.length;				
			for (listenerIndex = 0;  listenerIndex < listenerLength;  ++listenerIndex) {
				try {
					var event = {id: eventId, type: "eventRouter", eventRouterStatus: connected};
					var eventCallback = this.registry[eventId].listeners[listenerIndex].eventCallback;				
					eventCallback(event);									
				}
				catch (e1) {	
					// see if listener is still pingable				
					var listener = this.registry[eventId].listeners[listenerIndex].listener;
					try {
						listener.ping();
						this.error("***** event_router - Exception when routing eventId=" + eventId + " exception=" + e1);					
					}
					catch (e2) {
						// listener is dead we must remove it
						this.warn("***** event_router - routeEvent is removing invalid listener");
						this.removeListener(listener, eventId);
					}
				}				
			}	
		}
	}
};


/*
 * Send request to servlet for next available event
 */
ibm_tivoli_navmodel_EventRouter.sendPoll = function() {
	if (!this.terminated) {		
		var request;
    	if (window.XMLHttpRequest) {
        	// Mozilla or IE7+
        	request = new XMLHttpRequest();
    	} else if (window.ActiveXObject) {
        	// IE6
        	request = new ActiveXObject("Microsoft.XMLHTTP");
    	}
    
    	var that = this;
    	request.onreadystatechange = function() {
    		try {
    			// If the request's status is "complete"
            	if (request.readyState === 4) {
            		var nextPoll = this.pollDelay;
            		switch(request.status) {
            			case 200:
            				// connection is back up
            				if (!that.connected) {
            					that.routeStatus(true);
            				}
            				
            				that.routeEvent(request.responseText);            				
            				break;
            			case 504:
            				// normal microbroker timeout (i.e. no messages)
            				
            				// connection is back up
            				if (!that.connected) {
            					that.routeStatus(true);
            				}
            				break;
            			case 0:
            				// can get this when request is killed
            				that.error("***** event_router - HTTP connection is unavailable");
            				
            				// connection is down
            				if (that.connected) {
            					that.routeStatus(false);
            				}
            				
            				nextPoll = that.pollErrorDelay;
            				break;
            			default:            			
            				that.error("***** event_router - HTTP error=" + request.status + " responseText=" + request.responseText);
            				
            				// connection is down
            				if (that.connected) {
            					that.routeStatus(false);
            				}
            				         			
            				nextPoll = that.pollErrorDelay; 
            		} 
            		
            		setTimeout(function() {that.sendPoll();}, nextPoll);           	            		 
            	}
        	} 
        	catch (ex) {
            	that.error("***** event_router - unexpected exception=" + ex + " when handling response");
       		}
    	};

		var now = new Date();		
		request.open("DELETE", this.contextRoot + "/msg/queue/" + this.eventDestination + "?d=" + now.getTime() + "&r=" + Math.random(), true);
    	request.setRequestHeader("x-msg-wait", this.timeout * 1000);    
    	request.send(null);
    }
    else {
    	this.terminated = false;
    }
};


/*
 * Log info message
 */
ibm_tivoli_navmodel_EventRouter.info = function(msg) {
	if (typeof console !== 'undefined') {
		console.info(msg);
	}	
};


/*
 * Log warning message
 */		
ibm_tivoli_navmodel_EventRouter.warn = function(msg) {
	if (typeof console !== 'undefined') {
		console.warn(msg);
	}
};


/*
 * Log error message
 */	
ibm_tivoli_navmodel_EventRouter.error = function(msg) {
	if (typeof console !== 'undefined') {
		console.error(msg);
	}
};
