/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2010 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/

dojo.provide("ibm.tivoli.tip.charts.dijit.Builder");
dojo.require("ibm.tivoli.tip.dojo.EventUtil");

dojo.declare("ibm.tivoli.tip.charts.dijit.Builder",
	          null,
{
	// Note: See https://w3.tap.ibm.com/w3ki03/display/TIP/Event%20Names
	//  and  https://w3.tap.ibm.com/w3ki03/display/TIP/Advanced%20Content%20Passing
	//  for information about this design
	global: null,
	// e.g. mssName: "ibm-cdm:///CDMMSS/Hostname=loco.raleigh.ibm.com+ManufacturerName=IBM+ProductName=IBM Tivoli Monitoring Services"
	mssName_: null,
	matchOn_: null,
	contextType_: null,
	event_: null,
	
	processEvent: function (event) {
		console.log ("enter Builder.processEvent");
		console.dir (event);
		try {
			if (event.name == "http://ibm.com/isclite#portletResize") {
				// handled outside of this code
			} else if (event.name == "http://ibm.com/tip#NodeClickedOn") {
				this.handleNodeClickedOnEvent (event);
			} else if (event.name == "http://ibm.com/tip#TimeSet") {
				this.handleTimeSetEvent (event);
			} else if (event.name == "http://ibm.com/isclite#launchPage") {
				this.handleNodeClickedOnEvent (event);
			} else if (event.name == "http://ibm.com/isclite#switchPage") {
				this.handleNodeClickedOnEvent (event);
			} else if (event.name == "http://ibm.com/isclite#launchPortlet") {
				console.log ("handling launchPortlet event");
			} else if (event.name == "http://ibm.com/tip#changeTheme") {
				console.log ("received changeTheme event");
				this.changeTheme(event);
			} else {
				console.log ("received unknown event: ");
				console.dir (event);
			}
		} catch (e) {
			console.error ("caught exception in processEvent");
			console.error (e);
		}
		console.log ("exit Builder.processEvent");
	},
	changeTheme: function (event) {
		// default is do nothing.  override if needed
	},
	handleNodeClickedOnEvent: function (event) {
		console.log ("enter Builder.handleNodeClickedOnEvent");

		if ((this.global.builder.widgetDefinition.datasets[0].supportWires === undefined) ||
		    (this.global.builder.widgetDefinition.datasets[0].supportWires)) {
		    console.log ("supportsWire is set to true. processing event: " + this.global.builder.widgetDefinition.datasets[0].supportsWire);
			this.event_ = null;
			if (this.mssName_ === null) {
				console.log ("mssName is null... fetching it: " + this.mssName_);
				this.event_ = event;
				this.fetchMSSName(false, dojo.hitch(this, this.afterFetchMSSName));
			} else {
				console.log ("already retreived mssName. it is: " + this.mssName_);
				this.matchMSSName(event);
			}
		} else {
			console.info ("via widget definition, wiring is off for this chart widget. ignoring event.");
		}
	},
	afterFetchMSSName: function (data) {
      	console.log ("return from fetching mss name: " + data.MSSName);
		this.mssName_ = data.MSSName;
		this.matchOn_ = data.matchOn;
		this.contextType_ = data.contextType;
		this.matchMSSName (this.event_);
		this.event_ = null;
	},
	matchMSSName: function (event) {							
		console.log ("enter matchMSSName"); 
//		var matchingMSSIndex = this.getMatchingMSSIndex (event, this.getSourceMSSName());		
			// alert ("matchingMSSIndex: " + matchingMSSIndex)
//			if (matchingMSSIndex < 0) {
				// no matching MSS found, so call to transform
//				event = this.transformEvent (event);
//			} else {
				// matching MSS found, so no need to transform, 
				// just invoke post transform processing
		this.onTransformEnd (event);
//			}
//		} else {
//			console.log ("receive context is off or not determined yet. cacheing event.");
//			this.event_ = event;
//		}
	
	},
	getMatchingMSSIndex : function (event, srcMssName) {
		console.log ("src MSS: " + srcMssName);
		var index = -1;
		
		if (this.equalsIgnoreCase (event.MSSName, srcMssName)) {
			index = 0;
		} else {
			var i = 1;
			console.log ("entering for loop");
			forloop:
			for (i=1; ; i++) {
				console.log ("in for loop with index: " + i);
				var value = event["MSSName" + i];
				console.log ("event.MSSName" + i + ": " + value);
				if (value) {
					if (this.equalsIgnoreCase (value, srcMssName)) {
						index = i;
						break forloop;
					}
				} else {
					break forloop;
				}
			}
		}
		
		console.log ("returning matching mss index: " + index);
		return (index);
	},
	equalsIgnoreCase: function (str1, str2) {
		if ((str1) && (str2)) {
			if ((str1 === undefined) || (str2 === undefined)) {
				return false;
			}
			if ((str1 === null) && (str2 === null)) {
				return (true);
			} else if ((str1 === null) || (str2 === null)) {
				return (false);
			} else {
				return (str1.toLowerCase() == str2.toLowerCase());
			}
		} else {
			return (false);
		}
	},	
	transformEvent: function (event) {
		console.log ("enter transformEvent with event:");
		console.dir (event);
		var transformInfo = {
                              "uniqueName": "com.ibm.tivoli.tip.TIPTransformation",
                              "locationUniqueName": "com.ibm.tivoli.tip.TIPTransformation",
                              "eventObj": event,
                              "srcPortletNamespace": this.global.iscPortletNamespace,
                              "handler": dojo.hitch (this, this.onTransformEnd)
                            };
        EventBroker.executeTransform(transformInfo);
	},
	onTransformEnd: function (event) {
		console.log ("enter onTransformEnd with event: ");
		console.dir (event);
		
		// check for multiple resources and copy first resource to root of event
		// We do not support multiple resources yet, just 1
		var util = new ibm.tivoli.tip.dojo.EventUtil();
		util.addOldContext (event);
		console.log ("after adding old context to event: ");
		console.dir (event);
		
		var matchingMSSIndex = this.getMatchingMSSIndex (event, this.getSourceMSSName());
		if (matchingMSSIndex > 0) {
			console.log("MSSName: " + event["MSSName" + matchingMSSIndex]);
			console.log ("SourceToken: " + event["SourceToken" + matchingMSSIndex]);

			if ((event["MSSName" + matchingMSSIndex] !== undefined) &&
			 	(event["MSSName" + matchingMSSIndex] !== null)) {
				event.MSSName = event["MSSName" + matchingMSSIndex];
			}
			if ((event["SourceToken" + matchingMSSIndex] !== undefined) &&
			 	(event["SourceToken" + matchingMSSIndex] !== null)) {
				event.SourceToken = event["SourceToken" + matchingMSSIndex];
			}
			
			console.log ("event: ");
			console.dir (event);
		}
		
		this.updateChart (event);
		/*
		// Update properties
		if (this.global.builder.isChart == true) {
			this.updateChart (event);
		} else if (this.global.builder.isTable == true) {	
			//rebuilds the table
			this.global.builder.updateTableData (event);
		} else {
			console.info ("Builder.onTransformEnd: no method for update");
		} 
		*/
	},
	fetchMSSName: function (sync, callback) {
		console.log ("enter fetchMSSName (sync, callback): " + sync + ", " + callback);
		if (this.mssName_ !== null) {
			console.log ("returning mssName_: " + this.mssName_); 
			return (this.mssName_);
		}
		
		var encodedProviderID = encodeURIComponent(this.global.builder.providerID);	
		var kw = {
        			  url: "/ibm/tivoli/rest/providers/" + encodedProviderID,
					  handleAs: "json",
    				  mimetype: "application/json",
                      error: dojo.hitch (this, function (error) {											
							var errHandler = new ibm.tivoli.tip.charts.dojo.RESTErrorHandler(this.global);		
							errHandler.showError(dojo.fromJson(error.responseText));	
		 			  })
                   };
          if (sync) {
          	kw.sync = sync;
          }

          if (callback) {
          	kw.load = callback;
          } else {
          	kw.load = dojo.hitch (this, function (data) {
          		this.mssName_ = data.MSSName;
          		this.matchOn_ = data.matchOn;
          		this.contextType_ = data.contextType;
          	});
          }

          dojo.xhrGet ( kw );
          
          // this is useful when calling synchronously
          return (this.mssName_);
	},
	getSourceMSSName: function () {
		if (this.mssName_ == null) {
			console.log ("getSourceMSSName: mssName is null... fetching it: " + this.mssName_);
			this.fetchMSSName(true);
		}
	
		//	e.g., "ibm-cdm:///CDMMSS/Hostname=pico.raleigh.ibm.com,ManufacturerName=IBM,ProductName=IBM Tivoli Monitoring"
		return this.mssName_;
	},
	eventParametersToRestParameters: function (event) {
		console.log ("enter Builder.eventParametersToRestParameters: ");
		if (event === undefined) {
			return (null);
		}
		
		var restParams = [];
		
		if(event.payload && event.payload.product){
			if(event.payload.cdm && event.payload.cdm.attributes){
				restParams.push ({
					id: "ClassType",
					value: event.payload.cdm.attributes.ClassType
				});
			}
			for (x in event.payload.product){				
				for(y in event.payload.product[x]){					
					if(y == "cdm"){
						for(z in event.payload.product[x][y]){
							restParams.push ({
								id: z,
								value: event.payload.product[x][y][z]
							});
						}
					}else{
						restParams.push ({
							id: y,
							value: event.payload.product[x][y]
						});
					}					
				}
			}			
		} 

		// we should always use these base event parameters as REST parameters				
		var param = null;
		
		for (var i in event) {
			if(i!="name" && i!="cdm" && i!="payload" && i!="product"){
				param = {};
				param.id = i;
				param.value = event[i];
				restParams.push (param);
			}
		}

		console.log ("exit Builder.eventParametersToRestParameters: ");
		console.log (restParams);
		
		return (restParams);
	},
	
	handleTimeSetEvent: function(event){		
		if(event.payload){
			this.updateChartByTimeSet(event.payload.Time);
		}else{
			this.updateChartByTimeSet(event.Time);
		}
	},	
	
	createCMSEventCallback: function(data){
		var event = {
			name: data.event,
			payload: {}
		};
		var mssName = null;
		var sourceToken = null;
		var cdmClass = null;		
		var contextData = {};
		for(var i=0;i<data.parameters.length;i++){			
			var param = data.parameters[i];			
			if(param.name=="SourceToken"){
				sourceToken = param.value.toString();
				if (sourceToken === "{SourceToken}") {
					sourceToken = null;
				}
			}else if(param.name=="MSSName"){
				mssName = param.value.toString();
				if (mssName === "{MSSName}") {
					mssName = null;
				}
			}else if(param.name=="ClassType"){
				cdmClass = param.value.toString();
			}else{
				contextData[param.name] = param.value.toString();
			}
		}
		if ((cdmClass) && (cdmClass !== null)) {
			event.payload.cdm = {
				attributes: {
					ClassType: cdmClass
				}
			};
		}
		if ((mssName) && (mssName !== null)) {	
			event.payload.product = {};
			var productName = mssName.substring(mssName.lastIndexOf('=')+1).replace(/ /g,"");
			event.payload.product[productName] = {
				cdm: {
					MSSName: mssName
				}
			};
			if((sourceToken) && (sourceToken !== null)) {
				event.payload.product[productName].cdm.SourceToken = sourceToken;
			}
			//  do not mixin generic properties under the productName... Only include SourceToken there (i.e., things specific to that MSSName)
			// dojo.mixin(event.payload.product[productName], contextData);
		}
		dojo.mixin(event.payload, contextData);
		
		return event;
	},		
	
	updateChart: function (event) {
		console.log ("enter Builder.updateChart");				
		var widgetDef = this.global.builder.widgetDefinition;		

		//To restore parameters in widgetDef
		//var oldParams = dojo.clone(widgetDef.datasets[0].parameters);
	
		// We are NOT implementing matchOn provider argument in this release... so commenting this out.
		// Beware that the matchOn code was NOT working anyway	
//		if ((this.matchOn_ !== null) && (this.matchOn_ !== undefined) && (this.matchOn_ === "all")) {
//			console.log ("provider matchOn is all... sending event context as JSON object");
//			widgetDef.datasets[0].parameters.eventContext = dojo.toJson(event);
//		} else {
			console.log ("using default value for provider matchOn. flattening parameters for REST call.");
			var newParams = this.eventParametersToRestParameters(event);
			if (newParams !== null) {
				//widgetDef.datasets[0].parameters = newParams;
				for(var i=0; i<widgetDef.datasets[0].parameters.length; i++){
					for(var j=0; j<newParams.length; j++){
						if(widgetDef.datasets[0].parameters[i].id == newParams[j].id){
							widgetDef.datasets[0].parameters[i] = newParams[j];
							newParams.splice(j,1);
							break;
						}
					}
				}
				widgetDef.datasets[0].parameters = widgetDef.datasets[0].parameters.concat(newParams);
			}
//		} 
		
		var builder = this.global.builder;			
		/* duplicate encoding
		var encodedCID = null;
		if (this.global.builder.isChart == true) {			
			encodedCID = encodeURIComponent(builder.chartID);
		}
		else if (this.global.builder.isTable == true) {			
			encodedCID = encodeURIComponent(builder.tableID);
		}
		var encodedProviderID = encodeURIComponent(builder.providerID);		
		var encodedDS = encodeURIComponent(builder.datasourceID);
		var encodedPID = encodeURIComponent(this.global.chartProfileID);
		*/
		var widgetID = null;
		if (this.global.builder.isChart == true) {			
			widgetID = builder.chartID;
		}else if (this.global.builder.isTable == true) {			
			widgetID = builder.tableID;
		}
		var providerID = builder.providerID;
		var datasourceID = builder.datasourceID;
		var profileID = this.global.chartProfileID;
		
		var drillUpItems = builder.drillUpItems;
		//show busyPane
		var busyPane = dijit.byId(this.global.namespace+"busy_pane");
	    if ( busyPane ) {	      
	      busyPane.show(true); // size to portlet
		}	
		if(!this.global.widgetFetcher){
			this.global.widgetFetcher = new ibm.tivoli.tip.charts.dojo.WidgetFetcher ({
				global: this.global
			});
		}	
		var fetcher = this.global.widgetFetcher;
		builder.destroyRecursive();		
		var domNode = dojo.byId (this.global.namespace+"widgetDiv");		
		dojo.destroy(domNode);
		domNode = dojo.doc.createElement("div");
		domNode.id = this.global.namespace+"widgetDiv";
		dojo.byId (this.global.namespace + "widgetContainer").appendChild(domNode);
		/* Separate Dojo Chart from Birt
		var domNode = dojo.byId (this.global.namespace+"widget");
		builder.destroy();		
		dojo.empty(domNode);
		dojo.place (domNode, dojo.byId (this.global.namespace + "birtChartWidget"), "after");			
		*/
		// There is no need to read user or shared preferences from the embedded JSON
		// Those values are merged into the base widget def by CURI before it is returned
		// from the server.
		var chartType = widgetDef.details.preferredView;

		if (chartType == "Table") {			
			this.global.builder = fetcher.buildTable (providerID, datasourceID, widgetDef, domNode, widgetID, null, builder.isTableFirst, drillUpItems);
			this.global.builder.mssName_ = this.mssName_;						
		} else {
			this.global.builder = fetcher.buildChart (providerID, datasourceID, widgetDef, widgetID, domNode, null, drillUpItems);
			this.global.builder.mssName_ = this.mssName_;			
		}		
		//this.global.builder.widgetDefinition.datasets[0].parameters = oldParams;
	}
});
