/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2009 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/

dojo.provide("ibm.tivoli.tip.charts.dijit.ChartPager");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Toolbar");
dojo.require("dijit.form.Button");

// NLS
dojo.requireLocalization("ibm.tivoli.tip.dijit", "resources");

dojo.declare("ibm.tivoli.tip.charts.dijit.ChartPager",
              [ dijit._Widget, dijit._Templated],
{
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("ibm.tivoli.tip.charts.dijit","templates/ChartPager.html"),
	chartFetcher: null,
	totalRows: null,
	
	constructor: function () {
		this.currentPage = 0;
		this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");    
	},
	postCreate: function () {
		this.totalRows = this.chartFetcher.getTotalRows();
		this.updateFooter();
		this.configureTextBox();
	},
	configureTextBox: function(){
		var consts = {};
		consts.min = 1;
		consts.max = this.getMaxPages();
		this.page_number.attr('constraints', consts);
		//this.page_number.attr('value', 1);
	},
	removeCharts: function() {
		if(this.chartDef.type =="Pie"){
			var div = dojo.byId (this.chartBuilder.id + "chart0");			
			div.removeChild(div.firstChild);			
			this.chartBuilder.chart = null;
			for(var i=0; i< this.chartBuilder.charts.length;i++){
				this.chartBuilder.charts[i].destroy();
			}
		}
	},
	jumpPage: function (event) {
		var pageNumber = this.page_number.attr('value');
		if (!isNaN(pageNumber)){
			if(pageNumber > this.getMaxPages()){
				this.currentPage = this.getMaxPages()-1;
			}else if(pageNumber<=0){
				this.currentPage = 0;
			}else{
				this.removeCharts();
				this.currentPage = pageNumber - 1;
				var start = this.currentPage * this.chartFetcher.rowsPerPage;
				this.chartFetcher.fetchData (start, this.chartFetcher.rowsPerPage);
				this.updateFooter();
			}
		}
	},
	updatePageNumber: function () {
		this.page_number.attr('value', this.currentPage+1);
	},
	getMaxPages: function () {
		
		
		return (Math.ceil (this.totalRows / this.chartFetcher.rowsPerPage));
	},
	getCurrentPage: function () {
		return (this.currentPage);
	},
	updateFooter: function() {
		//this.setInfoAreaHTML( dojo.string.substitute( this.resources_.PAGE_X_OF_Y, { 0: this.currentPage + 1, 1: this.getMaxPages() } ) );
		this.setPageLabelHTML( dojo.string.substitute( this.resources_.PAGE_LABEL ) );		
		this.setTotalPageHTML( dojo.string.substitute( this.resources_.PAGE_OF_Y, { 0: this.getMaxPages() } ) );
		this.setRowsShowRateHTML( dojo.string.substitute( this.resources_.ROW_SHOW_RATE, { 0: this.chartFetcher.rowsPerPage, 1: this.totalRows }) );
	},
	setPageLabelHTML: function (html) {
		this.pageLabel.innerHTML = html;
	},
	setTotalPageHTML: function (html) {
		this.totalPage.innerHTML = html;
	},
	setRowsShowRateHTML: function (html) {
		this.rowsShowRate.innerHTML = html;
	},
	setInfoAreaHTML: function (html) {
		this.infoArea.innerHTML = html;
	},
	onRefresh: function () {
		var start = this.currentPage * this.chartFetcher.rowsPerPage;
		this.chartFetcher.fetchData (start, this.chartFetcher.rowsPerPage);
	}
});
