dojo.provide("ibm.tivoli.tip.charts.dijit.ChartVisualization");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.data.ItemFileWriteStore");
dojo.require("dijit.form.FilteringSelect");
dojo.require("dojox.grid.DataGrid");
dojo.require("ibm.tivoli.tip.charts.dijit.LabelSelector");
dojo.require("dijit.form.TextBox");

dojo.declare("ibm.tivoli.tip.charts.dijit.ChartVisualization",[dijit._Widget,dijit._Templated],{
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("ibm.tivoli.tip.charts.dijit","templates/ChartVisualization.html"),	
	SEARCH_ATTR: "label",	
	widgetDef: null,
	_dataStore: null,
	_labelStore: null,
	_gridStore: null,
	_buttonDisa: null,
	_conns: null,
	_xAxisLabelValue: null,
	_yAxisLabelValue: null,
	_chartLabelValue: null,
	
	selectorLabel: null,	
	gridHeaders: null,	
	
	chartType: null,
	
	constructor: function(){		
		this._resources = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");	
		this.chartType = null;
		this._conns = [];
		this._xAxisLabelValue = null;
		this._yAxisLabelValue = null;
		this._chartLabelValue = null;
	},
	
	_buildSingleGridStore: function(){
		var store = dojo.clone(this._gridStore);
		if(this.chartType === "dojot.visual.ColumnsEmbossed" || this.chartType === "dojot.visual.BarsEmbossed"){
			store.items = [store.items[0]];			
		}
		return store;		
	},
	
	_updateLabelSelector: function(){
		//set current value for label selector		
		if(this.widgetDef && this.widgetDef.charts){
			this.chartLabelSelector.attr("value",this.widgetDef.charts[0].xAxis.dataColumn);			
		}else{
			this.chartLabelSelector.reset();
		}
	},
	
	_buildGridLayout: function(){
		var layout = [{ field: 'value',	name: this.gridHeaders[0], formatter: dojo.hitch(this,"_comboBoxFormatter"),width: '40%'},
					  { field: 'label', name: this.gridHeaders[1], formatter: dojo.hitch(this,"_buttonFormatter"), width: '60%'}];
		return layout;
	},
	
	postMixInProperties: function(){		
		if(this.widgetDef && this.widgetDef.charts){			
			this._xAxisLabelValue = this.widgetDef.charts[0].xAxis.label.key;			
			this._yAxisLabelValue = this.widgetDef.charts[0].yAxis.label.key;
			this.widgetDef.charts[0].label.key = this.widgetDef.meta.label.key;
			this.widgetDef.charts[0].label.value = this.widgetDef.meta.label.value;				
		} else if (this.widgetDef && this.widgetDef.tables) {
			this.widgetDef.tables[0].label.key = this.widgetDef.meta.label.key;
			this.widgetDef.tables[0].label.value = this.widgetDef.meta.label.value;				
		}
	},	
	
	postCreate: function(){		
		if(this.chartType === "Pie"){
			dojo.style(this.axisLabelDiv,{"display": "none"});
		}		
				
		this.chartLabelSelector = new dijit.form.FilteringSelect({
			store: this._labelStore,
			searchAttr: this.SEARCH_ATTR, 
			value:  this._chartLabelValue}, 
			this.chartLabelSelectorDiv
		);
		
		if(!dojox.grid.util.oriFire){
			dojox.grid.util.oriFire = dojox.grid.util.fire;
			dojox.grid.util.fire = function(n, evt, args){			
				if(evt === 'focus' && dojox.grid.util.comboOpened){ return; }
				dojox.grid.util.oriFire(n, evt, args);
			};
		}	

		this.dataGrid = new dojox.grid.DataGrid({
			structure: this._buildGridLayout(),			
			store: new dojo.data.ItemFileWriteStore({data: this._buildSingleGridStore()})
		},this.gridDiv);				
		this.dataGrid.startup();
		
		this.connect(this.chartLabelSelector,"onChange","onChange");
		this.connect(this.xAxisLabel,"onChange","onChange");
		this.connect(this.yAxisLabel,"onChange","onChange");
	},
	
	setDataStore: function(dataStore){
		this._dataStore = dataStore;
	},
	
	setLabelStore: function(labelStore){
		this._labelStore = labelStore;
	},
	
	setWidgetDefinition: function(widgetDef){
		this.widgetDef = widgetDef;
		this.chartLabelSelector.store = this._labelStore;
		this._updateLabelSelector();
		this._updateDataGrid();
	},	
	
	_addRow: function(event){
		var newItem = {            
            value: null,
			label: null
		};
		this._gridStore.items.splice(event.rowIndex+1,0,newItem);		
		this._updateDataGrid();
		this.onChange();
	},
	
	_removeRow: function(event){		
		this._gridStore.items.splice(event.rowIndex,1);
		this._updateDataGrid();
		this.onChange();
	},
	
	_setButtonState: function(){
		if(this._gridStore.items.length===1 && this._buttonDisa){
			this._buttonDisa.attr('disabled', true);
		}
		if(this.chartType === "dojot.visual.ColumnsEmbossed" || this.chartType === "dojot.visual.BarsEmbossed"){
			if(this._buttonDisaAdd){
				this._buttonDisaAdd.attr('disabled', true);
			}
			if(this._buttonDisa){
				this._buttonDisa.attr('disabled', true);
			}
		}
	},
	
	_updateDataGrid: function(){		
        //this.dataStore.newItem(newItem);  //the new item will be appended to the end.
		//this.dataGrid._addRow({id: "0", value: "222"},);	//dojox.grid.DataGrid has not really implement this method		
		this.dataGrid.setStore(new dojo.data.ItemFileWriteStore({data: this._buildSingleGridStore()}));		
		this.dataGrid.update();	
		this._setButtonState();
	},
	
	
	_updateGridStoreValue: function(newValue){		
		this.gridStore.items[this.rowIndex].value = newValue;	
		
	},
	
	_updateGridStoreLabel: function(newValue){		
		this.gridStore.items[this.rowIndex].label = newValue;
		
	},
	
	
	_comboBoxFormatter: function(cellValue, rowIndex, row){
		var valueSel = new dijit.form.FilteringSelect({
			store: this._dataStore,
			searchAttr: this.SEARCH_ATTR, 
			value: cellValue, 
			rowIndex: rowIndex, 
			gridStore: this._gridStore});
		valueSel.connect(valueSel, 'onClick', function(){
			dojox.grid.util.comboOpened = !this._opened;			
		});
		valueSel.rowIndex = rowIndex;
		dojo.style(valueSel.domNode,{"width":"90%"});
		this._conns.push(dojo.connect(valueSel,"onChange",valueSel, this._updateGridStoreValue)); 
		//this.connect(valueSel,"onChange","onChange");
		this.tempValueSelector = valueSel;
		return valueSel;
	},
	
	_buttonFormatter: function(cellValue, rowIndex, row){ 
		var labelSel = new ibm.tivoli.tip.charts.dijit.LabelSelector({
			dataStore: this._labelStore, 
			rowIndex: rowIndex, 
			currentValue: cellValue, 
			gridStore: this._gridStore});
		
		if(rowIndex===0){
			this._buttonDisa = labelSel.delRowButton;
			this._buttonDisaAdd = labelSel.addRowButton;
		}
		this.connect(labelSel, "addRow", "_addRow");
		this.connect(labelSel, "removeRow", "_removeRow");
		this._conns.push(dojo.connect(labelSel.labelBox,"onChange",labelSel, this._updateGridStoreLabel));
		labelSel.labelBox.dataSelectHandle = dojo.connect(this.tempValueSelector,"onChange",labelSel.labelBox, function(newValue){			
			this.set('value', newValue);
		}); 
		labelSel.labelBox.dataSelectRef = this.tempValueSelector;
		labelSel.labelBox.changeHandle = dojo.hitch(this,"onChange");
		this.connect(labelSel.labelBox,"onChange","onChange");
		this._conns.push(dojo.connect(labelSel.labelBox,"onChange",labelSel.labelBox, function(){
			// dojo.disconnect(this.dataSelectHandle);
			dojo.connect(this.dataSelectRef,"onChange",this.changeHandle);
		}));
		return labelSel;
	},
	
	resize: function(size){		
		dojo.style (this.domNode, {
				"width": size.w,
				"height": size.h
		});
		var coords = dojo.coords (this.selLabelDiv);
		size.h = size.h - coords.h;		
		coords = dojo.coords (this.chartLabelSelector.domNode);	
		if(size.w<120){
			dojo.style(this.chartLabelSelector.domNode,{"width": "80%"});
		}
		size.h = size.h - coords.h;
		size.w = size.w - 5;
		this.dataGrid.resize({w:size.w+1,h:size.h});//add 1px to get rid of horizontal scrollbar
		dojo.style(this.gridBorder, {"width": size.w+"px"});
		this._updateDataGrid();
	},
	
	onChange: function(){
		console.log("ChartVisualization.onChange");
	},
	
	isValid: function(){
		var i=0;		
		if(!this.chartLabelSelector.getValue()){
			return false;
		}
		if(this._gridStore.items.length<1){
			return false;
		}
		for(i=0;i<this._gridStore.items.length;i++){
			var item = this._gridStore.items[i];
			if(!item.value||!item.label){
				return false;
			}
		}	
		/*
		if(this.chartType != "Pie" && !(this.xAxisLabel.attr("value") && this.yAxisLabel.attr("value"))){
			return false
		}
		*/
		return true;
	},
	
	noChartConfig: function(){
		var i=0;
		for(i=0;i<this._gridStore.items.length;i++){
			var item = this._gridStore.items[i];
			if(item.value&&item.label){
				return false;
			}
		}	
		return true;		
	},
	
	getWidgetDefinition: function(){	
		var yAxisDataAttrs = [];
		var yAxisTooltipAttrs = [];
		var i=0;
		for(i=0; i<this._gridStore.items.length; i++){
			yAxisDataAttrs.push(this._gridStore.items[i].value);
			yAxisTooltipAttrs.push(this._gridStore.items[i].label);
			if(this.chartType === "dojot.visual.ColumnsEmbossed" || this.chartType === "dojot.visual.BarsEmbossed"){		
				break;
			}
		}			
		var widgetDefJson = new ibm.tivoli.tip.charts.dojo.WidgetDefJsoner({
			widgetDef: this.widgetDef, 	
			xAxisLabel: dojo.string.trim(this.xAxisLabel.attr("value")),
			yAxisLabel: dojo.string.trim(this.yAxisLabel.attr("value")),
			xAxisDataAttr: this.chartLabelSelector.getValue(),
			yAxisDataAttrs: yAxisDataAttrs,
			yAxisTooltipAttrs: yAxisTooltipAttrs,
			chartType: this.chartType
		});		
		return widgetDefJson.getJsonDef();
	},
	
	destroy: function(){
		if(dojox.grid.util.oriFire){
			delete dojox.grid.util.comboOpened;
			dojox.grid.util.fire = dojox.grid.util.oriFire;
			delete dojox.grid.util.oriFire;
		}
		var i=0;		
		for(i=0;i<this._conns.length;i++){
			dojo.disconnect(this._conns[i]);
		}
		if (this.dataSelectHandle) {
			dojo.disconnect(this.dataSelectHandle);
		}
		this.dataGrid.destroyRecursive();
		this.inherited(arguments);
	}
});