dojo.provide("ibm.tivoli.tip.charts.dijit.DojoChartBuilder");

dojo.require("ibm.tivoli.tip.charts.dijit.ChartBuilder");

dojo.declare("ibm.tivoli.tip.charts.dijit.DojoChartBuilder",[ibm.tivoli.tip.charts.dijit.ChartBuilder],{		
	
	constructor: function(){
		this.isChartBuilt = false;
		this._widgetSize = null;
		this.isResizeLate = false;
	},	
	
	startup: function(){
		this.chartDef = this.getChartDef (this.chartID);
		this.preBuild = setTimeout(dojo.hitch(this,function(){
			this.setSize(this._widgetSize);
			this.setupTheme();
			this.dataset = this.getDataSet (this.dataSetID);
			this.common = this.getCommonData ();			
			this.constructChart();	
			if(this.chartDef.type != "Pie") {
				this.buildAxisChart();	
			}
			this.buildToolbar();				
			clearTimeout (this.preBuild);
		}),0);					
		this.fetchData();				
	},
	
	fetchData: function(){
		this.chartFetcher = new ibm.tivoli.tip.charts.dojo.ChartFetcher ({
			global: this.global, 
			chartPreferences: this.chartPreferences,
			chartDef: this.chartDef,
			rowsPerPage: this.chartDef.common.pagingSize,
			widgetDefinition: this.widgetDefinition,
			providerID: this.providerID,
			chartParams: this.widgetDefinition.datasets[0].parameters,
			datasourceID: this.datasourceID	
		});
		this.dataFetchSub = dojo.subscribe ("ChartFetcher.dataFetched" + this.chartFetcher.id, dojo.hitch (this, this.dataFetched)); 		
		this.subscribe ("ChartFetcher.totalRowsFetched" + this.chartFetcher.id, dojo.hitch (this, this.buildPager));	
		if(this.chartDef.common.paging){		
			this.chartFetcher.fetchData(0,this.chartFetcher.rowsPerPage);
		}else{
			this.chartFetcher.fetchData();
		}		
	},
	
	renderChart: function(){
		try{
		if(this.chartDef.type == "Pie") {			
			this.buildPieChart();	
		} else {	
			this.feedAxisChartData();		
		} 			
		if ((this.chartDef.legend.hidden !== undefined) && (this.chartDef.legend.hidden === false)) {
			this.addLegend (this.chart);			
		}	
		}catch(e){
			console.error("Render Chart: "+e.message);
		}finally{
			this.isChartBuilt = true;
		}		
		if(this.isResizeLate){
			this.resize();
		}
		dojo.publish ("ChartBuilder.chartBuilt" + this.id, []);		
		this.updateMetaData();
		if (this.chartDef.common.showLastUpdateTime) {
			dojo.style (this.metaDiv, "display", "");
		} else {
			dojo.style (this.metaDiv, "display", "none");
		}
		if(this.postRenderChart){
			clearTimeout(this.postRenderChart);
		}
	},
	
	dataFetched: function(columns, rows, store){		
		dojo.unsubscribe(this.dataFetchSub);
		this.subscribe ("ChartFetcher.dataFetched" + this.chartFetcher.id, dojo.hitch (this, this.afterFetch)); 
		this.columns = columns;
		this.rows = rows;
		this.store = store;		
		if(this.chart){
			this.renderChart();
		}else{
			this.postRenderChart = setTimeout(dojo.hitch(this,this.renderChart),0);
		}
		this.afterBuild = setTimeout(dojo.hitch(this,function(){
			if(!this.standalone) {			
				this.chartUtil = new ibm.tivoli.tip.charts.dojo.DojoChartUtil ({
					namespace: this.global.namespace,
					chartBuilder: this,
					providerID: this.providerID,
					datasourceID: this.datasourceID,
					savePrefsOption: false
				});
			}
			this.addDrillButtons();	
			if(!this.standalone || (this.standalone && !this.ignoreContextMenu)){
				this.makeChartContextMenu();			
			}	
			if ((this.interval !== null) && (!this.chartDef.common.autoUpdate)) {
				clearTimeout (this.interval);
				this.interval = null;
			}	
			this.configAutoUpdate();				
			clearTimeout (this.afterBuild);
		}),0);					
	},
	
	setWidgetSize: function(size){
		this._widgetSize = size;		
	},
	
	setSize: function (size) {	
		dojo.style (this.domNode, {"width": size.w + "px","height": size.h + "px"});					
		if (this.chartDef.common.showLastUpdateTime) {			
			size.h = size.h - 15;				
		} 				
		if (!this.standalone || (this.standalone && !this.ignoreToolbar)) {			
			if(this.chartDef.toolbar.hidden){				
				size.h = size.h - 16;
			}else{					
				size.h = size.h - 35;
			}			
		}
		if(this.standalone && !this.ignoreTitlebar){			
			size.h = size.h - 25;
		}			
		if(this.chartDef.common.paging){			
			size.h = size.h - 25;
		}
		dojo.style (dojo.byId (this.id + "chartContainer"), {
			"width": size.w + "px",
			"height": size.h + "px"
		});	
		var label = this.chartDef.yAxis.label;
		if(label && label.value && label.value != ""){
			var yAxisTitleNode = dojo.byId (this.id + "axistitle");
			if(this.chartDef.type != "Pie") {
				yAxisTitleNode.style.width = "20px";
			}			
			size.w = size.w - 20;
		}			
		label = this.chartDef.xAxis.label;
		if(label && label.value && label.value != ""){			
			size.h = size.h - 15;
		}		
		if ((this.chartDef.legend.hidden !== undefined) && (this.chartDef.legend.hidden === false)) {
			if(this.chartDef.legend.location == "right") {				
				size.w = size.w - 150;	
			} else {				
				size.h = size.h - 30;				
			}		
		}		
		if (dojo.isIE) {			
			size.h = size.h - 10;
			size.w = size.w - 10;
		}		
		// without this, we get scroll bars on FF freeform pages
		if (dojo.isFF) {
			size.w = size.w - 4;
			size.h = size.h - 4;
		}		
		this.chartDivSize = size;
		dojo.style (dojo.byId(this.id + "chart0"), {
			"width": this.chartDivSize.w + "px",
			"height": this.chartDivSize.h + "px"
		});		
	}
		
})
