dojo.provide("ibm.tivoli.tip.charts.dijit.VisualOptions");

dojo.require("dijit.form.CheckBox");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.NumberTextBox");
dojo.require("dijit.form.FilteringSelect");
dojo.require("dijit.form.RadioButton");
dojo.require("ibm.tivoli.tip.charts.dijit.DropdownColorPicker");
dojo.require("ibm.tivoli.tip.dijit.themes.ChartThemeTivoliStandard");
dojo.require("ibm.tivoli.tip.dijit.themes.ChartThemeTivoliDark");

dojo.declare("ibm.tivoli.tip.charts.dijit.VisualOptions",[dijit._Widget, dijit._Templated],{
	
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("ibm.tivoli.tip.charts.dijit", "templates/VisualOptions.html"),
	_jsonDef: {},
	widgetDef: {},
	isPagingSet: false,
	_unSkipTypeEnum: [
		"java.lang.String", "string", "string[]", 
		"java.util.List", "enum", "status", "itemName", "itemType", 
		"linkName", "linkType", "relationshipName", "relationshipType"
	],
	
	constructor: function(){
		this._jsonDef = null;
		this.widgetDef = null;
		this._resources = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");		
	},	
	
	postMixInProperties: function(){
		this._jsonDef = {
			charts: [
				{
					theme:{
						name: "TIPLight",
						startIndex: 0
					},
					common:{
						autoUpdate: false,
						paging: true,
						pagingSize: 25,
						showLastUpdateTime: false
					},
					legend: {
						hidden:true,
							location:"bottom"								
						},
						xAxis: {
							min: null,
							max: null,
							rotation: 0,
							majorTicks: true,
							minorTicks: false,
							majorLabels: true,
							minorLabels: false
						},
						yAxis: {
							min: null,
							max: null,
							rotation: 0,
							majorTicks: true,
							minorTicks: false,
							majorLabels: true,
							minorLabels: false			
						}
					}
				]
		};			
		if(!this._editMode){
			if(this.widgetDef.tables){	
				this._jsonDef.charts[0].common.paging = false;						
			}else{
				this._jsonDef.charts[0].common.paging = true;
				if(this.widgetDef.charts[0].type == "Lines"){
					this._jsonDef.charts[0].common.pagingSize = 10;			
				}else{
					this._jsonDef.charts[0].common.pagingSize = 2;
				}						
			}
		}else{
			if(this.widgetDef.tables){	
				dojo.mixin(this._jsonDef.charts[0].common,this.widgetDef.tables[0].common);
			}else{
				this._jsonDef = {
					charts: this.widgetDef.charts
				};
				if(this._jsonDef.charts[0].xAxis.max == undefined){
					this._jsonDef.charts[0].xAxis.max = null;
				}
				if(this._jsonDef.charts[0].xAxis.min == undefined){
					this._jsonDef.charts[0].xAxis.min = null;
				}
				if(this._jsonDef.charts[0].yAxis.min == undefined){
					this._jsonDef.charts[0].yAxis.min = null;
				}
				if(this._jsonDef.charts[0].yAxis.max == undefined){
					this._jsonDef.charts[0].yAxis.max = null;
				}
			}			
		}
		this._updateUI();
	},
	
	isSkipable: function(columnType){
		for(var i=0;i<this._unSkipTypeEnum.length;i++){
			if(this._unSkipTypeEnum[i] == columnType){
				return false;
			}
		}
		return true;
	},
	
	_setThemeColors: function(){
		if(this.chart.theme.name=="TIPDark"){
			this.themeColors = ibm.tivoli.tip.dijit.themes.ChartThemeTivoliDark.colors;
		}else{
			this.themeColors = ibm.tivoli.tip.dijit.themes.ChartThemeTivoliStandard.colors;
		}	
	},
	
	_updateUI: function(){		
		this.chart = this._jsonDef.charts[0];		
		if(this._jsonDef.charts[0].legend.hidden){
			this.uiLegendLocationValue = "hidden";			
		}else{
			this.uiLegendLocationValue = this._jsonDef.charts[0].legend.location;			
		}		
		this.uiNoPagingValue = !this._jsonDef.charts[0].common.paging;		
		this._setThemeColors();		
	},
	
	postCreate: function(){
		this.inherited(arguments);
		
		this.noPaging = new dijit.form.RadioButton({name: "paging", checked: this.uiNoPagingValue},this.noPagingDiv);
		this.doPaging = new dijit.form.RadioButton({name: "paging", checked: this.chart.common.paging},this.doPagingDiv);
		this.autoUpdate = new dijit.form.CheckBox({checked: this.chart.common.autoUpdate}, this.autoUpdateDiv);
		this.showLastUpdate = new dijit.form.CheckBox({checked: this.chart.common.showLastUpdateTime}, this.showLastUpdateDiv);
		this.majorAxisTick = new dijit.form.CheckBox({checked: this.chart.yAxis.majorTicks}, this.majorAxisTickDiv);
		this.minorAxisTick = new dijit.form.CheckBox({checked: this.chart.yAxis.minorTicks}, this.minorAxisTickDiv);
		this.majorAxisLabel = new dijit.form.CheckBox({checked: this.chart.yAxis.majorTicks}, this.majorAxisLabelDiv);
		this.minorAxisLabel = new dijit.form.CheckBox({checked: this.chart.yAxis.minorTicks}, this.minorAxisLabelDiv);
		
		this.startColor.setColors(this.themeColors);
		this.connect(this.chartTheme,"onChange","_updateTheme");
		this.connect(this.startColor,"onChange","_updateStartColor");
		this.connect(this.legendLocation,"onChange","_updateLegendLocation");
		this.connect(this.noPaging,"onClick","_updatePagingDisable");
		this.connect(this.doPaging,"onClick","_updatePagingEnable");
		this.connect(this.pageSize,"onChange","_updatePageSize");
		this.connect(this.autoUpdate,"onChange","_updateAutoUpdate");
		this.connect(this.showLastUpdate,"onChange","_updateShowLastUpdate");
		this.connect(this.xAxisRotation,"onChange","_updateXAxisRotation");
		this.connect(this.yAxisRotation,"onChange","_updateYAxisRotation");
		this.connect(this.xAxisScaleMin,"onChange","_updateXAxisMin");
		this.connect(this.xAxisScaleMax,"onChange","_updateXAxisMax");
		this.connect(this.yAxisScaleMin,"onChange","_updateYAxisMin");
		this.connect(this.yAxisScaleMax,"onChange","_updateYAxisMax");
		this.connect(this.majorAxisTick,"onChange","_updateAxisMajorTicks");
		this.connect(this.minorAxisTick,"onChange","_updateAxisMinorTicks");
		this.connect(this.majorAxisLabel,"onChange","_updateAxisMajorLabels");
		this.connect(this.minorAxisLabel,"onChange","_updateAxisMinorLabels");		
	},
	
	hideComponents: function(){	
		this.axisConfigDiv.style.display = "inline";
		this.commonChartConfigDiv.style.display = "block";
		if(this.widgetDef.tables || this.widgetDef.charts[0].type=="Pie"){			
			this.axisConfigDiv.style.display = "none";			
			if(this.widgetDef.tables){				
				this.commonChartConfigDiv.style.display = "none";
			}
		}
	},
	
	_updateTheme: function(){
		this._jsonDef.charts[0].theme.name = this.chartTheme.get("value");
		this._setThemeColors();	
		this.startColor.setColors(this.themeColors);
		this._updateNotify();
	},
	
	_updateStartColor: function(){		
		this._jsonDef.charts[0].theme.startIndex = this.startColor.getValue();
		this._updateNotify();
	},
	
	_updateLegendLocation: function(){
		var legendValue = this.legendLocation.get("value");
		if(legendValue == "hidden"){
			this._jsonDef.charts[0].legend.hidden = true;
		}else{
			this._jsonDef.charts[0].legend.hidden = false;
			this._jsonDef.charts[0].legend.location = legendValue;
		}		
		this._updateNotify();
	},		
	
	_updatePagingEnable: function(){
		this.isPagingSet = true;
		this._jsonDef.charts[0].common.paging = true;		
		this.pageSize.set('disabled',false);
		this._updateNotify();
	},
	
	_updatePagingDisable: function(){
		this.isPagingSet = true;
		this._jsonDef.charts[0].common.paging = false;		
		this.pageSize.set('disabled',true);
		this._updateNotify();
	},
	
	_updatePageSize: function(){
		if(this.pageSize.isValid()){
			this.isPagingSet = true;
			this._jsonDef.charts[0].common.pagingSize = this.pageSize.get("value");
			this._updateNotify();
		}
	},
	
	_updateAutoUpdate: function(){
		this._jsonDef.charts[0].common.autoUpdate = this.autoUpdate.get("checked");
		this._updateNotify();
	},
	
	_updateShowLastUpdate: function(){
		this._jsonDef.charts[0].common.showLastUpdateTime = this.showLastUpdate.get("checked");
		this._updateNotify();
	},
	
	_updateXAxisRotation: function(){
		this._jsonDef.charts[0].xAxis.rotation = dojo.number.parse(this.xAxisRotation.get("value"));
		this._updateNotify();
	},
	
	_updateYAxisRotation: function(){
		this._jsonDef.charts[0].yAxis.rotation = dojo.number.parse(this.yAxisRotation.get("value"));
		this._updateNotify();
	},
	
	_updateXAxisMin: function(){	
		if(this.xAxisScaleMin.isValid()){			
			var scaleValue = this.xAxisScaleMin.get("value");
			this._jsonDef.charts[0].xAxis.min = isNaN(scaleValue) ? null : scaleValue ;
			this._updateNotify();
		}
	},
	
	_updateXAxisMax: function(){		
		if(this.xAxisScaleMax.isValid()){
			var scaleValue = this.xAxisScaleMax.get("value");
			this._jsonDef.charts[0].xAxis.max = isNaN(scaleValue) ? null : scaleValue;
			this._updateNotify();
		}
	},
	
	_updateYAxisMin: function(){		
		if(this.yAxisScaleMin.isValid()){
			var scaleValue = this.yAxisScaleMin.get("value");
			this._jsonDef.charts[0].yAxis.min = isNaN(scaleValue) ? null : scaleValue;
			this._updateNotify();
		}
	},
	
	_updateYAxisMax: function(){		
		if(this.yAxisScaleMax.isValid()){
			var scaleValue = this.yAxisScaleMax.get("value");
			this._jsonDef.charts[0].yAxis.max = isNaN(scaleValue) ? null : scaleValue;
			this._updateNotify();
		}
	},
	
	_updateAxisMajorTicks: function(){
		this._jsonDef.charts[0].xAxis.majorTicks = this.majorAxisTick.get("checked");
		this._jsonDef.charts[0].yAxis.majorTicks = this.majorAxisTick.get("checked");
		this._updateNotify();
	},
	
	_updateAxisMinorTicks: function(){
		this._jsonDef.charts[0].xAxis.minorTicks = this.minorAxisTick.get("checked");
		this._jsonDef.charts[0].yAxis.minorTicks = this.minorAxisTick.get("checked");
		this._updateNotify();
	},
	
	_updateAxisMajorLabels: function(){
		this._jsonDef.charts[0].xAxis.majorLabels = this.majorAxisLabel.get("checked");
		this._jsonDef.charts[0].yAxis.majorLabels = this.majorAxisLabel.get("checked");
		this._updateNotify();
	},
	
	_updateAxisMinorLabels: function(){
		this._jsonDef.charts[0].xAxis.minorAxisLabel = this.minorAxisLabel.get("checked");
		this._jsonDef.charts[0].yAxis.minorAxisLabel = this.minorAxisLabel.get("checked");	
		this._updateNotify();	
	},
	
	_updateNotify: function(){
		if(this.isValid()){			
			this.onChange();
		}
	},	
	
	onChange: function(){
		console.log("VisualOptions.onChange");
	},
	
	getWidgetDefinition: function(){
		if(this.widgetDef.tables){	
			dojo.mixin(this.widgetDef.tables[0].common, this._jsonDef.charts[0].common);
		}else{
			if(this._jsonDef.charts[0].xAxis.max==null){
				delete this._jsonDef.charts[0].xAxis.max;
			}
			if(this._jsonDef.charts[0].xAxis.min==null){
				delete this._jsonDef.charts[0].xAxis.min;
			}
			if(this._jsonDef.charts[0].yAxis.min==null){
				delete this._jsonDef.charts[0].yAxis.min;
			}
			if(this._jsonDef.charts[0].yAxis.max==null){
				delete this._jsonDef.charts[0].yAxis.max;
			}	
			var dataSetAttrs = this.widgetDef.datasets[0].columns;
			for(var i=0;i<dataSetAttrs.length;i++){
				if(dataSetAttrs[i].id == this.widgetDef.charts[0].xAxis.dataColumn){
					if(!this.isSkipable(dataSetAttrs[i].type)){
						this._jsonDef.charts[0].xAxis.majorTickStep = 1;
					}
					break;
				}						
			}	
			this.widgetDef.charts[0].theme = this._jsonDef.charts[0].theme;		
			dojo.mixin(this.widgetDef.charts[0].common,this._jsonDef.charts[0].common);
			dojo.mixin(this.widgetDef.charts[0].legend,this._jsonDef.charts[0].legend);
			dojo.mixin(this.widgetDef.charts[0].xAxis,this._jsonDef.charts[0].xAxis);
			dojo.mixin(this.widgetDef.charts[0].yAxis,this._jsonDef.charts[0].yAxis);
		}	
		return this.widgetDef;
	},
	
	resize: function(size){
		this.inherited(arguments);
		dojo.style(this.domNode, {width: size.w+"px",height: size.h+"px"});
	},	
	
	isValid: function(){
		var validInput = true;		
		if(this.doPaging.get("checked")){
			if(!this.pageSize.isValid() || isNaN(this.pageSize.get('value'))){
				validInput = false;
			}
		}		
		if(!this.xAxisScaleMin.isValid()||!this.xAxisScaleMax.isValid()||!this.yAxisScaleMin.isValid()||!this.yAxisScaleMax.isValid()){
			validInput = false;
		}		
		return validInput;
	}	

});
