/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2009 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/


dojo.provide("ibm.tivoli.tip.charts.dojo.ChartFetcher");
dojo.require("ibm.tivoli.tip.dojo.Global");
dojo.require("ibm.tivoli.tip.dojo.Server");
dojo.require ("ibm.tivoli.tip.charts.dojo.RESTErrorHandler");
dojo.require("ibm.tivoli.navmodel.Dataset");
dojo.require("ibm.tivoli.navmodel.DatasetItemsStore");
dojo.require("ibm.tivoli.tip.navwidget.dijit.DefaultMessageHandler");

dojo.declare("ibm.tivoli.tip.charts.dojo.ChartFetcher",
	          null,
{
	/* Default timeout for requests made to the server */
	requestTimeout: 30000,
	global: null,
	rowsPerPage: null,
	chartPreferences: null,
	chartDef: null,
	widgetDefinition: null,
	datasourceID: null,
	providerID: null,
	chartParams: null,
	dataSet: null,

	constructor: function (mixinData) {
	    dojo.mixin( this, mixinData );
	    
		this.id = dijit.getUniqueId(this.declaredClass.replace(/\./g,"_"));	
		this.columns = null;
		this.rows = null;
		this.start = null;
		this.query = null;
		this.sortColumn = null;
		this.totalCount = -1;
		this.tableID = this.global.namespace + "states";
		this.store = null;
		this.dataSet = null;
	},
	getQuery: function () {
		var query = this.widgetDefinition.datasets[0].query;
		
		if ((this.chartPreferences) && (this.chartPreferences.getParameters() !== null)) {
			
			query = query + this.chartPreferences.getParameters();
		}
		
		query = query + "&widgetID=" + this.chartDef.id + "&applicationID=" + this.widgetDefinition.datasource.id;
		
		return (query);
	},
	fetchData: function (inRowIndex, inCount) {
		console.log ("enter ChartFetcher.fetchData");
		if(inRowIndex===undefined||inRowIndex===null||inCount===undefined||inCount===null){
			inRowIndex = 0;
			inCount = -1;
		}	
		this.inRowIndex = inRowIndex;
		this.inCount = inCount;
		var url = "/ibm/tivoli/rest";
		var dsParams = {
			widgetID: this.chartDef.id
		};
		if(this.chartParams !== null){
			console.log ("adding chartParams");
			console.log (this.chartParams);
			for (var row in this.chartParams) {
				console.log ("processing row in chartParams");
				if (row) {
					console.log ("processing row in chartParams");
				 	var param = this.chartParams[row];
				 	console.log ("param: ");
				 	console.log (param);
					//queryParam.navModel.parameters[param.id] = param.value;
					if(typeof param.value == "string"){
						dsParams[param.id] = param.value.split(",");
					}else{
						dsParams[param.id] = param.value;
					}				 	
				} else {
					console.log ("row has no value");
				}
			}
		}
		if(this.global.restTimeParam){
			dsParams[this.global.restTimeParam.id] = this.global.restTimeParam.value;
		}
		this.destroyDataSet();
		this.dataSet = new ibm.tivoli.navmodel.Dataset({
			baseUrl: url,	
			provider: this.providerID,
			datasource: this.datasourceID,
			dataset: this.widgetDefinition.datasets[0].id,
			createEventRouter: false,
			fetchMetaData: false,
			parameters: dsParams,
			onComplete: dojo.hitch(this, function(dataSet){
				var navStore = new ibm.tivoli.navmodel.DatasetItemsStore({
					dataset: dataSet,						
					returnItem: false		
				});		
				var query = {};		
				var filCond = this.widgetDefinition.datasets[0].conditions;
				var dsColumns = this.widgetDefinition.datasets[0].columns;
				if(filCond && filCond.filterString && filCond.filter.length>0){
					query.expression = filCond.filterString;	
					query.ids = {};			
					for(var i=0;i<filCond.filter.length;i++){
						if(filCond.filter[i].column == "anycolumn"){
							query.ids = {};
							for(var j=0;j<dsColumns.length;j++){
								query.ids[dsColumns[j].id] = true;
							}
							break;
						}else{
							query.ids[filCond.filter[i].column] = true;
						}
					}
				}		
				this.store = navStore;
				this.store.fetch ( {				
					//onBegin: dojo.hitch(this, "beginReturn"),
					query: query,
					onComplete: dojo.hitch (this, "processRows"),
					onBegin: dojo.hitch(this, "reportTotalRows"),
					onError: dojo.hitch (this, function (error) {										
						var errHandler = new ibm.tivoli.tip.charts.dojo.RESTErrorHandler(this.global);		
						errHandler.showError(dojo.fromJson(error.responseText));	
                	}),
					start: this.inRowIndex,
					count: this.inCount
				});				
			}),
			onError: dojo.hitch(this, function(error, exception, ioArgs) {
				var busyPane = dijit.byId(ibm.tivoli.tip.dojo.Server.getSingleton (this.global).busyPaneID);
				if(busyPane){
					busyPane.hide();
				}
				var msgHandler = new ibm.tivoli.tip.navwidget.dijit.DefaultMessageHandler({});
				msgHandler.displayException(exception);
			})
		});
		console.log ("exit ChartFetcher.fetchData");
	},
	
	reportTotalRows: function(inCount){
		console.log("enter reportTotalRows");
		if(this.totalCount==-1){
			this.totalCount = inCount;
			dojo.publish ("ChartFetcher.totalRowsFetched" + this.id, [inCount]);
		}		
		console.log("exit reportTotalRows");
	},
	beginReturn: function (inCount) {
		
		this.totalCount = inCount;
	},
	processRows: function (items, query) {
		this.onRowLoad (items);
	},
//	onColumnLoad: function (data, request) {
//		this.columns = data;
//		this.fetchData ();
//	},
	onRowLoad:function (data) {
		this.rows = data;
		var fetchTime = new Date() - this.start;
		
		
		dojo.publish ("ChartFetcher.dataFetched" + this.id, 
					  [this.columns, this.rows, this.store]
					 );
	},
	getChartDefinition: function () {
		return (this.chartDef);
	},
	getColumns: function () {
		return (this.widgetDefinition.datasets[0].columns);
		//return (this.columns);
	},
	getRows: function () {
		return (this.rows);
	},
	getTotalRows: function () {
		return (this.totalCount);
	},
	destroyDataSet: function(){
		if(this.dataSet){
			this.dataSet.destroy();
			delete this.dataSet;
		}
	},
	destroy: function(){
		this.destroyDataSet();
	}
});
