/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2009 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/


dojo.provide("ibm.tivoli.tip.charts.dojo.WidgetFetcher");
dojo.require("ibm.tivoli.tip.dojo.Global");
dojo.require("ibm.tivoli.tip.dojo.Server");
dojo.require ("ibm.tivoli.tip.charts.dijit.ChartBuilder");
dojo.require ("ibm.tivoli.tip.charts.dijit.TableBuilder");
dojo.require ("ibm.tivoli.tip.charts.dojo.ChartPreferences");
dojo.require ("ibm.tivoli.tip.charts.dojo.RESTErrorHandler");
dojo.declare("ibm.tivoli.tip.charts.dojo.WidgetFetcher",
	          null,
{
	/* Default timeout for requests made to the server */
	requestTimeout: 30000,
	global: null,
	autoUpdateConfig: null,

	constructor: function (mixinData) {
	    dojo.mixin( this, mixinData );
	    this.widgetDef = null;
	    this.preferences = null;
	    this.autoUpdateConfig = {};
	    if(this.global.autoUpdateConfig){
	    if(this.global.autoUpdateConfig.maxUpdateInterval){
			this.autoUpdateConfig.updateMaxInterval = this.global.autoUpdateConfig.maxUpdateInterval * 1000;
		}
		if(this.global.autoUpdateConfig.minUpdateInterval){
			this.autoUpdateConfig.updateMinInterval = this.global.autoUpdateConfig.minUpdateInterval * 1000;
		}
		if(this.global.autoUpdateConfig.multiplier){
			this.autoUpdateConfig.updateMultiplier = this.global.autoUpdateConfig.multiplier;
		}
		}
	},
	buildWidgetAPI: function (parameters) {
		return (this.buildWidget (parameters.providerID, parameters.datasourceID, parameters.widgetID,
						  parameters.domNode, parameters.chartVersion, parameters.widgetParameters,
						  parameters.drillUpItems, parameters.initSize));
	},
	fetchWidgetDefinitionAPI: function (parameters) {
		return (this.fetchWidgetDefinition (parameters.providerID, 
							parameters.datasourceID, parameters.widgetID, 
							parameters.widgetVersion, parameters.profileID));
	},
	/*
	 * widgetParams: {
	 * 	 standalone: boolean   run as a standalone widget if true
	 *   ignoreTitlebar: boolean	without titlebar if true
	 *   ignoreToolbar: boolean	 	without toolbar if true
	 *   ignoreContextMenu: boolean     without contextmenu if true
	 * }
	 */
	buildWidget: function (providerID, datasourceID, widgetID, domNode, chartVersion, widgetParams, drillUpItems, initSize) {
		console.log ("enter WidgetFetcher.buildWidget");
		this.widgetDef = this.fetchWidgetDefinition (providerID, datasourceID, widgetID, chartVersion);
		// If "all" is passed in as the datasource, then we must assume the datasource type in the widget definition
		// provides the ID of the datasource.  This is because tip.charts does not list datasources for providers
		// other than tip.charts provider.  The datasource is assumed to be "any" or "all".  However, to allow providers
		// to use datasource as organization mechanism, we provide this mechanism for assuming the datasource is just
		// organziation and not connection instance.
		if (datasourceID == "all") {
			datasourceID = this.widgetDef.datasource.type;
		}
		
		if ((this.widgetDef.charts) && (this.widgetDef.charts[0].type == "Table") || ((this.widgetDef.tables) && (this.widgetDef.tables.length > 0))) {
			var isTableFirst = false;
			if(this.widgetDef.tables) {
	            isTableFirst = true;
			}
			return (this.buildTable (providerID, datasourceID, this.widgetDef, domNode, widgetID, widgetParams, isTableFirst, drillUpItems));
		} else {
			return (this.buildChart (providerID, datasourceID, this.widgetDef, widgetID, domNode, widgetParams, drillUpItems, initSize));
		}
	},
	fetchWidgetDefinition: function (providerID, datasourceID, widgetID, chartVersion, profileID) {	
		var decodedPID = null;
		if(profileID === null || profileID === undefined) {
			//encodedPID = encodeURIComponent(this.global.chartProfileID);
			decodedPID = this.global.chartProfileID;
		}
		else {
			//encodedPID = encodeURIComponent(profileID);
			decodedPID = decodeURIComponent(profileID);
		}		
		// e.g, /ibm/tivoli/rest/providers/tip.charts/datasources/ITM/widgets/charts-2.1.0/definitions/ProcessorUsage
		// TODO: Do not hard code widgetVersion
		//var widgetVersion = "2.1.0";
		var widgetVersion = chartVersion;
		var controller = "/ibm/tivoli/rest";
		var result = null;
		//TODO passing profile Id within content modofies the profileId 
		var profileIdUriParam = decodedPID==null||decodedPID==undefined||decodedPID=="null"||decodedPID=="undefined" ? "" : "&profileID=" + decodedPID;
		
		var ecdProviderID = encodeURIComponent(providerID);
		var ecdDatasourceID = encodeURIComponent(datasourceID);
		var ecdWidgetID = encodeURIComponent(widgetID);
		
        var kw = {
        			  sync: true,
					  handleAs: "json",
					  preventCache: true,
    				  mimetype: "application/json",
        			  //url: controller + "/providers/" + providerID + "/datasources/" + datasourceID + "/widgets/charts"+ "-" + widgetVersion + "/definitions/" + widgetID + "-" + widgetVersion + "?profileID=" + decodedPID,
        			  url: controller + "/providers/" + ecdProviderID + "/datasources/" + ecdDatasourceID + "/widgets/charts-all/definitions/" + ecdWidgetID + "-" + widgetVersion + "?registryLocation=local" + profileIdUriParam,
                      load: function (data) {
                           	result = data;
                      },
                      error: dojo.hitch (this, function (error) {												
							var errHandler = new ibm.tivoli.tip.charts.dojo.RESTErrorHandler(this.global);		
							errHandler.showError(dojo.fromJson(error.responseText));	
	 				   })
                   };

          //ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );
          dojo.xhrGet (kw);
        
          console.log ("exit WidgetFetcher.fetchWidgetDefinition");
          return (result);
	},
	buildChart: function (providerID, datasourceID, widgetDef, widgetID, domNode, widgetParams, drillUpItems, initSize) {
		console.log ("enter buildChart with providerID & datasourceID: " + providerID + ", " + datasourceID);
		//this.fetchPreferences();
        if(drillUpItems === null || drillUpItems === undefined) {
			drillUpItems = [];
		}
		var params = {
				global: this.global,
				providerID: providerID,
				datasourceID: datasourceID,
				chartID: widgetID,
				chartPreferences: this.preferences,
				widgetDefinition: widgetDef,
				drillUpItems: drillUpItems,
				initSize: initSize
		};
		
		dojo.mixin (params, widgetParams);
		dojo.mixin (params, this.autoUpdateConfig);		
		
		var chartBuilder = new ibm.tivoli.tip.charts.dijit.ChartBuilder(params, domNode); 
		console.log ("created chartBuilder");
		chartBuilder.startup();
		
		return (chartBuilder);
	},
	buildTable: function (providerID, datasourceID, widgetDef, domNode, widgetID, widgetParams, isTableFirst, drillUpItems) {
		console.log ("enter buildTable");
		if(isTableFirst === null || isTableFirst === undefined)
		{
			isTableFirst = false;
		}
		//this.fetchPreferences();
		if(drillUpItems === null || drillUpItems === undefined) {
				drillUpItems = [];
		}
		
		var params = {
			global: this.global,
			providerID: providerID,
			datasourceID: datasourceID,
            tableID: widgetID,
			widgetDefinition: widgetDef,
			widgetParams: widgetParams,
			isTableFirst: isTableFirst,
			drillUpItems: drillUpItems,
			div: domNode
		};
		
		dojo.mixin (params, this.autoUpdateConfig);	
		
		var tableBuilder = new ibm.tivoli.tip.charts.dijit.TableBuilder(params);
		
		console.log ("created tableBuilder");
		tableBuilder.startup();
		
		return (tableBuilder);
	},
	fetchPreferences: function () {
		this.preferences = new ibm.tivoli.tip.charts.dojo.ChartPreferences();
		//this.preferences.setParameters ("?param_ManagedSystemName=*NT_SYSTEM");
					
		return (this.preferences);
	}
});
