/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
dojo.provide("ibm.tivoli.tip.dijit.BreadcrumbBar");

dojo.require("dijit._Widget");
dojo.require("dijit._Container");
dojo.require("dijit._Templated");
dojo.require("dijit._KeyNavContainer");
dojo.require("dijit.form.Button");
dojo.require("ibm.tivoli.tip.dijit.Breadcrumb");
dojo.require("ibm.tivoli.tip.dijit.BreadcrumbSeparator");

dojo.declare(
	"ibm.tivoli.tip.dijit.BreadcrumbBar",
	[dijit._Widget, dijit._Templated, 
	dijit._KeyNavContainer],
{
	templateString:
		'<div class="dijit dijitToolbar tipBreadcrumbBar" waiRole="toolbar" tabIndex="${tabIndex}" dojoAttachPoint="containerNode">' +
		'</div>',

	tabIndex: "0",
	breadcrumbWidget: [],
	breadcrumbsHidden: false,

	postCreate: function(){
		this.connectKeyNavHandlers(
			this.isLeftToRight() ? [dojo.keys.LEFT_ARROW] : [dojo.keys.RIGHT_ARROW],
			this.isLeftToRight() ? [dojo.keys.RIGHT_ARROW] : [dojo.keys.LEFT_ARROW]
		);
	},

	startup: function(){
		this.startupKeyNavChildren();
	},
	
	destroy: function () {
		this.destroyDescendants();
		this.inherited (arguments);
	},
	
	addChild: function(widget, insertIndex){
		var children = this.getChildren();
		var childToUpdate = null;
		if(children.length>0 && children[children.length -1]._setStateClass){
			childToUpdate=children[children.length -1]; //child which style class needs to be updated after addition
		}
		this.inherited (arguments);	//adding child
		if(childToUpdate!=null){
			childToUpdate._setStateClass(); //updating style class
		}
		//Breadcrumbs Tivoli Guidelines implementation
		var size = dojo.contentBox(this.domNode);
		this.breadcrumbWidget.push(widget);
		var i = 1;
		var isRtL = false;
		// for RtL languages, we need to manipulate the space left in breadcrumbbar considering that it is filled from right.
		// breadcrumbWidth  and  breadcrumbTrailWidth are calculated differently for R to L languages.  isRtL holds if the locale is Right to Left or otherwise.
		if(dojo.locale=="iw" || dojo.locale=="he"|| dojo.locale.indexOf("ar")>-1)
			isRtL = true;
		var breadcrumbWidth = this.breadcrumbWidget[0].containerNode.offsetLeft;
		if(isRtL)
			breadcrumbWidth = size.w - breadcrumbWidth;
		while(i < this.breadcrumbWidget.length){
			if(this.breadcrumbWidget[i].declaredClass == "ibm.tivoli.tip.dijit.BreadcrumbSeparator")
				breadcrumbWidth = breadcrumbWidth + this.breadcrumbWidget[i-1].containerNode.offsetWidth + this.breadcrumbWidget[i].domNode.offsetWidth;
			else 
				breadcrumbWidth = breadcrumbWidth + this.breadcrumbWidget[i-1].domNode.offsetWidth + this.breadcrumbWidget[i].containerNode.offsetWidth;
			if(breadcrumbWidth > size.w){
				this.breadcrumbsHidden = true;
				var breadcrumbTrailWidth = breadcrumbWidth;
				//Vyshantha: Mar31,2011: #907: Long breadcrumb trail puts ellipses in bad location
				//for(j = i - 2; j > 0 && breadcrumbTrailWidth > size.w ; j = j - 1){
				for(j = i - 4; j > 0 && breadcrumbTrailWidth > size.w ; j = j - 1){
					breadcrumbTrailWidth = 0;
					if(this.breadcrumbWidget[j].declaredClass == "ibm.tivoli.tip.dijit.BreadcrumbSeparator"){
						dojo.byId(this.breadcrumbWidget[j].id).style.display = "none";
						breadcrumbTrailWidth = this.breadcrumbWidget[j-1].containerNode.offsetLeft + this.breadcrumbWidget[j-1].containerNode.offsetWidth;
					} else {
						dojo.byId(this.breadcrumbWidget[j].id).style.display = "none";
						dojo.byId(this.breadcrumbWidget[j-1].id).style.display = "none";
						breadcrumbTrailWidth = this.breadcrumbWidget[j-2].domNode.offsetLeft + this.breadcrumbWidget[j-2].domNode.offsetWidth;
					}
					if(isRtL)
						breadcrumbTrailWidth = size.w - breadcrumbTrailWidth;
				}
				//Adding Ellipsis to make long breadcrumb trails guideline compliant
				//Vyshantha: Mar31,2011: #907: Long breadcrumb trail puts ellipses in bad location
				//if(this.breadcrumbWidget[i - 1].declaredClass == "ibm.tivoli.tip.dijit.BreadcrumbSeparator"){
				if(this.breadcrumbWidget[i - 3] !== undefined && this.breadcrumbWidget[i - 3].declaredClass == "ibm.tivoli.tip.dijit.BreadcrumbSeparator"){ // added check for undefined as it would throw exception in R to L languages.
					//dojo.byId(this.breadcrumbWidget[i - 1].id).innerHTML = "&gt;&#160;...&#160;&#160;&gt;";
					dojo.byId(this.breadcrumbWidget[i - 3].id).innerHTML = "&gt;&#160;...&#160;&#160;&gt;";
				} else {
					//dojo.byId(this.breadcrumbWidget[i].id).innerHTML = "&gt;&#160;...&#160;&#160;&gt;";
					if(this.breadcrumbWidget[i - 2] !== undefined) {
						dojo.byId(this.breadcrumbWidget[i - 2].id).innerHTML = "&gt;&#160;...&#160;&#160;&gt;";
						}
				}
			} else {
				this.breadcrumbsHidden = false;
			}
			i = i + 1;
		}
	},
	
	areBreadcrumbsHidden: function(){
		return this.breadcrumbsHidden;
	}
}
);

