/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2007, 2008, 2009 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
/*created 2009, Todd M. Eischeid, IBM Corporation*/
/*
  test pages for using the wizard:
   - basic example: http://eischeid.bluehost.ibm.com/designs/misc/wizdemo/wizdemo_widget.html
   - fixed size: http://eischeid.bluehost.ibm.com/designs/misc/wizdemo/wizdemo_widget_fixed.html
   - horizontal table of contents: http://eischeid.bluehost.ibm.com/designs/misc/wizdemo/wizdemo_widget_horizontal.html
   - displaying the wizard in a dijit Dialog: http://eischeid.bluehost.ibm.com/designs/misc/wizdemo/wizdemo_widget_fixed_dialog.html
*/

 
dojo.provide("ibm.tivoli.tip.dijit.TIPWizard");
dojo.require("dijit._Widget");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.layout.BorderContainer");
dojo.require("dijit.layout.StackContainer");
dojo.require("dojo.i18n");
dojo.require("dijit.form.Button");

dojo.declare(
	"ibm.tivoli.tip.dijit.TIPWizard", 
    [ dijit._Widget ],
{

  //unique string identifier that will be used to make elements of this widget instance unique. 
  id: "",
  
  //array of objects; each object has fields: label, location, init, onShow, onBeforeHide
  /*
   * label - Label that will appear in the table of contents
   * url - location of the page content 
   * onLoad - will fire when the page has been loaded and parsed during initialization
   * onShow - this will fire when the page is navigated to by the user
   * onBeforeHide -   this will fire before the page is navigated away from. Return false from your function to keep the user on the current page. return true to allow navigation to the next page.
   * 
   */  
  pages:null,

  //user-specified event/function for what action the Finish button performs
  onFinish: null,
  
  //user-specified event/function for what action the Cancel button performs
  onCancel: null,
  
  //table of contents layout for the wizard; values are either "vertical" or "horizontal"
  tocLayout: "vertical",
  
  //these will be references to dijit button objects; should probably make these private vars
  backBtn: null,
  nextBtn: null,
  finishBtn: null,
  cancelBtn: null,
  
  //if true, wizard will size to dimensions of source node; if false, will resize to 100% container and resize with the container
  //TODO: setting fixedSize to false causes rendering problems in IE, I think mainly due to the percentage css size values
  fixedSize: true,
  
  //fixed width; only applies when fixedSize = true
  fixedWidth: "800px",
  
  //fixed height; only applies when fixedSize = true
  fixedHeight: "400px",
  
  //width of table of contents area
  tocWidth: "150px",
  
  //whether or not wiz should be auto destroyed when user clicks cancel button
  //if you have a function defined for the onCancel event, then destroyOnCancel has no effect 
  destroyOnCancel: false,


/******************************************************************************************
******************************************************************************************/
constructor: function() {

   //private vars
   this.TAB_COUNT = 0;
   this.MAX_TAB_COUNT = 0;   
   this.currTabIdx = 0;  
   this.nextBtnConnectHandle = null;
   this.backBtnConnectHandle = null;
   this.finishBtnConnectHandle = null;
   this.cancelBtnConnectHandle = null;
   
   this.borderContainer = null;
   this.stackContainer = null;
  
   this._onPageShow = [];
   this._onPageBeforeHide = [];
  
   /* this will be an array of booleans, one array element per page. use the setPageVisible method
      to take pages in or out of the wizard flow. You can use this for branching type of functionality
	  or however you wish.
   */
   this.pageVisibilityFlags = null;   

   this.newChild = null;
   this.contentDiv = null;
   this.contentDivId = null;
   this.navTab = null;
   this.navTabId = null;
   this.navTabClass = null;
   
   this.tabCompleteClass = "";
   this.tabCurrentClass = "";
   this.tabNotCompleteClass = "";
   this.tocBackgroundClass = "";
   
   this.subStepClassHidden = "WizardTabSubStepHidden";
   this.subStepClassVisible = "WizardTabSubStepVisible";
   
   //this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");

},

/******************************************************************************************
******************************************************************************************/
postCreate: function() {
  
  if (this.tocLayout == "vertical")
  {
  	this.tabCompleteClass = "WizardTabCompleted";
    this.tabCurrentClass = "WizardSelTab";
    this.tabNotCompleteClass = "WizardTab";
    this.tocBackgroundClass = "WizardTabBackground";
    
  }
  
  if (this.tocLayout == "horizontal")
  {
  	this.tabCompleteClass = "WizardTabCompletedHorizontal";
    this.tabCurrentClass = "WizardSelTabHorizontal";
    this.tabNotCompleteClass = "WizardTabHorizontal";
    this.tocBackgroundClass = "WizardTabBackgroundHorizontal";
  	
  }
  
  
  //if no id is given, we can still garantee uniqueness by using the unix epoch as the id
  if (this.id == "")
  {
  	d = new Date();
  	this.id = "" + d.getTime();
  }
	
	
  this.wizInit();
},

/******************************************************************************************
  main wizard initialization function; this is the function to construct and show the 
  wizard. Call this only after page is loaded and parsed.
******************************************************************************************/
wizInit: function()
{
  this.writeWizardTemplate();
  this.loadContent();
  this.showTab(0);
  
   
},	

/******************************************************************************************
  Call this function to show or hide specific pages. This can be useful
  for creating branch points, for example.
  
  If you set a page to false with this function, it will not appear in the wizard sequence. There
  is one element in the pageVisibilityFlags array for each page in the wizard.
  
  note that this function will not work well if you set the currently visible page to false; 
  the current page won't be hidden. This behavior could be changed, but I don't see a use case for it.

******************************************************************************************/
setPageVisible: function(idx, visible)
{
   var i = 0;
   var visPageCount = 0;
   var currVisPage = 0;

   this.pageVisibilityFlags[idx] = visible;

   if (visible)
   {
   	 //TODO: make sure "tableCell" works below IE7 and below FF3.5
   	 if (this.tocLayout == "horizontal")
   	   dojo.style(dojo.byId(this.id + "wizard_t" + idx), "display", "tableCell");
     
	 if (this.tocLayout == "vertical")
       dojo.style(dojo.byId(this.id + "wizard_t" + idx), "display", "block");
   }    
   else
   {
   	 dojo.style(dojo.byId(this.id + "wizard_t" + idx), "display", "none");
   }
    

   //get number of visible pages
   //for (i = 0; i < this.TAB_COUNT; i++) if (this.pageVisibilityFlags[i]) visPageCount++;

},


/******************************************************************************************
  internal function to dynamically create the content pages and populate the 
  wizard; this is only called after the wizard framework/template has been
  initialized via writeWizardTemplate()
******************************************************************************************/
loadContent: function()
{

var i = 0;

this.TAB_COUNT = this.pages.length;
this.MAX_TAB_COUNT = this.pages.length;

this.pageVisibilityFlags = new Array(this.TAB_COUNT);
this.newChild = new Array(this.TAB_COUNT);

var navTabTable;
var navTabTableBody;
var navTabTableRow;

//var navTabImg = null;

if (this.tocLayout == "horizontal")
{
  navTabTable = dojo.create("table", {border:"0"}, dojo.byId(this.id + "wizardNavTabContainer"));
    
  dojo.style(navTabTable, "borderCollapse", "collapse");
  dojo.style(navTabTable, "borderSpacing", "0");
  dojo.style(navTabTable, "border", "none");
  
  navTabTableBody = dojo.create("tbody", {}, navTabTable);
  navTabTableRow = dojo.create("tr", {}, navTabTableBody);
  
}

for (i = 0; i < this.TAB_COUNT; i++)
{
	 this.newChild[i] = undefined;
	 
	 //if a page initialization function is defined, add it in, otherwise no page init
	 
	 var onLoadType = typeof this.pages[i].onLoad;

	 if (onLoadType == "undefined") {
	     paneObj = {
                 id:this.id + "page" + i,
                 href:this.pages[i].url,
                 title:this.pages[i].label,
                 refreshOnShow:false,
                 style:{padding:"0"}
               };
	 }
	 
	 if (onLoadType == "string") {
	     if (this.pages[i].onLoad == "") {
             paneObj = {
                               id:this.id + "page" + i,
                               href: this.pages[i].url,
                               title:this.pages[i].label,
                               refreshOnShow:false,
                               style:{padding:"0"}
                             };
           
         }
         else {
             paneObj = {
                               id:this.id + "page" + i,
                               href: this.pages[i].url,
                               title:this.pages[i].label,
                               refreshOnShow:false,
                               onLoad: window[this.pages[i].onLoad],
                               style:{padding:"0"}
                             };
         }
	 
	 
	 }
	 
	 if (onLoadType == "function") {
	     
	     paneObj = {
                 id:this.id + "page" + i,
                 href: this.pages[i].url,
                 title:this.pages[i].label,
                 refreshOnShow:false,
                 onLoad: this.pages[i].onLoad,
                 style:{padding:"0"}
               };
	     
	 }
	 
     this.newChild[i] =  new dijit.layout.ContentPane(paneObj);
    
     
     var onShowType = typeof this.pages[i].onShow;
     
     if (onShowType == "undefined") {
         this._onPageShow[i] = undefined;
     }
     
     if (onShowType == "string") {
         if (this.pages[i].onShow != "")
             this._onPageShow[i] = window[this.pages[i].onShow];      
           else
             this._onPageShow[i] = undefined;    
     }
     
     if (onShowType == "function") {
        this._onPageShow[i] = this.pages[i].onShow;      
             
     }
     
     var onBeforeHideType = typeof this.pages[i].onBeforeHide;
	  
     if (onBeforeHideType == "undefined") {
         this._onPageBeforeHide[i] = undefined;
     }
     
     if (onBeforeHideType == "string") {
         if (this.pages[i].onBeforeHide != "")
             this._onPageBeforeHide[i] = window[this.pages[i].onBeforeHide];      
           else
             this._onPageBeforeHide[i] = undefined;       
     }
	 
     
     if (onBeforeHideType == "function") {
         this._onPageBeforeHide[i] = this.pages[i].onBeforeHide;
     }
     
	 
								  
    dijit.byId(this.id + "mainTabContainer").addChild(this.newChild[i]);
    this.newChild[i].startup();

    if (this.tocLayout == "vertical") {
      this.navTab = dojo.create("div", {id:this.id + "wizard_t" + i, innerHTML:this.pages[i].label}, dojo.byId(this.id + "wizardNavTabContainer"));	
    }
    
    if (this.tocLayout == "horizontal") {
      this.navTab = dojo.create("td", {id:this.id + "wizard_t" + i, vAlign:"bottom",  innerHTML:this.pages[i].label}, navTabTableRow);
	  	
      
	  /*
      navTabImg = document.createElement("img");
      dojo.addClass(navTabImg, "WizardTabCompletedMarker");
      dojo.attr(navTabImg, "id", this.id + "wizard_t" + i + "Img");
      dojo.attr(navTabImg, "border", "0");        
      
      this.navTab.appendChild(navTabImg.cloneNode(true));
      */
    }    
    
        
    //assumes starting on first page of wizard
    if (i == 0)
		dojo.addClass(this.navTab, this.tabCurrentClass);
    else
		dojo.addClass(this.navTab, this.tabNotCompleteClass);


    if (this.pageLevels) {
      if (this.pageLevels[i] == "2")
        dojo.addClass(this.navTab, this.subStepClassVisible);
    }
    
    
    this.borderContainer.layout(); 
}

this.currTabIdx = 0;

//default is all pages visible
for (i = 0; i < this.TAB_COUNT; i++) this.pageVisibilityFlags[i] = true;

},

//figure out which, if any, substeps to show in the table of contents.
//hide or show as appropriate, based on current page
//this only applies when substeps are defined
calcSubStepToc: function()
{
  if (!(this.pageLevels)) return;	
	
  var i = 0;
  var parentFlag = false;
  var currentHasChildren = ((this.pageLevels[i] == "1") && (this.pageLevels[this.currTabIdx + 1] == "2"));
  
 // alert('currentHasChildren ' + currentHasChildren); 
  
  for (i = this.currTabIdx; i >= 0; i--)
  {  	
  	if (this.pageLevels[i] == "1") parentFlag = true;
  	
  	if (this.pageLevels[i] == "2")
  	{
  	  if (!parentFlag)  	       
        dojo.removeClass(dojo.byId(this.id + "wizard_t" + i), this.subStepClassHidden);	   	
  	  else  	         
       dojo.addClass(dojo.byId(this.id + "wizard_t" + i), this.subStepClassHidden); 
  	}
  }
  
  var hitNextParent = false; 
  
  //ensure all appropriate, non-active substeps are hidden
  for (i = (this.currTabIdx + 1); i < this.pageLevels.length; i++)
  {
  	if (currentHasChildren)
  	{
  	  hitNextParent = (this.pageLevels[i] == "1");
     // alert('hitNextParent ' + hitNextParent);
      
  	  if ((this.pageLevels[i] == "2") && (!hitNextParent))  	      
        dojo.removeClass(dojo.byId(this.id + "wizard_t" + i), this.subStepClassHidden);
      else
        dojo.addClass(dojo.byId(this.id + "wizard_t" + i), this.subStepClassHidden);
  	}
  	else
  	{
  	  if (this.pageLevels[i] == "2") dojo.addClass(dojo.byId(this.id + "wizard_t" + i), this.subStepClassHidden);
  	}
  }
  
  
  
  	
},

/**
 * @abstract Use to jump to a specific tab page. For example, you may need to skip a page based on the user's choices. if no index is passed in, then uses this.currTabIdx as the value for the page to show
 * @param idx
 * @returns void
 */
showTab: function(idx)
{

  if (idx) this.currTabIdx = idx;
  
    
  //this is what actually changes the visible stackcontainer page
  dijit.byId(this.id + "mainTabContainer").selectChild(dijit.byId(this.id + "page" + this.currTabIdx));
  
  dijit.byId(this.id + "btnNext").attr("disabled", !((this.currTabIdx + 1) < this.TAB_COUNT));
  dijit.byId(this.id + "btnFinish").attr("disabled", ((this.currTabIdx + 1) < this.TAB_COUNT));
  dijit.byId(this.id + "btnBack").attr("disabled", !(this.currTabIdx > 0));
    
  //if next button is enabled  
  if ((this.currTabIdx + 1) < this.TAB_COUNT)
  {
    if (!(this.nextBtnConnectHandle))
    {
      //this.nextBtnConnectHandle = dojo.connect(this.nextBtn.domNode, "onclick", this, "navNext");
      this.nextBtnConnectHandle = dojo.connect(this.nextBtn, "onClick", this, "navNext");
    }
  }
  else
  {    	
  	//note: the disconnect is needed because the onclick event will still fire in IE even
  	//      when the button is disabled
    if (this.nextBtnConnectHandle) dojo.disconnect(this.nextBtnConnectHandle);   
    this.nextBtnConnectHandle = null;
  }
  
  //if back button is enabled
  if (this.currTabIdx > 0)
  {
  	if (!(this.backBtnConnectHandle))
  	{
  	  this.backBtnConnectHandle = dojo.connect(this.backBtn, "onClick", this, "navBack");
  	}
  }
  else
  {
  	//note: the disconnect is needed because the onclick event will still fire in IE even
  	//      when the button is disabled
    if (this.backBtnConnectHandle) dojo.disconnect(this.backBtnConnectHandle);	
    this.backBtnConnectHandle = null;
  }
  
 
  for (i = 0; i < this.currTabIdx; i++) dojo.byId(this.id + "wizard_t" + i).className = this.tabCompleteClass;
  dojo.byId(this.id + "wizard_t" + this.currTabIdx).className = this.tabCurrentClass;
  for (i = (this.currTabIdx + 1); i < this.TAB_COUNT; i++) dojo.byId(this.id + "wizard_t" + i).className = this.tabNotCompleteClass;
  
  
  
/*
  if (this.tocLayout == "horizontal")
  {
    for (i = 0; i < this.currTabIdx; i++) dojo.style(dojo.byId("wizard_t" + i + "Img"), "display", "inline");
    for (i = this.currTabIdx; i < this.TAB_COUNT; i++) dojo.style(dojo.byId("wizard_t" + i + "Img"), "display", "none");  	
  }
*/

  //this.calcSubStepToc();

  if (typeof this._onPageShow[this.currTabIdx] != "undefined")
  {
    this._onPageShow[this.currTabIdx](this.currTabIdx, this.id);
  }
    
},



/**
 * @abstract Called when the back or next button is clicked.
 * @param direction
 * @returns void
 */
navWizTab: function(direction)
{
  var i = 0;
  
  //have your custom function return false to keep the user on this page. For example, if there is an invalid field value, do a 'return false', and the wizard will stay on the current page. you will need to show a message or something to let the user know what is going on  
  if (typeof this._onPageBeforeHide[this.currTabIdx] != "undefined")
  {
  	if (direction == "next")
      pgResult =  this._onPageBeforeHide[this.currTabIdx](this.currTabIdx, this.id);
    
    //on false, don't allow user to go forward, but also do not prevent back
    if ((!pgResult) && (direction == "next")) return;
  }
  
  if (direction == "next")
  {
    this.currTabIdx++;
    
    while ((!(this.pageVisibilityFlags[this.currTabIdx])) && (((this.currTabIdx) < this.TAB_COUNT)))
    {   
   	  this.currTabIdx++;     
    }
  }

  if (direction == "back")
  {
    this.currTabIdx--;
    while ((!(this.pageVisibilityFlags[this.currTabIdx])) && (this.currTabIdx > 0))
    {
      this.currTabIdx--;      
    }
  }

  this.showTab(this.currTabIdx);

},



/**
 * @abstract internal function to create the basic wizard framework; this does not populate the wizard with user content pages, this only creates the guts of the wizard.
 * @returns void
 */
writeWizardTemplate: function()
{
	var tocRegion;
	var attachNode;
	
	if (this.srcNodeRef)
	  attachNode = this.srcNodeRef;
	else
      attachNode = document.body;
     
    dojo.style(attachNode, "margin", "0");
	dojo.style(attachNode, "padding", "0");    
	
    if (!(this.fixedSize))
    {
    //  dojo.style(attachNode, "height", "95%");
	//  dojo.style(attachNode, "width", "97%");	  
 
	
    }
    else
    {
      dojo.style(attachNode, "height", this.fixedHeight);
	  dojo.style(attachNode, "width", this.fixedWidth); 
    }
	
	bdrNode = dojo.create("div", {id: this.id + "borderMainNode"}, attachNode);

	/*
	var bdrWidth, bdrHeight;
	bdrWidth = parseInt(this.fixedWidth) * 0.97; // height: 95%;
	bdrHeight= parseInt(this.fixedHeight)* 0.95; // width: 97%;
	*/

	//TODO: currently this won't result in a resizable wizard due to IE rendering quirks; have to specify px values for bordercontainer size
	
	//this.borderContainer = new dijit.layout.BorderContainer({id:this.id + "wizMainContainer",gutters:false, design:"headline",style:{height:"100%", width:"100%", margin:"0", padding:"0", borderColor: "#e3e3e3", borderWidth: "1px", borderStyle: "solid"}}, dojo.byId(this.id + "borderMainNode"));
	
	this.borderContainer = new dijit.layout.BorderContainer({id:this.id + "wizMainContainer",gutters:false, design:"headline",style:{height:this.fixedHeight, width:this.fixedWidth, margin:"0", padding:"0", borderColor: "#e3e3e3", borderWidth: "1px", borderStyle: "solid"}}, dojo.byId(this.id + "borderMainNode"));


	/* table of contents region; for vertical layout will be in 
	   left region, for horizontal layout, will be in top region */	
	
	if (this.tocLayout == "vertical")
	  tocRegion = new dijit.layout.ContentPane({region:"left",style:{padding:"0", width: this.tocWidth},id:this.id + "tocRegion"});
	  
    if (this.tocLayout == "horizontal")
	  tocRegion = new dijit.layout.ContentPane({region:"top",style:{padding:"0", height:"35px", borderBottom:"1px solid #dcdcdc"},id:this.id + "tocRegion"});	  
	  
    this.borderContainer.addChild(tocRegion);
		
	navBg = dojo.create("div", {}, tocRegion.containerNode);	
	dojo.addClass(tocRegion.domNode, this.tocBackgroundClass);		
	navBgCont = dojo.create("div", {id: this.id + "wizardNavTabContainer"}, navBg);

    centerRegion = new dijit.layout.ContentPane({style:{padding:"0"}, region:"center",id:this.id + "centerRegion"});   
    this.borderContainer.addChild(centerRegion);
    
    pageStackNode = dojo.create("div", {id: this.id + "pageStackContainer", style:{height:"99%", width:"99.9%"}}, centerRegion.containerNode);        
    this.stackContainer = new dijit.layout.StackContainer({style:{padding:"0"}, id: this.id + "mainTabContainer"}, pageStackNode);
   
    bottomRegion = new dijit.layout.ContentPane({style:{padding:"0"}, region:"bottom"});       
    this.borderContainer.addChild(bottomRegion);	

    buttonArea = dojo.create("div", {id: this.id + "wizardButtonBar"}, bottomRegion.containerNode);
        
    bBack = dojo.create("button", {id: this.id + "btnBack"}, buttonArea);
    bNext = dojo.create("button", {id: this.id + "btnNext"}, buttonArea);
    bFinish = dojo.create("button", {id: this.id + "btnFinish"}, buttonArea);
    bCancel = dojo.create("button", {id: this.id + "btnCancel"}, buttonArea);

    this.backBtn = new dijit.form.Button({label:"< Back"}, bBack);
    this.nextBtn = new dijit.form.Button({label:"Next >"}, bNext);
    this.finishBtn = new dijit.form.Button({style:{marginLeft:"10px"}, label:"Finish"}, bFinish);
    this.cancelBtn = new dijit.form.Button({label:"Cancel"}, bCancel);

    this.backBtnConnectHandle = dojo.connect(this.backBtn, "onClick", this, "navBack");
    this.nextBtnConnectHandle = dojo.connect(this.nextBtn, "onClick", this, "navNext");
    
    if (this.onFinish)
      this.finishBtnConnectHandle = dojo.connect(this.finishBtn, "onClick", this, window[this.onFinish]);
    
    
    if (this.onCancel) {
      this.cancelBtnConnectHandle = dojo.connect(this.cancelBtn, "onClick", this, window[this.onCancel]);	
    }
    else {
      if (this.destroyOnCancel)
	    this.cancelBtnConnectHandle = dojo.connect(this.cancelBtn, "onClick", this, this.destroyRecursive);	
    }

    this.borderContainer.startup();    
    this.stackContainer.startup();
    
    
    
},

navBack: function()
{
  this.navWizTab("back");	
},

navNext: function()
{
  this.navWizTab("next");	
},

/******************************************************************************************
******************************************************************************************/
destroyRecursive: function()
{

  if (this.srcNodeRef) {
      dojo.query('[widgetId]', this.srcNodeRef).forEach(function(node){
          var widget = dijit.byNode(node);
          if(widget) {if (widget.destroyRecursive) { widget.destroyRecursive(false); } }
        });      
  }
 
  this.inherited(arguments);
  
}

});





