console.warn("'ibm.tivoli.tip.dijit.TivWizard' is deprecated, use 'dojoe.wizard.Wizard'(TED) instead.");
dojo.provide("ibm.tivoli.tip.dijit.TivWizard");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.requireLocalization("ibm.tivoli.tip.dijit", "resources");
dojo.require("ibm.tivoli.tip.dijit.TivPage");
dojo.require("ibm.tivoli.tip.dijit.TivSubstep");

/**
 * Inherit from TivPageContainer which contains the common functions of both
 * the notebook and wizard.
 */
dojo.require("ibm.tivoli.tip.dijit.TivPageContainer");

/**
 * 
 * This class is an extension of TivPageContainer that handles all the
 * wizard specific functions as specified in the UI guidelines.  The table
 * of contents layout may be vertical or horizontal.  It also includes the 
 * advanced navigation features such as the ability to have sub-steps, 
 * sub-steps that loop and to jump backwards to a previous page by clicking 
 * on an item in the table of contents.
 * 
 * Extra capabilities that have been added include the ability to skip from 
 * whatever page is displayed to the next main step, enable and disable steps
 * and buttons, hide the button area, indicate a step is required and restart 
 * the wizard. 
 * 
 * Creating a TivWizard in markup
 * <div id="note" dojoType="ibm.tivoli.tip.dijit.TivWizard">
 *    <div dojoType="ibm.tivoli.tip.dijit.tests.tivpages.Welcome"></div>
 *    <div dojoType="ibm.tivoli.tip.dijit.tests.tivpages.BaseWidget"></div>
 *    <div dojoType="ibm.tivoli.tip.dijit.tests.tivpages.General"></div>
 *    <div dojoType="ibm.tivoli.tip.dijit.tests.tivpages.Security"></div>
 *    <div dojoType="ibm.tivoli.tip.dijit.tests.tivpages.Summary" ></div>
 *  </div> 
 *
 * Events:
 *  OnPageSelected: when a page is made active
 *  OnBack: when the Back button is clicked
 *  OnNext: when the Next button is clicked
 *  OnOK: when the Finish/OK button is clicked
 *  OnCancel: when the Cancel button is clicked
 */
dojo.declare("ibm.tivoli.tip.dijit.TivWizard", 
            [dijit._Widget, dijit._Templated, ibm.tivoli.tip.dijit.TivPageContainer], 
{  
  widgetsInTemplate: true,
  templatePath: "",

  mode: "w",               // wizard
  renderNoArrows: false,   // if true render ToC without arrow graphics, only applicable in horizontal layout
  slider: null,            // for scrolling ToC, only applicable in horizontal layout
  tocStepsArea: null,      // for main steps of ToC, only applicable in horizontal layout
  tocSubstepsArea: null,   // for sub steps of ToC, only applicable in horizontal layout
  lastPageSelected: null,  // keep track of the last page that was selected
  
  backLabel: "",
  nextLabel: "",
  finishLabel: "",
  cancelLabel: "",
  
  postMixInProperties: function() 
  {
    console.log("TivWizard.postMixInProperties");    
    if (this.nested)
      this.templatePath = dojo.moduleUrl("ibm.tivoli.tip.dijit", "templates/TivWizardNested.html");
    else
      this.templatePath = dojo.moduleUrl("ibm.tivoli.tip.dijit", "templates/TivWizard.html");
    console.log("templatePath is " + this.templatePath);
    
    var string_table = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");
    this.backLabel = string_table.BACK;
    this.nextLabel = string_table.NEXT;
    this.finishLabel = string_table.FINISH;
    this.cancelLabel = string_table.CANCEL;
    this.inherited(arguments);
  },
  
  buildRendering: function()
  {
    console.log("TivWizard.buildRendering");        

    this.inherited(arguments);

    // if ToC layout is horizontal... 
    if ('h' === this.layout) {
      var child = this._tocArea;
      
      // add main step area
      this.tocStepsArea = dojo.create("div", {});          
      dojo.place(this.tocStepsArea, child.domNode, 0);
      
      if (this.renderNoArrows) {
        dojo.addClass(this.tocStepsArea, "steps-area-noarrows");            
        dojo.place(this._toc, this.tocStepsArea, 0);
      }
      else {
        dojo.addClass(this.tocStepsArea, "steps-area");            
        
        // create slider components which are hidden until needed
        this.slider = new ibm.tivoli.tip.dijit.TivWizardToCSlider({
          toc: this._toc, 
          tocSteps: this.tocStepsArea, 
          ltr: this.isLeftToRight()}, null);
      }
      
      // add hidden sub-step area
      this.tocSubstepsArea = dojo.create("div", {className: "substeps-area"});
      dojo.place(this.tocSubstepsArea, child.domNode, 1);          
    }    
  },
  
  _setContainerType: function(page)
  {
    page.attr("wizard", this);
  },
  
  updateUI: function()
  {        
    // disable the Back/Next buttons if we are on the first/last page
    var curr_page = this._pageContainer.selectedChildWidget;
    var page_count = this._pages.length;

    var isFirst = true, isLast = true;
    if(page_count > 0) {
      isFirst = curr_page === this._pages[0];
      isLast =  curr_page === this._pages[this._pages.length-1];
    }
    this._backButton.attr("disabled", isFirst);
    this._nextButton.attr("disabled", isLast);
        
    // disable the Next button if the selected page does not validate
    var isCurrentPageValid = curr_page && curr_page.isValid(); 
    if( !isCurrentPageValid )
      this._nextButton.attr("disabled", true);    

    // disable the Finish/OK button until all pages are valid
    var allValid = dojo.every(this._pages, function(p){
      var v = p.isValid();
      v = v && (p.visited || p === curr_page);
      return v;
     }, this);
    this._finishButton.attr("disabled", !allValid);
    
    this.inherited(arguments);

    if ('h' === this.layout) {     
      // update UI for all substeps
      // substeps may be hidden due to jump back and need to be updated too
      dojo.forEach(this._pages, function(aPage) {
        if (aPage.hasSubsteps()) {
          dojo.query("li[pageID]", aPage.getSubstepsList()).forEach(function(n, i) {
            var pid = dojo.attr(n, "pageID");
            var p = dijit.byId(pid);
            dojo[(p && p.visited) ? "addClass" : "removeClass"](n, "tiv-page-visited");
            dojo[(p && (!p.visited || p.validated)) ? "removeClass" : "addClass"](n, "tiv-page-invalid");
            dojo[(p == curr_page) ? "addClass" : "removeClass"](n, "tiv-page-selected");                    
          }, this);
        }
      }, this);
      
      // if current page is a substep update main step to look selected
      if (!curr_page.isMainStep()) {
        var mainPage = curr_page.getMainPage();
        dojo.query("li[pageID]", this._toc).every(function(n, i) {
          var pid = dojo.attr(n, "pageID");
          if (pid == mainPage.id) {
            dojo.addClass(n, "tiv-page-selected");
            return false;
          }
          else
            return true;
        });              
      }
      
      // show or hide substeps area
      var newHeight;
      if (curr_page.hasSubsteps() || !curr_page.isMainStep())
        newHeight = this.tocStepsArea.offsetHeight + this.tocSubstepsArea.offsetHeight;
      else
        newHeight = this.tocStepsArea.offsetHeight;
      if (dojo.style(this._pageContainer.domNode, "top") != newHeight)
      {
        dojo.style(this._pageContainer.domNode, "top", newHeight + "px");        
        dojo.style(this._tocArea.domNode, "height", newHeight + "px");
      }
      
      if (this.slider != null) {
        // calculate required width for all step arrows in ToC
        this.slider.recalcToCWidth();
      
        // update slider
        this.slider.updateSliderUI(null);      
      }
    }
  },
  
  /**
   * Overrides base function to also resize the slider.  
   * (Note: for updating the slider, cannot rely on connecting a handler
   * to the resize event for the _tocArea div.  Problems occur in Firefox
   * where the event is not fired often enough so sometimes the slider
   * button will disappear.)
   */
  resize: function(newSize) 
  {
    if (newSize != null) {
      this.inherited(arguments);
      if (this.slider && newSize.w != null)
        this.slider.updateSliderUI(newSize.w);    
    }
  },
  
  /** 
   * Restart the wizard
   * 
   * Enable jumping backwards (may have been disabled)
   * 
   * Show the buttons (may have been hidden)
   * 
   * Remove the "visited" flag
   * 
   * Collapse substeps
   * 
   * Give each page a chance to reset to the original values
   * 
   * Reset to the first page
   */
  restartWizard: function() 
  {
    console.log("TivWizard.restartWizard");

    if (this.slider != null)
      this.slider.reset();
    
    // the bottom button area might have been hidden
    this.showAllButtons();

    // individual buttons might have been hidden
    this.hideBackButton(false);
    this.hideNextButton(false);
    this.hideCancelButton(false);
    this.hideFinishButton(false);

    // or maybe disabled
    this.enableNextButton(true);
    this.enableCancelButton(true);
    this.enableFinishButton(false);

    // get all of the children
    var theKids = this._pageContainer.getChildren();

    // do for all children
    for (var i = 0; i < theKids.length; i++) {
      // mark as not visited
      theKids[i].visited = false;

      // hide any kids
      theKids[i].hideSubsteps();

      // if the child supports the "reset" method
      if ("function" == typeof theKids[i].reset) {

        // give the kid a chance to reset
        theKids[i].reset();
      }
    }

    // the TOC might have been disabled
    this.enableJumpBack();

    // and then select the first page
    this.selectPage(theKids[0]);
  },

  /**
   * Reset the label for the "Finish" button
   */
  resetFinishLabel: function(newName) 
  {
    console.log("TivWizard.resetFinishLabel - " + newName);
    dojo.attr(this._finishButton, "label", newName);
  },

  /**
   * Mark pages as having been visited up to the one specified
   */
  markPreviousPagesVisited: function(aPage) 
  {
    console.log("TivWizard.markPreviousPagesVisited");

    // get all of the children
    var theKids = this._pageContainer.getChildren();

    // do for all children
    var foundPage = false;
    for (var i = 0; !foundPage && i < theKids.length; i++) {
      // if this is not the page specified
      if (aPage != theKids[i]) {
        // if the kid is not disabled
        if (theKids[i].isDisabled() != true) {

          // mark as visited
          theKids[i].visited = true;
        }

        // else this is the end of the line
      } 
      else {
        // mark as found
        foundPage = true;
      }
    }
  },

  /**
   * Disable jumping back to every page except for the one specified
   */
  disableJumpBack: function(exceptFor) 
  {
    console.log("TivWizard:disableJumpBack");

    // get all of the children
    var theKids = this._pageContainer.getChildren();

    // do for all children
    for (var i = 0; i < theKids.length; i++) {

      // if this is not the exception)
      if (exceptFor != theKids[i]) {

        // disable it
        theKids[i].disableJumpBack();
      }
    }
  },

  /**
   * Enable jumping back to every page
   */
  enableJumpBack: function() 
  {
    console.log("TivWizard.enableJumpBack");

    // get all of the children
    var theKids = this._pageContainer.getChildren();

    // do for all children
    for (var i = 0; i < theKids.length; i++) {
      // enable it
      theKids[i].enableJumpBack();
    }
  },

  /**
   * When a page is being added it might be a substep. If so it needs to
   * be added to the previous main step.
   */
  _onPageAdded: function(page, insertIndex) 
  {
    console.log("TivWizard._onPageAdded");

    this.inherited(arguments);

    // if this is a substep
    if (page.isMainStep() == false) {
      // get the substep's main page
      var mainPage = page.getMainPage();
      if (mainPage != null) {
        // add as a substep
        mainPage.addSubstep(page);
      } 
      else {
        // message
        console.error("failed to add substep to main page");
      }
    }
  },
  
  /*
   * Gets the previous main page.  If page is a substep, gets the main
   * page for this substep.  Note the page may be disabled.
   */
  getPreviousMainPage: function(page)
  {
    // get the previous page
    var previousPage = this._pageContainer._getSiblingOfChild(page, -1);

    // while the previous page is a substep
    while (previousPage.isMainStep() == false) {
      // get the previous page
      previousPage = this._pageContainer._getSiblingOfChild(previousPage, -1);
    }
    
    // if the previous page is a main page
    if (previousPage.isMainStep() == false) {
      console.log("no previous main page");
      previousPage = null;
    }
    return previousPage;
  },
  
  /**
   * Gets the next main page.  Note the page may be disabled.
   */
  getNextMainPage: function(page)
  {
    // get the next page
    var nextPage = this._pageContainer._getSiblingOfChild(page, 1);

    // while the next page is a substep
    while (nextPage.isMainStep() == false) {
      // get the next page
      nextPage = this._pageContainer._getSiblingOfChild(nextPage, 1);
    }
    
    // if the next page is a main page
    if (nextPage.isMainStep() == false) {
      console.log("no next main page");
      nextPage = null;
    }
    return nextPage;    
  },
  
  /**
   * Jumps to the next enabled step in the loop which may be a substep
   * or the main step.
   */
  jumpToNextLoopStep: function()
  {
    console.log("TivWizard.jumpToNextLoopStep");
    var nextPage = null;
    
    // get the current page
    var tempPage = this.lastPageSelected;
    
    if (tempPage.isMainStep())
    {
      // next step must be a substep so return first enabled substep
      if (tempPage.hasSubsteps())
      {
        dojo.every(tempPage.getSubsteps(), function(substep) {
          if (!substep.isDisabled()) {
            nextPage = substep;
            return false;
          }
          else
            return true;
        }, this);
      }
    }
    else {
      // current is a substep, get the next substep
      var nextPossible = this._pageContainer._getSiblingOfChild(tempPage, 1);

      // while the next page is a substep and disabled
      while (nextPossible.isMainStep() == false && nextPossible.isDisabled()) {
        // get the next page
        nextPossible = this._pageContainer._getSiblingOfChild(nextPossible, 1);
      }
      
      // if next page is a substep save it for selection
      if (nextPossible != null && !nextPossible.isMainStep())
        nextPage = nextPossible;
      else {
        // otherwise next page is a main page, make sure we
        // loop around and load the main page in this loop
        nextPage = tempPage.getMainPage();
      }
    }

    if (nextPage != null) {
      // select the new page
      this.selectPage(nextPage);
    }
  },
  
  /**
   * Jumps to the previous enabled step in the loop which may be a substep
   * or the main step.  Note if the current page is the main step, the jump
   * will not occur.
   */
  jumpToPreviousLoopStep: function()
  {
    console.log("TivWizard.jumpToPreviousLoopStep");
    
    // get the current page
    var tempPage = this.lastPageSelected;
     
    if (!tempPage.isMainStep()) {
      // get the previous page
      var previousPossible = this._pageContainer._getSiblingOfChild(tempPage, -1);
      
      // while the next page is a substep and disabled
      while(previousPossible.isMainStep() == false && previousPossible.isDisabled()) {
        // get the previous page
        previousPossible = this._pageContainer._getSiblingOfChild(previousPossible, -1);
      }
      
      if (previousPossible != null) {
        // select the new page
        this.selectPage(previousPossible);
      }
    }
  },

  /**
   * Using the skip function takes you to the next main page that is enabled. 
   * But you still want to hide any substeps if you happen to be in a main
   * page with substeps, and you want to hide all of the substeps for a main
   * page if you happen to be in a substep.
   * 
   * Also if the next main step contains substeps but the main step should not
   * show its page, the first enabled substep page will be selected.
   */
  skipToNextMainStep: function() 
  {
    console.log("TivWizard:skipToNextMainStep");

    // get the current page
    var tempPage = this.lastPageSelected;

    // assume it is a main step
    var mainPage = tempPage;

    // while still on a substep
    while (mainPage.isMainStep() == false) {
      // back up another page
      mainPage = this._pageContainer._getSiblingOfChild(mainPage, -1);
    }

    // hide any substeps
    mainPage.hideSubsteps();

    // mark it as visited
    tempPage.visited = true;

    // find the next page
    var whoIsNext = this._pageContainer._getSiblingOfChild(tempPage, 1);

    // while that page is a substep
    while (whoIsNext != null && (whoIsNext.isMainStep() == false || whoIsNext.isDisabled())) {
      // find the next page
      whoIsNext = this._pageContainer._getSiblingOfChild(whoIsNext, 1);
    }
    
    // select the new page
    if (whoIsNext != null) {
      // check if we need to show substep directly
      if (whoIsNext.showTitleOnly())
        this.goToSubstep(whoIsNext);
      else 
        this.selectPage(whoIsNext);
    }     
  },

  /**
   * The user has clicked on something in the table of contents, so the
   * container needs to display that page.
   * 
   * Hide any substeps of the jump off page if the page is not a substep
   * of the jump off page.
   * 
   * Let the developer decide when to mark the pages invalid.
   * 
   * This also means that the page where the jump originated needs to be
   * marked as visited in order to return to that page.  This can be
   * overridden if allowMarkAsComplete returns false.
   */
  jumpBackToPage: function(aPage) 
  {
    console.log("TivWizard.jumpBackToPage");

    // the jump off point might have substeps or be a substep
    // hide them as long as page to jump to isn't one of those substeps
    var jumpOffMainPage = (this.lastPageSelected.isMainStep()) ? this.lastPageSelected
                                                               : this.lastPageSelected.getMainPage();
    var jumpToMainPage = (aPage.isMainStep()) ? aPage : aPage.getMainPage();    
    if (jumpOffMainPage != jumpToMainPage)
      jumpOffMainPage.hideSubsteps();       

    // mark where we were as having been visited
    if (this.lastPageSelected.allowMarkAsComplete())
      this.lastPageSelected.visited = true;
    
    // check if we need to show substep directly
    if (aPage.showTitleOnly())
      this.goToSubstep(aPage);
    else {
      // then let the parent class handle selecting the page
      this.selectPage(aPage);
    }
  },
  
  /**
   * Selects the first enabled substep for the given main page.
   */
  goToSubstep: function(mainPage)
  {
    console.log("TivWizard.goToSubstep");
    mainPage.showSubsteps();
    dojo.every(mainPage.getSubsteps(), function(substep) {
      if (!substep.isDisabled()) {
        this.selectPage(substep);
        return false;
      }
      else 
        return true;
    }, this);    
  },

  /**
   * Hide all of the buttons by adding a class
   */
  hideAllButtons: function() 
  {
    console.log("TivWizard.hideAllButtons");

    dojo.addClass(this._buttonArea.domNode, "tiv-wiz-footer-hidden");
  },

  /**
   * Show all of the buttons by removing a class
   */
  showAllButtons: function() 
  {
    console.log("TivWizard.hideAllButtons");

    dojo.removeClass(this._buttonArea.domNode, "tiv-wiz-footer-hidden");
  },

  /** 
   * Hide (or show) the "back" button 
   */
  hideBackButton: function(shouldHide) 
  {
    if (shouldHide) 
      dojo.style(this._backButton.domNode, "display", "none");
    else 
      dojo.style(this._backButton.domNode, "display", "inline-block");
  },

  /**
   * Hide (or show) the "next" button
   */
  hideNextButton: function(shouldHide) 
  {
    if (shouldHide) 
      dojo.style(this._nextButton.domNode, "display", "none");
    else 
      dojo.style(this._nextButton.domNode, "display", "inline-block");
  },

  /**
   * Enable (or disable) the "back" button
   */
  enableBackButton: function(shouldEnable) 
  {
    this._backButton.attr("disabled", !shouldEnable);
  },

  /**
   * Enable (or disable) the "next" button
   */
  enableNextButton: function(shouldEnable) 
  {
    this._nextButton.attr("disabled", !shouldEnable);
  },

  /**
   * If a change is made to a page that was reached by jumping back,
   * that might cause subsequent pages to have information or choices
   * that are no longer valid.
   * 
   * Call this function to invalidate the selected page and all
   * subsequent pages.
   */
  unmarkVisitedPages: function(aPage) 
  {
    while (aPage != null) {
      aPage.visited = false;
      aPage = this._pageContainer._getSiblingOfChild(aPage, 1);
    }
    this.updateUI();
  },

  /**
   * There are some differences when a page is selected.
   * 
   * This is where the last page selected is reset.
   * 
   * The selected page should not be marked as having been visited, a
   * change from the parent class.
   */
  _onPageSelected: function(page) 
  {
    console.log("TivWizard._onPageSelected");

    // keep track of which page was last selected
    this.lastPageSelected = page;

    // let the parent class handle this
    this.OnPageSelected(this, page);

    // if the page can handle this function
    if ("function" == typeof page.OnPageSelected) {
      // let the page know it has been selected
      page.OnPageSelected(this);
    }

    // update the display
    this.updateUI();

    // if the page can handle this function
    if ("function" == typeof page.AfterPageSelected) {
      // let the page get final control
      page.AfterPageSelected(this);
    }
  },

  /**
   * When "Finish" is clicked any substeps that are open should be
   * hidden
   */
  _onClickFinish: function(event) 
  {
    console.log("TivWizard._onClickFinish");

    // get the page current in view
    var tempPage = this.lastPageSelected;

    // see if the page wants to allow the customer to move on
    var shouldAllow = tempPage.allowFinish();

    // if so
    if (shouldAllow) {

      // while this is a substep
      while (tempPage.isMainStep() == false) {
        // get the page preceding this one
        tempPage = this._pageContainer._getSiblingOfChild(tempPage, -1);
      }

      // then hide any substeps
      tempPage.hideSubsteps();

      // and allow any additional processing to occur
      this.OnOK(this);
    }
  },

  /**
   * This has a number of differences from the parent class.
   * 
   * If a page has substeps, then they should be hidden from view at
   * this time.
   * 
   * If the previous page is really a substep then the page that should
   * be shown is the parent of that substep.
   * 
   * When backing up to the parent of a substep the substeps should be
   * marked as not having been visited.
   * 
   * Disabled pages should be avoided.
   * 
   * Changing behavior:
   * - To completely override the behavior of the Back button, have
   * allowBack() of current page return false and connect customized 
   * handler function to _onClickBack event.
   * - To perform additional processing after previous page has been
   * selected, connect customized handler function to OnBack event.
   */
  _onClickBack: function(evt) 
  {
    console.log("TivWizard._onClickBack");
    
    // allow any additional processing to occur
    this.OnBack(this);
    
    // get the current position
    var tempPage = this.lastPageSelected;
    
    // see if the page wants to allow the customer to move back
    var shouldAllow = tempPage.allowBack();
    
    // if so
    if (shouldAllow) {
      
      // get the next page
      var whoIsNext = tempPage;
      whoIsNext = this._pageContainer._getSiblingOfChild(whoIsNext, 1);   
      
      // if the next page is marked as unvisited (no jump back)
      if (whoIsNext != null && !whoIsNext.visited) {
        // mark current page as unvisited
        tempPage.visited = false;      
      }
  
      var startingPage = tempPage;
      
      // if the current page is not a substep
      if (tempPage.isMainStep() == true) {
  
        // hide any substeps for this page
        tempPage.hideSubsteps();         
      }
      else {
        // current page is a substep
        var mainPage = tempPage.getMainPage();
		//Vyshantha: April7,2011: Fix for the back button issue on TivWizard
		if (whoIsNext != null && !whoIsNext.visited) {
          // mark main step as unvisited as long as no jump back
          mainPage.visited = false;
        }
        
        if (mainPage.showTitleOnly()) {
          // get the previous enabled page
          var previous = this._pageContainer._getSiblingOfChild(tempPage, -1);
          while (previous != null && previous.isDisabled())
              previous = this._pageContainer._getSiblingOfChild(previous, -1);      
          
          // if previous page is this substep's main page, then skip it
          if (previous == mainPage) {
            startingPage = previous;
            mainPage.hideSubsteps();
          }
        }
      }
      
      // get the previous page that is not disabled
      var whoIsPrevious = this._pageContainer._getSiblingOfChild(startingPage, -1);
      
      // while that page is marked disabled or if page is a substep and its
      // main page is disabled
      while (whoIsPrevious != null && (whoIsPrevious.isDisabled() || 
       !whoIsPrevious.isMainStep() && whoIsPrevious.getMainPage().isDisabled())) {
          // get the previous page
          whoIsPrevious = this._pageContainer._getSiblingOfChild(whoIsPrevious, -1);      
      }
          
      // if there is still a page to load
      if (whoIsPrevious != null) {      
        
        // if that page is a substep 
        if (whoIsPrevious.isMainStep() == false) {
          
          // keep track of substep
          var subStepToSelect = whoIsPrevious;
          
          // get previous main page
          whoIsPrevious = subStepToSelect.getMainPage();
          
          // if main page is not part of a loop
          // (otherwise select main page)
          if (!whoIsPrevious.isPartOfLoop()) {
            // make all substeps for this page visible then the substep can be selected
            whoIsPrevious.OnPageSelected(this);
            
            // reset page to be selected to our substep
            whoIsPrevious = subStepToSelect;
          }        
        }
        
        // select that page
        this.selectPage(whoIsPrevious);
      }
  
      // mark this page widget as not having been visited if
      // we didn't get here from jumping back
      if (!tempPage.visited)
        this._pageContainer.selectedChildWidget.visited = false;     
      
    }
  },

  /** 
   * Moving to the next page is slightly different if there are substeps
   * involved.
   * 
   * Substeps should only be seen when their parent is the last main
   * step to be visited. They need to disappear when the next main step
   * is entered.
   * 
   * Disabled pages should be avoided.
   * 
   * Changing behavior:
   * - To completely override the behavior of the Next button, have
   * allowNext() of current page return false and connect customized 
   * handler function to _onClickNext event.
   * - To perform additional processing after next page has been
   * selected, connect customized handler function to OnNext event.
   */
  _onClickNext: function (evt) 
  {
    console.log("TivWizard._onClickNext");
    
    // allow any additional processing to occur
    this.OnNext(this);

    // get the page current in view
    var tempPage = this.lastPageSelected;

    // see if the page wants to allow the customer to move on
    var shouldAllow = tempPage.allowNext();
    
    // if so
    if (shouldAllow) {
      
      // if current page in view has no substeps, mark the page as 
      // having been visited
      if (!tempPage.hasSubsteps())
        tempPage.visited = true;

      // get the next page
      var nextPossible;    
      if (tempPage.isPartOfLoop()) {
        // current page is part of a loop so get next main page
        nextPossible = this.getNextMainPage(tempPage);
      }
      else
        nextPossible = this._pageContainer._getSiblingOfChild(tempPage, 1);

      // while that page is marked disabled
      while (nextPossible != null && nextPossible.isDisabled()) {
        // if disabled page has substeps
        if (nextPossible.hasSubsteps()) {
          // get next main page
          nextPossible = this.getNextMainPage(nextPossible);
        }
        else  {
          // get the next page
          nextPossible = this._pageContainer._getSiblingOfChild(nextPossible, 1);
        }
      }

      // if there is still a next page to load
      if (nextPossible != null) {
        
        // if next page is a main page and current page is a substep or main page for loop
        if (nextPossible.isMainStep() && (!tempPage.isMainStep() || tempPage.isPartOfLoop())) {
          
          // get the current main page
          var currentMainPage;
          
          if (tempPage.isPartOfLoop()) {
            if (tempPage.isMainStep())
              currentMainPage = tempPage;
            else 
              currentMainPage = tempPage.getMainPage();
            
            // mark all loop substeps as visited
            dojo.forEach(currentMainPage.getSubsteps(), function(page) {
              page.visited = true;
            }, this);
          }
          else {
            currentMainPage = tempPage.getMainPage();          
          }
          
          // hide substeps for that main step
          currentMainPage.hideSubsteps();
          
          // mark the main step as having been visited
          currentMainPage.visited = true;
        }       
        
        // check if we need to show substep directly
        if (nextPossible.showTitleOnly())
          this.goToSubstep(nextPossible);
        else {
          // select that page
          this.selectPage(nextPossible);
        }
      }      
    }        
  },
  
  OnBack: function(wiz)
  {
    // override or connect to me
  },
  
  OnNext: function(wiz)
  {
    // override or connect to me
  }
  
});

/**
 * Slider support for horizontal navigation
 */
dojo.declare("ibm.tivoli.tip.dijit.TivWizardToCSlider", null, 
{
  toc: null,        // ordered list for ToC
  tocSteps: null,   // div containing toc main steps
  ltr: true, 
  tocStepsMargin: 10,
  tocMarginLeft: 2,
  oldTocStepsWidth: 0,
  
  totalWidthToCItems: 0,
  oneButtonWidth: 29,
  visibleButtonWidth: 0,  // width plus margin of visible buttons 

  sliderOuterDiv: null,
  sliderLeft: null,
  sliderRight: null,
  sliderLeftImg: dojo.moduleUrl("ibm.tivoli.tip.dijit", "themes/images/tivwizard/slider_left.png"),
  sliderRightImg: dojo.moduleUrl("ibm.tivoli.tip.dijit", "themes/images/tivwizard/slider_right.png"),
  
  t: null,
  slideDelay: 2,  // 2 milliseconds
  slidePixel: 2,
  pos: 0,
  slideDirection: "",

  slideRightLabel: "",
  slideLeftLabel: "",
  

  /**
   * Constructor which adds nodes to ToC content pane for the slider, e.g.
   * <div for main steps - passed in>
   *   <img for left slider button />
   *   <div for containing ToC (outer) >
   *     <ol for ToC (inner) - passed in />
   *   </div>
   *   <img for right slider button>
   * </div>
   * 
   * Takes three arguments:
   *   toc       <ol> for ToC
   *   tocSteps  div for main steps
   *   ltr       flag indicating whether container text direction is left-to-right
   *             or not (right-to-left)
   */
  constructor: function(arguments)
  {
    console.log("TivWizardToCSlider.constructor");
    this.toc = arguments.toc;
    this.tocSteps = arguments.tocSteps;
    this.ltr = arguments.ltr;
        
    // strings for slider buttons
    var string_table = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");
    this.slideRightLabel = string_table.SLIDE_RIGHT;
    this.slideLeftLabel = string_table.SLIDE_LEFT;

    this.sliderOuterDiv = dojo.create("div", { className: "slider-outer" });          
    dojo.place(this.toc, this.sliderOuterDiv, 0);
    dojo.addClass(this.toc, "slider-inner");        
    
    this.sliderLeft = dojo.create("img", {
      src: this.sliderLeftImg.toString(),
      alt: this.slideLeftLabel,
      className: "slider-left"
    });
    this.sliderRight = dojo.create("img", {
      src: this.sliderRightImg.toString(),
      alt: this.slideRightLabel,
      className: "slider-right"
    });
    dojo.place(this.sliderLeft, this.tocSteps, 0);
    dojo.place(this.sliderOuterDiv, this.tocSteps, 1);
    dojo.place(this.sliderRight, this.tocSteps, 2);
    
    // initially hide buttons
    dojo.style(this.sliderLeft, "display", "none");        
    dojo.style(this.sliderRight, "display", "none");        
    
    // connect button handlers
    dojo.connect(this.sliderLeft, "onclick", this, this.slideLeft);
    dojo.connect(this.sliderRight, "onclick", this, this.slideRight);
  },
  
  /**
   * Re-calculate total width of all ToC items (step arrows).
   */
  recalcToCWidth: function()
  {
    // With IE arrow image parts may wrap as the arrow styles change.  
    // To ensure we have the correct total width, we temporarily increase
    // the width of the ToC list to prevent any truncation of parts and 
    // then add up the widths of the arrow parts, i.e. front, middle and 
    // tail.
    dojo.style(this.toc, "width", (this.totalWidthToCItems * 2) + "px");  // should be sufficient   
    this.totalWidthToCItems = 0;
    dojo.query("i", this.toc).forEach(function(n) {
      this.totalWidthToCItems += n.offsetWidth; 
    }, this);
    dojo.query("span", this.toc).forEach(function(n) {
      this.totalWidthToCItems += n.offsetWidth;      
    }, this);
    dojo.query("b", this.toc).forEach(function(n) {
      this.totalWidthToCItems += n.offsetWidth;      
    }, this);
    dojo.style(this.toc, "width", this.totalWidthToCItems + "px");
  },
    
  /**
   * Right slider button is made visible if not all step arrows can
   * be displayed in the pane.  Once user clicks on button and slides
   * the arrows, then the left slider button is added.  Both buttons
   * remain visible unless the pane is resized so that all steps can
   * be displayed.
   */
  updateSliderUI: function(newWidth)
  {
    // newWidth will be passed in when toc content pane is being resized;
    // use it instead of scrollWidth since that value may not be updated yet
    var slideWidth = (newWidth != null) ? newWidth : dojo.style(this.tocSteps, "width");
    if (this.totalWidthToCItems >= slideWidth - this.tocStepsMargin)
    {
      if (this.ltr) {
        // Show right slider button only if no buttons currently showing
        if (this.sliderRight.style.display == "none") {
          dojo.style(this.sliderRight, "display", "inline");
          this.visibleButtonWidth = this.oneButtonWidth;        
        }
        else 
          this.finishSlide(this.sliderLeft, slideWidth);
      }
      else {
        // Show left slider button only if no buttons currently showing
        if (this.sliderLeft.style.display == "none") {
          dojo.style(this.sliderLeft, "display", "inline");
          this.visibleButtonWidth = this.oneButtonWidth;        
        }        
        else
          this.finishSlide(this.sliderRight, slideWidth);
      }
      dojo.style(this.sliderOuterDiv, "width", (slideWidth - this.visibleButtonWidth) + "px");
      this.oldTocStepsWidth = slideWidth;
    }              
    else if (this.visibleButtonWidth > 0)
    {
      // no need for slider, hide everything
      dojo.style(this.sliderLeft, "display", "none");        
      dojo.style(this.sliderRight, "display", "none");        
      dojo.style(this.sliderOuterDiv, "width", "100%");
      dojo.style(this.toc, "left", "0px");
      this.pos = 0;
      this.slideDirection = "";
      this.visibleButtonWidth = 0;
    }    
  },

  /**
   * Handles case where both slider buttons are visible, toc has been 
   * scrolled so first and last step(s) are no longer visible and window 
   * is being resized.  As the window width is increased, once the last 
   * step is visible the toc needs to be slid so that the first step(s) 
   * become visible too.
   */
  finishSlide: function(button, slideWidth) 
  {
    if (button.style.display != "none") {
      var tocLeft = dojo.style(this.toc, "left");
      if (this.ltr) {
        // left-to-right direction
        if (tocLeft < 0) {
          var tocWidthShowing = dojo.style(this.toc, "width") + tocLeft;
          if (slideWidth > (tocWidthShowing + this.visibleButtonWidth + this.tocMarginLeft))
          {
            this.pos = tocLeft + slideWidth - this.oldTocStepsWidth;
            dojo.style(this.toc, "left", this.pos + "px");
          }                  
        }
      }
      else {
        // right-to=left direction
        if (tocLeft > 0) {
          var tocWidthShowing = 0 - (dojo.style(this.toc, "width") - tocLeft);
          if (slideWidth > (tocWidthShowing + this.visibleButtonWidth + this.tocMarginLeft))
          {
            this.pos = tocLeft - slideWidth + this.oldTocStepsWidth;
            dojo.style(this.toc, "left", this.pos + "px");
          }                            
        }
      }
    }
  },
  
  /**
   * Called to show the second slider button after first slider button
   * has been clicked.
   */  
  showSecondButton: function(button, marginStyle, multiplier)
  {
    this.visibleButtonWidth *= 2;
    this.pos = multiplier * this.oneButtonWidth;
    dojo.style(button, "display", "inline");
    dojo.style(this.toc, "left", this.pos + "px");
    dojo.style(this.sliderOuterDiv, "width", (dojo.style(this.tocSteps, "width") - this.visibleButtonWidth) + "px");
    dojo.style(this.toc, marginStyle, this.tocMarginLeft + "px");    
  },
  
  /**
   * Returns the number of pixels that have been scrolled due to
   * tabbing.
   */
  getScrollAmount: function()
  {
    var scrollAmt = 0;
    if (this.sliderOuterDiv.scrollLeft > 0)  // IE
      scrollAmt = this.sliderOuterDiv.scrollWidth - this.sliderOuterDiv.scrollLeft - this.sliderOuterDiv.offsetWidth;
    else if (this.sliderOuterDiv.scrollLeft < 0)  // FF
      scrollAmt = 0 - this.sliderOuterDiv.scrollLeft;
    return scrollAmt;
  },
  
  /**
   * Invoked when user clicks on right slider button
   */
  slideRight: function()
  {
    if (this.slideDirection == "")
      this.showSecondButton(this.sliderLeft, "marginLeft", -1);
    else 
      this.slideStop();
    this.slideDirection = "R";
    
    // determine number of pixels to slide adjusting for any scrolling
    // from tabbing if necessary
    var slideWidth = dojo.style(this.sliderOuterDiv, "width") - this.tocMarginLeft;    
    var leftToSlide;
    if (this.ltr)
      leftToSlide = this.totalWidthToCItems - slideWidth + this.pos - this.sliderOuterDiv.scrollLeft;
    else
      leftToSlide = this.pos + this.getScrollAmount();

    // don't slide too much
    if (leftToSlide > slideWidth)
      leftToSlide = slideWidth;
    
    // call recursive function to do actual slide
    if (leftToSlide > 0)
      this._slideRight(leftToSlide - this.pos);
  },
  
  _slideRight: function(totalToSlide)
  {    
    if (this.pos == 0) 
      this.pos = -1;
    this.pos -= this.slidePixel;
    if (this.pos < -totalToSlide)
    {
      var diff = -this.pos - totalToSlide;
      this.pos += this.slidePixel;
      if (diff < this.slidePixel) {
        this.pos -= diff;  // any last pixels if not multiple of slidePixel number
        dojo.style(this.toc, "left", this.pos + "px");
      }
      return;
    }
    dojo.style(this.toc, "left", this.pos + "px");
    this.t = setTimeout(dojo.hitch(this, this._slideRight, totalToSlide), this.slideDelay);    
  },
  
  /**
   * Invoked when user clicks on left slider button
   */
  slideLeft: function()
  {    
    if (this.slideDirection == "")
      this.showSecondButton(this.sliderRight, "marginRight", 1);
    else 
      this.slideStop();
    this.slideDirection = "L";
            
    // determine number of pixels to slide adjusting for any scrolling
    // from tabbing
    var slideWidth = dojo.style(this.sliderOuterDiv, "width") - this.tocMarginLeft;    
    var leftToSlide;
    if (this.ltr)
      leftToSlide = -this.pos + this.sliderOuterDiv.scrollLeft;
    else    
      leftToSlide = this.totalWidthToCItems - slideWidth - this.pos - this.getScrollAmount();
    
    // don't slide too much
    if (leftToSlide > slideWidth)
      leftToSlide = slideWidth;
    
    // call recursive function to do actual slide
    if (leftToSlide > 0)
      this._slideLeft(leftToSlide);
  },
  
  _slideLeft: function(totalToSlide)
  {
    if (this.pos == 0) 
      this.pos = 1;
    this.pos += this.slidePixel;
    totalToSlide -= this.slidePixel;
    if (totalToSlide < 0) 
    {
      this.pos -= this.slidePixel;
      if (-totalToSlide < this.slidePixel) {
        this.pos += -totalToSlide;  // any last pixels if not multiple of slidePixel number
        dojo.style(this.toc, "left", this.pos + "px");
      }
      return;

    }
    dojo.style(this.toc, "left", this.pos + "px");
    this.t = setTimeout(dojo.hitch(this, this._slideLeft, totalToSlide), this.slideDelay);    
  },

  /**
   * Cancels the set timer so sliding is stopped
   */
  slideStop: function()
  {
    clearTimeout(this.t);
  },
  
  /**
   * Resets slider so first arrow is visible.
   */
  reset: function()
  {
    this.pos = 0;
    dojo.style(this.toc, "left", "0px");
    if (this.ltr)
      this.sliderOuterDiv.scrollLeft = 0;
  }
});
