/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
dojo.provide("ibm.tivoli.tip.dojo.EventUtil");

dojo.declare("ibm.tivoli.tip.dojo.EventUtil",
	          null,
{
	// Simple method to return whether or not a value has a value
	_hasValue: function(obj){
		return typeof obj != "undefined" && obj != null;
	},

	_doEval: function(scope,str){
		if (typeof scope == "undefined" || scope == null)
			return null;
	    var obj = null;
	    try {
	    	var subParts = str.split(".");
	    	obj=scope;
	    	for (var i=0; i<subParts.length; i++){
	    		obj = obj[subParts[i]];
	    	}
	    	//obj=eval("scope."+str);
	    }
	    catch (e) {
	    	obj=null
	    }
	    return obj;
	},
	// Indicates if the expression has a member
	// Scope is the object being checked, str indicates the member to find.
	_hasMember: function(scope, str){
	    var obj = this._doEval(scope,str);
		return typeof obj != "undefined" && obj != null;
	},

	// Pass in a string and this will return a value associated with it.
	// If the string has an error, the returned value is null.  Very
	// useful for deeply nested references.
	_getValue: function(scope,str){
	    var obj = this._doEval(scope,str);
	    if (typeof obj == "undefined")
	    	obj = null;
		return obj;
	},

	// Only adds the specified attribute to the specified object if
	// the attribute does not already exist
	_safeAdd: function(obj,attr,value){
		if (!this._hasValue(obj[attr]))
			obj[attr]=value;
	},

	// This function takes an event using the new event context and 
	// modifies the event by adding the old context to the event by
	// interrogating the proper fields of the new event.  Note that
	// multiple resources are not supported in the old event context.
	// srcEvent: is the event base, it will be updated upon return
	// payload: optional parameter to point to the payload of the event
    //          if specified, this will be used instead of srcEvent.payload
	addOldContext: function(srcEvent, /*optional*/ payload){
		if (this._hasMember(srcEvent,"payload") || this._hasValue(payload)){
			var productCount=1;
			var increment=false;
			var evtPayload=(this._hasValue(payload)?payload:this._getValue(srcEvent,"payload"));
			
			// Check for case if emitter created a resources array of 1
			if (this._hasMember(evtPayload,"resources.length") && (evtPayload.resources.length==1))
				evtPayload=evtPayload.resources[0];

			var clazztype=this._getValue(evtPayload,"cdm.attributes.ClassType");
			
			// Enumerate through all of the payload
			for (var attr in evtPayload) {
			   if (evtPayload[attr]!=null){
				   if (attr=="product"){
						for (var name in evtPayload.product) {
							increment=false;
							var product = evtPayload.product[name];
							// Get all product targeted attributes
							for (var prodName in product){
								// If the attribute is the cdm attribute, need to enumerate it.
								if (prodName == "cdm"){
									for (var prodCdm in product.cdm){
										if (prodCdm == "ClassType")
											this._safeAdd(srcEvent,"CDMClass"+productCount,product.cdm[prodCdm]);
										else
											this._safeAdd(srcEvent,prodCdm+productCount,product.cdm[prodCdm]);
										increment=true;
									}
									if (clazztype != null)
										this._safeAdd(srcEvent,"CDMClass"+productCount,clazztype);
								} else {
									this._safeAdd(srcEvent,prodName+productCount,product[prodName]);
									increment=true;
								}
							}
							if (increment)
								productCount++;
						}
				   }
				   else if (attr=="cdm" && this._hasMember(evtPayload,"cdm.attributes")){
						for (var attr in evtPayload.cdm.attributes) {
							// Add all attributes with their given name; including ClassType
							this._safeAdd(srcEvent,attr,evtPayload.cdm.attributes[attr]);
						}			
				   }
				   else if (attr != "framework" && attr != "resources"){
					   this._safeAdd(srcEvent,attr,evtPayload[attr]);
				   }
			   }
			}
		}
		return srcEvent;
	},

	_parseProductKey: function(mssname){
		var key = null;
		//var pattern = /\+ProductName=.+$/g;
		//var key = mssname.match(pattern);
		//console.debug(key);
		var pattern = "+ProductName=";
		var i = mssname.indexOf(pattern);
		if (i>0){
			var start = i + pattern.length;
			var end = mssname.indexOf("+",start);
			if (end>0)
				key = mssname.substr(start,end-start);
			else
				key = mssname.substr(start);
		}
		if(key != null)
			key = key.replace(/ /g,"");
			
		return key;
	},

	_addCdmAttribute: function(evtPayload, attribute, value){
		if (!this._hasValue(evtPayload.cdm))
			evtPayload.cdm = {};
		if (!this._hasValue(evtPayload.cdm.attributes))
			evtPayload.cdm.attributes = {};
		evtPayload.cdm.attributes[attribute] = value;
	},

	_addMssMember: function(evtPayload, attrList, mssNumber, productkey, attribute){
		if (this._hasMember(attrList,attribute+mssNumber)){

			// Some special processing for CDMClass
			if (attribute=="CDMClass"){
				if (!this._hasMember(evtPayload,"cdm.attributes.ClassType")){
					this._addCdmAttribute(evtPayload, "ClassType", attrList[attribute+mssNumber]);
				}
				this._safeAdd(evtPayload.product[productkey].cdm,"ClassType",attrList[attribute+mssNumber]);
			} else {
				this._safeAdd(evtPayload.product[productkey].cdm,attribute,attrList[attribute+mssNumber]);
			}
			
			delete attrList[attribute+mssNumber];
		}
	},

	_getMssMembers: function(evtPayload, attrList, mssNumber){
		var mssname=attrList["MSSName"+mssNumber];
		var productkey = this._parseProductKey(mssname);
		if (productkey != null){
			if (!this._hasValue(evtPayload.product))
				evtPayload.product={};
			if (!this._hasValue(evtPayload.product[productkey]))
				evtPayload.product[productkey]={};
			if (!this._hasValue(evtPayload.product[productkey].cdm))
				evtPayload.product[productkey].cdm={};
			
			// This case if there are no numbers after MSSName in the old context
			if (mssNumber == ""){
				// The old format only ever added numbers to MSSName, SourceToken, and CDMClass
				this._addMssMember(evtPayload, attrList, mssNumber, productkey, "MSSName");
				this._addMssMember(evtPayload, attrList, mssNumber, productkey, "SourceToken");
				this._addMssMember(evtPayload, attrList, mssNumber, productkey, "CDMClass");
			} else {
				// If here, there are numbers after MSSName (like MSSName1), we include all
				// attributes ending with the same number in that providertype.
				for (var attr in attrList){
					if (attr.lastIndexOf(mssNumber)==(attr.length-(""+mssNumber).length)){
						var prop = attr.substr(0,attr.length-(""+mssNumber).length);
						
						if (prop=="CDMClass" || prop=="MSSName" || prop=="SourceToken"){
							this._addMssMember(evtPayload, attrList, mssNumber, productkey, prop);
						} else {
							this._safeAdd(evtPayload.product[productkey],prop,attrList[attr]);
						}

						delete attrList[attr];
					}
				}
			}
			
		}
	},

	// The purpose of this method is to attempt to create a "new" context out of
	// an event constructed with old context.
	// srcEvent - the source event
	// payload - optional parameter.  By default this method will look for srcEvent.payload
    //           to pull the context from, but you can specify and alternate location if needed.
	addNewContext: function(srcEvent, /*optional*/ payload){
		// bypass if event already has a payload; therefore, only run this method
		// if a payload object was specified, or there is not already a payload
		if ((this._hasValue(payload) || !this._hasMember(srcEvent,"payload")) && this._hasValue(srcEvent)){  
		    // Gather all of the attributes into an array so we can track what has been processed.
			var attrList={};
			for (var attr in srcEvent) {
				attrList[attr]=srcEvent[attr];
			}
			var evtPayload={};
			if (this._hasValue(payload)){
				evtPayload=payload;
			} else {
				srcEvent.payload = {};
				evtPayload = srcEvent.payload;
			}
			
			// See if ClassType was coded in the old context.  If so, add it; otherwise, we will
			// try to get it from CDMClass later.
			if (this._hasMember(attrList,"ClassType") && !this._hasMember(evtPayload,"cdm.attributes.ClassType")){
				this._addCdmAttribute(evtPayload, "ClassType", attrList["ClassType"]);
				delete attrList["ClassType"];
			}
			
			if (this._hasMember(attrList,"MSSName")){
				// There is just "MSSName", so assume just one MSSName, SourceToken, CDMClass.
				this._getMssMembers(evtPayload,attrList,"");
			} else {
				var mssNumber = 1;
				while (this._hasMember(attrList,"MSSName"+mssNumber))
				{
					this._getMssMembers(evtPayload,attrList,mssNumber);
				    mssNumber++;	
				}
			}
			
			// Now place all the other properties inline
			for (var mem in attrList){
				// do not add name, type, or source as they are part of the event meta data
				if (mem != "name" && mem != "source" && mem != "type"){
					this._safeAdd(evtPayload,mem,attrList[mem]);
				}
			}
		}
     }   
});
