/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.dijit.DefaultMessageHandler");

dojo.require("ibm.tivoli.navmodel.Trace");
dojo.require("ibm.tivoli.navmodel.Exception");
dojo.require("dojoe.messagedialog.MessageDialog");
dojo.requireLocalization("ibm.tivoli.tip.navwidget", "resources");
dojo.requireLocalization("ibm.tivoli.tip.navwidget", "messages");

ibm.tivoli.tip.navwidget.dijit.DefaultMessageHandler._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.dijit.DefaultMessageHandler");

dojo.declare("ibm.tivoli.tip.navwidget.dijit.DefaultMessageHandler",
	      null,
{		
	
	// _trace: [private] ibm.tivoli.navmodel.Trace
	//		Utility for trace and logging
	_trace: ibm.tivoli.tip.navwidget.dijit.DefaultMessageHandler._trace,
	
	//Base constructor
	constructor: function () {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		
		//NLS
		this._resources = dojo.i18n.getLocalization("ibm.tivoli.tip.navwidget", "resources");
		this._messages = dojo.i18n.getLocalization("ibm.tivoli.tip.navwidget", "messages");
	},
	
	displayException: function (/*Object*/ ex, /*Object*/dontTraceMessage) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon

		var msgId = null;
		var msgSeverity = null;
		var msgText = null;

		//Checking for existence of isInstanceOf before calling it because normal JS exception won't have an isInstanceOf
		//Also check that msgId, msgSeverity and msgText exist otherwise popup may not display
		if (ex && ex.isInstanceOf && ex.isInstanceOf(ibm.tivoli.navmodel.Exception) && ex.getMsgId() && ex.getMsgSeverity() && ex.getMsgText()) { 
			msgId = ex.getMsgId();
			msgSeverity = ex.getMsgSeverity();
			msgText = ex.getMsgText();
		} else {
			
			//We have one of the following two messages to choose from. I don't think we're going to have enough in the exception to know if one of these is more appropriate than the
			//other. Also, I don't think we want to do the substitution for ${error_txt} since likely not translated.
			//
			//	- "TCTUI1002E":"An error has occurred.  The data provider could not return the data. ${error_txt}", // ${error_txt} will contain the error message returned from the server
			//	- "TCTUI1003E":"An error has occurred communicating with the server. ${error_txt}"  // ${error_txt} will contain the error message returned from the server
			
			msgId = "TCTUI1003E";
			msgSeverity = "Error";

			var messageTemplate = this._messages.TCTUI1003E;
			msgText = dojo.string.substitute(messageTemplate,
                    {error_txt: ""}); //Doing an empty substitution since message in generic exception probably won't be translated
		}

		//Display the message
		this.displayMessage(msgId, msgSeverity, msgText); 
		
		//Let's trace to the console window too
		if (!dontTraceMessage) {
			if (this._trace.isError()) {
				var methodName = "handleException";
				this._trace.error(ex, methodName);
			}
		}
	},
	
	displayMessage: function(/*Object*/ msgId, /*Object*/ msgSeverity, /*Object*/ msgText) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		
		var msgDlg = new dojoe.messagedialog.MessageDialog(
				{message: msgText,
				 messageId: msgId,
				 type: msgSeverity});
		msgDlg.show();
	}
});