/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.dijit.NavWidgetObjectPluggableView"); 

// includes
dojo.require("ibm.tivoli.navmodel.Trace");
dojo.require("ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView");
dojo.require("dijit._Widget");
ibm.tivoli.tip.navwidget.dijit.NavWidgetObjectPluggableView._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.dijit.NavWidgetObjectPluggableView");
 
dojo.declare("ibm.tivoli.tip.navwidget.dijit.NavWidgetObjectPluggableView", [ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView], {
	// summary:
	//		Base class for an object-specific view that plugs into a NavWidget
	// description:
	//		Coming soon
	
    //**************************************************************************************/
	
	// _items: [private] ibm.tivoli.navmodel.Item[]
	//		The items within the dataset to render
	_items: null,
	
	// _links: [private] ibm.tivoli.navmodel.Link[]
	//		The links within the dataset to render
	_links: null,
	
	// _trace: [private] Trace
	//		Utility for trace and logging
	_trace: ibm.tivoli.tip.navwidget.dijit.NavWidgetObjectPluggableView._trace,
	
	// _setObjectsLoadDataCompletesSynchronously: [protected] Boolean
	//		Allows subclasses to specify whether they load data asynchronously or not
	_setObjectsLoadDataCompletesSynchronously: true,
	
	isApplicableForObjects: function(/*ibm.tivoli.navmodel.Item[]*/ items, /*ibm.tivoli.navmodel.Link[]*/ links, /*ibm.tivoli.navmodel.Link[]*/ relationships){
		// summary:
		//		Analyzes object data and/or metadata to decide if this view makes sense for the given objects.
		// tags:
		//		public

		return true;
	},
	
	_setObjectsActive: false,
	setObjects: function(/*ibm.tivoli.navmodel.Item[]*/ items, /*ibm.tivoli.navmodel.Link[]*/ links, /*ibm.tivoli.navmodel.Link[]*/ relationships){
		// summary:
		//		Kick off the setObjects life-cycle of a widget. Default implementation will call (in order) the
		//      following protected functions:
		//			- _setObjectsInit
		//			- _setObjectsLoadData
		//			- _setObjectsComplete
		//
		//		It is NOT recommended for subclasses to override setObjects, but instead one or more of the
		//		protected methods that get called. In particular, it is expected subclasses would override 
		//		_setObjectsLoadData. Be sure to call super on any of the overridden methods.
		// items:
		//		An array of items
		// links:
		//		An array of links
		// relationships:
		//		An array of relationships
		// description:
		//		Tells this pluggable view it's time to render new objects
		// tags:
		//		public
	
		if (this._setObjectsActive) {
			if (this._trace.isFinest()) {
				this._trace.finest("Previous request to setObject still active, so ignoring; view = " + this.id, "setObjects");
			}
			return;
		}
		
		this._setObjectsInit(items, links, relationships);
		this._setObjectsLoadData(items, links, relationships);
	},
	
	_setObjectsInit: function(/*ibm.tivoli.navmodel.Item[]*/ items, /*ibm.tivoli.navmodel.Link[]*/ links, /*ibm.tivoli.navmodel.Relationship[]*/ relationships){
		// summary:
		//		Called during the setObjects lifecycle for any initialization that should take place in the view before
		//		starting to pull information out of the objects for rendering. Subclasses will probably NOT need to override.
		// items:
		//		An array of items
		// links:
		//		An array of links
		// relationships:
		//		An array of relationships
		// description:
		//		Tells this pluggable view it's time to initialize based on the new objects
		// tags:
		//		protected
		
		//Remember we're in setDataset
		this._setObjectsActive = true;
		
		//Store away the object
		this._items = items;
		this._links = links;
		this._relationships = relationships;
	},
	
	_setObjectsLoadData: function(/*ibm.tivoli.navmodel.Item[]*/ items, /*ibm.tivoli.navmodel.Link[]*/ links,  /*ibm.tivoli.navmodel.Relationship[]*/ relationships){
		// summary:
		//		Called during the setObjects lifecycle to tell the implementor to process the objects to set up
		//		rendering (e.g., start pulling info out of objects, set data models on any children, etc.). The 
		//		base implementation does nothing, so all work delegated to subclass.
		// items:
		//		An array of items
		// links:
		//		An array of links
		// relationships:
		//		An array of relationships
		// description:
		//		Tells the pluggable view it's time to process the new dataset
		// tags:
		//		protected
		
		//Delegate impl to subclass.
	},
	
	_setObjectsLoadDataActive: false,
	_setObjectsLoadData: function(/*ibm.tivoli.navmodel.Item[]*/ items, /*ibm.tivoli.navmodel.Link[]*/ links, /*ibm.tivoli.navmodel.Relationship[]*/ relationships){
		// summary:
		//		Called during the setObjects lifecycle to tell the implementor to process the dataset to set up
		//		rendering (e.g., start pulling info out of dataset, set data models on any children, etc.). The 
		//		base implementation does nothing, so all work delegated to subclass.
		// dataset:
		//		The Client Nav Model Dataset that the pluggable view should render.
		// description:
		//		Tells the pluggable view it's time to process the new objects
		// tags:
		//		protected
		
		this._setObjectsLoadDataActive = true;
		if (this._setObjectsLoadDataCompletesSynchronously) {
			this._onSetObjectsLoadComplete(items, links, relationships);
		}
	},
	
	_onSetObjectsLoadComplete: function(/*ibm.tivoli.navmodel.Item[]*/ items, /*ibm.tivoli.navmodel.Link[]*/ links, /*ibm.tivoli.navmodel.Relationship[]*/ relationships){ 
		// summary:
		//		Event fired when an the load of the dataset is completed (potentially asynchronously). It is recommended to use dojo.connect rather than overriding this method.
		// dataset: Dataset
		//		The client nav model dataset that this view is now rendering
		// tags:
		//		protected callback
		
		this._setObjectsLoadDataActive = false;
		this._setObjectsComplete(items, links, relationships);
	},
	
	_setObjectsComplete: function(/*ibm.tivoli.navmodel.Item[]*/ items, /*ibm.tivoli.navmodel.Link[]*/ links, /*ibm.tivoli.navmodel.Relationship[]*/ relationships){
		// summary:
		//		Called during the setObjects lifecycle as the last step. Subclasses will probably NOT need to override.
		// items:
		//		An array of items
		// links:
		//		An array of links
		// relationships:
		//		An array of relationships
		// description:
		//		Tells the pluggable view we're done setting up the new objects
		// tags:
		//		protected
		
		//Create an "inner class" to help us determine if all of the subviews have called back with onSetObjectsComplete
		var mainOnSetObjectsHandlerInnerClass = {
				view: this,
				callbackCounter: 0,
				
				//Should be invoked in the scope of mainOnSetObjectsHandlerInnerClass, so "this" refers to this inner class rather than the pluggable view
				onSetObjectsHandler: 
					function(items, links, relationships) {
						//Increment counter
						this.callbackCounter++;
						
						if (this.callbackCounter >= this.view.getPluggableViewCount()) {
							//Fire event
							dojo.hitch(this.view, this.view.onSetObjects, items, links, relationships).call();
						}
					}
		};
		var mainDataSetObjectsHandlerFunc = dojo.hitch( mainOnSetObjectsHandlerInnerClass, mainOnSetObjectsHandlerInnerClass.onSetObjectsHandler, items, links, relationships);
		
		//Notify the sub-views of the new objects (if they are object pluggable views)
		var iterator = this.getPluggableViewIterator();
		if (iterator.atEnd()) {
			//There are no pluggable views, so we let's complete the setDataset process
			mainDataSetObjectsHandlerFunc.call();
		} else {
			//We want to iterate through the pluggable views and listen for when they hit onSetDatasetComplete
			while (!iterator.atEnd()) {
				var view = iterator.get().value;

					//Provide an "inner class" to support a handler for the "onSetDataset" event that can remove itself as a listener when done
					var subHandlerInnerClass = {
						mainHandler: mainDataSetObjectsHandlerFunc,
						handle: null,
						
						//Should be invoked in the scope of subHandlerInnerClass, so "this" refers to this inner class
						handler: function(items, links, relationships){
								this.mainHandler.call();
								
							     //Disconnect the handler
							     dojo.disconnect(this.handle);
							}
					};
					var handle = dojo.connect(view, "onSetObjects", dojo.hitch(subHandlerInnerClass, subHandlerInnerClass.handler));
					subHandlerInnerClass.handle = handle;
						
					if (view.isInstanceOf(ibm.tivoli.tip.navwidget.dijit.NavWidgetObjectPluggableView) &&
							view.isApplicableForObjects(items, links, relationships)) {
						try {
							//Since views are pluggable... let's take some care so that a custom one doesn't take us out if it craps
							//out handling the new objects
							view.setObjects(items, links, relationships);
						} catch (ex) {
							if (this._trace.isError()) {
								this._trace.error("Cannot set objects on view = " + view.id + "; ex = " + ex.name + ": " + ex.message, "_setObjectsComplete");
							}
						}
					} else {
						//View not applicable, so let's clear out the view
						try {
							view.setObjects(null, null, null, null);
						} catch (ex) {
							if (this._trace.isError()) {
								this._trace.error("Unable to clear objects on view = " + view.id + "; ex = " + ex.name + ": " + ex.message, "_setObjectsComplete");
							}
						}
					}
			}
		}
	},

	_setDatasetLoadData: function(/*ibm.tivoli.navmodel.Dataset*/ dataset){
		// summary:
		//		Overriden to clear out any objects that may have previously been set with setObjects.
		// dataset:
		//		The Client Nav Model Dataset that the pluggable view should render.
		// description:
		//		Tells the pluggable view we're done setting up the dataset
		// tags:
		//		protected
		
		this.clearObjects();
		this.inherited(arguments)
	},
	
	clearObjects: function(){
		// summary:
		//		Overriden to clear out any objects that may have previously been set with setObjects.
		// dataset:
		//		The Client Nav Model Dataset that the pluggable view should render.
		// description:
		//		Tells the pluggable view we're done setting up the dataset
		// tags:
		//		protected
		
		this._items = null;
		this._links = null;
		this._relationships = null;

		//Clear out the views too
		var iterator = this.getPluggableViewIterator();
		while (!iterator.atEnd()) {
			var view = iterator.get().value;
			view.clearObjects();
		}
	},
	
	getItemCount: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
 		if (this._items) {
 			return this._items.length;
 		}
 	},
 	
 	getLinkCount: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
 		if (this._links) {
 			return this._links.length;
 		}
 	},
 	
 	getRelationshipCount: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
 		if (this._relationships) {
 			return this._relationships.length;
 		}
 	},
 	
 	
 	isMixedObjects: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
 		var objectTypeCount = 0;
 		
 		if (this.getItemCount() > 0) {
 			//We have items
 			objectTypeCount++;
 		}
 		if (this.getLinkCount() > 0) {
 			//We have items
 			objectTypeCount++;
 		}
 		if (this.getRelationshipCount() > 0) {
 			//We have items
 			objectTypeCount++;
 		}
 		return (objectTypeCount > 1);
 	},
 	
 	//*****************************************************************************************************
	// Events we fire
	//*****************************************************************************************************
	onSetObjects: function(/*ibm.tivoli.navmodel.Item[]*/ items, /*ibm.tivoli.navmodel.Link[]*/ links, /*ibm.tivoli.navmodel.Link[]*/ relationships){
		// summary:
		//		Event fired when the objects are set
		// dataset: Dataset
		//		The client nav model objects that this view is now rendering
		// tags:
		//		callback
		
		//Mark that we're done setting the objects
		this._setObjectsActive = false;
		
		//Delegate anything else to listeners
	}
});