/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView"); 

// includes
dojo.require("ibm.tivoli.tip.navwidget.dijit.DefaultMessageHandler");
dojo.require("ibm.tivoli.navmodel.Trace");
dojo.require("dijit._Widget");
dojo.require("dojox.collections.SortedList");
dojo.require("dojox.collections.ArrayList");
dojo.require("ibm.tivoli.tip.dijit.BusyPane");
dojo.require("ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil");
dojo.require("ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewSelection");
dojo.require("ibm.tivoli.tip.navwidget.dojo.NavWidgetBaseViewState");
ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView");
 
dojo.requireLocalization("ibm.tivoli.tip.navwidget", "resources");


dojo.declare("ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView", 
	[dijit._Widget], {
	// summary:
	//		Base class for a view that plugged into a NavWidget
	// description:
	//		Coming soon
	
    //**************************************************************************************/
	
	// viewId: [const] String
	//		Data model
	viewId: null,
	
	// viewTypeSpec: [const] String
	//		Coming soon
	viewTypeSpec: null,
	
	// dataset: [const] ibm.tivoli.navmodel.Dataset
	//		Data model
	dataset: null,
	
	// initializeWithNullDataset: [const] ibm.tivoli.navmodel.Dataset
	//		Determines whether setDataset(null) will be called during initialization. Generally, this can be left to true.
	//
	//		The typical scenario is to pass in null on the constructor, and then when the Dataset to be rendered finishes it's
	//		asynchronous initialization manually calling setDataset with the initialized dataset. In those cases, it is 
	//		often desirable to allow the UI to start rendering while the dataset initializes. 
	//
	//		This works pretty well if you expect the widgetDefinition passed in to the constructor to remain active 
	//		when the initialized dataset is later passed in via setDataset. However, in cases where you expect the
	//		dataset to specify a custom widget definition to override the default, it is less desirable because the
	//		UI will be set-up using the default def and then entirely rebuilt whenever the intialized dataset is passed in
	initializeWithNullDataset: true,
	
	// parentPluggableView: [const] ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView
	//		The parent pluggable view. If null, this instance is a stand-alone view not part of a larger view. This
	//		should only be set when this view is a true child of the parent (e.g., has been added to the parent
	//		with addPluggableView). This will typically be the case, but occasionally there are more "transient"
	//		pluggable views (like dialogs and tooltips) that aren't truly child views.
	parentPluggableView: null,
	
	// managingPluggableView: [const] ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView
	//		The managing pluggable view. This is intended for use in the special cases of more "transient" pluggable 
	//		views that aren't truly children (such as dialogs and tooltips), but might have a need to for some
	//		support from a higher-level view to help manage busy pane's, etc.
	managingPluggableView: null,
	
	// widgetDefinitionId: [const] String
	//		The id for the current widget definition used in this view
	widgetDefinitionId: "",		
	
	// widgetDefinitionId: [const] String
	//		The current widget definition used in this view
	widgetDefinition : null,
	
	// labelIconClass: [const] String
	//		An appropriate iconClass for use next to a label in a menu for switching views, etc.
	labelIconClass : null,
	
	// _originalWidgetDefinition: [private] String
	//		The first widget definition used in this view
	_originalWidgetDefinition : null,

	// cmProvider: [const] TIPContextMenuProvider
	//		Custom context menu provider (must be provided on construction to be used)
	cmProvider: null,
	
	//daHelper: [const] DefaultActionHandler
	//      Custom overwritten DefaultActionHandler (must be provided on construction to be used)
	daHelper: null,
	
	//defaultMessageHandler: [const] defaultMessageHandler
	//      Custom overwritten DefaultActionHandler (must be provided on construction to be used)
	defaultMessageHandler: null,
	
	// destroyDatasetWhenDone: [const] Boolean
	//		Determine whether to destroy the dataset when we clean ourselves up (but only if no parent view and no one else listening to dataset)
	destroyDatasetWhenDone: true,
	
	// _pluggableViews: [private] Dictionary
	//		Data structure of pluggable views
    _childPluggableViews: null,
    
    // selection: [const] ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewSelection
	//		Manages the selection of the view
	selection: null,
	
	// _setDatasetLoadDataCompletesSynchronously: [protected] Boolean
	//		Allows subclasses to specify whether they load data asynchronously or not
	_setDatasetLoadDataCompletesSynchronously: true,
	
	// _usesWidgetDefinition: [protected] Boolean
	//		Allows subclasses to specify whether they use a widget definition or not
	_usesWidgetDefinition: true,
	
	// _busyPane: [private] ibm.tivoli.tip.dijit.BusyPane
	//      Reference to the BusyPane for this view.  Note, only standalone views
	//      or the topmost parent pluggable view have a BusyPane to ensure only
	//      one per NavWidget
	_busyPane: null,
	
	// _busyPaneCount: [private] Number (integer)
	//      A counter that represents the number of times showBusyPane() has
	//      been called before hideBusyPane() is called.  Calling hideBusyPane()
	//      decrements the count and will only truly hide the busy pane when the
	//      count returns to zero.  This allows multiple child pluggable views to
	//      request the busy pane, but it will only be displayed on the first
	//      request and only hidden once the last request cycle calls hideBusyPane()
	_busyPaneCount: 0,

	// commonResources: [protected] dojo.i18n
	//		Shared resource bundle

	commonResources: null,
	
	// _isDestroyed: [private] Boolean
	//  	Flag as to whether we've been destroyed or not
	_isDestroyed: false,
	
	// _trace: [private] Trace
	//		An instance of Trace customized for this class
	_trace: ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView._trace, 

	//***********************************************************************************************/
	
	constructor: function () {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon

        	this.commonResources = dojo.i18n.getLocalization("ibm.tivoli.tip.navwidget", "resources");	

		//The sorted list can suffer from performance issues if there are a lot of insertions. But, the number of views managed should typically be less
		//than a handful.
		this._childPluggableViews = new dojox.collections.SortedList();
	},
	
	postMixInProperties: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		
		//Set up the widgetDef
		this._createWidgetDefinition();
		
		this.inherited(arguments);
	},

	buildRendering: function(){				
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		
		//Create the selection manager
		this.selection = this._createSelection();
		
		//Call superclass
		this.inherited(arguments);
	},
	
	postCreate: function(){
		// summary:
		//		Override's _Widgets postCreate to call setDataset if a dataset was set in the initial 
		//		parms.
		// dataset:
		//		The Client Nav Model Dataset that the pluggable view should render.
		// description:
		//		Coming soon
		// tags:
		//		protected
		
		//Create menu provider
		this._createCMProvider();
		
		//Create supporting widgets (like toolbars, etc.)
		this._createSupportingWidgets();
		
		//Create the busy pane
		if (!this.parentPluggableView && !this.managingPluggableView) {
			this._createBusyPane();
		}

		//Initialize ourselves with the dataset. 
		this._initializeWithDataset();
	
		//Call superclass
		this.inherited(arguments);
	},
	
	startup: function(){
		this.inherited(arguments);
	},
	
	_createSelection: function(){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// tags:
		//		protected
		
		var kwArgs = {
			pluggableView: this
		};
		return new ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewSelection(kwArgs); 
	},
	
	_createWidgetDefinition: function(){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// tags:
		//		protected

		if (this.widgetDefinition) {
			//If a widget def instance was passed in, let's use that
			this.widgetDefinitionId = this.widgetDefinition.getWidgetId();
			
		} else if (this._usesWidgetDefinition) {
			var defaultWidgetDefInfo = this._getDefaultWidgetDefInfo();
			if (defaultWidgetDefInfo) {
				if(!this.widgetDefinitionId){
					this.widgetDefinitionId = defaultWidgetDefInfo.definition;
				}
				
				this.widgetDefinition 
					= ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.createWidgetDefinition(defaultWidgetDefInfo.className, 
							this.widgetDefinitionId, defaultWidgetDefInfo, this.dataset, this.getParentPluggableViewWidgetDefinition());
			} else {
				if (this._trace.isError()) {
					this._trace.error("Missing default widget def info for view = " + this.id, "_createWidgetDefinition");
				}
			}
		}
		
		//Let's store away in case we need to revert later
		this._originalWidgetDefinition = this.widgetDefinition;
	},
	
	_getDefaultWidgetDefInfo: function(){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// tags:
		//		protected
	
		return null;
	},
	
	_createCMProvider: function(){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// tags:
		//		protected
		
		
	},
	
	_createSupportingWidgets: function(){
		// summary:
		//		Give subclass a chance to create some widgets (like toolbars, etc.) during the setup of the view.
		// description:
		//		Give subclass a chance to create some widgets (like toolbars, etc.). This is called in postCreate BEFORE the view 
		//		has been initialized with the Dataset, so the widgets created here would typically not be dependent on info in the
		//		dataset.
		// tags:
		//		protected
		
		
	},
	
	_createBusyPane: function() {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// tags:
		//		protected
		
        this._busyPane = new ibm.tivoli.tip.dijit.BusyPane({id: this.id + "_busyPane"});
        dojo.style(this._busyPane.domNode, "zIndex", "10");
                
		//Bobby Joseph, 16Nov2011 - BEGIN
		//254806 Dojo1.6: nav...WithWidgetDefProcessing doesnot work with dojo16
		//http://bugs.dojotoolkit.org/ticket/12795
		//Need to set region for all BorderContainer children from dojo 1.6
		this._busyPane.region = "center";
		//Bobby Joseph, 16Nov2011 - END
		
		this.domNode.appendChild(this._busyPane.domNode);
		this._busyPane.startup();
	},
	
	_initializeWithDataset: function(){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// tags:
		//		protected
		
		//Initialize ourselves with the dataset. If the parentPluggableView has been specified, we can skip
		//because setDataset will be called when parentPluggableView initializes its views.
		if (!this.parentPluggableView) {
			if (this.dataset || this.initializeWithNullDataset) {
				this.setDataset(this.dataset);
			}
		}
	},
		
	forceHideBusyPane: function() {
		// summary:
		//		Hides the busy pane if it is currently displayed regardless of the count
		// description:
		//		If this view has the busy pane, it forces the count to 1 and then calls
		//      hideBusyPane() to hide the busy pane.
       //      If this view does not have the busy pane, the request is passed on to the parent 
       //      pluggable view or managing pluggable view.
		// tags:
		//		public
		if (this._busyPane) {
			this._busyPaneCount = 1;
			this.hideBusyPane();
		}		
		else {
			// Send to parent view, if one exists
			if (this.parentPluggableView) {
				this.parentPluggableView.forceHideBusyPane();
			// Otherwise, send to managingPluggableView if one exists
			} else if (this.managingPluggableView) {
				this.managingPluggableView.forceHideBusyPane();
			}
		}
	},
	
	hideBusyPane: function() {
		// summary:
		//		Hides the busy pane if it is currently displayed and the decremented count is zero
		// description:
		//		If this view has the busy pane, it decrements the count and it checks if the busy 
		//      pane's current state is displayed, and if the count is back to zero, hides it.  
		//      If this view does not have the busy pane, the request is passed on to the parent 
		//      pluggable view or managing pluggable view.
		// tags:
		//		public
		
		// Does this view have the busy pane?
		if (this._busyPane) {
			// decrement the count, but not past zero
			if (this._busyPaneCount > 0) {
				this._busyPaneCount--;
			}
			// Is the busy pane currently visible? (ibm.tivoli.tip.dijit.BusyPane uses
			// display = "none" when it is hidden)
			if (dojo.style(this._busyPane.domNode, "display") != "none") {
	            if (this._busyPaneCount == 0) {
	            	this._busyPane.hide();
	            }
			}
		}
		else {
			// Send to parent view, if one exists
			if (this.parentPluggableView) {
				this.parentPluggableView.hideBusyPane();
			// Otherwise, send to managingPluggableView if one exists
			} else if (this.managingPluggableView) {
				this.managingPluggableView.hideBusyPane();
			}
		}
	},

	showBusyPane: function() {
		// summary:
		//		Shows the busy pane if it is not currently displayed
		// description:
		//		If this view has the busy pane, it increments the count and checks if the 
		//      busy pane's current state is hidden, and if so, shows it.  If this view does 
		//      not have the busy pane, the request is passed on to the parent pluggable view
      //      or managing pluggable view
		// tags:
		//		public
		
		// Does this view have the busy pane?
		if (this._busyPane) {
			this._busyPaneCount++;
			// Is the busy pane currently hidden? (ibm.tivoli.tip.dijit.BusyPane uses
			// display = "none" when it is hidden)
			if (dojo.style(this._busyPane.domNode, "display") == "none") {
				this._busyPane.show(true);
			}
		}
		else {
			// Send to the parent view, if one exists
			if (this.parentPluggableView) {
				this.parentPluggableView.showBusyPane();
			}
			// Otherwise, send to the managing pluggable view, if one exists
			else if (this.managingPluggableView) {
				this.managingPluggableView.showBusyPane();
			}
		}
	},
	
	displayException: function (/*Object*/ ex, /*Object*/dontTraceMessage) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon

		// Send to the parent view, if one exists
		if (this.parentPluggableView) {
			this.parentPluggableView.displayException(ex, dontTraceMessage);
		}
		// Otherwise, send to the managing pluggable view, if one exists
		else if (this.managingPluggableView) {
			this.managingPluggableView.displayException(ex, dontTraceMessage);
		} else {
			//No where else to turn, so we need to deal with it
			if (!this.defaultMessageHandler) {
				//Create our own handler if necessary
				this.defaultMessageHandler = new ibm.tivoli.tip.navwidget.dijit.DefaultMessageHandler({});
			}
			this.defaultMessageHandler.displayException(ex, dontTraceMessage);
		}
	},
	
	displayMessage: function(/*Object*/ msgId, /*Object*/ msgSeverity, /*Object*/ msgText) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		
		// Send to the parent view, if one exists
		if (this.parentPluggableView) {
			this.parentPluggableView.displayMessage(msgId, msgSeverity, msgText);
		}
		// Otherwise, send to the managing pluggable view, if one exists
		else if (this.managingPluggableView) {
			this.managingPluggableView.displayMessage(msgId, msgSeverity, msgText);
		} else {
			//No where else to turn, so we need to deal with it
			if (!this.defaultMessageHandler) {
				//Create our own handler if necessary
				this.defaultMessageHandler = new ibm.tivoli.tip.navwidget.dijit.DefaultMessageHandler({});
			}
			this.defaultMessageHandler.displayMessage(msgId, msgSeverity, msgText);
		}
	},
	
	isApplicable: function(/*ibm.tivoli.navmodel.Dataset*/dataset){
		// summary:
		//		Analyzes dataset data and/or metadata to decide if this view makes sense for the given dataset.
		// dataset:
		//		The Client Nav Model Dataset that the pluggable view should render.
		// description:
		//		Analyzes dataset data and/or metadata to decide if this view makes for the given dataset. For
		//		example, if this view represents a topology, it might not make sense for it to render a dataset
		//		without relationships. Default implementation checks the dataset's viewType against the explicitly
		//		excluded viewType's.
		// tags:
		//		public
	
		var applicable = true;
		
		//Let's see if the datseet viewType matches one of the view primaryViewType's
		if (dataset) {
			var datasetViewType = dataset.getViewType();
			var excludedViewTypes = this.getExcludedViewTypes();
			for (var i = 0; i < excludedViewTypes.length; i++) {
				var exViewType = excludedViewTypes[i];
				if (exViewType == datasetViewType) {
					applicable = false;
					break;
				}
			}
		} else {
			applicable = false;
		}
		
		return applicable;
	},
	
	getPrimaryViewTypes: function(){
		// summary:
		//		Coming soon
		// dataset:
		//		Coming soon
		// description:
		//		Coming soon
		// tags:
		//		public
	
		var primaryViewTypes = [];
		if (this.viewTypeSpec) {
			//Split apart the spec string
			var breakdown = this.viewTypeSpec.split(",");
			
			//Examine the individual viewTypes
			for (var i = 0; i < breakdown.length; i++) {
				var viewType = breakdown[i];
				var firstChar = viewType.substring(0, 1);
				if (firstChar != "!") { //If starts with "!" it would indicate an excluded viewType, so skip
					primaryViewTypes.push(viewType);
				}
			}
		}
		return primaryViewTypes;
	},
	
	getExcludedViewTypes: function(){
		// summary:
		//		Coming soon
		// dataset:
		//		Coming soon
		// description:
		//		Coming soon
		// tags:
		//		public
	
		var excludedViewTypes = [];
		if (this.viewTypeSpec) {
			//Split apart the spec string
			var breakdown = this.viewTypeSpec.split(",");
			
			//Examine the individual viewTypes
			for (var i = 0; i < breakdown.length; i++) {
				var viewType = breakdown[i];
				var firstChar = viewType.substring(0, 1);
				if (firstChar == "!") { //If starts with "!" we've found an excluded viewType
					excludedViewTypes.push(viewType.substring(1));
				}
			}
		}
		return excludedViewTypes;
	},

	getPreferredPluggableView: function(/*ibm.tivoli.navmodel.Dataset*/dataset){
		// summary:
		//		Coming soon
		// dataset:
		//		Coming soon
		// description:
		//		Comming soon
		// tags:
		//		public
	
		var datasetViewType = dataset;
		var preferredView = null;
		var iterator = this.getPluggableViewIterator();
		while (!iterator.atEnd()) {
			var view = iterator.get().value;
			if (view.isApplicable(dataset)) {
				//Let's see if the dataset viewType matches one of the view primaryViewType's
				var datasetViewType = dataset.getViewType();
				var matchesPrimaryViewType = false;
				var primaryViewTypes = view.getPrimaryViewTypes();
				for (var i = 0; i < primaryViewTypes.length; i++) {
					var primViewType = primaryViewTypes[i];
					if (primViewType == datasetViewType) {
						matchesPrimaryViewType = true;
						break;
					}
				}
				
				if (matchesPrimaryViewType) {
					//We've found an applicable view matching a primary view type, so we can quit looking
					preferredView = view;
					break;
				} else {
					//The view is applicable, but primary view type isn't a match... so if we haven't found another
					//applicable view along the way, let's save as best match so far
					if (!preferredView) {
						preferredView = view;
					}
				}
			}
		}
		return preferredView;
	},
	
	getLabel: function(){
		// summary:
		//		Provides an appropriate label appropriate for use in a menu for switching views, etc.
		// description:
		//		Provides an appropriate label appropriate for use in a menu for switching views, etc.
		// tags:
		//		public
	
		return this.title;
	},
	
	getLabelIconClass: function(){
		// summary:
		//		Provides an appropriate iconClass for use next to a label in a menu for switching views, etc.
		// description:
		//		Provides an appropriate iconClass for use next to a label in a menu for switching views, etc.
		// tags:
		//		public
	
		return this.labelIconClass;
	},
	
	saveWidgetDelta: function(){
		// summary:
		//		Save any changes to the user delta of the current widget def's for the view.
		// description:
		//		Save any changes to the user delta of the current widget def's for the view.
		// tags:
		//		public
		//
		// sub-class would want to save any information to the delta and then call base class method
		
		if (this.widgetDefinition && this.widgetDefinition.hasPendingChanges()) {
			this.widgetDefinition.saveWidgetDelta();
		}
	},

	clearWidgetDelta: function(){
		// summary:
		//		Clear information from user delta of the current widget def's for the view.
		// description:
		//		Clear information from user delta of the current widget def's for the view.
		// tags:
		//		public
		//		
		// sub-class would want to remove any information from the delta and then call base class method

		if (this.widgetDefinition) {
			this.widgetDefinition.saveWidgetDelta();
		}
	},
	
	hasWidgetDeltaPendingChanges: function(){
		// summary:
		//		Indicates whether the user delta has any updates to be saved.
		// description:
		//		Indicates whether the user delta has any updates to be saved. Intended to be 
		//		used as a convenience for a toolbar or menu providers (e.g., help to control enablement
		//		of "Save" action)
		// tags:
		//		public
		
		return (this.widgetDefinition && this.widgetDefinition.hasPendingChanges()); 
	},

	_setDatasetActive: false,
	setDataset: function(/*ibm.tivoli.navmodel.Dataset*/dataset, /*ibm.tivoli.navmodel.AbstractWidgetDef?*/datasetWidgetDefinition, /*ibm.tivoli.tip.navwidget.dojo.NavWidgetBaseViewState?*/state){
		// summary:
		//		Kick off the setDataset life-cycle of a widget. Default implementation will call (in order) the
		//      following protected functions:
		//			- _setDatasetCleanUpAfterLastDataset
		//			- _setDatasetInit
		//			- _setDatasetSetWidgetDefs
		//			- _setDatasetCreatePluggableViews();
		//			- _setDatasetProcessWidgetDefs
		//			- _setDatasetLoadData
		//			- _setDatasetComplete
		//
		//		It is NOT recommended for subclasses to override setDataset, but instead one or more of the
		//		protected methods that get called. In particular, it is expected subclasses would override 
		//		_setDatasetLoadData. Be sure to call super on any of the overridden methods.
		// dataset:
		//		The Client Nav Model Dataset that the pluggable view should render.
		// datasetWidgetDefinition:
		//		Optional widget definition to use while visualizing this particular dataset
		// description:
		//		Tells this pluggable view it's time to render a new dataset
		// tags:
		//		public
	
		this._setDatasetCleanUpAfterLastDataset();
		this._setDatasetInit(dataset);
		this._setDatasetSetWidgetDefs(dataset, datasetWidgetDefinition, state);
		this._setDatasetCreatePluggableViews();
		this._setDatasetProcessWidgetDefs(dataset);
		this._setDatasetLoadData(dataset);
	},
	
	_setDatasetCleanUpAfterLastDataset: function(){
		// summary:
		//		Called during the setDataset lifecycle to allow clean-up related to previous dataset to occur. The base class implementation
		//		takes care of deregistering listening for event changes. Subclasses will probably NOT need to override.
		// description:
		//		Tells the pluggable view it's time to clean up before the new dataset is used.
		// tags:
		//		protected

		//Stop listening for changes to the old dataset
		this._unregisterDatasetEventListener(this.dataset);
	},
	
	_setDatasetInit: function(/*ibm.tivoli.navmodel.Dataset*/ dataset){
		// summary:
		//		Called during the setDataset lifecycle for any initialization that should take place in the view before
		//		starting to pull information out of the dataset for rendering. Subclasses will probably NOT need to override.
		// dataset:
		//		The Client Nav Model Dataset that the pluggable view should render.
		// description:
		//		Tells the pluggable view it's time do initialization for the new dataset
		// tags:
		//		protected
		
		//Remember we're in setDataset
		this._setDatasetActive = true;
		
		//Store away the dataset
		this.dataset = dataset;
		
		//Start listening for changes to the new Dataset. If the parentPluggableView has been specified, we don't
		//need to explicitly listen as the parentPluggableView will pass changes along to us.
		if (!this.parentPluggableView && this.dataset) {
			this._registerDatasetEventListener();
		}
		
		//Let selection know there's a new dataset; also clears the selection
		this.selection.setDataset(this.dataset);
	},
	
	_setDatasetNewWidgetDef: false,
	_setDatasetState: null,
	_setDatasetSetWidgetDefs: function(/*ibm.tivoli.navmodel.Dataset*/ dataset, /*ibm.tivoli.navmodel.AbstractWidgetDef?*/datasetWidgetDefinition, /*ibm.tivoli.tip.navwidget.dojo.NavWidgetBaseViewState?*/state){
		// summary:
		//		Called during the setDataset lifecycle for setting widget defs specified in the dataset to the view
		//		Subclasses will probably NOT need to override.
		// dataset:
		//		The Client Nav Model Dataset that the pluggable view should render.
		// datasetWidgetDefinition:
		//		Optional widget definition to use while visualizing this particular dataset
		// description:
		//		Sets widget defs specified in the dataset to the view
		// tags:
		//		protected
		
		//Remember the current def id
		var oldDefId = this.widgetDefinitionId;
		var oldDefIdContainsVersion = false;
		if (oldDefId && oldDefId.indexOf("-") > 0) {
			oldDefIdContainsVersion = true;
		}
 
		if (datasetWidgetDefinition) {
			//We have a widget def explicitly set to go with this dataset, so respect above everything else
			this.widgetDefinitionId = datasetWidgetDefinition.getWidgetId() + (oldDefIdContainsVersion ? ("-" + datasetWidgetDefinition.getWidgetVersion()) : "");
			this.widgetDefinition = datasetWidgetDefinition;
		} else if (dataset && this._usesWidgetDefinition) {
			//See if the dataset specifies it's own widget def
			var defaultWidgetDefInfo = this._getDefaultWidgetDefInfo();
			if (defaultWidgetDefInfo) {
				var kwArgs = {
						widgetType: defaultWidgetDefInfo.widget
					};
				var datasetWidgetDef = dataset.getWidgetDef(kwArgs);
				if(datasetWidgetDef){
					this.widgetDefinitionId = datasetWidgetDef.getWidgetId() + (oldDefIdContainsVersion ? ("-" + datasetWidgetDef.getWidgetVersion()) : "");
					this.widgetDefinition = datasetWidgetDef;
				} else {
					// We didn't get a widget def from the dataset, let's revert to the original widget deft
					if (this._originalWidgetDefinition) {
						this.widgetDefinitionId = this._originalWidgetDefinition.getWidgetId() + (oldDefIdContainsVersion ? ("-" + this._originalWidgetDefinition.getWidgetVersion()) : "");
						this.widgetDefinition = this._originalWidgetDefinition;
					}
				}
			}
		}
		
		//Let's store the state away as well in case it has somethng in it that should override the widget def
		this._setDatasetState = state;
		
		//Let's see if the widget def is different than before
		this._setDatasetNewWidgetDef = (this.widgetDefinitionId != oldDefId);
	},
	
	_setDatasetProcessWidgetDefs: function(/*ibm.tivoli.navmodel.Dataset*/ dataset){
		// summary:
		//		Called during the setDataset lifecycle for processing widet definitions specified in the dataset.
		// dataset:
		//		The Client Nav Model Dataset that the pluggable view should render.
		// description:
		//		Processes widget definition specified in the datasets. This widget definition is already apllied to the view.
		// tags:
		//		protected
		//
		
		//Delegate impl to subclass.
	},
	
	_setDatasetLoadDataActive: false,
	_setDatasetLoadData: function(/*ibm.tivoli.navmodel.Dataset*/ dataset){
		// summary:
		//		Called during the setDataset lifecycle to tell the implementor to process the dataset to set up
		//		rendering (e.g., start pulling info out of dataset, set data models on any children, etc.). The 
		//		base implementation does nothing, so all work delegated to subclass.
		// dataset:
		//		The Client Nav Model Dataset that the pluggable view should render.
		// description:
		//		Tells the pluggable view it's time to process the new dataset
		// tags:
		//		protected
		
		this._setDatasetLoadDataActive = true;
		if (this._setDatasetLoadDataCompletesSynchronously) {
			this._onSetDatasetLoadComplete(dataset);
		}
	},
	
	_onSetDatasetLoadComplete: function(/*ibm.tivoli.navmodel.Dataset*/ dataset){ 
		// summary:
		//		Event fired when an the load of the dataset is completed (potentially asynchronously). It is recommended to use dojo.connect rather than overriding this method.
		// dataset: Dataset
		//		The client nav model dataset that this view is now rendering
		// tags:
		//		protected callback
		
		this._setDatasetLoadDataActive = false;
		this._setDatasetComplete(dataset);
	},
	
	isSetDatasetActive: function() {
		return this._setDatasetActive;
	},
	
	isSetDatasetNewWidgetDef: function() {
		return this._setDatasetNewWidgetDef;
	},
	
	getSetDatasetState: function() {
		return this._setDatasetState;
	},
	
	isSetDatasetLoadDataActive: function() {
		return this._setDatasetLoadDataActive;
	},
	
	_setDatasetComplete: function(/*ibm.tivoli.navmodel.Dataset*/dataset){
		// summary:
		//		Called during the setDataset lifecycle as the last step. Base implementation will make sure
		//		this view is setup to receive data change events from the Dataset. Subclasses will probably NOT need to override.
		// dataset:
		//		The Client Nav Model Dataset that the pluggable view should render.
		// description:
		//		Tells the pluggable view we're done setting up the dataset
		// tags:
		//		protected
		
		//Create an "inner class" to help us determine if all of the subviews have called back with onSetDatasetComplete
		var mainOnSetDatasetHandlerInnerClass = {
				view: this,
				callbackCounter: 0,
				
				//Should be invoked in the scope of mainOnSetDatasetHandlerInnerClass, so "this" refers to this inner class rather than the pluggable view
				onSetDatasetHandler: 
					function(dataset) {
						//Increment counter
						this.callbackCounter++;
						
						if (this.callbackCounter >= this.view.getPluggableViewCount()) {
							//Fire event
							dojo.hitch(this.view, this.view.onSetDataset, dataset).call();
						}
					}
		};
		var mainDataSetHandlerFunc = dojo.hitch(mainOnSetDatasetHandlerInnerClass, mainOnSetDatasetHandlerInnerClass.onSetDatasetHandler, dataset);
		
		//Notify the sub-views of the new dataset
		var iterator = this.getPluggableViewIterator();
		if (iterator.atEnd()) {
			//There are no pluggable views, so we let's complete the setDataset process
			mainDataSetHandlerFunc.call();
		} else {
			//We want to iterate through the pluggable views and listen for when they hit onSetDatasetComplete
			while (!iterator.atEnd()) {
				var view = iterator.get().value;

					//Provide an "inner class" to support a handler for the "onSetDataset" event that can remove itself as a listener when done
					var subHandlerInnerClass = {
						mainHandler: mainDataSetHandlerFunc,
						handle: null,
						
						//Should be invoked in the scope of subHandlerInnerClass, so "this" refers to this inner class
						handler: function(dataset){
								this.mainHandler.call();
								
							     //Disconnect the handler
							     dojo.disconnect(this.handle);
							}
					};
					var handle = dojo.connect(view, "onSetDataset", dojo.hitch(subHandlerInnerClass, subHandlerInnerClass.handler));
					subHandlerInnerClass.handle = handle;
						
					if (view.isApplicable(dataset)) {
						try {
							//Since views are pluggable... let's take some care with try/catch, so that a custom one doesn't take us out if it craps
							//out while handling the dataset
							var state = this.getSetDatasetState();
							var viewState = (state) ? state.getPluggableViewState(view.viewId) : null;
							view.setDataset(dataset, null, viewState);
						} catch (ex) {
							if (this._trace.isError()) {
								this._trace.error("Cannot set dataset on view = " + view.id + "; ex = " + ex.name + ": " + ex.message, "_setDatasetComplete");
							}
						}
					} else {
						//View not applicable, so let's clear out the view
						try {
							view.setDataset(null);
						} catch (ex) {
							if (this._trace.isError()) {
								this._trace.error("Unable to clear dataset on view = " + view.id + "; ex = " + ex.name + ": " + ex.message, "_setDatasetComplete");
							}
						}
					}
			}
		}
	},
	
	destroyRecursive: function(/*Boolean?*/ preserveDom){		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// preserveDom:
		//		
		
		//Overriding simply because on more than one occasion we've had trouble tracking down why this.destroy isn't being called on this
		//widget. The superclass's implementation is to do the following:
		//
		//			this._beingDestroyed = true;
		//			this.destroyDescendants(preserveDom);
		//			this.destroy(preserveDom); 
		//
		//If this.destroyDescendants craps out, then this.destroy doesn't get reached.
		
		var methodName = "destroyRecursive";
		
		this._beingDestroyed = true;
		try {
			this.destroyDescendants(preserveDom);
		} catch (ex) {
			if (this._trace.isError()) {
				this._trace.error("problem destroyingDescendants of " + this.id + "; ex = " + ex.name + ": " + ex.message, methodName);
			}
		}
		
		try {
			this.destroy(preserveDom);
		} catch (ex) {
			if (this._trace.isError()) {
				this._trace.error("problem destroying " + this.id  + "; ex = " + ex.name + ": " + ex.message, methodName);
			}
		}
	}, 
	
	destroy: function(){
		// summary:
		//		Override's the widget's destroy to clean up some view-specific things.
		// description:
		//		Cleans up some view-specific things. For example, will destroy the dataset if
		//		this view is NOT part of a Nav Widget.
		// tags:
		//		public
		
		if (this._isDestroyed) {
			return;
		} else {
			this._isDestroyed = true;
		}
		
		// Destroy the busy pane if it exists
		if (this._busyPane) {
			this._busyPane.destroyRecursive(false);
			this._busyPane = null;
		}
		
		//Do clean up related to dataset
		if (this.dataset) {
			//Stop listening for changes to the old dataset
			this._unregisterDatasetEventListener(this.dataset);
			
			//destroy the dataset if this view is NOT a subview of another view, AND no one else is listening for changes
			if (!this.parentPluggableView && this.dataset.getEventListenerCount() <= 0 && this.destroyDatasetWhenDone) {
				this.dataset.destroy();
			}
		}
		
		//Do clean up related to widget def
		if(this.widgetDefinition) {
			this.widgetDefinition.destroy();
		}
		
		this.inherited(arguments);
	},
	
	isDestroyed: function() {
		return this._isDestroyed;
	},
	
	getParentPluggableViewWidgetDefinition: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		var parent = (this.parentPluggableView) ? this.parentPluggableView : this.managingPluggableView;
		if (parent) { 
			return parent.widgetDefinition;
		}
		return null;
	},
	
	getTopLevelPluggableViewWidgetDefinition: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		var topLevelPluggableView = this.getTopLevelPluggableView();
		return topLevelPluggableView.widgetDefinition;
	},
	
	getTopLevelPluggableView: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		var topLevelPluggableView = this;
		
		var parent = (this.parentPluggableView) ? this.parentPluggableView : this.managingPluggableView;
		while (parent != null) {
			topLevelPluggableView = parent;
			parent = parent.parentPluggableView;
		}
		return topLevelPluggableView;
	},
	
	getNearestPluggableViewWidgetDefinition: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		var nearestPluggableViewWidgetDefinition = this.widgetDefinition;
		
		var parent = (this.parentPluggableView) ? this.parentPluggableView : this.managingPluggableView;
		while (parent != null && nearestPluggableViewWidgetDefinition == null) {
			nearestPluggableViewWidgetDefinition = parent.widgetDefinition;
			parent = parent.parentPluggableView;
		}
		return nearestPluggableViewWidgetDefinition;
	},
	
	isRootLevelPluggableView: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		return this.parentPluggableView == null && this.managingPluggableView == null;
	},
	
	isAncestorOfPluggableView: function(viewToFind) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// viewToFind:
		//		
		var isAnscestorOf = (this.viewId == viewToFind.viewId) || this.getPluggableView(viewToFind.viewId);
		var iterator = this.getPluggableViewIterator();
		while (!isAnscestorOf && !iterator.atEnd()) {
			var view = iterator.get().value;
			isAnscestorOf = view.isAncestorOfPluggableView(viewToFind);
		}
		return isAnscestorOf;
	},
	
	//*********************************************************************
	
	_cleanUpImagePath: function(imagePath) {
		// summary:
		//		Convenience for subclasses to clean up image path (e.g., look for [widget] tag
		// 		in path and replace appropriately).
		// imagePath:
		//		Coming soon
		// description:
		//		Convenience for subclasses to clean up image path (e.g., look for [widget] tag
		// 		in path and replace appropriately).
		// tags:
		//		protected
		
		return ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.cleanUpImagePath(imagePath);
	},
	
	//******************************************************************************************
	// Functions for pluggable subview management
	//******************************************************************************************
    addPluggableView: function(view) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// view:
		//		
		this._childPluggableViews.add(view.viewId, view);
		
		this._addPluggableViewToContainer(view);  
	},
	
	removeAllPluggableViews: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// view:
		//	
		
		var iterator = this.getPluggableViewIterator();
		while (!iterator.atEnd()) {
			var view = iterator.get().value;
			this.removePluggableView(view);
		}
	},
	
	removePluggableView: function(view) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// view:
		//	
		
		//Remove from the list of views
		this._childPluggableViews.remove(view.viewId);
		
		//Remove from the Dojo container
		this._removePluggableViewFromContainer(view); 
		
		//Destroy the view
		view.destroyRecursive();
	},

	getPluggableView: function(viewId) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// viewId:
		//		
		return this._childPluggableViews.item(viewId);
	},
	
	isPluggableViewApplicable: function(viewId, dataset) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// viewId:
		//		
		
		var applicable = false;
		
		var view = this.getPluggableView(viewId);
		if (view) {
			applicable = view.isApplicable(dataset);
		}
		
		return applicable;
	},
	
	makePluggableViewVisible: function(viewId) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// viewId:
		//		
		
		//Delegate to subclass
	},
	
	getVisiblePluggableView: function(viewId) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// viewId:
		//		

		//Delegate to subclass
		return null;
	},
	
    getPluggableViewIterator: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		return this._childPluggableViews.getIterator();
	},
	
	forEachPluggableView: function(fn, scope) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// fn:
		//		
		// scope:
		//		
		this._childPluggableViews.forEach(fn, scope);
	},
	
	getPluggableViewCount: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		return this._childPluggableViews.count;
	},
	
	getState: function() {
		// summary:
		//		Return an instance of NavWidgetBaseViewState representing the current state of the view. 
		// description:
		//		Return an instance of NavWidgetBaseViewState representing the current state of the view. 
		//
		//		It is NOT recommended for subclasses to override getState, but instead override
		//		_getStateData (which will be called by this function).
		// tags:
		//		public

		//Create a base state 
		var stateData = this._getStateData();
		var state = new ibm.tivoli.tip.navwidget.dojo.NavWidgetBaseViewState({
			view: this,
			dataset: this.dataset,
			stateData: stateData
		});
		
		//Get child states
		this._populatePluggableViewStates(state);
		
		//Return the state
		return state;
	},
	
	_getStateData: function() {
		// summary:
		//		Returns an object of the implementer's choosing which will be put in the 
		//		NavWidgetBaseViewState returned by getState.
		// description:
		//		Coming soon
		// tags:
		//		protected
		
		return null;
	},
	
	_populatePluggableViewStates: function(state) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// tags:
		//		protected
		
		var iterator = this.getPluggableViewIterator();
		while (!iterator.atEnd()) {
			var view = iterator.get().value;
			state.addPluggableViewState(view.viewId, view.getState());
		}
	},
	
	getPluggableViewState: function(viewId) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		
		var viewState = null;
		var view = this.getState(viewId);
		if (view) {
			viewState = view.getState();
		}
		return state;
	},
	
	restoreFromState: function(state) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon

		this._restorePluggableViewsFromStates(state);
	},
	
	_restorePluggableViewsFromStates: function(state) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// tags:
		//		protected
		
		var iterator = this.getPluggableViewIterator();
		while (!iterator.atEnd()) {
			var view = iterator.get().value;
			if (state) {
				view.restoreFromState(state.getPluggableViewState(view.viewId));
			} else {
				view.restoreFromState(null);
			}
		}
	},
	
	getViewSpecificTasks: function() {		
		// summary:
		//		Get tasks / actions specific to the pluggable view.
		// description:
		//		Get tasks / actions specific to the pluggable view.
		//		Subclasses should override this function.
	},

	executeViewSpecificTask: function(commandName) {		
		// summary:
		//		Executes the commandName in the pluggable view.
		// description:
		//		Executes the commandName in the pluggable view.
		//		Subclasses should override this function.
	},
	
	getSiblings: function(){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		var siblings = [];
		if(this.parentPluggableView){
			var iterator = this.parentPluggableView.getPluggableViewIterator();
			while (!iterator.atEnd()) {
				siblings.push(iterator.get().value);
			}
		}
		return siblings;
	},	
	
	_getPluggableViewDefinitions: function() {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		var childPluggableViewDefinitions = [];
		if (this.widgetDefinition) {
			childPluggableViewDefinitions = this.widgetDefinition.getViews();
		}
		return childPluggableViewDefinitions;
	},
	
	_setDatasetCreatePluggableViewsPreviouslyCalled: false, //Flag to keep track of whether we've created views before or not
	_setDatasetCreatePluggableViews: function() {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// tags:
		//		protected
		
		//If this is the first time being called or we have a new widget def, then go ahead with creation
		if (!this._setDatasetCreatePluggableViewsPreviouslyCalled || this.isSetDatasetNewWidgetDef()) {
			//Let's brute force it and remove all existing pluggable views. NOTE: Could potentially be smarter and try to reuse
			//views that already exist (e.g., have same ID, etc. as on of the pluggable view defs we'll be processing)
			this.removeAllPluggableViews(); 

			//Process the new pluggable views
			var childPluggableViewDefinitions = this._getPluggableViewDefinitions();
			if (childPluggableViewDefinitions) {
				for (var i = 0; i < childPluggableViewDefinitions.length; i++) {
					var viewDef = childPluggableViewDefinitions[i];
					
					var viewArgs = {
							id: this.id + "_" + viewDef.viewId,
							viewId: viewDef.viewId,
							viewTypeSpec: viewDef.viewType,
							dataset: this.dataset,
							cmProvider: this.cmProvider,
							daHelper: this.daHelper,
							widgetDefinitionId: viewDef.definition,
							parentPluggableView: this
						};
	
					var viewInstance = ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getObjectInstance(viewDef.className, viewArgs);
					this._setDatasetCreatePluggableViewsAugmentViewAttributes(viewInstance, viewDef);
					if (viewInstance) {
						this.addPluggableView(viewInstance);
					}
				}
			}
			
			//Set up to listener for selection changes on all the views
			var iterator = this.getPluggableViewIterator();
			while (!iterator.atEnd()) {
				var view = iterator.get().value;
				var chainSelectionChanged = function (objectContext) {
					if (!this.selection.isChanging()) {
						this.selection.chainSelectionChanged(objectContext);
					}
				};
				dojo.connect(view.selection, "onSelectionChanged", this, chainSelectionChanged);
			}
		}
		
		//Remember that we've created pluggable views before
		this._setDatasetCreatePluggableViewsPreviouslyCalled = true;
	},
	
	_setDatasetCreatePluggableViewsAugmentViewAttributes: function(view, viewDef) {
		// summary:
		//		TODO
		// description:
		//		TODO
		// tags:
		//		protected
		
	},
	
	_addPluggableViewToContainer: function(view) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// tags:
		//		protected
		
		this.addChild(view);
	},
	
	_removePluggableViewFromContainer: function(view) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// tags:
		//		protected
		
		this.removeChild(view);
	},
	
	//*****************************************************************************************************
	// Events we fire
	//*****************************************************************************************************
	onFilterChange: function(){
		// summary:
		//		Event fired when an filter is changed
		// tags:
		//		callback
	},
	
	onSetDataset: function(/*ibm.tivoli.navmodel.Dataset*/ dataset){
		// summary:
		//		Event fired when an the dataset is set
		// dataset: Dataset
		//		The client nav model dataset that this view is now rendering
		// tags:
		//		callback
		
		//Mark we're done setting the dataset
		this._setDatasetNewWidgetDef = false;
		this._setDatasetActive = false;
		
		//Delegate anything else to listeners
	},
	
	//*****************************************************************************************************
	// Data change event handling management
	//*****************************************************************************************************
	// summary:
	//		Base class for a handler to deal with Dataset change events. By itself, it doesn't maintain
	//		any state to speak of, but it more of a convenience to mixin to another class.
	// description:
	//		Coming soon
	//
	
	//***********************************************************************************************/
	
	_registerDatasetEventListener: function(){
		// summary:
		//		Called during the setDataset lifecycle. Base implementation does... Subclasses may not have a need to override.
		// description:
		//		Tells the pluggable view it's time to register an event listener for the dataset
		// tags:
		//		private
		
		//Start listening for changes
		if (this.dataset) {
			this.dataset.addEventListener(this, dojo.hitch(this, this._datasetEventListenerCallback));
		}
	},
	
	_unregisterDatasetEventListener: function(){
		// summary:
		//		Called during the setDataset lifecycle. Base implementation does... Subclasses may not have a need to override.
		// description:
		//		Tells the pluggable view it's time to register an event listener for the dataset
		// tags:
		//		private
		
		//Stop listening for dataset changes
		if (this.dataset) {
			this.dataset.removeEventListener(this);
		}
	},
	
	//***************************************************************************************************************
	_datasetEventListenerCallback: function (/*Object*/ listenerObj, /*Object*/ event) {
		// summary:
		//		Callback for when there are changes within the Dataset we've registered to listen for updates to.
		// listenerObj:
		//		An object representing the listener registered with the Dataset.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Callback for when there are changes within the Dataset currently being rendered. It will call one of the 
		//		specific handlers based on the change type in the Event. It's not anticipated any subclass should need to 
		//		override this callback.
		// tags:
		//		private
		var type = event.getType();
		if (this._trace.isFinest()) {
			this._trace.entering("event received, type = " + type, "_datasetEventListenerCallback");
		}
		//Handle the different event types
		switch (type) {
			case "changed":
				this._handleChangedEvent(event);
				break;
			case "labelChanged":
				this._handleLabelChangedEvent(event);
				break;
			case "statusChanged":
				this._handleStatusChangedEvent(event);
				break;	
			case "itemsAdded":
				this._handleItemsAddedEvent(event);
				break;
			case "itemsRemoved":
				this._handleItemsRemovedEvent(event);
				break;
			case "itemStatusChanged":
				this._handleItemStatusChangedEvent(event);
				break;
			case "itemPropertiesChanged":
				this._handleItemPropertiesChangedEvent(event);
				break;
			case "linksAdded":
				this._handleLinksAddedEvent(event);
				break;
			case "linksRemoved":
				this._handleLinksRemovedEvent(event);
				break;
			case "linkStatusChanged":
				this._handleLinkStatusChangedEvent(event);
				break;
			case "linkPropertiesChanged":
				this._handleLinkPropertiesChangedEvent(event);
				break;
			case "linkChanged":
				this._handleLinkChangedEvent(event);
				break;
			case "relationshipStatusChanged":
				this._handleRelationshipStatusChangedEvent(event);
				break;
			case "relationshipPropertiesChanged":
				this._handleRelationshipPropertiesChangedEvent(event);
				break;
			case "exception":
				this._handleExceptionEvent(event);
				break;
			case "eventRouter":
				this._handleEventRouterEvent(event);
				break;
			default:
				if (this._trace.isError()) {
					this._trace.error("Invalid event type = " + type, "_datasetEventListenerCallback");
				}
				break;
		}
		
		//Notify pluggable views
		var iterator = this.getPluggableViewIterator();
		while (!iterator.atEnd()) {
			var view = iterator.get().value;
			view._datasetEventListenerCallback(listenerObj, event);
		}
		
		if (this._trace.isFinest()) {
			this._trace.exiting("event received, type = " + type, "_datasetEventListenerCallback");
		}
	},

	_handleChangedEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "changed" event type.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Subclass should override to if it wishes to deal with "changed" events.
		// tags:
		//		protected
		
	},

	_handleLabelChangedEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "labelChanged" event type.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Subclass should override to if it wishes to deal with "labelChanged" events.
		// tags:
		//		protected
		
	},
	
	_handleStatusChangedEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "statusChanged" event type.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Subclass should override to if it wishes to deal with "statusChanged" events.
		// tags:
		//		protected
		
	},
	
	_handleItemsAddedEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "itemsAdded" event type.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Subclass should override to if it wishes to deal with "itemsAdded" events.
		// tags:
		//		protected

	},

	_handleItemsRemovedEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "itemsRemoved" event type.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Subclass should override to if it wishes to deal with "itemsRemoved" events.
		// tags:
		//		protected
		
	},

	_handleItemStatusChangedEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "itemStatusChanged" event type.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Subclass should override to if it wishes to deal with "itemStatusChanged" events.
		// tags:
		//		protected
		
	},
	
	_handleItemPropertiesChangedEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "itemPropertiesChanged" event type.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Subclass should override to if it wishes to deal with "itemPropertiesChanged" events.
		// tags:
		//		protected
		
	},

	_handleLinksAddedEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "linksAdded" event type.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Subclass should override to if it wishes to deal with "linksAdded" events.
		// tags:
		//		protected
		

	},
	
	_handleLinksRemovedEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "linksRemoved" event type.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Subclass should override to if it wishes to deal with "linksRemoved" events.
		// tags:
		//		protected
		
	},
	
	_handleLinkStatusChangedEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "linkStatusChanged" event type.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Subclass should override to if it wishes to deal with "linkStatusChanged" events.
		// tags:
		//		protected
		
	},
	
	_handleLinkPropertiesChangedEvent: function(/*Object*/ event) {		
		// summary:
		//		Handle "linkPropertiesChanged" event type.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Subclass should override to if it wishes to deal with "linkPropertiesChanged" events.
		// tags:
		//		protected
		
	},

	_handleLinkChangedEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "linkChanged" event type.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Subclass should override to if it wishes to deal with "linkChanged" events.
		// tags:
		//		protected
		
	},
	
	_handleRelationshipPropertiesChangedEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "relationshipPropertiesChanged" event type.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Subclass should override to if it wishes to deal with "relationshipPropertiesChanged" events.
		// tags:
		//		protected
		
	},
	
	_handleRelationshipStatusChangedEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "relationshipStatusChanged" event type.
		// event:
		//		An object containing information about what changed in the Dataset.
		// description:
		//		Subclass should override to if it wishes to deal with "relationshipStatusChanged" events.
		// tags:
		//		protected

	},
	
	_handleExceptionEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "exception" event type.
		// event:
		//		An object containing information the "exception" event
		// description:
		//		Subclass should override to if it wishes to deal with "exception" events.
		// tags:
		//		protected

	},
	
	_handleEventRouterEvent: function(/*Object*/ event) {
		// summary:
		//		Handle "eventRouter" event type.
		// event:
		//		An object containing information the "eventRouter" event
		// description:
		//		Subclass should override to if it wishes to deal with "eventRouter" events.
		// tags:
		//		protected

	}
});