/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewSelection");

dojo.require("ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil");
dojo.require("ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewObjectContext");
dojo.require("dojox.collections.SortedList");

ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewSelection._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewSelection");

dojo.declare(
    "ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewSelection",
    ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewObjectContext,
    {
    	// summary:
    	//		Manages selection of items, links, relationships, and child datasets in a pluggable view. It is used internally
    	//		by the view. Override to implement custom selection.
    	// description:
    	//		Coming soon
    	
    	
  		// _changing: [private] ibm.tivoli.navmodel.Dataset
    	//		True is selection state is currently being updated
  		_changing: false,
  		
  		// _changing: [private] ibm.tivoli.navmodel.Dataset
    	//		True is selection state is currently being updated
  		_mode: "extended",
  		
  		// _trace: [private] Trace
  		//		Manages tracing for this class
  		_trace: ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewSelection._trace,
    
  		// operation:  this could be either a select or a deselect
		operation: "select",
  		
	 	isChanging: function(){
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		public
		
	 		return this._changing;
		},
		
		setMode: function(mode){
			//Let's clear selection on any mode changes
			if(this.getTotalSelected() > 0){
				this.deselectAll();
			}
			
			//Check that we have a valid mode passed in
			if(mode != "extended" && mode != "multiple" && mode != "single" && mode != "none"){
				this._mode = "extended";
				
				if (this._trace.isWarn()) {
					this._trace.warn("Invalid selection mode specified; mode = " + mode, "setMode");
				}
			}else{
				this._mode = mode;
			}
		},
		
		getMode: function(m){		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon
			// m:
			//		
			return this._mode;
		},
		
	 	setDataset: function(/*ibm.tivoli.navmodel.Dataset*/ dataset){
			// summary:
			//		TODO
			// dataset:
			//		The dataset we're managing selection fot
			// description:
			//		Coming soon
			// tags:
			//		public
		
			//Mark we're changing
			this._changing = true;
			
	 		//Set dataset
			this._dataset = dataset;
			
			//Clear out current selection
			if (!this.isEmpty()) {
				this._setSelectedObjectsBase(null, null, null, null);
			}
			
			//pass along the new dataset to the selection objects of the pluggable views
			var iterator = this._pluggableView.getPluggableViewIterator();
			while (!iterator.atEnd()) {
				var view = iterator.get().value;
				view.selection.setDataset(dataset);
			}
			
			//Mark we're done changing
			this._changing = false;
		},
	 	
	 	setSelectedObjectIds: function(/*String[]*/ itemIds, /*String[]*/ linkIds, /*String[]*/ relationshipIds){
			// summary:
			//		Coming soon
			// itemIds:
			//		An array of absolute item ids
			// linkIds:
			//		An array of absolute link ids
			// relationshipIds:
			//		An array of absolute relationship ids
			// description:
			//		Coming soon
			// tags:
			//		public
	 		
	 		//Mark we're changing
			this._changing = true; 
			
	 		//Clear out current selection
	 		this._items.clear();
	 		this._links.clear();
	 		this._relationships.clear();
	 		
	 		//Look up the objects in the dataset and add to hash tables
	 		this._setSelectedObjectIds(itemIds, linkIds, relationshipIds);

	 		//Pass along to selection objects on the sub-views
			var iterator = this._pluggableView.getPluggableViewIterator();
			while (!iterator.atEnd()) {
				var view = iterator.get().value;
				view.selection.setSelectedObjectIds(itemIds, linkIds, relationshipIds);
			}
			
			//Mark we're done changing
			this._changing = false;
	 		
	 		//Fire selection changed
			this.onSelectionChanged(this);
		},
		
		setSelectedObjects: function(/*ibm.tivoli.navmodel.Item[]*/ items, /*ibm.tivoli.navmodel.Link[]*/ links, /*ibm.tivoli.navmodel.Relationship[]*/ relationships){
			// summary:
			//		Coming soon
			// items:
			//		An array of items
			// links:
			//		An array of links
			// relationships:
			//		An array of relationships
			// description:
			//		Coming soon
			// tags:
			//		public
		
			//Mark we're changing
			this._changing = true;

			//Copy items/links/relationships into ourselves
	 		this._setSelectedObjectsBase(items, links, relationships);

	 		//Pass along to selection objects on the sub-views
			var iterator = this._pluggableView.getPluggableViewIterator();
			while (!iterator.atEnd()) {
				var view = iterator.get().value;
				view.selection.setSelectedObjects(items, links, relationships);
			}
			
			//Mark we're done changing
			this._changing = false;
	 		
	 		//Fire selection changed
			this.onSelectionChanged(this);
		},

		deselectAll: function() {
			this.setSelectedObjects(null, null, null, null);
		},
		
		setSelectedItems: function(/*ibm.tivoli.navmodel.Item[]*/ items){
			// summary:
			//		Coming soon
			// items:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		public
		
			this.setSelectedObjects(items, null, null, null);
		},
		
		setSelectedItem: function(/*ibm.tivoli.navmodel.Item*/ item){
			// summary:
			//		Coming soon
			// item:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		public
		
			this.setSelectedItems([item]);
		},
		
		setSelectedLinks: function(/*ibm.tivoli.navmodel.Link[]*/ links){
			// summary:
			//		Coming soon
			// links:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		public
		
			this.setSelectedObjects(null, links, null, null);
		},
		
		setSelectedLink: function(/*ibm.tivoli.navmodel.Link*/ link){
			// summary:
			//		Coming soon
			// link:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		public
		
			this.setSelectedLinks([link]);
		},
		
		setSelectedRelationships: function(/*ibm.tivoli.navmodel.Relationship[]*/ relationships){
			// summary:
			//		Coming soon
			// relationships:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		public
		
			this.setSelectedObjects(null, null, relationships, null);
		},
		
		setSelectedLink: function(/*ibm.tivoli.navmodel.Relationship*/ relationship){
			// summary:
			//		Coming soon
			// relationship:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		public
		
			this.setSelectedRelationships([relationship]);
		},

		chainSelectionChanged: function(objectContext){
			// summary:
			//		Coming soon
			// objectContext:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		protected
		
			//Mark we're changing
			this._changing = true;
			
			//Copy items/links into ourselves
	 		this._setSelectedObjectsBase(objectContext.getItems(), objectContext.getLinks(), objectContext.getRelationships());
	 		
	 		//NOTE: Let's not pass along to subviews at in this case
	 		//Pass along to pluggable views that are NOT an anscestor of the view managed by this selection object
			var iterator = this._pluggableView.getPluggableViewIterator();
			while (!iterator.atEnd()) {
				var view = iterator.get().value;
				if (!view.isAncestorOfPluggableView(objectContext._pluggableView)) {
					view.selection.setSelectedObjects(objectContext.getItems(), objectContext.getLinks(), objectContext.getRelationships());
				}
			}
	 		
	 		//Mark we're done changing
			this._changing = false;
			
	 		//Fire selection changed
			this.onSelectionChanged(this);
		},
		
	 	deselectObjectIds: function(/*String[]*/ itemIds, /*String[]*/ linkIds, /*String[]*/ relationshipIds){
			// summary:
			//		Coming soon
			// itemIds:
			//		An array of absolute item ids
			// linkIds:
			//		An array of absolute link ids
			// relationshipIds:
			//		An array of absolute relationship ids
			// description:
			//		Coming soon
			// tags:
			//		public
	 		
	 		//Mark we're changing
			this._changing = true; 
	 		
	 		//Look up the objects in the dataset and remove from hash tables
	 		this._deselectObjectIds(itemIds, linkIds, relationshipIds);

			// Now select what remains					
	 		//Look up the objects in the dataset and add to hash tables
	 		this._setSelectedObjectIds(this._items.getKeyList(), this._links.getKeyList(), this._relationships.getKeyList());

	 		//Pass along to selection objects on the sub-views
			var iterator = this._pluggableView.getPluggableViewIterator();
			while (!iterator.atEnd()) {
				var view = iterator.get().value;
				view.selection.setSelectedObjectIds(this._items.getKeyList(), this._links.getKeyList(), this._relationships.getKeyList());
			}
			
			//Mark we're done changing
			this._changing = false;

	 		//Fire selection changed
			this.operation = "deselect";
			this.onSelectionChanged(this);

			//Reset to default "select" operator
			this.operation = "select";
		},
		
		deselectObjects: function(/*ibm.tivoli.navmodel.Item[]*/ items, /*ibm.tivoli.navmodel.Link[]*/ links, /*ibm.tivoli.navmodel.Relationship[]*/ relationships){
			// summary:
			//		Coming soon
			// items:
			//		An array of items
			// links:
			//		An array of links
			// relationships:
			//		An array of relationships
			// description:
			//		Coming soon
			// tags:
			//		public
		
			//Mark we're changing
			this._changing = true;

			//Copy items/links/relationships into ourselves
	 		this._deselectObjects(items, links, relationships);
			
			// Now select what remains			
	 		// Look up the objects in the dataset and add to hash tables
	 		this._setSelectedObjectIds(this._items.getKeyList(), this._links.getKeyList(), this._relationships.getKeyList());

	 		//Pass along to selection objects on the sub-views
			var iterator = this._pluggableView.getPluggableViewIterator();
			while (!iterator.atEnd()) {
				var view = iterator.get().value;
				view.selection.setSelectedObjectIds(this._items.getKeyList(), this._links.getKeyList(), this._relationships.getKeyList());
			}

			//Mark we're done changing
			this._changing = false;
	 		
	 		//Fire selection changed
			this.operation = "deselect";
			this.onSelectionChanged(this);

			//Reset to default "select" operator
			this.operation = "select";
		},

		deselectItems: function(/*ibm.tivoli.navmodel.Item[]*/ items){
			// summary:
			//		Coming soon
			// items:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		public
		
			this.deselectObjects(items, null, null);
		},
		
		deselectItem: function(/*ibm.tivoli.navmodel.Item*/ item){
			// summary:
			//		Coming soon
			// item:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		public
		
			this.deselectItems([item]);
		},
		
		deselectLinks: function(/*ibm.tivoli.navmodel.Link[]*/ links){
			// summary:
			//		Coming soon
			// links:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		public
		
			this.deselectObjects(null, links, null);
		},
		
		deselectLink: function(/*ibm.tivoli.navmodel.Link*/ link){
			// summary:
			//		Coming soon
			// link:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		public
		
			this.deselectLinks([link]);
		},
		
		deselectRelationships: function(/*ibm.tivoli.navmodel.Relationship[]*/ relationships){
			// summary:
			//		Coming soon
			// relationships:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		public
		
			this.deselectObjects(null, null, relationships);
		},
		
		deselectRelationship: function(/*ibm.tivoli.navmodel.Relationship*/ relationship){
			// summary:
			//		Coming soon
			// relationship:
			//		Coming soon
			// description:
			//		Coming soon
			// tags:
			//		public
		
			this.deselectRelationships([relationship]);
		},


		//*****************************************************************************************************
		// Events we fire
		//*****************************************************************************************************
		onSelectionChanged: function(objectContext){
			// summary:
			//		Event fired when an item, link, or relationship is selected
			// description:
			//		Coming soon
			// objectContext: ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewObjectContext
			//		Representation of all of the items, links, and relationships in the selection
			
			//Do nothing
		}
    }
);