/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * "Restricted Materials of IBM"
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil");

/*=====
ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil = {
	// summary: Miscellaneous utilities
};
=====*/

//initializing variable for tab index values to specify a starting index
var _tabIndex=100;
// a multiplication factor to have different starting index for each of the views, 100,200,300 etc 
var _multIndex=1;

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil");

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.constants = {
	THEMES_NAME: "themes/",
	THEMES_FULL_PATH: dojo.moduleUrl("ibm.tivoli.tip.navwidget", "themes/").uri,
	WIDGET_TAG : "[widget]/"
};

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.cleanUpImagePath = function(/*String*/ imagePath,/*String?*/ basePath){
	// summary:
	//		Coming soon
	// description:
	//		Coming soon
	// imagePath:
	//		Coming soon
	// tags:
	//		public
	
	var returnVal = imagePath;
	
	//If we find "[widget]/", we want to assume the provider wants the image retrieved from the widget's default relative location, so let's replace
	//the "[widget]/" string with the right base path
	if (imagePath) {
		var widgetTagStartIndex = imagePath.indexOf(ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.constants.WIDGET_TAG);
		if (widgetTagStartIndex == 0) {
			var relativePath = imagePath.substring(ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.constants.WIDGET_TAG.length);
			if (basePath) {
				returnVal = basePath + ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.constants.THEMES_NAME + relativePath;
			} else {
				returnVal = ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.constants.THEMES_FULL_PATH + relativePath;
			}
		}
	}
	return returnVal;
};

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getObjectInstance = function(/*String*/ fullClassName, /*Object*/ args){
	// summary:
	//		Coming soon
	// description:
	//		Coming soon
	// fullClassName:
	//		Coming soon
	// args:
	//		Coming soon
	// tags:
	//		public

	var returnVal = null;
	
	dojo["require"](fullClassName);
	var objectClass = dojo.getObject(fullClassName);
	if (objectClass) {
		returnVal = new objectClass(args);
	} else {
		if (ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil._trace.isError()) {
			var methodName = "ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getObjectInstance";
			ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil._trace.error("Could not load " + fullClassName, methodName);
		}
	}

	return returnVal;
};

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.createWidgetDefinition = function(/*String*/ widgetDefClassName, /*String*/ widgetDefId, /*Object*/ widgetDefInfo, /*ibm.tivoli.navmodel.Dataset*/ dataset, /*ibm.tivoli.navmodel.WidgetDef?*/ parentWidgetDef){
	// summary:
	//		Coming soon
	// description:
	//		Coming soon
	// widgetDefClassName:
	//		Coming soon
	// widgetDefId:
	//		Coming soon
	// widgetDefInfo:
	//		Coming soon
	// dataset:
	//		Coming soon	
	// parentWidgetDef:
	//		Coming soon	
	// tags:
	//		public
	
	var methodName = "createWidgetDefinition";
	
	var widgetDefinition = null;
	var baseUrl = null;
	var provider = null;
	var datasource = null;
	var profileID = null;

	if (dataset) {
		baseUrl = dataset.getArgs().baseUrl;
		provider = dataset.getProviderId();
		datasource = dataset._args.datasource;
	} else {
		baseUrl = widgetDefInfo.baseUrl;
		provider = widgetDefInfo.provider;
		datasource = widgetDefInfo.datasource;
		profileID = widgetDefInfo.profileID;
	}
		
	var args = {
		baseUrl: baseUrl,
        	provider: provider,
        	datasource: datasource,
        	widget: widgetDefInfo.widget,
        	definition: widgetDefId,
		profileID: profileID,	 
		parentWidgetDef: parentWidgetDef
	};
	try {
		widgetDefinition = ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getObjectInstance(widgetDefClassName, args);		
	} catch (exception){
		// Check specifically for case where cannot connect to provider
		if (exception && exception.getMsgId() == "ATKRST102E") {
			if (ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil._trace.isWarn()) {
				ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil._trace.warn(exception.getMsgText(), methodName);
			}
			throw exception;
		}
		if (ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil._trace.isWarn()) {
			ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil._trace.warn("Specified widget definition cannot be found. Using default one. args = " + dojo.toJson(args), methodName);
		}
		
		var args = {
			baseUrl: widgetDefInfo.baseUrl,
            	provider: widgetDefInfo.provider,
            	datasource: widgetDefInfo.datasource,
            	widget: widgetDefInfo.widget,
            	definition: widgetDefInfo.definition,
			profileID: widgetDefInfo.profileID,
			parentWidgetDef: parentWidgetDef
		};
		try {
			widgetDefinition = ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getObjectInstance(widgetDefClassName, args);				
		} catch (exception){
			if (ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil._trace.isWarn()) {
				ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil._trace.warn("Default widget definition cannot be found. Using default settings. Attempted args = " + dojo.toJson(args), methodName);
			}
		};
	};
	
	return widgetDefinition;
};

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getItemByAbsoluteId = function(/*ibm.tivoli.navmodel.Dataset*/ rootDataset, /*String*/ absItemId){
	// summary:
	//		Finds an Item by absolute id.
	// description:
	//		Finds an Item by absolute id.
	// rootDataset:
	//		The top-level dataset to start the search from.
	// absItemId:
	//		The absolute item id is a ":" delimited list of ids of the form "CHILD_DATASET_A:CHILD_DATASET1:...:ITEM_1". The last
	// 		entry is the item id. Entries leading up to the last entry allow us to walk to the child dataset tree from the root dataset
	//      to find the child dataset containing the item.
	// tags:
	//		public

	//Get the item out of whatever dataset we determined contains the item based on the absolute id
	var item = rootDataset.getItem({itemId: absItemId});

	return item;
};

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getLinkByAbsoluteId = function(/*ibm.tivoli.navmodel.Dataset*/ rootDataset, /*String*/ absLinkId){
	//	summary:
	//		Finds an Link by absolute id.
	//	description:
	//		Finds an Link by absolute id.
	//	rootDataset:
	//		The top-level dataset to start the search from.
	//  absLinkId:
	//		The absolute link id is a ":" delimited list of ids of the form "CHILD_DATASET_A:CHILD_DATASET1:...:LINK_1". The last
	// 		entry is the link id. Entries leading up to the last entry allow us to walk to the child dataset tree from the root dataset
	//      to find the child dataset containing the link.
	//  tags:
	//		public

	//Get the link out of whatever dataset we determined contains the link based on the absolute id
	var link = rootDataset.getLink({linkId: absLinkId});

	return link;
};

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getRelationshipByAbsoluteId = function(/*ibm.tivoli.navmodel.Dataset*/ rootDataset, /*String*/ absRelationshipId){
	// summary:
	//		Finds a Relationship by absolute id.
	// description:
	//		Finds a Relationship by absolute id.
	// rootDataset:
	//		The top-level dataset to start the search from.
	// absRelationshipId:
	//		The absolute relationship id is a ":" delimited list of ids of the form "CHILD_DATASET_A:CHILD_DATASET1:...:RELATIONSHIP_1". The last
	// 		entry is the relationship id. Entries leading up to the last entry allow us to walk to the child dataset tree from the root dataset
	//      to find the child dataset containing the relationship.
	// tags:
	//		public

    //Get the relationship out of whatever dataset we determined contains the relationship based on the absolute id
	var relationship = rootDataset.getRelationship({relationshipId: absRelationshipId});

	return relationship;
};

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getItemAbsoluteId = function(/*ibm.tivoli.navmodel.Item*/ item){
	// summary:
	//		calculates absolute id of an item
	// item:
	//		An item
	// description:
	//		calculates absolute id of an item
	// tags:
	//		public

	//Return id
	return ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil._getItemOrLinkOrRelationshipAbsoluteId(item);
};

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getLinkAbsoluteId = function(/*ibm.tivoli.navmodel.Link*/ link){
	// summary:
	//		calculates absolute id of a link
	// link:
	//		A link
	// description:
	//		calculates absolute id of a link
	// tags:
	//		public

	//Return id
	return ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil._getItemOrLinkOrRelationshipAbsoluteId(link);
};

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getRelationshipAbsoluteId = function(/*ibm.tivoli.navmodel.Relationship*/ relationship){
	// summary:
	//		calculates absolute id of a relationship
	// link:
	//		A relationship
	// description:
	//		calculates absolute id of a relationship
	// tags:
	//		public

	//Return id
	return ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil._getItemOrLinkOrRelationshipAbsoluteId(relationship);
};

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil._getItemOrLinkOrRelationshipAbsoluteId = function(/*Object*/ object){
	// summary:
	//		calculates absolute id of an item or link
	// object:
	//		An item or a link
	// description:
	//		calculates absolute id of an item or link
	// tags:
	//		private

	//Initialize the abs id
	var absId = object.getId();
	
	//Return id
	return absId;
};

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getStringWithEllipsis = function(/*String*/ str, /*Boolean*/isAltTitle){
	// summary:
	//		For whatever reason, it was only discovered after PII freeze that we had no ellipses
	//		next to strings in our resource bundle. This is a temporary approach to add ellipses
	//		while avoiding string concatenation. Long-term solution is to fix the resource bundle.
	// description:
	//		For whatever reason, it was only discovered after PII freeze that we had no ellipses
	//		next to strings in our resource bundle. This is a temporary approach to add ellipses
	//		while avoiding string concatenation. Long-term solution is to fix the resource bundle.
	// upperBound:
	//		Coming soon
	// tags:
	//		public

	var templateString = "${label}";
	if (isAltTitle) {
		templateString += "..."; //Button flyovers don't seem to deal very well with HTML char codes like &hellip;
	} else {
		templateString += "&hellip;";
	}
	return dojo.string.substitute(templateString, {label: str});
};

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getNextRandomInt = function(/*int*/ upperBound){
	// summary:
	//		Generates random int value between 0 (inclusive) and upperBound (exclusive)
	// description:
	//		Generates random int value between 0 (inclusive) and upperBound (exclusive)
	// upperBound:
	//		Coming soon
	// tags:
	//		public

	var rand = Math.floor(upperBound*Math.random());
	return rand;
};

ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.getNextRandomBoolean = function(){
	// summary:
	//		Generates true or false (like a coin flip)
	// description:
	//		Generates true or false (like a coin flip)
	// tags:
	//		public

	var rand = Math.random();
	return rand >= 0.5;
};


//for tab index values
ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.tabIndexFunc = function(){
	// Summary:
	//		Assigning different tab index values to start with for each view
	// Description:
	// 		Assigning different tab index values to start with for each view
	// tags:
	//  	public
		
	return ((_multIndex++) * (_tabIndex));
};

// Replace special characters with a given replaceChar
ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil.replaceSpecialChars = function(/*Char*/ replaceChar, /*String*/ inputString) {  
	if (inputString && replaceChar) {
		inputString =  inputString.replace(/[^a-zA-Z 0-9]+/g,replaceChar);
	}
	return inputString;
};

