/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.require("dijit.Menu");
dojo.require("dijit.MenuItem");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.ComboButton");
dojo.require("dijit.form.DropDownButton");
dojo.require("dijit.form.TextBox");
dojo.require("dojo.data.ItemFileWriteStore");
dojo.require("ibm.tivoli.tip.navwidget.topology.dijit.TopologyLabel");

dojo.provide("ibm.tivoli.tip.navwidget.dojo.ToolbarUtil");

/*=====
ibm.tivoli.tip.navwidget.dojo.ToolbarUtil = {
	// summary: Toolbar creation utilities
};
=====*/

ibm.tivoli.tip.navwidget.dojo.ToolbarUtil._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.dojo.ToolbarUtil");

ibm.tivoli.tip.navwidget.dojo.ToolbarUtil.customLabel = function(/*Object*/ kwArgs) {
	// summary:
	//		Create a label object for the toolbar item	
	// id: String
	//		base id value for the component; will be appended to
	// item: Object
	//		the toolbar item from the widget definition
	// tags:
	//		public
	var label = null;
	var item = kwArgs.item;

	if (item.label && item.label.showLabel){
		label = new ibm.tivoli.tip.navwidget.topology.dijit.TopologyLabel({ 
			labelFor : kwArgs.id + "_" + item.id,
			labelText: item.label.value});
	}

	return label;
};

ibm.tivoli.tip.navwidget.dojo.ToolbarUtil.customText = function(/*Object*/ kwArgs) {
	// summary:
	//		Create a text object for the toolbar item	
	// id: String
	//		base id value for the component; will be appended to
	// item: Object
	//		the toolbar item from the widget definition
	// tags:
	//		public
	var item = kwArgs.item;
	return new dijit.form.TextBox({id: kwArgs.id + "_" + item.id});				
};

ibm.tivoli.tip.navwidget.dojo.ToolbarUtil.customButton = function(/*Object*/ kwArgs) {
	// summary:
	//		Create a button object for the toolbar item	
	// id: String
	//		base id value for the component; will be appended to
	// item: Object
	//		the toolbar item from the widget definition
	// tags:
	//		public
	var item = kwArgs.item;
	var buttonArgs = {
		id: kwArgs.id + "_" +  item.id,
		label: item.label.value,
		showLabel: item.label.showLabel,
		iconClass:item.iconClass				
	};

	var button = new dijit.form.Button(buttonArgs);
	if (item.alt) {
		button.attr("alt", item.alt.value); 
		button.attr("title", item.alt.value); 
	}
				
	if (item.action) {		
		ibm.tivoli.tip.navwidget.dojo.ToolbarUtil._connect(button, item.action.event, item.action.handler);
	};

	return button;
};


ibm.tivoli.tip.navwidget.dojo.ToolbarUtil.customComboButton = function(/*Object*/ kwArgs) {
	// summary:
	//		Create a combo button object for the toolbar item	
	// id: String
	//		base id value for the component; will be appended to
	// item: Object
	//		the toolbar item from the widget definition
	// tags:
	//		public
	var item = kwArgs.item;

	var options = new dojo.data.ItemFileWriteStore({data: {identifier: 'name', items:[]}});
	if (item.options) {
		for (var i = 0; i < item.options.length; ++i) {
			options.newItem({name: item.options[i].name, value: item.options[i].value});
		}
	};	

	var buttonArgs = {
		id: kwArgs.id + "_" + item.id,
		name: kwArgs.id + "_" + item.id,
		store: options,
		searchAttr: "name",
		value:item.options[0].value //TODO add possibility of specifying selected value
	};
							
	return new dijit.form.ComboBox(buttonArgs);
};

ibm.tivoli.tip.navwidget.dojo.ToolbarUtil.customDropDown = function(/*Object*/ kwArgs) {
	// summary:
	//		Create a drop down object for the toolbar item	
	// id: String
	//		base id value for the component; will be appended to
	// item: Object
	//		the toolbar item from the widget definition
	// tags:
	//		public
	var item = kwArgs.item;
			
	var dropDownArgs = {
		id: kwArgs.id + "_" +  item.id + "_menu",
		style: "display: none;",
		autoSize: true
	};
	var dropDownMenu = new dijit.Menu(dropDownArgs);
				
	if (item.options) {
		for(var i = 0; i < item.options.length; ++i) {
			var menuArgs = {
				id: kwArgs.id + "_" +  item.id + "_menu_" + i,
				label: item.options[i].label.value,
				iconClass: item.options[i].iconClass
			};
						
			var menuItem = new dijit.MenuItem(menuArgs);
			ibm.tivoli.tip.navwidget.dojo.ToolbarUtil._connect(menuItem, "onClick", item.options[i].action.handler);
			dropDownMenu.addChild(menuItem);
		}
	}

	var buttonArgs = {
		id: kwArgs.id + "_" +  item.id,
		label: item.label.value,
		title: item.label.value,
		showLabel: item.label.showLabel,					
		dropDown: dropDownMenu,
		iconClass: item.iconClass
	};
	var button = new dijit.form.DropDownButton(buttonArgs);
			
	if (item.alt){
		button.attr("alt", item.alt.value);
		button.attr("title", item.alt.value); 
	}
		
	return button;
};

ibm.tivoli.tip.navwidget.dojo.ToolbarUtil.viewToggler = function(/*Object*/ kwArgs) {
	// summary:
	//		Create a view toggler for the toolbar item	
	// view: ibm.tivoli.tip.navwidget.dijit.NavWidgetPluggableView
	//		pluggable view associated with the toolbar
	// item: Object
	//		the toolbar item from the widget definition
	// tags:
	//		public
	var item = kwArgs.item;
			
	var pulldownArgs = {
		id: kwArgs.view.id + "_viewMenu",
		style: "display: none;",
		autoSize: true
	};
	var viewPulldown = new dijit.Menu(pulldownArgs);
			
	var pluggableViews = kwArgs.view.getSiblings();
	var defaultLabel = null;
	var defaultIconClass = null;
	if (pluggableViews) {
		for (var i = 0; i < pluggableViews.length; ++i) {
			var label = pluggableViews[i].getLabel();
			var iconClass = pluggableViews[i].getLabelIconClass();
			var viewMenuItem = new dijit.MenuItem({
 				id: kwArgs.view.id + "_viewMenuItem_" + pluggableViews[i].viewId,
 				label: label,
 				viewId: pluggableViews[i].viewId,
       			iconClass: iconClass
    			});

			ibm.tivoli.tip.navwidget.dojo.ToolbarUtil._connect(viewMenuItem, "onClick", kwArgs.view, function(/*Object*/ event) {							
				var viewMenuItem = event.target;
				while (!viewMenuItem.parentNode.parentNode.id || viewMenuItem.parentNode.parentNode.id != this.id + "_viewMenu") {
					viewMenuItem = viewMenuItem.parentNode;
				};
				viewMenuItem = dijit.byId(viewMenuItem.id);
				if (this.parentPluggableView) {
					this.parentPluggableView.makePluggableViewVisible(viewMenuItem.viewId);
				}							
			});
			viewPulldown.addChild(viewMenuItem);	
						
			// mark default view as selected
			if (pluggableViews[i].viewId == kwArgs.view.viewId) {
				defaultLabel = label;
				defaultIconClass = iconClass;
			}
		}
	};

	ibm.tivoli.tip.navwidget.dojo.ToolbarUtil._connect(kwArgs.view, "onSetDataset", function() {
		var pluggableViews = this.getSiblings();
		if (pluggableViews) {
			for (var i = 0; i < pluggableViews.length; ++i) {
				var menuItem = dijit.byId(this.id + "_viewMenuItem_" + pluggableViews[i].viewId);
				if (menuItem) {
					var applicable = this.parentPluggableView.isPluggableViewApplicable(pluggableViews[i].viewId, this.dataset);
					menuItem.attr("disabled", !applicable);
				}
			}
		}
	});
	    		
	var buttonArgs = {
		label: defaultLabel,
		dropDown: viewPulldown,
		title:  defaultLabel,
		iconClass: defaultIconClass,
		showLabel: false //OID requests we don't show the label in the toolbar
	};
	return new dijit.form.ComboButton(buttonArgs);
};				

ibm.tivoli.tip.navwidget.dojo.ToolbarUtil._connect = function(object, event, context, handler) {
	try {
		dojo.connect(object, event, context, handler);
	} catch (error) {
		var msg = "Failed to dojo.connect(" + object + ", " + event + ", " + context + ", " + handler + ")";
		ibm.tivoli.tip.navwidget.dojo.ToolbarUtil._trace.error(msg, "ibm.tivoli.tip.navwidget.dojo.ToolbarUtil._connect");
	}
};


