dojo.provide("ibm.tivoli.tip.navwidget.iwidget.ConditionDataset");

dojo.require("ibm.tivoli.navmodel.Dataset");

dojo.declare("ibm.tivoli.tip.navwidget.iwidget.ConditionDataset", [ ibm.tivoli.navmodel.Dataset ], {
	// summary:
	//		An extension to the ibm.tivoli.navmodel.Dataset that adds a hook to specify a condition to apply on the dataset, 
	//		and adds a convenience fetchParameters method
	relevantItemIds:null,
	getItems: function(/*getItems_kwArgs*/kwArgs) {
		// summary:
		//		Overrides getItems in order to inject the condition into the rest call.
		if(this._args.condition){
			if(kwArgs.condition){
				kwArgs.condition="("+this._args.condition+") and ("+kwArgs.condition+")";
			} else {
				kwArgs.condition=this._args.condition;
			}
		}
		this.inherited(arguments);
	},
	setRelevantItemIds: function setRelevantItemIds(itemIds){
		this.relevantItemIds=itemIds;
	},
	getRelevantItemIds: function getRelevantItemIds(){		
		return this.relevantItemIds;
	},
	equals: function equals(dataset){
		var basicsEqual=(this.getProviderId()===dataset.getProviderId()&&
				this.getDatasourceId()===dataset.getDatasourceId()&&
				this.getId()===dataset.getId()
		);
		basicsEqual=(basicsEqual)?this._compareParamsOneWay(this._args.parameters,dataset._args.parameters):false;
		basicsEqual=(basicsEqual)?this._compareParamsOneWay(dataset._args.parameters,this._args.parameters):false;
		return basicsEqual;
	},
	_compareParamsOneWay:function _compareParamsOneWay(params1,params2){
		var param=null;
		for (param in params1) {
			if (params1.hasOwnProperty(param)) {
				if(!params2.hasOwnProperty(param)||params1[param]!==params2[param]){
					console.log("Datasets differ on parameter:"+param);
					console.log("One Dataset has :",params1[param]);
					console.log("The other Dataset has :",params2[param]);
					return false;
				}
			}
		}
		return true;
	},
	/*=====
	fetchParameters_kwArgs: function(onComplete, onError, type, properties, start, count, sync) {
		// onComplete: cols_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/	
	fetchParameters: function(/*fetchColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously fetch columns from rest service		
		var content = ibm.tivoli.navmodel.Utils.getContent(this.getVersion());
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		if (kwArgs.source) {
			content.source = kwArgs.source;
		}
		if (kwArgs.type) {
			content.type = kwArgs.type;
		}
		if (kwArgs.properties) {
			content.properties = kwArgs.properties.join(",");
		}
		if (kwArgs.metadata) {
			content.metadata = kwArgs.metadata;
		}
		if (kwArgs.start && kwArgs.start > 0) {
			content.start = kwArgs.start;
		}
		if (kwArgs.count && kwArgs.count > 0) {
			content.count = kwArgs.count;
		}

		return dojo.xhrGet({
			url: this._getParametersURL(),
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var params = [];
				var paramsLength = result.items.length;
				var index=0;
				for (index = 0;  index < paramsLength;  ++index) {
					params[index] = result.items[index];
				}
				
				kwArgs.onComplete(params, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchParameters"), ioArgs);
			})
		});
	},
    _getParametersURL: function() {
		// summary:
		//		Returns url for fetching current parameters for a dataset
		return this.getArg("baseUrl") + this.getAttribute("uri") + "/parameters";	
	}
});
