dojo.provide("ibm.tivoli.tip.navwidget.iwidget.IWPropertiesView");

dojo.require("ibm.tivoli.tip.navwidget.iwidget._NavView");
dojo.require("ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebook");
dojo.require("ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebookGeneralTab");

dojo.declare("ibm.tivoli.tip.navwidget.iwidget.IWPropertiesView",
    [ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebook,ibm.tivoli.tip.navwidget.iwidget._NavView],{
	_tabMap:null,
	_MAX_TABS:10,
	constructor: function constructor() {
		// summary:
		//		The Properties Notebook View to iWidget bridge.	
		this._tabMap=[];
	    dojo.connect(this, "onSetDataset", this, "startProcessingEvents");
	},
    startup: function startup() {
        this.inherited(arguments);
        this.selectItems(this._readItemIds());
    },
    postCreate: function postCreate() {
		this.inherited(arguments);
	},
	setDataset: function setDataset(dataset) {
		this.inherited(arguments);
		if(this.tablist){
			if(dojo.exists("getRelevantItemIds",dataset) && dataset.getRelevantItemIds()){
				this.selectItems(dataset.getRelevantItemIds());
			}else{
				this.selectItems(this._readItemIds());
			}
		}
	},
    _readItemIds: function _readItemIds(){
		var itemIds=[];
		var readItem=function(items,kwArgs){
			dojo.forEach(items,function(item){
				itemIds.push(item.getId());
			});
		};
		var args={
			sync:true,
			count:this._MAX_TABS,
			onComplete:readItem
		};
		this.dataset.getItems(args);
		return itemIds;
    },
	selectItems: function selectItems(/*array*/itemIds) {
		// summary:
		//		Shows the attributes of the selected items as tabs in this notebook
		// description:
		//		The item for each item id is fetched and added as a tab in this notebook. The tab label is the name of the item,
		//		and the order of the tabs matches the order of the given array of item ids. The last item id, representing the 
		//		latest item selected, is then selected.
		// tags:
		//		public
    	if(!itemIds){return false;}
		if(itemIds.length===0){
		    this.destroyDescendants();
		    this._tabMap=[];
		} else {
		    dojo.forEach(itemIds,dojo.hitch(this,function(itemId,index){
	            while(this._tabMap.length>index && this._tabMap[index]._internalId!==itemId){
	                this.removeChild(this._tabMap[index]);
	                this._tabMap[index].destroyRecursive();
	                this._tabMap.splice(index,1);
	            }	            
	            if(this._tabMap.length<=index && this._tabMap.length<this._MAX_TABS){
	                var tab=new ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebookGeneralTab({_internalId:itemIds[index]});
	                tab.setDataset(this.dataset);
	                var item=this.dataset.getItem({itemId:itemIds[index]});
					tab.title=this._getTabTitle(item);
	                tab.setObjects([item], null, null, null);
	                this._tabMap.push(tab);
	                this.addChild(tab);
	            }
	            if(index===itemIds.length-1){
	                while(this._tabMap.length-1>index){
	                    var oldTab=this._tabMap.pop();
	                    this.removeChild(oldTab);
	                    oldTab.destroyRecursive();
	                }
	            }
		    }));
		}
		this.selectChild(this._tabMap[this._tabMap.length-1]);
		this.setObjects();
	},
	_getTabTitle: function _getTabTitle(item){
		var nameProp=item.getProperty('NAME');
		if(nameProp){
			return nameProp.getDisplayValue();
		} else if(item.getLabel()){
			return item.getLabel();
		} else if(item.getId()){
			return item.getId();
		} else {
			return "General";
		}
	}
});