dojo.provide("ibm.tivoli.tip.navwidget.iwidget._NavView");

dojo.require("ibm.tivoli.tip.navwidget.iwidget.Trace");
dojo.require("ibm.tivoli.tip.navwidget.iwidget.event.NodeClickedOnEvent");
dojo.require("ibm.tivoli.tip.navwidget.iwidget.event.EventReceiver");
dojo.declare("ibm.tivoli.tip.navwidget.iwidget._NavView", [ibm.tivoli.tip.navwidget.iwidget.event.EventReceiver], {
	// _connections:
	//		used to store all the dojo.connect handles that this view uses for easy cleanup.
	_connections:null,
	// _selectedItems:
	//		cache of the previous set of selected items, used to identify the chronological order of selections.
	_selectedItems:null,
	_iw_nv_trace:null,
	constructor: function constructor() {
		// summary:
		//		Base class for all Nav Views that need a bridge to the iWidget wrapper. It defines the API that the iWidget wrapper expects
		//		to communicate events to the Nav Views.
		this._connections=[];//each Nav View should have its own array of connections
		this._selectedItems=[];//each Nav View should have its own array of selected items
		this._iw_nv_trace=new ibm.tivoli.tip.navwidget.iwidget.Trace(this,["connectListeners","disconnectListeners","onNavItemSelected",
		                                                             "sendNavItemSelected","getEventId","selectItems","destroy"],
		                                                             this._iw_er_trace,"ibm.tivoli.tip.navwidget.iwidget._NavView");
	},
	connectListeners: function connectListeners(/*ibm.tivoli.tip.navwidget.iwidget.IWidgetPluggableView*/iWidget) {
		// summary:
		//		Connects common events from the iWidget wrapper to this view.
		this._connections.push(dojo.connect(this.selection,"onSelectionChanged",this,dojo.partial(this.sendNavItemSelected,iWidget.iContext)));
		this._connections.push(dojo.connect(iWidget,"onNavItemSelected",this,"queueEvent"));
		this._connections.push(dojo.connect(iWidget,"onLaunchPage",this,"queueEvent"));
		this._connections.push(dojo.connect(this,"onEvent",this,"onNavItemSelected"));
	},
	disconnectListeners: function disconnectListeners() {
		// summary:
		//		Disconnects all of the listeners that this view is listening to.
		dojo.forEach(this._connections,dojo.disconnect);
		this._connections=[];
	},
	onNavItemSelected: function onNavItemSelected(/*ibm.tivoli.tip.navwidget.iwidget.event.Event*/event,/*ibm.tivoli.tip.navwidget.iwidget.ConditionDataset*/dataset) {
		// summary:
		//		This is the Nav View subscriber for iWidget selection events.
		// description:
		//		The iWidget events are passed into it, the event is parsed and the selected items are analyzed.
		//		The algorithm detects the chronological order of when the items where selected based on previous invocations
		//		and the chronologically adjusted array is fed to the selectedItems function.
		//		Nav Views should attach the iWidget event to this function, and implement their own selectedItems function
		//		to act upon the parsed out selection event.
		var data=dataset.getRelevantItemIds();
		if(!data){
			var items=[];
			dataset.getItems({sync:true,onComplete:function(results){items=results;}});
			data=[];
			var pullItemIds=function(item){
				data.push(item.getId());
			};
			dojo.forEach(items,pullItemIds);			
		}
		if(dataset.equals(this.dataset)){
			var selectedItems=this._selectedItems;
			var removedArray=dojo.filter(selectedItems,function(itemId) {
				return dojo.indexOf(data,itemId)===-1;
			});
			dojo.forEach(removedArray,function(itemId) {
				var removeIndex=dojo.indexOf(selectedItems,itemId);
				selectedItems.splice(removeIndex,1);
			});
			dojo.forEach(data,function(itemId) {
				if(dojo.indexOf(selectedItems,itemId)===-1){
					selectedItems.push(itemId);
				}
			});
			this.selectItems(this._selectedItems);
		}else{
			this.setDataset(dataset);
		}
	},
	sendNavItemSelected: function sendNavItemSelected(/*iContext*/iContext,/*ibm.tivoli.tip.navwidget.dojo.NavWidgetPluggableViewSelection*/selection) {
		// summary:
		//		Adapts a Nav View selection event into a tip event and uses the iWidget infrastructure to fire the event.
		// description:
		//		The Nav View selection event is used to create an event that is compatible with the tip event schema. This function is
		//		automatically connected to the Nav View's selection event by the connectListeners function
		var event=new ibm.tivoli.tip.navwidget.iwidget.event.NodeClickedOnEvent();
		event.addItems(selection.getItems());
		console.log("FIRING EVENT:"+dojo.toJson(event.getPayload()));
		iContext.iEvents.fireEvent(event.name,null,event.getPayload());
	},
	selectItems: function selectItems(/* array */itemIds) {
		// summary:
		//		Called with a hard array of ids for the selected items. The array is ordered in chronological order, with the most recent
		//		item selected at the end of the array.
		// tags:
		//		public extension
	},
	destroy: function destroy(){
		this.inherited(arguments);
		if(this._iw_nv_trace){this._iw_nv_trace.destroy();}
	}
});