dojo.provide("ibm.tivoli.tip.navwidget.iwidget.event.AbstractDatasetFactory");

dojo.require("ibm.tivoli.tip.navwidget.iwidget.ConditionDataset");
dojo.require("ibm.tivoli.tip.navwidget.iwidget.Trace");

dojo.declare("ibm.tivoli.tip.navwidget.iwidget.event.AbstractDatasetFactory",[],{
	providerType:null,
	providerId:null,
	provider:null,
	datasetParamsCache:{},
	templateDataset:null,
	constructor: function constructor(templateDataset){
		ibm.tivoli.tip.navwidget.iwidget.Trace(this,['createDataset','_handleWithEventContext','_handleWithStandardContext','_handleWithMissingAttrs','_handleWithUnknownParams',
		                                                 '_parametersCheck','_eventAttrsCheck','_createDatasetWithEventContext','createDatasetWithStandardContext','_createDatasetPlusItemIds',
		                                                 '_cannotHandleEvent','_getDatasetParameters','_getCommonAttributes','_intersectArrays','_isParameterRequired','_areRequiredParamsAvailable',
		                                                 '_initProviderInfo','_createLikeDataset','_shrinkFixParams']);
		this.templateDataset=templateDataset;
	},
	createDataset: function createDataset(resources,event){
		if(this.provider.getAttribute("contextType")==='event'){
			return this._handleWithEventContext(resources,event);
		}else{
			return this._handleWithStandardContext(resources,event);
		}
	},
	//HANDLER HOOKS
	_handleWithEventContext: function _handleWithEventContext(resources,event){
		return this._createDatasetWithEventContext(event);
	},
	_handleWithStandardContext: function _handleWithStandardContext(resources,event){
		var parameters=this._getDatasetParameters();
		if(!this._parametersCheck(parameters)){
			return this._handleWithUnknownParams(resources);
		}
		var commonAttrs=this._getCommonAttributes(resources);
		if(!this._eventAttrsCheck(parameters,commonAttrs)){
			return this._handleWithMissingAttrs(resources,commonAttrs);
		}
		return this._createDatasetWithStandardContext(resources,commonAttrs,parameters);
	},
	_handleWithMissingAttrs: function _handleWithMissingAttrs(){
		return this._cannotHandleEvent();
	},
	_handleWithUnknownParams: function _handleWithUnknownParams(){
		return this._cannotHandleEvent();
	},
	//VALIDATIONS
	_parametersCheck: function _parametersCheck(params){
		return params.length!==0;
	},
	_eventAttrsCheck: function _eventAttrsCheck(params,commonAttrs){
		return this._areRequiredParamsAvailable(params,commonAttrs);
	},
	//POSSIBLE RETURNS
	_createDatasetWithEventContext: function _createDatasetWithEventContext(event){
		var parameters={eventContext:event.toJson()};
		return this._createLikeDataset(parameters);
	},
	_createDatasetWithStandardContext: function _createDatasetWithStandardContext(resources,commonAttrs,parameters){
		var paramsToSend=dojo.filter(parameters,function(param){
			return dojo.indexOf(commonAttrs,param.id)!==-1;
		});
		var params={};
		var provType=this.providerType;
		dojo.forEach(paramsToSend,function(param){
			params[param.id]=[];
			dojo.forEach(resources,function(resource){
				params[param.id].push(resource.getAttrValue(param.id,provType));
			});
		});
		params=this._shrinkFixParams(paramsToSend,params);
		return this._createLikeDataset(params);
	},
	_createDatasetPlusItemIds: function _createDatasetPlusItemIds(resources){
		var itemIds=[];
		var provType=this.providerType;
		var templateDatasetId=this.templateDataset.getId();
		dojo.forEach(resources,function(resource){
			var value=resource.getAttrValue("ItemId",provType);
			var datasetIdVal=resource.getAttrValue("DatasetId",provType);
			if(value&&datasetIdVal===templateDatasetId){itemIds.push(value);}
		});
		var dataset=this._createLikeDataset({});
		if(itemIds.length>0){dataset.setRelevantItemIds(itemIds);}
		return dataset;
	},
	_cannotHandleEvent: function _cannotHandleEvent(){
		return false;
	},
	//HELPERS
	_getDatasetParameters: function _getDatasetParameters(){
		if(!this.datasetParamsCache[this.providerId]){
			var ds=this._createLikeDataset({});
			var parameters={};
			ds.fetchParameters({
				sync:true,
				onComplete:function(params){
					parameters=params;
				}
			});
			this.datasetParamsCache[this.providerId]=parameters;
		}
		return this.datasetParamsCache[this.providerId];	
	},
	_getCommonAttributes: function _getCommonAttributes(resources){
		var commonTypedAttrs={};
		var commonAttrs=[];
		if(resources.length>0){
			dojo.forEach(resources,function(resource,index){
				if(index===0){
					commonTypedAttrs=resource.getAvailableAttributes(this.providerType);
				}else{
					commonTypedAttrs=this._intersectTypedAttrs(commonTypedAttrs,resource.getAvailableAttributes(this.providerType));
				}
			},this);
		}
		for(type in commonTypedAttrs){
			if(commonTypedAttrs.hasOwnProperty(type)){
				dojo.forEach(commonTypedAttrs[type],function(attr){
					commonAttrs.push(attr);
				});
			}
		}
		return commonAttrs;
	},
	_intersectTypedAttrs: function _intersectTypedAttrs(orig,filter){
		var results={};
		for(type in orig){
			if(orig.hasOwnProperty(type)){
				if(!filter[type]){
					results[type]=[];
				}else{
					results[type]=this._intersectArrays(orig[type],filter[type]);
				}
			}
		}
		return results;
	},
	_intersectArrays: function _intersectArrays(orig,filter){
		return dojo.filter(orig,function(item){
			return dojo.indexOf(filter,item)>-1;
		});
	},
	_isParameterRequired: function _isParameterRequired(params,paramId){
		var i=0;
		for(i=0;i<params.length;i++){
			var param=params[i];
			if(param.id===paramId){
				return param.required;
			}
		}
		return false;
	},
	_areRequiredParamsAvailable: function _areRequiredParamsAvailable(params,commonAttrs){
		var checkRequired=function(param){
			if(param.required){
				return dojo.indexOf(commonAttrs,param.id)>-1;
			}
			return true;
		};
		return dojo.every(params,checkRequired);
	},
	_initProviderInfo: function _initProviderInfo(provider){
		this.providerType=provider.getType();
		this.providerId=provider.getId();
		this.provider=provider;
	},
	_createLikeDataset: function createLikeDataset(parameters){
		var args={
			baseUrl:this.templateDataset.getArg("baseUrl"),
			provider:this.providerId,
			datasource: this.templateDataset.getDatasourceId(),
			dataset:this.templateDataset.getId(),
			parameters:dojo.clone(parameters),
			version:this.templateDataset.getVersion()			
		};
		return new ibm.tivoli.tip.navwidget.iwidget.ConditionDataset(args);
	},
	_shrinkFixParams: function _shrinkFixParams(paramsToSend,params){
		var shrunkParams={};
		dojo.forEach(paramsToSend,function(param){
			var vals=params[param.id];
			if(vals.length>0){
				shrunkParams[param.id]=vals[0];
			}
		});
		return shrunkParams;
	}
});