/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.properties.PropertyRenderer");

dojo.require("ibm.tivoli.navmodel.Trace");
dojo.require("ibm.tivoli.navmodel.Property");
dojo.require("ibm.tivoli.navmodel.Status");
dojo.require("ibm.tivoli.navmodel.ItemStyle");
dojo.require("ibm.tivoli.navmodel.Item");
dojo.require("ibm.tivoli.tip.navwidget.dojo.NavWidgetUtil");
dojo.require("dijit.ProgressBar");

ibm.tivoli.tip.navwidget.properties.PropertyRenderer._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.properties.PropertyRenderer");

//declare global link handler
ibm.tivoli.tip.navwidget.properties.handleLinkClick = function(domNode) {		
	// summary:
	//		handle a link click and make sure it gets handled to right place
		
	//get the attributes from the domNode that are set when the link is set up
	var propID = dojo.attr(domNode, "propid");
	var objectID = dojo.attr(domNode, "objectid");
	var viewID = dojo.attr(domNode, "viewid");
	var objectType = dojo.attr(domNode, "objecttype");
	
	//get refrence to the original viewobject that created this property
	var viewObj = dijit.byId(viewID);
	
	//now that we found the view, let it handle the link click 
	// ( this method will get the proper object and prop objects and then call renders
	//   handle link class in order handle the action on the link click )
	viewObj.handleLinkClickAction(viewID, objectID, propID, objectType);
};

dojo.declare("ibm.tivoli.tip.navwidget.properties.PropertyRenderer",
    null,
    {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
	
        //***************************
        // "Public"
        //***************************  	
		//summary: 
		//		callback for determining if a property needs a threshold value. 
	    //description:
		//		This callback is a method to determine if a the given view/prop needs thresholding
		//		if this value is null, then it assumed that no thresholding is needed.
		propThresholdCallback: null,
		
        //***************************
        // "Private"
        //***************************
	
		// _trace: [private] ibm.tivoli.navmodel.Trace
		//		Utility for trace and logging
        _trace: ibm.tivoli.tip.navwidget.properties.PropertyRenderer._trace, 
        _daHelper: null,

        
        //***************************
        // "Public Methods"
        //***************************  	
        
        constructor: function() {		
			// summary:
			//		Coming soon
			// description:
			//		Coming soon

        },

	// rendered node for a simple string type
	getRenderedItemNode: function(/*ibm.tivoli.navmodel.Item*/ item, plugviewref) {		
    	// summary:
    	//		Coming soon
    	// description:
    	//		Coming soon
        // item:
        //		Coming soon
        // plugviewref:
        //		Coming soon
		var contentNode = dojo.doc.createElement("div");
		var label = "";
		if (item) {
			label = item.getLabel();

			var style = item.getItemStyle({
        			mergeWidgetDef: true,
        			widgetDef: plugviewref.getNearestPluggableViewWidgetDefinition()
        		});
        				
        		if (style) {
        			//create the image node
        			nameImageNode = dojo.doc.createElement("img");
 			    					       					
        			//add the attributes to the img node based on the item style
        			dojo.attr(nameImageNode, "src", plugviewref._cleanUpImagePath(style.getSmallImage()));
        			dojo.attr(nameImageNode, "height", style.getAttribute("smallImageHeight"));
        			dojo.attr(nameImageNode, "width", style.getAttribute("smallImageWidth"));
        			dojo.attr(nameImageNode, "style", "vertical-align:sub");
				dojo.attr(nameImageNode, "hspace", "3px");
        			dojo.addClass(nameImageNode, "tctGridCellImg"); //253021

				var altText = style.getLabel();
				if (altText) {
					dojo.attr(nameImageNode, "alt", altText);
				}
        			        					
        			//append the image node
        			dojo.place(nameImageNode, contentNode,"last");
        		} 
			else if (this._trace.isWarn()) {
        			this._trace.warn("Item style is NULL for item = " + item.getId(), "getRenderedPropertyNode");
        		}
		}       	

		dojo.place(dojo.doc.createTextNode(label), contentNode, "last");						
		return contentNode;
	},

	// rendered node for a simple string type
	getRenderedValueNode: function(value) {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// value:
		//		Coming soon
		var contentNode = dojo.doc.createElement("div");
		dojo.place(dojo.doc.createTextNode(value), contentNode, "last");						
		return contentNode;
	},

    getRenderedPropertyNode: function(/*ibm.tivoli.navmodel.Item*/ object, prop, plugviewref, rowID, cell) {		
		// summary:
		//		Depending on the specific type of a property from the Property.getValueType() function
		//		return a node with the proper displayable content from the value.
		// description:
		//		|	values:	"itemName", "itemType", "linkName", "linkType", "relatonshipName", "relationshipType",
		//		|		"string", "string[]",
		//		|		"byte", "short", "int", "long", "byte[]", "short[]", "int[]", "long[]",
		//		|		"ubyte, "ushort", "uint", "ulong", "ubyte[]", "ushort[]", "uint[]", "ulong[]",
		//		|		"boolean", "boolean[]"
		//		|		"enum", "map",
		//		|		"status",
		//		|		"date", "datetime", "time", "date[]", "datetime[]", "time[]",
		//		|		"isodate", "isotime", "isodatetime", "isointerval", "isoduration", "isodate[]", "isotime[]", "isodatetime[]"
		//		|		"float", "double", "percentage", "float[]", "double[]"
		// object:
		//		Coming soon
		// prop:
		//		Coming soon
		// plugviewref:
		//		Coming soon

		//first check if there is a valid value
		switch(prop.getValueState()){
		case "ok":

				switch (prop.getValueType()) {
				case "itemName":

					var contentNode = dojo.doc.createElement("div");

					if(prop.getValueUnits()) {
						dojo.place(dojo.doc.createTextNode(prop.getDisplayValue() + " " + prop.getValueUnits()), contentNode,"last");
					} else {
						var style = object.getItemStyle({
							mergeWidgetDef: true,
							widgetDef: plugviewref.getNearestPluggableViewWidgetDefinition()
						});

						if (style) {
							//create the image node
							var nameImageNode = this._getImageContentNode( plugviewref, prop, object, style.getSmallImage(), style.getAttribute("smallImageHeight"), style.getAttribute("smallImageWidth"), style.getLabel());
							dojo.place(nameImageNode, contentNode,"last");
							
						} else {
							if (this._trace.isWarn()) {
								this._trace.warn("Item style is NULL for object = " + object.getId(), "getRenderedPropertyNode");
							}
						}
						
						//Add the text node        			    
						dojo.place(this._getLabelContentNode(prop.getDisplayValue(), prop, object, plugviewref ),contentNode,"last");

					}	
					
					//color background if thresholding needed
					this._thresholdBackground(cell, prop, object);
			
					//dojo.place(contentNode, outerNode, "last");
					return contentNode;

				case "status":


					var contentNode = dojo.doc.createElement("div");

					//the getStatusValue returns and object, which can be used to get the image
					//in order to use this we must mixin the widget def as well as the dataset and object type
					var sobject = prop.getStatusValue({
						widgetDef: plugviewref.getNearestPluggableViewWidgetDefinition(),
						dataset: plugviewref.dataset,
						objectType: object.getObjectType()
					});
					var statusImageNode = this._getImageContentNode( plugviewref, prop, object, sobject.getImage(), sobject.getImageHeight(), sobject.getImageWidth(), sobject.getLabel());
				
					//append the image node, and create then append the text node
					dojo.place(statusImageNode, contentNode,"last");

					var statusLabel = "";
					if(prop.getDisplayValue() && !prop.getDisplayValue().id){
						statusLabel = prop.getDisplayValue();
					}
					if (!statusLabel && cell) {
						cell.customStyles.push("text-align: center");
					}  
	
					dojo.place(this._getLabelContentNode(statusLabel, prop, object, plugviewref  ), contentNode,"last");
					
					//color background if thresholding needed
					this._thresholdBackground(cell, prop, object);

					return contentNode;
				case "percentage":
					var contentNode = dojo.doc.createElement("div");

					if(prop.getValueUnits()) {
						dojo.place(dojo.doc.createTextNode(prop.getDisplayValue() + " " + prop.getValueUnits()), contentNode,"last");
					} else {    					
                  // 255204
                  // Check if there is a maximum value specified on the property and if it exists and is non-zero, use it,
                  // otherwise use 1 which is the default assumption that percentage values will be between 0 and 1.
                  var maxValue = prop.getValueMax();    
                  maxValue = ((maxValue == null) || (maxValue <= 0)) ? 1 : maxValue;
						var pbar = new dijit.ProgressBar({ progress: prop.getValue(), 
															places: 0, 
															maximum: maxValue, 
															style:"width:100%"});
						
						dojo.query(".dijitProgressBarTile", pbar.domNode).addClass("tableCellDefaultProgressBarTile"); //SAPM desired a different color for the bar
						
						dojo.place(pbar.domNode, contentNode, "last");
					}
					
					//a threshold callback is defined, so use it to check if we need
					//to threshold this value and ADD an label, which we normally wouldnt render
					if(this.propThresholdCallback){
						var threshold = this.propThresholdCallback(prop, object);
						//only if a label is defined on the threshold for this value
						if(threshold && threshold.label){
							//render the node, and add it
							dojo.place(this._getLabelContentNode("", prop, object, plugviewref  ), contentNode, "last");	
						}
	
						if(threshold) {
							//set the background color
							if (threshold.color){
								//get the node for the progress bar portion that holds the color
								var barNode = dojo.query(".dijitProgressBarTile", pbar.domNode);
								dojo.style(barNode[0], "backgroundColor", threshold.color );
							}
							
							//set the foreground color
							if (threshold.fontColor){
								var labelNode = dojo.query(".dijitProgressBarLabel", pbar.domNode);
								dojo.style(labelNode[0], "color", threshold.fontColor );
							}
						}
					}
					
					//dont color background of progress bar since the bar itself is colored
					//this._thresholdBackground(cell, prop, object);
					
					return contentNode;

				default:
					var contentNode = dojo.doc.createElement("div");

					//a threshold callback is defined, so use it to check if we need
					//to threshold this value and ADD an image which we normally wouldnt render
					if(this.propThresholdCallback){
						var threshold = this.propThresholdCallback(prop, object);
						
						//only if an image is defined on the threshold for this value
						if(threshold && threshold.icon){
							var altText = (threshold.label ? threshold.label.value : null);
							if (!altText) {
								altText = prop.getDisplayValue();
							}

							//render the node, and add it
							dojo.place(this._getImageContentNode( plugviewref, prop, object, "", "16px", "16px"), contentNode, "last", altText);	
						}
					}
				
					if(prop.getValueUnits()) {
						dojo.place(this._getLabelContentNode(prop.getDisplayValue() + " " + prop.getValueUnits(), prop, object, plugviewref ), contentNode,"last");
					} else {
						dojo.place(this._getLabelContentNode(prop.getDisplayValue(), prop, object, plugviewref  ), contentNode,"last");
					}	
					
					//color background if thresholding needed
					this._thresholdBackground(cell, prop, object);
					
					return contentNode;
				//probably need to add some error handling in here in case the input value is invalid	
				}
			

		case "pending":
			var contentNode = dojo.doc.createElement("div");
		
			//get the image node			
			var nameImageNode = this._getImageContentNode( plugviewref, prop, object, "[widget]/images/processing-toolbar.gif", "16px", "16px", plugviewref.commonResources.PENDING);
			
			//append the image node
			dojo.place(nameImageNode, contentNode,"last");
			
			//a threshold callback is defined, so use it to check if we need
			//to threshold this value and ADD an label, which we normally wouldnt render
			if(this.propThresholdCallback){
				var threshold = this.propThresholdCallback(prop, object);
				//only if a label is defined on the threshold for this value
				if(threshold && threshold.label){
					//render the node, and add it
					dojo.place(this._getLabelContentNode("", prop, object, plugviewref  ), contentNode, "last");	
				}
			}
			
			//color background if thresholding needed
			this._thresholdBackground(cell, prop, object);
			
			return contentNode;
			
			
		case "error":
			var contentNode = dojo.doc.createElement("div");			
			var nameImageNode = this._getImageContentNode( plugviewref, prop, object, "[widget]/images/ac16_connstatnotoperational_24.png", "16px", "16px", plugviewref.commonResources.ERROR);
			
			//append the image node
			dojo.place(nameImageNode, contentNode,"last");
			
			//a threshold callback is defined, so use it to check if we need
			//to threshold this value and ADD an label, which we normally wouldnt render
			if(this.propThresholdCallback){
				var threshold = this.propThresholdCallback(prop, object);
				//only if a label is defined on the threshold for this value
				if(threshold && threshold.label){
					//render the node, and add it
					dojo.place(this._getLabelContentNode("", prop, object, plugviewref  ), contentNode, "last");	
				}
			}
			
			//color background if thresholding needed
			this._thresholdBackground(cell, prop, object);
				
			return contentNode;
		
		case "none":
			var contentNode = dojo.doc.createElement("div");
			contentNode.innerHTML = "&mdash;";
			return contentNode;

		default:
			
		}
	},
	
	getPropertyDisplayValue: function(object, prop, plugviewref) {		
		// summary:
		//		Depending on the specific type of a property from the Property.getValueType() function
		//		return a string suitable for the user. This method mostly intended for export.
		// description:

		// item:
		//		Coming soon
		// prop:
		//		Coming soon
		// plugviewref:
		//		Coming soon

		//first check if there is a valid value
		var displayValue = null;
		switch (prop.getValueState()){
			case "ok":
				if(prop.getValueUnits()) {
					displayValue = prop.getDisplayValue() + " " + prop.getValueUnits();
				} else {
					displayValue = prop.getDisplayValue();
				}	
				break;
			
			case "pending":
				displayValue = plugviewref.commonResources.PENDING;
				break;
	
			case "error":
				displayValue = plugviewref.commonResources.ERROR;
				break;
				
			default:
				break;
		}
		
		if (!displayValue) {
			displayValue = "";
		}
	
		return displayValue;
	},

	_thresholdBackground: function(cell, prop, object){
		//if a threshold is defined use the callback to determine if we need to color it
		if(this.propThresholdCallback && cell){
			var threshValue = this.propThresholdCallback(prop, object);
			if(threshValue) {
				if (threshValue.color){
					cell.customStyles.push("background-color: " + threshValue.color);
				}
				if (threshValue.fontColor){
					cell.customStyles.push("color: " + threshValue.fontColor);
				}
			}
		}
	},
	_getImageContentNode: function( plugviewref, prop, object, imagePath, height, width, altText){
		//create the image node
		var fullImageNode;
		var linkNode;
	
		var imageNode= dojo.doc.createElement("img");
		//var thresholded = false;
		
		//a threshold callback is defined,
		if(this.propThresholdCallback){
		//use it to check if we need to see if we need to threshold this image
			var threshold = this.propThresholdCallback(prop, object);
			if(threshold && threshold.icon){
				//threshold condition met, so use the threshold image instead
				//dojo.attr(imageNode, "src", plugviewref._cleanUpImagePath(threshold.icon));
				imagePath = plugviewref._cleanUpImagePath(threshold.icon);
			}
		}
			

		//add the attributes to the img node based on the object style
		dojo.attr(imageNode, "src", plugviewref._cleanUpImagePath(imagePath));
		dojo.attr(imageNode, "height", height);
		dojo.attr(imageNode, "width", width);
		dojo.attr(imageNode, "style", "vertical-align:sub");
		dojo.attr(imageNode, "hspace", "3px");
		dojo.addClass(imageNode, "tctGridCellImg"); //253021
		if (altText) {
			dojo.attr(imageNode, "alt", altText);
		}

		if(prop.getTask()){
			linkNode = dojo.doc.createElement("a");
			
			dojo.attr(linkNode, "onclick", "ibm.tivoli.tip.navwidget.properties.handleLinkClick(this);return false;" );
			dojo.attr(linkNode, "id", object.getId() + "_"+ prop.getId()+".img.link");
			dojo.attr(linkNode, "propid", prop.getId());
			dojo.attr(linkNode, "objectid", object.getId());
			dojo.attr(linkNode, "viewid", plugviewref.id);
			//check to see which type of object this is and set on ojbecttype attr for link
			var objType = null;
			if(object instanceof ibm.tivoli.navmodel.Item){
				objType = "ibm.tivoli.navmodel.Item";
			}
			else if(object instanceof ibm.tivoli.navmodel.Link){
				objType = "ibm.tivoli.navmodel.Link";
			}
			else if(object instanceof ibm.tivoli.navmodel.Relationship){
				objType = "ibm.tivoli.navmodel.Relationship";
			}
			dojo.attr(linkNode, "objecttype", objType);
		
			
			dojo.attr(linkNode, "href", "#");
			
			dojo.place(imageNode, linkNode, "last");
			
			fullImageNode = linkNode;
			
		}else{
			fullImageNode = imageNode;
		}
		
		
		return fullImageNode;

	},
	
	//create either a normal label, or a link if needed
	_getLabelContentNode: function(labelText, prop, object, plugviewref){
		//a threshold callback is defined,
		if(this.propThresholdCallback){
			//use it to check if we need to threshold this label
			var threshold = this.propThresholdCallback(prop, object);
			if(threshold && threshold.label && threshold.label.value){
				//use the threshold label instead of the normal label passed in
				labelText = threshold.label.value;
			}
		}
		
		if(prop.getTask()){
		
			var propTask = prop.getTask();
			//TODO:: ADM need better check for task, getCategory shuold work, but returns extra info

			//if this is a property task
			//this returns more than just "property" so lets use a diff check
			//if(propTask.getCategory() == "property"){

			//render SECOND! link for test
			var linkNode = dojo.doc.createElement("a");
				
			//var linkHelper2 = dojo.hitch(this, this._handleLinkClick, prop, plugviewref);
			dojo.attr(linkNode, "onclick", "ibm.tivoli.tip.navwidget.properties.handleLinkClick(this);return false;" );
			//render the link
			dojo.attr(linkNode, "id", object.getId() + "_"+ prop.getId()+".link");
			
			dojo.attr(linkNode, "propid", prop.getId());	
			dojo.attr(linkNode, "objectid", object.getId());	
			dojo.attr(linkNode, "viewid", plugviewref.id);
			//check to see which type of object this is and set on ojbecttype attr for link
			var objType = null;
			if(object instanceof ibm.tivoli.navmodel.Item){
				objType = "ibm.tivoli.navmodel.Item";
			}
			else if(object instanceof ibm.tivoli.navmodel.Link){
				objType = "ibm.tivoli.navmodel.Link";
			}
			else if(object instanceof ibm.tivoli.navmodel.Relationship){
				objType = "ibm.tivoli.navmodel.Relationship";
			}
			dojo.attr(linkNode, "objecttype", objType);
				
			dojo.attr(linkNode, "href", "#");

			dojo.place(dojo.doc.createTextNode(labelText), linkNode, "last");
	
			return linkNode;
		//else render as normal	
		}else{	
			return dojo.doc.createTextNode(labelText);
		}
	},
	
	   //Event callback method to handle updating the tooltip content with updated content
	_handleLinkClick: function(property, objectContext, e){

		//first stop the default browser event:
		if(e){
			dojo.stopEvent(e);
		}
		//Create DefaultActionHelper
		//By default launches a link url else executes the task and launches it (via callback)
		if(!this._daHelper){
			this._daHelper = new ibm.tivoli.tip.navwidget.topology.dijit.DefaultActionHelper( 
					{
						defaultActionCallback: dojo.hitch(this, "_executeLink",objectContext, property)
					}
			);
		}
		//actually execute the default action/task (defined in constuctor)
		this._daHelper.executeDefaultAction(objectContext, property);
 		   
		//this._daHelper.destroy();
		this._daHelper = null;
		return false;
	 },
	 
	 _executeLink: function(objectContext, prop){
		//check if this is a url
		console.log("The default link task did not execute as expected");
	 }
			
	
    });
