/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.topology.dojo.CSVWriter");

dojo.require("ibm.tivoli.tip.navwidget.topology.dojo.Exporter");

ibm.tivoli.tip.navwidget.topology.dojo.Exporter.registerWriter("csv",
		"ibm.tivoli.tip.navwidget.topology.dojo.CSVWriter");

dojo.declare("ibm.tivoli.tip.navwidget.topology.dojo.CSVWriter",null, 
{
	// summary:
	//		Coming soon
	// description:
	//		Coming soon
	
	// _separator: [private] String
	//      The string to use as a delimiter between values.
	//      The default is a comma.
	_separator: ',',
	
	constructor: function(/* object? */writerArgs) {
		// summary:
		//		CSV default separator is ',', but can be customized
		// description: 
		//      Coming soon
		// writerArgs: object?
		//		{separator:'...'}
	
		if (writerArgs && writerArgs.separator) {
			this._separator = writerArgs.separator;
		}
		this._headers=[];
		this._dataRows=[];
	},
	
	beforeHeader: function(/* TopologyView */ topology) {
		// summary:
		//		We are going to start the travel in the topology graph
		//		Is there anything we should do now?
		// description:
		//		Coming soon
		// return:
		//		true: go on handling the header row and then call afterHeader.
		//		false: skip the header row, won't call afterHeader.
		
		return true;
	},
	
	afterHeader: function() {
		// summary:
		//		The header line has been handled
		// description:
		//		Coming soon
	},
	
	beforeContent: function(/* Array */items) {
		// summary:
		//		We are ready to go through all the contents (items).
		// description:
		//		Coming soon
		// items:
		//		All the items fetched from the store
		// return:
		//		true: go on handling the contents and then call afterContent.
		//		false: skip all the contents, won't call afterContent.
		
		return true;
	},
	
	afterContent: function() {
		// summary:
		//		We have finished the entire graph travel.
		//		Do some clean up work if you need to.
		// description:
		//		Coming soon
	},
	
	beforeItem: function(/* object */argObj) {
		// summary:
		//		Before handling a line of data for an item (not header).
		// description:
		//		Coming soon
		// argObj:
		//		An object with at least the following context properties...
		// return:
		//		true: go on handling the current item and then call afterItem.
		//		false: skip the current item, won't call afterItem.
		
		return true;
	},
	
	afterItem: function(/* object */argObj) {
		// summary:
		//		After handling a line of data for an item (not header).
		// description:
		//		Coming soon
		// argObj:
		//		An object with at least the following context properties...
		
		this._dataRows.push(dojo.map(argObj.rowValues, this._formatCSVValue));
	},

	toString: function() {
		// summary:
		//		The exported CSV string
		// description:
		//		Coming soon

		var result = this._headers.join(this._separator),
			endl = "\r\n";
		for (var i = this._dataRows.length-1; i >= 0; --i) {
			this._dataRows[i] = this._dataRows[i].join(this._separator);
		}
		return result + endl + this._dataRows.join(endl);
	},
	
	handleValue: function(/* object */argObj) {
		// summary:
		//		Handle a header row value or item row value.
		// description:
		//		Coming soon
		// argObj:
		//		An object with at least the following context properties available:
		//		{
		//			isHeader,
		//			item,
		//			value
		//		}
		
		if (argObj.isHeader){
			this._headers.push(this._formatCSVValue(argObj.value));
		}
	},
	
	_formatCSVValue: function(/* string */value) {
		// summary:
		//		Format value to follow CSV standard.
		//		See: http://en.wikipedia.org/wiki/Comma-separated_values
		// description:
		//		Coming soon
		// value: string
		//		The value for a property.
		// returns:
		//		The formatted value as a string
		
		if (!value) {
			return '';
		}
		var result = String(value).replace(/"/g, '""');
		if (result.indexOf(this._separator) >= 0 || result.search(/[" \t\n]/) >= 0) {
			result = '"' + result + '"';
		}
		return result;
	}
});
