/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.topology.dojo.Exporter");

(function(){
dojo.declare("ibm.tivoli.tip.navwidget.topology.dojo.Exporter",null, 
{
	// summary:
	//		Basic approach derived from dojox.grid.enhanced.plugins.Exporter
	// description:
	//		Coming soon
	
	name: "Exporter",
	
	topology: null,

	constructor: function(inTopology) {
		// summary:
		//		only newed by TopologyView
		// inTopology: ibm.tivoli.tip.navwidget.topology.dijit.TopologyView
		//		The topology to plug in to.

	    // doing same mixin() approach from EnhancedGrid
		var props = {};
		for (p in this) {
			if(p == '_inherited' || p == 'declaredClass' || p == 'constructor' 
			   || this['privates'] && this['privates'][p]){
			   	continue; 
			}
			props[p] = this[p];
		}
		dojo.mixin(inTopology, props);
		
		this.topology = inTopology;
	},
	
	exportTo: function(/* object? */keywordArgs,/* string */fileType,
			/* object? */writerArgs,/* function(string) */onExported){
		// summary:
		//		Export required items and links to a kind of format(fileType)
		//		using the corresponding writer with given arguments(writerArgs),
		//		then pass the exported text to a given function(onExported).
		// description:
		//		Coming soon
		// keywordArgs: object?
		//		Any arguments for controlling aspects of the export
		// fileType: string
		//		A registered export format name
		// writerArgs: object?
		//		Arguments for the given format writer
		// onExported: function(string)
		//		Call back function when export result is ready
		
		// if first arg is a string, no keywordArgs, so shift parms up
		if (dojo.isString(keywordArgs)){
			onExported = writerArgs;
			writerArgs = fileType;
			fileType = keywordArgs;
			keywordArgs = {};
		}
		// if no writerArgs, shift up the callback function
		if (dojo.isFunction(writerArgs)){
			onExported = writerArgs;
			writerArgs = {};
		}
		// check for export format type
		if (!dojo.isString(fileType)){
			throw new Error("Exporter.exportTo: 'fileType' is not a string");
		}
		// check for callback function
		if (!dojo.isFunction(onExported)){
			throw new Error("Exporter.exportTo: Missing callback function 'onExported'.");
		}
		
		this.showBusyPane();
		var writer = _getExportWriter(fileType,writerArgs),
		    oldfunc = keywordArgs.onComplete,
		    _this = this;
		keywordArgs.onComplete = function(items, rels, request){
									oldfunc && oldfunc(items, rels, request);
									onExported(_goThroughGraphData(_this, items, rels, writer));
								};
		// this branch does not apply unless we support dojo datastore level access						
		if (this.store){
			// do nothing for now
			// this.store.fetch(keywordArgs);
			console.log("??? did we get here in the this.store branch?");
		}
		else {
			// Get items and relationships from the dataset
			this.exportItems = [];
			this.exportRels = [];
			this.dataset.getItems({sync: true, onComplete: dojo.hitch(this, this._gotItems), onError: dojo.hitch(this, this._errorItems)});
			this.dataset.getRelationships({sync: true, onComplete: dojo.hitch(this, this._gotRels), onError: dojo.hitch(this, this._errorRels)});
			onExported(_goThroughGraphData(this, this.exportItems, this.exportRels, writer));
		}
	},
	
	exportSelectedTo: function(/* string */ fileType, /* object? */ writerArgs){
		// summary:
		//		Only export selected items and links.
		// description:
		//		Coming soon
		// fileType: string
		//		A registered export format name
		// writerArgs: object?
		//		Arguments for the given format writer
		// returns:
		//		The exported string
		
		var writer = _getExportWriter(fileType, writerArgs);
		return _goThroughGraphData(this, this.selection.getSelected(), writer);
	},
	
	_gotItems: function(/*Object[]*/items, /*Object*/kwArgs) {
		if (items && items.length > 0) {
			this.exportItems = items;			
		}
	},
	
	_gotRels: function(/*Object[]*/relationships, /*Object*/kwArgs) {
		if (relationships && relationships.length > 0) {
			this.exportRels = relationships;			
		}
	},
	
	_errorItems: function(error, exception, ioArgs) {
		console.log("### Error retrieving items for export: " + error);
		
		//Give the user some feedback
        this.topology.displayException(error);
	},
	
	_errorRels: function(error, exception, ioArgs) {
		console.log("### Error retrieving relationships for export: " + error);	
		
		//Give the user some feedback
        this.topology.displayException(error);
	}
});

var 
_buildRow = function(/* object */arg_obj,/* ExportWriter */writer){
	// summary:
	//		Use the given export writer(writer) to go through a single row
	//		which is given in the context object(arg_obj).
	// tags:
	//		private

	var cells = [];
	if (arg_obj.isHeader) {
		// TODO: Need to load column titles from resource bundle
		cells = ["Name", "Type"];
		if (arg_obj.colTotal > cells.length) {
			cells = cells.concat(["From", "RelType", "To"]);
		}
		dojo.forEach(cells, function(cell) {
			writer.handleValue({value: cell, isHeader: arg_obj.isHeader});
		});
	} else if (arg_obj.row.getObjectType() == "item") {
		cells.push(arg_obj.row.getLabel());
		cells.push(arg_obj.row.getTypeLabel());
		var filledCols = cells.length;
		for (var i = 0; i < arg_obj.colTotal - filledCols; i++) {
			cells.push("");
		}
		arg_obj.rowValues = cells;
	} else if (arg_obj.row.getObjectType() == "relationship") {
		for (var j = 0; j < arg_obj.colOffset; j++) {
			cells.push("");
		}
		cells.push(arg_obj.row.getSourceItem().getLabel());
		cells.push(arg_obj.row.getLabel());
		cells.push(arg_obj.row.getTargetItem().getLabel());
		arg_obj.rowValues = cells;
	}
},

_goThroughGraphData = function(/* TopologyView */topology, /* Array */items, /* Array */relationships, /* ExportWriter */writer) {
	// summary:
	//		Use the given export writer(writer) to go through the graph 
	//		and the given items and relationships, then return the writer output.
	// tags:
	//		private
	// returns:
	//		String
	
	var itemColCount = 2,
		relColCount = (relationships.length > 0) ? 3 : 0;
	var arg_obj = {
			'topology': topology,
			isHeader: true,
			colTotal: itemColCount + relColCount,
			colOffset: 0
	};
	
	//go through header
	if(writer.beforeHeader(topology)){
		_buildRow(arg_obj, writer);
		writer.afterHeader();	
	}
	//go through content
	arg_obj.isHeader = false;
	if(writer.beforeContent(items)){
		dojo.forEach(items, function(item){
			arg_obj.row = item;
			if(writer.beforeItem(arg_obj)){
				_buildRow(arg_obj, writer);
				writer.afterItem(arg_obj);
			}
		});
		arg_obj.colOffset = itemColCount;
		dojo.forEach(relationships, function(item){
			arg_obj.row = item;
			if(writer.beforeItem(arg_obj)){
				_buildRow(arg_obj, writer);
				writer.afterItem(arg_obj);
			}
		});
		writer.afterContent();
	}

	this.exportItems = null;
	this.exportRels = null;
	return writer.toString();
},

_getExportWriter = function(/* string */ fileType, /* object? */ writerArgs){
	// summary:
	//		Use the given export format type(fileType)
	//		and writer arguments(writerArgs) to create 
	//		a ExportWriter and return it.
	// description:
	//		Coming soon
	
	var writerName,
		cls,
		expCls = ibm.tivoli.tip.navwidget.topology.dojo.Exporter;
	if (expCls.writerNames && (writerName = expCls.writerNames[fileType.toLowerCase()])){//intentional assignment
		if (cls = dojo.getObject(writerName)){ //intentional assignment
			return new cls(writerArgs);	//ExportWriter
		}else{
			throw new Error('Please make sure class "' + writerName + '" is required.');
		}
	}
	throw new Error('The writer for "' + fileType + '" has not been registered.');
};

ibm.tivoli.tip.navwidget.topology.dojo.Exporter.registerWriter = function(/* string */fileType,/* string */writerClsName){
	// summary:
	//		Register a writer(writerClsName) to a export format type(fileType).
	//		This function separates the Exporter from all kinds of writers.
	// description:
	//		Coming soon
	// returns:
	//		undefined
	var expCls = ibm.tivoli.tip.navwidget.topology.dojo.Exporter;
	expCls.writerNames = expCls.writerNames || {};
	expCls.writerNames[fileType] = writerClsName;
};
})();