/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.tree.dijit.NavModelTreeTable");

dojo.require("ibm.tivoli.tip.navwidget.dijit.AsynchContextMenu");
dojo.require("ibm.tivoli.tip.navwidget.topology.dijit.MenuProviderHelper");
dojo.require("ibm.tivoli.tip.navwidget.dijit.CustomContentTooltip");
dojo.require("ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebookGeneralTab");

dojo.require("dojoe.treetable.TreeTableAll"); 

dojo.requireLocalization("ibm.tivoli.tip.navwidget", "resources");

dojo.declare ("ibm.tivoli.tip.navwidget.tree.dijit.NavModelTreeTable",
	[dojoe.treetable.TreeTable],
{

	// viewLoaded: boolean
	//		Coming soon
	viewLoaded : false,
	
	// _propOnSetObjectsConnection: [private] Object
	//		Coming soon
	_propOnSetObjectsConnection: null,
	
	// _propOnSetDatasetConnection: [private] Object
	//		Coming soon
	_propOnSetDatasetConnection: null,

	// _tooltip: [private] Object
	//      Coming soon
	_tooltip : null,
	
	// _resizing: [private] Number
	//      Coming soon
	_resizing : 0,

	postCreate : function() {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon

		//first let the tree grid get generated as normal
		this.inherited(arguments);
	
		// create tooltip onto the tree grid domNode
		this._tooltip = new ibm.tivoli.tip.navwidget.dijit.CustomContentTooltip(
					{
						connectId : [ this.grid.domNode.id ],
						getCustomContentCallback : dojo.hitch(this, this._customTooltip),
						position : ["below", "right"]
					});
		
		// Override the tooltip behavior specifically for the tree grid
		dojo.safeMixin(this._tooltip, {
			
			// Override open to set tooltip position over the tree grid
			// near table row
			open : function(target) {
				// Create a rectangle for positioning the tooltip, otherwise default
				// position is outside the DOM node of the tree grid
				var aroundRect = target.rolloverTargetRect;
		
				// make sure the rolloverTargetRect has been set to actually display the tooltip
				// (it may not be if we got a rollover while the context menu is displayed)
				if (aroundRect != null) {
					// replace the target node, the tree grid DOM node, with this rectangle
					arguments[0] = aroundRect;
					this.inherited(arguments);
				}
			},
			_onTargetMouseEnter : function(e) {
				// ignore since the tree grid DOM node is not the real
				// target (we handle the tooltip)
			},
			_onTargetMouseLeave : function(e) {
				// ignore since the tree grid DOM node is not the real
				// target (we handle the tooltip)
			},
			_onTargetBlur : function(e) {
				// ignore since the tree grid DOM node is not the real
				// target (we handle the tooltip)
			},
			_onTargetFocus : function(e) {
				// ignore since the tree grid DOM node is not the real
				// target (we handle the tooltip)
			},
			// added helper method to trigger the timer to display the tooltip
			hover : function(target) {
				this._onHover( {
					target : target
				});
			},
			// added helper method to dismiss the tooltip
			unHover : function(target) {
				this._onUnHover( {
					target : target
				});
			}
		});
		this._tooltip.startup();

		dojo.connect( this.grid, "onCellMouseOver", this, "_onCellMouseOver");
		dojo.connect( this.grid, "onCellMouseOut", this, "_onCellMouseOut");
		dojo.connect( this, "onTableLoad", this, "notifyLoadComplete");	  	
		
	},
	
	notifyLoadComplete: function(){
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		
		this.viewLoaded = true;
		
  		//Send notification we're done loading the dataset/prepping the view now that the table has loaded
  		var parentPluggableView = this.tableView;
  		parentPluggableView._onSetDatasetLoadComplete(parentPluggableView.dataset);
		
	},
	
	resize : function() {
		// summary:
		//		Handle special resize case for IE8
		// description:
		//		Resize events that can occur on IE8 while processing the CURI REST
		//      callback to populate the context menu can sometimes interfere with
		//      the menu rendering the returned new task items.  (The resize event
		//      processing is dispatched after the menu has been repopulated but
		//      before it can actually render to the screen in IE8.  The result is
		//      the menu can be dismissed before it renders correctly.)  This
		//      workaround delays the processing of the resize event slightly thus
		//      allowing the menu to render.

        if (dojo.isIE) {
        	if (this.tableView && this.tableView.isMenuOpen() && (this._resizing < 1)) {
        		this._resizing++;
        		setTimeout(dojo.hitch(this, this._doResize), 0);
            }
            else {
            	this.inherited(arguments);
            }
        } 
        else {
        	this.inherited(arguments);
        }
    },

    _doResize: function() {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon

    	this.resize();
    	this._resizing--;
    },
	
	_customTooltip: function(/*Object*/ target) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		// target:
		//		Coming soon

		if (this._rolloverTarget) {
			// target is this.grid.domNode, so we use this._rolloverTarget
			var content = "";
			if (typeof this._rolloverTarget == "string") {
				content = content + this._rolloverTarget;
			} else {
				content = content + this._rolloverTarget.getLabel();
			}

			// first check if a server side cell property tooltip is defined, use that instead of other defaults
			if(this._rolloverCellTarget && this._rolloverCellTarget.getTooltip){
				var tooltip = this._rolloverCellTarget.getTooltip();
				if(tooltip){
					return tooltip;
				}
			}
			
			//if the server side tooltip content is set, use this instead of defaults.
			if(this._rolloverTarget.getTooltip) { //Need first test because getTooltip is not a function on child datasets right now
				var tooltip = this._rolloverTarget.getTooltip();
				if (tooltip){ 
					return tooltip;
				}
			}

			parentPluggableView = this.tableView;
			
			if(this._tooltipProperties){
				return this._tooltipProperties.getContent();
			}
			
			//create properties content object
			this._tooltipProperties = new ibm.tivoli.tip.navwidget.properties.dijit.PropertiesNotebookGeneralTab({
				id: this.id + "_tooltipProperties",
				managingPluggableView: parentPluggableView, 
				maxProperties: 3,  //TODO: ADM should probably be something a widgetDef sets.
				_isTooltipProperties: true
			});

			//Call back for when the dataset it set
			var objectType = (this._rolloverTarget.getObjectType) ? this._rolloverTarget.getObjectType() : null;
			var setDatasetComplete = function(dataset) {
				//connect to the onSetObjects Event
				this._propOnSetObjectsConnection = dojo.connect(this._tooltipProperties, "onSetObjects", dojo.hitch(this, this._updateTooltipContent));

				if (objectType === "item"){
					this._tooltipProperties.setObjects([this._rolloverTarget], null, null, null);
				} else if(objectType === "link"){
					this._tooltipProperties.setObjects(null, [this._rolloverTarget], null, null);
				} else if(objectType === "relationship"){
					this._tooltipProperties.setObjects(null, null, [this._rolloverTarget], null);
				}
			};
			this._propOnSetDatasetConnection = dojo.connect(this._tooltipProperties, "onSetDataset", dojo.hitch(this, setDatasetComplete));
			this._tooltipProperties.setDataset(parentPluggableView.dataset);

			return this._createTemporaryContentDiv(content, objectType);
		}
	   },
	   
	   //Event callback method to handle updating the tooltip content with updated content
	   _updateTooltipContent: function(){
		
		this._tooltip.updateContent(this._tooltipProperties.getContent(), this.grid.domNode);
	   },
	   
	   _createTemporaryContentDiv: function(content, objectType) {		   
		   //Background div
		   var backgroundDiv = dojo.doc.createElement("div");
		   var processingPane = new dijit.layout.ContentPane({
			   content: backgroundDiv
		   });
			
			// text div		
		   var textDiv = dojo.create("div",{
			   "innerHTML": content
		   });
		   backgroundDiv.appendChild(textDiv);
		
		   if (objectType) {
			   // processing image div		
			   var processDiv = dojo.create("div",{
					"class": "processingImage"
			   });
			   backgroundDiv.appendChild(processDiv);
		   }

		   //Return
		   return processingPane;
	   },
	   
	   //function to close tooltips
	   closeTooltip: function() {
			if (this._rolloverTarget) {
				dojo.disconnect(this._propOnSetObjectsConnection);
				dojo.disconnect(this._propOnSetDatasetConnection);
				
				// Clear out the reference in the tooltip to the content
				this._tooltip.updateContent("", this.grid.domNode); 
				if (this._tooltipProperties){
					this._tooltipProperties.destroyRecursive();
					this._tooltipProperties = null;
				}
				
				// this._tooltip.close();
				this._tooltip.unHover(this.grid.domNode);
				this._rolloverTarget = null;
				this._rolloverCellTarget = null;
				this.grid.domNode.rolloverTargetRect = null;
		    }
		},

	_onCellMouseOut: function( e ){
		// console.log("mouse rolled out of row");
		this.closeTooltip();
		dojo.stopEvent(e);  
	},
		
	_onCellMouseOver: function( e ){
		// console.log("rolled onto row ");
		
		dojo.stopEvent(e);
		
		// ignore if the menu is open or if we already have a rollover target
		if (!this.tableView.isMenuOpen() && !this._rolloverTarget) {
	    	var aroundRect = {type: "rect"};

	    	this._rolloverTarget = e.grid.getItem(e.rowNode.gridRowIndex);
		if (this._rolloverTarget.hasCachedProperty(e.cell.field)) {
	    	this._rolloverCellTarget = this._rolloverTarget.getProperty(e.cell.field);
		}
		else {
			this._rolloverCellTarget = null;
		}

	    	var boundingInfo = dojo.position(e.cellNode, true);
	    	aroundRect.x = boundingInfo.x; 
	    	aroundRect.y = boundingInfo.y;

	    	aroundRect.width = e.cellNode.clientWidth;
	    	aroundRect.height = e.cellNode.clientHeight;
	    
	    	// adjust relative coordinates to absolute, and remove fractions
	    	var lt = dojo.coords(this.grid.domNode, true);
	    	aroundRect.x = Math.round(aroundRect.x);
	    	aroundRect.y = Math.round(aroundRect.y);
			
	    	this.grid.domNode.rolloverTargetRect = aroundRect;
	    	this._tooltip.hover(this.grid.domNode);		
	    }			
	}
});	
	
