dojo.provide("ibm.tivoli.tip.richhover.HoverListener");

/* 
 * The HoverListener will listen for a onMouseOver event and will provide a short delay before
 * taking any action.
 */
dojo.declare("ibm.tivoli.tip.richhover.HoverListener", null, {
	_callback : null,
	//_callbackScope : null,
	_overConnectionHandle : null,
	_outConnectionHandle : null,
	_onMouseoutCallback : null,
	_param : null,
	_delay : 1000,
	_hoverTimer : null,
	_pdCallback : null,
	_pdParam : null,

	/*
	 * This callback is used to call the actual client callback.  We use this to also
	 * clear the hover timer handle.
	 */
	_hoverCallback: function()
	{
	    console.log("hovercallback invoked");
		this._hoverTimer = null;
		//(this._callbackScope[this._callback])(this._param);
		this._callback(this._param);
	},
	
	/*
	 * Called when an onMouseOver event occurs, and this method provides a delay.
	 */
	_mouseOverDelay: function()
	{
		console.log("mouseOverDelay called delay="+this._delay);
		if (this._pdCallback) {
			this._pdCallback(this._pdParam);
			//(this._callbackScope[this._pdCallback])(this._pdParam);
		}
		// This will wait for some specified time before dataset is prepared.

		this._hoverTimer = setTimeout(dojo.hitch(this,this._hoverCallback), this._delay);
	},
	
	/*
	 * If the user moves the mouse off of the element, this will clear the timer so
	 * the action does not occur.
	 */
	_mouseOutCtrl: function()
	{
		console.log("mouseOutCtrl called");
		if (typeof this._hoverTimer != "undefined" && this._hoverTimer != null)
		{
			clearTimeout(this._hoverTimer);
		}
		this._hoverTimer = null;
		if (typeof this._onMouseoutCallback != "undefined" && this._onMouseoutCallback != null) {
			this._onMouseoutCallback();
		}
	},
	
	/* Sets the hover listener 
	 *   elem - dom element to set the listener on
	 *   callback - to be called when there is a mouse hover
	 *   param - an optional parameter to be included in the callback
	 *   delay - milliseconds the mouse must hover for the callback to be called (default=1000)
	 *   pdCallback - predelay callback is called when onMouseOver happens, but before the delay
	 *                scope from the scope parameter is used.
	 *   pdParam - parameters passed to predelay callback
	 * */
	setListener : function(elem, callback, param, onMouseoutCallback, delay, pdCallback, pdParam) {
		this._param = param;
		this._callback = callback;
		if (typeof delay != "undefined" && delay != null) {
			this._delay = delay;
		}
		if (typeof pdCallback != "undefined" && pdCallback != null) {
			this._pdCallback = pdCallback;
		}
		if (typeof pdParam != "undefined" && pdParam != null) {
			this._pdParam = pdParam;
		}
		if (typeof onMouseoutCallback != "undefined" && onMouseoutCallback != null) {
			this._onMouseoutCallback = onMouseoutCallback;
		}
		
		console.log("onMouseOver setup");
		this._overConnectionHandle = dojo.connect(elem, 'onmouseenter', this, "_mouseOverDelay");
		this._outConnectionHandle = dojo.connect(elem, 'onmouseleave', this, "_mouseOutCtrl");

	},
	
	/**
	 * Frees resources used by the hover listener.
	 */
	clearListener : function() {
		this._mouseOutCtrl();     /* clear hover timer */
		
		if (typeof this._overConnectionHandle != "undefined" && this._overConnectionHandle != null)
			dojo.disconnect(this._overConnectionHandle);
		if (typeof this._outConnectionHandle != "undefined" && this._outConnectionHandle != null)
			dojo.disconnect(this._outConnectionHandle);
		
		this._outConnectionHandle = null;
		this._overConnectionHandle = null;
	},
	
	/*
	 * This should be called whenever you no longer need this hover listener.  
	 */
	destroy : function() {
		this.clearListener();
	}
});
