/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* Tivoli Extensions for Dojo
*
* (C) Copyright IBM Corp. 2011.  All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} **/
dojo.provide("ibm.tivoli.tip.richhover.MessageDialog");

dojo.require("dijit._Widget");
dojo.require("dijit.Dialog"); 
dojo.require("dijit.form.CheckBox");
dojo.require("dijit.form.Button");
dojo.require("dojo.i18n");
dojo.require("dojox.layout.ResizeHandle");

//Localization of resource bundles
dojo.requireLocalization("ibm.tivoli.tip.richhover", "RichHover", null, "ROOT,ar,cs,da,de,es,fi,fr,he,hr,hu,it,ja,ko,nb,nl,pl,pt-br,ru,sl,sv,tr,zh,zh-tw");


dojo.declare("ibm.tivoli.tip.richhover.MessageDialog",[ dijit._Widget ],
{
		// summary:  MessageDialog is a widget to show message dialogs of type error,warning,info,confirm. 
		// description: MessageDialog widget uses the dijit.Dialog for its implementation.
	
	//theme: String
	// defined the theme for Message Dialog
	theme:null,	
	
	//resources:object
	//Reference to the resources
	 resources_:null,
	 
    // resizeAxis: String
    //  x | xy | y to limit pane's sizing direction.
    resizeAxis: "xy",

	// type: String
	title: "",
	
	content:null,

	// message: String
	//the displayable translated message
	message: null,

	// messageId: String
	//the displayable translated message's unique identifier
	messageId: null,
	
	_resourceUrl:null,
	// width: String
	//gives the pixel width of the dialog (Default: 500px)
	width: "380px",
	
	// height: String
	//gives the pixel height of the dialog (Default: 200px)
	height: "225px",
	
	// callback: Function
	//a callback function when the dialog is closed
	callback: null,
	
	// parameters to pass on callback when dialog closes
	callbackParms: null,
	
	// url: String
	//a URL that links the message to an InfoCenter page
	url: null,

	// contentIsolation: boolean parameter to open PopupUtil in content isolation mode in portlets
	contentIsolation: false,
	
	// Destroy the dialog on close
	destroyOnClose: true,
	
	// namespace: user defined namespace for widget
	namespace :"",
	
	_footerHeight : 29,
	
	// tells if the locale is RTL.
	dir : "",

	constructor: function() {
		// summary: An extension to the Dojo Dialog widget that supports 
		//      advanced features.
		// resourceMessages_: JSON
		//		Resource bundle.
		// dialog_: dojo.widget.FloatingPane
		//		An instance of the Dialog widget.
		// title_: String
		//		Title bar string.
		// icon_: String
		//		The dialog icon's uri.
		// buttonPressedId_: String
		//		The ID of the button pressed; this value is passed to the callback function.

	    this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.richhover", "RichHover",this.lang);
		
		if(dojo.locale=="iw" || dojo.locale=="he"|| dojo.locale.indexOf("ar")>-1)
			this.dir= "rtl";
	    
	    this.content = null;
	    this.title = null;
	    
	},


	postCreate: function() { 
		// summary:  Creates and initialises the message Dialog.
		// tags: public
    
		var div = document.createElement('div'); 
		document.body.appendChild(div);
		var theme=this.theme;
		var dir = this.dir;
        this.dialog_ = new dijit.Dialog (
			                             { title: this.title,
										   dir: dir,
										   content: this.content
		                                 },
		                                 div); 

		var dojoAttachpoint = this.dialog_.containerNode;
		this.dialog_.closeButtonNode.setAttribute("title", this.resources_.DIALOG_BUTTON_CLOSE);
		this.dialog_.closeButtonNode.setAttribute("tabIndex", 0);
		dojo.connect(this.dialog_.closeButtonNode,'onkeypress',this,'onKeyPress');
		dojo.style(div,'class',theme);
		var moreInfo = dojo.create("a", {href: "javascript:void(0)", align:"left", title: this.resources_.MORE_INFO, id: "moreInfoMsg_dialog", onclick: dojo.hitch(this, this.onClickMore), innerHTML: this.resources_.MORE_INFO});
		var moreInfoDiv = dojo.create("div",{id:"moreInfoDiv", style:{"float":"left",padding:"5px"}});
		moreInfoDiv.appendChild(moreInfo);
		
		if(this.dir == "rtl") // For RTL locales footer got truncated.
			var buttonDiv = dojo.create("div",{id:"dialog_footer", style:{width:"100%",position:"absolute", bottom:"11px", padding:"2px 5px 12px 7px"}});
		else
			var buttonDiv = dojo.create("div",{id:"dialog_footer", style:{width:"100%",position:"absolute",bottom:"0px", padding:"2px 5px 12px 7px"}});
			
		buttonDiv.className = "dijitDialogPaneActionBar";
		var buttonSpan = dojo.create("span", {style:{position: "relative", right: "2em", "float":"right"}});
		var footerButton = dojo.create("button",{id:"dialog_footer_button", innerHTML:this.resources_.DIALOG_BUTTON_OK});
		dojo.connect(footerButton, 'onclick', this, 'onClick');   
		buttonSpan.appendChild(footerButton);
		
		buttonDiv.appendChild(moreInfoDiv);
		dojo.place( buttonSpan, buttonDiv, "right" );
		dojo.place( buttonDiv, this.dialog_.domNode, "last" );
		
		// Place the resize handle for the dialog
        var resizeHandleDOM = dojo.doc.createElement( 'span' );
        resizeHandleDOM.className = "dojoxFloatingResizeHandle";    
        dojo.place( resizeHandleDOM, this.dialog_.domNode, "last" );
        this.resizeHandle = new dojox.layout.ResizeHandle(
        { 
              targetId: this.dialog_.id, 
            resizeAxis: this.resizeAxis 
        }, resizeHandleDOM );   
	
        this.dialog_.domNode.style.width = this.width;	
        this.dialog_.domNode.style.height = this.height;	
        this.dialog_.domNode.style.overflow = "hidden";        
        this.onCloseListener  = dojo.connect( this.dialog_, "hide", this, "_onHide" );
	this.onResizeListener = dojo.connect( this.dialog_, "resize", this, "_fixLayout" );
		
	},
	
    /**************************************************************************
    * The dialog has been hidden.  Wait 500 miliseconds to destroy the dialog.
    *
    * HACK -- Can't figure out a better way to destroy the dialog on close.
    **************************************************************************/
    _onHide:function()
    {// summary: Closes the MessageDialog .
		// tags: private
      setTimeout( dojo.hitch( this, "destroy" ), 500 );
    },  
	
	onKeyPress: function (e) {
		if (e.keyCode == 13 || e.keyCode == 32 || e.keyCode == 0) { //  Space keycode is 32 on IE8 and 0 on mozilla.  Enter keycode is 13 for both. 
			this.dialog_.hide();
		}
	},
	
	onClickMore: function()
	{
		window.open(this._resourceUrl);
	},
	
	onClick: function(/* HTMLEvent */e) {
    	// summary:  Calls the callback on click of Ok else destroys.
		// tags: public
		//this.buttonPressedId_ = parseInt(e.target.id.substring(index), 0);
	    this.dialog_.hide();
	    
	    if (this.callback) {
	    	this.callback("dialog_footer_button", this.messageId, this.callbackParms);
	    }
    },

	show: function() {
    	// summary:  Displays the messageDialog.
		// tags: public
	    if( this.contentIsolation == true ){
			// Get the namespace identifying the content that launched the context menu
			console.info("Inside Show if PopupUtil Undefined");
			var namespace="";
			if(this.namespace != null)
			 namespace = this.namespace;
			
			//Close (just in case) and then open the popup covering the whole page

			var xcord=100;
			var ycord=100;
			var dialogNamespace=namespace+'ibm.tivoli.tip.richhover.MessageDialog';
			var old_popup_iframe = PopupUtil.ClosePopup( namespace+'ibm.tivoli.tip.richhover.MessageDialog' );
			//Changes for fixing Content Isolation issue with Claro theme
			//var theme = dojo.body().className;
			var theme=this.theme;
			this.baseUrl=dojo.baseUrl;
			
			var index = this.baseUrl.lastIndexOf("dojo/dojo/");
			
			if (index != -1) {
				this.baseUrl=this.baseUrl.substring(0,index);
			
			}
			var popupUrl =this.baseUrl+'ibm/tivoli/tip/richhover/TIPMessageDialogPopUp.html?PopupID='+dialogNamespace+'&eLeft='+xcord+'&eTop='+ycord+'&locale='+djConfig.locale+"&theme="+theme;
			console.info("url :"+popupUrl);
			var popup_iframe = PopupUtil.OpenPopup( 
				namespace, 
				namespace+'ibm.tivoli.tip.richhover.MessageDialog', 
				true, 
				null, 
				null, 
				null, 
				null, 
				popupUrl, 
				null
			);
			console.info("after popup opening code");
			//Initialize the contextMenu when the iframe popup is finished loading
			var tThis = this;
			if( popup_iframe.addEventListener ){
				popup_iframe.addEventListener(
					"load", 
					function(){
						popup_iframe.contentWindow.makeMessageDialog( tThis );
					}, 
					false
				);
			}
			else if (popup_iframe.attachEvent){
				popup_iframe.attachEvent(
					"onload", 
					function(){ 
						popup_iframe.contentWindow.makeMessageDialog( tThis);
					}
				);
			}
		}
		else{
			this.dialog_.show();
			this._fixLayout();
        }
	},
	
	
	_fixLayout: function() { // This fn adjusts the IFrame and its parent div height
		var clientHeight = this.dialog_.titleBar.clientHeight;
		var footerHeight = (document.getElementById('dialog_footer')).clientHeight;
		var domHeight = (this.dialog_.domNode).clientHeight;
		if(this.dir=="rtl") {
			footerHeight = this._footerHeight;
			var footer = dojo.byId("dialog_footer");
			footer.style.height = this._footerHeight + "px";
		}
		var iFrameHeight;
		
		if(this.dir == "rtl") { // this fixes the issue with rtl locales.
			iFrameHeight= domHeight - footerHeight - clientHeight - 10;
			if(dojo.isIE < 8)
				iFrameHeight= domHeight - footerHeight - clientHeight - 32;
			else if(dojo.isIE >= 8)
				iFrameHeight= domHeight - footerHeight - clientHeight - 9;
		}
		else {
			if(dojo.isIE < 8)
				iFrameHeight= domHeight - footerHeight - clientHeight - 16;
			else if(dojo.isIE >= 8)
				iFrameHeight= domHeight - footerHeight - clientHeight + 7;
			else // if its not IE
				iFrameHeight= domHeight - footerHeight - clientHeight + 7;
		}
		var largeHover = document.getElementById('largeHover');
		largeHover.style.height = iFrameHeight + "px";
		largeHover.parentNode.style.height = iFrameHeight + "px";
		
	},
	
	destroy: function () {
		// summary:  destroys the message Dialog.
		// tags: public
        dojo.disconnect( this.onCloseListener );
        dojo.disconnect( this.onResizeListener );
               
        if ( this.dialog_ ) { this.dialog_.destroy(); dijit.registry.remove (this.dialog_.id); delete this.dialog_; }
        
		delete this.type;
		delete this.message;
		delete this.messageId;
		delete this.width;
		delete this.height;
		delete this.callbackParms;
		delete this.url;
		delete this.destroyOnClose;
		delete this.title_;
		delete this.icon_;
		delete this.resources_;

		dijit.registry.remove (this.id);
	}

});

