/* NOTICE
 * This dataset is an override/hack to the basic Dataset.js that is provided in CURI.
 * The reason for not just using the base dataset is to reduce the number of requests to 
 * the server down to only one request instead of two.  The base Dataset will always create
 * and initial request to the server to get information about the dataset itself.  Since we
 * want the hover to be as fast as possible, I only want one request.  This code should be
 * revisited when productization happens to make sure it's done more robustly.  The usage of
 * this dataset (in RichHoverDialog.js) can easily be replace with using the Dataset.js because
 * the APIs are the same.
 */

/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.richhover.RichHoverDataset"); 
 
// includes
dojo.require("ibm.tivoli.navmodel.Dataset");

////////////////////////////////////
//RichHoverDataset
//
//NOTE: do not call any abstract methods directly;
//call the public version of the methods defined in
//AbstractDataset instead and it will in turn call
//the abstract overrides.
////////////////////////////////////
dojo.declare("ibm.tivoli.tip.richhover.RichHoverDataset", ibm.tivoli.navmodel.Dataset, {
	_oneTimeSetup : false,
	_createEventRouterFlag : false,
	
	/*=====
	 constructor_kwArgs: function(onComplete, onError, baseUrl, provider, datasource, dataset, 
	 					parameters, defaultProperties, defaultLinkProperties, defaultRelProperties, 
	 					cacheThreshold, cacheValidationInterval, cacheExpirationTime, version) {
		// onComplete: constructor_onComplete
	 	//		callback to be invoked when the Dataset is initialized and ready for methods
	 	//		to be invoked  (i.e. asynchronous initialization must be performed). 
	 	//		Note: A null value will result in synchronous initialization.
	 	// onError: constructor_onError
	 	//		callback to be invoked if there is a problem constructing the Dataset
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// dataset: String
		//		dataset id
		// parameters: Object
		//		optional: extra parameters sent to dataset provider
		// defaultProperties: String[]
		//		optional: item property ids to fetch by default (can specify "all")
		// defaultLinkProperties: String[]
		//		optional: link property ids to fetch by default (can specify "all")
		// defaultRelProperties: String[]
		//		optional: relationship property ids to fetch by default (can specify "all")
		// cacheThreshold: int[]
		//		optional: cache objects if number of dataset items >= threshold
		// cacheValidationInterval: int[]
		//		optional: number of minutes in between cache validations
		// cacheExpirationTime: int[]
		//		optional: number of minutes objects can stay in cache while inactive
		// version: Float
		//		optional: version used to communicate with rest service (i.e. json version)
	},
	=====*/

	constructor: function(/*constructor_kwArgs_kwArgs*/ kwArgs, dataset) {	
		// summary: 
		//		Constructs Dataset object.
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception		
	},

	//////////////////////////////////////////////////////
	// abstract method overrides
	//
	// NOTE: do not call any abstract methods directly;
	// call the public version of the methods defined in
	// AbstractDataset instead and it will in turn call
	// the abstract overrides.
	//////////////////////////////////////////////////////

	/*=====
	initDataset_kwArgs: function(createEventRouter, optimize) {
		// createEventRouter: Boolean
		//		boolean indicating whether new event router should be created
		// optimize: Boolean
		//		optimize json that is returned; remove items that can be calculated
		//		or can be retrieved via additional urls
	},
	=====*/
	
	_initDataset: function(/*initDataset_kwArgs*/ kwArgs) {
		// summary:
		//		Initialize internal dataset object
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
console.log("_initDataset called for RichHoverdataset.js");		
		// start out with empty dataset
		this._initEmptyDataset();		
		this._optimize = kwArgs.optimize;
		
		// make synchronous call if no onComplete callback
		var synchronous = (kwArgs.onComplete ? false : true);
		var args = arguments;
		var exception = null;
		
		// dataset was NOT provided; fetch it
		if (!kwArgs.dataset) {			
			var content = ibm.tivoli.navmodel.Utils.getContent();
			this._mixinStandard(content, kwArgs);			
			ibm.tivoli.navmodel.Utils.mixinParams(content, this);
			
			if (false){
			dojo.xhrGet({
				url: this._getDatasetURL(),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "dataset")
				},
				handleAs: "json",
				sync: synchronous,
				content: content,
				load: dojo.hitch(this, function(result, ioArgs) {
					// pull version from the Content-Type response header
					this._setVersion(this._extractContentTypeVersion(ioArgs));
					
					// mixin all dataset attributes
					this.mixinAttributes(result);
					
					// listen for asynchronous events
					if (kwArgs.createEventRouter && this.isEventsEnabled()) {
						this._createEventRouter();
					}
		
					// call base class
					this.inherited(args);
					// call onComplete callback
					if (kwArgs.onComplete) {
						kwArgs.onComplete(this, kwArgs);
					}							
				}),
				error: dojo.hitch(this, function(error, ioArgs){
					exception = this._getException(error, ioArgs, "_initDataset");					
					if (kwArgs.onError) {
						kwArgs.onError(error, exception, ioArgs);
					}						
				})
			});	
			} else {
			console.log("URL = " + this._getDatasetURL());
				this._createEventRouterFlag = kwArgs.createEventRouter;
				// call onComplete callback
				if (kwArgs.onComplete) {
					kwArgs.onComplete(this, kwArgs);
				}							

			}
			
			if (synchronous && exception) {
				throw exception;
			}						
		}
		
		// dataset was provided
		else {
			this.mixinAttributes(kwArgs.dataset);
			
			// listen for asynchronous events
			if (kwArgs.createEventRouter && this.isEventsEnabled()) {
				this._createEventRouter();
			}
		
			// call base class
			this.inherited(args);			
		}
	},
	
	/*=====
	fetchItems_kwArgs: function(onComplete, onError, recursive, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	_fetchItems: function(/*fetchItems_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch items from rest service

		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		return dojo.xhrGet({
			url: this._getItemsURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemViewList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				
				if (this._oneTimeSetup == false) {
				  this._oneTimeSetup = true;
				  // pull version from the Content-Type response header
				  this._setVersion(this._extractContentTypeVersion(ioArgs));
				
				  // listen for asynchronous events
				  if (this._createEventRouterFlag && this.isEventsEnabled()) {
					  this._createEventRouter();
				  }
				}
				
				var items = [];				
				if (result.items) {
					var expiration = this.getNewExpirationTime();
					var allProps = this._isAllProps(kwArgs.properties);
					var propIds = this._getPropIds(kwArgs.properties, allProps);
					var itemsLength = result.items.length;
					for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
						var item = result.items[itemIndex];
						if (item.datasetId && item.datasetId !== this.getId()) {
							var dataset = this.getDescendentDataset(item.datasetId);
							items[itemIndex] = new ibm.tivoli.navmodel.Item({item: item, dataset: dataset, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});							
						}
						else {
							items[itemIndex] = new ibm.tivoli.navmodel.Item({item: item, dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
						}
					}
				}
				
				// make sure our count is accurate
				console.log("Total Rows : " + result.totalRows);
				console.debug(result);
				this._setItemCount(result.totalRows);
					
				// place counts in the arguments map
				kwArgs.numRows = result.numRows;
				kwArgs.filteredRows = result.filteredRows;
				kwArgs.totalRows = result.totalRows;
				kwArgs.onComplete(items, kwArgs);	
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchItems"), ioArgs);
			})
		});
	},

	
	/*=====
	getItems_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getItems: function(/*getItems_kwArgs*/ kwArgs) {
console.log("Inside getItems");		
		// summary:
		//		Returns items associated with the dataset
		var args = {                
			onComplete: function(items, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			start: 0,
			count: null,
			properties: this._args.defaultProperties,
			metadata: false, 
			sort: null,
			condition: null,
			sync: false
		};   
		dojo.mixin(args, kwArgs);         
		
		///////////////////////////////////////////////////////////
		// make sure we always return asynchronously in case we are 
		// being invoked by a dojo datastore
		///////////////////////////////////////////////////////////
		
		// are NOT any items
		//if (this.getItemCount() === 0) {		
		if (false) {		
			args.numRows = 0;
			args.filteredRows = 0;
			args.totalRows = 0;				
			if (args.sync) {
				args.onComplete(null, args);
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, null, args), 0);
			}			
		}
		// fetch items from rest service
		else {
			var onComplete = args.onComplete;		
			args.onComplete = dojo.hitch(this, function(items, args) {
				// cache items
				items = this._cacheItems(items);
				
				// call orignal callback
				onComplete(items, args);
			});
			this._fetchItems(args);
		}		
	}

	
});