dojo.provide("ibm.tivoli.tip.richhover.RichHoverUrlInfo");

/* 
 *  The RichHoverUrlInfo object is used to query/provide information for the rich hover.
 *  It provides capability to preload information as needed.
 */
dojo.declare("ibm.tivoli.tip.richhover.RichHoverUrlInfo", null, {
	
	curiUrl : null,       /* URL to the curi service to use */
	urls : [],            /* An array of URL strings to query for compact XML */
    timeout : 5000,       /* Timeout for connection to each provider in miliseconds */
	namespaces : null,    /* Namespaces specified as 'shortname' : 'namespace'.  Example: {"rdf":"http://www.w3.org/1999/02/22-rdf-syntax-ns#","tivoli":"http://ism.oslc.elements/1.0"} */
	predicates : null,    /* Predicate names scoped by namespace.  Example {"monitoredAs":"tivoli","assetInfo":"rdf"} */
	cmsparms : null,      /* Object containing cms query information.  See CMS rest info for formatting. Example: {"rcfs":[{"name":"StreetAddress","value":"3901+S+Miami+Blvd,+Durham,+North+Carolina+27703","isNamingAttribute":false}],"launchTypes":["WEB_URL"]}} */
	hasProviders : false, /* Indicates the resquery should be used to find resource URLs using the 'urls' property.  This should be true when 'urls' contains resource registry urls */
	resquery : ["PREFIX im: <http://tivoli-OSLC-prototypes/ns/im#> PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#> PREFIX oslc: <http://open-services.net/ns/core#> PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>SELECT ?url WHERE { ?x im:providerRecord ?rec . ?rec im:sourceRecord ?url}"],
	delay:1000,
	myNodeId:null,
	_timer:null,
	_previews : null,   /* Array of preview data */
	_id : null,
	_callback : null,
	_callbackParm : null,
	_isLoading : false,
	_isLoaded : false,
	_hasError : false,
	
	_providerName : "uberhover",
	_datasourceName : "any",
	_datasetName : "hoverinfo",
	_datasetProperties : ["all"],
	_dataset : null,
	
	constructor: function(args)
	{
	  dojo.mixin(this,args);
	  
		
	  if (this.urls == null)
		  this.urls = [];

	},

	/**
	 * This function is called to load the data.  This call will result in REST requests going to 
	 * query the OSLC providers for their compact XML data.
	 * 
	 * callback - called when the load is complete.
	 * param - the parameter to pass on the callback.
	 */
	load: function(callback, param)
	{
		this._previews = null;   /* Null out previews in case this was used before */
		this._callback = callback;
		this._callbackParm = param;
		
		var urlinfo = {};
		urlinfo.urls = this.urls;
		urlinfo.timeout = this.timeout;

		if (typeof this.predicates != "undefined" && this.predicates != null)
		{
			urlinfo.predicates = this.predicates;
		}
		
		if (typeof this.namespaces != "undefined" && this.namespaces != null)
		{
			urlinfo.namespaces = this.namespaces;
		}
		
		if (typeof this.cmsparms != "undefined" && this.cmsparms != null)
		{
			urlinfo.cms = this.cmsparms;
		}
		
		if (this.hasProviders)
		{
			urlinfo.resquery = this.resquery;
		}

		console.log("creating dataset");
		try
		{
	    var dataSet = new ibm.tivoli.tip.richhover.RichHoverDataset({
	          baseUrl: this.curiUrl,
	          provider: this._providerName,
	          datasource: this._datasourceName,
	          dataset: this._datasetName,
	          defaultProperties: this._datasetProperties,
	          createEventRouter: false,
	          parameters: {urlinfo: dojo.toJson(urlinfo)},
	          onComplete: dojo.hitch(this, this._datasetReady ),
	          onError: dojo.hitch(this, this._datasetError )
	        });
		}
		catch(e)
		{
			console.log("got an exception")
			console.dir(e);
		}
	    
	    this._isLoading = true;
	    this._dataset = dataSet;
		console.log("exit load call");
		console.debug(dataSet);
	},
	
	/**
	 * This method can be used instead of the load method to actually load the preview information
	 * directly.  This allows you to specify the url that points to the html that will be filled
	 * into the preview pane.  That is, no negotiation for OSLC compact xml will take place.
	 * The input has a specific format so that you can specify all parts of the preview.
	 * The following shows the input object.  It should contain a smallPreview, and optionally
	 * a largePreview.
	 * 
	 * ** NOTE: In current prototype, the largePreview is not supported!!!
	 * 
	 * {
	 * 	smallPreview:{
	 * 					url: "http://foo.com/small/preview/html",
	 * 					shortTitle: "Tab Label",
	 * 					resourceUrl: "http://foo.com/LIC/to/resource",
	 * 					heighthint: "200px",                             * Optional                     
	 * 					widthhint: "200px"                               * Optional
	 * 				},
	 * 	largePreview:{                                                   * Optional
	 * 					url: "http://foo.com/small/preview/html",
	 * 					shortTitle: "Tab Label",
	 * 					resourceUrl: "http://foo.com/LIC/to/resource",
	 * 					heighthint: "200px",                             * Optional                     
	 * 					widthhint: "200px"                               * Optional
	 * 				}
	 * }
	 * 
	 */
	addPreview: function(preview)
	{
		console.log("ADD_PREVIEW");
		console.debug(preview);
		if (typeof this._previews == "undefined" || this._previews == null)
			this._previews = [];
		
		
		if (preview.shortTitle != null && preview.resourceUrl != null)
		{
			var result = {};
			result.shortTitle = preview.shortTitle;
			result.title = preview.title;
			result.resourceUrl=preview.resourceUrl;
			result.resourceType=preview.resourceType;

			result.smallPreview = {};
			result.smallPreview.url = preview.smallPreview.url;
			result.smallPreview.heighthint = preview.smallPreview.heighthint;
			result.smallPreview.widthhint = preview.smallPreview.widthhint;

			result.largePreview = {};
			result.largePreview.url = preview.largePreview.url;
			result.largePreview.heighthint = preview.largePreview.heighthint;
			result.largePreview.widthhint = preview.largePreview.widthhint;
			
			this._previews.push(result);
		}

		this._isLoaded = true;  /* Set this true since this is essentially the same as loading */
	},
	
	/**
	 * Returns an array of preview information.  A null is returned if the information has not yet been
	 * loaded either because 'load' has not been called, or 'load' as been called and has not completed yet.
	 * An empty array may be returned if the data was loaded, but there was no data.
	 */
	getPreviews: function()
	{
		if (typeof this._previews == "undefined")
			this._previews = null;
		
		return this._previews;
	},
	
	/**
	 * Returns true if load was called, and completed successfully or in error.
	 */
	isLoaded: function()
	{
		return this._isLoaded;
	},
	
	/**
	 * Returns true if load has been called, but has not yet completed.
	 */
	isLoading: function()
	{
		return this._isLoading;
	},
	
	/**
	 * Returns true if load has been called and has completed in error.
	 */
	hasError: function()
	{
		return this._hasError;
	},
	
	/**
	 * Calling this will destroy the internal CURI dataset.  It results in the 
	 * cache down on the server being destroyed too.  So use this when you want
	 * to kill the cache.
	 */
	destroy: function()
	{
		/* We could destroy the dataset here, but if we do, it removes the cache on the server */
		if (typeof this._dataset != "undefined" && this._dataset != null)
		{
			this._dataset.destroy();
			this._dataset = null;
		}
		
	},
	
	_getItemValue: function(item, name)
	{
		var result = null;
		if (item)
		{
			if (name)
			{
				var prop = item.getProperty(name);
				if (prop)
				{
					result = prop.getValue();
				}
			}
		}
		return result;
	},
	  
	_getPreview: function(item)
	{
		console.debug("_getPreview : ", item);
		var result = null;
		  
		var shortTitle = this._getItemValue(item,"uber.col.shorttitle");
		var title = this._getItemValue(item,"uber.col.title");
		var resourceUrl = this._getItemValue(item,"uber.col.resourceurl");
		var resourceType = this._getItemValue(item,"uber.col.resourcetype");
		
		// small preview details.
		var smallpreviewUrl = this._getItemValue(item,"small.col.htmlurl");
		var smallPreHeight = this._getItemValue(item,"small.col.hintheight");
		var smallPreWidth = this._getItemValue(item,"small.col.hintwidth");
		  
		// large preview details
		var largepreviewUrl = this._getItemValue(item,"large.col.htmlurl");
		var largePreHeight = this._getItemValue(item,"large.col.hintheight");
		var largePreWidth = this._getItemValue(item,"large.col.hintwidth");

		if (shortTitle != null && resourceUrl != null)
		{
			result = {};
			result.shortTitle = shortTitle;
			result.title = title;
			result.resourceUrl=resourceUrl;
			result.resourceType=resourceType;

			result.smallPreview = {};
			result.smallPreview.url = smallpreviewUrl;
			result.smallPreview.heighthint = smallPreHeight;
			result.smallPreview.widthhint = smallPreWidth;

			result.largePreview = {};
			result.largePreview.url = largepreviewUrl;
			result.largePreview.heighthint = largePreHeight;
			result.largePreview.widthhint = largePreWidth;
		}
		console.debug("result : ", result);
		return result;
	},
	
	_getLargePreview: function(item)
	{
		var result = null;
		var type = this._getItemValue(item,"uber.col.type");
		  
		//if (type != null && type == "largePreview")
		//{
			var shortTitle = this._getItemValue(item,"large.col.shorttitle"); // common
			var resourceUrl = this._getItemValue(item,"uber.col.resourceurl");
			var previewUrl = this._getItemValue(item,"large.col.htmlurl");
			var height = this._getItemValue(item,"large.col.hintheight");
			var width = this._getItemValue(item,"large.col.hintwidth");
			  
			console.log(shortTitle);
			console.log(resourceUrl);
			console.log(previewUrl);
			console.log(height);
			console.log(width);
			if (shortTitle != null && resourceUrl != null && previewUrl != null)
			{
				result = {};
				result.shortTitle = shortTitle;
				result.largePreview = {};
				result.largePreview.url = previewUrl;
				result.resourceUrl = resourceUrl;
				result.heighthint = height;
				result.widthhint = width;
			}
		//}
		console.debug("result : ", result);
		return result;
	},
	
	
	_itemFetchComplete: function(items)
	{
		console.debug("_itemFetchComplete enter", items);
		this._previews=[];
		if (typeof items != "undefined" && items != null)
		{
			for (var index in items)
			{
				var item = items[index];
				console.debug("item : ", item);
				this._id = item.getId();
				var preview = this._getPreview(item);
				console.debug("preview ??? : ", preview);
				console.log(this._previews);
				if (preview != null)
				{
					this._previews.push(preview);
				}
			}
		}
		dojo.connect(dojo.byId(this.myNodeId), 'onmouseleave', this, this._onNodeLeave);
		console.log("waiting for ... " + this.delay+ "ms");
		this._timer=setTimeout(dojo.hitch(this, this._invokeCallback), this.delay);
		//this._invokeCallback();
	},
	  
	_onNodeLeave: function()
	{
		console.log("_onNodeLeave called");
		if (typeof this._timer != "undefined" && this._timer != null)
		{
			clearTimeout(this._timer);
		}
		this._timer = null;
	},
	
	_itemFetchError: function(data)
	{
		console.error("Failed fetching items.");
		this._hasError = true;
		this._invokeCallback();
	},
	
	/**
	 * Actually calls the client callback function.
	 */
	_invokeCallback: function()
	{
		console.log("_invokeCallback enter");
		this._isLoaded = true;
		this._isLoading = false;
		console.debug("this._callback : " + this._callback);
		if (typeof this._callback == "function" && this._callback != null) {
			this._callback(this._callbackParm);
		}
		
	},
	
	/**
	 * This private method is called when the dataset retrieval encountered an error
	 */
	_datasetError: function(dataset)
	{
		console.error("Failed loading the dataset.");
		this._hasError = true;
		this._previews=[];
		this._invokeCallback();
	},
	
	/**
	 * This private method is called when the dataset is returned from the CURI provider.
	 * Once we get this, we then collect the URL info from each of the items.
	 */
	_datasetReady: function(dataset)
	{
		console.log("_datasetReady enter");
		this._dataset = dataset;
		this._previews=[];
		  
		var kwArgs = {};
		kwArgs.onComplete = dojo.hitch(this,this._itemFetchComplete);
		kwArgs.onError = dojo.hitch(this,this._itemFetchError);
		kwArgs.properties = ["all"];
		  
		dataset.getItems(kwArgs);
	}

	
});
