/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

(c) Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.analysis.AnalysisPane");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Menu");
dojo.require("dijit.MenuItem");

dojo.require("com.ibm.security.modeling.dijit.layout.BorderContainer");
dojo.require("com.ibm.security.modeling.dijit.layout.ContentPane");
dojo.require("com.ibm.security.modeling.dijit.layout.TabContainer");
dojo.require("com.ibm.security.modeling.layouts.TemplatedPanelWithLayout");
dojo.require("ibm.tivoli.tip.dijit.TIPButton");
dojo.require("dijit.form.DropDownButton");


dojo.require("com.ibm.security.modeling.widgets.MultiMessageArea");
dojo.require("com.ibm.security.modeling.help.Help");
dojo.require("com.ibm.security.modeling.util.WidgetUtilities");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");

dojo.require("com.ibm.security.modeling.analysis.base._AnalysisPaneUtil");
dojo.require("com.ibm.security.modeling.analysis.base._AnalysisTaskPanelController");
dojo.require("com.ibm.security.modeling.analysis.AnalysisPaneCatalogPanel");

dojo.require("com.ibm.security.modeling.util.Trace");
dojo.requireLocalization("com.ibm.security.modeling.analysis", "AnalysisPaneText");
dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");

/**
 * Main controller templated widget for the Analysis pane UI.
 */
dojo.declare("com.ibm.security.modeling.analysis.AnalysisPane", 
	[com.ibm.security.modeling.layouts.TemplatedPanelWithLayout, com.ibm.security.modeling.analysis.base._AnalysisTaskPanelController], {
	
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.analysis","templates/AnalysisPane.html"),	
	widgetsInTemplate: true,
	
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.analysis.AnalysisPane"),
	
	// _widgetUtilities: com.ibm.security.modeling.util.WidgetUtilities
	//		Utilities for widgets.
	// _widgetUtilities: null,
	
	// _isPaneOpen: boolean
	//		Panel state indicator
	//		true: represents that pane is open
	//		false: represents that pane is closed.
	
	_isPaneOpen: false,
	_defaultHelpFile: "ui_analysis_pane.html",
	
	attributeMap: {
		_titleText: {node: "_titleTextNode", type: "innerHTML"},
		id:"", dir:"", lang:"", "class":"", style:"", title:""
	},
	
	postMixInProperties: function() {
		// Summary:
		//		Widget life-cycle method.
		
		this._trace.entering(this.id, "postMixInProperties");
		
		this.inherited(arguments);
		if (!this.globalResources) {
			this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		}
		if (!this.localResources) {
			this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.analysis", "AnalysisPaneText");
		}
		this._widgetUtilities = new com.ibm.security.modeling.util.WidgetUtilities();
		
		this._tabPosition = "right-h";
		if (this.isLeftToRight()) {
			this._tabPosition = "left-h";
		}
		
		this._trace.exiting(this.id, "postMixInProperties");
	},
	
	postCreate: function() { 
		// Summary:
		//		Widget life-cycle method. 
		
		this._trace.entering(this.id, "postCreate");
		this.inherited(arguments);
		this.subscribe(this.analysisContainer.id+"-selectChild", function(page) {this._onTabSwitch(page);});
		this._setupMessageArea();
		this._setupHelp();
		this._widgetUtilities.disableButton(this._recentViewsButton);
		this._trace.exiting(this.id, "postCreate");
	},
	
	startup: function() {
		// Summary:
		//		Widget life-cycle method.
		//		Starts up the widget
		
		this._trace.entering(this.id, "startup");
		this.inherited(arguments);
		
		this.help.startup();
		
		if (this.analysisContainer.selectedChildWidget) {
			this._defaultTab = this.analysisContainer.selectedChildWidget;
		} else {
			this._defaultTab = this.catalogTab;
		}
		
		this.updatePaneDisplay(/*preferences*/ this._getPreferences());
		dojo.subscribe("LAUNCH_ANALYSISTASK", dojo.hitch(this, this.showAnalysisTask));
		this._trace.exiting(this.id, "startup");
	},
	
	destroy: function() {
		//	Summary:
		//		Widget life-cycle method.
		
		this._trace.enteringWithStack(this.id, "destroy", true);
		this._trace.entering(this.id, "destroy");
		try {
			this.inherited(arguments);
			
			this._trace.exiting(this.id, "destroy");
		} catch(e) {
			this._trace.exception(e, "destroy");
			throw e;
		}
	},
	
	updatePaneDisplay: function(/*Object*/ preferences) {
		// Summary:
		//		Displays the analysis pane per the preferences.
		//		If currently selected tab matched the one per the preferences,
		//			then it its GUI is updated per the latest data.
		//		Else
		//			Tab switch takes place per preferences. 
		
		this._trace.entering(this.id, "updatePaneDisplay");
		
		if (!preferences) {
			return;
		}
		
		var tabId = preferences.tabId;
		if (tabId == this.analysisContainer.selectedChildWidget.id) {
			// Tab already selected.
			// refresh the panel dependent elements.
			if (this._isCatalogTabSelected()) {
				this._updateCatalogTabContents();
			} else {
				this._updateAnalysisTabContents();
			}
		} else {
			this.analysisContainer.selectChild(tabId);
		}
		
		this._trace.exiting(this.id, "updatePaneDisplay");
	},
	
	resetPane: function() {
		// Summary:
		//		Resets this pane widget.
		
		this.reset();
		this._clearCachedData();
		this._widgetUtilities.disableButton(this._recentViewsButton);
		this.messageArea.hideMessages();
		this.updatePaneDisplay(/*preferences*/ this._getPreferences(this._defaultTab.id));
	},
	
	showCatalog: function() {
		// Summary:
		//		Displays the analysis catalog tab in its last state
		
		this._selectCatalogTab();
	},
	
	showAnalysis: function(/*_AnalysisTaskPanelFactory*/ factory) {
		// Summary:
		//		Method connected to AnalysisPaneCatalogPanel::_showAnalysisTask() 
		//
		// factory: com.ibm.security.modeling.analysis.base._AnalysisTaskPanelFactory
		//		Factory to create analysis task panel.
		
		this.messageArea.hideMessages();
		this.showAnalysisTask(factory);
	},
	
	_onTabSwitch: function(page) {
		// Summary:
		//		Callback method subscribed to the analysis tab container's select tab event.
		//
		// page:	string|widget
		
		this._trace.entering(this.id, "_onTabSwitch");
		if (page && (page == this.catalogTab.id || page.id == this.catalogTab.id)) {
			this._updateCatalogTabContents();
		} else if (page && (page == this.analysisTab.id || page.id == this.analysisTab.id)) {
			this._updateAnalysisTabContents();
		}
		this._trace.exiting(this.id, "_onTabSwitch");
	},
	
	_updateCatalogTabContents: function() {
		// Summary:
		//		Updates catalog tab contents per the latest data.
		
		this._trace.entering(this.id, "_updateCatalogTabContents");
		if (!this._catalogPanel) {
			this._createCatalogPanel();
			this._catalogPanel.startup();
		}
		
		this._widgetUtilities.disableButton(this._refreshButton);
		this._updateTitle();
		this._updateHelp();
		this._trace.exiting(this.id, "_updateCatalogTabContents");
	},
	
	_updateAnalysisTabContents: function() {
		// Summary:
		//		Updates analysis tab contents per the latest data.
		
		this._trace.entering(this.id, "_updateAnalysisTabContents");
		
		var analysisTask = this._getAnalysisTask(); // _AnalysisTaskPanelController::_getAnalysisTask()
		var helpFile = null;
		if (analysisTask) {
			if (!analysisTask._started) {
				analysisTask.startup();
			}
			if (this._needsRefresh == true) {
				this._updateAnalysisTask(this._getAnalysisTaskData());
				this._needsRefresh = false;
			}
			this._widgetUtilities.enableButton(this._refreshButton);
			helpFile = analysisTask.getHelpFile();
		} else {
			this._widgetUtilities.disableButton(this._refreshButton);
		}
		
		this._updateTitle();
		this._updateHelp(helpFile);
		this._trace.exiting(this.id, "_updateAnalysisTabContents");
	},
	
	_updateTitle: function() {
		// Summary:
		//		Updates pane's title per the selected tab.
		
		this._trace.entering(this.id, "_updateTitle");
		var titleText = "";
		if (this._isCatalogTabSelected()) {
			titleText = this.localResources.ANALYSIS_PANE_TITLE_ON_CATALOG_TAB;
		} else {
			var analysisTask = this._getAnalysisTask();
			if (analysisTask) {
				titleText = analysisTask.getTitle();
			}
			
			if (!titleText) {
				titleText = this.localResources.ANALYSIS_PANE_TITLE_ON_ANALYSIS_TAB;
			}
		}
		
		this.set("_titleText", titleText);
		this._trace.exiting(this.id, "_updateTitle");
	},
	
	showAnalysisTask: function(/*_AnalysisTaskPanelFactory?*/ arg1, /*Object?*/ arg2) {
		// Summary:
		//		Displays the analysis tab with panel created with the specified panel factory
		//		and sets the 'currentTaskFactory' to the passed in factory passing the contextData
		//		to the panel factory to open the analysis panel with the specified state.
		//	
		//		this method can be invoked in following ways:
		//		-	showAnalysisTask(factory, contextData);
		//		-	showAnalysisTask(factory);
		//		-	showAnalysisTask(null, contextData);
		//		-	showAnalysisTask(contextData);
		//
		//	taskFactory: _AnalysisTaskPanelFactory
		//	contextData: Object
		
		// resolve arguments.
		var taskFactory = null;
		var data = null;
		if (arg1 && arg1.isInstanceOf && arg1.isInstanceOf(com.ibm.security.modeling.analysis.base._AnalysisTaskPanelFactory)) {
			// arg1 is a factory
			taskFactory = arg1;
		} 
		
		if (arg2) {
			// arg2 is data
			data = arg2;
		} else if (arg1 && !arg2 && (!arg1.isInstanceOf || !arg1.isInstanceOf(com.ibm.security.modeling.analysis.base._AnalysisTaskPanelFactory))) {
			// only arg1 is provided, and it is not a factory.
			// i.e user invoked this method passing only Context Data.
			data = arg1;
		} 
		
		this._setContextData(data);
		
		if (this.isPaneOpen()) { // Update the current analysis task.
			this._loadAnalysisTask(taskFactory, this._getAnalysisTaskData(data));
			// Data has been loaded (or refreshed). No need to refresh again.
			this._needsRefresh = false;
			this.updatePaneDisplay(/*preferences*/ this._getPreferences(/*desiredTabId*/ this.analysisTab.id));
		} 
	},
	
	show: function(/*Object?*/ contextData, /*boolean?*/ roleAnalysis) {
		// Summary:
		//		Shows the current widget and updates its data per the supplied contextData.
		// contextData: Object
		//		Context data.
		// roleAnalysis: boolean
		//		Optional. Default false.
		//		true - 	
		//			A) 	Given an analysis pane where a task is selected and the Analysis task pane is selected
		//				Then the pane will be updated to show the selected role
		//			B)	Given an analysis pane where a task is selected and the Analysis catalog  pane is selected
		//				Then the tab will switch to the analysis task pane with the selected role
		//			C)	Given an analysis pane where a task has not been selected
		//				Then the 'context data' will be set and a message will be shown in the message area for the 
		//				analysis pane indicating that an analysis task must be chosen from the catalog to analyze the role.  
		if (!(roleAnalysis === true)) {
			roleAnalysis = false;
		}
		
		this._setContextData(contextData);
		if (this.isPaneOpen()) {
			var desiredTabId = null;// default
			if (roleAnalysis === true) {
				var analysisTask = this._getAnalysisTask(); // _AnalysisTaskPanelController::_getAnalysisTask()
				if (analysisTask) {
					desiredTabId = this.analysisTab.id;
				} else {
					this.messageArea.showInfoMessage(this.localResources.ANALYSIS_TASK_NOT_SELECTED);
				}
			}
			this.updatePaneDisplay(/*preferences*/ this._getPreferences(desiredTabId));
		}
	},
	
	onShow: function() {
		// Summary:
		//		Invoked when the analysis panel transitions from a hidden state to a displayed state.
		//		This can be used for example to reconnect listeners
		//		or perform other updates need on display of the pane.
		
		this._trace.entering(this.id, "onShow");
		if (this.isPaneOpen()) {
			return;
		}
		this._subscribeToRoleUpdate();
		
		this.inherited(arguments);
		this._isPaneOpen = true;
		//NG: Added the call to the updateAnalysisTabContents method and commented the call above to the _onRefres.
		this._updateAnalysisTabContents();
		this._trace.exiting(this.id, "onShow");
	},
	
	onHide: function() {
		// Summary:
		//		Invoked when the analysis panel transitions from a displayed state to a hidden state.
		//		This can be used for example to disconnect listeners
		//		or perform other updates needed on hide of the panel.
		
		this._trace.entering(this.id, "onHide");
		if (!this.isPaneOpen()) {
			return;
		}
		this._unSubscribeToRoleUpdate();
		
		this.inherited(arguments);
		this._isPaneOpen = false;
		this._trace.exiting(this.id, "onHide");
	},
	
	isPaneOpen: function() {
		return this._isPaneOpen;
	},
	
	_setupMessageArea: function() {
		// summary: 
		//			sets the resize callback for the message area widget to resize the container for the page.
		// tags:
		//			private
		this._trace.entering(this.id, "setupMessageArea");
		this.messageArea.borderContainer = this.contentContainer;
		this._trace.exiting(this.id, "setupMessageArea");
	},
	
	_onRefresh: function() {
		// Summary:
		//		onClick call-back function for refresh button.
		//		
		//		Delegates control to inherited _AnalysisTaskPanelController::refreshCurrentTask().
		//		to refresh the currently displayed analysis task panel on analysis tab.
		
		this._trace.entering(this.id, "_onRefresh");
		if (this._isAnalysisTabSelected()) {
			this.refreshCurrentTask();
			this._needsRefresh = false;
		}
		this._trace.exiting(this.id, "_onRefresh");
	},
	
	_onHelp: function() {
		if (!this.help) {
			return;
		}
		this.help.onHelp();
	},
	_getPreferences: function(/*string?*/ desiredTabId) {
		// Summary:
		//		Loads the preferences.
		//
		// desiredTabId: string
		//		id of the tab to be selected.
		//		if not supplied then default tab id is taken.
		// returns: Object
		//		Preferences object.
		
		var preferences = null;
		
		var defaultTabId = this._defaultTab.id;
		var currentTabId = null;
		if (this.analysisContainer.selectedChildWidget) {
			currentTabId = this.analysisContainer.selectedChildWidget.id;
		}
		
		// TODO:
		//		Load preferences.
		if (!preferences) {
			preferences = {
				tabId: desiredTabId? desiredTabId: (currentTabId? currentTabId: defaultTabId)
			};
		}
		return preferences;
	},
	
	_selectCatalogTab: function() {
		// Summary:
		//		Selects the catalog tab.
		this.analysisContainer.selectChild(this.catalogTab.id);
	},
	
	_selectAnalysisTab: function() {
		// Summary:
		//		Selects the analysis tab.
		this.analysisContainer.selectChild(this.analysisTab.id);
	},
	
	_isCatalogTabSelected: function() {
		return this.catalogTab.selected;
	},
	
	_isAnalysisTabSelected: function() {
		return this.analysisTab.selected;
	},
	
	_getAnalysisTabHelpDiv: function() {
		// Summary:
		//		Creates (if not created earlier) a DIV with analysis tab help text.
		
		return this.analysisTabHelpDiv;
	},
	
	_createCatalogPanel: function() {
		this._catalogPanel = new com.ibm.security.modeling.analysis.AnalysisPaneCatalogPanel({
			id: this.id+"_catalogPanel",
			localResources: this.localResources,
			messageArea: this.messageArea
		}, this.id+"_catalogContent");
		
		this.connect(this._catalogPanel, "_showAnalysisTask", this.showAnalysis);
	},
	
	/*
	 * Context and state data
	 */
	_getContextData: function() {
		return dojo.clone(this.contextData);
	},
	
	_getAnalysisTaskData: function(/*Object?*/ data) {
		var contextData = this._getContextData();
		var analysisTaskData = null;
		if (data) {
			analysisTaskData = dojo.clone(data);
			dojo.mixin(analysisTaskData, contextData);
		} else {
			analysisTaskData = contextData;
		}
		return analysisTaskData;
	},
	
	_setContextData: function(/*Object*/ data) {
		
		var contextData = this._extractContextData(data);
		if (!contextData) {
			return;
		}
		var oldContextData = {};
		if (this.contextData) {
			dojo.mixin(oldContextData, this.contextData);
			dojo.mixin(this.contextData, contextData);
		} else {
			this.contextData = contextData;
		}
		
		// Determine if refresh is needed.
		if (this._needsRefresh == true || com.ibm.security.modeling.analysis.base._AnalysisPaneUtil.isContextDifferent(oldContextData, this.contextData, false, false)) {
			// Context is updated. Hence refresh is needed.
			this._needsRefresh = true;
		} else {
			this._needsRefresh = false;
		}
	},
	
	_extractContextData: function(/*Object*/ data) {
		this._trace.entering(this.id, "_extractContextData");
		
		var contextData = null;
		if (data) {
			var attributes = ["projectUrl", "modelId", "roleUri"];
			var attribute = null;
			var attributeValue = null;
			for (var i=0; i<attributes.length; i++) {
				attribute = attributes[i];
				attributeValue = data[attribute];
				if (attributeValue) {
					if (!contextData) {
						contextData = {};
					}
					contextData[attribute] = attributeValue;
				}
			}
		}
		
		this._trace.exiting(this.id, "_extractContextData");
		return contextData;
	},
	
	_resetContextData: function() {
		delete this.contextData;
		this.contextData = null;
	},
	
	/*
	 * Overridden methods from _AnalysisTaskPanelController.
	 */
	_clearCachedData: function() {
		// Overrides _AnalysisTaskPanelController::_clearCachedData
		this.inherited(arguments);
		
		// Empty the recently viewed tasks list  
		this._spliceRecentlyViewedTasksList(/*startIndex*/ 0);
		
		this._resetContextData();
		this._needsRefresh = true;
	},
	
	_getTaskPanelInitData: function(/*Object?*/ analysisTaskData) {
		// Summary:
		//		Overrides _AnalysisTaskPanelController::_getTaskPanelInitData().
		//		Appends messageArea to analysisTaskData 
		
		analysisTaskData = this.inherited(arguments);
		
		analysisTaskData["messageArea"] = this.messageArea;
		analysisTaskData["class"] = "contentArea";
		return analysisTaskData;
	},
	
	
	_setAnalysisTask: function(currentTask) {
		// Summary:
		//		Overrides _AnalysisTaskPanelController::_getAnalysisTaskPaneAttachPoint().
		//		What ever is the currentTask, it should be attached to DOM before being referenced by this.__currentTask
		//
		// currentTask: com.ibm.security.modeling.analysis.base._AnalysisTaskPanel
		
		if (currentTask == this._getAnalysisTask()) {
			return;
		} else {
			// Update internal variables.
			this.inherited(arguments);
		}
		
		if (currentTask && dojo.byId(currentTask.id)) {
			return;
		}
		
		// Update DOM
		this._orphanAnalysisTabContents();
		var contentNode = null;
		if (currentTask) {
			contentNode = currentTask.domNode;
		} else {
			contentNode = this._getAnalysisTabHelpDiv();
		}
		dojo.place(contentNode, this.analysisTabContent);
		if(currentTask) dojo.publish(currentTask.id+"_RESET");
	},
	
	_getAnalysisTaskPaneAttachPoint: function(id) {
		// Summary:
		//		Sub types must implement this.
		//		A valid dom node should be provided.
		//	
		// returns
		//		String|domNode by dojo.byId
		// tags:
		//		protected
		
		var attachPoint = dojo.create("div", {
			id: id,
			"class": "contentArea"
		});
		
		this._orphanAnalysisTabContents();
		
		dojo.place(attachPoint, this.analysisTabContent);
		return attachPoint;
	},
	
	_orphanAnalysisTabContents: function() {
		var analysisTaskPaneNodeList = dojo.query("#"+this.analysisTabContent.id+" > div");
		
		if (analysisTaskPaneNodeList.length > 0) {
			analysisTaskPaneNodeList.orphan();
		}
	},
	
	_setupHelp: function() {
		this.help = com.ibm.security.modeling.help.Help({
			id: this.id+"_help"
		});
		
		this._updateHelp();
	},
	
	_updateHelp: function(/*string*/ fileName) {
		if (!this.help) {
			return;
		}
		
		if (!fileName) { 
			fileName = this._defaultHelpFile;
		}
		
		this.help.setHelpFile(fileName); 
	},
	
	_subscribeToRoleUpdate: function() {
		if (!this.__roleUpdateListener) {
			this.__roleUpdateListener = this.subscribe("ROLE_PROPERTY_UPDATE_EVENT", this._onRoleUpdate);
		}
	},
	
	_unSubscribeToRoleUpdate: function() {
		if (this.__roleUpdateListener) {
			this.unsubscribe(this.__roleUpdateListener);
			this.__roleUpdateListener = null;
		}
	},
	
	_onRoleUpdate: function() {
		this._needsRefresh = true;
		if (this._analysisTaskPanelCache) {
			for (var taskId in this._analysisTaskPanelCache) {
				var task = this._analysisTaskPanelCache[taskId];
				task.needsRefresh = true;
			}
		}
	},
	
	_updateRecentlyViewedTasksList: function() {
		// If recent view is changed then:
		//	1. Add current task to recent view list.
		// 	2. Enforce Max limit of 10 on recent tasks list. 
		
		var currentTask = this._getAnalysisTask();
		if (!this._isMostRecentlyViewed(currentTask)) {
			// If current task was viewed earlier for the current context
			// then remove that entry [that will become duplicate]
			this._removeFromRecentlyViewedTasksList(currentTask);
			
			this._addToRecentlyViewedTasksList(currentTask);
			// Enforce limit of maximum 10 recent tasks. 
			this._spliceRecentlyViewedTasksList(/*startIndex*/ 10);
		} else {
			// update title.
			var menuItem = this._getRecentlyViewedTaskListMenuItemForTask(currentTask); 
			menuItem.set("label", currentTask.getTitle());
		}
	},
	
	_addToRecentlyViewedTasksList: function(analysisTask) {
		
		this._widgetUtilities.enableButton(this._recentViewsButton);
		
		var analysisTaskData = {};
		dojo.mixin(analysisTaskData, analysisTask.getAnalysisTaskData());
		com.ibm.security.modeling.analysis.base._AnalysisPaneUtil.cleanseRecentViewItemData(analysisTaskData);
		
		var analysisTaskFactory = analysisTask.getFactory();
		
		var menuItem = new dijit.MenuItem({
			label: analysisTask.getTitle(),
			onClick: dojo.hitch(this, function(event) {
				this._onRecentlyViewedTasksListItemClick(menuItem);
			}),
			style: "white-space: normal; word-wrap: break-word;",
			"class": "analysisPaneMenuItem",
			analysisTaskFactory: analysisTaskFactory,
			analysisTaskData: analysisTaskData
		});
		this._recentViewsMenu.addChild(menuItem, 0); // recent task is placed on top of the menu.
		//menuItem.startup();
	},
	
	_removeFromRecentlyViewedTasksList: function(analysisTask) {
		var menuItemIndex = this._getRecentlyViewedTaskListsItemIndexForTask(analysisTask);
		if (menuItemIndex < 0) {
			return;
		}
		this._spliceRecentlyViewedTasksList(menuItemIndex, menuItemIndex);
	},
	
	_onRecentlyViewedTasksListItemClick: function(menuItem) {
		
		var taskFactory = menuItem["analysisTaskFactory"];
		var analysisTaskData = menuItem["analysisTaskData"];
		if (this._isFirstMenuItem(menuItem)) {
		    /* Just refreshing might not help here. We need to do a tab switch as well to focus on the right tab. */
			this.updatePaneDisplay(/*preferences*/ this._getPreferences(/*desiredTabId*/ this.analysisTab.id));
		} else {
			this._resetContextData();
			this.showAnalysisTask(taskFactory, analysisTaskData);
		}
	},
	
	_isFirstMenuItem: function(menuItem) {
		return menuItem.getIndexInParent() == 0;
	},
	
	_isMostRecentlyViewed: function(analysisTask) {
		// Most recently viewed task is on top of the recently viewed task list [index 0].
		// 1. Either analysis Task changes.
		// 2. Or Task is same but context changes.
		
		var menuItemIndexForCurrentTask = this._getRecentlyViewedTaskListsItemIndexForTask(analysisTask);
		if (menuItemIndexForCurrentTask == 0) {
			// Analysis task is most recently viewed.
			return true;
		}
		return false;
	},
	
	_getRecentlyViewedTaskListsItemIndexForTask: function(analysisTask) {
		if (!analysisTask) {
			return -1;
		}
		
		var menuItems = this._recentViewsMenu.getChildren();
		if (!menuItems || menuItems.length <=0) {
			return -1;
		}
		
		var analysisTaskFactory = analysisTask.getFactory();
		var analysisTaskData = analysisTask.getAnalysisTaskData();
		for (var i=0; i<menuItems.length; i++) {
			var menuItem = menuItems[i];
			if (analysisTaskFactory == menuItem.analysisTaskFactory 
					&& !com.ibm.security.modeling.analysis.base._AnalysisPaneUtil.isContextDifferent(
							/*oldContextData*/ menuItem.analysisTaskData, 
							/*newContextData*/ analysisTaskData,
							/*strict*/ true,
							false,
							false)
				) {
				
				// Task and its context matched.
				return i;
			}
		}
		
		return -1;
	},
	
	_getRecentlyViewedTaskListMenuItemForTask: function(analysisTask) {
		var menuItemIndex = this._getRecentlyViewedTaskListsItemIndexForTask(analysisTask);
		if (menuItemIndex < 0) {
			return null;
		}
		
		var menuItems = this._recentViewsMenu.getChildren();
		return menuItems[menuItemIndex];
	},
	
	_spliceRecentlyViewedTasksList: function(/*int*/ startIndex, /*int?*/endIndex) {
		// Items at startIndex, endIndex are included.
		// Number of removed items can be calculated as: (endIndex - startIndex) +1
		// Example: 
		//		To empty a list invoke (0, list.length-1) or (0)
		//		To remove three items [3rd, 4th, and 5th], invoke (2, 4)
		//		To remove 2nd item invoke (3, 3)
		
		var menuItems = this._recentViewsMenu.getChildren();
		if (menuItems && endIndex !==0 && !endIndex) {
			endIndex = menuItems.length -1;
		}
		
		if (startIndex > endIndex) {
			return;
		}
		var count = (endIndex - startIndex) +1;
		
		if (!menuItems || (startIndex+count > menuItems.length)) {
			return;
		}
		
		for (var i=0; i<count; i++) {
			var menuItem = menuItems[(startIndex+i)];
			this._recentViewsMenu.removeChild(menuItem);
			menuItem.destroyRecursive();
		}
	},
	
	_updateAnalysisDataOfRecentViewItem: function(/*Object*/ analysisTask) {
		var menuItemIndex = this._getRecentlyViewedTaskListsItemIndexForTask(analysisTask);
		if (menuItemIndex < 0) {
			return;
		}
		
		var analysisTaskData = {};
		dojo.mixin(analysisTaskData, analysisTask.getAnalysisTaskData());
		
		var menuItems = this._recentViewsMenu.getChildren();
		var recentViewItem = menuItems[menuItemIndex];
		recentViewItem["analysisTaskData"] = analysisTaskData;
	}
	
});