/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

(c) Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.analysis.AnalysisPaneCatalogPanel");

dojo.require("dojo.data.ItemFileReadStore");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.tree.ForestStoreModel");

dojo.require("com.ibm.security.modeling.breadcrumbs.BreadcrumbBuilder");
dojo.require("com.ibm.security.modeling.layouts.TemplatedPanelWithLayout");
dojo.require("com.ibm.security.modeling.util.MessageUtilities");
dojo.require("com.ibm.security.modeling.util.Trace");

dojo.require("ibm.tivoli.common.listtree.dijit.ModelListTree");

dojo["requireLocalization"]("customizations", "AnalysisCatalogCustomizationsText");
dojo.requireLocalization("com.ibm.security.modeling", "PluginMessages");
dojo.requireLocalization("com.ibm.security.modeling.analysis", "AnalysisCatalogDefaultsText");
dojo.requireLocalization("com.ibm.security.modeling.analysis", "AnalysisPaneText");

/**
 * Main controller templated widget for the Analysis pane UI.
 */
dojo.declare("com.ibm.security.modeling.analysis.AnalysisPaneCatalogPanel", 
	[com.ibm.security.modeling.layouts.TemplatedPanelWithLayout], {
	
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.analysis","templates/AnalysisPaneCatalogPanel.html"),	
	widgetsInTemplate: true,
	
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.analysis.AnalysisPaneCatalogPanel"),

	postMixInProperties: function() {
		// Summary:
		//		Widget life-cycle method.
		
		this._trace.entering(this.id, "postMixInProperties");
		
		this.inherited(arguments);
		
		if (!this.localResources) {
			this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.analysis", "AnalysisPaneText");
		}
		this._pluginResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "PluginMessages");
		this._messageUtilities = new com.ibm.security.modeling.util.MessageUtilities();
		this._analysisCatalogDefaultsResources = dojo.i18n.getLocalization("com.ibm.security.modeling.analysis", "AnalysisCatalogDefaultsText");
		
		this._trace.exiting(this.id, "postMixInProperties");
	},
	
	postCreate: function() { 
		// Summary:
		//		Widget life-cycle method. 
		
		this._trace.entering(this.id, "postCreate");

		this.inherited(arguments);

		this._loadAnalysisCatalogItems();

		this._trace.exiting(this.id, "postCreate");
	},
	
	_adjustAnalysisCatalogItems: function(analysisCatalogItems, nextId, resources) {
		// Summary:
		//		Adjusts the analysis catalog items by adding additional properties.
		//
		// analysisCatalogItems: Array of Object
		//		Analysis catalog items that should be adjusted.
		// nextId: Number
		//		Next identifier that should be used for the analysis catalog items.
		// resources: Object
		//		Resources for text.

		for (var i = 0; i < analysisCatalogItems.length; i++) {
			var analysisCatalogItem = analysisCatalogItems[i];
			analysisCatalogItem.id = nextId;
			nextId++;
			analysisCatalogItem.shortDescription = this._getAnalysisCatalogItemText(analysisCatalogItem.shortDescriptionKey, resources);
			analysisCatalogItem.longDescription = this._getAnalysisCatalogItemText(analysisCatalogItem.longDescriptionKey, resources);
			analysisCatalogItem.factory = this._getAnalysisCatalogItemFactory(analysisCatalogItem);
			if ((analysisCatalogItem.subItems != undefined) &&
				(analysisCatalogItem.subItems != null) &&
				(analysisCatalogItem.subItems.length > 0)) {
				nextId = this._adjustAnalysisCatalogItems(analysisCatalogItem.subItems, nextId, resources);
			}
		}

		return nextId;
	},

	_createAnalysisCatalog: function(defaultAnalysisCatalogItems, customizedAnalysisCatalogItems) {
		// Summary:
		//		Creates the analysis catalog and breadcrumbs.
		//
		// defaultAnalysisCatalogItems: Array of Object
		//		Default analysis catalog items that were loaded.
		// customizedAnalysisCatalogItems: Array of Object
		//		Customized analysis catalog items that were loaded.

		var concatenatedAnalysisCatalogItems = this._getConcatenatedAnalysisCatalogItems(defaultAnalysisCatalogItems, customizedAnalysisCatalogItems);

		var analysisCatalogStore = new dojo.data.ItemFileReadStore({
			data: {items: concatenatedAnalysisCatalogItems}
		});

		this.analysisCatalogModel = new dijit.tree.ForestStoreModel({
			childrenAttrs: ["subItems"],
			query: {},
			store: analysisCatalogStore
		});

		var analysisCatalogListLabel = this.localResources.ANALYSIS_CATALOG_LIST_TITLE;
		var analysisCatalogListDescription = this.localResources.ANALYSIS_CATALOG_LIST_DESCRIPTION;
		this.analysisCatalogList = new ibm.tivoli.common.listtree.dijit.ModelListTree({
				childField: "subItems",
				descField: "longDescription",
				description: analysisCatalogListDescription,
				fillOrder: "",
				iconField: "imagePath",
				iconSizeField: "iconSize",
				keyField: "id",
				label: analysisCatalogListLabel,
				labelField: "shortDescription",
				leafField: "subItems",
				model: this.analysisCatalogModel,
				onClick: dojo.hitch(this, this._onAnalysisCatalogItemClick),
				formatChildDescription: dojo.hitch(this, this._formatChildDescription)
			}, "analysisCatalogList");

		var analysisCatalogBreadcrumbLabel = this.localResources.ANALYSIS_CATALOG_BREADCRUMB;
		var analysisCatalogBreadcrumb = new ibm.tivoli.tip.dijit.Breadcrumb({
			label: analysisCatalogBreadcrumbLabel,
			onClick: dojo.hitch(this, function() {
				if (this.analysisCatalogBreadcrumbData.getLength() > 1) {
					this.analysisCatalogBreadcrumbData.removeLastBreadCrumb();
				}
				this.analysisCatalogList.getTreeRoot().show();			
			})
		});

		this.analysisCatalogBreadcrumbData = new com.ibm.security.modeling.breadcrumbs.BreadcrumbBuilder()
			.withId("analysisCatalogBreadcrumb")
			.withBreadcrumb(analysisCatalogBreadcrumb)
			.build();
		
		//Override the default toolbar style that the breadcrumb gets
		this.analysisCatalogBreadcrumbData.addClass("analysisPaneBreadcrumb");
		
		if (this._hasFolder()) {
			this.analysisCatalogBreadcrumbData.addClass("show");
		} else {
			this.analysisCatalogBreadcrumbData.addClass("hide");
		}

		dojo.connect(this.analysisCatalogList.getTreeRoot(), "onPushList", dojo.hitch(this, function (previousList, newList) {
			this.analysisCatalogBreadcrumbData.addBreadCrumb(new ibm.tivoli.tip.dijit.Breadcrumb({
				label: newList.data.shortDescription
			}));
		}));

		this.analysisCatalogList.startup();
		this.analysisCatalogList.show();
	},
	_formatChildDescription : function (description) {
		var formattedDescription;
		if(description) {
			formattedDescription = description.htmlencode();
		} else {
			formattedDescription = "&nbsp;";
		}
		return formattedDescription;
	},
	_displayMessage: function(severity, key, error) {
		// Summary:
		//		Displays a message.
		//
		// severity: String
		//		Severity of the message that should be displayed.
		// key: String
		//		Key of the message that should be displayed.
		// error: Object
		//		Error that occurred.

		if ((this.messageArea != undefined) &&
			(this.messageArea != null)) {
			var unformattedMessage = this._pluginResources[key];
			var inserts = new Array();
			inserts[0] = error.message;
			var formattedMessage = this._messageUtilities.formatMessage(unformattedMessage, inserts);
			if (severity == "info") {
				this.messageArea.showInfoMessage(formattedMessage);
			} else if (severity == "warning") {
				this.messageArea.showWarningMessage(formattedMessage);
			} else if (severity == "error") {
				this.messageArea.showErrorMessage(formattedMessage);
			} else {
				this.messageArea.showErrorMessage(formattedMessage);
			}
		}
	},

	_getAnalysisCatalogItemFactory: function(analysisCatalogItem) {
		// Summary:
		//		Returns the factory for the specified analysis catalog item.
		//
		// analysisCatalogItem: Object
		//		Analysis catalog item for which the factory should be returned.

		var factory = null;
		if ((analysisCatalogItem.factoryName != undefined) &&
			(analysisCatalogItem.factoryName != null)) {
			var factoryName = new String(analysisCatalogItem.factoryName);
			dojo["require"](factoryName);
			factory = dojo.getObject(factoryName);
		}
		return factory;
	},

	_getAnalysisCatalogItemText: function(key, resources) {
		// Summary:
		//		Returns the text for the specified key from the specified resources.
		//
		// key: String
		//		Key for which the text should be returned.
		// resource: Object
		//		Resources from which the text should be retrieved.

		var text = "";
		if ((key != undefined) &&
			(key != null)) {
			if ((resources != undefined) &&
				(resources != null)) {
				text = resources[key];
			} else {
				text = null;
			}
			if ((text == undefined) ||
				(text == null)) {
				text = key;
			}
		}
		return text;
	},

	_hasFolder: function() {
		// Summary:
		//		Indicates if any of the analysis tasks are contained in a folder.
		//
		//      Returns true, if there is at least one folder in the analysis catalog.
		//      Returns false, if there are no folders in the catalog. This is the default.
		//
		var analysisCatalogItems = null;
		
		if ( this.analysisCatalogList && this.analysisCatalogList.subitems ) {
			analysisCatalogItems = this.analysisCatalogList.subitems;
			for (var i = 0; i < analysisCatalogItems.length; i++) {
				var analysisCatalogItem = analysisCatalogItems[i];
				
				/* Check if the the current catalog item is a folder. */
				if ((analysisCatalogItem.subitems != undefined) &&
					(analysisCatalogItem.subitems != null) &&
					(analysisCatalogItem.subitems.length > 0)) {
					return true;
				}
			}
		}
		
		return false;
	},

	_getConcatenatedAnalysisCatalogItems: function(defaultAnalysisCatalogItems, customizedAnalysisCatalogItems) {
		// Summary:
		//		Returns a concatenated list of the specified analysis catalog items.
		//
		// defaultAnalysisCatalogItems: Array of Object
		//		Default analysis catalog items that were loaded.
		// customizedAnalysisCatalogItems: Array of Object
		//		Customized analysis catalog items that were loaded.

		var nextId = 0;
		if ((defaultAnalysisCatalogItems == undefined) ||
			(defaultAnalysisCatalogItems == null)) {
			defaultAnalysisCatalogItems = new Array();
		} else if (defaultAnalysisCatalogItems.length > 0) {
			nextId = this._adjustAnalysisCatalogItems(defaultAnalysisCatalogItems, nextId, this._analysisCatalogDefaultsResources);
		}
		if ((customizedAnalysisCatalogItems == undefined) ||
			(customizedAnalysisCatalogItems == null)) {
			customizedAnalysisCatalogItems = new Array();
		} else if (customizedAnalysisCatalogItems.length > 0) {
			try {
				this._analysisCatalogCustomizationsResources = dojo.i18n.getLocalization("customizations", "AnalysisCatalogCustomizationsText");
			} catch (error) {
				this._displayMessage("warning", "ERROR_LOADING_CUSTOMIZED_ANALYSIS_CATALOG_ITEM_TEXT", error);
				this._analysisCatalogCustomizationsResources = null;
			}
			nextId = this._adjustAnalysisCatalogItems(customizedAnalysisCatalogItems, nextId, this._analysisCatalogCustomizationsResources);
		}
		var concatenatedAnalysisCatalogItems = defaultAnalysisCatalogItems.concat(customizedAnalysisCatalogItems);
		return concatenatedAnalysisCatalogItems;
	},

	_loadAnalysisCatalogItems: function() {
		// Summary:
		//		Loads the analysis catalog items.

		dojo.xhrGet({
			error: dojo.hitch(this, this._onLoadDefaultAnalysisCatalogItemsFailure),
			handleAs: "json-comment-optional",
			load: dojo.hitch(this, this._onLoadDefaultAnalysisCatalogItemsSuccess),
			url: "/securitymodeling/js/src/com/ibm/security/modeling/analysis/AnalysisCatalogDefaults.json"
		});
	},

	_loadCustomizedAnalysisCatalogItems: function(defaultAnalysisCatalogItems) {
		// Summary:
		//		Loads the customized analysis catalog items.
		//
		// defaultAnalysisCatalogItems: Array of Object
		//		Default analysis catalog items that were loaded.

		dojo.xhrGet({
			error: dojo.hitch(this, this._onLoadCustomizedAnalysisCatalogItemsFailure, defaultAnalysisCatalogItems),
			handleAs: "json-comment-optional",
			load: dojo.hitch(this, this._onLoadCustomizedAnalysisCatalogItemsSuccess, defaultAnalysisCatalogItems),
			url: "/securitymodeling/customizations/AnalysisCatalogCustomizations.json"
		});
	},

	_onAnalysisCatalogItemClick: function(analysisCatalogItem) {
		// Summary:
		//		Receives control when an analysis catalog item is clicked.
		//
		// analysisCatalogItem: ibm.tivoli.common.listtree.dijit.ModelListTree
		//		Analysis catalog item that was clicked.

		if ((analysisCatalogItem.data.factory != undefined) &&
			(analysisCatalogItem.data.factory != null) &&
			(analysisCatalogItem.data.factory[0] != undefined) &&
			(analysisCatalogItem.data.factory[0] != null)) {
			this._showAnalysisTask(analysisCatalogItem.data.factory[0]);
		}
	},
	
	_onLoadCustomizedAnalysisCatalogItemsFailure: function(defaultAnalysisCatalogItems, error) {
		// Summary:
		//		Receives control when loading the customized analysis catalog items fails.
		//
		// defaultAnalysisCatalogItems: Array of Object
		//		Default analysis catalog items that were loaded.
		// error: Object
		//		Error that occurred.

		this._displayMessage("error", "ERROR_LOADING_CUSTOMIZED_ANALYSIS_CATALOG_ITEMS", error);
		this._createAnalysisCatalog(defaultAnalysisCatalogItems, null);
	},

	_onLoadCustomizedAnalysisCatalogItemsSuccess: function(defaultAnalysisCatalogItems, customizedAnalysisCatalogItems) {
		// Summary:
		//		Receives control when loading the customized analysis catalog items succeeds.
		//
		// defaultAnalysisCatalogItems: Array of Object
		//		Default analysis catalog items that were loaded.
		// customizedAnalysisCatalogItems: Array of Object
		//		Customized analysis catalog items that were loaded.

		this._createAnalysisCatalog(defaultAnalysisCatalogItems, customizedAnalysisCatalogItems);
	},

	_onLoadDefaultAnalysisCatalogItemsFailure: function(error) {
		// Summary:
		//		Receives control when loading the default analysis catalog items fails.
		//
		// error: Object
		//		Error that occurred.

		this._displayMessage("error", "ERROR_LOADING_DEFAULT_ANALYSIS_CATALOG_ITEMS", error);
		this._loadCustomizedAnalysisCatalogItems(null);
	},

	_onLoadDefaultAnalysisCatalogItemsSuccess: function(defaultAnalysisCatalogItems) {
		// Summary:
		//		Receives control when loading the default analysis catalog items succeeds.
		//
		// defaultAnalysisCatalogItems: Array of Object
		//		Default analysis catalog items that were loaded.

		this._loadCustomizedAnalysisCatalogItems(defaultAnalysisCatalogItems);
	},

	_showAnalysisTask: function(/*_AnalysisTaskPanelFactory*/ factory) {
		// Summary:
		//		Shows the analysis task panel for the specified factory.
		//
		// factory: com.ibm.security.modeling.analysis.base._AnalysisTaskPanelFactory
		//		Factory to create analysis task panel.
	}

});
