dojo.provide("com.ibm.security.modeling.analysis.AnalysisTableWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.CheckBox");
dojo.require("com.ibm.security.modeling.analysis.base._AnalysisTaskPanel");
dojo.require("com.ibm.security.modeling.dijit.layout.ContentPane");
dojo.require("com.ibm.security.modeling.dijit.layout.StackContainer");
dojo.require("com.ibm.security.modeling.layouts.TemplatedPanelWithLayout");
dojo.require("dijit.layout.StackController");
dojo.require("com.ibm.security.modeling.tables.Table");
dojo.require("com.ibm.security.modeling.help.HelpLink");
dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");
dojo.require("dojo.number");
dojo.require("com.ibm.security.modeling.util.Trace");

dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling.analysis", "AnalysisText");
dojo.requireLocalization("com.ibm.security.modeling", "LogMessages");

dojo.declare("com.ibm.security.modeling.analysis.AnalysisTableWidget", 
		[ com.ibm.security.modeling.layouts.TemplatedPanelWithLayout,com.ibm.security.modeling.analysis.base._AnalysisTaskPanel] , {	
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.analysis.AnalysisTableWidget"),
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.analysis","templates/AnalysisTableWidget.html"),	
	widgetsInTemplate: true,
	container: null,
	// isClientSide: boolean
	//     flag determining whether the table should be client or server side in its
	//     searching/filtering/sorting
	isClientSide: false,
	// defaultPageSize: int
	//     The number of rows to be rendered on a page by default
	defaultPageSize: 10,
	// isFilterVisible: boolean
	//     Whether to show the filter on the toolbar
	isFilterRendered: true,
	// isSelectAllRendered: boolean
	//     Indicates whether the select all, remove all toolbar buttons are visible
	isSelectAllRendered: false,
	// isRefreshVisible: boolean
	//     Indicates whether the refresh toolbar button is visible
	isRefreshRendered: false,
	// isMultiSelect: boolean
	//     Indicates whether selection mode is single or multi
	isMultiSelect: false,
	// isSearchMultiSelect: boolean
	//     Indicates whether selection mode of search options is single or multi
	isSearchMultiSelect: false,
	isDynamic: true,
	// searchString: String
	//     Possible values USER/PERMISSION
	type: "",
	// entityTypeText: String
	//     Translated String describing the items in the table.
	entityTypeText: "",
	// searchInstructions: String
	//     Translated String giving search instructions
	searchInstructions: "",
	// needSelection: boolean
	//     whether the table should support selection of rows
	needSelection:false,
	// tableQuery: object
	//     default query for table contents
	tableQuery:{},
	// structure: Object[]
	//     a definition of the table columns
	structure:null,
	// publishTrigger: String
	//     event name used trigger refresh call
	//     datastore object.
	publishTrigger:null,
	// dataStore: dojo.data.DataStore
	//     data to be used to populate the table. Plugin any compatible dojo
	//     datastore object.
	dataStore:null,
	// unSortableColumns: Number[]
	//     1-based integer array of the columns where sort is disabled (starting
	//     with the selection column).
	unSortableColumns:[],
	// defaultSort: Integer
	//     the 0-based column index of the default sort
	//     changed in postMixIn depending upon needSelection , i.e with/without
	//     indirectSelection plugin
	defaultSortIndex: null,
	// noAnalysisMessage: String
	//     Log message key if no analysis is present
	noAnalysisMessage: "",
	// helpUrl: String
	//     html file name for Learn More link
	helpUrl:"",
	// learnMoreFile: String
	//     html file name for Learn More link
	learnMoreFile:"",
	// analysisUri: String
	//     url to query for getting the analysis data
	analysisUri:"",
	// entityType: String
	//     Entity type for table and query param for analysis URL
	entityType:"",
	// querySet: String
	//     Query param for analysis URL, provides which data set to fetch
	querySet:"",
	// title: String
	//     Title to to be shown on analysis pane
	title:"",
	isInheritanceRequired:false,
	isDefaultSortAscending:true,
	isAttributeCheckRequired:true,
	constructor: function(args) {
		// tags:
		// protected
		this._trace.entering("", "constructor");
		this._trace.object(args, "constructor", "Constructor args");
		if(!this.globalResources){
			this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		}
		this.analysisResources = dojo.i18n.getLocalization("com.ibm.security.modeling.analysis", "AnalysisText");
		this._logMessageResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "LogMessages");
		this.htmlUtilities=new com.ibm.security.modeling.util.StringUtils();
		this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
		this._trace.exiting("", "constructor");
		
	},	
	postCreate:function() {
		// tags:
		// protected
		this._trace.entering(this.id, "postCreate");
		this.inherited(arguments);
		this._trace.exiting(this.id, "postCreate");
	},	
	startup:function() {
		// tags:
		// protected
		this._trace.entering(this.id, "startup");
		this.inherited(arguments);
		this._trace.exiting(this.id, "startup");
	},
	destroy:function() {
		// tags:
		//     protected
		this._trace.enteringWithStack(this.id, "destroy", true);
		try {
			dojo.forEach(this.connections, dojo.disconnect);
			if(this.table){	
				this.table.destroy();		
			}
			if(this.learnMoreWidget){
				this.learnMoreWidget.destroy();
			}
			this.inherited(arguments);
			this._trace.exiting(this.id, "destroy");
		} catch(e) {
			this._trace.exception(e, "destroy");
			throw e;
		}
	},
	refresh: function() {
		// Summary:
		//		Refreshes the analysis panel. 
		this._trace.entering(this.id, "refresh");
		try {
			this._updateUIStatePerSupplementData();
			this.onLoadingStart();	
			this._updateTitle();
			this._loadTable();
		} finally {
		}
		
		this._trace.exiting(this.id, "refresh");
	},
	_updateTitle: function() {
		//need to be implemented by the deriving table
	},
	
	attachTableToDOM: function() {
		
		var attach = function(containerId, chartId) {
			if (!dojo.byId(chartId)) { // not in DOM.
				var widget = dijit.byId(chartId); 
				if (widget) {
					dojo.place(widget.domNode, containerId);
				}
			}
		};
		
		attach(this.tablePane.id, this.id+"_Table");
		attach(this.messagePane.id, this.id+"_learnMore");
		attach(this.noRoleMessagePane.id, this.id+"_noRolePane");
	},
	_setupStructure:function(cols){
		// Summary:
		//		Setups structure of table to show first name column as hyperlink  
		// cols: Array
		//		Array of columns of table
		this._trace.entering(this.id, "_setupStructure");
		this.toolTipRequired=[];
		var i;
         for(i=0;i< cols.length;i++){
            if(cols[i].field === "name"){
                  cols[i].get=function(rowIndex,item){return item;};
                  cols[i].formatter=dojo.hitch(this, this._formatName);
             }
			 this.toolTipRequired.push(cols[i].field);
         }
		this._trace.exiting(this.id, "_setupStructure");
        return  cols;		
	},
	_formatAssignedToRoleColumn:function(list){
		// Summary:
		//		Formatter for Assigned Role Column 
		// list: Array
		//		Array of Role names
		//this._trace.entering(this.id, "_formatAssignedToRoleColumn");
		var i;
		var translatedList=[];
		for(i=0;i<list.length;i++){
			var str;
			switch(list[i]){
				case "Directly assigned":
					str=this.analysisResources.DIRECTLY_ASSIGNED;
					break;
				case "By filter":
					str=this.analysisResources.BY_FILTER;
					break;
				case "Inherited":
					str=this.analysisResources.INHERITED;
					break;
			}
			translatedList.push(str);
		}
		//this._trace.exiting(this.id, "_formatAssignedToRoleColumn");
		return this._createCommaSeparatedStringFromNameArray(translatedList);
	},
	_formatInheritedFromRolesColumn:function(list){
		// Summary:
		//		Formatter for Inherited From Roles Column 
		// list: Array
		//		Array of types on how entity is attached to role
		//this._trace.entering(this.id, "_formatInheritedFromRolesColumn");
		//this._trace.exiting(this.id, "_formatInheritedFromRolesColumn");
		return this._createCommaSeparatedStringFromNameArray(list);
	},
	_formatName:function(item){
		// Summary:
		//		Formatter for Name Column 
		//item: Object
		//		Entity to be shown in the name column with hyperlink to show its properties
		//this._trace.entering(this.id, "_formatName");
        var anchor=null;
	    if(item){
	 	    anchor=new com.ibm.security.modeling.util.Anchor({
						input:item.name,
						callback:dojo.hitch(this, this._entitySelected,item)
			});											
		}
		//this._trace.exiting(this.id, "_formatName");
		return anchor;
    },
	_formatAttributeValues:function(value){
		// Summary:
		//		Formatter for Attribute values column
		//value: Array
		//		Contains list of values for attribute used as column
		//this._trace.entering(this.id, "_formatAttributeValues");
		var values= [];
		var valueString = "";
		if(value && value[0]){
			values = this._createNamesArrayFromObjectArray(value,"value");
			valueString = this._createCommaSeparatedDivStringFromNameArray(values); 
		}
		//this._trace.exiting(this.id, "_formatAttributeValues");
		return valueString;
	},
	_createNamesArrayFromObjectArray: function(/*Object[]*/items,property) {
		// Summary:
		//		Converts Object to  Array
		//items: Object
		//		object to be coverted
		//this._trace.entering(this.id, "_createNamesArrayFromObjectArray");
		var values = [];
		var i;
		for(i=0,len=items.length;i<len;i++){
			var name=items[i][property];
			values.push(name);
		}
		//values.sort();//adding sort due to problem in automation testing D11385
		//this._trace.exiting(this.id, "_createNamesArrayFromObjectArray");
		return values;
	},
	_createCommaSeparatedStringFromNameArray: function(/*String[]*/values) {
		// Summary:
		//		Converts Array to comma separated String value
		//values: Array
		//		Array to be coverted
		//this._trace.entering(this.id, "_createCommaSeparatedStringFromNameArray");
		var namesString = "";
		var j;
		for(j=0;j<values.length;j++){
			namesString = namesString + this.htmlUtilities.parseHTMLandNormalize(values[j]);
			if(j+1 !== values.length){
				namesString=namesString + ", ";
			}
		}	
		//this._trace.exiting(this.id, "_createCommaSeparatedStringFromNameArray");
		return namesString;
	},
	_createCommaSeparatedDivStringFromNameArray: function(/*String[]*/values) {
		// Summary:
		//		Converts Array to comma separated String value
		//values: Array
		//		Array to be coverted
		//this._trace.entering(this.id, "_createCommaSeparatedDivStringFromNameArray");
		var namesString = "";
		var j;
		values=values||[];
		for(j=0;j<values.length - 1;j++){
			namesString = namesString + "<div style='height:17px; overflow: hidden;' >"+this.htmlUtilities.parseHTMLandNormalize(values[j])+",</div>";
		}
		if(values.length) namesString = namesString + "<div style='height:17px; overflow: hidden;' >"+this.htmlUtilities.parseHTMLandNormalize(values[j])+"</div>";
	
		//this._trace.exiting(this.id, "_createCommaSeparatedDivStringFromNameArray");
		return namesString;
	},
	_formatAttributePercentage:function(value){
		// Summary:
		//		Formatter for Attribute percentage column
		//value: Array
		//		Contains list of percentages for attribute values used as column
		//this._trace.entering(this.id, "_formatAttributePercentage");
		var progressHtml="";
		var percentages= [];
		if(value && value[0]){
			percentages = this._createNamesArrayFromObjectArray(value,"inPercent");
			var i;
			for(i=0;i<percentages.length;i++){
				progressHtml+=this._getFormatTags(percentages[i],100);
			}
		}
		
		return progressHtml;
	},
	_getFormatTags:function(value,maximum){
		var percent = 1;
		if(String(value).indexOf("%") != -1){
			percent = Math.min(parseFloat(value)/100, 1);
			value = percent * maximum;
		}else{
			value = Math.min(value, maximum);
			percent = value / maximum;
		}
		var text = dojo.number.format(percent, { type: "percent", places: 0, locale: this.lang });
		var width = (percent * 100) + "%";
		var templateString="<div class='dijitProgressBar dijitProgressBarEmpty' style='text-align:center;width:90%;height:100%;margin-left:3px;margin-right:3px;'>";
		templateString+="<div waiRole='progressbar'  class='dijitProgressBarFull' style='height:13px;' aria-valuenow='"+value+"' aria-valuemin='0' aria-valuemax='"+maximum+"'>";
		templateString+="<div class='dijitProgressBarTile' style='width:"+width+";'></div><span style='visibility:hidden'></span></div>";
		templateString+="<div  class='dijitProgressBarLabel'>"+text+"</div></div>";
		return templateString;
	},
    _entitySelected:function(item){
		// Summary:
		//		Gets called when the name column hyperlink gets clicked
		//item: object
		//		Row data 
		this._trace.entering(this.id, "_entitySelected");
        dojo.require("com.ibm.security.modeling.widgets.PropertiesDialog");
        var attributeField="uri";
        this.entityAttributePanel=new com.ibm.security.modeling.widgets.PropertiesDialog({
									type:this.type,
									panelHeaderName:item.name,
									entityItem:item[attributeField]
								},null);
        this.entityAttributePanel.show();
        this.entityAttributePanel.startup();
		this._trace.exiting(this.id, "_entitySelected");
     },
	 _loadTable:function(e){
		// Summary:
		//		Loads the table to show the data
		this._trace.entering(this.id, "_loadTable");
		
		if(this.structure){
			this._setUpTableQuery(e);
			
			var storeChanged = true;
			if (this.dataStore && this.dataStore.target == this.analysisUri) {
				storeChanged = false;
			}
			this._setupStore();
			
			this.messageArea.hideMessages();
			if(this.table){
				this.table.clearSearch();
				this.table.resetCurrentPageVariable();
				
				if (storeChanged) {
					this.table.setStore(null,this.table.query);
					this._updateTableStructure();
					this.table.setStore(this.dataStore,this.table.query);
				} else {
					this._updateTableStructure();
				}
			}else{
				this._createTable();
			}
		}else{
			this.showNoAnalysisMessagePane();
		}
		this._trace.exiting(this.id, "_loadTable");
	 },
	 _updateTableStructure: function() {
		// stub.
		// For overriding.
	 },
	 _setUpTableQuery : function(e){
		// Summary:
		//		Prepares query param for URL of table
		this._trace.entering(this.id, "_setUpTableQuery");
		this.tableQuery={};
		if(this.isInheritanceRequired) {
			this.tableQuery.includeInheritance = this._getIncludeInheritance();
		}
		
		this.tableQuery.entityType=this.entityType;
		this.tableQuery.querySet=this.querySet;
		this._trace.exiting(this.id, "_setUpTableQuery");
	},
	_setupStore:function(){
		// Summary:
		//		Creates the store for table
		this._trace.entering(this.id, "_setupStore");
		
		if (this.dataStore && this.dataStore.target == this.analysisUri) {
			// no update in store is required.
			return; 
		}
		
		this.dataStore = new com.ibm.security.modeling.tables.JsonRestStore({
			target: this.analysisUri,
			idAttribute: 'uri'
		});
		
		var getValue = this.dataStore.getValue;
		this.dataStore.getValue = function(/*Object*/item, /*String*/property, /*value?*/defaultValue) {
		   
			if(property.lastIndexOf("InPercent") > 0){
				property = property.replace("InPercent","");
			}
		   
			return getValue(item, property, defaultValue);
		};
		this._trace.exiting(this.id, "_setupStore");
	},
	getInheritanceLabel: function() {
		// Summary:
		//		Gets the label to be shown against the inheritance check-box of each task. 
		//
		// returns: string
		//		inheritanceLabel
		
		if(!this.analysisResources) {
			this.analysisResources = dojo.i18n.getLocalization("com.ibm.security.modeling.analysis", "AnalysisText");	
		}
		
		return this.analysisResources.INCLUDE_ROLE_MEMBER_INHERITANCE;
	},		 
	_createTable: function() {
		// summary:
		// creates the grid for this table with initial values
		// tags:
		// private
		this._trace.entering(this.id, "_createTable");
		var tableArgs = {
			id: this.id+"_Table",
			dataStore: this.dataStore, 
			entityTypeText: this.entityTypeText,
			isClientSide: this.isClientSide,
			isFilterRendered: this.isFilterRendered,
			isRefreshRendered: this.isRefreshRendered,
			parentContainer: this.container,
			defaultPageSize: this.defaultPageSize,
			isSelectAllRendered: this.isSelectAllRendered,
			searchInstructions:this.searchInstructions,
			isMultiSelect: this.isMultiSelect,
			isSearchMultiSelect: this.isSearchMultiSelect,
			isDynamic: this.isDynamic,
			type: this.type,
			messageArea:this.messageArea,
			unSortableColumns:this.unSortableColumns,
			isDefaultSortAscending:this.isDefaultSortAscending,
			defaultSortIndex:this.defaultSortIndex,
			needSelection:this.needSelection,
			structure: this.structure,
			query: this.tableQuery
		};
		
		
		this.table = new com.ibm.security.modeling.tables.Table (tableArgs, this.id+"_tableContainer");
		dojo.addClass(this.table.domNode, "contentArea");
		this.table.startup();
		dojo.connect(this.table.grid, "_onFetchComplete", this, this._handleFetchComplete);
		//dojo.subscribe(this.table.id+"_handleFetchComplete", this, dojo.hitch(this,function() {this._handleFetchComplete();}));
		dojo.connect(this.table.searchField,"_onFocus",this,dojo.hitch(this,this.clearDrillDownSearchQuery));
		if(this.isInheritanceRequired){
			this.inheritanceCheckBox = new com.ibm.security.modeling.analysis.IncludeInheritanceWidget({
				id: this.id +"_includeInheritance",
				isChecked: this._getIncludeInheritance(),
				changeEventHandler: dojo.hitch(this, this._onIncludeInheritanceChange),
				labelText: this.getInheritanceLabel()
			});
			this.table.header.toolbar.addChild(this.inheritanceCheckBox);
		}
		this._trace.exiting(this.id, "_createTable");
		
	},
	
	_handleFetchComplete:function(items){
		this.onLoadingComplete();
	},
	clearDrillDownSearchQuery:function(){
		
	},
	destroyTable:function(){
		// Summary:
		//		Destroys the table
		this._trace.entering(this.id, "destroyTable");
		if(this.table){
			this.table.destroy();
			var div=dojo.create("div",{id:this.id+"_tableContainer"});
			this.tablePane.setContent(div);
		}
		this._trace.exiting(this.id, "destroyTable");
	},
	_loadRoleItem: function(contextData) {
		if (!contextData || !contextData.roleUri) {
			this.roleItem = null;
			this.showNoRoleSelectedMessagePane();
		} else {
			var roleUri = contextData.roleUri;
			var roleManager = new com.ibm.security.modeling.api.RoleRestManager({
				resourceUri: roleUri,
				syncMode: true,
				_onError : dojo.hitch(this, this._onFetchError),
				msgArea : this.messageArea
			});
			this.roleItem = roleManager.loadResource();
			this.showTablePane();
		}
	},
	_setupMessagePane:function(){
		// summary:
		//     setup the message pane if the no analysis data found
		// tags:
		//     private 
		this._trace.entering(this.id, "_setupMessagePane");
		this._trace.exiting(this.id, "_setupMessagePane");
	},
	showTablePane:function(){
		
		// Summary:
		//		Shows analysis table pane
		this._trace.entering(this.id, "showTablePane");
		this.container.selectChild(this.tablePane);
		this._trace.exiting(this.id, "showTablePane");
	},
	showNoAnalysisMessagePane:function(){
		// Summary:
		//		Shows LeanMore link if no analysis data is to be shown
		this._trace.entering(this.id, "showNoAnalysisMessagePane");
		this.onLoadingComplete();
		if(!this.learnMoreWidget){
			this.learnMoreWidget = new com.ibm.security.modeling.util.LearnMoreWidget({
				id:this.id+"_learnMore",
				message: this.analysisResources[this.noAnalysisMessage],
				linkText: this.globalResources.LEARN_MORE,
				helpFileName: this.learnMoreFile
			},this.id+"_noAnalysisMsgContainer");
			this.learnMoreWidget.startup();
		}
		this.container.selectChild(this.messagePane);
		this._trace.exiting(this.id, "showNoAnalysisMessagePane");
		//dojo.place(this.learnMoreWidget.domNode,this.roleAnalysisUserPermissions,"only");
	},
	showNoRoleSelectedMessagePane:function(){
		
		// Summary:
		//		Shows info message for role selection
		this._trace.entering(this.id, "showNoRoleSelectedMessagePane");
		this.onLoadingComplete();
		this.container.selectChild(this.noRoleMessagePane);
		this._trace.exiting(this.id, "showNoRoleSelectedMessagePane");
	},
	_onFetchError: function(err, req) {
		// summary:
		//     handles an error fetching data from the datastore.
		// description:
		//     Parses the error response text and displays the appropriate error in the
		//     table message area.
		// tags:
		//     private callback
		this._trace.entering(this.id, "_onFetchError");
		this.onLoadingComplete();
		this._errorUtil.handleRESTError(err, this.messageArea);
		this._trace.exiting(this.id, "_onFetchError");
	},
	
	_updateUIStatePerSupplementData: function() {
		if (!this.inheritanceCheckBox) {
			return;
		}
		
		var supplementData = this.getSupplementData();
		var includeInheritance = false;
		if (supplementData && ("includeInheritance" in supplementData)) {
			includeInheritance = supplementData.includeInheritance;
		} else {
			// No inheritance information in supplement data.
			// In this case, if widget exists then keep it in its state.
			if (this.inheritanceCheckBox) {
				this._updateIncludeInheritanceInSupplementData(this.inheritanceCheckBox.get("checked"));
				return;
			}
		}
		
		this.inheritanceCheckBox.setChecked(includeInheritance);
	},
	_onIncludeInheritanceChange: function(includeInheritance) {
		console.log("Include inheritance selected : "+includeInheritance);
		this._updateIncludeInheritanceInSupplementData(includeInheritance);
		this.refresh();
	},
	_updateIncludeInheritanceInSupplementData: function(/*boolean*/ includeInheritance) {
		var supplementData = this.getSupplementData();
		if (!supplementData) {
			this.setSupplementData({});
			supplementData = this.getSupplementData();
		}
		supplementData["includeInheritance"] = includeInheritance;
		this.onSupplementDataChange();
	},
	
	_getIncludeInheritance: function() {
		if(this.inheritanceCheckBox){
			return this.inheritanceCheckBox.getCheckedValue();
		} 
		var supplementData = this.getSupplementData();
		if(supplementData && "includeInheritance" in supplementData){
			return this.supplementData.includeInheritance;
		}
		return false;
	}
	
});

dojo.declare("com.ibm.security.modeling.analysis.IncludeInheritanceWidget", [ dijit._Widget, dijit._Templated],{
	
	templateString: "<span><input id='${id}_checkbox' dojoType='dijit.form.CheckBox' dojoAttachPoint='checkBox' ><label for='${id}_checkbox'>${labelText}</label></input></span>",
	widgetsInTemplate: true,
	labelText: null,
	// changeEventHandler: null,
	isChecked: false,
	_ignoreChange: false,
	postCreate : function(){
		this.inherited(arguments);
		if (this.changeEventHandler) {
			this.checkBox.onChange = dojo.hitch(this, function(newValue) {
				if (this._ignoreChange) { 
					this._ignoreChange = false;
				} else {
					this.changeEventHandler(newValue);
					this.checkBox.focus();
				}
			});
		}
		this.setChecked(this.isChecked);
	},
	getCheckedValue:function(){
		return this.checkBox.get("checked");
	},
	setChecked:function(isChecked){
		if (this.checkBox.get("checked") != isChecked) {
			this._ignoreChange = true;
			
			// setting checked attribute invokes on change handler. But on change handler is invoked using setTimeOut.
			// Handling it by _ignoreChange variable.
			this.checkBox.attr("checked",isChecked);
		}
	}
});
