dojo.provide("com.ibm.security.modeling.analysis.RoleMembersIdentityPermissionTable");

dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");
dojo.require("com.ibm.security.modeling.widgets.ProgressBar");
dojo.require("com.ibm.security.modeling.tables.RoleAnalysisTable");
dojo.require("com.ibm.security.modeling.analysis.base._AnalysisTaskPanel");

dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling.analysis", "AnalysisText");
dojo.requireLocalization("com.ibm.security.modeling.analysis", "RoleMembersIdentityPermissionTableText");

dojo.declare("com.ibm.security.modeling.analysis.RoleMembersIdentityPermissionTable",[dijit._Widget,dijit._Templated, com.ibm.security.modeling.analysis.base._AnalysisTaskPanel],{
	analysisUri: "/isrm/rest/models/${modelId}/analysis/userAssignmentAnalysis",
	restUri: null,
	messageArea: null,
	widgetsInTemplate : true,
	query: null,
	projectUri: null,
	roleUri: null,
	modelId:null,
	querySet:"roleMembers",
	isUserPermissionAnalysis: true,
	_analysisQuery: {},
	_learnMoreHelpFileName: "ui_analysis_learn_more.html",
	
	templatePath : dojo.moduleUrl("com.ibm.security.modeling.analysis","templates/IdentityPermissionAnalysisTable.html"),
	
	helpFile: "", 
	
	constructor: function(){
		//Initialize the resource bundles
		this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.analysis","RoleMembersIdentityPermissionTableText");
		this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
	},
	
	postMixInProperties : function(){
		//Here we will have the following attributes set
		//
		this.inherited(arguments);
		if(this.isUserPermissionAnalysis === true){
			this.analysisUri = "/isrm/rest/models/${modelId}/analysis/userAssignmentAnalysis";
			this.helpFile = "ui_user_to_permission.html";
		}else{
			//We have to show the permission analysis information
			this.analysisUri = "/isrm/rest/models/${modelId}/analysis/permissionAnalysis";
			this.helpFile = "ui_permission_to_user.html";
		}
	},
	
	postCreate: function(){
		this.inherited(arguments);
		//this._replaceModelId(this.modelId);
		this._getQuery();
	},
	
	startup: function() {
		this.inherited(arguments);
		this._checkIdentityPermissionMappings(this.projectUri);
	},
	
	_checkIdentityPermissionMappings : function(projUri){
		
		if(this.roleUri && this.modelId){
			if (this.table) {
				// make UI in synch with data.
	    		this.table.setIncludeInheritance(this.includeInheritance);
			}
			this.onLoadingStart();
			this._getProjectDetails(projUri);
			this.onLoadingComplete();
		}else{
			this._displayNoRoleSelectedMessage();
		}
	},
	
	_getProjectDetails : function(projectUri){
		var xhrArgs = {
                url: projectUri,
                handleAs: "json",
                sync:true,
                headers: {
					Accept:  'application/json'
				},
                load: function(data) {
                    return data;
                    
                },
                error: function(error) {
                   return error;
                }
            };
        
        var projectRequest = dojo.xhrGet(xhrArgs);
        projectRequest.addCallback(dojo.hitch(this, this._showWidget));
        projectRequest.addErrback(dojo.hitch(this, this._errorCallBack));
	},
	
	_errorCallBack:  function(error){
		dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
		var errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
		errorUtil.handleRESTError(error, this.messageArea);
	},
	
	_showWidget : function(projectData){
		if(projectData.totalIdentityPermissionMappings > 0){
			this._displayTable();
		}else{
			this._displayLearnMoreWidget();
		}	
	},
	_replaceModelId : function(modelId){
		this.restUri = this.analysisUri;
		this.restUri = this.restUri.replace("${modelId}", modelId);
		return this.restUri;
	},
	//Will be called when need to refresh panel
	//data should have essential argument needed for refresh
	_refreshWidget :function(){
	       //Reset the search field to empty
		   //this._replaceModelId(this.modelId);
		   this._getQuery();
	       this._checkIdentityPermissionMappings(this.projectUri);
	},
	_getQuery : function(){
		this._analysisQuery.roleUri = this.roleUri;
		this._analysisQuery.includeInheritance = this.includeInheritance;
		this._analysisQuery.querySet = this.querySet;
	},
	_displayLearnMoreWidget : function(){
	       if(!this.learnMoreWidget){
	       	var learnMoreWidgetMsg = this.localResources.NO_USER_PERMISSION_MAPPING_MESSAGE;
	       	var learnMoreLink = this.localResources.LEARN_MORE_LINK;
			this.learnMoreWidget = new com.ibm.security.modeling.util.LearnMoreWidget({
				message: learnMoreWidgetMsg,
				linkText: learnMoreLink,
				helpFileName: this._learnMoreHelpFileName
			});
			this.learnMoreWidget.startup();
		}
		
		dojo.place(this.learnMoreWidget.domNode,this.roleAnalysisUserPermissions,"only");
		
	},
	resize:function(){
		if(this.table) this.table.resize();
	},
	_displayTable : function(){
		try{
			
			if(!this.table){
				this._createDatastore();
				this.table = new com.ibm.security.modeling.analysis.RoleAnalysisUserPermTable({
					                id:this.id+"roleAnalysisPermissionTable",
									tableStore: this.store,
									localResources: this.localResources,
									query: this._analysisQuery,
									messageArea: this.messageArea,
									isUserPermissionAnalysis: this.isUserPermissionAnalysis,
									handleEventCallback: dojo.hitch(this, this.handleEvent),
									isTableForRoleMembers: this.querySet === 'potentialRoleMembers' ? false : true,
									includeInheritance: this.includeInheritance
						});
				dojo.place(this.table.domNode,this.roleAnalysisUserPermissions,"only");
				this.table.startup();
				this.table.resize();
			}else{
				if(this._analysisQuery!=this.table.query){
					this.table.destroy();
		    		delete this.table;
		    		this._displayTable();
				}else{
					this._setSearchInProgress(true);
					this.table.refresh();
				}
				
			}
		}catch(e){
			console.log(e);
			console.log("Here is the error");
		}
	   
	},
    _reset:function(){
		this.includeInheritance=false;
		this._analysisQuery={};
	},
	handleEvent: function(eventData){
		this.includeInheritance = eventData['includeInheritance'];
		this._updateIncludeInheritanceInSupplementData(this.includeInheritance);
		var query = {includeInheritance: this.includeInheritance};
		dojo.mixin(this._analysisQuery, query);
		var tableQuery = this.table.grid.query ||{};
		dojo.mixin(tableQuery , this._analysisQuery);
		this._setSearchInProgress(true);
		this.table.grid.setQuery(tableQuery);
	},
	
	_setSearchInProgress : function(/*boolean*/ enable){
		/**
		 * The method is added to enable the table to update the search
		 * text which is displayed as part of the results. 
		 * The method does a check to verify if the searchQuery is set on the table
		 * this is done when there is some text typed in the search field.
		 */
		if(this.table.searchQuery){
		   this.table.searchInProgress = enable;
		}
	},
	_updateIncludeInheritanceInSupplementData: function(/*boolean*/ includeInheritance) {
		var supplementData = this.getSupplementData();
		if (!supplementData) {
			this.inherited("setSupplementData",[{}]);
			supplementData = this.getSupplementData();
		}
		supplementData["includeInheritance"] = includeInheritance;
		this.onSupplementDataChange();
	},
	
	getTableId : function(){
		if(this.table)
			return this.table.id;
		else
			return null;
	},
		
	//
	// The method creates a datastore which is passed to the table.
	//
	_createDatastore: function(){
			dojo.require("com.ibm.security.modeling.tables.JsonRestStore");
			this.restUri = this._replaceModelId(this.modelId);
			this.store = new com.ibm.security.modeling.tables.JsonRestStore({
				target : this.restUri
			});
			
			var self = this;
			var getValue = this.store.getValue;
			this.store.getValue = function(/*Object*/item, /*String*/property, /*value?*/defaultValue) {
				var value = getValue(item, property, defaultValue);
				var permAnalysisProperties = { "assignmentCount":"assignmentCount", 
											   "assignmentPercentage":"assignmentPercentage",
											   "nonAssignmentCount": "nonAssignmentCount",
											   "userHavingPermissionButNotInRoleCount" : "userHavingPermissionButNotInRoleCount", 
											   "userHavingPermissionButNotInAnyRoleCount" : "userHavingPermissionButNotInAnyRoleCount",
											   "permissionNonAssignmentCount" : "permissionNonAssignmentCount", 
											   //"userHavingPermissionGrantedByRoleAndAnyOtherRoleNotInHierarchyCount": "userHavingPermissionGrantedByRoleAndAnyOtherRoleNotInHierarchyCount",
											   "overlappedUserCount": "overlappedUserCount",
											   "permissionAssignment" : "permissionAssignment"
											};
				
				var attachmentDetails = {
											"assignedToRoles" : "assignedToRoles",
											"inheritedFromRoles" : "inheritedFromRoles"
										};
				if(property in permAnalysisProperties){
					var permissionAnalysis = getValue(item, 'permissionAnalysis', null);
					return permissionAnalysis[property];
				}else if(property in attachmentDetails){
					//TODO: Change identityAttachments to attachmentDetails.
					var attachmentDetails = getValue(item, 'attachmentDetails', null);	
					if(property == 'assignedToRoles'){
						var lstAttachmentType = new Array();
						for(var iCnt = 0; iCnt < attachmentDetails.length; iCnt++){
							var attachmentType = attachmentDetails[iCnt]['attachmentType'];
							var found = false;
							for(var i = 0 ; i < lstAttachmentType.length && !found; i++){
								if(lstAttachmentType[i] === attachmentType){
									found = true;
								}
							}
							if(!found){
								lstAttachmentType.push(attachmentType);
							}
						}
						return lstAttachmentType;
					}else if(property == 'inheritedFromRoles'){
						var lstRoles = new Array();
						for(var iCnt = 0; iCnt < attachmentDetails.length; iCnt++){
							var roleDetails = attachmentDetails[iCnt]['roleDetails'];
							var attachmentType = attachmentDetails[iCnt]['attachmentType']
	 						if(attachmentType !== 'DIRECT')
	 							lstRoles.push(roleDetails['name']);
						}
						return lstRoles.length > 0 ? lstRoles.join(", ") : "";
					}
				}
				return value;
		};
		
		return this.store;
	},
	
	_displayNoRoleSelectedMessage : function(){
		//Using the AnalysisText resource bundle for fetching the "NO_ROLE_SELECTED" message. Since this message is the same for all the panels
		this.analysisResources = dojo.i18n.getLocalization("com.ibm.security.modeling.analysis","AnalysisText");
		var noRoleMsgElement = "<div id='${id}_noRolePane' class='dijitInline' style='width:100%;' ><table style='width:100%;border:0;'><tr><td width='38px'><span class='infoIcon'></span></td><td>"+this.analysisResources.NO_ROLE_SELECTED+"</td></tr></table></div>";
		dojo.place(noRoleMsgElement, this.roleAnalysisUserPermissions, "only");
	},
	
	
	//Start Overriding the methods from the _AnalysisTaskPane
	
	getTitle: function() {
		// Summary:
		//		Gets the translated title of the panel.  
		//		The Analysis pane will use this method 
		//		to updated the title on the Analysis pane when the panel is displayed.
	
		// returns: string
		//		title
		var titleText = this.isUserPermissionAnalysis === true ? this.localResources.IDENTITY_ANALYSIS_PANE : this.localResources.PERMISSION_ANALYSIS_PANE;
		if(this.roleUri){
			this._loadRoleItem();
			if (this.roleItem) {
				var roleName = this.roleItem.name;
				roleName = com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(roleName);
				titleText = dojo.string.substitute(titleText, {
					"roleName": roleName? roleName:""
					//,"attributeName": attributeName? attributeName:""		
				});
			}
		}else{
			titleText = this.isUserPermissionAnalysis === true ? this.localResources.IDENTITY_ANALYSIS_PANE_TITLE : this.localResources.PERMISSION_ANALYSIS_PANE_TITLE;
		}
			
		return titleText;
	},
	
	_loadRoleItem: function() {
		
		var roleManager = new com.ibm.security.modeling.api.RoleRestManager({
			resourceUri: this.roleUri,
			syncMode: true,
			_onError : dojo.hitch(this, this._onError),
			msgArea : this.messageArea
		});
		this.roleItem = roleManager.loadResource();
	},
	_onError: function (error, request) {
		this._errorUtil.handleRESTError(error, this.messageArea);
	},
	
	/**
	 * 
	 *   This panel should update / reload as necessary.
	 * @param contextData
	 */
	setContextData: function(/*Object*/ contextData) {
		// Summary:
		//		This method is used to set new data in the task panel.  
		//		For example if the panel is current displayed 
		//		and a new role notebook is opened.  
		//		The new role uri would be passed in.
		//
		//		This panel should update / reload as necessary.
		// contextData: Object
		//		Context data needed to display this panel
		
		this.inherited(arguments);
		
		this.__populateData(contextData);
	},
	
	setSupplementData: function(/*Object*/ supplementData) {
		// Summary:
		//		This method is used to set new data in the task panel.  
		//		For example if the panel is current displayed 
		//		and a new role notebook is opened.  
		//		The new role uri would be passed in.
		//
		//		This panel should update / reload as necessary.
		// supplementData: Object
		//		Supplement data needed to display this panel
		
		this.inherited(arguments);
		if (!supplementData) {
			supplementData = this.getSupplementData();
		}
		
		if(supplementData && "includeInheritance" in supplementData){
        	this.includeInheritance = supplementData.includeInheritance;
        } else {
        	if (this.table) { // retain the check box state.
        		this.includeInheritance = this.table.getIncludeInheritance();
        	}
        }
	},
	
	__populateData: function(/*Object*/ contextData) {
		if (!contextData) {
			contextData = this.getContextData();
		}
		
		if (!contextData) {
			return;
		}
		if(contextData.projectUrl){
			this.projectUri = contextData.projectUrl;
		}else{
			//TODO:Have a pluginResources message which should be shown in the messageArea
		}
		if(contextData.modelId){
			this.modelId = contextData.modelId;
		}
		if(contextData.roleUri){
			if(this.roleUri != contextData.roleUri){
				this.roleUri = contextData.roleUri;
				this._reset();
			}
		}
//		else{
//			//TODO:Throw an exception in the messageArea mentioning that the role is not selected.
//		}
        
	},
	
	
	refresh: function() {
		// Summary:
		//		Refreshes the analysis panel. 
		//Assuming the contextData will have the includeInheritance and roleItem as its attributes.
		this._refreshWidget();
	}
	
	//Ends Overriding the methods from th _AnalysisTaskPane
	
});

dojo.declare("com.ibm.security.modeling.analysis.RoleAnalysisUserPermTable", com.ibm.security.modeling.tables.RoleAnalysisTable,{
	tableStore: null,
	//
	//Enables to distinguish between the user permission and permisson analysis tables
	//
	isUserPermissionAnalysis: true,
	//
	//The variable enables to determine if the table is to be displayed for role members or potential role members.
	//
	isTableForRoleMembers: true,
	
	constructor : function(){
		this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
	},
	postMixInProperties: function() {
		// tags:
		// protected
		this._trace.entering(this.id, "postMixInProperties");
		this.entityTypeText= this.isUserPermissionAnalysis === true ? this.localResources.IDENTITY_ENTITY_TYPE_TEXT : this.localResources.PERMISSION_ENTITY_TYPE_TEXT;
		this.inherited(arguments);
		this._trace.exiting(this.id, "postMixInProperties");
	},
	postCreate: function(){
		
		this.structure = this._getTableStructure();
		this.dataStore = this.tableStore;
		this.toolTipRequired = this._getTooltipColumnNames();
		this.defaultSortIndex=(!this.isUserPermissionAnalysis && !this.isTableForRoleMembers) ? 2 : 1;
		this.isDefaultSortAscending= !(!this.isUserPermissionAnalysis && !this.isTableForRoleMembers);
		this.unSortableColumns= this.isUserPermissionAnalysis === true ? [5, 6] : [7, 8];
		this.inherited(arguments);
	},
	
	getInheritanceLabel: function() {
		if (this.isTableForRoleMembers) {
			if (!this.localResources) {
				this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.analysis","RoleMembersIdentityPermissionTableText");
			}
			
			if(this.isUserPermissionAnalysis == true){ 
				return this.localResources.INCLUDE_ROLE_MEMBER_INHERITANCE;
			} else {
				return this.localResources.INCLUDE_ROLE_PERMISSION_INHERITANCE;
			}			
		} else {
			this.localResourcesForPotentialMembers = dojo.i18n.getLocalization("com.ibm.security.modeling.analysis","PotentialRoleMembersIdentityPermissionTableText");
			
			if(this.isUserPermissionAnalysis == true){ 
				return this.localResourcesForPotentialMembers.INCLUDE_USER_INHERITANCE_LABEL;
			} else {
				return this.localResourcesForPotentialMembers.INCLUDE_PERMISSION_INHERITANCE_LABEL;
			}
		}
		
	},
	
	_getTableStructure: function(){
		var tableStructure = [];
		
		if(this.isUserPermissionAnalysis === true){
			tableStructure =
				[		
					{field: 'name',name: this.localResources.IDENTITY_HEADER_COLUMN_0,width: '15%', get: function(rowIndex, item){return item;}, formatter: dojo.hitch(this, this._formatName)},
					{field: 'assignmentPercentage',name:this.localResources.IDENTITY_HEADER_COLUMN_1,width: '20%', isSearchable:false, formatter: dojo.hitch(this, this._formatAssignmentPercentage)},
					{field: 'assignmentCount',name: this.localResources.IDENTITY_HEADER_COLUMN_2,width: '20%', isSearchable:false},
					{field: 'nonAssignmentCount',name: this.localResources.IDENTITY_HEADER_COLUMN_3,width: '15%', isSearchable:false},
					{field: 'assignedToRoles',name: this.localResources.IDENTITY_HEADER_COLUMN_4,width: '20%',  isSearchable:false, formatter: dojo.hitch(this, this._formatAssignedToRoles)},
					{field: 'inheritedFromRoles',name: this.localResources.IDENTITY_HEADER_COLUMN_5,width: '20%', isSearchable: this.isTableForRoleMembers}
				];
		}else{
			//Prepare the table structure for the permission analysis data
			tableStructure =
				[		
				 	{field: 'name',name: this.localResources.PERMISSION_HEADER_COLUMN_0,width: '10%', get: function(rowIndex, item){return item;}, formatter: dojo.hitch(this, this._formatName)},
					{field: 'permissionNonAssignmentCount',name: this.localResources.PERMISSION_HEADER_COLUMN_1,width: '10%', isSearchable:false},
					{field: 'permissionAssignment',name:this.localResources.PERMISSION_HEADER_COLUMN_2,width: '10%', isSearchable:false},
					{field: 'userHavingPermissionButNotInRoleCount',name: this.localResources.PERMISSION_HEADER_COLUMN_3,width: '15%', isSearchable:false},
					{field: 'userHavingPermissionButNotInAnyRoleCount',name: this.localResources.PERMISSION_HEADER_COLUMN_4,width: '20%', isSearchable:false},
					//{field: 'userHavingPermissionGrantedByRoleAndAnyOtherRoleNotInHierarchyCount',name: this.localResources.PERMISSION_HEADER_COLUMN_5,width: '15%', isSearchable:false},
					{field: 'overlappedUserCount',name: this.localResources.PERMISSION_HEADER_COLUMN_5,width: '15%', isSearchable:false},
					{field: 'assignedToRoles',name: this.localResources.PERMISSION_HEADER_COLUMN_6,width: '10%',  isSearchable:false, formatter: dojo.hitch(this, this._formatAssignedToRoles)},
					{field: 'inheritedFromRoles',name: this.localResources.PERMISSION_HEADER_COLUMN_7,width: '10%', isSearchable: this.isTableForRoleMembers}

				];
		}
		  
		return tableStructure;
	},

	
	_formatName : function(item, rowIndex, cell){
		var anchor=null;
		if(item){
		 
	          anchor=new com.ibm.security.modeling.util.Anchor({
					input:item.name,
					callback:dojo.hitch(this, this._showAttributesDialog,item)
				});											
	    }
		return anchor;
	},
	
	_formatAssignmentPercentage : function(value, rowIndex, cell){
		dojo.require("dijit.ProgressBar");
			var bar;
			if(value){
				bar=new dijit.ProgressBar({maximum: 100,progress:value, width:"90%"});
			}else{
				bar=new dijit.ProgressBar({maximum: 100,progress:0, width:"90%"});
			}
			return bar;
	},
	
	//Displays the attributes dialog for the identity OR permission entity types.
	_showAttributesDialog : function(item){
		var entityType = "";
		var attributeField="";
		
		if(this.isUserPermissionAnalysis === true){
			entityType = "USER";
			attributeField = "user";
		}else{
			entityType = "PERMISSION";
			attributeField = "permission";
		}
		dojo.require("com.ibm.security.modeling.widgets.PropertiesDialog");
		this._attributesDialog=new com.ibm.security.modeling.widgets.PropertiesDialog({
           	type:entityType,
           	panelHeaderName:item.name,
            entityItem:item[attributeField]
           },null);
		
       this._attributesDialog.show();
       this._attributesDialog.startup();
	},
	
	_formatAssignedToRoles : function(item,rowIndex,cell){
		var assignedToRoles = new Array();
		if(item instanceof Array){
			for(var i = 0; i < item.length; i++){
				var attachmentType = item[i];
				var localResourceKey = "ATTACHMENT_TYPE_"+attachmentType;
				if(this.localResources[localResourceKey])
					assignedToRoles.push(this.localResources[localResourceKey]);
				else
					assignedToRoles.push(attachmentType);
			}
			return assignedToRoles.length > 0 ? assignedToRoles.join(",") : "";
		}else
			return item;
	},
	
	_getSearchInstructions : function(){
		return this.localResources.TABLE_SEARCH_INSTRUCTION;
	},
	
	_getTooltipColumnNames : function(){
		var toolTipColNames = [];
		if(this.isUserPermissionAnalysis === true){
			toolTipColNames = [1, 2, 3, 4, 5, 6];
			
		}else{
			toolTipColNames = [1, 2, 3, 4, 5, 6, 7, 8];
			
		}
		return toolTipColNames;
	},
	
	_onFetchError: function(err, req) {
			// summary:
			//     handles an error fetching data from the datastore.
			// description:
			//     Parses the error response text and displays the appropriate error in the
			//     table message area.
			// tags:
			//     private callback
			
			this._errorUtil.handleRESTError(err, this.messageArea);
	}
});
