dojo.provide("com.ibm.security.modeling.analysis.UserAttributeAnalysisTable");

dojo.require("com.ibm.security.modeling.tables.CustomDisplayStructureFactory");
dojo.require("com.ibm.security.modeling.analysis.AnalysisTableWidget");
dojo.require("com.ibm.security.modeling.api.RoleRestManager");
		
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.requireLocalization("com.ibm.security.modeling.tables", "UserTableText");


dojo.declare("com.ibm.security.modeling.analysis.UserAttributeAnalysisTable", 
		com.ibm.security.modeling.analysis.AnalysisTableWidget , {	
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.analysis.UserAttributeAnalysisTable"),
	// type: String
	//     Possible values USER/PERMISSION
	type: "USER",
	// noAnalysisMessage: String
	//     Log message key if no analysis is present
	noAnalysisMessage: "NO_ANALYSIS_SCHEMA",
	// helpUrl: String
	//     html file name for Learn More link
	helpFile:"ui_user_attributes_analysis.html",
	// learnMoreFile: String
	//     html file name for Learn More link
	learnMoreFile:"tsk_attributes_analysis_show_analysis_pane.html",
	// analysisUri: String
	//     url to query for getting the analysis data
	analysisUri:"/isrm/rest/models/{modelId}/analysis/attributeAnalysis",
	// entityType: String
	//     Entity type for table and query param for analysis URL
	entityType:"user",
	// querySet: String
	//     Query param for analysis URL, provides which data set to fetch
	querySet:"roleMembers",
	isInheritanceRequired:true,
	panelTitleKey:"USER_ATTRIBUTE_ANALYSIS_TITLE",
	isInheritedFromSearchable : true,
	drilDownAttribute:null,
	postMixInProperties: function(args) {
		// tags:
		// protected
		this._trace.entering("", "postMixInProperties");
		this._trace.object(args, "postMixInProperties", "postMixInProperties args");
		this.userTableResource = dojo.i18n.getLocalization("com.ibm.security.modeling.tables", "UserTableText");
		this.inherited(arguments);
		this.searchInstructions=this.userTableResource.ALL_ROLES_USERTABLE_SEARCH;
		this.defaultSortIndex=2;
		this.customCols={12:{field: 'assignedToRole',name: this.analysisResources.ASSIGNED_TO_ROLE,width: '8%', formatter: dojo.hitch(this, this._formatAssignedToRoleColumn),isSearchable:false},
        		13:{field: 'inheritedFrom',name: this.analysisResources.INHERITED_FROM,width: '10%', formatter: dojo.hitch(this, this._formatInheritedFromRolesColumn),isSearchable:this.isInheritedFromSearchable}};
		this.structure = this._getStructure();
		//this.title=this.analysisResources.USER_ATTRIBUTE_ANALYSIS;
		
		this._trace.exiting("", "postMixInProperties");
		
	},	
	startup:function() {
		// tags:
		// protected
		this._trace.entering(this.id, "startup");
		this.inherited(arguments);
		if(this.roleUri && this.modelId){
			this.showTablePane();
			this.refresh();
		}else{
			this.showNoRoleSelectedMessagePane();
		}
		this._trace.exiting(this.id, "startup");
	},
	refresh: function() {
		// Summary:
		//		Refreshes the analysis panel. 
		this._trace.entering(this.id, "refresh");
		if(this.contextData.modelId && this.contextData.roleUri){
			this.showTablePane();
			this.inherited(arguments);
		}else{
			this.showNoRoleSelectedMessagePane();
		}
		this._trace.exiting(this.id, "refresh");
	},
	createAnalysisUri : function(){
		this._trace.entering(this.id, "createAnalysisUri");
		this.analysisUri = "/isrm/rest/models/"+this.modelId+"/analysis/attributeAnalysis";
		this._trace.exiting(this.id, "createAnalysisUri");
	},
	/**
	 * 
	 *   This panel should update / reload as necessary.  
	 * 
	 * @param contextData
	 */
	setContextData: function(/*Object*/ contextData) {
		// Summary:
		//		This method is used to set new data in the task panel.  
		//		For example if the panel is current displayed 
		//		and a new role notebook is opened.  
		//		The new role uri would be passed in.
		//
		//		This panel should update / reload as necessary.
		// contextData: Object
		//		Context data needed to display this panel
		this._trace.entering(this.id, "setContextData");
		this.inherited(arguments);
		if(this.contextData){
			if(this.contextData.modelId){
				this.modelId = this.contextData.modelId;
			}
			if(this.contextData.roleUri){
				this.roleUri = this.contextData.roleUri;
			}
		}
		this._trace.exiting(this.id, "setContextData");
	},
	
	_updateTitle: function() {

		var titleText;
		this._loadRoleItem(this.getContextData());
		if (this.roleItem) {
			var roleName=this.htmlUtilities.parseHTMLandNormalize(this.roleItem.name);
			titleText = dojo.string.substitute(this.analysisResources[this.panelTitleKey], {
				"roleName": roleName? roleName:""
			});
		}
		this.title = titleText;
	},
	_getStructure:function(){
		// Summary:
		//		This method is used to fetch the dynamic table structure from Custom display factory.  
		//		Also sets up the formatters for table schema columns
		this._trace.entering(this.id, "_getStructure");
       var cols= com.ibm.security.modeling.tables.CustomDisplayStructureFactory.loadUserAttributeAnalysisStructure(this.customCols,dojo.hitch(this,this._onFetchError));
		if(cols.length ===3){
			return null;
		}
		var i;
		
		for(i=1;i<cols.length;){
			if(cols[i].field === "assignedToRole" || cols[i].field === "inheritedFrom"){
				break;
			}else{
				cols[i++].formatter=dojo.hitch(this, this._formatAttributeValues);
				cols[i].name=this.globalResources.PERCENTAGE;
				cols[i].toolTipText=this.analysisResources.PERCENTAGE_TOOLTIP;
				cols[i++].formatter=dojo.hitch(this, this._formatAttributePercentage);
			}
			
		}
		if(cols.length === 13){
			this._attribColsAreFull=true;
		}
		this.unSortableColumns=[cols.length-1,cols.length];
		cols=this._setupStructure(cols);
		this.oldStructure=cols;
		this._trace.exiting(this.id, "_getStructure");
		return cols;
	},
	
	_loadTable:function(e){
		this._trace.entering(this.id, "_loadTable");
		
		var connection = dojo.connect(this, "_handleFetchComplete", this, dojo.hitch(this, function(items) {
			//dojo.disconnect(connection);
			if(this.drilDownAttribute && this.table){
				this.table.searchField.selectColumnCheckBox(this.supplementData.attributeName);
				this.table.searchField.setSearchValue(this.supplementData.attributeValue);
				//this.table.mockSearch(this.supplementData.attributeName,this.supplementData.attributeValue);
				this._mockSearchResultsDisplay(items,this.supplementData.attributeValue);
			}
		}));
		
		this.inherited(arguments);
		this._trace.exiting(this.id, "_loadTable");
	},
	clearDrillDownSearchQuery:function(){
		if(this.drilDownAttribute && this.tableQuery){
			delete this.tableQuery[this.drilDownAttribute];
			this.drilDownAttribute=null;
		}
	},
	_mockSearchResultsDisplay:function(items,searchString){
		var numberOfResults = this.table.grid.pagination.plugin.getTotalRowCount();
		var displayTemplate = this.table.tableResources.SEARCH_DISPLAY;
		var displayText = dojo.string.substitute(displayTemplate, {
				"numResults":numberOfResults, 
				"entityType": this.table.entityTypeText, 
				"searchString": this.table.htmlUtilities.parseHTMLTag(searchString)
		});
			this.table.searchResultsDisplay.innerHTML = displayText;
			dojo.removeClass(this.table.searchResultsDisplay, "searchResultsHidden");
			dojo.addClass(this.table.searchResultsDisplay, "searchResultsVisible");	
		
		if (items.length === 0) {
			dojo.addClass(this.table.grid.messagesNode,"noDataMessageStyle");
			this.table.grid.messagesNode.innerHTML = this.table.tableResources.NO_DATA;
		} else {
			dojo.removeClass(this.table.grid.messagesNode,"noDataMessageStyle");
		}
		this.table.resize();
	},
	_setUpTableQuery : function(){
		// Summary:
		//		Setups the query parameters that needs to be send to REST
		//		adds columns used for displaying the table
		this._trace.entering(this.id, "_setUpTableQuery");
		this.inherited(arguments);
		
		var i;
		if(!this.structure){
			return;
		}
		if(!this.contextData || !this.contextData.roleUri || !this.contextData.modelId){
			this.reset();
		}
		if(this.contextData && this.contextData.roleUri){
			this.tableQuery.roleUri=this.contextData.roleUri;
			this.showTablePane();
		}else if(this.roleItem && this.roleItem.uri){
			this.tableQuery.roleUri=this.roleItem.uri;
		}else{
			this.showNoRoleSelectedMessagePane();
			return;
		}
		if(this.contextData && this.contextData.modelId){
			this.modelId=this.contextData.modelId;
			this.createAnalysisUri();
		}
		
		var index = 1;
		var struct;
		this.clearDrillDownSearchQuery();
		if(this.supplementData && this.supplementData.attributeName){
			this.tableQuery[this.supplementData.attributeName]=this.supplementData.attributeValue;
			this.drilDownAttribute=this.supplementData.attributeName;
			this.updateStructure(this.supplementData.attributeName);
			struct=this.structure;
		}else{
			struct=this.oldStructure;
		}
		if(struct){
			var i;
			var len=struct.length;
			var perColWidth=100/len;
			for(i = 0 ; i < len ; i++){
				struct[i].width=perColWidth+"%";
				if((struct[i].field).search(/InPercent/) !== -1  || struct[i].field === "inheritedFrom" || struct[i].field === "name" || struct[i].field === "assignedToRole"){
					continue;
				}
				this.tableQuery["col"+(index++)]= struct[i].field;
			}
		}
		
		this._updateTableQuery();
		
		this._trace.exiting(this.id, "_setUpTableQuery");
	},
	
	_updateTableQuery: function() {
		if (this.table) {
			this.table.grid.query=this.tableQuery;
			this.table.query=this.tableQuery;
			if(this.table.isPaginationEnabled && this.table.grid.pagination && this.table.grid.pagination.plugin){//setting pagination query as well...coz it gets init at time of create only
				this.table.grid.pagination.plugin.query=this.tableQuery;
			}
		}
	},
	
	_updateTableStructure: function() {
		var struct;
		if(this.supplementData && this.supplementData.attributeName){
			struct=this.structure;
		}else{
			struct=this.oldStructure;
		}
		this.setTableStructure(struct);
	},
	
	updateStructure:function(attributeName){
		var isAttribColPresent=false;
		if(this.structure){
			this.structure=this.oldStructure;
			var newCols=[];
			var i;
			for(i=0;i<this.structure.length;){
				if(i===1){
					var newAttribColumn={};
					newAttribColumn.name=this.htmlUtilities.parseHTMLandNormalize(attributeName);
					newAttribColumn.field=attributeName;
					newAttribColumn.width="10%";
					newAttribColumn.formatter=dojo.hitch(this, this._formatAttributeValues);
					newAttribColumn.isSearchable=true;
					newCols.push(newAttribColumn);
					var newAttribePercentColumn={};
					newAttribePercentColumn.name=this.globalResources.PERCENTAGE;
					newAttribePercentColumn.field=attributeName+"InPercent";
					newAttribePercentColumn.width="10%";
					newAttribePercentColumn.toolTipText=this.analysisResources.PERCENTAGE_TOOLTIP;
					newAttribePercentColumn.formatter=dojo.hitch(this, this._formatAttributePercentage);
					newAttribePercentColumn.isSearchable=false;
					newCols.push(newAttribePercentColumn);
				}
				if(this.structure[i].field === attributeName){
						i=i+2;
						isAttribColPresent=true;
						continue;
						
				}
				if(this._attribColsAreFull && !isAttribColPresent && i === 9){
					i=i+2;
					continue;
				}
				newCols.push(this.structure[i]);
				i++;
			
			}
			if(this.table){
				this.table.unSortableColumns=[newCols.length-1,newCols.length];
			}
			this.structure=newCols;
		}
		
	},
	setTableStructure:function(structure){
		if(this.table && structure){
			this.table.grid.set('structure', structure);
			this.table.searchField.setTable(this.table.grid);
		}
	}
});