/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

� Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.analysis.base._AnalysisPaneUtil");


dojo.require("com.ibm.security.modeling.util.Trace");

(function() {
	// 'var' should be used to declare local variables.
	var trace = new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.analysis.base._AnalysisPaneUtil");
	
	// Declaration in global scope.
	var _apu = com.ibm.security.modeling.analysis.base._AnalysisPaneUtil = {
			
		isContextDifferent: function(/*Object*/ oldContextData, /*Object*/ newContextData, /*boolean?*/ strict, /*boolean?*/ attributeCheckReq,/*boolean?*/ needInheritanceCheck) {
			// Summary:
			//		Determines if refresh is needed.
			//		Decision depends upon old and new context data.
			//		If context data is changed then refresh is needed.
			
			if ((!oldContextData && newContextData)
					||(oldContextData && !newContextData)) {
				return true;
			}
			if (oldContextData && newContextData) {
				
				if (oldContextData["modelId"] != newContextData["modelId"]) {
					return true;
				}
				
				if (strict !== true) {
					strict = false;
				}
				var i;
				var attributeToCheck=["roleUri"];
				if(attributeCheckReq){
					attributeToCheck.push("attributeName");
					attributeToCheck.push("attributeValue");
				}
				
				if(needInheritanceCheck !==false){
					needInheritanceCheck=true;
				}
				
				for(i=0;i<attributeToCheck.length;i++){
					var attrib=attributeToCheck[i];
					if ((oldContextData[attrib] && newContextData[attrib] && (oldContextData[attrib] != newContextData[attrib])) // Role changed.
						|| (!strict && !oldContextData[attrib] && newContextData[attrib])
						|| (strict && (
								(!oldContextData[attrib] && newContextData[attrib]) ||
								oldContextData[attrib] && !newContextData[attrib])
							)) 
					{
						
						return true;
					}
				}
				if(needInheritanceCheck){
					var inheritanceValue="includeInheritance";
					if (( inheritanceValue in oldContextData && inheritanceValue in newContextData && (oldContextData[inheritanceValue] != newContextData[inheritanceValue])) // Inheritance changed.
						|| (!(inheritanceValue in oldContextData) && (inheritanceValue in newContextData))
						|| ((inheritanceValue in oldContextData) && !(inheritanceValue in newContextData))
						) 
					{
						
						return true;
					}
				}
			}
			
			return false;
		},
		
		extractContextData: function(/*Object?*/ data) {
			trace.entering("", "extractContextData");
			
			var contextData = {};
			if (data) {
				var attributes = ["projectUrl","modelId","roleUri"];
				var attribute = null;
				var attributeValue = null;
				for (var i=0; i<attributes.length; i++) {
					attribute = attributes[i];
					attributeValue = data[attribute];
					if (attributeValue) {
						contextData[attribute] = attributeValue;
					}
				}
			}
			
			trace.exiting("", "extractContextData");
			return contextData;
		},
		
		extractSupplementData: function(/*Object?*/ data) {
			trace.entering("", "extractSupplementData");
			
			var supplementData = null;
			if (data) {
				supplementData = dojo.clone(data);
				
				var attributesToExclude = ["modelId", "roleUri", "messageArea", "class"];
				for (var i=0; i < attributesToExclude.length; i++) {
					var attribute = attributesToExclude[i];
					if (attribute in supplementData) {
						supplementData[attribute] = null;
						delete supplementData[attribute];
					}
				}
			} else {
				supplementData = {};
			}
			trace.exiting("", "extractSupplementData");
			return supplementData;
		},
		
		cleanseRecentViewItemData: function(/*Object?*/ data) {
			if (data) {
				var attributesToRetain = ["projectUrl", "modelId", "roleUri", "includeInheritance"];
				var retain = false;
				for (attribute in data) {
					retain = false;
					for (var i=0; i<attributesToRetain.length; i++) {
						if (attributesToRetain[i] == attribute) {
							retain = true;
							break;
						}
					}
					if (!retain) {
						data[attribute] = null;
						delete data[attribute];
					}
				}
			}
		}
		
	};
})();