/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

(c) Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.analysis.base._AnalysisTaskPanelController");


dojo.require("com.ibm.security.modeling.analysis.base._AnalysisTaskPanelFactory");
dojo.require("com.ibm.security.modeling.util.Trace");

/**
 * Main controller class for interactions between 
 * the Analysis Task panel and Analysis pane
 */
dojo.declare("com.ibm.security.modeling.analysis.base._AnalysisTaskPanelController", null,  {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.analysis.base._AnalysisTaskPanelController"),
	
	// _analysisTaskPanelCache
	//currentTaskFactory: null,
	//currentTask: null,
	
	_loadAnalysisTask: function(/*_AnalysisTaskPanelFactory?*/ taskFactory, /*Object?*/ analysisTaskData) {
		if (taskFactory) { // factory and/or analysisTaskData
			this._createAnalysisTask(taskFactory, analysisTaskData);
		} else if (analysisTaskData) { // contains context data
			this._updateAnalysisTask(analysisTaskData);
		} else { // no factory, no context data
			this.refreshCurrentTask();
		}
	},
	
	_createAnalysisTask: function(/*_AnalysisTaskPanelFactory*/ taskFactory, /*Object?*/ analysisTaskData) {
		// Summary:
		//		Displays the analysis tab with panel created with the specified panel factory
		//		and sets the 'currentTaskFactory' to the passed in factory. 
		//		
		//		Tasks launched from the analysis catalog before the related entity such as a role
		//		has been opened would call this method opening the task in its 'base' state.
		//		Each of the analysis panels should support a state before the relevant context data or state
		//		is available.
		//
		//	taskFactory: _AnalysisTaskPanelFactory
		
		if (!this._analysisTaskPanelCache) {
			this._analysisTaskPanelCache = {};
		}
		
		if (taskFactory) {
			this.__currentTaskFactory = taskFactory;
		}
		
		var taskPanel = null;
		if (this.__currentTaskFactory) {
			taskPanel = this._analysisTaskPanelCache[this.__currentTaskFactory.id];
			if (taskPanel) {
				this._setAnalysisTask(taskPanel);
				this._updateAnalysisTask(analysisTaskData);
			} else {
				analysisTaskData = this._getTaskPanelInitData(analysisTaskData);
				taskPanel = this.__currentTaskFactory.createAnalysisTaskPanel(analysisTaskData, this._getAnalysisTaskPaneAttachPoint(this.__currentTaskFactory.id+"_panel"));
				this._setAnalysisTask(taskPanel);
				this._analysisTaskPanelCache[this.__currentTaskFactory.id] = taskPanel;
				this.connect(taskPanel, "onLoadingComplete", this._updateTitle);
				this.connect(taskPanel, "onLoadingComplete", this._updateRecentlyViewedTasksList);
				this.connect(taskPanel, "onSupplementDataChange", function() { this._updateAnalysisDataOfRecentViewItem(taskPanel); });
				
			}
		}
	},
	
	_updateAnalysisTask: function (/*Object*/ analysisTaskData) {
		// Summary:
		//		Displays the analysis tab using the current taskFactory 
		//		to create the panel to display with the updated context data.
		//		This method would be used when the context data has changed
		//		such as opening the role notebook
		//		and we simply want to show the current analysis task
		//		with the updated data.  
		//		
		//	analysisTaskData: Object
		
		var currentTask = this._getAnalysisTask();
		if (currentTask) {
			var oldData = currentTask.getAnalysisTaskData();
			
			if (currentTask.needsRefresh) {
				currentTask.needsRefresh = false;
				this._needsRefresh = true;
			} else if (this._needsRefresh == false) {
				var attributeCheck = false;
				if(currentTask.isAttributeCheckRequired){
					attributeCheck = true;
				}
				this._needsRefresh = com.ibm.security.modeling.analysis.base._AnalysisPaneUtil.isContextDifferent(oldData,analysisTaskData,true,attributeCheck,false);
			}
			
			if (currentTask.postReset && !("includeInheritance" in analysisTaskData)) {
				currentTask.postReset = false;
				analysisTaskData["includeInheritance"] = false;
			}
			// Update context data of the current task and refresh.
			currentTask.setAnalysisTaskData(analysisTaskData);
			if (this._needsRefresh == true) {
				this.refreshCurrentTask();
			} else {
				// No refresh needed. It can be considered that loading is done.
				currentTask.onLoadingComplete();
			}
		} else {
			// Create analysis task using existing factory.
			this._createAnalysisTask(null, analysisTaskData);
		}
	},
	
	refreshCurrentTask: function() {
		// Summary:
		//		Invokes refresh on the current analysis task panel
		
		var currentTask = this._getAnalysisTask();
		if (currentTask) {
			currentTask.refresh();
		}
	},
	
	_getTaskPanelInitData: function(/*Object?*/ analysisTaskData) {
		if (!analysisTaskData) {
			analysisTaskData = {};
		}
		return analysisTaskData;
	},
	
	_clearCachedData: function() {
		// Summary:
		//		Clears the cache.
		
		if (this._getAnalysisTask()) {
			this._setAnalysisTask(null);
		}
		if (this.__currentTaskFactory) {
			this.__currentTaskFactory = null;
		}
	},
	
	_getAnalysisTask: function() {
		// Summary:
		//		Provides the analysis task widget to display.
		
		return this.__currentTask;
	},
	
	_setAnalysisTask: function(currentTask) {
		this.__currentTask = currentTask;
	},
	
	reset: function() {
		var taskPanel = this._getAnalysisTask();
		if (!taskPanel) {
			return;
		}
	
		taskPanel.reset();
		
		if (this._analysisTaskPanelCache) {
			for (var taskId in this._analysisTaskPanelCache) {
				var task = this._analysisTaskPanelCache[taskId];
				task.postReset = true;
			}
		}
	}
});
