/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

(c) Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.analysis.base._AnalysisTaskPanelFactory");

dojo.require("com.ibm.security.modeling.analysis.base._AnalysisTaskPanel");
dojo.require("com.ibm.security.modeling.util.Trace");

/**
 * A type that all analysis task panel factories must extend. 
 */
dojo.declare("com.ibm.security.modeling.analysis.base._AnalysisTaskPanelFactory", null,  {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.analysis.base._AnalysisTaskPanelFactory"),
	
	constructor: function() {
		// panelType: _AnalysisTaskPanel
		//panelData: Custom Panel data which needs to be passed while initializing the panel.
		//		Panel type instantiated by this factory.
		dojo.mixin(this, arguments[0]);
		
		// Factory should create instance of only those types, which extend _AnalysisTaskPanel
		if (!this.panelType) {
			this._setDefaultPanelType();
		}
	},
	
	_setDefaultPanelType: function() {
		this.panelType = com.ibm.security.modeling.analysis.base._AnalysisTaskPanel;
	},
	
	createAnalysisTaskPanel: function(/*Object?*/ initData, attachPoint) {
		// Summary:
		//		Creates instance of type per the this.panelType
		//		And sets the contextData
		//
		// initData: Object
		//		Data for panel.
		//
		// returns:	_AnalysisTaskPanel
		
		var panelData = this._extractPanelData(initData);
		panelData["factory"] = this;
		this._cleanInitData(initData);
		panelData["analysisTaskData"] = initData;
		
		var panel = null;
		if (attachPoint) {
			attachPoint = dojo.byId(attachPoint);
		}
		
		if (attachPoint) {
			panel = new this.panelType(panelData, attachPoint);
		} else {
			panel = new this.panelType(panelData);
		}
		
		if (panel && !panel.isInstanceOf(com.ibm.security.modeling.analysis.base._AnalysisTaskPanel)) {
			this._setDefaultPanelType();
			
			if (dojo.isFunction(panel.destroyRecursive)) {
				panel.destroyRecursive();
			} else if (dojo.isFunction(panel.destroy)) {
				panel.destroy();
			}
			panel = null;
			
			panel = new this.panelType(panelData);
		}
		return panel;
	},
	
	_extractPanelData: function(/*Object?*/ initData) {
		// initData: Object
		//		Data for panel.
		//
		var panelData = this._getPanelData();
		          
		if (initData) {
			var attributes = ["messageArea", "class"];
			for (var i=0; i<attributes.length; i++) {
				var attribute = attributes[i];
				if (initData[attribute]) {
					panelData[attribute] = initData[attribute];
				}
			}
		}
		
		return panelData;
	},
	
	_cleanInitData: function(/*Object?*/ initData) {
		// initData: Object
		//		Data for panel.
		//
		
		if (!initData) {
			return;
		}
		
		var attributes = ["messageArea", "class"];
		for (var i=0; i<attributes.length; i++) {
			var attribute = attributes[i];
			if (initData[attribute]) {
				initData[attribute] = null;
				delete initData[attribute];
			}
		}
	},
	
	_getPanelData : function(){
		var panelData = {};
		return panelData;
	}
});
