dojo.provide("com.ibm.security.modeling.api.HttpOverrideRestServiceFactory");

dojo.declare("com.ibm.security.modeling.api.HttpOverrideRestServiceFactory", null, {
	createService: function(targeturl, postData) {
		//summary:
		//	Creates a REST service that overrides the get to do a post using http-override.
		//description:
		//	We had to extend Rest.js in a non-standard way, since Rest is essentially a function that uses private scoped variables.
		//	To override the function we had to create our own and mix in the Rest attributes.  The side affect is we currenly can't
		//  use put/post/delete operations because they currenly leverage a private variable to do their work.
		//targeurl: String
		//	URL path to the list of resources
		//postData: Object
		//	Query information to be put in the post.
		this.path = targeturl;
		var jsonRestService = dojox.rpc.Rest(targeturl, true, null, dojo.hitch(this, this.getRequest));		
		var extendedService;
		var withFullCount = this.addCountToResponseObject;
		//we want our service to do a post for its default query instead of a get.
		extendedService = function(id, args){
			args = args || {};
			return withFullCount(dojo.xhrPost(extendedService._getRequest(id, args)), extendedService, (args.start >= 0 || args.count >= 0), id);
		};
		//add the rest service's supporting methods and variables to our post service
		dojo.mixin(extendedService, jsonRestService);
		//format the data to post on the request.
		this.postData = dojo.toJson(postData);
		return extendedService;
	},
	addCountToResponseObject: function(deferred, service, range, id){
		deferred.addCallback(function(result){
			if(deferred.ioArgs.xhr && range){
					rangeHeader = deferred.ioArgs.xhr.getResponseHeader("Content-Range");
					//content range header is in the form of Content-Range: 1-100/240 - we want the last number so we grab the text after the '/'
					deferred.fullLength = rangeHeader && (rangeHeader=rangeHeader.match(/\/(.*)/)) && parseInt(rangeHeader[1]);
			}
			return result;
		});
		return deferred;
	},
	
	getRequest: function(/*String*/id, /*Object*/args){
		//summary:
		//	Creates the Http request for the query post.
		//id: String
		//	data to put on the request's query params
		//args: Object
		//	data about the request to be added to the object
		id = this._makeQueryString(id, args);
		var request = this._createRequestObject(id, args);		
		dojox.rpc._sync = false;
		return request;//Object
	},
	
	_makeQueryString:function(id, args){
		if(dojo.isObject(id)){
			id = dojo.objectToQuery(id);
			id = id ? "?" + id: "";
		}
		id = this._addSortSpecToQueryString(id, args);
		return id;
	},
	
	_addSortSpecToQueryString:function(id, args) {
		if(args && args.sort && !args.queryStr){
			id += (id ? "&" : "?") + "sort(";
			for(var i = 0; i<args.sort.length; i++){
				var sort = args.sort[i];
				id += (i > 0 ? "," : "") + (sort.descending ? '-' : '+') + encodeURIComponent(sort.attribute); 
			}
			id += ")";
		}
		return id;
	},
	
	_createRequestObject:function(id, args) {
		var request = {
				url: this.path + (id == null ? "" : id),
				handleAs: 'json', 
				contentType: 'application/json',
				sync: dojox.rpc._sync,
				postData: this.postData,
				headers: {
					"X-HTTP-Method-Override": "GET",
					Accept: 'application/json,application/javascript'
				}
			};
		request = this._setupRangeHeader(request, args);
		return request;
	},
	
	_setupRangeHeader:function(request, args) {
		if(args && (args.start >= 0 || args.count >= 0)){
			request.headers.Range = "items=" + (args.start || '0') + '-' + ((args.count && args.count != Infinity && (args.count + (args.start || 0) - 1)) || '');
		}
		return request;
	}

});