/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

� Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.api.LifecycleRestManager");

dojo.require("com.ibm.security.modeling.api.RestManager");
dojo.require("com.ibm.security.modeling.util.Trace");

dojo.declare("com.ibm.security.modeling.api.LifecycleRestManager",com.ibm.security.modeling.api.RestManager,{
	
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.api.LifecycleRestManager"),
	
	constructor: function(args) {
		// summary:
		//     A reusable lifecycle rest manager widget
		this._trace.entering("", "constructor");
		
		dojo.mixin(this, args);
		
		this._trace.exiting("", "constructor");
	}, 

	submitRequestResource: function (
			/*long*/ modelId,
			/*string*/ entity,
			/*string*/ requestType,
			/*Array*/ entityItems,
			/*Stirng*/ comments,
			/*function (item, request)*/ successCallBack, 
			/*function (error, request)*/ errorCallBack) {
		// summary:
		//		Submit a lifecycle request using the incoming request type
		// modelId: long
		//     modelId id to be used to create URI.
		// entity: string
		//     entity string to be submitted.
		// requestType: string
		//     request type string to be submitted.
		// entityItems: Array
		//     entityItems array contains reference to entity to be submitted.
		// comments: string
		//     comments string to be submitted.
		// successCallBack:	function (item, request)
		//		callback function, used when syncMode is false, used to handle fetch success result
		// errorCallBack: function (error, request)
		//		callback function, used when syncMode is false, used to handle fetch error result
		
		var data = "";
		
		if (requestType && entityItems){
			var itemArray = [];
			for(var i=0;i<entityItems.length;i++){
				itemArray.push({uri:this.getUri(modelId,entity,entityItems[i].id)});
			}
			
			data = {
					type: requestType,
					comments: comments,
					items: itemArray
				};
		}
		
		this.doHttpPost(dojo.toJson(data),
				successCallBack,
				errorCallBack
			);
	},
	
	getUri : function (
			/*long*/ modelId,
			/*string*/ entity,
			/*string*/ entityId){
		// summary:
		//		To create entity uri to be submitted as entity reference.
		return "/isrm/rest/models/"+modelId+"/"+entity+"/"+entityId;
	}
	
	
});