/*
Licensed Materials - Property of IBM

Role and Policy Modeler
Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.api.ProjectRestManager");

dojo.require("com.ibm.security.modeling.api.RestManager");

dojo.declare("com.ibm.security.modeling.api.ProjectRestManager", 
	com.ibm.security.modeling.api.RestManager, 
	{
	
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.api.ProjectRestManager"),
	
	URI_PROJECTS_REST_STATUS_RESOURCE: "/isrm/rest/projects/status",
	
	constructor: function(args) {
	    if (this.resourceUri == null) {
		    this.resourceUri = "/isrm/rest/projects/";
	    }
		//dojo.mixin(this, args);
	    
	    /*
	    this._resourceSchema = {
			"description":"A Project",
			"type":"object",
			
			"properties": {
				"uri":{"type":"string","format":"url"},
				"id":{"type":"integer"},
				"name":{"type":"string"},
				"description":{"type":"string"},
				"lastModifiedOn":{"type":"date"},
				"coveragePercentage":{"value":"number"},
				"coveredPoliciesCount":{"type":"integer"},
				"sodRoleConstraintCount":{"type":"integer"},
				"coveredRolesCount":{"type":"integer"},
				"totalIdentityCount":{"type":"integer"},
				"totalPermissionCount":{"type":"integer"}
			}
		};
		*/
	},

	// Create a new role as root or under a given parent
	createProjectResource: function (
			/*string*/ projectName,
			/*string*/ projectDescription,
			/*Object*/ userScopingRuleItems,
			/*Object*/ permissionScopingRuleItems,
			/*function (item, request)*/successCallBack, 
			/*function (error, request)*/errorCallBack) {
		// summary:
		//	Creates a new role resource using the incoming name 
		// successCallBack:	function (item, request)
		//		callback function, used when syncMode is false, used to handle fetch success result
		// errorCallBack: function (error, request)
		//		callback function, used when syncMode is false, used to handle fetch error result
		var data = "";
	
		// Root role
		if (parent == null || parent == ""){
			//data = "{\"name\": \""+projectName+"\"}";
		}
		else{
			var dataobject = {
						name: projectName,
						description: projectDescription,
						userScoping: userScopingRuleItems,
						permissionScoping: permissionScopingRuleItems
					};
			data = dojo.toJson(dataobject);
		}
		
		
		this.doHttpPost(data, successCallBack, errorCallBack);
	
	},
	getStore:function(){
		var store = dojox.data.JsonRestStore.getStore({
			target:this.resourceUri
		});
		return store ;
	},
	updateProjectNameAndDescription: function(
			/*string*/ resourceId,
			/*string*/ projectName,
			/*string*/ projectDescription,
			/*function*/ successCallBack, 
			/*function*/ errorCallBack) {
		this._trace.entering(this.id, "updateProjectNameAndDescription");
		// summary:	makes rest call to update project name and/or description
		// resourceId: string
		//	 	id of the resource that is updated. 
		//		If it is null then it is considered that rest api is pointing to that resource.
		// projectName: string
		//			Project Name
		// projectDescription: string
		//			Project Description
		// successCallBack: function
		//			Success callback handler function.
		// errorCallBack: function
		//			Error callback function.
		
		var dataobject = {
					name: projectName,
					description: projectDescription
				};
		var data = dojo.toJson(dataobject);
			
		this.doHttpPut(
			resourceId,
			data,
			successCallBack,
			errorCallBack
		);
		this._trace.exiting(this.id, "updateProjectNameAndDescription");
	},

	updateProjectScopingInformation: function(
			/*string*/ resourceId,
			/*object*/ userScoping,
			/*object*/ permissionScoping,
			/*function*/ successCallBack, 
			/*function*/ errorCallBack) {
		this._trace.entering(this.id, "updateProjectScopingInformation");
		// summary:	makes rest call to update project scoping information
		// resourceId: string
		//	 	id of the resource that is updated. 
		//		If it is null then it is considered that rest api is pointing to that resource.
		// userScoping: object
		//			User Scoping details
		// permissionScoping: object
		//			Permission Scoping details
		// successCallBack: function
		//			Success callback handler function.
		// errorCallBack: function
		//			Error callback function.
		
		var dataobject = {
					userScoping: userScoping,
					permissionScoping: permissionScoping
				};
		var data = dojo.toJson(dataobject);
			
		this.doHttpPut(
			resourceId,
			data,
			successCallBack,
			errorCallBack
		);
		this._trace.exiting(this.id, "updateProjectScopingInformation");
	},

	copyRolesToProject: function(
			/*string*/ resourceId,
			/*string*/ scope, 
			/*function*/ successCallBack, 
			/*function*/ errorCallBack) {
		this._trace.entering(this.id, "copyRolesToProject");
		// summary:	makes rest call to update project with post roles to be copied
		// resourceId: string
		//	 	id of the resource that is updated. 
		//		If it is null then it is considered that rest api is pointing to that resource.
		// scope: array
		//			copy of roles
		// successCallBack: function
		//			Success callback handler function.
		// errorCallBack: function
		//			Error callback function.
		
		this.doHttpPut(
			resourceId,
			scope,
			successCallBack,
			errorCallBack
		);
		this._trace.exiting(this.id, "copyRolesToProject");
	},
	checkProjectEditState:function (uri,/*function (item, request)*/successCallBack,/*function (item, request)*/errorCallback) {
		// summary:
		//	checks project editable state
		// successCallBack:	function (item, request)
		//		callback function, used when syncMode is false, used to handle fetch success result
		
		dojo.xhrGet({
			url: uri,
			handleAs: "json",
			sync: true,
			load: successCallBack,
			error:errorCallback
		});
	
	},
		
	checkProjectCreateAllowed:function (/*function (item, request)*/successCallBack,/*function (item, request)*/errorCallback) {
		
		dojo.xhrGet({
			url: this.URI_PROJECTS_REST_STATUS_RESOURCE,
			handleAs: "json",
			sync: true,
			content: {
							"action": "createProject" 
						},
			load: successCallBack,
			error:errorCallback
		});
	
	}

});