/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

� Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.api.RoleRestManager");

dojo.require("com.ibm.security.modeling.api.RestManager");
dojo.require("com.ibm.security.modeling.util.Trace");

dojo.declare("com.ibm.security.modeling.api.RoleRestManager", 
	com.ibm.security.modeling.api.RestManager, 
	{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.api.RoleRestManager"),
	constructor: function(args) {
		this._trace.entering("", "constructor");
		this._trace.object(arguments, "constructor", "Constructor args");
		dojo.mixin(this, args);
		//Uncommenting the below line to ensure that multiple delete works.
		this._identifier = "skipidattributeprocessing";
		
		this._resourceSchema = {
			"description":"Role",
			"type":"object",
			"properties": {
				"id":{"type":"integer"},
				"uri":{"type":"string","format":"url"},
				"name":{"type":"string"},
				"children":{
					"description":"Role Children",
					"items":{
						"type":"object",
						"properties": {
							"id":{"type":"integer"},
							"uri":{"type":"string","format":"url"},
							"name":{"type":"string"}
						}
					}
				},
				"childrenUri":{"type":"string","format":"url"},
				"description":{"type":"string"},
				"ownersUri":{"type":"string","format":"url"},
				"permissionsUri":{"type":"string","format":"url"},
				"parents":{
					"description":"Role Parents",
					"items":{
						"type":"object",
						"properties": {
							"id":{"type":"integer"},
							"uri":{"type":"string","format":"url"},
							"name":{"type":"string"}
						}
					}
				},
				"parentsUri":{"type":"string","format":"url"},
				"type":{
					"description":"Role Type",
					"type":"object",
					"properties": {
						"id":{"type":"integer"},
						"uri":{"type":"string","format":"url"},
						"description":{"type":"string"},
						"name":{"type":"string"}
					}
				},
				"totalIdentitiesCount":{"type":"integer"},
				"inheritedIdentitiesCount":{"type":"integer"},
				"totalPermissionsCount":{"type":"integer"},
				"inheritedPermissionsCount":{"type":"integer"},
				"storeId":{"type":"integer"},
				"sodconstraints":{
					"description":"SOD Constraints",
					"items":{
						"type":"object",
						"properties":{
							"id":{"type":"integer"},
							"uri":{"type":"string","format":"url"},
							"description":{"type":"string"},
							"cardinality":{"type":"integer"},
							"violations":{"type":"integer"},
							"roles":{
								"description":"Roles",
								"items":{
									"type":"object",					    
									"properties": {
										"id":{"type":"integer"},
										"uri":{"type":"string","format":"url"},
										"name":{"type":"string"}
									}           
								} 
							}
						}
					}
				}
			}
		};	// end schema
		this._trace.exiting("", "constructor");
	}, // end constructor

	updateRoleName: function(
			/*string*/ resourceId,
			/*string*/ value, 
			/*function*/ successCallBack, 
			/*function*/ errorCallBack) {
		this._trace.entering(this.id, "updateRoleName");
		// summary:	makes rest call to update role name
		// resourceId: string
		//	 	id of the resource that is updated. 
		//		If it is null then it is considered that rest api is pointing to that resource.
		// value: string
		//			Role Name
		// successCallBack: function
		//			Success callback handler function.
		// errorCallBack: function
		//			Error callback function.
		var roleObj = {name:value.toString()};
//		this.doHttpPut(
//			resourceId,
//			"{\"name\": \""+value.toString()+"\"}",
//			successCallBack,
//			errorCallBack
//		);
		this.doHttpPut(
				resourceId,
				dojo.toJson(roleObj),
				successCallBack,
				errorCallBack
			);
		this._trace.exiting(this.id, "updateRoleName");
	},
	updateRoleDescription: function(
			/*string*/ resourceId,
			/*string*/ value, 
			/*function*/ successCallBack, 
			/*function*/ errorCallBack) {
		this._trace.entering(this.id, "updateRoleDescription");
		// summary:	makes rest call to update role description
		// resourceId: string
		//	 	id of the resource that is updated. 
		//		If it is null then it is considered that rest api is pointing to that resource.
		// value: string
		//			Role Description
		// successCallBack: function
		//			Success callback handler function.
		// errorCallBack: function
		//			Error callback function.
		var roleObj = {description:value.toString()};
		this.doHttpPut(
			resourceId,
			dojo.toJson(roleObj),
			successCallBack,
			errorCallBack
		);
		this._trace.exiting(this.id, "updateRoleDescription");
	},
	updateRoleType: function(
			/*string*/ resourceId,
			/*string*/ value, 
			/*function*/ successCallBack, 
			/*function*/ errorCallBack) {
	
		// summary:	makes rest call to update role type
		// resourceId: string
		//	 	id of the resource that is updated. 
		//		If it is null then it is considered that rest api is pointing to that resource.
		// value: string
		//			Role Type
		// successCallBack: function
		//			Success callback handler function.
		// errorCallBack: function
		//			Error callback function.
		this._trace.entering(this.id, "updateRoleType");
		var roleObj = null;
		if ((value) &&
			(value.toString().length > 0)) {
			roleObj = {type:{uri:value.toString()}};
		} else {
			roleObj = {type:{}};
		}
		this.doHttpPut(
				resourceId,
				dojo.toJson(roleObj),
				successCallBack,
				errorCallBack
			);
		this._trace.exiting(this.id, "updateRoleType");
	},
	// Create a new role as root or under a given parent
	createRoleResource: function (
			/*string*/ roleName,
			/*string*/ parent,
			/*function (item, request)*/successCallBack, 
			/*function (error, request)*/errorCallBack) {
		// summary:
		//		Creates a new role resource using the incoming name 
		// successCallBack:	function (item, request)
		//		callback function, used when syncMode is false, used to handle fetch success result
		// errorCallBack: function (error, request)
		//		callback function, used when syncMode is false, used to handle fetch error result
		var data = "";
		
		// Root role
		if (parent == null || parent == ""){
			data = "{\"name\": \""+roleName+"\"}";
		}
		else{
			data = "{\"name\": \""+roleName+"\"," +
			       "\"parents\": " +
			       "[{\"id\": "+"\""+parent+"\"}]}";
		}
		this.doHttpPost(data,
				successCallBack,
				errorCallBack
			);
		
	},
	// Copy an existing role
	copyRoleResource: function (
			/*string*/ roleName,
			/*string*/ srcRoleId,
			/*function (item, request)*/successCallBack, 
			/*function (error, request)*/errorCallBack) {
		// summary:
		//		Creates a new role resource using the incoming name 
		// successCallBack:	function (item, request)
		//		callback function, used when syncMode is false, used to handle fetch success result
		// errorCallBack: function (error, request)
		//		callback function, used when syncMode is false, used to handle fetch error result
		var data = "{\"id\": \""+srcRoleId+"\", \"name\": \""+roleName+"\"}";
		this.doHttpPost(data,
				successCallBack,
				errorCallBack
			);
		
	}
});