/*
 * 
 */
dojo.provide("com.ibm.security.modeling.breadcrumbs.BreadCrumManager");
dojo.require("dijit._KeyNavContainer");
dojo.require("ibm.tivoli.tip.dijit.BreadcrumbBar");
dojo.require("ibm.tivoli.tip.dijit.Breadcrumb");
dojo.require("ibm.tivoli.tip.dijit.BreadcrumbSeparator");
dojo.require("com.ibm.security.modeling.breadcrumbs.BreadcrumbBar");
dojo.require("com.ibm.security.modeling.breadcrumbs.BreadcrumbBuilder");
dojo.require("com.ibm.security.modeling.util.ConsoleUtilities");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");

dojo.declare("com.ibm.security.modeling.breadcrumbs.BreadCrumManager", null, {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.breadcrumbs.BreadCrumManager"),
	constructor: function(args) {
		// summary:
		// 		Manages the breadcrumb bar for a portlet
		// description:
		// 		BreadCrum manager manages the breadCrum . Instantiation take place at the root node , every othe node publish 
		//		when come on viewport . 
		//		On Publish each panel gives it hiding function .
		//		Manager internally connect to that function , so that breadcrum get updated when that action take place from some where else .
		//		NOTE : do not pass this hiding function as a hitched function to any event management 
		//			like : button.onClick = dojo.hitch(this, this.hide)
		//		Connection fails on such hide function , because your onclick is hitched function and not actual hide .
		//
		//		Whenever click on any breadcrumb , manager start calling hide function in reverser order , i.e keep hiding all the pages 
		//		so that currect get visible . 
		//
		//		By Default : It add a special case here home panel and manage this special case . 
		this._trace.entering("", "constructor");
		dojo.subscribe("AddBreadCrum",dojo.hitch(this,this.addBreadCrum));
		dojo.subscribe("UpdateBreadCrum",dojo.hitch(this,this.updateBreadCrumLabel));
		dojo.subscribe("NAVIGATE_AWAY_EVENT",dojo.hitch(this,function(){
			this.overrideCanHide=true;
			this.updateBreadCrum();
			this.overrideCanHide=false;
		}));
		dojo.subscribe("/com/ibm/security/modeling/panelChanged", dojo.hitch(this, this.updateBreadcrumbsOnPanelChange));
		
		this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		this.connections=[];
		this.breadcrumbData = new com.ibm.security.modeling.breadcrumbs.BreadcrumbBuilder().withId(args.placeHolderId).build();
		var bar = this.breadcrumbData.getBreadCrumbBar();
		dojo.addClass(bar.domNode,"breadCrumbBarStyle");
		dojo.addClass(bar.domNode,"leftPadding");
		//will hold panelid vs id 
		this._panelIDmap={};
		this._pendingList = {};
		this.addBreadCrum({
			context : this,
			hideFunction : "homePanel",
			label:this.globalResources.HOME
		}, "HOME");
		
		this._trace.exiting("", "constructor");
		
	},
	homePanel:function(){
		// summary: 
		//		Invokes the home panel breadcrumb.
		var homeLink=dojo.byId("homeLink");
		ConsoleUtilities.clickLink(homeLink);
	},
	
	addBreadCrum:function(args, panelId){
		// summary: 
		//		Addes a breadcrumb to the bar.
		// context: Object
		//		The context on which the hide panel function should run
		// hideFunction: Function
		//		The function that executes the panel change
		this._trace.entering("", "addBreadCrum");
		args = dojo.isArray(args)?args[0]:args;
		delete args.onClick;
		//incorporate in any pending list is there . 
		args.label = this._pendingList[panelId]||args.label||" ";
		delete this._pendingList[panelId];
		var breadCrum = new ibm.tivoli.tip.dijit.Breadcrumb(args);
		//faking index of home panel , so that it do not remove bread crum of project
		if(this.getCurrentBreadcrumbData().getLength()==1){
			//this method opens home panel , hence hard-coded this behaviour for first bread-crum
			breadCrum._BreadCrumonClick = dojo.hitch(this,"homePanel");
		}else{
			breadCrum._BreadCrumonClick = dojo.hitch(args.context,args.hideFunction);
		}
		breadCrum._index = this.getCurrentBreadcrumbData().getLength();
		breadCrum.panelId = panelId;
		if(panelId) this._panelIDmap[panelId]=breadCrum.id;
		this.getCurrentBreadcrumbData().addBreadCrumb(breadCrum);
		//updating the most latest added breadcrum with selected
		this.selectedBreadCrum = breadCrum ; 
		var handle1 , handle2;
		if(args.context && args.hideFunction) {
			handle1=dojo.connect(args.context,args.hideFunction ,this,dojo.hitch(this,this.updateBreadCrum,{breadCrum:breadCrum,clickedOnClose:true}));
		}
		handle2 = dojo.connect(breadCrum,"onClick" ,this, dojo.hitch(this, 
			function(evt){
				//stop event from bubbling up , onClick was invoked twice 
				dojo.stopEvent(evt);
				this.updateBreadCrum({breadCrum:breadCrum});
			}
		));
		this.connections.push({handle1:handle1 ,handle2:handle2});
		this._trace.exiting("", "addBreadCrum");
	},
	
	updateBreadCrumLabel:function(/*Object*/args){
		// summary:
		//		Updates the label of a specific breadcrumb, or the last breadcrumb if no info provided.
		// id: String?
		//		The id of the breadcrumb widget to update
		// label: String?
		//		The label to update the breadcrumb to
		this._trace.entering("", "updateBreadCrumLabel");
		args.id = args.id||this._panelIDmap[args.panelId];
		if(args.id){
			var widget = dijit.byId(args.id);
			if(widget) widget.set("label",args.label);
			else{
				//widget is not created , storing in pending list so that if added in future we can update 
				this._pendingList[args.panelId] = args.label ;
			}
		}else if(args.panelId){
			this._pendingList[args.panelId] = args.label ;
		}
		else{
		
			//update the last breadCrumb
			//this.getCurrentBreadcrumbData().breadcrumbs[this.getSelectedIndex()].set("label",args.label);
			this.getCurrentBreadcrumbData().breadcrumbs[this.getCurrentBreadcrumbData().getLength()-1].set("label",args.label);
		}
		this._trace.exiting("", "updateBreadCrumLabel");
	},
	
	updateBreadcrumbsOnPanelChange: function(/*Object*/breadcrumbKwArgs, panelId) {
		// summary: 
		//		Updates breadcrumbs on panel changes.
		// description:
		//		if the breadcrumb representing that panel exists, then update the breadcrumbs, otherwise add a new one to the end.
		// breadcrumbKwArgs: Objects
		//		arguments for creating a breadcrumb
		// panelId
		//		Id representing the panel for the breadcrumb.  get the panel ids from dojo.config.panels.<panel name>.id
		if (breadcrumbKwArgs.isHidden === true) {
			this.hideBreadCrumbBar();
		} else if (this.breadcrumbData.isInBreadcrumbBar(panelId) === true) {//assume we are going back if already there
			this.showBreadCrumbBar();
			this.overrideCanHide =  breadcrumbKwArgs.overrideCanHide;
			breadcrumbKwArgs.breadCrum=this.breadcrumbData.getBreadCrumbForPanelID(panelId);
			breadcrumbKwArgs.clickedOnClose = false;
			this.updateBreadCrum(breadcrumbKwArgs);
		} else { //assume going forward if not
			this.showBreadCrumbBar();
			this.addBreadCrum(breadcrumbKwArgs, panelId);
		}
	},
	hideBreadCrumbBar:function(){
		var bar = this.breadcrumbData.getBreadCrumbBar();
		dojo.removeClass(bar.domNode, "show");
		dojo.addClass(bar.domNode, "hide");
		dojo.removeClass(bar.domNode,"breadCrumbBarStyle");
		dojo.removeClass(bar.domNode,"leftPadding");
	},
	showBreadCrumbBar:function(){
		var bar = this.breadcrumbData.getBreadCrumbBar();
		dojo.removeClass(bar.domNode, "hide");
		dojo.addClass(bar.domNode, "show");
		dojo.addClass(bar.domNode,"breadCrumbBarStyle");
		dojo.addClass(bar.domNode,"leftPadding");
	},
	updateBreadCrum : function(/*Object*/args){
		// summary:
		//		Updates the breadcrumb bar
		// breadCrum: ibm.tivoli.tip.dijit.Breadcrumb
		//		Breadcrumb widget that was clicked
		// clickedOnClose: Boolean
		//		Whether we are updating breadcrumbs on a panel close event.
		this._trace.entering("", "updateBreadCrum");
		//will remove the all breadcrum in front of current breadCrums
		//first disconnect then removeif no arguments come , then update will take place according to currently selected breadcrum , and with assumption clickedonClose = false
		args = args ||{breadCrum:this.breadcrumbData.breadcrumbs[this.getSelectedIndex()],clickedOnClose:false};
		if(this.updatingBreadCrum) return ;
		this.updatingBreadCrum = true ; 
		this.selectedBreadCrum = args.breadCrum;
		this.disconnectAndRemoveBreadCrums(args);
		//breadcrum update the class when click happens on breadcrum , but this will help when we we click on close button and breadcrum updates
		this.getCurrentBreadcrumbData().breadcrumbs[this.getCurrentBreadcrumbData().getLength()-1]._setStateClass();
		this.updatingBreadCrum = false;		
		this._trace.exiting("", "updateBreadCrum");
	},
	getSelectedIndex:function(){
		// summary: 
		//		Gets the index of breadcrumb that was clicked
		// returns: 
		//		the index of the clicked breadcrumb, or the index to the last breadcrumb
		this._trace.entering("", "getSelectedIndex");
		var selectedIndex;
		if(this.selectedBreadCrum) {
			selectedIndex = this.selectedBreadCrum._index;
		} else{
			selectedIndex = this.getCurrentBreadcrumbData().getLength()-1;
		}
		this._trace.exiting(selectedIndex, "getSelectedIndex");
		return selectedIndex;
	},
	
	getCurrentBreadcrumbData :function(){
		return this.breadcrumbData;
	},
	
	setCurrentBreadcrumbData:function(data){
		this.breadcrumbData = data;
	},
	disconnectAndRemoveBreadCrums:function(/*Object*/args){
		// summary: 
		//		Completely clears the breadcrumbs down to the selected breadcrumb
		// clickedOnClose: Boolean
		//		Whether this call is a result of closing panels (going back in breadcrumbs)
		this._trace.entering("", "disconnectAndRemoveBreadCrumbs");
		var breadCrumbData = this.getCurrentBreadcrumbData();
		var i = this.getSelectedIndex();
		//if close button is clicked , remove current breadacrum  , override the canhide invocation 
		if(args.clickedOnClose===true){
			i=i-1;
			this.overrideCanHide = true;
		}
		
		for(var j = breadCrumbData.getLength()-1;j>i;j--){
			//here we check if canHide exist and close method is not invoked , it it believed that when ever this close method will be 
			//invoked , panel will close itself , i.e this close will never fail .
			//if(breadCrumBuilder.breadcrumbs[j].canHide && this.overrideCanHide) canHide = breadCrumBuilder.breadcrumbs[j].canHide();
			if(this.canHide(breadCrumbData, j)){
				this.disconnectLastIndex();
				//since close method has already been invoked for this don't do this
				//also, since our browser back handler handles page launching, we don't want the onClick to happen what the fromBrowserEvent attr is set.
				if(!args.clickedOnClose && !args.fromBrowserEvent) {
					breadCrumbData.breadcrumbs[j]._BreadCrumonClick();
				}
				this.removeLastBreadCrum();
			}else{
				break;
			}
		}
		//bringing this flag to original position
		this.overrideCanHide = false;
		this._trace.exiting("", "disconnectAndRemoveBreadCrums");
	},
	canHide:function(breadCrumbData,j){
		// summary:
		//		Check if the breadcrumb can be removed.  Usually breadcrumbs used in one of our wizards cannot be removed.
		// breadCrumbData: com.ibm.security.modeling.breadcrumbs.BreadcrumbData
		//		Current breadcrumb data
		// j: Number
		//		breadcrumb index
		// returns: Boolean
		//		Whether we can remove the breadcrumb at the given index
		this._trace.entering("", "canHide");
		var canHide = true;
		//here we check if canHide exist and close method is not invoked , it it believed that when ever this close method will be 
		//invoked , panel will close itself , i.e this close will never fail .
		if(breadCrumbData.breadcrumbs[j].canHide && !this.overrideCanHide) canHide = breadCrumbData.breadcrumbs[j].canHide();
		return canHide;
		this._trace.exiting("", "canHide");
	},
    removeLastBreadCrum:function(){
		// summary:
		//		Removes the last breadcrumb in the bar.
		this._trace.entering("", "removeLastBreadCrum");
		//do not remove last 2 breadcrum , e.g home > project , this two will remain 
		var breadCrumbData = this.getCurrentBreadcrumbData();
		if(breadCrumbData.getLength() > 2) {
			breadCrumbData.removeLastBreadCrumb();
		}
		this._trace.exiting("", "removeLastBreadCrum");
	},
	
	disconnectLastIndex : function(){
		this._trace.entering("", "disconnectLastIndex");
		//donot disconnect last 2 breadcrum , e.g home > project , this two will remain 
		if(this.connections.length<=2) return ;
		try{
	    	dojo.disconnect(this.connections[this.connections.length-1].handle1);
	    }catch(e){}
		try{
			dojo.disconnect(this.connections.pop().handle2);
		}catch(e){}
		this._trace.exiting("", "disconnectLastIndex");
	}
	
});