/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/


/*
 * 
 * Copying the ibm.tivoli.tip.dijit.BreadcrumbBar code into this widget to fix an issue.
 * 
 * The ibm.tivoli.tip.dijit.BreadcrumbBar cannot be extended to fix this, because the 
 * issue is in addChild method which again calls the this.inherited(). 
 * 
 * So instead of modifying the addChild of ibm.tivoli.tip.dijit.BreadcrumbBar, I am 
 * creating this new widget and modifying the addChild to fix the breadCrumb issue.
 * 
 * 
 * Currently ignoring the code that creates ... for long breadcrumbs. Doing this as
 * a temporary fix till the TIP module gets fixed. 
 * The impact is that the long BreadCrumbs will not have >...> appearing as per the guidelines.
 * 
 * The bug is filed for TIP Dojo team against this issue.
 */
dojo.provide("com.ibm.security.modeling.breadcrumbs.BreadcrumbBar");

dojo.require("dijit._Widget");
dojo.require("dijit._Container");
dojo.require("dijit._Templated");
dojo.require("dijit._KeyNavContainer");
dojo.require("dijit.form.Button");
dojo.require("ibm.tivoli.tip.dijit.Breadcrumb");
dojo.require("ibm.tivoli.tip.dijit.BreadcrumbSeparator");

dojo.declare(
	"com.ibm.security.modeling.breadcrumbs.BreadcrumbBar",
	[dijit._Widget, dijit._Templated, 
	dijit._KeyNavContainer],
{
	templateString:
		'<div class="dijit dijitToolbar tipBreadcrumbBar" waiRole="toolbar" tabIndex="${tabIndex}" dojoAttachPoint="containerNode">' +
		'</div>',

	tabIndex: "0",
	constructor:function(){
		this.breadcrumbWidget = [];
	},
	

	postCreate: function(){
		this.connectKeyNavHandlers(
			this.isLeftToRight() ? [dojo.keys.LEFT_ARROW] : [dojo.keys.RIGHT_ARROW],
			this.isLeftToRight() ? [dojo.keys.RIGHT_ARROW] : [dojo.keys.LEFT_ARROW]
		);
	},

	startup: function(){
		this.startupKeyNavChildren();
	},
	
	destroy: function () {
		this.destroyDescendants();
		this.inherited (arguments);
	},
	
	addChild: function(widget, insertIndex){
		var children = this.getChildren();
		var childToUpdate = null;
		if(children.length>0 && children[children.length -1]._setStateClass){
			childToUpdate=children[children.length -1]; //child which style class needs to be updated after addition
		}
		this.inherited (arguments);	//adding child
		if(childToUpdate!=null){
			childToUpdate._setStateClass(); //updating style class
		}
		//Breadcrumbs Tivoli Guidelines implementation
		var size = dojo.contentBox(this.domNode);
		this.breadcrumbWidget.push(widget);
		var i = 1;
		var breadcrumbWidth = this.breadcrumbWidget[0].containerNode.offsetLeft;
		while(i < this.breadcrumbWidget.length){
			if(this.breadcrumbWidget[i].declaredClass == "ibm.tivoli.tip.dijit.BreadcrumbSeparator")
				breadcrumbWidth = breadcrumbWidth + this.breadcrumbWidget[i-1].containerNode.offsetWidth + this.breadcrumbWidget[i].domNode.offsetWidth;
			else 
				breadcrumbWidth = breadcrumbWidth + this.breadcrumbWidget[i-1].domNode.offsetWidth + this.breadcrumbWidget[i].containerNode.offsetWidth;
			
			/*
			 * Ignoring this code path to be fixed in TIP BreadCrumbBar.js.
			 * The impact is that the long BreadCrumbs will not have >...> appearing as per the guidelines.
			 * But currently for RM, we do not have long breadcrumbs, so we can go ahead with this workaround for now.
			 */ 
			if(false) { //(breadcrumbWidth > size.w){
				var breadcrumbTrailWidth = breadcrumbWidth;
				for(j = i - 2; j > 0 && breadcrumbTrailWidth > size.w ; j = j - 1){
					breadcrumbTrailWidth = 0;
					if(this.breadcrumbWidget[j].declaredClass == "ibm.tivoli.tip.dijit.BreadcrumbSeparator"){
						dojo.byId(this.breadcrumbWidget[j].id).style.display = "none";
						breadcrumbTrailWidth = this.breadcrumbWidget[j-1].containerNode.offsetLeft + this.breadcrumbWidget[j-1].containerNode.offsetWidth;
					} else {
						dojo.byId(this.breadcrumbWidget[j].id).style.display = "none";
						dojo.byId(this.breadcrumbWidget[j-1].id).style.display = "none";
						breadcrumbTrailWidth = this.breadcrumbWidget[j-2].domNode.offsetLeft + this.breadcrumbWidget[j-2].domNode.offsetWidth;
					}
				}
				//Adding Ellipsis to make long breadcrumb trails guideline compliant
				if(this.breadcrumbWidget[i - 1].declaredClass == "ibm.tivoli.tip.dijit.BreadcrumbSeparator"){
					dojo.byId(this.breadcrumbWidget[i - 1].id).innerHTML = "&gt;&#160;...&#160;&#160;&gt;";
				} else {
					dojo.byId(this.breadcrumbWidget[i].id).innerHTML = "&gt;&#160;...&#160;&#160;&gt;";
				}
			}
			i = i + 1;
		}
	}
}
);

