dojo.provide("com.ibm.security.modeling.controller.BackHandler");

dojo.require("com.ibm.security.modeling.util.Trace");

dojo.declare("com.ibm.security.modeling.controller.BackHandler", null, {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.controller.BackHandler"),
	// _launcherMap: Object
	//		A map of launcher id to com.ibm.security.modeling.controller.PanelLauncher object
	_launcherMap: null,
	// _unhashedLauncher: com.ibm.security.modeling.controller.PanelLauncher
	//		A reference to a launcher for a launched panel that has not yet been set to the url
	_unhashedLauncher: null,
	// _lastHash: Number
	//		Url hash of the last panel that the bach handler knows about
	_lastHash: 0,
	// isActive: Boolean
	//		Whether the back handler is being used.
	isActive: false,
	
	constructor:function(isActive) {
		// summary:
		//		Handles browser back and forward event handling for the application.
		// isActive: Boolean
		//		Whether to create this object with its back event handling initially on (polling for url hash changes)
		this._launcherMap = {};
		if (isActive) {
			this.enableBrowserChangeChecking();
		}
	},
	
	enableBrowserChangeChecking:function() {
		// summary:
		//		Enables the browser back forward handling for the application.
		// description:
		//		We can't detect the back button using dojo.hash because we have no control of our parent window (TIP).
		//		We con't receive the onhashchange event becasue we are an iframe with no control of our parent window's javascript.
		//		Therefore we will set a constant interval to check whether the hash of the top window location has changed.
		//		If it has changed, then we can assume that the user has moved panels. 
		setInterval(dojo.hitch(this, this.pollLocation), 100);
		dojo.subscribe("/dojo/panelchange", this, this.onHashChange);
		this.isActive = true;		
	},
	
	pollLocation: function() {
		// summary:
		//		Checks if the last url hash we know about is different than what's currently there.  Sends an event if different.
		// tags:
		//		callback
		if(this._lastHash !== 0 && window.top.location.hash !== this._lastHash ){
			this._trace.finest("hash change - last hash: " + this._lastHash + ", new hash: " + window.top.location.hash, "pollLocation");
			this._setLastHash(window.top.location.hash);
			this.sendEvent();
		}
	},
	
	sendEvent: function() {
		dojo.publish("/dojo/panelchange", [window.top.location.hash.substring(1)]);
	},
	
	registerPanel:function(launcher) {
		this._trace.entering("location: " + window.top.location.hash, "registerPanel");
		var date = new Date();
		var launcherId = date.getTime();
		this._unhashedLauncher = launcher;
		var objectToHash = {'launcherId':launcherId};
		var hash = dojo.objectToQuery(objectToHash);
		if (this._lastHash === 0) {
			this._setLastHash(hash);
			window.top.location.hash = hash;
			this.sendEvent();
		} else {
			window.top.location.hash = hash;
		}
		this._trace.exiting("location: " + window.top.location.hash + " hash: " + hash, "registerPanel");
	},
	
	_setLastHash:function(hash) {
		if (hash.length > 0 && hash.substr(0,1) !== "#") {
			hash = "#" + hash;
		}
		this._lastHash = hash;
	},
	
	onHashChange: function(hash) {
		// summary:
		//		Changes the panel based on the given hash.
		// description:
		//		Called when the hash is different from the last known. Changes the panel based on the the launcher
		//		corresponding to the url hash.  If no launcher has been added to the map, then it is assumed that 
		//		the panel has already been launched.
		// tags:
		//		callback
		this._trace.entering("location: " + window.top.location.hash + " hash: " + hash, "onHashChange");
		var obj = dojo.queryToObject(hash);
		var launcherId = obj.launcherId;
		if(this._launcherMap[launcherId]) {
			var launcher = this._launcherMap[launcherId];
			if (launcher && launcher.breadcrumbArgs) {
				//this attribute makes sure we don't call the panel breadcrumb onclick method
				launcher.breadcrumbArgs.fromBrowserEvent = true;
				launcher.launch();
			}
		} else if (this._unhashedLauncher) {
			//if the id is not in the the map, then we assume the launcher is already launched (initial panel case)
			this._launcherMap[launcherId] = this._unhashedLauncher;
		}
		this._trace.exiting("location: " + window.top.location.hash + " hash: " + hash, "onHashChange");
	}
});