/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

 Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.controller.ModelingController");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("com.ibm.security.modeling.dijit.layout.BorderContainer");
dojo.require("com.ibm.security.modeling.dijit.layout.StackContainer");
dojo.require("com.ibm.security.modeling.dijit.layout.ContentPane");

dojo.require("com.ibm.security.modeling.widgets.MultiMessageArea");
dojo.require("com.ibm.security.modeling.projects.ProjectsTablePanel");
dojo.require("com.ibm.security.modeling.controller.BackHandler");
dojo.require("com.ibm.security.modeling.controller.PanelFactory");
dojo.require("com.ibm.security.modeling.hierarchy.HierarchyPanelLauncher");
dojo.require("com.ibm.security.modeling.notebook.RolePanelLauncher");
dojo.require("com.ibm.security.modeling.membership.FilterBuilderPanelLauncher");
dojo.require("com.ibm.security.modeling.projects.ProjectsPanelLauncher");
dojo.require("com.ibm.security.modeling.util.TitleUtilities");
dojo.require("com.ibm.security.modeling.util.HelpUtilities");
dojo.require("com.ibm.security.modeling.util.XhrUtilities");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.breadcrumbs.BreadCrumManager");
dojo.require("com.ibm.security.modeling.dijit.layout.ToggleSplitter");
dojo.require("com.ibm.security.modeling.layouts.TemplatedPanelWithLayout");

dojo.require("com.ibm.security.modeling.analysis.base._AnalysisPaneContainerMixin");
dojo.require("com.ibm.security.modeling.lifecycle.LifecycleProvider");

dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling.projects", "ProjectsTableWizardText");



dojo.declare("com.ibm.security.modeling.controller.ModelingController",
            [ com.ibm.security.modeling.layouts.TemplatedPanelWithLayout, com.ibm.security.modeling.analysis.base._AnalysisPaneContainerMixin ],
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.controller.ModelingController"),
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.controller","templates/ModelingController.html"),	
	widgetsInTemplate: true,
	// globalResources: Object
	//			Reference to the global shared resource bundle
	globalResources: null,
	// localResources: Object
	//			Reference to the resource bundle unique to this widget
	localResources: null,
	// namespace: [readonly] String
	// 			Unique identifier to the instance of portlet page where this item will exist
	namespace: "",
	// hostname: String
	//			The hostname of the server from which Content is being served.
	hostname: "",
	// port: String
	//			The port of the server from which Content is being served.
	port: "16311",
	// projectsTablePanel: com.ibm.security.modeling.projects.ProjectsTablePanel
	//			The widget representing the users resource selection page
	projectsTablePanel: null,
	// connection: Object[]
	//		summary: The array of dojo method connections for this widget
	connections: [],
	// contextRoot: String
	//			The context root of the server from which Content is being served.
	contextRoot: "",
	// roleHierarchyPanel: com.ibm.security.modeling.hierarcy.RoleHierarchyPanel
	//			The widget representing the hierarchy panel page
	roleHierarchyPanel:null,
	// rolePanel:  com.ibm.security.modeling.notebook.RoleNotebookDialogView
	//			the widget representing the roles page
	rolePanel: null,
	// filterBuilderPanel:  com.ibm.security.modeling.membership.FilterBuilder
	//			the widget representing the filter builder page
	filterBuilderPanel: null,
	//_workingDialogCount: [private] Number
	//			current number of working dialogs (dead?)
	_workingDialogCount:0,
	// msgArea: com.ibm.security.modeling.widgets.MultiMessageWidget
	//			widget to display messages to the user as they occur.
	msgArea: null,
	// helpUtil: com.ibm.security.modeling.util.HelpUtilities 
	//			sets current help file
	helpUtil: new com.ibm.security.modeling.util.HelpUtilities(),
	// titleUtil: com.ibm.security.modeling.util.TitleUtilities
	//			sets the portlet title to the current panel's
	titleUtil: null,	
	// backHandler: com.ibm.security.modeling.controller.BackHandler
	//			Special object for handling back events.
	backHandler: null,
	
	constructor: function (args) {
		// summary:
		// 			Widget that provides the UI navigation and basic layout for project-based modeling
		// description: 
		//			This widget handles page transitions between the projects hub, roles and policies page, and project creation wizard.
		this._trace.entering("", "constructor");
		this.titleUtil = new com.ibm.security.modeling.util.TitleUtilities();		
		this.backHandler = new com.ibm.security.modeling.controller.BackHandler(true);
		this.hostname = dojo.config.serverProperties.hostname;
		this.contextRoot = dojo.config.serverProperties.contextRoot;
		this.namespace = dojo.config.serverProperties.namespace;
		this.port = dojo.config.serverProperties.port;
		dojo.subscribe(dojo.config.panels.PROJECTS.launchEventAddress, dojo.hitch(this, this.launchProjectTablePanel));
		dojo.subscribe(dojo.config.panels.HIERARCHY.launchEventAddress, dojo.hitch(this, this.launchHierarchyPanel));
		dojo.subscribe(dojo.config.panels.ROLE.launchEventAddress, dojo.hitch(this, this.launchRolePanel));
		dojo.subscribe(dojo.config.panels.FILTER.launchEventAddress, dojo.hitch(this, this.launchFilterBuilderPanel));
		dojo.subscribe(dojo.config.panels.PROJECT_CREATE.launchEventAddress, dojo.hitch(this, this.launchProjectCreationPanel));
		dojo.subscribe(dojo.config.panels.MESSAGE_DIALOG.launchEventAddress, dojo.hitch(this, this.showMessageDialog));
		dojo.subscribe(dojo.config.panels.ANALYSIS.launchEventAddress, dojo.hitch(this, this.showRoleAnalysisByRoleId));
		
		this._trace.exiting("", "constructor");
		
	},
	postMixInProperties:function() {
		this.inherited(arguments);
		this.configObject = {
			parentPanel: this,
			id: this.id,
			hostname: this.hostname,
			contextRoot: this.contextRoot,
			namespace: this.namespace,
			port: this.port
			
		};
		this.panelFactory = new com.ibm.security.modeling.controller.PanelFactory(this.configObject);
		XhrUtilities.addSessionDetailInRequest();
	},
	postCreate:function(){
		// tags:
		//			protected
		this._trace.entering(this.id, "postCreate");	
		this.inherited(arguments);
		
		//have to wait until the template is processed to add the common message area to the config object.
		this.configObject.msgArea = this.msgArea;
		this.msgArea.borderContainer = this.container;
		var manager = new com.ibm.security.modeling.breadcrumbs.BreadCrumManager({
			placeHolderId : this.id + "_BreadCrumbPlaceHolder"
		});

		/* Globalization calls should be made after the object is created, not in the constructor. 
		 * Dojo Custom Build doesn't seem to load the Globalization files while the layers are being created and loaded. 
		 * Given the nature of the Custom Build, delaying the initialization of Globalization vars, makes sense. */
		this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.projects", "ProjectsTableWizardText");
		this._setupProgressDialog();
		this.projectsTablePanel = this.panelFactory.createProjectsPanel();
		this.projectsPanel.makeAccessible(this.projectsTablePanel);
		this.titleUtil.updateTitle(this.globalResources.PROJECTS);
		this._trace.exiting(this.id, "postCreate");
	},
	startup: function(){
		// tags:
		//			protected
		this._trace.entering(this.id, "startup");
		try {
			this._setupProjectsPanelLaunch();
			this.inherited(arguments);
            var handleTopLevelResize = function() {
				this._trace.entering("##############", "handleTopLevelResize");
				this.resize();
				this._trace.exiting("", "handleTopLevelResize");
			};
			if (!customizer)
			{
				this._trace.finest("Using window.onresize for resize handling", "startup");
				this.connect(dojo.global, "onresize", handleTopLevelResize);				
			}
		} catch (e) {
			this._trace.exception(e, "startup");
			this.msgArea.showErrorMessage(e);
		}
		
		this._trace.exiting(this.id, "startup");
	},
	_setupProjectsPanelLaunch: function() {
		this._trace.entering("", "_setupProjectsPanelLaunch");
		this.projectsTablePanel.startup();
		var projectsPanelLauncher = new com.ibm.security.modeling.projects.ProjectsPanelLauncher(this.wizardContainer, this.projectsPanel, this.projectsTablePanel, this.globalResources);
		this.launchPanel(projectsPanelLauncher);
		this._trace.exiting("", "_setupProjectsPanelLaunch");
	},
	resize:function() {
		// summary: 
		//			Resizes the inner layout of this panel.
		// tags:
		//			public
		this._trace.entering("", "resize");
		this.inherited(arguments);
		this._trace.exiting("", "resize");
	},
	_findTitleNode:function() {
		// summary:
		//			finds the title of the portlet and sets the node object to this panel object for future updates.
		// returns:
		//			dom node of the title
		// tags:
		//			private
		return dojo.query("h1[name='portletTitleInnerDIV']",parent.document)[0];
	},
	_setupProgressDialog:function() {
		// summary:
		//			sets up the progress dialog.
		// tags:
		//			private
		this.workingDialog.titleBar.style.display='none';//hide the title bar
		dojo.removeClass(this.workingDialog.containerNode, "dijitDialogPaneContent");		
	},	

	switchToTableView:function(){
		// summary: 
		//			switches the current view from a topology to the table.  
		// description:
		// 			Called from the topology toolbar.  Toolbar calls a global function defined at the jsp level, 
		//			which calls down into this panel widget.
		// tags:
		// 			public
		this._trace.entering(this.id, "switchToTableView");
		this.roleHierarchyPanel.switchToTableView();
		//this.roleHierarchyPanel.resize();
		this._trace.exiting(this.id, "switchToTableView");
	},
	launchExportProjectDialog:function(){
		// summary: 
		//			Launches the export project dialog.  
		// description:
		// 			Called from the topology toolbar.  Toolbar calls a global function defined at the jsp level, 
		//			which calls down into this panel widget.
		// tags:
		// 			public
		this._trace.entering(this.id, "launchExportProjectDialog");
		this.msgArea.hideMessages();// clear previous messages
		this.roleHierarchyPanel.launchExportProjectDialog();
		//this.roleHierarchyPanel.resize();
		this._trace.exiting(this.id, "launchExportProjectDialog");
	},
	launchPanel:function(launcher, /*boolean*/shouldKeepMessages) {
		// summary: 
		//			Launches the panel given by the provide launcher object
		// launcher: Object
		//			Object that provides the launch method.  Should be a subclass of com.ibm.security.modeling.controller.PanelLauncher
		// shouldKeepMessages: boolean
		//			if true, then we will maintain any displayed messages, otherwise we will hide them.
		this._trace.entering(this.id, "launchPanel");
		if(!shouldKeepMessages) {
			this.msgArea.hideMessages();
		}
		launcher.launch();
		this.backHandler.registerPanel(launcher);
		this._trace.exiting(this.id, "launchPanel");
	},
	launchHierarchyPanel:function(projectUrl, modelId, loadData){
		// summary:
		//			Launches the roles and policies panel, based on a given project.
		// projectUrl: String
		//			REST Url to the project data
		// modelId: String
		//			ID of the model
		// tags:
		//			public
		this._trace.entering("Model ID:"+modelId+", Project URL: "+projectUrl +", Load data: "+loadData, "launchHierarchyPanel");
		try {
			if(!this.roleHierarchyPanel){//if the panel doesn't exist, create it
				this.roleHierarchyPanel = this.panelFactory.createHierarchyPanel(projectUrl, modelId, this.getRoleLifecycleProvider());
				this.roleHierarchyPanel.startup();
			}
			var hierarchyLauncher = new com.ibm.security.modeling.hierarchy.HierarchyPanelLauncher(this.wizardContainer, this.hierarchyPanel, this.roleHierarchyPanel, projectUrl, modelId, this.globalResources, loadData);
			this.launchPanel(hierarchyLauncher);
		} catch (e) {
			this._trace.exception(e, "launchHierarchyPanel");
			this.msgArea.showErrorMessage(e);
		}
		this._trace.exiting(this.id, "launchHierarchyPanel");
	},
	getRoleLifecycleProvider: function(){
		if(! this.roleLifecycleProvider){
			this.roleLifecycleProvider = new com.ibm.security.modeling.lifecycle.LifecycleProvider({
				id : this.id+"_role_lifecycle_provider",
				entity : "role",
				messageArea: this.msgArea
			});	
		}
		return this.roleLifecycleProvider;
		
	},
	launchRolePanelBySelectedTopologyItem:function() {
		var selectedItemIds = this.roleHierarchyPanel.navWidget._canvas.selection.getItemIds();
		if(selectedItemIds && selectedItemIds.length===1){
			this.launchRolePanelByRoleId(selectedItemIds[0]);
		}
	},
	launchRolePanelByRoleId:function(roleId, initialPanel) {
		var roleUri=this.roleHierarchyPanel.getRoleUriFromId(roleId, initialPanel);
		this.launchRolePanel(roleUri, initialPanel, this.roleHierarchyPanel.projectUrl, this.roleHierarchyPanel._modelId);
	},
	
	launchRolePanel:function(roleUri, initialPanel, projectUrl, modelId) {
		// summary: 
		//			Launches the roles notebook, based on the given role
		// roleUri: String
		//			Uri of the role to be displayed in the role panel
		// initialPanel: Object
		//			Object representing the panel that initiated the launch of this role panel.
		// projectUrl: String
		//
		// modelId: String
		//
		// tags:
		//			public
		this._trace.entering("Role Url: "+roleUri+", Project Url: "+projectUrl+", "+"Model ID: "+modelId, "launchRolePanel");
		try {
			if(!this.rolePanel){//if the panel doesn't exist, create it
				this.rolePanel = this.panelFactory.createRolePanel(roleUri, initialPanel, this.hierarchyPanel, this.getRoleLifecycleProvider());
				this.rolePanel.startup();
				this.rolePanel.isNew = true;
			} else {
				this.rolePanel.isNew = false;
			}
			var rolePanelLauncher = new com.ibm.security.modeling.notebook.RolePanelLauncher(this.wizardContainer, this.rolePane, this.rolePanel, roleUri, initialPanel, projectUrl, modelId);
			this.launchPanel(rolePanelLauncher);
		}catch (e) {
			this._trace.exception(e, "launchRolePanel");
			this.msgArea.showErrorMessage(e);
		}
		this._trace.exiting("Role Url: "+roleUri, "launchRolePanel");
		
	},
	launchFilterBuilderPanel:function(filterData, roleItem, notebookTitle) {
		// summary: 
		//			Launches the Filter Builder, based on the given role
		// filterData: Object
		//
		// roleItem: Object
		//			Object representing the role for whom filter is being built for.
		// tags:
		//			public
		this._trace.entering(this.id, "launchFilterBuilderPanel");
		try {
			if(!this.filterBuilder){//if the panel doesn't exist, create it
				this.filterBuilder = this.panelFactory.createFilterBuilderPanel(filterData, roleItem, notebookTitle, this.rolePane, this.hierarchyPanel);
				this.filterBuilder.startup();
				this.filterBuilder.isNew = true;
			} else {
				this.filterBuilder.isNew = false;
			}
			var filterBuilderPanelLauncher = new com.ibm.security.modeling.membership.FilterBuilderPanelLauncher(this.wizardContainer, 
					this.filterBuilderPane, 
					this.filterBuilder,
					filterData, 
					roleItem, 
					notebookTitle, 
					this.globalResources);
			this.launchPanel(filterBuilderPanelLauncher);
		}catch (e) {
			this._trace.exception(e, "launchFilterBuilderPanel");
			this.msgArea.showErrorMessage(e);
		}
		this._trace.exiting(this.id, "launchFilterBuilderPanel");		
	},
	
	launchProjectCreationPanel:function() {
		// summary:
		//			Launches the roles and policies panel, based on a given project.
		// tags:
		//			public
		this._trace.entering(this.id, "launchProjectCreationPanel");
		try {
			//TODO : check with expected as this does not start afresh from first panel
			if(!this.projectCreationWiz){ // After discussion, creating new on every launch
				this.projectCreationWiz = this.panelFactory.createNewProjectWizardPanel(dojo.hitch(this, this._onFinishCreation));
				//this._setTableFooterResourcesToProjectCreationWizardPanels();//please find a better way
				this.projectCreationWiz.startup();
				this.projectCreationWiz.isNew = true;
			}
			else {
				this.projectCreationWiz.isNew = false;
			}
			dojo.require("com.ibm.security.modeling.projects.NewProjectWizardPanelLauncher");
			var projectCreationWizardPanelLauncher = new com.ibm.security.modeling.projects.NewProjectWizardPanelLauncher(this.wizardContainer, this.newProjectPanel, this.projectCreationWiz);
			this.launchPanel(projectCreationWizardPanelLauncher);
			//breadcrumb will hide hence rezing to take full space
			this.resize();
			this.projectCreationWiz.setInitialFocus();
		}catch (e) {
			this._trace.exception(e, "launchProjectCreationPanel");
			this.msgArea.showErrorMessage(e);
		}
		this._trace.exiting(this.id, "launchProjectCreationPanel");

	},

	_setTableFooterResourcesToProjectCreationWizardPanels:function() {
		// summary: 
		//			Sets the footer internationalization information to the project creation wizard tables.
		if(this.projectsTablePanel._projectsTable.isTotalsFooterRendered && this.projectsTablePanel._projectsTable.footer){
			this.projectCreationWiz.usersResourcesSelectionPanel.tableFooterResource = this.projectsTablePanel._projectsTable.footer.resources_;
			this.projectCreationWiz.permissionsSelectionPanel.tableFooterResource = this.projectsTablePanel._projectsTable.footer.resources_;
		}
	},
	launchProjectTablePanel:function(isStartupNeeded, shouldKeepMessages){
		// summary:
		//			Returns to the project table panel from the roles and policies panel
		// description:
		//			Assumes we are come back from the roles and policies panel.  We will need to expand this functionality to include
		//			The project creation wizard, which means we will have to advance the stack by id instead of using "back"
		// tags:
		//			public
		this._trace.entering(this.id, "launchProjectTablePanel");
		var projectsPanelLauncher = new com.ibm.security.modeling.projects.ProjectsPanelLauncher(this.wizardContainer, this.projectsPanel, this.projectsTablePanel, this.globalResources);
		this.launchPanel(projectsPanelLauncher, shouldKeepMessages);
		//resizing to show breadcrumb , if hidden due to new project creation .
		this.resize();
		this._trace.exiting(this.id, "launchProjectTablePanel");
	},
	
	_onFinishCreation: function(msg){
		// summary:
		//			Refreshes the projects table after a create
		// description:
		//			This method is out of place in this class.  We should probably do an event connection
		//			between the projects table/panel and the project creation wizard instead of executing
		//			logic here.
		// msg: String
		//			The info message indicating a successful project creation.
		// tags:
		//			callback
		this._trace.entering(this.id, "_onFinishCreation");
		this.msgArea.showInfoMessage(msg);// Temp success msg

		if (this.projectsTablePanel && this.projectsTablePanel._projectsTable) {
			this.projectsTablePanel._projectsTable.deselectAll();
			// Key is to refresh the table, but we need not refresh on all instances. We need to refresh only when
			// the table was under the influence of a search, basically search with a non-default filter. 
			// Further we do a client side filtering on the table, so that None gets displayed if no rows
			// show up in the search results.
			if ( this.projectsTablePanel._projectsTable._isSearchFilterActive() ){
				this.projectsTablePanel._projectsTable.grid.filter(this.projectsTablePanel._projectsTable.grid.query,false);
			}
		}
		
		this._trace.exiting(this.id, "_onFinishCreation");
	},
	
	showRoleAnalysisByRoleId:function(roleId) {
		var roleUri=this.roleHierarchyPanel.getRoleUriFromId(roleId);
		
		var roleAnalysisData = {
			"projectUrl": this.roleHierarchyPanel.projectUrl,
			"modelId": this.roleHierarchyPanel._modelId,
			"roleUrl":roleUri
		};
		this._showRoleAnalysis(roleAnalysisData);
	},
	showMessageDialog:function(/*string*/message,/*string*/messageId,/*string*/msgType,/*function*/callback,/*object?*/callbackParms){
		// summary: 
		//			displays the message dialog
		// message: String
		//		Message that will be shown in the dialog
		// messageId: String
		//		TMS  message Id
		// msgType: String
		//		 defines the type of message dialog ("Error","Critical", "Warning", "Info", "Confirm").
		// callback: Function?
		//		Function to be called after the dialog is closed
		// callbackParms: Object?
		//		Any parameters to be passed to callback function
		// tags:
		//			public
		this._trace.entering(this.id, "showMessageDialog");
		var msgDialog=this.panelFactory.createMessageDialog(message,messageId,msgType,callback,callbackParms);
		msgDialog.show();
		this._trace.exiting(this.id, "showWorkingDialog");
	},
	
	showWorkingDialog:function(msg){
		// summary: 
		//			displays the progress dialog
		// tags:
		//			public
		this._trace.entering(this.id, "showWorkingDialog");
		this._workingDialogCount++;
		if(msg !=null && typeof msg == "string"){
			this.msgPlaceHolder.innerHTML=msg;
		}else{
			msg=this.globalResources.LOADING;
		}
		this.workingDialog.show();
		this._trace.exiting(this.id, "showWorkingDialog");
	},
	
	hideWorkingDialog:function()  {
		// summary:
		//			hides the progress dialog
		// tags:
		//			public
		this._trace.entering(this.id, "hideWorkingDialog");
		this._workingDialogCount--;
		if ( this._workingDialogCount <= 0 ) {
			this.workingDialog.hide();
			this._workingDialogCount = 0;
		}
		this._trace.exiting(this.id, "hideWorkingDialog");	
	},
	
	destroy: function() {	
		// tags: 
		//			protected
		this._trace.enteringWithStack(this.id, "destroy", true);
		try {
			dojo.forEach(this.connections, dojo.disconnect);
			if (this.projectsTablePanel) {
				this.projectsTablePanel.destroy();
			}
			if (this.roleHierarchyPanel) {
				this.roleHierarchyPanel.destroy();
			}
			if (this.projectCreationWiz){
				this.projectCreationWiz.destroy();
			}
			this.inherited(arguments);
			this._trace.exiting(this.id, "destroy");
		} catch(e) {
			this._trace.exception(e, "destroy");
			throw e;
		}
	}
	
});