dojo.provide("com.ibm.security.modeling.controller.PanelFactory");

dojo.require("com.ibm.security.modeling.projects.ProjectsTablePanel");
dojo.require("com.ibm.security.modeling.hierarchy.RoleHierarchyPanel");
dojo.require("com.ibm.security.modeling.notebook.RoleNotebookDialogView");
dojo.require("com.ibm.security.modeling.membership.FilterBuilder");
dojo.require("ibm.tivoli.tip.dijit.TIPMessageDialog");
dojo.require("com.ibm.security.modeling.util.Trace");

dojo.declare("com.ibm.security.modeling.controller.PanelFactory", null, {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.controller.PanelFactory"),
	configObject: null,
	constructor: function(config) {
		// summary: 
		//			Creates panel widget objects for use in role modeling
		// description:
		//			Uses a basic factory pattern to create the appropriate panel objects.
		// config: Object
		//			An object containing basic environment options for the panels. 
		this.configObject = config;
	},
	createProjectsPanel: function() {
		this._trace.entering(this.configObject.id, "createProjectsTablePanel");
		var projectsTablePanel = new com.ibm.security.modeling.projects.ProjectsTablePanel(
			{
				namespace:this.configObject.namespace,
				messageArea:this.configObject.msgArea,
				mainContainer: this.configObject.parentPanel
			},
			this.configObject.namespace+"projectsTablePanel");
		this._trace.exiting(this.configObject.id, "createProjectsTablePanel");
		return projectsTablePanel;
	},
	
	createNewProjectWizardPanel: function(_onFinishCreation) {
			this._trace.entering(this.configObject.id, "createProjectCreationWizardPanel");
			dojo.require("com.ibm.security.modeling.projects.NewProjectWizard");
			var projectCreationWiz = new com.ibm.security.modeling.projects.NewProjectWizard (
					{id:this.configObject.namespace+"ProjectCreationWizard",
						namespace: this.configObject.namespace,
						msgArea: this.configObject.msgArea,
						hostname: this.configObject.hostname,
						port: this.configObject.port,
						callback_finish:_onFinishCreation,
						contextRoot: this.configObject.contextRoot,
						isCreateView: true,
						_parent:this.configObject.parentPanel},
					this.configObject.id+"_mainContainerProjectCreate"				
			);
			this._trace.exiting(this.configObject.id, "_createProjectCreationPanel");
			return projectCreationWiz;
	},
	createHierarchyPanel: function(projectUrl, modelId, roleLifecycleProvider) {
		this._trace.entering(this.configObject.id, "createHierarchyPanel");
		var roleHierarchyPanel = new com.ibm.security.modeling.hierarchy.RoleHierarchyPanel(
				{id:this.configObject.namespace+"RoleHierarchyPanel",
					namespace: this.configObject.namespace,
					messageArea: this.configObject.msgArea,
					projectUrl: projectUrl,
					hostname: this.configObject.hostname,
					port: this.configObject.port,
					contextRoot: this.configObject.contextRoot,
					_parent:this.configObject.parentPanel,
					_modelId: modelId,
					messageArea: this.configObject.msgArea,
					roleLifecycleProvider : roleLifecycleProvider
					},
				this.configObject.id+"_mainContainerHierarchyPlaceHolder");
		this._trace.exiting(this.configObject.id, "createHierarchyPanel");
		return roleHierarchyPanel;
	}, 
	createRolePanel: function(roleUri, initialPanel, hierarchyPanel,roleLifecycleProvider) {
		this._trace.entering(this.configObject.id + " - roleUri: " + roleUri, "createRolePanel");
		var _isErrorWarningPanelFirst=false;
		if(initialPanel == "errorWarning"){
			_isErrorWarningPanelFirst=true;
		}
		var rolePanel = new com.ibm.security.modeling.notebook.RoleNotebookDialogView({
				id:this.configObject.id+"_rolePanel", 
				namespace:this.configObject.namespace,
				roleUri: roleUri,
				contextRoot: this.configObject.contextRoot,
				parent:this.configObject.parentPanel,
				roleHierarchyPane: hierarchyPanel,
				designUrl: this.configObject.designUrl,
				_isErrorWarningPanelFirst:_isErrorWarningPanelFirst,
				_isGeneralPanelFirst:!(_isErrorWarningPanelFirst),
				msgArea: this.configObject.msgArea,
				projectUrl: hierarchyPanel.projectUrl,
				modelId: hierarchyPanel._modelId,
				roleLifecycleProvider : roleLifecycleProvider
			},
			this.configObject.id+"_mainContainerRolePanel");
		this._trace.exiting(this.configObject.id, "createRolePanel");
		return rolePanel;
	}, 
	createFilterBuilderPanel: function(filterData, roleItem, notebookTitle, rolePane, hierarchyPanel) {

		this._trace.entering(this.configObject.id + " - filterData: " + filterData, "createFilterBuilderPanel");
		var filterBuilder = new com.ibm.security.modeling.membership.FilterBuilder({
				id:this.configObject.id+"_filterBuilder",
				namespace:this.configObject.namespace,
				contextRoot: this.configObject.contextRoot,
				filterData: filterData,
				roleItem: roleItem,
				notebookTitle: notebookTitle,
				roleNotebookPane: rolePane,
				hierarchyPanel: hierarchyPanel,
				parent:this.configObject.parentPanel,
				designUrl: this.configObject.designUrl,
				"class": "contentArea",
				messageArea: this.configObject.msgArea
			},this.configObject.id+"_mainContainerfilterBuilderPanel");
		this._trace.exiting(this.configObject.id, "createFilterBuilderPanel"); 
		return filterBuilder;
	},
	createAnalysisPanel: function(contextData) {
		// Summary:
		//		Creates instance of Analysis Pane widget.
		
		this._trace.entering(this.configObject.id, "createAnalysisPanel");
		
		var analysisPane = new com.ibm.security.modeling.analysis.AnalysisPane({
			id: this.configObject.id+"_analysisPane",
			"class": "fullSize",
			contextData: contextData
		}, this.configObject.id+"_analysisPane");
		this._trace.exiting(this.configObject.id, "createAnalysisPanel");
		
		return analysisPane;
	},
	createMessageDialog:function(/*string*/message,/*string*/messageId,/*string*/msgType,/*function*/callback,/*object?*/callbackParms){
		// summary: 
		//			displays the message dialog
		// message: String
		//		Message that will be shown in the dialog
		// messageId: String
		//		TMS  message Id
		// msgType: String
		//		 defines the type of message dialog ("Error","Critical", "Warning", "Info", "Confirm").
		// callback: Function?
		//		Function to be called after the dialog is closed
		// callbackParms: Object?
		//		Any parameters to be passed to callback function
		var theme="tundra";
		if(!message){
			message="No message passed";
		}
		if(!messageId){
			messageId="";
		}
		var msgDialog= new ibm.tivoli.tip.dijit.TIPMessageDialog({theme:theme,
																type:msgType,
																message:message,
																messageId:messageId,
																callback:callback,
																callbackParms:callbackParms,
																namespace:this.configObject.namespace
															});
		msgDialog._oldOnHide=msgDialog._onHide;
		msgDialog._onHide=dojo.hitch(msgDialog,function(){
			if(this.callback){
				this.callback();
			}
			this._oldOnHide(arguments);
		});
		return msgDialog;
	}
});