dojo.provide("com.ibm.security.modeling.controller.PanelLauncher");

dojo.require("com.ibm.security.modeling.util.HelpUtilities");
dojo.require("com.ibm.security.modeling.util.TitleUtilities");
dojo.require("com.ibm.security.modeling.util.Trace");

dojo.declare("com.ibm.security.modeling.controller.PanelLauncher", null, {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.controller.PanelLauncher"),
	helpUtil: new com.ibm.security.modeling.util.HelpUtilities(),
	titleUtil: new com.ibm.security.modeling.util.TitleUtilities(),
	panelStack: null,
	constructor: function(panelStack) {
		// summary:
		//			Abstract class whose children launch specific panels for role modeling. 
		// description:
		//			This object contains stateful information that informs the data that is eventually
		//			loaded to the panel being launched.
		// panelStack: com.ibm.security.modeling.dijit.layout.StackContainer
		//			The dijit stack container that manages display of widgets.
		this.panelStack = panelStack;
	},
	launch:function(contentPane, titleName, fileName, panelId) {
		// summary:
		//			Performs common launch logic for any panel being launched
		// contentPane: com.ibm.security.modeling.dijit.layout.ContentPane
		//			Content pane to display in the stack.
		// titleName: String
		//			Key of the title to displayed for the panel
		// fileName: String
		//			Name of the help file to associate with the panel.
		// tags:
		//			extension
		this._trace.entering(titleName, "launch");
		this.panelStack.selectChild(contentPane);
		this.titleUtil.updateTitle(titleName);
		if (fileName) {
			this.helpUtil.setHelp(fileName);	
		}
		this._trace.exiting(titleName, "launch");
	},
	publishPanelChangedEvent:function(/*Object*/breadcrumbArgs, panelId, /*Object*/roleAnalysisData) {
		// summary: 
		//		sends an event telling anyone listening that the panel has changed.
		// breadcrumbArgs.context: Object?
		//		What object to run the breadcrumb methods in
		// breadcrumbArgs.hideFunction: Function?
		//		What function to execute when the breadcrumb is clicked 
		// breadcrumbArgs.label: String
		//		What label to apply to the breadcrumb
		// panelId: String
		//		Id representing a panel being launched/switched to.
		// roleAnalysisData.modelId: String
		//		Address of the currently selected model.
		// roleAnalysisData.roleUrl.String
		// 		Address of the currently selected role.
		// roleAnalysisData.projectUrl.String
		// 		Address of the currently selected project.
		this._trace.entering("Panel ID: " + panelId, "publishPanelChangedEvent");
		if (this._trace.isFinest() && roleAnalysisData) {
			this._trace.finest("Model ID: " + roleAnalysisData.modelId + ", Role URL: " + roleAnalysisData.roleUrl + ", Project URL: " + roleAnalysisData.projectUrl, "publishPanelChangedEvent");
		}
		//this method is invoked , when stackcontainer has already taken its action , so we dont want breadcrumbs to use can hide
		//as that will bring unwanted side effects .
		breadcrumbArgs.overrideCanHide=true;
		dojo.publish("/com/ibm/security/modeling/panelChanged", [breadcrumbArgs, panelId, roleAnalysisData]);
		this._trace.exiting("Panel ID: " + panelId, "publishPanelChangedEvent");
	}
});