/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

� Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.exporter.ExportDialog");

dojo.require("ibm.tivoli.tip.dijit.TIPMessageDialog");
dojo.require("com.ibm.security.modeling.dijit.layout.BorderContainer");
dojo.require("com.ibm.security.modeling.dijit.layout.ContentPane");
dojo.require("dijit.Dialog");
dojo.require("dojo.parser");
dojo.require("com.ibm.security.modeling.widgets.IsrmDialogBase");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.requireLocalization("com.ibm.security.modeling.exporter", "ExportDialogText");

dojo.declare("com.ibm.security.modeling.exporter.ExportDialog",com.ibm.security.modeling.widgets.IsrmDialogBase,{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.exporter.ExportDialog"),
	templatePath : dojo.moduleUrl("com.ibm.security.modeling.exporter", "templates/ExportDialog.html"),
	roles_icon: dojo.moduleUrl("com.ibm.security.modeling.themes", "images/roles.png"),
	identities_icon: dojo.moduleUrl("com.ibm.security.modeling.themes", "images/userIcon22.png"),
	sod_constraints_icon: dojo.moduleUrl("com.ibm.security.modeling.themes", "images/policies.png"),
	autofocus:"false",
	heightFactor:10,
	minHeight:275,
    minWidth:320,
	widthFactor:3, 
	widgetsInTemplate : true,
	killOnHide : false,
	exportURI : "",
	exportFileName : "",
	attributeMap : {
		rolesCount : {node : "rolesCountNode", type : "innerText"},
		sodCount : {node : "sodConstraintsCountNode", type : "innerText"},
		identitiesCount : {node : "identitiesCountNode", type : "innerText"},
		fileName : {node : "fileNameNode", type : "innerHTML"}
		
	},
	
	/**
	 * Overriden method to set property.
	 * @return
	 */
	postMixInProperties:function() {
		this._trace.entering(this.id, "postMixInProperties");
		try {			
			this.inherited(arguments);
			this.localResorce = dojo.i18n.getLocalization("com.ibm.security.modeling.exporter", "ExportDialogText");

			this.helpFile = "ui_export_project.html";
			this.attachedIdentities = this.localResorce.ATTACHED_IDENTITIES;
			this.download = this.localResorce.DOWNLOAD;
			this.dialogInstructon = this.localResorce.DIALOG_INSTRUCTION;
			this.savedToFileInstrunction = this.localResorce.SAVE_TO_FILE_INSTRUCTION;
			this.dialogTitle = this.localResorce.DIALOG_TITLE;
			
		} catch (e) {
			this._trace.exception(e, "postMixInProperties");
			throw e;
		} finally {
			this._trace.exiting(this.id, "postMixInProperties");
		}
	},
	
	/**
	 * Overriden method.
	 * 
	 * @return
	 */
	postCreate: function() {
		this._trace.entering(this.id, "postCreate");
		
		this.inherited(arguments);
		this._hookUpButtons();
		
		this._trace.exiting(this.id, "postCreate");		
	},
	
	/**
	 * To connect button with events
	 * 
	 * @return
	 */
	_hookUpButtons:function() {
		this._trace.entering(this.id, "_hookUpButtons");
	
		this.closeButton = this.addCommonButton({
			onClick:dojo.hitch(this, this.closeExportDialog),
			type:"CLOSE"
		});
		
		this.connect(
				dijit.byId(this.id+"_attached_identities"), 
				"onClick",
				function(attachedIdentities) {
					console.log("Attached identities selected : "+attachedIdentities);
					this.setExportURL();
				}
			); 
		this._trace.exiting(this.id, "_hookUpButtons");		
	},
	setExportURL : function(){
		var attachIdentities = dijit.byId(this.id+"_attached_identities").get("checked");
		var fileParsedFileName=com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(this.exportFileName);
		this.set("fileName","<a id='"+this.id+"_file_Name' href='"+this.exportURI+"&attachIdentities="+attachIdentities+"' target='_blank'>"+fileParsedFileName+"</a>");
	},
	
	/**
	 * To hide the export dialog box.
	 * 
	 * @return
	 */
	closeExportDialog : function (){
		this._trace.entering(this.id, "closeButtonPressed");
	
		try {
			this.hide();			
		} catch(e) {
			this._trace.exception(e, "closeButtonPressed");
			throw e;
		} finally {
			this._trace.exiting(this.id, "closeButtonPressed");			
		}		
	},
	
	/**
	 * To populate value in the export dialog dynamically. 
	 * @param 	argument.rolesCount 
	 * 						Roles count
	 * 			argument.sodCount
	 * 						Separation of Duty Constraints count
	 * 			argument.exportURI
	 * 						Project export URI
	 * 			argument.fileName
	 * 						Project name with extension
	 * @return
	 */
	populate: function(argument){
		this._trace.entering(this.id, "populate");
		
		try {
			if (argument) {
				var identitiesCountLabel = dojo.string.substitute(this.localResorce.IDENTITIES_COUNT, {"identitiesCount":argument.identitiesCount});
				var rolesCountLabel = dojo.string.substitute(this.localResorce.ROLES_COUNT, {"rolesCount":argument.rolesCount});
				var sodCountLabel = dojo.string.substitute(this.localResorce.SEPERATION_OF_DUTY_CONSTRAINTS_COUNT, {"sodCount":argument.sodCount});
				this.set("identitiesCount",identitiesCountLabel);
				this.set("rolesCount",rolesCountLabel);
				this.set("sodCount",sodCountLabel);
				this.exportURI = argument.exportURI;
				this.exportFileName = argument.fileName;
				dijit.byId(this.id+"_attached_identities").checked=true;//checked 
				dijit.byId(this.id+"_attached_identities").setAttribute("checked", "true");
				this.setExportURL();
			}
		} catch (e) {
			this._trace.exception(e, "populate");
		} 
		this.show();
	
		this._trace.exiting(this.id, "populate");			
	}
		
});
