/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

? Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.exporter.ExportHelper");

dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.util.MessageUtilities");
dojo.require("com.ibm.security.modeling.api.ProjectRestManager");

dojo.requireLocalization("com.ibm.security.modeling", "PluginMessages");
dojo.requireLocalization("com.ibm.security.modeling.projects", "ProjectsTableWizardText");

dojo.declare("com.ibm.security.modeling.exporter.ExportHelper",null,{
	id:null,
	globalResources : null,
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.exporter.ExportHelper"),
	
	/**
	 * To construct ExportHelper object.
	 * @param argu
	 * @return
	 */
	constructor : function(argu) {
		this.id = argu.id;
		this.globalResources = argu.globalResources;
		this.pluginMessages = dojo.i18n.getLocalization("com.ibm.security.modeling", "PluginMessages");
		this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.projects", "ProjectsTableWizardText");
		this._messageUtils = new com.ibm.security.modeling.util.MessageUtilities();
		this.messageArea=argu.messageArea;
	},
	
	getProjectItem: function(projectId){
		this._trace.entering(this.id, "getProjectItem");
		// Construct the project rest manager 
		this.projectRestManager = new com.ibm.security.modeling.api.ProjectRestManager({
			resourceUri: "/isrm/rest/projects/",
			syncMode: true,
			messageArea:this.messageArea
		});

		var projectItem = this.projectRestManager.loadResource(projectId);
		this._trace.exiting(this.id, "getProjectItem");
		return projectItem;
	},
	
	/**
	 * To launch the export dialog box. It validate count of roles and sod, if
	 * no roles or sod found, it creates information dialog box, else it creates
	 * export dialog box.
	 * 
	 * @param project Project object.
	 * @return
	 */
	exportProject:function(projectId){
		this._trace.entering(this.id, "exportProject");
		project = this.getProjectItem(projectId);
		var exportModels = project.models;
		if(exportModels == null && exportModels.length != 1){
			this._trace.error(this.id, "Export Error : Project dose not have model or has multiple model.");
			return;
		}	
		var exportModel = exportModels[0];
	
		var exportArgument =  new Array();
		exportArgument.rolesCount = exportModel.roleCount;
		exportArgument.identitiesCount = exportModel.identityRoleAttchementCount;
		exportArgument.sodCount = exportModel.sodRoleConstraintCount;
		exportArgument.fileName = project.name+".xml"; 
		exportArgument.exportURI = project.exportURI; 
		
		if(project.status !== "R"){//project export allowed only in Ready to edit state
			var status="";
		    var statusKey = "PROJECT_STATUS_"+project.status;
			if(this.localResources[statusKey]){
				status = this.localResources[statusKey];
			}else{
				status = this.localResources.PROJECT_STATUS_U;
			}
				
			var syncStatus="";
			var syncStatusKey = "PROJECT_SYNC_STATUS_"+project.synchronizationStatus;
			if(this.localResources[syncStatusKey]){
				syncStatus = this.localResources[syncStatusKey];
			}else{
				syncStatus = this.localResources.PROJECT_SYNC_STATUS_U;
			}
			var inserts=[];
			inserts.push(project.name);
			inserts.push(status);
			inserts.push(syncStatus);
			var msg = this._messageUtils.formatMessage(this.pluginMessages.STATE_MANAGEMENT_INVALID_EXPORT_FOR_PROJECT, inserts);
			dojo.publish(dojo.config.panels.MESSAGE_DIALOG.launchEventAddress,[msg,"","Error",function(){
														dojo.publish(dojo.config.panels.PROJECTS.launchEventAddress,[false,true]);
						}]);
			return;
		}
		
		if(exportModel.roleCount > 0 || exportModel.sodRoleConstraintCount > 0){
			if(this._exportDialog == null){
				this._exportDialog = this.createExportDialog();
			}
			this._exportDialog.populate(exportArgument);		
		}else{
			if(this._exportInfoMessageDialog == null){
				this._exportInfoMessageDialog = this.createExportInfoMessageDialog();
			}
			this._exportInfoMessageDialog.startup();
			this._exportInfoMessageDialog.show();
		}
		
		this._trace.exiting(this.id, "exportProject");
	},
	
	/**
	 * To create the export dialog box.
	 * 
	 * @return
	 */
	createExportDialog : function (){
		this._trace.entering(this.id, "createExportDialog");
		
		dojo.require("com.ibm.security.modeling.exporter.ExportDialog");
		
		var _exportDialog = new com.ibm.security.modeling.exporter.ExportDialog({
					id : this.id+"ExportDialog",
					globalResources : this.globalResources
					},
			null);
		_exportDialog.startup();
		this._trace.exiting(this.id, "createExportDialog");
		return _exportDialog;
	},
	
	/**
	 * To create export information dialog box.
	 * 
	 * @return
	 */
	createExportInfoMessageDialog : function(){
		this._trace.entering(this.id, "createExportInfoMessageDialog");
		
		dojo.require("com.ibm.security.modeling.widgets.InfoMessageDialog");
		dojo.requireLocalization("com.ibm.security.modeling", "LogMessages");
		dojo.requireLocalization("com.ibm.security.modeling.exporter", "ExportDialogText");
	
		var logMessages = dojo.i18n.getLocalization("com.ibm.security.modeling", "LogMessages");
		var exportResorce = dojo.i18n.getLocalization("com.ibm.security.modeling.exporter", "ExportDialogText");
		
		var _exportInfoMessageDialog = new com.ibm.security.modeling.widgets.InfoMessageDialog({
					id : this.id+"InfoMessageDialog",
					globalResources : this.globalResources,
					title : exportResorce.INFORMATION_DIALOG_TITLE,
					infoMessages : logMessages.CONSOLE_COULD_NOT_EXPORT_INFO
					},
			null);
		this._trace.exiting(this.id, "createExportInfoMessageDialog");
		return _exportInfoMessageDialog;
	}
	
});
