dojo.provide("com.ibm.security.modeling.hierarchy.HierarchyPanelLauncher");

dojo.require("com.ibm.security.modeling.controller.PanelLauncher");
dojo.require("com.ibm.security.modeling.util.UrlUtilities");
dojo.require("com.ibm.security.modeling.util.Trace");

dojo.declare("com.ibm.security.modeling.hierarchy.HierarchyPanelLauncher", com.ibm.security.modeling.controller.PanelLauncher, {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.hierarchy.HierarchyPanelLauncher"),
	contentPane: null,
	panelWidget: null,
	projectUrl: null,
	modelId: null,
	titleName: null,
	fileName: "ui_roles_policies.html",
	urlUtil: new com.ibm.security.modeling.util.UrlUtilities(),
	
	constructor: function(panelStack, contentPane, panelWidget, projectUrl, modelId, globalResources, loadData) {
		// summary:
		// 			Launches the role hierarchy panel
		// panelStack: com.ibm.security.modeling.dijit.layout.StackContainer
		//			Stack container controlling display of panels
		// contentPane: com.ibm.security.modeling.dijit.layout.ContentPane
		//			Content pane for the stack container to show.
		// panelWidget: Object
		//			The panel widget object to be loaded.
		// projectUrl: String
		//			The uri representing the project to be shown
		// modelId: String
		// 			The ID representing the model to show roles for
		// loadData: Boolean
		//			Whether the contents of the the panel should be loaded/reloaded
		// globalResources: Object
		//			Resource bundle to load title text from.
		this._trace.entering("Project Url: " + projectUrl + ", Model ID: " + modelId + ", Load data: " + loadData, "constructor");
		this.projectUrl = projectUrl;
		this.modelId = modelId;
		this.loadData = loadData;
		this.contentPane = contentPane;
		this.panelWidget = panelWidget;
		this.titleName = globalResources.ROLES_AND_POLICIES;
		this.breadcrumbArgs = {
			context:this.panelWidget,
			hideFunction: "closePanel",
			label: this.titleName
		};
		this.roleAnalysisData = {
			modelId: this.modelId,
			projectUrl: this.projectUrl
		};
		this.panelWidget.launcher = this;
		this._trace.exiting("Project Url: " + projectUrl + ", Model ID: " + modelId + ", Load data: " + loadData, "constructor");
	},

	launch:function() {
		this._trace.entering("", "launch");
		this.inherited(arguments, [this.contentPane, this.titleName, this.fileName]);
		this.publishPanelChangedEvent(this.breadcrumbArgs, dojo.config.panels.HIERARCHY.id, this.roleAnalysisData);
		if (this.projectUrl && this.modelId && this.loadData) {
			this.panelWidget.loadInitialData(this.projectUrl, this.modelId);
		}
		this._trace.exiting("", "launch");
	}
});