/*
Licensed Materials - Property of IBM

Role and Policy Modeler

? Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.hierarchy.RoleScopingPanel");
dojo.require("com.ibm.security.modeling.widgets.AttributeScopingPanel");

// NLS
dojo.requireLocalization("com.ibm.security.modeling.hierarchy", "RoleCopyDialogText");

dojo.require("com.ibm.security.modeling.util.Trace");


dojo.declare("com.ibm.security.modeling.hierarchy.RoleScopingPanel",
            [ com.ibm.security.modeling.widgets.AttributeScopingPanel],
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.hierarchy.RoleScopingPanel"),
	// localResources: Object
	//		The resource bundle for this widget set
	localResources: null,
	// namespace: String
	//		The identifier for the current portlet
	namespace: null,
	// messageArea: ibm.tivoli.tip.dijit.TIPMessageArea
	//		Message area for error display
	//messageArea: null,
	// hostname: String
	//		Hostname for server requests
	hostname: "",
	// port: String
	//		Port for server requests
	port: "16311",
	// contextRoot: String
	//			The context root of the server from which Content is being served.
	contextRoot: "",
	// searchAttribute: String
	//			The context root of the server from which Content is being served.
	searchAttribute:"displayName",
	// globalPoolRolesUri: String
	//			Uri used for RoleName attribute in the dropdown
	globalPoolRolesUri:"/isrm/rest/models/-1000/roles/",
	// viewAllUrl: String
	//			The uri which will fetch the ViewAll dialog box content and query for count
	viewAllUrl:"/isrm/rest/rolescope/?queryFormat=AttributeFilter",
	// defaultValue: String
	//			Default value for the attribute dropdown
	defaultValue:"ROLENAME",
	// nameAttributeSelectionLimit: number
	//			Max Role Name Selection Limit (direct selection)
	nameAttributeSelectionLimit: 15,
	// attrValueSelectionLimit: number
	//			Max Attribute Values Selection Limit 
		attrValueSelectionLimit: 15,
	// noResourceSelectionMsg: String
	//			The message key use to be show message if no roles are selected
	noResourceSelectionMsg:"NO_ROLES_OR_RESOURCES_SELECTED",
	// selectionLimitMsg: String
	//			The message key use to be show message if roles/attribute value are above the limit.
	selectionLimitMsg:"ROLE_MAX_LIMIT_FOR_COPY_ROLES_INTO_PROJECT",
	postMixInProperties: function() {
		this._trace.entering(this.id, "postMixInProperties");
		
		this.inherited(arguments);
		this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.hierarchy", "RoleCopyDialogText");
		this.dropDownLabel=this.localResources.DROP_DOWN_LABEL;
		this.roleNameAttributeUri="/isrm/rest/roles/";
		this.idAttribute="storeId";
		
		
		this._trace.exiting(this.id, "postMixInProperties");
	},
	postCreate: function() {
		// tags:
		//		protected
		
		
		
		this._loadAttributeDropDown();
		this.inherited(arguments);
		
	},
	_loadAttributeDropDown:function(){
		dojo.require("com.ibm.security.modeling.api.AttributeScopeRestManager");
		if(!this.attributeScopeManager){
			this.attributeScopeManager = new com.ibm.security.modeling.api.AttributeScopeRestManager({
					scopeType: "role"
			});
		}
		
		var stateStore=this.attributeScopeManager.getJsonRestStore();
		var attrItems = null;
        var gotItems = function(items, request){
        	
        	//this._trace.finest('Number of attributes found: ' + items.length,"_getStoreFromUrl");
		  attrItems = items;
		};
		stateStore.fetch({onComplete: gotItems, onError:dojo.hitch(this,this._errorHandler),syncMode: "true", cache: "false"});

		
		var attrDataItems = [];
		if( (attrItems!= null) && (attrItems.length > 0) )
		{
			for(var i = 0; i < attrItems.length; i++){
				var customAttributeId="customAttributeId"+i;
				attrDataItems.push({displayName:attrItems[i].name,name:attrItems[i].name, id:attrItems[i].name, dbId:attrItems[i].id, type:attrItems[i].type,uri:attrItems[i].uri});
			}
		}
		attrDataItems.push({displayName:this.localResources.ROLE_NAME,name:"ROLENAME",id:"ROLENAME", type:"NAME",uri:this.roleNameAttributeUri});
		var attrData = {
		  identifier:"id",
		  label: "displayName",
		  items: attrDataItems
		};
		dojo.require("dojo.data.ItemFileReadStore");
		this.attributeStore = new dojo.data.ItemFileReadStore({
   				data: attrData
		});
	},

	startup: function() {
		// tags:
		//		protected
		this.inherited(arguments);
	},
	_createNameAttributeTable:function(value,item){
		dojo.require("com.ibm.security.modeling.api.RoleRestManager");
		var schemaRestMgr= new com.ibm.security.modeling.api.RoleRestManager({
			resourceUri: this.globalPoolRolesUri,
			syncMode: false,
			_identifier:"uri",
			messageArea: this.messageArea
		});
        var store = schemaRestMgr.getIsrmTableRestStore();

		dojo.require("com.ibm.security.modeling.tables.GlobalRoleTable");
		table = new com.ibm.security.modeling.tables.GlobalRoleTable(
		{
			id : this.id + value,
			dataStore:store,
			isSearchMultiSelect: false,
			messageArea: this.messageArea
		});
		table.grid.attributeDetail=item;
		this._setupTableConnections(table);
		return table;
	},
	getHelpFileName: function(){
		return "ui_users_select.html";
	},
	getViewAllTable:function(store,queryParams){
		dojo.require("com.ibm.security.modeling.hierarchy.ViewAllGlobalRoleTable");
		var table = new com.ibm.security.modeling.hierarchy.ViewAllGlobalRoleTable(
		{
			id:this.id+"_viewAllTable",
			dataStore:store,
			query:queryParams,
			isSearchMultiSelect: false,
			messageArea: this.messageArea
		});
		return table;
		
	}
});
