/* 
Licensed Materials - Property of IBM

5724-Z11??

ï¿½ Copyright IBM Corp. 2009 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.hierarchy.SummaryContentPane");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.security.modeling.api.ProjectRestManager");
dojo.require("dojo.string");
dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
dojo.require("com.ibm.security.modeling.util.Trace");

dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling.hierarchy", "SummaryContentPaneText");
dojo.requireLocalization("com.ibm.security.modeling", "PluginMessages");

dojo.declare("com.ibm.security.modeling.hierarchy.SummaryContentPane",
		     [dijit._Widget, dijit._Templated],
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.hierarchy.SummaryContentPane"),

	// Provides utilities for handling errors.
	_errorUtil: null,

	// Template string for summary content
	/*templateString: dojo.cache("com.ibm.security.modeling.hierarchy",
			                   "templates/SummaryContentTemplate.html",
			                   {sanitize: true}),*/
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.hierarchy","templates/SummaryContentTemplate.html"),
	widgetsInTemplate: true,
	
	// The context root of the server from which Content is being served.
	contextRoot: "",
			               	
	// Object: Reference to the local resource bundle
	localResources: null,
	
	// Object: Reference to the global resource bundle
	globalResources: null,
	
	// Reference to the project rest store
	_projectItem: null,
	
	// project url
	projectUrl: "",

	// namespace: String
	//		The identifier for the current portlet
	namespace: "com.ibm.security.modeling.hierarchy.SummaryContentPane",
	
	// It is an interface for HTTP REST communication using dojo stores api. 
	projectRestManager: null,
	
	// Project description 
	_projectDescp: "",
	
	//Edit Dialog for modifying project's general information like - name and description
	_projectEditGeneralInfoDialog: null,
	
	//Edit Dialog for modifying project's scope information like - user and permission selections
	_projectEditScopeInfoDialog: null,

	msgArea: null,

	pluginResources: null,

	// hostname: String
	//		Hostname for server requests
	hostname: "",
	// port: String
	//		Port for server requests
	port: "16311",

    constructor: function (args, node) {
	    this._trace.entering(this.namespace, "constructor");
	    
	    dojo.mixin(this, args);

		this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();

		this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.hierarchy", "SummaryContentPaneText");
		this.pluginResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "PluginMessages");
		
		// Construct the project rest manager 
		this.projectRestManager = new com.ibm.security.modeling.api.ProjectRestManager({
			resourceUri: "/isrm/rest/projects/",
			syncMode: false
		});
			
		this._trace.exiting(this.namespace, "constructor");
	},
	
	postCreate:function(){
		this._trace.entering(this.namespace, "postCreate");
		
		this.inherited(arguments);

		// Load summary information
		this.loadSummaryInfo(this.projectUrl, dojo.hitch(this, function(result){
			this._projectItem = result;
			// Update the summary information 
			this.updateSummaryInfo();		
		}));

		this._trace.exiting(this.namespace, "postCreate");
	},
	
	loadSummaryInfo:function(projectUrl, successCallback){
		this._trace.entering(this.namespace, "loadSummaryInfo");
		this.projectUrl = projectUrl;
		
		// Extract the project id associated with the incoming uri
		var index = this.projectUrl.lastIndexOf("/");
		var projectsUri = this.projectUrl.substring(0, index+1);	// with "/"
		var projectId = this.projectUrl.substring(index+1);
        
		// Load the pproject for the given id
		this.projectRestManager.loadResource(projectId, successCallback);
		this._trace.exiting(this.namespace, "loadSummaryInfo");
	},

	incrementSummaryRoleCount:function(){
		this._trace.entering(this.namespace, "incrementSummaryRoleCount");
		
		dojo.query("span[id="+this.namespace+"coveredRolesCount]", this.domNode)[0].innerHTML=++this._projectItem.coveredRolesCount;
		
		this._trace.exiting(this.namespace, "incrementSummaryRoleCount");
	},
	
	updateSummaryInfo:function(){
		this._trace.entering(this.namespace, "updateSummaryInfo");
		
		this._projectDescp = "";
		if (this._projectItem.description != null){
			this._projectDescp = this._projectItem.description;
		}
		
		var userCountDisplay = dojo.string.substitute(this.localResources.USERS,{userCount:this._projectItem.totalIdentityCount});		
		var permissionCountDisplay = dojo.string.substitute(this.localResources.PERMISSIONS,{permissionsCount:this._projectItem.totalPermissionCount});
		var roleCountDisplay = dojo.string.substitute(this.localResources.ROLES,{rolesCount:this._projectItem.coveredRolesCount});
		var sodCountDisplay = dojo.string.substitute(this.localResources.SEPARATION_OF_DUTY_POLICIES,{SODCount:this._projectItem.coveredPoliciesCount});
		
		dojo.query("span[id="+this.namespace+"projectName]", this.domNode)[0].innerHTML=this._parseHTMLTag(this._projectItem.name);
		dojo.query("span[id="+this.namespace+"projectDescp]", this.domNode)[0].innerHTML=this._parseHTMLTag(this._projectDescp);
		dojo.query("span[id="+this.namespace+"totalIdentityCount]", this.domNode)[0].innerHTML=userCountDisplay;
		dojo.query("span[id="+this.namespace+"totalPermissionCount]", this.domNode)[0].innerHTML=permissionCountDisplay;
		dojo.query("span[id="+this.namespace+"coveredRolesCount]", this.domNode)[0].innerHTML=roleCountDisplay;
		dojo.query("span[id="+this.namespace+"sodRoleConstraintCount]", this.domNode)[0].innerHTML=sodCountDisplay;
		
		this.coverageContainerNode.setCoveragePercentage(this._projectItem.coveragePercentage);
		
		this._trace.exiting(this.namespace, "updateSummaryInfo");
	},
	
	refreshSummaryInfo:function(projectUrl){
		this._trace.entering(this.namespace, "refreshSummaryInfo");
	    this.loadSummaryInfo(projectUrl, dojo.hitch(this, function(result){
	    	this._projectItem = result;
	    	this.updateSummaryInfo();	    
	    }));
		this._trace.exiting(this.namespace, "refreshSummaryInfo");
	},
	
	onClickProjectEditGeneralInfo: function()
	{
		dojo.require("com.ibm.security.modeling.projects.ProjectEditGeneralInformation");

		var args = { projectUrl:this.projectUrl, projectName:this._projectItem.name, projectDescr:this._projectDescp, parentWidget:this };

		if (!this._projectEditGeneralInfoDialog)
		{
			this._projectEditGeneralInfoDialog = new com.ibm.security.modeling.projects.ProjectEditGeneralInformation(args);
		}
		else
		{
			this._projectEditGeneralInfoDialog = null;
			this._projectEditGeneralInfoDialog = new com.ibm.security.modeling.projects.ProjectEditGeneralInformation(args);
		}
		this._projectEditGeneralInfoDialog.startup();
		this._projectEditGeneralInfoDialog.show();
	},

	setUpdatedProjectGerenalInformation_Success: function(projectName, projectDescr, msg)
	{
		this._showMessage("info", msg);
		
		this.loadSummaryInfo(this.projectUrl, dojo.hitch(this, function(result){
			this._projectItem = result;
			this._projectDescp = this._projectItem.description;
		
			dojo.query("span[id="+this.namespace+"projectName]", this.domNode)[0].innerHTML=this._parseHTMLTag(this._projectItem.name);
			dojo.query("span[id="+this.namespace+"projectDescp]", this.domNode)[0].innerHTML=this._parseHTMLTag(this._projectDescp);
		}));
		
	},
	
	_parseHTMLTag: function(colHeader){
		colHeader= new String(colHeader);
		return colHeader.replace(/</g, "&#60;").replace(/\//g, "&#47;").replace(/>/g, "&#62;");
	},
	
	setMsgAreaFromParent: function(msgArea)
	{
		this.msgArea = msgArea;
	},
	
	_showMessage: function(/*String*/type, /*String*/message) {
		// summary:
		//			shows the given message and type in the local message area
		// tags:
		//			private
		if (type === "critical") {
			this.msgArea.showErrorMessage(message);
		}else if (type === "info") {
			this.msgArea.showInfoMessage(message);
		}
		/*this.messageArea.setType(type);
		this.messageArea.setMessage(message);
		this.messageArea.show();	*/	
	},
	
	destroy: function() {// TODO: cleanup.
		this.inherited(arguments);
	},
	
	onClickProjectEditScopeInfo: function()
	{
		dojo.require("com.ibm.security.modeling.projects.ProjectEditScopeInformation");

		var args = { projectUrl:this.projectUrl, projectName:this._projectItem.name, projectDescr:this._projectDescp, parentWidget:this };

		if (this._projectEditScopeInfoDialog!=null)
		{
			this._projectEditScopeInfoDialog = null;
		}	
			//this._projectEditScopeInfoDialog = new com.ibm.security.modeling.projects.ProjectEditScopeInformation(args);
			//this._projectEditScopeInfoDialog = new com.ibm.security.modeling.projects.ProjectEditScopeInformation(args);
		try {	
			this._projectEditScopeInfoDialog = new com.ibm.security.modeling.projects.ProjectEditScopeInformation({
				id:this.namespace+"_ProjectEditScope", 
				namespace:this.namespace,
				contextRoot: this.contextRoot,
				hostname:this.hostname,
				port:this.port,
				projectUrl: this.projectUrl,
				_projectItem: this._projectItem
			},
			null);

			this._projectEditScopeInfoDialog.startup();
			this._projectEditScopeInfoDialog.performInitialFillingOfScopingInfo();
			this._projectEditScopeInfoDialog.show();
		}
		catch(error)
		{
			this._trace.error(error, "onClickProjectEditScopeInfo");
			if (this.msgArea) {
				this.msgArea.hideMessages();
			}
			if ((error) && (error.responseText)) {
				this._errorUtil.handleRESTError(error, this.msgArea);
			} else if (this.msgArea) {
				this.msgArea.showErrorMessage(this.pluginResources.PLUGIN_REST_PROJECT_GET_FAILED);
			}
		}
	}

});