/* 
Licensed Materials - Property of IBM

5724-Z11??
Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.hierarchy.UndoHandler");

dojo.require("dijit.Menu");
dojo.require("dijit.MenuItem");
dojo.require("dijit.ToolbarSeparator");

dojo.require("com.ibm.security.modeling.api.UndoRestManager");
dojo.require("com.ibm.security.modeling.notebook.RoleTypeStore");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");
dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
dojo.require("com.ibm.security.modeling.util.MessageUtilities");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.util.ValidationUtilities");
dojo.require("com.ibm.security.modeling.util.WidgetUtilities");
dojo.require("com.ibm.security.modeling.widgets.ProgressBar");

dojo.requireLocalization("com.ibm.security.modeling", "PluginLabels");
dojo.requireLocalization("com.ibm.security.modeling.hierarchy", "RoleHierarchyText");

// Handles processing related to the undo stack.
dojo.declare("com.ibm.security.modeling.hierarchy.UndoHandler", null, {
	// messageArea: ibm.tivoli.tip.dijit.TIPMessageArea
	//		Message area for error display
	messageArea: null,
	// roleHierarchyPanel: com.ibm.security.modeling.hierarchy.RoleHierarchyPanel
	//		Role hierarchy panel in which this UndoHandler is contained.
	roleHierarchyPanel: null,
	// _errorUtilities: com.ibm.security.modeling.util.ErrorUtilities
	//		Utilities for handling errors.
	_errorUtilities: null,
	// _localResources: Object
	//		Reference to the resource bundle unique to this widget.
	_localResources: null,
	// _messageUtilities: com.ibm.security.modeling.util.MessageUtilities
	//		Utilities for formating messages.
	_messageUtilities: null,
	// _modelId: String
	//		Identifier of the model.
	_modelId: null,
	// _pluginResources: Object
	//		Reference to the resource bundle for plug-in labels.
	_pluginResources: null,
	// _progressBar: com.ibm.security.modeling.widgets.ProgressBar
	//		Progress bar displayed while undoing items.
	_progressBar: null,
	// _restManager: com.ibm.security.modeling.api.UndoRestManager
	//		REST manager.
	_restManager: null,
	// _roleHierarchyPanelConnection: Connection
	//		Connection used for displaying an error message on the role hierarchy panel.
	_roleHierarchyPanelConnection: null,
	// _roleTypes: Array of Object
	//		Role types that are defined in the system.
	_roleTypes: null,
	// _stringUtils: com.ibm.security.modeling.util.StringUtils
	//		Utilities for strings.
	_stringUtils: null,
	// _table: com.ibm.security.modeling.tables.RoleModelingTable
	//		Role table containing the undo button.
	_table: null,
	// _tableUndoButton: dijit.form.DropDownButton
	//		Undo button in the role table toolbar.
	_tableUndoButton: null,
	// _topology: ibm.tivoli.tip.navwidget.topology.dijit.TopologyView
	//		Role topology containing the undo button.
	_topology: null,
	// _topologyConnection: Connection
	//		Connection used for adding the undo button to the topology toolbar.
	_topologyConnection: null,
	// _topologyUndoButton: dijit.form.DropDownButton
	//		Undo button in the role topology toolbar.
	_topologyUndoButton: null,
	// _trace: com.ibm.security.modeling.util.Trace
	//		Trace handler.
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.hierarchy.UndoHandler"),
	// _validator: com.ibm.security.modeling.util.Validator
	//		Validator used for escaping strings.
	_validator: null,
	// _widgetUtilities: com.ibm.security.modeling.util.WidgetUtilities
	//		Utilities for widgets.
	_widgetUtilities: null,

	constructor: function(args) {
		// Summary:
		//		Constructs a com.ibm.security.modeling.hierarchy.UndoHandler.
		this._trace.entering("constructor");
		this._trace.object(arguments, "constructor", "Constructor args");
		dojo.mixin(this, args);
		this._errorUtilities = new com.ibm.security.modeling.util.ErrorUtilities();
		this._localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.hierarchy", "RoleHierarchyText");
		this._messageUtilities = new com.ibm.security.modeling.util.MessageUtilities();
		this._pluginResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "PluginLabels");
		this._getRoleTypes(dojo.hitch(this, function(results){
			this._roleTypes = results;
		}));
		this._stringUtils = new com.ibm.security.modeling.util.StringUtils();
		this._validator = new com.ibm.security.modeling.util.Validator();
		this._widgetUtilities = new com.ibm.security.modeling.util.WidgetUtilities();
		this._trace.exiting("constructor");
	},

	destroy: function() {
		// Summary:
		//		Destroys this com.ibm.security.modeling.hierarchy.UndoHandler.
		this.inherited(arguments);
		if (this._roleHierarchyPanelConnection != null) {
			dojo.disconnect(this._roleHierarchyPanelConnection);
			this._roleHierarchyPanelConnection = null;
		}
		if (this._topologyConnection != null) {
			dojo.disconnect(this._topologyConnection);
			this._topologyConnection = null;
		}
	},

	openDropDown: function(undoHandler, originalOpenDropDown) {
		// Summary:
		//		Receives control when the undo menu opens.
		//		This function executes under the scope of the undo button.
		//
		// undoHandler: com.ibm.security.modeling.hierarchy.UndoHandler
		//		UndoHandler associated with this undo button.
		// originalOpenDropDown: Function
		//		Original openDropDown function associated with this undo button.
		
		this.inherited(arguments);
		this.dropDown.domNode.style.width = "auto";
		this.openDropDown = originalOpenDropDown;
	},

	refresh: function() {
		// Summary:
		//		Asynchronously refreshes the undo stack and buttons.
		this._trace.entering("refresh");
		this._restManager.requestResource(
				null,
				dojo.hitch(this, this._refreshSuccess),
				dojo.hitch(this, this._refreshFailure));
		this._trace.exiting("refresh");
	},

	setTable: function(table, modelId) {
		// Summary:
		//		Sets the table containing the undo button.
		//
		// table: com.ibm.security.modeling.tables.RoleModelingTable
		//		Role table containing the undo button.
		// modelId: String
		//		Identifier of the model.
		this._trace.entering(table.id, "setTable");
		if (this._table != table) {
			this._table = table;
			this._tableUndoButton = dijit.byId(this._table.id + "undoBtn");
			this._tableUndoButton.openDropDown = dojo.hitch(this._tableUndoButton, this.openDropDown, this, this._tableUndoButton.openDropDown);
			this._setRestManager(modelId);
			this.refresh();
		}
		this._trace.exiting(table.id, "setTable");
	},

	setTopology: function(topology, modelId) {
		// Summary:
		//		Sets the topology containing the undo button.
		//
		// topology: ibm.tivoli.tip.navwidget.topology.dijit.TopologyView
		//		Role topology containing the undo button.
		// modelId: String
		//		Identifier of the model.
		this._trace.entering(topology.id, "setTopology");
		if (this._topology != topology) {
			if (this._topologyConnection != null) {
				dojo.disconnect(this._topologyConnection);
				this._topologyConnection = null;
			}
			this._topology = topology;
			this._setRestManager(modelId);
			this._topologyConnection = dojo.connect(this._topology, "_createToolbar", dojo.hitch(this, this._addUndoButtonToTopologyToolbar));
		}
		this._trace.exiting(topology.id, "setTopology");
	},

	_addUndoButtonToTopologyToolbar: function() {
		// Summary:
		//		Adds the undo button to the topology toolbar.
		this._trace.entering("_addUndoButtonToTopologyToolbar");
		var toolbar = this._topology._toolbar.getToolbar();
		var separator = new dijit.ToolbarSeparator();
		toolbar.addChild(separator, 7);
		var undoMenu = this._getUndoMenu();
		this._topologyUndoButton = this._widgetUtilities.createButton(
			this._localResources.UNDO,
			false,
			"undoIcon",
			null,
			this._topology.id + "undoBtn",
			false,
			undoMenu,
			null);
		this._topologyUndoButton.openDropDown = dojo.hitch(this._topologyUndoButton, this.openDropDown, this, this._topologyUndoButton.openDropDown);
		toolbar.addChild(this._topologyUndoButton, 8);
		this.refresh();
		this._trace.exiting("_addUndoButtonToTopologyToolbar");
	},

	_disableUndoButton: function(undoButton) {
		// Summary:
		//		Disables the specified undo button.
		//
		// undoButton: dijit.form.DropDownButton
		//		Undo button that should be disabled.
		this._trace.entering(undoButton.id, "_disableUndoButton");
		var undoMenu = this._getUndoMenu(null, undoButton.dropDown);
		this._widgetUtilities.disableButton(undoButton);
		this._trace.exiting(undoButton.id, "_disableUndoButton");
	},

	_displayUndoError: function(error) {
		// Summary:
		//		This function receives control upon the completion of refresh
		//		after an undo error occurs.  The refresh is performed just in
		//		case there was partial success.  This function displays the
		//		error message.  The error message cannot be displayed before
		//		the refresh because the refresh would wipe it out.
		//
		// error: Object
		//		Error that occurred.
		this._trace.entering("_displayUndoError");
		if (this._roleHierarchyPanelConnection != null) {
			dojo.disconnect(this._roleHierarchyPanelConnection);
			this._roleHierarchyPanelConnection = null;
		}
		this._errorUtilities.handleRESTError(error, this.messageArea);
		this._trace.exiting("_displayUndoError");
	},

	_enableUndoButton: function(undoButton, undoStack) {
		// Summary:
		//		Enables the specified undo button with the specified undo stack.
		//
		// undoButton: dijit.form.DropDownButton
		//		Undo button that should be enabled.
		// undoStack: Array of Object
		//		Undo stack that should be associated with the specified undo button.
		this._trace.entering(undoButton.id, "_enableUndoButton");
		var undoMenu = this._getUndoMenu(undoStack, undoButton.dropDown);
		
		this._widgetUtilities.enableButton(undoButton);
		this._trace.exiting(undoButton.id, "_enableUndoButton");
	},

	_getRoleTypes: function(/*function*/onComplete) {
		// Summary:
		//		Returns the role types.
		this._trace.entering("_getRoleTypes");
		var roleTypeStore = new com.ibm.security.modeling.notebook.RoleTypeStore({
			target: "/isrm/rest/types/role/", 
			idAttribute: "id",
			syncMode: false
        });
		var roleTypes = roleTypeStore.fetch({
			query: "",
			onComplete: onComplete,
			onError: dojo.hitch(this, this._getRoleTypesFailure)}).results;
		this._trace.exiting("_getRoleTypes");
	},

	_getRoleTypesFailure: function(error, request) {
		// Summary:
		//		Receives control when a failure occurs while retrieving the role types.
		//
		// error: Object
		//		Error that occurred.
		// request: Object
		//		Request that failed.
		this._trace.entering("_getRoleTypesFailure");
		this._errorUtilities.handleRESTError(error, this.messageArea);
		this._trace.exiting("_getRoleTypesFailure");
	},

	_getRoleTypeTranslatedName: function(roleTypeUntranslatedName) {
		// Summary:
		//		Returns the translated name for the role type having the specified name.
		//
		// roleTypeUntranslatedName: String
		//		Untranslated name of the role type for which the translated name
		//		should be returned.
		this._trace.entering(roleTypeUntranslatedName, "_getRoleTypeTranslatedName");
		var roleTypeTranslatedName = null;
		if ((roleTypeUntranslatedName != undefined) &&
			(roleTypeUntranslatedName != null)) {
			if ((roleTypeUntranslatedName.length > 0) &&
				(roleTypeUntranslatedName.charAt(0) == '"') &&
				(roleTypeUntranslatedName.charAt(roleTypeUntranslatedName.length - 1) == '"')) {
				roleTypeUntranslatedName = roleTypeUntranslatedName.substring(1, roleTypeUntranslatedName.length - 1);
			}
			if ((this._roleTypes != null) &&
				(this._roleTypes.length != undefined) &&
				(this._roleTypes.length > 0)) {
				for (var i = 0; i < this._roleTypes.length; i++) {
					var currentRoleType = this._roleTypes[i];
					if ((currentRoleType) &&
						(roleTypeUntranslatedName == currentRoleType.name)) {
						roleTypeTranslatedName = currentRoleType.translatedName;
						i = this._roleTypes.length;
					}
				}
			}
		}
		if ((roleTypeTranslatedName == undefined) ||
			(roleTypeTranslatedName == null)) {
			roleTypeTranslatedName = roleTypeUntranslatedName;
			if ((roleTypeTranslatedName == undefined) ||
				(roleTypeTranslatedName == null)) {
				roleTypeTranslatedName = "";
			}
		}
		roleTypeTranslatedName = "\"" + roleTypeTranslatedName + "\"";
		this._trace.exiting(roleTypeUntranslatedName, "_getRoleTypeTranslatedName");
		return roleTypeTranslatedName;
	},

	_getUndoItemText: function(undoItem) {
		// Summary:
		//		Returns the text for the specified undo item.
		//
		// undoItem: Object
		//		Undo item for which the text should be returned.
		this._trace.entering(undoItem, "_getUndoItemText");
		var undoItemText = undoItem.label.text;
		var key = undoItem.label.key;
		var roleTypeChangeKey = "PLUGIN_REST_UNDO_ROLE_TYPE_CHANGE";
		if (key == roleTypeChangeKey) {
			var untranslatedInserts = undoItem.label.inserts;
			var translatedInserts = new Array();
			translatedInserts[0] = this._getRoleTypeTranslatedName(untranslatedInserts[0]);
			translatedInserts[1] = this._getRoleTypeTranslatedName(untranslatedInserts[1]);
			var translatedText = this._pluginResources[roleTypeChangeKey];
			undoItemText = this._messageUtilities.formatMessage(translatedText, translatedInserts);
		}
		undoItemText = this._validator.escapeHTMLAndScriptTags(undoItemText);
		undoItemText = this._stringUtils.breakString(undoItemText);
		this._trace.exiting(undoItem, "_getUndoItemText");
		return undoItemText;
	},

	_getUndoMenu: function(undoStack, undoMenu) {
		// Summary:
		//		Returns a dijit.Menu representing the specified undo stack.
		//
		// undoStack: Array of Object
		//		Undo stack for which the menu should be returned.
		// undoMenu: dijit.Menu
		//		Reusable menu for the specified undo stack.
		this._trace.entering(undoStack, "_getUndoMenu");
		if (!undoMenu) {
			undoMenu = new dijit.Menu({
				style: "display: none;"
			});
		} else {
			dojo.forEach(undoMenu.getChildren(), function(child){ child.destroyRecursive(); });
		}
		if ((undoStack != undefined) &&
			(undoStack != null) &&
			(undoStack.length > 0)) {
			for (var i = 0; i < undoStack.length; i++) {
				var undoItem = undoStack[i];
				var undoMenuItem = this._getUndoMenuItem(undoItem);
				undoMenu.addChild(undoMenuItem);
			}
		}
		
		this._trace.exiting(undoStack, "_getUndoMenu");
		return undoMenu;
	},

	_getUndoMenuItem: function(undoItem) {
		// Summary:
		//		Returns a dijit.MenuItem representing the specified undo item.
		//
		// undoItem: Object
		//		Undo item for which the menu item should be returned.
		this._trace.entering(undoItem, "_getUndoMenuItem");
		var undoItemText = this._getUndoItemText(undoItem);
		var hitchedHandleUndo = dojo.hitch(this, this._handleUndo);
		var undoMenuItem = new dijit.MenuItem({
			label: undoItemText,
			onClick: function() { hitchedHandleUndo(undoItem); },
			style: "white-space: normal; word-wrap: break-word;"
		});
		dojo.addClass(undoMenuItem.iconNode.parentNode, "hidden");
		this._trace.exiting(undoItem, "_getUndoMenuItem");
		return undoMenuItem;
	},

	_handleUndo: function(undoItem) {
		// Summary:
		//		Receives control when the specified undo item is clicked.
		//
		// undoItem: Object
		//		Undo item that was clicked.
		this._trace.entering(undoItem, "_handleUndo");
		this._showProgressBar();
		this._restManager.undo(
				undoItem.id,
				dojo.hitch(this, this._undoSuccess),
				dojo.hitch(this, this._undoFailure));
		this._trace.exiting(undoItem, "_handleUndo");
	},

	_hideProgressBar: function() {
		// Summary:
		//		Hides the progress bar.
		this._trace.entering("_hideProgressBar");
		if (this._progressBar) {
			this._progressBar.hide();
			/* 
			 * Sometimes, hide doesn't seem to destroy the dialog. 
			 * Hence, destroying dialog explicitly. 
			 */
			this._progressBar.destroyProgressDialog(); 
		}
		this._trace.exiting("_hideProgressBar");
	},

	_refreshFailure: function(error, request) {
		// Summary:
		//		Receives control when a failure occurs while retrieving the undo stack.
		//
		// error: Object
		//		Error that occurred.
		// request: Object
		//		Request that failed.
		this._trace.entering("_refreshFailure");
		if (this._tableUndoButton != null) {
			this._updateUndoButton(this._tableUndoButton, null);
		}
		if (this._topologyUndoButton != null) {
			this._updateUndoButton(this._topologyUndoButton, null);
		}
		this._errorUtilities.handleRESTError(error, this.messageArea);
		this._trace.exiting("_refreshFailure");
	},

	_refreshSuccess: function(undoStack) {
		// Summary:
		//		Receives control when the undo stack is successfully retrieved.
		//
		// undoStack: Array of Object
		//		Undo stack that should be associated with the undo buttons.
		this._trace.entering(undoStack, "_refreshSuccess");
		if (this._tableUndoButton != null) {
			this._updateUndoButton(this._tableUndoButton, undoStack);
		}
		if (this._topologyUndoButton != null) {
			this._updateUndoButton(this._topologyUndoButton, undoStack);
		}
		this._trace.exiting(undoStack, "_refreshSuccess");
	},

	_setRestManager: function(modelId) {
		// Summary:
		//		Sets the REST manager.
		//
		// modelId: String
		//		Identifier of the model.
		this._trace.entering(modelId, "_setRestManager");
		if (this._modelId != modelId) {
			this._modelId = modelId;
			var undoUri = "/isrm/rest/models/" + this._modelId + "/undo/";
			this._restManager = new com.ibm.security.modeling.api.UndoRestManager({messageArea: this.messageArea, resourceUri: undoUri});
		}
		this._trace.exiting(modelId, "_setRestManager");
	},

	_showProgressBar: function() {
		// Summary:
		//		Creates (if not created earlier) progress bar and shows it.
		this._trace.entering("_showProgressBar");
		if (!this._progressBar) {
			this._progressBar = new com.ibm.security.modeling.widgets.ProgressBar({
				id: this.roleHierarchyPanel.id + "_undoProgressBar"
			});
		}
		this._progressBar.show(this._localResources.UNDOING);
		this._trace.exiting("_showProgressBar");
	},

	_undoFailure: function(error, request) {
		// Summary:
		//		Receives control when a failure occurs while undoing an item.
		//
		// error: Object
		//		Error that occurred.
		// request: Object
		//		Request that failed.
		this._trace.entering("_undoFailure");
		/*
		 * Just in case there was partial success, we need to refresh everything.
		 * If we displayed the error message now, it would be wiped out by the
		 * refresh.  Therefore, we use a connection to display the error message
		 * after the refresh is complete.
		 */
		if (this._roleHierarchyPanelConnection != null) {
			dojo.disconnect(this._roleHierarchyPanelConnection);
			this._roleHierarchyPanelConnection = null;
		}
		var hitchedDisplayUndoError = dojo.hitch(this, this._displayUndoError);
		this._roleHierarchyPanelConnection = dojo.connect(this.roleHierarchyPanel, "roleEditComplete", function() { hitchedDisplayUndoError(error); });
		this._undoSuccess();
		this._trace.exiting("_undoFailure");
	},

	_undoSuccess: function() {
		// Summary:
		//		Receives control when an item is successfully undone.
		this._trace.entering("_undoSuccess");
		this._hideProgressBar();
		dojo.publish("ROLE_EDIT_EVENT");
		this.refresh();
		this._trace.exiting("_undoSuccess");
	},

	_updateUndoButton: function(undoButton, undoStack) {
		// Summary:
		//		Updates the specified undo button with the specified undo stack.
		//
		// undoButton: dijit.form.DropDownButton
		//		Undo button that should be updated.
		// undoStack: Array of Object
		//		Undo stack that should be associated with the specified undo button.
		this._trace.entering(undoButton.id, "_updateUndoButton");
		if ((undoStack == undefined) ||
			(undoStack == null) ||
			(undoStack.length == 0)) {
			this._disableUndoButton(undoButton);
		} else {
			this._enableUndoButton(undoButton, undoStack);
		}
		this._trace.exiting(undoButton.id, "_updateUndoButton");
	}

});
