dojo.provide("com.ibm.security.modeling.importer.CreateSessionDialog");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Dialog");
dojo.require("dijit.Toolbar");
dojo.require("dijit.form.Button");
dojo.require("com.ibm.security.modeling.dijit.layout.BorderContainer");
dojo.require("dijit._Container");
dojo.require("ibm.tivoli.tip.dijit.TIPButton");
dojo.require("dojox.grid.enhanced._Plugin");
dojo.require("com.ibm.security.modeling.widgets.IsrmDialogBase");
dojo.require("com.ibm.security.modeling.tables.Table");
dojo.require("com.ibm.security.modeling.widgets.MultiMessageArea");
dojo.require("com.ibm.security.modeling.widgets.RequiredTextField");
dojo.require("com.ibm.security.modeling.widgets.MultiFileInput");
dojo.require("com.ibm.security.modeling.widgets.ProgressBar");
dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
dojo.require("com.ibm.security.modeling.util.Trace");

dojo.require("dijit._base.place");
dojo.require("dijit.form.RadioButton");
dojo.require("dojox.form.FileInput");


dojo.requireLocalization("com.ibm.security.modeling", "PluginMessages");
dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling.importer", "CreateSessionDialogText");

dojo.declare("com.ibm.security.modeling.importer.CreateSessionDialog", com.ibm.security.modeling.widgets.IsrmDialogBase, {
	/**
	 * Implements the Session creation ui dialog including the business logic for creating the session
	 * and queuing files for upload. 
	 * 
	 * This dialog is build for a show / hide pattern where the dialog can be reused on multiple 
	 * invocations.    
	 * 
	 */
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.importer.CreateSessionDialog"),
	// Provides utilities for handling errors.
	_errorUtil: null,
	templatePath : dojo.moduleUrl("com.ibm.security.modeling.importer", "templates/CreateSessionDialog.html"),
	image_path: dojo.moduleUrl("com.ibm.security.modeling.importer","../themes/images"),
	//store used to create sessions
	//passed in by parent panel
	sessionStore : null,
	// uploadTracker in memory store that queues and tracks pending 
	// uploads.  Used by this dialog to queue new uploads
	uploadTracker : null,
	widgetsInTemplate : true,
	parentWidget : null,
	// dialog used to show progress spinner passed in by parent panel
	progressBar : null,
	killOnHide:false,	
	//Acts as a flag indicating that an error
	//has occurred during session creation
	//using separate flag from error message
	//in case error occurs resolving the message
	sessionCreateFailed : false,
	//this will contain the error message
	//to be displayed in the message area
	//about the creation failure.  
	sessionCreationFailureMessage : null,

	constructor : function() {
	    this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
		this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.importer", "CreateSessionDialogText");
		this.helpFile="ui_session_new.html";
	},
	postCreate : function() {
		
		this.inherited(arguments);
		//this.setupMessageArea();
		//this.addHelpLink("ui_session_new.html");
	    this.sessionName._inputNode.style.width = "70%";
		//this.createSessionDialog.onCancel = dojo.hitch(this, this.onCancel);
		//this.createSessionDialog.onClose = dojo.hitch(this, this.onCancel);  
	    this.addCommonButton({
			onClick:dojo.hitch(this, this.onCancel),
			type:"CANCEL"
		});
	    this.okButton = this.addCommonButton({
			onClick:dojo.hitch(this, this.onOK),
			type:"OK"
		});
	    
	},
 postMixInProperties:function(){
		
		this.inherited(arguments);
		this.dialogTitle= this.localResources.PANEL_NAME;
		this.dialogInstruction=this.localResources.PANEL_INSTRUCTION;
		
	},
//	startup : function() {
//	    this.inherited(arguments);
//	    this.createSessionDialog.startup();    
//	},
	   
//    setupMessageArea : function() {
//			callBack = dojo.hitch(this, this.startResizeChain);
//			this.msgArea.resizeCallback = callBack;
//	},
//    startResizeChain : function() {
//		    this.container.resize();          
//	},

	 onOK : function() {
		 
		 this.msgArea.hideMessages();
		 
		 var successful = false;
			if (this.validate()) {			
				successful = this._createSessionAndQueueUpload();
				if (successful) {
					this.hide();
				}
				return successful;
			} else {
				//data was not valid
				return false;
			}
		},
	onCancel : function() {
		
		this.csMultiFileInput.deleteAndReplaceForm();
		this.hide();
	},
	validate : function() {
			/** validates that the session name is filled in and an acceptable length*/
			var valid = false;
			var errorMsg = "";
			var sessionName = this.sessionName.getValue().trim();
			if (sessionName != null) {
				if (sessionName.length == 0){
					errorMsg = this.pluginResources.PLUGIN_SESSION_NAME_NOT_FOUND;
					this.msgArea.showErrorMessage(errorMsg);
					this.sessionName.setFieldStatusToError();
				}			
			else if (sessionName.length > this.sessionName.constraints.maxlen){
				errorMsg = this.pluginResources.PLUGIN_SESSION_NAME_TOO_LONG;
				this.msgArea.showErrorMessage(errorMsg);
				this.sessionName.setFieldStatusToError();
			}
			else {
				valid = true;
			}
		
		  }	  
		  return valid;
	    },
//	show : function() {					
//			this.createSessionDialog.show();		
//		},
		
	hide : function() { 	
	    	//hide the dialog
	    	this.inherited(arguments);
	    	//this.createSessionDialog.hide();  
	    	//clear the values in the form so the 
	    	//next time the dialog is displayed it is clear
	    	//ie resetting for reuse of the dialog
	    	this.reset();
		},
	reset:function(){
			/**
			 * Summary:
			 * 		Resets form data to defaults
			 */
		this.sessionName.setValue("");
		//this.csMultiFileInput.reset();
		this.sessionTypeData.set("checked", true);
		this.msgArea.hideMessages();
		},
	_createSessionAndQueueUpload : function() {
			/**
			 * Summary:
			 * 		Orchestrates the creation of the session and optionally queuing of files for upload.  
			 *		
			 *		Process:
			 *			1) Display progress spinner
			 *			2) Synchronously create session
			 *			3) Queue files for upload
			 *			4) Close progress spinner
			 *	
			 * @returns boolean true if the process is successful false otherwise
			 * 
			 */
			//display progress spinner dialog
			this._displaySpinner();
			try {
			
				//create the session 
				var sessionObj = this._createSessionSynch();
				
				//queue uploads
				//enqueue the file upload
				this._enqueueUpload(sessionObj);
				return true;
			} catch (e) {
				this.msgArea.showErrorMessage(e);		
				//return false so the dialog stays open
				return false;
			}finally {
				this._closeSpinner();
			}	
	},
	
	_createSessionSynch : function() {
		/** 
		 * Summary:
		 * 	Synchronously creates a new session based on the data in the dialog
		 * 
		 * @pre Assumes the data in the form is valid
		 * 
		 * @throws String session creation failure error message
		 */
		
		//reset the flag indicating an error has occurred 
		this.sessionCreateFailed = false;
		this.sessionCreationFailureMessage = null;
		
		//Add the session to the sessionStore
		var newSessionData = this.sessionStore.newItem({sessionName: this.sessionName.getValue(), sessionType: (this.sessionTypeData.checked ? this.sessionTypeData.value : this.sessionTypeSchema.value)});
		
		//Synchronous save will ensure save method blocks until save is complete.  newSessionData referenced
		//above will also be updated with proper URIs  / default data retunred in the rest response
		//Note call back handler used to listen for errors,  revertOnError set so if this failes there 
		//is no need to refresh sessionStore it will automatically be upadted based on the failure.  
		var result = this.sessionStore.save({syncMode:true, revertOnError: true,onError : dojo.hitch(this, this._onSessionCreateError)});
					
		//an error occurred during the save
		//throw the error
		if (this.sessionCreationFailed) {
			throw this.sessionCreationFailureMessage;
		}
		
		//return the new session
		return newSessionData;
			
	},
	_enqueueUpload : function (session) {
		/**
		 * Summary:
		 * 	Adds files selected in the dialog to the pending upload queue
		 * 
		 * @param session the session object to upload the files to
		 * 		 * 
		 */
		
		//get the forms to upload
		var uploadData = this.csMultiFileInput.getFormsForUploadAndReplace();
		//add the files to the upload queue
		this.uploadTracker.startUpload(session.filesURI, uploadData, {sessionId : session.sessionId, sessionName : session.sessionName});

	},
	_displaySpinner : function() {
		/** Display the progress spinner */
		this.progressBar.show(this.localResources.CREATING_SESSION);	
	},
	_closeSpinner : function() {
		/** Hide the progress spinner */		
		this.progressBar.hide();
	},
	_onSessionCreateError : function(error) {
		/**
		 * Summary:
		 * 	Callback handler invoked on a session creation failure
		 * 
		 * This callback will set a member variable as a flag to 
		 * indicate that session create failed.  
		 */
		this.sessionCreateFailed = true;
		this.sessionCreationFailureMessage = this._errorUtil.getErrorMessageFromRestCallback(this.pluginResources,error);
		this._errorUtil.handleRESTError(error, this.parentWidget.msgArea);
	}
	
//	destroy : function() {
//		this.inherited(arguments);
//		this.createSessionDialog.destroyRecursive();	
//	},
//	kill : function() {
//		this.destroyRecursive();
//	}
	
}
	
	



);