/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.importer.ErrorDialog");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Dialog");
dojo.require("dijit.Toolbar");
dojo.require("dijit.form.Button");
dojo.require("com.ibm.security.modeling.dijit.layout.BorderContainer");
dojo.require("dijit._Container");
dojo.require("ibm.tivoli.tip.dijit.TIPButton");
dojo.require("com.ibm.security.modeling.widgets.ErrorsWarningsDialog");
dojo.require("com.ibm.security.modeling.tables.Table");
dojo.require("com.ibm.security.modeling.util._DialogUtilities");
dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling.importer", "ErrorText");
dojo.declare("com.ibm.security.modeling.importer.ErrorDialog", com.ibm.security.modeling.widgets.ErrorsWarningsDialog, {
				
	isSesssion:true,
	sessionItem:null,
	
	constructor : function() {
		
		this.globalResources = dojo.i18n.getLocalization(
				"com.ibm.security.modeling", "GlobalText");
		this.localResources = dojo.i18n.getLocalization(
				"com.ibm.security.modeling.importer", "ErrorText");
	
	},
	//method take care of , whether this is session error/warning or file error/warning
	postMixInProperties:function(){
	  	this.inherited(arguments);
	  	this.entityTypeName=this.localResources.SESSION;
      	this.entityTypeValue=this.sessionItem.sessionName  ; 
      	
      	this.storeUri = this.sessionItem.errorsURI;
      	this.tableArgs={
    			toolTipRequired : [ 'description' ]
    	};
      	
	    if(this.isSesssion){
	    	 this.dialogTitle=this.localResources.SESSION_ERROR_DIALOG_TITLE;
	    	 this.entityTypeNameLabelTextMarginClass = "sessionLabelTextMargin";
	    	 this.dialogInstruction=this.localResources.SESSION_ERROR_DIALOG_INSTRUCTION;
	    	 this.searchInstructionsText = this.localResources.SEARCH_SESSION;
	    	 this.entityTypeText=this.localResources.TYPE;
	    	 this.entityTypeTableText=this.localResources.ENTITY_SESSION;
	         this.entityValue= this.sessionType;
	         
	        this.errorTableStructure = [
								{field : 'fileName',name : this.localResources.HEADER_COLUMN_0,width : '20%'},
								{field : 'lineNumber',name : this.localResources.HEADER_COLUMN_1,width : '7%',isSearchable : false},
								{field : 'severity',name : this.localResources.HEADER_COLUMN_2,width : '10%',formatter : dojo.hitch(this,this.formatSeverity)},					
								{field : 'errorType',name : this.localResources.HEADER_COLUMN_3,width : '10%' ,formatter:dojo.hitch(this,this.getInternalizedLabels)},
								{field : 'description',name : this.localResources.HEADER_COLUMN_4,width : '16%',get:function(rowIndex,item){return item;},toolTipFormatter:dojo.hitch(this, this.formatMessage),formatter:dojo.hitch(this, function(item, rowIndex, cell){
									return com.ibm.security.modeling.tables.FormatterUtils.formatLongCellValue(this.formatMessage(item, rowIndex, cell));
								})},
								{field : 'entityType',name : this.localResources.HEADER_COLUMN_5,width : '17%',formatter:dojo.hitch(this,this.getInternalizedLabels) },
								{field : 'attrSource',name : this.localResources.HEADER_COLUMN_6,width : '20%',formatter:dojo.hitch(this,this.getInternalizedLabels)},
								
						];
			this.helpFile="ui_data_import_errors.html";
			//Add the Close and Edit button to the DOM node
		 }else{
		 	this.dialogTitle=this.localResources.FILE_ERROR_DIALOG_TITLE;
		 	this.entityTypeNameLabelTextMarginClass = "fileLabelTextMargin";
		 	this.entityTypeTextLabelMarginClass="fileTextLabelMarginClass";
		 	this.dialogInstruction=this.localResources.FILE_ERROR_DIALOG_INSTRUCTION;
	        this.entityTypeText=this.localResources.FILE;
	        this.entityTypeTableText=this.localResources.ENTITY_FILE;
	        this.entityValue= this.sessionItem.fileName;
	        this.searchInstructionsText = this.localResources.SEARCH_FILE;
	        
	        this.errorTableStructure = [
								
								{field : 'lineNumber',name : this.localResources.HEADER_COLUMN_1,width : '7%',isSearchable : false},
								{field : 'severity',name : this.localResources.HEADER_COLUMN_2,width : '7%',formatter : dojo.hitch(this,this.formatSeverity)},					
								{field : 'errorType',name : this.localResources.HEADER_COLUMN_3,width : '11%',formatter:dojo.hitch(this,this.getInternalizedLabels)},
								{field : 'description',name : this.localResources.HEADER_COLUMN_4,width : '25%',get:function(rowIndex,item){return item;},toolTipFormatter:dojo.hitch(this, this.formatMessage),formatter:dojo.hitch(this, function(item, rowIndex, cell){
									return com.ibm.security.modeling.tables.FormatterUtils.formatLongCellValue(this.formatMessage(item, rowIndex, cell));
								})},
								{field : 'entityType',name : this.localResources.HEADER_COLUMN_5,width : '25%',formatter:dojo.hitch(this,this.getInternalizedLabels) },
								{field : 'attrSource',name : this.localResources.HEADER_COLUMN_6,width : '25%',formatter:dojo.hitch(this,this.getInternalizedLabels)},
								
						];
			this.helpFile="ui_session_errors.html";
	    }
	   
	},
	
	postCreate : function() {

		this.inherited(arguments);
		if(this.isSession){
			//Place the Close and View Session button after the Close Button in the Dialog.
			
			this.closeButton = this.addCommonButton({
				onClick:dojo.hitch(this,this._onCloseAndView),
				label: this.localResources.CLOSE_VIEW,
				type:"OTHER"
			});
		}
		
		//If I remove the below line the table touches the bottom
		//TODO: remove this line and see if the issue can be addressed at the template level
		dojo.addClass(this.domNode, "contentArea");
	},
	
	_onCloseAndView:function(){
            	dojo.publish("OpenEdit",[this.sessionItem]);
            	this.onCancel();   
	}
});
