dojo.provide("com.ibm.security.modeling.importer.ImportPanel");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.security.modeling.layouts.TemplatedPanelWithLayout");
dojo.require("com.ibm.security.modeling.tables.Table");
dojo.require("com.ibm.security.modeling.dijit.layout.BorderContainer");
dojo.require("com.ibm.security.modeling.dijit.layout.ContentPane");
dojo.require("dojox.grid.enhanced.plugins.Pagination"); 
dojo.require("com.ibm.security.modeling.util.CommonUtilities");
dojo.require("com.ibm.security.modeling.widgets.MultiMessageArea");
dojo.require("com.ibm.security.modeling.tables.JsonRestStore");
dojo.require("com.ibm.security.modeling.widgets.UploadTracker");
dojo.require("com.ibm.security.modeling.importer.FilesDialog");
dojo.require("com.ibm.security.modeling.importer.CreateSessionDialog");
dojo.require("com.ibm.security.modeling.util.MessageUtilities");
dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
dojo.require("com.ibm.security.modeling.importer.UploadStatusDialog");
dojo.require("com.ibm.security.modeling.home.HomePageHelper");
dojo.require("com.ibm.security.modeling.util.NlsUtilities");
dojo.require("com.ibm.security.modeling.util.XhrUtilities");
dojo.require("com.ibm.security.modeling.breadcrumbs.BreadCrumManager");
dojo.require("com.ibm.security.modeling.importer.ImportStatusImages");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");

dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling.importer","ImportPanelText");
dojo.requireLocalization("com.ibm.security.modeling", "PluginMessages");
dojo.requireLocalization("com.ibm.security.modeling", "LogMessages");

dojo.declare("com.ibm.security.modeling.importer.ImportPanel",
				[ com.ibm.security.modeling.layouts.TemplatedPanelWithLayout ],{

	_messageUtil : new com.ibm.security.modeling.util.MessageUtilities(),
	widgetsInTemplate : true,
	contextRoot:null,
	homeUrl:null,
	uploadTracker: new com.ibm.security.modeling.widgets.UploadTracker(),
	templatePath : dojo.moduleUrl("com.ibm.security.modeling.importer","templates/ImportPanel.html"),		
	
		constructor : function(args) {	
						this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
	                    this._widgetUtilities = new com.ibm.security.modeling.util.WidgetUtilities();
	                    this.statusImages =  com.ibm.security.modeling.importer.ImportStatusImages;
						this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
						this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.importer","ImportPanelText");
						this.pluginResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "PluginMessages");
						this.logMessages = dojo.i18n.getLocalization("com.ibm.security.modeling", "LogMessages");
		},
		postCreate : function() {
			XhrUtilities.addSessionDetailInRequest();
			dojo.addClass(this.domNode, "contentArea");
			this.inherited(arguments);
			this._setUpStore();
			this._setUpTable();
			this.createFileUploadDialog();
			this._setUpToolbar();
			this._createBreadcrumbBar();
			this.msgArea.borderContainer = this.container;
			this.homeLink = ConsoleUtilities.placeLink(this.homeUrl);
			// connect to upload tracker events in order to display appropriate messages
			this.connect(this.uploadTracker, "onComplete",	"_displayUploadComplete");
			this.connect(this.uploadTracker, "onError",	"_displayUploadError");
			// this publish subscribe used to notify panel to open the edit panel
			// is used to incorporate switch button between error/edit
			this.subscribe("OpenEdit", "openEditSelected");
		},
		//method is called when OpenEdit event is published
		openEditSelected:function(item){
			//opens the edit panel for this item , timeout is given so that the panel which publishes 
			//the event hides itself first and then this dialog comes into picture . 
			setTimeout(dojo.hitch(this,this.onEditSelected,item),200);
		},
		destroy : function() {
						if(this.createSessionPanel) 	
							this.createSessionPanel.destroyRecursive();
            			this.inherited(arguments);			
		},
		startup : function() {    
			this.table.startup();    		
			this.inherited(arguments);
			this.connect(dojo.isIE ? this.domNode : dojo.global, 'onresize', function(){
				this.resize();
			});
			this.uploadStatusDialog.startup();
			this.uploadFilesDialog.startup();
        			
		},
		_setUpStore : function(){
		
                        this.sessionImportUri ='/isrm/rest/sessions';                    
						this.store = new com.ibm.security.modeling.tables.JsonRestStore( {
                        	localResources:this.localResources,
							target : this.sessionImportUri,
							idAttribute : "id"							
						});
                        var getValue=this.store.getValue;
                        //sorting is done on getValue , so if data is varying from store then sorted result get wrong .
						this.store.getValue = function(/*Object*/ item, /*String*/property, /*value?*/defaultValue){
						    var value = getValue(item,property,defaultValue);
						    if(property == 'sessionType'){
								if(value && value.toUpperCase() in this.localResources){
							        return this.localResources[value.toUpperCase()];
							    }
							}
						    else if (property == 'status'){
								 var statusTextKey = "importStatus_" + value;
								 var statusText = this.localResources.importStatus_DEFAULT;
								 if(this.localResources[statusTextKey]) {
									 statusText = this.localResources[statusTextKey];
								 } 
								 //trim is to prevent PII hit , a space is appended in few status . 
								 return statusText.trim();
							}
				            return value;
						};
		},
         addToStore : function(item){
		    
		                  this.store.newItem(item);
		             
		             },
		_setUpTable:function(){
		    
						this.importTableStructure = [
								{field : 'sessionName',name : this.localResources.HEADER_COLUMN_0,width : '14%',get:function(rowIndex,item){return item;},formatter : dojo.hitch(this,this.formatSession)},
								{field : 'sessionType',name : this.localResources.HEADER_COLUMN_1,width : '6%',isSearchable : false},
								{field : 'status',name : this.localResources.HEADER_COLUMN_2,width : '10%',isSearchable : false,get:function(rowIndex,item){return item;},formatter : dojo.hitch(this,this.formatStatus)},	
								{field : 'messageCount',name : this.localResources.HEADER_COLUMN_3,width : '5%',isSearchable : false,get:function(rowIndex,item){return item;},formatter : dojo.hitch(this,this.formatmsgCount)},
								{field : 'usersCount',name : this.localResources.HEADER_COLUMN_4,width : '5%',isSearchable : false},
								{field : 'roleCount',name : this.localResources.HEADER_COLUMN_5,width : '5%',isSearchable : false },
								{field : 'permissionCount',name : this.localResources.HEADER_COLUMN_6,width : '10%',isSearchable : false},
								{field : 'sodCount',name : this.localResources.HEADER_COLUMN_7,width : '11%',isSearchable : false},
								{field : 'userToPermissionCount',name : this.localResources.HEADER_COLUMN_8,width : '13%',isSearchable : false},
								{field : 'userToRoleCount',name : this.localResources.HEADER_COLUMN_9,width : '10%',isSearchable : false},
								{field : 'roleToPermissionCount',name : this.localResources.HEADER_COLUMN_10,width : '13%',isSearchable : false}
						];
						this.table = new com.ibm.security.modeling.tables.Table(
								{
									id : this.id + "_importTable",
									structure : this.importTableStructure,		
									dataStore:this.store,						
									isClientSide : true,
									isFilterRendered : true,
									isRefreshRendered : true,
									toolTipRequired:['sessionName',4],
									isMultiSelect : false,
									isSelectAllRendered:false,
									isDynamic:true,
									autoHeight:false,
									messageArea: this.msgArea,
									entityTypeText:this.localResources.ENTITY_TEXT,
									searchInstructions: this.localResources.SEARCH_INSTRUCTION
								}, dojo.byId(this.id+"_ImportTable"));
							
						dojo.addClass(this.table.domNode,"fullSizeTable");
						this.connect(this.table, "refresh",dojo.hitch(this,function(){
							this.validCommitButton.commit=false;
							
						}));
						this.connect(this.table, "_updateButtons", dojo.hitch(this,this._updateCommitButtons));
							
                                                    
		},
		
		_updateCommitButtons:function(){
			//returns the single seleted item  , else will throw exception
			var selectedSession;
			try{
				 selectedSession = this._getSelectedSession();
			}catch(e){
                 return ;
			}	
			if (selectedSession.status=='COMMITTED'||selectedSession.status=='COMMITTING'||selectedSession.status=='COMMIT_SESSION_ERROR') {
				this.table.toggleButtonActivation([this.validCommitButton.attachButton],true);
			}
	
		},
		formatSession:function(item,rowIndex,cell){
		    var anchor=null;
		    
		    if(item){
		    	var formattedSessionName = item.sessionName;
		    	          anchor=new com.ibm.security.modeling.util.Anchor({
								input:formattedSessionName,
								callback:dojo.hitch(this, this.onEditSelected,item)
							});											
		    }
			return anchor;
		   
		},
		onEditSelected:function(item){
			/**
			 * Summary:
			 * 		Launches the FilesDialog (session details)
			 *      for the selected session.  This method
			 *      is used both for the link anchor (which passed in the item)
			 *      and button handling table selection.  
			 *      
			 *      If the dialog has already been created then it will 
			 *      show the existing dialog.  Otherwise it will show 
			 *      the existing dialog passing in information about 
			 *      the current selection.
			 *      
			 *      !!!Note We tried initializing the dialog with a null
			 *      store and then always using the pass-in session on 
			 *      show version and a bug in enhnaced grid handling
			 *      of null store prevented this from working.  
			 *      
			 *      @param item SessionObject object representing the selected session
			 *      			or null if the table selection should be consulted.  
			 * 
			 */
			
			if(!item) {
				item = this._getSelectedSession();
			
			}
			//this variable store the active session, currenty used to control the upload messages .
			this.currentActiveSession = item.sessionName;
			var readOnly=(item.status=='COMMITTED'||item.status=='COMMITTING'||item.status=='COMMIT_SESSION_ERROR')?true:false;
			if (!this.filesDialog) {
				//if the dialog doesn't exist then create it
				this.filesDialog = new com.ibm.security.modeling.importer.FilesDialog({readOnly:readOnly,contextRoot:this.contextRoot,sessionName : item.sessionName, sessionType : this.store.getValue(item, "sessionType"), filesURI: item.filesURI, uploadTracker : this.uploadTracker, uploadFilesDialog : this.uploadFilesDialog, logMessages : this.logMessages}, this.filesDialogAnchor);
				
				this.filesDialog.startup();
				this.filesDialog.show();
			} else {		
				//show the existing dialog passing in new values for the session
				this.filesDialog.show({readOnly:readOnly,sessionName : item.sessionName, sessionType : this.store.getValue(item, "sessionType"), filesURI: item.filesURI});	
			}
			
					
		},
		_getSelectedSession : function() {
			/**
			 * Summary:
			 * 		Returns the current table selection
			 * 		Given that this is a single select table 
			 * 		we should not encounter situations where more
			 * 		than one row is selected.  Throwing error
			 * 		if this situation is encountered as it indicates
			 * 		a code bug.  
			 * 
			 */
			var items = this.table.getSelectedItems();
			//Fix for 41190 , items.length should always be 1 , as we are using radio button
			//But in creation scenarion , preserver is having two ids  one client side , and one server side . though the value will be same .
			if (items && items.length >= 1) {
				return items[0];	
			} else {
				//This should only occur if a coding error is made.  
				throw("Assertion failed single item not selected");
			}

			
		},
		_showSessionMessages : function(item){
			dojo.require("com.ibm.security.modeling.widgets.EntityStatusMsgDialog");
			this._messagesDialog = new com.ibm.security.modeling.widgets.EntityStatusMsgDialog({
				contextRoot:this.contextRoot,
				entityItem: item,
				helpFile:"ui_session_status.html",
				entityType: "session",
				//Below is hack to have the Dialog title for Session Staus messages being set to Status
				//Done to reduce PII impact. Refer defect 43786.
				dialogTitle: this.localResources.HEADER_COLUMN_2
			}, null);
			this._messagesDialog.show();
			this._messagesDialog.startup();
		},
		formatStatus:function(item,rowIndex,cell){
			
			 // Summary:
			 // 	 formats the status of the session
			 // 	 using a lookup pattern that keys into
			 // 	 the resource bundle for both text and image
			 //   this will prevent the need to update this 
			 //  code everytime there is a new status code
			 //   only the resource bundle needs to be updated
			 //
			
			//updating commiting flag , if any status is commiting
			if(!item) return null;
			var statusText = this.store.getValue(item,'status');
			 var statusImageKey = "importStatusImage_" + item.status;
			 var statusImage = this.statusImages.importStatusImage_DEFAULT;
			 if (this.statusImages[statusImageKey]) {
				 statusImage = this.statusImages[statusImageKey];
			 }
		 		
			var imgStatus = "<img class='imageHeight' src='"+statusImage+"'/>";
		 	//Create a hyperlink of the formatted text and return the same.
		 	var sessionItem = {
		 					"name": item.sessionName,
							"status": statusText,
							"messagesURI": item.messagesURI
		 				   };
			 				   
			 var anchor=new com.ibm.security.modeling.util.Anchor({
			 								add: imgStatus,
											input:statusText,
											callback:dojo.hitch(this, this._showSessionMessages,sessionItem)
								});		
			 	
			return anchor ;
			
			
		},
			
		formatmsgCount : function(item) {
						
						var temp_String;
						var anchor='';	
						if(item){
							//Level : 3 means error , 2 means warning , 1 means info
							var imageLevel = 1;
							var warningCount = this.store.getValue(item,"warningCount")||0;
							if(warningCount > 0) imageLevel = 2;
							var errorsCount = this.store.getValue(item,"errorCount")||0;
							if(errorsCount > 0) imageLevel = 3;
							
							
							 var totalCount = this.store.getValue(item,"messageCount");
							 var imagePath = "/securitymodeling/js/src/com/ibm/security/modeling/importer/themes/images/";
				
							
							if (item.messageCount != undefined && item.messageCount > 0) {
						 		 switch(imageLevel){
						 		 case 1: imagePath=imagePath+"info.png";
						 		        break;
						 		 case 2: imagePath=imagePath+"violation_warning.png";
						 		    	break;
						 		 case 3:imagePath=imagePath+"error.png";
						 		        break;
						 		 default:
						 			imagePath=imagePath+"info.png";
						 		         break;
						 		 }
						 		 
						 		temp_String= "<img class='imageHeight' src="+imagePath+" />";
								 anchor=new com.ibm.security.modeling.util.Anchor({
										add:temp_String,
										input:totalCount,
										callback:dojo.hitch(this,this.showAllErrors,item)
									});				
							} else {
								temp_String="<img class='imageHeight' src=/securitymodeling/js/src/com/ibm/security/modeling/importer/themes/images/no_violation.png />";
								anchor=temp_String+" "+"0";			
							}		
	         	}
						return anchor;				
		},			
		showAllErrors:function(item){
			dojo.require("com.ibm.security.modeling.importer.ErrorDialog");
			var errorPanel=new com.ibm.security.modeling.importer.ErrorDialog({
				contextRoot:this.contextRoot,
				sessionItem:item,
				
				sessionType:this.store.getValue(item,'sessionType'),
				isSession:true
			},null);
			
			errorPanel.show();
			errorPanel.startup();
		},
	
	_setUpToolbar:function(){				
			
			this.table.addToolbarButton(this.localResources.NEW, false,"newIcon", dojo.hitch(this, this.onCreateNew),this.id + "_new", true, true, true);
			this.table.addToolbarButton(this.localResources.EDIT, false,"editIcon", dojo.hitch(this,this.onEditSelected,null),this.id + "_edit", true, false, false);
			var deleteButton=this.table.addToolbarButton(this.localResources.DELETE, false,"deleteIcon", dojo.hitch(this,this.onDeleteSelected), this.id	+ "_delete", true, true, false);
			var attachButton = this.table.addToolbarButton(this.localResources.ADD_FILE, false, "attachIcon", dojo.hitch(this, this.onAddFilesSelected), this.id + "_upload", true, false, false);
			this.createUploadStatusDialog();
			this.table.addToolbarSeparator();	
			
			var validButton=this.table.addToolbarButton(this.localResources.VALIDATE, true, "validateIcon", dojo.hitch(this, this.onValidateSelected), this.id + "_validate", true, false, false);
			var commitButton=this.table.addToolbarButton(this.localResources.COMMIT, true, "commitIcon", dojo.hitch(this, this.onCommitSelected), this.id + "_commit", true, false, false);
			this.validCommitButton={
					validButon:validButton,
					commitButton:commitButton,
					attachButton:attachButton,
					commit:false
			};
	},
	
	createUploadStatusDialog:function() {
		/**
		 * Summary:
		 * 		Creates the upload status tooltip dialog and ties it to a new drop down button. 
		 * 		***Note we are setting visibility instead of display because we want the space 
		 *      on the toolbar to be allocated.  
		 * 
		 * 
		 */
		//This must refer to the common upload tracker
	    this.uploadStatusDialog = new com.ibm.security.modeling.importer.UploadStatusDialog({uploadTracker: this.uploadTracker});
	    var uploadStatusButton = this._widgetUtilities.createButton(
	    		this.localResources.UPLOAD,
				true,
				"loadingIcon",
				null,
				 this.id + "_uploadStatus",
				true,
				this.uploadStatusDialog,
				true);
	    dojo.place(uploadStatusButton.domNode ,this.uploadStatusDialogattach ,"first");
	    dojo.style(uploadStatusButton.domNode,"visibility","hidden");
		this.uploadStatusDialog.set("uploadStatusButton", uploadStatusButton);
	},
	createFileUploadDialog: function() {
		/**
		 * Summary:
		 * 		Creates a default file upload dialog.   Additional parameters need to be passed in when this dialog 
		 * 		is displayed in order to have the correct filesURI		 * 
		 */
		this.uploadFilesDialog = new com.ibm.security.modeling.importer.UploadFilesDialog({uploadTracker: this.uploadTracker, filesURI : "", sessionName :""},this.uploadFilesDialogAnchor);
	}, 
	
	_createBreadcrumbBar: function() {
		
			var manager = new com.ibm.security.modeling.breadcrumbs.BreadCrumManager({
				placeHolderId : this.id + "_BreadCrumbPlaceHolder"
			});
			dojo.publish("AddBreadCrum",[{
				context : this,
				hideFunction : "",
				label:this.globalResources.IMPORT
			}]);
	
	},
	onCreateNew:function(){
		/**
		 * Summary:  Launches create session dialog
		 * 		if the dialog has been previously created then it 
		 * 		launches the existing dialog, otherwise it creates
		 * 		a new one
		 */
			
			if(this.createSessionPanel){
				this.createSessionPanel.show();
			}else{
				this.createSessionPanel=new com.ibm.security.modeling.importer.CreateSessionDialog({
					contextRoot:this.contextRoot,
					parentWidget:this,
					sessionStore:this.store,
					uploadTracker : this.uploadTracker,
					progressBar : this.progressBar
				},this.createSessionPanelAnchor);
				this.createSessionPanel.startup();						
				this.createSessionPanel.show();
				
			}		
			
		},
		onAddFilesSelected:function(){
			/**
			 * Summary:  Launches the add files to session dialog
			 * 		note this code assumes there is a selection
			 */
			var selectedSession = this._getSelectedSession();
			this.uploadFilesDialog.show({sessionName : selectedSession.sessionName, filesURI : selectedSession.filesURI});
		},	
		onValidateSelected:function(){
			var selectedSession = this._getSelectedSession();
			var sessionName = com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(selectedSession.sessionName);
			this.validationConfirmationMsg = dojo.string.substitute(this.pluginResources.CONSOLE_IMPORT_SESSION_VALIDATION_CONFIRMATION, {"sessionName":sessionName});
	
			dojo.require("com.ibm.security.modeling.widgets.ConfirmMessageDialog");
			var validateConfirmDialog = dijit.byId(this.id+"_importSessionValidateDialog");
			if(!validateConfirmDialog){
				 validateConfirmDialog = new com.ibm.security.modeling.widgets.ConfirmMessageDialog({
							   id:this.id+"_importSessionValidateDialog",
							   submitButtonText:this.globalResources.OK,
							   cancelButtonText:this.globalResources.CANCEL,
							   completeMessage:this.validationConfirmationMsg,
							   title:this.localResources.VALIDATE_CONFIRM_DIALOG_TITLE,
							   onSubmit: dojo.hitch(this,this.doValidation)
							});
				validateConfirmDialog.show();
				validateConfirmDialog.startup();										
			}else{
				
				//Replacing the innerHTML text of the contentPane to display the updated session name.
				
				validateConfirmDialog.setMessage(this.validationConfirmationMsg);
				validateConfirmDialog.show();
			}
		},
		//setting status to "VALIDATING" indicate server , this post request is to trigger validation on this session .
		doValidation : function(){
			var selectedSession = this._getSelectedSession();
			this.store.changing(selectedSession);
			selectedSession.status='VALIDATING';
			this.store.save({
			onComplete:dojo.hitch(this, function(){
				 //this.msgArea.showInfoMessage(dojo.string.substitute(this.pluginResources.CONSOLE_IMPORT_SESSION_VALIDATION_REQUEST_SUBMITTED, {"sessionName":selectedSession.sessionName}));
				 var sessionName = com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(selectedSession.sessionName);
				 this.msgArea.showInfoMessage(dojo.string.substitute(this.pluginResources.CONSOLE_IMPORT_SESSION_VALIDATION_REQUEST_SUBMITTED, {"sessionName":sessionName}));
			}),
			onError:dojo.hitch(this, this._validationError)});
		},
		_validationError: function(error){
			
            this._errorUtil.handleRESTError(error,this.msgArea);
                     
		},
		onCommitSelected:function(){
		
			
			var selectedSession = this._getSelectedSession();
			var errorsCount = this.store.getValue(selectedSession,"errorCount")||0;
			var warningCount = this.store.getValue(selectedSession,"warningCount")||0;
			var totalCount = Number(errorsCount) + Number(warningCount);
			var sessionName = com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(selectedSession.sessionName);
			if(totalCount > 0 ){
					
				this.commitConfirmMessage = dojo.string.substitute(this.pluginResources.CONSOLE_IMPORT_SESSION_WITH_ERROR_COMMIT_CONFIRMATION, {"sessionName":sessionName,"htmldata":totalCount});
			}
			else 
			 this.commitConfirmMessage = dojo.string.substitute(this.pluginResources.CONSOLE_IMPORT_SESSION_COMMIT_CONFIRMATION, {"sessionName":sessionName});
			
			dojo.require("com.ibm.security.modeling.widgets.ConfirmMessageDialog");
			var commitDialog = dijit.byId(this.id+"_importSessionCommitDialog");
			if(!commitDialog){
				var commitDialog = new com.ibm.security.modeling.widgets.ConfirmMessageDialog({
							   id:this.id+"_importSessionCommitDialog",
							   submitButtonText:this.globalResources.OK,
							   cancelButtonText:this.globalResources.CANCEL,
							   completeMessage:this.commitConfirmMessage,
							   title:this.localResources.COMMIT_TITLE,
							   onSubmit: dojo.hitch(this,this.doCommit)
							});
				commitDialog.show();
				commitDialog.startup();										
			}else{
				
				//Replacing the innerHTML text of the contentPane to display the updated session name.
				
				commitDialog.setMessage(this.commitConfirmMessage);
				commitDialog.show();
			}
			
		},	
	//	setting status to "COMMITTING" indicate server , this post request is to trigger commit operation on this session .
		doCommit:function(){
			var selectedSession = this._getSelectedSession();
			this.store.changing(selectedSession);
			selectedSession.status='COMMITTING';  
			this.validCommitButton.commit=true;
			this.save({
				onComplete:dojo.hitch(this,function(){
					//this.msgArea.showInfoMessage(dojo.string.substitute(this.pluginResources.CONSOLE_IMPORT_SESSION_COMMIT_REQUEST_SUBMITTED, {"sessionName":selectedSession.sessionName}));
					var sessionName = com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(selectedSession.sessionName);
					this.msgArea.showInfoMessage(dojo.string.substitute(this.pluginResources.CONSOLE_IMPORT_SESSION_COMMIT_REQUEST_SUBMITTED, {"sessionName":sessionName}));
				}),
				onError:dojo.hitch(this,function(error){
					this._errorUtil.handleRESTError(error,this.msgArea);
				})
					
			});
			//updating the validate/commit button after a commit has been posted
			this.validCommitButton.commit = false;
			
			
		},
	
		onDeleteSelected:function(){
			var items = this.table.grid.selection.getSelected();
		    this.selectedSession = items[0];
			//Since we will be allowing the user to work with only one session at a time we have hardcoded the index of the items array
		    var sessionName = com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(items[0].sessionName);
		    this.deleteConfirmMessage = dojo.string.substitute(this.logMessages.CONSOLE_IMPORT_SESSION_DELETE_CONFIRMATION, {"sessionName":sessionName});
		    
			//Changes made to popup the common ConfirmMessageDialog widget for confirming if a user wants to delete a session.
			dojo.require("com.ibm.security.modeling.widgets.ConfirmMessageDialog");
			var deleteDialog = dijit.byId("importSessionDeleteDialog");
			if(!deleteDialog){
				var deleteDialog = new com.ibm.security.modeling.widgets.ConfirmMessageDialog({
							   id:"importSessionDeleteDialog",
							   submitButtonText:this.globalResources.OK,
							   cancelButtonText:this.globalResources.CANCEL,
							   completeMessage:this.deleteConfirmMessage,
							   title:this.localResources.DELETE_TITLE,
							   onSubmit: dojo.hitch(this,this.doDelete)
							});
				deleteDialog.show();
				deleteDialog.startup();										
			}else{
				deleteDialog.setMessage(this.deleteConfirmMessage);
				deleteDialog.show();
			}
			
		 },
		   doDelete:function(items){
				if(this.selectedSession !== null){
					//Changed the store.deleteItem call to PUT overwriting for resolving defect 42641
					this.store.changing(this.selectedSession);
					var putMethod = this.store.service["put"];
					this.store.service["put"]=dojo.hitch(this,function(id,content){
						return this.store.service["delete"](id,content);
					});	
					
					this.save({
						revertOnError: true
					});
					//Restoring the put method back on the store.
					this.store.service["put"] = putMethod;
				}
		  },
		  //Common save call : can be used to show or hide indicator if needed in future
		  save:function(args){
		      if(!args) args={};
		      if(!("onError" in args)){ 
		    	  args.onError=dojo.hitch(this,function(error){
		    	                     this._errorUtil.handleRESTError(error,this.msgArea);
		                             });
		      }
		      if(!("onComplete" in args))
		      { 
		          args.onComplete=dojo.hitch(this,function(){
		              });
		       }
		      this.store.save(args);
		  }, 
		  _displayUploadComplete:function(item) {
				/**
				 * Displays upload completion message
				 */
			 if(!(this.currentActiveSession && item.sessionName==this.currentActiveSession && this.filesDialog._IsrmDialog.open))
				 
				  this.msgArea.showInfoMessage(dojo.string.substitute(this.pluginResources.CONSOLE_IMPORT_SESSION_FILEUPLOAD_REQUEST_COMPLETED, {"fileName":item.fileName}));		
		},
		_displayUploadError:function(item, error) {
			/**
			 * Displays upload error message
			 */
		
			 var errorText= this._messageUtil.formatMessage(this.pluginResources[error.key],error.inserts,null,null); 	
      	
		    if(!(this.currentActiveSession && item.sessionName==this.currentActiveSession && this.filesDialog._IsrmDialog.open))
		    	this.msgArea.showErrorMessage(errorText);				
	}
});