/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

ï¿½ Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.lifecycle.LifecycleDialog");

dojo.require("ibm.tivoli.tip.dijit.TIPMessageDialog");
dojo.require("com.ibm.security.modeling.dijit.layout.BorderContainer");
dojo.require("com.ibm.security.modeling.dijit.layout.ContentPane");
dojo.require("dijit.Dialog");
dojo.require("com.ibm.security.modeling.widgets.IsrmDialogBase");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("dijit.form.SimpleTextarea");
dojo.require("com.ibm.security.modeling.lifecycle.LifecycleRequestTable");
dojo.require("com.ibm.security.modeling.api.LifecycleRestManager");
dojo.require("com.ibm.security.modeling.widgets.ProgressBar");
dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
dojo.require("dojo.data.ItemFileReadStore");

dojo["requireLocalization"]("customizations", "LifecycleText");

dojo.declare("com.ibm.security.modeling.lifecycle.LifecycleDialog",com.ibm.security.modeling.widgets.IsrmDialogBase,{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.lifecycle.LifecycleDialog"),
	templatePath : dojo.moduleUrl("com.ibm.security.modeling.lifecycle", "templates/LifecycleDialog.html"),
	autofocus:"false",
	heightFactor:1.7,
	minHeight:375,
    minWidth:320,
	widthFactor:1.5, 
	widgetsInTemplate : true,
	killOnHide : false,
	
	logMessages : null,
	
	//entity: String 
	//		Name of the Entity type which is to be used to load request types.
	entity:null,
	
	//submitRequestUri: String 
	//		URI for submit lifecycle request.
	submitRequestURI : null,
	
	// modelId: String
	//		Reference to current model Id
	modelId : null,
	
	// messageArea: ibm.tivoli.tip.dijit.TIPMessageArea
	//		Message area for error display
	parentMessageArea: null,
	
	lifecycleRoleStausKeyPrefix : "LIFECYCLE_ROLE_STATUS_",
	
	attributeMap : {
		lifecycleInstruction : {node : "lifecycleInstructionNode", type : "innerHTML"}
	},
	
	/**
	 * Overriden method to set property.
	 * @return
	 */
	postMixInProperties:function() {
		this._trace.entering(this.id, "postMixInProperties");
		try {			
			this.inherited(arguments);
			this.helpFile = "ui_role_submit_request.html";
			this.commentsLabel = this.globalResources.COMMENT;
			this.selectedRolesLabel = this.globalResources.SELECTED_ROLES;
			this.progressBar = new com.ibm.security.modeling.widgets.ProgressBar();
			this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
			if (!this.customText) {
				this.customText = dojo.i18n.getLocalization("customizations", "LifecycleText");
			}
		} catch (e) {
			this._trace.exception(e, "postMixInProperties");
			throw e;
		} finally {
			this._trace.exiting(this.id, "postMixInProperties");
		}
	},
	
	showWorkingDialog:function(msg){
		this.progressBar.show(msg);
	},
	
	hideWorkingDialog:function(){
		this.progressBar.hide();
	},
	
	/**
	 * Overriden method.
	 * 
	 * @return
	 */
	postCreate: function() {
		this._trace.entering(this.id, "postCreate");
		
		this.inherited(arguments);
		this._hookUpButtons();
		
		this._trace.exiting(this.id, "postCreate");		
	},
	
	/**
	 * To connect button with events
	 * 
	 * @return
	 */
	_hookUpButtons:function() {
		this._trace.entering(this.id, "_hookUpButtons");
	
		this.addCommonButton({
			onClick:dojo.hitch(this, this.closeLifecycleRequestDialog),
			type:"CLOSE"
		});
		
		this.addCommonButton({
			onClick:dojo.hitch(this, this.submitLifecycleRequest),
			type:"SUBMIT"
		});
		
		this._trace.exiting(this.id, "_hookUpButtons");		
	},
	/**
	 * To hide the submit lifecycle request dialog box.
	 * 
	 * @return
	 */
	closeLifecycleRequestDialog : function (){
		this._trace.entering(this.id, "closeLifecycleRequestDialog");
	
		try {
			this.hide();			
		} catch(e) {
			this._trace.exception(e, "closeLifecycleRequestDialog");
			throw e;
		} finally {
			this._trace.exiting(this.id, "closeLifecycleRequestDialog");			
		}		
	},
	
	/**
	 * To submit the lifecycle request.
	 * 
	 * @return
	 */
	submitLifecycleRequest : function (){
		this._trace.entering(this.id, "submitLifecycleRequestBtnPressed");
	
		try {
			this.showWorkingDialog(this.customText.LIFECYCLE_ROLE_REQUEST_STATUS_SUBMITTING);
			if(! this.lifecycleRestManager){
				this.lifecycleRestManager = this._createLifecycleRestManager();
			}
			
			this.lifecycleRestManager.submitRequestResource(
					this.modelId,
					this.entity,
					this.lifecycleArgument.requestType,
					this.lifecycleArgument.items,
					this.lifecycleSubmitDescription.get("value"),
					dojo.hitch(this,this.submitLifecycleSuccess),
		            dojo.hitch(this,this.submitLifecycleError)
		        );
			
						
		} catch(e) {
			this._trace.exception(e, "submitLifecycleRequestBtnPressed");
			throw e;
		} finally {
			this._trace.exiting(this.id, "submitLifecycleRequestBtnPressed");			
		}		
	},
	
	submitLifecycleSuccess : function(item, request){
		this._trace.entering(this.id, "submitLifecycleSuccess");
		this.hideWorkingDialog();
		this.hide();
		if(this.parentMessageArea){
			this.parentMessageArea.showInfoMessage(this.logMessages.LIFECYCLE_REQUEST_SUBMITTED_SUCCESSFULLY);	
		}
		this._viewUpdater();
		this._trace.exiting(this.id, "submitLifecycleSuccess");
	},
	
	submitLifecycleError : function(error){
		this._trace.entering(this.id, "submitLifecycleError");
		this.hideWorkingDialog();
		this.hide();
		if(this.parentMessageArea){
			this._errorUtil.handleRESTError(error, this.parentMessageArea,false,this.logMessages);
		}
		if(! (error && error.responseText && dojo.fromJson(error.responseText).key == "ROLE_REQUEST_SUBMIT_FAILED")){
			this._viewUpdater();
		}
		
		this._trace.exiting(this.id, "submitLifecycleError");
	},
	_viewUpdater : function(){
		dojo.publish("UPDATE_VIEW_FOR_SUBMIT_REQUEST","");
	},
	_createLifecycleRestManager : function(){
		this._trace.entering(this.id, "createLifecycleRestManager");
		
		var lifecycleRestManager = new com.ibm.security.modeling.api.LifecycleRestManager({
			resourceUri: this.submitRequestURI,
			syncMode: false,
			modelId : this.modelId
		});
		
		this._trace.exiting(this.id, "createLifecycleRestManager");

		return lifecycleRestManager;
		
	},
	
	/**
	 * To populate value in the submit lifecycle request dialog dynamically. 
	 * @param 	argument.requestType
	 * 						Lifecycle submit request type 
	 * 			argument.items
	 * 						Items to be submitted. 
	 * @return
	 */
	populate: function(lifecycleArgument,_modelId){
		this._trace.entering(this.id, "populate");
		
		this.modelId = _modelId;
		try {
				this.lifecycleArgument = lifecycleArgument;
				
				if(!this.tableWidget){
					this.tableWidget = this.createTable(this.createDataStore(this.lifecycleArgument.items));
					this.tableWidget.startup();
				}else{
					this.tableWidget.setStore(this.createDataStore(this.lifecycleArgument.items));	
				}				
				
				this.set("lifecycleInstruction",
						dojo.string.substitute(this.globalResources.LIFECYCLE_INSTRUCTION, 
						{ 
							"request_type":this.lifecycleArgument.requestLabel
						}
					));
				
				this._IsrmDialog.titleNode.innerHTML = dojo.string.substitute(
						this.globalResources.LIFECYCLE_DIALOG_TITLE, 
						{
							"request_type":this.lifecycleArgument.requestLabel
						}
					); 
				
				this.lifecycleSubmitDescription.set("value", "");
		} catch (e) {
			this._trace.exception(e, "populate");
		} 
		this.show();
	
		this._trace.exiting(this.id, "populate");			
	},
	createDataStore: function(dataItems) {
		// summary:
		//		Creates a ItemFileReadStore for the table from the given data.
		// dataItems: Object
		//		Server data for store in json format.
		var tableItems = [];
		
		for(var i=0;i<dataItems.length;i++){
			tableItems.push({
				id: dataItems[i].id,
				name: dataItems[i].name,
				description: dataItems[i].description,
				status: this._getLifecycleStatusPerLocale(this.lifecycleRoleStausKeyPrefix, dataItems[i].status)
			});
		}
		
		var dataStore = new dojo.data.ItemFileReadStore({
			data: {
				label: "name",
				id: "id",
				items: tableItems					
			}
		});
		
		return dataStore;
	},
	
	_getLifecycleStatusPerLocale: function(/*prefix key*/ keyPrefix,/*status enum key*/ key) {
		var text = null;
		if (keyPrefix && key) {
			text = this.customText[keyPrefix+key.toUpperCase()];
		}
		return text ? text : key;
	},
	createTable :function(dataStore){
		// summary:
		//		Creates a table object for the table from the given data store.
		// dataStore: Object
		//		dataStore for store in json format.
		
		var tableWidget = new com.ibm.security.modeling.lifecycle.LifecycleRequestTable(
				{
					id:this.id+"lifecycleRequestTable",
					globalResources:this.globalResources,
					dataStore:dataStore
				},this.id+"lifecycleRequestTable");									
		dojo.addClass(tableWidget.domNode, "fullSize");
		
		return tableWidget;
	}
		
});
