dojo.provide("com.ibm.security.modeling.lifecycle.LifecycleMenuHelper");

dojo.require("dijit._Widget");
dojo.require("dijit.MenuItem");
dojo.require("dijit.PopupMenuItem");
dojo.require("dijit.Menu");

dojo.declare("com.ibm.security.modeling.lifecycle.LifecycleMenuHelper", [dijit._Widget],{
	
	//requestTypes: RequestType 
	//			Reference to the array of request types
	requestTypes : null,
	
	// globalResources: Object
	//			Reference to the global shared resource bundle
	globalResources : null,
	
	constructor: function (args) {
		// summary:
		//     A reusable menu helper widget
	},
	
	getMenuItem : function(launcher) {
		// summary:
		//     To provide menu item based on the request type
		// launcher: Object
		//     Object contains reference to the launcher function, 
		// which is to be called on onClick event of menu.
		
		var submitRequestMenu = null;
				
		if(this.requestTypes.length == 1){
			// If there is only one request type is available.
			var rqstType = this.requestTypes[0];
			
				submitRequestMenu = new dijit.MenuItem({
	                label: dojo.string.substitute(this.globalResources.SUBMIT_REQUEST, {"request_type":rqstType.label}),
	                tooltip : rqstType.description,
	                onClick: dojo.hitch(this,launcher,rqstType),
	                iconClass: "tipMenuIcon submitIcon"
	            });
			
		}else if (this.requestTypes.length > 1) {
			// If there are more than one request type are available.
			
			var subMenu = new dijit.Menu();
			
			// To create main menu
			submitRequestMenu = new dijit.PopupMenuItem({
				label: this.globalResources.SUBMIT_REQUESTS,
				iconClass: "tipMenuIcon submitIcon",
				popup: subMenu
			});
			
			//To create sub menu items
			for (var i = 0 ; i < this.requestTypes.length; i++) {
					
				var rqstType = this.requestTypes[i];
				this._addMenuItem(subMenu,new dijit.MenuItem({
		                label: rqstType.label,
		                tooltip : rqstType.description,
		                onClick: dojo.hitch(this,launcher,rqstType)
		            })
				);
			}
		}
		return submitRequestMenu;
	},
	_addMenuItem : function(menu,menuItem) {
		// summary:
		//     To hide icon class place holder from the submenu items.
		// menu: Object
		//     Object contains reference of the menu object
		// menuItem: Object
		//     Object contains reference of the menu item object
		
		//Add hidden class to remove icon place holder from menu.
		dojo.addClass(menuItem.iconNode.parentNode,"dijitHidden");	
		
		menu.addChild(menuItem);
	}
		
});