dojo.provide("com.ibm.security.modeling.lifecycle.LifecycleProvider");

dojo.require("dijit._Widget");
dojo.require("com.ibm.security.modeling.api.RestManager");
dojo.require("com.ibm.security.modeling.lifecycle.LifecycleDialog");
dojo.requireLocalization("com.ibm.security.modeling", "LogMessages");

dojo["requireLocalization"]("customizations", "LifecycleText");

dojo.declare("com.ibm.security.modeling.lifecycle.LifecycleProvider", [dijit._Widget],{
	
	//_requestTypesURI: String  
	//			String representation of URL, which is to be used to load request types.
	_requestTypesURI : "/isrm/rest/lifecycle/request/types",
	
	//_requestTypesURI: String  
	//			String representation of URL, which is to be used to load request types.
	_submitRequestURI : "/isrm/rest/lifecycle/${entity}/requests",
	
	//_requestTypes: Object 
	//			Reference to the array of request types
	_requestTypes : null, 
	
	// messageArea: ibm.tivoli.tip.dijit.TIPMessageArea
	//		Message area for error display
	messageArea: null,
	
	//entity: String 
	//		Name of the Entity type which is to be used to load request types.
	entity:null,
	
	//localResources: Object 
	//		Reference to the local shared resource bundle
	localResources : null,
	
	// logMessages: Object
	//			Reference to the TMS Log messages resource bundle
	logMessages: null,
	
	constructor: function (args) {
		// summary:
		//     A reusable lifecycle provider widget
		
		if(! this.logMessages){
			this.logMessages = dojo.i18n.getLocalization("com.ibm.security.modeling", "LogMessages");
		}
		this.localResources = dojo.i18n.getLocalization("customizations", "LifecycleText");
	},
	
	getRequestTypes : function() {
		// summary:
		//     To provide available request types

		//To set request types only once 
		if(! this._requestTypes) {
			this._buildRequestTypes(this._loadRequestTypes());
		}
		
		return this._requestTypes;
	},
	isBPMEnabled: function() {
		var types = this.getRequestTypes();
		return types && types.length > 0;
	},
	_loadRequestTypes : function() {
		// summary:
		//     To load actual request types from REST API
		
		// Query string to be used for entity type to load request types
		var qyeryString = '?entity='+this.entity;
		
		// Create an object of REST manager with resource URI
		var restManager = new com.ibm.security.modeling.api.RestManager({
				resourceUri:this._requestTypesURI,
				syncMode:true});
		
		// Load request types with query parameter and return request types. 
		return restManager.loadResource(qyeryString);
	},
	
	_buildRequestTypes : function(item) {
		// summary:
		//     To build the array of request types from item
		// item: Object
		//     Object contains reference of request type item
		
		if (item !== null) {
			this._requestTypes = [];
			
			//To create array of actual request types
			for (var i = 0; i < item.types.length; i++) {
				var rqstType = {
						type: item.types[i],
						label: this._getLabelForType(item.types[i]),
						description: this._getDescriptionForType(item.types[i]),
						requestHandler: dojo.hitch(this,this._requestHandler)
				};
				
				this._requestTypes.push(rqstType);
			}
		}
		
	},
	
	_getLabelForType : function(key) {
		// summary:
		//     To translate labels in local language.
		// key: String
		//     String request type
		var text = null;
		if (key) {
			text = this.localResources["LIFECYCLE_ROLE_REQUEST_TYPE_"+key.toUpperCase()];
		}
		return text ? text : key;
	},
	
	_requestHandler : function (items,requestType,modelId) {
		// summary:
		//     To create request handler for request type
		
			
			if(! this._dialog){
				this._dialog = new com.ibm.security.modeling.lifecycle.LifecycleDialog({
					entity : this.entity,
					submitRequestURI : dojo.string.substitute(this._submitRequestURI, {"entity":this.entity}),
					parentMessageArea : this.messageArea,
					logMessages : this.logMessages
				});	
				this._dialog.startup();
			}
			
			var lifecycleArgument = {};
			
			lifecycleArgument.requestLabel = requestType.label;
			lifecycleArgument.requestType = requestType.type;
			lifecycleArgument.items = items;	
			
			this._dialog.populate(lifecycleArgument,modelId);
	},
	
	_getDescriptionForType : function(type) {
		//TODO this needs to be implemented
		//This method should look up the type description from a custom resource bundle with the following format:
		// lifecycle.role.request.type.description.<type>  if it doesn't exist in the resource bundle then return the type string itself
		return type;		
	}
});