dojo.provide("com.ibm.security.modeling.lifecycle.LifecycleRequestTable");


dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.tables.Table");

dojo["requireLocalization"]("customizations", "LifecycleText");

dojo.declare("com.ibm.security.modeling.lifecycle.LifecycleRequestTable", com.ibm.security.modeling.tables.Table, {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.lifecycle.LifecycleRequestTable"),
	
	constructor:function(args) {
		// summary:
		//     A reusable lifecycle request table widget
	},
	postMixInProperties:function() {
		this.isClientSide = true;
		this.autoHeight = false;  
		this.isMultiSelect = false;
		this.toolTipRequired = [1,2,3];
		this.isDynamic = true;
		this.needSelection = false,
		this.structure = this._getStructure();
		this.dataStore.comparatorMap = this._sortMapper();
		this.inherited(arguments);
		this.defaultSortIndex = 0;
		this.isDefaultSortAscending = false;
		this.customText = dojo.i18n.getLocalization("customizations", "LifecycleText");
		
	},
	
	startup:function() {
		this._trace.entering("", "startup");
		this.inherited(arguments);
		this._trace.exiting("", "startup");
	},
	_sortMapper : function(){
		var comparatorMap = {};
		var columns = this._getStructure();
		for(var i=0;i<columns.length;i++){
			comparatorMap[columns[i].field]= this._comparator;
		}
		return comparatorMap;
	},
	_comparator : function(a, b){
		var av = "";
		if(a){
			av = a.toLowerCase();	
		}
		var bv = "";
		if(b){
			bv = b.toLowerCase();	
		}
		
		if(av != bv){
			return av < bv ? 1 : -1;
		}
		return 0;
	},
	_getStructure: function() {
		var baseStructure = [
		        {
		        	field: 'name',
		        	name: this.globalResources.ROLE,
		        	width: '25%'
		        },
		        {
		        	field: 'description',
		        	name: this.globalResources.DESCRIPTION,
		        	width: '50%',
		        	toolTipFormatter:function(value,rowIndex,cell){
			        	return com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(value );
		    		},
		    		formatter:function(value,rowIndex,cell){
		    			return com.ibm.security.modeling.tables.FormatterUtils.formatLongCellValue(value);
		    		}
		        },
		        {  
		        	field: 'status',
					name: this.globalResources.LIFECYCLE_STATUS,
					width: '25%', 
					toolTipFormatter: dojo.hitch(this,this._formatLifecycleStatusTooltip),
					formatter: dojo.hitch(this,this._formatLifecycleStatus)
				}];
		
		return baseStructure;
	},
	_formatLifecycleStatusTooltip: function(value,rowIndex,cell){
		var statusText = "";
		if(value){
			var statusText = this._getValuePerLocale(value,"LIFECYCLE_ROLE_STATUS_");
			statusText = com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(statusText);
			
		}	
		return statusText;
	},
	_formatLifecycleStatus: function(value,rowIndex,cell){
		var statusText = "";
		if(value){
			var statusText = this._getValuePerLocale(value,"LIFECYCLE_ROLE_STATUS_");
			statusText = com.ibm.security.modeling.tables.FormatterUtils.formatLongCellValue(statusText);
			statusText = com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(statusText);
			
		}	
		
		return statusText;
	},
	_getValuePerLocale: function(/*string key*/ key,
			/*string prefix*/ prefix) {
		prefix = prefix ? prefix : "";
		
		var text = null;
		if (key) {
		var mappedKey = prefix+(key.toUpperCase());
		text = this.customText[mappedKey]; 
		}
		return text ? text : key;
	},
	postCreate:function() {
		this.inherited(arguments);
		
		// To render table 
		dojo.addClass(this.domNode,"fullSize");

		// To hide the toolbar.
		dojo.addClass(this.header.domNode, "hide");
	}
});