/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

(c) Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.membership.EditableElement");

dojo.require("dijit._Templated");

dojo.require("dojo.fx");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.util.WidgetUtilities");

// define the widget class
dojo.declare (
	"com.ibm.security.modeling.membership.EditableElement", 
     [ dijit._Widget ], {
	
	
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.membership.EditableElement"),
	
	// _focusNode: a DOM node or a widget with focus() method.
	
	// _isFocused: boolean
	//		Indicates if widget is on focus
	_isFocused: false,
	
	_isFocusAdjusted: false,
	_tabIndex: 0,
	
	// _isFocused: boolean
	//		Indicates if widget is in edit view.
	//		default: true:	// when an element is added on UI it is in edit mode.
	_editState: true,	
	
	// _editNode: DOMNode 
	//		Node to be displayed in edit mode. Can be specified in template.
	// _readNode: DOMNode 
	//		Node to be displayed in read mode. Can be specified in template.
	
	// _stringUtils: com.ibm.security.modeling.util.StringUtils
	//		Utilities for strings.
	// _stringUtils: null,
	
	// _widgetUtilities: com.ibm.security.modeling.util.WidgetUtilities
	//		Utilities for widgets.
	// _widgetUtilities: null,
	
	postCreate: function () {
		this._trace.entering(this.id, "postCreate");
		
		this.inherited (arguments);
		
		this._makeFocusable(); // focusable.
		
		this._stringUtils = new com.ibm.security.modeling.util.StringUtils();
		this._widgetUtilities = new com.ibm.security.modeling.util.WidgetUtilities();
		
		//dojoAttachEvent="onmousedown:_onStartEditing"
		if (this._editButton) {
			this.connect(this._editButton.domNode, "onmousedown", dojo.hitch(this, 
				function(e) { 
					this._onStartEditing(e);
				})
			);
		}
		
		this._trace.exiting(this.id, "postCreate");
	},
	
	startup:function() {
		this._trace.entering(this.id, "startup");
		this.inherited(arguments);
		
		this.domNode.tabIndex = this._tabIndex;	// initial tab index.
		
		this._trace.exiting(this.id, "startup");
	},
	
	setFocusNode: function(/*Widget|DOMNode*/ node) {
		this._focusNode = node;
	},
	
	getFocusNode: function() {
		return this._focusNode;
	},
	
	focus: function(forced) {
		this._trace.entering(this.id, "focus");
		// Focus is done via code. 
		// this.domNode is not focused yet.
		// onFocus should not have executed yet.
		
		if (forced !== true) {
			forced = false;
		}
		
		this._handleFocus(forced);
		
		this._trace.exiting(this.id, "focus");
	},
	
	onFocus: function() {
		this._trace.entering(this.id, "base::onFocus");
		// this.domNode has been focused.
		// Widget knows that it has been focused.
		
		this._handleFocus(false); // no forced.
		
		this._trace.exiting(this.id, "base::onFocus");
	},
	
	onBlur: function() {
		this._trace.entering(this.id, "onBlur");
		if (!this._isFocused) {
			this._trace.exiting(this.id+" - Already Blured", "base::_whenFocused");
			return;
		}
		this._isFocused = false;
		this._isFocusAdjusted = false;
		this._makeFocusable(); // After blur, widget should be re-focusable.
		this.displayInReadMode();
		this._trace.exiting(this.id, "onBlur");
	},
	
	_handleFocus: function(forced) {
		this._whenFocused(forced);
		this._adjustFocus(forced);
	},
	
	_whenFocused: function(forced) {
		this._trace.entering(this.id, "base::_whenFocused");
		
		if (this._isFocused && !forced) {
			this._trace.exiting(this.id+" - Already focused", "_whenFocused");
			return;
		}
		this._isFocused = true;
		
		this._makeNonFocusable(); // Shift+Tab should focus out from widget.
		// When focus is to be moved to itself, make sure that the edit mode is ON.
		this.displayInEditMode(forced);
		
		dojo.window.scrollIntoView(this.domNode); // defect# 33584
		this._trace.exiting(this.id, "base::_whenFocused");
	},
	
	_adjustFocus: function(forced) {
		this._trace.exiting(this.id, "_adjustFocus");
		if (this._isFocusAdjusted == true && !forced) {
			this._trace.exiting(this.id+" - Already adjusted the focus", "_adjustFocus");
			return;
		}
		this._isFocusAdjusted = true;
		// Shift the focus to the _focusNode
		if (this._focusNode && this._focusNode.focus) {
			this._focusNode.focus();
		}
		this._trace.exiting(this.id, "_adjustFocus");
	},
	
	_makeFocusable: function() {
		this._trace.entering(this.id, "_makeFocusable");
		this.domNode.tabIndex = this._tabIndex; // focusable.
		this._trace.exiting(this.id, "_makeFocusable");
		
	},
	
	_makeNonFocusable: function() {
		this._trace.entering(this.id, "_makeNonFocusable");
		this.domNode.tabIndex = -1;	// non-focusable. 
		this._trace.exiting(this.id, "_makeNonFocusable");
	},
	
	// override it.
	displayInReadMode: function(forced) {
		this._trace.entering(this.id, "base::displayInReadMode");
		if (! this.isInEditMode() && !forced) {
			// not in edit mode: already in read mode.
			this._trace.exiting(this.id+" - alreay in read mode.", "base::displayInReadMode");
			return;
		}
		
		if (this._readNode && this._editNode) {
			this._showHide(/*hideNode*/this._editNode, /*showNode*/ this._readNode);
		}
		
		this._editState = false;
		
		
		this._adjustLayout();
		
		
		this._trace.exiting(this.id, "base::displayInReadMode");
	},
	
	
	_adjustLayout: function() {
		if(dojo.isIE) {
			// Defect# 33590, Defect# 33592
			dojo.removeClass(this.domNode, "adjustLayOut");
			dojo.addClass(this.domNode, "adjustLayOut");
		}
	},
	
	// Override it.
	displayInEditMode: function(forced) {
		this._trace.entering(this.id, "base::displayInEditMode");
		if (this.isInEditMode() && !forced) {
			// in edit mode: already in edit mode.
			this._trace.exiting(this.id+" - alreay in edit mode.", "base::displayInEditMode");
			return;
		}
		
		
		if (this._readNode && this._editNode) {
			this._showHide(/*hideNode*/this._readNode, /*showNode*/ this._editNode);
		}
		
		this._editState = true;
		
		this._adjustLayout();
		this._trace.exiting(this.id, "base::displayInEditMode");
	},
	
	_showHide: function(/*DOMNode*/ hideNode, /*DOMNode*/ showNode) {
		// Summary:
		//			Hides the 'hideNode' and shows the 'showNode'
		
		// Hide the 'hideNode'
		dojo.removeClass(hideNode, "show");
		dojo.addClass(hideNode, "hide");
		
		// Show the 'showNode'
		dojo.removeClass(showNode, "hide");
		dojo.addClass(showNode, "show");
		
	},
	
	
	isInEditMode: function() {
		return this._editState;
	},
	
	_setButtonLabel: function(/*Button Widget*/ button, /*string*/label) {
		button.set("label", (label? label:""));
	},
	
	_enableButton: function(/*Button Widget*/ button, /*string*/label) {
		// Update Label.
		this._setButtonLabel(button, label);
		// Enable button.
		this._widgetUtilities.enableButton(button);
	},
	
	_disableButton: function(/*Button Widget*/ button, /*string*/label) {
		// Update Label.
		this._setButtonLabel(button, label);
		// Enable button.
		this._widgetUtilities.disableButton(button);
	},
	
	_onStartEditing: function(event) {
		// Summary:
		//		Executes when edit icon/button (pencil) is clicked.
		//		Focuses itself [which intern renders itself in edit mode]
		
		this._trace.entering(this.id, "_onStartEditing");
		
		dojo.stopEvent(event);
		
		this.focus(/*forced*/true);
		this._trace.exiting(this.id, "_onStartEditing");
	},
	
	showClickToEditPencil: function() {
		// Summary:
		//		Shows the pencil button [this._editButton]
		
		this._trace.entering(this.id, "showClickToEditPencil");
		//if (this._editButton) {
			dojo.removeClass(this._editButton.domNode, "hide");
			dojo.addClass(this._editButton.domNode, "showInline");
		//}
		this._trace.exiting(this.id, "showClickToEditPencil");
	},
	
	hideClickToEditPencil: function() {
		// Summary:
		//		Hides the pencil button [this._editButton]
		
		this._trace.entering(this.id, "hideClickToEditPencil");
		//if (this._editButton) {
			dojo.removeClass(this._editButton.domNode, "showInline");
			dojo.addClass(this._editButton.domNode, "hide");
		//}
		this._trace.exiting(this.id, "hideClickToEditPencil");
	}
});
