/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

(c) Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.membership.Filter");

dojo.require("com.ibm.security.modeling.membership._ElementContainer");

dojo.require("dojo.data.ItemFileReadStore");
dojo.require("dojox.data.JsonRestStore");

dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.util.ValidationUtilities");
dojo.require("com.ibm.security.modeling.membership.FilterData");
dojo.require("com.ibm.security.modeling.membership.FilterGroup");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");

// A controller class, representing a filter.
dojo.declare("com.ibm.security.modeling.membership.Filter", com.ibm.security.modeling.membership._ElementContainer,  {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.membership.Filter"),

	validator : new com.ibm.security.modeling.util.Validator(),	
	
	constructor: function () {
		this._trace.entering("", "constructor");
		
		// id: string
		//		Filter id.
		// globalResources: Object
		//			Reference to the global shared resource bundle
		// localResources: Object
		//			Reference to the local shared resource bundle
		// data: object
		//		An object representing a filter
		// filterBody: object
		//		Div reference of filter container DIV, obtained using dojoAttachPoint in template
		// messageArea: ibm.tivoli.tip.dijit.TIPMessageArea
		//		This is to display messages on GUI when any error occurs.
		
		dojo.mixin(this, arguments[0]);
		
		this._postMixInProperties(arguments);
		this._trace.exiting("", "constructor");
	},
	
	createElement: function(/*string*/ elementId, /*json object*/ elementData) {
		// Summary:
		//		Creates contained elements (group) widget's instance.
		// tags:	overridden from _ElementContainer.js
		//		
		// returns: com.ibm.security.modeling.membership.FilterGroup
		
		return new com.ibm.security.modeling.membership.FilterGroup({
			id: elementId,
			data: elementData,
			filter: this,
			globalResources: this.globalResources,
			localResources: this.localResources,
			messageArea: this.messageArea,
			attributeStore: this.attributeStore,
			attributeOperatorStore: this.attributeOperatorStore,
			operatorStore: this.phraseLevelOperatorStore // temporary
			
		});
	},
	
	_getElementsJsonFromRestData: function(/*json object*/ filterData) {
		// Summary: 
		//		Extracts Elements (groups) from the json object from REST.
		// tags: overridden from _ElementContainer.js
		// filterData: json object
		//		Represents details about the container (filter), and its elements (groups).
		//
		// returns: Array
		return com.ibm.security.modeling.membership.FilterData.getFilterGroups(filterData);
	},
	
	_getElementOperatorFromRestData: function(/*json object*/ filterData) {
		// Summary: 
		//		Extracts Elements operator (group operator) from the json object from REST.
		// tags: overridden from _ElementContainer.js
		// filterData: json object
		//		Represents details about the container (filter), and its elements (groups).
		//
		// returns: string
		var operator = null;
		if (filterData) {
			operator = com.ibm.security.modeling.membership.FilterData.getFilterGroupOperator(filterData);
		} 
		
		if (!operator) {
			operator = this._getSynchronizedSelectorDefaultValue();
		}
		
		return operator;
	},
	
	render: function() {
		this._trace.entering(this.id, "render");
		this.displayInReadMode();
		this._trace.exiting(this.id, "render");
	},
	
	reRender: function(/*json*/ data) {
		this._trace.entering(this.id, "reRender");
		this.data = data;
		this._loadAttributes();
		
		// remove current elements.
		this.empty();
		// create elements per the data.
		this._createElements(data);
		this._startupElements();
		this.render();
		
		/*
		var firstGroup = this._firstElement;
		if (firstGroup) {
			firstGroup.displayInEditMode(true);
			firstGroup.groupNameInputWidget.displayInEditMode(true);
		}
		*/
		this._trace.exiting(this.id, "reRender");
	},
	
	addGroup: function(/*json object*/ groupData) {
		this._trace.entering(this.id, "addGroup");
		
		// First show existing filter in read mode.
		this.displayInReadMode();
		
		// inherited.
		var group = this.addElement(groupData);
		group.focus(/*forced*/ true);
		
		this._trace.exiting(this.id, "addGroup");
	},
	
	removeGroup: function(group) {
		this._trace.entering(this.id, "removeGroup");
		this.removeElement(group);
		this._trace.exiting(this.id, "removeGroup");
	},
	
	displayInReadMode: function() {
		this._trace.entering(this.id, "displayInReadMode");
		this._synchronizedSelector.displayInReadMode();
		for (var groupId in this._elements) {
			this._elements[groupId].displayInReadMode();
		}
		this._trace.exiting(this.id, "displayInReadMode");
	},
	
	_postMixInProperties: function() {
		this._trace.entering(this.id, "_postMixInProperties");
		
		// inherited
		this.postMixInProperties(arguments);
		
		// Element container
		this.elementContainer = this.filterBody;
		this._elementType = "group";
		
		this._createUIData();	// temporary
		
		this._postCreate(arguments);
		
		this._trace.exiting(this.id, "_postMixInProperties");
	},
	
	_postCreate: function () {
		this._trace.entering(this.id, "_postCreate");
		
		// inherited
		this.postCreate (arguments);
		
		this._trace.exiting(this.id, "_postCreate");
	},
	
	_getSynchronizedSelectorDefaultValue: function() {
		return 'or';
	},
	
	_createUIData: function() {
		this._trace.entering(this.id, "_createUIData");
		
		this._loadGroupLogicalOperators();
		
		this._loadPhraseLogicalOperators();
		
		this._createAttributeJsonService();
		this._loadAttributes();
		
		this._loadAttributeOperators();
		
		this._trace.exiting(this.id, "_createUIData");
	},
	
	_createAttributeJsonService: function() {
		this._trace.entering(this.id, "_createAttributeJsonService");
		
		this._attributeJsonService = new dojox.data.JsonRestStore({
			target: "/isrm/rest/userscope/attributes",
			idAttribute: 'uri'
		});
		
		this._trace.exiting(this.id, "_createAttributeJsonService");
	},
	
	_loadGroupLogicalOperators: function() {
		this._trace.entering(this.id, "_loadGroupLogicalOperators");
		
		var groupLevelOperatorsJson = {
			identifier:"value",
			label: "operator",
            items: [
                {operator: this.localResources.LOGICAL_OPERATOR_AND,value: 'and'},
	            {operator: this.localResources.LOGICAL_OPERATOR_OR,value: 'or'}
	        ]
		};
		this.operatorStore = new dojo.data.ItemFileReadStore({
            data: groupLevelOperatorsJson
        });
		
		this._trace.exiting(this.id, "_loadGroupLogicalOperators");
	},
	
	_loadPhraseLogicalOperators: function() {
		this._trace.entering(this.id, "_loadPhraseLogicalOperators");
		
		var phraseLevelOperatorsJson = {
			identifier:"value",
			label: "operator",
            items: [
                {operator: this.localResources.LOGICAL_OPERATOR_AND,value: 'and'},
	            {operator: this.localResources.LOGICAL_OPERATOR_OR,value: 'or'}
	        ]
		};
		this.phraseLevelOperatorStore = new dojo.data.ItemFileReadStore({
            data: phraseLevelOperatorsJson
        });
		
		this._trace.exiting(this.id, "_loadPhraseLogicalOperators");
	},
	
	_loadAttributes: function() {
		this._trace.entering(this.id, "_loadAttributes");
		
		this._attributeJsonService.fetch({
			onComplete: function(items) {
				var attributes = [];
				// A blank one
				attributes.push({
					"name": " ", 
					"type": "invalidtype",
					"uri": "https://localhost:16311/isrm/rest/userscope/attributes/0"
				});
				
				// fetched items.
				if (items && items.length > 0) {
					for (var i=0; i<items.length; i++){
						var name = (items[i].name.toUpperCase() in this.attributeResources) ? 
								this.attributeResources[items[i].name.toUpperCase()] :
									items[i].name;
						
						// In case there are any <script> OR HTML formatting tags in this name, need to escape it.
						var label = this.validator.escapeHTMLAndScriptTags(name);
						if (dojo.isIE) {
							label = com.ibm.security.modeling.util.HtmlEscapingUtils.breakString(label, 80);
						}
						var obj = {
							"name": name,
							"label": label, 
							"type": items[i].type.toUpperCase(),
							"uri": items[i].uri
						};
						
						attributes.push(dojox.json.ref.fromJson(dojox.json.ref.toJson(obj)));
					}
				}
				this.attributeStore = new dojo.data.ItemFileReadStore({
					data: {
						identifier:"uri",
						label: "label",
						items:attributes
					}
				});
			},
			scope: this,
			syncMode: "true"
		});
		
		this._trace.exiting(this.id, "_loadAttributes");
	},
	
	_loadAttributeOperators: function() {
		this._trace.entering(this.id, "_loadAttributeOperators");
		
		var attributeOperatorsJson = {
			identifier: "value",
			label: "operator",
            items: [
                {operator: ' ',				value: 'invalidtype', 	applicableto: "invalidtype,STRING,INTEGER,HIERARCHY,IDENTITY"},	// applicable to all types.
                {operator: this.localResources.CONDITION_ATTRIBUTE_OPERATOR_CONTAINS,		value: 'contains', 		applicableto: "STRING"},
                {operator: this.localResources.CONDITION_ATTRIBUTE_OPERATOR_STARTSWITH,	value: 'startsWith', 	applicableto: "STRING"},
	            {operator: this.localResources.CONDITION_ATTRIBUTE_OPERATOR_ENDSWITH,		value: 'endsWith', 		applicableto: "STRING"},
                {operator: this.localResources.CONDITION_ATTRIBUTE_OPERATOR_EQUALS,		value: 'equals', 		applicableto: "STRING,INTEGER,IDENTITY,HIERARCHY"},
                {operator: this.localResources.CONDITION_ATTRIBUTE_OPERATOR_NOTEQUALS,	value: 'notEquals', 	applicableto: "STRING,INTEGER,IDENTITY,HIERARCHY"},
                {operator: this.localResources.CONDITION_ATTRIBUTE_OPERATOR_GREATERTHAN,	value: 'greaterThan', 	applicableto: "INTEGER"},
                {operator: this.localResources.CONDITION_ATTRIBUTE_OPERATOR_LESSTHAN,		value: 'lessThan', 		applicableto: "INTEGER"},
                {operator: this.localResources.CONDITION_ATTRIBUTE_OPERATOR_GREATERTHAN_OREQUAL,	value: 'greaterThanOrEqual', 	applicableto: "INTEGER"},
                {operator: this.localResources.CONDITION_ATTRIBUTE_OPERATOR_LESSTHAN_OREQUAL,		value: 'lessThanOrEqual', 		applicableto: "INTEGER"},
                {operator: this.localResources.CONDITION_ATTRIBUTE_OPERATOR_CONTAINED,		value: 'containedIn', 		applicableto: "HIERARCHY"},
                {operator: this.localResources.CONDITION_ATTRIBUTE_OPERATOR_NOTCONTAINED,		value: 'notContainedIn', 		applicableto: "HIERARCHY"}
	        ]
		};
		this.attributeOperatorStore = new dojo.data.ItemFileReadStore({
            data: attributeOperatorsJson
        });
		
		this._trace.exiting(this.id, "_loadAttributeOperators");
	}
});
