/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

(c) Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.membership.FilterBuilder");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("com.ibm.security.modeling.dijit.layout.BorderContainer");

dojo.require("com.ibm.security.modeling.membership.Filter");
dojo.require("com.ibm.security.modeling.membership.FilterData");
dojo.require("com.ibm.security.modeling.widgets.MultiMessageArea");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.indicator.SaveIndicator");

dojo.require("com.ibm.security.modeling.util.TitleUtilities");
dojo.require("com.ibm.security.modeling.util.HelpUtilities");
dojo.require("com.ibm.security.modeling.util.ConsoleUtilities");
dojo.require("com.ibm.security.modeling.widgets.ProgressBar");
dojo.require("com.ibm.security.modeling.membership.FilterButtonUtil");
dojo.require("com.ibm.security.modeling.membership.FilterEvaluationUtil");
dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
dojo.require("com.ibm.security.modeling.breadcrumbs.BreadCrumInterface");
dojo.require("com.ibm.security.modeling.widgets.ConfirmMessageDialog");
// NLS
dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling", "PluginMessages");

dojo.requireLocalization("com.ibm.security.modeling.membership", "FilterText");

// define the widget class
dojo.declare (
	"com.ibm.security.modeling.membership.FilterBuilder", 
    [ dijit._Widget, dijit._Templated ,com.ibm.security.modeling.breadcrumbs.BreadCrumInterface],
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.membership.FilterBuilder"),
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.membership","templates/FilterBuilder.html"),
	widgetsInTemplate: true,
	
	// namespace: [readonly] String
	// 			Unique identifier to the instance of portlet page where this item will exist
	namespace:"",
	
	// contextRoot: [readonly] String
	//			The context root of the server from which Content is being served.
	contextRoot: "",
	
	// parent: com.ibm.security.modeling.notebook.RoleNotebookDialogView
	//		Reference to parent container
	parent: null,

	// helpFile: [readonly] String
	//			name of the HTML help file for filter.
	helpFile: "",
	
	// filterData: String
	// 			Filter content pertaining to a given role.
	// filterData: null,
	
	// The title to reset the page to on switching back to the notebook.
	notebookTitle: null,
	
	// pageToLaunch: Integer
	//		Indicates the page to be launched at a given point of time, when working through the
	//		BreadCrumbs
	pageToLaunch: -1,
	
	// progressBarShown: boolean
	//		Indicator of whether the progress bar for filter evaluation is already shown.
	progressBarShown: false,	
	
	// roleNotebookPane: String
	//		Reference to the Role Notebook Panel.
	roleNotebookPane: null,
	
	// titleUtil: com.ibm.security.modeling.util.TitleUtilities
	//			sets the portlet title to the current panel's	
	titleUtil: new com.ibm.security.modeling.util.TitleUtilities(),	

	// helpUtil: com.ibm.security.modeling.util.HelpUtilities 
	//			sets current help file
	helpUtil: new com.ibm.security.modeling.util.HelpUtilities(),
	
	// filterUri: The URI received in response to a filter create/edit request.
	filterUri: null,

	// filterEvaluationRequest: The evaluation request created after the OK was hit.
	filterEvaluationRequest: null,
	
	// Flag to track if there were any edits in the current page.
	editHappened: false,
	
	/*
	 * Special case is when no filter existed, and Save/Apply was clicked
	 * then CREATE should take place,
	 * and there on UPDATE should take place on both Save, Apply.
	 * 
	 * Such scenario(s) has been handled by getting me taget uri and htt method using
	 * - 	_getTargetUri()
	 * -	_getHttpRequestMethod()
	 */
	
	_targetUri: "",
	
	FILTER_BUILDER_CREATE_MODE: "Create", // Filter Does not exists.
	FILTER_BUILDER_EDIT_MODE: "Edit", // Filter exists.
	
	// Internal variable to determine the filter builder mode [create/edit]
	_filterBuilderMode: null, 
	
	
	constructor: function() {
		// roleItem:
		//		Role Item from Json Store for filter.
		
		this._trace.entering("", "constructor");		
		this._trace.object(arguments, "constructor", "Constructor args");
		this._trace.exiting("", "constructor");
	},
	
	postMixInProperties: function() {
		// Summary:
		//		Widget life-cycle method. 
		//		Sets NLS resources.
		
		this._trace.entering(this.id, "postMixInProperties");
		
		this.inherited(arguments);
		
		if (!this.globalResources) {
			this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		}
		if (!this.pluginResources) {
			this.pluginResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "PluginMessages");
		}
		if (!this.localResources) {
			this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.membership", "FilterText");
		}
		
		if (!this.filterData) {
			this.filterData = null;
		}
		this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
		// Setup builder state per the supplied data.
		this._updateFilterBuilderState();

		this._trace.exiting(this.id, "postMixInProperties");
	},
	
	postCreate: function() { 
		// Summary:
		//		Widget life-cycle method. 
		//		Creates filter.
		this._trace.entering(this.id, "postCreate");
		
		this.inherited(arguments);
		
		this._filter = new com.ibm.security.modeling.membership.Filter({
			id: this.id+"_filter",
			data: this.filterData,
			filterBody: this.filterContainer,
			globalResources: this.globalResources,
			localResources: this.localResources,
			messageArea: this.messageArea,
			emptyContainerInstructions: this.emptyFilterInstructions 
		});
		// this._filter.startup();
		
		this._defineButtonBehaviors();
		
		this._trace.exiting(this.id, "postCreate");
	},
	
	startup: function() {
		// Summary:
		//		Widget life-cycle method.
		//		Starts up the widget and the filter inside it.
		this._trace.entering(this.id, "startup");
		this.inherited(arguments);
		
		// Hide previous messages [if any]
		this.messageArea.hideMessages();
		this._filter.render();
		this._trace.exiting(this.id, "startup");
	},
	
	destroy: function() {
		//	Summary:
		//		Widget life-cycle method.
		//		Destroys the filter recursively.
		
		this._trace.enteringWithStack(this.id, "destroy", true);
		this._trace.entering(this.id, "destroy");
		try {
			if (this._filter) {
				this._filter.destroyRecursive();
			}
			
			this.inherited(arguments);
			
			this._trace.exiting(this.id, "destroy");
		} catch(e) {
			this._trace.exception(e, "destroy");
			throw e;
		}
		this._trace.exiting(this.id, "destroy");
	},
	
	adjustLayout: function() {
		// Summary
		//		Invokes resize on the container to fit in the available view space
		
		this._trace.entering(this.id, "adjustLayout");
		this.container.resize();
		this._trace.exiting(this.id, "adjustLayout");
	},
	
	_resetPanelScrolling: function() {
		// Summary:
		//			Resets panel scrolling.
		this._trace.entering(this.id, "_resetPanelScrolling");
		this.centerRegion.domNode.scrollTop = 0;
		this.centerRegion.domNode.scrollLeft = 0;
		this._trace.exiting(this.id, "_resetPanelScrolling");
	},
	
	_defineButtonBehaviors: function() {
		// summary:
		//			connects methods in this widget with button clicks
		// tags:
		//			private
		this._trace.entering(this.id, "_defineButtonBehaviors");
		this.addSectionButton.onClick = dojo.hitch(this, this._onAddSection);
		this.okButton.onClick = dojo.hitch(this, this._onOK);
		this.applyButton.onClick = dojo.hitch(this, this._onApply);
		this.cancelButton.onClick = dojo.hitch(this, this._onCancel);
		
		com.ibm.security.modeling.membership.FilterButtonUtil.updateButtonClick(
			this, 
			this.addSectionButton.domNode, 
			dojo.hitch(this, function(event) {
				this._onAddSection(event);
			})
		);
		
		this._trace.exiting(this.id, "_defineButtonBehaviors");
	},	
	
	populate: function(/*json*/filterData, /*json*/roleItem, /*string*/notebookTitle) {
		// Summary:
		//		Populates filter builder per the supplied data.

		this._trace.entering(this.id, "populate");
		this.filterData = filterData;
		
		this.roleItem = roleItem;
		this.notebookTitle = notebookTitle;
		
		// Setup builder state per the supplied data.
		this._updateFilterBuilderState();
		
		// Hide previous messages [if any]
		this.messageArea.hideMessages();
		
		// Re-render the filter with supplied filter data.
		this._filter.reRender(filterData);
		
		this._resetPanelScrolling();
		
		this._trace.exiting(this.id, "populate");
	},
	
	_onAddSection: function () {
		// Summary:
		//		Adds an empty group to filter.
		try {
			this._trace.entering(this.id, "_onAddSection");
			
			this._filter.addGroup(/*groupData*/null); // no data: Add a new empty group
			
		} finally {
			this._trace.exiting(this.id, "_onAddSection");
		}
	},
	
	//Overridding breadcrum interface 
	canHide: function(){
		this._trace.entering(this.id, "canHide");
		
		if (this.isSaveRunning()) { // Save is running.
			this._handleCloseOnRunningSave(true);
			setTimeout (dojo.hitch (this, this.canHide), 100);
		} else if (this._isSavePending()) { // Some changes are yet to be saved.
			return this._canHideOnPendingSave();
		} else {
			this._hideProgressBar();
			this.progressBarShown = false;
			dojo.publish("NAVIGATE_AWAY_EVENT");
			return true;
		}
		this._trace.exiting(this.id, "canHide");
		return false;
	},
	
	_canHideOnPendingSave: function(event) {
		// Summary:
		//		Handles page behaviour, when close is requested and some changes are pending to be saved.
		// event: dojo normalized dijit event
		
		this._trace.entering(this.id, "_canHideOnPendingSave");
		// Ask for confirmation.
		this._launchConfirmationDialog(
			this.localResources.CONFIRM_EXIT_ON_PENDING_SAVE_MESSAGE, 	// messageText
			dojo.hitch(this, function() { 								// onSubmitCallback
				//	User wanted to navigate away from this page.
				this._trace.finest(this.id+" - User wanted to navigate away from this page.", "_canHideOnPendingSave");
				
				dojo.publish("NAVIGATE_AWAY_EVENT");
			}),
			dojo.hitch(this, function() { 								// onCancelCallback
				// User wanted to stay on this page.
				this._trace.finest(this.id+" - User wanted to stay on this page.", "_canHideOnPendingSave");
				// Do nothing, stay on the same page.
			})
		);
		this._trace.exiting(this.id, "_canHideOnPendingSave");
		
		return false;
	},
	
	_navigateAway: function() {
		this._trace.entering(this.id, "_navigateAway");
		
		// breadcrumb invoking this method means that 'navigate away' should happen.
		// if navigation was not supposed to happen, then that condition should have had captured in canHide() and 'false' had been returned.
		// because, when canHide() returns 'false' then _navigateAway() is not invoked.
		
		dojo.publish(dojo.config.panels.ROLE.launchEventAddress);
		if(this.editHappened) {
			/* Refresh the filter viewer section automatically. */
			/* 
			 * This is a fake event that'll notify the Membership Panel that filter edit is complete,
			 * and there was some save done, either through Apply / OK. The panel won't refresh
			 * immediately after the Apply is complete. It would rather refresh on receiving this 
			 * event. 
			 */
			this._publishFilterEdit(this.filterUri, this.filterEvaluationRequest);
			this.editHappened = false;
		}
		this._trace.exiting(this.id, "_navigateAway");
	},
	
	_onClose: function(event) {
		try {
			this._trace.entering(this.id, "_onClose");
			
			if ((dojo.isIE) && (event != null)) {
				setTimeout (dojo.hitch (this, this._onClose), 100);
			} else if (this.isSaveRunning()) { // Save is running.
				this._handleCloseOnRunningSave(false);
				setTimeout (dojo.hitch (this, this._onClose), 100);
			} else {
				this._hideProgressBar();
				this.progressBarShown = false;
				this._navigateAway(); // to let breadcrumb listener know about the close.
			}
		} finally {
			this._trace.exiting(this.id, "_onClose");
		}	
	},
	
	_onCancel: function(event) {
		// Summary:
		//		Call back method, executes when "Cancel" button is clicked.
		this._onClose(event);
	},
	
	isSaveRunning: function() {
		return this.saveIndicator.isSaving();
	},
	
	_handleCloseOnRunningSave: function(/* boolean */ fromBreadCrumb) {
		// Summary:
		//		Handles page behaviour, when close is requested and some changes are getting saved.
		// event: dojo normalized dijit event
		
		this._trace.entering(this.id+" - filter evaluation is running. Can not navigate away from Filter Editor.", "_handleCloseOnRunningSave");
		
		// We need to launch the Filter Evaluation Status in the form of a Progress Bar, only when the invocation was through the breadcrumb.
		if(fromBreadCrumb && !this.progressBarShown) {
			this._showFilterEvalProgressBar();
			this.progressBarShown = true;
		}				
		
		this._trace.exiting(this.id, "_handleCloseOnRunningSave");
	},
	
	_launchConfirmationDialog: function(/*string*/ messageText, /*function*/ onSubmitCallback, /*function*/ onCancelCallback) {
		// Summary:
		//		Launches the ConfirmMessageDialog and supplies the arguments to that.
		
		this._trace.entering(this.id, "_launchConfirmationDialog");
		var confirmationDialog = com.ibm.security.modeling.widgets.ConfirmMessageDialog({
			id: this.id+"_confirmationDialog",
			submitButtonText: this.globalResources.YES,
			cancelButtonText: this.globalResources.NO,
			completeMessage: messageText,
			onSubmit: onSubmitCallback,
			onCancel: onCancelCallback
		});
		
		confirmationDialog.hide = function() {
			this.inherited("hide", arguments );
			setTimeout (dojo.hitch (this, this.destroyRecursive),300);
		};
		
		confirmationDialog.startup();
		confirmationDialog.show();
		this._trace.exiting(this.id, "_launchConfirmationDialog");
	},
	
	
	_isSavePending: function() {
		// Summary:
		//		Tells if some changes on Filter Builder are yet to be saved.
		
		return this._isFilterEdited(null);
	},
	
	_isCancelling: function(event) {
		// Summary:
		//		Tells if event is from Cancel button.
		
		var isCancel = false;
		if (event && event.target && event.target.id) {
			isCancel = (event.target.id.indexOf(this.cancelButton.id) != -1);
		}
		
		return isCancel;
	},
	
	_onApply: function() {
		// Summary:
		//		Call back method, executes when "Apply" button is clicked.
		//		
		//		Apply submits the filter and remains on the same panel
		//		-	Save should happen with progress spinner
		//		-	Statistics evaluation should happen with saving indicator 
		//			(with same button/breadcrumb behaviour as on notebook)
		
		this._trace.entering(this.id, "_onApply");
		this._onSave(/*closePage*/ false);
		this._trace.exiting(this.id, "_onApply");
	},
	
	_onOK: function(/*dijit normalized event*/ event) {
		/*
		 * OK submits the filter and after successful save it closes panel.
		 * 	-	Save should happen with progress spinner
		 * 	-	Statistics evaluation should be triggered and control should move back to notebook.
		 */
		
		this._trace.entering(this.id, "_onOK");
		this._onSave(/*closePage*/ true);
		this._trace.exiting(this.id, "_onOK");
	},
	
	
	_onSave: function(/*boolean*/ closePage) {
		// Summary:
		//		Call back method, executes when "Save" button is clicked.
		//		Initiates the process of saving (create/update) the filter.
		//
		// closePage: boolean
		//		Boolean indicator to close page after successful save.
		//		true - close page after save completes with success
		//		false - Do NOT close page after save completes with success
		
		try {
			this._trace.entering(this.id, "_onSave");
			if (this._filter) {
				var filterUIJson = this._filter.getJsonFromUI();
				this._trace.object(filterUIJson, "_onSave", this.id+" - json object from UI elements.");
				if (filterUIJson) {
					this._saveFilter(filterUIJson, closePage);
				} else {
					// show error message that no filter data is supplied.
					this.messageArea.showErrorMessage(this.pluginResources.VALIDATION_EMPTY_FILTER);
				}
			}
		} catch (e) {
			this._trace.exception(e, "_onSave");
			throw e;
		} finally {
			this._trace.exiting(this.id, "_onSave");
		}
	},
	
	_saveFilter: function(/*json*/ filterUIJson, /*boolean*/ closePage) {
		// Summary:
		//		Initiates filter create/update process.
		//
		// filterUIJson: json object
		//		json representing a filter from UI elements.
		// closePage: boolean
		//		Boolean indicator to close page after successful save.
		//		true - close page after save completes with success
		//		false - Do NOT close page after save completes with success
		
		try {
			this._trace.entering(this.id, "_saveFilter");
			var filterRestJson = com.ibm.security.modeling.membership.FilterData.getFilterJsonForREST(filterUIJson, this.roleItem);
			this._trace.object(filterRestJson, "_saveFilter", this.id+" - json object for REST request.");
			
			if (filterRestJson) {
				if (this._isFilterEdited(filterRestJson)) {
					this._onSaveStart();
					this._sendHttpRequest(filterRestJson, closePage);
				} else { // nothing to save.
					this._handleCloseAfterSave(/*isError*/ false, closePage);
				}				
			} else {
				// show error message that no filter data is supplied.
				this.messageArea.showErrorMessage(this.pluginResources.VALIDATION_EMPTY_FILTER);
			}
		} catch (e) {
			this._trace.exception(e, "_saveFilter");
			throw e;
		} finally {
			this._trace.exiting(this.id, "_saveFilter");
		}
	},
	
	_isFilterEdited: function(/*json*/ filterRestJsonFromUI) {
		// Summary:
		//		Determines if filter has been edited and contains some changes which are to be saved.
		//
		// filterRestJsonFromUI: json
		//		Json object for REST interface, retrieved from UI element's Json.
		
		var isFilterEdited = false;
		if (!filterRestJsonFromUI) {
			var filterUIJson = this._filter.getJsonFromUI();
			filterRestJsonFromUI = com.ibm.security.modeling.membership.FilterData.getFilterJsonForREST(filterUIJson, this.roleItem);
		}
		
		var isEqual = com.ibm.security.modeling.membership.FilterData.areFiltersEqual(this.filterData, filterRestJsonFromUI);
		if (isEqual === false) { // filter data differs from the data on UI. 
			// User has done some editing.
			isFilterEdited = true;
		}
		return isFilterEdited;
	},
	
	_sendHttpRequest: function(/*json*/ jsonData, /*boolean*/ closePage) {
		// Summary:
		//		Makes an HTTP [PUT/POST] request for filter.
		// jsonData
		//		Request body
		// closePage: boolean
		//		Boolean indicator to close page after successful save.
		//		true - close page after save completes with success
		//		false - Do NOT close page after save completes with success
		
		var httpMethod = this._getHttpRequestMethod();
		
		if (httpMethod != 'put' && httpMethod != 'post') {
			throw "Invalid HTTP method name";
		}
		
		var filterRestService = null; 
		var request = null;
		
		var targetUri =  this._getTargetUri();
		var filterRestService = new dojox.rpc.Rest(targetUri, true);
		var filterId = "";
		request = filterRestService[httpMethod](
			filterId, 	// id
			dojox.json.ref.toJson(jsonData) 	// request payload
		);
		
		var onResponse = dojo.hitch(this, function(/*boolean*/isError, /*uri string*/filterUri) {
			// Summary
			//		Performs activities when HTTP response comes back (success, failure).
			//
			// isError: boolean
			//		true represents that HTTP request should be considered failed.
			//		false represents that HTTP requests completed with success (with success codes)
			// filterUri: uri string
			//		URI of the filter.
			//		It should be used to retrieve the filter and to create URI for filter evaluation.
			if (isError === false) {
				this.filterData = jsonData;
			}
			this._onSaveComplete(/*boolean*/isError, /*filter uri string*/filterUri, closePage);
		});
		
		// Success handler
		var successCallBack = function( result ) {
			var isError = false;
			var filterUri = null;
			try {
				var responseStatus = request.ioArgs.xhr.status;
				
				if (responseStatus == 201) { // filter created successfully
					// Location header value is the URI of the created filter.
					// Extract the location header
					filterUri = request.ioArgs.xhr.getResponseHeader("Location");
					if (!filterUri) {
						isError = true;
					}
				} 
				else if (responseStatus == 200) { // for PUT (Edit filter) 
					// updated successfully
					filterUri = targetUri;
				} else { // Consider it an error.
					isError = true;
				}
			} catch (e) {
				this._trace.exception(e, "successCallBack");
				throw e;
			} finally {
				onResponse(isError, filterUri);
			}
			return result;
		};
		
		// Failure handler
		var errorCallBack = function( error, request ) {
			var isError = true;
			try {
				this._handleRESTError(error, request);
			} catch (e) {
				this._trace.exception(e, "errorCallBack");
				throw e;
			} finally {
				onResponse(isError);
			}
			return error;
		};
		
		// attach callbacks with the request 
		request.addCallback(dojo.hitch(this, successCallBack));
		request.addErrback(dojo.hitch(this, errorCallBack));
		
		return request;
	},
	
	_onSaveStart: function() {
		// Summary
		//		Performs activities when save is performed.
		//		Activities like show Progress dialog [blocks screen]
		try {
			this._showProgressBar();
		} finally {
			
		}
	},
	
	_onSaveComplete: function(/*boolean*/isError, /*uri string*/filterUri, /*boolean*/ closePage) {
		// Summary
		//		Performs activities when save is completed [success, failure both].
		//		Activities like hide Progress dialog [blocks screen]
		//		On success, invokes filter evaluation, and publishes ROLE_PROPERTY_UPDATE_EVENT event.
		//		Success, Failure call-backs should be added to xhr request [filterEvaluationRequest]
		//		by the listeners of ROLE_PROPERTY_UPDATE_EVENT event for Role's filter create/edit.
		//
		// isError: boolean
		//		true represents that HTTP request should be considered failed.
		//		false represents that HTTP requests completed with success (with success codes)
		// filterUri: uri string
		//		URI of the filter.
		//		It should be used to retrieve the filter.
		// closePage: boolean
		//		Boolean indicator to close page after successful save.
		//		true - close page after save completes with success.
		//		false - Do NOT close page after save completes with success
		//				Add Success, Failure call-backs should be added to xhr request [filterEvaluationRequest] 
		//				So that evaluation status can be updated in the message area.
		//				Restrict page close, till the filter evaluation request is in progress. 
		
		var filterEvaluationRequest = null;
		try {
			if (isError === false && filterUri) {
				
				// Initiate the analysis asynchronously.
				filterEvaluationRequest = com.ibm.security.modeling.membership.FilterEvaluationUtil.startFilterEvaluation(filterUri);
				
				// If we've got a valid filter Eval Request object, go ahead with the evaluation.
				if (filterEvaluationRequest) {
					if(closePage == false) {
						var argsObject = this._getFilterEvaluationUtilArguments(filterUri, filterEvaluationRequest);
						com.ibm.security.modeling.membership.FilterEvaluationUtil.handleFilterEvaluationRequest(argsObject);
						this.filterEvaluationRequest = null;
					} else {
						this.filterEvaluationRequest = filterEvaluationRequest;
					}
					/* Collect the filterUri here. That'll be needed when we throw the event when we are actually
					 * closing the page. At this point of time closePage can be false. So, the Filter Panel Refreshes
					 * on the Membership Panel might not happen.*/
					this.filterUri = filterUri;
					this.editHappened = true;
				}
				
				// Filter exists now.
				this._updateFilterBuilderState(filterUri);
			} 
		} finally {
			this._hideProgressBar();
			this._handleCloseAfterSave(/*boolean*/ isError, /*boolean*/ closePage);
		}
	},
	
	_handleCloseAfterSave: function(/*boolean*/isError, /*boolean*/ closePage) {
		// Summary:
		//		Handles close of this page, after save completes.
		//
		// isError: boolean
		//		true represents no error occurred during save
		//		false represents error occurred during save: stay on the same page [irrespective of 'closePage']
		// closePage: boolean
		//		Boolean indicator to close page
		
		if (isError == false) {
			if (closePage) {
				this._onClose();
			} 
		}
		// else: stay on the same page.
	},
	
	_publishFilterEdit: function(/*uri string*/filterUri, /*xhr*/filterEvaluationRequest) {
		// Summary:
		//		Publishes ROLE_PROPERTY_UPDATE_EVENT event.
		//		And provides evaluation xhr object, filter uri as event data.
		//
		// filterUri: uri string
		//		URI of the filter.
		//		It should be used to retrieve the filter.
		// filterEvaluationRequest: xhr request
		//		xhr request obtained by invoking filter evaluation HTTP URI.
		//
		//		Success, Failure call-backs should be added to this xhr request
		//		by the listeners of ROLE_PROPERTY_UPDATE_EVENT event for Role's filter create/edit.
		
		this._trace.entering(this.id, "_publishFilterEdit");
		
		// Create event object
		var eventData = {};
		eventData.property = "filter";
		eventData.activity = this.isInEditMode()? "update": "create";
		eventData.data = {
			"filterEvaluationRequest" : filterEvaluationRequest,
			"filterUri": filterUri
		};
		
		dojo.publish("ROLE_PROPERTY_UPDATE_EVENT",[eventData]);
		/* Ensure that we set the Membership Panel dirty here. */
		this.parent.rolePanel._setPanelDirtyState(true, "membership");
		this._trace.exiting(this.id, "_publishFilterEdit");
	},
	
	_getFilterEvaluationUtilArguments: function(filterUri, filterEvaluationRequest) {
		// filterUri: uri string
		//		URI of the filter.
		//		It should be used to retrieve the filter.
		// filterEvaluationRequest: xhr request
		//		xhr request obtained by invoking filter evaluation HTTP URI.
		
		var saveIndicatorText = this.globalResources.SAVE_MEMBERSHIP_FILTER_CHANGE;
		
		return {
			filterUri: filterUri,
			filterEvaluationRequest: filterEvaluationRequest,
			caller: this,
			saveIndicator: this.saveIndicator,
			saveIndicatorText: saveIndicatorText,
			onFilterEvaluationStart: dojo.hitch(this, function() {}),
			onFilterEvaluationComplete: dojo.hitch(this, function() {}),
			handleRESTError: dojo.hitch(this, function(error, request) {this._handleRESTError(error, request);}),
			messageArea: this.messageArea
		};
	},
	
	_showProgressBar: function() {
		// Summary:
		//		Creates [if not created earlier] progress bar and shows that.
		if (!this._progressBar) {
			this._progressBar = new com.ibm.security.modeling.widgets.ProgressBar({
				id: this.id+"_progressBar"
			});
		}
		this._progressBar.show(this.localResources.SAVING_FILTER);
	},
	
	_showFilterEvalProgressBar: function() {
		// Summary:
		//		Creates [if not created earlier] progress bar and shows that.
		if (!this._progressBar) {
			this._progressBar = new com.ibm.security.modeling.widgets.ProgressBar({
				id: this.id+"_progressBar"
			});
		}
		this._progressBar.show(this.localResources.EVALUATING_FILTER);
	},	
	
	_hideProgressBar: function() {
		// Summary:
		//		Hides the progress bar.
		if (this._progressBar) {
			this._progressBar.hide();
			
			/* 
			 * Sometimes, hide doesn't seem to destroy the dialog. 
			 * Hence, destroying dialog explicitly. 
			 */
			this._progressBar.destroyProgressDialog(); 
		}
	},
	
	_handleRESTError: function (error, request) {
		this._trace.entering(this.id, "_handleRESTError");
		this._trace.finest(this.id + " - Error: " + error, "_handleRESTError");
		if ((this.messageArea != null) && (error != null) && (error.responseText != null)) {
			var message = error.responseText;
			if (message.search("CTJRP1170") != -1) {
					var errorMessage = this.pluginResources.PLUGIN_REST_MEMBERSHIP_FILTER_EVALUATE_MEMBERSHIP_FILTER_EVALUATION_EXCEPTION ;
					this.messageArea.showErrorMessage(errorMessage);
			}else{
				this._errorUtil.handleRESTError(error, this.messageArea);
			}
		}
		this._trace.exiting(this.id, "_handleRESTError");
	},
	
	_updateFilterBuilderState: function(/*uri string*/ filterUri) {
		// Summary
		//		Sets the filter builder internal state variables.
		//
		// filterUri: uri string
		//		Filter Uri
		
		// Update the target URI [useful when continuing on the same page (when Apply is used).]
		this._updateTargetUri(filterUri);
		
		// Update the filter builder mode [create/edit] to have next request per that. [useful when continuing on the same page (when Apply is used).]
		this._updateFilterBuilderMode(filterUri);
	},
	
	_updateFilterBuilderMode: function(/*uri string*/ filterUri) {
		// Summary:
		//		Sets the mode [create/edit] of the filter builder per the existence of filter.
		// filterUri: uri string
		//		Filter Uri
		
		this._trace.entering(this.id, "_updateFilterBuilderMode");
		
		if (this.filterData || filterUri) {
			this._setFilterBuilderModeWhenFilterExist(); // Filter data represents filter exists.
		} else {
			this._setFilterBuilderModeWhenFilterDoesNotExist();
		}
		
		this._trace.exiting(this.id, "_updateFilterBuilderMode");
	},
	
	_setFilterBuilderModeWhenFilterDoesNotExist: function() {
		//	Summary:
		//		Sets the filter builder state as 'filter does not exists' yet.
		
		this._trace.entering(this.id, "_setFilterBuilderModeWhenFilterDoesNotExist");
		this._filterBuilderMode = this.FILTER_BUILDER_CREATE_MODE;	
		this._trace.exiting(this.id, "_setFilterBuilderModeWhenFilterDoesNotExist");
	},
	
	_setFilterBuilderModeWhenFilterExist: function() {
		//	Summary:
		//		Sets the filter builder state as 'filter exists'.
		
		this._trace.entering(this.id, "_setFilterBuilderModeWhenFilterExist");
		this._filterBuilderMode = this.FILTER_BUILDER_EDIT_MODE;
		this._trace.exiting(this.id, "_setFilterBuilderModeWhenFilterExist");
	},
	
	isInEditMode: function() {
		// Summary:
		//		A query method, to know if filter builder is in edit mode
		// returns - boolean
		//		true - if filter builder is in edit mode
		//		false - otherwise.
		
		return this._filterBuilderMode == this.FILTER_BUILDER_EDIT_MODE;
	},
	
	isInCreateMode: function() {
		// Summary:
		//		A query method, to know if filter builder is in create mode
		// returns - boolean
		//		true - if filter builder is in create mode
		//		false - otherwise.
		
		return this._filterBuilderMode == this.FILTER_BUILDER_CREATE_MODE;
	},
	
	_updateTargetUri: function(targetUri) {
		// Summary:
		//		Updates the this._targetUri per the latest data of the filter
		// targetUri: uri string
		//		If provided, then this will be treated as the target URI.
		
		if (targetUri) {
			this._targetUri = targetUri;
		} else if (this.filterData && 'uri' in this.filterData && this.filterData['uri']) {
			this._targetUri = this.filterData['uri'];
		} else {
			this._targetUri = "/isrm/rest/membershipfilters";
		}
	},
	
	_getTargetUri: function() {
		return this._targetUri? this._targetUri: "/isrm/rest/membershipfilters";
	},
	
	_getHttpRequestMethod: function() {
		// Summary:
		//		returns http method [post/put] to invoke per the filter editor state.
		// returns: string
		//		'post' 	- if filter builder is in create mode.
		//		'put'	- if filter builder is in edit mode.
		//		null	- if none of the above.
		
		var httpMethod = null;
		
		if (this.isInCreateMode()) {
			httpMethod = 'post';
		} else if (this.isInEditMode()) {
			httpMethod = 'put';
		}
		
		return httpMethod;
	}
	
});