/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

� Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.membership.FilterButtonUtil");

(function() {
	
	// Declaration in global scope.
	var _fbu = com.ibm.security.modeling.membership.FilterButtonUtil = {
		updateButtonClick: function(/*widget*/ containerWidget, /*button dom Node*/ buttonNode, /*function*/ callback) {
			// Summary:
			//		Due to the contained elements' Edit view, Read view switching, 
			//		Target button [add condition, remove group, add section] changes position on onmousedown.
			//		This causes onClick handler not triggering, as onmouseup takes place somewhere else than on the button where mousedown happen.
			//		- To resolve this issue, keep track of the button position.
			//		- If button moved in between mouse down and mouse up and is not the target of mouse up event, 
			//			then click should take place [and onclick hanlder should be invoked].
			//		- If button remained at its place, but mouse is moved out of button and then mouse up took place, 
			//			then it is NOT a click, and [and onclick hanlder should NOT be invoked].
			//			
			// buttonNode: domNode
			//		Button dom node.
			// callback: function.
			//		onClick callback.
			
			var oldPosition = null;
			var hasButtonMoved = function() {
				var newPosition = dojo.position(buttonNode);
				if (oldPosition.x != newPosition.x ||
						oldPosition.y != newPosition.y) {
					
					return true; // button moved.
				}
				
				return false; // button not moved.
			};
			
			containerWidget.connect(buttonNode, "onmouseover",function (mouseOverEvent) {
				oldPosition = dojo.position(buttonNode); // update old position to the position when mouse enters.
			});
			
			containerWidget.connect(buttonNode, "onmousedown", function (mouseDownEvent) {
				if (!mouseDownEvent.ctrlKey) {
					var mouseUpConnect = containerWidget.connect(dojo.body(), "onmouseup", function(mouseUpEvent) {
						
						containerWidget.disconnect(mouseUpConnect);
						
						if (mouseDownEvent.target != mouseUpEvent.target) { // either mouse was moved to somewhere else, of button moved to some other place.
							// if button moved somewhere else, then do onClick logic.
							if (hasButtonMoved()) {
								
								callback(mouseUpEvent);
							}
							// else its the mouse which has moved; i.e. user moved it. Hence no click.
							
						}
						// else mouse down, mouse up took place on the button itself, hence onClick handler will get invoked.
						// so, nothing to do.
						
					});
				}
			});	
		}
	};
})();