/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

(c) Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.membership.FilterDialog");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Dialog");
dojo.require("dijit.form.Button");
dojo.require("com.ibm.security.modeling.dijit.layout.BorderContainer");


dojo.require("com.ibm.security.modeling.membership.Filter");
dojo.require("com.ibm.security.modeling.util._DialogUtilities");

dojo.require("com.ibm.security.modeling.widgets.MultiMessageArea");

dojo.require("com.ibm.security.modeling.util.Trace");

// NLS
dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling.membership", "FilterText");


// define the widget class
dojo.declare (
	"com.ibm.security.modeling.membership.FilterDialog", 
    [ dijit._Widget, dijit._Templated, com.ibm.security.modeling.util._DialogUtilities ],
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.membership.FilterDialog"),
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.membership","templates/FilterDialog.html"),
	widgetsInTemplate: true,
	
	// namespace: [readonly] String
	// 			Unique identifier to the instance of portlet page where this item will exist
	namespace:"",
	
	// contextRoot: [readonly] String
	//			The context root of the server from which Content is being served.
	contextRoot: "",
	
	// helpFile: [readonly] String
	//			name of the HTML help file for filter.
	helpFile: "",	
	
	// _filter: com.ibm.security.modeling.membership.Filter
	//			A filter.
	
	constructor: function() {
		// globalResources: Object
		//			Reference to the global shared resource bundle
		// localResources: Object
		//			Reference to the local shared resource bundle
		// filterData: json object
		//		An object representing a filter
		this._trace.entering("", "constructor");
		
		this._trace.object(arguments, "constructor", "Constructor args");
		this._trace.exiting("", "constructor");
	},
	
	postMixInProperties: function() {
		// Summary:
		//		Widget life-cycle method. 
		//		Sets Dialog's height, width related parameters. And NLS resources.
		
		this._trace.entering(this.id, "postMixInProperties");
		
		this.inherited(arguments);
		
		// dialog size
		this.heightFactor=1.2;
		this.widthFactor=1.2;
		this.minHeight=300;
		this.minWidth=400;
		
		if (!this.localResources) {
			this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.membership", "FilterText");
		}
		
		this._trace.exiting(this.id, "postMixInProperties");
	},
	buildRendering: function(){
		this.dialogInstruction="";
		this.dialogTitle=this.localResources.TITLE;
		this.inherited(arguments);
	},
	postCreate: function() { 
		// Summary:
		//		Widget life-cycle method. 
		//		Creates filter.
		this._trace.entering(this.id, "postCreate");
		
		this.inherited(arguments);
		
		this._filter = new com.ibm.security.modeling.membership.Filter({
			id: this.id+"_filter",
			filterData: this.filterData,
			filterBody: this.filterBodyContent,
			globalResources: this.globalResources,
			localResources: this.localResources,
			messageArea: this.messageArea
		});
		this.okButton = this.addCommonButton({
			onClick:dojo.hitch(this, this._onAddGroup),
			type:"OTHER",
			label:this.localResources.ADD_SECTION
		});
		this._trace.exiting(this.id, "postCreate");
	},
	
	startup: function() {
		// Summary:
		//		Widget life-cycle method.
		//		Starts up the widget, its dialog, and filter inside it.
		this._trace.entering(this.id, "startup");
		
		//this._filter.startup();
		
		this.inherited(arguments);
		this._trace.exiting(this.id, "startup");
	},
	
	destroy: function() {
		//	Summary:
		//		Widget life-cycle method.
		//		Destroys dialog and is content (filter) recursively.
		
		this._trace.enteringWithStack(this.id, "destroy", true);
		try {
			if (this._filter) {
				this._filter.destroyRecursive();
			}
			
			this.inherited(arguments);
			
			this._trace.exiting(this.id, "destroy");
		} catch(e) {
			this._trace.exception(e, "destroy");
			throw e;
		}
	},
	
	show: function () {
		// Summary:
		//		Displays dialog widget, and renders filter inside that.
		
		this._trace.entering(this.id, "show");
		this._filter.render();
		this.inherited(arguments);
		this._trace.exiting(this.id, "show");
	},
	
	_onAddGroup: function () {
		// Summary:
		//		Invoked when dialog's add button is clicked.
		//		Adds an empty group to filter.
		try {
			this._trace.entering(this.id, "_onAddGroup");
			this._filter.addGroup();
		} finally {
			this._trace.exiting(this.id, "_onAddGroup");
		}
	}
});