/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

(c) Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.membership.FilterGroup");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("ibm.tivoli.tip.dijit.TextInputBox");

dojo.require("com.ibm.security.modeling.membership.FilterButtonUtil");
dojo.require("com.ibm.security.modeling.membership.FilterData");
dojo.require("com.ibm.security.modeling.membership._ElementContainer");
dojo.require("com.ibm.security.modeling.membership.SynchronizedSelector");
dojo.require("com.ibm.security.modeling.membership.FilterGroupName");
dojo.require("com.ibm.security.modeling.membership.EditableElement");
dojo.require("com.ibm.security.modeling.membership.FilterGroupPhrase");

dojo.require("com.ibm.security.modeling.util.Trace");


// define the widget class
dojo.declare (
	"com.ibm.security.modeling.membership.FilterGroup", 
    [ com.ibm.security.modeling.membership.EditableElement, 
      dijit._Templated,
      com.ibm.security.modeling.membership._ElementContainer],
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.membership.FilterGroup"),
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.membership","templates/FilterGroup.html"),
	widgetsInTemplate: true,
	
	_maxPhraseLimit: 10,
	
	constructor: function () {
		this._trace.entering("", "constructor");
		// globalResources: Object
		//			Reference to the global shared resource bundle
		// pluginResources: object
		//			Reference to the global shared plugin resource bundle
		// localResources: Object
		//			Reference to the resource bundle unique to this widget
		// messageArea: ibm.tivoli.tip.dijit.TIPMessageArea
		//		This is to display messages on GUI when any error occurs.
		// data: object
		//		An object representing a group
		// filter:	com.ibm.security.modeling.membership.Filter
		//		Controller object, representing a filter.
		// attributeStore: dojox.data.JsonRestStore
		//		common store for attribute dropdown in filter group phrase.
		// attributeOperatorStore: dojox.data.JsonRestStore
		//		common store for attribute operator dropdown in filter group phrase.
		
		this._trace.exiting("", "constructor");
	},
	
	postMixInProperties: function() {
		this._trace.entering(this.id, "postMixInProperties");
		
		this.inherited (arguments);

		this._elementType = "phrase";
		
		// CSS class for logical operator between any two phrases.
		this.operatorEditNodeClass = "dojoxGridRowSelected elementPadding";
		this.operatorReadNodeClass = "dojoxGridRowOdd dojoxGridCell";
		
		this._trace.exiting(this.id, "postMixInProperties");
	},
	
	postCreate: function() {
		this._trace.entering(this.id, "postCreate");		
		
		this.inherited (arguments);
		
		// while editing.
		this.populateGroupName(this.data);
		
		if(dojo.isIE) {
			this.updatePageStyle();
		}
		
		this._setupButtons();
		
		this._trace.exiting(this.id, "postCreate");		
	},
	
	_setupButtons: function() {
		// Summary:
		//		Updates the button click behaviour.
		
		com.ibm.security.modeling.membership.FilterButtonUtil.updateButtonClick(
			this, 
			this._removeGroupButton.domNode, 
			dojo.hitch(this, function(event) {
				this._onRemoveGroup(event);
			})
		);
		
		com.ibm.security.modeling.membership.FilterButtonUtil.updateButtonClick(
			this, 
			this._addPhraseButton.domNode, 
			dojo.hitch(this, function(event) {
				this._addNewPhrase(event);
			})
		);
	},
	
	startup: function() {
		this._trace.entering(this.id, "startup");
		
		this.inherited(arguments);
		
		// After startup, it is sure that dojoAttachPoint is attached to DOM for addClass, removeClass operations to work.
		if (this.isEmpty()) {
			this._showEmptyContainerInstructions();
		} else {
			this._hideEmptyContainerInstructions();
		}
		
		this._trace.exiting(this.id, "startup");
	},
	
	populateGroupName: function(/*groupData*/ groupData) {
		// Summary:
		//		Populates group name with the value from groupData
		// groupData: json object
		//		A group from REST.
		// 		If it is not supplied then, this.data is considered to get name value.
		var group = groupData ? groupData: (this.data? this.data: null);
		var name = com.ibm.security.modeling.membership.FilterData.getFilterGroupName(group);
		this.groupNameInputWidget.setValue(name);
	},
	
	updatePageStyle: function() {
		this._trace.entering(this.id, "updatePageStyle");
		
		dojo.addClass(this._removeGroupButton.titleNode, "removeGroupIconContainer");
		
		this._trace.exiting(this.id, "updatePageStyle");		
	},
	
	getJsonFromUI: function() {
		// Summary:
		//		constructs a JSON representation from UI elements. 
		// returns:
		//		null if this container is empty
		//		json object otherwise.
		
		try {
			this._trace.entering(this.id, "getJsonFromUI");
			var jsonObj = null;
			
			// json object per element container
			jsonObj = this.inherited (arguments);
			
			var nameJson = this.groupNameInputWidget.getJson();
			if (nameJson) {
				if (!jsonObj) { 
					jsonObj = {}; // A group with no phrase can be saved.
				}
				// Add group name json
				jsonObj["name"] = nameJson;
			} 
			
			return jsonObj;
		} finally {
			this._trace.exiting(this.id, "getJsonFromUI");
		}
	},
	
	_getElementsJsonFromRestData: function(/*json object*/ groupData) {
		// Summary: 
		//		Extracts Elements (phrases) from the json object from REST.
		// tags: overridden from _ElementContainer.js
		// groupData: json object
		//		Represents details about the container (group), and its elements (phrases).
		//
		// returns: Array
		return com.ibm.security.modeling.membership.FilterData.getFilterGroupPhrases(groupData);
	},
	
	_getElementOperatorFromRestData: function(/*json object*/ groupData) {
		// Summary: 
		//		Extracts Elements operator (phrase operator) from the json object from REST.
		// tags: overridden from _ElementContainer.js
		// groupData: json object
		//		Represents details about the container (group), and its elements (phrases).
		//
		// returns: string
		var operator = null;
		if (groupData) {
			operator = com.ibm.security.modeling.membership.FilterData.getFilterGroupPhraseOperator(groupData);
		} 
		
		if (!operator) {
			operator = this._getSynchronizedSelectorDefaultValue();
		}
		
		return operator;
	},
	
	_getSynchronizedSelectorDefaultValue: function() {
		return 'and';
	},
	
	// overridden.
	createElement: function(elementId, data) {
		// Summary:
		//		Creates contained elements (phrase) widget's instance.
		// tags:	overridden from _ElementContainer.js
		//		
		// returns: com.ibm.security.modeling.membership.FilterGroupPhrase
		return new com.ibm.security.modeling.membership.FilterGroupPhrase({
			id: elementId,
			phraseData: data,
			filterGroup: this,
			globalResources: this.globalResources,
			localResources: this.localResources,
			attributeStore: this.attributeStore,
			attributeOperatorStore: this.attributeOperatorStore
		});
	},

	_addNewPhrase: function(event) {
		// summary:
		//		Adds a new phrase to the group.
		//		Consider the maximum phrase limit into consideration.
		//		And does not add a new phrase if limit has reached.
		//		returns true: if a phrase is added
		//				false: otherwise.
		// event:
		//		Normalized dijit event
		var isAdded = false;
		try {
			this._trace.entering(this.id, "_addNewPhrase");
			
			if (this._isMaxPhraseLimitReached()) {
				// this code should never execute, because on limit all phrase's Add Phrase buttons gets disabled.
				// Still checking it, if someone tries to invoke this method from somewhere else. 
				this._trace.finest(this.id+" - Max phrase inside a group limit reached. Not adding phrase.", "_addNewPhrase");
				
				return isAdded;	// control goes to finally.
			}
			
			// inherited.
			var phrase = this.addElement();
			phrase.focus(/*forced*/ true);
			
			isAdded = true;
		} finally {
			this._onAfterAddPhrase(isAdded);
			this._trace.exiting(this.id, "_addNewPhrase");
		}
		return isAdded;
	},
	
	removePhrase: function(phrase) {
		// Summary:
		//		Removes phrase from the group.
		//		Please see inherited removeElement method.
		//		
		//		After remove is done, it runs the logic per max limit of phrases in a group.
		this._trace.entering(this.id, "removePhrase");
		
		var isRemoved = false;
		try {
			isRemoved = this.removeElement(phrase);// inherited
		} finally {
			this._onAfterRemovePhrase(isRemoved);
			this._trace.exiting(this.id, "removePhrase");
		}
	},
	
	
	empty: function() {
		// Summary:
		//		Empties this group.
		//		see inherited empty method for more details.
		//		
		//		An empty group has no name, and an empty phrase
		
		this._trace.entering(this.id, "empty");
		
		// Make name field empty.
		this.groupNameInputWidget.empty();
		
		// inherited from _ElementContainer.
		this.inherited(arguments);
		
		this._trace.exiting(this.id, "empty");
	},
	
	isEmpty: function() {
		// Summary:
		//		Empty represents that group has no phrase 
		
		if (this._activeElementCounter > 0) {
			return false;
		} else {
			return true;
		}
		
	},
	
	isBlank: function() {
		// Summary:
		//		Blank represents that:
		//		 	all phrases are empty.
		//			and group name does not matter.
		
		var isBlank = true;
		for (var phraseId in this._elements) {
			isBlank = isBlank && this._elements[phraseId].isEmpty();
			if (!isBlank) {
				break;
			}
		}
		
		return isBlank;// && this.groupNameInputWidget.isEmpty();
	},
	
	_onAfterAddPhrase: function(/*boolean*/isAdded) {
		// summary:
		//		What to do after if could not add phrase: Nothing as yet.
		//		What to do if add was successful:
		//			Lets but a limit in of 10 phrases per group. [this._maxPhraseLimit]
		//			When we reach the limit we would disable all the add icons 
		//			and the tool tip of the disabled button should indicate that a limit has been reached.
		
		if (isAdded === true && this._isMaxPhraseLimitReached()) {
			var message = this.pluginResources.GROUP_PHRASE_LIMIT_EXCEEDED.replace("{0}", this._maxPhraseLimit);
			
			// Notify every phrase to update its 'Add Phrase' button.
			for (var phraseId in this._elements) { 
				this._updateAddPhraseButton(false, message);	// false: disable
			}
		} 
		//else {
			// do nothing yet. 
		//}
	},
	
	_onAfterRemovePhrase: function(/*boolean*/isRemoved) {
		// summary:
		//		What to do after, if could not remove phrase: Nothing as yet.
		//		What to do if remove was successful:
		//			Lets but a limit in of 10 phrases per group. [this._maxPhraseLimit]
		//			When we are in the limit we would enable all the add icons 
		//			and the tool tip of the disabled button should indicate that a limit has been reached.
		
		if (isRemoved === true && this._isMaxPhraseLimitButOne()) {
			// Notify every phrase to update its 'Add Phrase' button.
			for (var phraseId in this._elements) { 
				this._updateAddPhraseButton(true);	// true: enable
			}
		} 
		//else {
			// do nothing yet. 
		//}
	},
	
	_updateAddPhraseButton: function(/*boolean*/ shallEnable, /*string?*/ label) {
		// Summary:
		//		Enables/Disables the 'Add Phrase' button on phrase.
		// shallEnable:
		//		true: enable the 'Add Phrase' button on phrase.
		//		false: disable the 'Add Phrase' button on phrase.
		// label?:
		//		New label for the button
		//		
		if (shallEnable === true) {
			
			this._enableButton(this._addPhraseButton, (label? label: this.localResources.CONDITION));
		} else {
			
			this._disableButton(this._addPhraseButton, (label));
		}
		
	},
	
	_isMaxPhraseLimitReached: function() {
		// Summary
		//		Returns true if present number of phrases inside this group is equal to maximum limit.
		//		false otherwise.
		return this._activeElementCounter == this._maxPhraseLimit;
	},
	
	_isMaxPhraseLimitButOne: function() {
		// Summary:
		//		Returns true if present number of phrases inside this group is one less than the maximum limit.
		//		false otherwise.
		return this._activeElementCounter == (this._maxPhraseLimit -1);
	},
	
	_onStartEditing: function() {
		// Summary:
		//		Executes when edit icon/button (pencil) is clicked.
		//		Focuses itself [which intern renders itself in edit mode]
		
		this._trace.entering(this.id, "_onStartEditing");
		if (this.isEmpty()) {
			this.inherited(arguments);
		} else {
			// Focus the first element [phrase/condition].
			this._firstElement.focus();
		}
		this._trace.exiting(this.id, "_onStartEditing");
	},
	
	_onRemoveGroup: function(event) {
		// Summary:
		//		Executes when remove group icon/button is clicked.
		//		Removes the group from filter.
		// event
		//		Normalized dijit event.
		this._trace.entering(this.id, "_onRemoveGroup");
		this.filter.removeGroup(this);
		this._trace.exiting(this.id, "_onRemoveGroup");
	},
	
	displayInReadMode: function(/*boolean*/ forced) {
		// Summary:
		//		Displays this group in read mode.
		// forced: boolean
		//		true: 	run logic of displaying in read mode, 
		//				even if group is currently in read mode.
		//		false:	do not run logic of displaying in read mode, 
		//				if group is currently in read mode.
		
		this._trace.entering(this.id, "displayInReadMode");
		
		if (! this.isInEditMode() && !forced) {
			// not in edit mode: already in read mode.
			this._trace.exiting(this.id+" - alreay in read mode.", "displayInReadMode");
			return;
		}
		
		this.inherited(arguments);
		
		// show name is readonly mode
		this.groupNameInputWidget.displayInReadMode();
		
		// show all phrase operators in read mode.
		this._synchronizedSelector.displayInReadMode();
		// show all phrases in read mode
		for (var phraseId in this._elements) {
			this._elements[phraseId].displayInReadMode();
		}
		
		
		if (this.isEmpty()) {// No Body
			dojo.removeClass(this.sectionBody, "show");
			dojo.addClass(this.sectionBody, "hide");
		}
		// Hide the add phrase icon
		dojo.removeClass(this._addPhraseContainer, "show");
		dojo.addClass(this._addPhraseContainer, "hide");
		
		// When group is in read mode, only one pencil [to indicate click to edit] is shown.
		this.showClickToEditPencil();
		this.groupNameInputWidget.hideClickToEditPencil();
		for (var phraseId in this._elements) {
			this._elements[phraseId].hideClickToEditPencil();
		}
		var phraseOperator = null;
		for (var phraseId in this._elementOperators) {
			phraseOperator = this._elementOperators[phraseId];
			//if (phraseOperator) {
				phraseOperator.hideClickToEditPencil();
			//}
		}
		
		dojo.removeClass(this.container, "sectionDataContainerSelected");
		dojo.addClass(this.container, "sectionDataContainer");
		
		this._trace.exiting(this.id, "displayInReadMode");
	},
	
	displayInEditMode: function(/*boolean*/ forced) {
		// Summary:
		//		Displays this group in edit mode.
		// forced: boolean
		//		true: 	run logic of displaying in edit mode, 
		//				even if group is currently in edit mode.
		//		false:	do not run logic of displaying in edit mode, 
		//				if group is currently in edit mode.
		
		this._trace.entering(this.id, "displayInEditMode");
		if (this.isInEditMode() && !forced) {
			// not in edit mode: already in read mode.
			this._trace.exiting(this.id+" - alreay in edit mode.", "displayInEditMode");
			return;
		}
		
		this.inherited(arguments);
		
		// Body
		dojo.removeClass(this.sectionBody, "hide");
		dojo.addClass(this.sectionBody, "show");
	
		// Show the add phrase icon
		dojo.removeClass(this._addPhraseContainer, "hide");
		dojo.addClass(this._addPhraseContainer, "show");
		
		this.hideClickToEditPencil();
		this.groupNameInputWidget.showClickToEditPencil();
		for (var phraseId in this._elements) {
			this._elements[phraseId].showClickToEditPencil();
		}
		var phraseOperator = null;
		for (var phraseId in this._elementOperators) {
			phraseOperator = this._elementOperators[phraseId];
			//if (phraseOperator) {
				phraseOperator.showClickToEditPencil();
			//}
		}
		/*
		// Empty group displays its only phrase in edit mode. 
		if (this.isEmpty()) {
			// show first phrase in edit mode.
			this._firstElement.displayInEditMode();
		}
		*/
		
		// Accordion CSS
		dojo.removeClass(this.container, "sectionDataContainer");
		dojo.addClass(this.container, "sectionDataContainerSelected");
		
		this._trace.exiting(this.id, "displayInEditMode");
	}
	
});
