/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

(c) Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.membership.FilterGroupName");

dojo.require("com.ibm.security.modeling.membership.EditableElement");
dojo.require("com.ibm.security.modeling.util.ValidationUtilities");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");
dojo.require("dijit._Templated");

// dojo.require("ibm.tivoli.tip.dijit.TextInputBox");
dojo.require("dijit.form.TextBox");

dojo.require("com.ibm.security.modeling.util.Trace");

// NLS
dojo.requireLocalization("com.ibm.security.modeling.membership", "FilterText");

// define the widget class
dojo.declare (
	"com.ibm.security.modeling.membership.FilterGroupName", 
    [com.ibm.security.modeling.membership.EditableElement, dijit._Templated ],
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.membership.FilterGroupName"),
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.membership","templates/FilterGroupName.html"),
	validator : new com.ibm.security.modeling.util.Validator(),
	widgetsInTemplate: true,
	
	// nameValue: string
	//		Value that should be assigned as name value.
	//		Useful when an existing group is opened for editing.
	value: "",
	
	// Text to be displayed while in read mode.
	_nameText: "",
	attributeMap: {
		_nameText: {node: "_nameTextNode", type: "innerHTML"},
		id:"", dir:"", lang:"", "class":"", style:"", title:""
	},
	
	postMixInProperties: function() {
		this._trace.entering(this.id, "postMixInProperties");
		
		this.inherited(arguments);
		if (!this.localResources) {
			this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.membership", "FilterText");
		}
		
		this._trace.exiting(this.id, "postMixInProperties");
	},
	
	displayInReadMode: function(forced) {
		// Summary:
		//		Displays the current widget in read mode
		//		Updates the text in the read mode DOM elements 
		//		per the current values from Edit mode DOM elements.
		//		
		//		Overrides _EditableElement.displayInReadMode().
		
		this._trace.entering(this.id, "displayInReadMode");
		
		if (! this.isInEditMode() && !forced) {
			// not in edit mode: already in read mode.
			this._trace.exiting(this.id+" - alreay in read mode.", "displayInReadMode");
			return;
		}
		
		this.inherited(arguments);
		
		// Update the displayed name text.
		this._updateNameText();
		
		this._trace.exiting(this.id, "displayInReadMode");
	},
	
	_updateNameText: function() {
		// Summary:
		//		Updates the name text in DOM, per the values entered.
		//		If name field is empty [see isEmpty()] 
		//		then default name [see _getDefautlName()] is considered for display.
		
		// Display read only text
		var nameText = this._getDefautlName();
		
		if (! this.isEmpty()) {
			nameText = this.getValue();
			nameText = com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(nameText);
			nameText = this._stringUtils.breakString(nameText, 80);
		}
		
		// Populate DOM with the text.
		this.set("_nameText", nameText);
	},
	
	getValue: function() {
		return this.groupNameInput.get('value');
	},
	
	setValue: function(value) {
		this.groupNameInput.set('value', value);
	},
	
	empty: function() {
		this._trace.entering(this.id, "isEmpty");
		this.setValue("");
		this._trace.exiting(this.id, "isEmpty");
	},
	
	isEmpty: function() {
		// Summary:
		//		Returns true if value is still the same as default [Not changed].
		//		
		this._trace.entering(this.id, "isEmpty");
		var name = this.getValue();
		if (!name || name == null || name == "") {
			this._trace.exiting(this.id+" - true", "isEmpty");
			return true;
		}
		this._trace.exiting(this.id+" - false", "isEmpty");
		return false;
	},
	
	getJson: function() {
		// Summary:
		//		constructs a JSON representation from UI element. 
		// returns:
		//		Default name, if value is not set yet.
		//		Else the value.
		
		try {
			this._trace.entering(this.id, "getJson");
			var jsonObj = null;
			
			if (this.isEmpty()) {
				jsonObj = this._getDefautlName();
			} else {
				jsonObj = this.getValue()+"";
			}
			return jsonObj;
		} finally {
			this._trace.exiting(this.id, "getJson");
		}
	},
	
	_getDefautlName: function() {
		return this.localResources.NEW_SECTION_NAME;
	}
});
