/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

(c) Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.membership.FilterGroupPhrase");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dijit.form.Select");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");

dojo.require("com.ibm.security.modeling.membership.FilterData");
dojo.require("com.ibm.security.modeling.membership.EditableElement");
dojo.require("com.ibm.security.modeling.membership.FilterGroupPhraseValueController");

// NLS
dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling.membership", "FilterText");
dojo.requireLocalization("com.ibm.security.modeling","PluginMessages");

// define the widget class
dojo.declare (
	"com.ibm.security.modeling.membership.FilterGroupPhrase", 
    [ com.ibm.security.modeling.membership.EditableElement, dijit._Templated ],
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.membership.FilterGroupPhrase"),
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.membership","templates/FilterGroupPhrase.html"),
	widgetsInTemplate: true,
	
	defaultAttributeValue: null,
	defaultAttributeOperatorValue: 'equals',
	defaultValues: null,
	
	_emptyAttribute: " ",
	_emptyAttributeValue: null,
	
	_emptyAttributeOperator: "invalidtype",
	_emptyAttributeOperatorValue: null,
	
	_attributeSelectedItem: null,
	_attributeOperatorSelectedItem: null,
	
	// _valueWidget: defined in template
	//			An instance of com.ibm.security.modeling.membership.FilterGroupPhraseValueController
	
	// Text to be displayed while in read mode.
	_attributeText: "",
	_operatorText: "",
	_valuesText: "",
	
	attributeMap: {
		_attributeText: {node: "_attributeTextNode", type: "innerHTML"},
		_operatorText: {node: "_attributeOperatorTextNode", type: "innerHTML"},
		_valuesText: {node: "_attributeValuesTextNode", type: "innerHTML"},
		id:"", dir:"", lang:"", "class":"", style:"", title:""
	},
	
	constructor: function () {
		this._trace.entering("", "constructor");
		// globalResources: Object
		//			Reference to the global shared resource bundle
		// pluginResources: object
		//			Reference to the global shared plugin resource bundle
		// localResources: Object
		//			Reference to the resource bundle unique to this widget
		// phraseData: object
		//		An object representing a phrase
		// attributeStore: dojox.data.JsonRestStore
		//		common store for attribute dropdown in filter group phrase.
		// attributeOperatorStore: dojox.data.JsonRestStore
		//		common store for attribute operator dropdown in filter group phrase.
		
		// filterGroup: com.ibm.security.modeling.membership.FilterGroup
		//		reference to the Filter Group, containing this phrase.
		this._trace.exiting("", "constructor");
	},
	
	postMixInProperties: function() {
		this._trace.entering(this.id, "postMixInProperties");
		
		this.inherited (arguments);
		
		if (!this.globalResources) {
			this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		}
		
		if (!this.pluginResources) {
			this.pluginResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "PluginMessages");
		}
		
		if (!this.localResources) {
			this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.membership", "FilterText");
		}
		
		this._trace.exiting(this.id, "postMixInProperties");
	},

	_tuneWidgetsBiggerWidth: function() {
		this._trace.entering(this.id, "_tuneWidgetsBiggerWidth");
		dojo.removeClass(this.removePhraseDiv, "phraseActionIconContainer");
		dojo.removeClass(this.removePhraseLabel, "phraseActionIconContainer");
		dojo.removeClass(this.removePhraseButton, "phraseActionIconContainer");
		dojo.addClass(this.removePhraseDiv, "phraseActionIconContainerBiggerWidth");
		dojo.addClass(this.removePhraseLabel, "phraseActionIconContainerBiggerWidth");
		dojo.addClass(this.removePhraseButton, "phraseActionIconContainerBiggerWidth");
		this._trace.exiting(this.id, "_tuneWidgetsBiggerWidth");
	},
	
	_tuneWidgetsSmallerWidth: function() {
		this._trace.entering(this.id, "_tuneWidgetsSmallerWidth");
		dojo.removeClass(this.removePhraseDiv, "phraseActionIconContainerBiggerWidth");
		dojo.removeClass(this.removePhraseLabel, "phraseActionIconContainerBiggerWidth");
		dojo.removeClass(this.removePhraseButton, "phraseActionIconContainerBiggerWidth");
		dojo.addClass(this.removePhraseDiv, "phraseActionIconContainer");
		dojo.addClass(this.removePhraseLabel, "phraseActionIconContainer");
		dojo.addClass(this.removePhraseButton, "phraseActionIconContainer");
		this._trace.exiting(this.id, "_tuneWidgetsSmallerWidth");
	},
	
	_tuneWidgets: function() {
		this._trace.entering(this.id, "_tuneWidgets");
		
		/* We cannot keep this locale based, since that would form a big condition list and things 
		 * will start looking messy. Comparing on string length seemed most reliable. 
		 * 
		 * If this approach is not followed then the alignment of the Remove Condition label & 
		 * field is broken. 
		 * */
		if(this.localResources.LABEL_REMOVE_CONDITION.length <= 20) {
			this._tuneWidgetsSmallerWidth();
		} else {
			this._tuneWidgetsBiggerWidth();
		}

		this._trace.exiting(this.id, "_tuneWidgets");
	},
	
	postCreate: function () {
		this._trace.entering(this.id, "postCreate");
		this.inherited (arguments);
		this._tuneWidgets();
		this._trace.exiting(this.id, "postCreate");
	},
	
	startup: function() {
		this._trace.entering(this.id, "startup");
		
		//this._valueWidget.startup();
		
		// Operator
		this._setupAttributeOperatorSelector();
		this._setupAttributeSelector();
		
		// update _focusNode
		this.setFocusNode(this.attributeSelector);
		
		// startup child widgets.
		//this.attributeOperatorSelector.startup();
		//this.attributeSelector.startup();
		
		// startup itself.
		this.inherited(arguments);
		
		if (this.phraseData) { // while editing.
			this._populateFromRestData(this.phraseData);
		}
		
		this._trace.exiting(this.id, "startup");
	},
	
	getJsonFromUI: function() {
		// Summary:
		//		Constructs a JSON representation from UI elements representing a phrase.
		// returns:
		//		null - if no phrase value is selected/specified or phrase value is invalid
		//		json object - containing attribute information, operator, values items array
		try {
			this._trace.entering(this.id, "getJsonFromUI");
			
			var jsonObj = null;
			if (!this.isEmpty()) {
				jsonObj = {};
				jsonObj["attribute"] = {
					item: this._attributeSelectedItem,
					idAttribute: "uri"
				};
				jsonObj["operator"] = {
					item: this._attributeOperatorSelectedItem,
					idAttribute: "value"
				};
				
				jsonObj["values"] = this._valueWidget.getJson();
			}
			return jsonObj;
		} finally {
			this._trace.exiting(this.id, "getJsonFromUI");
		}
	},
	
	_populateFromRestData: function(/*json object*/ data) {
		// Summary:
		//		Populates this phrase with the data from Rest response [data]. 
		//		
		// data: json object
		//		Represents details about the phrase
		
		this._trace.entering(this.id, "_populateFromRestData");
		if (data) {
			this.defaultAttributeValue = com.ibm.security.modeling.membership.FilterData.getAttributeNameIdentifier(data);
			this.defaultAttributeOperatorValue = com.ibm.security.modeling.membership.FilterData.getAttributeOperatorIdentifier(data);
			this.defaultValues = com.ibm.security.modeling.membership.FilterData.getAttributeValuesIdentifiers(data);
			
			// Populate Attribute and other data.
			if (this.defaultAttributeValue) {
				this._trace.finest(this.id, "defaultAttributeValue: "+this.defaultAttributeValue);
				this.attributeSelector.setStore(this.attributeStore, this.defaultAttributeValue);
			}
		}
		this._trace.exiting(this.id, "_populateFromRestData");
	},
	
	_setupAttributeSelector: function() {
		this._trace.entering(this.id, "_setupAttributeSelector");
		
		// this._emptyAttributeValue will be used to reset attribute filter.
		this.attributeStore.fetch({
			query: this._getAttributeQuery(this._emptyAttribute),	// query to fetch the attribute that has " " as its name [blank]
			onComplete: function(items) {
				if (items && items.length > 0) {
					this._emptyAttributeValue = this.attributeStore.getIdentity(items[0]);
				}
				this._trace.finest(this.id, "_emptyAttributeValue: "+this._emptyAttributeValue);
				this.attributeSelector.setStore(this.attributeStore, this._emptyAttributeValue);
			},
			scope: this
		});
		
		// need to track down menu table using xPath while testing
		this.attributeSelector.dropDown.menuTableNode.id = this.attributeSelector.id+"_menu_table";
		
		this._updateSelectorStyle(this.attributeSelector);

		this.connect(this.attributeSelector.dropDown, "_onKeyPress", dojo.hitch(this.attributeSelector.dropDown, this._selectorDropDownOnKeyPress));
		this.connect(this.attributeSelector, "onChange", this._onAttributeChange);
		
		this._trace.exiting(this.id, "_setupAttributeSelector");
	},
	
	_updateSelectorStyle: function(/*selector widget*/ selector) {
		if (!selector) {
			return;
		}
		
		if (!dojo.isIE) {
			selector.forceWidth = true;
		}
		selector.maxHeight = "200"; //px;
		this.connect(selector.dropDown, "resize", dojo.hitch(this, function() {
			dojo.style(selector.dropDown.domNode, {overflowX: "auto"});
		}));
	},
	
	_setupAttributeOperatorSelector: function() {
		this._trace.entering(this.id, "_setupAttributeOperatorSelector");
		// this._emptyAttributeOperatorValue will be used to reset attribute operator filter.
		this.attributeOperatorStore.fetch({
			query: this._getAttributeOperatorQuery(this._emptyAttributeOperator),
			onComplete: function(items) {
				if (items && items.length > 0) {
					this._emptyAttributeOperatorValue = this.attributeOperatorStore.getIdentity(items[0]);
				}
				this._trace.finest(this.id, "_emptyAttributeOperatorValue: "+this._emptyAttributeOperatorValue);
				this.attributeOperatorSelector.setStore(this.attributeOperatorStore, this._emptyAttributeOperatorValue);
			},
			scope: this	
		});
		
		// need to track down menu table using xPath while testing
		this.attributeOperatorSelector.dropDown.menuTableNode.id = this.attributeOperatorSelector.id+"_menu_table";
		
		this._updateSelectorStyle(this.attributeOperatorSelector);
		
		this.connect(this.attributeOperatorSelector.dropDown, "_onKeyPress", dojo.hitch(this.attributeOperatorSelector.dropDown, this._selectorDropDownOnKeyPress));
		this.connect(this.attributeOperatorSelector, "onChange", this._onAttributeOperatorChange);
		
		this._trace.exiting(this.id, "_setupAttributeOperatorSelector");
	},
	
	empty: function() {
		this._trace.entering(this.id, "empty");
		// setting attribute selector to blank makes attribute operator to clear  
		// and clearing attribute operator clears the description (value) field 
		// and makes 'select value' anchor disabled.
		
		this._resetAttributeSelector();
		
		this._trace.exiting(this.id, "empty");
	},
	
	isEmpty: function() {
		// Summary:
		//		To check if the phrase is empty.
		//		A phrase is considered empty, till no value/invalid value is selected/provided.
		
		this._trace.entering(this.id, "isEmpty");
		var empty = true;
		try {
			empty = this._isPhraseAttributeEmpty();
			if (empty == true) {
				// unset  attribute selector represents that the phrase is empty.
				return empty;
			}
			
			// attribute is selected, but operator could be empty
			// Check if any operator is selected.
			empty = this._isPhraseAttributeOperatorEmpty();
			if (empty == true) {
				// unset  operator selector represents that the phrase is empty.
				return empty;
			}
			
			// check if the value is empty
			empty = this._valueWidget.isEmpty();
			return empty;
		} finally {
			this._trace.exiting(this.id+" - empty: "+empty, "isEmpty");
		}
	},
	
	_onRemovePhrase: function() {
		this._trace.entering(this.id, "_onRemovePhrase");
		this.filterGroup.removePhrase(this);
		// remove operator selector.
		this._trace.exiting(this.id, "_onRemovePhrase");
	},
	
	_onAttributeChange: function(value) {
		// Summary:
		//		When selected phrase attribute changes, then 
		//		-	Corresponding phrase value widget [Integer|String|IdentityOrHierarchy] should be displayed
		//		-	And attribute operator list should be reset to default value, 
		//			which intern will reset the value widget to default state [empty].
		
		this._trace.entering(this.id+" - value: "+value, "_onAttributeChange");
		this.attributeStore.fetchItemByIdentity({
			identity: value,
			onItem: function(item) { // selected attribute item
				this._attributeSelectedItem = item;
				
				// display phrase value widget per the selected attribute type.
				this._renderValueWidget(item, this.defaultValues);
				this.defaultValues = null;
				
				// Enable, disable the attribute operator list.
				if (this._isPhraseAttributeEmpty() === true) {
					// reset the attribute operator list.
					this._resetAttributeOperatorSelector(item["type"]);
					
					// if no value is selected, then disable the attribute operator drop down.
					this.attributeOperatorSelector.set('disabled', true);
				} else {
					// else enable the attribute operator drop down.
					this.attributeOperatorSelector.set('disabled', false);
					
					// Populate Attribute Operator
					if (this.defaultAttributeOperatorValue) {
						var valueToSelect = this.defaultAttributeOperatorValue;
						this._resetAttributeOperatorSelector(item["type"], valueToSelect);
						this._resetDefaultAttributeOperatorValue();
					}
				}
				
				// update phrase text
				this._updatePhraseText();
			},
			scope: this
		});
		this._trace.exiting(this.id, "_onAttributeChange");
	},
	
	_resetDefaultAttributeOperatorValue: function() {
		this._trace.entering(this.id, "_resetDefaultAttributeOperatorValue");
		this.defaultAttributeOperatorValue = 'equals';
		this._trace.exiting(this.id, "_resetDefaultAttributeOperatorValue");
	},
	
	_renderValueWidget: function(/*json object*/ attribute, defaultValues) {
		// Summary:
		//		Passes control to FilterGroupPhraseValueController widget via _valueWidget [dojoAttachPoint from phrase template.]
		//	attribute: json object
		//		Attribute item
		// defaultValues: array
		//		default values [while editing].
		
		this._trace.entering(this.id, "_renderValueWidget");
		this._valueWidget.render(/*type*/ attribute["type"].toString(), /*data*/ attribute, /*defaultValues*/ defaultValues);
		
		this._adjustLayout();
		
		this._trace.exiting(this.id, "_renderValueWidget");
	},
	
	_resetAttributeOperatorSelector: function(type, valueToSelect) {
		this._trace.entering(this.id+" - ", "_resetAttributeOperatorSelector");
		if (!valueToSelect) {
			valueToSelect = this._emptyAttributeOperatorValue;
		}
		this.attributeOperatorSelector.setStore(this.attributeOperatorStore, valueToSelect, {query: this._getAttributeOperatorQuery(type)});
		
		this._trace.entering(this.id, "_resetAttributeOperatorSelector");
	},
	
	_resetAttributeSelector: function() {
		this._trace.entering(this.id, "_resetAttributeSelector");
		this.attributeSelector.setStore(this.attributeStore, this._emptyAttributeValue, {query: this._getAttributeQuery("*")}); // all attributes and empty one selected by default.
		this._trace.entering(this.id, "_resetAttributeSelector");
	},
	
	_getAttributeOperatorQuery: function(value) {
		if (value == this._emptyAttributeOperator) {
			return {"applicableto": '*'+this._emptyAttributeOperator+'*'};
		}
		return {"applicableto": '*'+value+'*'};
	},
	
	_getAttributeQuery: function(value) {
		if (value == this._emptyAttribute) {
			return {"name": this._emptyAttribute};
		}
		return {"name": '*'};
	},
	
	_onAttributeOperatorChange: function(value) {
		this._trace.entering(this.id, "_onAttributeOperatorChange");
		
		this.attributeOperatorStore.fetchItemByIdentity({
			identity: value,
			onItem: function(item) { // selected attribute operator item
				this._attributeOperatorSelectedItem = item;
				
				// update phrase text
				this._updatePhraseText();
			},
			scope: this
		});
		
		
		// clear phrase value field, selected values stack
		// this._valueWidget.empty();
		
		// Phrase attribute value can be selected/provided only when some operator is selected.
		// So if no phrase attribute operator is selected, then make the phrase value disabled.
		if (this._isPhraseAttributeOperatorEmpty() === true) {
			this._disablePhraseValueWidget();
		} else {
			this._enablePhraseValueWidget();
		}
		
		this._trace.exiting(this.id, "_onAttributeOperatorChange");
	},
	
	destroy: function() {
		this._trace.enteringWithStack(this.id, "destroy", true);
		try {
			/*
			// destroy attribute operator and attribute selection lists. 
			this.attributeOperatorSelector.destroyRecursive();
			this.attributeSelector.destroyRecursive();
			
			this._valueWidget.destroyRecursive();
			*/
			
			this.inherited(arguments);
			
			this._trace.exiting(this.id, "destroy");
		} catch(e) {
			this._trace.exception(e, "destroy");
			throw e;
		}
	},
	
	displayInReadMode: function(forced) {
		// Summary:
		//		Displays the current widget in read mode
		//		Updates the text in the read mode DOM elements 
		//		per the current values from Edit mode DOM elements.
		//		
		//		Overrides _EditableElement.displayInReadMode().
		
		this._trace.entering(this.id, "displayInReadMode");
		if (! this.isInEditMode() && !forced) {
			// not in edit mode: already in read mode.
			this._trace.exiting(this.id+" - already in read mode.", "displayInReadMode");
			return;
		}
		// Update phrase text and show
		// update phrase text
		this._updatePhraseText();
		
		// show in read mode.
		this.inherited(arguments);
		
		// Grid row like style
		dojo.removeClass(this.container, "dojoxGridRowSelected");
		dojo.addClass(this.container, "dojoxGridRow");
		
		
		this._trace.exiting(this.id, "displayInReadMode");
	},
	
	_updatePhraseText: function() {
		this._trace.entering(this.id, "_updatePhraseText");
		
		// Default
		var attributeText = this.localResources.EMPTY_SECTION_ATTRIBUTE_TEXT;
		attributeText = com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(attributeText);
		var operatorText = "";
		var valuesText = "";
		
		if (! this._isPhraseAttributeEmpty()) { // some attribute
			attributeText = this.attributeSelector.get('displayedValue');
			//attributeText = com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(attributeText);
			
			if (!dojo.isIE) {
				attributeText = this._stringUtils.breakString(attributeText, 80);
			}
			
			operatorText = this.localResources.EMPTY_SECTION_ATTRIBUTE_OPERATOR_TEXT;
			if (!this._isPhraseAttributeOperatorEmpty()) { // some operator
				operatorText = this.attributeOperatorSelector.get('displayedValue');
			}
			operatorText = com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(operatorText);
			
			valuesText = this._valueWidget.getValueText();
			if (!valuesText) { // No value
				valuesText = this.localResources.EMPTY_SECTION_ATTRIBUTE_VALUES_TEXT;
				valuesText = com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(valuesText);
			} else {
				valuesText = com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(valuesText);
				valuesText = this._stringUtils.breakString(valuesText, 80);
			}
		}
		
		// Populate DOM with the texts.
		this.set("_attributeText", attributeText + " ");
		this.set("_operatorText", operatorText + " ");
		this.set("_valuesText", valuesText + " ");
		
		this._trace.exiting(this.id, "_updatePhraseText");
	},
	
	_isPhraseAttributeEmpty: function() {
		// Summary:
		//		Checks if phrase attribute list is empty: no option selected or blank option is selected. 
		//		
		this._trace.entering(this.id, "_isPhraseAttributeEmpty");
		
		var value = this.attributeSelector.get('value');
		if (!value || value == null || value == '' || value == this._emptyAttributeValue) {
			this._trace.entering(this.id+" returning true.", "_isPhraseAttributeEmpty");
			return true;
		}
		this._trace.exiting(this.id+" returning false.", "_isPhraseAttributeEmpty");
		return false;
	},
	
	_isPhraseAttributeOperatorEmpty: function() {
		// Summary:
		//		Checks if phrase attribute operator list is empty: no option selected or blank option is selected. 
		//		
		this._trace.entering(this.id, "_isPhraseAttributeOperatorEmpty");
		
		var value = this.attributeOperatorSelector.get('value');
		if (!value || value == null || value == '' || value == this._emptyAttributeOperatorValue) {
			this._trace.entering(this.id+" returning true.", "_isPhraseAttributeOperatorEmpty");
			return true;
		} 
		this._trace.exiting(this.id+" returning false.", "_isPhraseAttributeOperatorEmpty");
		return false;
	},
	
	_disablePhraseValueWidget: function() {
		// Summary:
		//		Disables the phrase value widget.
		this._trace.entering(this.id, "_disablePhraseValueWidget");
		this._valueWidget.disableValueWidget();
		this._trace.exiting(this.id, "_disablePhraseValueWidget");
	},
	
	_enablePhraseValueWidget: function() {
		// Summary:
		//		Enables the phrase value widget.
		this._trace.entering(this.id, "_enablePhraseValueWidget");
		this._valueWidget.enableValueWidget();
		this._trace.exiting(this.id, "_enablePhraseValueWidget");
	},
	
	displayInEditMode: function(forced) {
		this._trace.entering(this.id, "displayInEditMode");
		
		if (this.isInEditMode() && !forced) {
			// already in edit mode
			this._trace.exiting(this.id+" - alreay in edit mode.", "displayInEditMode");
			return;
		}
		
		this.inherited(arguments);
		
		// Grid row like style
		dojo.removeClass(this.container, "dojoxGridRow");
		dojo.addClass(this.container, "dojoxGridRowSelected");
		
		this._trace.exiting(this.id, "displayInEditMode");
	},
	
	_selectorDropDownOnKeyPress: function(evt) {
		// Summary:
		//		Handles keyboard based menu navigation.
		if ((!evt.ctrlKey) &&
			(!evt.altKey) &&
			(evt.charOrCode != this._openSubMenuKey) &&
			(evt.charOrCode != this._closeSubMenuKey) &&
			(evt.charOrCode != ' ')) {
			if ((evt.charOrCode.toLowerCase != undefined) &&
				(evt.charOrCode.toLowerCase != null)) {
				var lowerCaseCharOrCode = evt.charOrCode.toLowerCase();
				var children = this.getChildren();
				if ((children != undefined) &&
					(children != null)) {
					var child = null;
					var i = 0;
					while ((child == null) &&
						   (i < children.length)) {
						var currentChild = children[i];
						if ((currentChild != undefined) &&
							(currentChild != null)) {
							var currentLabel = currentChild.label;
							if ((currentLabel != undefined) &&
								(currentLabel != null) &&
								(currentLabel.length > 0) &&
								(currentLabel.charAt(0).toLowerCase() == lowerCaseCharOrCode)) {
								child = currentChild;
							}
						}
						i++;
					}
					if (child != null) {
						this.onItemClick(child);
					}
				}
			}
		}
	}
	
});
