/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

(c) Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.membership.FilterGroupPhraseIntegerValue");

dojo.require("dijit._Templated");

dojo.require("ibm.tivoli.tip.dijit.NumberInputBox");
dojo.require("dijit.form.NumberTextBox");

dojo.require("com.ibm.security.modeling.membership._FilterGroupPhraseValue");
dojo.require("com.ibm.security.modeling.util.Trace");

// define the widget class
dojo.declare (
	"com.ibm.security.modeling.membership.FilterGroupPhraseIntegerValue", 
    [ com.ibm.security.modeling.membership._FilterGroupPhraseValue, dijit._Templated], {
		
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.membership.FilterGroupPhraseIntegerValue"),
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.membership","templates/FilterGroupPhraseIntegerValue.html"),
	widgetsInTemplate: true,
	
	_type:"INTEGER",
	
	constructor: function() {
		// Summary:
		//		Widget life-cycle method. 

		this._trace.entering("", "constructor");
		this._trace.object(arguments, "constructor", "Constructor args");
		this._trace.exiting("", "constructor");
	},
	
	postMixInProperties: function() {
		// Summary:
		//		Widget life-cycle method. 
		
		this._trace.entering(this.id, "postMixInProperties");
		this.inherited(arguments);
		
		this.hintText = this.localResources.HINT_CONDITION_VALUE_INTEGER;
		this._trace.exiting(this.id, "postMixInProperties");
	},
	
	postCreate: function() { 
		// Summary:
		//		Widget life-cycle method. 
		
		this._trace.entering(this.id, "postCreate");
		this.inherited(arguments);
		this._valueWidget._isEmpty = dojo.hitch(this._valueWidget, this._valueWidgetIsEmpty);
		this._trace.exiting(this.id, "postCreate");
	},
	
	startup: function() {
		// Summary:
		//		Widget life-cycle method.
		//		Starts up the widget, and its child widget.
		
		this._trace.entering(this.id, "startup");
		this.inherited(arguments);
		this._trace.exiting(this.id, "startup");
	},
	
	destroy: function() {
		//	Summary:
		//		Widget life-cycle method.
		//		Destroys this widget and is contents recursively.
		
		this._trace.enteringWithStack(this.id, "destroy", true);
		try {
			this.inherited(arguments);
			this._trace.exiting(this.id, "destroy");
		} catch(e) {
			this._trace.exception(e, "destroy");
			throw e;
		}
	},
	
	getJson: function() {
		// Summary:
		//		Constructs a JSON representation from UI elements representing phrase values.
		// returns:
		//		null - if no value is specified or value is invalid
		//		Array - containing value item.
		try {
			this._trace.entering(this.id, "getJson");
			var jsonObj = null;
			if (!this.isEmpty()) {
				jsonObj = [];
				
				var data = {
					item: { // data item
						"value": this._valueWidget.get('value')
					},
					valueAttribute: "value"	// data item value attribute
				};
				
				jsonObj.push(data);
			}
			
			return jsonObj;
		} finally {
			this._trace.exiting(this.id, "getJson");
		}
	},
	
	_isEmpty: function() {
		// Summary:
		//		Overridden from _FilterGroupPhraseValue
		if (!this._valueWidget.isValid()) {
			return true;	// invalid value is like no value
		}
		return isNaN(this._valueWidget.get('value'));
	}, 
	
	_render: function(/*primitive value*/ data) {
		// Summary:
		//		Overridden from _FilterGroupPhraseValue
		
		// data:	primitive value
		//		Valid value is integer value [can be a string representation]
		//		Default data, to be populated in value widget.
		//		Used when filter phrase existed, and filter UI is opened for editing.
		
		this._trace.entering(this.id, "_render");
		
		this._valueWidget.set('value', data);
		
		this._trace.exiting(this.id, "_render");
	},
	
	_empty: function() {
		// Summary:
		//		Overridden from _FilterGroupPhraseValue
		
		this._valueWidget.set('value', '');
	},
	
	_disableWidget: function() {
		// Summary:
		//		Overridden from _FilterGroupPhraseValue
		this.inherited(arguments);

		this._valueWidget.set('disabled',true);
	},
	
	_enableWidget: function() {
		// Summary:
		//		Overridden from _FilterGroupPhraseValue
		this.inherited(arguments);

		this._valueWidget.set('disabled',false);
	},
	
	getValueText: function() {
		if (!this._valueWidget.isValid()) {
			return null;
		}
		return this._valueWidget.get('displayedValue');
	},
	
	_valueWidgetIsEmpty: function(value) {
		var result = false;
		if (value.length == 0) {
			result = true;
		}
		return result;
	}
	
});