/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

(c) Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.membership.FilterGroupPhraseStringValue");

dojo.require("dijit._Templated");

dojo.require("com.ibm.security.modeling.membership._FilterGroupPhraseFilteringValue");
dojo.require("com.ibm.security.modeling.util.Trace");

// define the widget class
dojo.declare (
	"com.ibm.security.modeling.membership.FilterGroupPhraseStringValue", 
    [ com.ibm.security.modeling.membership._FilterGroupPhraseFilteringValue], {
	
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.membership.FilterGroupPhraseStringValue"),
	_type: "STRING",
	
	// Overrides _allowCustomValues from _FilterGroupPhraseFilteringValue
	_allowCustomValues: true,
	
	// Overrides idAttribute from _FilterGroupPhraseFilteringValue
	idAttribute: "value",
	
	postMixInProperties: function() {
		// Summary:
		//		Widget life-cycle method. 
		
		this._trace.entering(this.id, "postMixInProperties");
		this.inherited(arguments);
		
		this.hintText = this.localResources.HINT_CONDITION_VALUE_STRING;
		this._trace.exiting(this.id, "postMixInProperties");
	},
	
	_onFilterKeyPress: function(/*json object*/ data) {
		// Summary:
		//		Overrides method from _FilterGroupPhraseFilteringValue
		//		This is to prevent next/previous page requests being treated as user input value to add.
		
		if (this._isPageRequest() == false) {
			// default behavior.
			this.inherited(arguments);
		}
	},
	
	_isPageRequest: function() {
		// Summary:
		//		Checks if the user activity is to get to the next/previous page.
		// returns: boolean
		//		true - if page is requested.
		//		false - otherwise.
		
		var pageRequest = false;
		// Hack to prevent the next, previous page options from being treated as user inputs.
		// And prevents adding as selected value.
		var popup = null;
		var option = null;
		if (this._filteringSelect && this._filteringSelect._popupWidget) {
			popup = this._filteringSelect._popupWidget; // hack
			option = popup.getHighlightedOption();
		}
		if (popup && (
				(option == popup.nextButton) ||
				(option == popup.previousButton))) {
			pageRequest = true;
		}
		
		return pageRequest;
	},
	
	_setupFilteringConfig: function(/*json object*/ data) {
		// Summary:
		//		Adds filtering config arguments.
		//		Overrides method from _FilterGroupPhraseFilteringValue
		//
		// data:	json object
		//		Default data, to be populated in value widget.
		
		try {
			this._trace.entering(this.id, "_setupFilteringConfig");
			
			// default configuration.
			this.inherited(arguments);
			
			// custom configuration for this value widget.
			// No arrow on select widget.
			// this._filterConfig["hasDownArrow"] = false;  // Defect# 30131
			
			// User might want to add non complete string.
			// This is so, because, String widget should allow custom values too.
			this._filterConfig["autoComplete"] = false;
			
			// User inputs are allowed, hence nothing is invalid.
			// So 
			this._filterConfig["displayMessage"] = dojo.hitch(this, function(message) { this._displayMessage(message);});
			this._filterConfig["validate"] = dojo.hitch(this, function(isFocused) { this._validate(isFocused);});
			
			// Hack.
			// On Tab out: if filter text is empty then first option is considered selected by default.
			// "_callbackSetLabel" method does so. Hence overriding it.
			this._filterConfig["_callbackSetLabel"] = dojo.hitch(this, 
				function() { 
					this._trace.finest(this.id+" - not setting label from response.", "_callbackSetLabel");
				});
			
		} finally {
			this._trace.exiting(this.id, "_setupFilteringConfig");
		}
	},
	
	_displayMessage: function(/*String*/ message) {
		// Summary:
		//		Overrides the filtering select's displayMessage from ValidationTextBox
		//		Skips display message.
		
		this._trace.entering(this.id, "_displayMessage");
		this._trace.finest(this.id+" - skipping: "+message, "_displayMessage");
		this._trace.exiting(this.id, "_displayMessage");
	},
	
	_validate: function(/*boolean*/ isFocused) {
		// Summary:
		//		Overrides the filtering select's validate from ValidationTextBox
		//		Skips display message.
		return true; // valid.
	}
	
	// Hide tooltip using: 
	// this._filteringSelect.displayMessage("");
});