/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

(c) Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.membership.FilterGroupPhraseValueController");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.membership.FilterGroupPhraseIntegerValue");
dojo.require("com.ibm.security.modeling.membership.FilterGroupPhraseStringValue");
dojo.require("com.ibm.security.modeling.membership.FilterGroupPhraseIdentityValue");
dojo.require("com.ibm.security.modeling.membership.FilterGroupPhraseHierarchyValue");
dojo.require("com.ibm.security.modeling.membership.StackContainer");

//NLS
dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling.membership", "FilterText");
dojo.requireLocalization("com.ibm.security.modeling","PluginMessages");

// define the widget class
dojo.declare (
	"com.ibm.security.modeling.membership.FilterGroupPhraseValueController", 
    [ dijit._Widget, dijit._Templated ], {
		
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.membership.FilterGroupPhraseValueController"),
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.membership","templates/FilterGroupPhraseValueController.html"),
	widgetsInTemplate: true,
	
	// _currentValueWidget: reference to a value widget [_integerValue, _stringValue, _identityValue, _hierarchyValue]
	//		Holds reference to the currently rendered value widget.
	
	// _valueWidgetsMap: map
	//		Maps widget type to widget class.
	_valueWidgetsMap: {
		"INTEGER": com.ibm.security.modeling.membership.FilterGroupPhraseIntegerValue,
		"STRING": com.ibm.security.modeling.membership.FilterGroupPhraseStringValue,
		"IDENTITY": com.ibm.security.modeling.membership.FilterGroupPhraseIdentityValue,
		"HIERARCHY": com.ibm.security.modeling.membership.FilterGroupPhraseHierarchyValue
	},
	
	// _valueWidgetsHookMap: map
	//		Maps the value widget type to their corresponding hook in template.
	
	// _valueWidgets: map
	//		Holds references to the Phrase Value widget instances, mapped by type.
	
	// _valueStack: com.ibm.security.modeling.membership.StackContainer
	//		Widget to display selected values in a stack.
	
	// _maxValuesLimit: integer
	//		Maximum number of values limit.
	
	_maxValuesLimit: 20,
	
	// _valueLimitMessage: string
	//		Message to display as tooltip when maximum limit of values is reached.
	_maxValuesLimitMessage: "",
	
	constructor: function() {
		// globalResources: Object
		//			Reference to the global shared resource bundle
		// pluginResources: object
		//			Reference to the global shared plugin resource bundle
		// localResources: Object
		//			Reference to the resource bundle unique to this widget
		
		this._trace.entering("", "constructor");
		this._trace.object(arguments, "constructor", "Constructor args");
		
		// initialize to null.
		this._currentValueWidget = null;
		
		this._valueWidgets = {};
		this._valueStack = null;
		
		this._trace.exiting("", "constructor");
	},
	
	postMixInProperties: function() {
		// Summary:
		//		Widget life-cycle method. 
		
		this._trace.entering(this.id, "postMixInProperties");
		
		this.inherited(arguments);
		
		if (!this.globalResources) {
			this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		}
		
		if (!this.pluginResources) {
			this.pluginResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "PluginMessages");
		}
		
		if (!this.localResources) {
			this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.membership", "FilterText");
		}
		//var messageFormat = "Values cannot be added because it exceeds the maximum number of {0}.";
		var messageFormat = this.pluginResources.PHRASE_VALUE_LIMIT_EXCEEDED.replace("{0}", this._maxValuesLimit);
		
		this._maxValuesLimitMessage = messageFormat.replace("{0}", this._maxValuesLimit);
		
		this._trace.exiting(this.id, "postMixInProperties");
	},
	
	postCreate: function() { 
		// Summary:
		//		Widget life-cycle method. 
		
		this._trace.entering(this.id, "postCreate");
		
		this._valueWidgetsHookMap= {
			"INTEGER": this._integerType,
			"STRING": this._stringType,
			"IDENTITY": this._indentityType,
			"HIERARCHY": this._hierarchyType
		},	
			
		this.inherited(arguments);
		
		this._trace.exiting(this.id, "postCreate");
	},
	
	startup: function() {
		// Summary:
		//		Widget life-cycle method.
		//		Starts up the widget, and its child widget.
		this._trace.entering(this.id, "startup");
		this.inherited(arguments);
		
		// Initially no value widget is shown.
		this._hideAllValueWidgets();
		
		this._trace.exiting(this.id, "startup");
	},
	
	destroy: function() {
		//	Summary:
		//		Widget life-cycle method.
		//		Destroys this widget and is contents recursively.
		
		this._trace.enteringWithStack(this.id, "destroy", true);
		try {
			// Destroy the child widgets
			
			if (this._valueStack) {
				this._valueStack.destroyRecursive();
			} 
			
			for (var type in this._valueWidgets) {
				this._valueWidgets[type].destroyRecursive();
			}
			
			this.inherited(arguments);
			
			this._trace.exiting(this.id, "destroy");
		} catch(e) {
			this._trace.exception(e, "destroy");
			throw e;
		}
	},
	
	getJson: function() {
		// Summary:
		//		Constructs a JSON representation from UI elements representing phrase values.
		// returns:
		//		null - if no value is selected/specified or value is invalid
		//		Array - containing values items.
		try {
			this._trace.entering(this.id, "getJson");
			
			var jsonObj = null;
			var valueContainer = this._getValueContainer();
			if (valueContainer) {
				jsonObj = valueContainer.getJson();
			}
			
			return jsonObj;
		} finally {
			this._trace.exiting(this.id, "getJson");
		}
	},
	
	render: function(/*string*/ type, /*json object*/ data, /*array*/ values) {
		// Summary:
		//		Displays one of the phrase value widgets [Integer|String|Identity|Hierarchy] per the 'type' parameter
		
		// type:	string
		//		Supported values: STRING,INTEGER,HIERARCHY,IDENTITY
		// data:	json object
		//		Default data, to be populated in value widget.
		// values
		//		Values from filter data.
		//		Used when filter phrase existed, and filter UI is opened for editing.
		
		try {
			this._trace.entering(this.id, "render");
			// TODO: make sure which on of "INT" or "INTEGER" is supported?
			if (type == "INT") {
				type = "INTEGER";
			}
			// empty current value (if any)
			this.empty();
			
			if (this._currentValueWidget && this._currentValueWidget.getType() != type) {
				// currently rendered widget is not of of the same type.
				// whichever value widget is on display (if any), hide it.
				this._hideCurrentValueWidget();
			}
			
			// render value widget per the 'type'
			this._renderValueWidget(type, data, values);
			
			// show currently rendered widget.
			this._showCurrentValueWidget();
			
			// populate the value stack with the values
			this._populateValueStack(values);
		} finally {
			this._trace.exiting(this.id, "render");
		}
	},
	
	addSelectedValue: function(/*json object*/ item) {
		// Summary:
		//		Adds the selected value item to the _valueStack
		//		This method is connected to onValueAdd method of the value widget.
		// 		Refer to this._createValueWidget for this.connect() call, when value widget is created.
		
		// item: json object
		//		Selected phrase value item json object
		// valueAttribute: string
		//		attribute, which should be used to display on UI.
		
		this._addItemToStack(item, /*valueAttribute*/ this._currentValueWidget.getValueAttribute());
	},
	getValueText: function() {
		// Summary:
		//		Returns the text from this._currentValueWidget
		//		Or this._valueStack
		//		Useful when it is to be displayed on UI when read mode is on.
		
		// returns: string
		//		representing phrase value text to be displayed on UI.
		
		var valueContainer = this._getValueContainer();
		
		if (valueContainer) {
			return valueContainer.getValueText();
		}
		
		return null;
	},
	
	isEmpty: function() {
		// Summary:
		//		Function to check if phrase value is empty.
		
		var valueContainer = this._getValueContainer();
		
		if (valueContainer) {
			return valueContainer.isEmpty();
		}
		return true;
	},
	
	_getValueContainer: function() {
		// Summary:
		//		Returns reference of the object, 
		//		which should be queried for isEmpty(), getValueText() for value/s
		//
		// returns:
		//		this._currentValueWidget: When phrase value widget allows only a single value: Integer 
		//		or this._valueStack		: When phrase value widget allows multi-values: string, identity, hierarchy 
		var valueContainer = null;
		if (this._currentValueWidget && this._currentValueWidget.getValueText) {
			// At present only Phrase Integer value has getValueText method.
			valueContainer = this._currentValueWidget;
		} else if (this._valueStack) {
			// For all other phrase value types,
			// Selected Values stack is the one that holds values.
			valueContainer = this._valueStack;
		}
		return valueContainer;
	},
	
	disableValueWidget: function() {
		// Summary:
		//		Function to disable the current value widget.
		if (this._currentValueWidget) {
			this._currentValueWidget.disableWidget();
		}
	},
	
	enableValueWidget: function() {
		// Summary:
		//		Function to enable the current value widget.
		if (this._currentValueWidget) {
			this._currentValueWidget.enableWidget();
		}
	},
	
	empty: function() {
		// Summary:
		//		Empties the value stack and currently displayed value widget.
		this._empty();
	},
	
	_empty: function() {
		// Summary:
		//		Empties the value stack and currently displayed value widget.
		
		if (this._valueStack) {
			this._valueStack.empty();
			this._adjustStackLayout(/*scroll top*/false);
		}
		
		if (this._currentValueWidget) {
			// empty (clear) the currently rendered value widget.
			this._currentValueWidget.empty();
			if (this._currentValueWidget.setAddAllowed) {
				this._currentValueWidget.setAddAllowed(true);
			}
		}
	},
	
	_renderValueWidget: function(/*string*/ type, /*json object*/ data, /*array*/ values) {
		// Summary:
		//		Renders phrase value widgets [Integer|String|Identity|Hierarchy] per the 'type' parameter
		//		If widget (per type) is not instantiated yet, then it instantiates the widget
		//		and renders the widget.
		//		Also, if widget requires ValueStack, and value stack is not created yet, it creates that too.
		
		// type:	string
		//		Supported values: STRING,INTEGER,HIERARCHY,IDENTITY
		// data:	json object
		//		Default data, to be populated in value widget.
		//		Used when filter phrase existed, and filter UI is opened for editing.
		// values
		//		Values from filter data.
		//		Used when filter phrase existed, and filter UI is opened for editing.
		
		if (!this._valueWidgets[type]) {
			// widget for this type is not created yet.
			// Create the instance and keep it in this._valueWidgets
			this._createValueWidget(type);
		}
		// Update _currentValueWidget reference to point to currently rendered value widget.
		this._currentValueWidget = this._valueWidgets[type];
		
		// render the current value widget.
		if (this._currentValueWidget) {
			if (type == "INTEGER") {
				data = "";
				if (dojo.isArray(values) && values.length > 0) {
					var value =  values[0];
					if (value && "value" in value) {
						data = value["value"];
					}
				}
			}
			this._currentValueWidget.render(data);
		}
	},
	
	_createValueWidget: function(/*string*/ type) {
		// Summary:
		//		Creates instance of value widget.
		
		// type:	string
		//		Supported values: STRING,INTEGER,HIERARCHY,IDENTITY
		try {
			this._trace.entering(this.id, "_createValueWidget");
			this._trace.entering(this.id+" - type: "+type, "_createValueWidget");
			
			// Do not recreate widget, if already created for this type.
			if (this._valueWidgets[type]) {
				this._trace.entering(this.id+" - Widget already created for type: "+type, "_createValueWidget");
				return;
			}
			
			var valueTypeWidget = this._valueWidgetsMap[type];
			if (!valueTypeWidget) {
				// No type widget is configured for the 'type' value.
				// throw "Invalid type: "+type;
				return;
			}
			
			// create instance of the value widget
			var valueWidget = this._valueWidgets[type] = new valueTypeWidget({
				id: this.id+"_"+type,
				globalResources: this.globalResources,
				localResources: this.localResources
			});
			
			// Attach the value widget to the DOM.
			dojo.place(valueWidget.domNode, this._valueWidgetsHookMap[type], "only");
			
			// Connect this.addSelectedValue method to widget's onValueAdd
			if (valueWidget.onValueAdd) {
				// This function is available only in those value widgets which extends _FilterGroupPhraseValue
				// [String, Identity, Hierarchy]
				this.connect(valueWidget, "onValueAdd", this.addSelectedValue);
			}
			
			if (dojo.isFunction(valueWidget.removeAllValues)) {
				// This function is available only in those value widgets which extends _FilterGroupPhraseValue
				// [Identity, Hierarchy]
				this.connect(valueWidget, "removeAllValues", this.empty);
			}
			
			
			// invoke startup
			valueWidget.startup();
		} finally {
			this._trace.exiting(this.id, "_createValueWidget");
		}
	},
	
	_createValueStack: function() {
		// Summary:
		//		Instantiate _valueStack
		// 		Its call should be avoided till, any value is actually selected.
		
		try {
			this._trace.entering(this.id, "_createValueStack");
			if (this._valueStack) {
				this._trace.entering(this.id+" - value stack already created.", "_createValueStack");
				return;
			}
			this._valueStack = new com.ibm.security.modeling.membership.StackContainer({
				id: this.id+"_valuesStack",
				globalResources: this.globalResources,
				localResources: this.localResources,
				"class": "valuesStackStyle"
			});
			
			// Attach the value stack to the DOM
			dojo.place(this._valueStack.domNode, this._stackContainer, "only");
			
			this._valueStack.startup();
			
			this.connect(this._valueStack, "addItem", this._afterValueAdd);
			this.connect(this._valueStack, "onRemoveItem", this._afterValueRemove);
		} finally {
			this._trace.exiting(this.id, "_createValueStack");
		}
	},
	
	_populateValueStack: function(/*array*/ values) {
		// Summary:
		//		Populates the value stack with the values from argument array.
		//
		// value: array
		//		An array holding json objects, representing data to populate stack items.
		try {
			this._trace.entering(this.id, "_populateValueStack");
			if (!values || values.length <1 || !this._currentValueWidget.getValueAttribute) {
				return;
			}
			var valueAttribute = this._currentValueWidget.getValueAttribute();
			for (var i= values.length-1; i >= 0; i--) {
				this._addItemToStack(values[i], valueAttribute);
			}
		} finally {
			this._trace.exiting(this.id, "_populateValueStack");
		}
	},
	
	_addItemToStack: function(item, valueAttribute) {
		// Summary:
		//		Adds item to stack.
		// returns: boolean
		//		- true: if item was added.
		//		- false: otherwise.
		
		try {
			
			this._trace.entering(this.id, "_addItemToStack");
			
			// Create value stack if not created yet.
			if (!this._valueStack) {
				this._createValueStack();
			}
			
			if (!(valueAttribute in item) &&
					(valueAttribute  != "value") &&
					("value" in item) && (item["value"])) {
				
				item[valueAttribute] = item["value"];
			}
			
			return this._valueStack.addItem(item, /*valueAttribute*/ valueAttribute);
		} finally {
			this._trace.exiting(this.id, "_addItemToStack");
		}	
	},
	
	_hideAllValueWidgets: function() {
		for (var type in this._valueWidgetsHookMap) {
			this._showHide(this._valueWidgetsHookMap[type], /*show*/ false);
		}
	},
	
	_showCurrentValueWidget: function() {
		if (!this._currentValueWidget) {
			dojo.removeClass(this.phraseValueInputSection, "phraseValueInputSection");
			dojo.addClass(this.phraseValueInputSection, "phraseValueInputSectionEmpty");
			return;
		} else {
			dojo.removeClass(this.phraseValueInputSection, "phraseValueInputSectionEmpty");
			dojo.addClass(this.phraseValueInputSection, "phraseValueInputSection");
		}
		
		this._showHide(this._valueWidgetsHookMap[this._currentValueWidget.getType()], /*show*/ true);
		// show its container as well.
		if (this._currentValueWidget.isMultiValued() == true) {
			this._showHide(this._multiValueWidgetContainer, /*show*/ true);
			
			dojo.window.scrollIntoView(this.domNode); // defect# 33584
		} else {
			this._showHide(this._singleValueWidgetContainer, /*show*/ true);
		}
	},
	
	_hideCurrentValueWidget: function() {
		if (!this._currentValueWidget) {
			return;
		}
		
		this._showHide(this._valueWidgetsHookMap[this._currentValueWidget.getType()], /*show*/ false);
		
		// hide its container as well.
		if (this._currentValueWidget.isMultiValued() == true) {
			this._showHide(this._multiValueWidgetContainer, /*show*/ false);
		} else {
			this._showHide(this._singleValueWidgetContainer, /*show*/ false);
		}
	},
	
	
	
	_showHide: function(/*DOMNode*/ node, /*boolean*/ show) {
		// Summary:
		//		Hides/Shows the 'node' per 'show' value
		// node:	DOMNode
		// show:	boolean
		//		If true then node is shown.
		//		Else node is made hidden	
		
		if (show === true) { // Show the 'node'
			dojo.removeClass(node, "hide");
			dojo.addClass(node, "show");
		} else { // Hide the 'node'
			dojo.removeClass(node, "show");
			dojo.addClass(node, "hide");
		}
	},
	
	_afterValueAdd: function(/*json object*/ itemData, /*string*/ valueAttribute) {
		// Summary:
		//		Logic to execute, when item has been added to stack.
		//		And lets current value widget know that add has taken place.
		//		
		//		Also keeps track of the maximum allowed values limit.
		//		If limit has already reached, then does not do anything
		//		If limit reaches after this add, then performs logic when limit reaches.
		
		// itemData: json object
		//		Item which has been added to stack
		//		Contains json object reference, for which stack item has been shown on UI.
		// valueAttribute: string
		//		attribute, which should is used to display on UI.
		
		this._adjustStackLayout(/*scroll top*/true);
		
		// check if limit is reached.
		if ((this._valueStack.size() == this._maxValuesLimit) && 
				this._currentValueWidget.setAddAllowed) {
			this._currentValueWidget.setAddAllowed(false, this._maxValuesLimitMessage);	// no more add
		}
		
		if (!itemData) {
			return;
		}
		if (this._currentValueWidget.onAfterValueAdd) {
			this._currentValueWidget.onAfterValueAdd(itemData);
		}
	},
	
	_afterValueRemove: function(/*json object*/ itemData) {
		// Summary:
		//		Logic to execute, when item has been removed from stack.
		// itemData: json object
		//		Data of removed Value Item.
		
		this._adjustStackLayout(/*scroll top*/false);
		
		// check if just under the limit.
		if ((this._valueStack.size() == (this._maxValuesLimit-1)) && 
				this._currentValueWidget.setAddAllowed) {
			this._currentValueWidget.setAddAllowed(true);	// can add more.
		}
		
		// if stack has not items after this remove, 
		// then focus should move back to the add button on current value widget.
		if (this._valueStack.isEmpty() === true && 
				this._currentValueWidget.addValueButton && 
				this._currentValueWidget.addValueButton.focus) {
			this._currentValueWidget.addValueButton.focus();
		}
		
		
		if (!itemData) {
			return;
		}
		
		if (this._currentValueWidget.onAfterValueRemove) {
			this._currentValueWidget.onAfterValueRemove(itemData);
		}
	},
	
	_adjustStackLayout: function(/*scroll top*/ scrollTop) {
		// Summary:
		//		Adjusts the stack's, stack container's layout.
		// scrollTop: boolean
		//		Indicator if, scroll to the top of the stack.
		
		if (scrollTop === true) {
			this._stackContainer.scrollTop = 0;
		}
	}
	
});