/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

(c) Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.membership.FilterGroupPhraseViewer");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.membership.FilterData");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");
// NLS
dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling.membership", "FilterText");

// define the widget class
dojo.declare (
	"com.ibm.security.modeling.membership.FilterGroupPhraseViewer", 
    [ dijit._Widget, dijit._Templated], {
		
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.membership.FilterGroupPhraseViewer"),
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.membership","templates/FilterGroupPhraseViewer.html"),
	widgetsInTemplate: true,
	
	_attributeText: "",
	_operatorText: "",
	_valuesText: "",
	
	constructor: function() {
		// globalResources: Object
		//			Reference to the global shared resource bundle
		// localResources: Object
		//			Reference to the local shared resource bundle
		
		// phraseData: json object
		//		An object representing a phrase
		this._trace.entering("", "constructor");
		
		this._trace.object(arguments, "constructor", "Constructor args");
		this._trace.exiting("", "constructor");
	},
	
	postMixInProperties: function() {
		// Summary:
		//		Widget life-cycle method. 
		//		Provides NLS resources.
		
		this._trace.entering(this.id, "postMixInProperties");
		
		this.inherited(arguments);
		
		if (!this.globalResources) {
			this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		}
		if (!this.localResources) {
			this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.membership", "FilterText");
		}
		
		this._extractData();
		
		this._trace.exiting(this.id, "postMixInProperties");
	},
	
	_extractData: function() {
		// Summary
		//		Extracts phrase data from rest data [this.phraseData]
		
		if (this.phraseData) {
			this._attributeText = com.ibm.security.modeling.membership.FilterData.getAttributeName(this.phraseData);
			this._operatorText = com.ibm.security.modeling.membership.FilterData.getAttributeOperator(this.phraseData);
			this._valuesText = com.ibm.security.modeling.membership.FilterData.getAttributeValuesText(this.phraseData);
			
			// if null then blank should be shown.
			this._attributeText = this._attributeText == null? "" : com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(this._attributeText);
			this._operatorText = this._operatorText == null? "" : com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(this._operatorText);
			this._valuesText = this._valuesText == null? "" : com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(this._valuesText);
		}
	}
});