/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

(c) Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.membership.FilterGroupViewer");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.security.modeling.membership.FilterGroupPhraseViewer");
dojo.require("com.ibm.security.modeling.util.Trace");

// NLS
dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling.membership", "FilterText");

// define the widget class
dojo.declare (
	"com.ibm.security.modeling.membership.FilterGroupViewer", 
    [ dijit._Widget, dijit._Templated], {
		
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.membership.FilterGroupViewer"),
	templateString: 
		"<div>" +
			"<div id='${id}_panel' dojoType='dijit.TitlePane' title='${_groupNameText}' open='false' class='filterGroupPane'>" +
				"<div dojoAttachPoint='body' class='bodyContent p'>" +
				"</div>" +
			"</div>" +
		"</div>",
		
	widgetsInTemplate: true,
	
	_groupNameText: "<h2>GROUP_NAME_TEXT_TO_REPLACE</h2>",
	
	constructor: function() {
		// globalResources: Object
		//			Reference to the global shared resource bundle
		// localResources: Object
		//			Reference to the local shared resource bundle
		
		// groupData: json object
		//		An object representing a group
		this._trace.entering("", "constructor");
		
		this._trace.object(arguments, "constructor", "Constructor args");
		this._trace.exiting("", "constructor");
	},
	
	postMixInProperties: function() {
		// Summary:
		//		Widget life-cycle method. 
		//		Provides NLS resources.
		
		this._trace.entering(this.id, "postMixInProperties");
		
		this.inherited(arguments);
		
		if (!this.globalResources) {
			this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		}
		if (!this.localResources) {
			this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.membership", "FilterText");
		}
		
		this._extractData();
		
		this._elements = {};
		
		this._trace.exiting(this.id, "postMixInProperties");
	},
	
	postCreate: function() { 
		// Summary:
		//		Widget life-cycle method. 
		this._trace.entering(this.id, "postCreate");
		this.inherited(arguments);
		
		this._createElements();
		
		this._trace.exiting(this.id, "postCreate");
	},

	startup: function() {
		// Summary:
		//		Widget life-cycle method.
		
		this._trace.entering(this.id, "startup");
		this.inherited(arguments);
		
		for (var elementId in this._elements) {
			this._elements[elementId].startup();
		}
		
		this._trace.exiting(this.id, "startup");
	},
	
	destroy: function() {
		//	Summary:
		//		Widget life-cycle method.
		
		this._trace.enteringWithStack(this.id, "destroy", true);
		try {
			// destroy elements
			for (var elementId in this._elements) {
				this._elements[elementId].destroyRecursive();
				this._elements[elementId] = null;
			}
			
			this.inherited(arguments);
			
			this._trace.exiting(this.id, "destroy");
		} catch(e) {
			this._trace.exception(e, "destroy");
			throw e;
		}
	},
	
	_createElements: function() {
		// Summary
		//		Creates group elements.
		try {
			this._trace.entering(this.id, "_createElements");
			if (!this._phrases) { // No Data
				// nothing to create.
				this._trace.finest(this.id+" - No group data. Rendering nothing.", "_createElements");
				return;
			}
			
			for (var i = 0; i < this._phrases.length;  i++) {
				// phrase operator
				if (i != 0) { // except for the first phrase
					this._createOperatorElement();
				}
				
				var phraseData = this._phrases[i];
				this._createPhraseElement(phraseData);
			}
			
		} finally {
			this._trace.exiting(this.id, "_createElements");
		}
	},
	
	_elementCount: 0,
	_createPhraseElement: function(/*json object*/ phraseData) {
		// Summary
		//		Creates filter group element.
		//		Attaches it with the DOM node on document.
		//
		// phraseData: json object
		//		Phrase information as json object
		
		try {
			this._trace.entering(this.id, "_createPhraseElement");
			
			if (!phraseData) { // No Data
				// nothing to create.
				this._trace.finest(this.id+" - No phrase data.", "_createPhraseElement");
				return;
			}
			
			var phraseId = this.id+"_phraseViewer"+ (this._elementCount++);
			this._elements[phraseId] = new com.ibm.security.modeling.membership.FilterGroupPhraseViewer({
				id: phraseId,
				globalResources: this.globalResources,
				localResources: this.localResources,
				phraseData: phraseData
			});
			
			// place the phrase in DOM.
			dojo.place(this._elements[phraseId].domNode, this.body, "last");
		} finally {
			this._trace.exiting(this.id, "_createPhraseElement");
		}
	},
	
	_createOperatorElement: function(/*string*/ operator) {
		// Summary
		//		Creates filter phrase operator element.
		//		Attaches it with the DOM node on document.
		if (!this._operator) {
			return;
		}
		
		var text = ""; 
		if (this._operator == 'or') {
			text = this.localResources.LOGICAL_OPERATOR_OR;
		} else if (this._operator == 'and') {
			text = this.localResources.LOGICAL_OPERATOR_AND;
		}  
		var operatorTextDiv = dojo.create("div", {
			"innerHTML": text,
			"class": "phraseOperatorDiv bodyText"
		});
		
		// place the operator in DOM.
		dojo.place(operatorTextDiv, this.body, "last");
	},
	
	_extractData: function() {
		// Summary
		//		Extracts phrases array from rest data [this.groupData] and 
		//		Creates a reference this._phrases
		
		if (this.groupData) {
			this._operator = this.groupData["operator"];
			
			var rules = this.groupData["rules"];
			if (dojo.isArray(rules) && rules.length > 0) {
				this._phrases = rules;
			}
		}
	},

	_getGroupNameDisplayText: function(/*string*/ groupName) {
		if (!groupName) {
			groupName = "";
		}
		
		return "<h2>"+groupName+"</h2>";
	}
});