/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

(c) Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.membership.FilterOperator");

dojo.require("com.ibm.security.modeling.membership.EditableElement");
dojo.require("dijit._Templated");

dojo.require("dijit.form.FilteringSelect");

dojo.require("com.ibm.security.modeling.util.Trace");

// NLS
dojo.requireLocalization("com.ibm.security.modeling.membership", "FilterText");

// define the widget class
dojo.declare (
	"com.ibm.security.modeling.membership.FilterOperator", 
    [com.ibm.security.modeling.membership.EditableElement, dijit._Templated ],
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.membership.FilterOperator"),
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.membership","templates/FilterOperator.html"),
	widgetsInTemplate: true,
	
	editNodeClass: "",
	readNodeClass: "",
	
	constructor: function () {
		this._trace.entering("", "constructor");
		// value: string 
		//		Initial selected value of operator
		
		// store: JsonRestStore
		//		Operator store
		
		// searchAttr:	string
		//		attribute to pick operator label value from
		
		// editNodeClass: string
		//		CSS class for this._editNode [DOM node which gets displayed in Edit mode]
		
		// readNodeClass: string
		//		CSS class for this._readNode [DOM node which gets displayed in Read mode]
		
		this._trace.exiting("", "constructor");
	},
	
	
	postMixInProperties: function() {
		this._trace.entering(this.id, "postMixInProperties");
		
		this.inherited(arguments);
		if (!this.localResources) {
			this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.membership", "FilterText");
		}
		this._trace.exiting(this.id, "postMixInProperties");
	},
	
	postCreate: function () {
		this._trace.entering(this.id, "postCreate");
		// Create elements
		this._setupSelector();
		
		this.inherited (arguments);
		
		this._trace.exiting(this.id, "postCreate");
	},
	
	startup: function() {
		this.inherited (arguments);
		// this._operatorSelector.startup();
		this._operatorSelector.setStore(this.store, this.value);
	},
	
	getJson: function() {
		// Summary:
		//		Constructs a JSON representation from UI elements representing logical operator.
		// returns:
		//		null - if no operator is specified
		//		Array - containing operator item.
		try {
			this._trace.entering(this.id, "getJson");
			var jsonObj = null;
			var value = this._operatorSelector.get('value');
			if (value) {
				jsonObj = {
					item: {"value": value.toString()},
					idAttribute: "value"	// data item id attribute
				};
			}
			
			return jsonObj;
		} finally {
			this._trace.exiting(this.id, "getJson");
		}
	},
	
	_setupSelector: function() {
		this._trace.entering(this.id, "_setupSelector");
		// creates instance of a filtering select
		
		this._trace.finest(this.id, "value: "+this.value);
		
		// need to track down menu table using xPath while testing
		this._operatorSelector.dropDown.menuTableNode.id = this._operatorSelector.id+"_menu_table";
		this.connect(this._operatorSelector.dropDown, "_onKeyPress", dojo.hitch(this._operatorSelector.dropDown, this._operatorSelectorDropDownOnKeyPress));
		this.connect(this._operatorSelector, "onChange", function(value) {this.onChange(value);});
		
		this._trace.exiting(this.id, "_setupSelector");
	},
	
	onChange: function(value) {
		// selector's onChange is connected to it. 
		this._trace.entering(this.id, "onChange");
		this._trace.exiting(this.id, "onChange");
	},
	
	setValue: function(value) {
		this._trace.entering(this.id, "setValue");
		// update filtering select.
		this._operatorSelector.set('value', value);
		this._trace.exiting(this.id, "setValue");
	},
	
	displayInReadMode: function(forced) {
		this._trace.entering(this.id, "displayInReadMode");
		if (! this.isInEditMode() && !forced) {
			// not in edit mode: already in read mode.
			this._trace.exiting(this.id+" - alreay in read mode.", "displayInReadMode");
			return;
		}
		
		this.inherited(arguments);
		
		// Display read only text
		var value = this._operatorSelector.get('displayedValue'); // 'value' 
		if (value == null || value == '') {
			value = "&nbsp;";
		}
		this._operatorText.innerHTML = value;
		
		this._trace.exiting(this.id, "displayInReadMode");
	},
	
	displayInEditMode: function(forced) {
		this._trace.entering(this.id, "displayInEditMode");
		if (this.isInEditMode() && !forced) {
			// not in edit mode: already in read mode.
			this._trace.exiting(this.id+" - alreay in edit mode.", "displayInEditMode");
			return;
		}
		
		this.inherited(arguments);
		
		this._trace.exiting(this.id, "displayInEditMode");
	},
	
	
	setSynchronizer: function(synchronizer) {
		this._trace.entering(this.id, "setSynchronizer");
		this._synchronozer = synchronizer;
		this._trace.exiting(this.id, "setSynchronizer");
	},
	
	destroy: function() {
		this._trace.enteringWithStack(this.id, "destroy", true);
		try {
			
			if (this._operatorSelector) {
				this._operatorSelector.destroyRecursive();
			}
			this.inherited(arguments);
			
			this._trace.exiting(this.id, "destroy");
		} catch(e) {
			this._trace.exception(e, "destroy");
			throw e;
		}
	},
	
	_operatorSelectorDropDownOnKeyPress: function(evt) {
		// Summary:
		//		Handles keyboard based menu navigation.
		if ((!evt.ctrlKey) &&
			(!evt.altKey) &&
			(evt.charOrCode != this._openSubMenuKey) &&
			(evt.charOrCode != this._closeSubMenuKey) &&
			(evt.charOrCode != ' ')) {
			if ((evt.charOrCode.toLowerCase != undefined) &&
				(evt.charOrCode.toLowerCase != null)) {
				var lowerCaseCharOrCode = evt.charOrCode.toLowerCase();
				var children = this.getChildren();
				if ((children != undefined) &&
					(children != null)) {
					var child = null;
					var i = 0;
					while ((child == null) &&
						   (i < children.length)) {
						var currentChild = children[i];
						if ((currentChild != undefined) &&
							(currentChild != null)) {
							var currentLabel = currentChild.label;
							if ((currentLabel != undefined) &&
								(currentLabel != null) &&
								(currentLabel.length > 0) &&
								(currentLabel.charAt(0).toLowerCase() == lowerCaseCharOrCode)) {
								child = currentChild;
							}
						}
						i++;
					}
					if (child != null) {
						this.onItemClick(child);
					}
				}
			}
		}
	}
	
});
