/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

(c) Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.membership.FilterViewer");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.security.modeling.membership.FilterData");
dojo.require("com.ibm.security.modeling.membership.FilterGroupViewer");
dojo.require("com.ibm.security.modeling.util.Trace");

// NLS
dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling.membership", "FilterText");

// define the widget class
dojo.declare (
	"com.ibm.security.modeling.membership.FilterViewer", 
    [ dijit._Widget, dijit._Templated], {
		
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.membership.FilterViewer"),
	templateString: "<div class=\"filterViewerBodyContents\" dojoAttachPoint=\"body\"></div>",
	widgetsInTemplate: true,
	
	constructor: function() {
		// globalResources: Object
		//			Reference to the global shared resource bundle
		// localResources: Object
		//			Reference to the local shared resource bundle
		
		// filterData: json object
		//		An object representing a filter
		this._trace.entering("", "constructor");
		this._trace.object(arguments, "constructor", "Constructor args");
		this._trace.exiting("", "constructor");
	},
	
	postMixInProperties: function() {
		// Summary:
		//		Widget life-cycle method. 
		//		Provides NLS resources.
		
		this._trace.entering(this.id, "postMixInProperties");
		
		this.inherited(arguments);
		
		if (!this.globalResources) {
			this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		}
		if (!this.localResources) {
			this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.membership", "FilterText");
		}
		
		this._elements = {};
		
		this._extractData();
		
		this._trace.exiting(this.id, "postMixInProperties");
	},
	
	postCreate: function() { 
		// Summary:
		//		Widget life-cycle method. 
		this._trace.entering(this.id, "postCreate");
		this.inherited(arguments);
		
		this._createElements();
		this._trace.exiting(this.id, "postCreate");
	},
	
	startup: function() {
		// Summary:
		//		Widget life-cycle method.
		
		this._trace.entering(this.id, "startup");
		this.inherited(arguments);
		
		for (var elementId in this._elements) {
			this._elements[elementId].startup();
		}
		
		this._trace.exiting(this.id, "startup");
	},
	
	destroy: function() {
		//	Summary:
		//		Widget life-cycle method.
		
		this._trace.enteringWithStack(this.id, "destroy", true);
		try {
			// destroy elements
			for (var elementId in this._elements) {
				this._elements[elementId].destroyRecursive();
				this._elements[elementId] = null;
			}
			
			this.inherited(arguments);
			
			this._trace.exiting(this.id, "destroy");
		} catch(e) {
			this._trace.exception(e, "destroy");
			throw e;
		}
	},
	
	_createElements: function() {
		// Summary
		//		Creates filter elements.
		try {
			this._trace.entering(this.id, "_createElements");
			if (!this._groups) { // No Data
				// nothing to create.
				this._trace.finest(this.id+" - No filter data. Rendering nothing.", "_createElements");
				return;
			}
			
			for (var i = 0; i < this._groups.length;  i++) {
				// Group operator
				if (i != 0) { // except for the first group
					this._createOperatorElement();
				}
				
				var groupData = this._groups[i];
				this._createGroupElement(groupData);
			}
			
		} finally {
			this._trace.exiting(this.id, "_createElements");
		}
	},
	
	_createOperatorElement: function() {
		// Summary
		//		Creates filter group operator element.
		//		Attaches it with the DOM node on document.
		
		if (!this._operator) {
			return;
		}
		var text = "";
		if (this._operator == 'or') {
			text = this.localResources.LOGICAL_OPERATOR_OR;
		} else if (this._operator == 'and') {
			text = this.localResources.LOGICAL_OPERATOR_AND;
		}  
		
		var operatorTextDiv = dojo.create("div", {
			"innerHTML": text,
			"class": "groupOperatorDiv bodyText p"
		});
		
		// place the operator in DOM.
		dojo.place(operatorTextDiv, this.body, "last");
	},
	
	_elementCount: 0,
	_createGroupElement: function(/*json object*/ groupData) {
		// Summary
		//		Creates filter group element.
		//		Attaches it with the DOM node on document.
		//
		// groupData: json object
		//		Group information as json object
		
		try {
			this._trace.entering(this.id, "_createGroupElement");
			
			if (!groupData) { // No Data
				// nothing to create.
				this._trace.finest(this.id+" - No group data.", "_createGroupElement");
				return;
			}
			
			var groupId = this.id+"_groupViewer"+ (this._elementCount++);
			this._elements[groupId] = new com.ibm.security.modeling.membership.FilterGroupViewer({
				id: groupId,
				globalResources: this.globalResources,
				localResources: this.localResources,
				groupData: groupData
			});
			
			// place the group in DOM.
			dojo.place(this._elements[groupId].domNode, this.body, "last");
		} finally {
			this._trace.exiting(this.id, "_createGroupElement");
		}
	},
	
	_extractData: function() {
		// Summary
		//		Extracts groups array from rest data [this.filterData] and 
		//		Creates a reference this._groups
		
		this._groups = com.ibm.security.modeling.membership.FilterData.getFilterGroups(this.filterData);
		this._operator = com.ibm.security.modeling.membership.FilterData.getFilterGroupOperator(this.filterData);
	}
});