/*
Licensed Materials - Property of IBM

Role and Policy Modeler

? Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.membership.SearchDialogForIdentityAndHierarchyValues");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("com.ibm.security.modeling.widgets.IsrmDialogBase");
dojo.require("dijit.Dialog");
dojo.require("dijit.Toolbar");
dojo.require("dijit._Container");

dojo.require("ibm.tivoli.tip.dijit.TIPButton");
dojo.require("com.ibm.security.modeling.dijit.layout.BorderContainer");
dojo.require("com.ibm.security.modeling.dijit.layout.ContentPane");
dojo.require("com.ibm.security.modeling.util.NlsUtilities");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.membership.SearchScopingPanel");
dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling", "LogMessages");
dojo.requireLocalization("com.ibm.security.modeling.membership", "FilterText");
dojo.requireLocalization("com.ibm.security.modeling", "PluginMessages");
dojo.require("com.ibm.security.modeling.widgets.AttributeScopingPanel");
dojo.require("com.ibm.security.modeling.util.MessageUtilities");

dojo.declare("com.ibm.security.modeling.membership.SearchDialogForIdentityAndHierarchyValues",com.ibm.security.modeling.widgets.IsrmDialogBase,
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.membership.SearchDialogForIdentityAndHierarchyValues"),
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.membership","templates/SearchDialogForIdentityAndHierarchyValues.html"),	

	widgetsInTemplate : true,
	// parentWidget: domNode
	//		Widget which created this widget
	parentWidget : null,
	// heightFactor: number
	//		Gives the hight factor for widget
	heightFactor:1.2,
	// widthFactor: number
	//		Gives the width factor for widget
	widthFactor:1.1,
	// minHeight: number
	//		minimum height of widget
	minHeight:350,
	killOnHide:true,
	valueAttribute : null,
	//			Reference to the resource bundle unique to this widget
	localResources: null,
	// messageResources: Object
	//      The resource bundel
	// 			Unique identifier to the instance of portlet page where this item will exist

	//Instance of the Search Scoping Panel that actually displays table/topology and the shopping cart widget within this dialog
	_searchScopingPanel: null,
	
	//Selected Item (json object) that is provided from parent widget for launching the search dialog
	selectedItemFromMembershipFilterPanel: null,
	
		
	constructor: function (args) {
		// summary:
		// 			Widget that provides the UI for the Search Memmbership Values operation for identity and hierachical values.
		
	    this._trace.entering(this.id, "constructor");
		this.helpFile="ui_role_membership_values.html";
		this._trace.exiting(this.id, "constructor");
	},
	
	postMixInProperties:function(){
		this._trace.entering(this.id, "postMixInProperties");
        this.inherited(arguments);
        if (!this.localResources) {
        	this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.membership", "FilterText");
        }
        this._messageUtilities = new com.ibm.security.modeling.util.MessageUtilities();
        var inserts = new Array();
        inserts[0] = this.selectedItemFromMembershipFilterPanel.name[0];
        var formattedTitle = this._messageUtilities.formatMessage(this.localResources.SEARCH_DIALOG_TITLE, inserts);
        this.dialogTitle = formattedTitle;
        this.dialogInstruction = this.localResources.SEARCH_DIALOG_INSTRUCTION;
        
        this._trace.exiting(this.id, "postMixInProperties");
	},

	postCreate : function() {
		// tags:
		//		protected
		this._trace.entering(this.id, "postCreate");
		this.inherited(arguments);
		this._overrideContainerLtrFunction();
		this._createAttributeScopingPanel();
		this.addCommonButton({
			onClick:dojo.hitch(this, this.onCancel),
			type:"CANCEL"
		});		
		this.addCommonButton({
			onClick:dojo.hitch(this, this.onOk),
			type:"OK"
		});
		this._trace.exiting(this.id, "postCreate");
	},
	
	_overrideContainerLtrFunction: function() {
		// summary:
		// 			Setups the direction for the widget.
		this._trace.entering(this.id, "_overrideContainerLtrFunction");
		this._IsrmDialog.isLeftToRight = NlsUtilities.isLtr;
		this._trace.exiting(this.id, "_overrideContainerLtrFunction");
	},

	startup : function() {
		// tags:
		//		protected
		//this._defineMessageAreaBehavior();
		this._trace.entering(this.id, "startup");
	    this.inherited(arguments);
	    this._searchScopingPanel.startup();
		dojo.addClass(this._IsrmDialog.domNode,"roleCopy");
		this._trace.exiting(this.id, "startup");
	 },
	
	show : function() {
		// summary:
		// 			Launches and initializes the dialog.
		this._trace.entering(this.id, "show");
		this._initAttributePanel();
		
		this.inherited(arguments);
		this._trace.exiting(this.id, "show");
	},
	
	_initAttributePanel:function(){
		// summary:
		// 			Initializes the attribute/search panel.
		this._trace.entering(this.id, "_initAttributePanel");	
		this._searchScopingPanel.initPanel();
		this._trace.exiting(this.id, "_initAttributePanel");
	},

	_createAttributeScopingPanel : function() {
		// summary:
		// 			Creates the attribute/search attribute shopping cart panel
 	    this._trace.entering("", "_createAttributeScopingPanel");

		var selectedItem = null;
		
		var attrUri = this.selectedItemFromMembershipFilterPanel.uri[0].toString();
		var attrId = attrUri.substring(attrUri.lastIndexOf("/")+1, attrUri.length); 
			
		selectedItem = {id:attrId, name:this.selectedItemFromMembershipFilterPanel.name[0], type:this.selectedItemFromMembershipFilterPanel.type[0], uri:this.selectedItemFromMembershipFilterPanel.uri[0]};
		this._searchScopingPanel = new com.ibm.security.modeling.membership.SearchScopingPanel( {
			id : this.id + "_attribScopingContainer",
			selectedItem:selectedItem,
			preSelectValues: this.selectedValues,
			messageArea:this.msgArea,
			valueAttribute : this.valueAttribute
		}, this.id + "_attribScopingContainer");

		dojo.addClass(this._searchScopingPanel.domNode,"contentArea");
		this._trace.exiting("", "_createAttributeScopingPanel");
   },
  
	_showMessage: function(/*String*/type, /*String*/message) {
		// summary:
		//			shows the given message and type in the local message area
		// tags:
		//			private
		
		//this._hideMessages();
		
	   this._trace.entering(this.id, "_showMessage");
		if (type === "critical") {
			this.msgArea.showErrorMessage(message);
		}else if (type === "info") {
			this.msgArea.showInfoMessage(message);
		}else if (type === "warning") {
			this.msgArea.showWarningMessage(message);
		}
		this._trace.exiting(this.id, "_showMessage");

	},
	
	_hideMessages:function() {
		// summary:
		//			hide the given message and type in the local message area
		// tags:
		//			private
		
		this.msgArea.hideMessages();
	},
	
	onOk : function() {
		// summary:
		// 			Gets called when OK button is clicked.
		
		this._trace.entering(this.id, "onOk");
		if (this._searchScopingPanel.validateSelection()== true) {
			this.msgArea.hideMessages();
			this.getScopingValues();
			this.hide();
		}
		this._trace.exiting(this.id, "onOk");
	},
	
	getScopingValues: function(){

		// summary:
		// 			Gets called when OK button is clicked and all the validations are performed.
		//			This function gets the selected items and adds them in the selected values of the filter membership dialog
		// 			For Identity attribute, attribute name is "customAttributeId". Refer to SearchScopingPanel.js for the same.
		
		this._trace.entering(this.id, "getScopingValues");
		var type=this.selectedItemFromMembershipFilterPanel.type[0];
		var attrUri = this.selectedItemFromMembershipFilterPanel.uri[0];
		var attrName = this.selectedItemFromMembershipFilterPanel.name[0];
		var attrId = attrUri.substring(attrUri.lastIndexOf("/")+1, attrUri.length);

		var items=this._searchScopingPanel.summaryWidget.getSectionItems(attrName);
		this.parentWidget.removeAllValues();//removing old values
		var itemId;
		for(itemId in items){
					var val=items[itemId].item.text;
					var id = items[itemId].item.id;
					var uri = items[itemId].item.itemUri;
					var obj = {name:val, value:val, id:id, uri:uri};
					this.parentWidget.onValueAdd(obj);
		}
		this._trace.exiting(this.id, "getScopingValues");
	},
		
	destroy: function()	{
		this._trace.entering(this.id, "destroy");
		this._searchScopingPanel.destroy();
		this.inherited(arguments);
		this._trace.exiting(this.id, "destroy");
	}
});
