/*
Licensed Materials - Property of IBM

Role and Policy Modeler

? Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.membership.SearchScopingPanel");
dojo.require("com.ibm.security.modeling.widgets.AttributeScopingPanel");

// NLS
dojo.requireLocalization("com.ibm.security.modeling.projects", "UsersResourcesSelectionPanelText");

dojo.require("com.ibm.security.modeling.util.Trace");


dojo.declare("com.ibm.security.modeling.membership.SearchScopingPanel",
            [ com.ibm.security.modeling.widgets.AttributeScopingPanel],
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.membership.SearchScopingPanel"),
	// localResources: Object
	//		The resource bundle for this widget set
	localResources: null,
	// messageArea: ibm.tivoli.tip.dijit.TIPMessageArea
	//		Message area for error display
	//messageArea: null,
	// searchAttribute: String
	//			The context root of the server from which Content is being served.
	searchAttribute:"displayName",

	// viewAllUrl: String
	//			The uri which will fetch the ViewAll dialog box content and query for count
	viewAllUrl:"/isrm/rest/users/?queryFormat=AttributeFilter",
	// defaultValue: String
	//			Default value for the attribute dropdown
	defaultValue:"IDENTITY",
	// nameAttributeSelectionLimit: number
	//			Max Role Name Selection Limit (direct selection)
	nameAttributeSelectionLimit: 1000,
	// attrValueSelectionLimit: number
	//			Max Attribute Values Selection Limit 
	attrValueSelectionLimit: 15,
	// noResourceSelectionMsg: String
	//			The message key use to be show message if no roles are selected
	noResourceSelectionMsg:"MEMBERSHIP_FILTER_NO_VALUE_SELECTED",
	// selectionLimitMsg: String
	//			The message key use to be show message if roles/attribute value are above the limit.
	selectionLimitMsg:"MAX_LIMIT_OF_SELECTING_VALUES_FOR_MEMBERSHIP_FILTER",
	
	//selected item (json object) received from SearchDialog. 
	selectedItem: null,
	preSelectValues: null,
	includeSubtreeOption: false,
	includeSubtree: true,
	includeSubtreeText:"", //We dont need any text to mention Include Subtree
	
	postMixInProperties: function() {
		this._trace.entering(this.id, "postMixInProperties");
		
		this.inherited(arguments);
		this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.projects", "UsersResourcesSelectionPanelText");
		this.summaryHeaderLabel = this.localResources.SELECTED_VALUES;
		this.dropDownLabel="";
		
		this.idAttribute="storeId";
		this.bIsfilteringWidgetNeeded = false;
		
		this._trace.exiting(this.id, "postMixInProperties");
	},
	postCreate: function() {
		// tags:
		//		protected
				
		this._loadAttributeDropDown();
		this.inherited(arguments);
	},
	
	_loadAttributeDropDown:function(){
		
		//This function pre-populates selected item in the filtering select widget
		
		var attrDataItems = [];

		var attrUri = this.selectedItem.uri;
		var attrId = attrUri.substring(attrUri.lastIndexOf("/")+1, attrUri.length);
		attrDataItems.push({displayName:this.selectedItem.name,name:this.selectedItem.name,id:this.selectedItem.name, type:this.selectedItem.type,uri:this.selectedItem.uri});
		
		var attrData = {
		  identifier:"id",
		  label: "displayName",
		  items: attrDataItems
		};
		dojo.require("dojo.data.ItemFileReadStore");
		this.attributeStore = new dojo.data.ItemFileReadStore({
   				data: attrData
		});
	},

	startup: function() {
		// tags:
		//		protected
		this.inherited(arguments);
	},

	getHelpFileName: function(){
		return "ui_role_membership_values.html";
	},
	
	getViewAllTable:function(store,queryParams){
		dojo.require("com.ibm.security.modeling.hierarchy.ViewAllGlobalRoleTable");
		var table = new com.ibm.security.modeling.hierarchy.ViewAllGlobalRoleTable(
		{
			id:this.id+"_viewAllTable",
			dataStore:store,
			query:queryParams,
			messageArea: this.messageArea
		});
		return table;
		
	},

	initPanel: function(){
		this.inherited(arguments);
		this.prePopulateValues(this.preSelectValues);
		this._updateTotalCount();
	},
	
	destroy: function()	{
		this.inherited(arguments);
	}
});
