/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

(c) Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.membership.StackItem");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("ibm.tivoli.tip.dijit.TIPButton");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");

//NLS
dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling.membership", "FilterText");

// define the widget class
dojo.declare (
	"com.ibm.security.modeling.membership.StackItem", 
    [ dijit._Widget, dijit._Templated ], {
		
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.membership.StackItem"),
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.membership","templates/StackItem.html"),
	widgetsInTemplate: true,
	
	// _valueText: string
	//		Text representing data value for this Item.
	_valueText: "",
	
	// _displayValueText: string
	//		Text that is to be displayed on UI for this Item.
	_displayValueText: "",
	
	
	// _defaultValueAttribute: string
	//		attribute to access from the item, when no valueAttribute is provided.
	_defaultValueAttribute: "value",
	
	constructor: function() {
		// globalResources: Object
		//			Reference to the global shared resource bundle
		// localResources: Object
		//			Reference to the resource bundle unique to this widget
		// data:	json object
		//		Json Data object representing phrase attribute value item.
		// valueAttribute: string
		//		attribute, which should be used to display on UI.
		// container: StackContainer
		//		Stack which holds this item
		
		this._trace.entering("", "constructor");
		this._trace.object(arguments, "constructor", "Constructor args");
		
		this._trace.exiting("", "constructor");
	},
	
	postMixInProperties: function() {
		// Summary:
		//		Widget life-cycle method. 
		
		this._trace.entering(this.id, "postMixInProperties");
		
		this.inherited(arguments);
		if (!this.globalResources) {
			this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		}
		if (!this.localResources) {
			this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.membership", "FilterText");
		}
		
		if (!this.valueAttribute) {
			this.valueAttribute = this._defaultValueAttribute;
		}
		
		this._setupValueText();
		
		this._trace.exiting(this.id, "postMixInProperties");
	},
	
	focus: function() {
		// Summary:
		//		Method to set focus to this widget's focus node, via code.
		if (this._focusNode && this._focusNode.focus) {
			this._focusNode.focus();
		}
	},
	
	getValueText: function() {
		// Summary:
		//		returns the displayed value text for this item.
		
		return this._valueText;
	},
	
	_setupValueText: function() {
		// Summary:
		//		Sets value in _valueText from data.
		
		this._trace.entering(this.id, "postMixInProperties");
		this._valueText = this.data[this.valueAttribute];
		this._displayValueText = com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(this._valueText);
		this._trace.exiting(this.id, "postMixInProperties");
	},
	
	_onRemove: function() {
		// Summary:
		//		Callback attached to remove item button in template.
		//		Invokes removeItem on its container stack.
		
		this._trace.entering(this.id, "_onRemove");
		if (this.container && this.container.removeItem) {
			this.container.removeItem(this);
		}	
		this._trace.entering(this.id, "_onRemove");
	}
	
});