/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

(c) Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.membership.SynchronizedSelector");

dojo.require("dijit.form.ComboBox");

dojo.require("com.ibm.security.modeling.membership.FilterOperator");
dojo.require("com.ibm.security.modeling.util.Trace");


dojo.declare("com.ibm.security.modeling.membership.SynchronizedSelector", null,  {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.membership.SynchronizedSelector"),
	
	// _selectedValue: string
	//		Selected option value
	
	editNodeClass: "",
	readNodeClass: "",
	
	constructor: function() {
		this._trace.entering(this.id, "constructor");
		// id: string
		//		should be supplied by caller.
		
		// store: dojo store
		//		Store for the selection list.
		// defaultValue: string
		//		Default value for selection list
		
		// editNodeClass: string
		//		CSS class to be applied, when operator is displayed in Edit Mode
		
		// readNodeClass: string
		//		CSS class to be applied, when operator is displayed in Read Mode
		
		dojo.mixin(this, arguments[0]);
		
		this._connects = [];
		
		this._isSynchRunning = false;
		this._operators = {};
		this._selectedValue = 'and';
		
		if (this.defaultValue) {
			this._selectedValue = this.defaultValue;
		}
		
		this._trace.exiting(this.id, "constructor");
	},
	
	_activeOperatorCount: 0,
	
	createOperator: function(id) {
		this._trace.entering(this.id, "createOperator");
		var operatorWidget = new com.ibm.security.modeling.membership.FilterOperator({
			id: id,
			value: this._selectedValue,
			store: this.store,
			searchAttr: "operator",
			editNodeClass: this.editNodeClass,
			readNodeClass: this.readNodeClass,
			"class": "ignorePrevFloatElements" 
		});
		

		this._connects.push(dojo.connect(operatorWidget, "onChange", this, dojo.hitch(this, this._onChangeHandler, operatorWidget)));
		this._connects.push(dojo.connect(operatorWidget, "onFocus", this, this.displayInEditMode));
		this._connects.push(dojo.connect(operatorWidget, "onBlur", this, this.displayInReadMode));
		
		// So work around at present:
		operatorWidget.setSynchronizer(this);
		
		this._operators[operatorWidget.id] = operatorWidget;
		this._activeOperatorCount++;
		this._trace.exiting(this.id, "createOperator");
		return operatorWidget;
	},
	
	_onChangeHandler: function(operatorWidget, selectedValue) {
		this._trace.entering(this.id, "_onChangeHandler");
		if (this._isSynchRunning == true) {
			return;
		}
		
		this._selectedValue = selectedValue;
		// for each operator in this._operators
		//	update value.
		this._isSynchRunning = true;
		for (var operatorId in this._operators) {
			if (operatorId == operatorWidget.id) {
				continue;	// skip, as this one caused the synch up, so it is the source of this value.
			}
			this._operators[operatorId].setValue(this._selectedValue);
		}
		this._isSynchRunning = false;
		this._trace.exiting(this.id, "_onChangeHandler");
	},
	
	setSelectedValue: function(/*string*/ value) {
		// Summary
		//		Sets the _selectedValue per the supplied value parameters.
		// value: string
		//		Value that is to be set.
		this._selectedValue = value;
		
		for (var operatorId in this._operators) {
			this._operators[operatorId].setValue(this._selectedValue);
			// changing one operator has updated all operator.
			// hence break;
			break;
		}
	},
	
	destroyOperator: function(operatorWidget) {
		this._trace.entering(this.id, "destroyOperator");
		this._destroyOperator(operatorWidget.id);
		this._trace.exiting(this.id, "destroyOperator");
	},
	
	_destroyOperator: function(id) {
		this._trace.entering(this.id, "_destroyOperator");
		if (this._operators[id]) { 
			this._operators[id].destroyRecursive();
			delete this._operators[id];
			this._activeOperatorCount--;
		} 
		this._trace.exiting(this.id, "_destroyOperator");
	},
	
	destroyRecursive: function() {
		this._trace.entering(this.id, "destroyRecursive");
		for (var operatorId in this._operators) {
			this._destroyOperator(operatorId);
		}
		this._trace.exiting(this.id, "destroyRecursive");
	},
	
	displayInReadMode: function() {
		this._trace.entering(this.id, "displayInReadMode");
		for (var operatorId in this._operators) {
			this._operators[operatorId].displayInReadMode();
		}
		this._trace.entering(this.id, "displayInReadMode");
	},
	
	displayInEditMode: function() {
		this._trace.entering(this.id, "displayInEditMode");
		for (var operatorId in this._operators) {
			this._operators[operatorId].displayInEditMode();
		}
		this._trace.entering(this.id, "displayInEditMode");
	}
	
});
