/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

(c) Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.membership._ElementContainer");

dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.membership.SynchronizedSelector");

//NLS
dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling.membership", "FilterText");
dojo.requireLocalization("com.ibm.security.modeling","PluginMessages");

dojo["requireLocalization"]("customizations", "AttributeCustomization");

dojo.declare("com.ibm.security.modeling.membership._ElementContainer", null,  {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.membership._ElementContainer"),
	
	// data: object
	//		A json object representing container data.
	
	// _synchronizedSelector: object
	//		An instance of com.ibm.security.modeling.membership.SynchronizedSelector
	// _elements: object
	//		a map for elements in this filter. mapped by element id.
	// _elementOperators: object
	//		a map for element operator in this filter. mapped by element id.
	
	// _firstElement: An instance of 
	//		com.ibm.security.modeling.membership.FilterGroup
	//		or
	//		com.ibm.security.modeling.membership.FilterGroupPhrase
	//
	//		A reference to the first element of this filter.
	//		This reference should be updated when this element is removed from filter.
	
	// _elementType: null,
	//		element type: 'group' or 'phrase'
	
	operatorEditNodeClass: "",
	operatorReadNodeClass: "",
	
	constructor: function () {
		this._trace.entering("", "constructor");
		
		this._elements = {};
		this._elementOperators = {};
		this._firstElement = null;
		
		// emptyContainerInstructions: domNode
		//		DIV containing the instructions text, to be displayed when there is no element in it. 
		
		// operatorEditNodeClass: string
		//		CSS class to be applied, when element operator is displayed in Edit Mode
		
		// operatorReadNodeClass: string
		//		CSS class to be applied, when element operator is displayed in Read Mode
		
		this._trace.exiting("", "constructor");
	},
	
	postMixInProperties: function() {
		this._trace.entering(this.id, "postMixInProperties");
		
		this.inherited (arguments);
		
		if (!this.globalResources) {
			this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		}
		
		if (!this.pluginResources) {
			this.pluginResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "PluginMessages");
		}
		
		if (!this.localResources) {
			this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.membership", "FilterText");
		}
		
		if (!this.attributeResources) {
			this.attributeResources = dojo.i18n.getLocalization("customizations", "AttributeCustomization");
		}
		
		this._trace.exiting(this.id, "postMixInProperties");
	},
	
	postCreate: function () {
		this._trace.entering(this.id, "postCreate");
		this.inherited (arguments);
		
		this._synchronizedSelector = new com.ibm.security.modeling.membership.SynchronizedSelector({
			id: this.id+"_synchSelector",
			store: this.operatorStore,
			defaultValue: this._getElementOperatorFromRestData(this.data),
			editNodeClass: this.operatorEditNodeClass,
			readNodeClass: this.operatorReadNodeClass
		});
		
		// create elements.
		this._createElements(this.data);
		
		this._trace.exiting(this.id, "postCreate");
	},
	
	_getSynchronizedSelectorDefaultValue: function() {
		// nothing.
		// override it.
		return null;
	},
	
	getJsonFromUI: function() {
		// Summary:
		//		constructs a JSON representation from UI elements. 
		// returns:
		//		null if this container is empty
		//		json object otherwise.
		
		try {
			this._trace.entering(this.id, "getJsonFromUI");
			var jsonObj = null;
			
			if (!this.isEmpty()) {
				// add elements
				var elements = [];
				var elementJson = null;
				for (var elementId in this._elements) {
					elementJson = this._elements[elementId].getJsonFromUI();
					if (elementJson) {
						elements.push(elementJson);
					}
				}
				if (elements.length > 0) {
					jsonObj = {};
					jsonObj["elements"] = elements;
					
					// Add element operator
					for (var elementId in this._elementOperators) {
						jsonObj["operator"] = this._elementOperators[elementId].getJson();
						break;
					}
				}
			}
			return jsonObj;
		} finally {
			this._trace.exiting(this.id, "getJsonFromUI");
		}
	},
	
	_getJsonFromRestData: function(/*json object*/ data) {
		// Summary:
		//		constructs a JSON representation from Rest response data. 
		//		This JSON representation contains "elements" (Array representing contained elements) 
		//		and "operator" (string representing element operator) properties.
		//		
		// data: json object
		//		Represents details about the container, and its elements.
		//
		// returns:
		//		null if this container is empty
		//		json object otherwise.
		
		try {
			this._trace.entering(this.id, "_getJsonFromRestData");
			var jsonObj = null;
			
			if (data) {
				jsonObj = {};
				jsonObj["elements"] = this._getElementsJsonFromRestData(data);
				jsonObj["operator"] = this._getElementOperatorFromRestData(data);
			}
			return jsonObj;
		} finally {
			this._trace.exiting(this.id, "_getJsonFromRestData");
		}
	},
	
	_getElementsJsonFromRestData: function(/*json object*/ data) {
		// Summary: 
		//		Extracts Elements from the json object from REST.
		// tags: protected.
		//		Sub class should provide implementation for this.
		// data: json object
		//		Represents details about the container, and its elements.
		//
		// returns: Array
		return null;
	},
	
	_getElementOperatorFromRestData: function(/*json object*/ data) {
		// Summary: 
		//		Extracts Elements operator from the json object from REST.
		// tags: protected.
		//		Sub class should provide implementation for this.
		// data: json object
		//		Represents details about the container, and its elements.
		//
		// returns: string
		return null;
	},
	
	startup: function() {
		this._trace.entering(this.id, "startup");
		
		this.inherited(arguments);
		this._startupElements();
		
		this._trace.exiting(this.id, "startup");
	},
	
	_startupElements: function() {

		for (var elementId in this._elements) {
			this._elements[elementId].startup();
		}
		for (var elementId in this._elementOperators) {
			this._elementOperators[elementId].startup();
		}
	},
	
	isEmpty: function() {
		// Summary:
		//		To check if the container is empty (contains no element).
		
		return this._activeElementCounter == 0;
	},
	
	addElement: function(/*json object*/ elementData) {
		// Summary:
		//		Adds an element to this container.
		//		This method should be used when widget has been started.
		//		For example "add" button callback should invoke this method to add the element.
		//
		//		Sub types are supposed to provide implementation of 'createElement' method.
		//		
		// returns: created element
		
		this._trace.entering(this.id, "addElement");
		
		// add element and focus it.
		var element = this._createElement(elementData);// new empty element 
		
		// startup and display [read only mode] element operator 
		if (element.id in this._elementOperators) {
			this._elementOperators[element.id].startup();
			this._elementOperators[element.id].displayInReadMode();
		}
		
		// startup.
		element.startup();
		this._trace.exiting(this.id, "addElement");
		return element;
	},
	
	createElement: function(elementId, data) {
		// Summary:
		//			Supposed to return the element type instance, to be contained in this container.
		//			Override this method in sub types which extends this class.
		return null;
	},
	
	removeElement: function(element) {
		// Summary:
		//		Removes the element from container.
		var isRemoved = false;
		try {
			this._trace.entering(this.id, "removeElement");
			
			if (this._activeElementCounter == 1) { // last element is being removed.
				// remove element
				this._destroyElement(element.id);
				// no elements, hence no firstElement.
				this._firstElement = null;
				
			} else {
				// Which element should be focused after this delete.
				var elementToFocus = this._getElementToFocus(element.id);
				
				// If first element is being removed, then next element's (new first element to be) operator is to be destroyed as well.
				// But do it after deleting the element.
				var isResetFirstElementNeeded = element.id == this._firstElement.id;
				
				// remove element
				this._destroyElement(element.id);
				
				if (isResetFirstElementNeeded == true) {	// removed element was the first one.
					this._resetFirstElement(elementToFocus); // reset the first element reference.
					// remove first element's operator.
					this._synchronizedSelector.destroyOperator(this._elementOperators[this._firstElement.id]);
					delete this._elementOperators[this._firstElement.id];
				}
				
				elementToFocus.focus(true);	// true represents forced focus.
			}
			
			
			isRemoved =  true; // removed successfully.
			
		} finally {
			this._onAfterRemoveElement(isRemoved);
			this._trace.exiting(this.id, "removeElement");
		}
		return isRemoved;
	},
	
	_onAfterRemoveElement: function(/*boolean*/ isRemoved) {
		// Summary:
		//		Logic to execute after remove element.
		
		if (isRemoved === true) {
			if (this.isEmpty()) {
				this._showEmptyContainerInstructions();
			}
		}
	},
	
	_showEmptyContainerInstructions: function() {
		// Summary:
		//		Displays the empty container instructions section.
		if (this.emptyContainerInstructions) {
			dojo.removeClass(this.emptyContainerInstructions, "hide");
			dojo.addClass(this.emptyContainerInstructions, "show");
		}
	},
	
	_hideEmptyContainerInstructions: function() {
		// Summary:
		//		Displays the empty container instructions section.
		if (this.emptyContainerInstructions) {
			dojo.removeClass(this.emptyContainerInstructions, "show");
			dojo.addClass(this.emptyContainerInstructions, "hide");
		}
	},
	
	empty: function() {
		// Summary:
		//			Empties the container.
		//			An empty container contains no element in it.
		
		this._trace.entering(this.id, "empty");
		
		// empty container contains no element
		// delete all elements.
		for (var elementId in this._elements) {
			this._destroyElement(elementId);
		}
		this._firstElement = null;
		
		// show empty container instructions.
		this._showEmptyContainerInstructions();
		
		this._trace.exiting(this.id, "empty");
	},
	
	destroyRecursive: function() {
		this._trace.enteringWithStack(this.id, "destroy", true);
		try {
			// destroy elements
			for (var elementId in this._elements) {
				this._destroyElement(elementId);
			}
			
			if (this._synchronizedSelector) {
				this._synchronizedSelector.destroyRecursive();
			}
			
			this.inherited(arguments);
			
			this._trace.exiting(this.id, "destroy");
		} catch(e) {
			this._trace.exception(e, "destroy");
			throw e;
		}
	},
	
	_createElements: function(/*json object*/ data) {
		// Summary:
		//		Creates the elements of this containers, per the supplied data.
		//
		// data: json object
		//		Represents details about the container, and its elements.
		
		this._trace.entering(this.id, "_createElements");
		
		// Synchronized selector's selected option value. 
		this._synchronizedSelector.setSelectedValue(this._getElementOperatorFromRestData(data));
		
		if (!data) { // empty this container.
			this.empty();
		} else {
			var jsonObj = this._getJsonFromRestData(data);
			if (jsonObj && jsonObj["elements"]) {
				var elementsData = jsonObj["elements"];
				for (var i=0; i < elementsData.length; i++) {
					this._createElement(elementsData[i]);
				}
			}
		}
		this._trace.exiting(this.id, "_createElements");
		
	},
	
	_activeElementCounter: 0,
	_elementCounter: 0,
	_createElement: function(elementData) {
		try {
			// elementData: json object
			//		Data to populate in the element.
			this._trace.entering(this.id, "_createElement");
			
			// create element id.
			var elementId = this.id+"_"+this._elementType+"_"+(this._elementCounter++);
			
			//1. create element
			
			var element = this.createElement(elementId, elementData);	// get element from the sub-type.
			// If this is the first element of this container,
			// then update the first element reference.
			if (this._firstElement == null) {// no reference yet: this is the first element.
				this._firstElement = element;
			}
			this._elements[elementId] = element; // keep track of the created elements.
			this._activeElementCounter++;
			
			//2. create operator and place it in the end.
			this._createElementOperator(elementId);
			
			//3. Place the element in the DOM.
			if (elementId in this._elementOperators) { // place after the operator
				dojo.place(element.domNode, this._elementOperators[elementId].domNode, "after");
			} else { // place it in the end of the container.
				dojo.place(element.domNode, this.elementContainer, "last");
			}
			
			this._trace.exiting(this.id, "_createElement");
			
			this._onAfterCreateElement();
			
			return element;
		} catch (e) {
			this._trace.exception(e, "_createElement");
			throw e;
		} 
	},
	
	_onAfterCreateElement: function() {
		// Summary:
		//		Performs logic after create element.
		if (this._activeElementCounter == 1) {
			this._hideEmptyContainerInstructions();
		}
	},
	
	_resetFirstElement: function(element) {
		// Summary:
		//		Updates the _firstElement to point to the first element on UI.
		//		Possible use: when first element itself is removed.
		// element: _firstElement
		
		this._trace.entering(this.id, "_resetFirstElement");
		
		if (element) {
			// Update reference.
			this._firstElement = element;
		} else {
			// Query DOM
			var divs = dojo.query("> div", this.elementContainer);
			var firstElementId = firstDivId = dojo.attr(divs[0], "id");
			
			if (firstDivId.indexOf("_operator") == -1) { // first element's operator has not been removed yet.
				firstElementId = dojo.attr(divs[1], "id");
			}
			
			// Update reference.
			this._firstElement = this._elements[firstElementId];
		}
		this._trace.exiting(this.id, "_resetFirstElement");
	},
	
	_getElementToFocus: function(elementId) {
		// Summary:
		//		returns the next element on UI with respect to the element
		//		If no next element, then returns the previous element.
		// elementId: string
		//		Id of the reference element.
		
		this._trace.entering(this.id, "_getElementToFocus");
		// Query DOM
		var divs = dojo.query("> div", this.elementContainer);
		var nextIndex = -1;
		var previousIndex = -1;
		
		var element = null;
		for (var i=0; i < divs.length; i++) {
			if (i == nextIndex) {
				element = this._elements[dojo.attr(divs[i], "id")];
				break;
			}
			var id = dojo.attr(divs[i], "id");
			if (id.indexOf("_operator") == -1) { // not operator
				if (id == elementId) {
					nextIndex = i+2;	// +1 would be the element operator
					previousIndex = i-2;
					if (previousIndex >=0) {
						element = this._elements[dojo.attr(divs[previousIndex], "id")];
						// do not break
					}
				}
			}
		}
		
		this._trace.exiting(this.id, "_getElementToFocus");
		return element;
	},
	
	_createElementOperator: function(elementId) {
		// Summary:
		//		Creates an operator element for the element having its id: elementId
		//		And places it in the end of the container.
		//
		//		Operator is created only if the element is not the first element.
		//
		// elementId: string
		//		id of the element for which the operator is to be created.
		//		"_operator" is appended to it, to create the operator's id.
		
		this._trace.entering(this.id, "_createElementOperator");
		// every phrase has an operator. except the first one
		if (this._firstElement &&  this._firstElement.id != elementId) {
			var operator = this._synchronizedSelector.createOperator(elementId+"_operator");
			this._elementOperators[elementId] = operator;
			
			// place in the DOM.
			dojo.place(operator.domNode, this.elementContainer, "last");
			// apply class
			// dojo.addClass(operator.domNode, "filterOperator");
		}
		
		this._trace.exiting(this.id, "_createElementOperator");
	},
	
	_destroyElement: function(elementId) {
		this._trace.entering(this.id, "_destroyElement");
		if (elementId in this._elementOperators) { // first element does not have its operator.
			this._synchronizedSelector.destroyOperator(this._elementOperators[elementId]);
			delete this._elementOperators[elementId];
		}
		
		// element itself
		this._elements[elementId].destroyRecursive();
		delete this._elements[elementId];
		this._activeElementCounter--;
		this._trace.exiting(this.id, "_destroyElement");
	}
	
});
