/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

(c) Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.membership._FilterGroupPhraseEntityValue");

dojo.require("dijit._Templated");

dojo.require("com.ibm.security.modeling.membership._FilterGroupPhraseFilteringValue");
dojo.require("com.ibm.security.modeling.util.Trace");

// define the widget class
dojo.declare (
	"com.ibm.security.modeling.membership._FilterGroupPhraseEntityValue", 
    [ com.ibm.security.modeling.membership._FilterGroupPhraseFilteringValue], {
	
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.membership._FilterGroupPhraseEntityValue"),
	
	_filterAttribute: "name",
	
	// _data: json object
	//		Reference to the selected attribute item
	
	searchDialog: null,
	
	startup: function() {
		// Summary:
		//		Widget life-cycle method.
		//		Starts up the widget, and its child widget.
		
		this._trace.entering(this.id, "startup");
		
		// create search phrase value button.
		this._createSearchButton();
		
		this.inherited(arguments);
		this._trace.exiting(this.id, "startup");
	},
	
	destroy: function() {
		//	Summary:
		//		Widget life-cycle method.
		//		Destroys this widget and is contents recursively.
		
		this._trace.enteringWithStack(this.id, "destroy", true);
		try {
			if (this._searchValueButton) {
				this._searchValueButton.destroyRecursive();
			}
			
			this.inherited(arguments);
			this._trace.exiting(this.id, "destroy");
		} catch(e) {
			this._trace.exception(e, "destroy");
			throw e;
		}
	},
	
	setAddAllowed: function(/*boolean*/ isAllowed, /*string*/ label) {
		// Summary:
		//		Overridden from _FilterGroupPhraseFilteringValue
		
		this.inherited(arguments);
		if (isAllowed === true) {
			label = this.localResources.SEARCH_CONDITION_VALUE;// default button label.
		}
		
		if(label) {
			// Update label
			this._setButtonLabel(this._searchValueButton, label);
		}
	},
	
	_render: function(/*json object*/ data) {
		// Summary:
		//		Overridden from _FilterGroupPhraseFilteringValue
		//
		// data:	json object
		//		Default data, to be populated in value widget.
		
		this._trace.entering(this.id, "_render");
		this._data = data;
		
		this.inherited(arguments);
		
		this._trace.exiting(this.id, "_render");
	},
	
	_createSearchButton: function() {
		// Summary:
		//		Creates an instance of button for search phrase value functionality
		
		this._trace.entering(this.id, "_createSearchButton");
		
		this._searchValueButton = new ibm.tivoli.tip.dijit.TIPButton({
			id: this.id+"_searchValue",
			onClick: dojo.hitch(this,function() {this._onValueSearch();}),
			showLabel: false,
			iconClass: "searchPhraseValueIcon",
			label: this.localResources.SEARCH_CONDITION_VALUE
			
		});

		// Attach the button widget to the DOM.
		dojo.place(this._searchValueButton.domNode, this._buttonContainer, "last");
		
		// invoke button's startup right away.
		this._searchValueButton.startup();
		
		this._trace.exiting(this.id, "_createSearchButton");
	},
	
	_disableWidget: function() {
		// Summary:
		//		Overridden from _FilterGroupPhraseFilteringValue
		this.inherited(arguments);
		if (this._searchValueButton) {
			this._disableButton(this._searchValueButton);
		}
	},
	
	_enableWidget: function() {
		// Summary:
		//		Overridden from _FilterGroupPhraseFilteringValue
		this.inherited(arguments);
		if (this._searchValueButton) {
			this._enableButton(this._searchValueButton);
		}
	},
	
	_onValueSearch: function() {
		// Summary:
		//		Searches for the value
		// 		Gets invoked when search value icon is clicked.
		
		this._launchSearchDialog();
	},
	
	_launchSearchDialog:function(){
		// summary:
		//			Launch the Search Dialog. 
		
		this._trace.entering("", "launchSearchDialog");
		/*
		 * Let the derivatives of this class take care of overriding this method 
		 * to launch a Dialog for search.
		 * 
		 * Nothing to do here. The warning is just to make sure the implementor has to 
		 * have extended class to launch a Dialog.
		 */ 
		
		this._trace.warning("No Search dialog found. Derivatives need to launch the search dialog.", "launchSearchDialog");
		this._trace.exiting("", "_launchCopyRolesDialog");
	},
	removeAllValues:function(){
		// summary:
		//			Removes all values from list.
		
		this._trace.entering("", "removeAllValues");
		/*
		 * Let the derivatives of this class take care of overriding this method 
		 * to remove all values for a Search Dialog.
		 * 
		 * Nothing to do here. 
		 */ 
		
		this._trace.exiting("", "removeAllValues");
	}
	
});