/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

(c) Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.membership._FilterGroupPhraseFilteringValue");

dojo.require("dijit._Templated");
dojo.require("dojox.data.JsonRestStore");
dojo.require("dijit.form.FilteringSelect");

dojo.require("ibm.tivoli.tip.dijit.TIPButton");

dojo.require("com.ibm.security.modeling.membership._FilterGroupPhraseValue");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.util.WidgetUtilities");

dojo.provide("com.ibm.security.modeling.tables.JsonRestStore");

dojo.require("dojox.data.JsonRestStore");

// define the widget class
dojo.declare (
	"com.ibm.security.modeling.membership._FilterGroupPhraseFilteringValue", 
    [ com.ibm.security.modeling.membership._FilterGroupPhraseValue, dijit._Templated], {
		
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.membership._FilterGroupPhraseFilteringValue"),
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.membership","templates/FilterGroupPhraseFilteringValue.html"),
	widgetsInTemplate: true,
	
	// _widgetUtilities: com.ibm.security.modeling.util.WidgetUtilities
	//		Utilities for widgets.
	// _widgetUtilities: null,
	
	// _filterConfig: json object
	//		configuration parameters for filtering select widget.
	
	// _selectedValues: array
	//		An array of items which should be excluded from result set.
	
	// _filterAttribute: string
	//		Attribute to do the filtering on.
	//		Same attribute is used for label, sort.
	_filterAttribute: "value",
	
	// _allowCustomValues: boolean
	//		Indicator, if user supplied custom values [values not from filtering select options list] are supported.
	//		true: supported.
	//		false: not supported. [default]
	_allowCustomValues: false,
	
	// _isAddAllowed: boolean
	//		Indicator is add item is allowed.
	_isAddAllowed: true,
	
	// idAttribute: string
	//		idAttribute for the json rest store for the filtering select.
	idAttribute: "uri",
	
	// Overrides _FilterGroupPhraseValue
	//		After a selection, more selections can be done.
	_isMultiValued: true,
	
	postMixInProperties: function() {
		// Summary:
		//		Widget life-cycle method. 
		
		this._trace.entering(this.id, "postMixInProperties");
		this.inherited(arguments);
		
		this._selectedValues = [];
		
		this._widgetUtilities = new com.ibm.security.modeling.util.WidgetUtilities();
		
		this._trace.exiting(this.id, "postMixInProperties");
	},
	
	destroy: function() {
		//	Summary:
		//		Widget life-cycle method.
		//		Destroys this widget and its contents recursively.
		
		this._trace.enteringWithStack(this.id, "destroy", true);
		try {
			if (this._filteringSelect) {
				dojo.disconnect(this._keyPressConnect);
				if (this._onsearchConnect) {
					dojo.disconnect(this._onsearchConnect);
				}
				if (this._onChangeConnect) {
					dojo.disconnect(this._onChangeConnect);
				}
				this._filteringSelect.destroyRecursive();
			}
			
			this.inherited(arguments);
			this._trace.exiting(this.id, "destroy");
		} catch(e) {
			this._trace.exception(e, "destroy");
			throw e;
		}
	},
	
	setAddAllowed: function(/*boolean*/ isAllowed, /*string*/ label) {
		// Summary:
		//		Sets the _isAddAllowed flag.
		// isAllowed: boolean
		//		- true: add will be allowed.
		//		- false: add will not be allowed.
		
		if (isAllowed === true || isAllowed === false) {
			// when boolean
			this._isAddAllowed = isAllowed;
			
			if (this._isAddAllowed) {	// Add is allowed.
				this._enableWidget();	// enable widget
				label = this.localResources.ADD_CONDITION_VALUE;
				
			} else {	// Add is not allowed.
				this._disableWidget(this.addValueButton);	// disable widget
			}
			
			if (label) {
				// Update label
				this._setButtonLabel(this.addValueButton, label);
			}
		}		
	},
	
	_render: function(/*json object*/ data) {
		// Summary:
		//		Overridden from _FilterGroupPhraseValue
		//
		// data:	json object
		//		Default data, to be populated in value widget.
		
		this._trace.entering(this.id, "_render");
		
		this._setupFilteringSelect(data);
		
		this._trace.exiting(this.id, "_render");
	},
	
	_empty: function() {
		// Summary:
		//		Overridden from _FilterGroupPhraseValue
		
		// reset the items to exclude.
		this._selectedValues.splice(0, this._selectedValues.length);
		this._clearFilteringSelectValue();
	},
	
	onValueAdd: function(/*json object*/ valueItem) {
		// Summary:
		//		Invoked when a value is selected from the available values.
		//		Value controller listens to it [see: FilterGroupPhraseValueController.js: addSelectedValue().].
		// 
		// valueItem: json object
		//		Value item to be considered as selected value.
		
		this._trace.entering(this.id, "onValueAdd");
		this._trace.exiting(this.id, "onValueAdd");
	},
	
	onAfterValueAdd: function(/*json object*/ valueItem) {
		// Summary:
		//		Performs logic when an item has been added as selected value.
		//		like: 
		//			- filter the item from available values options list.
		//			- clear the filtering select
		//		Called from value controller.
		//
		// valueItem: json object
		//		Value item which has been added as selected value.
		
		this._trace.entering(this.id, "onValueAdd");
		if (valueItem && valueItem[this.idAttribute]) {
			var isNew = true;
			for (var itemIndex in this._selectedValues) {
				if (valueItem[this.idAttribute] == this._selectedValues[itemIndex][this.idAttribute]) {
					isNew = false;
					break;
				}
			}
			if (isNew == true) {
				this._selectedValues.push(valueItem);
			}
		}
		
		this._clearFilteringSelectValue();
		this._trace.exiting(this.id, "onValueAdd");
	},
	
	onAfterValueRemove: function(/*json object*/ valueItem) {
		// Summary:
		//		Performs logic when an item has been removed from selected value [stack].
		//		like: 
		//			- show item in available values options list.
		//		Called from value controller.
		//
		// valueItem: json object
		//		Value item which has been removed from selected value [stack].
		
		this._trace.entering(this.id, "onAfterValueRemove");
		
		if (valueItem && valueItem[this.idAttribute] && this._selectedValues.length > 0) {
			for (var i=0; i < this._selectedValues.length; i++) {
				if (valueItem[this.idAttribute] == this._selectedValues[i][this.idAttribute]) {
					this._selectedValues.splice(i, 1);
					break;
				}
			}
		}
		
		this._trace.exiting(this.id, "onAfterValueRemove");
	},
	
	getValueAttribute: function() {
		// Summary:
		//		Provides string value which should be used to get display value from data item.
		return this._filterAttribute;
	},
	
	_onValueAdd: function() {
		// Summary:
		//		Gets invoked when add value icon is clicked.
		//		Or when Enter key is hit on Filter list input box. refer: _onFilterDijitClick
		
		try {
			this._trace.entering(this.id, "_onValueAdd");
			if (this._isAddAllowed === false) {
				// add is not allowed.
				this._trace.entering(this.id+" - Add is not allowed.", "_onValueAdd");
				return;
			}
			// get the current selected value item.
			var valueItem = this._getValueItem();
			
			if (!valueItem) {
				return;
			}
			
			// invoke onValueAdd() and supply the item to add.
			this.onValueAdd(valueItem);
		} finally {
			this._trace.exiting(this.id, "_onValueAdd");
		}
	},
	
	_getValueItem: function() {
		// Summary:
		//		If custom values are not supported then retrieves current valid value from filtering select.
		//		else custom value is used to construct the value item.
		//
		// returns
		//		null -  if filtering select is not available,
		//				no value is entered: ""
		//				or if value in filtering select is no valid, and custom values are not supported
		//				
		//		Item - 
		//				from filtering select
		//				Or json object constructed from the user input custom value.
		
		try {
			this._trace.entering(this.id, "_getValue");
			var valueItem = null;
			if (!this._filteringSelect) { 
				// filtering select is not created yet.
				return valueItem;	// null
			}
			
			var displayedValue = this._filteringSelect.get('displayedValue');
			if (displayedValue == "") {
				// No value is entered.
				return valueItem;	// null
			}
			
			if (this._allowCustomValues === false) {		
				// do not allow user custom values. Only valid values are allowed.
				if(!this._filteringSelect.isValid()) {		
					// Value should be valid, but it is not valid.
					return valueItem;	// null
				}
				
				// value is valid and it is from filtering options
				valueItem = this._filteringSelect.item;
			} else if (this._allowCustomValues === true){
				// user supplied, custom values are supported.
				if (this._filteringSelect.item) {
					// but entered value is not custom rather it is a valid option/value is selected.
					// valueItem is the selected option's item.
					valueItem = this._filteringSelect.item;
				} else {
					// A value which is not from the options list, is entered.
					// Construct the valueItem, using value attribute, and displayed value.
					valueItem = {};
					valueItem[this.getValueAttribute()] = displayedValue;
				}
			}
			
			return valueItem;
		} finally {
			this._trace.exiting(this.id, "_getValue");
		}
	},
	
	_clearFilteringSelectValue: function() {
		// Summary:
		//		Clears the value from the filtering select.
		this._trace.entering(this.id, "_clearFilteringSelectValue");
		var emptySelection = {};
		emptySelection[this._filterAttribute] = "";
		this._filteringSelect.set('item', emptySelection);
		this._trace.exiting(this.id, "_clearFilteringSelectValue");
	},
	
	_setupFilteringSelect: function(/*json object*/ data) {
		// Summary:
		//		Setup filtering select widget.
		//
		// data:	json object
		//		Default data, to be populated in value widget.
		
		this._trace.entering(this.id, "_setupFilteringSelect");
		
		// setup filtering config first.
		if (this._filteringSelect) {
			dojo.disconnect(this._keyPressConnect);
			this._filteringSelect.destroyRecursive();
		}
		
		this._setupFilteringConfig(data);
		this._filteringSelect = new dijit.form.FilteringSelect(this._filterConfig);
		
		// Place this filtering select in DOM.
		dojo.place(this._filteringSelect.domNode, this._listContainer, "only");
		
		// startup
		this._filteringSelect.startup();
		
		this._filteringSelect._isEmpty = dojo.hitch(this._filteringSelect, this._isEmpty);
		
		// Enter key should select the value.
		this._keyPressConnect = dojo.connect(this._filteringSelect.focusNode, "onkeypress", this, function(event) {this._onFilterKeyPress(event);});
		this._configureAddOnOptionClick();
		
		this._trace.exiting(this.id, "_setupFilteringSelect");
	},
	
	_configureAddOnOptionClick: function() {
		if (this._filteringSelect) {
			this._onsearchConnect = dojo.connect(this._filteringSelect, "_startSearch", this, function() {
				if (this._filteringSelect._popupWidget) {
					dojo.disconnect(this._onsearchConnect);
					this._onsearchConnect = null;
					
					this._onChangeConnect = dojo.connect(this._filteringSelect._popupWidget, "onChange", this, function(event) {
						this._onFilterPopUpOptionSelect(event);
					});
				}
			}); 
		}
	},
	
	_setupFilteringConfig: function(/*json object*/ data) {
		// Summary:
		//		Setup filtering config.
		//		Override this method to provide additional parameters.
		//
		// data:	json object
		//		Default data, to be populated in value widget.
		
		try {
			this._trace.entering(this.id, "_setupFilteringConfig");
			this._setupStore(data["uri"].toString());
			
			if (this._filterConfig) {
				// update store.
				this._filterConfig["store"] = this.store;
				return this._filterConfig;
			}
			
			this._filterConfig = {
				id: this.id+"_selectList",
				store: this.store,
				
				searchAttr: this._filterAttribute,
				labelAttr: this._filterAttribute,
				fetchProperties: {
					sort: [{attribute:this._filterAttribute,descending: false}]
				},
				query: {
					structure: "FLAT",
					content: "summary"
				},
				pageSize: 15,
				
				highlightMatch: "all",
				hasDownArrow: true,
				required: false,
				maxLength: 240,
				placeHolder: this.hintText,
				"class": "valueList"
			};
			
		} finally {
			this._trace.exiting(this.id, "_setupFilteringConfig");
		}
	},
	
	_setupStore: function(/*string uri*/ target) {
		// Summary:
		//		sets up store for filtering select.
		//		Creates store instance, if not created yet.
		// 
		// returns:
		//		store instance.
		
		try {
			this._trace.entering(this.id, "_setupStore");
			if (this.store) {
				delete this.store;
				// reset the items to exclude.
				this._selectedValues.splice(0, this._selectedValues.length);
			}
			
			
			this.store = new dojox.data.JsonRestStore({
				target: target,
				idAttribute: this.idAttribute,
				allowNoTrailingSlash: true
			});
		} finally {
			this._trace.exiting(this.id, "_setupStore");
		}
	},
	
	_onFilterKeyPress: function(/*Event*/ event) {
		// Summary:
		//		Function which gets invoked, when a key event takes place on Filtering Select widget.
		//		If enter key is hit then current value from filtering select should be added to phrase selected values list.
		this._trace.entering(this.id, "_onFilterKeyPress");
		var key = event.charOrCode;
		if ((key == dojo.keys.ENTER) && (this._isAddAllowed === true)) {
			// enter key is hit and add is allowed.
			this._trace.entering(this.id+" - Enter key is hit.", "_onFilterKeyPress");
			this._onValueAdd();
		}
		this._trace.exiting(this.id, "_onFilterKeyPress");
	},
	
	_onFilterPopUpOptionSelect: function(/*Event*/ event) {
		// Summary:
		//		Function which gets invoked, when a an Option is selected on Filtering Select widget.
		
		this._trace.entering(this.id, "_onFilterPopUpOptionSelect");
		this._onValueAdd();
		this._trace.exiting(this.id, "_onFilterPopUpOptionSelect");
	},
	
	_disableWidget: function() {
		// Summary:
		//		Overridden from _FilterGroupPhraseValue
		this.inherited(arguments);
		if (this._filteringSelect) {
			this._filteringSelect.set('disabled', true);
		}
		this._disableButton(this.addValueButton);
	},
	
	_enableWidget: function() {
		// Summary:
		//		Overridden from _FilterGroupPhraseValue
		this.inherited(arguments);
		if (this._filteringSelect) {
			this._filteringSelect.set('disabled', false);
		}
		this._enableButton(this.addValueButton);
	},
	
	_setButtonLabel: function(/*Button Widget*/ button, /*string*/label) {
		button.set("label", (label? label:""));
	},
	
	_enableButton: function(/*Button Widget*/ button, /*string*/label) {
		if (label) {
			// Update Label.
			this._setButtonLabel(button, label);
		}
		// Enable button.
		this._widgetUtilities.enableButton(button);
	},
	
	_disableButton: function(/*Button Widget*/ button, /*string*/label) {
		if (label) {
			// Update Label.
			this._setButtonLabel(button, label);
		}
		// Enable button.
		this._widgetUtilities.disableButton(button);
	},
	
	getSelectedValuesItems: function() {
		// Summary:
		//		Provides an array, containing currently selected items. 
		// returns:	array
		//		If no items then returns an empty array.
		
		// Selected items are available in this._selectedValues array.
		return this._selectedValues;
	},
	
	_isEmpty: function(value) {
		// Summary:
		//		This function overrides the default _isEmpty function for the
		//		Filtering Select widget so that white space is not ignored.
		// returns:	boolean
		//		true if the specified value is empty; otherwise false.
		var result = false;
		if (value.length == 0) {
			result = true;
		}
		return result;
	}
	
});