/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

(c) Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.membership._FilterGroupPhraseValue");
dojo.require("dijit._Widget");

dojo.require("com.ibm.security.modeling.util.Trace");

//NLS
dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling.membership", "FilterText");
dojo.requireLocalization("com.ibm.security.modeling","PluginMessages");

// define the widget class
// This abstracts common functionality among the widgets, which represent Filter Group Phrase Value
//
// Available sub types: 
//		- FilterGroupPhraseIntegerValue
//		- FilterGroupPhraseStringValue
// 		- FilterGroupPhraseIndentityValue
//		- FilterGroupPhraseHierarchyValue

dojo.declare (
	"com.ibm.security.modeling.membership._FilterGroupPhraseValue", 
    [dijit._Widget], {
		
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.membership._FilterGroupPhraseValue"),
	
	// _container: DOMNode
	//		Defined in template.
	//		Referenced via dojoAttachPoint
	
	// _type: String
	//		Type of the value: INTEGER, STRING, IDENTITY, HIERARCHY
	
	// _isMultiValued: boolean
	//		An indicator, to specify if multiple values are allowed for selection one by one
	//		False by default.
	_isMultiValued: false,
	
	
	// hintText: string
	//		placeHolder attribute for the input node.
	hintText: "",
	
	// globalResources: Object
	//			Reference to the global shared resource bundle
	// pluginResources: object
	//			Reference to the global shared plugin resource bundle
	// localResources: Object
	//			Reference to the resource bundle unique to this widget
	
	postMixInProperties: function() {
		this._trace.entering(this.id, "postMixInProperties");
		
		this.inherited (arguments);
		
		if (!this.globalResources) {
			this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		}
		
		if (!this.pluginResources) {
			this.pluginResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "PluginMessages");
		}
		
		if (!this.localResources) {
			this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.membership", "FilterText");
		}
		
		this._trace.exiting(this.id, "postMixInProperties");
	},
	
	getType: function() {
		return this._type;
	},
	
	render: function(/*json object|primitive value*/ data) {
		// Summary:
		//		Renders the widget.
		//		If data is not provided, then empties the widget.
		
		// data:	json object|primitive value
		//		Default data, to be populated in value widget.
		
		this._trace.entering(this.id, "render");
		if (data) {
			this._render(data);
		} else {
			this.empty();
		}
		this._trace.exiting(this.id, "render");
	},
	
	empty: function() {
		// Summary:
		//		Empties the widget.
		this._empty();
	},
	
	isEmpty: function() {
		// Summary:
		//		Checks if the widget is empty
		// returns: boolean
		//		true - if widget is empty
		//		false - if widget is not empty
		
		return this._isEmpty();
	}, 
	
	isMultiValued: function() {
		// Summary:
		//		To check if this widget supports more than one selections one after another.
		return this._isMultiValued;
	},
	
	disableWidget: function() {
		// Summary:
		//		Function to disable the value widget.
		this._disableWidget();
	},
	
	enableWidget: function() {
		// Summary:
		//		Function to enable the value widget.
		this._enableWidget();
	},
	
	_disableWidget: function() {
		// Summary:
		//		Function to disable the value widget.
		//		Sub types should override this method.
	},
	
	_enableWidget: function() {
		// Summary:
		//		Function to enable the value widget.
		//		Sub types should override this method.
		
	},
	
	_render: function(/*json object*/ data) {
		// Summary:
		//		Override this method, and have rendering logic in it.
		
		// data:	json object
		//		Default data, to be populated in value widget.
		//		Used when filter phrase existed, and filter UI is opened for editing.
	},
	
	_empty: function() {
		// Summary:
		//		Override this method, and have emptying logic in it.
	},
	
	_isEmpty: function() {
		// Summary:
		//		Override this method, and have tell if widget is empty
		return true;
	}
});