dojo.provide("com.ibm.security.modeling.mining.RoleMiningDialog");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Dialog");
dojo.require("com.ibm.security.modeling.widgets.IsrmDialogBase");
dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
dojo.require("com.ibm.security.modeling.widgets.FilteringWidget");
dojo.require("com.ibm.security.modeling.util.UrlUtilities");
dojo.require("com.ibm.security.modeling.widgets.ConfirmMessageDialog");
dojo.require("com.ibm.security.modeling.api.ProjectRestManager");
dojo.requireLocalization("com.ibm.security.modeling.mining", "RoleMiningDialogText");
dojo.requireLocalization("com.ibm.security.modeling", "PluginMessages");
dojo.requireLocalization("com.ibm.security.modeling.mining", "AlgorithmNameText");

dojo.require("com.ibm.security.modeling.mining.RoleMiningFactory");

dojo.declare("com.ibm.security.modeling.mining.RoleMiningDialog", com.ibm.security.modeling.widgets.IsrmDialogBase, {

	templatePath : dojo.moduleUrl("com.ibm.security.modeling.mining", "templates/RoleMiningDialog.html"),
	// provides utility method for various error handling , like
	// RestErrors
	
	// provides utility methods for various url handling
	_urlUtilities : new com.ibm.security.modeling.util.UrlUtilities(),
	widgetsInTemplate : true,
	heightFactor : 2,
	widthFactor : 2.3,
	minHeight : 510,
	minWidth : 700,
	helpFile : "ui_generate_roles.html",
	// Handle to the projects table store.
	projectsTableStore : null,

	constructor : function(args) {
		 this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
		 this.projectsTableStore = args.projectsTable.dataStore;
			// duration is 0 show that hide/show of progressBar do
			// not kill hide/show of this dialog
		this.progressBar = new com.ibm.security.modeling.widgets.ProgressBar( {
			duration : 0
		});
	},
	postMixInProperties : function() {
	
		this.inherited(arguments);
		this.localResources = this.localResources || dojo.i18n.getLocalization("com.ibm.security.modeling.mining", "RoleMiningDialogText");
		this.algorithmResource = this.algorithmResource || dojo.i18n.getLocalization("com.ibm.security.modeling.mining", "AlgorithmNameText");
		this.pluginResources =  dojo.i18n.getLocalization("com.ibm.security.modeling", "PluginMessages");
		this.dialogTitle = this.localResources.TITLE;
		this.dialogInstruction = this.localResources.INSTRUCTION;
	
	},
	postCreate : function() {
		this._trace.entering("", "postCreate");
		dojo.addClass(this.domNode, "contentArea");
		this.inherited(arguments);
		try {
			this.addFilteringWidget();
			
			this.okButton = this.addCommonButton( {
				onClick : dojo.hitch(this, this.onCancel),
				type : "CANCEL"
			});
			this.addCommonButton( {
				onClick : dojo.hitch(this, this.onOK),
				type : "OTHER",
				label : this.localResources.GENERATE_ROLES
			});
		} catch (e) {
			// Fails saying no item at backend
		this.noItems(true);
		this._trace.exception(e, "Error creating FilteringWidget");
	    }
		
	},
	startup : function() {
		this.inherited(arguments);
		this.filteringWidget.startup();
	},
	getMiningUrl : function() {
		var id = this._urlUtilities.getIdfromUri(this.projectUrl);
		return this.projectUrl.split(id)[0] + "/mining";
	},
	// add the filtering widget , getValue is overridden to
		// support internalization for algorithm/approaches name
		addFilteringWidget : function() {
	
			var stateStore = new dojox.data.JsonRestStore( {
				idAttribute : "name",
				target : this.getMiningUrl()
			});
	
			var self = this;
			var getValue = stateStore.getValue;
			stateStore.getValue = function(/* Object */item, /* String */
			property, /* value? */defaultValue) {
				var value = getValue(item, property, defaultValue);
				if (property == 'name') {
					if (value && value.toUpperCase() in self.algorithmResource) {
						return self.algorithmResource[value.toUpperCase()];
					}
				}
				return value;
			};
	
			this.filteringWidget = new com.ibm.security.modeling.widgets.FilteringWidget( {
	
				searchAttr : "name",
				store : stateStore,
				selectedValueCallback : dojo.hitch(this, this.selectedValueCallback),
				hookForFilteringSelect : this.id + "_FilteringSelect",
				hookForStackContainer : this.id + "_stackContainer",
				isOneItem : dojo.hitch(this, this.ifOneItem),
				hideFilteringSelectForOne : true,
				noItems : dojo.hitch(this, this.noItems),
				onError : dojo.hitch(this, this.noItems, true)
			});
		},
		noItems : function(isError) {
			// more Information like no Algorithm exist , can be
		// shown here
		if (isError) {
			if (this.parentShowMessage)
				this.parentShowMessage("critical", this.pluginResources.REST_GENERIC_INTERNAL_ERROR);
		}
		this.progressBar.hide();
	},
	ifOneItem : function() {
		dojo.addClass(this.filteringWidgetPane.domNode, "hiddenFilteringWidget");
	},
	onOK : function() {
	
		this.messageArea.hideMessages();
		var currentSelectedWidget = this.filteringWidget.getSelectedChild();
		var validationResult = currentSelectedWidget.validate();
		if (validationResult.valid) {
			var dataObject = currentSelectedWidget.getRoleGeneration();
			 this.objectToReturn = {
				patternName : this.filteringWidget.getFilteringSelectItem().name,
				configurationValues : dataObject
			};
			//this.sendToServer(this.objectToReturn);
                      this.projectRestManager = new com.ibm.security.modeling.api.ProjectRestManager({
			resourceUri: this._urlUtilities.removeQueryFromUri(this.projectUrl)+"/analyze",
			syncMode: true
			//_identifier:"storeId",
			//messageArea: this.msgArea
		});
	    var data = dojo.toJson(this.objectToReturn);
	    this.progressBar.show(this.localResources.ANALYSE_ROLES);
		this.projectRestManager.doHttpPost(
		   	data,
				dojo.hitch(this,function(analysisObj){
					this.progressBar.hide();
					this.analyseResult(analysisObj);
					}),
				dojo.hitch(this,function(error){
					this.progressBar.hide();
					this.analysisError(error);
				})
		  );

		} else {
	
			this.messageArea.showErrorMessage(validationResult.message);
		}
	
	},
        analysisError :function(error){
		this.hideDialogIfStateEnforcementError(error);
		this._errorUtil.handleRESTError(error, this.messageArea);
        },
	analyseResult :function(analysisObj){
		 if(analysisObj.feasibilityResult=="PASSED")
		   this.sendToServer(this.objectToReturn);
		else if(analysisObj.feasibilityResult=="WARNING"){
		      
			var confirmDialog = dijit.byId(this.id+"_importSessionValidateDialog");
			if(!confirmDialog){
				var confirmDialog = new com.ibm.security.modeling.widgets.ConfirmMessageDialog({
					   id:this.id+"_importSessionValidateDialog",
					   submitButtonText:this.globalResources.OK,
					   cancelButtonText:this.globalResources.CANCEL,
					   completeMessage:analysisObj.message||" ",
					   icon: dojo.moduleUrl("com.ibm.security.modeling.themes", "images/warning_large.png"),
					   title:this.localResources.CONFIRM_ROLE_GENERATE,
					   onSubmit: dojo.hitch(this,this.submitToServer)
					});
				confirmDialog.show();
				confirmDialog.startup();									
			}else{
				
				//Replacing the innerHTML text of the contentPane to display the updated session name.
				
				confirmDialog.setMessage(analysisObj.message||" ");
				confirmDialog.show();
			}			
		                             
		   }
		 else { 
			 
				var errorDialog = dijit.byId(this.id+"errorDialog");
				if(!errorDialog){
					   var errorDialog = new com.ibm.security.modeling.widgets.ConfirmMessageDialog({
						   id:this.id+"errorDialog",
						   submitButtonText:this.globalResources.OK,
						   cancelButtonText:this.globalResources.CLOSE,
						   completeMessage:analysisObj.message||" ",
						   icon: dojo.moduleUrl("com.ibm.security.modeling.themes", "images/critical.png"),
						   title:this.localResources.CONFIRM_ROLE_GENERATE
						});
			          errorDialog.show();
			          errorDialog.startup();
			          dojo.style(dijit.byId(this.id+"errorDialogSubmitButton").domNode,"display","none");								
				}else{
					
					//Replacing the innerHTML text of the contentPane to display the updated session name.
					
					errorDialog.setMessage(analysisObj.message||" ");
					errorDialog.show();
				}
			 
		 }
		},
		
	submitToServer:function(){
		  this.sendToServer(this.objectToReturn);
		},
	sendToServer : function(content) {
	
		this.progressBar.show(this.localResources.GENERATE_ROLE);
	
		try {
			var projectId = this._urlUtilities.getIdfromUri(this.projectUrl);
			var projectItem = this.projectsTableStore.fetchItemByIdentity( {
				identity : projectId
			});
	
			this.projectsTableStore.changing(projectItem);
	
			// Need to remove the project status constants to a
		// separate file where in they can be referred by
		// both the projects and Mining panels.
		// Status "j" implies ROLE_GENRERATION_QUEUED.
		this.projectsTableStore.setValue(projectItem, "status", "j");
		this.projectsTableStore.setValue(projectItem, "patternConfigurations", content);
		this.projectsTableStore.save( {
			syncMode : false,
			onComplete : dojo.hitch(this, this.loadCallback),
			onError : dojo.hitch(this, function(error) {
				this.progressBar.hide();
				this.hideDialogIfStateEnforcementError(error);
				this._errorUtil.handleRESTError(error, this.messageArea);
			})
		});
	} catch (e) {
		// This catch block is added to take care of any JS errors which may occur.
		// The block is added to hide the progress bar, else the UI becomes
		// unusable.
		this.progressBar.hide();
		this.messageArea.showErrorMessage("Error while Generating Roles");
	}
	},
	// once request is sent to server for mining , this will
		// poll for mining Status
		loadCallback : function(data) {
	
			dojo.require("com.ibm.security.modeling.mining.polling");
			var checkSuccessStatus = function(item) {
			//if the project status in "mining evaluating" or "mining queued" wait 
			//else mining has completed
				if((item.status == 'j'|| item.status == 'k' )){
					return false ; 
				}else{
					return true;
				}
				//return (item.status != 'j'&& item.status != 'k');
	
			};
			var poller = new com.ibm.security.modeling.mining.polling( {
				url : this.projectUrl,
				localResources : this.localResources,
				messageArea : this.messageArea,
				errBack : dojo.hitch(this, this.onError),
				sucBack : dojo.hitch(this, this.onSuccess),
				checkSuccessStatus : checkSuccessStatus
			});
			poller.poll();
		},
		onError : function(error, isError) {
			if (isError && (error.dojoType != "timeout")) {
					// show the error message and let the use to decide
					// what to do
					// a real error has occured
					this.progressBar.hide();
					this.hideDialogIfStateEnforcementError(error);
					this._errorUtil.handleRESTError(error, this.messageArea);
		  } else {
			// move to project panel , tell use we are waiting
				
				if (this.closePanel)
					this.closePanel();
				if (this.parentShowMessage)
					this.parentShowMessage("info", this.pluginResources.PLUGIN_MINING_IN_PROGRESS);
				this.progressBar.hide();
				this.hide();
		}
	},
	// onSucces of mining publish role_edit_event so that role
		// modeling table can refresh itself for newly generated
		// roles
		onSuccess : function(item) {
		   //we have got for what we were polling , but role generation has failed . 
	       if(item.status == 'l'){
	    	   if (this.closePanel) this.closePanel();
				if (this.parentShowMessage) this.parentShowMessage("critical", this.pluginResources.PLUGIN_REST_ROLE_MINING_TASK_SUBMISSION_FAILED);
				this.projectsTable.refresh();
	       }else{
	    	   dojo.publish("ROLE_EDIT_EVENT");
				
	       }
	       this.progressBar.hide();
		   this.hide();
	
		},
		destroy : function() {
			var errorDialog = dijit.byId(this.id+"errorDialog");
			if(errorDialog && errorDialog.destroy) errorDialog.destroy();
			var confirmDialog = dijit.byId(this.id+"_importSessionValidateDialog");
			if(confirmDialog && confirmDialog.destroy) confirmDialog.destroy();
			this.filteringWidget.destroy();
			this.inherited(arguments);
		},
		// validate the child widget , widget should have validate
		// and getRoleGeneration function
		_isValidChildWidget : function(widget) {
			var valid = true;
			if (widget.validate && widget.getRoleGeneration) {
				valid = dojo.isFunction(widget.validate) && dojo.isFunction(widget.getRoleGeneration);
			} else {
				valid = false;
			}
			return valid;
		},
		// gives back the widget for currently selected Algorithms
		// throw : Exception if invalid exception is found
		selectedValueCallback : function(value, item) {
			// if this callback is called means filteringWidget
			// store is updated .
			if (!this.filteringWidgetLoaded) {
				this.filteringWidgetLoaded = true;
				this.show();
			}
			var args = {};
			var widgetFactory = new com.ibm.security.modeling.mining.RoleMiningFactory();
			var widget = widgetFactory.getWidget(item, args);
			if (!this._isValidChildWidget(widget))
				throw new Error("Invalid widget : Widget much support validation and getRoleGeneration");
			return widget;
		},
		// show progress bar if loading of algorithm is still in progress , once
		// loades show the rolemining dialog , if error occurs no item is called
		show : function() {
			if (this.filteringWidgetLoaded === true) {
				this.progressBar.hide();
				this.inherited(arguments);
				this._loading = false;
			} else {
				if (!this._loading) {
					this.progressBar.show(this.localResources.LOADING);
					this._loading = true;
				}
	
			}
	
		}
	
	});