/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

dojo.provide("com.ibm.security.modeling.notebook.AttributeDialog");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Dialog");
dojo.require("com.ibm.security.modeling.notebook.AttributeDialogFactory");
dojo.require("com.ibm.security.modeling.util.ValidationUtilities");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.widgets.FilteringWidget");
dojo.require("com.ibm.security.modeling.widgets.IsrmDialogBase");
dojo.requireLocalization("com.ibm.security.modeling.notebook", "AttributeDialogText");
dojo.declare("com.ibm.security.modeling.notebook.AttributeDialog", com.ibm.security.modeling.widgets.IsrmDialogBase, {
	_trace : new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.notebook.AttributeDialog"),
	templatePath : dojo.moduleUrl("com.ibm.security.modeling.notebook", "templates/AttributeDialog.html"),

	widgetsInTemplate : true,
	validator : new com.ibm.security.modeling.util.Validator(),
         _messageUtil : new com.ibm.security.modeling.util.MessageUtilities(),
	heightFactor : 1.4,
	widthFactor : 1.3,
	minHeight : 365,
	parentWidget : null,
	editMode:false,
	helpFile:"",
	
	attributeMap : {
	   attributeLabel : {node : "attributeLabelNode", type : "innerText"}
		
	},

	constructor : function() {
		this._trace.entering("", "constructor");
		this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.notebook", "AttributeDialogText");
		this._trace.exiting("", "constructor");

	},
	postMixInProperties:function(){
		this.inherited(arguments);
		this.stringUtil = new com.ibm.security.modeling.util.StringUtils();
		if (this.editMode) {
			if(this.item && this.item.attributeName.toUpperCase() in this.attributeResource){
				this.attrName=this.attributeResource[this.item.attributeName.toUpperCase()];
                           
			} else{
                               this.attrName=this.item.attributeName;
                        }
			this.dialogTitle= this._messageUtil.formatMessage(this.localResources.EDIT_ATTRIBUTE_TITLE, [this.attrName]);
			this.dialogInstruction=this.localResources.ATTRIBUTE_DIALOG_INSTRUCTION_EDIT;
			this.helpFile="ui_attributes_edit.html";
			
			
		}else{
			this.dialogInstruction=this.localResources.ATTRIBUTE_DIALOG_INSTRUCTION;
			this.dialogTitle=this.localResources.ATTRIBUTE_DIALOG_TITLE;
			this.helpFile="ui_attributes_add.html";
		}
		
	},
	postCreate : function() {
		this._trace.entering("", "postCreate");
		dojo.addClass(this.domNode, "contentArea");
		this.inherited("postCreate", arguments);
		if(this.editMode) {
			this.changeTextForEdit();
			this.addWidgetForEditMode();
		}
		else {
			this.changeText();
			this.addFilteringWidget();
		}
		this.addCommonButton({
			onClick:dojo.hitch(this, this.onCancel),
			type:"CANCEL"
		});
		this.okButton = this.addCommonButton({
			onClick:dojo.hitch(this, this.onOK),
			type:"OK"
		});
		
		

	},
	
	startup:function(){
		// tags:
		//		protected
		this.inherited(arguments);
		if (this.filteringWidget && !this.filteringWidget._started) {
			this.filteringWidget.startup();
		}
	},
	
	changeTextForEdit : function(){
	  //  var editValue="<table><tr><td><label for='Attribute' style='font-weight: bold;'>"+this.localResources.ATTRIBUTE_LABEL+"</label></td><td><span id='AttributeName' dojoAttachPoint='AttributeNameNode'></span></td></tr><tr id='currentRow'><td><label for='Current' style='font-weight: bold;'>"+this.localResources.CURRENT_VALUE+"</label></td><td><span id='CurrentValue' dojoAttachPoint='CurrentValueNode'></span></td></tr><tr><td><label for='NewValue' style='font-weight: bold;' id='NewValue' class='requiredField' >"+this.localResources.ATTRIBUTE_LABEL+"</label></td></tr></table>";
		 var editValue="<div class='padBottom'>" +
					          "<label for='Attribute' class='padLeft'>"+this.localResources.ATTRIBUTE+"</label>" +
					          "<span id='AttributeName' dojoAttachPoint='AttributeNameNode'></span>" +
					   "</div>" +
					   "<div id='currentRow' class='padBottom'>" +
					          "<label for='Current'  class='padLeft'>"+this.localResources.CURRENT_VALUE+"</label>" +
					          "<span id='CurrentValue' class='boldtext' dojoAttachPoint='CurrentValueNode'></span>" +
				       "</div>" +
				       "<div>" +
					          "<label for='NewValue'  id='NewValue' class='requiredField' >"+this.localResources.ATTRIBUTE_LABEL+"</label>" +
			   	       "</div>";
        dojo.place(editValue,this.contentdiv,"replace");
        this.placeValues(this.item);
	},
	placeValues : function(item){
         
        dojo.attr(  dojo.byId("AttributeName"),{innerHTML:this.stringUtil.parseHTMLTag(this.attrName)});
        if (item && item.type == "IDENTITY") {
           	dojo.attr(  dojo.byId("CurrentValue"),{innerHTML:this.stringUtil.parseHTMLTag(item.name)});  
		   	dojo.attr(  dojo.byId("NewValue"),{innerHTML:this.localResources.NEW_VALUE}); 
		} else if (item && item.type == "HIERARCHY") {
        	dojo.attr(  dojo.byId("CurrentValue"),{innerHTML:this.stringUtil.parseHTMLTag(item.name)});  
		    dojo.attr(  dojo.byId("NewValue"),{innerHTML:this.localResources.NEW_VALUE}); 
		} else {
		  	dojo.byId("currentRow").className="hideClass"; 
		    dojo.attr(  dojo.byId("NewValue"),{innerHTML:this.localResources.VALUE}); 
		}
    },
	addFilteringWidget : function() {

		var stateStore = new dojox.data.JsonRestStore( {
			target : this.attributeUri
		});

		var self = this;
		var getValue = stateStore.getValue;
		stateStore.getValue = function(/*Object*/item, /*String*/property, /*value?*/defaultValue) {
			var value = getValue(item, property, defaultValue);
			if (property == 'attributeName') {
				if (value && value.toUpperCase() in self.attributeResource) {
					return self.attributeResource[value.toUpperCase()];
				}
			}
			return value;
		};

		this.filteringWidget = new com.ibm.security.modeling.widgets.FilteringWidget( {
			searchAttr : "attributeName",
			store : stateStore,
			selectedValueCallback : dojo.hitch(this, this.selectedValueCallback),
			hookForFilteringSelect : this.id + "_FilteringSelect",
			hookForStackContainer : this.id + "_stackContainer",
			defaultName:(this.item && this.item.attributeName),
			onSelection : dojo.hitch(this, this.changeText),
			hideFilteringSelect:this.editMode,
			noItems : dojo.hitch(this, this.setItemFoundFromCallback),
			dropDownLabelNode: this.attributeLabelNode

		});

	},
	//edit mode
	//in place of filtering widget we will get attribute name , in place of stackContainer we should directly place the widget meeant for this attribute
	addWidgetForEditMode:function(){
		//requires : same what is required by selectedValueCallback : which need value , item : value is id for attribute 
		
		//TODO get the value and item 
		this.editModeWidget = this.selectedValueCallback(this.item.attributeId,this.item);
		if(this.editModeWidget.setEditValue)
			 this.editModeWidget.setEditValue(this.item);
		//placed the widget at correct place 
		dojo.place(this.editModeWidget.domNode, this.id + "_stackContainer", "replace");
		
	},
	
	setItemFoundFromCallback : function() {
		
		this.noItemFound = true;
	},
	show : function() {

		if(this.editMode){
		//	${localResources.ATTRIBUTE_LABEL}
			this.set("attributeLabel",this.localResources.ATTRIBUTE_EDIT_LABEL + this.item.attributeName);
			this.TableDialog.show();
		}
		else if (this.noItemFound === true) {
			
			dojo.publish(dojo.config.panels.MESSAGE_DIALOG.launchEventAddress,[this.pluginResources.NO_ATTRIBUTE_DESCRIPTION,"","Warning",dojo.hitch(this,function(){
				this.hide();
				})]);
														
			this.noItemFound = false;
		} else if (this.noItemFound === false){
			this.set("attributeLabel",this.localResources.ATTRIBUTE_LABEL);
			this.TableDialog.show();
		}	
		else {
			//wait for store to complete the loading 
		//can show the loading symbol here 
		setTimeout(dojo.hitch(this, this.show), 100);
	    }
	},
	selectedValueCallback : function(value, item) {
	//current value which is selected
		//this.value = value ;
		this.attributeDialogFactory = this.attributeDialogFactory || new com.ibm.security.modeling.notebook.AttributeDialogFactory();
		try{
			this.widgetFactory = this.attributeDialogFactory.getWidgetFactory(item, {
				id:this.id,
				msgArea : this.messageArea,
				editMode:this.editMode,
				storeUri : this.storeUri
			});
			return this.widgetFactory.getWidget(value);
		}catch(e){
			this._trace.exception(e, "Error while creatign widget");
		}
	
	},
	changeText : function(value, item) {
	
		this.messageArea.hideMessages();
		//test approach to solve the race condition
		if (value)
			this.noItemFound = false;
		if (this.filteringWidget) {
			var dataObject = this.extractValue(this.filteringWidget.getSelectedChild(), this.filteringWidget.getFilteringSelectItem());
			var isValid = this.validate(dataObject, {
				 "EMPTYNESS":true
			});
			this.highLightField(dataObject.type, isValid.valid);
			if (!isValid.valid) {
	
				this.messageArea.hideMessages();
				this.messageArea.showErrorMessage(isValid.message);
			}
		}
	
		var innerHTML;
		var node = dojo.create("label", null, this.textValue, "only");
		dojo.addClass(node, "label requiredField");
	
		if (item && item.type == "IDENTITY") {
			dojo.attr(node, {
				innerHTML : this.localResources.IDENTITY_LABEL
			});
		} else if (item && item.type == "HIERARCHY") {
			dojo.attr(node, {
				innerHTML : this.localResources.HIERARCHY_LABEL
			});
		} else if (item && item.type == "STRING") {
			dojo.attr(node, {
				innerHTML : this.localResources.STRING_LABEL
			});
		} else {
			dojo.attr(node, {
				innerHTML : this.localResources.INTEGER_LABEL
			});
		}
	
	},
	
	highLightField : function(type, valid) {
		var widget = this.getCurrentSelectedWidget();
		if (!valid && (type == "INT" || type == "STRING"))
			widget.setStatus( {
				state : widget.constants.ERROR
			});
		if (valid && (type == "INT" || type == "STRING"))
			widget.setStatus( {
				state : widget.constants.REQUIRED
			});
	
	},
	
	destroy : function() {
		this._trace.entering("", "destroy");
		if (this.noItemConfirmation)
			this.noItemConfirmation.destroyRecursive();
		if(this.TableDialog) this.TableDialog.destroyRecursive();
		if(this.filteringWidget) this.filteringWidget.destroy();
		this.inherited(arguments);
		this._trace.exiting("", "destroy");
	},
	
	extractValue : function(widget, item) {
		if (!widget)
			return;
		//return widget.extractValue(item);
		return widget.extractValue();
	
	},
	/*
	 * Validates the dataObject so that it can be passed to attributePanel,currently validation is off
	 * validation is done only for int/string
	 */
	//byPassTest expect an arvalueToValidateray 
	validate : function(dataObject, byBassTest) {
			byBassTest = byBassTest || [];
			
			return this.getCurrentSelectedWidget().widgetValidator(byBassTest);
		},
		getCurrentSelectedWidget : function(){
			if(this.editMode) return this.editModeWidget;
			return this.filteringWidget.getSelectedChild();
		},
	    getItemforSelectedWidget :function(){
			if(this.editMode) return this.item;
			return  this.filteringWidget.getFilteringSelectItem();
		},
		onOK : function() {
	
				//hide the parent messages
			this.parentWidget.messageArea.hideMessages();
			var dataObject = this.extractValue(this.getCurrentSelectedWidget(), this.getItemforSelectedWidget());
			
			
			var validationResult = this.validate(dataObject);
			if (validationResult.valid) {
				var args={}
				if(this.editMode){
					//TODO : problem detected in post request to server , id is not returned , post and get request are not similar
					args={editMode:true,item:this.item};
					this.parentWidget.editAttributeValue(dataObject, args);
				}else{
					this.parentWidget.addAttributeValue(dataObject,args);
				}
				
				this.hide();
			} else {
				this.highLightField(dataObject.type, false);
				this.messageArea.hideMessages();
				this.messageArea.showErrorMessage(validationResult.message);
			}
	
	}
	
});