/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

� Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.notebook.AttributePanel");

dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
dojo.require("com.ibm.security.modeling.layouts.TemplatedPanelWithLayout");
dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling.notebook", "AttributePanelText");
dojo.require("com.ibm.security.modeling.tables.AttributeTable");
dojo["requireLocalization"]("customizations", "AttributeCustomization");
dojo.declare(
				"com.ibm.security.modeling.notebook.AttributePanel",
				[ com.ibm.security.modeling.layouts.TemplatedPanelWithLayout ],
				{
				    templatePath : dojo.moduleUrl("com.ibm.security.modeling.notebook","templates/AttributePanel.html"),
					widgetsInTemplate : true,
					hyperLinkIdentities:true,	
					_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.notebook.AttributePanel"),
					
					//The RoleNotebookDialogView should be set as the parent widget of this panel. 
					// This is done during the initialization of the AttributePanel. 
					parentWidget : null,
				    needToolBar : true,
				    constructor:function(){
					this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
				    },
				    
					
					postMixInProperties:function(){
					    //this is done so that localResources, attributeResources can be applied from outside
					    this.localResources = dojo.mixin( dojo.i18n.getLocalization("com.ibm.security.modeling.notebook","AttributePanelText"),this.localResources||{});
					    this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
					    this.attributeInstruction = this.attributeInstruction||this.localResources.PANEL_INSTRUCTIONS;
						//Safety measure if file do not exist 
						try{
							
							this.attributeResource=dojo.mixin(dojo.i18n.getLocalization("customizations", "AttributeCustomization"),this.attributeResource||{});
						}
						catch(err){
							this.attributeResource={};
							this._trace.exception(err, "constructor");
						}
					},
					_setUpToolbar : function() {

                    		this.table.addToolbarButton(this.localResources.ADD_ATTRIBUTE, false, "addIcon", dojo.hitch(this, this.onAddSelected), this.table.id + "_addFile", true,
                    				true, true);
                    		this.table.addToolbarButton(this.localResources.DELETE_ATTRIBUTE, false, "removeIcon", dojo.hitch(this, this.onDeleteSelected),
                    				this.table.id + "_removeFile", true, true, false);
                    				//TODO
                    		this.table.addToolbarButton(this.localResources.EDIT_ATTRIBUTE, false,"editIcon", dojo.hitch(this,this.onEditSelected,null),this.id + "_edit", true, false, false);
                    		this._trace.exiting("", "_setUpToolbar");

	                },
					postCreate : function() {
						this._trace.entering("", "postCreate");
						this.inherited(arguments);
						dojo.addClass(this.domNode, "fullSize");							
						this._trace.exiting("", "postCreate");
					},
					destroy : function() {
						this.table.destroy();
						this.inherited(arguments);			
					},
					destroyPrevTableInstances: function() {
						this._trace.entering(this.id, "destroyPrevTableInstances");

						var oldAttributeTable = dijit.byId(this.id+"_attributeTable");
						if (oldAttributeTable) {
							oldAttributeTable.destroyRecursive();
						} 
						//creating a div so that table can be recreated at same div location
						this.TableContainer.set("content", "");
						var newdiv = document.createElement('div');
						newdiv.setAttribute('id',this.id+"_attributeTable");
						this.TableContainer.domNode.appendChild(newdiv);
						this._trace.exiting(this.id, "destroyPrevTableInstances");		
					},	
					//This method is needed for rolenotebookdialog which populate the panel when  this tab is selected
					populate:function(entityItem){
					    this.startup();
					    this.storeUri=entityItem.attributeUri?entityItem.attributeUri:this.storeUri;
					    this._setUpTable();
					    if (this.needToolBar)  {
					    	this._setUpToolbar();
					    }else{
					    	dojo.style(this.table.header.domNode ,"display","none");
					    }
						this.table.startup();
					},
					_setUpTable:function(){
					  this.table = new com.ibm.security.modeling.tables.AttributeTable(
                        		{
                        			id : this.id +"_attributeTable",
                        			needToolBar:this.needToolBar,
                        			storeUri:this.storeUri,
                        			parentWidget:this,
                        			attributeResource:this.attributeResource,
                        			hyperLinkIdentities:this.hyperLinkIdentities,
                        			messageArea: this.messageArea,
                        			localResources : this.localResources
                        		},dojo.byId(this.id+"_attributeTable"));
                        		dojo.addClass(this.table.domNode,"contentArea");  
                      },
                      
    					reBuild: function(entityItem) {
    						this.destroyPrevTableInstances();
    						this.populate(entityItem);
    					},
					getIdfromUri:function(uri){
					         //pattern extracts the id from uri , pick number which occurs at the end of string
                                                  var patt = new RegExp( "[0-9]*$", "g");
                                                  var resultId=patt.exec(uri);
                                                  return resultId;
					},
					editAttributeValue:function(attribute, args){
						//var  itemToEdit = this.getStore().fetchItemByIdentity({identity:args.id});
						var  itemToEdit = args.item;
						this.getStore().changing(itemToEdit);
						
						for(var i=0;i<attribute.value.length;i++){
						    var type=attribute.type;
						    if(type=="HIERARCHY"){
							      this.getStore().setValue(itemToEdit, "nodeuri", attribute.value[i].uri);
							      this.getStore().setValue(itemToEdit, "name", attribute.value[i].name);
							}else if(type=="IDENTITY"){
								      this.getStore().setValue(itemToEdit,"name",attribute.value[i].name);
								      this.getStore().setValue(itemToEdit, "identityURI", attribute.value[i].uri);
							}else{
								     this.getStore().setValue(itemToEdit, "value", attribute.value[i]);
					      	}
						}
						
					      
					      var eventData = {
							property: 'attributeValues', 
							activity: 'edit'
            					};
						this.parentWidget.showSaveInProgress();
						this.table.saveOnStore();
						
					},
					//args will have many thing 
					addAttributeValue:function(attribute,args){
					  args=args||{};
					  var type=attribute.type;
					  var saveNeeded=false;
					  //keeping this to remove or between name and value
					 // var valueToSetforEdit ;
					  for(var i=0;i<attribute.value.length;i++){
					      var item ={};
					      item.attributeId = attribute.attributeId;
					      item.attributeName= attribute.attributeName;
					      item.type=type;
					      if(type=="HIERARCHY"||type=="IDENTITY"){
					            
					          var id = attribute.value[i].id|| this.getIdfromUri(attribute.value[i].uri);
						      item.name=attribute.value[i].name;
						      if(type=="HIERARCHY"){
						      	item.nodeuri=attribute.value[i].uri;
						      } 
						      else {
						      	item.identityURI=attribute.value[i].uri;
						     }
					      }else{
						      item.value=attribute.value[i];
						  }
					  	this.getStore().newItem(item);
				          
				           var eventData = {
        							property: 'attributeValues', 
        							activity: 'add'
        						};
					      
				          
				          
				          saveNeeded=true;
				          
					  }
					  if(saveNeeded)this.parentWidget.showSaveInProgress();
					 
						this.table._isRefreshCallReq=true;
				      this.table.saveOnStore();
					 
					},
					
					//dataObject expects [{attribute:  , value},..............]
					//value is an array of values for a given attribute Object
					addItems:function(dataObject){
					    
					    dojo.forEach(dataObject,dojo.hitch(this,this.addAttributeValue));
					    
					},
					
					onAddSelected : function(){
						this.openAttributeDialog();
					   
					},
					openAttributeDialog:function(args){
					    dojo.require("com.ibm.security.modeling.notebook.AttributeDialog");
					    args=args||{};
						var modelsIndex = this.storeUri.indexOf("models");
						var schemaUri = this.storeUri.substring(0, modelsIndex);
						schemaUri = schemaUri + "schema/roles/attributes";
						var argument={
    						//attributeUri:this.storeUri.replace(/[0-9]*$/i,""),
    						attributeUri: schemaUri,
    						id:this.id+"_attributeDialog",
    						attributeResource:this.attributeResource,
    						parentWidget:this,
    						contextRoot : this.contextRoot,
    						storeUri:this.storeUri
    						
						};
						args=dojo.mixin(argument,args);
						var filteringSelect= new com.ibm.security.modeling.notebook.AttributeDialog(
						args,"dialog");
					
						filteringSelect.startup();
						filteringSelect.show();
					},
					onEditSelected:function(item){
						  if(!item){
						        item = this.table.grid.selection.getSelected();
	                            if(item.length==1)item=item[0];
							    else item={};
	                       }
	                    this.table.deselectAll();
					    this.openAttributeDialog({item:item,editMode:true});
					},
					onDeleteSelected : function(){
						var selectedRows = this.table.getSelectedItems();
						this.parentWidget.showSaveInProgress();
						this.table.deselectAll();
						dojo.forEach(selectedRows, dojo.hitch(this, function(selectedItem) {
								if (selectedItem !== null) {		
									this.getStore().deleteItem(selectedItem);
								} 
						}));
												
						var eventData = {
							property: 'attributeValues', 
							activity: 'remove'
						};					
						
						this.table._isRefreshCallReq=true;
				         this.table.saveOnStore();
   
				 },
				 getStore:function(){
				     return this.table.getStore();
				 },
				 hideSaveInProgress:function(){
				     this.parentWidget.hideSaveInProgress();
				 }
				 
					
					
				    												
});

