/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.notebook.RoleNotebookDialogView");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Dialog");
dojo.require("com.ibm.security.modeling.dijit.layout.TabContainer");
dojo.require("com.ibm.security.modeling.dijit.layout.ContentPane");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.ComboBox");
dojo.require("dijit.form.Select");
dojo.require("dijit.form.Textarea");
dojo.require("dijit.form.FilteringSelect");
dojo.require("dojox.data.JsonRestStore");
dojo.require("com.ibm.security.modeling.widgets.MultiMessageArea");
dojo.require("com.ibm.security.modeling.dijit.layout.BorderContainer");
dojo.require("com.ibm.security.modeling.projects.BreadCrumb");
dojo.require("com.ibm.security.modeling.layouts.TemplatedPanelWithLayout");
dojo.require("com.ibm.security.modeling.notebook.RoleNotebookGeneralPanel");
dojo.require("com.ibm.security.modeling.notebook.RoleNotebookErrorWarningPanel");
dojo.require("com.ibm.security.modeling.notebook.RoleNotebookHierarchicalRelationPanel");
dojo.require("com.ibm.security.modeling.notebook.RoleNotebookPermissionsPanel");
dojo.require("com.ibm.security.modeling.notebook.SeperationOfDutyPanel");
dojo.require("com.ibm.security.modeling.widgets.ProgressBar");
dojo.require("com.ibm.security.modeling.util.ValidationUtilities");
dojo.require("com.ibm.security.modeling.breadcrumbs.BreadCrumInterface");
dojo.require("com.ibm.security.modeling.indicator.SaveIndicator");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.util.TitleUtilities");
dojo.require("com.ibm.security.modeling.util.HelpUtilities");
dojo.require("com.ibm.security.modeling.util.ConsoleUtilities");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");

dojo.require("com.ibm.security.modeling.api.RoleRestManager");
// NLS
dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling", "LogMessages");
dojo.requireLocalization("com.ibm.security.modeling", "PluginMessages");
dojo.requireLocalization("com.ibm.security.modeling.notebook", "RoleNotebookDialogText");
dojo.requireLocalization("com.ibm.security.modeling", "EnumLabels");


// define the widget class
dojo.declare (
	"com.ibm.security.modeling.notebook.RoleNotebookDialogView", 
    [ com.ibm.security.modeling.layouts.TemplatedPanelWithLayout ,com.ibm.security.modeling.breadcrumbs.BreadCrumInterface],
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.notebook.RoleNotebookDialogView"),
	validator : new com.ibm.security.modeling.util.Validator(),
        _messageUtil : new com.ibm.security.modeling.util.MessageUtilities(),	
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.notebook","templates/RoleNotebookDialogView.html"),

	// homeUrl: String
	// 			Points to the home page of our product.
	homeUrl: null,
	
	// designUrl: String
	// 			Points to the Projects/Design Page of our product.
	designUrl: null,

	widgetsInTemplate: true,
	
	// namespace: [readonly] String
	// 			Unique identifier to the instance of portlet page where this item will exist
	namespace: "",
	
	// contextRoot: String
	//			The context root of the server from which Content is being served.
	contextRoot: "",
	
	// roleUri: string
	//		String containing uri of Role being worked upon.
	roleUri: null,
	
	// isNewRole: boolean
	//		Boolean indicator to specify whether this widget is invoked for creating a new role.
	isNewRole: false,
	
	// populateAllPanels: boolean
	//		Boolean indicator to specify if all panels are to be checked for populating OR not.
	populateAllPanels: true,
	
	// pageToLaunch: Integer
	//		Indicates the page to be launched at a given point of time, when working through the
	//		BreadCrumbs
	pageToLaunch: -1,
	
	// roleNameLimit: Integer
	// Maximum number of characters that we display in the notebook, for the roleName		
	roleNameLimit: 35,

	// projectsLimit: Integer
	// 		Maximum number of characters that we display in the notebook, 
	//		for the Projects Link in the breadcrumbs.
	projectsLimit: 7,

	// homeLimit: Integer
	// 		Maximum number of characters that we display in the notebook, 
	//		for the Home Link in the breadcrumbs.			
	homeLimit: 4,
	
	// rolesAndPoliciesLimit: Integer
	// 		Maximum number of characters that we display in the notebook, 
	//		for the Roles and Policies Link in the breadcrumbs.		
	rolesAndPoliciesLimit: 10,
	
	// progressBar: String
	//		The widget used to display the progress of various taks. For e.g. the progress of
	// 		the role save operation.
	progressBar: null,

	// progressBarShown: boolean
	//		Indicator of whether the progress bar is already shown.
	progressBarShown: false,
	
	// titleUtil: com.ibm.security.modeling.util.TitleUtilities
	//			sets the portlet title to the current panel's	
	titleUtil: new com.ibm.security.modeling.util.TitleUtilities(),
	// helpUtil: com.ibm.security.modeling.util.HelpUtilities 
	//			sets current help file
	helpUtil: new com.ibm.security.modeling.util.HelpUtilities(),
	
	globalpanelId : dojo.config.panels.ROLE.id,
	
	_ignorePopulateOnTabChange: false,
	
	//roleLifecycleProvider: com.ibm.security.modeling.lifecycle.LifecycleProvider 
	//			Reference to the lifecycle Provider
	roleLifecycleProvider : null,
	
	constructor: function() {
		this._trace.entering("", "constructor");
		
		// globalResources: Object
		//			Reference to the global shared resource bundle
		// logResources: Object
		//			Reference to the log shared resource bundle
		// pluginResources: Object
		//			Reference to the plugin shared resource bundle
		// localResources: Object
		//			Reference to the resource bundle unique to this widget
		// connection: Object[]
		//		summary: The array of dojo method connections for this widget
		// error: object
		//		An object that holds reference to error object. 
		// roleRestManager: com.ibm.security.modeling.api.RoleRestManager
		//		An instance of com.ibm.security.modeling.api.RoleRestManager. 
		//		It is an interface for HTTP REST communication using dojo stores api. 
		// roleItem: object
		//		An object representing a Role for 'roleUri' in dojo store.
		
		
		this.error=null;
		this.roleItem= null;	// role item in store.
		this.roleRestManager= null;
		
		this.inherited (arguments);
		this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		this.logResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "LogMessages");
		this.pluginResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "PluginMessages");
		this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.notebook", "RoleNotebookDialogText");
		
		this.enumLabelsResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "EnumLabels");
		
		// Setup links for the bread-crumbs.
		/* Am putting the homeUrl explicitly here. If this is not done, then while launching to the Home Page,
		 * the Projects Portlet is resetting back to the Projects page, before launching the Home Page. */
		this.homeUrl =   "navigation.do?pageID=com.ibm.security.modeling.console.home.navigationElement&moduleRef=com.ibm.security.modeling.console.portlets.SecurityRoleModeler.4aa2f72672&ISC.TASKTYPE=0";		

		// Event mechanism.
		this.isRoleEdited = false;
		this.panelIds = ["general", "hierarchy", "membership", "permissions", "SODTab", "errorWarning", "attributeTab","lifecycleRequest"];
		this._initPanelDirtyState();
		this._initPanelDirtyStateForSure();
		this._initPanelReBuildState();
		this._initPropertyUpdate();
		
		this.progressBar = new com.ibm.security.modeling.widgets.ProgressBar();		
		
		this._trace.exiting("", "constructor");
	},

    _updateTabListStyle: function() {
        // On non-Windows systems, there seems to be an issue with the TabList. The text in each Tab heading (General, Permissions etc.)
        // seems missing when a console deployed on AIX, is accessed via Internet Explorer from a desktop. This happens especially
        // in the BiDi mode.
        // The fix is to update the styles of the tablist headers, such that they don't carry the position attribute.
        this._trace.entering(this.id, "_updateTabListStyle");
        var childTabs = this.tabContainer.tablist.getChildren();

		if(childTabs) {
		   var loopCtr = 0;
		   for (loopCtr = 0; loopCtr < childTabs.length; loopCtr++) {
				var childTabTitle = childTabs[loopCtr].titleNode;
				if(childTabTitle) {
				   var innerDivNode = childTabTitle.children[0];
				   dojo.removeClass(innerDivNode,"dijitTabInnerDiv");
				   dojo.addClass(innerDivNode,"notebookTabInnerDiv");
				}				
			}
		}

        this._trace.exiting(this.id, "_updateTabListStyle");
    },

	postCreate: function() {
		this._trace.entering(this.id, "postCreate");
		
		//dojo.addClass(this.domNode,"fullSize");
		dojo.addClass(this.domNode,"contentArea");		
		
		this.inherited ("postCreate", arguments);
		
		try {
			// Setup REST store.
			this._setupRestStore();
			
			if (this.roleUri == null) {	
				// Editor is invoked for creating a new Role
				this.isNewRole = true;
			}
			
			if(this._isErrorWarningPanelFirst){
				console.log("create error");
				// Create Role Editor's Errors and Warnings Panel
				this.errorWarningPanel=this._createErrorWarningPanel();
			}else{
			// Create Role Editor's General Panel
				this.generalPanel=this._createGeneralPanel();
			}
			
			this.subscribe("RoleNotebook_TabContainer-selectChild", dojo.hitch(this, this.onTabSwitch));
			
			this._loadRole();
			this._setupConnections();
			this._updateDialogStyle();
			this.homeLink = ConsoleUtilities.placeLink(this.homeUrl);
			this.designLink = ConsoleUtilities.placeLink(this.designUrl);
		} catch (e) {
			this._trace.exception(e, "postCreate");
			
			// 'this.msgArea' is not available during constructor.
			// Hence this error should be considered while startup method execution where 'this.msgArea' is available.
			this.error = e;
		} finally {
			this._trace.exiting(this.id, "postCreate");
		}
	},
	
	_updateDialogStyle: function() {
		if(dojo.isFF) {
			this.tabContainer.set('class','fullSize tabContainerStyle');
		}
	},
	
	tagReBuildPanels:function() {
		
		/* Need to have the logic of marking a specific page is a REBUILD candidate. That way, when we tab-switch, the 
		 * logic mentioned below needs to be executed. Lets use this function just to tag a given tab as something 
		 * that requires a rebuild!! */

		var panelId = null;
		if (this._hierarchicalRelationPanel) {
			panelId = "hierarchy";
			this._setPanelReBuildState(true,panelId);
		}
		
		if (this._userMembershipPanel) {
			panelId = "membership";
			this._setPanelReBuildState(true,panelId);
		}
				
		if (this.generalPanel) {
			panelId = "general";
			this._setPanelReBuildState(true,panelId);
		}
		
		if (this._permissionsPanel) {
			panelId = "permissions";
			this._setPanelReBuildState(true,panelId);	
		}
		
		if (this.sodPanel) {
			panelId = "SODTab";
			this._setPanelReBuildState(true,panelId);
		}

		if (this.errorWarningPanel) {
			panelId = "errorWarning";
			this._setPanelReBuildState(true,panelId);
		}
		
		if (this.attributePanel) {
			panelId = "attributeTab";
			this._setPanelReBuildState(true,panelId);
		}
		
		if (this.lifecycleRequestPanel) {
			panelId = "lifecycleRequest";
			this._setPanelReBuildState(true,panelId);
		}
	},
	
	refreshBasicData:function(latestRoleUri, isWarningFirst, populatePanels){
		// summary:
		//			Does the initial load of the role data for the panel (needs a method rename)
		this._trace.entering("", "refreshBasicData");
		var panel = null;
		this.roleUri = latestRoleUri;
		this._isErrorWarningPanelFirst = isWarningFirst;
		this._isGeneralPanelFirst = !isWarningFirst;
		
		if (this.roleUri == null) {	
			// Editor is invoked for creating a new Role
			this.isNewRole = true;
		}

		//Get a new rest manager for the new Role URI.
		this.roleRestManager = null;
		this._setupRestStore();
		
		/* If the notebook is launched for a different role, we need to ensure that the 
		 * relevant page is already created. */
		if(this._isErrorWarningPanelFirst){
			console.log("create error");
			// Create Role Editor's Errors and Warnings Panel
			if (!this.errorWarningPanel) { 
				this.errorWarningPanel=this._createErrorWarningPanel();
			} else {
				this.errorWarningPanel.roleRestManager = this.roleRestManager;
			}
		} else{
			// Create Role Editor's General Panel
			if (!this.generalPanel) {
				this.generalPanel=this._createGeneralPanel();
			} else {
				this.generalPanel.roleRestManager = this.roleRestManager;
			}
		}

		this._resetPanelScrolling();

		this.populateAllPanels = populatePanels;

		/* On subsequent launches of the RoleNotebook, we should not show the messages 
		 * from the previous launch. */
		this.msgArea.hideMessages();
		
		this._loadRole();

		this.tagReBuildPanels();
		
		this._trace.exiting("", "refreshBasicData");
	},	
	
	_resetPanelScrolling: function() {
		// Summary:
		//			Resets panel scrolling.
		this._trace.entering("", "_resetPanelScrolling");
		if (this.generalPanel) {
			this.generalPanel.panelBodyArea.domNode.scrollTop = 0;
			this.generalPanel.panelBodyArea.domNode.scrollLeft = 0;
		}
		if (this._hierarchicalRelationPanel) {
			this._hierarchicalRelationPanel.container.domNode.scrollTop = 0;
			this._hierarchicalRelationPanel.container.domNode.scrollLeft = 0;
		}
		if (this._userMembershipPanel) {
			if(dojo.isIE) {
				this._userMembershipPanel.container.domNode.scrollTop = 0;
				this._userMembershipPanel.container.domNode.scrollLeft = 0;				
			} else if (dojo.isFF) {
				this.membershipTab.domNode.scrollTop = 0;
				this.membershipTab.domNode.scrollLeft = 0;
			}
		}
		if (this._permissionsPanel) {
			this._permissionsPanel.container.domNode.scrollTop = 0;
			this._permissionsPanel.container.domNode.scrollLeft = 0;
		}
		this._trace.exiting("", "_resetPanelScrolling");
	},
	
	_setupConnections: function() {
		this.connect(this.container, "show", "_loadRole");
	},

	setIgnorePopulateOnTabChange: function(/*boolean*/ shallIgnore) {
		this._ignorePopulateOnTabChange = shallIgnore;
	},
	
	onTabSwitch: function(page) {
		this._trace.entering("", "onTabSwitch");

		if (this._ignorePopulateOnTabChange) {
			this._trace.finest(this.id+" - Data population skipped.", "onTabSwitch");
			return;
		}

		// This will ensure that the warning messages, in fact any messages, displayed as part 
		// of the previous tab, for any stale data, are cleared.
		this.msgArea.hideMessages();//clear messages when moving between tabs
		
		var panel = null;
		var panelId = null;
		var isFirstTime = false;
		
		if (page === "general" || page.id === "general") {
			if (this.generalPanel == null) {
				this.generalPanel = this._createGeneralPanel();
				isFirstTime = true;
			}
			panel = this.generalPanel;
			panelId = "general";
		} else if (page === "hierarchy" || page.id === "hierarchy") {
			if (this._hierarchicalRelationPanel == null) {
				this._hierarchicalRelationPanel = this._createHierarchicalRelationPanel();
				isFirstTime = true;
			}
			panel = this._hierarchicalRelationPanel;
			panelId = "hierarchy";
		} else if (page === "membership" || page.id === "membership") {
			if(this._userMembershipPanel == null){
				this._userMembershipPanel = this._createUserMembershipPanel();
				isFirstTime = true;
			}
			panel = this._userMembershipPanel;
			panelId = "membership";
		} else if (page === "permissions" || page.id === "permissions") {
			if (this._permissionsPanel == null) {
				this._permissionsPanel = this._createPermissionsPanel();
				isFirstTime = true;
			}
			panel = this._permissionsPanel;
			panelId = "permissions";
		} else if (page === "SODTab" || page.id === "SODTab") {
			if (this.sodPanel == null) {
				this.sodPanel = this._createSeperationOfDutyPanel();
				isFirstTime = true;
			}
			panel = this.sodPanel;
			panelId = "SODTab";
		} else if (page === "errorWarning" || page.id === "errorWarning") {
			if (this.errorWarningPanel == null) {
				this.errorWarningPanel = this._createErrorWarningPanel();
				isFirstTime = true;
			}
			panel = this.errorWarningPanel;
			panelId = "errorWarning";
		}else if (page === "attributeTab" || page.id === "attributeTab") {
			if (this.attributePanel == null) {
				this.attributePanel = this._createAttributePanel();
				isFirstTime = true;
			}
			panel = this.attributePanel;
			panelId = "attributeTab";
		}else if (page === "lifecycleRequest" || page.id === "lifecycleRequest") {
			if (this.lifecycleRequestPanel == null) {
				this.lifecycleRequestPanel = this._createLifecycleRequestPanel();
				this.lifecycleRequestPanel.startup();
				isFirstTime = true;
			}
			panel = this.lifecycleRequestPanel;
			panelId = "lifecycleRequest";
		}

		if (isFirstTime && this.roleItem != null) {
			panel.populate(this.roleItem);
		}
		
		if ((!this._panelReBuildState[panelId]) && (!isFirstTime) && panelId && panel &&
				this._panelDirtyState && this._panelDirtyState[panelId] && panel.getRefreshConfig) {
			this._refreshPanel(panel.getRefreshConfig(), panelId);
			this._setPanelDirtyState(false, panelId);
		}

		if(this._panelReBuildState[panelId]) {
			//if(panelId != "general") {
			   panel.reBuild(this.roleItem);
			   /* Once rebuilt, ensure that we are not keeping it as a rebuild candidate anymore,
			    * else it'll keep rebuilding on each tabswitch! */
			   this._panelReBuildState[panelId] = false;
			
			   if (this._panelDirtyState && this._panelDirtyState[panelId]) {
				   /* If the panel is rebuilt and it was marked dirty earlier, ensure that we clear that. */
				   this._setPanelDirtyState(false, panelId);
			   }
			//}
		}
		
		this._trace.exiting("", "onTabSwitch");
	},
	
	startResizeChain: function() {
		// summary:
		//			resizes the widget container, which results into resizing its children.
		this._trace.entering(this.id, "startResizeChain");
		this._trace.warn("Deprecated: Use the method resize instead", "startResizeChain");
		//this.resize();
		this._trace.exiting(this.id, "startResizeChain");
	},
	
	startup: function(){
		this._trace.entering(this.id, "startup");
	
		if (this.error != null) {
			this._displayError();
		}
		
		try {
			this.inherited(arguments);
			if(this.roleLifecycleProvider.isBPMEnabled()){//add lifecyle request tab if BPM is enabled
				this.lifecycleRequestTab= new com.ibm.security.modeling.dijit.layout.ContentPane({
					id:"lifecycleRequest",
					title:this.localResources.LIFECYCLE_TAB_TITLE
				});
				dojo.addClass(this.lifecycleRequestTab,"contentArea");
				this.lifecycleRequestTab.setContent("<div id="+this.id+"_lifecycleRequestsPanel></div>");
				this.tabContainer.addChild(this.lifecycleRequestTab);
			}
		} catch (e) {
			this._trace.exception(e, "startup");
			this.error = e;
			this._displayError();
		} 
		this._trace.exiting(this.id, "startup");
	},
		
	_displayError: function() {
		// summary:
		//			Displays error using 'msgArea' widget.
		this._trace.entering(this.id, "_displayError");
		this.msgArea.showErrorMessage(this.error);
		this._trace.exiting(this.id, "_displayError");
	},

	destroy: function() {
		this._trace.enteringWithStack(this.id, "destroy", true);
		try {
			this.inherited(arguments);			
			this._trace.exiting(this.id, "destroy");
		} catch (e) {
			this._trace.exception(e, "destroy");
			throw e;
		}
	},
	
	_checkIfRoleNameValid: function() {
		this._trace.entering(this.id, "_checkIfRoleNameInvalid");
		if (this.generalPanel) {
			valid = this.generalPanel._validateRoleName(true);
		} else {
			valid = true;
		}
		this._trace.exiting(this.id, "_checkIfRoleNameInvalid");
		return valid;
	},

	_tailorString: function(srcString, limit) {
		var targetString = "";
		if ( (limit > 0) && (srcString.length > limit) ) {
			targetString = srcString.substring(0, limit-1);
			targetString += "...";
		} else {
			return srcString;
		}
		return targetString;
	},

	_updateBreadCrumbBar: function() {
		/* Strip-off the existing Role Name and append the Bread-crumb more suitable for the current page. */
		this._trace.entering(this.id, "_updateBreadCrumbBar");
		
		var breadCrumbLabel = this.roleItem.name  +  this.localResources.TITLE_NAMEVALUE_SEPARATOR + this.globalResources.ROLE_NOTEBOOK_TITLE;
		breadCrumbLabel = com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(breadCrumbLabel);
		this.updateBreadCrum(breadCrumbLabel);
		this._trace.exiting(this.id, "_updateBreadCrumbBar");
	},
	
	_createGeneralPanel: function () {
		this._trace.entering(this.id, "_createGeneralPanel");
		var generalPanel = new com.ibm.security.modeling.notebook.RoleNotebookGeneralPanel (
			{
				id:this.id+"_generalPanel",
				namespace:this.namespace,
				roleRestManager: this.roleRestManager,
				messageArea: this.msgArea,
				contextRoot: this.contextRoot,
				parentWidget:this,
				globalResources: this.globalResources,
				saveIndicator: this.saveIndicator
			},
			this.id+"_GeneralPanel");
		
		generalPanel.startup();
		
		this._trace.exiting(this.id, "_createGeneralPanel");
		return generalPanel;
	},
	_createHierarchicalRelationPanel: function () {
		this._trace.entering(this.id, "_createHierarchicalRelationPanel");
		var saveIndicatorText = this.globalResources.SAVE_HIERARCHY_CHANGE;
		var hierarchicalRelationPanel = new com.ibm.security.modeling.notebook.RoleNotebookHierarchicalRelationPanel (
				{
					id:this.id+"_hierarchicalRelationPanel",
					namespace:this.namespace,
					roleRestManager: this.roleRestManager,
					messageArea: this.msgArea,
					contextRoot: this.contextRoot,
					parentWidget:this,
					globalResources: this.globalResources,
					saveIndicator: this.saveIndicator,
					saveIndicatorText: saveIndicatorText
				},
				this.id+"_HierarchicalRelationPanel");
		hierarchicalRelationPanel.startup();
		this._trace.exiting(this.id, "_createHierarchicalRelationPanel");
		return hierarchicalRelationPanel;
		
	},
	_createUserMembershipPanel: function () {
		dojo.require("com.ibm.security.modeling.notebook.RoleNotebookUserMembershipPanel");
		this._trace.entering(this.id, "_createUserMembershipPanel");
		var saveIndicatorText = this.globalResources.SAVE_MEMBERSHIP_CHANGE;
		var userMembershipPanel = new com.ibm.security.modeling.notebook.RoleNotebookUserMembershipPanel (
				{
					id:this.id+"_userMembershipPanel",
					messageArea: this.msgArea,
					contextRoot: this.contextRoot,
					parentWidget:this,
					globalResources: this.globalResources,					
					saveIndicator: this.saveIndicator,
					saveIndicatorText: saveIndicatorText
				},
				this.id+"_UserMembershipPanel");
		userMembershipPanel.startup();
		this._trace.exiting(this.id, "_createUserMembershipPanel");
		return userMembershipPanel;
		
	},
	_createPermissionsPanel: function () {
		this._trace.entering(this.id, "_createPermissionsPanel");
		var permissionsPanel = new com.ibm.security.modeling.notebook.RoleNotebookPermissionsPanel (
				{
					id:this.id+"_permissionsPanel",
					namespace:this.namespace,
					//roleRestManager: this.roleRestManager,
					messageArea: this.msgArea,
					contextRoot: this.contextRoot,
					saveIndicator: this.saveIndicator,
					globalResources: this.globalResources,
					parentWidget:this
				},
				this.id+"_PermissionsPanel");
		permissionsPanel.startup();
		this._trace.exiting(this.id, "_createPermissionsPanel");
		return permissionsPanel;
		
	},
	_createSeperationOfDutyPanel: function () {
		this._trace.entering(this.id, "_createSeperationOfDutyPanel");
		var saveIndicatorText = this.globalResources.SAVE_SOD_CHANGE;
		//this._trace.entering(this.id+" - "+this.saveIndicator, "_createSeperationOfDutyPanel");	
		var sodPanel = new com.ibm.security.modeling.notebook.SeperationOfDutyPanel (
				{
					id:this.id+"_SODPanel",
					//roleItem: this.roleItem,
					//rolesUri:this.roleUri,
					messageArea: this.msgArea,
					contextRoot: this.contextRoot,
					parentWidget:this,
					globalResources: this.globalResources,
					saveIndicator: this.saveIndicator,
					saveIndicatorText: saveIndicatorText				
				},
				this.id+"_SODPanel");
		this._trace.exiting(this.id, "_createSeperationOfDutyPanel");
		return sodPanel;
	},
	_createErrorWarningPanel: function () {
		this._trace.entering(this.id, "_createErrorWarningPanel");	
		var errorWarningPanel = new com.ibm.security.modeling.notebook.RoleNotebookErrorWarningPanel (
				{
					id:this.id+"_errorWarningPanel",
					messageArea: this.msgArea,
					contextRoot: this.contextRoot,
					parentWidget:this,
					globalResources: this.globalResources,
					logResources:this.logResources,					
					saveIndicator: this.saveIndicator
				},
				this.id+"_errorWarningPanel");
		errorWarningPanel.startup();
		this._trace.exiting(this.id, "_createErrorWarningPanel");
		return errorWarningPanel;
	},
	_createAttributePanel :function(){
	    this._trace.entering(this.id, "_createAttributePanel");	
	    dojo.require("com.ibm.security.modeling.notebook.AttributePanel");	
		var attributePanel = new com.ibm.security.modeling.notebook.AttributePanel (
				{
					id:this.id+"_attributeValuePanel",
					messageArea: this.msgArea,
					globalResources: this.globalResources,
					saveIndicator: this.saveIndicator,
					contextRoot: this.contextRoot,
					parentWidget:this
				},
				this.id+"_attributeValuePanel");
		this._trace.exiting(this.id, "_createAttributePanel");
		return attributePanel;
	},
	_createLifecycleRequestPanel :function(){
	    this._trace.entering(this.id, "_createLifecycleRequestPanel");	
	    dojo.require("com.ibm.security.modeling.notebook.RoleNotebookLifecycleRequestsPanel");	
		var lifecycleRequestPanel = new com.ibm.security.modeling.notebook.RoleNotebookLifecycleRequestsPanel (
				{
					id:this.id+"_lifecycleRequestsPanel",
					messageArea: this.msgArea,
					globalResources: this.globalResources,
					parentWidget:this
				},
				this.id+"_lifecycleRequestsPanel");
		this._trace.exiting(this.id, "_createLifecycleRequestPanel");
		return lifecycleRequestPanel;
	},

	_setupRestStore: function () {
		this._trace.entering(this.id, "_setupRestStore");
		this.roleRestManager = new com.ibm.security.modeling.api.RoleRestManager({
			resourceUri: this.roleUri,
			syncMode: false,
			messageArea: this.msgArea
		});
		this._trace.exiting(this.id, "_setupRestStore");
	},

	_loadRole: function () {
		try {
			this._trace.entering(this.id, "_loadRole");
			this._trace.finest(this.id+" - Loading Role: "+this.roleUri, "_loadRole");
			this.roleRestManager.requestResource(null, dojo.hitch (this, this._loadRoleRequestComplete), null);
		} finally {
			this._trace.exiting(this.id, "_loadRole");
		}
	},
	
	_loadRoleRequestComplete: function (result) {
		// The item in this method has its id truncated because of its large size (as part of json
		// conversion). Hence reassigning the id from the uri
		// This is a temporary fix until a more centralized solution is in place
		if (result != null && result.uri != null){
			result.id = result.uri.substring(result.uri.lastIndexOf("/")+1, result.uri.length);
		}

		if(dojo.isIE) {
			this._updateTabListStyle();
		}
		
		this._trace.entering(this.id, "_loadRoleRequestComplete");
		this.roleItem = result;
		this._trace.object(this.roleItem, "_loadRoleRequestComplete", "Role from REST");
		if (this.roleItem != null) {
			this.updateDialogHeader();
			if (this.populateAllPanels) {
				this._populateNotebookPanels();
			} else {
				try {
					this.setIgnorePopulateOnTabChange(true);
					if(this._isGeneralPanelFirst) {
						if(this._panelReBuildState["general"]) {
							this.generalPanel.reBuild(this.roleItem);
							/* Once rebuilt, ensure that we are not keeping it as a rebuild candidate anymore,
							 * else it'll keep rebuilding on each tab-switch! */
							this._panelReBuildState["general"] = false;
						} else {
							this.generalPanel.populate(this.roleItem);
						}
						this.tabContainer.selectChild(this.generalTab);				
					} else {
						if(this._panelReBuildState["errorWarning"]) {
							this.errorWarningPanel.reBuild(this.roleItem);
							/* Once rebuilt, ensure that we are not keeping it as a rebuild candidate anymore,
							 * else it'll keep rebuilding on each tab-switch! */
							this._panelReBuildState["errorWarning"] = false;
						} else {
							this.errorWarningPanel.populate(this.roleItem);
						}
						this.tabContainer.selectChild(this.errorWarningTab);
					}
				} finally {
					this.setIgnorePopulateOnTabChange(false);
				}
			}
			this.subscribe("ROLE_PROPERTY_UPDATE_EVENT", this._handleRolePropertyEditEvent);
		}
		this._trace.exiting(this.id, "_loadRoleRequestComplete");
	},
	
	getCurrentSelectedTab: function() {
		this._trace.entering(this.id, "getCurrentSelectedTab");
		if (this.tabContainer) {
			selectedTab = this.tabContainer.selectedChildWidget; // object having the content pane
			if(selectedTab && (selectedTab.id)) {
				this._trace.exiting(this.id, "getCurrentSelectedTab");			
				return selectedTab.id;
			}
		}
		this._trace.exiting(this.id, "getCurrentSelectedTab");
		return null;
	},
	
	_panelEligibleForStaleDataWarning: function() {
		this._trace.entering(this.id, "_panelEligibleForStaleDataWarning");
	
		currentSelectedTab = this.getCurrentSelectedTab();
		
		if(currentSelectedTab) {
	
		/* Only the following four panels qualify for showing stale data refresh warnings.
		 * For the rest of the tabs, its an auto-refresh. */
			if( (currentSelectedTab == "membership") || 
				(currentSelectedTab == "permissions") ||  
				(currentSelectedTab == "SODTab") || 
				(currentSelectedTab == "errorWarning") ) {
				this._trace.exiting(this.id, "_panelEligibleForStaleDataWarning");				
				return true;
			}				
		}
				
		this._trace.exiting(this.id, "_panelEligibleForStaleDataWarning");
		return false;
	},
	
	_panelStateDirtyForSure: function(panelId) {
		// Function to return if a panel has been marked as dirty irrespective of 
		// traversals to and away from the panel. There are cases whereby a panel
		// is evaluated for stale data and the user is warned accordingly. However,
		// if the user doesn't refresh the panel at that instant of time and goes
		// away from the panel, we should not be evaluating the panel for stale data
		// when the user visits the panel again. This switch would help in ensuring
		// that we don't repeat the evaluation all over again.
	
		this._trace.entering(this.id, "_panelStateDirtyForSure");
		this._trace.exiting(this.id, "_panelStateDirtyForSure");
		return this._panelDirtyStateForSure[panelId];
	},
	
	_getSelectedTab: function() {
		this._trace.entering(this.id, "_getSelectedTab");
		var selectedTab = null;
		
		if (this.tabContainer) {
			selectedTab = this.tabContainer.selectedChildWidget; // object having the content pane, and its id
		}
		
		this._trace.exiting(this.id, "_getSelectedTab");
		return selectedTab;
	},
	
	_getTabWidget: function(selectedTab) {
		this._trace.entering(this.id, "_getTabWidget");
		var tabWidget = null;
		
		if (selectedTab) {
			var children = selectedTab.getChildren();
			if (children && dojo.isArray(children) && children.length>0) {
				tabWidget = children[0]; // the widget.
			}
		}
		this._trace.exiting(this.id, "_getTabWidget");		
		return tabWidget;
	},
	
	_extractRefreshConfig: function(tabWidget) {
		this._trace.entering(this.id, "_extractRefreshConfig");	
		var panelRefreshConfig = null;
		
		if (tabWidget && tabWidget.getRefreshConfig) {
			panelRefreshConfig = tabWidget.getRefreshConfig();
		}
		this._trace.exiting(this.id, "_extractRefreshConfig");			
		return panelRefreshConfig;
	},
	
	_panelQualifiesForStaleDataWarning: function(panelId, panelRefreshConfig) {
		this._trace.entering(this.id, "_panelQualifiesForStaleDataWarning");
		
		/* Check if the panel is eligible for throwing stale data warning. */
		if(!this._panelEligibleForStaleDataWarning()) {
			this._trace.exiting(this.id, "_panelQualifiesForStaleDataWarning");
			return false;
		}
		
		/* In case the panelRefreshConfig isn't sent, gather the selected tab details
		 * and fetch the refresh config. */
		if(!panelRefreshConfig) {
			// Get the refresh config of the current Tab.
			
			var tabWidget = null;
			var selectedTab = this._getSelectedTab();
			
			if(selectedTab) {
				panelId = selectedTab.id;
				tabWidget = this._getTabWidget(selectedTab);
			}
			
			panelRefreshConfig = this._extractRefreshConfig(tabWidget);

		}
		
		/* We need this state in cases where the user has made some changes, say in the Hierarchy Tab, later
		 * he's visited the Membership page to see the warning. He doesn't hit Refresh but goes to the next page.
		 * Next time he visits the page, he would get an indicator upfront that the panel was marked as dirty
		 * for sure and there need not be any eligibility checks on whether the dirty state would mean an
		 * automated update OR manual. */
		if(this._panelStateDirtyForSure(panelId)) {
			this._trace.exiting(this.id, "_panelQualifiesForStaleDataWarning");
			return true;
		}
		
		/* If panel Refresh config is still not available, that's an error that we need to flag. */
		if (!panelRefreshConfig) {
			this._trace.exiting(this.id, "_panelQualifiesForStaleDataWarning");
			return false;
		}
		
		/* Traverse through all properties in the refreshConfig of the selected tab. */
		for (var property in panelRefreshConfig) {
			if (property in this._propertyUpdates) { // check if this property is tracked for updates.
				var propertyUpdates = this._propertyUpdates[property];	// get updates (activities) that have happened on this property.
				var propertyConfig = panelRefreshConfig[property];
				// check if any of the activity matches with the one specified for panel refresh configuration.
				for (var activity in propertyConfig) {
					if (!(activity in propertyUpdates)) {	// this activity has not taken place yet.
						continue;
					}
					
					var activityConfig = propertyConfig[activity];
					if (!("version" in activityConfig)) {// no version yet.
						activityConfig["version"] = 0;
					}
					
					// Update only if the version has updated. In other words some new updates have taken place.
					if (propertyUpdates[activity]["version"] > activityConfig["version"]) {
						// update config version.
						activityConfig["version"] = propertyUpdates[activity]["version"];

						/* Record here the fact that the current panel was marked as dirty owing to some authentic
						 * updates. The next time we revisit this page and the user had not cleared the dirty status
						 * on the table, the above if clause won't hold true anymore. Hence, we won't be able to 
						 * throw the right message to the user.
						 */
						this._setPanelDirtyStateForSure(true, panelId);
					
						// If the change is any of these properties, we would do an auto-refresh. Hence,
						// we would tag that the panel is not eligible for stale data warning. Instead
						// refresh the table ourselves.
						if ( (property == "name") || (property == "type") || (property == "description") ) {
							var methods = activityConfig["methods"];
							for (var i=0; i<methods.length; i++) {
								// Invoke configured methods and pass event data.
								methods[i](propertyUpdates[activity]["data"]);
							} 							
							this._trace.exiting(this.id, "_panelQualifiesForStaleDataWarning");
							return false;
						}
						this._trace.exiting(this.id, "_panelQualifiesForStaleDataWarning");
						return true;
					}
				}
			}
		}
		
		this._trace.exiting(this.id, "_panelQualifiesForStaleDataWarning");
		return false;
	},
	
	_throwStaleDataWarning: function() {
		this._trace.entering(this.id, "_throwStateDataWarning");
		
		/* Each SoD Delete seems to be performed as an independent operation. Hence, each delete is throwing a 
		 * stale data warning. Need to check if the message is already displayed. If yes, don't show it again. */
		if(!this.msgArea.messageDisplayedAtTop(this.pluginResources.ROLE_STALE_DATA_WARNING)) {
			var messageID = this._messageUtil.getMessageID(this.pluginResources.ROLE_STALE_DATA_WARNING);
			this.msgArea.showWarningMessage(this.pluginResources.ROLE_STALE_DATA_WARNING, messageID);
		}
		
		this._trace.exiting(this.id, "_throwStaleDataWarning");
	},
	
	_refreshHandler: function(panelId) {
		this._trace.entering(this.id, "_refreshHandler");
		// Function that'll ensure that we are resettig the dirty flags on the panels,
		// once the user refreshes the corresponding panel.
		
		if(this._panelDirtyState[panelId] || this._panelDirtyStateForSure[panelId]) {
			this._setPanelDirtyState(false, panelId);
			this._setPanelDirtyStateForSure(false, panelId);
		}
		this._trace.exiting(this.id, "_refreshHandler");
	},
	
	_getPanelInfo: function(event) {
		/* Function to yield the ID of the panel that had generated 
		 * the event passed in the function.
         */
		this._trace.entering(this.id, "_getPanelInfo");
		
		var panelId = null;
		
		if(event && event.property && event.activity) {
			if( (event.property == "name") ||
				(event.property == "description") || 
				(event.property == "type") ) {
				panelId = "general";
			} else if ( (event.property == "children") ||
						(event.property == "parents") ) {
				panelId = "hierarchy";
			} else if ( event.property == "membership" ) {
				panelId = "membership";
			} else if ( event.property == "permissions" ) {
				panelId = "permissions";
			} else if ( event.property == "sodconstraints" ) {
				panelId = "SODTab";
			}
		}
		this._trace.exiting(this.id, "_getPanelInfo");
		return panelId;
	},
	
	_updatePanelDirtyState: function(event) {
		/* Function to mark a set of panels as dirty, based upon
		 * the event data passed in.
         */
		this._trace.entering(this.id, "_updatePanelDirtyState");

		var eventInvokedFrom = this._getPanelInfo(event);
		this._setAllPanelsDirtyExcept(eventInvokedFrom);
		
		this._trace.exiting(this.id, "_updatePanelDirtyState");
	},
	
	_handleRolePropertyEditEvent: function () {
		this._trace.entering(this.id, "_handleRolePropertyEditEvent");
		this.isRoleEdited = true;
		
		var event = arguments[0];
		// To log and event; event should have the property, activity in it.
		// for example: property: name, activity: update; represents that Role name has been updated.
		if (event && event.property && event.activity) {
			// log property edit
			this._logPropertyUpdate(event);
			
			// set all panels dirty, except the current one.
			this._updatePanelDirtyState(event);
			
			if(this._panelQualifiesForStaleDataWarning()) {
				this._throwStaleDataWarning();
			}
			else {
				// If the panel doesn't qualify for stale data warning, it would need to undergo an auto-refresh.
				this._refreshCurrentTab();
			}
		}
		this._trace.exiting(this.id, "_handleRolePropertyEditEvent");
	},
	
	_refreshCurrentTab: function() {
		// If currently selected panel needs refresh then do it here.
		// Else this panel will not be refreshed, as refresh logic runs only on tab switch
		//this.selectedChildWidget
		
		var tabWidget = null;
		var selectedTab = null;
		
		if (this.tabContainer) {
			selectedTab = this.tabContainer.selectedChildWidget; // object having the content pane, and its id
		}
		if (selectedTab) {
			var children = selectedTab.getChildren();
			if (children && dojo.isArray(children) && children.length>0) {
				tabWidget = children[0]; // the widget.
			}
		}
		if (tabWidget && tabWidget.getRefreshConfig) {
			this._refreshPanel(tabWidget.getRefreshConfig(), selectedTab.id);
			this._setPanelDirtyState(false, selectedTab.id);
		}
	},
	
	_populateNotebookPanels: function () {
		this._trace.entering(this.id, "_populateNotebookPanels");
		if (this.generalPanel) {
			this.generalPanel.populate(this.roleItem);
		}
		
		if (this._hierarchicalRelationPanel) {
			this._hierarchicalRelationPanel.populate(this.roleItem);
		}
		if (this._userMembershipPanel) {
			this._userMembershipPanel.populate(this.roleItem);
		}
		if (this._permissionsPanel) {
			this._permissionsPanel.populate(this.roleItem);
		}
		if (this.sodPanel) {
			this.sodPanel.populate(this.roleItem);
		}
		if (this.errorWarningPanel) {
			this.errorWarningPanel.populate(this.roleItem);
		}
		if (this.attributePanel) {
			this.attributePanel.populate(this.roleItem);
		}
		if (this.lifecycleRequestPanel) {
			this.lifecycleRequestPanel.populate(this.roleItem);
		}
		this._trace.exiting(this.id, "_populateNotebookPanels");
	},
	
	_validate: function() {
		this._trace.entering(this.id, "_validate");
		var valid = true;

		var generalPanelValid = this.generalPanel.validate();
		if (!generalPanelValid) {
			valid = false;
		}

		if (!generalPanelValid) {
			this.tabContainer.selectChild(this.generalTab);
		}
		this._trace.exiting(this.id, "_validate");
		return valid;
	},
	//Overridding breadcrum interface 
	canHide : function() {

		return this._handleCanClose(null, /*isViaBreadCrumb*/ true);
	},
	
	_navigateAway: function() {
		// breadcrumb invoking this method means that 'navigate away' should happen.
		// if navigation was not supposed to happen, then that condition should have had captured in canHide() and 'false' had been be returned.
		// because, when canHide() returns 'false' then _navigateAway() is not invoked.
		
		this._trace.entering(this.id, "_navigateAway");
		dojo.publish(dojo.config.panels.HIERARCHY.launchEventAddress, [this.projectUrl, this.modelId, false]);
		this._trace.exiting(this.id, "_navigateAway");
	},
	
	_showProgressBar: function(message) {
		this._trace.entering(this.id, "_showProgressBar");
		if(!this.progressBarShown) {
			this.progressBar.show(message);
			this.progressBarShown = true;
		}
		this._trace.entering(this.id, "_showProgressBar");
	},

	_hideProgressBar: function() {
		this._trace.entering(this.id, "_hideProgressBar");
		if(this.progressBarShown) {
			this.progressBar.hide();
			this.progressBar.destroyProgressDialog(); /* Sometimes, hide doesn't seem to destroy the dialog. Hence, invoking it explicitly. */
			this.progressBarShown = false;
		}
		this._trace.entering(this.id, "_hideProgressBar");
	},	
	
	_handleCanClose: function(/*dojo normalized dijit event*/event, /*boolean*/isViaBreadCrumb, /*boolean*/ isTimedOut) {
		
		this._trace.entering(this.id, "_handleCanClose");
		
		// In the IE browser, the close event is queued up before the event for
		// updating a field.  Therefore, we add a slight delay just in case an
		// update event is queued up behind the close event.
		
		if ((dojo.isIE) &&
			(event != null)) {			
			this._showProgressBar(this.localResources.SAVE_ROLE_DETAILS);
			setTimeout (dojo.hitch (this, this._handleCanClose, null, isViaBreadCrumb, /*isTimedOut*/ true), 100);
			
		} else if (this.saveIndicator.isSaving()) {
			
			if (this.isEvaluatingFilter()) {
				// Quick Patch.
				// Need to have a monitor like mechanism for notebook "navigation away" [close, breadcrumbs].
				this._trace.entering(this.id+" - filter evaluation is running. Cannot navigate away from Notebook.", "_checkAndClose");
				this.msgArea.showInfoMessage(this.localResources.NOTEBOOK_EXIT_NOT_ALLOWED_EVALUATING_ROLE_FILTER);
				return false;
			} else if(!this.progressBarShown) { // Save is taking place. Wait and try again.
				this._showProgressBar(this.localResources.SAVE_ROLE_DETAILS);
			}		
			setTimeout (dojo.hitch (this, this._handleCanClose, null, isViaBreadCrumb, /*isTimedOut*/ true), 100);
			
		} else {
			
			this._hideProgressBar();
			
			if(!this._checkIfRoleNameValid()) {
				return false;
			}
			
			if (this.isRoleEdited) {
				// Edit took place.
				this._trace.info(this.id+" - Role Edited. Publishing Event.", "_handleCanClose");
				var eventData = {};
				eventData.roleUri = this.roleItem.uri;
				dojo.publish("ROLE_EDIT_EVENT",[eventData]);
				this._trace.info(this.id+" - Role Edited. Event Published.", "_handleCanClose");
			}
			
			if (isViaBreadCrumb === true) {
				if (isTimedOut === true) {
					dojo.publish("NAVIGATE_AWAY_EVENT");
				} 
			} else {
				this._navigateAway();
			}
			
			return true;
		}
		this._trace.exiting(this.id, "_handleCanClose");
		
		return false;
	},
	
	_onClose: function(event) {
		this._trace.entering(this.id, "_onClose");
		this._handleCanClose(event, /*isViaBreadCrumb*/ false);
		this._trace.exiting(this.id, "_onClose");
	},
	
	/* 
	 * Set the title of the dialog with the role-name, in case the dialog is launched for editing role details. 
	 * Also this function will be invoked to update the header of the role editor dialog in case of role name 
	 * changes as part of the update activities. 
	 */
	updateDialogHeader: function() {
		this._trace.entering(this.id, "updateDialogHeader");
		
		var titleText = this.roleItem.name + this.localResources.TITLE_NAMEVALUE_SEPARATOR + this.globalResources.ROLE_NOTEBOOK_TITLE;
		titleText = com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(titleText);
		this.titleUtil.updateTitle(titleText);
		
		/* Need to update the Bread-crumb bar accordingly. */
		this._updateBreadCrumbBar();
		
		this._trace.exiting(this.id, "updateDialogHeader");
	},
	
	getTitle: function() {
		return /*this.roleItem.name + this.localResources.TITLE_NAMEVALUE_SEPARATOR +*/ this.globalResources.ROLE_NOTEBOOK_TITLE;
	},
	
	getFullTitle: function() {
		return this.roleItem.name + this.localResources.TITLE_NAMEVALUE_SEPARATOR + this.globalResources.ROLE_NOTEBOOK_TITLE;
	},

	showSaveInProgress: function() {
		this._trace.entering(this.id, "showSaveInProgress");
		this.saveIndicator.show();
		this._trace.exiting(this.id, "showSaveInProgress");
	},
	
	hideSaveInProgress: function() {
		this._trace.entering(this.id, "hideSaveInProgress");
		this.saveIndicator.hide();
		this._trace.exiting(this.id, "hideSaveInProgress");
	},
	
	_initPanelDirtyState: function() {
		// marks every panel as not dirty.
		this._trace.entering(this.id, "_initPanelDirtyState");
		this._panelDirtyState = {};
		for (var i=0; i<this.panelIds.length; i++) {
			this._panelDirtyState[this.panelIds[i]] = false;
		}
		this._trace.exiting(this.id, "_initPanelDirtyState");
	},

	_initPanelDirtyStateForSure: function() {
		// marks every panel as not dirty.
		this._trace.entering(this.id, "_initPanelDirtyStateForSure");
		this._panelDirtyStateForSure = {};
		for (var i=0; i<this.panelIds.length; i++) {
			this._panelDirtyStateForSure[this.panelIds[i]] = false;
		}
		this._trace.exiting(this.id, "_initPanelDirtyStateForSure");
	},
	
	_initPanelReBuildState: function() {
		// marks every panel as not dirty.
		this._trace.entering(this.id, "_initPanelReBuildState");
		this._panelReBuildState = {};
		for (var i=0; i<this.panelIds.length; i++) {
			this._panelReBuildState[this.panelIds[i]] = false;
		}
		this._trace.exiting(this.id, "_initPanelReBuildState");		
	},
	
	_setAllPanelsDirtyExcept: function(/* string */ panelID) {
	    /* Function to set/unset the dirty state of all available panels
         * in the notebook, except for the panelID passed in. */
		this._trace.entering(this.id, "_setAllPanelsDirtyExcept");		
		var panelDirtyState = true;
		if(panelID == -1) {
			this._setPanelDirtyState(true, null);
		} else {
			for (var i=0; i<this.panelIds.length; i++) {
				if(this.panelIds[i] == panelID) {
					panelDirtyState = false;
				}
				this._setPanelDirtyState(panelDirtyState, this.panelIds[i]);
				panelDirtyState = true;
			}
		}
		this._trace.exiting(this.id, "_setAllPanelsDirtyExcept");		
	},
	
	_setPanelDirtyState: function(/*boolean*/ isDirty, /*string*/ panelId) {
		this._trace.entering(this.id, "_setPanelDirtyState");			
		if (!panelId) { // If panel is not specified then apply isDirty on all the panels.
			for (var i=0; i<this.panelIds.length; i++) {
				this._panelDirtyState[this.panelIds[i]] = isDirty;
			}
		} else if (panelId in this._panelDirtyState) {
			this._panelDirtyState[panelId] = isDirty;
		}
		this._trace.exiting(this.id, "_setPanelDirtyState");		
	},

	_setPanelDirtyStateForSure: function(/*boolean*/ isDirtyForSure, /*string*/ panelId) {
		this._trace.entering(this.id, "_setPanelDirtyStateForSure");			
		if (!panelId) { // If panel is not specified then apply isDirtyForSure on all the panels.
			for (var i=0; i<this.panelIds.length; i++) {
				this._panelDirtyStateForSure[this.panelIds[i]] = isDirtyForSure;
			}
		} else if (panelId in this._panelDirtyStateForSure) {
			this._panelDirtyStateForSure[panelId] = isDirtyForSure;
		}
		this._trace.exiting(this.id, "_setPanelDirtyStateForSure");		
	},	
	
	_setPanelReBuildState: function(/*boolean*/ needToReBuild, /*string*/ panelId) {
		this._trace.entering(this.id, "_setPanelReBuildState");	
		if (!panelId) { // If panel is not specified then apply isDirty on all the panels.
			for (var i=0; i<this.panelIds.length; i++) {
				this._panelReBuildState[this.panelIds[i]] = needToReBuild;
			}
		} else if (panelId in this._panelReBuildState) {
			this._panelReBuildState[panelId] = needToReBuild;
		}
		this._trace.exiting(this.id, "_setPanelReBuildState");
	},	
	
	_initPropertyUpdate: function() {
		/*
		 * Object has following structure:
		 * 	{
		 * 		role property1: {	// updates on this property
		 * 			event1 name: { 	// event data.
		 * 			},
		 * 			event2 name: { 	// event data.
		 * 			},
		 * 			version: 2
		 * 		},
		 * 		role property2: {	// updates on this property
		 * 			event1 name: { 	// event data.
		 * 			},
		 * 			event2 name: { 	// event data.
		 * 			},
		 * 			version: 4
		 * 		}
		 * 	}
		 * 
		 * Example: 
		 * 	Sometime during execution if 
		 * 	a) role name has been updated and 
		 * 	b) new child has been added to a role 
		 * 	then structure will be:
		 * 	{
		 * 		"name": {
		 * 			"update": {	//event data & version}
		 * 		},
		 * 		"children": {
		 * 			"add": {	//event data & version}
		 * 		}
		 * 	}
		 */
		
		// Properties that are tracked for updates.
		this._propertyUpdates = {
			"id":{},
			"children":	{},
			"description":{},
			"filter": {},
			"inheritedIdentitiesCount":{},
			"inheritedPermissionsCount":{},
			"name":	{},
			"owners":{},
			"parents": {},
			"membership":{},
			"permissions": {},
			"sodconstraints":{},
			"totalIdentitiesCount":{},
			"totalPermissionsCount":{},
			"type":{},
			"uri":{}
		};
	},
	
	_logPropertyUpdate: function(event) {
		if (event.property in this._propertyUpdates) {	// check if this property is to be tracked.
			var data = event.data;
			if (!data) {	// if no data then empty object.
				data = {};
			}
			
			var update;
			if ((event.activity in this._propertyUpdates[event.property]) && 
					(this._propertyUpdates[event.property][event.activity])) {
				update  = this._propertyUpdates[event.property][event.activity];
			} else {
				update = {"version": 0};
				this._propertyUpdates[event.property][event.activity] = update;
			}
			
			update["data"] = data;
			// Increase the version.
			update["version"] = update["version"]+1; 
		} else {
			this._trace.info(this.id+" - Property with name: "+event.property+" is not configured for panel refresh tracking.", "_logPropertyUpdate");
		}
	},
	
	_refreshPanel: function(panelRefreshConfig, panelId) {
		if (!panelRefreshConfig) {
			return;
		}
		for (var property in panelRefreshConfig) {
			if (property in this._propertyUpdates) { // check if this property is tracked for updates.
				var propertyUpdates = this._propertyUpdates[property];	// get updates (activities) that have happened on this property.
				var propertyConfig = panelRefreshConfig[property];	 
				// check if any of the activity matches with the one specified for panel refresh configuration.
				for (var activity in propertyConfig) {
					if (!(activity in propertyUpdates)) {	// this activity has not taken place yet.
						continue;
					}
					
					var activityConfig = propertyConfig[activity];
					if (!("version" in activityConfig)) {// no version yet.
						activityConfig["version"] = 0;
					}
					
					// Update only if the version has updated. In other words some new updates have taken place.
					if ( (propertyUpdates[activity]["version"] > activityConfig["version"]) ||
						 (panelId) && (this._panelDirtyStateForSure[panelId]) ) {
						// update config version.
						activityConfig["version"] = propertyUpdates[activity]["version"];
						// run the logic per the configuration.
						var methods = activityConfig["methods"];
						for (var i=0; i<methods.length; i++) {
							// Invoke configured methods and pass event data.
							methods[i](propertyUpdates[activity]["data"], this.roleItem);
						}
					}
				}
			}
		}
	},
	
	//Method used for displaying of messages when doing batch processing.
	showMessages: function(value) {
		var mu = new com.ibm.security.modeling.util.MessageUtilities();
		var msgs = mu.extractBatchMessages(value, null, true);
		mu.showMessages(msgs, this.msgArea, true);
	},
	
	
	/*
	 * Need to have a notebook monitor, and move following functions to that.
	 *  - isEvaluatingFilter
	 *  - onFilterEvaluationStart
	 *  - onFilterEvaluationComplete
	 */
	_isEvaluatingFilter: false,
	isEvaluatingFilter: function() {
		// Summary
		//		Function to query if role filter evaluation is running.
		// returns: boolean
		//		true: represents the filter evaluation is running.
		//		false: represents the filter evaluation is not running.
		
		return this._isEvaluatingFilter;
	},
	
	onFilterEvaluationStart: function() {
		// Summary:
		//		Performs activities when role filter evaluation starts.
		
		this._isEvaluatingFilter = true;
	},
	
	onFilterEvaluationComplete: function() {
		// Summary:
		//		Performs activities when role filter evaluation completes.
		
		this._isEvaluatingFilter = false;
	},
	selectGeneralTab:function() {
		this.tabContainer.selectChild(this.generalTab);
	},
	selectErrorWarningTab: function() {
		this.tabContainer.selectChild(this.errorWarningTab);
	}
	
});
