/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

� Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.notebook.RoleNotebookErrorWarningPanel");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.security.modeling.dijit.layout.BorderContainer");
dojo.require("com.ibm.security.modeling.dijit.layout.ContentPane");
dojo.require("com.ibm.security.modeling.layouts.TemplatedPanelWithLayout");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.util.MessageUtilities");

// NLS
dojo.requireLocalization("com.ibm.security.modeling.notebook", "RoleNotebookDialogText");


// define the widget class
dojo.declare (
	"com.ibm.security.modeling.notebook.RoleNotebookErrorWarningPanel", 
    [ com.ibm.security.modeling.layouts.TemplatedPanelWithLayout ],
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.notebook.RoleNotebookErrorWarningPanel"),
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.notebook","templates/RoleNotebookErrorWarningPanel.html"),
	widgetsInTemplate: true,
	// globalResources: Object
	//			Reference to the global shared resource bundle
	globalResources: null,

	// localResources: Object
	//			Reference to the resource bundle unique to this widget
	localResources: null,
	
	// namespace: [readonly] String
	// 			Unique identifier to the instance of portlet page where this item will exist
	namespace: "",
	
	// contextRoot: String
	//			The context root of the server from which Content is being served.
	contextRoot: "",
	
	// roleUri: string
	//		String containing uri of Role being worked upon.
	roleUri: null,

	// messageArea:	ibm.tivoli.tip.dijit.TIPMessageArea
	//			This is to display error/info messages on GUI.
	messageArea: null,
	
	// restManager:	com.ibm.security.modeling.api.RestManager
	//		An instance of com.ibm.security.modeling.api.restManager to retrieve, errors and warnings attached to roles.
	//		It is an interface for HTTP REST communication using dojo stores api
	restManager: null,
	
	// _errorWarningTableStore: com.ibm.security.modeling.tables.JsonRestStore
	//		Reference to Common Table store
	_errorWarningTableStore:null,
	
	// _errorWarningTable: com.ibm.security.modeling.tables.RoleErrorWarningTable
	//		Reference to Common Table store
	_errorWarningTable:null,
	
    // saveIndicator: com.ibm.security.modeling.indicator.SaveIndicator
    //		An instance of com.ibm.security.modeling.indicator.SaveIndicator.
    //		Displays save indicator.
    saveIndicator: null,
	 // roleItem: Object
    //		An instance of role object.
	roleItem:null,
    
	constructor: function () {
		// tags:
		//		protected
		this._trace.entering("", "constructor");
		this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.notebook", "RoleNotebookDialogText");
		this._trace.exiting("", "constructor");
	},

	postCreate: function () {
		// tags:
		//		protected
		this._trace.entering(this.id, "postCreate");
		dojo.addClass(this.domNode,"contentArea");
		this.inherited("postCreate", arguments);
		this._trace.exiting(this.id, "postCreate");
	},
	
	startup: function () {
		// tags:
		//		protected
		this._trace.enteringWithStack(this.id, "startup", true);
		try {
			this.inherited("startup", arguments);
		} catch (e) {
			this._trace.finest(this.id + " - Error while starting errors and warning panel", "startup");
			this._trace.exception(e, "startup");
		}
		
		this._trace.exiting(this.id, "startup");
	},

	renderWidget: function () {
		// summary:	
		//			Need the container to resize.
		this._trace.entering(this.id, "renderWidget");
		this.resize();
		this._trace.exiting(this.id, "renderWidget");
	},

	destroy: function () {
		// tags:
		//		protected
		this._trace.enteringWithStack(this.id, "destroy", true);
		try {
			this.inherited(arguments);
			
			this._trace.exiting(this.id, "destroy");
		} catch (e) {
			this._trace.exception(e, "destroy");
			throw e;
		}
	},
	
	destroyPrevTableInstances: function() {
		this._trace.entering(this.id, "destroyPrevTableInstances");
		var oldErrorTable = dijit.byId(this.id+"_errorWarningTable");
		if (oldErrorTable) {
			oldErrorTable.destroyRecursive();
		}

		//creating a div so that table can be recreated at same div location
		this.errorWarningContainer.set("content", "");
		var newdiv = document.createElement('div');
		newdiv.setAttribute('id',this.id+"_errorWarningTable");
		this.errorWarningContainer.domNode.appendChild(newdiv);
		this._trace.exiting(this.id, "destroyPrevTableInstances");		
	},
	
	reBuild: function(roleItem) {
		this.destroyPrevTableInstances();
		this._errorWarningTable = null;
		this.populate(roleItem);
	},	
	
	populate: function(roleItem) {
		// summary:
		//		Sets role item to panel for populating data
		// tags:
		//		protected
		this._trace.entering(this.id, "populate");
		this.roleItem = roleItem;
		this.roleUri=roleItem.uri;
		this._loadErrorWarningTable();
		this._trace.exiting(this.id, "populate");
		
	},
	_refreshErrorWarningTable:function(editParameters){
		// summary:
		//		Refresh panel/table if some other panel is making it dirty
		// tags:
		//		protected
		this._trace.entering(this.id, "_refreshErrorWarningTable");
		if(this._errorWarningTable){
			this._errorWarningTable.refresh();
		}
		this._trace.exiting(this.id, "_refreshErrorWarningTable");
	},
	_refreshHandler: function() {
		this._trace.entering(this.id, "_refreshHandler");
		this.parentWidget._refreshHandler("errorWarning");
		this._trace.exiting(this.id, "_refreshHandler");
	},
	getRefreshConfig: function() {
		// Role Property to Activity mapping.
		// Representing that if the specified activity takes place on the specific role property
		//	then which function should be executed on Tab switch to this panel.
		if (!this.refreshConfig) {
			var scopedMethod = dojo.hitch(this, this._refreshErrorWarningTable);
			
			this.refreshConfig = {
				"name":{	// on update of role name; refresh error/warning field table.
					"update": {
						"methods": [scopedMethod]
					}
				},
				"children":{	// on add or remove of children; refresh error/warning field table.
					"add": {
						"methods": [scopedMethod]
					},
					"remove": {
						"methods": [scopedMethod]
					}
				},
				"parents": {	// on add or remove of parent; refresh error/warning field table.
					"add": { 
						"methods": [scopedMethod]
					},
					"remove": {
						"methods": [scopedMethod]
					}
				},
				"membership":{	// on add or remove of membership; refresh error/warning field table.
					"add": {
						"methods": [scopedMethod]
					},
					"remove": {
						"methods": [scopedMethod]
					}
				},
				"sodconstraints":{	// on add or remove of sodconstraints; refresh error/warning field table.
					"add": {
						"methods": [scopedMethod]
					},
					"remove": {
						"methods": [scopedMethod]
					}
				},
				"filter": {
					"create": {
						"methods": [scopedMethod]
					},
					"update": {
						"methods": [scopedMethod]
					},
					"delete": {
						"methods": [scopedMethod]
					}
				}		
			};
		}
		
		return this.refreshConfig;
	},
	_loadErrorWarningTable:function(){
		// summary:
		//		Loads the errors and  warnings in table for a given role
		// tags:
		//		private
		this._trace.entering(this.id, "_loadErrorWarningTable");
			if(this._errorWarningTable ==null){
				this._errorWarningTableStore=this._setupRestStore();
				this._errorWarningTable=this._createErrorWarningTable(this.id+"_errorWarningTable",this._errorWarningTableStore);
				dojo.addClass(this._errorWarningTable.domNode,"contentArea");
				this._errorWarningTable.startup();
			}
		this._trace.exiting(this.id, "_loadErrorWarningTable");
	},
	_createErrorWarningTable:function(/*string*/placeHolderId,
								/*Table store*/jsonTableStore){
		// summary:
		//			Creates the error warning table for a given role.
		// tags:
		//			private
		
		this._trace.entering(this.id, "_createErrorWarningTable");
		dojo.require("com.ibm.security.modeling.tables.RoleErrorWarningTable");
		//additionalQuery={includeMembers:"direct"};
		var errorWarningTable = new com.ibm.security.modeling.tables.RoleErrorWarningTable({dataStore: jsonTableStore, 
				id:placeHolderId,
				contextRoot:this.contextRoot,
				parentContainer:this.container,
				saveIndicator:this.saveIndicator,
				isRefreshRendered : true,
				roleItem:this.roleItem,
				parentWidget: this,
				messageArea: this.messageArea,
				globalResources:this.globalResources,
				logResources:this.logResources
		},placeHolderId);
		
		this._trace.exiting(this.id, "_createErrorWarningTable");
		return errorWarningTable;
		
		
	},
	_setupRestStore: function () {
		// summary:
		//		Setups the JsonRestStore for errors and  warnings in table for a given role
		// tags:
		//		private
		this._trace.entering("", "_setupRestStore");
		dojo.require("com.ibm.security.modeling.api.RestManager");
		
		this.restManager = new com.ibm.security.modeling.api.RestManager({
				resourceUri: this.roleItem.messagesUri,
				syncMode: false,
				messageArea: this.messageArea,
				allowNoTrailingSlash: false,
				_identifier: 'uri',
				 _resourceSchema : {
					"description":"Messages for role",
					"type":"object",
					
					"properties": {
						"uri":{"type":"string","format":"url"},
						"messageKey":{"type":"string"},
						"messageType":{"type":"string"}
					}
				}
			});
		
		var errorWarningTableStore = this.restManager.getIsrmTableRestStore();
		this._trace.exiting("", "_setupRestStore");
		return errorWarningTableStore;
		
	},
	showMessages: function(value) {
		// summary:
		//		Function used for MessageUtilities
		// tags:
		//		protected
		this._trace.entering(this.id, "showMessages");
		var mu = new com.ibm.security.modeling.util.MessageUtilities();
		var msgs = mu.extractBatchMessages(value, null, true);
		mu.showMessages(msgs, this.messageArea, true);
		this._trace.exiting(this.id, "showMessages");
	},
	
	_handleRESTError: function (error, request) {
		// summary:
		//		Handles REST layers for table. Used by table to show errors in message area
		// tags:
		//		private
		this._trace.entering(this.id, "_handleRESTError");
		this._trace.finest(this.id + " - Error: " + error, "_handleRESTError");
//		if ((this.messageArea != null) && (error != null) && (error.responseText != null)) {
//			try {
//				var message = JSON.parse(error.responseText, null);
//				if ((message != null) &&
//					(message.text != null)) {
//					this.messageArea.showErrorMessage(message.text);
//				}
//			} catch (e){
//				var messageResponseSplits = error.responseText.split("text",-1);
//				if (messageResponseSplits.length > 1) {
//					messageResponseSplits[1] = messageResponseSplits[1].replace(/^\":\"/g,"");
//					message = messageResponseSplits[1].replace(/\"}$/g,"");
//				} else {
//					message = messageResponseSplits[0];
//				}
//				if (message != null) {
//					this.messageArea.showErrorMessage(message);
//				}
//			}
//		}
		dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
		this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
		this._errorUtil.handleRESTError(error, this.messageArea);
		this._trace.exiting(this.id, "_handleRESTError");
	}
	
});
