/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.notebook.RoleNotebookGeneralPanel");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.security.modeling.layouts.TemplatedPanelWithLayout");
dojo.require("dijit.form.SimpleTextarea");

dojo.require("com.ibm.security.modeling.dijit.layout.BorderContainer");
dojo.require("com.ibm.security.modeling.dijit.layout.ContentPane");
dojo.require("com.ibm.security.modeling.dijit.layout.TabContainer");

dojo.require("com.ibm.security.modeling.api.UserRestManager");
dojo.require("com.ibm.security.modeling.notebook.RoleTypeStore");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.util.ValidationUtilities");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");
dojo.require("com.ibm.security.modeling.widgets.RequiredTextField");

dojo.require("com.ibm.security.modeling.tables.JsonBatchRest");
dojo.require("com.ibm.security.modeling.util.MessageUtilities");
dojo.require("com.ibm.security.modeling.notebook.RoleOwnerTable");
// NLS
dojo.requireLocalization("com.ibm.security.modeling", "PluginMessages");
dojo.requireLocalization("com.ibm.security.modeling.notebook", "RoleNotebookDialogText");

// define the widget class
dojo.declare (
	"com.ibm.security.modeling.notebook.RoleNotebookGeneralPanel", 
    [ com.ibm.security.modeling.layouts.TemplatedPanelWithLayout ],
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.notebook.RoleNotebookGeneralPanel"),
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.notebook","templates/RoleNotebookGeneralPanel.html"),
	validator : new com.ibm.security.modeling.util.Validator(),
	widgetsInTemplate: true,
	
	// globalResources: Object
	//			Reference to the global shared resource bundle
	globalResources: null,
	
	// pluginResources: Object
	//			Reference to the plugin shared resource bundle
	pluginResources: null,
	
	// localResources: Object
	//			Reference to the resource bundle unique to this widget
	localResources: null,
	
	// namespace: [readonly] String
	// 			Unique identifier to the instance of portlet page where this item will exist
	namespace: "",
	
	// contextRoot: String
	//			The context root of the server from which Content is being served.
	contextRoot: "",
	
	// roleItem: object
	//		An object representing a Role in dojo store.
	roleItem: null,
	
	// roleRestManager: com.ibm.security.modeling.api.RoleRestManager
	//		An instance of com.ibm.security.modeling.api.RoleRestManager. 
	//		It is an interface for HTTP REST communication using dojo stores api. 
	roleRestManager: null,
	
	// messageArea: ibm.tivoli.tip.dijit.TIPMessageArea
	//		This is to display messages on GUI when any error occurs.
	messageArea: null,

	// saveIndicator: com.ibm.security.modeling.indicator.SaveIndicator
    //		An instance of com.ibm.security.modeling.indicator.SaveIndicator.
    //		Displays save indicator.
    saveIndicator: null,
    
	
	// roleOwnerManager:	com.ibm.security.modeling.api.UserRestManager
	//		An instance of com.ibm.security.modeling.api.UserRestManager to retrieve, delete role owners.
	//		It is an interface for HTTP REST communication using dojo stores api
	roleOwnerManager: null,
	
	_ownerFieldTable: null,
	
	_selectBox: null,
	
	// connections: Object[]
    //		summary: Array of dojo method connections
    connections: [], 
    
    parentWidget: null,
    
    // _ignoreRoleDescriptionChangeEvent: boolean
    //		Setting description field causes onChange handler to execute, 
    // 		but it is needed only when user interacts with UI. 
    //		And it should not happen if description field is set via javascript program.
    _ignoreRoleDescriptionChangeEvent: false, 
    
	constructor: function () {
		this._trace.entering("", "constructor");
		this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.notebook", "RoleNotebookDialogText");
		this.ownerPickerResources = dojo.i18n.getLocalization("com.ibm.security.modeling.notebook", "OwnersTableDialogText");
		this.pluginResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "PluginMessages");
		this._trace.exiting("", "constructor");
	},

	postCreate: function () {
		this._trace.entering(this.id, "postCreate");
		dojo.addClass(this.domNode,"fullSize");
		this.inherited ("postCreate", arguments);
		
		this.roleNameField.setMessageArea(this.messageArea);
		
		// mapping widget's events with methods
		this.roleNameField._onChange = dojo.hitch(this, this.updateRoleName);
		this.roleDescriptionField.onChange = dojo.hitch(this, this.updateRoleDescription);
		
		this._trace.exiting(this.id, "postCreate");
	},

	destroy: function () {
		this._trace.enteringWithStack(this.id, "destroy", true);
		try {
			this.inherited(arguments);
			dojo.forEach(this.connections, dojo.disconnect);
			this._trace.exiting(this.id, "destroy");
		} catch (e) {
			this._trace.exception(e, "destroy");
			throw e;
		}
	},
	
	destroyPrevTableInstances: function() {
		this._trace.entering(this.id, "destroyPrevTableInstances");
		var oldOwnerFieldTable = dijit.byId(this.id+"_roleOwnersFieldTable");
		if (oldOwnerFieldTable) {
			oldOwnerFieldTable.destroyRecursive();
		} 

		//creating a div so that table can be recreated at same div location
		var newdiv = document.createElement('div');
		newdiv.setAttribute('id',this.id+"_roleOwnersFieldTable");
				
		var newdiv = document.createElement('div');
		newdiv.setAttribute('id',this.id+"_roleOwnersFieldTable");
		newdiv.setAttribute('dojoAttachPoint',"roleOwnersField");
				
		this.roleOwnerContainerDiv.appendChild(newdiv);
		this._trace.exiting(this.id, "destroyPrevTableInstances");		
	},
	
	populate: function(roleItem) {
		this._trace.entering(this.id, "populate");
		this.roleItem = roleItem;
		
		// Value should not be altered.
		// Rather its representation text should be altered while displaying it on browser. 
		this.roleNameField.setValue(this.roleItem.name);
		this._populateRoleDescriptionField();
		
		this._createRoleTypeList();
		
		this.destroyPrevTableInstances();
		this._displayRoleOwnersTable();
		
		this._trace.exiting(this.id, "populate");
	},
	
	_populateRoleDescriptionField: function() {
		if (this.roleItem && this.roleDescriptionField) {
			var descriptionOldValue = this.roleDescriptionField.get("value");
			var descriptionNewValue = this.roleItem.description;
			if (descriptionNewValue != descriptionOldValue) {
				this._ignoreRoleDescriptionChangeEvent = true;
				this.roleDescriptionField.set("value", descriptionNewValue);
			}
		}
	},
	
	reBuild: function(roleItem) {
		this.populate(roleItem);
	},
	
	
	renderWidget: function () {	
		this._trace.entering(this.id, "renderWidget");
		// summary:	
		//			Renders grid.
		this._ownerFieldTable.grid.render();
		this._trace.exiting(this.id, "renderWidget");
	},

	_createRoleTypeList: function () {
		this._trace.entering(this.id, "_createRoleTypeList");
		var initialRoleTypeValue = null;
		if (this.roleItem && this.roleItem.type) {
			initialRoleTypeValue = this.roleItem.type.uri;
			if (initialRoleTypeValue == undefined || (initialRoleTypeValue != null && initialRoleTypeValue.toString().trim() == "")) {
				initialRoleTypeValue = null;
			}
		}
		
		var roleTypeStore = new com.ibm.security.modeling.notebook.RoleTypeStore({
			target:"/isrm/rest/types/role/", 
			idAttribute:"id",
			syncMode: true
        });
		
		var roleTypeItems = roleTypeStore.fetch({query:"",onError:dojo.hitch(this,this._handleRESTError)}).results;
		
		var tempArr = new Array();
		var initialRoleTypeValueFound = false;
		selectedIndex = 0;
		if (roleTypeItems != null) {
			roleTypeItems.unshift({});
			for (var i=0; i<roleTypeItems.length; i++) {
				var name = roleTypeItems[i].name;
				var obj = {
						uri: roleTypeItems[i].uri,
						name: roleTypeItems[i].translatedName
				}
				tempArr[i]=obj;
				
				// Check if initial value found.
				if ((selectedIndex == 0) &&
					(initialRoleTypeValue != null) &&
					(initialRoleTypeValue == roleTypeItems[i].uri)) {
					selectedIndex = i;
				}
			}
		}
		
		// update HTML select element.
		this._selectBox = dojo.byId(this.id+"_roleTypeSelect");
		this._selectBox.options[0]=new Option("", "");
		for (var i=1; i< tempArr.length; i++) {
			this._selectBox.options[i]=new Option(tempArr[i].name.toString(), tempArr[i].uri.toString());
		}
		this._selectBox.selectedIndex = (selectedIndex);
		this._trace.exiting(this.id, "_createRoleTypeList");
	},
	
	_displayRoleOwnersTable: function () {
		this._trace.enteringWithStack(this.id, "_displayRoleOwnersTable", true);
		try {
			// Instantiate the User store as Role Owner Store.
			this.roleOwnerManager = new com.ibm.security.modeling.api.UserRestManager({
				resourceUri: this.roleItem.ownersUri.toString(), 
				syncMode: false,
				messageArea: this.messageArea,
				allowNoTrailingSlash: false,
				_identifier: 'id'
			});
			
			this._ownerFieldTableStore = this.roleOwnerManager.getIsrmTableRestStore();
			
			this._ownerFieldTable = new com.ibm.security.modeling.notebook.RoleOwnerTable(
	    		{
	    			id: this.id+"_roleOwnersFieldTable", 
	    			dataStore: this._ownerFieldTableStore,
	    			messageArea: this.messageArea,
					parentWidget:this,
					contextRoot: this.contextRoot,
					namespace: this.namespace,
					parentContainer:this.container,
					isPaginationEnabled:true,
					isTotalsFooterRendered:false,
					saveIndicator:this.saveIndicator,
					tableType:"roleOwnerTable"
	    		},
	    		dojo.byId(this.id+"_roleOwnersFieldTable"));
			this._ownerFieldTable.startup();
		} catch (e) {
			this._trace.exception(e, "_displayRoleOwnersTable");
		} finally {
			this._trace.exiting(this.id, "_displayRoleOwnersTable");
		}
	},
	
	refreshOwnersFieldTable: function () {
		this._trace.entering(this.id, "refreshOwnersFieldTable");
		this._trace.finest(this.id + " - Refreshing owners table", "refreshOwnersFieldTable");
		this._ownerFieldTable.refresh();
		this._trace.exiting(this.id, "refreshOwnersFieldTable");
	},
	validate: function () {
		this._trace.entering(this.id, "validate");
		// summary:
		//		Performs validations for this widget.
		var valid = false;
		valid = this._validateRoleName();
		this._trace.finest(this.id + " - Valid: " + valid, "validate");
		this._trace.exiting(this.id, "validate");
		return valid;
	},
	
	getRoleNameFromUI: function() {
		this._trace.entering(this.id, "getRoleNameFromUI");
		roleName = this.roleNameField.getValue();
		if (roleName != null) {
			roleName = roleName.trim();
		}
		this._trace.exiting(this.id, "getRoleNameFromUI");
		return roleName;
	},
	
	_validateRoleName: function (avoidDuplicateMessage) {
		this._trace.entering(this.id, "_validateRoleName");
		var valid = false;
		
		if ( (avoidDuplicateMessage == undefined) || 
			 (avoidDuplicateMessage == null) ) {
			avoidDuplicateMessage = false;
		}
		
		var roleName = this.getRoleNameFromUI();
		
		if (roleName != null) {
			roleName = roleName.trim();
			if ((roleName != null) &&
				(roleName.length > 0)) {
				valid = true;
			}
		}
		if (valid) {
			this.roleNameField.setStatus({state: this.roleNameField.constants.REQUIRED, message: ""});
		} else {
			this.roleNameField.setStatus({state: this.roleNameField.constants.ERROR, message: ""});
			if (this.messageArea != null) {
				if(avoidDuplicateMessage) {
					if(!this.messageArea.messageDisplayedAtTop(this.pluginResources.PLUGIN_REST_ROLE_NAME_MISSING)) {
						this.messageArea.showErrorMessage(this.pluginResources.PLUGIN_REST_ROLE_NAME_MISSING);
					}
				} else {
					this.messageArea.showErrorMessage(this.pluginResources.PLUGIN_REST_ROLE_NAME_MISSING);
				}
			}
		}
		this._trace.finest(this.id + " - Valid: " + valid, "_validateRoleName");
		this._trace.exiting(this.id, "_validateRoleName");
		return valid;
	},
	
	updateRoleName: function () {
		this._trace.entering(this.id, "updateRoleName");
		// 	summary: 
		//			Retrieves the data from Role Name field from GUI and makes REST call (Http PUT).
		
		// validate role name
		var valid = this._validateRoleName();
		if (!valid){
			this._trace.finest(this.id + " - Role name is not valid", "updateRoleName");
			this._trace.exiting(this.id, "updateRoleName");
			return;
		}
		
		this.saveIndicator.show();

		var roleName = this.getRoleNameFromUI();
		
		this.roleRestManager.updateRoleName(
				null,
				roleName,
				dojo.hitch (this, this.roleNameUpdateSuccess),
				dojo.hitch (this, this.rolePropertyUpdateFail));
		this._trace.exiting(this.id, "updateRoleName");
	},

	roleNameUpdateSuccess: function () {
		this._trace.entering(this.id, "roleNameUpdateSuccess");
		var roleName = this.getRoleNameFromUI();
		
		/* Using object for passing the event information. This will do away with the parsing logic required.*/
		var eventData = {};
		eventData.property = "name";
		eventData.activity = "update";
		eventData.data = {
			"oldValue" : this.roleItem.name,
			"newValue": roleName
		};
		
		/* Update the role item. */
		this.roleItem.name = roleName;
		
		this.rolePropertyUpdateSuccess(eventData);
		
		// Update Dialog header as it contains name of the role being edited in notebook.
		this.parentWidget.updateDialogHeader();		
		this._trace.exiting(this.id, "roleNameUpdateSuccess");
	},
	
	updateRoleDescription: function () {
		this._trace.entering(this.id, "updateRoleDescription");
		// summary: 
		//			Retrieves the data from Role Description field from GUI and makes REST call (Http PUT).
		if (this._ignoreRoleDescriptionChangeEvent) {
			this._ignoreRoleDescriptionChangeEvent = false;
			return;
		}
		
		var roleDescription = this.roleDescriptionField.get("value");
		
		// Create event
		var eventData = {};
		eventData.property = "description";
		eventData.activity = "update";
		eventData.data = {
			"oldValue" : this.roleItem.description,
			"newValue": roleDescription
		};
		
		this.saveIndicator.show();
		this.roleRestManager.updateRoleDescription(
				null,
				roleDescription,
				dojo.hitch (this, this.rolePropertyUpdateSuccess, eventData),
				dojo.hitch (this, this.rolePropertyUpdateFail));
		this._trace.exiting(this.id, "updateRoleDescription");
	},
	
	updateRoleType: function () {
		this._trace.entering(this.id, "updateRoleType");
		// summary: 
		//			Retrieves the data from Role Type field from GUI and makes REST call (Http PUT).
		var roleType = this._selectBox.value;
		
		// Create event
		var eventData = {};
		eventData.property = "type";
		eventData.activity = "update";
		eventData.data = {
			"oldValue" : this.roleItem.type,
			"newValue": roleType
		};
		
		
		this.saveIndicator.show();
		this.roleRestManager.updateRoleType(
				null,
				roleType,
				dojo.hitch (this, this.rolePropertyUpdateSuccess, eventData),
				dojo.hitch (this, this.rolePropertyUpdateFail));
		this._trace.exiting(this.id, "updateRoleType");
	},	

	rolePropertyUpdateSuccess: function (eventData) {
		this._trace.entering(this.id, "rolePropertyUpdateSuccess");
		this._publishRolePropertyEdit(eventData);	// publish role property edit event
		this.saveIndicator.hide();
		this._trace.exiting(this.id, "rolePropertyUpdateSuccess");
	},

	rolePropertyUpdateFail: function (error, request) {
		this._trace.entering(this.id, "rolePropertyUpdateFail");
		this.saveIndicator.hide();
		this._handleRESTError(error, request);
		this._trace.exiting(this.id, "rolePropertyUpdateFail");
	},
	
	_publishRolePropertyEdit: function(eventData) {
		this._trace.entering(this.id, "_publishRolePropertyEdit");
		dojo.publish("ROLE_PROPERTY_UPDATE_EVENT",[eventData]);
		this._trace.exiting(this.id, "_publishRolePropertyEdit");
	},
	
	showMessages: function(value) {
		var mu = new com.ibm.security.modeling.util.MessageUtilities();
		var msgs = mu.extractBatchMessages(value, null, true);
		mu.showMessages(msgs, this.messageArea, true);
	},
	
	_handleRESTError: function (error, request) {
		this._trace.entering(this.id, "_handleRESTError");
		this._trace.finest(this.id + " - Error: " + error, "_handleRESTError");
//		if ((this.messageArea != null) && (error != null) && (error.responseText != null)) {
//			try {
//				var message = JSON.parse(error.responseText, null);
//				if ((message != null) &&
//					(message.text != null)) {
//					this.messageArea.showErrorMessage(message.text);
//				}
//			} catch (e){
//				var messageResponseSplits = error.responseText.split("text",-1);
//				if (messageResponseSplits.length > 1) {
//					messageResponseSplits[1] = messageResponseSplits[1].replace(/^\":\"/g,"");
//					message = messageResponseSplits[1].replace(/\"}$/g,"");
//				} else {
//					message = messageResponseSplits[0];
//				}
//				if (message != null) {
//					if ( (error.status == 404) && (message.search("SRVE0255E") != -1) ) {
//						var serverDownMessage = this.pluginResources.VALIDATION_ROLE_REST_SERVER_NOT_REACHABLE ;
//						this.messageArea.showErrorMessage(serverDownMessage);
//					} else {
//						this.messageArea.showErrorMessage(message);
//					}
//				}
//			}
//		}
		dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
		this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
		this._errorUtil.handleRESTError(error, this.messageArea);
		this._trace.exiting(this.id, "_handleRESTError");
	}
	
});
