/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

Â© Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.notebook.RoleNotebookHierarchicalRelationPanel");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.security.modeling.layouts.TemplatedPanelWithLayout");
dojo.require("dijit.Dialog");
dojo.require("com.ibm.security.modeling.dijit.layout.BorderContainer");
dojo.require("com.ibm.security.modeling.dijit.layout.ContentPane");

dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.widgets.RolePickerDialog");
dojo.require("com.ibm.security.modeling.tables.RoleTable");
dojo.require("com.ibm.security.modeling.util.MessageUtilities");

// NLS
dojo.requireLocalization("com.ibm.security.modeling.notebook", "RoleNotebookDialogText");
dojo.requireLocalization("com.ibm.security.modeling.notebook", "RoleTableText");
dojo.requireLocalization("com.ibm.security.modeling.notebook", "ParentRolePickerText");
dojo.requireLocalization("com.ibm.security.modeling.notebook", "ChildRolePickerText");
dojo.requireLocalization("com.ibm.security.modeling", "LogMessages");


// define the widget class
dojo.declare (
	"com.ibm.security.modeling.notebook.RoleNotebookHierarchicalRelationPanel", 
    [ com.ibm.security.modeling.layouts.TemplatedPanelWithLayout ],
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.notebook.RoleNotebookHierarchicalRelationPanel"),
	_messageUtils: new com.ibm.security.modeling.util.MessageUtilities(),
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.notebook","templates/RoleNotebookHierarchicalRelationPanel.html"),
	widgetsInTemplate: true,

	// localResources: Object
	//			Reference to the resource bundle unique to this widget
	localResources: null,
	
	// localRoleTableResources: Object
	//			Reference to the resource bundle unique to role table for notebook.
	localRoleTableResources: null,
	
	// namespace: [readonly] String
	// 			Unique identifier to the instance of portlet page where this item will exist
	namespace: "",
	
	// contextRoot: String
	//			The context root of the server from which Content is being served.
	contextRoot: "",
	
	constructor: function () {
		this._trace.entering("", "constructor");
		this.logResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "LogMessages");
		
		this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.notebook", "RoleNotebookDialogText");
		this.localRoleTableResources = dojo.i18n.getLocalization("com.ibm.security.modeling.notebook", "RoleTableText");
		this.parentRolePickerResources = dojo.i18n.getLocalization("com.ibm.security.modeling.notebook", "ParentRolePickerText");
		this.childRolePickerResources = dojo.i18n.getLocalization("com.ibm.security.modeling.notebook", "ChildRolePickerText");
		
		
		// messageArea:	ibm.tivoli.tip.dijit.TIPMessageArea
		//			This is to display error/info messages on GUI.
		
		// saveIndicator: com.ibm.security.modeling.indicator.SaveIndicator
	    //		An instance of com.ibm.security.modeling.indicator.SaveIndicator.
	    //		Displays save indicator.
		
		// _childRolesFieldTable: com.ibm.security.modeling.tables.RoleTable
		//			Roles Table widget.
		
		// _childStore: Objects
		//			An instance of Write store for child roles table.
		
		// _parentRolesFieldTable: com.ibm.security.modeling.tables.RoleTable
		//			Roles Table widget.
		
		// _parentStore: Objects
		//			An instance of store for parent roles table.
		
		this._trace.exiting("", "constructor");
	},

	postCreate: function () {
		this._trace.entering(this.id, "postCreate");
		dojo.addClass(this.domNode,"fullSize");
		this.inherited ("postCreate", arguments);		
		this._trace.exiting(this.id, "postCreate");
	},
	
	startup: function () {
		this._trace.enteringWithStack(this.id, "startup", true);
		try {
			this.inherited ("startup", arguments);
		} catch (e) {
			this._trace.finest(this.id + " - Error while starting hierarchical relationship panel", "startup");
			this._trace.exception(e, "startup");
		}
		
		this._trace.exiting(this.id, "startup");
	},

	renderWidget: function () {
		this._trace.entering(this.id, "renderWidget");
		this.resize();
		this._childRolesFieldTable.grid.render();
		this._parentRolesFieldTable.grid.render();
		this._trace.exiting(this.id, "renderWidget");
	},

	destroy: function () {
		this._trace.enteringWithStack(this.id, "destroy", true);
		try {
			this.inherited(arguments);
			this._trace.exiting(this.id, "destroy");
		} catch (e) {
			this._trace.exception(e, "destroy");
			throw e;
		}
	},
	
	populate: function(roleItem) {

		try {
			this._trace.entering(this.id, "populate");
			this.roleItem = roleItem;
			// create hierarchical tables
			this._createChildRolesTable();
			this._createParentRolesTable();
			
			// startup hierarchical tables
			this._childRolesFieldTable.startup();
			this._parentRolesFieldTable.startup();
			
		} catch (e) {
			this._trace.exception(e, "startup");
			throw e;
		} finally {
			this._trace.exiting(this.id, "populate");
		}
		
	},
	
	reBuild: function(roleItem) {
		this.populate(roleItem);
	},
	
	
	// creates Role Hierarchy (Parent/Child) table.
	_createHierarchyTable: function(tableConfig) {
		this._trace.entering(this.id, "_createHierarchyTable");
		
		this._decorateStoreNewItem(tableConfig.store);
		// Setup child roles table
		
		var table = dijit.byId(tableConfig.tableId);
		if(table) {
			table.setStore(tableConfig.store,"");
			return table;
		}
		 table = new com.ibm.security.modeling.tables.RoleTable ({
			id: tableConfig.tableId,
			dataStore: tableConfig.store, 
			isClientSide: true,
			isFilterRendered: false,
			isRefreshRendered: true,
			saveIndicatorText: this.saveIndicatorText,
			saveIndicator: this.saveIndicator,
			messageArea: this.messageArea,
			pluginResources: this.pluginResources
		}, tableConfig.tableHook);
		
		table.addToolbarSeparator();
		// Add, Remove buttons.		
		table.createAddButton(tableConfig.addButton.label, tableConfig.addButton.callback);
		table.createRemoveButton(
			tableConfig.removeButton.label,		// label
			null,								// default delete function
			tableConfig.removeButton.callback, 	// onComplete
			true								// do not show REST errors
		);
		
		this._trace.exiting(this.id, "_createHierarchyTable");
		return table;
	},
	
	_createChildRolesTable: function () {
		this._trace.entering(this.id, "_createChildRolesTable");
		// Instantiate store for Role's Children.
		this._childStore = new com.ibm.security.modeling.tables.JsonRestStore({
			target: this.roleItem.childrenUri.toString(),
			idAttribute: 'uri'
		});
		
		var tableConfig = {
				store: this._childStore, 
				tableId: this.id+"_childRolesFieldTable",  
				tableHook: this.id+"_childRolesFieldTable",
				addButton: {
					label: this.localRoleTableResources.ADD_CHILD,
					callback: dojo.hitch(this, this.openAddChildDialog)
				},
				removeButton: {
					label: this.localRoleTableResources.DELETE_CHILD,
					callback: dojo.hitch(this, this._hierarchyEditComplete, {
						property: 'children', 
						activity: 'remove',
						deleteFailMsg: this.logResources.ROLE_CHILDREN_DELETE_FAILED,
						deletePartialMsg: this.logResources.ROLE_CHILDREN_DELETED_PARTIAL
					})
				}
		};
		
		this._childRolesFieldTable = this._createHierarchyTable(tableConfig);
		
		this._trace.exiting(this.id, "_createChildRolesTable");
	},
	
	_createParentRolesTable: function () {
		this._trace.entering(this.id, "_createParentRolesTable");
		
		this._parentStore = new com.ibm.security.modeling.tables.JsonRestStore({
			target: this.roleItem.parentsUri.toString(),
			idAttribute: 'uri'
		}); 
			
		var tableConfig = {
				store: this._parentStore, 
				tableId: this.id+"_parentRolesFieldTable", 
				tableHook: this.id+"_parentRolesFieldTable",
				addButton: {
					label: this.localRoleTableResources.ADD_PARENT,
					callback: dojo.hitch(this, this.openAddParentDialog)
				},
				removeButton: {
					label: this.localRoleTableResources.DELETE_PARENT,
					callback: dojo.hitch(this, this._hierarchyEditComplete, {
						property: 'parents', 
						activity: 'remove',
						deleteFailMsg: this.logResources.ROLE_PARENT_DELETE_FAILED,
						deletePartialMsg: this.logResources.ROLE_PARENT_DELETED_PARTIAL
					})
				}
		};
		
		this._parentRolesFieldTable = this._createHierarchyTable(tableConfig);
		this._trace.exiting(this.id, "_createParentRolesTable");
	},
	
	_refreshTables: function(editParameters) {
		
		if (editParameters.activity == 'remove') {
			if (editParameters.property == 'children') {
				this._parentRolesFieldTable.refresh();
			}
			if (editParameters.property == 'parents') {
				this._childRolesFieldTable.refresh();
			}
		}
		
		if (editParameters.activity == 'add') {
			this._parentRolesFieldTable.refresh();
			this._childRolesFieldTable.refresh();
		}
	},
	
	getRefreshConfig: function() {
		// Role Property to Activity mapping.
		// Representing that if the specified activity takes place on the specific role property
		//	then which function should be executed on Tab switch to this panel.
		if (!this.refreshConfig) {
			var refreshParent = dojo.hitch(this, this.refreshParentFieldTable);
			var refreshChild = dojo.hitch(this, this.refreshChildFieldTable);
			this.refreshConfig = {
				"name":{	// on update of role name; refresh parent, child field table.
					"update": {
						"methods": [refreshParent, refreshChild]
					}
				},
				"membership":{	// on add or remove of membership; refresh parent field table.
					"add": {
						"methods": [refreshParent]
					},
					"remove": {
						"methods": [refreshParent]
					}
				},
				"permissions": {
					"add": { 
						"methods": [refreshChild]
					},
					"remove": {
						"methods": [refreshChild]
					}
				}
			};
		}
		
		return this.refreshConfig;
	},
	
	refreshParentFieldTable: function() {
		if (this._parentRolesFieldTable) {
			this._parentRolesFieldTable.refresh();
		}
	},
	
	refreshChildFieldTable: function() {
		if (this._childRolesFieldTable) {
			this._childRolesFieldTable.refresh();
		}
	},
	
	openAddChildDialog: function () {
		this._trace.entering(this.id, "openAddChildDialog");
		
		this.childRolePickerResources.TABLE_INSTRUCTION = dojo.string.substitute(
			this.childRolePickerResources.TABLE_INSTRUCTION_TEMPLATE,{"roleName":this.roleItem.name});
		
		var uri = this.roleItem.childCandidatesUri.toString();
		
		// Instantiate store for Role's Children.
		this._childCandidateStore = new com.ibm.security.modeling.tables.JsonRestStore({
			target: this._getTableStoreTargetUri(uri),
			idAttribute: 'id',
			syncMode: false
		});
		
		this._childCandidateStore.isUpdateable = function(request) {
			return false;
		};
		var editParameters = {
			property: 'children', 
			activity: 'add'
		};
		
		// Instantiate picker
		var childRolesPicker = new com.ibm.security.modeling.widgets.RolePickerDialog({
			id: this.id+"_childPickerDialog", 
			namespace:this.namespace,
			tableQuery: this._getTableQueryObject(uri),
			fromDataStore: this._childCandidateStore,	// for picker table.
			toDataStore: this._childStore,
			toDataStoreNoContent: true,
			callback: dojo.hitch(this, this._hierarchyEditComplete, editParameters),
			resourceUri: this.roleItem.childrenUri.toString(),
			parentWidget: this,
			parentMessageArea: this.messageArea,
			saveIndicatorText: this.saveIndicatorText,
			saveIndicator: this.saveIndicator,
			contextRoot: this.contextRoot,
			globalResources: this.globalResources,	// for picker table.
			localResources: this.childRolePickerResources,	// for picker title, instructions.
			helpFile: "ui_roles_child_add.html"
		}, null);
		
		// Startup and show picker
		childRolesPicker.startup();
		childRolesPicker.show();
		
		this._trace.exiting(this.id, "openAddChildDialog");
	},

	openAddParentDialog: function () {
		this._trace.entering(this.id, "openAddParentDialog");
		
		this.parentRolePickerResources.TABLE_INSTRUCTION = dojo.string.substitute(
			this.parentRolePickerResources.TABLE_INSTRUCTION_TEMPLATE,{"roleName":this.roleItem.name});
		
		var uri = this.roleItem.parentCandidatesUri.toString();
		
		// Instantiate store for Role's Parent.
		this._parentCandidateStore = new com.ibm.security.modeling.tables.JsonRestStore({
				target: this._getTableStoreTargetUri(uri),
				idAttribute: 'id',
				syncMode: false
			});
		
		this._parentCandidateStore.isUpdateable = function(request) {
			return false;
		};
		
		var editParameters = {
			property: 'parents', 
			activity: 'add'
		};
		
		// Instantiate picker
		var parentRolesPicker = new com.ibm.security.modeling.widgets.RolePickerDialog({
			id: this.id+"_parentPickerDialog", 
			namespace:this.namespace,
			tableQuery: this._getTableQueryObject(uri),
			fromDataStore: this._parentCandidateStore,	// for picker table.
			toDataStore: this._parentStore,
			toDataStoreNoContent: true,
			callback: dojo.hitch(this, this._hierarchyEditComplete, editParameters),
			resourceUri: this.roleItem.parentsUri.toString(),
			parentWidget: this,
			parentMessageArea: this.messageArea,
			saveIndicatorText: this.saveIndicatorText,			
			saveIndicator: this.saveIndicator,
			contextRoot: this.contextRoot,
			globalResources: this.globalResources,	// for picker table.
			localResources: this.parentRolePickerResources,
			helpFile: "ui_roles_parent_add.html"
		}, null);
		
		// Startup and show picker
		parentRolesPicker.startup();
		parentRolesPicker.show();
		
		this._trace.exiting(this.id, "openAddParentDialog");
	},
	
	
	_decorateStoreNewItem: function(store) {
		var oni = dojo.hitch(store, store.newItem);
		store.newItem = dojo.hitch(store, function(data, parentInfo) {
			console.log("Start: _childStore.newItem");
			var newData = {
				id: data.id,
				uri: data.uri,
				name: data.name,
				description: data.description,
				children: [],
				parents: [],
				totalIdentitiesCount: data.totalIdentitiesCount,
				inheritedIdentitiesCount: data.inheritedIdentitiesCount,
				totalPermissionsCount: data.totalPermissionsCount,
				inheritedPermissionsCount: data.inheritedPermissionsCount
			};
			/* console.dir seems to crash Javascript on IE. Hence, need to ensure that we are doing this only for FF. */
			if (!dojo.isIE) {
				console.dir(newData);
			}
			return oni(newData, parentInfo);
		});
		
	},
	
	_handleRESTError: function (error, request) {
		this._trace.entering(this.id, "_handleRESTError");
		this._trace.finest(this.id + " - Error: " + error, "_handleRESTError");
		dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
		this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
		this._errorUtil.handleRESTError(error, this.messageArea);
		this._trace.exiting(this.id, "_handleRESTError");
	},

	_hierarchyEditComplete: function (editParameters, args) {
		try {
			this._trace.entering(this.id, "_hierarchyEditComplete");
			this._refreshTables(editParameters);
			
			// some success
			var result = args[0];	// response 'value'
			var counts = args[1];	// success, failure counts
			if (counts.successCount > 0) {
				this._publishRolePropertyEdit(editParameters);
			}
	
			// some failure
			if (counts.failureCount > 0) {
				// Show messages
				if (editParameters.activity == 'remove' && this.messageArea) {	// some failure/s occurred during 'remove'
					var failureMessage = this._messageUtils.formatMessage(editParameters.deleteFailMsg, [counts.failureCount]);
					this.messageArea.showErrorMessage(failureMessage);
					if (counts.successCount>0) {
						// This should go more as an Info message.
						var warningMessage = this._messageUtils.formatMessage(editParameters.deletePartialMsg, [counts.successCount]);
						this.messageArea.showInfoMessage(warningMessage);
					}
				}	
			}
		} finally {
			this._trace.exiting(this.id, "_hierarchyEditComplete");
		}
	},
	
	_publishRolePropertyEdit: function(editParameters) {
		this._trace.entering(this.id, "_publishRolePropertyEdit");
		this._trace.finest(this.id+" - property: "+editParameters.property+", activity: "+editParameters.activity, "_publishRolePropertyEdit");
		// Edit took place.
		var eventData = {};
		eventData.property = editParameters.property;
		eventData.activity = editParameters.activity;
		dojo.publish("ROLE_PROPERTY_UPDATE_EVENT",[eventData]);
		this._trace.exiting(this.id, "_publishRolePropertyEdit");
	},
	
	_getTableStoreTargetUri: function(targetUri) {
		if (targetUri.indexOf("?") != -1) {
			return targetUri.split("?")[0];
		}
		return targetUri;
	},
	
	_getTableQueryObject: function(targetUri) {
		if (targetUri.indexOf("?") != -1) {
			return dojo.queryToObject(targetUri.split("?")[1]);
		}
		return dojo.queryToObject(targetUri);
	}
});
