/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

� Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.notebook.RoleNotebookLifecycleRequestsPanel");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.security.modeling.dijit.layout.BorderContainer");
dojo.require("com.ibm.security.modeling.dijit.layout.ContentPane");
dojo.require("com.ibm.security.modeling.layouts.TemplatedPanelWithLayout");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.util.MessageUtilities");

// NLS
dojo.requireLocalization("com.ibm.security.modeling.notebook", "RoleNotebookDialogText");
dojo.requireLocalization("com.ibm.security.modeling.lifecycle", "LifecycleText");

dojo["requireLocalization"]("customizations", "LifecycleText");
dojo.requireLocalization("com.ibm.security.modeling.tables", "TableText");


// define the widget class
dojo.declare (
	"com.ibm.security.modeling.notebook.RoleNotebookLifecycleRequestsPanel", 
    [ com.ibm.security.modeling.layouts.TemplatedPanelWithLayout ],
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.notebook.RoleNotebookLifecycleRequestsPanel"),
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.notebook","templates/RoleNotebookLifecycleRequestsPanel.html"),
	widgetsInTemplate: true,
	// globalResources: Object
	//			Reference to the global shared resource bundle
	globalResources: null,
	
	// instructionsResources: Object
	//			Reference to the resource bundle unique to this widget and provides instructions
	instructionsResources: null,
	// localResources: Object
	//			Reference to the resource bundle unique to this widget
	localResources: null,
	// lifecycleResources: Object
	//			Reference to the resource bundle unique to this widget
	lifecycleResources: null,
	// tableResources: Object
	//			Reference to the resource bundle unique to this widget
	tableResources: null,
	
	// roleUri: string
	//		String containing uri of Role being worked upon.
	roleUri: null,

	// messageArea:	ibm.tivoli.tip.dijit.TIPMessageArea
	//			This is to display error/info messages on GUI.
	messageArea: null,
	
	// restManager:	com.ibm.security.modeling.api.RestManager
	//		An instance of com.ibm.security.modeling.api.restManager to retrieve, lifecycle requests attached to roles.
	//		It is an interface for HTTP REST communication using dojo stores api
	restManager: null,
	
	// _lifecycleRequestTableStore: com.ibm.security.modeling.tables.JsonRestStore
	//		Reference to Common Table store
	_lifecycleRequestTableStore:null,
	
	// _lifecycleRequestTable: com.ibm.security.modeling.tables.RoleLifecycleRequestTable
	//		Reference to Common Table store
	_lifecycleRequestTable:null,
	
	 // roleItem: Object
    //		An instance of role object.
	roleItem:null,
	attributeMap : {
		lifecycleStatusLabel : {node : "lifecycleStatusNode", type : "innerHTML"}
	},
    
	constructor: function () {
		// tags:
		//		protected
		this._trace.entering("", "constructor");
		this.instructionsResources = dojo.i18n.getLocalization("com.ibm.security.modeling.notebook", "RoleNotebookDialogText");
		this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.lifecycle", "LifecycleText");
		if (!this.customText) {
			this.customText = dojo.i18n.getLocalization("customizations", "LifecycleText");
		}
		if(! this.tableResources){
			this.tableResources = dojo.i18n.getLocalization("com.ibm.security.modeling.tables", "TableText");
		}
		if (!this.lifecycleResources) {
			this.lifecycleResources = dojo.i18n.getLocalization("customizations", "LifecycleText");
		}
		this._trace.exiting("", "constructor");
	},

	postCreate: function () {
		// tags:
		//		protected
		this._trace.entering(this.id, "postCreate");
		dojo.addClass(this.domNode,"contentArea");
		this.inherited("postCreate", arguments);
		//this.lifecycleStatusField.setDisabled(true);
		this._trace.exiting(this.id, "postCreate");
	},
	
	startup: function () {
		// tags:
		//		protected
		this._trace.enteringWithStack(this.id, "startup", true);
		try {
			this.inherited("startup", arguments);
		} catch (e) {
			this._trace.finest(this.id + " - Error while starting lifecycle request panel", "startup");
			this._trace.exception(e, "startup");
		}
		
		this._trace.exiting(this.id, "startup");
	},

	renderWidget: function () {
		// summary:	
		//			Need the container to resize.
		this._trace.entering(this.id, "renderWidget");
		this.resize();
		this._trace.exiting(this.id, "renderWidget");
	},

	destroy: function () {
		// tags:
		//		protected
		this._trace.enteringWithStack(this.id, "destroy", true);
		try {
			this.inherited(arguments);
			
			this._trace.exiting(this.id, "destroy");
		} catch (e) {
			this._trace.exception(e, "destroy");
			throw e;
		}
	},
	
	destroyPrevTableInstances: function() {
		this._trace.entering(this.id, "destroyPrevTableInstances");
		var oldRequestTable = dijit.byId(this.id+"_lifecycleRequestTable");
		if (oldRequestTable) {
			oldRequestTable.destroyRecursive();
		}

		//creating a div so that table can be recreated at same div location
		this.lifecycleRequestContainer.set("content", "");
		
		var newdiv = document.createElement('div');
		newdiv.setAttribute('id',this.id+"_lifecycleRequestTable");
		this.lifecycleRequestContainer.domNode.appendChild(newdiv);
		this._trace.exiting(this.id, "destroyPrevTableInstances");		
	},
	
	reBuild: function(roleItem) {
		this.destroyPrevTableInstances();
		this._lifecycleRequestTable = null;
		this.populate(roleItem);
	},	
	
	populate: function(roleItem) {
		// summary:
		//		Sets role item to panel for populating data
		// tags:
		//		protected
		this._trace.entering(this.id, "populate");
		this.roleItem = roleItem;
		this.roleUri=roleItem.uri;
		this._setRoleStatus(roleItem.status);
		this._loadLifecycleRequestTable();
		this._trace.exiting(this.id, "populate");
		
	},
	_setRoleStatus: function(status){
		var label = "";
		if(status){
			label = this._getRoleLifecycleStatusPerLocale(status);
			label = com.ibm.security.modeling.util.HtmlEscapingUtils.breakString(label,80);
		}else{
			label = this.tableResources.NO_DATA;
		}
		
		this.set("lifecycleStatusLabel",label);	
	},
	_refreshLifecycleRequestTable:function(editParameters){
		// summary:
		//		Refresh panel/table if some other panel is making it dirty
		// tags:
		//		protected
		this._trace.entering(this.id, "_refreshErrorWarningTable");
		if(this._lifecycleRequestTable){
			this._lifecycleRequestTable.refresh();
		}
		this._trace.exiting(this.id, "_refreshErrorWarningTable");
	},
	_refreshRoleStatus: function (roleItem) {
		this._trace.entering("", "_refreshRoleStatus");

		if(roleItem && roleItem.status){
			
			if(this.roleItem.roleStatus != roleItem.status){
				this.parentWidget.isRoleEdited = true;
			}
			this.roleItem = roleItem;
			
			this._setRoleStatus(roleItem.status);	
		}
		
		this._trace.exiting("", "_refreshRoleStatus");
	},
	_refreshHandler: function() {
		this._trace.entering(this.id, "_refreshHandler");
		this.parentWidget._refreshHandler("lifecycleRequest");
		this._trace.exiting(this.id, "_refreshHandler");
	},
	_getRoleLifecycleStatusPerLocale: function(/*status enum key*/ statusCode) {
		var text = null;
		if (statusCode) {
			var mappedStatusCodeKey = "LIFECYCLE_ROLE_STATUS_"+(statusCode.toUpperCase());
			text = this.lifecycleResources[mappedStatusCodeKey];
		}
		return text ? text : statusCode;
	},
	_loadLifecycleRequestTable:function(){
		// summary:
		//		Loads the lifcycle requests in table for a given role
		// tags:
		//		private
		this._trace.entering(this.id, "_loadLifecycleRequestTable");
			if(this._lifecycleRequestTable ==null){
				this._lifecycleRequestTableStore=this._setupRestStore();
				this._lifecycleRequestTable=this._createLifecycleRequestTable(this.id+"_lifecycleRequestTable",this._lifecycleRequestTableStore);
				dojo.addClass(this._lifecycleRequestTable.domNode,"contentArea");
				this._lifecycleRequestTable.startup();
			}
		this._trace.exiting(this.id, "_loadLifecycleRequestTable");
	},
	_createLifecycleRequestTable:function(/*string*/placeHolderId,
								/*Table store*/jsonTableStore){
		// summary:
		//			Creates the lifcycle requests table for a given role.
		// tags:
		//			private
		
		this._trace.entering(this.id, "_createLifecycleRequestTable");
		dojo.require("com.ibm.security.modeling.tables.RoleLifecycleRequestTable");
		var additionalQuery={"roleId":this.roleItem.id};
		var _lifecycleRequestTable = new com.ibm.security.modeling.tables.RoleLifecycleRequestTable({dataStore: jsonTableStore, 
				id:placeHolderId,
				parentContainer:this.container,
				isRefreshRendered : true,
				roleItem:this.roleItem,
				parentWidget: this,
				messageArea: this.messageArea,
				// globalResources:this.globalResources,
				logResources:this.logResources,
				lifecycleResources:this.localResources,
				customText:this.customText,
				query:additionalQuery
		},placeHolderId);
		dojo.connect(_lifecycleRequestTable,"refresh",this,	dojo.hitch(this,function(){
						this.parentWidget.roleRestManager.requestResource(null,	dojo.hitch (this, this._refreshRoleStatus), null);
					}
				)
			);
		this._trace.exiting(this.id, "_createLifecycleRequestTable");
		return _lifecycleRequestTable;
		
		
	},
	_setupRestStore: function () {
		// summary:
		//		Setups the JsonRestStore for lifcycle requests in table for a given role
		// tags:
		//		private
		this._trace.entering("", "_setupRestStore");
		dojo.require("com.ibm.security.modeling.api.RestManager");
		
		this.restManager = new com.ibm.security.modeling.api.RestManager({
				resourceUri: "/isrm/rest/lifecycle/role/requests/",
				syncMode: false,
				messageArea: this.messageArea,
				allowNoTrailingSlash: false,
				_identifier: 'uri'
			});
		
		var lifecycleRequestTableStore = this.restManager.getIsrmTableRestStore();
		this._trace.exiting("", "_setupRestStore");
		return lifecycleRequestTableStore;
		
	},
	showMessages: function(value) {
		// summary:
		//		Function used for MessageUtilities
		// tags:
		//		protected
		this._trace.entering(this.id, "showMessages");
		var mu = new com.ibm.security.modeling.util.MessageUtilities();
		var msgs = mu.extractBatchMessages(value, null, true);
		mu.showMessages(msgs, this.messageArea, true);
		this._trace.exiting(this.id, "showMessages");
	},
	
	_handleRESTError: function (error, request) {
		// summary:
		//		Handles REST layers for table. Used by table to show errors in message area
		// tags:
		//		private
		this._trace.entering(this.id, "_handleRESTError");
		this._trace.finest(this.id + " - Error: " + error, "_handleRESTError");
		dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
		this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
		this._errorUtil.handleRESTError(error, this.messageArea);
		this._trace.exiting(this.id, "_handleRESTError");
	}
	
});
