/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

� Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.notebook.RoleNotebookPermissionsPanel");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.security.modeling.layouts.TemplatedPanelWithLayout");
dojo.require("com.ibm.security.modeling.dijit.layout.BorderContainer");
dojo.require("com.ibm.security.modeling.dijit.layout.ContentPane");

dojo.require("com.ibm.security.modeling.tables.RolePermissionTable");

dojo.require("com.ibm.security.modeling.util.Trace");
// NLS
dojo.requireLocalization("com.ibm.security.modeling.notebook", "RoleNotebookDialogText");

// define the widget class
dojo.declare (
	"com.ibm.security.modeling.notebook.RoleNotebookPermissionsPanel", 
    [ com.ibm.security.modeling.layouts.TemplatedPanelWithLayout ],
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.notebook.RoleNotebookPermissionsPanel"),
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.notebook","templates/RoleNotebookPermissionsPanel.html"),
	widgetsInTemplate: true,
	
	// namespace: [readonly] String
	// 			Unique identifier to the instance of portlet page where this item will exist
	namespace: "",
	
	// contextRoot: String
	//			The context root of the server from which Content is being served.
	contextRoot: "",
	
	constructor: function () {
		this._trace.entering("", "constructor");
		
		// localResources: Object
		//		Reference to the resource bundle unique to this widget
		// messageArea:	com.ibm.security.modeling.widgets.MultiMessageArea
		//		This is to display error/info messages on GUI.
		// saveIndicator: com.ibm.security.modeling.indicator.SaveIndicator
	    //		An instance of com.ibm.security.modeling.indicator.SaveIndicator.
	    //		Displays save indicator.
		// _permissionsFieldTable: com.ibm.security.modeling.tables.RolePermissionTable
		//			Permission Table.
		// _permissionsStore: Objects
		//		An instance of Write store for permissions table.
		// parentWidget: Widget Object
		//		Parent widget object
		
		this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.notebook", "RoleNotebookDialogText");
		
		this._trace.exiting("", "constructor");
	},

	postCreate: function () {
		this._trace.entering(this.id, "postCreate");
		
		dojo.addClass(this.domNode,"contentArea");
		this.inherited ("postCreate", arguments);
		this.tunePanelWidgets();
		this._trace.exiting(this.id, "postCreate");
	},
	
	startup: function () {
		this._trace.enteringWithStack(this.id, "startup", true);
		try {
			this.inherited ("startup", arguments);
		} catch (e) {
			this._trace.finest(this.id + " - Error while starting permissions panel", "startup");
			this._trace.exception(e, "startup");
		}
		
		this._trace.exiting(this.id, "startup");
	},
	
	destroy: function () {
		this._trace.enteringWithStack(this.id, "destroy", true);
		try {
			this.inherited(arguments);
		} catch (e) {
			this._trace.exception(e, "destroy");
			throw e;
		} finally {
			this._trace.exiting(this.id, "destroy");
		}
	},
	
	tunePanelWidgetsHighRes: function() {
		dojo.addClass(this.panelBodyArea.containerNode, "permissionTableHeight");
		dojo.removeClass(this.panelBodyArea.containerNode, "permissionTableHeightLowRes");
		
		if(dojo.isIE) {
			dojo.removeClass(this.panelHeaderArea.containerNode, "permissionTableWidthLowRes");
			dojo.removeClass(this.panelBodyArea.containerNode, "permissionTableWidthLowRes");
		}
		
	},

	tunePanelWidgetsLowRes: function() {
		dojo.addClass(this.panelBodyArea.containerNode, "permissionTableHeightLowRes");
		dojo.removeClass(this.panelBodyArea.containerNode, "permissionTableHeight");

		if(dojo.isIE) {
			dojo.addClass(this.panelHeaderArea.containerNode, "permissionTableWidthLowRes");
			dojo.addClass(this.panelBodyArea.containerNode, "permissionTableWidthLowRes");
		}
		
	},
	
	tunePanelWidgets: function() {
		var view = dijit.getViewport();
		
		if (view.w <= 978) {
			this.tunePanelWidgetsLowRes();
		} else {
			this.tunePanelWidgetsHighRes();
		}		
	},
		
	renderWidget: function () {
		this._trace.entering(this.id, "renderWidget");
		this.resize();
		this._permissionsFieldTable.grid.render();
		
		this._trace.exiting(this.id, "renderWidget");
	},
	
	destroyPrevTableInstances: function() {
		this._trace.entering(this.id, "destroyPrevTableInstances");
	
		var oldPermissionsFieldTable = dijit.byId(this.id+"_permissionsFieldTable");
		if (oldPermissionsFieldTable) {
			oldPermissionsFieldTable.destroyRecursive();
		} 

		//creating a div so that table can be recreated at same div location
		this.permissionTableContainer.set("content", "");
		var newdiv = document.createElement('div');
		newdiv.setAttribute('id',this.id+"_permissionsFieldTable");
		this.permissionTableContainer.domNode.appendChild(newdiv);
		this._trace.exiting(this.id, "destroyPrevTableInstances");		
	},

	reBuild: function(roleItem) {
		this.destroyPrevTableInstances();
		this.populate(roleItem);
	},	
	
	populate: function(roleItem) {

		try {
			this._trace.entering(this.id, "populate");
			this.roleItem = roleItem;
			// create hierarchical tables
			this._createPermissionsTable();
			
			// startup hierarchical tables
			this._permissionsFieldTable.startup();
		} catch (e) {
			this._trace.exception(e, "startup");
			throw e;
		} finally {
			this._trace.exiting(this.id, "populate");
		}
		
	},
	

	_createPermissionsTable: function () {
		this._trace.entering(this.id, "_createPermissionsTable");
		
		var editAddParameters = {
			property: 'permissions', 
			activity: 'add'
		};
		var editRemoveParameters = {
			property: 'permissions', 
			activity: 'remove'
		};
		
		// Instantiate store for permissions table.
		this._permissionsStore = new com.ibm.security.modeling.tables.JsonRestStore({
			target: this.roleItem.permissionsAnalysisUri.toString(),
			idAttribute: 'uri'
		});
		this._permissionsStore.isUpdateable = function(request) {
			return false;
		};
		
		// Setup permission table
		this._permissionsFieldTable = new com.ibm.security.modeling.tables.RolePermissionTable ({
			id: this.id+"_permissionsFieldTable",
			dataStore: this._permissionsStore,
			isAddRemoveRendered: true,
			updateSelectionLogic: true,
			parentContainer: this.panelBodyArea,
			globalResources: this.globalResources,
			messageArea: this.messageArea,
			isRefreshRendered: true,
			namespace: this.namespace,	// for picker dialog help launch
			contextRoot: this.contextRoot,	// for picker dialog help launch
			roleItem: this.roleItem,	// for picker candidate uri
			addPermissionCallback: dojo.hitch(this, this._permissionEditComplete, editAddParameters),
			removePermissionCallback: dojo.hitch(this, this._permissionEditComplete, editRemoveParameters),
			parentWidget: this,	// for UnderLay
			saveIndicator: this.saveIndicator,
			saveIndicatorText: this.saveIndicatorText,
			tableType:"rolePermissionTable",
			searchInstructions:this.localResources.PERMISSION_SEARCH_INSTRUCTION
		}, this.id+"_permissionsFieldTable");
		
		this._trace.exiting(this.id, "_createPermissionsTable");
	},
	
	_refreshHandler: function() {
		this._trace.entering(this.id, "_refreshHandler");

		this.parentWidget._refreshHandler("permissions");
		
		this._trace.exiting(this.id, "_refreshHandler");
	},
	
	_permissionEditComplete: function (editParameters, args) {
		this._trace.entering(this.id, "_permissionEditComplete");
		
		// some success
		var counts = args[1];	// success, failure counts
		if (counts.successCount > 0) {
			this._publishRolePropertyEdit(editParameters);
			this._permissionsFieldTable.refresh();
		}
	
		this._trace.exiting(this.id, "_permissionEditComplete");
	},
	
	_publishRolePropertyEdit: function(editParameters) {
		this._trace.entering(this.id, "_publishRolePropertyEdit");
		this._trace.finest(this.id+" - property: "+editParameters.property+", activity: "+editParameters.activity, "_publishRolePropertyEdit");
		// Edit took place.
		var eventData = {};
		eventData.property = editParameters.property;
		eventData.activity = editParameters.activity;
		dojo.publish("ROLE_PROPERTY_UPDATE_EVENT",[eventData]);
		this._trace.exiting(this.id, "_publishRolePropertyEdit");
	},
	
	getRefreshConfig: function() {
		// Role Property to Activity mapping.
		// Representing that if the specified activity takes place on the specific role property
		//	then which function should be executed on Tab switch to this panel.
		if (!this.refreshConfig) {
			var scopedMethod = dojo.hitch(this, this._refreshTable);
			this.refreshConfig = {
				"parents":{	// on add or remove of parent; refresh permission field table.
					"add": {
						"methods": [scopedMethod]
					},
					"remove": {
						"methods": [scopedMethod]
					}
				}
			};
		}
		
		return this.refreshConfig;
	},
	
	_refreshTable: function() {
		// refreshes permission table.
		this._permissionsFieldTable.refresh();
	}
	
	
	
});
