/* 
Licensed Materials - Property of IBM

Role and Policy Modeler


US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.notebook.RoleNotebookUserMembershipPanel");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.TitlePane");
dojo.require("dijit.Dialog");
dojo.require("com.ibm.security.modeling.layouts.TemplatedPanelWithLayout");
dojo.require("com.ibm.security.modeling.dijit.layout.BorderContainer");
dojo.require("com.ibm.security.modeling.dijit.layout.ContentPane");
dojo.require("com.ibm.security.modeling.dijit.layout.StackContainer");
dojo.require("dijit.form.FilteringSelect");

dojo.require("com.ibm.security.modeling.util.WidgetUtilities");
dojo.require("com.ibm.security.modeling.membership.FilterViewer");

dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.util.MessageUtilities");
dojo.require("com.ibm.security.modeling.util.ValidationUtilities");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");

dojo.require("com.ibm.security.modeling.membership.FilterEvaluationUtil");

// NLS
dojo.requireLocalization("com.ibm.security.modeling.notebook", "RoleNotebookDialogText");


// define the widget class
dojo.declare (
	"com.ibm.security.modeling.notebook.RoleNotebookUserMembershipPanel", 
    [ com.ibm.security.modeling.layouts.TemplatedPanelWithLayout ],
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.notebook.RoleNotebookUserMembershipPanel"),
	validator : new com.ibm.security.modeling.util.Validator(),
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.notebook","templates/RoleNotebookUserMembershipPanel.html"),
	widgetsInTemplate: true,
	// globalResources: Object
	//			Reference to the global shared resource bundle
	globalResources: null,

	// localResources: Object
	//			Reference to the resource bundle unique to this widget
	localResources: null,
	
	// namespace: [readonly] String
	// 			Unique identifier to the instance of portlet page where this item will exist
	namespace: "",
	
	// contextRoot: String
	//			The context root of the server from which Content is being served.
	contextRoot: "",
	
	// roleUri: string
	//		String containing uri of Role being worked upon.
	roleUri: null,

	// messageArea:	ibm.tivoli.tip.dijit.TIPMessageArea
	//			This is to display error/info messages on GUI.
	messageArea: null,
	
	// userRestManager:	com.ibm.security.modeling.api.UserRestManager
	//		An instance of com.ibm.security.modeling.api.UserRestManager to retrieve, delete users attached to roles.
	//		It is an interface for HTTP REST communication using dojo stores api
	userRestManager: null,
	
	// _directlyattachedUserTableStore: com.ibm.security.modeling.tables.JsonRestStore
	//		Reference to Common Table store
	_directlyattachedUserTableStore:null,
	
	// _directlyattachedUserTable: com.ibm.security.modeling.tables.UserTable
	//		Reference to Common Table store
	_directlyattachedUserTable:null,
	
	_allRoleMembersUserTable:null,
	
	_allRoleMembersUserTableStore:null,
	
	// connections: Object[]
    //		summary: Array of dojo method connections
    connections: [], 
    
    // saveIndicator: com.ibm.security.modeling.indicator.SaveIndicator
    //		An instance of com.ibm.security.modeling.indicator.SaveIndicator.
    //		Displays save indicator.
    saveIndicator: null,
    
    // _filterViewer: com.ibm.security.modeling.membership.FilterViewer
    //		A widget to display a filter in read mode.
    
	constructor: function () {
		this._trace.entering("", "constructor");
		this._trace.exiting("", "constructor");
	},
	
	postMixInProperties: function() {
		// Summary:
		//		Widget life-cycle method. 
		//		Provides NLS resources.
		
		this._trace.entering(this.id, "postMixInProperties");
		
		this.inherited(arguments);
		
		if (!this.globalResources) {
			this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		}
		if (!this.localResources) {
			this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.notebook", "RoleNotebookDialogText");
		}
		
		if (!this.widgetUtilities) {
			this.widgetUtilities = new com.ibm.security.modeling.util.WidgetUtilities();
		}
		
		this._trace.exiting(this.id, "postMixInProperties");
	},
	
	postCreate: function () {
		this._trace.entering(this.id, "postCreate");
		this.inherited ("postCreate", arguments);
		
		this.connections.push(dojo.connect(this.userViewType, "onChange", this, "_loadUserTable"));
		
		this.subscribe("ROLE_PROPERTY_UPDATE_EVENT", this._handleRolePropertyEditEvent);
		
		this._trace.exiting(this.id, "postCreate");
	},
	_refreshHandler: function() {
		this._trace.entering(this.id, "_refreshHandler");

		this.parentWidget._refreshHandler("membership");
		
		this._trace.exiting(this.id, "_refreshHandler");
	},	
	refreshAllRolesTable: function (){
		this._trace.entering(this.id, "refreshAllRolesTable");
		if(this._allRoleMembersUserTable){
			this._allRoleMembersUserTable.refresh();
			this._allRoleMembersUserTable.resetAllSelection();				
		}
		this._trace.exiting(this.id, "refreshAllRolesTable");
	},
	refreshDirectlyAssignedTable:function(){
		this._trace.entering(this.id, "refreshDirectlyAssignedTable");
		if(this._directlyattachedUserTable){
			this._directlyattachedUserTable.refresh();
			this._directlyattachedUserTable.resetAllSelection();
		}
		this._trace.exiting(this.id, "refreshDirectlyAssignedTable");
	},
	_refreshFilterPane: function(/*json*/ eventData) {
		// Summary:
		//		Refreshes the filter pane using eventData
		// eventData: json
		//		Json object containing "filterEvaluationRequest",  "filterUri" properties.
		//		- filterEvaluationRequest: is an xhr request. Callbacks can be added to it.
		//		- filterUri: is the URI to retrieve the filter Data
		
		this._trace.entering(this.id, "_refreshFilterPane");
		// Extract data from evenet data.
		
		var filterEvaluationRequest = null;
		var filterUri = null;
		
		if(eventData && eventData.data){
			filterEvaluationRequest = eventData.data["filterEvaluationRequest"];
			filterUri = eventData.data["filterUri"];
		}
		
		// perform activities using the extracted data.
		this._populateFilterPane(filterUri);
		
		/* If the refresh filter pane request was through a fake publish event, then its expected
		 * that we just refresh the filter pane and the filter evaluation is not required as such. */
		if (filterEvaluationRequest) {
			var argsObject = this._getFilterEvaluationUtilArguments(filterUri, filterEvaluationRequest);
			com.ibm.security.modeling.membership.FilterEvaluationUtil.handleFilterEvaluationRequest(argsObject);
		}
		
		this._trace.exiting(this.id, "_refreshFilterPane");
	},
	
	startup: function () {
	//	this._trace.enteringWithStack(this.id, "startup", true);
		try {
			this.inherited ("startup", arguments);
		} catch (e) {
			this._trace.finest(this.id + " - Error while starting user membership panel", "startup");
			this._trace.exception(e, "startup");
		}
		this._attachLabelElementForFilteringSelect();
		this._trace.exiting(this.id, "startup");
	},


	destroy: function () {
		this._trace.enteringWithStack(this.id, "destroy", true);
		try {
			if (this._createFilterButton) {
				this._createFilterButton.destroyRecursive();
			}

			this._destroyFilterViewer();
			
			this.inherited(arguments);
			
			dojo.forEach(this.connections, dojo.disconnect);
			this._trace.exiting(this.id, "destroy");
		} catch (e) {
			this._trace.exception(e, "destroy");
			throw e;
		}
	},

	destroyPrevTableInstances: function() {
		this._trace.entering(this.id, "destroyPrevTableInstances");
		
		var oldMembershipTable = dijit.byId(this.id+"_directlyAttachedUserTable");
		if (oldMembershipTable) {
			oldMembershipTable.destroyRecursive();
		} 

		//creating a div so that table can be recreated at same div location
		this._directlyAttachedUsersPanel.set("content", "");
		var newdiv = document.createElement('div');
		newdiv.setAttribute('id',this.id+"_directlyAttachedUserTable");
		newdiv.setAttribute('class',"userMembershipStackItem");
		this._directlyAttachedUsersPanel.domNode.appendChild(newdiv);
		

		var oldAllMembershipTable = dijit.byId(this.id+"_allRoleMembersUserTable");
		if (oldAllMembershipTable) {
			oldAllMembershipTable.destroyRecursive();
		} 

		//creating a div so that table can be recreated at same div location
		this._allRoleMembersUsersPanel.set("content", "");
		var newdiv = document.createElement('div');
		newdiv.setAttribute('id',this.id+"_allRoleMembersUserTable");
		newdiv.setAttribute('class',"userMembershipStackItem");
		this._allRoleMembersUsersPanel.domNode.appendChild(newdiv);
		
		this._trace.exiting(this.id, "destroyPrevTableInstances");		
	},
	
	reBuild: function(roleItem) {
		this._trace.entering(this.id, "reBuild");
		this.destroyPrevTableInstances();
		this._allRoleMembersUserTable = null;
		this._directlyattachedUserTable = null;
		this.userViewType.set('value','allRoleMembers');
		this.populate(roleItem);
		this.userTableContainer.selectChild(this._allRoleMembersUsersPanel);	
		this._trace.exiting(this.id, "reBuild");
	},
	
	launchFilterBuilder: function(/*json*/ filterData) {
		// Summary:
		//		Launches filter builder panel.
		// filterData: json
		//		Filter Data to be displayed (when editing an exiting filter)
		
		this._trace.entering(this.id, "launchFilterBuilder");
		// To display the filterData for the role, pass it as an argument to the function.
		this.parentWidget.parent.launchFilterBuilderPanel(filterData, this.roleItem, this.parentWidget.getFullTitle());
		this._trace.exiting(this.id, "launchFilterBuilder");
	},
	
	launchDeleteFilter: function(filterUri) {
		// Summary:
		//		Invokes the filter delete functionality. We are actually erasing the filter.
		//		A filter will always be associated with the role, its more of a matter whether
		//		the filter is null/not-null.
		// filterUri: URI
		//		The URI of the filter data to be deleted.
		this._trace.entering(this.id, "launchDeleteFilter");

		try {
			if (!filterUri) {
				filterUri = this._filterData.uri;
				/* If the Uri is still not valid, means an error. */
				if(!filterUri) {
					this._trace.finest(this.id, "filterUri passed for deletion is null.");
					this.messageArea.showErrorMessage(this.pluginResources.VALIDATION_INVALID_FILTER_URI);
					return;
				}
			}			
			this.deleteFilter(filterUri);
		} catch (e) {
			this._trace.exception(e, "launchDeleteFilter");
			throw e;
		} finally {
			this._trace.exiting(this.id, "launchDeleteFilter");
		}
	},
	
	_onDeleteStart: function() {
		// Summary
		//		Performs the initial activities when delete is to be performed.
		//		Activities like show Progress dialog [blocks screen]
		try {
			this._trace.entering(this.id, "_onDeleteStart");
			this.parentWidget._showProgressBar(this.localResources.DELETING_FILTER);
		} finally {
			this._trace.exiting(this.id, "_onDeleteStart");
		}
	},
	
	_publishDeleteComplete: function() {
		this._trace.entering(this.id, "_publishDeleteComplete");

		/* Create eventData for the delete operation just completed. */
		var eventData = {};
		eventData.property = 'filter';
		eventData.activity = 'delete';

		/* Publish the delete event. */
		this._publishRolePropertyEdit(eventData);
		
		this._trace.exiting(this.id, "_publishDeleteComplete");
	},
	
	_onDeleteComplete: function(/*boolean*/isError, /* Response Status */ responseStatus) {
		// Summary
		//		Performs activities when delete operation completes.
		//		Activities like hide Progress dialog and render blank filter.
		//		In case the delete operation had failed, display the relevant error message.
		try {
			this._trace.entering(this.id, "_onDeleteComplete");

			/* In case of no errors, render an empty filter. That'll ensure that the create button
			 * shows up again. */
			if(!isError) {
				this._renderFilter(null, isError);
				this._publishDeleteComplete();
			} else {
				/* Handle the error conditions here. Basically display the messages conveying the error
				 * condition to the end user. */
				
				if (responseStatus == 404) {
					/* Invalid filter URI was passed for deletion. */
					this.messageArea.showErrorMessage(this.pluginResources.VALIDATION_INVALID_FILTER_URI);
				}
				
				/* Am assuming the rest of the error codes will come along with the corresponding messages
				 * from the REST layer itself and the handleRESTError should display those. */				
			}

			// Hide the progress bar once the panel resets to the right status.
			this.parentWidget._hideProgressBar();			
			
		} finally {
			this._trace.exiting(this.id, "_onDeleteComplete");
		}		
	},
	
	_sendHttpRequest: function(/*URI*/ filterUri) {
		// Summary:
		//		Makes an HTTP [DELETE] request for filter represented by the filterUri passed-in.
		//
		// filterUri: URI
		//		The URI of the filter data to be deleted.
		
		var httpMethod = 'delete';
		
		// second boolean parameter 'true' indicates that data is in JSON format.
		var filterRestService = new dojox.rpc.Rest(filterUri, true);

		// Invoke the delete request.
		var request = filterRestService[httpMethod]();
		
		var onResponse = dojo.hitch(this, function(/*boolean*/isError, /*Response Code*/responseStatus) {
			// Summary
			//		Performs activities when HTTP response comes back (success, failure).
			//
			// isError: boolean
			//		true represents that HTTP request should be considered failed.
			//		false represents that HTTP requests completed with success (with success codes)
			this._onDeleteComplete(/*boolean*/isError, /*Response Code*/ responseStatus);
		});
		
		// Success handler
		var successCallBack = function( result ) {
			var isError = false;
			var responseStatus = 204;
			try {
				responseStatus = request.ioArgs.xhr.status;
		
				// IE goofs up with the responseStatus when fetched through xhr.status. 
				// It misinterprets 204 to be 1223. Hence, need to mask it to a common
				// status code.
				if( (dojo.isIE) && (responseStatus == 1223) ) {
					responseStatus = 204;
				}				
				
				if (responseStatus == 204) { // filter deleted successfully
					isError = false;
				} else { // Consider it an error.
					isError = true;
				}
			} catch (e) {
				this._trace.exception(e, "successCallBack");
				throw e;
			} finally {
				onResponse(isError, responseStatus);
			}
			return result;
		};
		
		// Failure handler
		var errorCallBack = function( error, request ) {
			var isError = true;
			var responseStatus = 204;
			try {
				if(request) {
					responseStatus = request.ioArgs.xhr.status;
				}
				this._handleRESTError(error, request);
			} catch (e) {
				this._trace.exception(e, "errorCallBack");
				throw e;
			} finally {
				onResponse(isError, responseStatus);
			}
			return error;
		};
		
		// attach call-backs with the request 
		request.addCallback(dojo.hitch(this, successCallBack));
		request.addErrback(dojo.hitch(this, errorCallBack));
		
		return request;
	},
		
	deleteFilter: function(filterUri) {
		// Summary:
		//		Deletes the filter identified by the filterUri passed-in.
		// filterUri: URI
		//		The URI of the filter data to be deleted.
		this._trace.entering(this.id, "deleteFilter");
		// Block the screen and start the delete operation.
		this._onDeleteStart();
		// Perform the actual delete operation.
		this._sendHttpRequest(filterUri);
		this._trace.exiting(this.id, "deleteFilter");		
	},
	
	populate: function(roleItem) {
		// summary:
		//			Populate user table with selected role.
		// tags:
		//			private

		this._trace.entering(this.id, "populate");
		this.roleItem = roleItem;
		this.roleUri=roleItem.uri;
		
		// Both of the filter, table title panes should be opened
		this.filterPane.set("open", true);
		this.tablePane.set("open", true);
		
		// Populate data in the filter, table panes.
		this._populateFilterPane();
		this._populateTablePane();
		this._trace.exiting(this.id, "populate");
	},
	
	_populateFilterPane: function(/*urin string?*/ filterUri) {
		// Summary:
		//		Populates the filter pane
		//
		// filterUri: uri string
		//		URI to be used for fetching the Filter Data from REST.
		//		If it is not provided then, 
		//			Role Item's "membershipFilterUri" attribute is considered as filter uri
		
		this._trace.entering(this.id, "_populateFilterPane");
		
		/*
		 * Fetch filter data
		 */
		if (!filterUri) {
			filterUri = this.roleItem["membershipFilterUri"];
		}
		// second boolean parameter 'true' indicates that data is in JSON format.
		var filterRestService = new dojox.rpc.Rest(filterUri, true);
		
		// make HTTP GET request
		var request = filterRestService("");
		
		// Success handler
		var successCallBack = function( result ) {
			try {
				var responseStatus = request.ioArgs.xhr.status;
				
				if( (dojo.isIE) && (responseStatus == 1223) ) {
					responseStatus = 204;
				}
				
				var isError = false;
				var filterData = null;
				if (responseStatus == 204 || responseStatus == 200) { 
					// No content if there is no membership filter for the specified model and role.
					filterData = this._extractFilterData(result);
					
				} else { 
					// Could not load filter, and can not determine filter. 
					// Consider it an error.
					isError = true;
				}
				
				// render filter on UI.
				this._renderFilter(filterData, isError);
			} catch (e) {
				this._trace.exception(e, "successCallBack");
				throw e;
			}
			return result;
		};
		
		// Failure handler
		var errorCallBack = function( result ) {
			try {
				// Error notification to user
				this._handleRESTError(arguments);
				
				// render filter on UI showing "! Error loading"
				this._renderFilter(/*data*/ null, /*isError*/ true);
			} catch (e) {
				this._trace.exception(e, "errorCallBack");
				throw e;
			}
			return result;
		};
		
		// attach callbacks with the request 
		request.addCallback(dojo.hitch(this, successCallBack));
		request.addErrback(dojo.hitch(this, errorCallBack));
		
		this._trace.exiting(this.id, "_populateFilterPane");
	},
	
	_normalizeFilterData: function(/*json object*/ result) {
		// Summary:
		//		Ensures that the attribute name is normalized such that
		//		all script and html tags that it contains are escaped.
		//		That way we won't be executing the script tags when
		//		they are entered through say a prestate OR imported 
		//		through a schema import.
		
		//		In case it turns out that there are other attributes 
		//		that can contain a script tags, this is the place to
		//		escape those.
		
		var fragments = result["fragments"];
		if (fragments && dojo.isArray(fragments) && fragments.length > 0) {
			var fragment = fragments[0];
			if (fragment) {				
				var children = fragment["children"];
				
				if (children && dojo.isArray(children) && children.length > 0) {
					for(var i=0; i<children.length; i++){
						var curRules = children[i].rules;
						if (curRules && dojo.isArray(curRules) && curRules.length > 0) {
							for(var j=0; j<curRules.length; j++){
								curRules[j].attributeName = this.validator.escapeHTMLAndScriptTags(curRules[j].attributeName);
							}
						}
					}	
				}								
			}
		}
		return result;
	},
	
	_extractFilterData: function(/*json object*/ result) {
		// Summary:
		//		Extracts data from REST response JSON object.
		
		if (dojo.isArray(result) && result.length > 0) {
			return this._normalizeFilterData(result[0]);
		} else if (result) {
			return this._normalizeFilterData(result);
		}
		
		return null;
	},
	
	_populateTablePane: function() {
		// Summary:
		//		Populates the table pane
		this._trace.entering(this.id, "_populateTablePane");
		var option=this.userViewType.store.fetchSelectedItem();
		if( option !=null && option!=undefined){
			this._showUserTable(option.value);
		}
		this._trace.exiting(this.id, "_populateTablePane");
	},
	
	_fetchGroupTitleNodes: function() {
		// Summary:
		//		Pulls the nodes in the current filter that correspond to the group/section title.
		this._trace.entering(this.id, "_fetchGroupTitleNodes");
		var titleNodes = null;
		query = "#" + this.id + "_filterPanel" + " h2";
		titleNodes = dojo.query(query, this.document);
		this._trace.exiting(this.id, "_fetchGroupTitleNodes");
		return titleNodes;
	},
	
	_extractGroupNames: function(filterData) {
		// Summary:
		//		Fetches the group names in sequence, as they appear in
		//		filterData. This info will later be used to do a 1:1 
		//		replacement of the place-holder group name that's 
		//		returned back by the FilterViewer.
		this._trace.entering(this.id, "_extractGroupNames");
		
		var groupNames = [];
		
		if (filterData) {
			var fragments = filterData["fragments"];
			if (dojo.isArray(fragments) && fragments.length > 0) {
				var fragment = fragments[0];
				if (!fragment) {
					return;
				}
				
				var groups = fragment["children"];
				if (dojo.isArray(groups) && groups.length > 0) {
					for(var i=0; i<groups.length; i++){
						groupNames[i] = groups[i].name;
					}
				}
			}
		}
		
		this._trace.exiting(this.id, "_extractGroupNames");
		return groupNames;
	},	
	
	_updateGroupNames: function(filterData) {
		// Summary:
		// The filter that we render on the screen till this point of time is one, where
		// all the groupnames are shown as GROUP_NAME_TEXT_TO_REPLACE.
		// We need to update this place-holder with the corresponding group names
		// Ideally the FilterGroupViewer should have done this, when it mixes the template of
		// each group in the filter, however, that creates problems when there are script/html
		// tags in the group name. While saving, <b> is actually saved as &lt;b&gt;. 
		// While reading it back, it should be displayed as <b> again, but it is displayed in
		// bold, which means, even the escaping is consumed and the tags are executed.
		// That'll be serious when its a script tag. Given that, the approach to be followed
		// was get the filter created fully and then replace the place-holder with the right
		// group name - d 31297.
		
		this._trace.entering(this.id, "_updateGroupNames");
		var titleNodes = this._fetchGroupTitleNodes();
		var groupNames = this._extractGroupNames(filterData);
		
		var groupName = null;
		for(var i=0; i<titleNodes.length; i++){
			groupName = com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(groupNames[i]);
			titleNodes[i].innerHTML = groupName;
		}

		this._trace.exiting(this.id, "_updateGroupNames");
	},
	
	_renderFilter: function(/*json object*/ filterData, /*boolean*/ isError) {
		// Summary:
		//		Renders filter on UI.
		//
		// filterData: json object
		//		JSON object from REST, representing a filter.
		
		this._trace.entering(this.id, "_renderFilter");
		
		// delete currently displayed filter
		this._deleteFilterBody();
		
		this._filterData = filterData;
		
		if (isError === true) { //Error loading filter
			this._renderFilterOnError();
			
		} else {
			if (!filterData) { // No filter
				// show empty filter.
				this._renderEmptyFilter(/*render*/ true);
				
			} else { // show filter with data
				// If filter status is failed, then re-evaluation message is provided to user.
				if (com.ibm.security.modeling.membership.FilterEvaluationUtil.isFilterEvaluationFailed(filterData)) {
					var filterUri = filterData["uri"];
					var argsObject = this._getFilterEvaluationUtilArguments(filterUri, null);
					com.ibm.security.modeling.membership.FilterEvaluationUtil.renderFilterEvaluationMessage(argsObject);
				}
				// Filter Pane with filter data.
				this._createFilterViewer(filterData);
				this._updateGroupNames(filterData);
			}
		}
		this._trace.exiting(this.id, "_renderFilter");
	},
	
	_renderFilterOnError: function() {
		// Summary:
		//		 Render filter on UI showing "! Error loading"
		// Render filter on UI showing "! Error loading"
		this._trace.entering(this.id, "_renderFilterOnError");
		var errorDiv = dojo.create("div", {
			"innerHTML": this.localResources.USERMEMBERSHIP_TAB_FILTER_SECTION_FILTER_LOADING_ERROR,
			"class": "bodyText"
		});
		dojo.place(errorDiv, this.filter, "only");
		this._trace.exiting(this.id, "_renderFilterOnError");
	},
	
	_deleteFilterBody: function() {
		// Summary:
		//		Removes the elements displayed in filter.
		
		this._trace.entering(this.id, "_deleteFilterBody");
		
		// If a nonempty filter is on display, then delete that.
		this._destroyFilterViewer(); 
		// If empty filter was displayed, then detach it from DOM.
		this._renderEmptyFilter(/*render*/ false);
		this._trace.exiting(this.id, "_deleteFilterBody");
	},
	
	_createFilterViewer: function(/*json object*/ filterData) {
		// Summary:
		//		Creates instance of com.ibm.security.modeling.membership.FilterViewer 
		//		and assigns it to this._filterViewer
		//
		// filterData: json object
		//		JSON object from REST, representing a filter.
		
		if (this._filterViewer) {
			// already created. nothing to do.
			return;
		}
		
		this._filterViewer = new com.ibm.security.modeling.membership.FilterViewer({
			id: this.id+"_filterViewer",
			globalResources: this.globalResources,
			filterData: filterData
		});	// create and attach to DOM in the document.
		
		dojo.place(this._filterViewer.domNode, this.filter, "only");
		
		this._filterViewer.startup();
		
		this._setupFilterButtons();
	},
	
	_setupFilterButtons: function() {
		// Summary:
		//		Creates and places the Edit and Delete Filter buttons.
		
		// Edit Filter instructions: Information DIV
		var filterButtonsDiv = dojo.create("div", {
			"innerHTML": this.localResources.USERMEMBERSHIP_TAB_FILTER_SECTION_FILTER_TEXT,
			"class": "description bodyText"
		});
		dojo.place(filterButtonsDiv, this.filter, "first");
		
		var filterButtonContainer = dojo.create("div", {
			"class": "filterButton"
		});
		dojo.place(filterButtonContainer, filterButtonsDiv, "after");
		
		// Edit filter button.
		this._editFilterButton = this._createLaunchFilterEditorButton(
			/*id*/ this.id+"_editFilter", 
			/*label*/ this.localResources.USERMEMBERSHIP_TAB_FILTER_EDIT_BUTTON,
			/*onClickHandler*/ dojo.hitch(this, function() { this.launchFilterBuilder(this._filterData); })
		);
		dojo.addClass(this._editFilterButton.domNode,"leftPadding");
		dojo.place(this._editFilterButton.domNode, filterButtonContainer, "first");
		
		// Delete filter button.
		this._deleteFilterButton = this._createLaunchFilterEditorButton(
			/*id*/ this.id+"_deleteFilter",
			/*label*/ this.localResources.USERMEMBERSHIP_TAB_FILTER_DELETE_BUTTON,
			/*onClickHandler*/ dojo.hitch(this, function() { this.launchDeleteFilter(); })
		);
		
		dojo.addClass(this._deleteFilterButton.domNode,"leftPadding");
		dojo.place(this._deleteFilterButton.domNode, filterButtonContainer, "last");
	},
	
	_createEmptyFilter: function() {
		// Summary:
		//		Creates an empty filter DOM Node, NodeList reference.
		
		if (this._emptyFilterDiv) {
			// DIV already created.
			return;
		}
		
		this._emptyFilterDiv = dojo.create("div");
		// Create NodeList for easy DOM manipulation.
		this._emptyFilterNode = new dojo.NodeList(this._emptyFilterDiv);	
		
		// Empty Filter content: Information DIV
		var emptyTextDiv = dojo.create("div", {
			"innerHTML": this.localResources.USERMEMBERSHIP_TAB_FILTER_SECTION_NOFILTER_TEXT,
			"class": "description bodyText"
		});
		// Empty Filter content: 'Create...' Button
		this._createFilterButton = this._createLaunchFilterEditorButton(
				/*id*/ this.id+"_createFilter", 
				/*label*/ this.localResources.USERMEMBERSHIP_TAB_FILTER_CREATE_BUTTON,
				/*onClickHandler*/ dojo.hitch(this, function() { this.launchFilterBuilder(); })
			);
		
		// Place in DOM (not attached to window document yet).
		this._emptyFilterNode.addContent(emptyTextDiv);
		this._emptyFilterNode.addContent(this._createFilterButton.domNode);

		dojo.addClass(this._createFilterButton.domNode,"topPadding");
		
	},
	
	_createLaunchFilterEditorButton: function(/*string*/ id, /*string*/ label, /*function*/ onClickHandler) {
		// Summary:
		//		Creates a button which will launch Filter Editor when clicked.
		//
		// label: string
		//		Button label.
		// onClickHandler: function
		//		Function to invoke when button is clicked.
		
		return this.widgetUtilities.createButton(
				label,	// label
				true,	// show label
				null,	// no icon class
				onClickHandler,	// onClick
				id,	// button id
				true);	// enabled
	},
	
	_renderEmptyFilter: function(/*boolean*/ render) {
		// Summary:
		//		Renders an empty filter on UI.
		// 		Multiple invocations have no side effect.
		// render: boolean
		//		Flag representing attach/detach the empty filter DIV on UI.
		//		true - attach
		//		false - detach
		this._trace.entering(this.id, "_renderEmptyFilter");
		if (!this._emptyFilterNode) {
			// need to create empty filter div
			
			// But check, if request is to detach empty div
			// then no need to create empty filter DIV.
			if (render == false) {
				// nothing to detach.
				return;
			}
			// create empty filter contents DOM structure.
			this._createEmptyFilter();
		} 
		
		// Attach/Detach empty filter contents DOM structure using NodeList reference.
		if (render == true) { // attach
			this._emptyFilterNode.place(this.filter, "only"); 
		} else { // detach
			this._emptyFilterNode.orphan();
		}
		this._trace.exiting(this.id, "_renderEmptyFilter");
	},
	
	_destroyFilterViewer: function() {
		// Summary
		//		Destroys filter viewer widget instance.
		//		Multiple invocations have no side effect.
		this._trace.entering(this.id, "_destroyFilterViewer");
		if (this._editFilterButton) {
			this._editFilterButton.destroyRecursive();
		}
		
		if (this._deleteFilterButton) {
			this._deleteFilterButton.destroyRecursive();
		}
		
		if (this._filterViewer) {
			this._filterViewer.destroyRecursive();
			this._filterViewer = null;
			delete this._filterViewer;
		}
		this._trace.exiting(this.id, "_destroyFilterViewer");
	},
	
	_loadUserTable:function(){
		// summary:
		//			Gets the user type dropdown selection.
		// tags:
		//			private
		this._trace.entering(this.id, "postCreate");
		this.messageArea.hideMessages(); // Clear all messages when changing tables
		if(this.userViewType.item && this.userViewType.item!=null){
			var userTableType=this.userViewType.item.value;
			this._showUserTable(userTableType);
		}
		
	},
	
	_showUserTable:function(/*string*/userTableType){
		var additionalQuery="";
		var tableType="";
		if(userTableType=="directlyAttachedUsers"){
			if(this._directlyattachedUserTable ==null){
				additionalQuery={includeMembers:"direct"};
				 tableType="directlyAttachedUserTable";
				this._directlyattachedUserTableStore=this._setupRestStore(this.roleItem.membershipUri);
				this._directlyattachedUserTable=this._createMembershipTable(userTableType,this.id+"_directlyAttachedUserTable",this._directlyattachedUserTableStore,additionalQuery,tableType);
				this._directlyattachedUserTable.startup();
				this.userTableContainer.selectChild(this._directlyAttachedUsersPanel);
			}else{					
				this.userTableContainer.selectChild(this._directlyAttachedUsersPanel);
			}
		}else if(userTableType=="allRoleMembers"){
		    
		    if(this._allRoleMembersUserTable ==null){
				additionalQuery={includeMembers:"all"};
				tableType="allRoleMembersUserTable";
				this._allRoleMembersUserTableStore=this._setupRestStore(this.roleItem.membershipAnalysisUri);
				this._allRoleMembersUserTable=this._createMembershipTable(userTableType,this.id+"_allRoleMembersUserTable",this._allRoleMembersUserTableStore,additionalQuery,tableType);
				this._allRoleMembersUserTable.startup();
			}else{			        
				this.userTableContainer.selectChild(this._allRoleMembersUsersPanel);
			} 
		}
	},
	_createMembershipTable:function(/*string*/userTableType,
							/*string*/placeHolderId,
								/*Table store*/userTableStore,/*object*/additionalQuery,tableType){
		// summary:
		//			Creates the usermembership table depending on type of selection.
		// tags:
		//			private
		dojo.require("com.ibm.security.modeling.tables.RoleMembershipTable");
		var userTable = new com.ibm.security.modeling.tables.RoleMembershipTable({dataStore: userTableStore, 
				id:placeHolderId,
				contextRoot:this.contextRoot,
				userTableType:userTableType,
				saveIndicator:this.saveIndicator,
				saveIndicatorText: this.saveIndicatorText,
				//query:additionalQuery,
				roleItem:this.roleItem,
				parentWidget: this,
				panel:this,
				messageArea: this.messageArea,
				tableType:tableType
			},placeHolderId);

		return userTable;		
	},
	_setupRestStore: function (/*string*/resourceUri,additionalQuery) {
		this._trace.entering("", "_setupRestStore");
		this.userRestManager = new com.ibm.security.modeling.api.UserRestManager({
					resourceUri: resourceUri,
					syncMode: false,
					messageArea: this.messageArea,
					allowNoTrailingSlash: false,
					idAttribute: 'identifier'
		});
		
		var userTableStore = this.userRestManager.getIsrmTableRestStore();
		userTableStore.isUpdateable=function(request){
			return false;
		};
		this._trace.exiting("", "_setupRestStore");
		return userTableStore;		
	},
	_publishRolePropertyEdit: function(editParameters) {
		this._trace.entering(this.id, "_publishRolePropertyEdit");
		this._trace.finest(this.id+" - property: "+editParameters.property+", activity: "+editParameters.activity, "_publishRolePropertyEdit");
		// Edit took place.
		var eventData = {};
		eventData.property = editParameters.property;
		eventData.activity = editParameters.activity;
		dojo.publish("ROLE_PROPERTY_UPDATE_EVENT",[eventData]);
		this._trace.exiting(this.id, "_publishRolePropertyEdit");
	},	
	_handleRolePropertyEditEvent: function () {
		this._trace.entering(this.id, "_setRoleEdited");
		this.isRoleEdited = true;
		
		var event = arguments[0];
		// To log and event; event should have the property, activity in it.
		// for example: property: name, activity: update; represents that Role name has been updated.
		// We need to handle the changes to the membership filter as well, since that'll dictate when
		// we would need to refresh the membership tables.
		if (event && event.property && event.activity) {		
			if ( event.property=="membership" ) {
				this.refreshAllRolesTable();
				this.refreshDirectlyAssignedTable();
			}

			if ( (event.property=="filter") && (event.data) ) {
				if(this.parentWidget._panelDirtyState && this.parentWidget._panelDirtyState["membership"]) {
					/* Need to automatically update the filter. */
					this._refreshFilterPane(event);
				}		
			}			
		}

		this._trace.exiting(this.id, "_setRoleEdited");
	},
	
	showMessages: function(value) {
		var mu = new com.ibm.security.modeling.util.MessageUtilities();
		var msgs = mu.extractBatchMessages(value, null, true);
		mu.showMessages(msgs, this.messageArea, true);
	},
	
	_handleRESTError: function (error, request) {
		this._trace.entering(this.id, "_handleRESTError");
		this._trace.finest(this.id + " - Error: " + error, "_handleRESTError");
//		if ((this.messageArea != null) && (error != null) && (error.responseText != null)) {
//			try {
//				var message = JSON.parse(error.responseText, null);
//				if ((message != null) &&
//					(message.text != null)) {
//					this.messageArea.showErrorMessage(message.text);
//				}
//			} catch (e){
//				var messageResponseSplits = error.responseText.split("text",-1);
//				if (messageResponseSplits.length > 1) {
//					messageResponseSplits[1] = messageResponseSplits[1].replace(/^\":\"/g,"");
//					message = messageResponseSplits[1].replace(/\"}$/g,"");
//				} else {
//					message = messageResponseSplits[0];
//				}
//				if (message != null) {
//					if ( (error.status == 404) && (message.search("SRVE0255E") != -1) ) {
//						if (!this.pluginResources) {
//							// create plugin i18n resource.
//							dojo.requireLocalization("com.ibm.security.modeling", "PluginMessages");
//							this.pluginResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "PluginMessages");
//						}						
//						var serverDownMessage = this.pluginResources.VALIDATION_ROLE_REST_SERVER_NOT_REACHABLE ;
//						this.messageArea.showErrorMessage(serverDownMessage);
//					} else {
//						this.messageArea.showErrorMessage(message);
//					}
//				}				
//			}
//		}
		dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
		this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
		this._errorUtil.handleRESTError(error, this.messageArea);
		this._trace.exiting(this.id, "_handleRESTError");
	},
	
	getRefreshConfig: function() {
		// Role Property to Activity mapping.
		// Representing that if the specified activity takes place on the specific role property
		//	then which function should be executed on Tab switch to this panel.
		if (!this.refreshConfig) {
			var refreshAllRoles = dojo.hitch(this, this.refreshAllRolesTable);
			var refreshDirectlyAssigned = dojo.hitch(this, this.refreshDirectlyAssignedTable);
			
			var refreshFilterPane = dojo.hitch(this, this._refreshFilterPane);
			
			this.refreshConfig = {
				"name":{	// on update of role name; 
					"update": {
						"methods": [refreshAllRoles,refreshDirectlyAssigned] 
					}
				},

				"children":{	// on add or remove of children
					"add": {
						"methods": [refreshAllRoles,refreshDirectlyAssigned]
					},
					"remove": {
						"methods": [refreshAllRoles,refreshDirectlyAssigned]
					}
				},
				"parents": {	// on add or remove of parent; 
					"add": {
						"methods": [refreshAllRoles,refreshDirectlyAssigned]
					},
					"remove": {
						"methods": [refreshAllRoles,refreshDirectlyAssigned]
					}
				},
				"filter": {
					"create": {
						"methods": [refreshFilterPane, refreshAllRoles, refreshDirectlyAssigned]
					},
					"update": {
						"methods": [refreshFilterPane, refreshAllRoles, refreshDirectlyAssigned]
					},
					"delete": {
						"methods": [refreshFilterPane, refreshAllRoles, refreshDirectlyAssigned]
					}
				}
			};
		}		
		return this.refreshConfig;
	},
	
	_getFilterEvaluationUtilArguments: function(filterUri, filterEvaluationRequest) {
		// filterUri: uri string
		//		URI of the filter.
		//		It should be used to retrieve the filter.
		// filterEvaluationRequest: xhr request
		//		xhr request obtained by invoking filter evaluation HTTP URI.
		
		var filterEvalSaveIndicatorText = this.globalResources.SAVE_MEMBERSHIP_FILTER_CHANGE;
		
		return {
			filterUri: filterUri,
			filterEvaluationRequest: filterEvaluationRequest,
			caller: this,
			saveIndicator: this.saveIndicator,
			saveIndicatorText: filterEvalSaveIndicatorText,
			onFilterEvaluationStart: dojo.hitch(this, function() {this.parentWidget.onFilterEvaluationStart();}),
			onFilterEvaluationComplete: dojo.hitch(this, function() {this.parentWidget.onFilterEvaluationComplete();}),
			handleRESTError: dojo.hitch(this, function(error, request) {this._handleRESTError(error, request);}),
			messageArea: this.messageArea
		};
	},
	
	_attachLabelElementForFilteringSelect: function() {
		// Summary:
		//		Sets the label element's 'for' attribute for filtering select.
		if (this.userViewType && this.dropDownLabelNode) {
			var filteringSelectId = this.userViewType.id;
			var labelFor = this.dropDownLabelNode.getAttribute("for");
			if (!labelFor) {  
				this.dropDownLabelNode.setAttribute("for", filteringSelectId);
			}
			var labelNodeId = (filteringSelectId+"_label");
			this.dropDownLabelNode.id = labelNodeId;
			dijit.setWaiState(this.userViewType.domNode, "labelledby", labelNodeId);
		}
	}
	
});
