/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

� Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.notebook.RoleOwnerTable");

dojo.require("com.ibm.security.modeling.tables.UserTable");
dojo.requireLocalization("com.ibm.security.modeling.tables", "UserTableText");
dojo.requireLocalization("com.ibm.security.modeling.notebook", "OwnersTableDialogText");
dojo.require("com.ibm.security.modeling.util.MessageUtilities");
dojo.requireLocalization("com.ibm.security.modeling", "LogMessages");



dojo.declare("com.ibm.security.modeling.notebook.RoleOwnerTable", com.ibm.security.modeling.tables.UserTable, {
	_roleId: null,
	// saveIndicator: com.ibm.security.modeling.indicator.SaveIndicator
	//		An instance of com.ibm.security.modeling.indicator.SaveIndicator.
    	//		Displays save indicator.
	saveIndicator:null,
	roleItem:null,
	maximumNumberOfOwners:100,
	selectedRows:[],
	// logResources: Object
	//			Reference to the log shared resource bundle
	logResources: null,
	// _messageUtils: com.ibm.security.modeling.util.MessageUtilities
	//			Reference to the log shared resource bundle
	_messageUtils: new com.ibm.security.modeling.util.MessageUtilities(),
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.notebook.RoleOwnerTable"),
	postMixInProperties:function() {
			this.inherited(arguments);
			this.isClientSide=true;
	    	this.isFilterRendered=false;
	    	this.isRefreshRendered= false;
	    	this.isSelectAllRendered = false;
	    	this.defaultSortIndex=1;
			this.logResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "LogMessages");
			this.ownerPickerResources = dojo.i18n.getLocalization("com.ibm.security.modeling.notebook", "OwnersTableDialogText");
	},
	postCreate:function() {
		this.inherited(arguments);
		this._setupToolbar();
		this._setupOwnerCandidateStore();
		this.tableType="roleOwnerTable";
	},
	_setupToolbar:function() {
		this.addToolbarButton(this.userTableResources.ADD_OWNER, false, "addIcon", dojo.hitch(this,this._openAddOwnersDialog), this.id+"_addOwners", true, true, true);
		this.addToolbarButton(this.userTableResources.REMOVE_OWNER, false, "removeIcon", dojo.hitch(this,this._removeOwners), this.id+"_removeOwners", true, true, false);
	},
	_setupOwnerCandidateStore:function(){
		this._trace.entering("", "_setupOwnerCandidateStore");
		this.userRestManager = new com.ibm.security.modeling.api.UserRestManager({
				syncMode: false,
				messageArea: this.messageArea
			});
		this.candidateStore = this.userRestManager.getIsrmTableRestStore();
		this._trace.exiting("", "_setupOwnerCandidateStore");
	},
	_showOwnerPicker:function(id,peopleStore){
		dojo.require("com.ibm.security.modeling.widgets.PeoplePickerDialog");
		var pickerDialog = new com.ibm.security.modeling.widgets.PeoplePickerDialog({
					id:id, 
					namespace:this.namespace,
					fromDataStore: peopleStore,	// for picker table.
					//toDataStore: toStore,
					parentWidget: this,
					parentMessageArea: this.messageArea,
					callback:dojo.hitch(this, this._onOK),
					contextRoot: this.contextRoot,
					globalResources: this.globalResources,	// for picker table.
					localResources:this.ownerPickerResources,
					saveIndicator:this.saveIndicator,
					helpFile: "ui_owners_add.html"
					
				});
		pickerDialog.startup();
		pickerDialog.show();
		
	},
	_openAddOwnersDialog:function(){
		this._setupOwnerCandidateStore();
		this._showOwnerPicker(this.id+"_candidateOwerPicker",this.candidateStore);
		
	},
	
	_removeOwners: function () {
		this._trace.entering(this.id, "_removeOwners");
		var selectedRows = this.getSelectedItems();
		if (selectedRows == null || selectedRows.length <=0) {
			this._trace.exiting(this.id, "_removeOwners");
			return;
		}
		// Show Save indicator
		this.parentWidget.parentWidget.showSaveInProgress();
		
		//this.deselectAll();
		// Update store
		for (var j=0; j<selectedRows.length; j++) {
			this.dataStore.deleteItem(selectedRows[j]);
		}
		
		var eventData = {
			property: 'owners', 
			activity: 'remove'
		};
		
		// When done.
		var processedItems = 0;
		var selectedItemsCount = selectedRows.length;
		var self = this;
		var pw = this.parentWidget;
		var store = this.dataStore;
		var handleComplete = function(actions, value) {
			
			processedItems = processedItems + actions.length;
			if (processedItems == selectedItemsCount) {
				pw.showMessages(value);
				pw._publishRolePropertyEdit(eventData);
				pw.parentWidget.hideSaveInProgress();	// hide save indicator
				self.refresh();
			}
		};
		
		
		// commit changes
		this.dataStore.save({
			handleAsBatch: true,
			
			onComplete: dojo.hitch(this, function(args) {
				handleComplete(args[0], args[1]);
			}),
			
			onError: dojo.hitch(this, function(args) {
				this._trace.error("Error while deleting owners.", "error callback");
				if (typeof args == Array) {
					handleComplete(args[0], args[1]);
				} else {
					this._handleRESTError(args);
					this.parentWidget.parentWidget.hideSaveInProgress();	// hide save indicator
				}
			})
		});
		this._trace.exiting(this.id, "_removeOwners");
	},
	_onOK: function (selectedRows) {
		this._trace.entering(this.id, "_onOK");
		this.selectedRows=selectedRows;
		if (this.selectedRows != null && this.selectedRows.length > 0) {
			// Some users have been selected.
			this._trace.info(this.id+" - User selected.", "_onOK");
			this.fetchRoleOwnerItems(
				dojo.hitch(this, this._validateAndAddOwners),	// onComplete callback
				dojo.hitch(this, this._handleRESTError)
			);		// onError callback
			
		}

		this._trace.exiting(this.id, "_onOK");
	},
	fetchRoleOwnerItems: function (/*function*/ onComplete, /*function*/ onError) {
		this._trace.entering(this.id, "fetchRoleOwnerItems");
		if (onError == null) {
			onError = dojo.hitch(this, this._handleRESTError); 
		}
		
		this.dataStore.fetch({
			onComplete: onComplete, 
			onError: onError 
		});
		this._trace.exiting(this.id, "fetchRoleOwnerItems");
	},
	_validateAndAddOwners: function (/*array*/roleOwners) {
		this._trace.entering(this.id, "_validateAndAddOwners");
		try {
			// Validate.
			var isValid = this._validate(roleOwners);
			if (isValid == false) {
				// validation failure. control should remain on dialog.
				var formattedMessage = this._messageUtils.formatMessage(this.logResources.ROLE_TOO_MANY_OWNERS, ["100"]);
				this._handleError(formattedMessage);
				return;
			}
			
			// Add Owner
			if (this.newOwnersToAdd.length <= 0) {
				// Success but, no new additions. Dialog should close. No need to refresh role owner table.
				this._trace.info(this.id+" - No owner to add.", "_validateAndAddOwners");
				return;
			}
			this._addOwners();
		} catch (e) {
			this._trace.exception(e, "_validateAndAddOwners");
			throw e;
		}
		this._trace.exiting(this.id, "_validateAndAddOwners");
	},
	
	_validate: function(/*array*/roleOwners) {
		this._trace.entering(this.id, "_validate");
		var isValid = true;
		if (roleOwners == null) {
			roleOwners == new Array();
		}
		var numberOfOwners = roleOwners.length;	// Current Role Owner count.
		this.newOwnersToAdd = [];
		var isNewOwner;
		this._trace.object(this.selectedRows, "_validate", "selected rows");
		this._trace.object(roleOwners, "_validate", "current owners");
		for (var i=0; (i<this.selectedRows.length) && (numberOfOwners <= this.maximumNumberOfOwners); i++) {
			isNewOwner = true;	// reset.
			for (var j=0; j<roleOwners.length; j++) {
				if (this.selectedRows[i].uri.toString() === roleOwners[j].owner.toString()) {	//comparing uri of user object with owner field of owners list as it contains that user's uri
					//this._trace.finest(this.id+" - Already an Owner: "+this.selectedRows[i].uiColumn1.toString(), "_validate");
					isNewOwner = false;	// update
					break;
				}
			}
			if (isNewOwner) {	// check
				numberOfOwners++;
				this.newOwnersToAdd[this.newOwnersToAdd.length] = this.selectedRows[i];
			}
		}
		this._trace.finest(this.id+" - Total number of Owners would be: "+numberOfOwners+
				" out of which new additions are: "+this.newOwnersToAdd.length,
				"_validate");
		
		if (numberOfOwners > this.maximumNumberOfOwners) {
			this._trace.info(this.id+" - validation failed for owners upper limit.", "_validate");
			isValid = false;
		}
		this._trace.exiting(this.id, "_validate");
		return isValid;
	},
	_addOwners: function() {
		this._trace.entering(this.id, "_addOwners");
		this.parentWidget.parentWidget.showSaveInProgress();
		for (var i=0; i<this.newOwnersToAdd.length; i++) {
			this.dataStore.newItem(this.newOwnersToAdd[i]);
		}
		
		var eventData = {
			property: 'owners', 
			activity: 'add'
		};
			
		
		
		var processedItems = 0;
		var selectedItemsCount = this.newOwnersToAdd.length;
		var pw = this.parentWidget;
		var self = this;
		var store = this.dataStore;
		var handleComplete = function(actions, value) {
			
			processedItems = processedItems + actions.length;
			if (processedItems === selectedItemsCount) {
				pw.showMessages(value);
				pw._publishRolePropertyEdit(eventData);
				pw.parentWidget.hideSaveInProgress();	// hide save indicator				
				self.refresh();				
			}
		};
		
		// commit changes
		this.dataStore.save({
			handleAsBatch: true,
			alwaysPostNewItems : true,	// makes POST request even if the item has 'uri' (idAttribute) in it. Else PUT request is made.
			
			onComplete: dojo.hitch(this, function(args) {
				handleComplete(args[0], args[1]);
			}),
			
			onError: dojo.hitch(this, function(args) {
				this._trace.error("Error while adding owners.", "error callback");
				if (args instanceof Array) {
					handleComplete(args[0], args[1]);
				} else {
					this._handleRESTError(args);
					this.parentWidget.parentWidget.hideSaveInProgress();	// hide save indicator
				}
			})
		});
	
		
		this._trace.exiting(this.id, "_addOwners");
		
	},
	_handleRESTError: function (error, request) {
		this._trace.entering(this.id, "_handleRESTError");
		
		this._trace.exception(error, "_handleRESTError");
		dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
		this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
		this._errorUtil.handleRESTError(error, this.messageArea);
		this._trace.exiting(this.id, "_handleRESTError");
	},
	_handleError: function (formattedMessage) {
		this._trace.entering(this.id, "_handleError");
		if ((this.messageArea != null) &&
			(formattedMessage != null)) {
			this.messageArea.showErrorMessage(formattedMessage);
		}
		this._trace.exiting(this.id, "_handleError");
	},
	destroy:function(){
		this.inherited(arguments);
	}	
});