/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.notebook.SeparationOfDutyDialog");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Dialog");
dojo.require("dijit.Toolbar");
dojo.require("dijit.form.Button");
dojo.require("com.ibm.security.modeling.dijit.layout.BorderContainer");
dojo.require("dijit._Container");
dojo.require("ibm.tivoli.tip.dijit.TIPButton");
dojo.require("dojo.data.ItemFileWriteStore");
dojo.require("com.ibm.security.modeling.tables._DisabledSelection");
dojo.require("com.ibm.security.modeling.widgets.IsrmDialogBase");
dojo.require("com.ibm.security.modeling.tables.Table");
dojo.require("com.ibm.security.modeling.widgets.RequiredTextField");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.requireLocalization("com.ibm.security.modeling.notebook", "SeparationOfDutyDialogText");

dojo.declare("com.ibm.security.modeling.notebook.SeparationOfDutyDialog", com.ibm.security.modeling.widgets.IsrmDialogBase, {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.notebook.SeparationOfDutyDialog"),
	templatePath : dojo.moduleUrl("com.ibm.security.modeling.notebook", "templates/SeparationOfDutyDialog.html"),
	cardinality : 0,
	totalRoles : 0,
	changing : false,
	constructor : function() {
		this._trace.entering("", "constructor");
		this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.notebook", "SeparationOfDutyDialogText");
		this.itemAdded = 0;
		this.connections=[];
		this._trace.exiting("", "constructor");
	},
	postMixInProperties:function(){
		this._trace.entering("", "postMixInProperties");
		this.inherited(arguments);
		
		if (this.changing) {
			this.dialogTitle= this.localResources.EDIT_TITLE;
			this.dialogInstruction=this.localResources.SOD_DIALOG_INSTRUCTION_EDIT;
			this.helpFile="ui_sod_edit.html";
			
			
		}else{
			this.dialogInstruction=this.localResources.SOD_DIALOG_INSTRUCTION;
			this.dialogTitle=this.localResources.SOD_DIALOG_TITLE;
			this.helpFile="ui_sod_create.html";
		}
		this._trace.exiting("", "postMixInProperties");
	},
	postCreate : function() {
		this._trace.entering("", "postCreate");
		dojo.addClass(this.domNode, "fullSize");
		this.inherited("postCreate", arguments);
		this._createSODTable();
		this._createCardinalityBox();
		this.addCommonButton({
			onClick:dojo.hitch(this, this.onCancel),
			type:"CANCEL"
		});
		this.okButton = this.addCommonButton({
			onClick:dojo.hitch(this, this.onOK),
			type:"OK"
		});
		//this.connections.push(dojo.connect(this._userTable.grid, "postrender", dojo.hitch(this, this.makeThisDisabled)));
		this._trace.exiting("", "postCreate");
	},
	startup : function() {
		this._trace.entering("", "startup");
		this.inherited(arguments);
		// TODO: Need to take care of this width in css.
		this.Sod_Description._inputNode.style.width="95%";
        this._trace.exiting("", "startup");
	},
	//seeing no use of this method ,will remove in few days 
	displayError : function(error) {
		this._trace.entering("", "displayError");
		this.messageArea.showErrorMessage(error);
		this._trace.exiting("", "displayError");
	},
	destroy : function() {
        this._trace.entering("", "destroy");
		this.TableDialog.destroyRecursive();
		for(var i=0;i<this.connections.length;i++) dojo.disconnect(this.connections[i]);
		this.inherited(arguments);
		this._trace.exiting("", "destroy");
	},
	_formatUserCount : function(rowIndex, item) {
		var countString = new String("");
		if (item) {
			if (item.inheritedIdentitiesCount == 0) {
				countString = (new String(item.totalIdentitiesCount));
			} else {
				countString = this.localResources.USER_COUNT_TABLE.replace("${0}", (new String(item.totalIdentitiesCount)));
				countString = countString.replace("${1}", (new String(item.inheritedIdentitiesCount)));
			}
		}
		return countString;

	},
	_formatPermissionCount : function(rowIndex, item) {
		var countString = new String("");
		if (item) {
			if (item.inheritedPermissionsCount == 0) {
				countString = (new String(item.totalPermissionsCount));
			} else {
				countString = this.localResources.PERMISSION_COUNT_TABLE.replace("${0}", (new String(item.totalPermissionsCount)));
				countString = countString.replace("${1}", (new String(item.inheritedPermissionsCount)));
			}
		}
		return countString;

	},
	
	_getChildObjectNameCommaSepList : function(rowIndex,item) {

var nameListString = "";
	if(item){
		var list=item.children;
		var nameList = new Array();
		
		if (list != null && list.length && list.length > 0) {
			for ( var i = 0, len = list.length; i < len; i++) {
				nameList.push(list[i].name);
			}
			nameList.sort();// adding sort due to problem in
			// automation testing D11385
			for ( var j = 0, len1 = nameList.length; j < len1; j++) {
				nameListString = nameListString + nameList[j];
				if (j + 1 != len1) {
					nameListString = nameListString + ", ";
				}
			}
		}
		}
		return nameListString;
		

	},
	_getParentObjectNameCommaSepList : function(rowIndex,item) {

var nameListString = "";
	if(item){
	var list=item.parents;

	
		var nameList = new Array();
		
		if (list != null && list.length && list.length > 0) {
			for ( var i = 0, len = list.length; i < len; i++) {
				nameList.push(list[i].name);
			}
			nameList.sort();// adding sort due to problem in
			// automation testing D11385
			for ( var j = 0, len1 = nameList.length; j < len1; j++) {
				nameListString = nameListString + nameList[j];
				if (j + 1 != len1) {
					nameListString = nameListString + ", ";
				}
			}
		}
		}
		return nameListString;

	},

       
	makeThisDisabled : function() {
		
		
		 var id,item,rowIndex;
		 id = this.roleItem.id; 
		 item= this.store._getItemByIdentity(id); 
		 rowIndex=this._userTable.grid.getItemIndex(item);
		 if(this._disabledrowIndex ||this._disabledrowIndex==0){
			 this._userTable.grid.rowSelectCell.setDisabled(this._disabledrowIndex, false);
		 }
		 this._disabledrowIndex = rowIndex;
		 this._userTable.grid.rowSelectCell.setDisabled(rowIndex, true);
		 this._userTable.grid.rowSelectCell.disabledIdMap={};//putting it in disabledMap to avoid getting in selection count
		 this._userTable.grid.rowSelectCell.disabledIdMap[id]=true;
		 
		  
		 
	},

	_setupStore : function() {
		this.store = new dojo.data.ItemFileWriteStore( {
			data : {
				items : [], 
				identifier : "id"
			},
			hierarchical : false		
		});
		 
		this.connect(this.store, "onNew",  function() {
		
			this.totalRoles++;
			this.cardinality = this.totalRoles - 1 || 1;
			try {
				this.dmStore.newItem( {
					name : this.cardinality
				});
			} catch (e) {
				console.log("Element repeated or error in cardinality box");
			}

			;
		});

		this.connect(this.store, "onDelete", function() {

			this.totalRoles--;
			if (this.cardinality >1) {
				try {
					this.dmStore.deleteItem(this.dmStore._getItemByIdentity(this.cardinality));
					this.dmStore.save();
					this.cardinality--;
					var temp_cardinality = this.comboBox.attr('value');
					if (temp_cardinality > this.cardinality)
						this.comboBox.attr('value', this.cardinality);
				} catch (e) {
					console.log(e);
					console.log("Delete: Element repeated or error in cardinality box");
				}
			}

		});

	},
	_createSODTable : function() {

		this._setupStore();

		var roleStructure = [
		                		{field: 'name',name: this.globalResources.ROLE,width: '20%'},
		                		{field: 'description',name: this.globalResources.DESCRIPTION,width: '25%'},
		                		{field: 'totalIdentitiesCount',name: this.globalResources.USERS,width: '11%',get: dojo.hitch(this,this._formatUserCount), isSearchable:false},
		                		{field: 'totalPermissionsCount',name: this.globalResources.PERMISSIONS,width: '15%',get: dojo.hitch(this,this._formatPermissionCount), isSearchable:false},
		                		{field: 'parents',name: this.globalResources.PARENT_ROLES,width: '12%',get : this._getParentObjectNameCommaSepList, isSearchable:false},
		                		{field: 'children',name: this.globalResources.CHILD_ROLES,width: '12%',get : this._getChildObjectNameCommaSepList, isSearchable:false}
		                	];

		this._userTable = new com.ibm.security.modeling.notebook.SodTable( {
			id : this.id + "_SoDTableContainer1",
			dataStore : this.store,
			structure : roleStructure,
			isClientSide : true,
			isFilterRendered : false,
			isRefreshRendered : false,
			isMultiSelect : true,
			autoHeight : false,
			messageArea : this.messageArea,
			isDynamic : true

		}, dojo.byId(this.id + "_RoleSelectionTable"));
		dojo.addClass(this._userTable.domNode, "fullSize");
		this._userTable.addToolbarButton(this.localResources.ADD_ROLES, false, "addIcon", dojo.hitch(this, this.openAddSodRolesDialog), this._userTable.id+ "_addRoles");
		this._userTable.addToolbarButton(this.localResources.DELETE_ROLES, false, "removeIcon", dojo.hitch(this, this.removeRoles), this._userTable.id+ "_removeRoles",true, true, false);

	},
	_createDmStore : function() {

		this.dmStore = new dojo.data.ItemFileWriteStore( {
			data : {
				identifier : 'name',
				items : []
			}
		});

		this.dmStore.newItem( {
			name : 1
		});

	},
	_createCardinalityBox : function() {

		this._createDmStore();
		var comboBoxItems = this.dmStore;
		this.comboBox = new dijit.form.FilteringSelect( {
			id : "stateSelect",
			name : "state",
			value : "1",
			store : this.dmStore,
			searchAttr : "name"
		}, dojo.byId("cardinalityBox"));

	},


addRoles : function(items) {

	dojo.forEach(items, dojo.hitch(this, function(item) {
		try {
			this.store.newItem(item);
		} catch (e) {
			console.log("may be some item is repeted");
		}

	}));
},
openAddSodRolesDialog : function() {

	dojo.require("com.ibm.security.modeling.notebook.RolesDialog");
	this.roledialog = new com.ibm.security.modeling.notebook.RolesDialog( {
		id : this.id + "_roleDialog",
		rolesUri : this.rolesUri.replace(/roles.*/i, "roles"),
		roleItem : this.roleItem,
		parentWidget : this,
		contextRoot : this.contextRoot
	}, dojo.byId(this.id + "_roleTableDialogContainer"));
	
	this.roledialog.startup();
	this.roledialog.show();
	
	
},

onOK : function() {
	this.messageArea.hideMessages();
	if (this.validate()){
		this.onApply();
		this.hide();	
	}

},
validate : function() {
	  var valid = false;
	    var errorMsg="";
		var description = this.Sod_Description.getValue();



		if(description !=null){
			if(description.length==0 )   errorMsg=this.pluginResources.PLUGIN_SOD_CONSTRAINT_DESCRIPTION_MISSING;
			else if(description.length > 1024) errorMsg= this.pluginResources.PLUGIN_REST_SOD_NOT_VALID.replace("{0}",description);
			else valid=true;

		}
		if(valid && this.totalRoles<=1) {
			errorMsg=this.pluginResources.PLUGIN_SOD_CONSTRAINT_INSUFFICIENT_ROLES;
			valid=!valid;
		}
		if(valid && !this.comboBox.isValid()) {
			errorMsg=this.pluginResources.PLUGIN_REST_SOD_INVALID_CARDINALITY;
			valid=!valid;
		}
		
		if(!valid && this.messageArea != null) {
			this.messageArea.showErrorMessage(errorMsg);
		}
		return valid;
},
onApply : function() {

	var description = this.Sod_Description.getValue().trim();
	dojo.hitch(this.parentWidget,this.parentWidget.addItemsToStore)( {
		"tempStore" : this.store,
		"cardinality" : this.comboBox.attr('value'),
		"description" : description,
		"id" : this.sodID
	});
},

removeRoles : function() {

	var items = this._userTable.grid.selection.getSelected();
	dojo.forEach(items, dojo.hitch(this, function(selectedItem) {
		if (selectedItem != null && selectedItem.id != this.roleItem.id) {

			this.store.deleteItem(selectedItem);
			this._userTable.grid.selection.clear();

		}
	}));

	this.store.save();
	this._userTable.deselectAll();
},

getRestResource : function() {
	var allRolesUri = this._calculateAllRolesUri(this.rolesUri);
	if (this.resourceRestStore == null)
		this.resourceRestStore = new dojox.rpc.Rest(allRolesUri, true);
	return this.resourceRestStore;
},
/**
 * The method calculates the URI for fetching all roles. This is required so
 * that we can append the specific role id for fetching futher information of
 * the roles present in the SOD Policy.
 * 
 * For example: Converts the URI
 * 
 * https://host:port/isrm/rest/models/<modelId>/roles/<roleId> TO
 * https://host:port/isrm/rest/models/<modelId>/roles/
 * 
 */
_calculateAllRolesUri : function(/* String */rolesUri) {
	var strRoleUri = rolesUri.toString();
	var strRolesUri = strRoleUri.substring(0, strRoleUri.lastIndexOf("/"));

	return strRolesUri.concat("/");
},

fillData : function(sodConstraint) {

	this.sodID = sodConstraint.id;
	var resourceRequest;
	this.Sod_Description.value = sodConstraint.description;
	this.sodCardinality = sodConstraint.cardinality;
	this.totalItem = sodConstraint.roles.length;

	this.okButton.attr("disabled", true);
	var Resource = this.getRestResource();
	for ( var i = 0; i < this.totalItem; i++) {
		resourceRequest = Resource(sodConstraint.roles[i].id);
		resourceRequest.addCallback(dojo.hitch(this, this.onSuccess));
		resourceRequest.addErrback( function(response) {
			alert("Error in loading roles");
		});

	}

},
onSuccess : function(roleItem) {
	this.itemAdded++;
	try {
		if(roleItem.id == this.roleItem.id){
			roleItem.directlyAttached=false;
		}
		this.store.newItem(roleItem);
	} catch (e) {
		console.log(e);
		console.log("error while adding data to sod dialog table");
	}
	if (this.itemAdded == this.totalItem) {

		if (this.sodCardinality > this.cardinality)
			this.comboBox.attr('value', this.cardinality || 1);
		else
			this.comboBox.attr('value', this.sodCardinality);
		this.itemAdded = 0;
		//setTimeout(dojo.hitch(this, this.makeThisDisabled), 1000);
		this.okButton.attr("disabled", false);
	}
}

});

dojo.declare("com.ibm.security.modeling.notebook.SodTable", [ com.ibm.security.modeling.tables.Table, com.ibm.security.modeling.tables._DisabledSelection],{
	postCreate : function(){
		this.inherited(arguments);
		this._setupSelectionLogic();
	}
});
