/* 
Licensed Materials - Property of IBM

Role and Policy Modeler


US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.notebook.SeperationOfDutyPanel");

dojo.require("dijit._Widget");
dojo.require("com.ibm.security.modeling.layouts.TemplatedPanelWithLayout");
dojo.require("dijit.Toolbar");
dojo.require("dojo.date.locale");
dojo.require("dijit.form.TextBox");
dojo.require("dojox.data.ClientFilter"); 
dojo.require("dojox.data.JsonRestStore");
dojo.require("dojox.data.AndOrWriteStore");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.tables.Table");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");
dojo.require("com.ibm.security.modeling.notebook.SODHelperClasses");
dojo.require("com.ibm.security.modeling.util.UrlUtilities");
dojo.requireLocalization("com.ibm.security.modeling.notebook", "SeparationOfDutyText");
dojo.requireLocalization("com.ibm.security.modeling", "LogMessages");

dojo.declare(
				"com.ibm.security.modeling.notebook.SeperationOfDutyPanel",
				[ com.ibm.security.modeling.layouts.TemplatedPanelWithLayout ],
				{
					templatePath : dojo.moduleUrl("com.ibm.security.modeling.notebook","templates/SeperationOfDutyPanel.html"),
					widgetsInTemplate : true,
					tableTitle : "",	
					_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.notebook.SeperationOfDutyPanel"),
					
					constructor : function(args) {
						this._trace.entering("", "constructor");
						this._trace.object(args, "constructor", "Constructor args");
						this.logMessages = dojo.i18n.getLocalization("com.ibm.security.modeling", "LogMessages");
						this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.notebook","SeparationOfDutyText");
						this.connections=[];
						this._urlUtilities = com.ibm.security.modeling.util.UrlUtilities();
						this._trace.exiting("", "constructor");
					},
					postCreate : function() {
						this._trace.entering("", "postCreate");
						this.inherited(arguments);
						dojo.addClass(this.domNode, "contentArea");
						this._trace.exiting("", "postCreate");
					},				
					destroy : function() {
						for(var i=0;i<this.connections.length;i++) dojo.disconnect(this.connections[i]);
						this.inherited(arguments);			
					},
					reBuild: function(roleItem) {
						this.refresh(roleItem);
					},
					//refresh the sod panel , with new data 
					refresh:function(roleItem){
						this.roleItem = roleItem;
						this.rolesUri = this.roleItem.uri.toString();
						this._refreshPanelTitle({newValue:this.roleItem.name});
						
						if(this.table) {
							this.table.resetAppliedSearch();
							if(this._getSodConstraintUri()!=this.store.target){
								this._setUpStore();
								this.table.grid.setStore(this.store,{roleId:this.roleItem.id});
							}else {
								this.table.grid.setQuery({roleId:this.roleItem.id});
							}
							
							this.table.refresh();
						}
					},
					_refreshHandler: function() {
						this._trace.entering(this.id, "_refreshHandler");
						this.parentWidget._refreshHandler("SODTab");
						this._trace.exiting(this.id, "_refreshHandler");
					},	
					populate: function(roleItem) {
						
						//refresh will add common data
						this.refresh(roleItem);
						this._setUpStore();
						this._setUpTable();
						this._setUpToolbar();
						this.startup();
						this.table.startup();
						
						// Forcily setting the query parameter , as after
						// startup it was getting reset
						// causing failure of client side sort				
						dojo.mixin(this.table.grid.query,{roleId:this.roleItem.id});
					},
					
					noAction: function() {
					},
					
					getRefreshConfig: function() {
						// Role Property to Activity mapping.
						// Representing that if the specified activity takes place on the specific role property
						//	then which function should be executed on Tab switch to this panel.
						if (!this.refreshConfig) {
							var scopedMethod = dojo.hitch(this, this.noAction);
							this.refreshConfig = {
								"name":{	// on role name update: update title of this panel.
									"update": {
										"methods": [dojo.hitch(this, this._refreshPanelTitle)]
									}
								},
								"children":{	// on add or remove of children
									"add": {
										"methods": [dojo.hitch(this, this._refreshPanel)]
									},
									"remove": {
										"methods": [dojo.hitch(this, this._refreshPanel)]
									}
								},
								"parents": {	// on add or remove of parent; 
									"add": { 
										"methods": [dojo.hitch(this, this._refreshPanel)]
									},
									"remove": {
										"methods": [dojo.hitch(this, this._refreshPanel)]
									}
								},
								"membership":{	// on add or remove of membership;
									"add": {
										"methods": [dojo.hitch(this, this._refreshPanel)]
									},
									"remove": {
										"methods": [dojo.hitch(this, this._refreshPanel)]
									}
								},								
								"filter": {
									"create": {
										"methods": [scopedMethod]
									},
									"update": {
										"methods": [scopedMethod]
									},
									"delete": {
										"methods": [scopedMethod]
									}
								}				
							};
						}
						
						return this.refreshConfig;
					},
					
					_refreshPanelTitle: function(data) {
						
						var escapedRolename = com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag( data.newValue);
						this.tableTitle = dojo.string.substitute(
							this.localResources.TABLE_TITLE, {
								roleName : escapedRolename
							});
						this.Title.innerHTML=this.tableTitle;
					},

					_refreshPanel: function(data, roleItem) {
						this.refresh(roleItem);
					},					
					
					formatRoleNamesCell : function(roles) {
						
						var roleString ="" ;
						for(var i=0;i<roles.length;i++){
								roleString +=roles[i].name+",";
						}
																	
						return roleString.substring(0,roleString.length-1);
					},
					formatDescription : function(item) {
						var anchor=null;
						if(item){
							var description = item.description;
							anchor=new com.ibm.security.modeling.util.Anchor({
								input:description,
								callback:dojo.hitch(this, this.onEditSelected,item)
							});							
						}
						return anchor;
					},
					
					showConflicts:function(target){
					
						this.conflicts=new com.ibm.security.modeling.notebook.MemberConflictDialog({
							localResources:this.localResources,
							globalResources:this.globalResources,
							contextRoot: this.contextRoot,
							target:target

							},dojo.byId(this.id+ "_TableDialogContainer"));
				
						this.conflicts.show();
						this.conflicts.startup();
						
					},
					formatViolationsCell : function(item) {
						
						var temp_String;
						var anchor;
						if(item){						
							if (item.violations > 0) {              
								temp_String= "<img class='imageHeight' src=/securitymodeling/js/src/com/ibm/security/modeling/notebook/images/violation_warning.png />";
								 anchor=new com.ibm.security.modeling.util.Anchor({
										add:temp_String,
										input:item.violations,
										callback:dojo.hitch(this,this.showConflicts,this.sodConstraintsUri+"/"+item.id+"/violations")
									});
										
							} else {
								temp_String="<img class='imageHeight' src=/securitymodeling/js/src/com/ibm/security/modeling/notebook/images/no_violation.png />";
								anchor=temp_String+" "+item.violations;			
							}
						}				
						return anchor;				
					},
				     _getSodConstraintUri:function(){
						 
							 return this.rolesUri.replace(/role.*/i,"sodconstraints");
					},
					_setUpStore : function(){
						this.sodConstraintsUri =this._getSodConstraintUri();
                        this._trace.info("Sod constraints Uri is " + this.sodConstraintsUri, "_setUpStore");
                  
						this.store = new com.ibm.security.modeling.tables.JsonRestStore( {
							target : this.sodConstraintsUri,
							ignoreMatchQuery:"roleId",
							idAttribute : "id"
						});
						var getValue=this.store.getValue;
						this.store.getValue = function(/*Object*/ item, /*String*/property, /*value?*/defaultValue){
						    var value = getValue(item,property,defaultValue);
						    if(property == 'roles'){
								var roles = value;
								var roleString ="" ;
								for(var i=0;i<roles.length;i++){
										roleString +=roles[i].name+",";
								}
																					
								return roleString.substring(0,roleString.length-1);
							}
				            return value;
						};
					},
					
					_setUpTable : function() {
						
						this._trace.entering("", "_setUpTable");
					
						this.sodTableStructure = [
								{field : 'description',name : this.localResources.HEADER_COLUMN_0,width : '30%',get:function(rowIndex,item){return item},formatter : dojo.hitch(this,this.formatDescription) },
								{field : 'roles',name : this.localResources.HEADER_COLUMN_1,width : '30%'},
								{field : 'cardinality',name : this.localResources.HEADER_COLUMN_2,width : '20%',isSearchable : false},
								{field : 'violations',name : this.localResources.HEADER_COLUMN_3,width : '20%',isSearchable : false,get:function(rowIndex,item){return item},formatter : dojo.hitch(this,this.formatViolationsCell)}
						];
							
						this.table = new com.ibm.security.modeling.tables.Table(
								{
									id : this.id + "_sodConstraintsTable",
									structure : this.sodTableStructure,
									dataStore:this.store,
									query:{roleId:this.roleItem.id},
									isClientSide : true,
									isFilterRendered : true,
									isRefreshRendered : true,
									isMultiSelect : true,
									isDynamic:true,
									autoHeight:false,
									messageArea:this.messageArea,
									parentWidget:this,
									entityTypeText:this.globalResources.SEPERATION_OF_DUTY_CONSTRAINTS,
									searchInstructions:this.localResources.SEARCH_INSTRUCTION
								}, dojo.byId(this.id+"_sodConstraintsTable"));
                        
                        dojo.addClass(this.table.domNode,"contentArea");
						

					},
					
					_setUpToolbar : function() {
						this.table.addToolbarButton(
								this.localResources.CREATE_SOD, false,
								"newIcon", dojo.hitch(this, this.onCreateNew),
								this.id + "_new", true, true, true);
						this.table.addToolbarButton(this.localResources.EDIT_SOD, false,"editIcon", dojo.hitch(this,this.onEditSelected,null),this.id + "_edit", true, false, false);
						this.table.addToolbarButton(this.localResources.DELETE_SOD, false,"deleteIcon", dojo.hitch(this,this.onDeleteSelected), this.id	+ "_delete", true, true, false);

					},			
					onCreateNew : function() {
						this.changing=false;
						this.dataToFill={};
						this.dataToFill.id=null;
						this.dataToFill.description="";
						this.dataToFill.cardinality="1";
						
						var defaultRole = {};
						defaultRole.id = this.roleItem.id;
						this.dataToFill.roles=[];
						this.dataToFill.roles[0]=defaultRole;
						this.openSeparationOfDutyDialog();

					},
					
					openSeparationOfDutyDialog : function() {

						dojo.require("com.ibm.security.modeling.notebook.SeparationOfDutyDialog");
												
						this.SodDialog = new com.ibm.security.modeling.notebook.SeparationOfDutyDialog(
								{
									id : this.id + "_SODDialog",
									namespace : this.namespace,
									rolesUri : this.rolesUri,
									roleItem : this.roleItem,
									changing : this.changing,
									parentWidget : this
								}, dojo.byId(this.id + "_SoDTableContainer"));
		
						this.SodDialog.fillData(this.dataToFill);	
						this.SodDialog.startup();
						this.SodDialog.show();
						
					},
					onEditSelected : function(item) {
	
						if(!item) {
							items = this.table.grid.selection.getSelected();
							this.table.deselectAll();
							
						}
						else items=[item];

						if(items.length!=1){
							alert("Two Elements can't be edited once");
							return;
						}
						//Fetch the item from JSONRestStore
					//	itemToEdit = this.store.fetchItemByIdentity({identity:items[0].id});
						itemToEdit = items[0];
						// Marking selected item to changing
						this.store.changing(itemToEdit);
						this.changingItem=itemToEdit;
						// flag to indicate action is "EDIT"
						this.changing=true;
						
						this.dataToFill={};
						this.dataToFill.id=itemToEdit.id;
						this.dataToFill.description=itemToEdit.description;
						this.dataToFill.cardinality=itemToEdit.cardinality;
						
						var tempRoles = {};
						this.dataToFill.roles=[];
			
						for(var i=0;i<itemToEdit.roles.length;i++){			
							tempRoles.id = this._urlUtilities.getIdfromUri(itemToEdit.roles[i].uri)[0];				
							this.dataToFill.roles[i]=dojo.clone(tempRoles);
						}					
						this.openSeparationOfDutyDialog();	
					},
		
					doDelete:function(items){
						

						dojo.forEach(items, dojo.hitch(this, function(
								selectedItem) {
							if (selectedItem !== null) {
								itemToDelete = selectedItem;	
								this.store.deleteItem(itemToDelete);
								this.store.save({
									onComplete:dojo.hitch(this,function(){
										var eventData = {};
										eventData.property = "sodconstraints";
										eventData.activity="remove";
										dojo.publish("ROLE_PROPERTY_UPDATE_EVENT", [eventData]);
										//this.table.refresh();
									}),
									onError:dojo.hitch(this,this.saveError)
							});
								this.table.grid.selection.clear();
							}
						}));

					},
					onDeleteSelected : function() {
						
						var items = this.table.grid.selection.getSelected();
						var SodName=items[0].description;
						for(var i=1;i<items.length;i++){
							SodName +=","+items[i].description;
						}
						
						SodName = com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(SodName);
						this.deleteConfirmMessage = dojo.string.substitute(this.logMessages.CONSOLE_SEPARATION_DUTY_DELETE_CONFIRMATION, {"sodNames":SodName});
						dojo.require("com.ibm.security.modeling.widgets.ConfirmMessageDialog");
						var deleteDialog = dijit.byId(this.id+"sodDeleteDialog");
						if(!deleteDialog){
							var deleteDialog = new com.ibm.security.modeling.widgets.ConfirmMessageDialog({
										   id:this.id+"sodDeleteDialog",
										   submitButtonText:this.globalResources.OK,
										   cancelButtonText:this.globalResources.CANCEL,
										   completeMessage:this.deleteConfirmMessage,
										   title:this.localResources.DELETE_SOD,
										   onSubmit: dojo.hitch(this,this.doDelete,items)
										});
							deleteDialog.show();
							deleteDialog.startup();										
						}else{
							deleteDialog.onSubmit =  dojo.hitch(this,this.doDelete,items);
							deleteDialog.setMessage(this.deleteConfirmMessage);
							deleteDialog.show();
						}

						
						
					},
					addItemsToStore : function(args) {	
						var roles = [];
						var i = 0;
						args.tempStore
								.fetch( {
									onComplete : function(items, request) {

										dojo.forEach(items,dojo.hitch(this,function(item) {
																			
																			var roleObj = {};
																			roleObj.name = args.tempStore.getValue(item,"name");
																			roleObj.uri = args.tempStore.getValue(item,"uri");
																			roles[i++] = roleObj;

																		}))

									}

								});
						var item = {};
						item.id = args.id;
						item.roles = roles;
						item.uri = null;
						item.description = args.description;
						item.cardinality = args.cardinality;
						item.violations = 0;
						if(this.changing){
							this.store.setValue(this.changingItem,"roles",roles);
							this.store.setValue(this.changingItem,"description",args.description);
							this.store.setValue(this.changingItem,"cardinality",args.cardinality);
							this.store.setValue(this.changingItem,"violations",0);
							this.changing=false;
							
							
							
						}else	{
							var data = this.store.newItem(item);
						}
						this.saveIndicator.show(this.saveIndicatorText);
						this.store.save({alwaysPostNewItems:true,onComplete:dojo.hitch(this,this.onComplete),onError:dojo.hitch(this,this.saveError)});
						
					},				
					saveError:function(error){
						dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
						this.saveIndicator.hide(this.saveIndicatorText);
						var errorUtils = new com.ibm.security.modeling.util.ErrorUtilities();
						errorUtils.handleRESTError(error,this.messageArea);
					},
					
					onComplete:function(args){
						//The warning information is shown for the RoleHierarchy Validation.
						var eventData = {};
						eventData.property = "sodconstraints";
						eventData.activity="add";
						//this.table.refresh();
						dojo.publish("ROLE_PROPERTY_UPDATE_EVENT", [eventData]);
						var warningMsg = "";	
                        var warnings=null;
    					warnings=args[0].deferred.results[0].warnings;
					    if(warnings){                            
                           for(var i=0;i<warnings.length;i++){
                               warningMsg=warnings[i].text;
							   this.messageArea.showWarningMessage(warningMsg);
						   }
                        }
					    this.saveIndicator.hide(this.saveIndicatorText);
						
	                },	
					renderWidget : function() {
						this.resize();
					}							
});
