/*
Licensed Materials - Property of IBM

Role and Policy Modeler

� Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.projects.NewProjectCreationPanel");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.security.modeling.dijit.layout.ContentPane");
dojo.require("com.ibm.security.modeling.dijit.layout.BorderContainer");
dojo.require("com.ibm.security.modeling.layouts.TemplatedPanelWithLayout");
dojo.require("dijit.Dialog");
dojo.require("dijit.form.FilteringSelect");
dojo.require("dijit.form.Textarea");
dojo.require("dijit.form.TextBox");
dojo.require("ibm.tivoli.navmodel.Exception");
dojo.require("com.ibm.security.modeling.projects.ProjectsRestStore");
dojo.require("ibm.tivoli.tip.navwidget.topology.dijit.TopologyView");
dojo.require("com.ibm.security.modeling.topology.TopologyDataSets");
dojo.require("com.ibm.security.modeling.help.HelpLink");
dojo.require("ibm.tivoli.tip.dijit.TextInputBox");
dojo.require("ibm.tivoli.tip.dijit.TextArea");
dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
dojo.require("com.ibm.security.modeling.util.Trace");

dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling", "PluginMessages");
dojo.requireLocalization("com.ibm.security.modeling.projects", "NewProjectCreationPanelText");

dojo.declare("com.ibm.security.modeling.projects.NewProjectCreationPanel",
            [ com.ibm.security.modeling.layouts.TemplatedPanelWithLayout ],
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.projects.NewProjectCreationPanel"),
	_errorUtil: null,
	projname: null,
	projdescr: null,
	store: null,
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.projects","templates/NewProjectCreationPanel.html"),	
	widgetsInTemplate:true,
	// globalResources: Object
	//		The global resource bundle
	globalResources: null,
	// localResources: Object
	//		The resource bundle for this widget set
	localResources: null,
	// messageResources: Object
	//      The resource bundel for TMS messages
	messageResources: null,
	// namespace: String
	//		The identifier for the current portlet
	namespace: "",
	/// connections: Object[]
	///		Array of event connections
	///connections: [],
	// messageArea: ibm.tivoli.tip.dijit.TIPMessageArea
	//		Message area for error display
	//messageArea: null,
	// hostname: String
	//		Hostname for server requests
	hostname: "",
	// port: String
	//		Port for server requests
	port: "16311",
	contextRoot: "",
	image_path: dojo.moduleUrl("com.ibm.security.modeling.projects","../themes/images"),

	//This flag is used to clear showing the "duplicate project name" message once unique project name is specified 
	projectname_exists: false,

	//This variable is set to the project name value when the panel is launched for Editing Project Name and Description. Otherwise its null in case of creating a new project 
	projectNameForEdit: null,

	//Max char limit for Project Name
	maxLenghtLimitforName: 240,
	
	//Max char limit for Project Description
	maxLenghtLimitforDescription: 1024,
	
	
	constructor: function (args) {
		// summary:
		// 			Widget that provides the UI for the user and resource selection page.
		// description: 
		//			this widget sets up the selection widgets and behavior
		this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
		this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.projects", "NewProjectCreationPanelText");
		this.pluginResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "PluginMessages");
	},
	postCreate: function() {
		this.inherited(arguments);
		//this._setupAttributeDropdown();
		dojo.addClass(this.domNode,"fullSize");
		 this.projname._inputNode.style.width = "376px";
		// Set the validator to validate the projectname.
		this.projname.constraints.validator = dojo.hitch(this, this._validateProjectName);
		this.projectname_exists = false;
	},
	getTitle:function(){
		return "Create ";
	},
	showEverything: function() {
		alert("showeverything");
	},
	getTest:function(){
		return "getTest value";
	},
	getHelpFileName: function(){
		return "ui_new_project.html";
	},
	_isProjectNameExist: function(projectname){
		dojo.require("com.ibm.security.modeling.projects.ProjectsTablePanel");
		//Since this is client side table , we can always fetch .
		this.restStore = dojox.data.JsonRestStore.getStore({
			target:'/isrm/rest/projects/'
		});
			
		var attritems;
		var storeSyncMode = this.restStore.syncMode;
		this.restStore.syncMode = true;
		var request = this.restStore.fetch({query:null, onError: dojo.hitch(this, this._handleError)});
		this.restStore.syncMode = storeSyncMode;
		if(request.results) {
			this._projects = request.results;
		} else {
			throw("Rest API failed.");
		}
		
		var returnVal = false;
		for(i=0; i<this._projects.length; i++){
			var name = this.restStore.getValue(this._projects[i], "name");
			if(projectname == name.trim()){
				returnVal = true;
				break;
			}
		}
		return returnVal;
	},
	_handleError: function(error) {
		this._errorUtil.handleRESTError(error, this.messageArea);
	},
	_validateProjectName: function(){
		var goAhead = true;
		this._trace.finest('next clicked',"_validateProjectName");
		//this.messageArea.hideMessages(); // clear previous messages before new validation
		
		var enteredname = this.projname.getValue().trim();
		if((enteredname == null)||(enteredname == "")){
			this.projname.setFieldStatusToError();
			this._showMessage("critical",this.pluginResources.PROJECT_NAME_CANNOT_BE_EMPTY);
			goAhead = false;	
		}
		else{
			try{
				
				//First If to check if entered project name is unique or not
				if(this._isProjectNameExist(enteredname)){
					//Second If to check if entered name is not the same name being Edited (in case of Project Edit operation)
					var isNewProject = this.projectNameForEdit==null || this.projectNameForEdit==undefined ;
					if (isNewProject || enteredname != this.projectNameForEdit.trim())	{
						this._showMessage("critical",this.pluginResources.PROJECT_NAME_ALREADY_EXISTS);
						this.projectname_exists = true;
						goAhead = false;
					}
					else
					{
						this.projectname_exists = false;
						this.messageArea.hideMessages();
						goAhead = true;
					}
				}
				else if(enteredname.length > this.maxLenghtLimitforName ) {
					if (dojo.isIE) {
						this._showMessage("critical",this.pluginResources.PROJECT_NAME_TOO_LONG);
						this.projectname_exists = true;
						goAhead = false;
					}
				}

			}catch(e){
				this._showMessage("critical",this.pluginResources.STORE_UNAVAILABLE + "error:" + e);
				goAhead = false;
			}
		}

		if (goAhead) { 
				this.messageArea.hideMessages();
		}
		if(goAhead){
			return null;
		}else{
			return { 
                state: 2,
                message: this.localResources.PROJECT_NAME_ALREADY_EXISTS  
            	};
		}
	},
	validateFields: function() {
		this.messageArea.hideMessages(); // clear previous messaged before new validation
		if(this._validateProjectName()){
			return false;
		}else{
			var enteredname = this.projname.getValue().trim();
			if ((enteredname.trim() == null)||(enteredname.trim().length == 0)){
				 this.projname.setFieldStatusToError();
					this._showMessage("critical",this.pluginResources.PROJECT_NAME_CANNOT_BE_EMPTY);
					return false;
			}
			else if(this.projdescr.getValue().length > this.maxLenghtLimitforDescription) {
				this._showMessage("critical",this.pluginResources.PROJECT_DESCRIPTION_TOO_LONG);
				return false;
			}
			else{
				return true;
			}	
		}
		return goAhead;
		//return goAhead; //Boolean
	},
	_showMessage: function(/*String*/type, /*String*/message) {
		// summary:
		//			shows the given message and type in the local message area
		// tags:
		//			private
		if (type === "critical") {
			this.messageArea.hideMessages();
			this.messageArea.showErrorMessage(message);
		}else if (type === "info") {
			this.messageArea.showInfoMessage(message);
		}
	},

	initPanel: function(){
		this.clearFields();
	},
	
	clearFields: function(){
		this.projdescr.setValue("");
		this.projname.setValue("");
		this._projects = null;
	},
	
	validate: function(){
			return this.validateFields();
	},
	
	destroy: function()	{
		//clearFields();
		this.inherited(arguments);
		//dojo.forEach(this.connections, dojo.disconnect);		
	}
});


/*
 * This TextBox is a wrapper over the ibm.tivoli.tip.dijit.TextInputBox
 * With additional Tivoli specific UI guidelines implementation.
 * This TextBox overrides setStatus method to suppress the tooltip on validation.
 * The tooltip is not required as per RM UI guidelines.
 */
dojo.provide("com.ibm.security.modeling.projects.TextBox");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("ibm.tivoli.tip.dijit.TextInputBox");

dojo.declare("com.ibm.security.modeling.projects.TextBox", ibm.tivoli.tip.dijit.TextInputBox, {
	maxlength: null,
	postCreate: function(){
		//Lets not allow the characters more than maxlength to be entered.
		if(this.maxlength !== null){
			dojo.attr(this._inputNode, "maxLength", this.maxlength);
		}
		dijit.setWaiRole(this._inputNode, "textbox");
		if (this.required === true || this.required === "true") {
			dijit.setWaiState(this._inputNode, "required", "true");
		}
		dojo.attr(this._inputNode, "id", this.id + "Input");
	},
	setStatus: function(status)
    {
		status.message = "";
        // update the status
        this.inherited("setStatus", arguments);
        // update the widget
        this._updateWidget();
    },
    setFieldStatusToError: function(){
    	this.domNode.className = "textInputError";
        this._iconNode.src = this.constants.ERROR_ICON;
    }
});

/*
 * This TextBox is a wrapper over the ibm.tivoli.tip.dijit.TextArea
 * With additional Tivoli specific UI guidelines implementation.
 * This TextArea overrides _updateCounter method to suppress the status
 * at the bottom of the TextArea.
 */

dojo.provide("com.ibm.security.modeling.projects.TextArea");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("ibm.tivoli.tip.dijit.TextArea");

dojo.declare("com.ibm.security.modeling.projects.TextArea", ibm.tivoli.tip.dijit.TextArea, {
	postCreate:function() {
		this.inherited(arguments);
		dijit.setWaiRole(this._textAreaNode, "textbox");
		if (this.required === true || this.required === "true") {
			dijit.setWaiState(this._textAreaNode, "required", "true");
		}
		dijit.setWaiState(this._textAreaNode, "multiline", "true");
		dojo.attr(this._textAreaNode, "id", this.id + "Input");
	},
	_updateCounter: function()
    {
		// Doing nothing to suppress the updation of status. 
    }
});