/*
Licensed Materials - Property of IBM

Role and Policy Modeler

� Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.projects.NewProjectWizard");


dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.widget.Wizard");
dojo.require("dijit.form.Button");
dojo.require("dijit.layout.BorderContainer");
dojo.require("dijit.layout.ContentPane");
dojo.require("com.ibm.security.modeling.breadcrumbs.BreadcrumbBuilder");
dojo.require("com.ibm.security.modeling.projects.BreadCrumb");
dojo.require("com.ibm.security.modeling.projects.NewProjectCreationPanel");
dojo.require("com.ibm.security.modeling.projects.ProjectCreationWelcomePanel");
dojo.require("com.ibm.security.modeling.util.NlsUtilities");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.api.ProjectRestManager");
dojo.require("com.ibm.security.modeling.projects.ProjectUtil");
dojo.require("com.ibm.security.modeling.widgets.ProgressBar");
dojo.require("com.ibm.security.modeling.widgets.RapmWizard");
dojo.require("com.ibm.security.modeling.util.MessageUtilities");
dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");

dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling", "LogMessages");
dojo.requireLocalization("com.ibm.security.modeling.projects", "ProjectCreationWizardText");

dojo.declare("com.ibm.security.modeling.projects.NewProjectWizard",
            com.ibm.security.modeling.widgets.RapmWizard ,
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.projects.NewProjectWizard"),
	
	globalResources: null,

	localResources: null,

	messageResources: null,

	namespace: "",

	requireBreadCrumb: true,
	
	hostname: "",

	port: "16311",

	contextRoot: "",

	//This booolean flag is used for swithing of topology and table between Create Wizard and Edit Scope dialog
	isCreateView: null,
	
	constructor: function () {
		this._trace.entering(this.id, "constructor");
		
		this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		this.messageResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "LogMessages");
		this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.projects", "ProjectCreationWizardText");
		this._progressBar = new com.ibm.security.modeling.widgets.ProgressBar();
		this._messageUtil = new com.ibm.security.modeling.util.MessageUtilities();
		this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
		
		this._trace.exiting(this.id, "constructor");
	},
	
	//Overriding createPanels
	
	createPanels: function(){
		this._trace.entering(this.id, "createPanels");
		
		this._createWelcomePanel();
		this._createNewProjectPanel();
		this._createUsersSelectionPanel(); 
		this._createPermissionsSelectionPanel();		
		
		// this.wizard.selectChild(this.wizard.getChildren()[1]);
		this.wizard.selectChild(this.wizard.getChildren()[0]);
		
		this._trace.exiting(this.id, "createPanels");
		
	},
	startup:function(){
		this.inherited(arguments);
		this.connect(this.wizard, "selectChild", this.initPanel);
	},
	initPanel:function(){
		if(!this._userScopingPanel.isInitialized && this.wizard.selectedChildWidget.widget == this._userScopingPanel){
			this._userScopingPanel.initPanel();
			this._userScopingPanel.isInitialized=true;
		}else if(!this._permissionScopingPanel.isInitialized && this.wizard.selectedChildWidget.widget == this._permissionScopingPanel){
			this._permissionScopingPanel.initPanel();
			this._permissionScopingPanel.isInitialized=true;
		}
	},
	
	setInitialFocus: function() {
		if(this.wizard.getChildren().length > 0){
			dijit.focus(this.wizard.getChildren()[0].domNode);
		}
	},

	_createWelcomePanel: function(){
		this._trace.entering(this.id, "_createWelcomePanel");
		
		this._projectCreationWelcomePanel = new com.ibm.security.modeling.projects.ProjectCreationWelcomePanel(
			{
				id:this.namespace+"ProjectCreationWelcomePanel",
				namespace: this.namespace,
				messageArea: this.msgArea,
				hostname: this.hostname,
				port: this.port,
				contextRoot: this.contextRoot,
				messageResources: this.messageResources,
				wizardResources: this.localResources
			});

		this.addChild(this._projectCreationWelcomePanel, this.localResources.WELCOME_PAGE, null, "welcomeText");
		this._trace.exiting(this.id, "_createWelcomePanel");
	},

	_createNewProjectPanel: function(){
		this._trace.entering(this.id, "_createNewProjectPanel");
		
		this._newProjectCreationPanel = new com.ibm.security.modeling.projects.NewProjectCreationPanel(
			{
				//id:this.namespace+"NewProjectCreationPanel",
				//namespace: this.namespace+"NewProjectCreationPanel",
				messageArea: this.msgArea,
				hostname: this.hostname,
				port: this.port,
				contextRoot: this.contextRoot,
				messageResources: this.messageResources
			});
		
		this.addChild(this._newProjectCreationPanel, this.localResources.ENTER_PROJECT_DETAILS, dojo.hitch(this, this._onFinish));

		this._trace.exiting(this.id, "_createNewProjectPanel");

	},
	
	_createUsersSelectionPanel: function(){
		this._trace.entering(this.id, "_createUsersSelectionPanel");

		dojo.require("com.ibm.security.modeling.projects.UserScopingPanel");

		this._userScopingPanel = new com.ibm.security.modeling.projects.UserScopingPanel( {
					id:this.id +"_userPanel",
					//id : "_attribScopingContainer",
					//selectedItem:selectedItem,
					//preSelectValues: this.selectedValues,
					//attributeStore: this.attributeStore,
					messageArea:this.msgArea
		});

		this.addChild(this._userScopingPanel, this.localResources.SELECT_USER_RESOURCES, dojo.hitch(this, this._onFinish));

		this._trace.exiting(this.id, "_createUsersSelectionPanel");
	},
	
	
	_createPermissionsSelectionPanel: function(){
			this._trace.entering(this.id, "_createPermissionsSelectionPanel");
	
			dojo.require("com.ibm.security.modeling.projects.PermissionScopingPanel");
	
			this._permissionScopingPanel = new com.ibm.security.modeling.projects.PermissionScopingPanel( {
						id:this.id +"_permissionPanel",
						//id : "_attribScopingContainer",
						//selectedItem:selectedItem,
						//preSelectValues: this.selectedValues,
						//attributeStore: this.attributeStore,
						messageArea:this.msgArea
			});
	
			this.addChild(this._permissionScopingPanel, this.localResources.SELECT_PERMISSIONS, dojo.hitch(this, this._onFinish));
	
			this._trace.exiting(this.id, "_createPermissionsSelectionPanel");
	},
	

	_showMessage: function(/*String*/type, /*String*/message) {
		this._trace.entering(this.id, "_showMessage");
		
		this.msgArea.hideMessages();

		if (type === "critical") {
			this.msgArea.showErrorMessage(message);
		}else if (type === "info") {
			this.msgArea.showInfoMessage(message);
		}
		
		this._trace.exiting(this.id, "_showMessage");
	
	},
	
	_onCancel: function(){
		//this._trace.entering(this.id, "_onCancel");
		//this.destroy(); // TODO : this will causing null wizard
		this.isCreateView = false;
		this.msgArea.hideMessages();// clear previous messages
		this._closeWizard();
		//this.wizard.destroyRecursive();
		
		//this._trace.exiting(this.id, "_onCancel");
	},
	
	_onFinish_old: function(){
		this._trace.entering(this.id, "_onFinish");
		
		// summary:
		//			Exits the wizards but persists state
		
		//Validations needed to be called explicitly
		if (this.newProjectCreationPanel.validateFields())
		{	
			if (this.usersResourcesSelectionPanel.validateUserSelections())
			{	
				if (this.permissionsSelectionPanel.validateUserSelections())
				{	
					this._saveProject();
				}
			}
		}
		////alert("_onFinish> " + this.wizard.selectedChildWidget.id + " > " + this.wizardpanels[this.wizard.selectedChildWidget.title] );
		
		this._trace.exiting(this.id, "_onFinish");
	},
	
	_onFinish: function(){
			this._trace.entering(this.id, "_onFinish");
			var passAllPanelValidations = true;
			var wizardpanes = this.wizard.getChildren();
			for(var i=0;i<wizardpanes.length;i++){
				if(wizardpanes[i].widget.validate){
					var result = wizardpanes[i].widget.validate();
					if(!result){
						passAllPanelValidations = false;
					}
				}
			}
			if(passAllPanelValidations){
			this._saveProject();
			}else{
				//TODO Trace here.
				console.log("Failed Panel validations");
			}
			
			this._trace.exiting(this.id, "_onFinish");
	},
	
	_closeWizard: function(){
		this._trace.entering(this.id, "_closeWizard");
		
		//this.newProjectCreationPanel.clearFields();
		
		this._destroyPanels();
		
		this._parent.launchProjectTablePanel(false, true);
		
		this._trace.exiting(this.id, "_closeWizard");
	},
	
	_saveProject: function () {
		this._trace.entering("", "_saveProject");
		
		/*
		 * 
		 * ProgressBar currently commented because this is causing relaunch issue.
		 * commenting the display of progressBar till I find some solution
		 * on this.
		 * 
		 * 
		 */
		 
			this._progressBar.show(this.localResources.CREATE_PROJECT_IN_PROGESS);
		
			
		var projectsUri = "/isrm/rest/projects/";
		var store = dojox.data.JsonRestStore.getStore({
			target:projectsUri
		});
		
			var userScopingRuleItems = this._userScopingPanel._getSelectedScope();
			var permissionScopingRuleItems = this._permissionScopingPanel._getSelectedScope();
		
			console.log(userScopingRuleItems);
			console.log(permissionScopingRuleItems);
		
		//TODO Do we need to normalize the description?
		//var projectDescr = this.normalizeString(this.newProjectCreationPanel.projdescr.getValue());
			var projectDescr = this._newProjectCreationPanel.projdescr.getValue();
			var projectname = this._newProjectCreationPanel.projname.getValue().trim();
			var projectItem = {
				name: projectname,
				description: projectDescr,
				userScoping: userScopingRuleItems,
				permissionScoping: permissionScopingRuleItems
			};
			
			store.newItem(projectItem);
			store.save({
				onComplete:dojo.hitch(this,this._projectCreationSuccessCallback,projectItem),
				onError:dojo.hitch(this,this._projectCreationFailCallback)
			});
      		this._trace.exiting("", "_saveProject");
	},

	_projectCreationSuccessCallback: function (item) {
		
		this._trace.entering("", "_projectCreationSuccessCallback");

		if (item == null){
			return;
		}
		this.isCreateView = false;
		if(this.callback_finish){ 
			var msg = this._messageUtil.formatMessage(this.messageResources.PROJECT_CREATED_SUCCESSFULLY, [item.name]);
			setTimeout(dojo.hitch(this,function(){
				this._progressBar.hide();
				this._closeWizard();
			}),200);
			this.callback_finish(msg, true);
			
		}
		
		this._trace.exiting("", "_projectCreationSuccessCallback");
	},

	_projectCreationFailCallback: function (error, request) {
		// summary:
		//			Callback called on project creation failure. 
		// error: String
		//          Error details
		this._trace.entering("", "_projectCreationFailCallback");
		
		this._trace.error("project creation failed "+ error);
		
		this._progressBar.hide();
		this.isCreateView = false;
		//var msg = this.messageResources.LOG_REST_PROJECT_CREATE_FAILED;
		//this._showMessage("critical", msg);
		this._errorUtil.handleRESTError(error, this.msgArea);
		this._trace.exiting("", "_projectCreationFailCallback");
	},

	normalizeString: function(inputString) {
		var normalizedString = "";
		normalizedString = inputString.replace(/\r/g, "\\r").replace(/\n/g, "\\n").replace(/\t/g, "\\t");
		return normalizedString;
	},
	
	
	_setMaxLimitCountsForUserSelection_old: function(){
			this.wizardpanels[this.localResources.SELECT_USER_RESOURCES].setMaxLimitCountsForUserSelection(this.directUserSelectionLimit, this.attrValueSelectionLimit);
		},
	_setMaxLimitCountsForPermissionSelection_old: function(){
		this.wizardpanels[this.localResources.SELECT_PERMISSIONS].setMaxLimitCountsForPermissionSelection(this.directPermissionSelectionLimit, this.attrValueSelectionLimit);
	},
	
	_updateBreadCrumbAndHelp_old: function(){
		this._updateBreadCrumb();
		this._setHelp();
	},
	
	reset_override: function(){
		this._trace.entering(this.id, "reset");
		
		//Need to reset all the panels. 		

		//this._projectCreationWelcomePanel
		
		this._newProjectCreationPanel.initPanel();
		this._userScopingPanel.initPanel();
		this._permissionScopingPanel.initPanel();
		
		this._trace.exiting(this.id, "reset");
		
	}	
});