/*
Licensed Materials - Property of IBM

Role and Policy Modeler

� Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dijit.Dialog");
dojo.require("dijit.Toolbar");
dojo.require("dijit._Container");
dojo.require("ibm.tivoli.tip.dijit.TIPButton");
dojo.require("com.ibm.security.modeling.dijit.layout.BorderContainer");
dojo.require("com.ibm.security.modeling.dijit.layout.ContentPane");
dojo.require("com.ibm.security.modeling.projects.NewProjectCreationPanel");
dojo.require("com.ibm.security.modeling.util.NlsUtilities");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.api.ProjectRestManager");
dojo.require("com.ibm.security.modeling.projects.ProjectUtil");
dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
dojo.require("com.ibm.security.modeling.widgets.ProgressBar");
dojo.require("com.ibm.security.modeling.widgets.IsrmDialogBase");
dojo.requireLocalization("com.ibm.security.modeling.projects", "ProjectCreationWizardText");

dojo.provide("com.ibm.security.modeling.projects.ProjectEditGeneralInformation");

dojo.declare("com.ibm.security.modeling.projects.ProjectEditGeneralInformation",com.ibm.security.modeling.widgets.IsrmDialogBase,
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.projects.ProjectEditGeneralInformation"),
	wizard: null,
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.projects","templates/ProjectEditGeneralInformation.html"),	
	widgetsInTemplate: true,
	// globalResources: Object
	//			Reference to the global shared resource bundle
	globalResources: null,
	// localResources: Object
	//			Reference to the resource bundle unique to this widget
	localResources: null,
	// messageResources: Object
	//      The resource bundel for TMS messages
	messageResources: null,
	// namespace: [readonly] String
	// 			Unique identifier to the instance of portlet page where this item will exist
	namespace: "",
	// hostname: String
	//			The hostname of the server from which Content is being served.
	hostname: "",
	// port: String
	//			The port of the server from which Content is being served.
	port: "16311",
	// contextRoot: String
	//			The context root of the server from which Content is being served.
	contextRoot: "",
	
	projectRestStore:null,
	// projectRestManager: com.ibm.security.modeling.api.ProjectRestManager
	//		An instance of com.ibm.security.modeling.api.ProjectRestManager. 
	//		It is an interface for HTTP REST communication using dojo stores api. 
	projectRestManager: null,

	// newProjectCreationPanel: com.ibm.security.modeling.projects.NewProjectCreationPanel
	//			The widget for the new project properties panel
	newProjectCreationPanel: null,

	//URL of the project to be edited
	projectUrl: null,

	//Name of the project to be edited
	projectName: null,
	
	//Description of the project to be edited
	projectDescr: null,
	
	parentWidget: null,

	// progressBar: String
	//		The widget used to display the progress of save task. 
	progressBar: null,
	
	minHeight:350,
	
    minWidth:500,
	
	constructor: function (args) {
		// summary:
		// 			Widget that provides the UI for the project edit operation.
		this.projectUrl = args.projectUrl;
		this.projectName = args.projectName;
		this.projectDescr = args.projectDescr;
		this.parentWidget = args.parentWidget;
		
		this.messageResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "LogMessages");
		this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.projects", "ProjectCreationWizardText");
		this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
		this.progressBar = new com.ibm.security.modeling.widgets.ProgressBar();
	},
	postMixInProperties:function(){ 
		this.inherited(arguments);
    	this.dialogTitle = this.localResources.PROJECT_EDIT_GENERAL_DIALOG_TITLE;
    	this.helpFile="ui_edit_project.html";
    },
	postCreate:function(){
		this.inherited(arguments);

		this._createnewProjectCreationPanel();

		//give the container a StyleUtilities global function to figure out bidi direction,
		//since TIP doesn't do bidi the way dojo expects.
		//this.container.isLeftToRight = NlsUtilities.isLtr;
		dojo.addClass(this.domNode,"fullSize");		
    	this.newProjectCreationPanel.startup();

		this.newProjectCreationPanel.projectNameForEdit = this.projectName;
		
		this.newProjectCreationPanel.projname.setValue(this.projectName);
		this.newProjectCreationPanel.projdescr.setValue(this.projectDescr);
		
		this.addCommonButton({
			onClick:dojo.hitch(this, this.onCancel),
			type:"CANCEL"
		});
		this.okButton = this.addCommonButton({
			onClick:dojo.hitch(this, this.onOK),
			type:"OK"
		});

		this.connect(this.projectEditDialog, "onCancel", "onCancel");
	
	},

	startup: function(){	
		try {
					
			this.inherited(arguments);

			this.newProjectCreationPanel.startup();
		
			var newPanel_bottom_id = dijit.byId("ProjectEditGeneralDialog" + "panel1_bottom");
			if (newPanel_bottom_id) {
				newPanel_bottom_id.destroy();
			}
			

			
		} catch (e) {
			this._showMessage("critical", e);
		}
		
	},

	_showMessage: function(/*String*/type, /*String*/message) {
		// summary:
		//			shows the given message and type in the local message area
		// tags:
		//			private

		this.messageArea.hideMessages();

		if (type === "critical") {
			this.messageArea.showErrorMessage(message);
		}else if (type === "info") {
			this.messageArea.showInfoMessage(message);
		}

	},
	

	_createnewProjectCreationPanel: function() {
		// summary:
		//			creates the wizard-panel1 i.e. newProjectCreationPanel		

		this.newProjectCreationPanel = new com.ibm.security.modeling.projects.NewProjectCreationPanel(
				{
					id:this.namespace+"ProjectEditGeneralDialog",
					namespace: this.namespace+"ProjectEditGeneralDialog",
					messageArea: this.messageArea,
					hostname: this.parentWidget.hostname,
					port: this.parentWidget.port,
					contextRoot: this.contextRoot,
					messageResources: this.messageResources
				},
				this.namespace + "ProjectEditGeneralDialog");
	},

	onCancel: function(){
		this._trace.entering(this.id, "_onCancel");
		this.messageArea.hideMessages();// clear previous messages
		this.hide();
		this._trace.exiting(this.id, "_onCancel");
	},

	onOK: function(){
		// summary:
		//			Exits the wizards but persists state
		
		if (this.validate()){
			this.messageArea.hideMessages();
			this.updateProjectNameAndDescription();
			//Removed from here - this.hide();
		}
		
	},

	validate : function() {
	
		if(!this._validateProjectName()){
			return false;
		}else{
			if(this.newProjectCreationPanel.projdescr.getValue().length > 1024) {
				this._showMessage("critical",this.pluginResources.PROJECT_DESCRIPTION_TOO_LONG);
				return false;
			}
			else{
				return true;
			}	
		}
	},
	_validateProjectName: function () {
		this._trace.entering(this.id, "_validateProjectName");
		var valid = false;
		var projectName = this.newProjectCreationPanel.projname.getValue();
		if (projectName != null) {
			projectName = projectName.trim();
			if ((projectName != null) && (projectName.length > 0)) {
				if(this.newProjectCreationPanel._validateProjectName()){
					return false;
				}	
				valid = true;
			}
		}
		if (valid) {
			this.newProjectCreationPanel.projname.setStatus({state: this.newProjectCreationPanel.projname.constants.REQUIRED, message: ""});
		} else {
			this.newProjectCreationPanel.projname.setStatus({state: this.newProjectCreationPanel.projname.constants.ERROR, message: ""});
			if (this.messageArea != null) {
				this.messageArea.hideMessages();
				this.messageArea.showErrorMessage(this.pluginResources.PROJECT_NAME_CANNOT_BE_EMPTY);
			}
		}
		this._trace.finest(this.id + " - Valid: " + valid, "_validateProjectName");
		this._trace.exiting(this.id, "_validateProjectName");
		return valid;
	},

	
	updateProjectNameAndDescription: function () {
		this._trace.entering(this.id, "updateProjectNameAndDescription");
		// 	summary: 
		//			Retrieves the data from Project Name and Description field from GUI and makes REST call (Http PUT).
		
		// validate project name
		//var valid = this._validateProjectName();
		//if (!valid){
		//	this._trace.finest(this.id + " - Project name is not valid", "updateProjectNameAndDescription");
		//	this._trace.exiting(this.id, "updateProjectNameAndDescription");
		//	return;
		//} 
		
		var projectName = this.newProjectCreationPanel.projname.getValue();
		projectName = projectName.trim();
		
		var projectDescription = this.newProjectCreationPanel.projdescr.getValue();
		//projectDescription = this.normalizeString(projectDescription);

		this.progressBar.show(this.localResources.SAVE_IN_PROGRESS_PROJECT_EDIT);
		
		var store = dojox.data.JsonRestStore.getStore({
			target:"/isrm/rest/projects/"
		});
		dojo.require("com.ibm.security.modeling.util.UrlUtilities");
		var urlUtilities = com.ibm.security.modeling.util.UrlUtilities();
		var id = urlUtilities.getIdfromUri(this.projectUrl)[0];
		var item = store.fetchItemByIdentity({identity:id});
		store.setValue(item,"name",projectName);
		store.setValue(item,"description",projectDescription);
		store.save({
			onComplete:dojo.hitch (this, this.projectNameAndDescriptionUpdateSuccess),
			onError:dojo.hitch (this, this.projectNameAndDescriptionUpdateFail)
		});
	},

	projectNameAndDescriptionUpdateSuccess: function () {
		this._trace.entering(this.id, "projectNameAndDescriptionUpdateSuccess");
		var projectName = this.newProjectCreationPanel.projname.getValue();
		projectName = projectName.trim();
		
		var projectDescription = this.newProjectCreationPanel.projdescr.getValue();
		projectDescription = this.normalizeString(projectDescription);

		this.projectName = projectName;
		this.projectDescr = projectDescription;

		this.parentWidget.setUpdatedProjectGerenalInformation_Success(this.projectName, this.projectDescr, this.pluginResources.PROJECT_EDIT_GENERAL_INFO_SUCCESS);

		this.progressBar.hide();
		
		this.hide();
		
		/* Using object for passing the event information. This will do away with the parsing logic required.*/
		/*
		var eventData = {};
		eventData.property = "name";
		eventData.activity = "update";
		eventData.data = {
			"oldValue" : this.roleItem.name,
			"newValue": roleName
		};
		*/
		/* Update the role item. */
		
		//this.rolePropertyUpdateSuccess(eventData);
		
		// Update Dialog header as it contains name of the role being edited in notebook.
		//this.parentWidget.updateDialogHeader();		
		this._trace.exiting(this.id, "projectNameAndDescriptionUpdateSuccess");
		
	},
	
	normalizeString: function(inputString) {
		var normalizedString = "";
		normalizedString = inputString.replace(/\r/g, "\\r").replace(/\n/g, "\\n").replace(/\t/g, "\\t");
		return normalizedString;
	},
	
	projectNameAndDescriptionUpdateFail: function (error) {
		this._trace.entering(this.id, "projectNameAndDescriptionUpdateFail");

		this.progressBar.hide();
		this.hideDialogIfStateEnforcementError(error);
		this._errorUtil.handleRESTError(error, this.messageArea);
		this._trace.exiting(this.id, "projectNameAndDescriptionUpdateFail");
	}
});