/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.projects.ProjectEditScopeInformation");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Dialog");
dojo.require("com.ibm.security.modeling.dijit.layout.TabContainer");
dojo.require("com.ibm.security.modeling.dijit.layout.ContentPane");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.ComboBox");
dojo.require("dijit.form.Select");
dojo.require("dijit.form.Textarea");
dojo.require("dijit.form.FilteringSelect");
dojo.require("dojox.data.JsonRestStore");

dojo.require("com.ibm.security.modeling.widgets.MultiMessageArea");
dojo.require("com.ibm.security.modeling.dijit.layout.BorderContainer");
dojo.require("com.ibm.security.modeling.projects.UserScopingPanel");
dojo.require("com.ibm.security.modeling.projects.PermissionScopingPanel");
dojo.require("com.ibm.security.modeling.widgets.IsrmDialogBase");
dojo.require("com.ibm.security.modeling.api.ProjectRestManager");
dojo.require("com.ibm.security.modeling.projects.ProjectUtil");
dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
dojo.require("com.ibm.security.modeling.widgets.ProgressBar");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.util.TitleUtilities");
dojo.require("com.ibm.security.modeling.util.HelpUtilities");

// NLS
dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling", "LogMessages");
dojo.requireLocalization("com.ibm.security.modeling.projects", "ProjectCreationWizardText");
dojo.requireLocalization("com.ibm.security.modeling", "PluginMessages");

// define the widget class
dojo.declare (
	"com.ibm.security.modeling.projects.ProjectEditScopeInformation", com.ibm.security.modeling.widgets.IsrmDialogBase,
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.projects.ProjectEditScopeInformation"),
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.projects","templates/ProjectEditScopeInformation.html"),

	widgetsInTemplate: true,
	
	// globalResources: Object
	//			Reference to the global shared resource bundle
	globalResources: null,
	// localResources: Object
	//			Reference to the resource bundle unique to this widget
	localResources: null,
	// messageResources: Object
	//      The resource bundel for TMS messages
	messageResources: null,
	
	//logResources: null,

	progressBar: null,
	
	// namespace: [readonly] String
	// 			Unique identifier to the instance of portlet page where this item will exist
	namespace: "",
	
	// contextRoot: String
	//			The context root of the server from which Content is being served.
	contextRoot: "",
	
	// titleUtil: com.ibm.security.modeling.util.TitleUtilities
	//			sets the portlet title to the current panel's	
	titleUtil: new com.ibm.security.modeling.util.TitleUtilities(),
	// helpUtil: com.ibm.security.modeling.util.HelpUtilities 
	//			sets current help file

	//Instance of User Selection Panel
	userPanel: null,

	//Instance of Permission Selection Panel
	permissionPanel: null,

	//The project object of which users and permissions scoping to be modified
	_projectItem: null,
	
	// heightFactor: number
	//		Gives the hight factor for widget
	heightFactor:1.1,
	// widthFactor: number
	//		Gives the width factor for widget
	widthFactor:1.05,
	// minHeight: number
	//		minimum height of widget
	minHeight:350,

	// hostname: String
	//		Hostname for server requests
	hostname: "",
	// port: String
	//		Port for server requests
	port: "16311",
	
	//project url to be modified
	projectUrl: null,

	// projectRestManager: com.ibm.security.modeling.api.ProjectRestManager
	//		An instance of com.ibm.security.modeling.api.ProjectRestManager. 
	//		It is an interface for HTTP REST communication using dojo stores api. 
	projectRestManager: null,

	constructor: function() {
		this._trace.entering("", "constructor");
		
		// globalResources: Object
		//			Reference to the global shared resource bundle
		// logResources: Object
		//			Reference to the log shared resource bundle
		// localResources: Object
		//			Reference to the resource bundle unique to this widget
		
		this.inherited (arguments);
		this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.projects", "ProjectCreationWizardText");
		this.messageResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "LogMessages");
		this.progressBar = new com.ibm.security.modeling.widgets.ProgressBar();
		this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
		
		this._trace.exiting("", "constructor");
	},
	postMixInProperties: function() {
		this.inherited(arguments);
		this.dialogTitle = this.localResources.PROJECT_EDIT_SCOPE_DIALOG_TITLE;
		this.helpFile = "ui_edit_project_scope.html";
	
	},
	postCreate: function() {
		this._trace.entering(this.id, "postCreate");
		
		this.inherited ("postCreate", arguments);
		
			//this.subscribe("ProjectEditScopeTopologySwitchTableView", dojo.hitch(this, this._switchToTableView));
			
			// Setup REST store.
			this._setupRestStore();
			
			this.userPanel=this._createUserPanel();
			this.permissionPanel=this._createPermissionPanel();
			this.userPanel.initPanel();
			this.permissionPanel.initPanel();
			this.subscribe("ProjectNotebook_TabContainer-selectChild", dojo.hitch(this, this.onTabSwitch));
		    this.connect(this.projectEditDialog, "onCancel", "onCancel");
		    this.addCommonButton({
				onClick:dojo.hitch(this, this.onCancel),
				type:"CANCEL"
			});
			this.okButton = this.addCommonButton({
				onClick:dojo.hitch(this, this.onOk),
				label:this.localResources.RE_EVALUATE_SCOPE_BUTTON,
				type:"OTHER"
			});
			this._trace.exiting(this.id, "postCreate");
	},
	
	onTabSwitch:function(){
	
	// Clear all messages when switching tabs
	this.msgArea.hideMessages();
	
	},
				
	startup: function(){
		this._trace.entering(this.id, "startup");
	
		
			this.inherited(arguments);
			
			this.userPanel.startup();
			this.permissionPanel.startup();
									
			//this._removeBottomPanels();
			
		this._trace.exiting(this.id, "startup");
	},

	_removeBottomPanels: function() {

	//This function removes the bottom panel of the users and permissions panel 
		
		var userPanel_bottom_id = dijit.byId(this.namespace+"panel2_bottom");
		
		if (userPanel_bottom_id) {
			dojo.style(userPanel_bottom_id.domNode, "display", "none"); 
		}
		
		var permissionPanel_bottom_id = dijit.byId(this.namespace+"panel3_bottom");
		
		if (permissionPanel_bottom_id) {
			dojo.style(permissionPanel_bottom_id.domNode, "display", "none"); 
		}
		
		var userPanel_SummaryWidegt_id = dijit.byId(this.namespace+"summaryWidget_user");
		if (userPanel_SummaryWidegt_id) {
			dojo.addClass(userPanel_SummaryWidegt_id.domNode,"summary_forEditScope");
		}
		
		var permissionPanel_SummaryWidegt_id = dijit.byId(this.namespace+"permission_summaryWidget");
		if (permissionPanel_SummaryWidegt_id) {
			dojo.addClass(permissionPanel_SummaryWidegt_id.domNode,"summary_forEditScope");
		}		
	},


	_displayError: function() {
		// summary:
		//			Displays error using 'msgArea' widget.
		this._trace.entering(this.id, "_displayError");
		this.messageArea.showErrorMessage(this.error);
		this._trace.exiting(this.id, "_displayError");
	},
	
	_createUserPanel: function () {
		
	//This function creates the users panel 	
	
		this._trace.entering(this.id, "_createUserPanel");
		var userPanel = new com.ibm.security.modeling.projects.UserScopingPanel (
			{
					id:this.id +"_userPanel",
					namespace:this.namespace,
					messageArea: this.messageArea,
					contextRoot: this.contextRoot,
					hostname:this.hostname,
					port:this.port,
					bIsProjectScopeEdit: true,
					learnMoreLinkFile:  "ui_users_select_edit_learn_more.html",
					messageResources: this.messageResources
			},
			this.id +"_UserPanel");
		
		dojo.addClass(userPanel.domNode,"contentArea");
		
		//userPanel.startup();
		this._trace.exiting(this.id, "_createUserPanel");
		return userPanel;
	},

	_createPermissionPanel: function () {
		
		//This function creates the permissions panel
		
		this._trace.entering(this.id, "_createPermissionPanel");
		var permissionPanel = new com.ibm.security.modeling.projects.PermissionScopingPanel (
			{
					id:this.id+"_permissionPanel",
					namespace:this.namespace,
					messageArea: this.messageArea,
					contextRoot: this.contextRoot,
					hostname:this.hostname,
					port:this.port,
					bIsProjectScopeEdit: true,
					learnMoreLinkFile:  "ui_permissions_select_edit_learn_more.html",
					messageResources: this.messageResources
			},
			this.id+"_PermissionPanel");
		
		dojo.addClass(permissionPanel.domNode,"contentArea");
		
		this._trace.exiting(this.id, "_createUserPanel");
		return permissionPanel;
	},

	onOk : function() {
		// summary:
		// 			Gets called when OK button is clicked.
		
		if (this.validateSelections()){
			this.messageArea.hideMessages();
			this.updateProjectScopingInformation();
		}

		//this.hide();
	},
	
	validateSelections : function() {
	
		//This function performs validations on the users and permissions panel
		
		if (this.userPanel.validate())
		{	
			if (this.permissionPanel.validate())
			{
				return(true);
			}
		}
		
		return(false);
	},
	
	updateProjectScopingInformation : function() {
	
		//This function performs the actual modify operation on the project
		
		this._setupRestStore();
		
		this.progressBar.show(this.localResources.SAVE_IN_PROGRESS_PROJECT_EDIT);
		
		//var selectedUsersData = this.userPanel.selectedValues;
		//var selectedPermisssionsData = this.permissionPanel.selectedValues;
		
		//var projUtil = new com.ibm.security.modeling.projects.ProjectUtil({});
		//var userScopingRuleItems = projUtil.getUserScopingRule(selectedUsersData);
		//var permissionScopingRuleItems = projUtil.getPermissionScopingRule(selectedPermisssionsData);

		var userScopingRuleItems = this.userPanel._getSelectedScope();
		var permissionScopingRuleItems = this.permissionPanel._getSelectedScope();

		this.projectRestManager.updateProjectScopingInformation(
				null,
				userScopingRuleItems,
				permissionScopingRuleItems,
				dojo.hitch (this, this.projectScopingInformationUpdateSuccess),
				dojo.hitch (this, this.projectScopingInformationUpdateFail));
		this._trace.exiting(this.id, "updateProjectScopingInformation");
		
	},
	
	_setupRestStore: function () {
	
		//Setup the rest store before performing the actual modify operation on the project
		
		this._trace.entering("", "_setupRestStore");
		this.projectRestManager = new com.ibm.security.modeling.api.ProjectRestManager({
			resourceUri: this.projectUrl,
			syncMode: true,
			messageArea: this.messageArea
		});
		
      	this._trace.exiting("", "_setupRestStore");
	},

	projectScopingInformationUpdateSuccess: function () {
		this._trace.entering(this.id, "projectScopingInformationUpdateSuccess");

		this._publishEditProjectScopeCompleteEvent();
		
		this.progressBar.hide();
		
		this.hide();
		
		this._trace.exiting(this.id, "projectScopingInformationUpdateSuccess");	
	},

	projectScopingInformationUpdateFail: function (error) {
		this._trace.entering(this.id, "projectScopingInformationUpdateFail");
		this.progressBar.hide();
		this.hideDialogIfStateEnforcementError(error);
		this._errorUtil.handleRESTError(error, this.messageArea);
		this._trace.exiting(this.id, "projectScopingInformationUpdateFail");
	},

	_publishEditProjectScopeCompleteEvent: function() {
		//This function publishes the event for project edit operation complete
		
		this._trace.entering(this.id, "_publishEditProjectScopeCompleteEvent");
		dojo.publish("PROJECT_EDIT_SCOPE_UPDATE_EVENT");
		this._trace.exiting(this.id, "_publishEditProjectScopeCompleteEvent");
	},
	
	destroy : function() {
		// tags:
		//		protected
		this._trace.enteringWithStack(this.id, "destroy", true);
			
		try {
			dojo.unsubscribe("ProjectEditScopeTopologySwitchTableView");
			this.userPanel.destroyRecursive();
			this.permissionPanel.destroyRecursive();
			this.inherited(arguments);
			
			this._trace.exiting(this.id, "destroy");
		} catch(e) {
			this._trace.exception(e, "destroy");
			throw e;
		}
	},


	getTitle: function() {
		return /*this.roleItem.name + this.localResources.TITLE_NAMEVALUE_SEPARATOR +*/ this.localResources.TITLE;
	},

	loadPanels: function() {
		this.tabContainer.selectChild(this.usersTab);
	},


	performInitialFillingOfScopingInfo: function() {
		//This function pre-populate the users and permissions selection
		
		//this.userPanel.prepareUserSelectionForEdit(this._projectItem.userScoping);
		//this.permissionPanel.prepareUserSelectionForEdit(this._projectItem.permissionScoping);
		
		this.userPanel.prePopulateScopingRules(this._projectItem.userScoping);
		this.permissionPanel.prePopulateScopingRules(this._projectItem.permissionScoping);
	},
	
	_switchToTableView:function(){
		/*
		 * TODO 
		 * Need to check whether to have different function for users and permissions
		 * or we should switch here itself based on the current selected panel.
		 */
		//alert("Inside switchToTableView Edit Project Scope .js");
		if(this.tabContainer.selectedChildWidget.id=="userSelection"){
			this.userPanel._switchToTableView();
		}else {
			this.permissionPanel._switchToTableView();
		}
		
		//this.permissionsSelectionPanel._switchToTableView();
	}

});
