/* 
Licensed Materials - Property of IBM

 Copyright IBM Corp. 2009 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

/*
 * This ProjectUtil will have common methods to do specific functions
 * that are required at many places.
 */

dojo.provide("com.ibm.security.modeling.projects.ProjectUtil");

dojo.require("dojox.rpc.Rest");
dojo.require("com.ibm.security.modeling.api.HttpOverrideRestServiceFactory");
dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling", "PluginMessages");

dojo.declare("com.ibm.security.modeling.projects.ProjectUtil",[],
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.projects.ProjectUtil"),
	_errorUtil: null,
	wizardPane: null,
	pluginMessages: null,
	constructor: function (args) {
		this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
		this.pluginMessages = dojo.i18n.getLocalization("com.ibm.security.modeling", "PluginMessages");
	},
	
	destroy: function()	{
		
	},
	
	_setCurrentPanel: function(wizardPane){ 
		this.wizardPane = wizardPane;
		//alert("SET : " + wizardPane);

	},

	_getCurrentPanel: function(wizardPane){ 
		return this.wizardPane;
		alert("GET : " + wizardPane);
	},

	getUserScopingRule: function(scopingRuleItems){
		return this._getScopingRuleForPost("USER", scopingRuleItems);
	},
	
	getPermissionScopingRule: function(scopingRuleItems){
		return this._getScopingRuleForPost("PERMISSION", scopingRuleItems);
	},
	
	_getScopingRuleForPost: function(entity_type, scopingRuleItems){
		this._trace.finest('No of Scoping Items = ' + scopingRuleItems.length);
		var scopingRules = [];
		var retval = [];
		
		for(i=0; i<scopingRuleItems.length; i++){
			var itemname = scopingRuleItems[i].attrname;
			if(retval[itemname] == undefined){
				retval.push(itemname);
				retval[itemname] = new Object();
				retval[itemname].type = scopingRuleItems[i].type;
			}
			if((retval[itemname].values == undefined)||(retval[itemname].values == null)){
				retval[itemname].values = [];
			}
			
			if((scopingRuleItems[i].type == "STRING")||(scopingRuleItems[i].type == "INT")){
				retval[itemname].values.push(scopingRuleItems[i].value);
			}else if(scopingRuleItems[i].type == "HIERARCHY"){
				if(scopingRuleItems[i].identityCount == -1){
					// Include SUBTREES in the hierarchy
					retval[itemname].values.push(scopingRuleItems[i].value+"/SUBTREE");
				}else{
					// Do not Include SUBTREES in the hierarchy
					retval[itemname].values.push(scopingRuleItems[i].value +"/BASE");
				}
				
			}else if(scopingRuleItems[i].type == "IDENTITY"){
					/*
					 * Ideally, instead of constructing the uris, 
					 * should use the obtained uri of resource.
					 */
				if(entity_type == "USER"){
					retval[itemname].values.push("/users/" + scopingRuleItems[i].value);
				}
				else {
					retval[itemname].values.push("/permissions/" + scopingRuleItems[i].value);
				}	
			}else if(scopingRuleItems[i].type == "NAME"){
				if(entity_type == "USER"){
					retval[itemname].values.push("/users/" + scopingRuleItems[i].value);
				}else{
					retval[itemname].values.push("/permissions/" + scopingRuleItems[i].value);
				}
			}else{
				alert("Unexpected error occurred. Invalid item type found." + scopingRuleItems[i].type);
			}
		}
		
		
		for(i=0; i<retval.length; i++){
			var itemname = retval[i];
			var itemtype = retval[itemname].type;
			var scopingRuleItem = new Object();
			if(itemtype == "NAME"){
				if(entity_type == "USER"){
					scopingRuleItem.attributeURI = "https://localhost:16311/isrm/rest/users";
				}else{
					scopingRuleItem.attributeURI = "https://localhost:16311/isrm/rest/permissions";
				}
			}else{
				if(entity_type == "USER"){
					scopingRuleItem.attributeURI = "https://localhost:16311/isrm/rest/userscope/attributes/" + itemname;
				}
				else {
					scopingRuleItem.attributeURI = "https://localhost:16311/isrm/rest/permissionscope/attributes/" + itemname;
				}
			}
			scopingRuleItem.values = retval[itemname].values;
			scopingRules.push(scopingRuleItem);
		}
		
		//alert("Total no. of items  = " + retval.length);
		/*
		var scopingRuleItem = new Object();
		scopingRuleItem.attributeURI = "https://localhost:16311/isrm/rest/userscope/attributes/12345678";
		scopingRuleItem.attributeType = "STRING";
		scopingRuleItem.values = ["one","two","three"];
		scopingRules.push(scopingRuleItem);
		*/
		return scopingRules;
	},
	
	XgetScopingRuleForPost: function(scopingRuleItems){
		this._trace.finest('No of Scoping Items = ' + scopingRuleItems.length);
		var scopingRules = [];

		var scopingRule = "";
		var ruletype = "";
		for(i=0; i<scopingRuleItems.length; i++){
			
			var scopingRuleItem = new Object();
			scopingRuleItem.attrtype = scopingRuleItems[i].type;
			scopingRuleItem.attrid = scopingRuleItems[i].attrname;
			scopingRuleItem.attrvalue = scopingRuleItems[i].value;

			
			if(scopingRuleItems[i].type == "HIERARCHY"){
				if(scopingRuleItems[i].identityCount == -1){
					// Include SUBTREES in the hierarchy
					scopingRuleItem.subtree = true;
				}else{
					// Do not Include SUBTREES in the hierarchy
					scopingRuleItem.subtree = false;
				}
			}else{
				scopingRuleItem.subtree = false;
			}
			
			scopingRules.push(scopingRuleItem);
		}
		
		this._trace.finest('scopingRule = ' + scopingRule);
		return scopingRules;
	},

	
	_closeviewall: function(){
		if(this.viewusers != null){
			alert("Closing...." + this.viewusers);
			this.viewusers.destroy();
		}
	},
		
	/*
	 * The function will return the count of Users or Permissions
	 * that satisfy the scopingRule
	 * 
	 * type - USERS/PERMISSIONS.
	 * returns - int (no. of users or permissions that satisfy the rule)
	 */
	
	getSelectionCount: function(type, scopingRuleItems){
		this.type = type;
		var targeturl = "";
				
		if(this.type == "USERS"){
			targeturl = '/isrm/rest/users/?countOnly=true&queryFormat=AttributeFilter';
			this._tempScopingRule = this.getUserScopingRule(scopingRuleItems);
		}else{
			targeturl = '/isrm/rest/permissions/?countOnly=true&queryFormat=AttributeFilter';
			this._tempScopingRule = this.getPermissionScopingRule(scopingRuleItems);
		}
		var serviceFactory = new com.ibm.security.modeling.api.HttpOverrideRestServiceFactory();
		var extendedService = serviceFactory.createService(targeturl, this._tempScopingRule);
		//var extendedService = this.createARestServiceWithAPostAsAGet(targeturl, this._tempScopingRule);
		var restStoreCount = new com.ibm.security.modeling.tables.JsonRestStore({
			target:"targeturl",
			service: extendedService,
			schema:{
				"description":"A User",
				"type":"object",
				
				"properties": {
					"uri":{"type":"string","format":"url"},
			        "id":{"type":"string"},
			        "uiColumn1":{"type":"string"},
			        "uiColumn2":{"type":"string"},
			        "uiColumn3":{"type":"string"},
			        "uiColumn4":{"type":"string"},
			        "uiColumn5":{"type":"string"},
			        "uid":{"type":"string"}
				}
			}
		});	
		var xitems = restStoreCount.fetch({
			onComplete: dojo.hitch(this,this._getItemsForCount),
			onError:dojo.hitch(this,this._errorHandler),
			syncMode: true, 
			cache: "true"});
		
		return this.currentTotalSelectedUsers;
			
	},
	
	_getItemsForCount: function(item, request){
		this._trace.finest('currentTotalSelectedUsers = ' + item.totalCount, "_getItemsForCount");
		this.currentTotalSelectedUsers = item.totalCount;
	},
	
	
	positionCentre: function(widget){
		// summary:
		//		Position modal dialog in the viewport. If no relative offset
		//		in the viewport has been determined (by dragging, for instance),
		//		center the node. Otherwise, use the Dialog's stored relative offset,
		//		and position the node to top: left: values based on the viewport.
		// tags:
		//		private
		if(!dojo.hasClass(dojo.body(),"dojoMove")){
			var node = widget.domNode,
				viewport = dijit.getViewport(),
				//p = this._relativePosition,
				bb =  dojo._getBorderBox(node),
				l = Math.floor(viewport.l + ( (viewport.w - bb.w) / 2)),
				t = Math.floor(viewport.t)
			;
			dojo.style(node,{
				left: l + "px",
				top: t + "px"
			});
		}
	},
	
	
	getIdentitiesFromScopeInformation: function(type, scopingRuleItems){
		this.type = type;
		var targeturl = "";
				
		if(this.type == "USERS"){
			targeturl = '/isrm/rest/users/?countOnly=false&queryFormat=AttributeFilter';
			this._tempScopingRule = this.getUserScopingRule(scopingRuleItems);
		}else{
			targeturl = '/isrm/rest/permissions/?countOnly=false&queryFormat=AttributeFilter';
			this._tempScopingRule = this.getPermissionScopingRule(scopingRuleItems);
		}
		var extendedService = dojox.rpc.Rest(targeturl, true);
		extendedService.postData = dojo.toJson(this._tempScopingRule);
		extendedService._get = function(service,id, args){
			args = args || {};
			// this is called to actually do the get
			return index(dojo.xhrPost(service._getRequest(id, args)), service, (args.start >= 0 || args.count >= 0), id);
		};
		
			var restStoreCount = new com.ibm.security.modeling.tables.JsonRestStore({
									target:"targeturl",
									service: extendedService,
									schema:{
									"description":"A User",
									"type":"object",
									
									"properties": {
										"uri":{"type":"string","format":"url"},
								        "id":{"type":"string"},
								        "uiColumn1":{"type":"string"},
								        "uiColumn2":{"type":"string"},
								        "uiColumn3":{"type":"string"},
								        "uiColumn4":{"type":"string"},
								        "uiColumn5":{"type":"string"},
								        "uid":{"type":"string"}
									}
								}
								});	
			var xitems = restStoreCount.fetch({
							onComplete: dojo.hitch(this,this._getItemsFromScopeInformation),
							onError:dojo.hitch(this,this._errorHandler),
							syncMode: true, 
							cache: "true"});
			
			return this.currentTotalSelectedUsers;
			
	},
	
	_getItemsFromScopeInformation: function(item, request){
		this._trace.finest('currentTotalSelectedUsers = ' + item.totalCount, "_getItemsForCount");
		//this.currentTotalSelectedUsers = item.totalCount;
	},
	
	_countOnlyInfo : 0,
	_countErrorMessage:null,
	getCountOnlyInfo: function(targeturl){	
	
		dojo.xhrGet({
			url: targeturl,
			handleAs: "json",
			sync: true,
			load: dojo.hitch(this, function(item, ioArgs) {
				console.info(item.length);
				if (item != null){
					console.log(item.totalCount);
					this._countOnlyInfo = item.totalCount;
				}
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				this._errorUtil.handleRESTError(error, null);
				if(error.status && (error.status==404) || (error.status==0) ){
					this._countErrorMessage=this.pluginMessages.PLUGIN_SERVER_CONNECTION_FAILD;
				}else if(error.status && error.status==500){
					this._countErrorMessage=this.pluginMessages.PLUGIN_DB_CANNOT_BE_REACHED;
				}else {
					var responseText=dojo.fromJson(error.responseText);
					this._countErrorMessage=responseText.text;
				}
				
		         this._trace.error("Error while getting countOnly", "error callback");
			})			
		});
		
		return this._countOnlyInfo;	
	},
	
	getCountOnlyErrorMessage:function(){
	 	  return this._countErrorMessage;
	}
});


