/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

? Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.projects.ProjectsTablePanel");

dojo.require("dijit._Widget");
dojo.require("dojo.date.locale");
dojo.require("dojox.grid.EnhancedGrid");
dojo.require("dojox.grid.enhanced.plugins.DnD");
dojo.require("dojox.grid.enhanced.plugins.Menu");
dojo.require("dojox.grid.enhanced.plugins.NestedSorting");
dojo.require("dojox.grid.enhanced.plugins.IndirectSelection");
dojo.require("dojox.data.ClientFilter");// Import this before JsonRestStore. And do not import it, if cachingFetch of store should not take place.
dojo.require("com.ibm.security.modeling.tables.JsonRestStore");
dojo.require("com.ibm.security.modeling.api.ProjectRestManager");
dojo.require("dijit.Toolbar");
dojo.require("dijit.form.TextBox");
dojo.require("ibm.tivoli.tip.dijit.TIPTableSearchField");
dojo.require("ibm.tivoli.tip.dijit.TIPTableToolbar");
dojo.require("ibm.tivoli.tip.dijit.TIPButton");
dojo.require("ibm.tivoli.tip.dijit.TIPTableFooter");
dojo.require("com.ibm.security.modeling.tables.Table");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.widgets.ProgressBar");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");
dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
dojo.require("com.ibm.security.modeling.widgets.DeleteProjectDialog");
dojo.require("com.ibm.security.modeling.api.ProjectRestManager");
dojo.require("com.ibm.security.modeling.layouts.TemplatedPanelWithLayout");
dojo.require("com.ibm.security.modeling.tables.JsonBatchRest");
dojo.require("com.ibm.security.modeling.util.UrlUtilities");
dojo.require("dojox.rpc.JsonRest");

dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling.projects", "ProjectsTableWizardText");

dojo.declare("com.ibm.security.modeling.projects.ProjectsTablePanel",
            [ com.ibm.security.modeling.layouts.TemplatedPanelWithLayout ],
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.projects.ProjectsTablePanel"),
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.projects","templates/ProjectsTablePanel.html"),	
	widgetsInTemplate: true,
	// globalResources: Object
	//			Reference to the global shared resource bundle
	globalResources: null,
	// localResources: Object
	//			Reference to the resource bundle unique to this widget
	localResources: null,
	// namespace: [readonly] String
	//          Unique identifier to the instance of portlet page where this item will exist
	namespace: "",
	// defaultRowHeight: int
	//		the default height of each row in the projects table.  Important when calculating the viewable area in the 
	//      table (especially in IE).  A value of -1 means we should just use the default set in the datagrid
	defaultRowHeight: 32,	
	// progressBar: String
	//		The widget used to display the progress of various taks. For e.g. the progress of
	//      the current invocation of search.
	progressBar: null,
	// messageArea: Object
	//			Area for messages
	messageArea: null,
	// restStore: dojox.data.JsonRestStore
	//		summary: The datastore for the table 
    restStore: null,
    // projectsTable: com.ibm.security.modeling.tables.Table
    //      summary: The table to hold the list of projects
    _projectsTable: null,
    mainContainer: null,
	_logMessageResources:null,
	constructor: function (args) {
		this._trace.entering("", "constructor");
		this._trace.object(args, "constructor", "Constructor args");
		this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.projects", "ProjectsTableWizardText");
		this._logMessageResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "LogMessages");
		this._urlUtilities = com.ibm.security.modeling.util.UrlUtilities();
		this.progressBar = new com.ibm.security.modeling.widgets.ProgressBar();
		this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
		this.label = this.localResources.PROJECTS_TITLE;
		this.descriptionId = "projectsTableInstructions";
		this._trace.exiting("", "constructor");
		
	},

	_exportDialog : null,
	_exportInfoMessageDialog : null,
	postCreate:function(){
		this._trace.entering(this.id, "postCreate");
		this.inherited(arguments);
		// this.setupMenus();		
		dojo.addClass(this.domNode,"fullSize");		
		this._setupRestStore();
		this._setupTable();
		this.subscribe("PROJECTSUPDATED", dojo.hitch(this,this._updateProjectsTable));
		this._trace.exiting(this.id, "postCreate");
	},
	
	_updateProjectsTable:function(activity){
		this._projectsTable.refresh();
	},
	
	// Toolbar functions
	onCreateNew: function() {
		this._trace.entering(this.id, "onCreateNew");
		this.messageArea.hideMessages();
		//alert('Create New');
		this.checkProjectCreationAllowed();
		this._trace.exiting(this.id, "onCreateNew");
	},
	
	getSelected: function() {
		this._trace.entering(this.id, "getSelected");
		var selectedRows = this._projectsTable.grid.selection.getSelected();
		var selectedRowsCount = selectedRows.length;
		this._trace.finest(this.id+' - Number of selected rows: '+selectedRowsCount, "getSelected");
		
		var result = [];
		var rowIndex;
		for (rowIndex = 0; rowIndex < selectedRowsCount; rowIndex++) {
			result[rowIndex] = selectedRows[rowIndex].uri;
			this._trace.finest(this.id+' - Project URI: '+selectedRows[rowIndex].uri, "getSelected");
		}
		this._trace.exiting(this.id, "getSelected");
		return result;
	},
	
	onEditSelected: function() {
		// summary: 
		//		function that executues when the Edit button is pushed on the projects table.
		this._trace.entering(this.id, "onEditSelected");
		this.messageArea.hideMessages();
		var result = this._projectsTable.getSelectedItems();
		if (result === null) {
			return;
		}

		this.checkProjectState(result[0]);
		
		// Refreshing grid after Edit. This should result in displaying latest data in Grid.
		this._projectsTable.refresh();
		
		this._trace.finest(this.id+' - URI of selected project is: '+result[0], "onEditSelected");
		this._trace.exiting(this.id, "onEditSelected");
	},
	onDeleteSelected: function() {
		// summary: 
		//		function that executues when the delete button is pushed on the projects table.
		// description:
		//		Iterates through the selected projects and submits a delete request on each one.  
		//		At the end of the deletes we refresh the grid, refetching data from the server
		//		to reflect any pending state projects.
		this._trace.entering(this.id, "onDeleteSelected");
		var projectDeleteDialog = dijit.byId("deleteProjectDialog");		
		projectDeleteDialog.destroyRecursive();
		this.messageArea.hideMessages();
		var selectedRows = this._projectsTable.getSelectedItems();
		if (selectedRows === null || selectedRows.length <=0) {
			this._trace.exiting(this.id, "onDeleteSelected");
			return;
		}
		this.progressBar.show(this.localResources.PREPARING_FOR_DELETION);		
		//define variables scoped to this function that can be accessed from save callbacks.
		var successCount = 0;
		var errorCount = 0;
		var deletedSuccess = [];		
		var self = this;
		var trace = this._trace;
		this.erroralreadyhandled = false;
		//define save handler functions for success and failure
		var handleProjectSaveComplete = function() {
			if(self.progressBar !== null && self.progressBar !== undefined) {
				self.progressBar.hide();
			}
			//successCount= action.length;
			trace.finest("Last item delete submitted, refreshing...", "onDeleteSelected");
			self._projectsTable.refresh(); //Refresh table to show latest state of project evaluation
			if(	successCount > 0 ){//show a success message if at least one successful delete
				self.messageArea.showInfoMessage(dojo.string.substitute(self._logMessageResources.DELETE_PROJECTS_SUCCESS_COMPLETE, {"number_of_projects":successCount}));
			}
			
		};
		var handleProjectSaveError = function(error) {			
			this._trace.error("Error while deleting Project.", "onDeleteSelected");
			this._handleRESTError(error);
			
		};		
		
		
		var putMethod = this._projectsTable.dataStore.service["put"];
		this._projectsTable.dataStore.service["put"]=dojo.hitch(this,function(id,content){
			return this._projectsTable.dataStore.service["delete"](id,content);
		});	
		// Iterate through the selected items and delete
		//We are anyway performing refresh finally on completion of all the request 
		//so there is no need for individual revert 
		for (var j=0; j<selectedRows.length; j++) {
			//this._projectsTable.dataStore.deleteItem(selectedRows[j]);
			this._projectsTable.dataStore.changing(selectedRows[j]);
			
			this._projectsTable.dataStore.save({
				handleAsBatch: false,	
				revertOnError :true,
				onComplete: dojo.hitch(this, function(){successCount++;
					if(successCount+errorCount == selectedRows.length ) handleProjectSaveComplete();
				}),
				onError: dojo.hitch(this, function(error){
					errorCount++;
					dojo.hitch(this,handleProjectSaveError)(error);
					if(successCount+errorCount == selectedRows.length ) handleProjectSaveComplete();
					})
			});	
		}
	
		this._projectsTable.dataStore.service["put"] = putMethod;
		
		//reset selections after delete so counts are updated properly
		//FIX for 45824 , if any ways we are resetting preserver , then selection is getting reset , so why not reset selection
		this._projectsTable.resetAllSelection();
		this._trace.exiting(this.id, "onDeleteSelected");		
	},
	
	//var for Export helper
	exportHelper:null,
	
	onExport:function(){
		this._trace.entering(this.id, "onExport");
		this.messageArea.hideMessages();
		var exportProjects = this._projectsTable.getSelectedItems();
		if (exportProjects === null && exportProjects.length !== 1) {
			this._trace.error(this.id,"Export Error : No project or multiple project has been selected.");
			return;
		}
		
		if(this.exportHelper === null){
			this.exportHelper = this.createExportHelper();
		}
		
		this.exportHelper.exportProject(exportProjects[0].id);
		
		// Refreshing grid after Edit. This should result in displaying latest data in Grid.
		this._projectsTable.deselectAll();
		
		this._trace.exiting(this.id, "onExport");
	},
	
	onBuildSuccess: function(data){
		var submitted = 0;
		var failed = 0;
		if(data && data[1] && data[1].content){
			var items = data[1].content;
		
			for(var i=0;i<items.length;i++){
				if(items[i].statusCode == "202"){
					submitted++;
				}else if(items[i].statusCode == "503"){
					failed++;
				} 
			}
		}
		
		this.messageArea.showInfoMessage(
				dojo.string.substitute(this._logMessageResources.BUILD_PROJECTS_SUCCESS_COMPLETE, {"number_of_projects":submitted,"number_of_projects_failed":failed})
			);
		
		//TODO : dont know whether we need this refresh here on not . 
		this._projectsTable.refresh();
		
	},
	
	onBuildFail: function(error){
		if(!(error && error[1] && error[1].content)){
			this._errorUtil.handleRESTError(error, this.messageArea);
		}
	},
	
	onBuild: function(){
		var items = this._projectsTable.getSelectedItems();
		for(var i=0; i<items.length; i++){
                   this.restStore.changing(items[i]);
		}
		this.restStore.save({
			handleAsBatch:true,
			noContent:true,
			onComplete:dojo.hitch(this, this.onBuildSuccess),
		    onError:dojo.hitch(this, this.onBuildFail)
		});
		
	},
	
	/**
	 *  To create export helper object
	 */
	createExportHelper : function(){
		this._trace.entering(this.id, "createExportHelper");
		
		dojo.require("com.ibm.security.modeling.exporter.ExportHelper");
		var exportHelper = new com.ibm.security.modeling.exporter.ExportHelper({
					id : this.id,
					globalResources : this.globalResources,
					messageArea:this.messageArea
					},
				null);
		this._trace.exiting(this.id, "createExportHelper");
		return exportHelper;
	},
	_setupRestStore: function() {
		this._trace.entering(this.id, "_setupRestStore");
		this.restStore = new com.ibm.security.modeling.tables.JsonRestStore({
				target:'/isrm/rest/projects/', 
				idAttribute:"id"
		});
		var self = this;
		var getValue=this.restStore.getValue;
		//overriding the getValue for status attribute , so that search also occurs on the returned(displayed) value instead of 
		//value returned from REST .
		this.restStore.getValue = function(/*Object*/ item, /*String*/property, /*value?*/defaultValue){
		    var value = getValue(item,property,defaultValue);
		    if(property == 'status'){
		    	var cellContent = "";
		    	var statusKey = "PROJECT_STATUS_"+value;
				if(self.localResources[statusKey]){
					cellContent = self.localResources[statusKey];
				}else{
					cellContent = self.localResources.PROJECT_STATUS_U;
				}
				
				
				if(item.synchronizationStatus != "S"){
					var syncStatusKey = "PROJECT_SYNC_STATUS_"+item.synchronizationStatus;
					if(self.localResources[syncStatusKey]){
						syncStatusContent = self.localResources[syncStatusKey];
					}else{
						syncStatusContent = self.localResources.PROJECT_SYNC_STATUS_U;
			        }
		
					cellContent = cellContent + " (" + syncStatusContent + ")";
				}
				
				
				return cellContent ;
				
			}
            return value;
		};
		
//		this.restStore.isUpdateable = function(request) {
//			return false;
//		};
		
		this._trace.exiting(this.id, "_setupRestStore");
	},
	
	_setupTable: function() {
		this._trace.entering(this.id, "_setupTable");

		var projectStructure = [
			{field: 'name',name: this.localResources.HEADER_COLUMN_0,width: '15%',formatter: dojo.hitch(this, this._formatProjectNameColumn),get:function(rowIndex,item){return item;}},
			{field: 'description',name: this.localResources.HEADER_COLUMN_1,width: '40%',toolTipFormatter:function(value,rowIndex,cell){return  com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(value ); ;},
    	     	formatter:function(value,rowIndex,cell){return com.ibm.security.modeling.tables.FormatterUtils.formatLongCellValue(value);}},
			{field: 'status',name: this.localResources.HEADER_COLUMN_2,width: '25%', get:function(rowIndex,item){return item;},formatter: dojo.hitch(this, this.formatStatusValue)},
			{field: 'coveredRolesCount',name: this.localResources.HEADER_COLUMN_4,width: '10%', isSearchable: false},
			{field: 'coveredPoliciesCount',name: this.localResources.HEADER_COLUMN_5,width: '10%', isSearchable: false}
		];

		this._projectsTable = new com.ibm.security.modeling.tables.Table({
				id: this.id+"_projectsTable",
				dataStore: this.restStore, 
				structure: projectStructure,
				entityTypeText: this.globalResources.PROJECTS,
				autoHeight:false,
				isClientSide: true,
				defaultSortIndex: 1,
				toolTipRequired: [2,3],
				isDynamic: true,
				messageArea: this.messageArea,
				searchInstructions:this.localResources.SEARCH_INSTRUCTION,
				loadingMessage: this.localResources.MSG_DATA_LOADING,
				errorMessage: this.localResources.MSG_DATA_LOADING_ERROR
			},
			dojo.byId(this.id+"_projectsTableHook")
		);
		
		dojo.addClass(this._projectsTable.domNode,"tableArea");
		
		this._projectsTable.addToolbarSeparator();
		this._projectsTable.addButtonWithBuilder()
			.withInvisibleLabel(this.localResources.CREATE_PROJECT)
			.withIconClass("newIcon")
			.withOnClickFunction(dojo.hitch(this, this.onCreateNew))
			.withId(this._projectsTable.id+"_new")
			.withActivationForSingleSelect()
			.withActivationForMultiSelect()
			.withActivationForNoSelect().build();
		this._projectsTable.addButtonWithBuilder()
			.withInvisibleLabel(this.localResources.EDIT_PROJECT)
			.withIconClass("editIcon")
			.withOnClickFunction(dojo.hitch(this, this.onEditSelected))
			.withId(this._projectsTable.id+"_edit")
			.withActivationForSingleSelect().build();
		this._projectsTable.addButtonWithBuilder()
			.withInvisibleLabel(this.localResources.DELETE_PROJECTS)
			.withIconClass("deleteIcon")
			.withOnClickFunction(dojo.hitch(this, this._launchDeleteProjectDialog))
			.withId(this._projectsTable.id+"_delete")
			.withActivationForSingleSelect()
			.withActivationForMultiSelect().build();
		this._projectsTable.addToolbarSeparator();
		this._projectsTable.addButtonWithBuilder()
			.withInvisibleLabel(this.localResources.EXPORT_PROJECT)
			.withIconClass("exportIcon activeImage")
			.withOnClickFunction(dojo.hitch(this, this.onExport))
			.withId(this._projectsTable.id+"_export")
			.withActivationForSingleSelect().build();
		this._projectsTable.addButtonWithBuilder()
			.withInvisibleLabel(this.localResources.BUILD_PROJECT)
			.withIconClass("buildIcon activeImage")
			.withOnClickFunction(dojo.hitch(this, this.onBuild))
			.withId(this._projectsTable.id+"_build")
			.withActivationForSingleSelect()
			.withActivationForMultiSelect().build();
		this._trace.exiting(this.id, "_setupTable");			
	},
	
	_launchDeleteProjectDialog:function(){
	
		var selectedRows = this._projectsTable.getSelectedItems();
		var projectNames = [];
		var i;
		for ( i=0;i < selectedRows.length; i++){
			projectNames[i]= selectedRows[i].name;
		}
		var  projectDeleteDialog = new com.ibm.security.modeling.widgets.DeleteProjectDialog({
					projectNamesToDisplay:projectNames,
					globalResources: this.globalResources,
					localResources: this.localResources,
					messageArea: this.messageArea,
					deleteCallback: dojo.hitch(this, this.onDeleteSelected),
					id: "deleteProjectDialog"
				});
		projectDeleteDialog.startup();
		projectDeleteDialog.show();	
	},
	
	_getProjectDetailFromTable:function(inRowIndex,inItem){
		var projectDetail = "";
		if (inItem) {
			var uri = this.restStore.getValue(inItem, "uri");
			var modelId = (this.restStore.getValue(inItem, "models"))[0].id;
			var name = this.restStore.getValue(inItem, "name");
			projectDetail = dojo.toJson({"uri":uri, "modelId":modelId, "name":name});
		}
		return projectDetail;
		
	},
	
	_showProjectMessages : function(item){
		dojo.require("com.ibm.security.modeling.widgets.EntityStatusMsgDialog");
		this._messagesDialog = new com.ibm.security.modeling.widgets.EntityStatusMsgDialog({
			contextRoot:this.contextRoot,
			entityItem: item,
			helpFile:"ui_projects_status.html",
			entityType: "project"
		}, null);
		this._messagesDialog.show();
		this._messagesDialog.startup();
	},

	_formatProjectNameColumn:function(/*object*/item, /*int?*/rowIndex, /*dojox.grid.cells.Cell?*/cell){
		
		var nameValueString = null;
		var statusString = ""; // other column
		if (item) {
			if ((statusString === 'C') || (statusString === 'M')) {
				nameValueString = item.name;
			} else {
				nameValueString=new com.ibm.security.modeling.util.Anchor({
							input:item.name,
							callback:dojo.hitch(this, this.checkProjectState,item)
					});											
			}
		} else {
			nameValueString = "";
		}
		return nameValueString;
	},
	formatStatusValue: function(item, rowIndex, cell) { //TODO: See if this can be moved to REST layer
		//this._trace.entering(this.id, "formatStatusValue");
		if(item){
			var cellContent = this.restStore.getValue( item, "status");
			
			
			
			var projectItem = {
								"name": item.name,
								"status": cellContent,
								"messagesURI": item.messagesURI
							 };
			var anchor=new com.ibm.security.modeling.util.Anchor({
											input:cellContent,
											callback:dojo.hitch(this, this._showProjectMessages,projectItem)
								});	
			//this._trace.exiting(this.id, "formatStatusValue");
			return anchor;
		}
	},
	
	checkProjectState:function(item){
		this._trace.entering(this.id, "checkProjectState");
		var projectManger=new com.ibm.security.modeling.api.ProjectRestManager({messageArea:this.messageArea});
		projectManger.checkProjectEditState(item.statusURI,dojo.hitch(this,this.launchRolesHierarchyPanel,item),dojo.hitch(this, function(error, request){this._handleRESTError(error, request,false);}));
		this._trace.exiting(this.id, "checkProjectState");
	},
	
	launchProjectCreationWizard: function(){
		this._trace.entering(this.id, "launchProjectCreationWizard");
		dojo.publish(dojo.config.panels.PROJECT_CREATE.launchEventAddress);
		this._trace.exiting(this.id, "launchProjectCreationWizard");
	},
	
	checkProjectCreationAllowed:function(){
		this._trace.entering(this.id, "checkProjectState");
		var projectManger=new com.ibm.security.modeling.api.ProjectRestManager({messageArea:this.messageArea});
		projectManger.checkProjectCreateAllowed(dojo.hitch(this, this.launchProjectCreationWizard), dojo.hitch(this, function(error, request){this._handleRESTError(error, request,false);}));
		this._trace.exiting(this.id, "checkProjectState");
	},
	
	launchRolesHierarchyPanel:function(item){
		this._trace.entering(this.id, "launchRolesHierarchyPanel");
		if(item.models && item.models.length === 1){
			var modelId = this._urlUtilities.getIdfromUri(item.models[0].uri)[0];
			dojo.publish(dojo.config.panels.HIERARCHY.launchEventAddress,[item.uri,modelId,true]);
		}else{
			this._trace.error(this.id,"launchRolesHierarchyPanel Error : No model or multiple model is present for project.");
		}
		this._trace.exiting(this.id, "launchRolesHierarchyPanel");
	},
	_handleRESTError: function (error, request,isMsgDialogReq) {
		this._trace.entering(this.id, "_handleRESTError");
		this._trace.finest(this.id + " - Error: " + error, "_handleRESTError");
		if( (this.progressBar !== null) && (this.progressBar !== undefined) ) {
			this.progressBar.hide();
		}
		//Check added to show server connection error only once
		if(error.status == 404){
			if(!this.erroralreadyhandled){
				this._errorUtil.handleRESTError(error, this.messageArea,isMsgDialogReq);
				this.erroralreadyhandled = true;
			}
		} else {
			this._errorUtil.handleRESTError(error, this.messageArea,isMsgDialogReq);
		}
		this._trace.exiting(this.id, "_handleRESTError");
	}
});