/*
Licensed Materials - Property of IBM

Role and Policy Modeler

? Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.projects.UserScopingPanel");
dojo.require("com.ibm.security.modeling.widgets.AttributeScopingPanel");

// NLS
dojo.requireLocalization("com.ibm.security.modeling.projects", "UsersResourcesSelectionPanelText");

dojo.require("com.ibm.security.modeling.util.Trace");


dojo.declare("com.ibm.security.modeling.projects.UserScopingPanel",
            [ com.ibm.security.modeling.widgets.AttributeScopingPanel],
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.projects.UserScopingPanel"),
	// localResources: Object
	//		The resource bundle for this widget set
	localResources: null,
	// messageArea: ibm.tivoli.tip.dijit.TIPMessageArea
	//		Message area for error display
	//messageArea: null,
	// searchAttribute: String
	//			The context root of the server from which Content is being served.
	searchAttribute:"displayName",

	// viewAllUrl: String
	//			The uri which will fetch the ViewAll dialog box content and query for count
	viewAllUrl:"/isrm/rest/users/?queryFormat=AttributeFilter",
	// defaultValue: String
	//			Default value for the attribute dropdown
	defaultValue:"IDENTITY",
	// nameAttributeSelectionLimit: number
	//			Max Role Name Selection Limit (direct selection)
	nameAttributeSelectionLimit: 1000,
	// attrValueSelectionLimit: number
	//			Max Attribute Values Selection Limit 
	attrValueSelectionLimit: 15,
	// noResourceSelectionMsg: String
	//			The message key use to be show message if no roles are selected
	noResourceSelectionMsg:"MEMBERSHIP_FILTER_NO_VALUE_SELECTED",
	// selectionLimitMsg: String
	//			The message key use to be show message if roles/attribute value are above the limit.
	selectionLimitMsg:"USER_MAX_LIMIT_FOR_PROJECT_SCOPING_ATTRIBUTES",
	
	//selected item (json object) received from SearchDialog. 
	selectedItem: null,
	preSelectValues: null,
	helpFile: "ui_preview_user_select.html",
	learnMoreLinkFile: "ui_users_select_learn_more.html",
	//constants
	USERS_RESOURCE_URI: '/isrm/rest/users',

	postMixInProperties: function() {
		this._trace.entering(this.id, "postMixInProperties");
		
		this.inherited(arguments);
		this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.projects", "UsersResourcesSelectionPanelText");
		this.dropDownLabel=this.localResources.SELECT_USERS;
		
		this.idAttribute="storeId";
		this._trace.exiting(this.id, "postMixInProperties");
	},
	postCreate: function() {
		// tags:
		//		protected
				
		this._loadAttributeDropDown();
		this.inherited(arguments);
	},
	
	_loadAttributeDropDown:function(){
		dojo.require("com.ibm.security.modeling.api.AttributeScopeRestManager");
		if(!this.attributeScopeManager){
			this.attributeScopeManager = new com.ibm.security.modeling.api.AttributeScopeRestManager({
					scopeType: "user"
			});
		}

		var stateStore=this.attributeScopeManager.getJsonRestStore();
		var attrItems = null;
		var gotItems = function(items, request){

		//this._trace.finest('Number of attributes found: ' + items.length,"_getStoreFromUrl");
		  attrItems = items;
		};
		stateStore.fetch({onComplete: gotItems, onError:dojo.hitch(this,this._errorHandler),syncMode: "true", cache: "false"});


		var attrDataItems = [];
		if( (attrItems!= null) && (attrItems.length > 0) )
		{
			for(var i = 0; i < attrItems.length; i++){
				attrDataItems.push({displayName:attrItems[i].name,name:attrItems[i].name, id:attrItems[i].name, dbId:attrItems[i].id, type:attrItems[i].type,uri:attrItems[i].uri});
			}
		}
		
		attrDataItems.push({displayName:this.localResources.USER_NAME,name:"USERNAME",id:"USERNAME", dbId:"USERNAME", type:"NAME",uri:this.USERS_RESOURCE_URI});
		
		var attrData = {
		  identifier:"id",
		  label: "displayName",
		  items: attrDataItems
		};
		dojo.require("dojo.data.ItemFileReadStore");
		this.attributeStore = new dojo.data.ItemFileReadStore({
				data: attrData
		});
	},

	getHelpFileName: function(){
		return "ui_users_select.html";
	},
	
	/*
	* Override
	* Overriding this to handle INT type attributes.
	*/
	
	selectedValueCallback:function(value,item){
		
	     var widget;
	    if(item.type=="INT"){
	       widget= this.attachTable(value,item);
	    }else{
	        widget= this.inherited(arguments);
	    }
		return widget;
	},

	/*
	* Override
	* Get the Users table for the option "User Name" selection.
	*/
	
	_createNameAttributeTable: function(value,item){
		var fetch = {onError:dojo.hitch(this,this._errorHandler)};
		this.restStoreA = new com.ibm.security.modeling.tables.JsonRestStore({
						target:this.USERS_RESOURCE_URI, 
						fetchProperties:fetch,	
						schema:{
						"description":"A User",
						"type":"object",
						
						"properties": {
							"uri":{"type":"string","format":"url"},
					        "id":{"type":"string"},
					        "uiColumn1":{"type":"string"},
					        "uiColumn2":{"type":"string"},
					        "uiColumn3":{"type":"string"},
					        "uiColumn4":{"type":"string"},
					        "uiColumn5":{"type":"string"},
					        "uid":{"type":"string"}
				}}
			});
		var usersTable = new com.ibm.security.modeling.tables.UserTable(
				{
					id: "flattable_User_Name",
					dataStore: this.restStoreA, 
					isClientSide: false,
					isFilterRendered: true,
					isRefreshRendered: true,
					isMultiSelect: true,
					autoHeight: false,
					isDynamic: true,
					isSearchMultiSelect: false,
					entityTypeText: this.localResources.USER_NAME,
					messageArea: this.messageArea,
					tableType:"usersResourceSelection",
					searchInstructions:this.localResources.SEARCH_INSTRUCTION
				});
		usersTable.grid.attributeDetail=item;
		this._setupTableConnections(usersTable);
		return usersTable;
	},

	//Override
	_getValueForScopeRule: function(id){
			return "/users/" + id;
	},
	
	
	getViewAllTable:function(store,queryParams){
		dojo.require("com.ibm.security.modeling.tables.UserTable");
		var viewAllUsersTable = new com.ibm.security.modeling.tables.UserTable(
		{
				id: "flattable_ViewAllUsers",
				dataStore:store,
				isClientSide: false,
				isFilterRendered: true,
				isRefreshRendered: true,
				isSelectAllRendered: false,	
				autoHeight: false,
				isDynamic: true,
				needSelection: false,
				query:queryParams,
				isSearchMultiSelect: false,
				defaultSortIndex :0,
				tableType:"viewUsersTable",
				searchInstructions:this.localResources.SEARCH_INSTRUCTION
				//entityTypeText: item.name
		});
				
		
		return viewAllUsersTable;
		
	},

	startup: function(){
		this.inherited(arguments);
		
		//this.prePopulateValues(this.preSelectValues);
		//this._updateTotalCount();
	},
	
	destroy: function()	{
		this.inherited(arguments);
	},
	
	hideWorkingDialog: function(){
		//Nothing to do. Just because the AttributeScopingPanel need override this method.
	},
	
	_errorHandler:function(error,ioargs){
		this._trace.entering(this.id, "_handleRESTError");

		dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
		this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
		this._errorUtil.handleRESTError(error, this.messageArea);

		this._trace.exiting(this.id, "_handleRESTError");
			
	},
	
	validate: function(){
		var currentScope=this._getSelectedScope();
		if(this.isThreasholdLimitReached(currentScope) == true){
			this._showMessage("critical", this.messageResources.USER_MAX_LIMIT_FOR_PROJECT_SCOPING_ATTRIBUTES);
			return false;
		}
		return true;
	}
});
