
/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

 Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.samples.widgets.ProjectAndRolePicker");

dojo.require("ibm.tivoli.tip.dijit.TIPMessageDialog");
dojo.require("com.ibm.security.modeling.dijit.layout.BorderContainer");
dojo.require("com.ibm.security.modeling.dijit.layout.ContentPane");
dojo.require("dijit.Dialog");
dojo.require("dojo.parser");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("dijit.form.ComboBox");
dojo.require("com.ibm.security.modeling.api.ProjectRestManager");
dojo.requireLocalization("com.ibm.security.modeling.samples.widgets", "ProjectAndRolePickerText");

dojo.declare("com.ibm.security.modeling.samples.widgets.ProjectAndRolePicker",[ dijit._Widget, dijit._Templated ],{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.samples.widgets.ProjectAndRolePicker"),
	templatePath : dojo.moduleUrl("com.ibm.security.modeling.samples.widgets", "templates/ProjectAndRolePicker.html"),
	autofocus:"false",
	widgetsInTemplate : true,
	killOnHide:false,
	
	/**
	 * Overriden method to set property.
	 * @return
	 */
	postMixInProperties:function() {
	
		this._trace.entering(this.id, "postMixInProperties");
		
		try {			
			this.inherited(arguments);
			
			this.localResorce = dojo.i18n.getLocalization("com.ibm.security.modeling.samples.widgets", "ProjectAndRolePickerText");

			this.closeButton = this.localResorce.CLOSE;
			this.projectAndRole = this.localResorce.PROJECT_AND_ROLE;
			this.projectAndRolePicker = this.localResorce.PROJECT_AND_ROLE_PICKER;
			this.projects = this.localResorce.PROJECTS;
			this.roles = this.localResorce.ROLES;

		} catch (e) {
			this._trace.exception(e, "postMixInProperties");
			throw e;
		} finally {
			this._trace.exiting(this.id, "postMixInProperties");
		}
	},
	
	/**
	 * Overriden method.
	 * 
	 * @return
	 */
	postCreate: function() {
		this._trace.entering(this.id, "postCreate");
		
		this.inherited(arguments);
		this._hookUpButtons();
		
		this._trace.exiting(this.id, "postCreate");		
	},
	
	/**
	 * To connect button with events
	 * 
	 * @return
	 */
	_hookUpButtons:function() {
		this._trace.entering(this.id, "_hookUpButtons");
	
		var closeButton = dijit.byId("closeButton");
		this.connect(closeButton, "onClick", "closeDialog");
		
		this._trace.exiting(this.id, "_hookUpButtons");		
	},
	
	/**
	 * To hide the dialog box.
	 * 
	 * @return
	 */
	closeDialog : function (){
		this._trace.entering(this.id, "closeButtonPressed");
	
		try {
			if(this.closeTrigger) 
				dojo.publish(this.closeTrigger, [{"projectUri":this.projectUri, "modelId":this.modelId,"roleItem":this.roleIItem}]);			
			if(this.publishTrigger && this.modelId && this.roleIItem){
				setTimeout(dojo.hitch(this,this.publish),400);
							
			}
			this.projectRolePickerDialog.hide();
			if(this.killOnHide){
				setTimeout(dojo.hitch(this,this.hidePicker),400);
			}
		} catch(e) {
			this._trace.exception(e, "closeButtonPressed");
			throw e;
		} finally {
			this._trace.exiting(this.id, "closeButtonPressed");			
		}		
	},
	hidePicker:function(){
		//this.projectRolePickerDialog.hide();	
		this.projectRolePickerDialog.destroyRecursive();
		this.destroy();
	},
	publish:function(){
		dojo.publish(this.publishTrigger, [{"modelId":this.modelId,"roleIItem":this.roleIItem}]);
	},
	
	/**
	 * To show value in the dialog dynamically. 
	 * @return
	 */
	show: function(argument){
		this._trace.entering(this.id, "show");
		
	this.loadProjects();	
		this.projectRolePickerDialog.show();
	
		this._trace.exiting(this.id, "show");			
	},

	loadProjects : function(){
		var projectRestManager = new com.ibm.security.modeling.api.ProjectRestManager({
			resourceUri: "/isrm/rest/projects/",
			syncMode: true
		});

		var projectStore = projectRestManager.getJsonRestStore();
		
		var projectCombo = dijit.byId('projects');
		projectCombo.attr('store', projectStore);
		projectCombo.attr('labelAttr','name');
		projectStore.fetch(
				{
					query:{
						name:"*"
					}, 
					onComplete:function(items){
						if(items[0] && items[0].name){
							projectCombo.setValue(items[0].name);	
							projectCombo.item = items[0];
						}
					}
				}
			);
		var onChangeConnect = this.connect(projectCombo, "onChange", 
												function(projectName){
													console.log("Project selected : "+projectName);
													this.modelId = projectCombo.item.models[0].id;
													this.projectUri = projectCombo.item.uri;
													this.loadProjectRoles();
												}
											); 
		
	},
	
	loadProjectRoles : function(){
	dojo.require("com.ibm.security.modeling.api.RoleRestManager");
		this.roleRestManager = new com.ibm.security.modeling.api.RoleRestManager({
					resourceUri: "/isrm/rest/models/"+this.modelId+"/roles",
					syncMode: true
				});
		
		var roleStore = this.roleRestManager.getJsonRestStore();
	
		var roleCombo = dijit.byId('roles');
		roleCombo.attr('store', roleStore);
		roleCombo.attr('labelAttr','name');
		roleStore.fetch(
				{
					query:{
						name:"*"
					}, 
					onComplete:function(items){
						if(items[0] && items[0].name){
							//roleCombo.setValue(items[0].name);	
							//roleCombo.item = items[0];
						}
					}
				}
			);
		var onChangeConnect = this.connect(roleCombo, "onChange", 
												function(roleName) {
													this.roleIItem = roleCombo.item; 
												}
											); 
		
	}
});
