dojo.provide("com.ibm.security.modeling.tables.AttributeTable");

dojo.require("com.ibm.security.modeling.tables.Table");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.widgets.PropertiesDialog");

dojo.declare("com.ibm.security.modeling.tables.AttributeTable", com.ibm.security.modeling.tables.Table, {
	
	hyperLinkIdentities:true,
	postCreate : function() {
		this.inherited(arguments);
		dojo.addClass(this.domNode, "contentArea");

	},
	postMixInProperties : function() {

		this.structure = this.getStructure();
		this.dataStore = this.getStore();
		this.toolTipRequired = [ 'attributeName', 2, 3 ];
		this.isClientSide = true;
		this.isFilterRendered = false;
		this.isRefreshRendered = false;
		this.isMultiSelect = true;
		this.isSelectAllRendered = false;
		this.isDynamic = true;
		this.needSelection = this.needToolBar;
		this.autoHeight = false;

		this.inherited(arguments);
		//this.isPaginationEnabled=false;
		//this.isTotalsFooterRendered=true;
		this.stringUtil = new com.ibm.security.modeling.util.StringUtils();

	},
	
	userSelected : function(item) {
		
		this.permissions = new com.ibm.security.modeling.widgets.PropertiesDialog( {
			type : "USER",
			entityItem : item.identityURI,
			panelHeaderName : item.name

		}, null);
		this.permissions.show();
		this.permissions.startup();

	},
	i18Attributes : function(item) {
		var value;  
        var anchor;
	    if(item){
		 	if ( item.attributeName && item.attributeName.toUpperCase() in this.attributeResource) {
				value= this.attributeResource[item.attributeName.toUpperCase()];
			}else  {
            	 value=item.attributeName;	                 
		    }         
	        
	        if(!this.hyperLinkIdentities)
	             return this.stringUtil.parseHTMLTag(value);
	         else{      
	              anchor = new com.ibm.security.modeling.util.Anchor( {
			   	  input : value,
				  callback : dojo.hitch(this.parentWidget,this.parentWidget.onEditSelected,item)
		           });
		           return anchor;
	           }
          }
	},
	_formatName : function(item) {

		var anchor = null;

		if(item){
			anchor = this.store.getValue(item, "value");

			if (item.type == 'IDENTITY' && this.hyperLinkIdentities) {
				anchor = new com.ibm.security.modeling.util.Anchor( {
					input : anchor,
					callback : dojo.hitch(this, this.userSelected, item)
				});		
			}else{
				anchor = this.stringUtil.parseHTMLTag(anchor);
			}
		}
		
		return anchor;

	},
	
	_setUpStore : function(storeUri) {

		this._trace.info("AttributeUri Uri is " + storeUri || this.storeUri, "_setUpStore");
		this.store = new com.ibm.security.modeling.tables.JsonRestStore( {
			target : storeUri || this.storeUri,
			idAttribute : "uri"
		});
		var self = this;
		var getValue = this.store.getValue;
		this.store.getValue = function(/* Object */item, /* String */property, /* value? */defaultValue) {
			var value = getValue(item, property, defaultValue);
			if (property == 'value') {
				if (item.type == "HIERARCHY" || item.type == "IDENTITY")  return item.name;
			}
			return value;
		}
		var setValue = this.store.setValue;
		this.store.setValue = function(item, attribute, value) {
			if (attribute == 'value') {
				if (item.type == "HIERARCHY" || item.type == "IDENTITY") {
					attribute = 'name';
				}
			}
			setValue(item, attribute, value);
		}
		
		return this.store;

	},

	getStructure : function() {
	
		this._trace.entering("", "_setUpTable");
		//getting Table Structure
		var attributeTableStructure = [ {
			field : 'attributeName',
			name : this.localResources.HEADER_COLUMN_0,
			width : '50%',
			get : function(rowIndex, item) {
				return item;
			},
			formatter : dojo.hitch(this, this.i18Attributes)
		}, {
			field : 'value',
			name : this.localResources.HEADER_COLUMN_1,
			width : '50%',
			get : function(rowIndex, item) {
				return item
			},
			formatter : dojo.hitch(this, this._formatName)
		}
	
		];
		return attributeTableStructure;
	
	},
	saveOnStore : function() {
		this.store.save( {
			handleAsBatch : true,
			onComplete : dojo.hitch(this, function(args) {
				this.parentWidget.hideSaveInProgress();
				/*if(this._isRefreshCallReq==true){
					this._isRefreshCallReq=false;
					this.refresh();
				}*/
				
				var actions = args[0];
				var value = args[1];
	
			}),
	
			onError : dojo.hitch(this, function(args) {
				this._trace.error("Error while removing directly attached permissions.", "error callback");
				this.parentWidget.hideSaveInProgress();
				if (typeof args == Array) {
					//TODO : currently not confident what to do 
					//handleComplete(args[0], args[1], true);
				} else {
					
					this._errorUtil.handleRESTError(args, this.messageArea);
	
				}
			})
	
		});
	},
	getStore : function() {
		return this.store||(this._setUpStore());
	}

});