dojo.provide("com.ibm.security.modeling.tables.CommonTableSearchField");

dojo.require("ibm.tivoli.tip.dijit.TIPTableSearchField");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.requireLocalization("com.ibm.security.modeling.tables", "TableText");

dojo.declare("com.ibm.security.modeling.tables.CommonTableSearchField", [ibm.tivoli.tip.dijit.TIPTableSearchField], {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.tables.Table"),
	selectedCheckBox: null,
	isMultiSelect: true,
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.tables.","templates/CommonTableSearchField.html"),
	searchInstructions: "",
	_searchEdited: false,
	_searchFieldFocused: false,
	constructor: function() {
		this.inherited(arguments);
		this.tableResources = dojo.i18n.getLocalization("com.ibm.security.modeling.tables", "TableText");
	},
	postMixInProperties:function(){
		//overriding tip table search field , all columns text 
		this.resources_.ALL_COLUMNS = this.tableResources.SEARCH_ALL_SEARCHABLE_COLUMNS;
	},
	startup: function() {
		this.inherited(arguments);
		//Set default search instructions
		dojo.attr(this.searchField, "value", this.searchInstructions);
		//dojo.attr(this.searchField, "maxlength", "50"); this state doesnt work for IE hence using maxLength
		this.searchField.setAttribute("maxLength", "50");
		if(this.table){
			var tooltipDialog =this.childrenWidgets[this.table.id + "gf_tooltipdialog"];
			if(tooltipDialog){
				this.connect(tooltipDialog,"onHide",dojo.hitch(this,this.selectIfAllColIfNoCheckBox));
			}
		}
	},
	setTable: function( table , widgetDef ) {
		this.inherited(arguments);
		if (!this.tooltip_dropdownhelp._started) {
			this.tooltip_dropdownhelp.startup();
		}
		//Replace "Filter On" with "Search By"
		var nodeList=dojo.query(".qf-col-filter-title");
		var i;
		for(i=0,len=nodeList.length;i<len;i++){//using for loop as a page may have multiple tables hence nodelist will have more than one item
			nodeList[i].innerHTML = this.tableResources.SEARCH_BY;
		}
		this._setCheckBoxEvents(table, widgetDef);
		
		if (this.childrenWidgets) {
			var dialogId = table.id + "gf_tooltipdialog";
			var tooltipDialog = this.childrenWidgets[dialogId];
			if (tooltipDialog) {
				dojo.query(".qf-col-chkbox-label-row", tooltipDialog.domNode).forEach(function(row){
					var checkBoxNode = dojo.query(".dijitCheckBox", row) [0];
					var labelNode = dojo.query(".qf-col-chkbox-label", row) [0];
					labelNode.setAttribute("for", checkBoxNode.getAttribute("widgetid"));
				});
			}
		}
	},
	_onFocus: function(){
		this.domNode.className="common-table-quick-filter-focus"; 
	},	  
	_onBlur: function(){
		this.domNode.className="common-table-quick-filter";
	},
	
	_onSearchFieldFocus: function() {
		this._searchFieldFocused = true;
		//if there is no currently set filter (the search instruction are still set), then remove the instructions on focus 
		if (!this.isSearchSet()) {
			this._hideSearchValuePlaceHolder();
		}
	},
	_onSearchFieldBlur: function() {
		this._searchFieldFocused = false;
		this.updateSearchEdited();
		//if there is no currently set filter (the search field is empty), then set the default instructions
		if (!this.isSearchSet()) {
			this._showSearchValuePlaceHolder();
		} 
	},
	isSearchFieldFocused: function() {
		return this._searchFieldFocused;
	},
	updateSearchEdited: function() {
		if (dojo.attr(this.searchField, "value") === "") {
			this._searchEdited = false;
		} else {
			this._searchEdited = true;
		}
	},
	_onKeyPress: function(event) {
		if (event && ("charOrCode" in event)) {
			var key = event["charOrCode"];
			if (key && (key == dojo.keys.ENTER)) {
				this.updateSearchEdited();
		  	}
		}
		
		this.inherited(arguments);
	},
	
	getSearchFieldValue: function() {
		var value = "";
		if (this.isSearchSet() || 
				(this.searchField.value != this.searchInstructions)) {
			value = this.searchField.value;
		}
		return value;
	},
	
	isSearchSet: function() {
		return this._searchEdited;
	},
	_setCheckBoxEvents: function(table, widgetDef){
		// Assuming we are not going to get the widgetDef ever. 
		/*
		 * We are going to set the events for checkbox and will subscribe to it too.
		 * The checkboxs were created by the TIPTableSearchField, and added in childrenWidgets
		 * with the id = table.id + columnname. We will fetch the checkboxes using
		 * the same id.
		 */

		this._trace.entering("", "_setCheckBoxEvents");
		var self=this;
		var onCheckedPublish = function(){ 
	  		if ( this.checked )
	  		{
				if(!table.isSearchMultiSelect){
					dojo.publish( this.pId + "_selectOnlyOneItem", [this] );
				}
	  		}else{
				self.selectIfAllColIfNoCheckBox();
			}
		};
		
		/* 
		 * Iterating on table structure to get column names.
		 * Current implementation of TIPTableSearchField adds the checkboxes into childrenWidgets.
		 * So if implementation gets changes and we do not find the checkbox, we need to fix our
		 * implementation here. 
		 */
		 var i;
		for(i=0;i<table.structure.length;i++){
			try{
				var checkbox = this.childrenWidgets[table.id+table.structure[i].name];
				if(checkbox!=undefined){
					checkbox._onCheckedPublish = onCheckedPublish;
					dojo.connect(checkbox, "onCheckedEvent", checkbox, "_onCheckedPublish");
				}else{
					this._trace.info("Checkbox not found. Skipping..." + table.id+table.structure[i].name);
				}

			}catch(e){
				this._trace.error("Error in handling checkbox for search options.", "_setCheckBoxEvents");
			}
		}

		if(!this.isMultiSelect){
			this.handle = dojo.subscribe( table.id + "_selectOnlyOneItem", this, "_unCheckPreviousCheckbox" );
		}
		this._trace.exiting("" + "_setCheckBoxEvents");
	},
	_unCheckPreviousCheckbox: function(checkbox){
		if((this.selectedcheckbox != null) && (this.selectedcheckbox != checkbox) ){
			  try{
				  this.selectedcheckbox.attr("checked", false);
			  }catch(e){
				  console.log("Unable to uncheck the previous selectedcheckbox." + e);
			  }
		  }
		  this.selectedcheckbox = checkbox;
	},
	
	onFilterOK: function () {
		//suppress search action on ok click on filter dropdown
		this.selectIfAllColIfNoCheckBox();
	},
	selectIfAllColIfNoCheckBox:function(){
		var columns = this.getSelectedColumns();
		if(columns.length === 0){
			var allColCheckBox=this.childrenWidgets[this.table.id + this._ALL_COLS_NAME];
			if(allColCheckBox){
				allColCheckBox.attr("checked", true);
			}
		}
	},
	clearSearchValue:function(){
		this._showSearchValuePlaceHolder();
		this._searchEdited = false;
	},
	setSearchValue:function(value){
		this._hideSearchValuePlaceHolder();
		dojo.attr(this.searchField, "value", value);
		this._searchEdited = true;
	},
	_showSearchValuePlaceHolder: function() {
		dojo.attr(this.searchField, "class", "qf-textfield-label-instructions");
		dojo.attr(this.searchField, "value", this.searchInstructions);
	},
	_hideSearchValuePlaceHolder: function() {
		dojo.attr(this.searchField, "class", "qf-textfield-label");
		dojo.attr(this.searchField, "value", "");
	},
	selectColumnCheckBox:function(field){
		var checkBox=this._getFilterCheckBoxForColumn(field);
		if(checkBox){
			checkBox.attr("checked", true);
		}
	},
	_getFilterCheckBoxForColumn:function(field){
		var id;
		var checkBox;
		for( id in this.childrenWidgets){
			var widget=this.childrenWidgets[id];
			if(widget && widget.columnID && widget.columnID === field){
				checkBox=widget;
			}
		}
		return checkBox;
	},
	destroy: function(){
		if(!this.isMultiSelect){
			if(this.handle){
				dojo.unsubscribe(this.handle);
			}
		}
		this.inherited( "destroy", arguments );
	},
    setDisable:function(){
		this.searchField.setAttribute("disabled",true);
	},
	setEnable:function(){
		this.searchField.removeAttribute("disabled");
		
	}
});