/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

 Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.tables.CustomDisplayStructureFactory");

dojo.require("com.ibm.security.modeling.api.SchemaRestManager");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");
dojo.require("dojox.collections.ArrayList");


(function() {

	// userSchema: Object
	//     Object that maps uiColumnX to attribute name for users
	var userSchema;
	// userAttributeAnalysisSchema: Object
	//     Object that maps mininig attribute name for users analysis
	var userAttributeAnalysisSchema;
	// permissionSchema: Object
	//     Object that maps uiColumnX to attribute name for permissions
	var permissionSchema;

	// 'var' should be used to declare local variables.
	var trace= new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.tables.CustomDisplayStructureFactory");
	
	dojo["requireLocalization"]("customizations", "IdentityCustomText");
	dojo["requireLocalization"]("customizations", "PermissionCustomText");
	
	

	var _getWidthAsNumber= function(/*string*/width){
		// summary:
		//    Private method, returns width of custom column structure as a number spliting %
		// width  : string
			var numWidth=0;
			if(dojo.isString(width)){
				numWidth=width.split("%");
				numWidth=numWidth[0];
			}
			return numWidth;
		};
	var _correctCustomColumnIndexes=function(customColumnIndexes,totalColumnsCount,schemaCount,customColumns){
		// summary:
		//    Private method, returns array of indexes correcting. If indexes provided are higher than the total count then it is converted to last and so on
		// customColumnIndexes  : array
		// contains the list of columns provided by table method. 
		// totalColumnsCount  : number
		// total columns count i.e. custom columns + schema columns
		// schemaCount  : number
		// custom attribute schema count
		// customColumns  : object
		// additional columns provided by table
			var len=customColumnIndexes.length;
			var j=-(len);
			for(var i=0;i<len;i++){
				if( customColumnIndexes[i] > schemaCount ||customColumnIndexes[i] >= totalColumnsCount){
					trace.info("Got a column which higher than total", "_correctCustomColumnIndexes");
					customColumns[totalColumnsCount+j]=customColumns[customColumnIndexes[i]];
				}
				j++;
			}
			return customColumns;
		};
	var _isCustomColumnsPresentAt=function(index,customColumnIndexes){
		// summary:
		//    Private method, returns array boolean if a custom column if present at given index
		// index  : number
		// index to be checked. 
		// customColumnIndexes  : array
		// total indexes of custom columns
			for(var i=0;i<customColumnIndexes.length;i++){
				if(customColumnIndexes[i]==index){
					return true;
				}
			}
			return false;
		};
	var _getSchemaColumnsList=function(schemaObject,availWidth,labelResource,nonSearchableCols){
		// summary:
		//    Private method, returns dojox.collections.ArrayList containing schema columns with available width equally divided
		// schemaObject  : object
		// Object contains schema. 
		// availWidth  : number
		// available width excluding the custom column if any
		var templabels = new Object();
			var schemaAttrib=new dojox.collections.ArrayList();
			var schemaColumnsList=new dojox.collections.ArrayList();
			var i=0;
			for(var attrib in schemaObject){
				if(attrib !="entity" && attrib !="uri" && attrib !="__id" ){
					schemaAttrib.insert(i,schemaObject[attrib]);
					i++;
				}
			}
			var schemaAttribCount=schemaAttrib.count;
			var perColWidth=Math.round(availWidth/schemaAttribCount);
			for(var j=0;j<schemaAttribCount;j++){
				var tempName=labelResource[schemaAttrib.item(j)];
				if(tempName=="" || tempName==undefined || tempName==null){
					tempName=schemaAttrib.item(j);
				}
				
				// If the label for the column name is already used by some other column,
				// we shall prefix this label with (1), (2) and so on.
				if(templabels[tempName]){
					tempName = tempName + " (" + templabels[tempName] +")";
					templabels[tempName] = templabels[tempName] + 1;
				}else{
					templabels[tempName] = 1;
				}
				var searchable=true;
				if(nonSearchableCols){
					var k;
					for(k=0; k < nonSearchableCols.length; k++){
						if(nonSearchableCols[k] == schemaAttrib.item(j)){
							searchable=false;
							break;
						}
					}
				}
				var colObj={field:schemaAttrib.item(j),name:tempName,width:(perColWidth+"%"),isSearchable:searchable};
				schemaColumnsList.insert(j,colObj);
			}
			trace.object(schemaColumnsList.toArray(), "_getSchemaColumnsList");
			return schemaColumnsList;
		};
		
	var _getSingleColumnAt=function(/*Object*/customColumns,/*number*/index){
		// summary:
		//    Private method, returns object of column structure at given index
		// customColumns  : object
		// Object contains custom column. 
		// index  : number
		// index for which column is to be fixed
			return customColumns[index]; 
			
		};
	var _getCustomColumnIndexes=function(/*Object*/customColumns){
		// summary:
		//    Private method, returns array of indexes for custom columns
		// customColumns  : object
		// Object contains custom column. 
			var indexes=[];
			for(var colmnIndex in customColumns){
				indexes.push(colmnIndex);
			}
			return indexes;
		};
	var _getAvailableWidth=function(/*Object*/customColumns,/*number*/availWidth){
		// summary:
		//    Private method, returns number for available width for custom columns
		// customColumns  : object
		// Object contains custom column. 
			for(var a in customColumns){
				var col=customColumns[a];
				if(col && dojo.isObject(col)){
					if(col.width){
						availWidth=availWidth - _getWidthAsNumber(col.width);
					}
				}else{
					return availWidth;
				}
			}
			return availWidth;
		};

	com.ibm.security.modeling.tables.CustomDisplayStructureFactory = {
		// summary:
		//     Gets the table structure for tables with custom display columns
		// description: 
		//     This class should be instantiated once and injected as a dependency to its callers.  This is because
		//     the schema is cached so we don't have to fetch it from the server every time we create a new table.

		
		availEntityTypes:{
			USERS: "users",
			USER_ATTRIBUTE_ANALYSIS:"userAttributeAnalysis",
			PERMISSIONS: "permissions"
		},

		/*************** Users Section*****************/
		loadUserStructure: function(/*Object*/customColumns,/*function*/errorCallback) {
			// summary:
			//    builds and returns a structure object for an enhanced grid showing users.
			// customColumns: Object
			//    An object defining custom columns in the form of -
			//        {
			//             <column index(0-based)>:{<column definition>}
			//        }
			//    For example - 
			//        {
			//             2: {field:"source",name:this.localResources.PERMISSION_SOURCE, formatter: this.permSourceFormatter, width: "15%"}
			//        }

				userSchema = this.getUserSchema(errorCallback);//Use the REST interface for fetching schema
				var identityLabelResources = dojo.i18n.getLocalization("customizations", "IdentityCustomText");
				return this.buildStructure(customColumns,userSchema,identityLabelResources);
			
		},
		fetchUserSchema: function(errorCallBack){
			var schemaRestMgr= new com.ibm.security.modeling.api.SchemaRestManager({entityType:this.availEntityTypes.USERS,syncMode:true});
			return schemaRestMgr.loadResource("",null,errorCallBack);
		},
		getUserSchema: function(/*function*/errorCallback) {
			if (userSchema == null) {
				trace.info("No schema found", "getUserSchema");
				//fetch and add the structure to the object.  don't replace the object or you lose its "staticness"
				userSchema = this.fetchUserSchema(errorCallback);//Use the REST interface for fetching schema
			}
			return userSchema;
		},
		/*************** User Analysis Section*****************/
		loadUserAttributeAnalysisStructure:function(/*Object*/customColumns,/*function*/errorCallback){
				userAttributeAnalysisSchema = this.getUserAttributeAnalysisSchema(errorCallback);//Use the REST interface for fetching schema
				var identityLabelResources = dojo.i18n.getLocalization("customizations", "IdentityCustomText");
				var nonSearchableCols=this._getNonSearchableAnalysisCols(userAttributeAnalysisSchema);
				return this.buildStructure(customColumns,userAttributeAnalysisSchema,identityLabelResources,nonSearchableCols);
		},
		fetchUserAttributeAnalysisSchema: function(errorCallBack){
			var schemaRestMgr= new com.ibm.security.modeling.api.SchemaRestManager({entityType:this.availEntityTypes.USER_ATTRIBUTE_ANALYSIS,syncMode:true});
			return schemaRestMgr.loadResource("",null,errorCallBack);
		},
		getUserAttributeAnalysisSchema: function(/*function*/errorCallback) {
			if (userAttributeAnalysisSchema == null) {
				trace.info("No schema found", "getUserAttributeAnalysisSchema");
				//fetch and add the structure to the object.  don't replace the object or you lose its "staticness"
				userAttributeAnalysisSchema = this.fetchUserAttributeAnalysisSchema(errorCallback);//Use the REST interface for fetching schema
				userAttributeAnalysisSchema = this.addPerencentColumnAnalysisSchema(userAttributeAnalysisSchema);
			}
			return userAttributeAnalysisSchema;
		},
		addPerencentColumnAnalysisSchema:function(schema){
			var newSchema={};
			var attr;
			for(attr in schema){
				if(attr == "entity" || attr == "uri" || attr == "name" || attr =="__id"){
					newSchema[attr]=schema[attr];
				}else{
					newSchema[attr]=schema[attr];
					newSchema[attr+"PercentValue"]=schema[attr]+"InPercent";
				}
			}
			return newSchema;
		},
		_getNonSearchableAnalysisCols:function(schema){
			var attr;
			var nonSearchableCols=[];
			for(attr in schema){
				var tempString=new String(schema[attr]+"InPercent");
				if(tempString.search(/InPercent/i) !== -1){
					nonSearchableCols.push(tempString);
				}
			}
			return nonSearchableCols;
		},

		/*************** Permissions Section*****************/
		 loadPermissionStructure: function(/*Object*/customColumns,/*function*/errorCallback) {
			// summary:
			//    builds and returns a structure object for an enhanced grid showing permissions.
			// customColumns: Object
			//    An object defining custom columns in the form of -
			//        {
			//             <column index(0-based)>:{<column definition>}
			//        }
			//    For example - 
			//        {
			//             2: {field:"source",name:this.localResources.PERMISSION_SOURCE, formatter: this.permSourceFormatter, width: "15%"}
			//        }

			permissionSchema=this.getPermissionSchema(errorCallback);//Use the REST interface for fetching schema
			var permissionLabelResources = dojo.i18n.getLocalization("customizations", "PermissionCustomText");
			return this.buildStructure(customColumns,permissionSchema,permissionLabelResources);
		},
		 fetchPermissionSchema:function(/*function*/errorCallBack){
			var schemaRestMgr= new com.ibm.security.modeling.api.SchemaRestManager({entityType:this.availEntityTypes.PERMISSIONS,syncMode:true});
			return schemaRestMgr.loadResource("",null,errorCallBack);

		},
		getPermissionSchema: function(/*function*/errorCallback) {
			if (permissionSchema == null) {
				trace.info("No schema found", "getPermissionSchema");
				//fetch and add the structure to the object.  don't replace the object or you lose its "staticness"
				permissionSchema = this.fetchPermissionSchema(errorCallback);//Use the REST interface for fetching schema
			}
			return permissionSchema;
		},
		buildStructure: function (/*Object*/customColumns,/*Object*/entitySchema,/*Object*/labelResource,/*array*/nonSearchableCols) {
			// summary:
			//     Creates a table structure object based on the user schema and provided custom columns
			// returns:
			//     A table structure object for use in the enhanced grid
			
			 var tableStructure=new dojox.collections.ArrayList();
			 this.htmlUtilities=new com.ibm.security.modeling.util.StringUtils();

			var availWidth=100;
			var customColumnIndexes=[];
			
			if(customColumns && dojo.isObject(customColumns)){
				customColumnIndexes=_getCustomColumnIndexes(customColumns);
				customColumnIndexes.sort();
				availWidth=_getAvailableWidth(customColumns,availWidth);
				trace.info("Available width for schema columns "+availWidth, "buildStructure");
			}
			
			var schemaColumnsList=_getSchemaColumnsList(entitySchema,availWidth,labelResource,nonSearchableCols);
			var totalColumnsCount=schemaColumnsList.count+customColumnIndexes.length;
			if(customColumns && dojo.isObject(customColumns)){
				customColumns=_correctCustomColumnIndexes(customColumnIndexes,totalColumnsCount,schemaColumnsList.count,customColumns);
				customColumnIndexes=_getCustomColumnIndexes(customColumns);
			}
			var j=0;
			var k=0;
			for(var i=0;i<totalColumnsCount;i++){
				var column=null;
				
				if(_isCustomColumnsPresentAt(i,customColumnIndexes)){
					column=_getSingleColumnAt(customColumns,i);
				}else{
					column=schemaColumnsList.item(j);
					j++;
				}
				column.name = this.htmlUtilities.parseHTMLandNormalize(column.name);
				tableStructure.insert(i,column);
			}
			return(tableStructure.toArray());
		}
	};


 //userSchema = com.ibm.security.modeling.tables.CustomDisplayStructureFactory.fetchUserSchema();
 //permissionSchema = com.ibm.security.modeling.tables.CustomDisplayStructureFactory.fetchPermissionSchema();
 
})();