dojo.provide("com.ibm.security.modeling.tables.DistributionAnalysisGrid");

dojo.require("dojox.grid.EnhancedGrid");
dojo.require("dojo.data.ItemFileReadStore");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");
dojo.requireLocalization("com.ibm.security.modeling.widgets", "ChartWidgetText");

dojo.declare("com.ibm.security.modeling.tables.DistributionAnalysisGrid", dojox.grid.EnhancedGrid, {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.tables.DistributionAnalysisGrid"),
	// attributeDistribution: [readOnly] Object
	//		attribute data from server in json form.
	attributeDistribution: null,
	translatedText: null,
	constructor: function(args) {
		// Summary:
		//		Shows distribution analysis statistics and a grid view.
		// isDrillDownCapable: boolean
		//		Whether this grid should support drilling down into data values via hyperlink
		// chartText: Object
		//		resouce bundle for translated chart text.
		this._trace.entering("", "constructor");
		this.id = args.id;
		this.autoHeight = true;
		this.rowSelector = "0px";
		this.store = this.createStore(args.attributeDistribution);
		var attributeText = dojo.i18n.getLocalization("com.ibm.security.modeling.notebook", "AttributePanelText");
		this.structure = [		                  
		                  {field:"name", name:attributeText.HEADER_COLUMN_1, width:"50%",toolTipFormatter:function(value,rowIndex,cell){
		  		        		return com.ibm.security.modeling.util.HtmlEscapingUtils.breakString(value );
		  	    			}},
		                  {field:"percentOfRole", name:args.chartText.DISTRIBUTION, width:"50%"}
		                  ];
		if (args.isDrillDownCapable === true) {
			this.structure[1].formatter = dojo.hitch(this, this.formatDistLink);
		}
		this._trace.exiting("", "constructor");
	},
	createStore: function(attributeDistribution) {
		// summary:
		//		Creates a ItemFileReadStore for the grid from the given data.
		// attributeDistribution: Object
		//		Server data for store in json format.
		var items = [];
		dojo.forEach(attributeDistribution.attributeValues, function(attrVal){
			var floatValue = parseFloat(attrVal.countPercent);
			if(!isNaN(floatValue)){
				items.push({
					id: attrVal.id,
					name: attrVal.attributeValue,
					percentOfRole: floatValue
				});
			}
        });
		var dataStore = new dojo.data.ItemFileReadStore({
			data: {
				label: "name",
				id: "id",
				items: items					
			}
		});
		return dataStore;
	},
	startup:function() {
		this._trace.entering("", "startup");
		this.inherited(arguments);
		this._trace.exiting("", "startup");
	},
	resize:function() {
		this._trace.entering("", "resize");
		this.inherited(arguments);
		this._trace.exiting("", "resize");
	},
	formatDistLink:function(percent, index, cell) {
		// summary:
		//		Formats a hyperlink on a distribution column cell to drill down
		// percent: String
		//		The distribution percentage text
		// index: integer
		//		the row index of the the current cell.
		// returns: com.ibm.security.modeling.util.Anchor
		//		An anchor object to render in the cell.
		this._trace.entering("percent: " + percent + ", index: " + index, "formatDistLink");
		var item = this.getItem(index);
		var attributeValue = this.store.getValue(item, "name");
		var nameAnchor = new com.ibm.security.modeling.util.Anchor({
			input: percent,
			callback: function() {
				dojo.publish(dojo.config.panels.ANALYSIS_DRILL_DOWN.launchEventAddress, [attributeValue, dojo.config.analysisFactories.DISTRIBUTION_DRILL_FACTORY]);
			}
		});
		this._trace.exiting("", "formatDistLink");
		return nameAnchor;
	}
});