dojo.provide("com.ibm.security.modeling.tables.InOutAnalysisGrid");

dojo.require("dojox.grid.EnhancedGrid");
dojo.require("dojo.data.ItemFileReadStore");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");

dojo.requireLocalization("com.ibm.security.modeling.notebook", "AttributePanelText");

dojo.declare("com.ibm.security.modeling.tables.InOutAnalysisGrid", dojox.grid.EnhancedGrid, {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.tables.InOutAnalysisGrid"),
	// attributeDistribution: [readOnly] Object
	//		attribute data from server in json form.
	attributeDistribution: null,
	constructor: function(args) {
		// Summary:
		//		Shows in/out analysis statistics and a grid view.
		// isDrillDownCapable: boolean
		//		Whether this grid should support drilling down into data values via hyperlink
		// chartText: Object
		//		resouce bundle for translated chart text.
		this._trace.entering("isDrillDownCapable: " + args.isDrillDownCapable, "constructor");
		this.id = args.id;
		this.autoHeight = true;
		this.rowSelector = "0px";
		this.store = this.createStore(args.attributeDistribution);
		var attributeText = dojo.i18n.getLocalization("com.ibm.security.modeling.notebook", "AttributePanelText");
		this.structure = [
		                  {field:"name", name:attributeText.HEADER_COLUMN_1, width:"34%",toolTipFormatter:function(value,rowIndex,cell){
		  		        		return com.ibm.security.modeling.util.HtmlEscapingUtils.breakString(value );
		  	    			}},
		                  {field:"inCount", name:args.chartText.IN, width:"33%"},
		                  {field:"outCount", name:args.chartText.OUT, width:"33%"}
		                  ];
		if (args.isDrillDownCapable === true) {
			this._trace.finest("setting formatters", "constructor");
			this.structure[1].formatter = dojo.hitch(this, this.formatInLink);
			this.structure[2].formatter = dojo.hitch(this, this.formatOutLink);
		}
		this._trace.exiting("", "constructor");
	},
	createStore: function(attributeDistribution) {
		// summary:
		//		Creates a ItemFileReadStore for the grid from the given data.
		// attributeDistribution: Object
		//		Server data for store in json format.
		var items = [];
		dojo.forEach(attributeDistribution.attributeValues, function(attrVal){
			if (attrVal.inCount !== 0) {
				items.push({
					id: attrVal.id,
					name: attrVal.attributeValue,
					inCount: attrVal.inCount,
					outCount: attrVal.outCount
				});
			}
        });
		var dataStore = new dojo.data.ItemFileReadStore({
			data: {
				label: "name",
				id: "id",
				items: items					
			}
		});
		return dataStore;
	},
	startup:function() {
		this._trace.entering("", "startup");
		this.inherited(arguments);
		this._trace.exiting("", "startup");
	},
	resize:function() {
		this._trace.entering("", "resize");
		this.inherited(arguments);
		this._trace.exiting("", "resize");
	},
	formatInLink:function(inCount, index, cell) {
		// summary:
		//		Formats a hyperlink on an in column cell to drill down
		// percent: String
		//		The in count text
		// index: integer
		//		the row index of the the current cell.
		// returns: com.ibm.security.modeling.util.Anchor
		//		An anchor object to render in the cell.
		var inAnchor = this._formatDrillDownLink(inCount, index, dojo.config.analysisFactories.IN_DRILL_FACTORY);
		return inAnchor;
	},
	formatOutLink:function(outCount, index, cell) {
		// summary:
		//		Formats a hyperlink on an out column cell to drill down
		// percent: String
		//		The out count text
		// index: integer
		//		the row index of the the current cell.
		// returns: com.ibm.security.modeling.util.Anchor
		//		An anchor object to render in the cell.
		var outAnchor = this._formatDrillDownLink(outCount, index, dojo.config.analysisFactories.OUT_DRILL_FACTORY);
		return outAnchor;		
	},
	_formatDrillDownLink:function(text, index, factoryName) {
		// summary:
		//		Formats a hyperlink on an column cell to drill down on.
		// text: String
		//		The text to display
		// index: integer
		//		the row index of the the current cell.
		// factoryName: String
		//		The class name of the factory used to handle the drill down operation.
		// returns: com.ibm.security.modeling.util.Anchor
		//		An anchor object to render in the cell.
		this._trace.entering("text: " + text + ", index: " + index + ", factoryName: " + factoryName, "_formatDrillDownLink");
		var item = this.getItem(index);
		var attributeValue = this.store.getValue(item, "name");
		var anchor = new com.ibm.security.modeling.util.Anchor({
			input: text,
			callback: function() {
				dojo.publish(dojo.config.panels.ANALYSIS_DRILL_DOWN.launchEventAddress, 
						[attributeValue, factoryName]);
			}
		});
		this._trace.exiting("", "_formatDrillDownLink");
		return anchor;
	}
});