dojo.provide("com.ibm.security.modeling.tables.JsonRestStore");

dojo.require("dojox.data.JsonRestStore");

dojo.declare("com.ibm.security.modeling.tables.JsonRestStore", dojox.data.JsonRestStore, {
	totalCount: 0,
	currentCount: 0,
	isServerSide: false,
	ignoreMatchQuery:null,
	getIdentity: function(item) {
		// summary: 
		//     Gets the identity value of the given datastore item
	
		//Since the superclass implementation of this method prefers to use clientId, and since at least
		//one of our UIs create new entries at a different target url than the rest store target, we run
		//into a problem where the clientid gets messed up.  Therefore just set the client id to the id,
		//since we don't have scenarios in this product there the two should ever be different.
	    if(!item) return null;
		if (item.__clientId && item.__id){
			item.__clientId = item.__id;
		}
		var id = this.inherited(arguments);
		
		//we run into a few scenarios with selection preservation in the product where __id is not set, in these cases we assume that
		//item.id is the identity, of if not that then uri.  These are the 2 id patterns we use in the product.
		if (!id) {
			id = item.id?item.id:item.uri;
		}
		return id;
	},
	_getIdFromUri:function(uri) {
		var id = "undefined";
		if (uri) {
			var index = uri.lastIndexOf("/");
			id = uri.substring(index+1);
		}
		return id;
	},
	processQuery:function(query){
	    
		if( (typeof(this.ignoreMatchQuery)=="string")  && this.ignoreMatchQuery in query){
		   delete query[this.ignoreMatchQuery];
		}else if( this.ignoreMatchQuery instanceof Array ){
			for(var i=0;i<this.ignoreMatchQuery.length ;i++){
				if(this.ignoreMatchQuery[i] in query) delete query[this.ignoreMatchQuery[i]];
			}
		
		}else if(this.ignoreMatchQuery instanceof Object){
			try{
				for(var i in this.ignoreMatchQuery){
					if(i in query && query[i]==this.ignoreMatchQuery[i])  delete query[i];
				}
			}catch(err){
				
			}
		}
		return query;
	},
	matchesQuery: function(item,request){
		// summary: 
		//		overrides the same method in ClientFilter so that client side filtering
		//		works when searching over all columns
		// item: Object
		//		The cell of the table
		// request: Object
		//		The filter request
		// returns: boolean
		//		true - if item matches query
		// 		false - otherwise.
		
		// Fix for defect: 23991 and 21802
		// For newly added items, matchesQuery results onto false when table is server side and have query parameters which are not item's (being queried) attribute.
		// Hence these items do not appear in result on screen.
		//Removing the ignore query parameter from query
		var query = dojo.clone(request.query);
		if(this.ignoreMatchQuery) query=this.processQuery(query);
		
	//	var ignoreCase = request.queryOptions && request.queryOptions.ignoreCase;
		var ignoreCase = true;
		/*
		 * In a scenario where 'query' is an empty object: {}
		 * 	- hence item should be considered a match
		 * loop does not execute, 'isMatching' remains undefined, and returned.
		 * 
		 * This results into the 'item' being treated as 'not match'
		 * Hence undesired results are produced. 
		 */
		var isQueryEmpty = true;
		for(var i in query){
			isQueryEmpty = false;
			// if anything doesn't match, than this should be in the query
			var match = query[i];
			//if(i in item )var value = this.getValue(item,i);
		//	else return true;
		var value = this.getValue(item,i);
			var isString = typeof match == 'string';
			var isWildcard = match.match(/[\*\.]/);
			var isIgnoreCase = ignoreCase;
			var isMatching = false;
			if (isString && (isWildcard || isIgnoreCase)) {
				match = match.replace(/\?/g, "\\?");
				isMatching = dojo.data.util.filter.patternToRegExp(match, ignoreCase).test(value);
			} else {
				isMatching = value == match;
			}
			//Break the loop on the first cell that matches the filter
			if (isMatching) {
				break;
			}
		}
		
		if (isQueryEmpty) { // for loop did not execute. query does not have anything.
			return true;
		}
		
		return isMatching;
	},
	makeComparator: function(sort){
		//	summary:
		// 		returns a comparator function for the given sort order array. 
		//		Overrides the clientfilter method of the same name to tweak
		//		the comparator to be case insensitive.
		//	sort:
		//		See dojox.data.api.Read.fetch
		var current = sort.shift();
		if(!current){
			// sort order for ties and no sort orders
			return function(){
				return 0;// keep the order unchanged
			}; 
		}
		var attribute = current.attribute;
		var descending = !!current.descending;
		var next = this.makeComparator(sort);
		var store = this;
		return function(a,b){
			var av = store.getValue(a,attribute);
			if (typeof av == 'string') {
				av = av.toLowerCase();
			}
			var bv = store.getValue(b,attribute);
			if (typeof bv == 'string') {
				bv = bv.toLowerCase();
			}
			if(av != bv){
				return av < bv == descending ? 1 : -1;
			}
			return next(a,b); 
		}; 
	}, 

	//Overridden 
	//Firstly updating the client cache , then doing any operation on that cache .
	cachingFetch:function(args){
		
		
		for(var i = 0; i < this._fetchCache.length;i++){
			var cachedArgs = this._fetchCache[i];
			var count = args.count;
			args.count=Infinity;
	        if(cachedArgs.cacheResults){
	        	this.updateResultSet(cachedArgs.cacheResults,args);
	        }
	        args.count = count ;
		}
		//storing original isisUpdateable
		//we have already done our updates , and do not need any more updates in request 
		//setting to return false always
		var originalUpdateable = this.isUpdateable;
		this.isUpdateable=function(request){
			return false;
		};
		var returnValue =  this.inherited(arguments);
		//reestablishing the value of function . 
		this.isUpdateable = originalUpdateable;
		return returnValue;	
		
		
		
	},
	//Overriding this method , so that jsonrest store clears all local updates also ,since we want to
	//synch up with server .
	clearCache: function(){
		//	summary:
		//		Clears the cache of client side queries
		this.gotoServer = true;
		this._updates= [];
		this.inherited(arguments);
	}


});