dojo.provide("com.ibm.security.modeling.tables.PermissionTable");
dojo.require("com.ibm.security.modeling.tables.CustomDisplayStructureFactory");
dojo.require("com.ibm.security.modeling.tables.Table");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");
dojo.require("dojox.grid.enhanced.plugins.IndirectSelection");

dojo.requireLocalization("com.ibm.security.modeling.tables", "PermissionTableText");

dojo.declare("com.ibm.security.modeling.tables.PermissionTable", com.ibm.security.modeling.tables.Table, {
	
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.tables.PermissionTable"),
        customCols:null,
        tableType:"default",
        attributesMap:{
		"rolePermissionTable":"permission",
		"permissionPicker":"uri",
		"default":"uri"
        },
	constructor: function (args) {
		this._trace.entering("", "constructor");
		/* By default we do not want to show the Refresh Table button in the tool-bar for the permission
		 * table. We would like to show it only in case passed explicitly through the table args. */
		this.isRefreshRendered = false;
		this.defaultSortIndex = 1;
		this.permissionTableResources = dojo.i18n.getLocalization("com.ibm.security.modeling.tables", "PermissionTableText");
		this.searchInstructions = args.searchInstructions || this.permissionTableResources.SEARCH_INSTRUCTUION ;
		this._trace.exiting("", "constructor");
	},
	
	postCreate:function() {
		this.inherited(arguments);
		dojo.addClass(this.domNode,"fullSize");
	},
	
	postMixInProperties:function() {
		
		this.isClientSide = false;
		this.isFilterRendered = true;
		this.autoHeight = false;  // adding this back due to screen freeze. Will need to find a solution
		
		this.isMultiSelect = true;
		
		this.isDynamic = true;
		
		this.entityTypeText = this.globalResources.PERMISSIONS;
		
		this.structure = this._getStructurePermissions();
		this.inherited(arguments);//this needs to be called after isDynamic is set
	},
	
	_getStructurePermissions: function() {
		
		 var cols= com.ibm.security.modeling.tables.CustomDisplayStructureFactory.loadPermissionStructure(this.customCols,dojo.hitch(this,this._onFetchError));
		 var i;
         for(i=0;i<cols.length;i++){
          
            if(cols[i].field=="name"){
                  cols[i].get=function(rowIndex,item){return item};
                  cols[i].formatter=dojo.hitch(this, this._formatName);
             }

         }
         return cols;		
	},
    _formatName:function(item){
          var anchor=null;
		  if(item){
	          anchor=new com.ibm.security.modeling.util.Anchor({
					input:item.name,
					callback:dojo.hitch(this, this.permissionSelected,item)
				});											
		    }
			return anchor;
          

         },
	permissionSelected:function(item){
            dojo.require("com.ibm.security.modeling.widgets.PropertiesDialog"); 
            var attributeField=this.attributesMap[this.tableType];
            this.permissions=new com.ibm.security.modeling.widgets.PropertiesDialog({
                                     id:this.id+"Permissions",
                                     panelHeaderName:item.name,
				                     type:"PERMISSION",
				                     entityItem:item[attributeField]
			         
		},null);
		this.permissions.show();
		this.permissions.startup();
     },
	startup:function() {
		this.inherited(arguments);
	},
	
	destroy:function(){
		this.inherited(arguments);
	}	
});
	