dojo.provide("com.ibm.security.modeling.tables.RoleAnalysisTable");

dojo.require("com.ibm.security.modeling.tables.Table");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");
dojo.require("com.ibm.security.modeling.util.Trace");

//The table will have the common finctionality which is required by the ROleAnalysisTable
//The things which are seen common for the role analysis table are listed below. 
//	1. There should be no selection column in the table
//  2. There should be no toolbar buttons
//	3. Only Search box should be available

dojo.declare("com.ibm.security.modeling.tables.RoleAnalysisTable", com.ibm.security.modeling.tables.Table, {
	
	inheritanceEventName: "inheritance",
	handleEventCallback: null,
	//LocalResources should be passed to the table from the panel/dialog where the table is being initialized.
	localResources : null,
	includeInheritance: false,
	postCreate : function() {
		this.inherited(arguments);
		this.topic = this.subscribe(this.id+"_"+this.inheritanceEventName, this.handleEventCallback);
		dojo.addClass(this.domNode, "contentArea");
		this._createIncludeInheritance();
	},
	
	postMixInProperties : function() {
		this.isClientSide = false;
		this.isFilterRendered = true;
		this.isMultiSelect = false;
		this.isSelectAllRendered = false;
		this.isDynamic = true;
		this.autoHeight = false;
		this.rowsPerPage = 10;
		this.needSelection = false;
		this.isRefreshRendered = false;
		this.isPaginationEnabled=true;
		this.isTotalsFooterRendered=true;
		this.inherited(arguments);
	},
	
	_clicked : function(isChecked){
		if (this._ignoreChange) {
			this._ignoreChange = false;
			return;
		}
		dojo.publish(this.id+"_inheritance", [{'includeInheritance':isChecked}]);
		this.includeInheritanceButton.focus();
	},
	_createIncludeInheritance : function(){
		try{
		this.includeInheritanceButton = new com.ibm.security.modeling.tables.IncludeInheritanceWidget({
			id: this.id +"_includeInheritance",
			value: "inheritance",
			changeEventHandler: dojo.hitch(this, this._clicked),
			labelText: this.getInheritanceLabel()
		});
		this.setIncludeInheritance(this.includeInheritance);
		this.header.toolbar.addChild(this.includeInheritanceButton);
	   }catch(e){
	   	console.log(e);
	   }	
	},
	
	setIncludeInheritance: function(/*boolean*/isChecked) {
		if (!this.includeInheritanceButton || !this.includeInheritanceButton.checkBox) {
			return;
		}
		if (this.includeInheritanceButton.checkBox.get("checked") != isChecked) {
			this._ignoreChange = true; // not from UI interaction, hence ignore.  
			this.includeInheritanceButton.checkBox.attr("checked", isChecked);
		}
	},
	getIncludeInheritance: function() {
		if (this.includeInheritanceButton && this.includeInheritanceButton.checkBox) {
			return this.includeInheritanceButton.checkBox.get("checked");
		}
		return false;
	},
	
	destroy: function(){
		this.inherited(arguments);
	}
	
});


dojo.declare("com.ibm.security.modeling.tables.IncludeInheritanceWidget", [ dijit._Widget, dijit._Templated],{
	
	templateString: "<span><input id='${id}_checkbox' dojoType='dijit.form.CheckBox' value='false' dojoAttachPoint='checkBox' ><label for='${id}_checkbox'>${labelText}</label></input></span>",
	widgetsInTemplate: true,
	labelText: null,
	id: null,
	changeEventHandler: null,
	value: "selected",
	
	postCreate : function(){
		this.inherited(arguments);
		this.checkBox.onChange = dojo.hitch(this, this.changeEventHandler);
	},
	focus:function(){
		this.checkBox.focus();
	}

});