/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

 Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.tables.RoleErrorWarningTable");

dojo.require("com.ibm.security.modeling.tables.Table");


dojo.declare("com.ibm.security.modeling.tables.RoleErrorWarningTable", com.ibm.security.modeling.tables.Table, {
	// roleItem: Object
    //		An instance of role object.
	roleItem:null,
	// parentWidget: com.ibm.security.modeling.notebook.RoleNotebookDialogView
	//		Parent widget to display error messages and do resizing
	parentWidget:null,
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.tables.RoleErrorWarningTable"),
	postMixInProperties:function() {
			// tags:
			//		protected
			this._trace.entering(this.id, "postMixInProperties");
			this.isClientSide=true;
			this.isFilterRendered=false;
			this.isSelectAllRendered=false;
			this.isRefreshRendered=true;
			this.isMultiSelect=false;
			this.isSearchMultiSelect=false;
			this.defaultSortIndex=0;
			this.isDefaultSortAscending=true;
			this.isDynamic=true;
			this.needSelection=false;
			this.structure=this._getStructure();
			this.inherited(arguments);//this needs to be called after isDynamic is set
			this._trace.exiting(this.id, "postMixInProperties");
	},
	postCreate:function() {
		// tags:
		//		protected
		this._trace.entering("", "postCreate");
		this.inherited(arguments);
		this._trace.exiting("", "postCreate");
	},
	_getStructure:function(){
		// summary:
		//		Sets the structue of error warniing table 
		// tags:
		//		private
		this._trace.entering(this.id, "_getStructure");
		this._trace.exiting(this.id, "_getStructure");
		return [{field:"severity",name:this.globalResources.SEVERITY,width:"10%", formatter:dojo.hitch(this,this._formatMessageType)},
				{field:"text",name:this.globalResources.MESSAGE,width:"90%"}];
	},
	_formatMessageType:function(/*String*/severity, /*int?*/inRowIndex,/*dojox.grid.cells.Cell?*/cell){
		// summary:
		//		Sets the icon for severity column using formatter. Set the icon and label depending on severity.
		// tags:
		//		private
		this._trace.entering(this.id, "_formatMessageType");
		var typeLabel="";
		if(severity=="E"){//represented as error
			typeLabel="<span class=\'dijitInline errorIcon\'></span> <span class=\'dijitInline\'>"+this.globalResources.ERROR+"</span>";
		}else if(severity=="W"){//represented as warning
			typeLabel="<span class=\'dijitInline warningIcon\'></span> <span class=\'dijitInline\'>"+this.globalResources.WARNING+"</span>";
		}
		this._trace.exiting(this.id, "_formatMessageType");
		return typeLabel;
		
	},
	_formatMessage:function(/*Object*/messageItem){
		var formattedMsg="";
		var roleName=messageItem.messageParameters[0];
		var sodConstraintDescription=messageItem.messageParameters[1];
		var userCount="";
		formattedMsg=this.logResources[messageItem.messageKey];
		formattedMsg=formattedMsg.replace("${role_name}", roleName);
		formattedMsg=formattedMsg.replace("${sod_constraint_description}", sodConstraintDescription);
		if(messageItem.messageParameters.length>2){
			userCount=messageItem.messageParameters[2];
			formattedMsg=formattedMsg.replace("${user_count}", userCount);
		}
		return formattedMsg;
	},
	destroy:function(){
		// tags:
		//		protected
		this._trace.entering(this.id, "destroy");
		this.inherited(arguments);
		this._trace.exiting(this.id, "destroy");
	}	
});