/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

ï¿½ Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.tables.RoleLifecycleRequestTable");

dojo.require("com.ibm.security.modeling.tables.Table");
dojo.require("dojo.date.locale");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");
//NLS
dojo["requireLocalization"]("customizations", "LifecycleText");


dojo.declare("com.ibm.security.modeling.tables.RoleLifecycleRequestTable", com.ibm.security.modeling.tables.Table, {
	// roleItem: Object
    //		An instance of role object.
	roleItem:null,
	// parentWidget: com.ibm.security.modeling.notebook.RoleNotebookDialogView
	//		Parent widget to display error messages and do resizing
	parentWidget:null,
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.tables.RoleLifecycleRequestTable"),
	postMixInProperties:function() {
			// tags:
			//		protected
			this._trace.entering(this.id, "postMixInProperties");
			this.isClientSide=true;
			this.isFilterRendered=false;
			this.isSelectAllRendered=false;
			this.isRefreshRendered=true;
			this.isMultiSelect=false;
			this.isSearchMultiSelect=false;
			this.defaultSortIndex=2;
			this.isDefaultSortAscending=false;
			this.isDynamic=true;
			this.needSelection=false;
			this.toolTipRequired = [1,2];
			this.unSortableColumns = [1,2];
			this.customText = dojo.i18n.getLocalization("customizations", "LifecycleText");
			this.structure=this._getStructure();
			this.inherited(arguments);//this needs to be called after isDynamic is set
			this._trace.exiting(this.id, "postMixInProperties");
	},
	postCreate:function() {
		// tags:
		//		protected
		this._trace.entering("", "postCreate");
		this.inherited(arguments);
		this._trace.exiting("", "postCreate");
	},
	_getStructure:function(){
		// summary:
		//		Sets the structure of error warning table 
		// tags:
		//		private
		this._trace.entering(this.id, "_getStructure");
		this._trace.exiting(this.id, "_getStructure");
		return [
		        	{
		        		field: "requestType",
		        		name: this.lifecycleResources.REQUEST_TYPE,
		        		width: "30%",
		        		toolTipFormatter: dojo.hitch(this,this._formatRequestTypeTooltip),
		        		formatter: dojo.hitch(this,this._formatRequestType)
		        	},{
		        		field: "status", 
		        		name: this.lifecycleResources.STATUS,
		        		width: "40%",
		        		get: dojo.hitch(this,this._getLifecycleStatusFrmTable), 
		        		toolTipFormatter: dojo.hitch(this,this._formatLifecycleStatusTooltip),
		        		formatter: dojo.hitch(this,this._formatLifecycleStatus)
		        	},{
		        		field: "timestamp",
		        		name: this.lifecycleResources.LAST_UPDATED,
		        		width: "30%",
		        		formatter: dojo.hitch(this,this._formatDate)
		        	}
		        ];
	},
	
	_getLifecycleStatusFrmTable:function(inRowIndex,inItem){
		return inItem;
	},
	
	_formatDate: function(value,rowIndex,cell){
		var parsedDate = dojo.date.locale.parse(value, {
								datePattern:"yyyy-MM-dd'T'HH:mm:ss.SSSZ", 
								selector:"date"
							});
		
		parsedDate = dojo.date.add(parsedDate, "minute",this._getTimeZoneOffset(value));
		
		parsedDate = dojo.date.locale.format(parsedDate, {formatLength:"medium"});
		
		return parsedDate;
	},
	_getTimeZoneOffset: function(value){
		//Ex : value = "2012-02-08T14:57:42.421+0500"
		var timezoneOffset = 0;
		if(value){
			var timeZone = value.substring(value.length-5,value.length);
			
			var timeZoneHH = parseInt(timeZone.substring(1,3));
			var timeZonemm = parseInt(timeZone.substring(3,5));
			
			timezoneOffset =  timeZoneHH*60+timeZonemm;
			
			var timeZoneSign = timeZone.substring(0,1);
			if(timeZoneSign == "+"){
				timezoneOffset = timezoneOffset * -1;
			}
		}
		return  timezoneOffset - new Date().getTimezoneOffset();
	},
	_formatLifecycleStatus: function(item,rowIndex,cell){
		var statusText = "";
		if(item){
			statusText = this._getValuePerLocale(item.status,"LIFECYCLE_ROLE_REQUEST_STATUS_");
			statusText = com.ibm.security.modeling.tables.FormatterUtils.formatLongCellValue(statusText);
			statusText = com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(statusText);
			var linkLocation = this.dataStore.getValue(item, "data1");
			if(linkLocation){
				statusText = "<a id='"+this.id+"_lifecycle_status_link' href='"+linkLocation+"' target='_blank'>"+statusText+"</a>";
				statusText = "<span class='externalLinkIconFloat'>"+statusText+"</span><span class='externalLinkIcon externalLinkIconFloat'></span>";
				}
			else{
				statusText = "<span class='externalLinkIconFloat'>"+statusText+"</span>";
			}
		}	
		return statusText;
	},
	_formatRequestType: function(value,rowIndex,cell){
		var statusText = this._getValuePerLocale(value,"LIFECYCLE_ROLE_REQUEST_TYPE_");
		statusText = com.ibm.security.modeling.tables.FormatterUtils.formatLongCellValue(statusText);
		statusText = com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(statusText);
		return statusText;
	},
	_formatLifecycleStatusTooltip: function(value,rowIndex,cell){
		
		var statusText = "";
		if(value && value.status){
			statusText = this._getValuePerLocale(value.status,"LIFECYCLE_ROLE_REQUEST_STATUS_");
			statusText = com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(statusText);
			
		}
		return statusText ;
	},
	_formatRequestTypeTooltip: function(value,rowIndex,cell){
		value = this._getValuePerLocale(value,"LIFECYCLE_ROLE_REQUEST_TYPE_");
		return com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(value);
	},
	_getValuePerLocale: function(/*string key*/ key,
									/*string prefix*/ prefix) {
		prefix = prefix ? prefix : "";
		
		var text = null;
		if (key) {
			var mappedKey = prefix+(key.toUpperCase());
			text = this.customText[mappedKey]; 
		}
		return text ? text : key;
	},
	destroy:function(){
		// tags:
		//		protected
		this._trace.entering(this.id, "destroy");
		this.inherited(arguments);
		this._trace.exiting(this.id, "destroy");
	}	
});