/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

� Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.tables.RoleMembershipTable");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");
dojo.require("com.ibm.security.modeling.tables.UserTable");
dojo.require("com.ibm.security.modeling.tables._DisabledSelection");
dojo.require("com.ibm.security.modeling.util.ErrorUtilities");

dojo.declare("com.ibm.security.modeling.tables.RoleMembershipTable",[com.ibm.security.modeling.tables.UserTable,com.ibm.security.modeling.tables._DisabledSelection], {
	panel: null,
	_roleId: null,
	// saveIndicator: com.ibm.security.modeling.indicator.SaveIndicator
	//		An instance of com.ibm.security.modeling.indicator.SaveIndicator.
    	//		Displays save indicator.
	saveIndicator:null,
	roleItem:null,
	parentWidget:null,
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.tables.RoleMembershipTable"),
	
	constructor:function(){
		this._errorUtil = com.ibm.security.modeling.util.ErrorUtilities();
        this.inherited(arguments);
    },
	postMixInProperties:function() {
			this.setCustomCols();
			this.inherited(arguments);
			if(this.userTableType=="directlyAttachedUsers"){
				this.searchInstructions=this.userTableResources.DIRECTLY_ATTCHED_USERTABLE_SEARCH;
				this.entityTypeText = this.userTableResources.DIRECTLY_ATTCHED_USERS;
				this.unSortableColumns = [4];
			}else{
				this.unSortableColumns = [5];
				this.searchInstructions=this.userTableResources.ALL_ROLES_USERTABLE_SEARCH;
				this.entityTypeText = this.userTableResources.ALL_ROLES_USERS;
			}
			this.defaultSortIndex = 1;
	},
	postCreate:function() {
		this.inherited(arguments);
		this._setupToolbar();
		this._setupCandidateStore();
		if(this.userTableType=="allRoleMembers"){
			this._setupSelectionLogic();
		}
	},
     setCustomCols:function(){
		if(this.userTableType=="directlyAttachedUsers"){
			this.customCols={1:{field: 'byFilter',name: this.userTableResources.BY_FILTER,width: '8%', formatter: dojo.hitch(this, this._formatCheckBoxColumn), isSearchable: false },
				2:{field: 'entitledBy',name: this.userTableResources.ENTITLED_BY,width: '12%', formatter: dojo.hitch(this, this._formatEntitledByColumn), isSearchable: false}};
		}else{
			this.customCols={1:{field: 'directlyAttached',name: this.userTableResources.HEADER_COLUMN_1,width: '11%', formatter: dojo.hitch(this, this._formatCheckBoxColumn), isSearchable: false },
				2:{field: 'byFilter',name: this.userTableResources.BY_FILTER,width: '8%', formatter: dojo.hitch(this, this._formatCheckBoxColumn), isSearchable: false},
				3:{field: 'entitledBy',name: this.userTableResources.ENTITLED_BY,width: '12%', formatter: dojo.hitch(this, this._formatEntitledByColumn), isSearchable: false}};
		}
    },
	// Column formatters.
	_formatCheckBoxColumn: function (value) {
		//this._trace.entering(this.id, "_formatCheckBoxColumn");
		var cellContent = "<span style='font-size: 0px !important;' ";
		if (value && (value == 'true' || value == true)) {
			cellContent = cellContent+ "class='checkIcon'";				
			cellContent = cellContent + ">yes";
		} else {			
			cellContent = cellContent + ">no";
		}
		cellContent = cellContent+ "</span>";
		//this._trace.exiting(this.id, "_formatCheckBoxColumn");
		return cellContent;
	},
	_formatEntitledByColumn:function(list){
		//this._trace.entering(this.id, "_formatEntitledByColumn");
		
		var nameListString = "";
		if(list !=null && list.length && list.length>0){
			var nameList = null;
			
			if (typeof list[0] == 'string') {
				nameList = list;
			} else if (typeof list[0] == 'object') { 
				nameList = [];
				for(var i = 0; i < list.length; i++){
					nameList.push(list[i].name.toString());
				}
			}
			if (nameList && nameList.length > 0) {
				nameList.sort();
				for(var j=0,len1=nameList.length;j<len1;j++){
					nameListString=nameListString+nameList[j];
					if(j+1!=len1){
						nameListString=nameListString+", ";
					}
				}
			}
		}
		//this._trace.exiting(this.id, "_formatEntitledByColumn");
		return com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLandNormalize(nameListString);
	},
	_setupToolbar:function() {
		this.addToolbarSeparator();
		this.addToolbarButton(this.userTableResources.ADD_USER,
	              false,
	              "addIcon",
	              dojo.hitch(this,this._launchAddUser),
	              this.id+"addUserBtn",
				  true,
				  true,
				  true);
        
		this.addToolbarButton(this.userTableResources.DELETE_USER,		// Text describing the button (used as tooltip/alt text)
				              false,								// Whether to show the label in the button itself
				              "removeIcon",							// String describing the css class describing the button image
				              dojo.hitch(this,this._removeUser),// The function to be executed when the button is clicked.
				              this.id+"newRoleBtn",					// The identifier used to locate the button.
				              true,									// boolean flag specifying whether the button should be active when a single row is selected.
				              true,									// boolean flag specifying whether the button should be active when multiple rows are selected
				              false);								// boolean flag specifying whether the button should be active when no rows are selected.
	},
	_setupCandidateStore:function(){
		this._trace.entering("", "_setupCandidateStore");
		this.userRestManager = new com.ibm.security.modeling.api.UserRestManager({
			resourceUri: this._getTableStoreTargetUri(this.roleItem.membershipCandidatesUri),
			syncMode: false,
			allowNoTrailingSlash:true,
			messageArea: this.messageArea
		});
		this.candidateStore = this.userRestManager.getIsrmTableRestStore();
		this._trace.exiting("", "_setupCandidateStore");
	},
	_showPeoplePicker:function(id,peopleStore,toStore,localResourceBuundle,tableQuery){
		dojo.require("com.ibm.security.modeling.widgets.PeoplePickerDialog");
		var editParameters = {
			property: 'membership', 
			activity: 'add'
		};
		var pickerDialog = new com.ibm.security.modeling.widgets.PeoplePickerDialog({
					id:id, 
					namespace:this.namespace,
					fromDataStore: peopleStore,	// for picker table.
					tableQuery:tableQuery,

					toDataStore: toStore,
					toDataStoreTargetUri: this.roleItem.membershipUri.toString(),
					toDataStoreResponsePayloadAttribute: 'analysis',

					parentWidget: this,
					parentMessageArea: this.messageArea,
					callback:dojo.hitch(this, this._membershipEditComplete, editParameters),
					contextRoot: this.contextRoot,
					globalResources: this.globalResources,	// for picker table.
					localResources: localResourceBuundle,
					saveIndicator:this.saveIndicator,
					saveIndicatorText: this.saveIndicatorText,
					helpFile: "ui_users_add.html"
				});
		pickerDialog.startup();
		pickerDialog.show();
		
		
	},
	
	
	_membershipEditComplete:function(editParameters, args){	// args contains 'value' (batch rest response)
		try {
			this._trace.entering(this.id, "_addMembershipComplete");
			var counts = args[1];	// success, failure counts
			if (counts.successCount > 0) {
				dojo.publish("ROLE_PROPERTY_UPDATE_EVENT",[editParameters]);
			}
			//this._resetSelectedAndTotal();
		} finally {
			this._trace.exiting(this.id, "_addMembershipComplete");
		}
	},
	_launchAddUser:function(){
		this._trace.entering(this.id, "_launchAddUser");
		this._setupCandidateStore();
		this._showPeoplePicker(this.id+"_candidatePicker",this.candidateStore,this.dataStore,this.userTableResources,this._getTableQueryObject(this.roleItem.membershipCandidatesUri));
		this._trace.exiting(this.id, "_launchAddUser");
	},
	_removeUser:function(){
		var selectedRows = this.getSelectedItems();
		var selectedRowsCount = selectedRows.length;
		this._trace.finer(this.id+' - Number of selected rows: '+selectedRowsCount, "_removeUser");
		
		// validation: At-least one selection is required
		if (selectedRowsCount == 0) { 
			// control should never come here, as button enable/disable functionality is provided by common grid.
			return null;
		}
		this._hideFooter();
		this.deselectAll();
		if (this.saveIndicator) {	
			this.saveIndicator.show(this.saveIndicatorText);	//show save indicator
		}
		var editParameters = {
			property: 'membership', 
			activity: 'remove'
		};
		// Update store
		for (var i=0; i<selectedRows.length; i++) {
			this.dataStore.deleteItem(selectedRows[i]);// marks item for deletion
		}
		
		// When done
		var selectedItemsCount = selectedRows.length;
		var processedItems = 0;
		var successCount = 0;
		var failureCount = 0;
		var self = this;
		var handleComplete = function(actions, value, isError) {
			if (isError == false) {
				successCount = actions.length;
			} else {
				failureCount = actions.length;
			}
			processedItems = processedItems + actions.length;
			if (processedItems == selectedItemsCount) {
				self.panel.showMessages(value);
				self._membershipEditComplete(editParameters,[value, {successCount: successCount, failureCount: failureCount}]);
				self.saveIndicator.hide(self.saveIndicatorText);
				self._showFooter();
			}
		};
		
		// commit changes
		var saveArgs={};
		saveArgs.handleAsBatch=true;
		saveArgs.targetUri=this.roleItem.membershipUri.toString();
		if(this.userTableType == "allRoleMembers"){
			saveArgs.uriAttribute="directMembership";
		}
		saveArgs.responsePayloadAttribute="analysis";
		
		
		saveArgs.onComplete=dojo.hitch(this, function(args) {
							handleComplete(args[0], args[1],false);
							});
		saveArgs.onError= dojo.hitch(this, function(args) {
				this._trace.error("Error while deleting.", "error callback");
				if (args instanceof Array) {
					handleComplete(args[0], args[1],true);
				} else {
					this._errorUtil.handleRESTError(args,this.messageArea);
					this.saveIndicator.hide(this.saveIndicatorText);
				}
			});
		this.dataStore.save(saveArgs);
		this._trace.exiting(this.id, "_removeUser");
	},
	
	_getTableStoreTargetUri: function(targetUri) {
		if (targetUri.indexOf("?") != -1) {
			return targetUri.split("?")[0];
		}
		return targetUri;
	},
	
	_getTableQueryObject: function(targetUri) {
		if (targetUri.indexOf("?") != -1) {
			return dojo.queryToObject(targetUri.split("?")[1]);
		}
		return dojo.queryToObject(targetUri);
	},
	destroy:function(){
		this.inherited(arguments);
	}
});