dojo.provide("com.ibm.security.modeling.tables.RoleModelingTable");

dojo.require("dijit.MenuItem");
dojo.require("dijit.PopupMenuItem");
dojo.require("com.ibm.security.modeling.hierarchy.UndoHandler");
dojo.require("com.ibm.security.modeling.tables.Table");
dojo.require("com.ibm.security.modeling.tables.TableContextMenuHandler");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("dojox.grid.enhanced.plugins.IndirectSelection");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");
dojo["requireLocalization"]("customizations", "LifecycleText");

dojo.declare("com.ibm.security.modeling.tables.RoleModelingTable", com.ibm.security.modeling.tables.Table, {
	parentContainer: null,
	_modelId: null,
	_undoHandler: null,
	roleConnections: [],
	_actionMenu : null,
	
	// showLifecycleStatusColumn: boolean
	//		Flag to include Role Lifecycle Status column in table and table searches.
	//		true - Role Lifecycle Status column will be displayed.
	//		false - Role Lifecycle Status column will not be displayed.
	showLifecycleStatusColumn: false,
	
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.tables.RoleModelingTable"),
	constructor:function(args) {
		this.contextMenuHandler = new com.ibm.security.modeling.tables.RoleModelingTableContextMenuHandler();
	},
	postMixInProperties:function() {
		
		if (!this.lifecycleResources) {
			this.lifecycleResources = dojo.i18n.getLocalization("customizations", "LifecycleText");
		}
		this.isClientSide = false;
		this.isFilterRendered = true;
		this.isRefreshRendered = true;
		this.autoHeight = false;  // adding this back due to screen freeze. Will need to find a solution
		this.isSelectAllRendered = false;
		this.isMultiSelect = true;
		this.unSortableColumns = [7,8];
		this.toolTipRequired = [3,4,7,8];
		if (this.showLifecycleStatusColumn === true) {
			this.toolTipRequired = [3,4,5,8,9]; // include lifecycle status column as well.
			this.unSortableColumns = [8,9];
		}
		this.isDynamic = true;
		this.searchInstructions = this.globalResources.SEARCH_ROLES;
		this.entityTypeText = this.globalResources.ROLES;
		this.structure = this._getStructure();
		this.inherited(arguments);//this needs to be called after isDynamic is set
		this.defaultSortIndex = 2;
	},
	_getStructure: function() {
		var baseStructure = [{field: 'messagesCount',name:"<div class=\'statusIcon\'></div>",width: '50px', isSearchable:false, get: dojo.hitch(this,this._getRoleStatusValueFrmTable),formatter:dojo.hitch(this,this._formatRoleStatusValue), toolTipText:this.globalResources.STATUS},
		        {field: 'name',name: this.globalResources.ROLE,width: '20%',get: dojo.hitch(this,this._getRoleDetailValueFrmTable),formatter:dojo.hitch(this,this._formatRoleNameValue)},
		        {field: 'description',name: this.globalResources.DESCRIPTION,width: '21%',toolTipFormatter:function(value,rowIndex,cell){
			        	return com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(value );
		    		},
		    		formatter:function(value,rowIndex,cell){
		    			return com.ibm.security.modeling.tables.FormatterUtils.formatLongCellValue(value);
		    		}
		        },
		        {field: 'totalIdentitiesCount',name: this.globalResources.USERS,width: '10%',get: dojo.hitch(this,this._getRoleUserCountDetailFrmTable),formatter: dojo.hitch(this,this._formatUserCount), isSearchable:false},
		        {field: 'totalPermissionsCount',name: this.globalResources.PERMISSIONS,width: '10%',get: dojo.hitch(this,this._getRolePermCountDetailFrmTable),formatter:dojo.hitch(this,this._formatPermissionCount), isSearchable:false},
		        {field: 'parents',name: this.globalResources.PARENT_ROLES,width: '12%',formatter: dojo.hitch(this,this.formatObjectsToCommaSeparatedNamesCell),toolTipFormatter: dojo.hitch(this,this.formatToolTipObjectsToCommaSeparatedNamesCell), isSearchable:true},
		        {field: 'children',name: this.globalResources.CHILD_ROLES,width: '12%',formatter: dojo.hitch(this,this.formatObjectsToCommaSeparatedNamesCell), toolTipFormatter: dojo.hitch(this,this.formatToolTipObjectsToCommaSeparatedNamesCell), isSearchable:true}
		];
		
		var tableStructure = null;
		if (this.showLifecycleStatusColumn === true) {
			tableStructure = new Array();
			for (var i=0; i<baseStructure.length; i++) {
				tableStructure[tableStructure.length] = baseStructure[i];
				
				// Status column is displayed after description column
				if (baseStructure[i]["field"] == "description") {
					tableStructure[tableStructure.length] = {
							field: 'status',
							name: this.globalResources.LIFECYCLE_STATUS,
							width: '10%', 
							isSearchable: true, 
							toolTipFormatter: dojo.hitch(this,this._formatLifecycleStatusTooltip),
							formatter: dojo.hitch(this,this._formatLifecycleStatus),
							valueLabelResourceConfiguration: this._getLifecycleStatusValueLabelResourceConfiguration()
						};
				}
			}
		} else { // No status column; hence increase the width of some columns.
			tableStructure = baseStructure;
			for (var i=0; i<tableStructure.length; i++) {
				switch(tableStructure[i]["field"]) {
				case 'description':
					tableStructure[i]["width"] = "25%";
					break;
				case 'parents':
				case 'children':
					tableStructure[i]["width"] = "15%";
					break;
				default:
					break;
				}
			}
		}
		return tableStructure;
	},
	
	_getLifecycleStatusValueLabelResourceConfiguration: function() {
		var valueLabelResourceConfiguration = {
			nlsResource: this.lifecycleResources,
			keyPrefix: "LIFECYCLE_ROLE_STATUS_",
			keyPostfix: null
		};
			
		return valueLabelResourceConfiguration;
	},
	
	_formatLifecycleStatusTooltip: function(value,rowIndex,cell){
		var statusText = this._getLifecycleStatusPerLocale(value);
		return com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(statusText);
	},
	_formatLifecycleStatus: function(value,rowIndex,cell){
		var statusText = this._getLifecycleStatusPerLocale(value);
		return com.ibm.security.modeling.tables.FormatterUtils.formatLongCellValue(statusText);
	},
	_getLifecycleStatusPerLocale: function(/*status*/ statusCode) {
		var i18nStatusText = null;
		if (statusCode) {
			var mappedStatusCodeKey = "LIFECYCLE_ROLE_STATUS_"+(statusCode.toUpperCase());
			i18nStatusText = this.lifecycleResources[mappedStatusCodeKey];
		}
		return i18nStatusText ? i18nStatusText : statusCode;
	},
	
	postCreate:function() {
		this.inherited(arguments);
		dojo.addClass(this.domNode,"fullSize");
		this._setupToolbar();
		this._setupRightClickMenu();
		this._setupLifecycleMenu();
		// connecting onCellClick event for table to launch error and warning panel on click status column
		this.connect(this.grid, "onCellClick",dojo.hitch(this,this._statusIconColumnClickHandler));

	},
	
	_setupToolbar:function() {
		
		//adding a button to switch to topology
		this.addToolbarButton(this.localResources.HIERARCHY_VIEW,
				              false,
				              "hierarchicalLayoutIcon",
				              this.launcher.launchTopology,
				              this.id+"swithView");
		this.addToolbarSeparator();
		
		//adding buttons to launch role operations
		this.addToolbarButton(this.localResources.CREATE_ROLE,		// Text describing the button (used as tooltip/alt text)
				              false,								// Whether to show the label in the button itself
				              "newIcon",							// String describing the css class describing the button image
				              dojo.hitch(this,this._createRootRole),// The function to be executed when the button is clicked.
				              this.id+"newRoleBtn",					// The identifier used to locate the button.
				              true,									// boolean flag specifying whether the button should be active when a single row is selected.
				              true,									// boolean flag specifying whether the button should be active when multiple rows are selected
				              true);								// boolean flag specifying whether the button should be active when no rows are selected.
		
		this.addToolbarButton(this.localResources.EDIT_ROLE,
				              false,
				              "editIcon",
				              dojo.hitch(this,this._launchRolePropertiesDialog),
				              this.id+"editRoleBtn",
				              true,
				              false,
				              false);
		
		this.addToolbarButton(this.localResources.COPY_ROLE,
							  false,
							  "copyIcon",
							  dojo.hitch(this,this._copyRole),
							  this.id+"copyRoleBtn",
							  true,
							  false,
							  false);
		
		this.addToolbarButton(this.localResources.DELETE_ROLE,
				              false,
				              "deleteIcon",
				              dojo.hitch(this,this._launchRoleDeleteDialog),
				              this.id+"deleteRoleBtn",
				              true,
				              true,
				              false);
		this.addToolbarSeparator();
		var undoMenu = new dijit.Menu({
            style: "display: none;"
        });
		var undoButton = this.addToolbarButton(this.localResources.UNDO,
	              false,
	              "undoIcon",
	              null,
	              this.id+"undoBtn",
	              false,
	              false,
	              false,
	              undoMenu);
		// this._widgetUtilities.enableButton(undoButton);
		this._undoHandler.setTable(this, this._modelId);
		this.addToolbarSeparator();
		/*this.addToolbarButton(this.localResources.EXPORT_PROJECT,
				              false,
				              "exportIcon",
				              dojo.hitch(this,this._launchExportProjectDialog),
				              this.id+"exportProjectBtn",
				              true,
				              true,
				              true);*/
		this._actionMenu = new dijit.Menu({
                    style: "display: none;"
                });
		this._addActionMenuItem(new dijit.MenuItem({
                    label: this.localResources.EXPORT_PROJECT,
                    onClick: dojo.hitch(this,this._launchExportProjectDialog),
                    iconClass: "tipMenuIcon exportIcon"
                }));
	
		
		this._addActionMenuItem(new dijit.MenuItem({
                    label: this.localResources.COPY_ROLES_PROJECT,
                    onClick: dojo.hitch(this,this._launchCopyRolesDialog),
                    iconClass: "tipMenuIcon copyIcon"
                }));
		
		this._addActionMenuItem(new dijit.MenuItem({
                    label: this.localResources.GENERATE_ROLES,
                    onClick: dojo.hitch(this,this._launchGenerateRoles),
                    iconClass: "tipMenuIcon generateIcon"
                }));
		
        //this.addToolbarSeparator();	
       this.addToolbarButton(this.localResources.ACTIONS,
				              true,
				              "",
				              null,
				              this.id+"actions",
							  true,
							  true,
							  true,
							  this._actionMenu 	
		);					  
		
	},
	_addActionMenuItem : function(menuItem) {
		this._addMenuItem(this._actionMenu,menuItem);
	},
	_addMenuItem : function(menu,menuItem) {
	//	dojo.addClass(menuItem.iconNode.parentNode,"hidden");	
		menu.addChild(menuItem);
	},
	_setupRightClickMenu:function() {
		var viewItem = new dijit.MenuItem({
			label: this.localResources.VIEW_DETAILS,
			disabled: true,
			onClick: dojo.hitch(this,this._launchRolePropertiesDialog),
			iconClass: "tipMenuIcon viewDetailIcon"
		});

		this.addSelectionMenuItem(viewItem, 
				true, 
				false,
				false);
				
		var createMenuItem = new dijit.MenuItem({
            label: this.localResources.CREATE_CHILD_ROLE,
            disabled: true,
            onClick: dojo.hitch(this,this._createChildRole),
            iconClass: "tipMenuIcon newIcon"
            });

		this.addSelectionMenuItem(createMenuItem, 
				true,   // boolean flag specifying whether the menu item should be active when a single row is selected.
				false,  // boolean flag specifying whether the menu item should be active when multiple rows are selected
				false); // boolean flag specifying whether the menu item should be active when no rows are selected.
				
		var deleteItem = new dijit.MenuItem({
			label: this.localResources.DELETE_ROLE,
			disabled: true,
			onClick: dojo.hitch(this,this._launchRoleDeleteDialog),
			iconClass: "tipMenuIcon deleteIcon"
		});

		this.addSelectionMenuItem(deleteItem, 
				true, 
				true,
				false);

		var copyMenuItem = new dijit.MenuItem({
			label: this.localResources.COPY_ROLE,
			disabled: true,
			onClick: dojo.hitch(this,this._copyRole),
			iconClass: "tipMenuIcon copyIcon"
		});

		this.addSelectionMenuItem(copyMenuItem, 
				true, 
				false,
				false);
		var errorWarningMenuItem = new dijit.MenuItem({
			label: this.localResources.ERRORS_WARNINGS,
			disabled: true,
			onClick: dojo.hitch(this,this._launchErrorWarningDialog),
			iconClass: "tipMenuIcon errorWarning"
		});

		this.addSelectionMenuItem(errorWarningMenuItem, 
				true, 
				false,
				false);
		
		var analyzeRoleMenuItem = new dijit.MenuItem({
			label: this.localResources.ANALYZE_ROLE,
			disabled: true,
			onClick: dojo.hitch(this,this._launchRoleAnalysisPane),
			iconClass: "tipMenuIcon analysisIcon"
		});

		this.addSelectionMenuItem(analyzeRoleMenuItem, 
				true, 
				false,
				false);
			
	},
	_setupLifecycleMenu : function(){
		
		if(this.lifecycleMenuHelper){
			this.addSelectionMenuItem(this.lifecycleMenuHelper.getMenuItem(
					dojo.hitch(this, this._launchLifecycleSubmitRequest)), 
				true, 
				true,
				false);
			
			var menuItem = this.lifecycleMenuHelper.getMenuItem(dojo.hitch(this, this._launchLifecycleSubmitRequest));
			
			this._addActionMenuItem(menuItem);
			this.setActivationBasedOnRoleSelection(menuItem,true,true,false);	
		}
	},
	destroy:function(){
		this.inherited(arguments);
		dojo.forEach(this.roleConnections, dojo.disconnect);
	},
	
	_getRoleUserCountDetailFrmTable:function(inRowIndex,inItem){
		var countString="";
		if(inItem){
			countString=this.dataStore.getValue(inItem, "totalIdentitiesCount")+","+this.dataStore.getValue(inItem, "inheritedIdentitiesCount");
		}
		return countString;
	},
	
	_formatUserCount:function(countString,inRowIndex, cell){
		var countDetail=countString.split(",");
		if(countDetail && countDetail.length===2){
			if(countDetail[1]==="0" || countDetail[1]===0){//the second index contains the inherited count
				countString=(""+countDetail[0]);//the first index contains the total count//the "" ensures its of type String
			}else{
				countString=dojo.string.substitute(this.localResources.USER_COUNT_TABLE, {"totalCount":countDetail[0],"inheritedCount":countDetail[1]});
			}
		}
		return com.ibm.security.modeling.util.HtmlEscapingUtils.normalizeString(countString);//calling the common tooltip formatter on Table;
	},
	
	_getRolePermCountDetailFrmTable:function(inRowIndex,inItem){
		var countString="";
		if(inItem){
			countString=this.dataStore.getValue(inItem, "totalPermissionsCount")+","+this.dataStore.getValue(inItem, "inheritedPermissionsCount");
		}
		return countString;
	},
	
	_formatPermissionCount:function(countString,inRowIndex, cell){
		var countDetail=countString.split(",");
		if(countDetail && countDetail.length===2){
			if(countDetail[1]==="0" || countDetail[1]===0){//the second index contains the inherited count
				countString=(""+countDetail[0]);//the first index contains the total count//the "" ensures its of type String
			}else{
				countString=dojo.string.substitute(this.localResources.PERMISSION_COUNT_TABLE, {"totalCount":countDetail[0],"inheritedCount":countDetail[1]});
			}
		}
		return com.ibm.security.modeling.util.HtmlEscapingUtils.normalizeString(countString);//calling the common tooltip formatter on Table
	},
	
	_getRoleDetailValueFrmTable:function(inRowIndex,item){
		return item;
	},
	
	_formatRoleNameValue:function(item,inRowIndex,cell){		
	        var anchor="";
		    if(item){
				var roleName=item.name;
				anchor=new com.ibm.security.modeling.util.Anchor({
						input:roleName,
						callback:dojo.hitch(this, this._launchRoleNotebook,item.uri)
				});											
		    }
			return anchor;

	},
	
	_getRoleStatusValueFrmTable:function(inRowIndex,inItem){
		var status = "normal";
		if(inItem){
			if (inItem.errorCount > 0) {status= "error";}
			else if (inItem.warningCount > 0) {status= "warning";}
		}
		return status;
		
	},
	
	_formatRoleStatusValue:function(status,inRowIndex,cell){
		//var statusDiv = "<div title=\'"+this.globalResources.OK+"\' class='normalIcon'></div>";
		var statusDiv = "";
		if (status==="error") {statusDiv = "<div title=\'"+this.globalResources.ERROR+"\' class='errorIcon'></div>";}
		else if (status==="warning") {statusDiv = "<div title=\'"+this.globalResources.WARNING+"\' class='warningIcon'></div>";}

		return statusDiv;
	},
	_statusIconColumnClickHandler:function(cell){
		this.messageArea.hideMessages();
		if(cell.cellIndex===1){//launching error panel only for first column(status column)
			var divClassName="";
			if(dojo.isIE){
				divClassName=cell.target.className;
			}else if(dojo.isFF){
				divClassName=cell.originalTarget.className;
			}
			if(divClassName==="errorIcon" || divClassName==="warningIcon" ){//if no need to launch for normal roles
				var roleItem=cell.grid.getItem(cell.rowIndex);
				if(roleItem){
					this.launcher.launchRoleErrorWarning(roleItem.uri.toString());
				}
			}
		}
	},
	formatObjectsToCommaSeparatedNamesCell:function(objects, rowIndex, cell){
		var value  = this.formatToolTipObjectsToCommaSeparatedNamesCell(objects, rowIndex, cell);
		return com.ibm.security.modeling.tables.FormatterUtils.formatLongCellValue(value);
	},
	formatToolTipObjectsToCommaSeparatedNamesCell:function(objects, rowIndex, cell){
		var names= [];
		var namesString = "";
		if(objects && objects.length > 0){
			names = this._createNamesArrayFromObjectArray(objects);
			namesString = this._createCommaSeparatedStringFromNameArray(names); 
		}
		return com.ibm.security.modeling.util.HtmlEscapingUtils.normalizeString(namesString);//calling the common formatter on Table		
	},
    
	_createNamesArrayFromObjectArray: function(/*Object[]*/items) {
		var names = [];
		for(var i=0,len=items.length;i<len;i++){
			var name=this.htmlUtilities.parseHTMLandNormalize(items[i].name);
			names.push(name);
		}
		names.sort();//adding sort due to problem in automation testing D11385
		return names;
	},
	_createCommaSeparatedStringFromNameArray: function(/*String[]*/names) {
		var namesString = "";
		for(var j=0;j<names.length;j++){
			namesString = namesString + names[j];
			if(j+1 !== names.length){
				namesString=namesString + ", ";
			}
		}	
		return namesString;
	},
	_launchErrorWarningDialog:function(){
		// summary:
		//			Launch role error warning dialog to see error/warning details. 
		this._trace.entering("", "_launchErrorWarningDialog");
		var items=this.getSelectedItems();
		if(items && items.length===1){
			this.launcher.launchRoleErrorWarning(items[0].uri.toString());
		}
		else{
			this._trace.error("Role error/warning view is only supported in single selection mode", "_launchErrorWarningDialog");
		}
		
		this._trace.exiting("", "_launchErrorWarningDialog");
	},

	_launchRoleDeleteDialog: function(){
		// summary:
		//			Launch role deletion dialog to choose the delete option. 
		this._trace.entering("", "_launchRoleDeleteDialog");
		this.messageArea.hideMessages();
		var items=this.getSelectedItems();
		if(items && items.length===1){
			this.launcher.launchDeleteRoleDialog(items[0].uri.toString(), items[0].id, items[0].name);
		}
		else{
			this.launcher.launchMultiRoleDelete(items);
		}
		
		this._trace.exiting("", "_launchRoleDeleteDialog");
	},
	
	_launchRolePropertiesDialog: function(){
		// summary:
		//			Launch the role properties dialog. 
		this._trace.entering("", "_launchRolePropertiesDialog");
		this.messageArea.hideMessages();
		var items=this.getSelectedItems();
		if(items && items.length===1){
			this.launcher.launchRolePropertiesDialog(items[0].uri.toString());
		}
		else{
			this._trace.error("Role edit is only supported in single selection mode", "_launchRolePropertiesDialog");
		}
		
		this._trace.exiting("", "_launchRolePropertiesDialog");
	},
	_launchRoleNotebook:function(uri){
		this.launcher.launchRolePropertiesDialog(uri.toString());
	},
	_launchExportProjectDialog:function(){
		// summary:
		//			Launch the Project export dialog. 
		this._trace.entering("", "_launchExportProjectDialog");
		this.messageArea.hideMessages();
		this.launcher.launchExportProjectDialog();
		
		this._trace.exiting("", "_launchExportProjectDialog");
	},	

	_launchGenerateRoles:function(){
		// summary:
		//			Launch the Generate role. 
		this._trace.entering("", "_launchGenerateRoles");
		this.messageArea.hideMessages();
		this.launcher.launchGenerateRoles();
		this._trace.exiting("", "_launchGenerateRoles");
	},
	
	_launchCopyRolesDialog:function(){
		// summary:
		//			Launch the Copy role dialog. 
		this._trace.entering("", "_launchCopyRolesDialog");
		this.messageArea.hideMessages();
		this.launcher.launchCopyRolesDialog();
		
		this._trace.exiting("", "_launchCopyRolesDialog");
	},
	
	_launchRoleAnalysisPane: function(){
		// summary:
		//			Launches role analysis pane. 
		this._trace.entering("", "_launchRoleAnalysisPane");
		this.messageArea.hideMessages();
		var items=this.getSelectedItems();
		if(items && items.length===1){
			this.launcher.launchRoleAnalysisPane(items[0].id);
		}
		
		this._trace.exiting("", "_launchRoleAnalysisPane");
	},
	_launchLifecycleSubmitRequest: function(requestType){
		var items=this.getSelectedItems();
		this.launcher.launchLifecycleSubmitRequest(items,requestType);		
	},
	
	_createRootRole:function(){
		// summary:
		//			Create new root role. 
		this._trace.entering("", "_createRootRole");
		this.messageArea.hideMessages();
		this.launcher.roleCreate(null, "", true);
		
		this._trace.exiting("", "_createRootRole");
	},

	_createChildRole:function(){
		// summary:
		//			Create new child role for the selected parent role. 
		this._trace.entering("", "_createChildRole");
		this.messageArea.hideMessages();
		var items=this.getSelectedItems();
		if(items && items.length===1){
			var parentRoleId = items[0].id;
			this.launcher.roleCreate(null, parentRoleId, false);
		}
		else{
			this._trace.error("Role creation is only supported in single selection mode", "_createChildRole");
		}
		
		this._trace.exiting("", "_createChildRole");
	},

	_copyRole: function(){
		// summary:
		//			Copies the selected role
		this._trace.entering("", "_copyRole");
		
		var items=this.getSelectedItems();
		if(items && items.length===1){
			this.launcher.roleCopy(null, items[0].id, items[0].name);
		}
		else{
			this._trace.error("Role Copy is only supported in single selection mode", "_copyRole");
		}
		
		this._trace.exiting("", "_copyRole");
	}
});

dojo.declare("com.ibm.security.modeling.tables.RoleModelingTableContextMenuHandler", com.ibm.security.modeling.tables.TableContextMenuHandler, {
	processContextMenuLaunch: function(e) {		
		// summary:
		//		This function will disable the right click option for Errors and Warnings if a role has zero error and warnings
		this.inherited(arguments);
		var items=this.table.getSelectedItems();
		if(items && items.length===1){
			if((items[0].errorCount==="0"||items[0].errorCount===0) && (items[0].warningCount==="0"||items[0].warningCount===0)){
				var menus=this.table.rightClickSelectionMenu.getChildren();
				if(menus[4]){//error and warning menu option
					menus[4].set('disabled', "true");
				}
			}
		}
	}
});