dojo.provide("com.ibm.security.modeling.tables.RolePermissionTable");

dojo.require("com.ibm.security.modeling.tables.PermissionTable");
dojo.require("com.ibm.security.modeling.tables._DisabledSelection");
dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("dojox.grid.enhanced.plugins.IndirectSelection");

dojo.requireLocalization("com.ibm.security.modeling.tables", "PermissionTableText");

dojo.declare("com.ibm.security.modeling.tables.RolePermissionTable", 
		[com.ibm.security.modeling.tables.PermissionTable, com.ibm.security.modeling.tables._DisabledSelection], {
	// namespace: [readonly] String
	// 			Unique identifier to the instance of portlet page where this item will exist
	namespace:"",
	
	isAddRemoveRendered: false,
	updateSelectionLogic: false,
	updateSelectionBehavior: false,
	
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.tables.RolePermissionTable"),
	

	postCreate:function() {
		this.inherited(arguments);
		dojo.addClass(this.domNode,"contentArea");
		if (this.isAddRemoveRendered) {
			this._setupToolbar();
		}
		if (this.updateSelectionLogic) {
			this._setupSelectionLogic();
		}
	},
	
	postMixInProperties:function() {
		
		// roleItem: object
		//		An object representing a Role for 'roleUri' in dojo store.
		
		// addPermissionCallback: function
		//		function to invoke Add permission
		
		// removePermissionCallback: function
		//		function to invoke remove permission 
		
		this.inherited(arguments);
		this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
		this.setCustomCols();
		this.structure = this._getStructurePermissions();
		if (this.updateSelectionLogic) {
			this.unSortableColumns = [3,4];
		}
	},
    setCustomCols:function(){
        if (this.updateSelectionLogic) { // Role's permissions.
        	this.customCols={1:{field: 'directlyAttached',name: this.permissionTableResources.HEADER_COLUMN_1,width: '8%', formatter: dojo.hitch(this, this._formatDirectlyAttachedColumn), isSearchable: false },
        		2:{field: 'inheritedFromRoles',name: this.permissionTableResources.HEADER_COLUMN_2,width: '15%', formatter: dojo.hitch(this, this._formatInheritedFromRolesColumn), isSearchable: false}};
	    }
    },
	
	_setupToolbar:function() {
		this.addToolbarSeparator();
		
		//add permission
		this.addToolbarButton(this.permissionTableResources.ADD,
				              false,
				              "addIcon",
				              dojo.hitch(this,this._launchAddPermission),
				              this.id+"_addPermissions");
		// remove permission
		this.addToolbarButton(this.permissionTableResources.REMOVE,		// Text describing the button (used as tooltip/alt text)
				              false,								// Whether to show the label in the button itself
				              "removeIcon",							// String describing the css class describing the button image
				              dojo.hitch(this,this._removePermission),// The function to be executed when the button is clicked.
				              this.id+"_removePermissions",					// The identifier used to locate the button.
				              true,									// boolean flag specifying whether the button should be active when a single row is selected.
				              true,									// boolean flag specifying whether the button should be active when multiple rows are selected
				              false);								// boolean flag specifying whether the button should be active when no rows are selected.
	},
	
	_launchAddPermission: function(){
		this._trace.entering(this.id, "_launchAddPermission");
		
		dojo.require("com.ibm.security.modeling.widgets.PermissionPickerDialog");
		
		var uri = this.roleItem.permissionCandidatesUri.toString();
		
		// Instantiate store for Role's Children.
		this._permissionCandidateStore = new com.ibm.security.modeling.tables.JsonRestStore({
			target: this._getTableStoreTargetUri(uri),
			idAttribute: 'id',
			syncMode: false
		});
		
		// Instantiate picker
		var permissionPicker = new com.ibm.security.modeling.widgets.PermissionPickerDialog({
			id: this.id+"_permissionPickerDialog", 
			tableQuery: this._getTableQueryObject(uri),
			fromDataStore: this._permissionCandidateStore,	// store for picker table.
			
			toDataStore: this.dataStore,	// this table's store
			toDataStoreTargetUri: this.roleItem.permissionsUri.toString(),
			toDataStoreResponsePayloadAttribute: 'analysis',
			
			callback: this.addPermissionCallback || null,
			parentWidget: this.parentWidget,	// for UnderLay
			parentMessageArea: this.messageArea,
			saveIndicator: this.saveIndicator,
			namespace:this.namespace,	// for help launch
			contextRoot: this.contextRoot,	// for help launch
			globalResources: this.globalResources,	// for picker table.
			helpFile: "ui_role_permissions_add.html"
		}, null);
		
		// Startup and show picker
		permissionPicker.startup();
		permissionPicker.show();
		
		this._trace.exiting(this.id, "_launchAddPermission");
	},
	
	_removePermission:function(){
		this._trace.entering(this.id, "_removePermission");
		var selectedRows = this.getSelectedItems();
		if (selectedRows == null || selectedRows.length <=0) {
			this._trace.exiting(this.id, "_removePermission");
			return;
		}
		// Show Save indicator
		if (this.saveIndicator) { this.saveIndicator.show(this.saveIndicatorText);}
		
		this.deselectAll();
		this._hideFooter();
		
		// Update store
		for (var j=0; j<selectedRows.length; j++) {
			this.dataStore.deleteItem(selectedRows[j]);
		}
		
		// When done.
		var processedItems = 0;
		var selectedItemsCount = selectedRows.length;
		var successCount = 0;
		var failureCount = 0;
		var self = this;
		
		var handleComplete = function(actions, value, isError) {
			if (isError == false) {
				successCount = actions.length;
			} else {
				failureCount = actions.length;
			}
			
			processedItems = processedItems + actions.length;
			if (processedItems == selectedItemsCount) {
				self.showMessages(value);
				if (self.removePermissionCallback) {
					self.removePermissionCallback([value, {successCount: successCount, failureCount: failureCount}]);
				}
				if (self.saveIndicator) {self.saveIndicator.hide(this.saveIndicatorText);}	// hide save indicator
				self._showFooter();
			}
		};
		
		
		// commit changes
		this.dataStore.save({
			handleAsBatch: true,
			
			targetUri: this.roleItem.permissionsUri.toString(),
			uriAttribute: "directPermission",
			responsePayloadAttribute: 'analysis',
			
			onComplete: dojo.hitch(this, function(args) {
				handleComplete(args[0], args[1], false);
			}),
			
			onError: dojo.hitch(this, function(args) {
				this._trace.error("Error while removing directly attached permissions.", "error callback");
				if (typeof args == Array) {
					handleComplete(args[0], args[1], true);
				} else {
					this._handleRESTError(args);
					if (this.saveIndicator) { this.saveIndicator.hide(this.saveIndicatorText);}	// hide save indicator
					this._showFooter();
				}
			})
		});
		this._trace.exiting(this.id, "_removePermission");
	},
	
	
	// Column formatters.
	_formatDirectlyAttachedColumn: function (value) {
		//this._trace.entering(this.id, "_formatDirectlyAttached");
		
		var cellContent = "<span ";
		if (value && (value == 'true' || value == true)) {
			cellContent = cellContent+ "class='checkIcon'";
			
			// hidden span for testing.
			cellContent = cellContent + "><span style='font-size: 0px;' class='formattedCell'>yes</span>";
		} else {
			// hidden span for testing.
			cellContent = cellContent + "><span style='font-size: 0px;' class='formattedCell'>no</span>";
		}
		cellContent = cellContent+ "</span>";
		//this._trace.exiting(this.id, "_formatDirectlyAttached");
		return cellContent;
	},
	
	_formatInheritedFromRolesColumn:function(list){
		//this._trace.entering(this.id, "_formatInheritedFromRolesColumn");
		
		var nameListString = "";
		if(list !=null && list.length && list.length>0){
			var nameList = null;
			
			if (typeof list[0] == 'string') {	// for Role Children, Role Parents tables
				nameList = list;
			} else if (typeof list[0] == 'object') { // for Role, Role Parent Candidates, Role Children Candidates table.
				nameList = new Array();
				for(var i = 0; i < list.length; i++){
					var inheritedFromRole = list[i].name.toString();
					var normalizedRoleName = com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLandNormalize(inheritedFromRole);
					nameList.push(normalizedRoleName);
				}
			}
			nameList.sort();//adding sort due to problem in automation testing D11385
			for(var j=0,len1=nameList.length;j<len1;j++){
				nameListString=nameListString+nameList[j];
				if(j+1!=len1){
					nameListString=nameListString+", ";
				}
			}
		}
		//this._trace.exiting(this.id, "_formatInheritedFromRolesColumn");
		return nameListString;
	},
	
	_getTableStoreTargetUri: function(targetUri) {
		if (targetUri.indexOf("?") != -1) {
			return targetUri.split("?")[0];
		}
		return targetUri;
	},
	
	_getTableQueryObject: function(targetUri) {
		if (targetUri.indexOf("?") != -1) {
			return dojo.queryToObject(targetUri.split("?")[1]);
		}
		return dojo.queryToObject(targetUri);
	},
	showMessages: function(value) {
		var mu = new com.ibm.security.modeling.util.MessageUtilities();
		var msgs = mu.extractBatchMessages(value, null, true);
		mu.showMessages(msgs, this.messageArea, true);
	},
	
	_handleRESTError: function (error, request) {
		this._trace.entering(this.id, "_handleRESTError");
		this._trace.finest(this.id + " - Error: " + error, "_handleRESTError");
		this._errorUtil.handleRESTError(error, this.messageArea);
		this._trace.exiting(this.id, "_handleRESTError");
	}
	
});