/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

�  Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.tables.RoleTable");

dojo.require("com.ibm.security.modeling.tables.Table");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.util.MessageUtilities");
dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
dojo.requireLocalization("com.ibm.security.modeling.tables", "RoleTableText");

dojo.declare("com.ibm.security.modeling.tables.RoleTable", com.ibm.security.modeling.tables.Table, {
	
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.tables.RoleTable"),
		
	// pluginResources: Object
	//			Reference to the plugin shared resource bundle
	pluginResources: null,
	
	//parentContainer: null,
	//saveIndicator: null,
	
	constructor: function() {
		this._trace.entering("", "constructor");
		if( (this.pluginResources == null) || (this.pluginResources == undefined) ) {
			this.pluginResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "PluginMessages");			
		}
		this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
		this._trace.exiting("", "constructor");
	},

	postMixInProperties:function() {
		this.autoHeight = false;
		this.isDynamic = true;
		this.isMultiSelect = true;
		this.unSortableColumns = [6,7];
		this.defaultSortIndex = 1;
		
		if (this.searchInstructions == this.tableResources.DEFAULT_SEARCH_INSTRUCTIONS) {
			this.searchInstructions = this.globalResources.SEARCH_ROLES;
		}
		if (this.entityTypeText == "") {
			this.entityTypeText = this.globalResources.ROLES;
		}
		
		if (this.structure == null || !this.structure || this.structure == undefined) {
			this.structure = this._getStructure();
		}
		
		if (this.isClientSide == true) { // client side, sort.
			this.dataStore.oldMakeComparator = this.dataStore.makeComparator;
			this.dataStore.makeComparator = this._makeComparator;
		}
		
		this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.tables", "RoleTableText");
		
		this.inherited(arguments);//this needs to be called after isDynamic is set
	},
	_getStructure: function() {
		return [
	        {field: 'name',name: this.globalResources.ROLE,width: '20%'}, 
	        {field: 'description',name: this.globalResources.DESCRIPTION,width: '25%',toolTipFormatter:function(value,rowIndex,cell){
	        	return com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(value );
    		},
    		formatter:function(value,rowIndex,cell){
    			return com.ibm.security.modeling.tables.FormatterUtils.formatLongCellValue(value);
    		}
	        
	        },
    		{field: 'totalIdentitiesCount',name: this.globalResources.USERS,width: '11%',formatter: dojo.hitch(this,this._formatUserCount), isSearchable:false},
    		{field: 'totalPermissionsCount',name: this.globalResources.PERMISSIONS,width: '15%',formatter: dojo.hitch(this,this._formatPermissionCount), isSearchable:false},
    		{field: 'parents',name: this.globalResources.PARENT_ROLES,width: '12%',toolTipFormatter: dojo.hitch(this, this._getParentChildObjectNameCommaSepList),formatter: dojo.hitch(this,this.formatObjectsToCommaSeparatedNamesCell), isSearchable:true},
    		{field: 'children',name: this.globalResources.CHILD_ROLES,width: '12%',toolTipFormatter: dojo.hitch(this, this._getParentChildObjectNameCommaSepList), formatter: dojo.hitch(this,this.formatObjectsToCommaSeparatedNamesCell) ,isSearchable:true}
    	];
	},
	formatObjectsToCommaSeparatedNamesCell:function(objects, rowIndex, cell){
		var value  = this._getParentChildObjectNameCommaSepList(objects, rowIndex, cell);
		return com.ibm.security.modeling.tables.FormatterUtils.formatLongCellValue(value);
	},
	postCreate:function() {
		this.inherited(arguments);
		dojo.addClass(this.domNode,"contentArea");
		
		this._setupGridCellGet();
	},
	
	createAddButton: function(/*string*/label,
			/*function*/callBackHandler) {
		this._trace.entering(this.id, "createAddButton");
		this.addToolbarButton(label, false, "addIcon",callBackHandler, this.id+"_add", true, true, true);
		this._trace.exiting(this.id, "createAddButton");
	},
	createRemoveButton: function(/*string*/label,
			/*function*/callBackHandler,
			/*function*/onComplete,
			/*boolean*/ showRESTError) {
		this._trace.entering(this.id, "createRemoveButton");
		if (callBackHandler == null) {
			callBackHandler = dojo.hitch(this, this._deleteSelectedRows);
			this.showRESTError = false;
		}
		if (onComplete) {
			this.onDeleteComplete = onComplete;
		}
		
		if (typeof showRESTError == 'boolean') {
			this.showRESTError = showRESTError;
		}
		
		this.addToolbarButton(label, false, "removeIcon", callBackHandler, this.id+"_delete", true, true, false);
		this._trace.exiting(this.id, "createRemoveButton");
	},
	_setupGridCellGet: function () {
		var usersCell = this.grid.getCell(3);
		if (usersCell != null) {
			usersCell.oldGet = usersCell.get;
			usersCell.get = this._get;
		}
		var permissionsCell = this.grid.getCell(4);
		if (permissionsCell != null) {
			permissionsCell.oldGet = permissionsCell.get;
			permissionsCell.get = this._get;
		}
	},
	
	_get: function(inRowIndex, inItem){	// Client side, sort.
		if ((this.field == "totalIdentitiesCount") ||
			(this.field == "totalPermissionsCount")) {
			return inItem;
		} else {
			return this.oldGet(inRowIndex, inItem);
		}
	},

	// Role Structure Field function.
	_formatUserCount:function(item){
		//this._trace.entering(this.id, "_formatUserCount");
		var countString=new String("");
		if(item){
			if(item.inheritedIdentitiesCount==0){
				countString=(new String(item.totalIdentitiesCount));
			}else{
				countString=this.localResources.USER_COUNT_TABLE.replace("${0}",(new String(item.totalIdentitiesCount)));
				countString=countString.replace("${1}",(new String(item.inheritedIdentitiesCount)));
			}
		}
		//this._trace.exiting(this.id, "_formatUserCount");
		return countString;
		
	},
	
	_formatPermissionCount:function(item){
		//this._trace.entering(this.id, "_formatPermissionCount");
		var countString=new String("");
		if(item){
			if(item.inheritedPermissionsCount==0){
				countString=(new String(item.totalPermissionsCount));
			}else{
				countString=this.localResources.PERMISSION_COUNT_TABLE.replace("${0}",(new String(item.totalPermissionsCount)));
				countString=countString.replace("${1}",(new String(item.inheritedPermissionsCount)));
			}
		}
		//this._trace.exiting(this.id, "_formatPermissionCount");
		return countString;
	},
	
	_makeComparator: function(sort) {	// Client side, sort
		//	summary:
		// 		returns a comparator function for the given sort order array.
		//		Overrides the JsonRestStore method of the same name to tweak
		//		the comparator to handle the users and permissions columns.
		//	sort:
		//		See dojox.data.api.Read.fetch
		if ((sort != null) &&
			(sort.length > 0) &&
			(sort[0] != null) &&
			((sort[0].attribute == "totalIdentitiesCount") || (sort[0].attribute == "totalPermissionsCount"))) {
			var current = sort.shift();
			if (!current) {
				// sort order for ties and no sort orders
				return function() {
					return 0;// keep the order unchanged
				};
			}
			var attribute = current.attribute;
			var descending = !!current.descending;
			var next = this.makeComparator(sort);
			var store = this;
			return function(a, b) {
				if (attribute == "totalIdentitiesCount") {
					var av = store.getValue(a, "totalIdentitiesCount");
					var bv = store.getValue(b, "totalIdentitiesCount");
					if (av == bv) {
						av = store.getValue(a, "inheritedIdentitiesCount");
						bv = store.getValue(b, "inheritedIdentitiesCount");
					}
					if (av != bv) {
						return av < bv == descending ? 1 : -1;
					}
				} else if (attribute == "totalPermissionsCount") {
					var av = store.getValue(a, "totalPermissionsCount");
					var bv = store.getValue(b, "totalPermissionsCount");
					if (av == bv) {
						av = store.getValue(a, "inheritedPermissionsCount");
						bv = store.getValue(b, "inheritedPermissionsCount");
					}
					if (av != bv) {
						return av < bv == descending ? 1 : -1;
					}
				}
				return next(a, b);
			};
		} else {
			return this.oldMakeComparator(sort);
		}
	}, 

	_getParentChildObjectNameCommaSepList:function(list, rowIndex, cell){
		//this._trace.entering(this.id, "_getParentChildObjectNameCommaSepList");
		
		var nameListString="";
		if(list!=null && list.length && list.length>0){
			var nameList = new Array();
			
			if (typeof list[0] == 'string') {	// for Role Children, Role Parents tables
				nameList = list;
			} else if (typeof list[0] == 'object') { // for Role, Role Parent Candidates, Role Children Candidates table.
				for(var i = 0; i < list.length; i++){
					nameList.push(list[i].name.toString());
				}
			}
			nameList.sort();//adding sort due to problem in automation testing D11385
			for(var j=0,len1=nameList.length;j<len1;j++){
				nameListString=nameListString+nameList[j];
				if(j+1!=len1){
					nameListString=nameListString+", ";
				}
			}
		}
		
		//this._trace.exiting(this.id, "_getParentChildObjectNameCommaSepList");		
		return com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLandNormalize(nameListString);
	},
	
	_deleteSelectedRows: function() {
		this._trace.entering(this.id, "_deleteSelectedRows");
		var selectedRows = this.getSelectedItems();
		var selectedRowsCount = selectedRows.length;
		this._trace.finer(this.id+' - Number of selected rows: '+selectedRowsCount, "_deleteSelectedRows");
		
		// validation: At-least one selection is required
		if (selectedRowsCount == 0) { 
			// control should never come here, as button enable/disable functionality is provided by common grid.
			return null;
		}
		
		this.deselectAll();
		this._hideFooter();
		if (this.saveIndicator) {
			this.saveIndicator.show(this.saveIndicatorText);	//show save indicator
		}
		// Update store
		for (var i=0; i<selectedRows.length; i++) {
			this.dataStore.deleteItem(selectedRows[i]);	// marks item for deletion
		}

		// When done
		var selectedItemsCount = selectedRows.length;
		var processedItems = 0;
		var successCount = 0;
		var failureCount = 0;
		var self = this;
		var handleComplete = function(actions, value, isError) {
			if (isError == false) {
				successCount = actions.length;
			} else {
				failureCount = actions.length;
			}
			processedItems = processedItems + actions.length;
			if (processedItems == selectedItemsCount) {
				self.showMessages(value);
				if (self.onDeleteComplete) {
					self.onDeleteComplete([value, {successCount: successCount, failureCount: failureCount}]);
				}
				self.saveIndicator.hide(self.saveIndicatorText);
				self._showFooter();
				self.refresh(); // refresh the table
			}
		};
		
		// commit changes
		this.dataStore.save({
			handleAsBatch: true,
			
			onComplete: dojo.hitch(this, function(args) {
				handleComplete(args[0], args[1], false);
			}),
			
			onError: dojo.hitch(this, function(args) {
				this._trace.error("Error while deleting.", "error callback");
				if (args instanceof Array) {
					handleComplete(args[0], args[1], true);
				} else if (this.showRESTError) {
					this.saveIndicator.hide(this.saveIndicatorText);
					this._handleRESTError(args);
					this._showFooter();
					this.refresh(); // refresh the table
				}
			})
		});
		this._trace.exiting(this.id, "_deleteSelectedRows");
		
	},
	
	_handleRESTError: function (error, request) {
		this._trace.entering(this.id, "_handleRESTError");
		this._trace.finest(this.id + " - Error: " + error, "_handleRESTError");
		this._errorUtil.handleRESTError(error, this.messageArea);
		this._trace.exiting(this.id, "_handleRESTError");
	},
	showMessages: function(value) {
		var mu = new com.ibm.security.modeling.util.MessageUtilities();
		var msgs = mu.extractBatchMessages(value, null, true);
		mu.showMessages(msgs, this.messageArea, true);
	}
	
});