dojo.provide("com.ibm.security.modeling.tables.Table");

dojo.require("dojox.data.ClientFilter"); 
dojo.require("com.ibm.security.modeling.tables.JsonRestStore");
dojo.require("dojo.data.ItemFileReadStore");
dojo.require("com.ibm.security.modeling.tables.CommonTableToolbar");
dojo.require("com.ibm.security.modeling.tables.TableStatus");
dojo.require("com.ibm.security.modeling.tables.CommonTableSearchField");
dojo.require("com.ibm.security.modeling.tables.TableToolbarButtonBuilder");
dojo.require("com.ibm.security.modeling.tables.TableContextMenuHandler");
dojo.require("com.ibm.security.modeling.layouts.TemplatedPanelWithLayout");
dojo.require("dojox.grid.EnhancedGrid");
dojo.require("dojox.grid.enhanced.plugins.Menu");
dojo.require("dojox.grid.enhanced.plugins.Pagination"); 
dojo.require("dojox.grid.enhanced.plugins.IndirectSelection");  
dojo.require("dojox.data.AndOrReadStore");
dojo.require("com.ibm.security.modeling.util.WidgetUtilities");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");
dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
dojo.require("com.ibm.security.modeling.widgets.ProgressBar");
dojo.require("dojox.grid.enhanced.plugins.Pagination");

dojo.require("dijit.form.Button");
dojo.require("dijit.Menu");
dojo.require("dijit.MenuItem");
dojo.require("dijit.ToolbarSeparator");
dojo.require("dijit.Tooltip");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("ibm.tivoli.tip.dijit.TIPTableToolbar");
dojo.require("ibm.tivoli.tip.dijit.TIPTableFooter");
dojo.require("com.ibm.security.modeling.tables.ToolTipHelper");
dojo.require("com.ibm.security.modeling.tables.FormatterUtils");

dojo.require("com.ibm.security.modeling.util.Trace");

dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling.tables", "TableText");
dojo.requireLocalization("com.ibm.security.modeling", "LogMessages");

dojo.declare("com.ibm.security.modeling.tables.Table", 
		[ com.ibm.security.modeling.layouts.TemplatedPanelWithLayout ] , {	
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.tables.Table"),
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.tables","templates/Table.html"),	
	widgetsInTemplate: true,
	container: null,
	id: "commonTable",
	totalSeletedInAllPages: [],
	totalSelected: 0,
	// isClientSide: boolean
	//     flag determining whether the table should be client or server side in its
	//     searching/filtering/sorting
	isClientSide: true,
	// autoHeight: int or boolean
	//     the height the table tries to set itself to in rows. False by default
	//     because we want the height to be dictated by the container.
	autoHeight: false,
	// defaultRowHeight: int
	//     the default height of each row in the table. Important when calculating
	//     the viewable area in the table (especially in IE). A
	//     value of -1 means we should just use the default set in the datagrid
	defaultRowHeight: -1,
	// rowsPerPage: Number
	//     The number of items to be fetched at a time
	rowsPerPage: 100,
	// defaultPageSize: Number
	//		The number of rows per page visible at a time by default
	defaultPageSize: 50,
	// isFilterVisible: boolean
	//     Whether to show the filter on the toolbar
	isFilterRendered: true,
	// isSelectAllRendered: boolean
	//     Indicates whether the select all, remove all toolbar buttons are visible
	isSelectAllRendered: false,
	// isRefreshVisible: boolean
	//     Indicates whether the refresh toolbar button is visible
	isRefreshRendered: true,
	// isPaginationEnabled: boolean
	//     Indicates whether the grid will use pagination
	isPaginationEnabled: true,
	// isTotalsFooterRendered: boolean
	//     Indicates whether the footer that displayes selected and total counts is
	//     visible
	isTotalsFooterRendered: false,
	// isMultiSelect: boolean
	//     Indicates whether selection mode is single or multi
	isMultiSelect: true,
	// isSearchMultiSelect: boolean
	//     Indicates whether selection mode of search options is single or multi
	isSearchMultiSelect: false,
	// defaultSortIndex: Integer
	//     the 0-based column index of the default sort
	//     changed in postMixIn depending upon needSelection , i.e with/without
	//     indirectSelection plugin
	defaultSortIndex: null,
	// isDefaultSortAscending: boolean
	//     true if the default sort is ascending, otherwise false
	isDefaultSortAscending: true,
	// isDynamic: boolean
	//     Specifies if the table will adapt based on window resizes.
	isDynamic: false,
	// searchString: String
	//     Currently applied search text for the table
	searchString: "*",
	// entityTypeText: String
	//     Translated String describing the items in the table.
	entityTypeText: "",
	// searchInstructions: String
	//     Translated String giving search instructions
	searchInstructions: "",
	// progressBar: String
	//     The widget used to display the progress of various taks. For e.g. the
	//     progress of the current invocation of search.
	progressBar: null,
	// sortInProgress: boolean
	//     A flag specifying if a sort request is already in progress, so that we
	//     don't fire recurrent sort requests on the same data, virtually leading to an
	//     infinite loop.
	sortInProgress: false,
	// searchInProgress: bool
	//     A flag specifying if a search request is already in progress, so that we
	//     can avoid
	//     computations that make more sense for a search request.
	searchInProgress: false,
	// _widgetUtilities: com.ibm.security.modeling.util.WidgetUtilities
	//     Utilities for widgets.
	_widgetUtilities: null,
	// readOnlyMode: boolean
	//     is the table read only?  will be used in toggleActivationButton
	readOnlyMode:false,
	// needSelection: boolean
	//     whether the table should support selection of rows
	needSelection:true,
	// query: object
	//     default query for table contents
	query:null,
	// htmlUtilities: com.ibm.security.modeling.util.StringUtils
	//     utility for escaping text to be displayed as html.
	htmlUtilities: new com.ibm.security.modeling.util.StringUtils(),
	_logMessageResources: null, 
	// loadingMessage: String
	//		Message to display when table data is getting loaded.
	loadingMessage: null,
	// errorMessage: String
	//		Message to display when table data fails to load.
	errorMessage: null,
	//by default search will not preserve selection 
	preserveSelectionOnSearch:false,
	
	// isSelectable: boolean
	//		Boolean to allow text selection within the grid.
	isSelectable: true,
	/********************Widget Lifecycle Methods************************/
	
	constructor: function(args) {
		// summary:
		//     A reusable table widget that provides an enhanced grid with header
		//     and footer toolbars.
		
		// grid: dojox.grid.EnhancedGrid
		//     the grid object itself
		// connections: connection[]
		//     array of connections made with this widget
		// unSortableColumns: Number[]
		//     1-based integer array of the columns where sort is disabled (starting
		//     with the selection column).
		// tableStatus: com.ibm.security.modeling.tables.TableStatus
		//     Object for handling the tables status stats in the footer.
		// singleSelectButtons: dijit.form.Button[]
		//     MenuItems in this table that are active when a single row is selected
		// multiSelectButtons: dijit.form.Button[]
		//     MenuItems in this table that are active when multiple rows are
		//     selected
		// noSelectButtons: dijit.form.Button[]
		//     Buttons in this table that are active when no rows are selected
		// contextMenuHandler: com.ibm.security.modeling.tables.TableContextMenuHandler
		//		Handles right click context menu processing
		// messageArea: ibm.tivoli.tip.dijit.TIPMessageArea
		//     Object into which messages should be placed.
		// searchField: ibm.tivoli.tip.dijit.TIPTableSearchField
		//     Represents the search filter "pill" area on the toolbar
		// structure: Object[]
		//     a definition of the table columns
		// dataStore: dojo.data.DataStore
		//     data to be used to populate the table. Plugin any compatible dojo
		//     datastore object.
		// header: ibm.tivoli.tip.dijit.TIPTableToolbar
		//     represents the toolbar/header for the table
		// footer: ibm.tivoli.tip.dijit.TIPTableFooter
		//     represents the footer area below the table that displays table stats
		// globalResources: Object
		//     Global resource bundle for common table text
		// tableResources: Object
		//     Resource bundle for table text
		// rightClickSelectionMenu: dijit.Menu
		//     The Object representing the menu you get when you right click a selected row
		// toolTipRequired: Number [] or String []
		//     Array containing 1-based column indices or names of columns that you want to show
		//     tooltips for on mouseover.
		
		this._trace.entering("", "constructor");
		this._trace.object(args, "constructor", "Constructor args");
		this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		this.tableResources = dojo.i18n.getLocalization("com.ibm.security.modeling.tables", "TableText");
		this._logMessageResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "LogMessages");
		this.progressBar = new com.ibm.security.modeling.widgets.ProgressBar();
		this.contextMenuHandler = new com.ibm.security.modeling.tables.TableContextMenuHandler();
		this.singleSelectButtons= [];
		this.noSelectButtons=[];
		this.multiSelectButtons= [];
		this.connections= [];
		this.unSortableColumns= [];
		this.toolTipRequired = [];
		this._widgetUtilities = new com.ibm.security.modeling.util.WidgetUtilities();
		this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
		dijit.Tooltip.defaultPosition = ["below", "above"];
		this._trace.exiting("", "constructor");
		
	},
	setSearchAndEntityText:function(){
	
		if(this.entityTypeText){
			this.searchInstructions = this.searchInstructions||dojo.string.substitute(this.tableResources.SEARCH_TEMPLATE,[this.entityTypeText.toLowerCase()]);
		}else if(this.searchInstructions){
			var searchInstruction = dojo.string.trim(this.searchInstructions);
			var arr = searchInstruction.split(this.globalResources.SEARCH);
			//first should be empty , assuming Search is first text in searchInstruction
			if( arr.length>=2 && arr[0]===""){
				this.entityTypeText = dojo.string.trim(searchInstruction.substr(this.globalResources.SEARCH.length));
			}
		}else{
			this.searchInstructions = this.tableResources.DEFAULT_SEARCH_INSTRUCTIONS;
		}
	},
	postMixInProperties: function() {
		// tags:
		// protected
		this._trace.entering(this.id, "postMixInProperties");
		this.inherited(arguments);
		if (this.isDynamic) {
			this.templatePath = dojo.moduleUrl("com.ibm.security.modeling.tables","templates/DynamicTable.html");
		} 
		// when indirectSelection is not there defaultSortIndex should be 0 else 1
		if(!this.defaultSortIndex) {this.defaultSortIndex = this.needSelection ? 1:0;}
		if (this.isPaginationEnabled) {
			this.tableStatus = new com.ibm.security.modeling.tables.PaginationTableStatus(this);
		} else {
			this.tableStatus = new com.ibm.security.modeling.tables.TableStatus(this);
		}
		this.setSearchAndEntityText();
		this._trace.exiting(this.id, "postMixInProperties");
	},
	postCreate:function() {
		// tags:
		// protected
		this._trace.entering(this.id, "postCreate");
		this.inherited(arguments);
		this._setupGrid();
		this.contextMenuHandler.attachToTable(this);
		this._setupDefaultButtons();
		this._setupFilter();
		if (this.isTotalsFooterRendered === true) {
			this._setupFooter();			
		}
		this.tooltipHandler = new com.ibm.security.modeling.tables.ToolTipHelper({table:this});
		this._setupSelectAllInfoMessageDisplay();
		this._trace.exiting(this.id, "postCreate");
	},	
	startup:function() {
		// tags:
		// protected
		this._trace.entering(this.id, "startup");
		if(this._started) return ;
		this.inherited(arguments);
		
		this._patchTable();
		this.grid.startup();
		this.rightClickSelectionMenu.startup();
		
		this._setupMappedValueColumnsStores();
		
		// For some reason, the client filtering logic needs to have the filter set.
		// Setting the query option on the creation of the grid doesn't do thetrick,
		// so we set it when the grid is started.
		if (this._isClientSideServiceStore()) {
			var query = this._getInitialClientSideQuery();	
			this.grid.filter(query);
		} 
		this._trace.exiting(this.id, "startup");
	},
	destroy:function() {
		// tags:
		//     protected
		this._trace.enteringWithStack(this.id, "destroy", true);
		try {
			dojo.forEach(this.connections, dojo.disconnect);	
			this.tableStatus.destroy();
			if(this.tooltipHandler) this.tooltipHandler.destroy();
			this.rightClickSelectionMenu.destroy();
			if(this.grid && this.grid.domNode){
				this.grid.destroy();
			}
			this.inherited(arguments);
			this._trace.exiting(this.id, "destroy");
		} catch(e) {
			this._trace.exception(e, "destroy");
			throw e;
		}
	},
	destroyRecursive: function() {
		this.destroy();
	},
	paginationUpdate:function(){
		if(this.isPaginationEnabled && this.grid.pagination.plugin.paginators[0]){
			this.grid.pagination.plugin.paginators[0].update();
		}
	},
	
	/***************Helper Methods for startup***************/
	
	_getInitialClientSideQuery : function() {
		// summary:
		//     gets the initial query for client side tables
		var query = {};			
		var firstField = this.structure[0].field;
		query[firstField] = "*";
		dojo.mixin(query,this.query);
		return query;
	},
	_patchTable:function() {
		// summary:
		//     Executes table modifications that are needed after the table is created.
		// tags:
		//     private
		if (this.isTotalsFooterRendered === true) {
			this._changeFooterText();
		}
		this._removeFilterWidgetsIfNotRendered();
	},
	_changeFooterText:function(){
		// summary:
		//     Sets the selected/total text at the bottom of the table.
		// description:
		//     defines the method to update the selected/total text in the DOM after footer creation, and sets it
		//     to the updateFooter method of the footer. The function is run within the scope of the footer. We
		//     keep this logic here because we use the 3rd party TIPFooter.
		//     Optimally, we should extend this class and keep this logic there.
		if (!this.needSelection) {
			this.footer.updateFooter = dojo.hitch(this, function(numSelected, totalRows) {
				var html = dojo.string.substitute(this.tableResources.TABLE_TOTAL, {
					0 : totalRows
				});
				//html = html.substring(html.indexOf(",") + 1, html.length); this code wont work as "," is also translated in arabic
				this.footer.setInfoAreaHTML(html);
			});
		}
	},
	_removeFilterWidgetsIfNotRendered: function() {
		// summary:
		// Removes the filter objects from the table after they've been created
		// description:
		// Right now we need to destroy these objects that are automatically
		// created by the tip table.
		// Optimally, we wouldn't create these in the first place.
		if (!this.isFilterRendered) {
			this.searchField.destroy();
			try{
			this.header.searchButton.destroy();
			}catch(e){
			    this._trace.exception(e, "_removeFilterWidgetsIfNotRendered");
			}
		}
	},
	
	/****************Helper Methods for postCreate****************/
	
	_setupFooter: function() {
		// summary:
		//     Initializes and the footer data algorithm, so the footer data can be
		//     updated as the contents and state of the table are changed.
		var footerId = this.id + "_footer";
		var footerDiv = '<div id="' + footerId + '"/>';
		if (this.isDynamic) {
			var footerContentPane = new com.ibm.security.modeling.dijit.layout.ContentPane({region:"bottom"});
			footerContentPane.set("content", footerDiv);
			this.container.addChild(footerContentPane);
		} else {
			dojo.place(footerDiv, this.gridContainer, "after");
		}
		this.footer = new ibm.tivoli.tip.dijit.TIPTableFooter({}, footerId);
		this.tableStatus.connectFooterStatus(this.connections);
	},	
	//For Client side table , refresh can be broken down into two parts
	//first bring all data from server ,then apply client side search . 
	//hence we are taking query and notification(onItem , onBegin , onComplete) from original request .
	//and onComplete of first request , passing the original request back for query portion 
	//NOTE : this is only done for client side table , and during refresh call only hence the variable dataStore.gotoServer
	overrideGridFetchForClientSide :function(){
		var originalFetch  = this.dataStore.fetch;
		var self = this;
		
		this.dataStore.fetch=function(request){
			
			if(self.dataStore.gotoServer){
				var secondQueryRequest = request;
				var onBegin = request.onBegin;
				var onItem = request.onItem;
				var onComplete = request.onComplete;
				var query = request.query;
				delete request.onBegin;
				delete request.onItem;
				delete request.onComplete;
				request.query = self.query ;
				dojo.hitch(self.dataStore ,originalFetch)(request);
				request.onComplete=function(arguments){
					self._updatePreservedSelections(self, arguments);
					self.dataStore.gotoServer = false;
					secondQueryRequest.onBegin = onBegin;
					secondQueryRequest.onItem = onItem;
					secondQueryRequest.onComplete = onComplete;
					secondQueryRequest.query = query;
					return dojo.hitch(self.dataStore ,originalFetch)(secondQueryRequest);
				};
				
				return request;
			}
						 
			return dojo.hitch(self.dataStore ,originalFetch)(request);
			
		};
	},
	
	_updatePreservedSelections: function(/*this*/ self, /*Array*/ allItems) {
		if (allItems && dojo.isArray(allItems) &&
				self.grid && self.grid.store && self.grid.selection 
				&& self.grid.selection.preserver && self.grid.selection.preserver._selectedById) {
			
			var itemIdsMap = {};
			for (var i=0; i<allItems.length; i++) {
				var item = allItems[i];
				if(item && self.grid._hasIdentity){
					var id = self.grid.store.getIdentity(item);
					if (id) {
						itemIdsMap[id] = null;
					}
				}
			}
			
			var preservedIdsMap = self.grid.selection.preserver._selectedById;
			for(var selection in preservedIdsMap) {
				if (selection && !(selection in itemIdsMap)) {
					delete preservedIdsMap[selection];
				}
			}
		}
	},
	
	_setupGrid: function() {
		// summary:
		// creates the grid for this table with initial values
		// tags:
		// private
		
		this.rightClickSelectionMenu = new dijit.Menu({id: this.id + "_rowMenu"});
		var pluginsObject = {};
		if (this.needSelection === true) {
			pluginsObject.indirectSelection = {"headerSelector": true};
		}
		pluginsObject.menus = {rowMenu: this.id+"_rowMenu"};
		if (this.isPaginationEnabled === true) {
			pluginsObject.pagination = {position: "bottom", sizeSwitch:true, pageSizes: ["10", "25", "50", "100"], defaultPageSize:this.defaultPageSize};	
		}
		if(this.isClientSide){
			this.overrideGridFetchForClientSide();
		}
		
		this.grid = new dojox.grid.EnhancedGrid({
				id: this.id+"_grid",
				queryOptions: {ignoreCase: true, cache: this.isClientSide},
				store: this.dataStore,
				structure: this.structure,
				autoHeight: this.autoHeight,
				query:this.query,
				clientSort: !this._isServerSideServiceStore(),
				elasticView: 1,
				rowSelector: "0",// hide the selector column since we use indirect selection
				rowsPerPage: this.rowsPerPage,
				canSort: dojo.hitch(this, this._canSort),
				selectionMode: this.isMultiSelect?"extended":"single",				
				plugins: pluginsObject,
				keepSelection: true,
				selectable: this.isSelectable
			},
			this.gridContainer
		);
		if (this._isServerSideServiceStore()) {//set initial sort for server tables
			//sortInfo is 1 based - default sort info is 0 based (needed for setSortIndex method).  Convert to one based by adding one.
			this.grid.sortInfo = (this.isDefaultSortAscending?1:-1) * (this.defaultSortIndex + 1);		
		}
		if (this.defaultRowHeight > -1) {
			this.grid.rowHeight = this.defaultRowHeight;
		}
		this.grid.noDataMessage = "<span class='noDataMessageStyle'>"+this.tableResources.NO_DATA+"</span>";
		
		this.grid.loadingMessage= this.progressBar.generateProgressBarDivWithMessage(this.loadingMessage? this.loadingMessage: this.grid.loadingMessage);
		this.grid.tableLoadingMessage=this.grid.loadingMessage;
		
		this.grid.onFetchError = dojo.hitch(this, this._onFetchError);
		this.grid.onCanSelect = function(inIndex) {
			var canSelect = true;
			if (this.rowSelectCell && this.rowSelectCell.disabledMap[inIndex]) {
				canSelect = false;
			}
			return canSelect;
		};
				
		if (this.isPaginationEnabled === true) {
			if(this.needSelection === true){
				this.tableStatus.connectFooterStatus(this.connections);
			}
		}
		this.grid.onHeaderCellClick = dojo.hitch(this, this._sortInvoked);
	},	
	_setupDefaultButtons:function() {
		// summary:
		// Sets up initial buttons based off of table preferences.
		// tags:
		// private
		if (this.isRefreshRendered) {
			//using function call in hitched function so that we can connect on refresh function
		    this.addToolbarButton(this.globalResources.REFRESH, false, "refreshIcon", dojo.hitch(this, function(){this.refresh();}), this.id+"_refresh");
		    //Add a seperator only if is select all is rendered.
		   
		}
		if(this.tableStatus) this.connect(this.tableStatus, "onSelectionChanged", "_updateButtons");
	
	},
	_updateButtons:function(numberOfSelectedRows) {	
		// summary:
		//     Updates Button activation based on rows selected in the table.
		// tags:
		//     private callback
		
		this._trace.info(this.id + " - Event received on this grid, updating buttons", "_updateButtons");
		var numberOfSelectedRows = numberOfSelectedRows || this.getSelectedCount();
		
		//Grid misfires onSelected event when Selectall is selected, even when no rows are there 
		var rowCount =this.grid.get('rowCount');
		if(rowCount==0) {numberOfSelectedRows=0;}
		// disable single selection menu Buttons
		this.toggleButtonActivation(this.singleSelectButtons, true);
		// disable multiple selection menu Buttons
		this.toggleButtonActivation(this.multiSelectButtons, true);
		// disable no selection menu Buttons
		this.toggleButtonActivation(this.noSelectButtons, true);
		// then only enable Buttons appropriate to the selections
		if(numberOfSelectedRows === 1) {
			/* There are instances whereby the number selected shows 1, but the actual content that 
			 * gets selected is null. One of the cases this happens is when we've got None displayed
			 * in the table. Given that, we need to ensure that we do not open up buttons for doing
			 * any activity if the number of selected is 1 and the value that gets selected is null. */
			var curSelectedItems = this.getSelectedItems();
			if(curSelectedItems[0] !== null) {
				this.toggleButtonActivation(this.singleSelectButtons, false);
			} else {
				this.toggleButtonActivation(this.noSelectButtons, false);
			}
		}
		if (numberOfSelectedRows > 1) {
			this.toggleButtonActivation(this.multiSelectButtons, false);			
		}
		if (numberOfSelectedRows === 0) {
			this.toggleButtonActivation(this.noSelectButtons, false);		
		}
	},
	toggleButtonActivation:function(/* dijit.form.Button[] */buttons, /* Boolean */isDisabled) {
		// summary:
		//     iterates through the given buttons and enables/disables them
		// buttons: dijit.form.Button[]
		//     array of buttons to activate/deactivate
		// isDisabled: Boolean
		//     Whether you want the buttons enabled or disabled.
		// tags: private
		dojo.forEach(buttons, dojo.hitch(this, function(button) {
		// if readOnlyMode is on disable all the button
			if ( this.readOnlyMode || isDisabled === true) {
				this._widgetUtilities.disableButton(button);
			} else {
				this._widgetUtilities.enableButton(button);
			}
		}));
	},
	_setupFilter: function() {
		// summary:
		// sets up the filter for use
		// description:
		// tests the text inside the filter field, sets the grid to the filter
		// widget, and sets up
		// necessary event handling for filter events.
		// tags:
		// private
		this.searchField = this.header.searchField;
		if (this.isFilterRendered) {
			this.searchField.setTable(this.grid);
			this.searchField.searchInstructions = this.searchInstructions;
			this.subscribe(this.searchField.id+"searchInvoked", dojo.hitch(this, this._tableSearchInvoked));
			this.connections.push(dojo.connect(this.header.searchButton, "onClick", this, this._searchButtonClicked));
			this.connections.push(dojo.connect(this.grid, "_onFetchComplete", this, this._handleFetchComplete));
			//Since we need this before grid fetch , so wrapping grid fetch method , can think of a better solution .
			var originalFetch = this.grid._fetch ;
			var self = this;
			this.grid._fetch = function(start, isRender){
				if(self.isFilterRendered) self.header.setSearchDisable();
				dojo.hitch(self.grid,originalFetch)(start, isRender);
			};
			
			
			
			
		} 
		
		/*
		 * We don't seem to handle the table fetch complete event, if we are
		 * dealing with client side tables that don't have search enabled.
		 * That's causing issues in having the records sorted by default, when
		 * the table comes up. We need to know if the table load is complete,
		 * since there are many client side tables working with async JSON REST
		 * stores and hence, fetchcomplete seems to be the event to get hold of
		 * to sort the results.
		 */
		if (!this._isServerSideServiceStore() && 
			!this.isFilterRendered) {
				this.connections.push(dojo.connect(this.grid, "_onFetchComplete", this, this._handleFetchComplete));
		}
	},
	_searchButtonClicked:function(event) {
		// summary:
		//     Handle search button click by publishing a search event
		// tags:
		//     private callback
		this._trace.info(this.id + " - search button clicked", "_searchButtonClicked");
		dojo.publish(this.searchField.id + "searchInvoked", [this.searchField, event]);
	},
	_updateSearchResultsDisplay: function(items){
		// summary:
		// updates the text that appears above the table that specifies the
		// number of search results
		// and what the currently applied filter is.
		// items: Object[]
		// the search results.
		// tags:
		// private callback
		var noDataMessage = this.tableResources.NO_DATA;
		// We need this section for evaluating whether or not to display None,
		// in case there are no records
		// in the table. Hence, pulled it out to the top of the function.
		var numberOfResults = 0;
		// if (this.dataStore instanceof
		// com.ibm.security.modeling.tables.JsonRestStore) {
		
			if (this.isPaginationEnabled) {
				numberOfResults = this.grid.pagination.plugin.getTotalRowCount();			
			}else{
				numberOfResults = items.length;			
			}
	
		// change the style class to hide the text if there is no filter
		// applied, otherwise display with approriate data
		if (this.searchString === "*" || this.searchString === "" || (!this.searchField.isSearchSet())) {
			dojo.removeClass(this.searchResultsDisplay, "searchResultsVisible");
			dojo.addClass(this.searchResultsDisplay, "searchResultsHidden");

			if(this.isPaginationEnabled){
				this.searchQuery=this.query;
				this.grid.pagination.plugin.query = this.searchQuery;
			}
		} else {
			var displayTemplate = this.tableResources.SEARCH_DISPLAY;
			var displayText = dojo.string.substitute(displayTemplate, {
				"numResults":numberOfResults, 
				"entityType": this.entityTypeText, 
				"searchString": this.htmlUtilities.parseHTMLTag(this.searchString)
			});
			this.searchResultsDisplay.innerHTML = displayText;
			dojo.removeClass(this.searchResultsDisplay, "searchResultsHidden");
			dojo.addClass(this.searchResultsDisplay, "searchResultsVisible");			
			if(this.isPaginationEnabled){
				this.grid.pagination.plugin.query = this.searchQuery;
			}
		}

		// Need to change the style of the messagenode of the grid. We need to
		// display None as left-aligned
		// as is typically shown in prototypes. If we have results, we need to
		// ensure that we set back the
		// style.
		if (items.length === 0) {
			dojo.addClass(this.grid.messagesNode,"noDataMessageStyle");
			/*
			 * There seems to be a glitch with the common grid code. The
			 * noDataMessage parameter seems to have stopped working all of a
			 * sudden. Explicitly setting the noDataMessage and pulling it off
			 * from the grid setting. It was working just couple of days back.
			 */
			this.grid.messagesNode.innerHTML = noDataMessage;
		} else {
			dojo.removeClass(this.grid.messagesNode,"noDataMessageStyle");
		}
	
	},
	
	_isSearchFilterActive: function() {
		// summary:
		//     Checks if the table is under the influence of a search, i.e. a search with non-default 
		//	   filter. 
		if (this.searchString === "*" || this.searchString === "" || (!this.searchField.isSearchSet())) {
			return false;
		}
		return true;
	},
	
	_handleFetchComplete: function(items) {
		// summary:
		//     piggy backs on the completion of fetching data for the table, to do
		//     some additional common table updates based on the new result set.
		// tags:
		//     private callback
		this._trace.entering("", "_handleFetchComplete");
		/*
		 * Ensure that the search results are displayed only in the event we are
		 * here because of a search completion rather than purely a fetch
		 * completion (for e.g. the table startup). For e.g. the Owners table.
		 * There we are not having any search operations, but we do need to sort
		 * the records once they are loaded.
		 */
		if(this.isFilterRendered) this.header.setSearchEnable();
		if (this.searchInProgress) {
			this._trace.finest("search in progress: updating results display", "_handleFetchComplete");
			this._updateSearchResultsDisplay(items);
			if (this.isDynamic) {
				this._trace.finest("search in progress: resizing container for search results display", "_handleFetchComplete");
				this.resize();
			}
		}
		
		/*
		 * If its a client side table that we are dealing with, sort the records
		 * rendered currently in the table.
		 */
		if(!this._isServerSideServiceStore()) {
			this._trace.finest("client table: sorting...", "_handleFetchComplete");
			this.sortClientTable();
		}
		
		/*
		 * We need to hide the progress bar, in case we are in this function as
		 * a result of a search request, rather than the basic startup request.
		 */
		if (this.searchInProgress) {
			this._trace.finest("search in progress: hiding progress bar", "_handleFetchComplete");
			this.progressBar.hide();
			this.grid.loadingMessage=this.grid.tableLoadingMessage;//reset loading progress bar after search is complete
			this.searchInProgress = false;
			if (this.isPaginationEnabled) {
				this._trace.finest("search in progress: reseting pager to first page", "_handleFetchComplete");
				this.grid.pagination.plugin.gotoFirstPage();
			}
		}
		else{
			//if no item is return we will go to last page , if we already on last page , gotoPage will prevent any filter call to server 
			//since the solution is written in handle page complete , the solution is recursive , and break point is goPage itself . 
			if(!items || items.length == 0) this.grid.pagination.plugin.gotoLastPage();
		}
		
		dojo.publish(this.id+"_handleFetchComplete", "");
		this._trace.exiting("", "_handleFetchComplete");
	},

	sortClientTable: function() {	
		// summary:
		// executes a client side sort based on current sort variables
		
		/*
		 * Need to invoke this request only in case sort is not already running.
		 * The sort request would end up in this exact same function, hence, it
		 * essential that we break the request and end-up in an infinite loop.
		 */
		this._trace.entering(this.id, "sortClientTable");
		if(!this.sortInProgress) {
			this.sortInProgress = true;
			/* In case its a new sort from scratch. */
			if (this.grid.sortInfo === 0) {
				this._trace.finest("sorting table column " + this.defaultSortIndex, "sortClientTable");
				this.setSortedColumn(this.defaultSortIndex, this.isDefaultSortAscending);
			} 
		} else {
			this.sortInProgress = false;
		}
		this._trace.exiting(this.id, "sortClientTable");
	},
	_sortInvoked:function(event) {
		this._trace.entering(this.id, "_sortInvoked");
		if(!this._isSelectColumnIndex(event.cellIndex)){ 
			dojo.publish(this.id+"_sortInvoked","");
			var columnIndex = event.cell.index;
			var isColumnSortable = this.grid.canSort(columnIndex+1);
			this.grid.setSortIndex(columnIndex);
			this.grid.onHeaderClick(event);
		}
		this._trace.exiting(this.id, "_sortInvoked");
	},
	_isSelectColumnIndex:function(index) {
		return this.needSelection && index===0?true:false;
	},
	_tableSearchInvoked:function(searchObj) {
		// summary:
		//     Executes search when a search event is published by the filter object.
		// tags:
		//     private callback
		this.searchInProgress = true;
		//Search is invoked , flush the selection count 
		dojo.publish(this.id + "_tableSearchInvoked", [searchObj]);
		this.grid.loadingMessage="";//clearing loading message as table has separate progress bar for search
		//remove all the selection 
		this.resetAllSelection();
		//Show the progress bar for the current search operation.
		this.progressBar.show(this.tableResources.SEARCH_PROGRESS);
		this.resetCurrentPageVariable();
		var selections = searchObj.getSelectedColumns();
		this.searchString = searchObj.getSearchFieldValue();
		this._trace.info(this.id + " - Search string is " + this.searchString, "_searchButtonClicked");
		var decoratedSearchString = this._decorateSearchString();		
		var query = null;
		if (this.dataStore instanceof dojox.data.AndOrReadStore) {
			query = this._handleAndOrQuery(selections, decoratedSearchString);
		} else {
			query = this._handleRestQuery(selections, decoratedSearchString);
		}
		dojo.mixin(query,this.query);
		this.searchQuery=query;
		this._trace.object(query, "_tableSearchInvoked", this.id + " - Search Query");
		this.grid.filter(query, true);
	},
	_handleAndOrQuery:function(selections, decoratedSearchString) {
		// summary:
		//     Builds the query string for an dojox.data.AndOrReadStore
		// selections: String[]
		//     An array of selected columns to search on
		// decoratedSearchString: String
		//     The search string entered by the user decorated with appropriate wildcards
		// returns:
		//     A query string to be passed to a grid filter
		// tags:
		//     private
		
		var queryString = "";
		dojo.forEach(selections, function(selection, index) {
			if (index !== 0) {
				queryString = queryString + " OR ";
			}
			
			queryString = queryString + selection + ":" + decoratedSearchString;
		});
		return queryString;	// return String
	},
	
	_setupMappedValueColumnsStores: function() {
		var mappedValueColumns = this._getMappedValueColumns();
		if (mappedValueColumns && mappedValueColumns.length > 0) {
			dojo.forEach(mappedValueColumns, function(column) {
				this._setupMappedValueColumnStore(column);
			}, this);
		}
	},
	
	_setupMappedValueColumnStore: function(/*Object*/ column) {
		if (column.mappedValueStore) { // already setup
			return;
		}
		var valueLabelResourceConfiguration = column["valueLabelResourceConfiguration"];
		var nlsResource = valueLabelResourceConfiguration["nlsResource"];
		var keyPrefix = ("keyPrefix" in valueLabelResourceConfiguration) ? valueLabelResourceConfiguration["keyPrefix"] : null;
		var keyPostfix = ("keyPostfix" in valueLabelResourceConfiguration) ? valueLabelResourceConfiguration["keyPostfix"] : null;
		column.mappedValueStore = this._createMappedValueStore(nlsResource, keyPrefix, keyPostfix);
	},
	_getMappedValueColumnStore: function(/*Object*/ column) {
		if (!column.mappedValueStore) {
			this._setupMappedValueColumnStore(column);
		}
		return column.mappedValueStore;
	},
	_createMappedValueStore: function(/*Object*/ nlsResource, /*string*/ keyPrefix, /*string*/keyPostfix) {
		var startsWith = function(key, prefix) {
			return key.indexOf(prefix) === 0;
		};
		var endsWith = function(key, postfix) {
			return key.match(postfix+"$") == postfix;
		};
		
		var items = [];
		for (var mappedKey in nlsResource) {
			if ((keyPrefix && !startsWith(mappedKey, keyPrefix))
					|| (keyPostfix && !endsWith(mappedKey, keyPostfix))) {
				// skip this key
				continue;
			}
			
			var fromCharIndex = (keyPrefix && dojo.string.trim(keyPrefix) != "")? (keyPrefix.length) : 0; 
			var toCharIndex = (keyPostfix && dojo.string.trim(keyPostfix) != "")? (mappedKey.lastIndexOf(keyPostfix)) : mappedKey.length -1;
			var i18nKeyValue = {
				"code": mappedKey.substring(fromCharIndex, toCharIndex+1),
				"text": nlsResource[mappedKey]
			};
			items.push(i18nKeyValue);
		}
		
		return new dojo.data.ItemFileReadStore({
			data: {
				"items" : items
			}
		});
	},
	
	_getCodesForMappedValueColumn: function(/*String*/ columnName, /*Array?*/ mappedValueColumns, /*string*/ decoratedSearchString) {
		var codes = null;
		if (!mappedValueColumns) {
			mappedValueColumns = this._getMappedValueColumns();
		}
		if (mappedValueColumns && mappedValueColumns.length > 0) {
			var mappedColumns = dojo.filter(mappedValueColumns, function (column) {
				return column.field == columnName;
			});
			if (mappedColumns && mappedColumns.length > 0) {
				var selectedMappedColumn = mappedColumns[0];
				var columnStore = this._getMappedValueColumnStore(selectedMappedColumn);
				// perform search on 'text' and collect 'code'
				codes = this._fetchMappedCodes(columnStore, decoratedSearchString);
			}
		}
		return codes;
	},
	
	_fetchMappedCodes: function(/*dojo.data.ItemFileReadStore*/ store, /*string*/ decoratedSearchString) {
		var codes = [];
		// synchronous
		store.fetch({
			query: {text: decoratedSearchString},
			onComplete: function(items) {
				if (items && items.length > 0) {
					dojo.forEach(items, function(item) {
						if (item && item.code) {
							if (dojo.isArray(item.code) && item.code.length > 0) {
								codes.push(item.code[0]);
							} else {
								codes.push(item.code);
							}
						}
					});
				}
			},
			queryOptions: {
				ignoreCase: true
			},
			scope: this
		});
		
		return codes;
	},
	
	_getMappedValueColumns: function() {
		// Summary:
		//		Mapped value column is a column that displays text from nls resources per the actual value.
		//		It is determined by 'valueLabelResourceConfiguration' field of the table structure column object.
		//		'valueLabelResourceConfiguration' has following object structure:
		//		{
		//			"nlsResource": 	<nls resource bundle object>,
		//			"keyPrefix":	<prefix to the actual value>,
		//			"keyPostfix":	<postfix to the actual value>
		//		}
		//
		//	Returns:
		//		Array - Containing mapped value columns.
		
		return dojo.filter(this.structure, function (column) {
			return (
					column && ("valueLabelResourceConfiguration" in column) 
					&& column["valueLabelResourceConfiguration"] != null 
					&& ("nlsResource" in column["valueLabelResourceConfiguration"]) 
					&& column["valueLabelResourceConfiguration"]["nlsResource"] != null); 
		});
	},
	
	_handleRestQuery:function(selections, decoratedSearchString) {
		// summary:
		//     Builds the query string for an dojox.data.JsonRestStore
		// selections: String[]
		//     An array of selected columns to search on
		// decoratedSearchString: String
		//     The search string entered by the user decorated with appropriate wildcards
		// returns:
		//     A query string to be passed to a grid filter
		// tags:
		//     private
		var filterParams = {};
		
		var mappedValueColumns = null;
		if (this.isClientSide === false) {
			mappedValueColumns = this._getMappedValueColumns();
		}
		
		dojo.forEach(selections, function(selection, index) {
			// Handle search for the mapped value column
			var mappedValueColumnCodes = null;
			if (mappedValueColumns && mappedValueColumns.length > 0) {
				mappedValueColumnCodes = this._getCodesForMappedValueColumn(selection, mappedValueColumns, decoratedSearchString);
			}
			if (mappedValueColumnCodes && mappedValueColumnCodes.length > 0) {
				var searchedAndMappedValues = [];
				searchedAndMappedValues.push(decoratedSearchString);
				dojo.forEach(mappedValueColumnCodes, function(code) {
					searchedAndMappedValues.push(code);
				});
				
				filterParams[selection] = searchedAndMappedValues;
			} else {
				filterParams[selection] = decoratedSearchString;
			}
		}, this);
		return filterParams;
	},
	_decorateSearchString:function() {
		// summary:
		//     decorates the currently set search string with wildcards, enabling a substring
		//     search for user entered searches.
		// description:
		//     If the user has already entered wildcard characters (*), then we
		//     honor the user's original intentions and do not add the substring search decorations.
		//if search is client side , we need to do the escaping 
		
		//escaping all \ , so that javascript do not understand it as an special character .
		// say \" means " in javascript , similary \\ means \ . so changing all \ by \\ will make javascript understand that 
		// we have give a \ for search .
		var newStr = (this.isClientSide && dojo.isString(this.searchString))?this.searchString.replace(/\\/g,"\\\\"):this.searchString;
		
		if (this.searchString === "") {
			newStr = "*";
		} else if (this.searchString.indexOf("*") === -1) {
			newStr = "*" + newStr + "*";
		}
		return newStr;// return String
	},
	mockSearch:function(/* String */field,/* String */value){
		// summary:
		//     Makes search to table, puts value in search box and clicks on search button.
		// field: String
		//     Column field on which it is to be searched if nothing is specified it will search on all column
		// value: String
		//     Value to be searched
		
		this.searchField.selectColumnCheckBox(field);
		this.searchField.setSearchValue(value);
		this._tableSearchInvoked(this.searchField);
	},
	clearSearch:function(){
		// summary:
		//     Clears search from table
		
		if(dojo.hasClass(this.searchResultsDisplay,"searchResultsVisible")){
			dojo.removeClass(this.searchResultsDisplay, "searchResultsVisible");
			dojo.addClass(this.searchResultsDisplay, "searchResultsHidden");
			this.resize();
		}
		if(dojo.hasClass(this.grid.messagesNode,"noDataMessageStyle")){
			dojo.removeClass(this.grid.messagesNode,"noDataMessageStyle");
		}
		this.searchField.clearSearchValue();
	},
	
	_canSort:function(/* Integer */index) {
		// summary:
		//     Answers whether a column at the given index can be sorted.
		// index: Integer
		//     1-based index representing columns
		// returns:
		//     Boolean representing whether the given column is sortable
		// tags:
		//     private
		var isSortable = true;
		dojo.forEach(this.unSortableColumns, function(colNum, i){
			if (colNum === index) {
				isSortable = false;
			}
		});
		return isSortable; // return Boolean
	},
	
	_setupSelectAllInfoMessageDisplay: function() {
		
	//	since there is no triger to know selectall checkbox is checked or not , we are capturing this .
		this.connections.push(dojo.connect(this.grid.rowSelectCell, 'toggleAllSelection', this, '_handleSelectAllMessageDisplay'));
		this.connections.push(dojo.connect(this ,"_handleFetchComplete", this, function(){
		this._handleSelectAllMessageDisplay(false);
			
		}));

	},
	_handleSelectAllMessageDisplay:function(checked){
	  
		this._trace.entering("Checked status: " + checked, "_handleSelectAllMessageDisplay");
		
			if (this.isPaginationEnabled === true){
				if(checked){
					 this.displaySelectAllMessage();				
				}else{
					this.hideSelectAllMessage();
				}			
			}
		this._trace.exiting("", "_handleSelectAllMessageDisplay");
	},	
	
	displaySelectAllMessage:function(){
		this._trace.entering("", "displaySelectAllMessage");
		if (this.isPaginationEnabled === true){			
			var numberOfRows = this.grid.pagination.plugin.getTotalRowCount();
			if (numberOfRows > this.grid.get('rowCount')){
				this._trace.finest("rows is greater than page", "displaySelectAllMessage");
				var disabledCount = 0;	
				if(this.grid.rowSelectCell && this.grid.rowSelectCell.disabledMap){//get disable count for checkboxes of table
					for (var i in this.grid.rowSelectCell.disabledMap) {
						if (this.grid.rowSelectCell.disabledMap[i] === true){
							disabledCount++;
						}
					}
					var enabledCount = numberOfRows - disabledCount;
					if(enabledCount <=0){ //check if any item is present for selection
						return;//dont display select all message
					}
				}
				this._trace.finest("publishing show message event", "displaySelectAllMessage");
				if ((this.messageArea !== undefined) && (this.messageArea !== null)){
					this._trace.finest("found a message area to post to", "displaySelectAllMessage");					
					if(this.messageArea.hideMessages !== undefined) {this.messageArea.hideMessages();}// In case it is a TIPMessageArea instead of a MultiMessageArea
					this._selectMessageId = this.messageArea.showInfoMessage(this._logMessageResources.TABLE_SELECTION_CURRENT_PAGE_ONLY);
					
				}
			}
		}
		this._trace.exiting("", "displaySelectAllMessage");
	 },
	
	
	
	hideSelectAllMessage:function(){
		
		if(this.messageArea && this.messageArea.hideMessages !== undefined && this._selectMessageId ) {this.messageArea.hideMessage(this._selectMessageId);}// In case it is a TIPMessageArea instead of a MultiMessageArea
		this._selectMessageId  = null;
			
	},
	
	_isClientSideServiceStore:function() {
		// summary:
		//     Answers whether the the table is "client side" enabled, and that it is an instance of dojox.data.ServiceStore
		// description:
		//     Client caching for the column table means that all items are fetched into
		//     the datastore and searching and sorting are done with the data loaded in the store (client only).
		// returns:
		//     Whether this table is client enabled, and it has a store that supports it.
		// tags:
		//     private
		return this.isClientSide === true && this.grid.store instanceof dojox.data.ServiceStore;
	},
	
	_isServerSideServiceStore:function() {
		// summary:
		//     Answers whether the the table is "server side" enabled, and if it is using
		//     an instance of dojox.data.ServiceStore
		// description:
		//     Server side enabled means that the table data is loaded incrementally
		//     in "pages", and that searching and sorting are done server-side.
		// returns:
		//     Whether this table is client enabled, and it has a store that supports it.
		// tags:
		//     private
		return this.isClientSide===false && this.dataStore instanceof dojox.data.ServiceStore;
	},
	
	_onFetchError: function(err, req) {
		// summary:
		//     handles an error fetching data from the datastore.
		// description:
		//     Parses the error response text and displays the appropriate error in the
		//     table message area.
		// tags:
		//     private callback
		try{
		if(this.isFilterRendered) this.header.setSearchEnable();
		this.grid.showMessage(this.grid.errorMessage);
		this._errorUtil.handleRESTError(err, this.messageArea);
		}catch(e){
			this._trace.exception(e, "_onFetchError");
		}finally{
			if (this.searchInProgress) {// hide the search progress bar, since
				this.progressBar.hide();
				this.searchInProgress = false;
			}
		}
	},
	
	/****************Public methods available for clients to call********************/
	
	addButtonWithBuilder: function() {
		// summary:
		//     Enables adding buttons to the table through a builder object.
		// description:
		//     Chain builder methods to configure behavior of the button.
		//     Sample Usage:
		//     <pre>
		//         myTable.addButtonWithBuilder()
		//             .withVisibleLabel(this.localResources.DELETE)
		//             .withIconClass("deleteIcon")
		//             .withId("deleteButton")
		//             .withActivationForSingleSelect()
		//             .withActivationForMultiSelect().build();
		//     </pre>
		return new com.ibm.security.modeling.tables.TableToolbarButtonBuilder(this);
	},
	
	
	addToolbarButton: function(/* String */label, 
			/* boolean */showLabel,
			/* String */iconClass,
			/* function */onClick,
			/* String */id,
			/* boolean */activateOnSingleSelect,
			/* boolean */activateOnMultiSelect,
			/* boolean */activateOnNoSelect,
			/* menu Object */menuObject,
			/* Tooltip dialog */ tooltipDialog,
			onFooter) {
		// summary:
		//     Adds a toolbar button the the header bar with the specified
		//     attributes.  Use addButtonWithBuilder for a more readable interface.
		// label: String
		//     Text describing the button (used as tooltip/alt text)
		// showLabel: boolean
		//     Whether to show the label in the button itself
		// iconClass: String
		//     String describing the css class describing the button image
		// onClick: function
		//     The function to be executed when the button is clicked.
		// id: String
		//     The identifier used to locate the button.
		// activateOnSingleSelect: boolean
		//     boolean flag specifying whether the button should be active when a
		//     single row is selected.
		// activateOnMultiSelect: boolean
		//     boolean flag specifying whether the button should be active when
		//     multiple rows are selected
		// activateOnNoSelect: boolean
		//     boolean flag specifying whether the button should be active when no
		//     rows are selected.
		// menuObject: dijit.Menu
		//     Object contains menu object
		// tags: 
		//     public deprecated
		var button = this._widgetUtilities.createButton(
				label,
				showLabel,
				iconClass,
				onClick,
				id,
				true,
				menuObject,
				tooltipDialog);
		
		if (activateOnSingleSelect === true || activateOnSingleSelect === undefined) {
			this.singleSelectButtons.push(button);
		}
		if (activateOnMultiSelect === true || activateOnMultiSelect === undefined) {
			this.multiSelectButtons.push(button);
		}
		if (activateOnNoSelect === true || activateOnNoSelect === undefined) {
			this.noSelectButtons.push(button);
		} else {
			this._widgetUtilities.disableButton(button);
		}
		
		if (!onFooter) {
			this.header.toolbar.addChild(button);
		} else {
			this.footer.toolbar.addChild(button);
		}
		return  button;
	},
	
	setActivationBasedOnRoleSelection: function(
			/* menu Object */menuObject,
			/* boolean */activateOnSingleSelect,
			/* boolean */activateOnMultiSelect,
			/* boolean */activateOnNoSelect) {
		// summary:
		//     Enable or disable item specified attributes.
		// menuObject: dijit.Menu
		//     Object contains menu object
		// activateOnSingleSelect: boolean
		//     boolean flag specifying whether the button should be active when a
		//     single row is selected.
		// activateOnMultiSelect: boolean
		//     boolean flag specifying whether the button should be active when
		//     multiple rows are selected
		// activateOnNoSelect: boolean
		//     boolean flag specifying whether the button should be active when no
		//     rows are selected.
		if (activateOnSingleSelect === true || activateOnSingleSelect === undefined) {
			this.singleSelectButtons.push(menuObject);
		}
		if (activateOnMultiSelect === true || activateOnMultiSelect === undefined) {
			this.multiSelectButtons.push(menuObject);
		}
		if (activateOnNoSelect === true || activateOnNoSelect === undefined) {
			this.noSelectButtons.push(menuObject);
		} else {
			this._widgetUtilities.disableButton(menuObject);
		}
	},
	
	addToolbarSeparator:function() {
		// summary:
		//     Adds a separator to the toolbar.
		// tags:
		//     public
		this.header.toolbar.addChild(new dijit.ToolbarSeparator());
	},
	
	addSelectionMenuItem:function(/* dijit.MenuItem */menuItem, 
			/* boolean */activateOnSingleSelect, 
			/* boolean */activateOnMultiSelect,
			/* boolean */activateOnNoSelect) {
		// summary:
		//     Addes a menu item to show up when a user right clicks selected items.
		// menuItem: dijit.MenuItem
		//     The object representing the menu item.
		// activateOnSingleSelect: boolean
		//     boolean flag specifying whether the menu item should be active when a
		// single row is selected.
		//     activateOnMultiSelect: boolean
		// boolean flag specifying whether the menu item should be active when
		//     multiple rows are selected
		// activateOnNoSelect: boolean
		//     boolean flag specifying whether the menu item should be active when
		//     no rows are selected.
		// tags: public
		this.contextMenuHandler.addSelectionMenuItem(menuItem, activateOnSingleSelect, activateOnMultiSelect, activateOnNoSelect);
	},
	
	selectAll:function() {
		// summary:
		//     Selects all items in the grid
		this.grid.rowSelectCell.toggleAllSelection(true);
		
		this.displaySelectAllMessage();
	},
	
	deselectAll : function(event) {
		// summary:
		//     Deselects all items in the grid
		//no selection is need just return 
		this._trace.entering(event, "deselectAll");
		if(!this.needSelection) {return;} 
		if (this.isMultiSelect) {
			this.grid.rowSelectCell.toggleAllSelection(false);
		} else {
			var selected = this.grid.selection.getFirstSelected();
			if (selected && selected !== -1) {
				this.grid.rowSelectCell.toggleRow(selected, true);
			}
			this.grid.selection.clear();
		}
		if((event !== null) && (event !== undefined)){ //Check to make sure this is called because of an user event 
		  //console.log(event);
		  this.hideSelectAllMessage();
		}
		this._trace.exiting("", "deselectAll");
		
	},
	
	resetAllSelection:function(){
        this.deselectAll();
		this.grid.selection.preserver.reset();
	},
	//Reset table to initial state , currently search box 
	resetAppliedSearch:function(){
		
		this.searchString="";
		this.header.searchField.clearSearchValue();
		this._updateSearchResultsDisplay([]);
		this.resize();
	},
	resize:function(){
		this.inherited(arguments);
		try{
			if(dojo.isIE){// a bad fix for table resize on IE.For Defect 46117
				var containerChilds=this.container.getChildren();
				var i;
				var childrenHeight=0;
				for(i=0;i<containerChilds.length;i++){
					childrenHeight=childrenHeight+dojo.marginBox(containerChilds[i].domNode).h;
				}
				var containerHeight=dojo.marginBox(this.container.domNode).h;
				if(containerHeight > childrenHeight){
					this.inherited(arguments);
				}
			}
		}catch(e){
			this._trace.log("resize on IE breaking", "resize");
		}
	},
	refresh: function() {
		// summary:
		//     Refetches all grid item from the datastore
		// tags:
		//     public
		this._trace.entering(this.id, "refresh");
		// Need to ensure that we don't break the alignment of any other text apart from
		// None (the one which gets displayed when there's no data in the table).
		dojo.removeClass(this.grid.messagesNode,"noDataMessageStyle");	
		this.grid.showMessage(this.grid.tableLoadingMessage);
		// This is to tell store that consider the Server rest response as the most updated one.
		// So that _updates within the store are not merged with the result from server.
		if (this.dataStore && this.dataStore._updates) {
			this.dataStore.serverVersion = this.dataStore._updates && this.dataStore._updates.length;
		}
		if (this._isClientSideServiceStore()) {
			this._trace.info(this.id + "Clearing client side filter cache", "refresh");
			this._refreshForClientCaching();
		} else {
			this.grid.filter(this.grid.query, true);
		}	
		
		if(this.parentWidget) {
			this.parentWidget._refreshHandler();
		}
		/*if (this.isPaginationEnabled) {//update the table stats if data changes
			this.grid.pagination.plugin.paginators[0].update();
		}*/
		
		this._trace.exiting(this.id, "refresh");
	},	
	_refreshForClientCaching:function() {
		this._trace.entering("", "_refreshForClientCaching");
		this._trace.object(this.grid.query, "_refreshForClientCaching");
		this.grid.store.clearCache();
		if(this.grid.rowSelectCell) this.grid.rowSelectCell.map={};
		this.grid.filter(this.grid.query,false);
        this._trace.exiting("", "_refreshForClientCaching");
	},	
	
	setSortedColumn: function(/* int */columnIndex, /* boolean */isAscending) {
		// summary:
		//     Sets the sorted column
		// columnIndex: int
		//     zero-based index indicating the column to sort.
		// isAscending: boolean
		//     acending if true, descending if false
		this.grid.setSortIndex(columnIndex, isAscending);
	},
	
	getSelectedItems: function() {
		// summary:
		//     gets references to selected items
		// returns:
		//     Array of selected items.
		var selected = [];
		var syncMode = this.dataStore.syncMode;
		this.dataStore.syncMode = true;
		try {
			for(var selection in this.grid.selection.preserver._selectedById) {
				if (selection !== "undefined" && this.grid.selection.preserver._selectedById[selection] === true) {
					var item = this.dataStore.fetchItemByIdentity({identity:selection});
					selected.push(item);	
				}
			}
		} finally {
			this.dataStore.syncMode = syncMode;
		}
		return selected;//Object[]
	},
	
	getSelectedCount: function() {
		// summary:
		//     gets count of selected items
		// returns:
		//     Count of selected items.
		var count = 0;
		for (var selection in this.grid.selection.preserver._selectedById) {
			if (selection !== "undefined" && this.grid.selection.preserver._selectedById[selection] === true) {
				count++;
			}
		}
		return count;//Number
	},
	setStore : function(store, query, queryOptions){ 
		// summary:
		//     changes the datastore on the table. Note - this has only been implemented with client side tables.
		this.dataStore = store;
		//when setting new store  we should start fetching from page 1 , hence resetting the current page 
		this.resetCurrentPageVariable();
		if(this.isClientSide){
			
		    var clientQuery = this._getInitialClientSideQuery();
		    clientQuery = dojo.mixin(clientQuery,query||{});
			this.grid.setStore(store,clientQuery,{ignoreCase: true, cache: this.isClientSide});
		}else{
			this.grid.setStore(store, query, queryOptions);
		}
			
	},
	_showFooter: function() {
		// summary:
		//     Shows the footer count area for table.
		if(this.isTotalsFooterRendered){
			dojo.removeClass(this.footer.infoArea, "hideEntity");
			dojo.addClass(this.footer.infoArea, "showEntity");	
		}
	},
	_hideFooter: function() {
		// summary:
		//      hides the footer count area for table.
		if(this.isTotalsFooterRendered){
			dojo.removeClass(this.footer.infoArea, "showEntity");
			dojo.addClass(this.footer.infoArea, "hideEntity");		
		}
	},
	
	resetCurrentPageVariable: function() {
		if(this.grid && this.isPaginationEnabled){
			this.grid.pagination.plugin._currentPage = 0;
		}
	}
	
});