dojo.provide("com.ibm.security.modeling.tables.TableContextMenuHandler");

dojo.declare("com.ibm.security.modeling.tables.TableContextMenuHandler", null, {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.tables.TableContextMenuHandler"),
	constructor: function() {	
		// table: com.ibm.security.modeling.tables.Table
		//		Table to handle right click events for.
		// singleSelectMenuItems: dijit.MenuItem[]
		//     MenuItems in this table that are active when a single row is selected
		// multiSelectMenuItems: dijit.MenuItem[]
		//     MenuItems in this table that are active when multiple rows are selected
		// noSelectMenuItems: dijit.MenuItem[]
		//     MenuItems in this table that are active when no rows are selected
		this.singleSelectMenuItems = [];
		this.multiSelectMenuItems = [];
		this.noSelectMenuItems = [];		
	},
	attachToTable:function(table) {
		this.table = table;
		this.table.connections.push(dojo.connect(this.table.grid, "onRowContextMenu", this, this.processContextMenuLaunch));
	},
	processContextMenuLaunch: function(e) {
		// summary:
		//     handles a right click on a row to launch the context menu
		// e: Object
		//     Event information, mainly the grid reference
		// tags:
		//     callback
		this._trace.entering("", "_processContextMenuLaunch");
		if (this._isMenuAvailable() === true) {
			var rowIndex = e.rowIndex;
			var itemRightClicked = this.table.grid.getItem(rowIndex);
			var isSelected = this._isItemCurrentlySelected(itemRightClicked);
			if (!isSelected) {
				this._clearOthersAndSelect(rowIndex);
			}
			var numberOfSelectedRows = this.table.getSelectedCount();
			this._setAvailableMenuOptions(numberOfSelectedRows);
		}
		this._trace.exiting("", "_processContextMenuLaunch");
	},
	_isMenuAvailable:function() {
		var isAvailable = false;
		var menus=this.table.rightClickSelectionMenu.getChildren();
		if (menus.length > 0) {
			isAvailable = true;
		}
		return isAvailable;
	},
	_clearOthersAndSelect:function(rowIndex) {
		this._trace.entering("", "_clearOthersAndSelect");
		this.table.deselectAll();
		this.table.grid.rowSelectCell.toggleRow(rowIndex, true);
		this._trace.exiting("", "_clearOthersAndSelect");
	},
	_isItemCurrentlySelected:function(item) {
		this._trace.entering("", "_isItemCurrentlySelected");
		var isSelected = false;
		var selectedItems = this.table.getSelectedItems();
		for (var i=0;i<selectedItems.length;i++) {
			if (this.table.dataStore.getIdentity(item) === this.table.dataStore.getIdentity(selectedItems[i])) {
				isSelected = true;
				break;
			}
		}
		this._trace.exiting("", "_isItemCurrentlySelected");
		return isSelected;
	},
	_setAvailableMenuOptions: function(numberOfSelectedRows) {
		this._trace.entering(numberOfSelectedRows, "_setAvailableMenuOptions");
		// disable all menu items
		this._toggleItemActivation(this.table.rightClickSelectionMenu.getChildren(), true);
		// then only enable items appropriate to the selections
		if(numberOfSelectedRows === 1) {
			this._toggleItemActivation(this.singleSelectMenuItems, false);
		}
		if (numberOfSelectedRows > 1) {
			this._toggleItemActivation(this.multiSelectMenuItems, false);			
		}
		if (numberOfSelectedRows === 0) {
			this._toggleItemActivation(this.noSelectMenuItems, false);			
		}
		this._trace.exiting("", "_setAvailableMenuOptions");
	},
	_toggleItemActivation: function(/* Object[] */items, /* boolean */isDisabled) {
		// summary:
		//     Sets the disabled attribute on the array of items passed in.
		// items:
		//     Array of objects to be set
		// isDiabled:
		//     Whether you want the objects to be disabled or enabled.
		// tags:
		//     private
		dojo.forEach(items, function(item) {
			item.set('disabled', isDisabled);
		});
	},
	addSelectionMenuItem:function(/* dijit.MenuItem */menuItem, 
			/* boolean */activateOnSingleSelect, 
			/* boolean */activateOnMultiSelect,
			/* boolean */activateOnNoSelect) {
		// summary:
		//     Addes a menu item to show up when a user right clicks selected items.
		// menuItem: dijit.MenuItem
		//     The object representing the menu item.
		// activateOnSingleSelect: boolean
		//     boolean flag specifying whether the menu item should be active when a
		// single row is selected.
		//     activateOnMultiSelect: boolean
		// boolean flag specifying whether the menu item should be active when
		//     multiple rows are selected
		// activateOnNoSelect: boolean
		//     boolean flag specifying whether the menu item should be active when
		//     no rows are selected.
		// tags: public
		if (activateOnSingleSelect) {
			this.singleSelectMenuItems.push(menuItem);
		}
		if (activateOnMultiSelect) {
			this.multiSelectMenuItems.push(menuItem);
		}
		if (activateOnNoSelect) {
			this.noSelectMenuItems.push(menuItem);
		}
		this.table.rightClickSelectionMenu.addChild(menuItem);
	}
});