dojo.provide("com.ibm.security.modeling.tables.TableStatus");

dojo.require("com.ibm.security.modeling.util.Trace");

dojo.declare("com.ibm.security.modeling.tables.TableStatus", null, {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.tables.TableStatus"),
	// numSelected: int
	//		The current number of selected items
	numSelected: 0,
	// numTotalItems: int
	//		The current total number of items of this type, regardless of set filter
	numTotalItems: 0,
	// table:  com.ibm.security.modeling.tables.Table
	//		the table object itself
	table: null,
	
	// pluginResources: Object
	//			Reference to the plugin shared resource bundle
	pluginResources: null,
	
	constructor: function(/*com.ibm.security.modeling.tables.Table*/table) {
		// summary: 
		//		Keeps table status such as number of items and selections and 
		//		updates their display on the table.
		this.table = table;
	}, 
	
	connectFooterStatus: function(/*Object[]*/connections) {
		connections.push(dojo.connect(this.table.grid.store, "onDelete", this, this.updateTotalItemsForRemove));
		connections.push(dojo.connect(this.table.grid.store, "onNew", this, this.updateTotalItemsForAdd));
		if(this.table.needSelection){
			connections.push(dojo.connect(this.table.grid.selection, "onSelected", this, this.updateSelected));
			connections.push(dojo.connect(this.table.grid.selection, "onDeselected", this, this.updateDeSelected));
		}
		connections.push(dojo.connect(this.table, "_handleFetchComplete", this, this.updateTotalItems));
		return connections;
	},
	
	updateTotalItems:function(/*Object[]*/items) {
		try {
			if (this.table._isServerSideServiceStore()) {
				this.numTotalItems = this.table.dataStore.totalCount;
			} else {
				this.numTotalItems = items.length;
			}
			this.table.footer.updateFooter(this.numSelected, this.numTotalItems);
		} catch (e) {
			this._trace.exception(e, "updateTotalItems");
		}
	},
	
	updateTotalItemsForRemove: function(item) {
		// summary:
		//		Updates the footer for a single item being removed from the grid.
		this._trace.entering(this.table.id, "updateTotalItemsForRemove");
		this._trace.object(item, "updateTotalItemsForRemove");
		try {
			this.table.grid.selection.clear();
			this.numTotalItems--;
			this.table.footer.updateFooter(this.numSelected, this.numTotalItems);
			this._trace.exiting(this.table.id, "updateTotalItemsForRemove");
		} catch (e) {
			this._trace.exception(e, "updateTotalItemsForRemove");
		}
	},
	
	updateTotalItemsForAdd: function(item) {
		// summary:
		//		Updates the footer for a single item being added to the grid.
		this._trace.entering(this.table.id, "updateTotalItemsForAdd");
		this._trace.object(item, "updateTotalItemsForAdd");
		try {
			this.numTotalItems++;
			this.table.footer.updateFooter(this.numSelected, this.numTotalItems);
			this._trace.exiting(this.table.id, "updateTotalItemsForAdd");
		} catch (e) {
			this._trace.exception(e, "updateTotalItemsForAdd");
		}
	},
	
	updateSelected:function(index) {
		this._trace.entering(this.table.id, "updateSelected");
		try {
			this.numSelected = this.table.grid.selection.getSelectedCount();
			this.table.footer.updateFooter(this.numSelected, this.numTotalItems);
			this.onSelectionChanged(this.numSelected);
			this._trace.exiting(this.table.id, "updateSelected");
		} catch (e) {
			this._trace.exception(e, "updateSelected");
		}
	},
	updateDeSelected:function(index) {
		this._trace.entering(this.table.id, "updateSelected");
		try {
			this.numSelected = this.table.grid.selection.getSelectedCount();
			this.table.footer.updateFooter(this.numSelected, this.numTotalItems);
			this._trace.exiting(this.table.id, "updateSelected");
		} catch (e) {
			this._trace.exception(e, "updateSelected");
		}
	},
    onSelectionChanged:function(numSelected){
		
	}
});

dojo.declare("com.ibm.security.modeling.tables.PaginationTableStatus", com.ibm.security.modeling.tables.TableStatus, {
	constructor:function(){
	   this._toggleHandler = null;
    },
    destroy:function(){
    	dojo.disconnect(this._toggleHandler);
    },
    //Overriding method to connect , and disconnect our toggleHandler to improve performance in grid rendering
    //when selectall is called , we disconnect the handle , so that for every selection , the connection is not called
    overRideSelectionfunction:function(){
    	//var originalSelectAll = this.table.grid.selection.selectRange;
		var self = this;
		var originaltoggleAllSelection = this.table.grid.rowSelectCell.toggleAllSelection;
		this.table.grid.rowSelectCell.toggleAllSelection = function(checked){
				dojo.disconnect(self._toggleHandler);
				self._toggleHandler = null;
				dojo.hitch(self.table.grid.rowSelectCell,originaltoggleAllSelection)(checked);
				self._toggleHandler = dojo.connect(self.table.grid.selection.preserver, "_selectById", self, self.updateSelected);
				self.updateSelected();
	   };
    },
	connectFooterStatus: function(/*Object[]*/connections) {
		this._trace.entering(this.table.id, "connectFooterStatus");
		
		
		connections.push(dojo.connect(this.table.grid.store, "onDelete", this, this.updateTotalItemsForRemove));
		if(this.table.needSelection){
			this.overRideSelectionfunction();
			this._toggleHandler = dojo.connect(this.table.grid.selection.preserver, "_selectById", this, this.updateSelected);
			//reset connection is to handle , deselect all , and set store condition 
			//deselect all not equavalent to onDeselection , since selection only invokes onDeselected for current page entries
			connections.push(dojo.connect(this.table.grid.selection.preserver, "reset", this, this.updateSelected));
		}
			
		
		connections.push(dojo.connect(this.table, "_handleFetchComplete", this, this.updateTotalItems));
		this.paginationFooterDiv=this.table.grid.pagination.plugin.paginators[0].descriptionDiv;
		this._trace.entering(this.table.id, "connectFooterStatus");
		return connections;
	},
	
	updateTotalItems:function(/*Object[]*/items) {
		this._trace.entering(this.table.id, "updateTotalItems");
		//this.table.grid.pagination.plugin.paginators[0].update();
		if(this.table.needSelection)
			this.updateSelectedCountOnPaginator(this.paginationFooterDiv);	
		this._trace.exiting(this.table.id, "updateTotalItems");	
	},
	
	updateTotalItemsForRemove: function(item) {
		// summary:
		//		Resets the selected items on a delete.  
		// description: 
		//		The principle is that delete should delete all selected items.  Selection preserver will leave behind
		//		selection references when things are removed from the store, so we have to remove the selections programmatically
		//		or the hanging references will mess up table stats (rows, selected, etc)
		this._trace.entering(this.table.id, "updateTotalItemsForRemove");
		this._trace.object(item, "updateTotalItemsForRemove");
		this.table.grid.selection.preserver.reset();
		this._trace.exiting(this.table.id, "updateTotalItemsForRemove");
	},
	
	updateSelected:function(index) {
		this._trace.entering(this.table.id, "updateSelected");
		this.updateSelectedCountOnPaginator(this.paginationFooterDiv);
		this._trace.exiting(this.table.id, "updateSelected");	
	},
	updateSelectedCountOnPaginator: function(originalDescription,totalSelected){
		// summary:
		//		Pins the selected count onto the end of the table stats in the footer.
		this._trace.entering(this.table.id, "updateSelectedCountOnPaginator");
		if (totalSelected === undefined) {
			totalSelected=this._getSelectionPreserverCount();
		}
		this.table.grid.pagination.plugin.paginators[0]._updateDescription();
		originalDescription.innerHTML = originalDescription.innerHTML + this.table.tableResources.COMMA_TO_SEPARATE + " " + totalSelected + this.table.tableResources.SELECTED;
		this.onSelectionChanged(totalSelected);
		this._trace.exiting(this.table.id, "updateSelectedCountOnPaginator");
	},
	_getSelectionPreserverCount:function(){
		var count=0;
		if(this.table.grid.selection.preserver){
			var selectionList=this.table.grid.selection.preserver._selectedById;
			var disableMap=this.table.grid.rowSelectCell.disabledIdMap;
			for(var i in selectionList){
				var isDisabled=false;
				if(disableMap && disableMap[i]){
					isDisabled=true;
				}
				if(selectionList[i] && !isDisabled){
					count++;
				}
			}
		}
		return count;
	}
});