dojo.provide("com.ibm.security.modeling.tables.TableToolbarButtonBuilder");

dojo.require("com.ibm.security.modeling.util.WidgetUtilities");

dojo.declare("com.ibm.security.modeling.tables.TableToolbarButtonBuilder", null, {
	_widgetUtilities: new com.ibm.security.modeling.util.WidgetUtilities(),
	isActiveForSingleSelect: false,
	isActiveForMultiSelect: false,
	isActiveForNoSelect: false,
	isOnFooter: false,
	
	constructor: function(table) {
		this.table = table;
	},
	
	withVisibleLabel:function(/*string*/label) {
		this.isShowLabel = true;
		this.label = label;
		return this;
	},
	
	withInvisibleLabel: function(/*string*/label) {
		this.isShowLabel = false;
		this.label = label;
		return this;
	},
	
	withIconClass: function(/*string*/iconClass) {
		this.iconClass = iconClass;
		return this;
	},
	
	withOnClickFunction: function(/*function*/onClick) {
		this.onClick = onClick;
		return this;
	},
	
	withId: function(/*string*/id) {
		this.buttonId = id;
		return this;
	},
	
	withActivationForSingleSelect: function() {
		this.isActiveForSingleSelect = true;
		return this;
	},
	
	withActivationForMultiSelect: function() {
		this.isActiveForMultiSelect = true;
		return this;
	},
	
	withActivationForNoSelect: function() {
		this.isActiveForNoSelect = true;
		return this;
	},
	
	withMenuObject: function(/*dijit.Menu*/menuObject) {
		this.menu = menuObject;
		return this;
	},
	
	withTooltipDialog: function(/*dijit.TooltipDialog*/tooltipDialog) {
		this.tooltipDialog = tooltipDialog;
		return this;
	},
	
	onFooter:function(/*boolean*/isOnFooter) {
		this.isOnFooter = isOnFooter;
		return this;
	},
	
	build: function() {
		var button = this._createButton();
		this._defineBehavior(button);
		this._addToTable(button);
		return button;
	},
	
	_createButton: function() {
		return this._widgetUtilities.createButton(
				this.label,
				this.isShowLabel,
				this.iconClass,
				this.onClick,
				this.buttonId,
				true,
				this.menu,
				this.tooltipDialog);
	},
	
	_defineBehavior:function(button) {
		if (this.isActiveForSingleSelect === true) {
			this.table.singleSelectButtons.push(button);
		}
		if (this.isActiveForMultiSelect === true) {
			this.table.multiSelectButtons.push(button);
		}
		if (this.isActiveForNoSelect === true) {
			this.table.noSelectButtons.push(button);
		} else {
			this._widgetUtilities.disableButton(button);
		}
	},
	
	_addToTable:function(button) {
		if (this.isOnFooter === true) {
			this.table.footer.toolbar.addChild(button);
		} else {
			this.table.header.toolbar.addChild(button);
		}
	}
	
});