dojo.provide("com.ibm.security.modeling.tables.UserTable");
dojo.require("com.ibm.security.modeling.tables.CustomDisplayStructureFactory");

dojo.require("com.ibm.security.modeling.tables.Table");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("dojox.grid.enhanced.plugins.IndirectSelection");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");
dojo.require("com.ibm.security.modeling.notebook.SODHelperClasses");

dojo.requireLocalization("com.ibm.security.modeling.tables", "UserTableText");

dojo.declare("com.ibm.security.modeling.tables.UserTable", com.ibm.security.modeling.tables.Table, {
	userTableResources:null,
	parentContainer: null,
	userTableType:null,
	userConnections: [],
    customCols:null,
    isMultiSelect:true,
    tableType:"default",
    attributesMap:{
	"roleOwnerTable":"owner",
    "directlyAttachedUserTable":"member",
    "allRoleMembersUserTable":"member",
    "usersResourceSelection" : "uri",
    "viewUsersTable":"uri",
    "peoplePicker":"uri",
    "identity":"identityURI",
    "default":"uri"
    },
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.tables.UserTable"),
	constructor: function (args) {
		this._trace.entering("", "constructor");
		this.userTableResources = dojo.i18n.getLocalization("com.ibm.security.modeling.tables", "UserTableText");
		this.searchInstructions = args.searchInstructions || this.userTableResources.SEARCH_INSTRUCTUION ;
		this.defaultSortIndex = 1;
		this._trace.exiting("", "constructor");
	},
	postCreate:function() {
		this.inherited(arguments);
		dojo.addClass(this.domNode,"contentArea");
	},
	postMixInProperties:function() {
		
			this.isClientSide = false;
			this.isFilterRendered = true;
			this.isRefreshRendered = true;
			
			this.isDynamic = true;
			this.entityTypeText = this.globalResources.USERS;
			this.structure = this._getStructureUsers();
			
			this.inherited(arguments);//this needs to be called after isDynamic is set
	},
	_getStructureUsers: function() {
	  var  cols= com.ibm.security.modeling.tables.CustomDisplayStructureFactory.loadUserStructure(this.customCols,dojo.hitch(this,this._onFetchError));
      var i;
      for(i=0;i< cols.length;i++){
          if( cols[i].field=="name"){
            cols[i].get=function(rowIndex,item){return item; };
            cols[i].formatter=dojo.hitch(this, this._formatName);
          }else{
        	  if(cols[i].formatter){
        		  cols[i].toolTipFormatter=function(value,rowIndex,cell){ return com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(cols[i].formatter(value,rowIndex,cell) ); };
        	  }
        	  cols[i].toolTipFormatter=cols[i].toolTipFormatter||function(value,rowIndex,cell){ return com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(value); };
  		      cols[i].formatter=cols[i].formatter||function(value,rowIndex,cell){return com.ibm.security.modeling.tables.FormatterUtils.formatLongCellValue(value);};
          }
      }
      return  cols;		
			
	},
        _formatName:function(item){
                   var anchor=null;
		    
		    if(item){
		    	          anchor=new com.ibm.security.modeling.util.Anchor({
								input:item.name,
								callback:dojo.hitch(this, this.userSelected,item)
							});											
		    }
			return anchor;
          

         },
         userSelected:function(item){
             dojo.require("com.ibm.security.modeling.widgets.PropertiesDialog");
             var attributeField=this.attributesMap[this.tableType];
               this.usersAttributePanel=new com.ibm.security.modeling.widgets.PropertiesDialog({
                   type:"USER",
                   panelHeaderName:item.name,
                    entityItem:item[attributeField]
                   
                   
                    },null);
                    this.usersAttributePanel.show();
                     this.usersAttributePanel.startup();
                  
                  
                   
         },
	_formatUserName:function(nameValueString,inRowIndex){
		nameValueString="<span class=\'userIcon dijitInline\'></span> <span class=\'dijitInline\'> "+nameValueString+"</span>";
		
		return nameValueString;
	},

	destroy:function(){
		this.inherited(arguments);
		dojo.forEach(this.userConnections, dojo.disconnect);
	}	
});