dojo.provide("com.ibm.security.modeling.topology.RoleModelingMenuProviderHelper");

dojo.require("ibm.tivoli.tip.navwidget.topology.dijit.MenuProviderHelperBase");
dojo.require("com.ibm.security.modeling.util.Trace");

dojo.requireLocalization("com.ibm.security.modeling.hierarchy", "RoleHierarchyText");
dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");

dojo.declare("com.ibm.security.modeling.topology.RoleModelingMenuProviderHelper", ibm.tivoli.tip.navwidget.topology.dijit.MenuProviderHelperBase, {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.topology.RoleModelingMenuProviderHelper"),
	// localResources: Object
	//			Reference to the resource bundle unique to this widget
	localResources: null,
	globalResources:null,
	_requestTypes:null,

	
	constructor:function(){
		this._trace.entering("", "constructor");
		this.inherited(arguments);
		this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.hierarchy", "RoleHierarchyText");
		this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		
		this._trace.exiting("", "constructor");
	},
	// required, as it is used as the getMenuItemsCallback with the menu provider
	getMenuItems: function(context) {
		this._trace.entering("", "getMenuItems");
		//this._trace.object(context, "getMenuItems");
		var items =[];
		var itemCount= context.selection.getItemCount();
		if (itemCount == 0) {
			this._trace.exiting("No items selected, returning items", "getMenuItems");
			return items;
		}
				
		var isSingleSelect=true; //setting true by default
		// The create label must reflect create child when a right click is performed on a node or row 
		var createLabel = this.localResources.CREATE_ROLE;
		// Disable Add is more than one node / row is selected
		var disableAdd=false;
		
		
		if(itemCount == 1){// setting enable/disable for single selection
			isSingleSelect=false;
        }
        
        if(itemCount >= 1){
			createLabel = this.localResources.CREATE_CHILD_ROLE;
		}
		
		if(itemCount > 1){
			disableAdd=true;
        }

		var disableErrorWarning=isSingleSelect;
		//Errors and Warnings must be disabled for role for zero errors and warnings
		if(!isSingleSelect){
			var roleitems=context.selection.getItems();
			if(roleitems[0]){
				var statusObj=roleitems[0].getAttribute("status");
				if(statusObj[0].value==6 || statusObj[0].value=="6"){//topology considers status value 6 as OK value hence comparing with numeric status
					disableErrorWarning=true;
				}
				
			}
		}
		
		items.push(
				{ 
				  "actionID": "_separator_" 
				},
				{ "actionID": "viewRolesPortlet",
				  "label": this.localResources.VIEW_DETAILS,
				  "launchType":"JavaScript",
				  "iconClass": "tipMenuIcon viewDetailIcon",
				  "disabled": isSingleSelect
				},
				{ "actionID": "addRolesPortlet",
				  "iconClass": "tipMenuIcon newIcon",
				  "label": createLabel,
				  "launchType":"JavaScript",
				  "disabled": disableAdd
				},
				{ "actionID": "deleteRolesPortlet",
				  "label": this.localResources.DELETE_ROLE,
				  "iconClass": "tipMenuIcon deleteIcon",
				  "launchType":"JavaScript",
				  "disabled": false
				},
				{ "actionID": "copyRolesPortlet",
				  "label": this.localResources.COPY_ROLE,
				  "iconClass": "tipMenuIcon copyIcon",
				  "launchType":"JavaScript",
				  "disabled": isSingleSelect
				},
				{ "actionID": "showErrorWarningPortlet",
				  "label": this.localResources.ERRORS_WARNINGS,
				  "iconClass": "tipMenuIcon errorWarning",
				  "launchType":"JavaScript",
				  "disabled": disableErrorWarning
				},
				{ "actionID": "showRoleAnalysis",
				  "label": this.localResources.ANALYZE_ROLE,
				  "iconClass": "tipMenuIcon analysisIcon",
				  "launchType":"JavaScript",
				  "disabled": isSingleSelect
				}
			);
		
		this.getLifecycleContextmenuItems(items);
		
		items.push({ 
			"actionID": "_separator_" 
		});
		
		this._trace.exiting("", "getMenuItems");
		return items;
	},
	getRoleRequestTypes : function(){
		var roleLifecycleProvider = dojo.hitch(null,"getRoleLifecycleProvider")();
		if(roleLifecycleProvider && roleLifecycleProvider.isBPMEnabled()){
			return roleLifecycleProvider.getRequestTypes();
		}
	},
	getLifecycleContextmenuItems: function(items){
		// summary:
		//     To get context menu defintion   
		// items: Object
		//     Object contains reference of menuItem object
		
		if(! this._requestTypes){
			this._requestTypes = this.getRoleRequestTypes();
		}
		
		if(this._requestTypes && this._requestTypes.length == 1){
			
			items.push(
				{ "actionID": "submitRequest",
				  "label": dojo.string.substitute(this.globalResources.SUBMIT_REQUEST, {"request_type":this._requestTypes[0].label}),
				  "iconClass": "tipMenuIcon submitIcon",
				  "launchType":"JavaScript",
				  "disabled": false
				});
			
		}else if(this._requestTypes && this._requestTypes.length > 1){
			var subMenuItems = [];
			
			for (var i = 0 ; i < this._requestTypes.length; i++) {
				subMenuItems.push({
					"label": this._requestTypes[i].label,
			        "iconClass": "hidden",
			        "actionID": "submitRequest",
			        "launchType":"JavaScript"
				});
			}
			items.push({
				"label": this.globalResources.SUBMIT_REQUESTS,
				"iconClass": "tipMenuIcon submitIcon",
				"launchType":"JavaScript",
				"submenu_items":subMenuItems,
				"disabled": false
			});
		}
	},
	// required, as it is used as the getMenuCommandCallback with the menu provider
	getMenuCommand: function(commandName, context, menuItem) {
		this._trace.entering("", "getMenuCommand");
		var result = {"launchType": "NOOP"};
		var functionName="";
        switch (commandName) {
             case "viewRolesPortlet":            	
           		 var items=context.selection.getItemIds();
				 functionName="launchRoleDetailPanelFromTopologyContextMenu";
				 this._handleMenuLaunch(functionName,items[0]);
                 break;
             case "addRolesPortlet":
            	 var items=context.selection.getItemIds();
            	 var parentIdList=items.join();
				  functionName="launchCreateRolePanel";
				  this._handleMenuLaunch(functionName,null,parentIdList,false);
                 break;
             case "deleteRolesPortlet":
           		 var items=context.selection.getItems();
                         console.log("The items in the topology");
                         console.log(items.length);
                         if(items.length==1){
					  functionName="launchDeleteRolePanel";
					 this._handleMenuLaunch(functionName,null,items[0]._item.id);
                         }else{
                          var itemsList='';
                          for(var i=0;i<items.length;i++){
                             itemsList=itemsList+items[i]._item.id+",";
                           }
                           itemsList=itemsList.substring(0,itemsList.length-1);
						  functionName="launchMultiDeleteRolePanel";
						  this._handleMenuLaunch(functionName,null,itemsList);
                         }

                 break;
             case "copyRolesPortlet":
         	 	var items=context.selection.getItems();
				 functionName="launchCopyRolePanel";
				 this._handleMenuLaunch(functionName,null,items[0]._item.id);
         		break;
             case "showErrorWarningPortlet":
         	 	var items=context.selection.getItems();
				 functionName="launchRoleErrorWarningPanel";
				 this._handleMenuLaunch(functionName,items[0]._item.id);
         		break;
             case "showRoleAnalysis":
            	var items=context.selection.getItems();
				 functionName="launchRoleAnalysisPane";
				 this._handleMenuLaunch(functionName,items[0]._item.id);
          		break;
             case "submitRequest":
            	 var items=context.selection.getItems();
 				 functionName="launchLifecycleSubmitRequest";
 				
 				 this._handleMenuLaunch(functionName,items,this._getRequestType(menuItem));
           		break;
             default:
                  // Note: If this is NOT one of our client-side actions, DO NOT set result and we
                  // will let the parent class determine the command to be returned
        }
        var menuCommand = (result) ? result : this.inherited(arguments);
        this._trace.object(menuCommand, "getMenuCommand");
        this._trace.exiting(menuCommand, "getMenuCommand");
        return menuCommand;
	},
	_getRequestType : function(menuItem){
		// summary:
		//     To get request type based on the menuItem
		// menuItem: Object
		//     Object contains reference of menuItem object
		for(var i=0;i<this._requestTypes.length;i++){
			if(menuItem.label == this._requestTypes[i].label ||
					menuItem.label == dojo.string.substitute(this.globalResources.SUBMIT_REQUEST, {"request_type":this._requestTypes[i].label}) ){
				return this._requestTypes[i];
			}
		}
	},
	_handleMenuLaunch:function(functionName,arg1,arg2,arg3){
		 setTimeout(dojo.hitch(null,functionName,arg1,arg2,arg3),100);
	}
});