dojo.provide("com.ibm.security.modeling.topology.TopologyDataSets");

dojo.require("ibm.tivoli.navmodel.Dataset");
dojo.require("com.ibm.security.modeling.topology.TopologyTypes");

dojo.declare("com.ibm.security.modeling.topology.TopologyDataSets", null, {
	identityAttributeDataSets: {},
	permissionAttributeDataSets: {},
	roleDataSets: {},
	createDataSet:function(/*String*/id, /*String*/type) {
		var dataset = null;
		if (type === TopologyTypes.ROLE) {
			dataset = this.createRoleDataSet(id, true);
		} else if (type === TopologyTypes.PERMISSION_ATTRIBUTE) {
			dataset = this.createPermissionAttributeDataSet(id, true);
		} else if (type === TopologyTypes.IDENTITY_ATTRIBUTE) {
			dataset = this.createIdentityAttributeDataSet(id, true);
		} else {
			throw "Unknown type: " + type + " encountered when creating dataset";
		}
		return dataset;
	},
	createIdentityAttributeDataSet:function(/*String*/id, /*boolean*/useCache) {
		var isCached = this.identityAttributeDataSets[id] instanceof ibm.tivoli.navmodel.Dataset;
		if (!useCache || !isCached){ 
			//console.info("creating new dataset for id: " + id);
			var dataset = new ibm.tivoli.navmodel.Dataset({
				baseUrl: "/ibm/tivoli/rest",
				provider: "com.ibm.security.modeling.console.topology.users",                	
				datasource: "com.ibm.security.modeling.console.topology.users.datasource",
				dataset: id,			
				defaultProperties: [],
				parameters: {modelId:id},
				useDatastore: false});
			this.identityAttributeDataSets[id] = dataset;			
		}
		return this.identityAttributeDataSets[id];
	},
	createHierarchyAttributeDataSet:function(/*String*/id,/*String*/modelid,/*String*/roleid, /*boolean*/useCache) {
		var isCached = this.identityAttributeDataSets[id] instanceof ibm.tivoli.navmodel.Dataset;
		if (!useCache || !isCached){ 
			//console.info("creating new dataset for id: " + id);
			var dataset = new ibm.tivoli.navmodel.Dataset({
				baseUrl: "/ibm/tivoli/rest",
				provider: "com.ibm.security.modeling.console.topology.users",                	
				datasource: "com.ibm.security.modeling.console.topology.rolesHierarchy.datasource",
				dataset: id,			
				defaultProperties: [],
				parameters: {modelId:modelid||id,roleId:roleid||id},
				useDatastore: false});
			this.identityAttributeDataSets[id] = dataset;			
		}
		return this.identityAttributeDataSets[id];
	},
	
	createPermissionAttributeDataSet:function(/*String*/id, /*boolean*/useCache) {
		var isCached = this.permissionAttributeDataSets[id] instanceof ibm.tivoli.navmodel.Dataset;
		if (!useCache || !isCached){ 
			//console.info("creating new dataset for id: " + id);
			var dataset = new ibm.tivoli.navmodel.Dataset({
				baseUrl: "/ibm/tivoli/rest",
				provider: "com.ibm.security.modeling.console.topology.permissions",                	
				datasource: "com.ibm.security.modeling.console.topology.permissions.datasource",
				dataset: id,			
				defaultProperties: [],
				useDatastore: false});
			this.permissionAttributeDataSets[id] = dataset;			
		}
		return this.permissionAttributeDataSets[id];
	},

	createRoleDataSet:function(/*String*/id, /*boolean*/useCache) {
		var isCached = this.roleDataSets[id] instanceof ibm.tivoli.navmodel.Dataset;
		if (!useCache || !isCached) {
			//console.info("creating new dataset for id: " + id);
			var dataset =  new ibm.tivoli.navmodel.Dataset({
				baseUrl: "/ibm/tivoli/rest",
				provider: "com.ibm.security.modeling.console.topology.roles",
				dataset: "com.ibm.security.modeling.console.topology.roles",	
				defaultProperties: [],
				parameters: {modelid:id},
				useDatastore: false});
			this.roleDataSets[id] = dataset;			
		}
		return this.roleDataSets[id];
	},
	createRoleAttributesDataSet:function(/*String*/id, /*boolean*/useCache) {
		var isCached = this.roleDataSets[id] instanceof ibm.tivoli.navmodel.Dataset;
		if (!useCache || !isCached) {
			//console.info("creating new dataset for id: " + id);
			var dataset =  new ibm.tivoli.navmodel.Dataset({
				baseUrl: "/ibm/tivoli/rest",
				provider: "com.ibm.security.modeling.console.topology.users",                	
				datasource: "com.ibm.security.modeling.console.topology.users.datasource",
				dataset: id,			
				defaultProperties: [],
				parameters: {entityType:"ROLE"},
				useDatastore: false});
			this.roleDataSets[id] = dataset;			
		}
		return this.roleDataSets[id];
	}
	
});