dojo.provide("com.ibm.security.modeling.util.ErrorUtilities");

dojo.require("com.ibm.security.modeling.util.MessageUtilities");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");
dojo.requireLocalization("com.ibm.security.modeling", "PluginMessages");
dojo.requireLocalization("com.ibm.security.modeling", "LogMessages");

dojo.declare("com.ibm.security.modeling.util.ErrorUtilities",null,{
	
	
	constructor:function(){
	   this._messageUtils = new com.ibm.security.modeling.util.MessageUtilities(),
	   this.pluginResources =  dojo.i18n.getLocalization("com.ibm.security.modeling", "PluginMessages");
	   this.logResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "LogMessages");
    }, 
	
	getErrorMessageFromMessageObject : function(rb, error) {
		/**
		 * Summary:
		 * 	Gets the translated message from a message object
		 *  that potentially has an id, text, and message inserts
		 *  
		 *  it will create in the following order
		 *  
		 *  1) If it is a SecurityModeling error message and has a message id
		 *  		then get the message from our local message resources
		 *  2) Else If it is a SecurityModeling error message and has message text
		 * 
		 */
    
    	
    	var message = {text:"",severity:"error",inserts:[],id:""};
    	//assumes error is just string , we can not help in translating .
    	if(!dojo.isObject(error)) {
    		message.text=error;
    		return message ;
    		
    	}
    	message=dojo.mixin(message,error);
		if (message.key) {
			var inserts = message.inserts ? message.inserts : null;
			//if the message key is contained in the resource bundle
			if (rb && rb[message.key]) {
				 message.text = rb[message.key];
			} else if(this.pluginResources[message.key]){
				message.text = this.pluginResources[message.key];
			}else if(this.logResources[message.key]){
				message.text = this.logResources[message.key];
			}else{
				message.text = message.key;
			}
		} else if (error.message) {
			message.text= error.message;
		}
		
			var severity = message.severity;
			switch (severity) {
			case 'I':
			case 'i':
				message.severity= 'info';
				break;
			case 'W':
			case 'w':
				message.severity= 'warn';
				break;
			case 'E':
			case 'e':
				message.severity= 'error';
				break;
			case 'info':	// already mapped.
				break;
			case 'warn':	// already mapped.
				break;
			case 'error':	// already mapped.
				break;
			default:
				message.severity= 'error';
			    break;
			}
	
		return message;
		
	},
	
	getErrorMessageFromRestCallback : function(rb, error) {
		/**Summary:
		 *  Gets the error message from the call back error object
		 *  using the following order of preference.
		 *  
		 *  1) If it is a SecurityModeling error message and has a message id
		 *  		then get the message from our local message resources
		 *  2) Else If it is a SecurityModeling error message and has message text in the json payload
		 *  		then return the text returned in the message
		 *  3) Else return the error message set in the error object.
		 *  		
		 *  @param rb Resource bundle to use to translate the message key
		 *  @param error callback error object to get the message from
		 *  
		 *  @returns String error message text.
		 */
		if (error.responseText) {
			var message = dojo.fromJson(error.responseText);
			if (message.key) {
				var inserts = message.inserts ? message.inserts : null;
				//if the message key is contained in the resource bundle
				if (rb[message.key]) {
					return this._messageUtils.formatMessage(rb[message.key],inserts);
				} else {
					return this._messageUtils.formatMessage(message.key,inserts);
				}
			} else if (message.text){
				return message.text;
			} else  {
				return error.message;
			}
		} else {
			return error.message;
		}
	},
	
//Takes resource bundle as fourth parameter . This will help in internalization of error messages .  
	handleRESTError: function (error, messageArea,isMsgDialogReq , rb) {
		if (this._isTIPSessionTerminated(error)) {
			window.location.href = dojo.config.serverProperties.contextRoot + "/jsp/login/login.jsp";
			return;
		}
		this.messageArea = messageArea || this.messageArea || null;
		if ((this.messageArea != null) &&
			(error != null)) {
			
			if(isMsgDialogReq !== false){
				isMsgDialogReq=true;
			}
			if(error.status == 403) {
				this.messageArea.showErrorMessage(this.pluginResources.PLUGIN_REST_INSUFFICIENT_ACCESS);
			} else {
				var messages = new Array();
				//messages.rb = rb||this.pluginResources;
				try {
					//getting response object
					var responseObject = dojo.fromJson(error.responseText);//JSON.parse(error.responseText, null);
					if (responseObject != null) {
						if (responseObject.text != null) {
							//every thing went well , and we got the response object . 
							//getting the message from object . 
							messages.push(responseObject);
						} else if ((responseObject.messages != null) ||
								   (responseObject.content != null)) {
	
							var extractedMessages = this._messageUtils.extractBatchMessages(responseObject,null,false);
							// Merge
							if (extractedMessages && extractedMessages.length > 0) {
								for (var i=0; i<extractedMessages.length; i++) {
									messages.push(extractedMessages[i]);
								}
							} 
						}
					}
					else{
						messages.push(this.pluginResources.PLUGIN_REST_SERVER_UNAVAILABLE);
					}
				} catch (e) {
					messages.length = 0;
					if (error.responseText == null ||  error.responseText == '') {
						messages.push(this.pluginResources.PLUGIN_REST_SERVER_UNAVAILABLE);
					} else {
						var messageResponseSplits = error.responseText.split("\"text\":\"", -1);
						if (messageResponseSplits.length == 1) {
							if (error.status === 404 || error.status === 0) {
								
								//Change the message to a user friendly message.			 
								messages.push(this.pluginResources.PLUGIN_REST_SERVER_UNAVAILABLE);
							}else{
								messages.push(messageResponseSplits[0]);
							}
						} else {
							for (var i = 1; i < messageResponseSplits.length; i++) {
								var messageResponseSplit = messageResponseSplits[i];
								var messageSplits = messageResponseSplit.split("\"}", -1);
								if (messageSplits.length > 1) {
									messages.push(messageSplits[0]);
								}
							}
						}
					}
				}
				for (var i = 0; i < messages.length; i++) {
				//	var escapedMessage = com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(messages[i]);
				  var messageObj = this.getErrorMessageFromMessageObject(rb,messages[i]);
					if(error.status === 409 && dojo.config.panels && isMsgDialogReq){
							dojo.publish(dojo.config.panels.MESSAGE_DIALOG.launchEventAddress,[this._messageUtils.formatMessage(messageObj.text, messageObj.inserts),"","Error",function(){
															dojo.publish("PROJECTSUPDATED","");
															dojo.publish(dojo.config.panels.PROJECTS.launchEventAddress,[false,true]);
							}]);
					}else{
						this._messageUtils.showMessage(messageObj,this.messageArea);
						//this.messageArea.showErrorMessage(escapedMessage);
					}
				}
				
			}
		}
	},
   
	_addMessages: function (messagesContainer, messages) {
		if ((messagesContainer != null) &&
			(messagesContainer.messages != null)) {
			for (var i = 0; i < messagesContainer.messages.length; i++) {
				var message = messagesContainer.messages[i];
				if ((message != null) &&
					(message.text != null)) {
					messages.push(message.text);
				}
			}
		}
	},

	_isTIPSessionTerminated: function(error) {
		/**Summary:
		 *  Returns true if the TIP session has been terminated.
		 *  		
		 *  @param error The error received in response to a REST call.
		 *  
		 *  @returns true if TIP session has been terminated; otherwise false.
		 */
		var isTIPSessionTerminated = false;

		// This is the response text returned by REST if the TIP session
		// has been terminated.  This value is intended to match the value
		// of the following Java constant used by the REST code:
		// com.ibm.security.modeling.rest.Constants.TIP_SESSION_TERMINATED_RESPONSE_CONTENT.
		var tipSessionTerminatedResponseText = "TIP Session Terminated";

		if ((error) &&
			(error.status === 401) &&
			(error.responseText === tipSessionTerminatedResponseText)) {
			isTIPSessionTerminated = true;
		}

		return isTIPSessionTerminated;
	}

});
