dojo.provide("com.ibm.security.modeling.util.MessageUtilities");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");

dojo.declare("com.ibm.security.modeling.util.MessageUtilities", null, {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.util.MessageUtilities"),	

	formatMessage: function (/*String*/message, /*Array*/inserts , cssClassName,placeholders) {
		this._trace.entering(this.id,"formatMessage");
		var formatTags = ["",""];
		var formattedInserts = [];
		if(cssClassName) formatTags = ["<span class="+cssClassName+">", "</span>"];
		if (inserts != null && inserts.length > 0) {
		    for(var i=0;i<inserts.length;i++){     
		    	formattedInserts[i]=com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLandNormalize(inserts[i]);	    
		    	formattedInserts[i]=formatTags[0]+formattedInserts[i]+formatTags[1];
			}
		}
		
		var result = message;
		if(placeholders!=null && inserts != null && inserts.length > 0 && inserts.length >= placeholders.length){
            var sub="{";
            for(var i=0;i<inserts.length;i++){
               sub=sub+"\""+placeholders[i]+"\":\""+formattedInserts[i]+"\"\,";
            }
            sub=sub.substring(0,sub.length-1)+"}";
	        dojo.string.substitute(sub,dojo.fromJson(sub));
	        result = dojo.string.substitute(message,dojo.fromJson(sub));
	        return result;
        }
		if ((message !== null) &&
			(inserts !== null)) {
			for (var i = 0; i < inserts.length; i++) {
				var insertTarget = "{" + i + "}";
				var indexOfInsert = result.indexOf(insertTarget);
				if (indexOfInsert < 0) {
					//if {i} not found , we are here 
					//then we will try to replace ${var} things .
					//replace ${var} withe {i} , i-- , and continue 
					//i = inserts.length;
					var regex = new RegExp( "\\${[^}]*}");
					if(regex.test(result)){
						result = result.replace(regex,"{"+i+"}");
						i--;
						continue ;
					}
						
				} else if (indexOfInsert === 0) {
					if (indexOfInsert < (result.length - insertTarget.length)) {
						result = formattedInserts[i] + result.substring(indexOfInsert + insertTarget.length);
					} else {
						result = formattedInserts[i];
					}
				} else if (indexOfInsert < (result.length - insertTarget.length)) {
					var splitResult = result.split(insertTarget);
					if ((splitResult !== null) &&
						(splitResult.length == 2)) {
						result = splitResult[0] + formattedInserts[i] + splitResult[1];
					}
				} else {
					result = result.substring(0, indexOfInsert) + formattedInserts[i];
				}
			}
		}
		this._trace.exiting(this.id,"formatMessage");
		return result;
	},

	extractBatchMessages: function (/*Object batch response*/value, /*Array*/ severityFilter,/*boolean?*/makeGroups) {
		var filter = {'info': true, 'warn': true, 'error': true};
		if (severityFilter) {
			filter = {'info': false, 'warn': false, 'error': false};
			if (severityFilter instanceof Array){
				for (var i=0; i<severityFilter.length; i++) {
					var severity = severityFilter[i];
					if (severity in filter) {
						filter[severity] = true;
					}
				}
			} else if (severityFilter instanceof String) {
				if (severityFilter in filter) {
					filter[severityFilter] = true;
				}
			}
		}
		
		var mesgs = [];	// Contains all messages.
		if (!value) {
			return mesgs;
		}
		
		var infoMsgs = [];
		var warnMsgs = [];
		var errMsgs = [];
		var collectMessage = function(msg) {
			
			if (filter[msg.severity] === false) {
				return; // Message is not of specified severity filter, skip it.
			}
			
			if (makeGroups === true) {
				switch (msg.severity) {
				case 'info': infoMsgs.push(msg); break;
				case 'warn': warnMsgs.push(msg); break;
				case 'error': errMsgs.push(msg); break;
				default: errMsgs.push(msg); break;
			}
			} else {
				mesgs.push(msg);
			}
		};
		
		var getMessageSeverity = function(message) {
			if (message && message.id) {
				var severityChar = message.severity;
				switch (severityChar) {
				case 'I':
				case 'i':
					return 'info';
				case 'W':
				case 'w':
					return 'warn';
				case 'E':
				case 'e':
					return 'error';
				default:
					return 'error';
				}
			}
			return 'info';
		};
		
		var extractMessages = function(messages) {
			var message;
			if (messages && messages.length > 0) {
				for (var iCount=0; iCount<messages.length; iCount++) {
					message = messages[iCount];
					var msg = {
						id: message.id || '',
						severity: getMessageSeverity(message),
						text: com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLandNormalize(message.text),
						inserts: message.inserts || null
					};
					
					collectMessage(msg);
				}
			}
		};
		
		// Extract
		// Global (batch request) messages
		extractMessages(value.messages);	
		// Messages per sub request
		if (value.content && value.content.length > 0) {
			for (var iCnt=0; iCnt<value.content.length; iCnt++) {	
				extractMessages(value.content[iCnt].messages);
			}
		}
		
		// return
		if (mesgs.length === 0) {
			mesgs = mesgs.concat(errMsgs);
			mesgs = mesgs.concat(warnMsgs);
			mesgs = mesgs.concat(infoMsgs);
		}
		return mesgs;
	},

	showMessage: function (msg, msgArea, preformatted) {
		preformatted = preformatted || false;
		switch (msg.severity) {
		case 'info': msgArea.showInfoMessage(preformatted ? msg.text : this.formatMessage(msg.text, msg.inserts), msg.id);  break;
		case 'warn': msgArea.showWarningMessage(preformatted ? msg.text : this.formatMessage(msg.text, msg.inserts), msg.id);  break;
		case 'error': msgArea.showErrorMessage(preformatted ? msg.text : this.formatMessage(msg.text, msg.inserts), msg.id);  break;
		default: msgArea.showErrorMessage(preformatted ? msg.text : this.formatMessage(msg.text, msg.inserts), msg.id);  break;
		}
	},
	
	showMessages: function (msgs, msgArea, preformatted) {
		for (var i=0; i<msgs.length; i++) {
			this.showMessage(msgs[i], msgArea, preformatted);
		}
	},
	
	getMessageID: function(message) {
	    if (message != null && message.length >= 10) {
	        return message.substring(0, 9);
	    }

	    return null;
	}
});
