dojo.provide("com.ibm.security.modeling.util.TopologyTransformer");

dojo.declare("com.ibm.security.modeling.util.TopologyTransformer", null, {
	
	transformRoleItems: function(roleItems){
		// summary:
		//     Transform the roles raw data 
		// roleItems: Object
		//     Object contains reference to the raw role items object
		
		if(this.validateRoleItem(roleItems)){
			var roleList = [];
			for(var i=0;i<roleItems.length;i++){
				
				var role = {};
				role.id = roleItems[i].getId();
				
				// To load all properties from nav model.
				roleItems[i].getProperties({sync:true});
				
				for(var propIndex=0;propIndex<roleItems[i]._props.length;propIndex++){
					
					var property = roleItems[i]._props[propIndex]._prop;
					
					if(property["id"] == "NAME"){
						role.name = property["value"];
					}else if(property["id"] == "DESCRIPTION"){
						role.description = property["value"];
					}else if(property["id"] == "LIFECYCLE_STATUS"){
						role.status = property["value"];
					}
				}
				
				roleList.push(role);
			}
			return roleList;
		}else{
			throw  "Object not suported";
		}
	},
	
	validateRoleItem: function(roleItems){
		// summary:
		//     Validate role item data to confirm that, transformer understand raw data.
		// roleItems: Object
		//     Object contains reference to the raw role items object
		
		if(roleItems && roleItems.length > 0 && roleItems[0] && roleItems[0]._item && roleItems[0]._item.label){
			return true;
		}
		
		return false;
	},
	
	transformTopologyItems : function(resourceBundle,topologyItems){
		// summary:
		//     Transform the roles raw data 
		// roleItems: Object
		//     Object contains reference to the raw role items object
		
		if(this.validateTopologyItems(topologyItems)){
			for(var i=0;i<topologyItems.length;i++){
				for(var j=0; j<topologyItems[i]._item.properties.length;j++){
					var itemProperties = topologyItems[i]._item.properties[j];
					if(itemProperties.id == "LIFECYCLE_STATUS"){
						if(itemProperties.displayValue){
							itemProperties.displayValue = this._getPerLocale(resourceBundle,"LIFECYCLE_ROLE_STATUS_", itemProperties.displayValue);	
						}
						if(itemProperties.value){
							itemProperties.value = this._getPerLocale(resourceBundle,"LIFECYCLE_ROLE_STATUS_", itemProperties.value);	
						}
					}
				}
			}	
			return topologyItems;
		}
	},
	
	validateTopologyItems: function(topologyItems){
		// summary:
		//     Validate topology item data to confirm that, transformer understand raw data.
		// topologyItems: Object
		//     Object contains reference to the raw topology items object
		
		if(topologyItems && topologyItems.length > 0 && topologyItems[0] && topologyItems[0]._item && topologyItems[0]._item.properties){
			return true;
		}
		
		return false;
	},
	
	_getPerLocale: function(/*resourceBundle*/ resourceBundle, /*status keyPrefix*/ keyPrefix, /*status enum key*/ key) {
		var text = null;
		if (key) {
			text = resourceBundle[keyPrefix+key.toUpperCase()];
		}
		return text ? text : key;
	}
});