dojo.provide("com.ibm.security.modeling.util.TopologyViewSummaryMediator");

dojo.requireLocalization("com.ibm.security.modeling.projects", "UsersResourcesSelectionPanelText");

/*
 * 
 * This Class is a mediator between TopologyView and Summary Widget.
 * This encapsulates connecting to widget's item selection/deselection events,
 * deselect item in widget on removing the item from summary widget. 
 */
dojo.declare("com.ibm.security.modeling.util.TopologyViewSummaryMediator", [],
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.util.TopologyViewSummaryMediator"),
	topologyWidget: null,
	tableWidget: null,
	summaryWidget: null,
	
	sectionDisplayName: "SectionDisplayName",
	sectionName: "sectionName",
	sectionId: "sectionId",
	attributeType: "attributeType",
	attributeUri: "attributeUri",
	countCallback: null,
	topologyWidgetWrapper: null,
	localResources: null,
	unSelectItemFromTableCallback: null,
	
	_currentSelection: null,
	
	/*
	 * Creates connections to the events for TopologyView to add the selected 
	 * items from Topology into Summary widget.
	 */
	constructor: function(){
	
		this._trace.entering(this.id, "constructor");
	
		dojo.mixin(this, arguments[0]);
				
		this._currentSelection = [];
		
		this._setTopologyWidget(this.topologyWidget);
		
		this._trace.exiting(this.id, "constructor");
	},
	
	_setTopologyWidget: function(/*dijit widget*/ widget) {
		this.topologyWidget = widget;
		if (this.topologyWidget) {
			dojo.connect(this.topologyWidget.selection, "onSelectionChanged", this,dojo.hitch(this, "_onTopologySelectionChanged", "navigatorWidgetContainer1"));
		} else {
			// TODO: remove the connect handle
		}
	},
	
	//Need to get the details of the selected item to put into summary.
	_getItem: function(itemid){
		return this.topologyWidgetWrapper.getItem(itemid);
	},
	
	_getItem_old: function(itemId){
		this._trace.entering(this.id, "_getItem");
		
		//this._trace.info("Entering", "_getItem");
		var attrId = this.topologyWidget.attributeId;
		var valueStore = new com.ibm.security.modeling.tables.JsonRestStore( {
			target:"/isrm/rest/userscope/attributes/" + attrId,
			idAttribute:"id",
			syncMode: true
		});

		var item = valueStore.fetch( {
			query: {
				id:itemId
			}
		});
		this._trace.exiting(this.id, "_getItem");
		
		return item.results[0];	
	},
	
	_removeItemsFromSummaryWidget: function(unselectedItems){
		var i;
		for(i=0; i<unselectedItems.length;i++){
			this.summaryWidget.removeSectionItem(this.sectionName,unselectedItems[i]);
		}
	},
	
	_onTopologySelectionChanged: function(navwidgetid)
		{
		this._trace.entering(this.id, "_onTopologySelectionChanged");

		var itemIdArray = this.topologyWidgetWrapper.getSelectedItemIds();
		
		var unselectedItems = this.topologyWidgetWrapper.getUnselectedItems(this._currentSelection);
		this._removeItemsFromSummaryWidget(unselectedItems);
		
		for(var i=0; i<itemIdArray.length; i++){
			this._addTopoogyItemInSummaryWidget(itemIdArray[i]);
		}
			
		this.countCallback();
		
		this._currentSelection = itemIdArray;
		this._trace.exiting(this.id, "_onTopologySelectionChanged");
			
	},
	
	_addTopoogyItemInSummaryWidget: function(itemId){
		
		this._trace.entering(this.id, "_addTopoogyItemInSummaryWidget");

		var item = this._getItem(itemId);
		var section = this.summaryWidget.addSection(
					{
						heading: this.sectionDisplayName,
						attrName:this.sectionName, 
						id: this.sectionName, 
						sectionId:this.sectionName, 
						attributeType: this.attributeType, 
						attributeUri: this.attributeUri, 
						headingClass: "sectionHeading", 
						onCloseCallBack: dojo.hitch(this,this._removeAttributeFromSelectionList,"section")
					});
		if(this.topologyWidgetWrapper.includeSubtree){
			textValue = item.value + this.topologyWidgetWrapper.includeSubtreeText;
		}else{
			textValue = item.value;
		}
		this.summaryWidget.addSectionItem(
					section, 
					{
						id:itemId, 
						sectionId:this.sectionName, 
						attributeType: this.attributeType,
						text: textValue, 
						itemUri: item.uri, 
						contentClass: null, 
						includeSubtree: this.topologyWidgetWrapper.includeSubtree,
						onCloseCallBack:dojo.hitch(this,this._removeValueFromSelectionList,"item")
			});
					
		this._trace.exiting(this.id, "_addTopoogyItemInSummaryWidget");
	},
	
	// TBDs
	_removeAttributeFromSelectionList: function(sectionItem, item){
		this._trace.entering(this.id, "_removeAttributeFromSelectionList");
		
		// We need to deselect all the items on Topology.
		if(item.sectionId == this.sectionName){
			if(this.topologyWidgetWrapper.getCurrentView() == "TABLE"){
				this.unSelectItemFromTableCallback(this.sectionName, item.id, "HIERARCHY", true);
			}else{
			//this._setSelectedItems([]);
			this.topologyWidgetWrapper.setSelectedItems([]);
		}
		}

		//var sectionitems = this.summaryWidget.getSectionItems(item.id);
		/*
		
		for(var i=0; i<sectionitems.length; i++){
			console.log("Removing an item from section " + sectionItem.heading + "sectionitem" + sectionitems[i]);
		}
		
		*/
		this.countCallback();
		// This need to trigger unselecting the items from the topologyWidget.
		this._trace.exiting(this.id, "_removeAttributeFromSelectionList");
	},
	
	_removeValueFromSelectionList: function(param,item){
		
		this._trace.entering(this.id, "_removeValueFromSelectionList");
		
		if(this.topologyWidgetWrapper.getCurrentView() == "TABLE"){
			this.unSelectItemFromTableCallback(this.sectionName, item.id, "HIERARCHY", false);
		}else{
		this.topologyWidgetWrapper.deselectItem(item.id);
		}
		
		//console.log("Removing an item " + sectionValueItem);	
		// This need to trigger unselecting the item from the topologyWidget.
		this.countCallback();
		this._trace.exiting(this.id, "_removeValueFromSelectionList");
	}
	
});