dojo.provide("com.ibm.security.modeling.util.TransformerUtility");

dojo.declare("com.ibm.security.modeling.util.TransformerUtility", null, {
	
	roleItemsTransformer: function(roleItems){
		// summary:
		//     To search for the appropriate transformer and 
		// pass the roleItem to transform it.
		// roleItems: Object
		//     Object contains reference to the raw data of role items
		
		// Get array of transformers
		if( ! this.roleItemsTransformers){
			this.roleItemsTransformers = this.getRoleItemsTransformers();
		}
		
		// Find appropriate transformer and pass roleItems to transform it.
		for(var i =0; i < this.roleItemsTransformers.length;i++){
			var transformer = this.roleItemsTransformers[i];
			if(this._isSuitableTransformer(transformer, roleItems)) {
				return transformer.transformRoleItems(roleItems);
			}
		}
	},
	
	topologyItemsTransformer: function(resourceBundle, topologyItems){
		// summary:
		//     To search for the appropriate transformer and 
		// pass the roleItem to transform it.
		// roleItems: Object
		//     Object contains reference to the raw data of role items
		
		// Get array of transformers
		if( ! this.topologyItemsTransformers){
			dojo.require("com.ibm.security.modeling.util.TopologyTransformer");
			this.topologyItemsTransformers = new com.ibm.security.modeling.util.TopologyTransformer();
		}
		
		return this.topologyItemsTransformers.transformTopologyItems(resourceBundle, topologyItems);
	},
	
	
	_isSuitableTransformer: function(transformer, roleItems) {
		return transformer.validateRoleItem(roleItems);
	},
	
	getRoleItemsTransformers: function(){
		// summary:
		//     Return array of registered transformers
		
		dojo.require("com.ibm.security.modeling.util.TopologyTransformer");
		dojo.require("com.ibm.security.modeling.util.TableTransformer");
		
		// Create array of object of transformers
		var roleItemsTransformers = [
								        new com.ibm.security.modeling.util.TopologyTransformer(),
								        new com.ibm.security.modeling.util.TableTransformer()
								     ];
		return roleItemsTransformers;
	}
});