/* 
Licensed Materials - Property of IBM

5724-Z11??
Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.util.WidgetUtilities");

dojo.require("dijit.form.DropDownButton");
dojo.require("ibm.tivoli.tip.dijit.TIPButton");

// Utilities for widgets.
dojo.declare("com.ibm.security.modeling.util.WidgetUtilities", null, {

	// Creates a button with the specified attributes.
	//
	// @param label: String
	//      Text describing the button (used as tooltip/alt text).
	// @param showLabel: boolean
	//      Whether to show the label in the button itself.
	// @param iconClass: String
	//      String describing the css class describing the button image.
	// @param onClick: function
	//      The function to be executed when the button is clicked.
	// @param id: String
	//      The identifier used to locate the button.
	// @param enabled: boolean
	//      boolean flag specifying whether the button should be initially enabled.
	// @param menuObject: dijit.Menu
	//      Object contains menu object.
	// @param tooltipDialog: dijit.TooltipDialog
	//      Tooltip dialog object.
	//
	// @returns dijit.form.DropDownButton or ibm.tivoli.tip.dijit.TIPButton
	//          with the specified attributes.
	createButton: function(
			label,
			showLabel,
			iconClass,
			onClick,
			id,
			enabled,
			menuObject,
			tooltipDialog) {
		var button = null;
		if (onClick == null) {
			onClick = function() { return true; };
		}
		if ((menuObject != null) && (dojo.isObject(menuObject))) {
			button = new dijit.form.DropDownButton({
				label: label,
				showLabel: showLabel,
				iconClass: iconClass,
				onClick: onClick,
				id: id,
				title:label,
				dropDown: menuObject
			});
		} else if ((tooltipDialog != null) && (dojo.isObject(tooltipDialog))) {
			button = new dijit.form.DropDownButton({
				label: label,
				showLabel: showLabel,
				iconClass: iconClass,
				onClick: onClick,
				id: id,
				dropDown: tooltipDialog,
				isContainer: true
			});
		} else {
			button = new ibm.tivoli.tip.dijit.TIPButton({
				label: label,
				showLabel: showLabel,
				iconClass: iconClass,
				onClick: onClick,
				id: id
			});
		}

		if (enabled != true) {
			this.disableButton(button);
		}

		return  button;
	},

	// Disables the specified button.
	//
	// @param button: dijit.form.DropDownButton or ibm.tivoli.tip.dijit.TIPButton
	//      Button that should be disabled.
	disableButton: function(
			button) {
		button.set("disabled", true);
		dojo.removeClass(button.domNode, "activeImage");
		dojo.addClass(button.domNode, "inactiveImage");
		//FIX FOR IE
		if (button.iconNode) {
			dojo.addClass(button.iconNode, "inactiveImage");
		}
	},

	// Enables the specified button.
	//
	// @param button: dijit.form.DropDownButton or ibm.tivoli.tip.dijit.TIPButton
	//      Button that should be enabled.
	enableButton: function(
			button) {
		button.set("disabled", false);
		dojo.removeClass(button.domNode, "inactiveImage");
		dojo.addClass(button.domNode, "activeImage");
		//FIX FOR IE
		if (button.iconNode) {
			dojo.removeClass(button.iconNode, "inactiveImage");
		}
	}

});
