/*
Licensed Materials - Property of IBM

Role and Policy Modeler

? Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.widgets.AttributeScopingPanel");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.security.modeling.dijit.layout.BorderContainer");
dojo.require("com.ibm.security.modeling.dijit.layout.ContentPane");
dojo.require("com.ibm.security.modeling.layouts.TemplatedPanelWithLayout");
dojo.require("dijit.Dialog");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.widgets.Summary");
dojo.require("com.ibm.security.modeling.util.NlsUtilities");
dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");
dojo.require("com.ibm.security.modeling.widgets.FilteringWidget");
dojo.require("com.ibm.security.modeling.widgets.RequiredTextField");
dojo.require("com.ibm.security.modeling.api.HttpOverrideRestServiceFactory");
dojo.require("dojox.collections.ArrayList");

dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling", "LogMessages");

dojo.declare("com.ibm.security.modeling.widgets.AttributeScopingPanel",
            [ com.ibm.security.modeling.layouts.TemplatedPanelWithLayout ],
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.widgets.AttributeScopingPanel"),
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.widgets","templates/AttributeScopingPanel.html"),	
	widgetsInTemplate: true,
	// globalResources: Object
	//		The global resource bundle
	globalResources: null,
	// localResources: Object
	//		The resource bundle for this widget set
	localResources: null,
	// messageResources: Object
	//      The resource bundel for TMS messages
	//messageResources: null,
	// namespace: String
	//		The identifier for the current portlet
	namespace: null,
	// messageArea: ibm.tivoli.tip.dijit.TIPMessageArea
	//		Message area for error display
	messageArea: null,
	// hostname: String
	//		Hostname for server requests
	hostname: "",
	// port: String
	//		Port for server requests
	port: "16311",
	// contextRoot: String
	//			The context root of the server from which Content is being served.
	contextRoot: "",
	dropDownLabel:"",
	attributeStore:null,
	searchAttribute:"attributeName",
	isStoreLocal:true,
	defaultValue:"",
	countParam:"&countOnly=true",
	noResourceSelectionMsg:"",
	selectionLimitMsg:"",
	performShopping:true,
	selectionType:"",
	idAttribute:"uri",
	_isThresholdMessageShown:false,
	ENTITYTYPE: "USER",
	bIsfilteringWidgetNeeded: true,
	htmlUtilities: new com.ibm.security.modeling.util.StringUtils(),
	
	includeSubtreeText: null,
	_mediators: null,
	learnMoreLinkFile: null,
	__learnmorelink: null,
	TABLE_ATTRIBUTE_NAME: "attributename",
	
	// summaryHeaderLabel: String
	//		Label to be shown on summary widget.
	//		If not specified then SUMMARY_HEADER_LABEL from localResources is considered.
	summaryHeaderLabel: null,
	
	constructor: function (args) {
		// summary:
		// 			Widget that provides the UI for the user selection page.
		// description: 
		//			this widget sets up the selection widgets and behavior
		if (!this.globalResources) {
			this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		} 
		this.messageResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "LogMessages");
		this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
		this.connections=[];
		this._mediators = []; 
		this._tableConnections={};
	},
	
	_setupDefaultSubtreeOptionText: function(){
		if(!this.includeSubtreeText && (this.includeSubtreeText != "")){
			this.includeSubtreeText = this.localResources.INCLUDING_SUBTREES;
		}
	},
	postCreate: function() {
		// tags:
		//		protected
		this._setupDefaultSubtreeOptionText();
		this._trace.entering("", "postCreate");
		this.inherited(arguments);
		this._overrideContainerLtrFunction();
		this.setupSummaryWidget();
		this._createAttributeSelectionWidget();
		dojo.addClass(this.domNode,"fullSize");
		if(this.learnMoreLinkFile != null){
			this._createLearnMoreLink();
		}
		this._trace.exiting("", "postCreate");
	
	},
	
	startup: function() {
			// tags:
			//		protected
			this.inherited(arguments);
			this.summaryWidget.updateFooterText(this.globalResources.TOTAL);
	},
	
	_destroyMediators: function(){
		var i;
		for(i=0;i<this._mediators.length;i++){
			delete this._mediators[i];
		}
	},
	destroy: function()	{
			dojo.forEach(this.connections, dojo.disconnect);
			this._destroyMediators();
			this.summaryWidget.removeAll();
			this.summaryWidget.destroy();
			if(this.__learnmorelink != null){
				this.__learnmorelink.destroy();
			}
	  	      this.filteringWidget.destroy();
	  	      
	  	    for(var i in this._tableConnections){
	  	    	var handlers = this._tableConnections[i];
	  	    	for (var j in handlers) dojo.disconnect(handlers[j]);
	  	    }
			this.inherited(arguments);
	},
	
	destroyRecursive: function()	{
				dojo.forEach(this.connections, dojo.disconnect);
				this.summaryWidget.removeAll();
				//this.summaryWidget.destroy();
		  	      //this.filteringWidget.destroy();
				this.inherited(arguments);
	},
	
	_overrideContainerLtrFunction: function() {
		// summary:
		// 			Setups the direction for the widget.
		this.container.isLeftToRight = NlsUtilities.isLtr;
	},
	
	setupSummaryWidget:function(){
			// summary:
			// 			Creates and setups summary widget
			this.summaryWidget =  new com.ibm.security.modeling.widgets.Summary ({
				id: this.id+"_attributeSelectionSummaryWidget",
				headerLabel: this.summaryHeaderLabel || this.localResources.SUMMARY_HEADER_LABEL,
				bodyLabel: "",
				footerContent: "sample footer",
				footerLink: "www.ibm.com",
				footerLinkText: this.localResources.VIEW_ALL
			});
			
			this.summaryWidget.startup();
			
			dojo.connect(this.summaryWidget.footer, "onClickViewAll", dojo.hitch(this, this._openViewAllDialog));
			dojo.place(this.summaryWidget.domNode, this.attributeSelectionSummaryWidget, "only");
	},
	
	_createAttributeSelectionWidget:function(){
		// summary:
		// 			Creates and setups dropdown widget
		this._trace.entering("", "_createAttributeSelectionWidget");
		this.filteringWidget=new com.ibm.security.modeling.widgets.FilteringWidget({
			searchAttr:this.searchAttribute,
			dropDownLabelNode: this._dropDownLabelNode,
			store:this.attributeStore,
			isStoreLocal:this.isStoreLocal,
			hookForFilteringSelect:this._dropDown,
			hookForStackContainer:this._stackContainer,
			selectedValueCallback:dojo.hitch(this,this.selectedValueCallback)
		},this.id +"_attributeSelection");
		dojo.addClass(this.filteringWidget.filteringSelect.domNode,"attributeDropDown");
		dojo.addClass(this.filteringWidget,"contentArea");
		this.filteringWidget.startup();
		this._trace.exiting(this.searchAttribute, "_createAttributeSelectionWidget");
	},
	selectedValueCallback:function(value,item){
		// summary:
		// 			Function to create widget depending on the type of attribute item in the dropdown
		// value:string
		//			value of the dropdown selected option
		// item:object
		//			item of the dropdown selected option
	     var widget;
		    if(item.type=="IDENTITY" ){
		       //widget= this.attachTable(value,item,parentContainer);
		    	widget= this.attachIdentityTable(value, item);
		    }else if(item.type=="HIERARCHY"){
		       widget= this.attachTopology(value, item);
		    }else if(item.type=="NAME"){
		       widget= this.attachNameTable(value,item);
		    }else if(item.type=="STRING"){
		       widget= this.attachTable(value,item);
		    }else{
		        widget= this.attachTextBox(value);
		    }
			return widget;
	},
	
	attachNameTable:function(value,item){
        var table = this._createNameAttributeTable(value,item);
        table.domNode.setAttribute(this.TABLE_ATTRIBUTE_NAME, value);
        return table;
	},
	
	attachTextBox:function(value){
	  
	    var textbox = new com.ibm.security.modeling.widgets.RequiredTextField({id:this.id +value});
	    return textbox;
	},
	
	attachTable:function(value,item){
		this._trace.entering(value, "attachTable");
		var uri=item.uri.toString();
		
		dojo.require("com.ibm.security.modeling.api.AttributeScopeRestManager");
		var schemaRestMgr= new com.ibm.security.modeling.api.AttributeScopeRestManager({resourceUri:uri, idAttribute:"value"});
        var store = schemaRestMgr.getIsrmTableRestStore();

		dojo.require("com.ibm.security.modeling.tables.Table");
		var tableHeader=this.htmlUtilities.parseHTMLTag(item.displayName);
		var structure=[{field: 'value',name:tableHeader,width: '100%'}];
		var table = new com.ibm.security.modeling.tables.Table({
			id : this.id + item.dbId,
			dataStore:store,
			messageArea: this.messageArea,
			isClientSide:false,
			isFilterRendered:true,
			isSelectAllRendered:false,
			isRefreshRendered:false,
			autoHeight:false,
			isDynamic:true,
			structure:structure,
			isMultiSelect:true,
			defaultSortIndex:1,
			entityTypeText:value
		});
		table.domNode.setAttribute(this.TABLE_ATTRIBUTE_NAME, value);  //Adding for FIT to find the table with attributename.
		table.grid.attributeDetail=item;
		this._setupTableConnections(table);
		this._trace.exiting(table.id, "attachTable");
        return table;	
	},
	
	attachIdentityTable:function(value,item){
		
		var url = item.uri.toString();
		//"https://localhost:16311/isrm/rest/userscope/attributes/1299740550726754220"
		this._trace.entering(value, "attachIdentityTable");

		dojo.require("com.ibm.security.modeling.tables.UserTable");
		attributeValueStructure = com.ibm.security.modeling.tables.CustomDisplayStructureFactory.loadUserStructure(customColumn,dojo.hitch(this,this._errorHandler));
		var restStoreA = new com.ibm.security.modeling.tables.JsonRestStore({
			target:url, 
			//fetchProperties:fetch,
			schema:{
			"description":"A project",
			"type":"object",
			"properties": {
			    "customAttr1":{"type":"string"},
		        "customAttr2":{"type":"string"},
		        "customAttr3":{"type":"string"},
		        "identityCount":{"type":"integer"}
			}
		}
		});
		
		var customColumn={10:{field: 'identityCount',name: "Total No. of Employees" ,width: '10%', isSearchable:false}};
		var table = new com.ibm.security.modeling.tables.UserTable(
				{
					id: this.id + item.dbId,
					dataStore: restStoreA, 
					customCols: customColumn, 
					isClientSide: false,
					isFilterRendered: true,
					isRefreshRendered: true,
					isMultiSelect: true,
					autoHeight: false,
					isDynamic: true,
					isSearchMultiSelect: false,
					entityTypeText: value,
					tableType:"usersResourceSelection",
					entityTypeText:value
					//searchInstructions:this.globalResources.SEARCH +" "+value
				});

		table.domNode.setAttribute(this.TABLE_ATTRIBUTE_NAME, value); //Adding for FIT to find the table with attributename.
		table.grid.attributeDetail=item;
		this._setupTableConnections(table);
		this._trace.exiting(table.id, "attachIdentityTable");
        return table;			
	},
	attachTopology:function(value, item){					
					dojo.require("com.ibm.security.modeling.widgets.HierarchicalAttributeView");
					var attrUri = item.uri.toString();
					var attrId = attrUri.substring(attrUri.lastIndexOf("/")+1, attrUri.length); 
					var selectedAttribute = this.filteringWidget.getFilteringSelectItem();
					var navWidget = new com.ibm.security.modeling.widgets.HierarchicalAttributeView(
						{
							id:this.id+selectedAttribute.dbId+"topology",
							includeSubtreeOption: this.includeSubtreeOption,
							entityType: this.ENTITYTYPE,
							attributeId:attrId,
							localResources:this.localResources,
							messageArea: this.messageArea,
							includeSubtreeText:this.includeSubtreeText,
							includeSubtree: this.includeSubtree,
							entityTypeText:value
							//searchInstructions:this.globalResources.SEARCH +" "+value
						}
					);
					navWidget.domNode.setAttribute("tablename", selectedAttribute.name);
					this.onSwitchableHierarchicalAttributeViewSwitch("TABLE", navWidget);
					this.connect(navWidget, "onSwitch", "onSwitchableHierarchicalAttributeViewSwitch");
					return(navWidget);
	},
	
	onSwitchableHierarchicalAttributeViewSwitch: function(currentView, widget){
		if(currentView == "TABLE"){
			var table = widget.getTable();
			table.domNode.setAttribute(this.TABLE_ATTRIBUTE_NAME, widget.entityTypeText);
			this._setupTableConnections(table);	
		}else{
				
			var selectedAttribute = this.filteringWidget.getFilteringSelectItem();
			var attrUri = selectedAttribute.uri.toString();
			var attrId = attrUri.substring(attrUri.lastIndexOf("/")+1, attrUri.length); 
			dojo.require("com.ibm.security.modeling.util.TopologyViewSummaryMediator");
			var mediator = new com.ibm.security.modeling.util.TopologyViewSummaryMediator(
				{
					sectionDisplayName: selectedAttribute.displayName,
					sectionName: selectedAttribute.name,
					sectionId: attrId,
					attributeType: selectedAttribute.type,
					attributeUri: selectedAttribute.uri,
					topologyWidget: widget.getTopology(),
					topologyWidgetWrapper: widget,
					summaryWidget: this.summaryWidget,
					countCallback: dojo.hitch(this, this._updateTotalCount),
					unSelectItemFromTableCallback:dojo.hitch(this, this._unSelectItemFromTable)
			});
			this._mediators.push(mediator);
		}
		this.resize();
	},
	_setupTableConnections:function(table){
		
		
		
		var self= this;
		this._setupSelectionConnection(table);
		var toggleAllSelection = table.grid.rowSelectCell.toggleAllSelection;
		table.grid.rowSelectCell.toggleAllSelection = function(checked){
			self.performShopping=false; 
			self._disconnecSelectionConnection(table);
			try{
				dojo.hitch(table.grid.rowSelectCell,toggleAllSelection)(checked);
				if(!self.preventToggleAllSelction) self._handleSelectAll(checked, table);
			}finally{
				self._setupSelectionConnection(table);
				self.performShopping=true;
			}
			
			
			
		};
		var resetAllSelection = table.resetAllSelection;
		table.resetAllSelection = function(){
			self.preventToggleAllSelction=true; 
			self._disconnecSelectionConnection(table);
			try{
				dojo.hitch(table,resetAllSelection)();
			}finally{
				self.preventToggleAllSelction=false;
				self._setupSelectionConnection(table);
			}
			
			
		};
	},
	_setupSelectionConnection:function(table){
		//table object is meant for shopping cart , so we should update then connection .
		//for the table wich involves connection with shopping cart we will preserve the selection .
		var selectHandler = dojo.connect(table.grid.selection, "onSelected",this, function(index){
			this.selectionType="Select";
			this._trackTableSelections(index);
		});
		var deSelectHandler = dojo.connect(table.grid.selection, "onDeselected", this ,function(index){
			this.selectionType="Deselect";
			this._trackTableSelections(index);
		});
		this._tableConnections[table.id] = [selectHandler,deSelectHandler];

	},
	_disconnecSelectionConnection:function(table){
		//table object is meant for shopping cart , so we should update then connection .
		//for the table wich involves connection with shopping cart we will preserve the selection .
		dojo.disconnect(this._tableConnections[table.id][0]);
		dojo.disconnect(this._tableConnections[table.id][1]);
		this._tableConnections[table.id] = [];
		
	},
	
	_handleSelectAll:function(checked, updatedtable){
		
			isSelectionRemoveType = !checked;
			var selectedChildWidget = this.filteringWidget.getSelectedChild();
			
			var isSwitchableHierarchicalAttributeView = false;
			
			var table = null;
			if(selectedChildWidget.declaredClass == "com.ibm.security.modeling.widgets.HierarchicalAttributeView"){
				table = selectedChildWidget.getTable();
				isSwitchableHierarchicalAttributeView = true;
			}else{
				table = selectedChildWidget;
			}
			
			if(updatedtable.id != table.id){
				return;
			}
			var currentAttrSelected = null;
			var selectedRowsCount = table.grid.get('rowCount');//using row count as when the select all checkbox is clicked it did not complete the row selections
			if(isSwitchableHierarchicalAttributeView){
				currentAttrSelected = this.filteringWidget.getFilteringSelectItem();
			}else{
				currentAttrSelected = table.grid.attributeDetail;//this.filteringWidget.getFilteringSelectItem();
			}
			var firstColumnName = table.grid.structure[0].field;
			if(isSelectionRemoveType){
				//Removing the entire section was causing problem in adding items again.
				//this.summaryWidget.removeSection(currentAttrSelected.id);
				//Hence iterating to remove each item of the section.
				
				var sectionItems=this.summaryWidget.getSectionItems(currentAttrSelected.id);
				if(sectionItems){
					var item;
					for(item in sectionItems){
						var itemId=item.toString();
								this.removeItemFromSummaryWidget(currentAttrSelected.id,currentAttrSelected.name,currentAttrSelected.displayName,currentAttrSelected.type, item,itemId);
							}
						}
				
			}else{
				//for adding new
				var i;
				for(i=0;i<selectedRowsCount;i++){
					var item=table.grid.getItem(i);
					if (item){
						//table.grid.rowSelectCell.toggleRow(i,false);
						var itemValue = item[firstColumnName];
						var itemId = item.id + "";
						if(item.id == undefined){
							itemId = item.value + "";	
						}
						var itemUri = null;
						if (item.uri) {
							itemUri = item.uri;
						}
						this._addToSummaryWidget(currentAttrSelected.id,currentAttrSelected.name,currentAttrSelected.displayName,currentAttrSelected.type, itemValue,itemId,currentAttrSelected.uri, itemUri, selectedChildWidget.includeSubtree);
					}
					
				}
			}
			this._updateTotalCount();

		
	},
	
	_trackTableSelections:function(index){
		this._trace.entering(index, "_trackTableSelections");
		if(!this.performShopping){
			return;
		}
		var selectedChildWidget = this.filteringWidget.getSelectedChild();
		
		var table = null;
		
		var isSwitchableHierarchicalAttributeView = false;
		if(selectedChildWidget.declaredClass == "com.ibm.security.modeling.widgets.HierarchicalAttributeView"){
			table = selectedChildWidget.getTable();
			isSwitchableHierarchicalAttributeView = true;
		}else{
			table = selectedChildWidget;
		}
		
		this._trace.finest('=========Inside _trackSelections',"_trackSelections");
		var currentAttrSelected = null;
		
		if(isSwitchableHierarchicalAttributeView){
			currentAttrSelected = this.filteringWidget.getFilteringSelectItem();
		}else{
			currentAttrSelected = table.grid.attributeDetail;//this.filteringWidget.getFilteringSelectItem();
		}
		
		var selectedRows = table.grid.selection.getSelected();
		var selectedRowsCount = selectedRows.length;
		var firstColumnName = table.grid.structure[0].field;
		
		if(this.selectionType=="Deselect"){
			var item=table.grid.getItem(index);
			if(item){
				this._hideMessages();
				var itemid = item.id;
				if(itemid == undefined){
					itemid = item.value;
				}
				this.removeItemFromSummaryWidget(currentAttrSelected.id,currentAttrSelected.name,currentAttrSelected.displayName,currentAttrSelected.type, itemid);
			}
		}else{
			//for adding new
			var i;
			for(i=0;i<selectedRowsCount;i++){
				var itemValue = selectedRows[i][firstColumnName];
				var itemId = selectedRows[i].id;
				if( (itemId == undefined) || (itemId == null) ){
					itemId = itemValue;
				}
				var itemUri = null;
				if (selectedRows[i].uri) {
					itemUri = selectedRows[i].uri;
				}
				
				this._addToSummaryWidget(currentAttrSelected.id,currentAttrSelected.name,currentAttrSelected.displayName,currentAttrSelected.type, itemValue,itemId,currentAttrSelected.uri, itemUri, selectedChildWidget.includeSubtree);
			}
		}
		this._updateTotalCount();
		this._trace.exiting("", "_trackTableSelections");
	},
	
	_addToSummaryWidget: function(/*attrbuteId*/sectionId, 
								/*attributeName*/sectionName, 
								/*attributesDisplayName*/sectionDisplayName,
								attributeType,itemValue,itemId,attributeUri, itemUri, includeSubtree)	{
		// summary:
		//		Adds an item into the summary widget if it does not exist earlier.
		
		this._trace.entering(this.id, "_addToSummaryWidget");
		var section = this.summaryWidget.addSection({heading: sectionDisplayName,attrName:sectionName, id: sectionId, sectionId:sectionId, attributeType: attributeType, attributeUri: attributeUri, headingClass: "sectionHeading", onCloseCallBack: dojo.hitch(this,this._removeFromSelectionList,"section")});
		
		if(attributeType == "HIERARCHY"){
			var itemValueWithSubtree = null;
			if(includeSubtree){
				itemValueWithSubtree = itemValue+this.includeSubtreeText;
			}else{
				itemValueWithSubtree = itemValue;
			}
			
			this.summaryWidget.addSectionItem(section, {id:itemId, sectionId:sectionId, attributeType: attributeType,text: itemValueWithSubtree, itemUri: itemUri, contentClass: null, onCloseCallBack:dojo.hitch(this,this._removeFromSelectionList,"item"), includeSubtree:includeSubtree});
		}else{
		this.summaryWidget.addSectionItem(section, {id:itemId, sectionId:sectionId, attributeType: attributeType,text: itemValue, itemUri: itemUri, contentClass: null, onCloseCallBack:dojo.hitch(this,this._removeFromSelectionList,"item")});		
		}
		
		this._trace.exiting(this.id, "_addToSummaryWidget");
	},
	
	removeItemFromSummaryWidget: function(/*attrbuteId*/sectionId,
										/*attributeName*/sectionName, 
										/*attributesDisplayName*/sectionDisplayName,
										attributeType,itemId){
		
		
		this.summaryWidget.removeSectionItem(sectionId,itemId);
		
	},
	removeSectionFromSummaryWidget: function(/*attrbuteId*/sectionId, /*attributeName*/sectionName, /*attributesDisplayName*/sectionDisplayName,attributeType)//(itemId, itemValueId, itemValue)
	{
		
		this.summaryWidget.removeSection(sectionId);
		
	},
	_removeFromSelectionList: function(param,item){
		//Summary
		// When-ever "remove value" icon image is clicked on the User Summary widget, this function gets invoked	
			var attrId=item.sectionId;
			var attributeType = item.attributeType;
			var itemId;
			var isAll=true;
			
			if ( attributeType== "NAME" || attributeType== "STRING" || attributeType== "IDENTITY" || attributeType== "HIERARCHY" || attributeType== "INT"){ 
				if(param=="item"){
					itemId = item.id;
					isAll=false;
				}
				this._unSelectItemFromTable(attrId,itemId,attributeType,isAll);
			}
		
	},
	_unSelectItemFromTable: function(attrId,itemId,attrType,isAll)
	{
		this._trace.entering(this.id, "_unSelectItemFromTable");
		var table=this.filteringWidget.getChild(attrId);
		if(!table){
			this._updateTotalCount();
			return;
		}
		if(table.declaredClass == "com.ibm.security.modeling.widgets.HierarchicalAttributeView"){
			if(table.getCurrentView() == "TOPOLOGY") {
				// TODO. Call the widget's deselection for the item and return.
				// table.deselect(attrId, itemId, attrType);
				if(isAll){
					// Means all the elements of this sections are removed. itemId is sectionId here.
					table.setSelectedItems([]);
				}else{
					// One element is being removed. itemId is itemId here.
					table.deselectItem(itemId);
				}
				return;
			}
			table = table.getTable();
			
		}
		
			if(table!=null){
				this.performShopping=false;
				if(isAll){
					table.deselectAll();
				}else{
					//var count= table.grid.indirectSelector.grid.rowCount;
					var count=table.grid.get('rowCount');
					var i;
					for (i =0;i<count;i++){
						var id=new String(table.grid.getItem(i).id);
						//When item does not have value, case of String/INT attribute values
						if(id == "undefined"){
							id=new String(table.grid.getItem(i).value);
						}
						if (id == itemId ){
							table.grid.rowSelectCell.toggleRow(i,false);
							break;
						}
					}
				}
				//setTimeout(dojo.hitch(this,this._updateTotalCount),200);
				this._updateTotalCount();
				this.performShopping=true;
			}
		
		this._trace.exiting(this.id, "_unSelectItemFromTable");
	},
	_updateTotalCount: function()
	{
		var scope=this._getSelectedScope();
		if(scope.length===0){//temp code as count is always returning some number
			this.summaryWidget.showFooterLink();
			this.summaryWidget.updateFooterText(this.globalResources.TOTAL);
			this.summaryWidget.updateFooterTotalNumber(0);
			this._hideMessages();
		}else{
			this._hideMessages();
			this._getCount(scope);
			if(this.isThreasholdLimitReached(scope) == true){
			  this.showMaxLimitExceedingWarning("warning");
			}
		}
		
	},
	isThreasholdLimitReached:function(scope){
		var isLimitReached=false;
		var attributeValueCount=0;
		var nameValueCount=0;
		var i;
		for(i=0;i<scope.length;i++){
			var item=scope[i];
			if(item.attributeType=="NAME"){
				nameValueCount=item.values.length;
			}else{
				attributeValueCount=attributeValueCount+item.values.length;
			}
		}
		
		if(nameValueCount > this.nameAttributeSelectionLimit || attributeValueCount > this.attrValueSelectionLimit){
			isLimitReached=true;
		}
		return isLimitReached;
	},
	showMaxLimitExceedingWarning: function(type) {
		this._isThresholdMessageShown=true;
		this._showMessage(type,this.messageResources[this.selectionLimitMsg]);
	},
	
	_handleAttributeFetchError: function(/*Object*/error) {
		// summary:
		//		Displays an error message on an error fetching the selected attribute from the store
		// error: Object
		//		The error that failed the fetch
		// tags:
		//		private
		this._isThresholdMessageShown=false;
		this._errorUtil.handleRESTError(error, this.messageArea);
		
	},
	_showMessage: function(/*String*/type, /*String*/message) {
		// summary:
		//			shows the given message and type in the local message area
		// tags:
		//			private
		
		this._hideMessages();
		
		if (type === "critical") {
			this.messageArea.showErrorMessage(message);
		}else if (type === "info") {
			this.messageArea.showInfoMessage(message);
		}else if (type === "warning") {
			this.messageArea.showWarningMessage(message);
		}

	},
	_hideMessages:function() {
		// summary:
		//			hide the given message and type in the local message area
		// tags:
		//			private
		this._isThresholdMessageShown=false;
		this.messageArea.hideMessages();

	},

	_errorHandler:function(error,ioargs){
		this.hideWorkingDialog();
		this._errorUtil.handleRESTError(error, this.messageArea);
		
	},
	initPanel:function(){
		this._hideMessages();
		this._loadAttributeDropDown();
		this.summaryWidget.updateFooterTotalNumber(0);
		this.summaryWidget.removeAll();
		this.filteringWidget._initialize(this.attributeStore);
		if(!this.bIsfilteringWidgetNeeded) {
			this.filteringWidget.toggleVisibility(true);
			this.filteringWidget.setUpdatedLabel(this.localResources.AVAILABLE_VALUES);
			dojo.addClass(this.summaryWidget.footer.summaryFooterTopNode, "hiddenFooter");
		}
	},
	
	_openViewAllDialog:function(){
		if (this._isTotalCountZero() ==  true) {
			return;
		} 			
		if (this.messageArea.isShowing) {
				this.messageArea._hideMessages();
		}
		
		var scope=this._getSelectedScope();
		this._showViewAllDialog(scope);
	},
	_getSelectedScope:function(){
		var attributes=this.summaryWidget.getAllSections();
		var scope=[];
		var attr;
		for(attr in attributes){
			var values=[];
			var items=this.summaryWidget.getSectionItems(attr);
			var type=attributes[attr].params.args.attributeType;
			if(type == "NAME"){
				var id;
				for( id in items){
					values.push(this._getValueForScopeRule(id));
				}
			}
			else if(type == "IDENTITY"){
				var id;
				for(id in items){
					values.push("users/" +id);
				}
			}
			else if(type == "HIERARCHY"){
				var id;
				for(id in items){
					if(items[id].item.includeSubtree){
						values.push(id + "/SUBTREE");
					}else{
					values.push(id + "/BASE");
					}
				}
			}
			else{
				var id;
				for(id in items){
					var val=new String((items[id]).item.text);
					values.push(val.toString());
				}
			}
			if(values.length == 0){
				continue;
			}
			var attributeUri=(attributes[attr].params.args.attributeUri).toString();
			var scopedItem={"attributeURI":attributeUri,"values":values,"attributeType":type};
			scope.push(scopedItem);
		}
		return scope;
	},
	
	_getValueForScopeRule: function(id){
		return id;
	},
	
	_showViewAllDialog:function(scope){
		dojo.require("com.ibm.security.modeling.widgets.AttributeScopingViewAllDialog");
		var dialog= new com.ibm.security.modeling.widgets.AttributeScopingViewAllDialog({
						targeturl:this.viewAllUrl,
						scopingRule:scope,
						localResources:this.localResources,
						globalResources:this.globalResources,
						helpFile: this.helpFile,
						parentWidget:this,
						idAttribute: this.idAttribute,
						messageArea:this.messageArea
						});
		dialog.startup();
		dialog.show();
		
	},
	_getCount:function(scope){
		var fetch = {onError:dojo.hitch(this,this._errorHandler)};
		var serviceFactory = new com.ibm.security.modeling.api.HttpOverrideRestServiceFactory();
		var extendedService = serviceFactory.createService(this.viewAllUrl+this.countParam, scope);
		
		var dataStore = new com.ibm.security.modeling.tables.JsonRestStore({
									target:"targeturl",
									service: extendedService,
									fetchProperties:fetch
							});
		dataStore.fetch({
					onComplete: dojo.hitch(this,this._setCount),
					onError:dojo.hitch(this,this._errorHandler),
					syncMode: true, 
					cache: "false"});
							
	},
	_setCount:function(item, request){
		var count=0;
		if(item && item.totalCount){
			count=item.totalCount;
		}
		if(count === -1){
			this.summaryWidget.hideFooterLink();
			this.summaryWidget.updateFooterText(this.globalResources.FILTER_LIMIT_EXCEEDED);
			this.summaryWidget.updateFooterTotalNumber("");
		}else{
			this.summaryWidget.showFooterLink();
			this.summaryWidget.updateFooterText(this.globalResources.TOTAL);
			this.summaryWidget.updateFooterTotalNumber(count);
		}

	},
	_isTotalCountZero:function(){
		var count=this.summaryWidget.getFooterTotalNumber();
		if((this.summaryWidget.getFooterText() == this.globalResources.TOTAL) && (count =="0" || count ==0)){
			this._showMessage("critical",this.messageResources[this.noResourceSelectionMsg]);
			return true;
		}
		return false;
	},
	validateSelection:function(){
		if(this._isTotalCountZero() ==  true){
			return false;
		}else{
		  var currentScope=this._getSelectedScope();
		  if(this.isThreasholdLimitReached(currentScope) == true){
			this.showMaxLimitExceedingWarning("critical");
			return false;
		 }else{
			return true;
		 }
		}
	},
	
	prePopulateValues: function(preSelectValues){

		var attrUri = this.selectedItem.uri;
		var attributeId = attrUri.substring(attrUri.lastIndexOf("/")+1, attrUri.length);
		var attributeName = this.selectedItem.name;
		var attributeType = this.selectedItem.type; 
		var i;
		for(i=0; i<preSelectValues.length; i++){
			var item = preSelectValues[i];
		
			var itemUri = item.uri;
			var itemId = itemUri.substring(itemUri.lastIndexOf("/")+1, itemUri.length);
			var itemValue = null
			if(item[this.valueAttribute]){
				itemValue = item[this.valueAttribute];	
			}else{
				itemValue = item.value;
			}
			
			this._addToSummaryWidget(
										attributeName,
										attributeName,
										attributeName,
										attributeType, 
										itemValue,
										itemId,
										attrUri, 
										item.uri);
		}
	},
	
	
	_getAttributeFromId: function(attrId){
		//TODO Need better way to get item by Id.
		var items = this.filteringWidget.store._arrayOfAllItems;
		var i;
		for(i=0;i<items.length;i++){
			if(items[i].dbId == attrId){
				return items[i];
			}
		}
		return null;
	},
	
	prePopulateScopingRules: function(userScoping) {
			var i;
			for(i = 0; i < userScoping.length; i++)
			{
				var attrType = null;
				var attributeURI = userScoping[i].attributeURI;
				var attrId = attributeURI.substring(attributeURI.lastIndexOf("/")+1, attributeURI.length);
				var value = null;
				var idCount = -1;
				var attrName = null;
				var actualValue = null;
				
				var currentAttrSelected;
				if( (attributeURI.lastIndexOf("/userscope/")>0) || (attributeURI.lastIndexOf("/permissionscope/")>0))
				{
					currentAttrSelected = this._getAttributeFromId(attrId);
					attrType = currentAttrSelected.type;
				}else if(attributeURI.lastIndexOf("/users")>0)
				{
					attrType = "NAME";
					attrId = "USERNAME";
					currentAttrSelected = this._getAttributeFromId(attrId);
				}else if(attributeURI.lastIndexOf("/permissions")>0)
				{
					attrType = "NAME";
					attrId = "PERMISSIONNAME";
					currentAttrSelected = this._getAttributeFromId(attrId);
				}
				
				var itemValue;
				var itemId;
				var itemUri;
				attrName = currentAttrSelected.name;
				var j;
				for(j = 0; j < userScoping[i].values.length; j++)
				{
					if((attrType=="STRING")||(attrType=="INT")) {
						itemValue = userScoping[i].values[j];
						itemId = itemValue;
						itemUri = "";
						//this.selectedValues.push({attrname:attrId,value:String(value), actualValue:String(value), type:attrType, identityCount:idCount});
						//this._addToSummaryWidget(attrId, value, value, attrName, attrType);
						this._addToSummaryWidget(currentAttrSelected.id,currentAttrSelected.name,currentAttrSelected.displayName,currentAttrSelected.type, itemValue,itemId,currentAttrSelected.uri, itemUri);
					}
					else if(attrType=="IDENTITY") {
						//this is for - /users/{valueId}
						value = userScoping[i].values[j].substring(userScoping[i].values[j].lastIndexOf("/")+1, userScoping[i].values[j].length);
						itemId = value;
						actualValue = userScoping[i].displayValues[j].displayValue;
						itemValue = actualValue;
						itemUri = userScoping[i].values[j];
						//this.selectedValues.push({attrname:attrId,value:String(value), actualValue:actualValue, type:attrType, identityCount:idCount});
						//this._addToSummaryWidget(attrId, actualValue, value, attrName, attrType);
						this._addToSummaryWidget(currentAttrSelected.id,currentAttrSelected.name,currentAttrSelected.displayName,currentAttrSelected.type, itemValue,itemId,currentAttrSelected.uri, itemUri);
						
					}
					else if(attrType=="HIERARCHY") {
						
						value = userScoping[i].values[j];
						itemId = value;
						var actualItemId = itemId.substring(0, value.lastIndexOf("/"));
						actualValue = userScoping[i].displayValues[j].displayValue;
						var valueWithoutAddedText = value.substring(0, value.lastIndexOf("/"));
						var includeSubtree = false;
						if (userScoping[i].values[j].indexOf("/SUBTREE")!= -1){
							includeSubtree = true;
						}
						else if (userScoping[i].values[j].indexOf("/BASE")!= -1) {
							includeSubtree = false;
						}
						itemValue = actualValue;
						itemUri = userScoping[i].values[j];
						//this.selectedValues.push({attrname:attrId,value:String(valueWithoutAddedText), actualValue:actualValue, type:attrType, identityCount:idCount});
						//this._addToSummaryWidget(attrId, actualValue, valueWithoutAddedText, attrName, "HIERARCHY_EDIT");
						this._addToSummaryWidget(currentAttrSelected.id,currentAttrSelected.name,currentAttrSelected.displayName,currentAttrSelected.type, itemValue,actualItemId,currentAttrSelected.uri, itemUri, includeSubtree);
					}
					else if(attrType=="NAME") {
						idCount = 1;
						value = userScoping[i].values[j].substring(userScoping[i].values[j].lastIndexOf("/")+1, userScoping[i].values[j].length);
						itemId = value;
						actualValue = userScoping[i].displayValues[j].displayValue;
						itemValue = actualValue; 
						itemUri = userScoping[i].values[j];
						//this.selectedValues.push({attrname:attrId,value:String(value), actualValue:actualValue, type:attrType, identityCount:idCount});
						
						//this.selectedValues.push({attrname:attrId,value:String(value), actualValue:actualValue, type:attrType, identityCount:idCount});
						//this._addToSummaryWidget(attrId, actualValue, value, attrName, attrType);
						
						this._addToSummaryWidget(currentAttrSelected.id,currentAttrSelected.name,currentAttrSelected.displayName,currentAttrSelected.type, itemValue,itemId,currentAttrSelected.uri, itemUri);
					}	
				}
			}
		
			this._updateTotalCount();
			//var projUtil = new com.ibm.security.modeling.projects.ProjectUtil({});
			//this.totalUsersSelected = projUtil.getIdentitiesFromScopeInformation("USERS", this.selectedValues);	
	},
	
	_createLearnMoreLink: function(){
		this.__learnmorelink = new com.ibm.security.modeling.projects.ScopingPanelLearnMoreLink({
			id: this.id + "helpLink",
			helpFileName: this.learnMoreLinkFile,
			contextRoot: this.contextRoot,
			linkText: this.globalResources.LEARN_MORE
		}, this._learnmorelink);
		
		dojo.style(dijit.byId(this.__learnmorelink.id+"_help").domNode, "display", "inline");
	}
	
});

dojo.declare("com.ibm.security.modeling.projects.ScopingPanelLearnMoreLink", com.ibm.security.modeling.util.LearnMoreWidget,{
	templateString:"<a dojoAttachPoint='link' ></a>"
});
