/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

? Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.widgets.AttributeScopingViewAllDialog");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Dialog");
dojo.require("dijit.form.Button");
dojo.require("com.ibm.security.modeling.dijit.layout.BorderContainer");
dojo.require("com.ibm.security.modeling.widgets.IsrmDialogBase");

dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.util.MessageUtilities");
dojo.require("com.ibm.security.modeling.api.HttpOverrideRestServiceFactory");
dojo.require("com.ibm.security.modeling.util.ErrorUtilities");

// NLS
dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");


// define the widget class
dojo.declare (
	"com.ibm.security.modeling.widgets.AttributeScopingViewAllDialog", com.ibm.security.modeling.widgets.IsrmDialogBase,
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.widgets.AttributeScopingViewAllDialog"),
	_errorUtil: null,
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.widgets","templates/AttributeScopingViewAllDialog.html"),
	widgetsInTemplate: true,
	
	
	// namespace: [readonly] String
	// 			Unique identifier to the instance of portlet page where this item will exist
	namespace:"",
	
	// contextRoot: [readonly] String
	//			The context root of the server from which Content is being served.
	contextRoot: "",
	// contextRoot: [readonly] String
	//			The context root of the server from which Content is being served.
	parentWidget : null,
	// heightFactor: number
	//		Gives the hight factor for widget
	heightFactor:1.3,
	// widthFactor: number
	//		Gives the width factor for widget
	widthFactor:1.4,
	// minHeight: number
	//		minimum height of widget
	minHeight:100,
	// targeturl:  String
	//			The url used to fetch the items for table.
	targeturl:"",
	// scopingRule: Array
	//			Data to be send through the post call
	scopingRule:null,
	// idAttribute:  String
	//			Id attribute for store
	idAttribute:"uri",
	
	
	constructor: function() {
		// tags:
		//		protected
		this._trace.entering("", "constructor");
		this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
		this.mu = new com.ibm.security.modeling.util.MessageUtilities();
		this._trace.object(arguments, "constructor", "Constructor args");
		this._trace.exiting("", "constructor");
	},
	postMixInProperties: function() {
		this._trace.entering(this.id, "postMixInProperties");
		
		this.inherited(arguments);
		if (!this.messageTypeFilter) {
			this.messageTypeFilter = null;
		}
		this._trace.exiting(this.id, "postMixInProperties");
	},
	buildRendering: function(){
	
		this.dialogTitle=this.localResources.SELECTED_ENTITY;
		this.inherited(arguments);
	},
	postCreate: function() { 
		// tags:
		//		protected
		this._trace.entering(this.id, "postCreate");
		
		dojo.addClass(this.domNode,"contentArea");
		this.inherited("postCreate", arguments);
		this._setupRestStore();
		this._setupTable();
		this.closeButton = this.addCommonButton({
			onClick:dojo.hitch(this, this.onCancel),
			type:"CLOSE"
		});
		
		this._trace.exiting(this.id, "postCreate");
		
	},
	_setupRestStore:function(){
		var fetch = {onError:dojo.hitch(this,this._showRESTError)};
		var target=this._getTableStoreTargetUri(this.targeturl);
		var serviceFactory = new com.ibm.security.modeling.api.HttpOverrideRestServiceFactory();
		var extendedService = serviceFactory.createService(target, this.scopingRule);
		
		this.dataStore = new com.ibm.security.modeling.tables.JsonRestStore({
									target:"targeturl",
									idAttribute: this.idAttribute,
									service: extendedService,
									fetchProperties:fetch
							});
	},
	_setupTable:function(){
		var queryParams=this._getTableQueryObject(this.targeturl);
		var table=this.parentWidget.getViewAllTable(this.dataStore,queryParams);
		table.startup();
		this.tableArea.set("content", table);
	},
	
	
	_showRESTError: function (error) {
		this._errorUtil.handleRESTError(error, null);
		var ma = this.messageArea;
		/*if (this.parentMessageArea) {
			ma = this.parentMessageArea;
		}*/

		if ((ma != null) && (error != null) && (error.responseText != null)) {
			var messageResponseSplits = error.responseText.split("text",-1);
			var message = null;
			
			if (messageResponseSplits.length > 1) {
				messageResponseSplits[1] = messageResponseSplits[1].replace(/^\":\"/g,"");
				message = messageResponseSplits[1].replace(/\"}$/g,"");
			} else {
				message = messageResponseSplits[0];
			}

			if (message != null) {
				ma.showErrorMessage(message);
			}
		}
	},
	
	_showErrorMessage : function(error) {
			//
			// Callback handler for file delete operations
			// 
			// displays error message if delete failes
			//
		this.messageArea.showErrorMessage(this._errorUtil.getErrorMessageFromRestCallback(this.pluginResources,error));	
	},
	_getTableStoreTargetUri: function(targetUri) {
		if (targetUri.indexOf("?") != -1) {
			return targetUri.split("?")[0];
		}
		return targetUri;
	},
	
	_getTableQueryObject: function(targetUri) {
		if (targetUri.indexOf("?") != -1) {
			return dojo.queryToObject(targetUri.split("?")[1]);
		}
		return dojo.queryToObject(targetUri);
	}
	
	
	
});