/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.widgets.ChartWidget");

dojo.require("dojox.charting.Chart2D");
dojo.require("dojox.charting.themes.CubanShirts");
dojo.require("dojox.layout.GridContainerLite");
dojo.require("dojox.charting.widget.Legend");
dojo.require("dojox.charting.Theme");
dojo.require("dijit.layout.BorderContainer");
dojo.require("dijit.layout.ContentPane");
dojo.require("com.ibm.security.modeling.layouts.TemplatedPanelWithLayout");
dojo.require("dijit.form.ComboBox");
dojo.require("dijit.form.CheckBox");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.api.UserRestManager");
dojo.require("com.ibm.security.modeling.tables.DistributionAnalysisGrid");
dojo.require("com.ibm.security.modeling.tables.InOutAnalysisGrid");
dojo.require("com.ibm.security.modeling.util.NlsUtilities");
dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");
dojo.require("com.ibm.security.modeling.analysis.base._AnalysisTaskPanel");
dojo.require("com.ibm.security.modeling.analysis.AnalysisTaskLaunchingHelper");
dojo.require("com.ibm.security.modeling.dijit.layout.StackContainer");
dojo.require("dijit.Toolbar");
dojo.require("dijit.ToolbarSeparator");
dojo.require("com.ibm.security.modeling.tables.ToolTipHelper");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");


dojo.requireLocalization("com.ibm.security.modeling.widgets", "ChartWidgetText");

dojo.declare("com.ibm.security.modeling.widgets.ChartWidget", 
	[ com.ibm.security.modeling.layouts.TemplatedPanelWithLayout, com.ibm.security.modeling.analysis.base._AnalysisTaskPanel],{
		
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.widgets.ChartWidget"),			
	widgetsInTemplate : true,
	templatePath : dojo.moduleUrl("com.ibm.security.modeling.widgets", "templates/ChartWidget.html"),
	titleText : "",
	defaultTitleText : "",
	titleTemplate : "",
	_showMousePointer : false,

	constructor:function() {
		this._trace.entering(this.id, "constructor");
		
		this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.widgets", "ChartWidgetText");
		this.includeInheritanceLabel = this.getInheritanceLabel();			
		this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();

		this._trace.exiting(this.id, "constructor");
	},

	postCreate: function() {
		this._trace.entering(this.id, "postCreate");
		
		this.inherited(arguments);
		this._overrideWidgetLtrFunction(); // This is temp method call, we dont need it in the main console project.
		this.loadAttributes();
		this._subscribeWidget();
		this.subscribe(this.id+"_RESET",dojo.hitch(this,function(){
				if(dojo.isIE){
					this.createCharts(this.attributeDistribution);
				}
		}));
		this._hookUpButtons();
		this._launcher= new com.ibm.security.modeling.analysis.AnalysisTaskLaunchingHelper({messageArea:this.messageArea});
		
		this._trace.exiting(this.id, "postCreate");		
	}, 
	
	startup: function() {
		
		this.inherited(arguments);
		
		var includeInheritanceWidget = dijit.byId(this.id+"_includeInheritanceId");
		includeInheritanceWidget.set("tabIndex", "0");
		this.connect(includeInheritanceWidget, "_onFocus", function() {
			includeInheritanceWidget.set("tabIndex", "0");
		});
		this.connect(includeInheritanceWidget, "_onBlur", function(){
			includeInheritanceWidget.set("tabIndex", "0");
		});
	},
	
	_overrideWidgetLtrFunction: function() {
		// summary:
		//		Setups the direction for the widget.
		if(!NlsUtilities.isLtr()){
			dojo.addClass(this.domNode,"dijitRtl");
		}
	},
	

	_hookUpButtons:function() {
		// summary:
		//		Connects buttons with events.
		this._trace.entering(this.id, "_hookUpButtons");
		
		var includeInheritanceWidget = dijit.byId(this.id+"_includeInheritanceId");
		
		includeInheritanceWidget.onChange = dojo.hitch(this, function(newValue) {
			if (this._ignoreIncludeInheritanceChange) { 
				this._ignoreIncludeInheritanceChange = false;
			} else {
				this._onIncludeInheritanceChange(newValue);
				includeInheritanceWidget.focus();
			}
		});
		
		
		this._trace.exiting(this.id, "_hookUpButtons");		
	},
	
	_subscribeWidget : function(){
		if(this.subscribeTrigger){
			this.subscribe(
				this.subscribeTrigger, 
				function(modelId,roleItem){
					this.modelId = modelId;
					this.roleItem = roleItem;
					this.refresh();
				}
			);
		}
	},

	setContextData: function(/*Object*/ contextData) {
		// Summary:
		//		This method is used to set new data in the task panel.  
		//		For example if the panel is current displayed 
		//		and a new role notebook is opened.  
		//		The new role uri would be passed in.
		//
		//		This panel should update / reload as necessary.
		// contextData: Object
		//		Context data needed to display this panel
		this.inherited(arguments);
		if (contextData) {
			this.modelId = contextData.modelId;
		}
	},
	
	getInheritanceLabel: function() {
		// Summary:
		//		Gets the label to be shown against the inheritance check-box of each task. 
		//
		// returns: string
		//		inheritanceLabel
		return this.localResources.INCLUDE_INHERITANCE;
	},
	
	_loadRoleItem: function(contextData) {
		if (!contextData || !contextData.roleUri) {
			this.roleItem = null;
		} else {
			var roleUri = contextData.roleUri;
			var roleManager = new com.ibm.security.modeling.api.RoleRestManager({
				resourceUri: roleUri,
				syncMode: true,
				_onError : dojo.hitch(this, this._onError),
				msgArea : this.messageArea
			});
			this.roleItem = roleManager.loadResource();
		}
	},
	
	loadAttributes : function(){
		dojo.require("com.ibm.security.modeling.tables.JsonRestStore");

		var attributeRestStore =  new com.ibm.security.modeling.tables.JsonRestStore({
			target: this.attributeUri,
			syncMode: true,
			_onError : dojo.hitch(this, this._onError),
			msgArea : this.messageArea
		});
		var attributeCombo = dijit.byId(this.id+"_attributesId");
		this.connect(
				attributeCombo, 
				"onChange", 
				function(attributeName) {
					console.log("Attribute selected : "+attributeName);
					this.attributeItem = attributeCombo.item; 
					this.refresh();
				}
		); 

		attributeCombo.attr("store", attributeRestStore);
		attributeCombo.attr("labelAttr","name");
		attributeRestStore.fetch(
				{
					query:{
						name:"*"
					}, 
					onComplete:function(items){
						if(items[0] && items[0].name){
							attributeCombo.setValue(items[0].name);	
							attributeCombo.item = items[0];
						}else{
							attributeCombo.setValue("");	
							attributeCombo.item = "";
						}
					},
					onError: dojo.hitch(this, this._onError) 
				}
		);
	},
	_onError: function (error, request) {
		this._trace.entering(this.id, "_onError");
		this._trace.object(arguments, "_onError", "Error function args");
		this._errorUtil.handleRESTError(error, this.messageArea);
		this._trace.exiting(this.id, "_onError");
	},
	getRoleRestManager : function(){
		var resourceUri= "/isrm/rest/models/"+this.modelId+"/analysis/roles/";
		if(!this.roleRestManager 
				|| (this.roleRestManager && this.roleRestManager.resourceUri != resourceUri)){
			
			dojo.require("com.ibm.security.modeling.api.RoleRestManager");
			
			this.roleRestManager = new com.ibm.security.modeling.api.RoleRestManager({
				resourceUri: resourceUri,
				syncMode: true,
				_onError : dojo.hitch(this, this._onError),
				msgArea : this.messageArea
			});
		}
		return this.roleRestManager;
	},
	_makeDrillDownData: function(attribValue){
		var drillDownData={};
		if(this.roleItem && this.modelId && this.attributeItem && attribValue){
			drillDownData.roleUri=this.roleItem.uri;
			drillDownData.modelId=this.modelId;
			drillDownData.attributeName=this.attributeItem.name;
			drillDownData.attributeValue=attribValue;
			var includeInheritance = dijit.byId(this.id+"_includeInheritanceId");
			drillDownData.includeInheritance=includeInheritance.get("checked");
		}
		// this.contextData=drillDownData;
		return drillDownData;
	},
	refresh: function() {
		try {
			this._updateUIStatePerSupplementData();
			
			this.onLoadingStart();
			this._loadRoleItem(this.getContextData());
			if(this.roleItem){
				if (this.attributeItem) {
					var _roleRestManager = this.getRoleRestManager();
					var includeInheritance = dijit.byId(this.id+"_includeInheritanceId");
					
					this.attributeDistribution = _roleRestManager.loadResource(
						this.roleItem.id+this.distributionPath+"/attributeOverview?attribute="+this.attributeItem.id+"&includeInheritance="+includeInheritance.get("checked")
					);
				
					this.createCharts(this.attributeDistribution);
					this.setupDistributionAnalysisTable(this.attributeDistribution);
					this.setupInOutAnalysisTable(this.attributeDistribution);
				} else {
					this.reset();
				}
				this.titleTemplate = this.titleText;
				this.showChartPane();
			}else{
				this.showNoRoleSelectedMessagePane();
				this.titleTemplate = this.defaultTitleText;
			}
		} finally {
			this._updateTitle();
			this.onLoadingComplete();
		}
	},
	
	_updateTitle: function() {
		
		if (this.roleItem) {
			var roleName = this.roleItem.name;
			roleName = com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(roleName);
			this.titleTemplate = dojo.string.substitute(this.titleTemplate, {
				"roleName": roleName? roleName:""
			});
		}
		
		this.title = this.titleTemplate;
	},
	launchAnalysisPane:function(factoryName, drillDownData){
		if(factoryName){
			this._launcher.launchAnalysisPane(factoryName, drillDownData);
		}
	},
	setupDistributionAnalysisTable: function(attributeDistribution, isDrillDownCapable) {
		// summary:
		//		Sets up grid view of distribution analysis data.
		// description:
		//		If the grid has not yet been instantiated, then it gets created.  If it is already there,
		//		then the datastore is switched out with a new one based on the given attribute distribution
		//		object.
		// attributeDistribution: Object
		//		Object in json format representing the attribute analysis data from the server.
		// distributionTableView: com.ibm.security.modeling.tables.DistributionAnalysisGrid
		//		A reference to the grid object on the chart widget.
		this._trace.entering("isDrillDownCapable: " + isDrillDownCapable, "setupDistributionAnalysisTable");
		if (!this.distributionTableView) {
			this._trace.finest("creating new table", "setupDistributionAnalysisTable");
			this.distributionTableView = new com.ibm.security.modeling.tables.DistributionAnalysisGrid({
				id: this.id + "_distributionTableId",
				attributeDistribution: attributeDistribution,
				chartText: this.localResources,
				sortInfo:-2,//shouuld be default decending on percentage column
				isDrillDownCapable: isDrillDownCapable
			},	this.id + "_distributionTableDivId");

			var table = {id:this.distributionTableView.id,grid:this.distributionTableView,toolTipRequired:[1],_isSelectColumnIndex:function(){return true}};
			this.tooltipHandler = new com.ibm.security.modeling.tables.ToolTipHelper({table:table});
			
			this.distributionTableView.startup();
		} else {
			this._trace.finest("table exists, setting store", "setupDistributionAnalysisTable");
			this.distributionTableView.setStore(this.distributionTableView.createStore(attributeDistribution));
		}
		this._trace.exiting("", "setupDistributionAnalysisTable");
	},
	setupInOutAnalysisTable: function(attributeDistribution, isDrillDownCapable) {
		// summary:
		//		Sets up grid view of in/out analysis data.
		// description:
		//		If the grid has not yet been instantiated, then it gets created.  If it is already there,
		//		then the datastore is switched out with a new one based on the given attribute distribution
		//		object.
		// attributeDistribution: Object
		//		Object in json format representing the attribute analysis data from the server.
		// distributionTableView: com.ibm.security.modeling.tables.InOutAnalysisGrid
		//		A reference to the grid object on the chart widget.
		this._trace.entering("isDrillDownCapable: " + isDrillDownCapable, "setupInOutAnalysisTable");
		if (!this.inOutTableView) {
			this.inOutTableView = new com.ibm.security.modeling.tables.InOutAnalysisGrid({
				id: this.id + "_inOutTableId",
				attributeDistribution: attributeDistribution,
				sortInfo:-2,//shouuld be default decending on in column
				chartText: this.localResources,
				isDrillDownCapable: isDrillDownCapable
			}, this.id + "_inOutTableDivId");
			var table = {id:this.inOutTableView.id,grid:this.inOutTableView,toolTipRequired:[1],_isSelectColumnIndex:function(){return true}};
			this.tooltipHandler = new com.ibm.security.modeling.tables.ToolTipHelper({table:table});
		
			this.inOutTableView.startup();
		} else {
			this.inOutTableView.setStore(this.inOutTableView.createStore(attributeDistribution));
		}
		this._trace.exiting("", "setupInOutAnalysisTable");
	},
	createCharts: function(attributeDistribution) {
		this.attachChartsToDOM();
		
		if( !this.inAndOutChart ){
			dojo.require("com.ibm.security.modeling.widgets.InAndOutChart");		
			this.inAndOutChart = new com.ibm.security.modeling.widgets.InAndOutChart({
					id:this.id+"_stackedChartId",
					localResources:this.localResources,
					_showMousePointer:this._showMousePointer,
					_parent:this
					},dojo.byId(this.id+"_stackedChartDivId")
				);
		}
		this.inAndOutChart.populate(attributeDistribution.attributeValues);
		
		if( !this.distributionChart ){
			dojo.require("com.ibm.security.modeling.widgets.DistributionChart");		
			this.distributionChart = new com.ibm.security.modeling.widgets.DistributionChart({
					id:this.id+"_pieChartId",
					localResources:this.localResources,
					_showMousePointer:this._showMousePointer,
					_parent:this
					},dojo.byId(this.id+"_pieChartDivId")
				);
		}
		this.distributionChart.populate(attributeDistribution.attributeValues);
		
		if( !this.scoreChart ){
			dojo.require("com.ibm.security.modeling.widgets.ScoreChart");		
			this.scoreChart = new com.ibm.security.modeling.widgets.ScoreChart({
					id:this.id+"_scoreChartId",
					localResources:this.localResources
					},dojo.byId(this.id+"_scoreChartDivId")
				);
		}
		this.scoreChart.populate(attributeDistribution.score);
	},
	showChartPane:function(){
		
		// Summary:
		//		Shows analysis table pane
		this._trace.entering(this.id, "showChartPane");
		this.stackContainer.selectChild(this.chartPane);
		this._trace.exiting(this.id, "showChartPane");
	},
	showNoRoleSelectedMessagePane:function(){
		
		// Summary:
		//		Shows info message for role selection
		this._trace.entering(this.id, "showNoRoleSelectedMessagePane");
		this.stackContainer.selectChild(this.noRoleMessagePane);
		this._trace.exiting(this.id, "showNoRoleSelectedMessagePane");
	},
	detachChartsFromDOM: function() {
		var chartWidgetId = [this.id+"_scoreChartId", this.id+"_pieChartId", this.id+"_stackedChartId"];
		for (var i=0; i<chartWidgetId.length; i++ ) {
			var chartWidget = dijit.byId(chartWidgetId[i]);
			if (chartWidget) {
				var chartWidgetNode = new dojo.NodeList(chartWidget.domNode);
				chartWidgetNode.orphan();
			}
		}
	},
	
	attachChartsToDOM: function() {
		
		var attach = function(containerId, chartId) {
			if (!dojo.byId(chartId)) { // not in DOM.
				var chartWidget = dijit.byId(chartId); 
				if (chartWidget) {
					dojo.place(chartWidget.domNode, containerId);
				}
			}
		};
		
		attach(this.id+"_scoreChartContainer", this.id+"_scoreChartId");
		attach(this.id+"_pieChartContainer", this.id+"_pieChartId");
		attach(this.id+"_stackedChartContainer", this.id+"_stackedChartId");
	},
	
	reset: function() {
		// Overrides _AnalysisTaskPanel:: reset
		this.inherited(arguments);
		this.detachChartsFromDOM();
	},
	
	_onIncludeInheritanceChange: function(includeInheritance) {
		console.log("Include inheritance selected : "+includeInheritance);
		
		this._updateIncludeInheritanceInSupplementData(includeInheritance);
		
		this.refresh();
	},
	
	_updateUIStatePerSupplementData: function() {
		var supplementData = this.getSupplementData();
		
		var includeInheritance = false;
		if (supplementData && ("includeInheritance" in supplementData)) {
			includeInheritance = supplementData.includeInheritance;
		} else {
			// No inheritance information in supplement data.
			// In this case, if widget exists then keep it in its state.
			var includeInheritanceWidget = dijit.byId(this.id+"_includeInheritanceId");
			if (includeInheritanceWidget) {
				this._updateIncludeInheritanceInSupplementData(includeInheritanceWidget.get("checked"));
				return;
			}
		}
		
		this._setIncludeInheritance(includeInheritance);
	},
	
	_setIncludeInheritance: function(/*boolean*/ includeInheritance) {
		var includeInheritanceWidget = dijit.byId(this.id+"_includeInheritanceId");
		if (includeInheritanceWidget && includeInheritanceWidget.get("checked") != includeInheritance) {
			this._ignoreIncludeInheritanceChange = true;
			includeInheritanceWidget.attr("checked", includeInheritance);
		} 
	},
	
	_updateIncludeInheritanceInSupplementData: function(/*boolean*/ includeInheritance) {
		var supplementData = this.getSupplementData();
		if (!supplementData) {
			this.setSupplementData({});
			supplementData = this.getSupplementData();
		}
		supplementData["includeInheritance"] = includeInheritance;
		this.onSupplementDataChange();
	}
});