/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

� Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.widgets.DeleteProjectDialog");
dojo.require("com.ibm.security.modeling.widgets.ConfirmMessageDialog");


dojo.declare("com.ibm.security.modeling.widgets.DeleteProjectDialog", com.ibm.security.modeling.widgets.ConfirmMessageDialog, {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.widgets.DeleteProjectDialog"),
	_projectsCount:0,
	
	postCreate : function() {
	  if (!(_projectsCount > 10 )){
			this._createProjectNamesList();			
		}
	   var submitButton = dijit.byId(this.id+"SubmitButton");	   
	   this.inherited(arguments);
	},
		
	_createProjectNamesList:function(){
		// summary:
		//		creates list of project names to display
		// tags:
		//			private
		this._trace.entering(this.id, "_createProjectNamesList");
		var dialogContent = dijit.byId(this.id+"_content");
		_projectsCount = this.projectNamesToDisplay.length;
		//dojo.query("#deleteProjul").forEach(dojo.destroy);	
		//Sort the project Names	
		this.projectNamesToDisplay.sort(function(a,b){
			a = a.toLowerCase(); b = b.toLowerCase();
			if (a>b) return 1;
			if (a <b) return -1;
			return 0; 			
		});
		
		
		var ul = dojo.create("ul", {id:"deleteProjul"}, dialogContent.domNode, "first");
		var projName = null;
		for(i=0;i < _projectsCount; i++){
				projName = com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLandNormalize(this.projectNamesToDisplay[i]);
				dojo.create("li", {
					innerHTML: projName,
					id:"deleteProjulli"+i	
				}, ul);
		}		
		var ul = dojo.create("div", {id:"deleteProjIntroText",innerHTML:this.localResources.DELETE_PROJECTS_TEXT,style:{marginTop:"10px"}}, dialogContent.domNode, "first");		
		
		this._trace.exiting(this.id, "_createProjectNamesList");

	},
	
	postMixInProperties:function() {
		// summary:
		//		Executes setup logic after constructor args and inherited vars are "mixed" into the widget.
		// description:
		//		In this method we define the contents of the dialog, which is the TMS message, followed by the list of 
		//		radio button options.  We provide the markup with an html template file to avoid defining layout in
		//		javascript.		
		this._trace.entering("", "postMixInProperties");
		try {
			this.title = this.localResources.DELETE_PROJECTS;
			this.submitButtonText = this.globalResources.DELETE;
			this.cancelButtonText = this.globalResources.CANCEL;
			this.onSubmit = dojo.hitch(this,this.deleteCallback);
			_projectsCount = this.projectNamesToDisplay.length;
			
			if(_projectsCount > 10 ){
				this.formTemplate= dojo.cache("com.ibm.security.modeling.widgets","templates/DeleteProjectDialogCount.html", {sanitize:true});
				this._createDialogContentFromTemplate();
				this.message = dojo.string.substitute(this.message,{"project_count":_projectsCount});
				
			} else {
				this.formTemplate= dojo.cache("com.ibm.security.modeling.widgets","templates/DeleteProjectDialog.html", {sanitize:true});
				this._createDialogContentFromTemplate();				
			}			
					
			//call inherited after our additional mixins so they are included in the creation of the dialog markup
			this.inherited(arguments);			
		} catch (e) {
			this._trace.exception(e, "postMixInProperties");
			throw e;
		} finally {
			this._trace.exiting("", "postMixInProperties");			
		}
	},
	
	
	
	_createDialogContentFromTemplate:function() {
		// summary:
		//		Gets the template of the delete dialog box containing radio button options
		// tags:
		//			private
		this._trace.entering(this.id, "_createDialogContentFromTemplate");
		this.message = dojo.string.substitute(this.formTemplate, this, function(value, key){
			if(typeof value == "undefined"){ throw new Error("com.ibm.security.modeling.widgets.DeleteProjectDialog template:"+key); } // a debugging aide
			if(value == null){ return ""; }
			return value.toString().replace(/"/g,"&quot;"); 
		}, this);
		this._trace.exiting(this.id, "_createDialogContentFromTemplate");
	},
	
	_showMessage: function(/*String*/type, /*String*/message) {
		// summary:
		//			shows the given message and type in the local message area
		// tags:
		//			private
		this.messageArea.setType(type);
		this.messageArea.setMessage(message);
		this.messageArea.show();		
	},
	submitButtonPressed:function(){
		this.inherited(arguments);
		setTimeout(dojo.hitch(this, this.kill), (this.duration+100));
	},
	cancelButtonPressed:function(){
		this.inherited(arguments);
		setTimeout(dojo.hitch(this, this.kill), (this.duration+100));
	}	,
	kill:function(){
		this.destroyRecursive();
	}

});