/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

 Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.widgets.DeleteRoleDialog");

dojo.require("com.ibm.security.modeling.widgets.ConfirmMessageDialog");
dojo.require("com.ibm.security.modeling.util.UrlUtilities");
dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");
dojo.require("dijit.form.RadioButton");

dojo.declare("com.ibm.security.modeling.widgets.DeleteRoleDialog", com.ibm.security.modeling.widgets.ConfirmMessageDialog, {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.widgets.DeleteRoleDialog"),
	_urlUtil: new com.ibm.security.modeling.util.UrlUtilities(),
	constructor:function() {
		this.inherited(arguments);
	},
	postMixInProperties:function() {
		// summary:
		//		Executes setup logic after constructor args and inherited vars are "mixed" into the widget.
		// description:
		//		In this method we define the contents of the dialog, which is the TMS message, followed by the list of 
		//		radio button options.  We provide the markup with an html template file to avoid defining layout in
		//		javascript.		
		this._trace.entering("", "postMixInProperties");
		try {
			this.deleteConfirmMessage = dojo.string.substitute(this.logMessages.CONSOLE_ROLE_DELETE_CONFIRMATION, {"roleName":com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(this.roleName)});
			this._getMessageIdFromMessage();
			this.title = this.localResources.DELETE_ROLE;
			this.submitButtonText = this.globalResources.OK;
			this.cancelButtonText = this.globalResources.CANCEL;
			this.onSubmit = dojo.hitch(this,this._getDeleteConfirmation);
			this.formTemplate = dojo.cache("com.ibm.security.modeling.widgets","templates/DeleteRoleDialog.html", {sanitize:true});
			this._createDialogContentFromTemplate();	
			//call inherited after our additional mixins so they are included in the creation of the dialog markup
			this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
			this.inherited(arguments);			
		} catch (e) {
			this._trace.exception(e, "postMixInProperties");
			throw e;
		} finally {
			this._trace.exiting("", "postMixInProperties");			
		}
	},
	_createDialogContentFromTemplate:function() {
		// summary:
		//		Gets the template of the delete dialog box containing radio button options
		// tags:
		//			private
		this._trace.entering(this.id, "_createDialogContentFromTemplate");
		this.message = dojo.string.substitute(this.formTemplate, this, function(value, key){
			if(typeof value == "undefined"){ throw new Error("com.ibm.security.modeling.widgets.DeleteRoleDialog template:"+key); } // a debugging aide
			if(value === null){ return ""; }
			return value.toString().replace(/"/g,"&quot;"); 
		}, this);
		this._trace.exiting(this.id, "_createDialogContentFromTemplate");
	},
	_getMessageIdFromMessage: function() {
		this._trace.entering(this.id, "_getMessageIdFromMessage");
		if (this.deleteConfirmMessage.indexOf("CTJ") === 0) {//startsWith
			this.messageId = this.deleteConfirmMessage.substring(0,10);		
			this.deleteConfirmMessage = this.deleteConfirmMessage.substring(10);
		}
		this._trace.exiting(this.id, "_getMessageIdFromMessage");
	},
	replaceRole:function(/*String*/roleName, /*String*/roleUri) {
		this.callbackParms = {"roleUri":roleUri,"roleName":roleName};
		this.roleName = com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(roleName);
		this.deleteConfirmMessage = dojo.string.substitute(this.logMessages.CONSOLE_ROLE_DELETE_CONFIRMATION, {"roleName":this.roleName});
		this.deleteConfirmMessage = this.deleteConfirmMessage.substring(10);
		this._trace.finest("Replacing message with: " + this.deleteConfirmMessage, "replaceRole");
		dojo.place("<p class='TUIG_description' id='deleteConfirmMessage'>" + this.deleteConfirmMessage + "</p>", "deleteConfirmMessage", "replace");
		this._resetRadioButtonSelection();
	},
	_resetRadioButtonSelection: function() {
		var option1 = dijit.byId("option1");
		option1.setChecked(true);
	},
	_getDeleteConfirmation:function(callBackRoleDetailParams){
		// summary:
		// 		Call back method when the delete dialog is completed with response
		// tags:
		//			private
		// btnNumClicked: number
		//		Gives the number to find which button was clicked
		// msgId: String
		//		Dialog box TMS message id.If any
		// checkBoxSelection: boolean
		//		Attribute of TIPMessageDialog. Specifies the dialog single/multiple times
		// callBackRoleDetailParams: Object
		//		Represent Object which was set while creation of the dialog box
		this._trace.finest("Delete role confirmed", "_getDeleteConfirmation");
		var radioBtnOptions=document.getElementsByName("roleDeleteOption");
		var selectedDeleteBtnOption=null;
		var i;
		var len;
		for(i=0,len=radioBtnOptions.length;i<len;i++){
			if(radioBtnOptions[i].checked=="true" || radioBtnOptions[i].checked==true){
				selectedDeleteBtnOption=radioBtnOptions[i].value;
				break;
			}
		}
		this._doDeleteRoleRestCall(callBackRoleDetailParams,selectedDeleteBtnOption,true);

	},

	launchUndoDeleteDialog:function(e,ioargs){
		this.hideWorkingDialog();
		if(this._isDeleteTooBigForUndo(e)){
			this._trace.finest("Got a 412 back from the server - Delete too big for UNDO", "launchUndoDeleteDialog");
			var params=dojo.fromJson(e.responseText);
			msg=this.logMessages.CONSOLE_ROLE_DELETE_UNDO_CONFIRMATION;
			var msgId=msg.substring(0,10);//assign message id to var
			msg = msg.substring(10);//peel off message id
			msg = "<p class='TUIG_description'>" + msg + "</p>";
			var roleUndoDeleteDialog = dijit.byId(this.id+"undoDialog");
			if (roleUndoDeleteDialog) {
				roleUndoDeleteDialog.callbackParms = {"roleUri":params.inserts[1]};
			} else {
				roleUndoDeleteDialog = new com.ibm.security.modeling.widgets.ConfirmMessageDialog({
					title:this.localResources.CANNOT_UNDO,
					id:this.id+"undoDialog",
					submitButtonText: this.globalResources.YES,
					cancelButtonText: this.globalResources.NO,
					onSubmit:dojo.hitch(this,this._getDeleteUndoConfirmation),
					message:msg,
					messageId:msgId,
					callbackParms:{"roleUri":params.inserts[1]}
				});
			}			 
			roleUndoDeleteDialog.show();
		}else{
			this.hide();
			this._errorHandler(e,ioargs);
		}
	},
	_isDeleteTooBigForUndo:function(event) {
		// summary:
		//		Executes setup logic after constructor args and inherited vars are "mixed" into the widget.
		// tags:
		//			private
		// description:
		//		In this method we define the contents of the dialog, which is the TMS message, followed by the list of 
		//		radio button options.  We provide the markup with an html template file to avoid defining layout in
		//		javascript.		
		return event.status && event.status===412;
	},
	_getDeleteUndoConfirmation:function(btnroleDetails){
		// summary:
		// 		Call back method when the undo dialog is completed with response
		// tags:
		//			private
		// btnNumClicked: number
		//		Gives the number to find which button was clicked
		// msgId: String
		//		Dialog box TMS message id.If any
		// checkBoxSelection: boolean
		//		Attribute of TIPMessageDialog. Specifies the dialog single/multiple times
		// btnroleDetails: Object
		//		Represent Object which was set while creation of the dialog box
		this._trace.finest("Delete role confirmed", "_getDeleteUndoConfirmation");
		btnroleDetails.roleUri=this._urlUtil.removeParamFromURL(btnroleDetails.roleUri,"checkUndoFlag");
		var deletionMode=this._urlUtil.getParamFromURL(btnroleDetails.roleUri,"modeOfDeletion");
		btnroleDetails.roleUri=this._urlUtil.removeParamFromURL(btnroleDetails.roleUri,"modeOfDeletion");
		this._doDeleteRoleRestCall(btnroleDetails,deletionMode,false);
	},
	_errorHandler:function(e,ioargs){
		this._errorUtil.handleRESTError(e, this.messageArea);
		
	},
	_doDeleteRoleRestCall:function(roleDetails,selectedDeleteBtnOption,undoFlag){//TODO:Use table deleteItem function
		// summary:
		// 		Rest call to delete a role
		// tags:
		//			private
		// selectedDeleteBtnOption: string
		//		Type of delete
		// undoFlag: boolean
		//		Specified whether threashold check needs to be done or not at server side.
		
		this.showWorkingDialog(this.localResources.DELETING_ROLE);
		var uri=roleDetails.roleUri;
		
		if(roleDetails.roleUri.indexOf("http")=== -1){
			uri="https://"+this.hostname+":"+this.port+roleDetails.roleUri;
		}
		var xhrArgs = {
            url:uri+"?modeOfDeletion="+selectedDeleteBtnOption+"&checkUndoFlag="+undoFlag,
            load: this.deleteCallback,
            handleAs:"json",
            error: dojo.hitch(this,this.launchUndoDeleteDialog)
        };
        dojo.xhrDelete(xhrArgs);
	}

});