/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

� Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.widgets.DistributionChart");

dojo.require("dojox.charting.Chart2D");
dojo.require("dojox.charting.action2d.Tooltip");
dojo.require("dojox.charting.action2d.Base");
dojo.require("dojox.charting.widget.Legend");
dojo.require("dojox.charting.Theme");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.util.ColorUtilities");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.security.modeling.util.CommonUtilities");

dojo.declare("com.ibm.security.modeling.widgets.DistributionChart", 
	[ dijit._Widget, dijit._Templated  ],{
		
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.widgets.DistributionChart"),			
	widgetsInTemplate : true,
	templatePath : dojo.moduleUrl("com.ibm.security.modeling.widgets", "templates/DistributionChart.html"),
	

	populate: function(attributeValues){
		this._trace.entering(this.id, "populate");
		
		if (this._hasValidData(attributeValues)) {
			
			if(this.chart == null){
				this.chart = this.getChart();
				this.clickAction=new dojox.charting.action2d.Base(this.chart, "default");
				this.clickAction.process=dojo.hitch(this,this._actionHandler);
				this.clickAction.connect();
			}else{
				this.chartTooltip.destroy();
			}

			this.chart.addSeries("Series A", this._getSeries(attributeValues));
			this.chartTooltip = new dojox.charting.action2d.Tooltip(this.chart, "default");
			this.chartTooltip.oldProcess=this.chartTooltip.process;
			this.chartTooltip.process=dojo.hitch(this.chartTooltip,function(e){
				if(e.type === "onclick"){
					e.type="onmouseout";
				}
				this.oldProcess(e);
			});
			this.chart.render();
			
			if(this.legend == null){
				this.legend = this.getLegend();
			}else{
				this.legend.refresh();
			}
			
			if(this._showMousePointer){
				dojo.addClass(dojo.byId(this.id+"_distributionChart"),"chartPointer");
			}
		}
		
		this._trace.exiting(this.id, "populate");			
	},
	getLegend : function(){
		this._trace.entering(this.id, "getLegend");
		
		var _legend = new dojox.charting.widget.Legend({
														chart: this.chart,
														horizontal:false
													}, this.id+"_distributionChartLegend");
		
		this._trace.exiting(this.id, "getLegend");	
		
		return _legend;
	},
	getChart : function(){
		this._trace.entering(this.id, "getChart");
		
		var _chart = (new dojox.charting.Chart2D(dojo.byId(this.id+"_distributionChart")))
						.addPlot("default", {
							type: "Pie", 
							radius: 90,
							labels:false
						})
						.setTheme(new dojox.charting.Theme({
							plotarea:{
								stroke: null,
								fill:   null
							},
							chart:{
								stroke: null,
								fill:   null
							}
						})
					);
		this._trace.exiting(this.id, "getChart");	
		
		return _chart;
	},
	_actionHandler:function(event){
		if(event.type === "onclick"){
			if(this._parent.distributionChartDrillFactoryName){
				var drillDownData=null;
				if(event.run.data){
					var dataObject=event.run.data[event.index];
					if(dataObject.text == this.localResources.NO_VALUE || dataObject.other){
						this._trace.info(this.id, "_actionHandler:Nothing to launch. Pie chart is empty");
						return;
					}
					drillDownData = this._parent._makeDrillDownData(dataObject.text);
				}
				this._parent.launchAnalysisPane(this._parent.distributionChartDrillFactoryName, drillDownData);
				
			}
		}
	},
	_hasValidData : function (attributeValues){
		this._trace.entering(this.id, "_hasValidData");
		
		if(attributeValues){
			if(attributeValues.length == 0){
				console.log("Distribution data is empty!");
				return true;
			}else{
				for(var i=0;i<attributeValues.length;i++){		
					if(!("attributeValue" in attributeValues[i])){
						console.log("Distribution data is invalid ! [Not Found : 'attributeValue']");
					}else if(!("count" in attributeValues[i])){
						console.log("Distribution data is invalid ! [Not Found : 'count']");
					}else if(!("countPercent" in attributeValues[i])){
						console.log("Distribution data is invalid ! [Not Found : 'countPercent']");
					}else{
						console.log("Distribution data is valid !");
						return true;
					}
				}
			}
		}else{
			console.log("Distribution data is null !");
		}
		
		this._trace.exiting(this.id, "_hasValidData");	
		
		return false;
	},
	
	_getSeries: function(attributeValues){
		this._trace.entering(this.id, "_getSeries");
		
		var series = new Array();
		var colorIndex = 0;
		
		if( ! this.primaryColors){
			var _primaryColors = new com.ibm.security.modeling.util.PrimaryColor();
			this.primaryColors = _primaryColors.getPrimaryColors(26);
		}
		
		for(var i=0 ; i<attributeValues.length;i++){
			var count = attributeValues[i].count;
			if(count>0){
				var attributeValue = attributeValues[i].attributeValue;
				series.push({ 
					y: count, 
					tooltip: dojo.string.substitute(this.localResources.DISTRIBUTION_COUNT_TOOLTIP, 
														{
															"attributeValue":com.ibm.security.modeling.util.HtmlEscapingUtils.breakString(attributeValue),
															"countPercent":attributeValues[i].countPercent
														}
													),
					text:attributeValue, 
					other:attributeValues[i].other,
					color:this.primaryColors[colorIndex++],
					stroke: {
						color: "white", 
						width: 1
					}
				});
			}
		}
		
		if(series.length == 0){
			series.push({ 
				y: 1, 
				tooltip: this.localResources.NO_VALUE, 
				text:this.localResources.NO_VALUE,
				color:this.primaryColors[colorIndex],
				stroke: {
					color: "white", 
					width: 1
				}
			});
		}
		
		this._trace.exiting(this.id, "_getSeries");	
		
		return series.sort(this._sortAttributeValues);
	},
	_sortAttributeValues : function (attribute1,attribute2){
		return attribute1.y - attribute2.y;
	}
});