/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.widgets.ErrorsWarningsDialog");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Dialog");
dojo.require("dijit.Toolbar");
dojo.require("dijit.form.Button");
dojo.require("com.ibm.security.modeling.dijit.layout.BorderContainer");
dojo.require("dijit._Container");
dojo.require("ibm.tivoli.tip.dijit.TIPButton");
dojo.require("com.ibm.security.modeling.tables.Table");
dojo.require("com.ibm.security.modeling.widgets.IsrmDialogBase");
dojo.require("com.ibm.security.modeling.util.MessageUtilities");
dojo.requireLocalization("com.ibm.security.modeling.importer", "ErrorText");
dojo.requireLocalization("com.ibm.security.modeling", "EnumLabels");
dojo.declare("com.ibm.security.modeling.widgets.ErrorsWarningsDialog", com.ibm.security.modeling.widgets.IsrmDialogBase, {
	
	templateString : dojo.cache("com.ibm.security.modeling.widgets", "templates/ErrorsWarningsDialog.html"),			
	widgetsInTemplate : true,
	storeUri:null,
	heightFactor:1.2,
	widthFactor:1.3,
	dialogInstruction:null,
	globalResources:null,
	localResources:null,
	entityTypeText:"",
	searchInstructionsText:"",
	tableArgs : null,
	constructor : function() {				
		this.dialogInstruction="";
		 this.entityTypeName="";
		 this.entityTypeText="";
		 this.entityValue= "";
		 this.entityTypeTextLabelMarginClass="defaultTextLabelMarginClass";
		 this.enumLabel=dojo.i18n.getLocalization("com.ibm.security.modeling", "EnumLabels");
	},
	
	attributeMap : {
		entityTypeValue : {node : "entityNameNode", type : "innerText"},
		entityValue : {node : "entityTypeNode", type : "innerText"}
	},
	
	/*
	 * The method enables to configure the table structure from outside.
	 * The table structure is used for displaying the data in the table on the
	 * ErrorsWarningsDialog
	 */
	setTableStructure:function(tableStructure){
		this.errorTableStructure = tableStructure;
	},
	
	postCreate : function() {
		
		dojo.addClass(this.domNode, "contentArea");
		this.inherited(arguments);

		this.set("entityTypeValue",this.entityTypeValue);
		this.set("entityValue",this.entityValue);
		this._setupStore();
		this._createErrorTable();
		this.closeButton = this.addCommonButton({
			onClick:dojo.hitch(this, this.onCancel),
			type:"CLOSE"
		});
		
	},
	
	_setupStore:function(){
	    this.store = new com.ibm.security.modeling.tables.JsonRestStore({
				target:this.storeUri,
				idAttribute: "id"
				});
	},
	_createErrorTable : function() {
        var args = {
        		id : this.id + "_CommonTable",
    			dataStore : this.store,
    			structure : this.errorTableStructure,
    			isClientSide : false,
    			isFilterRendered : true,
    			isRefreshRendered : false,
    			needSelection:false,
    			isSelectAllRendered:false,
    			isDynamic : true,
    			autoHeight:false,
    			entityTypeText:this.entityTypeTableText,                                        
    			searchInstructions: this.searchInstructionsText,
    			messageArea:this.messageArea
        };
        dojo.mixin(args,this.tableArgs);
		this._errorTable = new com.ibm.security.modeling.tables.Table( args, dojo.byId(this.id + "_ErrorTable"));
		dojo.addClass(this._errorTable.domNode,"contentArea");
	},
	//extra div is to align image to centre
	//imageHeight Class is so that image takes a 16px height which helps table to calculate row height
    formatSeverity:function(value){        
        var severityImage="";
        var iconClass="";
        if(value=="ERROR"){
        	iconClass="errorIcon";
        }else if(value=="INFO"){
			iconClass="infoIcon";
        }else{
			iconClass="warningIcon";
        }
        var severityAlt = this.getInternalizedLabels(value);
        severityImage= "<div title='"+severityAlt+"' class='"+iconClass+"'></div>";
        return severityImage;
       
    },
   
   //
   //The method formats the message which is to be displayed in the 
   //Description cell of the Error and Warnings dialog for the Session, files and Projects
   //
   formatMessage: function(item, rowIndex, cell){
   	dojo.require("com.ibm.security.modeling.util.MessageUtilities");
   	this._messageUtils = new com.ibm.security.modeling.util.MessageUtilities();
   	var result = null;
   	if(item){
   		var messageKey = item.messageKey;
   	   	var inserts = item.inserts;
   		if(this.pluginResources[messageKey] || this.logMessages[messageKey]){
	   		var message = this.pluginResources[messageKey] || this.logMessages[messageKey];
		   	//var formatTags = ["<b>","</b>"];
		   	var cssClass = "msgArgsClass";
		   	result = this._messageUtils.formatMessage(message, inserts,cssClass);
   		}else{
   	   		//log that the message id is not availablke in the pluginMessages resouce bundle.
   	   		//Display the error on the UI panel???
   	   		}
   	}
   	return result;
   },
   getInternalizedLabels:function(label){
	   if(label && label in this.enumLabel){
		   return this.enumLabel[label];
	   }
	   return label;
	     },
   
   setHelp: function(helpFileName){
   		this.helpFile=helpFileName;
   }
    

});
