/*Licensed Materials - Property of IBM

Role and Policy Modeler

Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.widgets.FilteringWidget");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Dialog");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.dijit.layout.StackContainer");
dojo.require("com.ibm.security.modeling.layouts.TemplatedPanelWithLayout");
dojo.require("dijit.form.FilteringSelect");
dojo.require("com.ibm.security.modeling.dijit.layout.BorderContainer");
dojo.require("com.ibm.security.modeling.dijit.layout.ContentPane");
dojo.declare("com.ibm.security.modeling.widgets.FilteringWidget", [ com.ibm.security.modeling.layouts.TemplatedPanelWithLayout  ], {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.widgets.FilteringWidget"),			
	
				
	widgetsInTemplate : true,
	templateString:	"<div>"+
			    "<div dojoType='com.ibm.security.modeling.dijit.layout.BorderContainer' dojoAttachPoint='container' gutters='false' style='overflow: auto;'>"+
					"<div dojoType='com.ibm.security.modeling.dijit.layout.ContentPane' region='top' >"+
						 "<div  id='${id}_FilteringSelect'> </div>"+				       
					"</div>"+
			 	    "<div dojoAttachPoint='TableContainer' dojoType='com.ibm.security.modeling.dijit.layout.ContentPane' region='center' style='margin-top:10px'  >"+
						"<div id='${id}_stackContainer' ></div>" +
					"</div>"+	
				"</div>"+
			"</div>",
	// clientSide: boolean
	//		true, If dropdown store is client side jsonreststore
	clientSide:true,
	// isStoreLocal: boolean
	//		true, If dropdown store is not a jsonreststore
	isStoreLocal:false,
	// defaultValue: string
	//		default value for the dropdown
	defaultValue:"",
	//
	defaultName:"",
	// hookForFilteringSelect: Domnode
	//		Attachs the dropdown to the panel
	hookForFilteringSelect:null,
	// hookForStackContainer: Domnode
	//		Attachs the stackcontainer to the panel
	hookForStackContainer:null,
	//hide FilteringSelect for only One element 
	hideFilteringSelectForOne:false,
	//hide FilteringSelect
	hideFilteringSelect:false,
	
	constructor : function() {
		// tags:
		//		protected
		this._trace.entering("", "constructor");
		
		this.stackList=[];
		this._trace.exiting("", "constructor");
		
		

	},
	postMixInProperties:function(){
		// summary:
		//			If the hooks are provided then it will create the widget at those place else it use the template string
		// tags:
		//			protected
	    if(this.hookForFilteringSelect && this.hookForStackContainer) {
		    
		}else{
		    if(this.hookForFilteringSelect) delete this.hookForFilteringSelect;
		    if(this.hookForStackContainer) delete this.hookForStackContainer;
		}
		//if every thing is hidden , this flag should automatically get on .
		hideFilteringSelectForOne = this.hideFilteringSelectForOne||this.hideFilteringSelect;
		
	},
	postCreate : function() {
		// tags:
		//		protected
        this._trace.entering("", "postCreate");
		this.inherited("postCreate", arguments);
		this.makeFilteringSelect();
		this.makeStachContainer();
		
	},
	_initialize:function(store){
		// summary:
		//			initializes the panel. Cleans up the stackcontainer
		// tags:
		//			private
		if(store){
			this.store=store;
			this.filteringSelect.store=store;
		}
		this.filteringSelect.reset();
		this.stackContainer.destroyDescendants();
		this.stackList=[];
		var sortAttributes = [{attribute:this.displayName||this.searchAttr}];
		this.store.fetch( {
		   
		queryOptions: {ignoreCase: this.ignoreCase||true,cache:true},
		sort:sortAttributes,
		query:this.buildQueryObject(this.searchAttr),
			onComplete : dojo.hitch(this,function(items, request) { 
			if(items.length>=0) result=items[0];
			if(result) this.filteringSelect.set('displayedValue', this.store.getValue(result,this.displayName||this.searchAttr));
		})
		});		
	},
	buildQueryObject:function(searchAttr,value){
		// summary:
		//			Builds query for dropdown widget for querying the REST call
		// searchAttr:string
		//			search attribute used for creating the dropdown
		// value:string
		//			value for dropdown
		var query ={};
   	    value=value||"*";
		query[searchAttr]=value;
        return query;
	},
	destroy:function(){
		// tags:
		//		protected
	    this.filteringSelect.destroy();
		this.stackContainer.destroyRecursive();
		this.inherited(arguments);
	},
	makeFilteringSelect:function(){
		// summary:
		//			setups the dropdown widget
	   
             //building query of form this.searchAttr=*
		if(this.clientSide)  {var query=this.buildQueryObject(this.searchAttr); }
		this.attachFilteringSelect(query);
		
		if(this.clientSide){
		    //if defaultValue is absent
		      if(!(this.defaultValue && this.defaultValue.trim()))this.setDefaultValue(this.store);
		      //firstly forced filtering select to do * query , so that it can 
		      //match any further query
		      //now delete query from fetchProperties so that it does not kill further client side query during mixin
		       delete this.filteringSelect.fetchProperties.query;
		}
	
       
      
	},
	attachFilteringSelect:function(query){
		// summary:
		//			creates the dropdown widget
	
		var hook = this.hookForFilteringSelect||dojo.byId(this.id +"_FilteringSelect");
		var sortAttributes = [{attribute:this.displayName||this.searchAttr}];
		
		var filteringSelectId = this.id+"_stateSelect";
		
		// Filtering Select widget takes care of any label which has its 'for' attribute set to filtering select id.
		
		this.filteringSelect = new dijit.form.FilteringSelect({
        
    		    id: filteringSelectId,
    		    name: this.displayName||this.searchAttr,
    		    value: this.defaultValue,
    		    ignoreCase:true,
    		    
    		    fetchProperties: {
    		        //client side search
    		    	    queryOptions: {ignoreCase: this.ignoreCase||true,cache:this.clientSide},
    		    	    //forcing to have query
    		    	    query:query,
						sort:sortAttributes
    		    	    
    		    },
    		    store: this.store,
    		    searchAttr: this.searchAttr
     	},hook); 
     	//construct and hide if hideFilteringSelect flag is on .
     	this.toggleVisibility(this.hideFilteringSelect)
     	this.connect(this.filteringSelect,"onChange",this.selectChild);
     	
	},
	
	_attachLabelElementForFilteringSelect: function() {
		// Summary:
		//		Sets the label element's 'for' attribute for filtering select.
		if (this.filteringSelect && this.dropDownLabelNode) {
			var filteringSelectId = this.filteringSelect.id;
			var labelFor = this.dropDownLabelNode.getAttribute("for");
			if (!labelFor) {  
				this.dropDownLabelNode.setAttribute("for", filteringSelectId);
			}
			var labelNodeId = (filteringSelectId+"_label");
			this.dropDownLabelNode.id = labelNodeId;
			dijit.setWaiState(this.filteringSelect.domNode, "labelledby", labelNodeId);
		}
	},
	
    toggleVisibility:function(hideFilteringSelect){
        
        if(hideFilteringSelect) {
            dojo.addClass( this.filteringSelect.domNode,"hiddenFilteringWidget");
            var node = dojo.byId(this.id+"_fitleringReadOnly");
            var labelNode;
            if (!node) {
            	var headerNode = dojo.place("<h1/>", this.filteringSelect.domNode, "after");
            	labelNode = dojo.create("label", {id:this.id+"_fitleringReadOnly"}, headerNode, "last");	 
            }	 
		    dojo.attr(labelNode, {
				innerHTML : this.getFilteringSelectDisplayedValue()
			});   
		    dojo.removeClass( labelNode,"hiddenFilteringWidget");
        }
     	else{
     	    var node = dojo.byId(this.id+"_fitleringReadOnly");
     	    if(node)dojo.addClass( node,"hiddenFilteringWidget");
     	    dojo.removeClass( this.filteringSelect.domNode,"hiddenFilteringWidget");
     	}
    },
   setUpdatedLabel: function(label){
    	
    	var labelId = dojo.byId(this.id+"_fitleringReadOnly");
    	if(labelId){
    		 //dojo.addClass(labelId,"changedLabel");
    		 dojo.attr(labelId, {
 				innerHTML : label
 			});
    	}
    },
	setDefaultValue:function(store){
		// summary:
		//			sets the default value for dropdown using rest call
		// store:dojo store
		//			store for dropdown
	var result;
	var sortAttributes = [{attribute:this.displayName||this.searchAttr}];
		if(!this.isStoreLocal){
		store.fetch( {
		   
		queryOptions: {ignoreCase: this.ignoreCase||true,cache:true},
		sort:sortAttributes,
		query:this.buildQueryObject(this.searchAttr),
		onComplete : dojo.hitch(this,function(items, request) { 
			if(items.length>=0) {
			    result=items[0];
			    if(this.defaultName && this.defaultName.trim()){
			        dojo.forEach(items,dojo.hitch(this,function(item){
			            var displayName=store.getValue(item,this.displayName||this.searchAttr);
			            if(displayName==this.defaultName){
			                result = item;
			            }
			        }));
			    }
			    
			}
			
			if(result) {
			    this.filteringSelect.set('displayedValue', store.getValue(result,this.displayName||this.searchAttr));
			   // this.toggleVisibility(this.hideFilteringSelect);
			}
			else {
			    if(this.noItems) this.noItems();
			}
			
			if(items.length==1){
			    //hide if only one item is available for filtering select
			    if(this.isOneItem) this.isOneItem();
			    this.toggleVisibility(this.hideFilteringSelectForOne);
			}else{
			    this.toggleVisibility(this.hideFilteringSelect);
			}
		}),
		onError:dojo.hitch(this,function(error){
		    
		    if(this.onError) this.onError();
		    else throw "Ajax Call failed";
		})
		
		});
			}

	},
	
	makeStachContainer:function(){
		// summary:
		//			setups the stackcontainer widget
	var hook = this.hookForStackContainer||this.id+"_stackContainer";
	    this.stackContainer= new com.ibm.security.modeling.dijit.layout.StackContainer({
	        id:this.id+"_stackContainer"
	    }, hook); 
	},
	startup:function(){
		// tags:
		//		protected
		this.inherited(arguments);
		this.stackContainer.startup();
		if(this.isStoreLocal){
			var val=this.filteringSelect.getValue();
			if(val){
				this.selectChild(val);
			}
		}
		this._attachLabelElementForFilteringSelect();
		
	},
	selectChild:function(value){
		// summary:
		//			selects a stack of the stack container depending on the value
		// value:string
		//			valid value of the dropdown
	    if(this.stackList && value in this.stackList){
	        this.stackContainer.selectChild(this.stackList[value]);
	        //called after a selection is done 
	        if(this.onSelection){
					this.onSelection(value,this.filteringSelect.item);
		}
	        
	    }else{	         
	   
	        var widget = this.selectedValueCallback(value,this.filteringSelect.item);
	        if(widget){
	        	if(! widget.domNode.style.height){
	        		   dojo.style(widget.domNode, "height", "100%");	
	        	}
	     	        var contentPane = new com.ibm.security.modeling.dijit.layout.ContentPane({parseOnLoad:false});
			        this.stackList[value]=contentPane;
			        contentPane.set("content", widget);
	                this.stackContainer.addChild(contentPane);
	                //Fix : few widget were not get resized for change , since addChild call this internally when no selection is there
	                //first child was comming right , but not from second on , so added this line
	                this.stackContainer.selectChild(this.stackList[value]);
			        
			        this.selectChild(value);	       
	        }
	    }
	},
	getSelectedChild:function(){
	//as we are wrapping all the widget is a contentPane ,hence unwrapping
	    var contentPane=this.stackContainer.selectedChildWidget;
	    
	    return contentPane.getChildren()[0];
	},
	getChild:function(value){
	//as we are wrapping all the widget is a contentPane ,hence unwrapping
	    var contentPane=this.stackList[value];
	    if(contentPane){
	    return contentPane.getChildren()[0];
	    }else{
	    	return null;
	    }
	},
	getFilteringSelectItem:function(){ 
	    return this.filteringSelect.item;
	},
	getFilteringSelectDisplayedValue:function(){ 
	    return this.filteringSelect.attr('displayedValue');
	}
			
});
