dojo.provide("com.ibm.security.modeling.widgets.HierarchicalAttributeView");

dojo.require("com.ibm.security.modeling.widgets.SwitchableTopologyView");
dojo.require("com.ibm.security.modeling.topology.TopologyGlobalFunctions");

dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling", "LogMessages");
dojo.requireLocalization("com.ibm.security.modeling", "PluginMessages");
dojo.requireLocalization("com.ibm.security.modeling.projects", "UsersResourcesSelectionPanelText");
/*
*  This widget will display the Hierarchical Attribute in Topology and Table views.
*/
dojo.declare("com.ibm.security.modeling.widgets.HierarchicalAttributeView", [ com.ibm.security.modeling.widgets.SwitchableTopologyView ],
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.widgets.HierarchicalAttributeView"),
	
	ENTITYTYPE_USER: "USER",
	ENTITYTYPE_PERMISSION: "PERMISSION",
	ENTITYTYPE_ROLE: "ROLE",
	entityType: "USER",  //Default to USER
	attributeId: null,
	localResources: null,
	includeSubtreeOption: null,
	includeSubtree: null,
	includeSubtreeText: null,
	pluginResources: null,
	
	projUtil: null,
	
	// Constants
	// Maximum number of items allowed to switch to TopologyView.
	ATTRIBUTE_TOPOLOGY_MAX_ITEMS : 1000,
	
	constructor: function(){
		this.inherited(arguments);
		this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.projects", "UsersResourcesSelectionPanelText");
		this.pluginResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "PluginMessages");
	},
	
	postCreate: function(){
		if(this.includeSubtreeOption != false){
			this.includeSubtreeOption = true;
		}
		if(this.includeSubtree != false){
			this.includeSubtree = true;
		}
		this.inherited(arguments);
	},
	getTableWidget: function(){
		this._trace.entering(this.id, "getTableWidget");
		
		dojo.require("com.ibm.security.modeling.api.AttributeScopeRestManager");
		var uri = null;
		if(this.entityType === "USER"){
			uri = "/isrm/rest/userscope/attributes/" + this.attributeId;
		}else if(this.entityType === "PERMISSION"){
			uri = "/isrm/rest/permissionscope/attributes/" + this.attributeId;
		}else if(this.entityType === "ROLE"){
			uri = "/isrm/rest/rolescope/attributes/" + this.attributeId;
		}
		var schemaRestMgr= new com.ibm.security.modeling.api.AttributeScopeRestManager({resourceUri:uri});
		var store = schemaRestMgr.getIsrmTableRestStore();

		dojo.require("com.ibm.security.modeling.projects.HierarchyTable");

		var fetch = {onError:dojo.hitch(this,this._errorHandler)};
		var lable1 = this.localResources.HIERARCHICALTABLE_HEADER_NAME;
		var lable2 = this.localResources.HIERARCHICALTABLE_HEADER_DESCRIPTION;
		var lable3 = this.localResources.HIERARCHICALTABLE_HEADER_PARENT;
		var lable4 = this.localResources.HIERARCHICALTABLE_HEADER_CHILD;

		attributeValueStructure = 
			[
				{field: 'value',name: lable1,width: '20%'},
				{field: 'description',name: lable2,width: '40%'},
				{field: 'parent',name: lable3,width: '20%', formatter: this._getParentName, isSearchable:false},
				{field: 'children',name: lable4,width: '20%', formatter: this._getChildNameCommaSepList, isSearchable:false}
		];

		var idWithoutSpaces = this.id.replace(/ /g,"_");

		this._navWidget = new com.ibm.security.modeling.projects.HierarchyTable(
					{
						id: idWithoutSpaces + "hierarchytable",
						dataStore: store, 
						structure: attributeValueStructure, 
						messageArea: this.messageArea,
						isClientSide: false,
						isFilterRendered: true,
						isRefreshRendered: true,
						isMultiSelect: true,
						autoHeight: false,
						isDynamic: true,
						isSearchMultiSelect: false,
						//entityTypeText: item.name,
						query: {"structure":"FLAT"},
						unSortableColumns: [5],
						defaultSortIndex:1,
						entityTypeText:this.entityTypeText,
						switch_view: dojo.hitch(this, this._switch)
			});

		this._trace.exiting(this.id, "getTableWidget");
		//return this._navWidget;
		return this._getWidgetInContainer(this._navWidget);
	},
	
	_getChildNameCommaSepList:function(list){
		var nameList= new Array();
		var nameListString="";
		if(list!=null && list.length && list.length>0){
			for(var i=0,len=list.length;i<len;i++){
				nameList.push(list[i].value);
			}
			nameList.sort();//adding sort due to problem in automation testing D11385
			for(var j=0,len1=nameList.length;j<len1;j++){
				nameListString=nameListString+nameList[j];
				if(j+1!=len1){
					nameListString=nameListString+", ";
				}
			}
		}
		return com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(nameListString);

	},
	_getParentName:function(item){
		if( (item.value == undefined) || (item.value == null)){
			return "";
		}else{
			return com.ibm.security.modeling.util.HtmlEscapingUtils.parseHTMLTag(item.value);
		}
	},
	
	getTopologyWidget: function(){
		this._trace.entering(this.id, "getTopologyWidget");
		
		dojo.require("ibm.tivoli.tip.navwidget.topology.dijit.TopologyView");
		dojo.require("com.ibm.security.modeling.topology.TopologyDataSets");

		var dataSetFactory = new com.ibm.security.modeling.topology.TopologyDataSets();

		var dataset = null;
		if(this.entityType === this.ENTITYTYPE_USER){
			dataset = dataSetFactory.createIdentityAttributeDataSet(this.attributeId, true);
		}else if(this.entityType === this.ENTITYTYPE_PERMISSION){
			dataset = dataSetFactory.createPermissionAttributeDataSet(this.attributeId, true);
		}else if(this.entityType === this.ENTITYTYPE_ROLE){
			dataset = dataSetFactory.createRoleAttributesDataSet(this.attributeId, true);
		}
		
		dojo.require("com.ibm.security.modeling.topology.HierarchicalAttributeTopologyFactory");
		var factory= new com.ibm.security.modeling.topology.HierarchicalAttributeTopologyFactory();
		var menuProvider=factory.createMenuProvider();
		dojo.require("ibm.tivoli.tip.navwidget.topology.dijit.TopologyView");

		this._navWidget = new ibm.tivoli.tip.navwidget.topology.dijit.TopologyView(
					{
						id: this.id + "topology",
						attributeId:this.attributeId,
						dataset: dataset,
						cmProvider:menuProvider,
						widgetDefinitionId: "HierarchicalAttrTopology-0.1.0"
					}
				); 

		this.connect(this._navWidget,"_createToolbar", dojo.hitch(this, this._updateTopologyButtonLabels) );
		
		this._trace.exiting(this.id, "getTopologyWidget");
		
		//return this._navWidget;
		return this._getWidgetInContainer(this._navWidget);
	},
	
	// Override. Validate the number of nodes less than 
	switchToTopologyView: function(){
		this._trace.entering(this.id, "switchToTopologyView");
		
		if(this._isAttributeValuesWithinLimit()){
			this.inherited(arguments);
		}else{
			return;
		}
		
		this._trace.exiting(this.id, "switchToTopologyView");
	},
	
	_isAttributeValuesWithinLimit: function(){
		this._trace.entering(this.id, "_isAttributeValuesWithinLimit");
		
		var targetUrl = null;
		if(this.entityType === this.ENTITYTYPE_USER){
			targetUrl ='/isrm/rest/userscope/attributes/' + this.attributeId + '/?countOnly=true';
		}else if(this.entityType === this.ENTITYTYPE_PERMISSION){
			targetUrl ='/isrm/rest/permissionscope/attributes/' + this.attributeId + '/?countOnly=true';
		}else if(this.entityType === this.ENTITYTYPE_ROLE){
			targetUrl ='/isrm/rest/rolescope/attributes/' + this.attributeId + '/?countOnly=true';
		}

		dojo.require("com.ibm.security.modeling.projects.ProjectUtil");
		this.projUtil = new com.ibm.security.modeling.projects.ProjectUtil();
	    
		var _attributeCountOnly = this.projUtil.getCountOnlyInfo(targetUrl);	
		
		if(this.projUtil.getCountOnlyErrorMessage() === null){
			
		   if(_attributeCountOnly > this.ATTRIBUTE_TOPOLOGY_MAX_ITEMS){
				this._trace.info("Cannot switch to Topology view because No of nodes are greater than 1000.", "_isAttributeValuesWithinLimit");
				this.messageArea.hideMessages();
				this.messageArea.showInfoMessage(this.pluginResources.COUNT_EXCEEDED_FOR_TOPOLOGY);
				return false;
			}else{
				return true;
			}
		}else{
			this.messageArea.showErrorMessage(this.projUtil.getCountOnlyErrorMessage());
			return false;
		}
		
		this._trace.exiting(this.id, "_isAttributeValuesWithinLimit");
	},
	
	setIncludeSubtree: function(trueOrFalse){
		this.includeSubtree = trueOrFalse;
	},
	
	_getWidgetInContainer_orig: function(widget){
			return widget;
	},
	
	_getWidgetInContainer_old: function(widget){
		return widget;
	},
	
	_toggleIncludeSubtree: function(){
		this.includeSubtree = !this.includeSubtree;
	},
	
	_getWidgetInContainer_old: function(widget){
		return widget;
	},
	_getWidgetInContainer: function(widget){
			var bc = new dijit.layout.BorderContainer({gutters:false});
			dojo.addClass(bc.domNode, "contentArea");
			dojo.style(bc, "gutters", "false");

			if(this.includeSubtreeOption){
				//var topContentPane = new dijit.layout.ContentPane({parseOnLoad:false});
				var topContentPane = new dijit.layout.ContentPane();
				var checkboxitem = new dijit.form.CheckBox({
									checked: this.includeSubtree,
									value: this.includeSubtree,
									onClick: dojo.hitch(this, this._toggleIncludeSubtree)
					});

				topContentPane.set('region', 'top');
				dojo.style(topContentPane.domNode, "height", "25px");
				bc.addChild(topContentPane);
				//topContentPane.set("content", checkboxitem);
				dojo.place(checkboxitem.domNode, topContentPane.domNode, "only");
				var labeldiv = dojo.create("label", {});
				labeldiv.innerHTML = this.localResources.INCLUDE_SUBUNITS;
				dojo.place(labeldiv, topContentPane.domNode, "last");
				//Want to remove border.
				//Not a good idea to use RapmWizardPanelCenterArea class as its related to 
				//RapmWizard and not with this widget. Either I define new .css or check 
				//for one existing.
				// Using this till I find better one
			
				dojo.addClass(topContentPane.domNode, "RapmWizardPanelCenterArea");
			}

			//var centerContentPane = new dijit.layout.ContentPane({parseOnLoad:false});
			var centerContentPane = new dijit.layout.ContentPane();
			centerContentPane.set('region', 'center');
			bc.addChild(centerContentPane);
			dojo.addClass(widget.domNode, "contentArea");
			
			//centerContentPane.set("content", widget);
			
			dojo.place(widget.domNode, centerContentPane.domNode, "only");
			//widget.startup();
			dojo.addClass(centerContentPane.domNode, "RapmWizardPanelCenterArea");
			
			bc.widget = widget;
			return bc;
	},
	
	getTable: function(){
		return this._navWidget;
	},
	
	getTopology: function(){
		return this._navWidget;
	},
	
	getItem: function(itemId){
			this._trace.entering(this.id, "_getItem");
			
			//this._trace.info("Entering", "_getItem");
			var attrId = this.attributeId;
			var targetUrl = null;
			if(this.entityType === this.ENTITYTYPE_USER){
				targetUrl ='/isrm/rest/userscope/attributes/' + this.attributeId;
			}else if(this.entityType === this.ENTITYTYPE_PERMISSION){
				targetUrl ='/isrm/rest/permissionscope/attributes/' + this.attributeId;
			}else if(this.entityType === this.ENTITYTYPE_ROLE){
				targetUrl ='/isrm/rest/rolescope/attributes/' + this.attributeId;
			}
			var valueStore = new com.ibm.security.modeling.tables.JsonRestStore( {
				target:targetUrl,
				idAttribute:"id",
				syncMode: true
			});
	
			var item = valueStore.fetch( {
				query: {
					id:itemId,
					structure:"FLAT"
				}
			});
			this._trace.exiting(this.id, "_getItem");
			
			return item.results[0];	
	},
	
	_updateTopologyButtonLabels: function(){
		this._trace.entering("", "_updateTopologyButtonLabels");
		var items = this._navWidget._toolbar.getToolbar().getChildren();
		var i;
		for(i=0;i<items.length;i++){
			var item = items[i];
			console.log(item);
			if(item.declaredClass === "dijit.form.Button"){
				console.log(item.label);
				if(item.label.indexOf("RAPM_LABEL_") === 0){
					//Workaround - Topology button event was not getting associated.
					//Hence we are explicitly connecting the switch method to the tableview button.
					if(item.label.indexOf("RAPM_LABEL_TABLEVIEW") === 0){
						this._tableviewButton = item;
						this.connect(this._tableviewButton, "_onClick", "_switch");
					}
					var suffix = item.label.substr(item.label.indexOf("RAPM_LABEL_")+11);
					item.set("label", this._getLocalizedLabel(suffix));
				}else if(item.id.indexOf(ibm.tivoli.tip.navwidget.topology.dijit.TopologyToolbar.itemTypes.ZOOM_IN_BUTTON) != -1){
					//Keeping this for - future implementation of focus to set to the selected node.
					this._zoomInButton = item;
				}
			}else if(item.declaredClass === "dijit.form.DropDownButton"){
				if(item.label.indexOf("RAPM_LABEL_") == 0){
					var suffix = item.label.substr(item.label.indexOf("RAPM_LABEL_")+11);
					item.set("label", this._getLocalizedLabel(suffix));
					item.set("title", this._getLocalizedLabel(suffix));
					//TODO Need to set localized labels for options of this dropdown.
					var menuItems = item.dropDown.getChildren();
					for(i=0;i<menuItems.length;i++){
						var menuItem = menuItems[i];
						menuItem.set("label", this._getLocalizedLabel(menuItem.label));
					}
				}
			}
		}
		//this._updateZoomIn();
		this._trace.exiting("", "_updateTopologyButtonLabels");
	},
	
	_getLocalizedLabel: function(key){
		this._trace.entering("", "_getLocalizedLabel");
		var label = key;
		var returnLabel = "";
		if((this._globalResources[label] != null) && (!(this._globalResources[label] == undefined)) ){
			returnLabel = this._globalResources[label]
		}else{
			returnLabel = label + "Not translated";
		}
		this._trace.exiting("", "_getLocalizedLabel");
		return returnLabel;
	}
});