/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

? Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.widgets.InAndOutChart");

dojo.require("dojox.charting.Chart2D");
dojo.require("dojox.charting.Theme");
dojo.require("dojox.charting.action2d.Base");
dojo.require("dojox.charting.action2d.Tooltip");
dojo.require("dojox.charting.widget.Legend");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.declare("com.ibm.security.modeling.widgets.InAndOutChart", 
	[ dijit._Widget, dijit._Templated  ],{
		
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.widgets.InAndOutChart"),			
	widgetsInTemplate : true,
	templatePath : dojo.moduleUrl("com.ibm.security.modeling.widgets", "templates/InAndOutChart.html"),
	_showMousePointer:false,
	
	populate: function(attributeValues){
		this._trace.entering(this.id, "populate");

		if (this._hasValidData(attributeValues)) {
			var _sortedAttributes = attributeValues.sort(this._sortAttributeValues);

			var series = this._getSeries(_sortedAttributes);
			if(this.chart == null){
				this.chart = (new dojox.charting.Chart2D(dojo.byId(this.id+"_inAndOutChart"))).
				addPlot("default", {type: "StackedBars", gap:5}).
				setTheme(new dojox.charting.Theme({
					plotarea:{
					stroke: null,
					fill:   null
				},
				chart:{
					stroke: null,
					fill:   null
				},
				axis:{
					stroke:    {color:"#959595", width:1},
					line:      {color:"#959595", width:.5},
					majorTick: {color: "#959595", width: .5, length: 6},
					minorTick: {color: "#959595", width: .5, length: 3},
					tick:      {font: ".8em Verdana", fontColor: "#959595"}
				}
				})).	
				addAxis("x",{
					minorTicks:false,
					majorLabels:false,
					majorTickStep:0,
					includeZero:true
				});
				this.clickAction=new dojox.charting.action2d.Base(this.chart, "default");
				this.clickAction.process=dojo.hitch(this,this._actionHandler);
				this.clickAction.connect();
			}
			this.chart.addAxis("y", {
				vertical: true, 
				fixLower: "major", 
				fixUpper: "major", 
				natural: true, 
				includeZero:true,
				maxLabelSize : 150,
				labels:this._getLabels(_sortedAttributes)
			}).
			addSeries(this.localResources.IN,series.inItems).
			addSeries(this.localResources.OUT,series.outItems);

			var animb = new dojox.charting.action2d.Tooltip(this.chart, "default");
			animb.oldProcess=animb.process;
			animb.process=dojo.hitch(animb,function(e){
				if(e.type === "onclick"){
					e.type="onmouseout";
				}
				this.oldProcess(e);
			});

			this.chart.render();

			if(this.legend == null){
				this.legend = new dojox.charting.widget.Legend({chart: this.chart,horizontal:true}, this.id+"_inAndOutChartLegend");
			}else{
				this.legend.refresh();
			}
			
			if(this._showMousePointer){
				dojo.addClass(dojo.byId(this.id+"_inAndOutChart"),"chartPointer");
			}
		}
		this._trace.exiting(this.id, "populate");			
	},
	_actionHandler:function(event){
		if(event.type === "onclick"){
			if(this._parent.inChartDrillFactoryName && this._parent.outChartDrillFactoryName){
				var drillDownData=null;
				if(event.run.data){
					var dataObject=event.run.data[event.index];
					var lunchFactoryName=null;
					
					drillDownData=this._parent._makeDrillDownData(dataObject.attributeValue);
					if(dataObject.barType && dataObject.barType === "IN"){
						lunchFactoryName=this._parent.inChartDrillFactoryName;
					}else if(dataObject.barType && dataObject.barType === "OUT"){
						lunchFactoryName=this._parent.outChartDrillFactoryName;
						drillDownData.includeInheritance = this._getOutDrillIncludeInheritanceFlag(drillDownData);
					}
					if(dataObject.other){
						this._trace.info(this.id, "_actionHandler:Nothing to launch.Bar chart is for other value");
						return;
					}
				}
				if(lunchFactoryName){
					this._parent.launchAnalysisPane(lunchFactoryName, drillDownData);
				}
			}
		}
	},
	_getOutDrillIncludeInheritanceFlag : function (drillDownData){
		return !(drillDownData.includeInheritance);
	},
	_hasValidData : function (attributeValues){
		if(attributeValues){
			if(attributeValues.length == 0){
				console.log("In And Out data is empty!");
				return true;
			}else{
				for(var i=0;i<attributeValues.length;i++){
					if(!("attributeValue" in attributeValues[i])){
						console.log("In And Out data is invalid ! [Not Found : 'attributeValue']");
					}else if(!("inCount" in attributeValues[i])){
						console.log("In And Out data is invalid ! [Not Found : 'inCount']");
					}else if(!("outCount" in attributeValues[i])){
						console.log("In And Out data is invalid ! [Not Found : 'outCount']");
					}else if(!("inPercent" in attributeValues[i])){
						console.log("In And Out data is invalid ! [Not Found : 'inPercent']");
					}else if(!("outPercent" in attributeValues[i])){
						console.log("In And Out data is invalid ! [Not Found : 'outPercent']");
					}else{
						console.log("In And Out data is valid !");
						return true;
					}
				}	
		}
		}else{
			console.log("In And Out data is null !");
		}
		return false;
	},
	_getSeries: function(attributeValues){
		var series = new Array();
		
		series.inItems = new Array();
		series.outItems = new Array();
		
		for(var i=0 ; i<attributeValues.length;i++){
			if((attributeValues[i].inCount+attributeValues[i].outCount)>0){
				var inCount = attributeValues[i].inCount;
				var outCount = attributeValues[i].outCount;
				var  percentInCount = attributeValues[i].inPercent;
				var  percentOutCount = attributeValues[i].outPercent;
				var attributeValue = attributeValues[i].attributeValue;
				var other=attributeValues[i].other;

				series.inItems.push({ 
					y: percentInCount*1.5, 
					tooltip: dojo.string.substitute(
							this.localResources.IN_COUNT_TOOLTIP, 
							{
								"inCount":inCount
							}
					), 
					barType:"IN",
					other:other,
					attributeValue:attributeValue,
					color:'#e28d00'
				});
				
				series.outItems.push({ 
					y: percentOutCount*1.5, 
					tooltip: dojo.string.substitute(
							this.localResources.OUT_COUNT_TOOLTIP, 
							{
								"outCount":outCount
							}
					),
					barType:"OUT",
					other:other,
					attributeValue:attributeValue,
					color:'#7f6699'
				});
			}
		}
		
		for(var i =series.inItems.length; i<=4;i++){
			series.inItems.push({ y: 0, tooltip: "", color:'#e28d00'});
			series.outItems.push({ y: 0, tooltip: "",color:'#7f6699'});
		}
		return series;
	},
	
	_getLabels: function(attributeValues) {
		var labels = new Array();
		var  index = 0;
		labels.push({ value: index++, text: ""});
		
		for(var i=0;i<attributeValues.length;i++){
			if((attributeValues[i].inCount+attributeValues[i].outCount)>0){
				labels.push({value: index++, text: this._truncateValue(attributeValues[i].attributeValue)});
			}
		}
	
		for(var i =labels.length; i<=6;i++){
			labels.push({ value: index++, text: ""});
		}
		
		return labels;
	},
	
	_truncateValue : function (longValue){
		if(longValue.length > 20){
			return longValue.substring(0,20)+"...";
		}else{
			return longValue;
		}
	},
	
	_sortAttributeValues : function (attribute1,attribute2){
		return attribute1.inPercent - attribute2.inPercent;
	}
});