/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.widgets.MultiFileInput");

dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("dojox.form.FileInput");
// NLS
dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");


dojo.declare ("com.ibm.security.modeling.widgets.MultiFileInput", 
		[ dijit._Widget] , {
	
	fileParameter: "filename",
	numInputs : 5,
	forms : null,
	fields : null,
	/*tracks the number of inputFields created and used
	 * to avoid dom id collisions with in the same form*/
	fieldCounter : 0,
		
	constructor : function (params, srcNode) {
		this._trace.entering(this.id, "constructor");
		this.inherited(arguments);
		this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		this.clearText = this.globalResources.CLEAR;
		this.browseText = this.globalResources.BROWSE;
		//initialize forms and fields array
		//if you initizlize them in the object properties
		//above there is cross talk between versions of the 
		//widget almost like static variables in java.
		this.forms = [];
		this.fields = [];
		this._trace.exiting(this.id, "constructor");
	},

	postCreate: function() { 
		this._trace.entering(this.id, "postCreate");
		this.inherited(arguments);		
		this._trace.exiting(this.id, "postCreate");
	},
	 buildRendering: function() {
		this.inherited(arguments); 
		this.domNode = dojo.create("div");
		for (var i = 0; i < this.numInputs; i++) {	
			
			var formField = this._createForm(this.domNode, "last");
			this.forms.push(formField.form);
			this.fields.push(formField.field);	
			
			dojo.create("br",null, this.domNode);	
		}   

    },
	startup: function () {
		this._trace.entering(this.id, "startup");
		this.inherited(arguments);
		
		for (var i = 0; i < this.fields.length; i++) {
			this.fields[i].startup();
		}
		
		this._trace.exiting(this.id, "startup");
	},

	_clearForm : function(form){
		var origForm = form;
		var formField = this._createForm(origForm, "after");
        for(var i=0;i<this.forms.length;i++){
             if(this.forms[i]===origForm) 
             {
                  this.forms[i] = formField.form;
	              this.fields[i] = formField.field;
              }
         }
         formField.field.startup();
         this.domNode.removeChild(origForm);
	},
	_createForm : function(node,pos) {
		/*
		 * Creates a form for each file input value.  This is done programatically instead
		 * of using a template because we will need to replace certain forms as they 
		 * are submitted for upload.  
		 */
		
		/*warning... edit this code very carefully.. there are some redundant calls made here
		 * to make different browser work.  If you change this ensure that you test on all
		 * supported browsers.  in IE if you get this wrong you may see the following symptoms
		 * 1) It will send a get instead of a post and then the client will get a warning about downloading files
		 * 2) The server will not recieve multipart form data and reject the request
		 */
		 var form = dojo.place("<form enctype='multipart/form-data' method='POST' accept-charset='UTF-8'></form>", node, pos);
		 form.encoding = "multipart/form-data";
		     
		var field = new dojox.form.FileInput({
                       cancelText: this.clearText, 
                       label : this.browseText, 
                       name : this.fileParameter,
                       reset:function(){}
                     
         }, dojo.create("div",null,form));
		this.connect(field.cancelNode,'onclick',dojo.hitch(this,'_clearForm',form));
		this.connect(field.cancelNode,'onkeyup',dojo.hitch(this,function(form, event){
				var key = event.keyCode;
				if(key === dojo.keys.SPACE || key === dojo.keys.ENTER){
					this._clearForm(form);
				}
			},form));
		dojo.style(field.titleNode,{"textAlign":"center","width":"5em"});
		dojo.style(field.cancelNode,{"width":"5em","textAlign":"center"});
		dojo.attr(field.inputNode,"disabled","disabled");
		dojo.attr(field.inputNode,"readOnly","true");
		dojo.attr(field.fileInput,"tabIndex","0");
		this.connect(field.fileInput,'onfocus',dojo.hitch(this,function(inputWidget,event){
				dojo.addClass(inputWidget.inputNode,"fileInputFocus");
			},field));
		this.connect(field.fileInput,'onblur',dojo.hitch(this,function(inputWidget,event){
				dojo.removeClass(inputWidget.inputNode,"fileInputFocus");
			},field));
		
		
		//Defect Fix for 26221: Clear button on the new import session page is not keyboard accessible. STARTS
		dojo.attr(field.cancelNode, "dojoAttachPoint", "focusNode, cancelNode");
		//Ideally we should not be required to set the tabIndex. The focusNode should enable the 
		//tabIndex attribute. However somehow it was not working in IE hence explicitly set the tabIndex here.
		dojo.attr(field.cancelNode, "tabIndex", "0");
		return {form: form,field: field};
		
	},	
	getFileCount : function() {
		var count = 0;
		
		for (var i = 0; i < this.fields.length;  i++)
		
		count += this._fileInputHasValue(this.fields[i]) ? 1 : 0;
	
        return count;
	},
	getFormsForUploadAndReplace : function () {
		/*
		 * Gathers the forms with filled in values to pass for upload processing
		 * 
		 * Note this method will replace any form with a filled in value with a new form
		 * hiding the existing form.  
		 * 
		 * @see _replaceForm for a detailed explaination
		 */
		var uploadData = [];
		for (var i = 0; i < this.fields.length;  i++) {
			if (this._fileInputHasValue(this.fields[i])) {
				var args = {form: this.forms[i], fileName : this._fileInputGetValue(this.fields[i])};
				uploadData.push(args);
				this._replaceForm(i);
			}			
		}
		return uploadData;
	}, 
    deleteAndReplaceForm : function(){
        for (var i = 0; i < this.numInputs; i++){
		var origForm = this.forms[i];
		var formField = this._createForm(origForm, "after");
            this.forms[i] = formField.form;
	        this.fields[i] = formField.field;
            formField.field.startup();
		    this.domNode.removeChild(origForm);

         }
  },
	_replaceForm : function(index) {	
		/*
		 * The iframe.io.send transport used to send this data to the server
		 * requires that the form be available in the dom with the correct
		 * values at the time the submit is sent to the server. The send to the 
		 * server could be minutes after the submission is made.  To support
		 * this process we will hide forms that are pending upload and replace
		 * them with new forms.   The forms will ultimately be removed from
		 * the dom when the upload completes.  
		 * 
		 * @see UploadTracker._removeTempForm for handling of the form removal  
		 */
		var origForm = this.forms[index];
		//create a new form directly after the one we are replacing
		var formField = this._createForm(origForm, "after");
		//replace the form/field in the index
		this.forms[index] = formField.form;
		this.fields[index] = formField.field;
		//startup is required as we are outside the normal widget life cycle
		formField.field.startup();
		//hide the form.  
		origForm.style.display = "none";
	},
	reset : function() {
		/*resets the value of the visible file input fields
		 * note this will not affect the hidden fields that are already
		 * inflight to the server via the iframe io
		 */
		for (var i = 0; i < this.numInputs; i++) {	
			this.fields[i].reset();		
		}
	},
	_fileInputHasValue : function(fileInput) {
		return fileInput.inputNode.value.trim().length > 0;
	},
	_fileInputGetValue : function(fileInput) {
		return fileInput.inputNode.value.trim();
	},
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.widgets.MultiFileInput")
	
	
	
});
