/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

� Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.widgets.MultiMessageArea");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("ibm.tivoli.tip.dijit.TIPMessageArea");
dojo.require("com.ibm.security.modeling.util.Trace");

// define the widget class
dojo.declare (
	"com.ibm.security.modeling.widgets.MultiMessageArea", 
    [ dijit._Widget, dijit._Templated ],
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.widgets.MultiMessageArea"),
	
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.widgets","templates/MultiMessageArea.html"),
	widgetsInTemplate: true,
	scrollNewMessageIntoView: true,
	constructor: function() {
		this._trace.entering("", "constructor");
		this._trace.object(arguments, "constructor", "Constructor args");
		
		// borderContainer: dijit.layout.BorderContainer
		//		BorderContainer widget that needs to be resized when shown or hidden.
		// _messageAreas: map
		//		holds ibm.tivoli.tip.dijit.TIPMessageArea objects mapped with "id" property.
		this._messageAreas = {};
		
		this.count = 0;
		this._trace.exiting("", "constructor");
	},
	postCreate: function() { 
		this._trace.entering(this.id, "postCreate");
		// dojo.addClass(this.domNode,"contentArea");
		this.inherited("postCreate", arguments);
		this._trace.exiting(this.id, "postCreate");
	},
	
	destroy: function() {
		this._trace.enteringWithStack(this.id, "destroy", true);
		try {
			for(messageId in this._messageAreas){
				this._messageAreas[messageId].destroyRecursive();
			}
			
			this.inherited(arguments);
			
			this._trace.exiting(this.id, "destroy");
		} catch(e) {
			this._trace.exception(e, "destroy");
			throw e;
		}
	},
	
	showInfoMessage: function(message, messageID) {
		var ma = this._getMessageArea();
		ma.showInfoMessage(message, null);
		return ma.id;
	},

	showWarningMessage: function(message, messageID) {
		var ma = this._getMessageArea();
		ma.showWarningMessage(message, null);
		return ma.id;
	},

	showErrorMessage: function(message, messageID) {
		var ma = this._getMessageArea();
		ma.showErrorMessage(message, null);
		return ma.id;
	},
	
	_getMessageArea: function() {
		this._trace.entering(this.id, "_getMessageArea");
		
		// get available (hidden) MessageArea 
		var ma = null;
		var mas = dojo.query("> div", this.container);
		
		if (mas && mas.length > 0) {
			for (var i=0; i< mas.length; i++) {
				var m = mas[i];
				var id = dojo.attr(m, "id");
				
				this._trace.finest(this.id+" - while querying, out of "+mas.length+" checking: "+(i+1)+" node, having id: "+id, "_getMessageArea");
				// dijit.byId(id) can be used here (in place of _messageAreas[id]). 
				// but used _messageAreas map instead, to get references while destroy().
				if (!this._messageAreas[id].isShowing && (this._messageAreas[id]._isAnimationInPlay === false)) {
					this._trace.finest(this.id+" - found available MessageArea.", "_getMessageArea");
					ma = this._messageAreas[id];
					break;
				}
			}
		}
		
		if (!ma) {	// create new one.
			this._trace.finest(this.id+" - creating new MessageArea.", "_getMessageArea");
			ma = this._createMessageArea();
		}
		this._trace.exiting(this.id, "_getMessageArea");
		return ma;
	},
	
	_createMessageArea: function() {
		try {
			this._trace.entering(this.id, "_createMessageArea");	
			this.count++;
			// create hook
			var hook = dojo.create("div", {id: this.id+"_MessageArea_"+this.count+"_hook"}, this.container, "last");
			
			// create TIPMessageArea and attach it to the hook
			var ma = new ibm.tivoli.tip.dijit.TIPMessageArea({
				id: this.id+"_MessageArea_"+this.count
			}, hook);
			
			// Animation
			ma._isAnimationInPlay = false;
			this.connect(ma._wipeOut, "play", function(){ 
				ma._isAnimationInPlay = true;
			});
			this.connect(ma._wipeOut, "onEnd", function(){ 
				ma._isAnimationInPlay = false;
				
			});
			if (this.scrollNewMessageIntoView) {
				this.connect(ma._wipeIn, "onEnd", function(){ 
					dojo.window.scrollIntoView(ma.domNode);
					
				});
			}
			if (this.borderContainer) {
				this._trace.finest("Connecting to borderContainer resize method because it exists", "_createMessageArea");
				this.connect(ma._wipeIn, "onEnd", this.resizeContainer);
				this.connect(ma._wipeOut, "onEnd", this.resizeContainer);
			}
			
			var _clearContents = dojo.hitch(this, this.clearContents, ma);
			var _placeLast = dojo.hitch(this, this.placeLast, ma);
			
			this.connect(ma._wipeOut, "onEnd",_clearContents);
			this.connect(ma._wipeOut, "onEnd", _placeLast);
			this._makeMessageAreaAccessible(ma);
			
			ma.startup();
			// put in map
			this._messageAreas[ma.id] = ma;
			
			// return
			return ma;
		} finally {
			this._trace.exiting(this.id, "_createMessageArea");
		}
	},
	
	_makeMessageAreaAccessible:function(ma) {
		// summary:
		//		Adds accessiblity features to the message area because it wasn't initially coded with them.
		// ma: ibm.tivoli.tip.dijit.TIPMessageArea
		//		The message area to make accessible
		
		//the role alert tells the screen reader to read the message when it becomes visible
		dijit.setWaiRole(ma.wipeNode, "alert");
		dijit.setWaiState(ma.wipeNode, "live", "assertive");
		//setting tab index 0 allows the user to tab through the elements in natural order
		ma.closeBtn.setAttribute("tabIndex", "0");
		dijit.setWaiRole(ma.closeBtn, "button");
	},
	
	resizeContainer: function() {
		// summary:
		//		Resizes the top border container for the needs of displayed message boxes.
		// description:
		//		Sizes the message display area to the cumulative height of visible message boxes.  Caps the
		//		height at 100px (Introducing a scroll)
		this._trace.entering("", "resizeContainer");
		var totalMessageHeight = 0;
		for (var messageAreaId in this._messageAreas) {
			var messageArea = this._messageAreas[messageAreaId];
			if (messageArea.isShowing === true) {
				var box = dojo.marginBox(messageArea.styleArea);
				totalMessageHeight += box.h;
				this._trace.finest("box height: " + box.h + " total height: " + totalMessageHeight, "resizeContainer");
			}
			//for some reason IE introduces a small bit of bottom margin, which causes a scrollbar if the height of the 
			//inner and outer node is equal, so we need to increment it just a bit
			if (dojo.isIE) {
				totalMessageHeight += 3;
			}
		}
		//Cap height at 106px, which is about the height of 5 single line messages.
		var topBorderHeight = totalMessageHeight < 106?totalMessageHeight:106;
		
		var childArr = dojo.filter(this.borderContainer.getChildren(), function (child) { return child.region == "top"; });
		if (childArr && childArr.length > 0) {
			var topRegionChild = childArr[0];
			this.borderContainer._layoutChildren(topRegionChild.id, topBorderHeight);
		}
		
		
		this._trace.exiting("new height: " + topBorderHeight, "resizeContainer");
	},
	
	clearContents: function(ma) {
		ma.setMessage("");
		ma.setMessageId(null);
	},
	
	placeLast: function(ma) {
		this._trace.entering(this.id, "placeLast");
		dojo.place(ma.domNode, this.container, "last");
		this._trace.exiting(this.id, "placeLast");	
	},
	
	hideMessage: function(/*string id*/ messageWidgetId) {
		if (messageWidgetId) {
			var ma = this._messageAreas[messageWidgetId];
			if (ma) {
				ma.hide();
			}
		}
	},
	
	/**
	 * The method is used to hide the TIPMessage areas which are created.
	 * This method is used in dialogs which are hidden on clicking of the OK/Cancel buttons.
	 * For example: CreateImportSession
	 */
	hideMessages: function(){
		
		var mas = dojo.query("> div", this.container);
		if (mas && mas.length > 0) {
			for (var i=0; i< mas.length; i++) {
				var m = mas[i];
				var id = dojo.attr(m, "id");
				if (this._messageAreas[id].isShowing) {
					this._messageAreas[id].hide();
				}
			}
		}
	},

	/**
	 * The method is used to hide the TIPMessage areas which are created. It will hide all messages
	 * excluding the one passed in the function. 
	 * This method is used in dialogs which are hidden on clicking of the OK/Cancel buttons.
	 * For example: CreateImportSession
	 */	
	hideMessagesExcluding: function(messageID) {
		var mas = dojo.query("> div", this.container);
		if (mas && mas.length > 0) {
			for (var i=0; i< mas.length; i++) {
				var m = mas[i];
				var id = dojo.attr(m, "id");
				if (this._messageAreas[id].isShowing) {
					/* Only in cases where messageID is not part of a given message, hide it. */
					if(this._messageAreas[id].domNode.innerHTML.search(messageID) == -1) {
						this._messageAreas[id].hide();
					}
				}
			}
		}
	},

	/**
	 * The method is used to hide the TIPMessage areas which are created.
	 * This method is used in dialogs which are hidden on clicking of the OK/Cancel buttons.
	 * For example: CreateImportSession
	 */
	numMessagesShown: function(){
		var msgCount = 0;
		var mas = dojo.query("> div", this.container);
		if (mas && mas.length > 0) {
			for (var i=0; i< mas.length; i++) {
				var m = mas[i];
				var id = dojo.attr(m, "id");
				if (this._messageAreas[id].isShowing) {
					msgCount++;
				}
			}
		}
		return msgCount;
	},
	
	/**
	 * The method is used to check if the message displayed at the top of the stack is the one
	 * that's passed in as the argument. That will ensure that we are not displaying duplicate
	 * messages one after the other.
	 */
	messageDisplayedAtTop: function(curMessage) {
		var msgCount = 0;
		var mas = dojo.query("> div", this.container);
		if (mas && mas.length > 0) {
			for (var i=0; i< mas.length; i++) {
				var m = mas[i];
				var id = dojo.attr(m, "id");
				if (this._messageAreas[id].isShowing) {
					if(this._messageAreas[id].domNode.innerHTML.search(curMessage) != -1) {
						return true;
					} else {
						return false;
					}
				}
			}
		}
		return false;
	}
	
});