/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

� Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.widgets.PeoplePickerDialog");

dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.widgets.PickerDialog");
dojo.require("com.ibm.security.modeling.tables.UserTable");

// NLS
dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");


dojo.declare ("com.ibm.security.modeling.widgets.PeoplePickerDialog", com.ibm.security.modeling.widgets.PickerDialog , {
		// summary:
	    //    Creates a dialog that includes a UserTable where users can pick people and add them to a datastore.
	    // description:
	    //    Sample Usage:
		//		var ownersDialog = new com.ibm.security.modeling.widgets.PeoplePickerDialog({
		//			id: this.id+"_parentPickerDialog", 
		//			namespace:this.namespace,
		//			fromDataStore: peopleStore,	// for picker table.
		//			toDataStore: this._ownerFieldTableStore,
		//			parentWidget: this,
		//			contextRoot: this.contextRoot,
		//			globalResources: this.globalResources,	// for picker table.
		//			localResources: this.parentPeoplePickerResources,
		//			helpFile: "/path/to/the/help/file.html"
		//		});
		//		ownersDialog.startup();
		//		ownersDialog.show();
	    // 
	// tableQuery: object
	//		query to be passed to table.
	tableQuery:null,
	widthFactor:1.4,
	// saveIndicator: com.ibm.security.modeling.indicator.SaveIndicator
	//		An instance of com.ibm.security.modeling.indicator.SaveIndicator.
    	//		Displays save indicator.
	saveIndicator:null,
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.widgets.PeoplePickerDialog"),
	postCreate: function() { 
		this._trace.entering(this.id, "postCreate");
		this.inherited(arguments);
		// create table
		this._setupTable();
		this._trace.exiting(this.id, "postCreate");
	},
	postMixInProperties:function() {
			this._trace.entering(this.id, "postMixInProperties");
			this.inherited(arguments);
			this.contentStyleClass="userPickerDialogContent";
			this._trace.exiting(this.id, "postMixInProperties");
	},
	startup: function () {
		this._trace.entering(this.id, "startup");
		this.inherited(arguments);
		
		// start table
		this._pickerTable.startup();
		this._trace.exiting(this.id, "startup");
	},
	
	_setupTable: function () {
		this._trace.entering(this.id, "_setupTable");
		
		var tableArgs = {
			id: this.id+"_peopleTable",
			dataStore: this.fromDataStore, 
			entityTypeText: this.globalResources.USERS,
			isClientSide: false,
			isFilterRendered: true,
			isRefreshRendered: true,
			parentContainer: this.container,
			messageArea: this.messageArea,
			structure: this.structure,
			tableType:"peoplePicker"
		};
		
		if (this.tableQuery) {
			tableArgs.query = this.tableQuery;
		}
		this._pickerTable = new com.ibm.security.modeling.tables.UserTable (tableArgs, this.id+"_pickerDialogTableContainer");
		
		this._trace.exiting(this.id, "_setupTable");
	}
	
});