/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

�  Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.widgets.PickerDialog");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Dialog");
dojo.require("dijit.form.Button");
dojo.require("com.ibm.security.modeling.dijit.layout.BorderContainer");
dojo.require("ibm.tivoli.tip.dijit.TIPButton");
dojo.require("com.ibm.security.modeling.widgets.IsrmDialogBase");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.util.MessageUtilities");

// NLS
dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");


// define the widget class
dojo.declare (
	"com.ibm.security.modeling.widgets.PickerDialog", 
	com.ibm.security.modeling.widgets.IsrmDialogBase,
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.widgets.PickerDialog"),
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.widgets","templates/PickerDialog.html"),
	widgetsInTemplate: true,
	
	// namespace: [readonly] String
	// 			Unique identifier to the instance of portlet page where this item will exist
	namespace:"",
	
	// contextRoot: [readonly] String
	//			The context root of the server from which Content is being served.
	contextRoot: "",

	// pluginResources: Object
	//			Reference to the plugin shared resource bundle
	pluginResources: null,
	
	// helpFile: [readonly] String
	//			name of the HTML help file for picker.
	helpFile: "",	
	contentStyleClass: "pickerDialogContent",
	contentTablePanelStyleClass: "pickerDialogContentTablePanel",
	
	// This attribute can be modified for Pickers that probably need a different z-Index down the line.
	zIndexStyling: "zIndexRolePicker",
	
	showWarningsOnAdd : true,	// true: if on add success, response header contains "Warning" then warnings will be shown.
	showErrorsOnAdd : true,	// true: if add fails, then error message will be shown.
	
	constructor: function() {
		// _pickerTable: com.ibm.security.modeling.tables.Table
		//		table widget to pick the items from.
		
		// fromDataStore: dojo.data.DataStore
		//		data to be used to populate the table.  Plugin any compatible dojo datastore object.
		
		// toDataStore: dojo.data.DataStore
		//		data to populate by the picked rows from the table.  Plugin any compatible dojo datastore object.
		
		
		// parentMessageArea: com.ibm.security.modeling.widgets.MultiMessageArea
		//		Message area of parent container. 
		//		If specified, then messages will be displayed in it.
		
		// callback: function
		//		Handles picker close:	callback should return boolean, indicating if the picker is to be closed.
		//
		//		if 'toDataStore' is not provided: called with selected items
		//		if 'toDataStore' is provided:  called with REST requests' success, failure details.
		
		// globalResources: Object
		//			Reference to the global shared resource bundle
		
		// localResources: Object
		//			Reference to the local shared resource bundle
	
		// saveIndicator: com.ibm.security.modeling.indicator.SaveIndicator
		//		An instance of com.ibm.security.modeling.indicator.SaveIndicator.
	    //		Displays save indicator.
		
		// messageTypeFilter: Array
		//		Specifies message types (from batch rest response), which should be shown.  
		//		Supported values: 'info', warn', 'error'
		
		
		// toDataStoreTargetUri: string,	
		// 		If specified, then batch Add request will be sent to this uri instead of Store's targetUri
		// toDataStoreNoContent: boolean
		// 		If specified and true, then batch Add request will send uri of object instead of content
		
		this._trace.entering("", "constructor");
		this.mu = new com.ibm.security.modeling.util.MessageUtilities();		
		this._trace.object(arguments, "constructor", "Constructor args");
		this._trace.exiting("", "constructor");
	},
	postMixInProperties: function() {
		this._trace.entering(this.id, "postMixInProperties");
		
		this.inherited(arguments);
		
		if (!this.messageTypeFilter) {
			this.messageTypeFilter = null;
		}
		this._trace.exiting(this.id, "postMixInProperties");
	},
	buildRendering: function(){
		this.dialogInstruction=this.localResources.TABLE_INSTRUCTION;
		this.dialogTitle=this.localResources.TITLE;
		this.inherited(arguments);
	},
	
	postCreate: function() { 
		this._trace.entering(this.id, "postCreate");
		
		dojo.addClass(this.domNode,"fullSize");
		this.inherited("postCreate", arguments);
		this.addCommonButton({
			onClick:dojo.hitch(this, this.onCancel),
			type:"CANCEL"
		});
		this.okButton = this.addCommonButton({
			onClick:dojo.hitch(this, this._onOK),
			type:"OK"
		});
		this._trace.exiting(this.id, "postCreate");
	},
	
	_onOK: function () {
		try {
			this._trace.entering(this.id, "_onOK");
			
			var selectedRows = null;
			if (this._pickerTable && 
				((selectedRows = this._pickerTable.getSelectedItems()) && selectedRows.length > 0) &&
					this.toDataStore) {				
				this._trace.finest(this.id+" - adding selected items to toStore", "_onOK");	
				this._trace.object(selectedRows, "_onOK");
				this._addPickedItems(selectedRows); // Add picked items to "toDataStore"
			} else {
				// No widget to pick from, or
				// Nothing is picked, or
				// Items are picked but no destination Store specified
				this._trace.finest(this.id+" - Widget to pick from: "+ (this._pickerTable ? "available": " not available"), "_onOK");
				this._trace.finest(this.id+" - Picked items count: "+ (selectedRows ? selectedRows.length: 0), "_onOK");
				this._trace.finest(this.id+" - Destination store: "+ (this.toDataStore ? "available": " not available"), "_onOK");
				
				if (selectedRows.length > 0) {
					this._trace.finest(this.id+" - Invoking callback", "_onOK");
					this.callback(selectedRows);	// Invoke call back (if provided) and close.
				}
			}
			
		} finally {
			this._trace.exiting(this.id, "_onOK");
		        this.hide();
		}
	},
	
	_addPickedItems: function (selectedRows) {
		this._trace.entering(this.id, "_addPickedItems");
		
		// show save indicator
		if (this.saveIndicator) {
			this.saveIndicator.show(this.saveIndicatorText);	
		}
		
		// Update store.
		for (var i = 0; i < selectedRows.length; i++) {
			try{
				this.toDataStore.newItem(selectedRows[i]);
			}catch(e){
				this._trace.info(selectedRows[i] + " already exist in data store ");
			}
		
		}
		
		// When done
		var selectedItemsCount = selectedRows.length;
		var processedItems = 0;
		var successCount = 0;
		var failureCount = 0;
		var self = this;
		var handleComplete = function(actions, value, isError) {
			if (isError == false) {
				successCount = actions.length;
			} else {
				failureCount = actions.length;
			}
			processedItems = processedItems + actions.length;
			if (processedItems == selectedItemsCount) {
				self.showMessages(value);
				if (self.callback) {
					self.callback([value, {successCount: successCount, failureCount: failureCount}]);	// invoke callback.
				}
				if (self.saveIndicator) {
					self.saveIndicator.hide(self.saveIndicatorText);	// hide save indicator
				}
			}
		};
		
		var ma = self.messageArea;
		if (self.parentMessageArea) {
			ma = self.parentMessageArea;
		}
		// Construct store save parameters.
		var kwArgs = {
			handleAsBatch: true,
			alwaysPostNewItems : true,	// makes POST request even if the item has 'uri' (idAttribute) in it. Else PUT request is made.
			
			onComplete: dojo.hitch(this, function(args) {
				//write store do not call onComplete with arrays 
				if(dojo.isArray(args)) handleComplete(args[0], args[1], false);
			}),
			
			onError: dojo.hitch(this, function(args) {
				this._trace.error("Error while adding.", "error callback");
				if (args instanceof Array) { // partial failure.
					handleComplete(args[0], args[1], true);
				} else {	// batch request itself failed.
					var errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
					errorUtil.handleRESTError(args, ma);
					this.saveIndicator.hide(this.saveIndicatorText);	// hide save indicator
				}
			})
		};
		
		if (this.toDataStoreTargetUri) {
			kwArgs.targetUri = this.toDataStoreTargetUri;
		}
		if (this.toDataStoreResponsePayloadAttribute) {
			kwArgs.responsePayloadAttribute= this.toDataStoreResponsePayloadAttribute;
		}
		if (this.toDataStoreNoContent) {
			kwArgs.noContent = this.toDataStoreNoContent;
		}
		
		// commit changes
		this.toDataStore.save(kwArgs);
		
		this._trace.exiting(this.id, "_addPickedItems");
	},
	
	showMessages: function(value) {
		
		var ma = this.messageArea;
		if (this.parentMessageArea) {
			ma = this.parentMessageArea;
		}
		
		var msgs = this.mu.extractBatchMessages(value, this.messageTypeFilter, true);
		this.mu.showMessages(msgs, ma, true);
	}
	
	
});