/*
Licensed Materials - Property of IBM

Role and Policy Modeler

� Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.widgets.ProgressBar");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.declare (
	"com.ibm.security.modeling.widgets.ProgressBar", 
    [ dijit._Widget, dijit._Templated ],
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.widgets.ProgressBar"),
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.widgets","templates/ProgressBar.html"),
	
	widgetsInTemplate: false,
		
	// _searchCount: integer
	//		A counter specifying number of search operations that are taking place
	//		This is to ensure that the launch of the progress bar is thread-safe.
	_searchCount: 0,
	
	// messageToDisplay: String
	//      The message to be displayed in the progress bar.
	messageToDisplay: null,
	
	progressBarDialog: null,
	
	constructor: function() {
		this._trace.entering("", "Progress Bar constructor");
		this.inherited (arguments);
		this._trace.exiting("", "Progress Bar constructor");
	},
	destroy: function() {
		this._trace.enteringWithStack(this.id, "Progress Bar destroy", true);
		try {
			this.inherited(arguments);
		} catch (e) {
			this._trace.exception(e, "Progress Bar destroy");
			throw e;
		}
	},
	generateProgressBarDivWithMessage: function(/*string*/ messsate) {
		this._trace.entering(this.id, "generateProgressBarDivWithMessage");
		var template = dojo.cache("com.ibm.security.modeling.widgets", "templates/ProgressBar.html", {sanitize:true});
		var div = template.toString().replace("${messageToDisplay}", messsate? messsate: this.messageToDisplay);		
		this._trace.exiting(this.id, "generateProgressBarDivWithMessage");
		return div;
	},
	/* Create the progress bar with the content as passed in, in the form of a div.
	 */
	createProgressBar: function(div) {
		this._trace.entering(this.id, "createProgressBar");
		/* Create a new dialog to show the progress bar. */
		//duration:0 , to make things simpler and less error prone . Some one can call hide progress bar , and hide of another dialog
		//which will create problem . 
		var newProgressBarDlg = new dijit.Dialog({ title: "Progress Bar", "class":"progressBarDialogStyle",duration:0});
		/* Hide the titlebar of the dialog. */
		newProgressBarDlg.titleBar.style.display='none';
		newProgressBarDlg.startup();	
		newProgressBarDlg.setAttribute("content",div);
		dojo.removeClass(newProgressBarDlg.containerNode, "dijitDialogPaneContent");
		this._trace.exiting(this.id, "createProgressBar");
		return newProgressBarDlg;
	},

	/* This function is to be invoked by callers, in case they intend to show progress bar of any task.
	 * Its more of a matter to pass the message they would like get displayed in the progress bar. It 
	 * should be a globalized string.
	 * For e.g. Search in progress... */
	show:function(msg) {
		// Increase the display count of the progress bar.
		this._trace.entering(this.id, "showProgressBar");
		this._searchCount++;
		this.messageToDisplay = msg;
		try {
			var div = this.generateProgressBarDivWithMessage(msg);
			if (this.progressBarDialog) {
				this.progressBarDialog.setAttribute("content", div);
			} else {
				this.progressBarDialog = this.createProgressBar(div);
			}
			this.progressBarDialog.show();
		} catch (e) {
			this._trace.exception(e, "showProgressBar");
		}
		this._trace.exiting(this.id, "showProgressBar");
	},
	
	/* Invoke this function when you intend to hide the progress bar.
	 */
	hide:function() {
		// Hide the progress bar if no operation is pending.
		this._trace.entering(this.id, "hideProgressBar");
		if (this._searchCount > 0) {
			this._searchCount--;
			if (this._searchCount == 0) {
				this.progressBarDialog.hide();
				//setTimeout(dojo.hitch(this,this.destroyProgressDialog),this.progressBarDialog.duration + 100);
				this.destroyProgressDialog();
			}
		}
		this._trace.exiting(this.id, "hideProgressBar");
	},
	destroyProgressDialog:function(){
		if (this._searchCount == 0 && this.progressBarDialog) {
			this.progressBarDialog.destroy();
			this.progressBarDialog = null;
		}
	}
	
});
