/*
Licensed Materials - Property of IBM

Role and Policy Modeler

� Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.widgets.RapmWizard");


dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.widget.Wizard");
dojo.require("dijit.form.Button");
dojo.require("com.ibm.security.modeling.dijit.layout.BorderContainer");
dojo.require("com.ibm.security.modeling.dijit.layout.ContentPane");
dojo.require("com.ibm.security.modeling.breadcrumbs.BreadcrumbBuilder");
dojo.require("com.ibm.security.modeling.layouts.TemplatedPanelWithLayout");
dojo.require("com.ibm.security.modeling.projects.BreadCrumb");
dojo.require("com.ibm.security.modeling.util.NlsUtilities");
dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.api.ProjectRestManager");
dojo.require("com.ibm.security.modeling.projects.ProjectUtil");
dojo.require("com.ibm.security.modeling.widgets.ProgressBar");
dojo.require("com.ibm.security.modeling.util.HelpUtilities");

dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling", "LogMessages");
dojo.requireLocalization("com.ibm.security.modeling.projects", "ProjectCreationWizardText");

dojo.declare("com.ibm.security.modeling.widgets.RapmWizard",
            [ com.ibm.security.modeling.layouts.TemplatedPanelWithLayout ],
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.widgets.RapmWizard"),
	
	_helpUtil: new com.ibm.security.modeling.util.HelpUtilities(),
	
	wizard: null,

	templatePath: dojo.moduleUrl("com.ibm.security.modeling.widgets","templates/RapmWizard.html"),	

	widgetsInTemplate: true,

	globalResources: null, //optional

	localResources: null,  //optional

	messageResources: null, //optional

	namespace: "",

	requireBreadCrumb: null,

	contextRoot: "",

// Private variables 

	_connections: [],
	
	_lastbreadcrumb: null,
	
	_progressBar: null,
	
	_breadCrumbBar: null,

	constructor: function () {
		this._trace.entering(this.id, "constructor");
		
		this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		this.messageResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "LogMessages");
		this._progressBar = new com.ibm.security.modeling.widgets.ProgressBar();
		if(this.requireBreadCrumb === null){
			this.requireBreadCrumb = false;
		}
		this._trace.exiting(this.id, "constructor");
	},
	
	postCreate:function(){
		this._trace.entering(this.id, "postCreate");		
		this.inherited(arguments);		
		this.createPanels();		
		//currently observed that Cancel button function can only be set before the wizard startup. 
		//After startup this isn't working.
		this._setWizardButtonCancel();
		this._makeAccessible();	
		this._trace.exiting(this.id, "postCreate");
	},
	
	_makeAccessible:function() {
		//remove the tabpanel role that gets put on stackcontainer, since we are using that role for its children.
		dijit.removeWaiRole(this.wizard.containerNode, "tabpanel");
		dojo.subscribe(this.wizard.id + "-selectChild", function(pane){	
			//Let the widget set its focus once loaded
			dijit.focus(pane.domNode);
		});	
		this._connections.push(dojo.connect(this.wizard, "_transition", function(newPane, oldPane) {
			if (newPane && newPane.domNode && oldPane && oldPane.domNode) {
				dijit.setWaiState(oldPane.domNode, "hidden", true);
				dijit.setWaiState(newPane.domNode, "hidden", false);
			}
		}));
	},
	
	startup: function(){	
	
		this._trace.entering(this.id, "startup");
		
		try {

			this.inherited(arguments);
			this._initialize();
			
		} catch (e) {
			this._showMessage("critical", e);
		}
		
		this._trace.exiting(this.id, "startup");
	},
	
	_initialize: function(){
			this._createBreadcrumbBar();
								
			//give the container a StyleUtilities global function to figure out bidi direction,
			//since TIP doesn't do bidi the way dojo expects.
			this.container.isLeftToRight = NlsUtilities.isLtr;

			this._updateBreadCrumb();
					
			this._setHelp();

			this._setWizardButtonValidations();
	},
	
	destroy: function() {
		this._trace.entering(this.id, "destroy");			
		this.msgArea.hideMessages();
		this.breadCrumbBar.destroy();
		dojo.forEach(this._connections, dojo.disconnect);			
		this.inherited(arguments);			
		this._trace.exiting(this.id, "destroy");
	},
	
	_setWizardButtonCancel: function(){
			this.wizard.cancelFunction=dojo.hitch(this, this._onCancel);
	},

	_setWizardButtonValidations: function() {
		this._trace.entering(this.id, "_defineWizardButtonBehaviors");			
		// Updating the breadCrumbs of wizard on Panel selection.
		var connection1 = dojo.connect(this.wizard, "selectChild", this, this._onNavigation);
		this._connections.push(connection1);
		var wizardpanes = this.wizard.getChildren();
		for(var i=0;i<wizardpanes.length;i++){
			if(wizardpanes[i].widget.validate){
				wizardpanes[i].passFunction = dojo.hitch(wizardpanes[i].widget, wizardpanes[i].widget.validate);
			}
		}		
		this._trace.exiting(this.id, "_defineWizardButtonBehaviors");	
	},
	

	_onNavigation: function(){
		this._trace.entering(this.id, "_onNavigation");
		this._clearMessages();
		this._setPanelNavigationButtons();

		this._updateBreadCrumb();
		
		this._setHelp();
		
		
		this._trace.exiting(this.id, "_onNavigation");
	},
	
	_setHelp: function(){
		fileName = this.wizard.selectedChildWidget.widget.getHelpFileName();
		if(fileName){
			this._helpUtil.setHelp(fileName);
		}
	},
	
	_clearMessages: function() {
		this._trace.entering(this.id, "_clearMessages");
		
		this.msgArea.hideMessages();
		
		this._trace.exiting(this.id, "_clearMessages");
	},

	
	//Need to set the bottom border for the buttons.
	_getWidgetContainer: function(widget){
		var bc = new com.ibm.security.modeling.dijit.layout.BorderContainer();
		dojo.addClass(bc.domNode, "fullsize");
		var centerContentPane = new com.ibm.security.modeling.dijit.layout.ContentPane();
		//var bottomContentPane = new com.ibm.security.modeling.dijit.layout.ContentPane();
		centerContentPane.set('region', 'center');
		//bottomContentPane.set('region', 'bottom');

		bc.addChild(centerContentPane);
		//bc.addChild(bottomContentPane);
		
		dojo.addClass(centerContentPane.domNode, "RapmWizardPanelCenterArea");
		//dojo.addClass(bottomContentPane.domNode, "RapmWizardButtonStyle");
		
		dojo.place(widget.domNode, centerContentPane.domNode, "only");
		bc.startup();
		return bc;
	},
	
	addChild: function(widget, breadcrumblabel, finishCallBack, instructions, /*Integer?*/ insertIndex){
	
		this._trace.entering(this.id, "addChild");
		try{
			var wizardPane = new dojox.widget.WizardPane();
			
			var paneid = widget.id +"_wizardpane";
			
			//Let me keep the reference of the widget added to the WizardPane.
			wizardPane.widget = widget;
			
			this.wizard.addChild(wizardPane);							
			
			var children = this.wizard.getChildren();
			if (children && children.length > 1) {
				
				dojo.forEach(children, function(child, index) {
					if (index == children.length -1) {
						child.isLastChild = true;
					} else {
						child.isLastChild = false;
					}
				}, this);
			}
			
			dojo.place(this._getWidgetContainer(widget).domNode, wizardPane.domNode, "only");
			
			if(finishCallBack){
				wizardPane.doneFunction = finishCallBack;
			}
			if(breadcrumblabel){
				wizardPane.breadcrumblabel = breadcrumblabel;
			}else{
				wizardPane.breadcrumblabel = wizardPane.id;
			}
			this._makeChildAccessible(wizardPane, breadcrumblabel, instructions);
			widget.startup();
			
		}catch(e){
			console.log("Failed to add the widget in wizard" + e);
		}
		this._trace.exiting(this.id, "addChild");
	},
	
	_makeChildAccessible: function(wizardPane, label, description) {
		dijit.setWaiRole(wizardPane.domNode, "region");
		dijit.setWaiState(wizardPane.domNode, "label", label);
		if(description) {
			dijit.setWaiState(wizardPane.domNode, "describedby", description);
		}
		dojo.attr(wizardPane.domNode, "tabindex", 0);
		dojo.attr(wizardPane.domNode, "title", label);
		
	},
	
	//Override this to create your panels. In this, use addChild to add the widget.
	createPanels: function(){
		this._trace.entering(this.id, "createPanels");
		
		this._trace.exiting(this.id, "createPanels");
		
	},
	

	_showMessage: function(/*String*/type, /*String*/message) {
		this._trace.entering(this.id, "_showMessage");
		
		this.msgArea.hideMessages();

		if (type === "critical") {
			this.msgArea.showErrorMessage(message);
		}else if (type === "info") {
			this.msgArea.showInfoMessage(message);
		}
		
		this._trace.exiting(this.id, "_showMessage");
	
	},
	
	_setPanelNavigationButtons: function(){
		this._trace.entering(this.id, "_setPanelNavigationButtons");
		
		/* 
		* This is for keeping Next button visible on the last panel otherwise it is hidden on 
		* the last panel as per the implementation from dojox.widget.Wizard
		*/
		
		if (this.wizard.selectedChildWidget.id === this.wizard.getChildren()[this.wizard.getChildren().length-1].id)
		{
			this.wizard.nextButton.domNode.style.display = "";
		}
		
		/* Adding validation so that on navigation between the wizard panels , any validation errors will reappear*/		
		if ((this.wizard.selectedChildWidget.widget.id.indexOf("userPanel") != -1) ||(this.wizard.selectedChildWidget.widget.id.indexOf("permissionPanel") != -1))
		{
			this.wizard.selectedChildWidget.widget.validate();
		}	
		
		this._trace.exiting(this.id, "_setPanelNavigationButtons");
	},
	
	_updateBreadCrumb:function(){
		this._trace.entering(this.id, "_updateBreadCrumb");
		
		if(this.requireBreadCrumb){
			if(this.breadCrumbBar){
				//Set the appropriate breadcrumb.
				if(this._lastbreadcrumb !== null){
					// Reset the last breadcrumb.
					dojo.removeClass(this._lastbreadcrumb, "wizardpaneactive");
				}
				dojo.addClass(dojo.byId("BreadCrumbpanel"+this.wizard.selectedChildWidget.id), "wizardpaneactive");
				this._lastbreadcrumb = dojo.byId("BreadCrumbpanel"+this.wizard.selectedChildWidget.id);
			}
		}
		
		this._trace.exiting(this.id, "_updateBreadCrumb");
	},
	
	_createBreadcrumbBar: function() {
		this._trace.entering(this.id, "_createBreadcrumbBar");
	
		if(!this.requireBreadCrumb){
			return;
		}

		this.breadCrumBuilder = new com.ibm.security.modeling.breadcrumbs.BreadcrumbBuilder();

		var wizardpanes = this.wizard.getChildren();
		for(var i=0;i<wizardpanes.length;i++){
			this.breadCrumBuilder.withBreadcrumb(
					new com.ibm.security.modeling.projects.BreadCrumb(
						{
							id:"BreadCrumbpanel" + wizardpanes[i].id,
							// TODO Get the label from the widgets added using addChild(widget)
							label:wizardpanes[i].breadcrumblabel
						})
			);
		}

		this.breadCrumbBar= this.breadCrumBuilder.withoutAccessibility().build().getBreadCrumbBar();
		
		dojo.place(this.breadCrumbBar.domNode, this.namespace + "mainContainerBreadCrumbPlaceHolder", "only");

		this.breadCrumbBar.startup();
		
		dojo.addClass(this.breadCrumbBar.containerNode,"wizardBreadcrumb");
		
		this._trace.exiting(this.id, "_createBreadcrumbBar");
	},
		
	switchToTableView:function(){
		this._trace.entering(this.id, "switchToTableView");
		
		/*
		 * TODO 
		 * Need to check whether to have different function for users and permissions
		 * or we should switch here itself based on the current selected panel.
		 */
		if(this.wizard.selectedChildWidget.id === "panel2"){
			this.usersResourcesSelectionPanel._switchToTableView();
		}else {
			this.permissionsSelectionPanel._switchToTableView();
		}
		
		//this.permissionsSelectionPanel._switchToTableView();
		
		this._trace.exiting(this.id, "switchToTableView");
	},
	
	
	createRolesPolicies:function(){	
		this._trace.entering(this.id, "createRolesPolicies");
		
		// summary:
		//			Finishes the wizard and persists all data.
		var selections = this.usersResourcesSelectionPanel.getPanelInput();
		
		this._trace.exiting(this.id, "createRolesPolicies");
	},
	
	_onCancel: function(){
		//this._trace.entering(this.id, "_onCancel");
		
		//this.destroy(); // TODO : this will causing null wizard
		this.isCreateView = false;
		this.msgArea.hideMessages();// clear previous messages
		this._closeWizard();
		//this.wizard.destroyRecursive();
		
		//this._trace.exiting(this.id, "_onCancel");
	},
	
	_onFinish: function(){
		this._trace.entering(this.id, "_onFinish");
		
		// summary:
		//			Exits the wizards but persists state
		
		//Validations needed to be called explicitly
		if (this.newProjectCreationPanel.validateFields())
		{	
			if (this.usersResourcesSelectionPanel.validateUserSelections())
			{	
				if (this.permissionsSelectionPanel.validateUserSelections())
				{	
					this._saveProject();
				}
			}
		}
		////alert("_onFinish> " + this.wizard.selectedChildWidget.id + " > " + this.wizardpanels[this.wizard.selectedChildWidget.title] );
		
		this._trace.exiting(this.id, "_onFinish");
	},
	
	_closeWizard: function(){
		this._trace.entering(this.id, "_closeWizard");
		
		//this.newProjectCreationPanel.clearFields();
		
		this._destroyPanels();
		
		this._parent.launchProjectTablePanel(false);
		
		this._trace.exiting(this.id, "_closeWizard");
	},
	
	
	_destroyPanels: function(){
		this._trace.entering(this.id, "_destroyPanels");
	
		try{
		// Remove the Panels. Hopefully, it will remove  all the Panels items.
		
		dojo.forEach(this._connections, dojo.disconnect);
		
		this.wizard._beingDestroyed = true;
		this.wizard.destroyDescendants();
		this.wizard.selectedChildWidget = undefined;
		this.wizard._beingDestroyed = false;
		/*
		var wizardpanes = this.wizard.getChildren();
		
		for(var i=0; i<wizardpanes.length; i++){
			this.wizard.removeChild(wizardpanes[i]);
			if(wizardpanes[i].destroyRecursive){
				wizardpanes[i].destroyRecursive();
			}
		}
		*/
		//Destroy the breadCrumb.
		if(this.requireBreadCrumb){
			if( this.breadCrumbBar ) {

				this.breadCrumbBar.destroyRecursive();
				this.breadCrumbBar = null;
			}
		}
		}catch(e){
			console.log(e);
		}
		
		this._trace.exiting(this.id, "_destroyPanels");
	},
	
	reset: function(){
		this._trace.entering(this.id, "reset");
		
		this._destroyPanels();
		this.createPanels();
		this._initialize();
		
		this._trace.exiting(this.id, "reset");
		
	}
});