/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

� Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.widgets.Summary");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.security.modeling.util.Trace");
dojo.requireLocalization("com.ibm.security.modeling.widgets", "SummaryWidgetText");

// define the summary widget class
dojo.declare ( "com.ibm.security.modeling.widgets.Summary", [ dijit._Widget, dijit._Templated ], {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.widgets.Summary"),
	
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.widgets","templates/Summary.html"),
	widgetsInTemplate: true,
	
	headerLabel: "",
	bodyLabel: "",
	// Widget API methods.
	addSection: function(args) {
		//	returns created body section widget id.
		//	this id can be used to add items to it.
		return this.body.addSection(args);
	},
	getAllSections:function(){
		return this.body.getAllSections();
	},
	addSectionItem: function(sectionId, item) {
		this.body.addSectionItem(sectionId, item);
	},
	
	//Dinesh
	removeSection: function(sectionId) {
		this.body.removeSection(sectionId);
	},
	
	removeSectionItem: function(sectionId, itemId) {
		this.body.removeSectionItem(sectionId, itemId);
	},
	getSectionItems:function(sectionId) {
		return this.body.getSectionItems(sectionId);
	},
	getSectionItemCount:function(sectionId) {
		return this.body.getSectionItemCount(sectionId);
	},
	updateFooterText: function(text)
	{
		this.footer.updateFooterText(text);
	},
	getFooterText: function(){
		return this.footer.getFooterText();
	},
	updateFooterTotalNumber: function(text)
	{
		this.footer.updateFooterTotalNumber(text);
	},
	getFooterTotalNumber:function(){
		return this.footer.getFooterTotalNumber();
	},
	setFooterLinkText: function(text)
	{
		this.footer.setFooterLinkText(text);
	},
	hideFooterLink : function(){
		this.footer.hideFooterLink();
	},
	showFooterLink : function(){
		this.footer.showFooterLink();
	},
	setFooterLink: function(text)
	{
		this.footer.setFooterLink(text);
	},
	removeAll:function(){
		if (this.body) {
			this.body.removeAll();
		}
		if (this.footer) {
			this.footer.updateFooterTotalNumber(0);
		}
	}
	/*,
	destroy:function(){
		this.body.destroy();
		this.footer.destroy();
		this.inherited();
	}*/
	// Function API method.
	
});

// define the summary header widget class
dojo.declare ("com.ibm.security.modeling.widgets.SummaryHeader", [ dijit._Widget, dijit._Templated ], {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.widgets.SummaryHeader"),
	
	// templatePath: dojo.moduleUrl("com.ibm.security.modeling.widgets","templates/Summary.html"),
	templateString: "<h1>${label}</h1>",
	widgetsInTemplate: true,
	label: "",
	postMixInProperties:function() {
		// TODO: build contents.
		// this.labelText = this.label || "header "+this.id;
		this.inherited(arguments);
	}
});

// define the summary body widget class
dojo.declare ("com.ibm.security.modeling.widgets.SummaryBody", [ dijit._Widget, dijit._Templated ], {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.widgets.SummaryBody"),
	
	// templatePath: dojo.moduleUrl("com.ibm.security.modeling.widgets","templates/Summary.html"),
	templateString: "<div>${label}</div>",
	widgetsInTemplate: true,
	sectionCount: 0,
	
	label: "",
	constructor: function() {
		this._trace.entering("", "constructor");
		this._trace.object(arguments, "constructor", "Constructor args");
		this._sections = {};
		this._trace.exiting("", "constructor");
	},
	
	postMixInProperties:function() {
		// TODO: build heading.
		// this.labelText = this.label || "body "+this.id;
		this.inherited(arguments);
	},
	
	// Widget API methods.
	addSection: function(args) {
		// summary: adds a new body section
		//	returns created body section widget id.
		
		// create hook
		//var id = this.id+"_Section_"+this.sectionCount++;

		//console.info("==========args.id: "+args.id);
		this._trace.entering("Entering addSection with args.id: " +args.id, "addSection");
						
		if(this._sections[args.id]){
			return (args.id);
		}
		
		var id = args.id;
		var hook = dojo.create(
			"div", 
			{}, 
			this.domNode,
			"last");
		
		// create SummaryBodySection and attach it to the hook
		var section = new com.ibm.security.modeling.widgets.SummaryBodySection({
			id: this.id + id,
			args: args
		}, hook);
		
		this._sections[id] = section; 
		//console.info("==========id: "+id);
		var tail = dojo.hitch(this, "_checkIfAttributeIsRemoved", id);
		dojo.connect(this._sections[id], "destroy", tail);
		this._trace.exiting("Exiting addSection with id: " +id, "addSection");
		return id;
	},
	getAllSections:function(){
		return this._sections;
	},
	_checkIfAttributeIsRemoved: function(sectionId) {
		this._trace.entering("===========Entering", "_checkIfAttributeIsRemoved");
		this._trace.object(this._sections, "_checkIfAttributeIsRemoved", "this._sections");
		
		if (this._sections[sectionId])
		{	
			this._sections[sectionId]=null;
			delete this._sections[sectionId];
		}
		
		this._trace.object(this._sections, "_checkIfAttributeIsRemoved", "this._sections");
		this._trace.exiting("Exiting", "_checkIfAttributeIsRemoved");
	},

	// Widget API methods.
	removeSection: function(sectionId) {
		// summary: removes an existsing body section
		
		//console.info("==========sectionId: "+sectionId);
		this._trace.entering("Entering removeSection with sectionId: " +sectionId, "removeSection");
		
		if(this._sections[sectionId]!=null)
		{
			this._sections[sectionId].destroyRecursive();
		}
		else{
			//console.info("==========Section Does not exist - sectionId: "+sectionId);
		}
	},
	getSectionItems:function(sectionId) {
		var bodySection = this._sections[sectionId];
		if (bodySection) {
			return bodySection.getItems();
		}
	},
	getSectionItemCount:function(sectionId) {
		var bodySection = this._sections[sectionId];
		if (bodySection) {
			return bodySection.getItemCount();
		}
	},
	addSectionItem: function(sectionId, item) {
		var bodySection = this._sections[sectionId];
		if (bodySection) {
			bodySection.addItem(item);
		}
	},
	
	removeSectionItem: function(sectionId, itemId) {
		var bodySection = this._sections[sectionId];
		if (bodySection) {
			bodySection.removeItem(itemId);
		}
	},
	removeAll:function(){
		var sections=this.getAllSections();
		for(var id in sections){
			var section=this._sections[id];
			if(section){
				section.removeAll();
				section.destroyRecursive();
			}
		}
		
		this._sections=[];
	}
	/*destroy:function(){
		this.removeAll();
		this.inherited();
	}*/
	
	// Function API method.
	
});

//define the summary body section widget class
dojo.declare ("com.ibm.security.modeling.widgets.SummaryBodySection", [ dijit._Widget, dijit._Templated ], {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.widgets.SummaryBodySection"),
	_StringUtils: new com.ibm.security.modeling.util.StringUtils(),
	//templateString: "<div><div class='${sectionHeadingClass}'><span class='${sectionIconClass}'></span><span>${heading}</span><span class='closeNode closeIcon' dojoAttachEvent='ondijitclick: onClose'></span></div></div>",
	templateString: "<div><div class='${sectionHeadingClass}'><div class='itemTextClass'><span class='${sectionIconClass}'></span><h2 class='sectionHeaderText'>${heading}</h2></div><span class='closeNode closeIcon' dojoAttachEvent='ondijitclick: onClose' tabIndex='0'></span></div></div>",
	widgetsInTemplate: true,
	
	// should be supplied as arguments.
	sectionIconClass: "",
	sectionHeadingClass: "",
	
	itemCount: 0,
	constructor: function() {
		this._trace.entering("", "constructor");
		this._trace.object(arguments, "constructor", "Constructor args");
		this._items = {};
		this._trace.exiting("", "constructor");
	},
	
	postMixInProperties:function() {
		
		// TODO: build heading from arguments
		this.sectionIconClass = (this.args && this.args.iconClass)? this.args.iconClass : this.sectionIconClass;
		this.sectionHeadingClass = (this.args && this.args.headingClass)? this.args.headingClass : this.sectionHeadingClass;
		var content = (this.args && this.args.heading)? this.args.heading : "body section "+this.id;
		this.heading = this._StringUtils.parseHTMLandNormalize(content.toString());
		
		this.inherited(arguments);
	},
	
	postCreate: function() { 
		this._trace.entering(this.id, "postCreate");
		dojo.addClass(this.domNode,"bodysection");
		this.inherited("postCreate", arguments);
		this._trace.exiting(this.id, "postCreate");
	},
	
	// Widget API methods.
	addItem: function(item) {
		// summary: adds a new body section item
		
		// create hook
		//var id = this.id+"_Item_"+this.itemCount++;
		
	try
	{
		this.itemCount++;
		var id;
		this._trace.entering("====Entering", "addItem");
		//console.info("======= inside addItem:");
		
		this._trace.object(item, "addItem", "addItem item");
		//console.dir(item);
		
		if (item) {
			if (item.id) {
				id = item.id;
			} else {
				id = this.id+"_Item_"+ this.itemCount;
			}
		} else {
			id = this.id+"_Item_"+ this.itemCount;
		}
		
		//var id = this.id+"_Item_"+ ((item && item.id) || this.itemCount);
		
		if(this._items[id]) {
			return;
		}
		
		var hook = dojo.create(
			"div", 
			{}, 
			this.domNode,
			"last");
		
		// create SummaryBodySection and attach it to the hook
		var sectionItem = new com.ibm.security.modeling.widgets.SummaryBodySectionItem({
			id: id + "_" + this.id + "_" + item.text,
			item: item
		}, hook);
		
		this._items[id] = sectionItem;
		
		var tail = dojo.hitch(this, "_checkIfLastItemIsRemoved", id);
		dojo.connect(this._items[id], "destroy", tail);
		this._trace.exiting("====Exiting addItem", "addItem");
	 }	
	 catch(e)
	 {
		 console.info("==============Error==========");
		 console.dir(e);
	 }
		
	},

	_checkIfLastItemIsRemoved: function(itemId) {
		this._items[itemId]=null;
		delete this._items[itemId];
		
		var flag = false;
		this._trace.entering("Entering", "_checkIfLastItemIsRemoved");
		//console.info("============Inside _checkIfLastItemIsRemoved");
		this._trace.object(this._items, "_checkIfLastItemIsRemoved", "this._items");
		//console.dir(this._items);
		for (var item in this._items) {
			flag = true;
			break;
		}
		if (!flag) {
			this.destroyRecursive();
		}
		this._trace.exiting("Exiting", "_checkIfLastItemIsRemoved");
	},
	
	// Widget API methods.
	removeItem: function(itemId) {
		// summary: removes an existing body section item
		
		this._trace.entering("Entering removeItem with itemId:"+itemId, "removeItem");
		//console.info("=========inside removeItem:" +itemId);
		this._trace.object(this._items, "removeItem", "this._items");
		//console.dir(this._items);
		if(this._items[itemId]!=null) {
			//console.info("=========deleting item:" +itemId);
			this._trace.finest("deleting item:" +itemId,"removeItem");
			this.itemCount--;
			this._items[itemId].destroyRecursive();
		}
		this._trace.exiting("Exiting removeItem", "removeItem");
	},
	getItems: function() {
		return this._items;
	},
	getItemCount: function() {
		var i=0;
		for(var item in this._items){
			i++;
		}
		return i;
	},
	onClose: function() {
		
		//alert('To do: remove at section level.');
		this._trace.entering("Entering onClose at section level", "onClose");
		var callback = this.args.onCloseCallBack;
		var parameter = this.args;
		
		try
		{
		// loop:	this._items this.removeItem(itemId));
		for (var itemId in this._items) {
			this._items[itemId].destroyRecursive();
		}
		}
		catch(e)
		{
			this._trace.exception(e, "=========onClose at attribute level");
		}
		
		if (callback) {
			callback(parameter);
		}

		//this.destroyRecursive();
		this._trace.exiting("Exiting onClose at section level", "onClose");
	},
	removeAll:function(){
		var items=this.getItems();
		for (var itemId in items) {
			var item=items[itemId];
			if(item){
				item.destroyRecursive();
			}
		}
		this._items={};
		this.itemCount=0;
	
	}
	/*destroy:function(){
		this.removeAll();
		this.inherited();
		
	}*/
});

//define the summary body section widget class
dojo.declare ("com.ibm.security.modeling.widgets.SummaryBodySectionItem", [ dijit._Widget, dijit._Templated ], {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.widgets.SummaryBodySectionItem"),
	_StringUtils: new com.ibm.security.modeling.util.StringUtils(),
	//templateString: "<div><span class='${itemIconClass}'></span><span class='${itemContentClass}'>${contents}</span><span class='closeNode closeIcon' dojoAttachEvent='onclick: onClose'></span></div>",
	templateString: "<div><div class='itemTextClass'><span class='${itemIconClass}'></span><span class='${itemContentClass}'>${contents}</span></div><span class='closeNode closeIcon' dojoAttachEvent='ondijitclick: onClose' tabIndex='0'></span></div>",
	widgetsInTemplate: true,
	
	// should be taken from arguments.item
	itemIconClass: "",
	itemContentClass: "",
	
	postMixInProperties:function() {
		// TODO: build contents from arguments.item
		var content = (this.item && this.item.text) ? this.item.text : "body section item "+this.id;
		this.contents = this._StringUtils.parseHTMLandNormalize(content);
		this.itemIconClass = (this.item && this.item.iconClass)? this.item.iconClass : this.itemIconClass;
		this.itemContentClass = (this.item && this.item.contentClass)? this.item.contentClass : this.itemContentClass;
		
		this.inherited(arguments);
	},
	
	postCreate: function() { 
		this._trace.entering(this.id, "postCreate");
		dojo.addClass(this.domNode,"bodysectionitem");
		dojo.addClass(this.domNode,"p");
		this.inherited("postCreate", arguments);
		this._trace.exiting(this.id, "postCreate");
	},
	
	onClose: function() {
		//alert('To do: remove at item level.');
		this._trace.entering("Entering onClose at item level", "onClose");
		var callback = this.item.onCloseCallBack;
		var item = this.item;
		
		this.destroyRecursive();
		
		if(callback) {
			callback(item);
		}
			
			
		this._trace.exiting("Exiting onClose at section level", "onClose");
	}

});

// define the summary footer widget class
dojo.declare ("com.ibm.security.modeling.widgets.SummaryFooter", [ dijit._Widget, dijit._Templated ], {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.widgets.SummaryFooter"),
	
	//templateString: "<div dojoType='com.ibm.security.modeling.dijit.layout.BorderContainer'><div dojoType='com.ibm.security.modeling.dijit.layout.ContentPane' class='footerContents' region='left' id='${id}_footerContent'>${contents}</div><div dojoType='com.ibm.security.modeling.dijit.layout.ContentPane' class='footeranchor' region='trailing' id='${id}_footerAnchor' class='footeranchor'><a href='' onClick='this.viewall();'>View All</a></div></div>",
	//templateString: "<div dojoType='com.ibm.security.modeling.dijit.layout.BorderContainer'><div dojoType='com.ibm.security.modeling.dijit.layout.ContentPane' class='footerContents' region='left' id='${id}_footerContent'>${contents}</div><div dojoType='com.ibm.security.modeling.dijit.layout.ContentPane' class='footeranchor' region='trailing' id='${id}_footerAnchor' class='footeranchor'><a dojoAttachEvent='onclick: onClickViewAll' style = 'color: #32659c'>View All</a></div></div>",
	//localResources: dojo.i18n.getLocalization("com.ibm.security.modeling.widgets", "FileUploadWidgetText");
	
	
	templateString: "<div class='p' dojoAttachPoint='summaryFooterTopNode'><span class='footerContents' id='${id}_footerContent'>${contents}</span>" +
		"<span class='footerTotalNumber' id='${id}_footerTotalNumber'>0</span>"+
		"<span class='footeranchor' id='${id}_footerAnchor' class='footeranchor'>" + 
		"<a href='javascript:void(0)' tabIndex='0' dojoAttachEvent='ondijitclick: onClickViewAll'>${localResources.VIEW_ALL}</a></span></div>",
	widgetsInTemplate: true,
	
	constructor : function() {
		this.localResources= dojo.i18n.getLocalization("com.ibm.security.modeling.widgets", "SummaryWidgetText");

	},

	postCreate : function() {
		//this.templateString= "<div dojoType='com.ibm.security.modeling.dijit.layout.BorderContainer'><div dojoType='com.ibm.security.modeling.dijit.layout.ContentPane' class='footerContents' region='left' id='${id}_footerContent'>${contents}</div><div dojoType='com.ibm.security.modeling.dijit.layout.ContentPane' class='footerTotalNumber' id='${id}_footerTotalNumber'>0</div><div dojoType='com.ibm.security.modeling.dijit.layout.ContentPane' class='footeranchor' region='trailing' id='${id}_footerAnchor' class='footeranchor'><a dojoAttachEvent='onclick: onClickViewAll' style = 'color: #006699'>${localResources.VIEW_ALL}</a></div></div>";
	},

	postMixInProperties:function() {
		// TODO: build contents.
		if (!this.contents) { 
			this.contents = "footer "+this.id;
		}
		/*
		if (!this.footerLink) {
			node = dojo.byId(this.id+"_footerAnchor");
			dojo.removeClass(node, "footeranchor");
			dojo.applyStyle(node, "display:none");
		}
		*/
		this.inherited(arguments);
	},
	
	updateFooterText: function(text) {
		if(dojo.byId(this.id+'_footerContent')){
			dojo.byId(this.id+'_footerContent').innerHTML = text;
		}
	},
	getFooterText : function(){
		if(dojo.byId(this.id+'_footerContent')){
			return dojo.byId(this.id+'_footerContent').innerHTML;
		}
	},
	updateFooterTotalNumber: function(text) {
		if(dojo.byId(this.id+'_footerContent')){
			dojo.byId(this.id+'_footerTotalNumber').innerHTML = text;
		}
	},
	getFooterTotalNumber:function(){
		return dojo.byId(this.id+'_footerTotalNumber').innerHTML;
	},
	setFooterLinkText: function(text) {
		//Yet to be implemented for generic purpose. Currently "View All" is displayed. 
	},
	hideFooterLink: function() {
		var node = dojo.byId(this.id+'_footerAnchor').childNodes[0];
		dojo.addClass(node,"hidden");
	},
	showFooterLink: function() {
		var node = dojo.byId(this.id+'_footerAnchor').childNodes[0];
		dojo.removeClass(node,"hidden");
	},
	setFooterLink: function(text) {
		dojo.byId(this.id+'_footerAnchor').childNodes[0].href=text;
		//dojo.byId(this.id+'_footerLink').innerHTML = text;
	},
	onClickViewAll: function(){
		//alert("View all clicked");
	}
});