dojo.provide("com.ibm.security.modeling.widgets.SwitchableTopologyView");

dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling", "LogMessages");
dojo.requireLocalization("com.ibm.security.modeling", "PluginMessages");
/*
*  This widget will display the Hierarchical view of attribute values with additional toolbar button
*  that can switch between the Topology and Table view.
*/
dojo.declare("com.ibm.security.modeling.widgets.SwitchableTopologyView", [ dijit._Widget, dijit._Templated ],
{
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.widgets","templates/SwitchableTopologyView.html"),
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.widgets.SwitchableTopologyView"),
	
	defaultView: "TABLE", //{"TABLE" , "TOPOLOGY"}
	// Dataset required for TopologyView.
	dataset: null, 
	widgetDefinitionId: null,
	
	//Data required for TableView.
	tableUri: null,
	
	messageArea: null,
	
	// Local variables.
	
	_currentView: null,
	_currentWidget: null,
	_globalResources: null,
	_messageResources: null,
	_pluginResources: null,
	
	constructor: function(){
		this._trace.entering(this.id, "constructor");
		
		this._globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		this._messageResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "LogMessages");
		this._pluginResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "PluginMessages");
		this._trace.exiting(this.id, "constructor");
	},
	
	
	// postMixin
	//	if (!this.defaultWidget) {this.defaultWidget = "TABLE";}
	postCreate: function(){
		this._trace.entering(this.id, "postCreate");
		
		this.inherited(arguments);
		if( (this.defaultWidget == null) || (this.defaultWidget == "TABLE") ) {
			this.switchToTableView();
		} else {
			this.switchToTopologyView();
		}
		
		this._trace.exiting(this.id, "postCreate");
	},
	
	// Override
	getTableWidget: function(){
		// Return a table widget to be displayed for TableView.
	},
	
	_createTable: function(){
		this._trace.entering(this.id, "_createTable");
		
		this._currentWidget = this.getTableWidget();
		dojo.style(this._currentWidget.domNode, "width", "100%");
		dojo.style(this._currentWidget.domNode, "height", "100%");
		dojo.place(this._currentWidget.domNode, this.container, "only"); 
		if (this._currentWidget.widget) {
			this._currentWidget.widget.startup();
		}
		this._currentWidget.startup();
		this._currentView = "TABLE";
		
		this._trace.exiting(this.id, "_createTable");
	},
	
	getTopologyWidget: function(){
		// Return a table widget to be displayed for TopologyView.
	},
	
	_createTopology: function(){
		this._trace.entering(this.id, "_createTopology");
		
		this._currentWidget = this.getTopologyWidget();
		dojo.style(this._currentWidget.domNode, "width", "100%");
		dojo.style(this._currentWidget.domNode, "height", "100%");
		dojo.place(this._currentWidget.domNode, this.container, "only"); 
		if (this._currentWidget.widget) {
			this._currentWidget.widget.startup();
		}
		this._currentWidget.startup();
		//this.subscribe("ProjectEditScopeTopologySwitchTableView", this.switchToTableView);
		
		this._currentView = "TOPOLOGY";
		
		this._trace.exiting(this.id, "_createTopology");
	},
	
	_switch: function(){
		this._trace.entering(this.id, "_switch");
		
		if(this._currentView == "TABLE" ){
			this.switchToTopologyView();
		}else{
			this.switchToTableView();
		}
			
		this._trace.exiting(this.id, "_switch");
	},
	
	switchToTableView: function(){
		this._trace.entering(this.id, "switchToTableView");
		
		if(this._currentWidget) {
			this._currentWidget.destroyRecursive();
		}
		this._createTable();
		this.onSwitch(this._currentView, this);
		
		this._trace.exiting(this.id, "switchToTableView");
	},
	
	switchToTopologyView: function(){
		this._trace.entering(this.id, "switchToTopologyView");
		
		if(this._currentWidget) {
			this._currentWidget.destroyRecursive();
		}
		this._createTopology();
		this.onSwitch(this._currentView, this);
		
		this._trace.exiting(this.id, "switchToTopologyView");
	},
	
	startup: function() {
		this._trace.entering(this.id, "startup");
		
		this.inherited(arguments);
		this._currentWidget.startup();
		
		this._trace.exiting(this.id, "startup");
	},
	
	onSwitch: function(currentView, widget){
	},
	
	onCreateWidget: function(currentView, widget){
		
	},

	destroy: function(){
		this._trace.entering(this.id, "destroy");
		
		this._currentWidget.destroyRecursive();
		
		this.inherited(arguments);

		this._trace.exiting(this.id, "destroy");
	},
	
	resize: function(){
		this._trace.entering(this.id, "resize");
		
		this._currentWidget.resize();
		
		this._trace.exiting(this.id, "resize");
	},
	
	getCurrentWidget: function(){
		return this._currentWidget;
	},
	
	/*
	 * Returns "TABLE" / "TOPOLOGY" depending on whether the current view is Topology or TABLE.
	*/
	getCurrentView: function(){
		return this._currentView;
	},
	
	getTable: function(){
		if(this._currentView == "TABLE"){
			return this._currentWidget;
		}else{
			return null;
		}
	},
	
	getTopology: function(){
		if(this._currentView == "TOPOLOGY"){
			return this._currentWidget;
		}else{
			return null;
		}
	},
	
	getSelectedItemIds: function(){
		this._trace.entering(this.id, "_getSelectedItemIds");
		if(this._currentView == "TOPOLOGY"){
			var topologyWidget = this.getTopology();
			var canvas = topologyWidget._canvas;
			
			var selectionsObject = canvas._getSelectedObjectIdsFromFlexBridge();
	
			this._trace.exiting(this.id, "_getSelectedItemIds");
			
			return selectionsObject.itemIds;
		}else{
			// Get items from Table.
			//TODO
		}
	},
	
	setSelectedItems: function(items){
		if(this._currentView == "TOPOLOGY"){
			var topologyWidget = this.getTopology();
			topologyWidget._canvas._setSelectedObjectIdsOnFlexBridge(items);
		}else{
			//TODO Set the selected items on TABLE
		}
	},
	
	deselectItem: function(itemid){
		currentSelection = this.getSelectedItemIds();
		if(currentSelection){
			var i;
			for(i=0; i<currentSelection.length;i++){
				if(currentSelection[i] == itemid){
					currentSelection.splice(i, 1);
					break;
				}
			}
			//this._setSelectedItems(this._currentSelection);
			this.setSelectedItems(currentSelection);
		}else{
			//TODO nothing to do.
		}
	},

	getUnselectedItems: function(previousSelection){
		if(this._currentView == "TOPOLOGY"){
			var topologyWidget = this.getTopology();
			var newSelections = this.getSelectedItemIds();
			var unselectedItems = [];
			var i;
			for(i=0; i<previousSelection.length; i++){
				var j;
				var foundElementi = false;
				for(j=0;j<newSelections.length; j++){
					if(newSelections[j] == previousSelection[i]){
						foundElementi = true;
						break;
					}
				}
				if(!foundElementi){
					unselectedItems.push(previousSelection[i]);
				}
			}
			
			return unselectedItems;
		}
		
	}
	
});